/*********************************************************************/
/*                                                                   */
/*  Cyrene.c :  Simulateur de code 65c816 et Hardware Apple IIgs.    */
/*                                                                   */
/*********************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe  *  Date : Mars 2018  */
/*********************************************************************/

#include <stdio.h>
#include <stdlib.h>

#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>
#include <Commdlg.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_Request.h"
#include "Dc_Output.h"
#include "Dc_Notification.h"
#include "Cyrene_Engine.h"
#include "Window_IPC.h"
#include "Window_Preference.h"
#include "Window_QuickWatch.h"
#include "Window_Main.h"

struct parameter *InitParameter(HINSTANCE);

/*******************************************************/
/*  WinMain() :  Fonction principale de l'application. */
/*******************************************************/
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
  MSG msg;
  BOOL bRet;
  int error;
  struct window_ipc *window_ipc;
  struct window_main *window_main;
  struct parameter *param;

  /* Init */
  window_ipc = NULL;
  window_main = NULL;

  /* Initialisation des Bibliothques partages */
  my_Memory(MEMORY_INIT_ALL,NULL,NULL);
 
  /** Initialisation des parametres d'application **/
  param = InitParameter(hInstance);
  if(param == NULL)
    {
      my_Memory(MEMORY_FREE_ALL,NULL,NULL);
      return(FALSE);
    }

  /* Initialisation des Notifications */
  my_Notification(NOTIFICATION_INIT,NULL,NULL);

  /** Initialisation des Controls Windows **/
  InitCommonControls();

  /* Charge la DLL du Rich Text Edit Control */
  param->rich_edit_dll = LoadLibrary("RICHED20.DLL"); 
  if(param->rich_edit_dll == NULL)
    {
      my_Memory(MEMORY_FREE_ALL,NULL,NULL);
      return(FALSE);
    }

  /** Charge la table Accelerateur **/
  param->hAccelerator = LoadAccelerators(hInstance,MAKEINTRESOURCE(IDR_ACCELERATOR_TABLE));

  /** Cration de la fentre principale 'Main' **/
  error = BuildMainWindow(hInstance);
  if(error)
    {
      my_Memory(MEMORY_FREE_ALL,NULL,NULL);
      return(FALSE);
    }

  /** Lance le Thread de gestion des Requtes **/
  error = StartRequestManager();
  if(error)
    {
      /* Ferme la fentre Main */
      my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
      if(window_main != NULL)
        SendMessage(window_main->hwnd_main,WM_CLOSE,0,0);
    }

  /** Lance le Thread de gestion de la fentre IPC + Cration de la fentre IPC **/
  error = StartWindowIPCThread();
  if(error)
    {
      /* Ferme la fentre Main */
      my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
      if(window_main != NULL)
        SendMessage(window_main->hwnd_main,WM_CLOSE,0,0);
    }

  /** Lance le Thread de gestion des Output Lines **/
  error = StartOutputManager();
  if(error)
    {
      /* Ferme la fentre Main */
      my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
      if(window_main != NULL)
        SendMessage(window_main->hwnd_main,WM_CLOSE,0,0);
    }

  /* Rcupre la handle de la fentre Main */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);

  /** On va charger en mmoire les fichiers Preference (Breakpoint/Quickwatch/Symbol/Trace System Calls) **/
  LoadInitPreferenceFiles();

  /** Message Loop pour toutes les fentres de ce Thread (Fentre Main, Fentre Project, Fentre Snapshot...) **/
  while((bRet = GetMessage(&msg,NULL,0,0)) != 0)
    {
      if(bRet == -1)
        break;

      /* Utilise la table des Accelerateurs pour transformer les touches en Evnements */
      if(!TranslateAccelerator(window_main->hwnd_main,param->hAccelerator,&msg))
        {
          TranslateMessage(&msg);
          DispatchMessage(&msg);
        }
    }

  /* Ferme le Thread Output et attend sa fin */
  param->stop_output_thread = 1;
  while(param->output_thread_status == 1)
    Sleep(1);

  /* Ferme le Thread Request */
  param->stop_request_thread = 1;

  /* Ferme la fentre IPC */
  my_Memory(MEMORY_GET_WINDOW_IPC,&window_ipc,NULL);
  if(window_ipc != NULL)
    SendMessage(window_ipc->hwnd_cyrene_ipc,WM_CLOSE,0,0);

  /* Libration des ressources */
  my_Memory(MEMORY_FREE_ALL,NULL,NULL);
  my_Notification(NOTIFICATION_FREE,NULL,NULL);

  return((int)msg.wParam);
}


/****************************************************/
/*  InitParameter() :  Initialiser les paramtres.  */
/****************************************************/
struct parameter *InitParameter(HINSTANCE hInstance)
{
  struct parameter *param;

  /** Allocation de la structure Param **/
  param = mem_alloc_parameter(hInstance);
  if(param == NULL)
    return(NULL);
    
  /* Dclare la structure Param */
  my_Memory(MEMORY_SET_PARAM,param,NULL);
  
  /* Initialisation du Log */
  CyreneLogLine("InitFile");

  /* OK */
  return(param);
}

/*************************************************************************/
