/********************************************************************/
/*                                                                  */
/*  Cyrene_Engine.c :  Module de gestion du simulateur du 65c816.   */
/*                                                                  */
/********************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe  *  Date : Mai 2018  */
/********************************************************************/

#include <stdio.h>
#include <stdlib.h>

#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>
#include <Commdlg.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_MMU.h"
#include "Dc_CPU.h"
#include "Dc_IIgsInternals.h"
#include "Dc_Snapshot.h"
#include "Window_QuickWatch.h"
#include "Window_Main.h"
#include "Cyrene_Engine.h"

static struct s65816 *BuildEngineFromSnapshot(struct snapshot *);

/*************************************************************/
/*  Initialize65816Engine() :  Dmarre la simulation 65c816. */
/*************************************************************/
void Initialize65816Engine(struct snapshot *current_snapshot)
{
  struct s65816 *current_65816;

  /* Allocation mmoire | Vrification / Remplissage */
  current_65816 = BuildEngineFromSnapshot(current_snapshot);
  if(current_65816 == NULL)
    return;
    
  /* Stocke la simulation en mmoire */
  my_Memory(MEMORY_SET_S65816,current_65816,NULL);
}


/****************************************************************************/
/*  BuildEngineFromSnapshot() :  Cration du moteur  partir d'un Snapshot. */
/****************************************************************************/
static struct s65816 *BuildEngineFromSnapshot(struct snapshot *current_snapshot)
{
  int rom_size;
  struct s65816 *current_65816;
  char buffer_error[1024];

  /* Vrification de la taille de la mmoire */
  my_Memory(MEMORY_GET_S65816,&current_65816,NULL);
  if(current_65816 != NULL)
    {
      /* On va devoir retailler la mmoire */
      if(current_65816->current_mmu->nb_extended_bank != current_snapshot->ram_size)
        {
          mem_free_s65816(current_65816);
          current_65816 = NULL;
        }
    }

  /* Allocation mmoire */
  if(current_65816 == NULL)
    {
      current_65816 = (struct s65816 *) calloc(1,sizeof(struct s65816));
      if(current_65816 == NULL)
        return(NULL);
      current_65816->current_mmu = (struct mmu *) calloc(1,sizeof(struct mmu));
      current_65816->current_cpu = (struct cpu *) calloc(1,sizeof(struct cpu));
      if(current_65816->current_mmu == NULL || current_65816->current_cpu == NULL)
        {
          mem_free_s65816(current_65816);
          return(NULL);
        }

      /* Ram Extended */
      current_65816->current_mmu->nb_extended_bank = current_snapshot->ram_size;
      current_65816->current_mmu->ram_extended     = (unsigned char *) calloc(current_snapshot->ram_size*64*1024,sizeof(unsigned char));
      current_65816->current_mmu->map_ram_extended = (unsigned char *) calloc(current_snapshot->ram_size*64*1024,sizeof(unsigned char));
      if(current_65816->current_mmu->ram_extended == NULL || current_65816->current_mmu->map_ram_extended == NULL)
        {
          mem_free_s65816(current_65816);
          return(NULL);
        }
    }

  /** Init : CPU **/
  /* Register */
  current_65816->current_cpu->proc_a          = current_snapshot->proc_a;
  current_65816->current_cpu->proc_x          = current_snapshot->proc_x;
  current_65816->current_cpu->proc_y          = current_snapshot->proc_y;
  current_65816->current_cpu->proc_dbr        = current_snapshot->proc_dbr;
  current_65816->current_cpu->proc_pbr        = current_snapshot->proc_pbr;
  current_65816->current_cpu->proc_pc         = current_snapshot->proc_pc;
  current_65816->current_cpu->proc_stack      = current_snapshot->proc_stack;
  current_65816->current_cpu->proc_directpage = current_snapshot->proc_directpage;
  /* Status */
  current_65816->current_cpu->status_c = current_snapshot->status_c;
  current_65816->current_cpu->status_z = current_snapshot->status_z;
  current_65816->current_cpu->status_i = current_snapshot->status_i;
  current_65816->current_cpu->status_d = current_snapshot->status_d;
  current_65816->current_cpu->status_x = current_snapshot->status_x;
  current_65816->current_cpu->status_m = current_snapshot->status_m;
  current_65816->current_cpu->status_v = current_snapshot->status_v;
  current_65816->current_cpu->status_n = current_snapshot->status_n;
  current_65816->current_cpu->status_e = current_snapshot->status_e;
  current_65816->current_cpu->status_b = current_snapshot->status_b;

  /** Init : MMU **/
  rom_size = (current_snapshot->rom_type == SNAPSHOT_APPLEIIGS_ROM01) ? 2*64*1024 : 2*64*1024;
  current_65816->current_mmu->rom_version = (current_snapshot->rom_type == SNAPSHOT_APPLEIIGS_ROM01) ? 1 : 3;

  /* Fast RAM : 00 01 */  
  memcpy(current_65816->current_mmu->ram_00,&current_snapshot->snapshot_data[0*64*1024],64*1024);
  memcpy(current_65816->current_mmu->ram_01,&current_snapshot->snapshot_data[1*64*1024],64*1024);

  /* Extended RAM */
  memcpy(current_65816->current_mmu->ram_extended,&current_snapshot->snapshot_data[2*64*1024],current_65816->current_mmu->nb_extended_bank*64*1024); 

  /* Slow RAM : E0 E1 */
  memcpy(current_65816->current_mmu->ram_e0,&current_snapshot->snapshot_data[(1+1+current_65816->current_mmu->nb_extended_bank+0)*64*1024],64*1024);
  memcpy(current_65816->current_mmu->ram_e1,&current_snapshot->snapshot_data[(1+1+current_65816->current_mmu->nb_extended_bank+1)*64*1024],64*1024);

  /* ROM */
  if(current_snapshot->rom_type == SNAPSHOT_APPLEIIGS_ROM01)
    {
      memcpy(current_65816->current_mmu->rom_fe,&current_snapshot->snapshot_data[current_snapshot->snapshot_length-3*64*1024],64*1024);
      memcpy(current_65816->current_mmu->rom_ff,&current_snapshot->snapshot_data[current_snapshot->snapshot_length-2*64*1024],64*1024);
    }
  else if(current_snapshot->rom_type == SNAPSHOT_APPLEIIGS_ROM03)
    {
      memcpy(current_65816->current_mmu->rom_fc,&current_snapshot->snapshot_data[current_snapshot->snapshot_length-5*64*1024],64*1024);
      memcpy(current_65816->current_mmu->rom_fd,&current_snapshot->snapshot_data[current_snapshot->snapshot_length-4*64*1024],64*1024);
      memcpy(current_65816->current_mmu->rom_fe,&current_snapshot->snapshot_data[current_snapshot->snapshot_length-3*64*1024],64*1024);
      memcpy(current_65816->current_mmu->rom_ff,&current_snapshot->snapshot_data[current_snapshot->snapshot_length-2*64*1024],64*1024);        
    }

  /* DOC RAM */
  memcpy(current_65816->current_mmu->ram_sound,&current_snapshot->snapshot_data[current_snapshot->snapshot_length-1*64*1024],64*1024);

  /* DOC Registers */
  memcpy(current_65816->current_mmu->doc_register,current_snapshot->doc_register,DOC_REGISTER_SIZE);

  /* Battery RAM */
  memcpy(current_65816->current_mmu->battery_ram,current_snapshot->battery_ram,BATTERY_RAM_SIZE);

  /** Construction des listes des Memory Handle (Used/Purged/Free) **/
  mem_free_handle_list(current_65816->list_used_handle);
  mem_free_handle_list(current_65816->list_purged_handle);
  mem_free_handle_list(current_65816->list_free_handle);
  current_65816->list_used_handle   = BuildHandleList(GS_FIRST_HANDLE_USED_ADDR,&current_65816->nb_used_handle,&current_65816->size_used_handle,current_65816->error_used,current_65816->current_mmu);
  current_65816->list_purged_handle = BuildHandleList(GS_FIRST_HANDLE_PURGED_ADDR,&current_65816->nb_purged_handle,&current_65816->size_purged_handle,current_65816->error_purged,current_65816->current_mmu);
  current_65816->list_free_handle   = BuildHandleList(GS_FIRST_HANDLE_FREE_ADDR,&current_65816->nb_free_handle,&current_65816->size_free_handle,current_65816->error_free,current_65816->current_mmu);

  /** Liste des File **/
  mem_free_fileidtable(current_65816->table_file_id);
  current_65816->table_file_id = LoadIDTable(current_65816->list_used_handle,buffer_error,current_65816->current_mmu);

  /** Liste des CDA **/
   mem_free_cdatable(current_65816->current_cda);
  current_65816->current_cda = GetCDATable(buffer_error,current_65816->list_used_handle,current_65816->current_mmu);

  /** Liste des NDA **/
  mem_free_ndatable(current_65816->current_nda);
  current_65816->current_nda = GetNDATable(buffer_error,current_65816->current_mmu);

  /** Liste des Vecteur Quickdraw **/
  mem_free_quickdrawvectortable(current_65816->current_quickdraw_vector);
  current_65816->current_quickdraw_vector = GetQuickdrawVectorTable(buffer_error,current_65816->current_mmu);

  /** Liste des Vecteur Interrupt **/
  mem_free_interruptvectortable(current_65816->current_interrupt_vector);
  current_65816->current_interrupt_vector = GetInterruptVectorTable(buffer_error,current_65816->current_mmu);

  /** Liste des File Segment **/
  mem_free_filesegmenttable(current_65816->current_file_segment);
  current_65816->current_file_segment = GetFileSegmentList(current_65816->list_used_handle,current_65816->current_mmu);

  /** Liste des Init **/
  mem_free_inittable(current_65816->current_init);
  current_65816->current_init = GetInitTable(current_65816);

  /** Liste des Driver **/
  mem_free_drivertable(current_65816->current_driver);
  current_65816->current_driver = GetDriverTable(current_65816);

  /** Liste des FST **/
  mem_free_fsttable(current_65816->current_fst);
  current_65816->current_fst = GetFSTTable(current_65816);

  /* Renvoi la structure */
  return(current_65816);
}


/**************************************************************/
/*  mem_free_s65816() :  Libration mmoire structure s65816. */
/**************************************************************/
void mem_free_s65816(struct s65816 *current_65816)
{
  if(current_65816)
    {
      if(current_65816->current_cpu)
        free(current_65816->current_cpu);
        
      if(current_65816->current_mmu)
        {
          if(current_65816->current_mmu->ram_extended)
            free(current_65816->current_mmu->ram_extended);

          if(current_65816->current_mmu->map_ram_extended)
            free(current_65816->current_mmu->map_ram_extended);

          free(current_65816->current_mmu);
        }
        
      free(current_65816);
    }
}

/********************************************************************/
