/************************************************************************/
/*                                                                      */
/*  Dc_Disassembler.c :  Module pour la bibliothque du dsassemblage.  */
/*                                                                      */
/************************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe Software  *  Aout 2018   */
/************************************************************************/

#include <stdio.h>
#include <time.h>
#include <sys/timeb.h>

#include <windows.h>
#include <windowsx.h>
#include <Commctrl.h>
#include <Winuser.h>
#include <Richedit.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_Notification.h"
#include "Dc_Graphic.h"
#include "Dc_MMU.h"
#include "Dc_IIgsInternals.h"
#include "Dc_Name.h"
#include "Cyrene_Engine.h"
#include "Window_Symbol.h"
#include "Window_Disassembler.h"
#include "Dc_Disassembler.h"

static int ComputeDesassemblyQuality(struct code_line *);
static int IsValidCodeLine(struct code_line *);

/**************************************************************/
/*  DisassembleDataBlock() :  Dsassemble un bloc de donnes. */
/**************************************************************/
struct code_line *DisassembleDataBlock(int data_size, unsigned char *data, int nb_line_max, int address_max, int address, int data_bank, int processor, int *register_mx_io, struct directive_line *first_directiveline, int *quality_coef_rtn)
{
  int i, nb_codeline, current_address, current_data_size, current_register_m, current_register_x, bit_x, bit_m, quality_coef;
  struct code_line *current_codeline;
  struct code_line *first_codeline;
  struct code_line *last_codeline;
  struct directive_line *current_directiveline;
  char name[1024];
  char type[1024];
  char application[1024];

  /* Init */
  nb_codeline = 0;
  first_codeline = NULL;
  last_codeline = NULL;
  current_address = address;
  current_data_size = data_size;
  current_register_m = (*register_mx_io == DISASSEMBLER_REGISTER_00 || *register_mx_io == DISASSEMBLER_REGISTER_01) ? 0 : 1;
  current_register_x = (*register_mx_io == DISASSEMBLER_REGISTER_00 || *register_mx_io == DISASSEMBLER_REGISTER_10) ? 0 : 1;

  /**********************************************/
  /***  On passe toutes les donnes en revue  ***/
  for(i=0; i<data_size; )
    {
      /** A t'on une ligne directive  injecter avant la ligne  dcoder ? **/
      for(current_directiveline = first_directiveline; current_directiveline; current_directiveline=current_directiveline->next)
        if(current_directiveline->address == current_address)
          {
            /** Transforme la directive_line en code_line **/
            current_codeline = (struct code_line *) calloc(1,sizeof(struct code_line));
            if(current_codeline == NULL)
              break;
            current_codeline->is_directive = 1;
            current_codeline->address = current_address;
            current_codeline->m = current_directiveline->m;
            current_codeline->x = current_directiveline->x;
            strcpy(current_codeline->opcode,"MX");
            sprintf(current_codeline->operand,"%%%d%d",current_directiveline->m,current_directiveline->x);
            
            /* Attache la structure  la liste (doublement chaine) */
            current_codeline->previous = last_codeline;
            if(first_codeline == NULL)
              first_codeline = current_codeline;
            else
              last_codeline->next = current_codeline;
            last_codeline = current_codeline;
            nb_codeline++;   
            
            /* On modifier les valeurs MX */
            current_register_m = current_directiveline->m;            
            current_register_x = current_directiveline->x;
            break;
          }
      /* Fin ? */
      if((nb_line_max > 0) && (nb_codeline == nb_line_max))
        break;
          
      /*** Dcode une ligne de Code (pas de call_num  fournir) ***/
      current_codeline = DisassembleOneLine(current_data_size,&data[i],current_address,data_bank,processor,current_register_m,current_register_x,0xFFFF,last_codeline);
      if(current_codeline == NULL)
        break;

      /** Condense les lignes HEX (hors Symbol) **/
      if(current_codeline->data_symbol != 1)
        {
          if(!stricmp(current_codeline->opcode,"HEX") && last_codeline != NULL)
            if(!stricmp(last_codeline->opcode,"HEX") && last_codeline->data_size < 4)
              {
                last_codeline->data[last_codeline->data_size] = current_codeline->data[0];
                last_codeline->data_size++;
                sprintf(&last_codeline->operand[strlen(last_codeline->operand)],",%s",current_codeline->operand);
                mem_free_codeline(current_codeline);
                continue;
              }
        }

      /* Attache la structure  la liste (doublement chaine) */
      current_codeline->previous = last_codeline;
      if(first_codeline == NULL)
        first_codeline = current_codeline;
      else
        last_codeline->next = current_codeline;
      last_codeline = current_codeline;
      nb_codeline++;

      /** MAJ valeurs **/
      i += current_codeline->data_size;
      current_address += current_codeline->data_size;

      /** On va grer les SEP et les REP pour influencer les MX **/
      if(!stricmp(current_codeline->opcode,"SEP"))
        {
          bit_m = (current_codeline->data[1] & 0x20) >> 5;
          if(bit_m == 1)
            current_register_m = 1;
          bit_x = (current_codeline->data[1] & 0x10) >> 4;
          if(bit_x == 1)
            current_register_x = 1;
        }
      if(!stricmp(current_codeline->opcode,"REP"))
        {
          bit_m = (current_codeline->data[1] & 0x20) >> 5;
          if(bit_m == 1)
            current_register_m = 0;
          bit_x = (current_codeline->data[1] & 0x10) >> 4;
          if(bit_x == 1)
            current_register_x = 0;
        }

      /** Fin ? **/
      if((nb_line_max > 0) && (nb_codeline == nb_line_max))
        break;
      if((nb_line_max == 0) && (current_address > address_max))
        break;

      /*******************************************************************/
      /*** On va grer les 3 bytes suivant un appel au MLI de Prodos 8 ***/
      if(!stricmp(current_codeline->opcode,"JSR") && current_codeline->data_value == 0xBF00 && (current_codeline->address >> 16) == 0x00)
        {
          /** 1 Byte de Commande **/
          current_codeline = (struct code_line *) calloc(1,sizeof(struct code_line));
          if(current_codeline == NULL)
            break;
          current_codeline->address = current_address;
          current_codeline->m = current_register_m;
          current_codeline->x = current_register_x;
          current_codeline->data[0] = data[i+0];
          current_codeline->data_value = data[i+0];
          current_codeline->data_size = 1;
          strcpy(current_codeline->opcode,"DFB");
          sprintf(current_codeline->operand,"$%02X",current_codeline->data_value);

          /* Identifie l'appel Prodos 8 */
          GetNameFromAddress(0x0000BF00,(WORD)data[i+0],&name[0],&type[0],&application[0]);
          if(strlen(name) > 0)
            sprintf(current_codeline->comment,"; %s",name);

          /* Attache la structure  la liste */
          current_codeline->previous = last_codeline;
          if(first_codeline == NULL)
            first_codeline = current_codeline;
          else
            last_codeline->next = current_codeline;
          last_codeline = current_codeline;
          nb_codeline++;

          /* MAJ valeurs */
          i += current_codeline->data_size;
          current_address += current_codeline->data_size;

          /** Fin ? **/
          if((nb_line_max > 0) && (nb_codeline == nb_line_max))
            break;
          if((nb_line_max == 0) && (current_address > address_max))
            break;

          /** Adresse des paramtres : 2 bytes **/
          current_codeline = (struct code_line *) calloc(1,sizeof(struct code_line));
          if(current_codeline == NULL)
            break;
          current_codeline->address = current_address;
          current_codeline->m = current_register_m;
          current_codeline->x = current_register_x;
          current_codeline->data[0] = data[i+0];
          current_codeline->data[1] = data[i+1];
          current_codeline->data_size = 2;
          current_codeline->data_value = data[i+0] + 256*data[i+1];
          strcpy(current_codeline->opcode,"DA");
          sprintf(current_codeline->operand,"$%04X",current_codeline->data_value);

          /* Attache la structure  la liste */
          current_codeline->previous = last_codeline;
          if(first_codeline == NULL)
            first_codeline = current_codeline;
          else
            last_codeline->next = current_codeline;
          last_codeline = current_codeline;
          nb_codeline++;

          /* MAJ valeurs */
          i += current_codeline->data_size;
          current_address += current_codeline->data_size;

          /** Fin ? **/
          if((nb_line_max > 0) && (nb_codeline == nb_line_max))
            break;
          if((nb_line_max == 0) && (current_address > address_max))
            break;
        }
      /***********************************************************************/
      /*** On va grer les 6 bytes suivant un appel  de GS/OS | Prodos 16 ***/
      else if(!stricmp(current_codeline->opcode,"JSL") && current_codeline->data_value == 0xE100A8)
        {
          /** 2 Bytes de Commande **/
          current_codeline = (struct code_line *) calloc(1,sizeof(struct code_line));
          if(current_codeline == NULL)
            break;
          current_codeline->address = current_address;
          current_codeline->m = current_register_m;
          current_codeline->x = current_register_x;
          current_codeline->data[0] = data[i+0];
          current_codeline->data[1] = data[i+1];
          current_codeline->data_value = data[i+0] + 256*data[i+1];
          current_codeline->data_size = 2;
          strcpy(current_codeline->opcode,"DA");
          sprintf(current_codeline->operand,"$%04X",current_codeline->data_value);
          if((current_codeline->data_value & 0x00FF) == 0x01)
            strcpy(current_codeline->comment,"; CREATE");
          else if((current_codeline->data_value & 0x00FF) == 0x02)
            strcpy(current_codeline->comment,"; DESTROY");
          else if((current_codeline->data_value & 0x00FF) == 0x03)
            strcpy(current_codeline->comment,"; OS_SHUTDOWN");
          else if((current_codeline->data_value & 0x00FF) == 0x04)
            strcpy(current_codeline->comment,"; CHANGE_PATH");
          else if((current_codeline->data_value & 0x00FF) == 0x05)
            strcpy(current_codeline->comment,"; SET_FILE_INFO");
          else if((current_codeline->data_value & 0x00FF) == 0x06)
            strcpy(current_codeline->comment,"; GET_FILE_INFO");
          else if((current_codeline->data_value & 0x00FF) == 0x07)
            strcpy(current_codeline->comment,"; JUDGE_NAME");
          else if((current_codeline->data_value & 0x00FF) == 0x08)
            strcpy(current_codeline->comment,"; VOLUME");
          else if((current_codeline->data_value & 0x00FF) == 0x09)
            strcpy(current_codeline->comment,"; SET_PREFIX");
          else if((current_codeline->data_value & 0x00FF) == 0x0A)
            strcpy(current_codeline->comment,"; GET_PREFIX");
          else if((current_codeline->data_value & 0x00FF) == 0x0B)
            strcpy(current_codeline->comment,"; CLEAR_BACKUP_BIT");
          else if((current_codeline->data_value & 0x00FF) == 0x0C)
            strcpy(current_codeline->comment,"; SET_SYS_PREFS");
          else if((current_codeline->data_value & 0x00FF) == 0x0D)
            strcpy(current_codeline->comment,"; NULL");
          else if((current_codeline->data_value & 0x00FF) == 0x0E)
            strcpy(current_codeline->comment,"; EXPAND_PATH");
          else if((current_codeline->data_value & 0x00FF) == 0x0F)
            strcpy(current_codeline->comment,"; GET_SYS_PREF");
          else if((current_codeline->data_value & 0x00FF) == 0x10)
            strcpy(current_codeline->comment,"; OPEN");
          else if((current_codeline->data_value & 0x00FF) == 0x11)
            strcpy(current_codeline->comment,"; NEWLINE");
          else if((current_codeline->data_value & 0x00FF) == 0x12)
            strcpy(current_codeline->comment,"; READ");
          else if((current_codeline->data_value & 0x00FF) == 0x13)
            strcpy(current_codeline->comment,"; WRITE");
          else if((current_codeline->data_value & 0x00FF) == 0x14)
            strcpy(current_codeline->comment,"; CLOSE");
          else if((current_codeline->data_value & 0x00FF) == 0x15)
            strcpy(current_codeline->comment,"; FLUSH");
          else if((current_codeline->data_value & 0x00FF) == 0x16)
            strcpy(current_codeline->comment,"; SET_MARK");
          else if((current_codeline->data_value & 0x00FF) == 0x17)
            strcpy(current_codeline->comment,"; GET_MARK");
          else if((current_codeline->data_value & 0x00FF) == 0x18)
            strcpy(current_codeline->comment,"; SET_EOF");
          else if((current_codeline->data_value & 0x00FF) == 0x19)
            strcpy(current_codeline->comment,"; GET_EOF");
          else if((current_codeline->data_value & 0x00FF) == 0x1A)
            strcpy(current_codeline->comment,"; SET_LEVEL");
          else if((current_codeline->data_value & 0x00FF) == 0x1B)
            strcpy(current_codeline->comment,"; GET_LEVEL");
          else if((current_codeline->data_value & 0x00FF) == 0x1C)
            strcpy(current_codeline->comment,"; GET_DIR_ENTRY");
          else if((current_codeline->data_value & 0x00FF) == 0x1D)
            strcpy(current_codeline->comment,"; BEGIN_SESSION");
          else if((current_codeline->data_value & 0x00FF) == 0x1E)
            strcpy(current_codeline->comment,"; END_SESSION");
          else if((current_codeline->data_value & 0x00FF) == 0x1F)
            strcpy(current_codeline->comment,"; SESSION_STATUS");
          else if((current_codeline->data_value & 0x00FF) == 0x20)
            strcpy(current_codeline->comment,"; GET_DEV_NUMBER");
          else if((current_codeline->data_value & 0x00FF) == 0x21)
            strcpy(current_codeline->comment,"; GET_LAST_DEV");
          else if((current_codeline->data_value & 0x00FF) == 0x22)
            strcpy(current_codeline->comment,"; READ_BLOCK");
          else if((current_codeline->data_value & 0x00FF) == 0x23)
            strcpy(current_codeline->comment,"; WRITE_BLOCK");
          else if((current_codeline->data_value & 0x00FF) == 0x24)
            strcpy(current_codeline->comment,"; FORMAT");
          else if((current_codeline->data_value & 0x00FF) == 0x25)
            strcpy(current_codeline->comment,"; ERASE_DISK");
          else if((current_codeline->data_value & 0x00FF) == 0x26)
            strcpy(current_codeline->comment,"; RESET_CACHE");
          else if((current_codeline->data_value & 0x00FF) == 0x27)
            strcpy(current_codeline->comment,"; GET_NAME");
          else if((current_codeline->data_value & 0x00FF) == 0x28)
            strcpy(current_codeline->comment,"; GET_BOOT_VOL");
          else if((current_codeline->data_value & 0x00FF) == 0x29)
            strcpy(current_codeline->comment,"; QUIT");
          else if((current_codeline->data_value & 0x00FF) == 0x2A)
            strcpy(current_codeline->comment,"; GET_VERSION");
          else if((current_codeline->data_value & 0x00FF) == 0x2B)
            strcpy(current_codeline->comment,"; GET_FST_INFO");
          else if((current_codeline->data_value & 0x00FF) == 0x2C)
            strcpy(current_codeline->comment,"; D_INFO");
          else if((current_codeline->data_value & 0x00FF) == 0x2D)
            strcpy(current_codeline->comment,"; D_STATUS");
          else if((current_codeline->data_value & 0x00FF) == 0x2E)
            strcpy(current_codeline->comment,"; D_CONTROL");
          else if((current_codeline->data_value & 0x00FF) == 0x2F)
            strcpy(current_codeline->comment,"; D_READ");
          else if((current_codeline->data_value & 0x00FF) == 0x30)
            strcpy(current_codeline->comment,"; D_WRITE");
          else if((current_codeline->data_value & 0x00FF) == 0x31)
            strcpy(current_codeline->comment,"; ALLOC_INTERRUPT");            
          else if((current_codeline->data_value & 0x00FF) == 0x32)
            strcpy(current_codeline->comment,"; DEALLOC_INTERRUPT");
          else if((current_codeline->data_value & 0x00FF) == 0x33)
            strcpy(current_codeline->comment,"; FST_SPECIFIC");
                                                                                                                                    
          /* Attache la structure  la liste */
          current_codeline->previous = last_codeline;
          if(first_codeline == NULL)
            first_codeline = current_codeline;
          else
            last_codeline->next = current_codeline;
          last_codeline = current_codeline;
          nb_codeline++;

          /* MAJ valeurs */
          i += current_codeline->data_size;
          current_address += current_codeline->data_size;

          /** Fin ? **/
          if((nb_line_max > 0) && (nb_codeline == nb_line_max))
            break;
          if((nb_line_max == 0) && (current_address > address_max))
            break;

          /** Adresse des paramtres : 4 Bytes **/
          current_codeline = (struct code_line *) calloc(1,sizeof(struct code_line));
          if(current_codeline == NULL)
            break;
          current_codeline->address = current_address;
          current_codeline->m = current_register_m;
          current_codeline->x = current_register_x;
          current_codeline->data[0] = data[i+0];
          current_codeline->data[1] = data[i+1];
          current_codeline->data[2] = data[i+2];
          current_codeline->data[3] = data[i+3];          
          current_codeline->data_size = 4;
          current_codeline->data_value = data[i+0] + 256*data[i+1] + 65536*data[i+2] + 16777216*data[i+3];
          strcpy(current_codeline->opcode,"ADRL");
          sprintf(current_codeline->operand,"$%08X",current_codeline->data_value);

          /* Attache la structure  la liste */
          current_codeline->previous = last_codeline;
          if(first_codeline == NULL)
            first_codeline = current_codeline;
          else
            last_codeline->next = current_codeline;
          last_codeline = current_codeline;
          nb_codeline++;

          /* MAJ valeurs */
          i += current_codeline->data_size;
          current_address += current_codeline->data_size;

          /** Fin ? **/
          if((nb_line_max > 0) && (nb_codeline == nb_line_max))
            break;
          if((nb_line_max == 0) && (current_address > address_max))
            break;
        }
    }

  /** Calcul la qualit de ce qui a t dsassembl **/
  quality_coef = ComputeDesassemblyQuality(first_codeline);

  /* Renvoi la liste */
  *register_mx_io = ((current_register_m << 1) | current_register_x);
  *quality_coef_rtn = quality_coef;
  return(first_codeline);
}


/***********************************************************/
/*  DisassembleOneLine() :  Dsassemble une ligne de Code. */
/***********************************************************/
struct code_line *DisassembleOneLine(int data_size, unsigned char *data, int pc_address, int data_bank, int processor, int register_m, int register_x, WORD call_num, struct code_line *last_codeline)
{
  int i, j, address, delta, delta_address, line_data_size;
  unsigned char opcode;
  DWORD data_address;
  char buffer_name[1024];
  char buffer_type[1024];
  char buffer_application[1024];
  struct code_line *previous_codeline;
  struct code_line *current_codeline;
  struct symbol *current_line_symbol;
  struct symbol *current_operand_symbol;
  struct reloc *current_reloc;

  /* Init */
  current_line_symbol = NULL;
  current_operand_symbol = NULL;

  /* Allocation mmoire */
  current_codeline = (struct code_line *) calloc(1,sizeof(struct code_line));
  if(current_codeline == NULL)
    return(NULL);

  /* Remplissage */
  current_codeline->address = pc_address;
  current_codeline->m = register_m;
  current_codeline->x = register_x;
  current_codeline->data[0] = data[0];

  /** Label : On va rechercher un Symbol  l'adresse de cette ligne **/
  current_line_symbol = GetSymbolByAddress(pc_address);
  if(current_line_symbol)
    {
      /* Adresse Extacte */
      strcpy(current_codeline->label,current_line_symbol->name);
      current_codeline->line_symbol = 1;
      delta_address = 0;
    }
  else
    {
      /* Recherche un Symbol  une adresse prcdante */
      current_line_symbol = GetSymbolByPreviousAddress(pc_address);
      if(current_line_symbol)
        {
          /* Adresse Partielle */
          current_codeline->line_symbol = 1;
          delta_address = pc_address - current_line_symbol->symbol_relocated_address;
        }
    }

  /* On va rechercher l'adresse de la ligne dans la Rom / Toolbox / GS/OS / Vecteur... */
  if(strlen(current_codeline->label) == 0)
    {
      GetNameFromAddress(pc_address,call_num,buffer_name,buffer_type,buffer_application);
      strcpy(current_codeline->label,buffer_name);
    }

  /*** Data ***/
  if(current_line_symbol != NULL)
    if(!stricmp(current_line_symbol->type,"Data"))
      {
        /* Init */
        current_codeline->address_mode = AM_IMPLICIT;
        line_data_size = current_line_symbol->size - delta_address;
        current_codeline->data_size = (line_data_size > 4) ? 4 : line_data_size;
        for(i=0; i<current_codeline->data_size; i++)
          current_codeline->data[i] = data[i];
        current_codeline->data_symbol = 1;

        /** ADR (3 bytes) **/
        if(!stricmp(current_line_symbol->data_type,"ADR"))
          {
            strcpy(current_codeline->opcode,"ADR");
            if((delta_address % 3) == 0 && current_codeline->data_size >= 3)
              {
                current_codeline->data_size = 3;
                data_address = data[0] + (data[1]*256) + (data[2]*65536);
                sprintf(current_codeline->operand,"$%06X",data_address);
                /* Recherche un Symbol */
                current_line_symbol = GetSymbolByAddress(data_address);
                if(current_line_symbol)
                  strcpy(current_codeline->operand,current_line_symbol->name);
              }
          }
        /** ADRL (4 bytes) **/
        else if(!stricmp(current_line_symbol->data_type,"ADRL"))
          {
            strcpy(current_codeline->opcode,"ADRL");
            if((delta_address % 4) == 0 && current_codeline->data_size == 4)
              {
                data_address = data[0] + (data[1]*256) + (data[2]*65536);
                sprintf(current_codeline->operand,"$%08X",data_address);
                /* Recherche un Symbol */
                current_line_symbol = GetSymbolByAddress(data_address);
                if(current_line_symbol)
                  strcpy(current_codeline->operand,current_line_symbol->name);
              }
          }
        /** ASC **/
        else if(!stricmp(current_line_symbol->data_type,"ASC"))
          {
            strcpy(current_codeline->opcode,"ASC");
            for(i=0; i<current_codeline->data_size; i++)
              {
                if(data[i] < 0x20)
                  {
                    if(i == 0)
                      sprintf(&current_codeline->operand[strlen(current_codeline->operand)],"%02X,",data[i]);
                    else if(current_codeline->operand[strlen(current_codeline->operand)-1] == ',')
                      sprintf(&current_codeline->operand[strlen(current_codeline->operand)],"%02X,",data[i]);
                    else
                      sprintf(&current_codeline->operand[strlen(current_codeline->operand)],",%02X,",data[i]);
                  }
                else
                  {
                    if(i == 0)
                      sprintf(&current_codeline->operand[strlen(current_codeline->operand)],"\"%c\"",data[i]&0x7F);
                    else if(current_codeline->operand[strlen(current_codeline->operand)-1] == '"')
                      sprintf(&current_codeline->operand[strlen(current_codeline->operand)-1],"%c\"",data[i]&0x7F);
                    else
                      sprintf(&current_codeline->operand[strlen(current_codeline->operand)],"\"%c\"",data[i]&0x7F);
                  }
              }
            if(current_codeline->operand[strlen(current_codeline->operand)-1] == ',')
              current_codeline->operand[strlen(current_codeline->operand)-1] = '\0';
          }
        /** DA **/
        else if(!stricmp(current_line_symbol->data_type,"DA"))
          {
            strcpy(current_codeline->opcode,"DA");
            if((delta_address % 2) == 0)
              {
                current_codeline->data_size = 2;
                sprintf(current_codeline->operand,"%02X%02X",data[1],data[0]);
                /* Recherche un Symbol */
                data_address = data[0] + (data[1]*256) + (pc_address & 0x00FF0000);
                current_line_symbol = GetSymbolByAddress(data_address);
                if(current_line_symbol)
                  strcpy(current_codeline->operand,current_line_symbol->name);
              }
            else
              {
                current_codeline->data_size = 1;
                sprintf(current_codeline->operand,"%02X",data[0]);
              }
          }
        /** DS / HEX **/
        else if(!stricmp(current_line_symbol->data_type,"DS") || !stricmp(current_line_symbol->data_type,"HEX"))
          {
            strcpy(current_codeline->opcode,"HEX");
            for(i=0; i<current_codeline->data_size; i++)
              sprintf(&current_codeline->operand[strlen(current_codeline->operand)],"%s%02X",(i==0)?"":",",data[i]);            
          }
        /** DW **/
        else if(!stricmp(current_line_symbol->data_type,"DW"))
          {
            strcpy(current_codeline->opcode,"DW");
            if((delta_address % 2) == 0)
              {
                current_codeline->data_size = 2;
                sprintf(current_codeline->operand,"%02X%02X",data[1],data[0]);
              }
            else
              {
                current_codeline->data_size = 1;
                sprintf(current_codeline->operand,"%02X",data[0]);
              }
          }
        /** STR **/
        else if(!stricmp(current_line_symbol->data_type,"STR"))
          {
            strcpy(current_codeline->opcode,"STR");
            if(delta_address == 0)
              {
                sprintf(current_codeline->operand,"%02X,",data[0]);
                j = 1;
              }
            else
              j = 0;
            for(i=j; i<current_codeline->data_size; i++)
              {
                if(data[i] < 0x20)
                  {
                    if(i == j)
                      sprintf(&current_codeline->operand[strlen(current_codeline->operand)],"%02X,",data[i]);
                    else if(current_codeline->operand[strlen(current_codeline->operand)-1] == ',')
                      sprintf(&current_codeline->operand[strlen(current_codeline->operand)],"%02X,",data[i]);
                    else
                      sprintf(&current_codeline->operand[strlen(current_codeline->operand)],",%02X,",data[i]);
                  }
                else
                  {
                    if(i == j)
                      sprintf(&current_codeline->operand[strlen(current_codeline->operand)],"\"%c\"",data[i]&0x7F);
                    else if(current_codeline->operand[strlen(current_codeline->operand)-1] == '"')
                      sprintf(&current_codeline->operand[strlen(current_codeline->operand)-1],"%c\"",data[i]&0x7F);
                    else
                      sprintf(&current_codeline->operand[strlen(current_codeline->operand)],"\"%c\"",data[i]&0x7F);
                  }
              }
            if(current_codeline->operand[strlen(current_codeline->operand)-1] == ',')
              current_codeline->operand[strlen(current_codeline->operand)-1] = '\0';
          }
        /** STRL **/
        else if(!stricmp(current_line_symbol->data_type,"STRL"))
          {
            strcpy(current_codeline->opcode,"STRL");
            if(delta_address == 0)
              {
                sprintf(current_codeline->operand,"%02X%02X,",data[1],data[0]);
                j = 2;
              }
            else if(delta_address == 1)
              {
                sprintf(current_codeline->operand,"%02X,",data[0]);
                j = 1;
              }
            else
              j = 0;
            for(i=j; i<current_codeline->data_size; i++)
              {
                if(data[i] < 0x20)
                  {
                    if(i == j)
                      sprintf(&current_codeline->operand[strlen(current_codeline->operand)],"%02X,",data[i]);
                    else if(current_codeline->operand[strlen(current_codeline->operand)-1] == ',')
                      sprintf(&current_codeline->operand[strlen(current_codeline->operand)],"%02X,",data[i]);
                    else
                      sprintf(&current_codeline->operand[strlen(current_codeline->operand)],",%02X,",data[i]);
                  }
                else
                  {
                    if(i == j)
                      sprintf(&current_codeline->operand[strlen(current_codeline->operand)],"\"%c\"",data[i]&0x7F);
                    else if(current_codeline->operand[strlen(current_codeline->operand)-1] == '"')
                      sprintf(&current_codeline->operand[strlen(current_codeline->operand)-1],"%c\"",data[i]&0x7F);
                    else
                      sprintf(&current_codeline->operand[strlen(current_codeline->operand)],"\"%c\"",data[i]&0x7F);
                  }
              }
            if(current_codeline->operand[strlen(current_codeline->operand)-1] == ',')
              current_codeline->operand[strlen(current_codeline->operand)-1] = '\0';
          }

        /* Renvoie la structure */
        return(current_codeline);
      }

  /** Opcode **/
  opcode = data[0];
  switch(opcode)
    {
      /** BRK data_8 (65816 65c02 6502) **/
      case 0x00 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_IMMEDIATE_8;
        strcpy(current_codeline->opcode,"BRK");      
        break;

      /** ORA (dp,x) (65816 65c02 6502) **/
      case 0x01 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDEXED_X_INDIRECT;
        strcpy(current_codeline->opcode,"ORA");
        break;

      /** COP data_8 (65816) **/
      case 0x02 :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_IMMEDIATE_8;
        strcpy(current_codeline->opcode,"COP");        
        break;

      /** ORA sr,s (65816) **/
      case 0x03 :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_STACK_RELATIVE;
        strcpy(current_codeline->opcode,"ORA");      
        break;

      /** TSB dp (65816 65c02) **/
      case 0x04 :
        if(processor == DISASSEMBLER_PROC_6502)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE;
        strcpy(current_codeline->opcode,"TSB");
        break;

      /** ORA dp (65816 65c02 6502) **/
      case 0x05 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE;
        strcpy(current_codeline->opcode,"ORA");
        break;

      /** ASL dp (65816 65c02 6502) **/
      case 0x06 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE;
        strcpy(current_codeline->opcode,"ASL");
        break;

      /** ORA [dp] (65816) **/
      case 0x07 :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDIRECT_LONG;
        strcpy(current_codeline->opcode,"ORA");
        break;

      /** PHP (65816 65c02 6502) **/
      case 0x08 :
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"PHP");
        break;

      /** ORA #const (65816 65c02 6502) **/
      case 0x09 :
        if(current_codeline->m == 1)
          {
            /* 8 Bit */
            if(data_size < 2)
              break;
            current_codeline->data_size = 2;
            current_codeline->data_value = data[1];
            current_codeline->data[1] = data[1];
            current_codeline->address_mode = AM_IMMEDIATE_8;
          }
        else
          {
            /* 16 Bit */
            if(data_size < 3)
              break;
            current_codeline->data_size = 3;
            current_codeline->data_value = data[1] + 256*data[2];
            current_codeline->data[1] = data[1];
            current_codeline->data[2] = data[2];
            current_codeline->address_mode = AM_IMMEDIATE_16;
          }
        strcpy(current_codeline->opcode,"ORA");
        break;

      /** ASL (65816 65c02 6502) **/
      case 0x0A :
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"ASL");
        break;

      /** PHD (65816) **/
      case 0x0B :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"PHD");
        break;

      /** TSB addr2 (65816 65c02) **/
      case 0x0C :
        if(processor == DISASSEMBLER_PROC_6502)
          break;
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE;
        strcpy(current_codeline->opcode,"TSB");
        break;

      /** ORA addr2 (65816 65c02 6502) **/
      case 0x0D :
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE;
        strcpy(current_codeline->opcode,"ORA");
        break;

      /** ASL addr2 (65816 65c02 6502) **/
      case 0x0E :
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE;
        strcpy(current_codeline->opcode,"ASL");
        break;

      /** ORA addr3 (65816) **/
      case 0x0F :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 4)
          break;
        current_codeline->data_size = 4;
        current_codeline->data_value = data[1] + 256*data[2] + 65536*data[3];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->data[3] = data[3];
        current_codeline->operand_address = current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE_LONG;
        strcpy(current_codeline->opcode,"ORAL");
        break;

      /** BPL (65816 65c02 6502) **/
      case 0x10 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_PC_RELATIVE;
        strcpy(current_codeline->opcode,"BPL");
        break;

      /** ORA (dp),y (65816 65c02 6502) **/
      case 0x11 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDIRECT_INDEXED_Y;
        strcpy(current_codeline->opcode,"ORA");
        break;

      /** ORA (dp) (65816 65c02) **/
      case 0x12 :
        if(processor == DISASSEMBLER_PROC_6502)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDIRECT;
        strcpy(current_codeline->opcode,"ORA");
        break;

      /** ORA (sr,s),y (65816) **/
      case 0x13 :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_STACK_RELATIVE_INDIRECT_INDEXED_Y;
        strcpy(current_codeline->opcode,"ORA");
        break;

      /** TRB dp (65816 65c02) **/
      case 0x14 :
        if(processor == DISASSEMBLER_PROC_6502)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE;
        strcpy(current_codeline->opcode,"TRB");
        break;

      /** ORA dp,x (65816 65c02 6502) **/
      case 0x15 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDEXED_X;
        strcpy(current_codeline->opcode,"ORA");      
        break;

      /** ASL dp,x (65816 65c02 6502) **/
      case 0x16 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDEXED_X;
        strcpy(current_codeline->opcode,"ASL");
        break;

      /** ORA [dp],y (65816) **/
      case 0x17 :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDIRECT_LONG_INDEXED_Y;
        strcpy(current_codeline->opcode,"ORA");
        break;

      /** CLC (65816 65c02 6502) **/
      case 0x18 :
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"CLC");
        break;

      /** ORA addr2,y (65816 65c02 6502) **/
      case 0x19 :
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE_INDEXED_Y;
        strcpy(current_codeline->opcode,"ORA");
        break;

      /** INC (65816 65c02) **/
      case 0x1A :
        if(processor == DISASSEMBLER_PROC_6502)
          break;
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"INC");
        break;

      /** TCS (65816) **/
      case 0x1B :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"TCS");
        break;

      /** TRB addr2 (65816 65c02) **/
      case 0x1C :
        if(processor == DISASSEMBLER_PROC_6502)
          break;
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE;
        strcpy(current_codeline->opcode,"TRB");
        break;

      /** ORA addr2,X (65816 65c02 6502) **/
      case 0x1D :
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE_INDEXED_X;
        strcpy(current_codeline->opcode,"ORA");
        break;

      /** ASL addr2,X (65816 65c02 6502) **/
      case 0x1E :
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE_INDEXED_X;
        strcpy(current_codeline->opcode,"ASL");
        break;

      /** ORA addr3,X (65816) **/
      case 0x1F :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 4)
          break;
        current_codeline->data_size = 4;
        current_codeline->data_value = data[1] + 256*data[2] + 65536*data[3];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->data[3] = data[3];
        current_codeline->operand_address = current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE_LONG_INDEXED_X;
        strcpy(current_codeline->opcode,"ORAL");
        break;

      /** JSR addr2 (65816 65c02 6502) **/
      case 0x20 :
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (pc_address & 0x00FF0000) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE;
        strcpy(current_codeline->opcode,"JSR");
        break;

      /** AND (dp,x) (65816 65c02 6502) **/
      case 0x21 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDEXED_X_INDIRECT;
        strcpy(current_codeline->opcode,"AND");
        break;

      /** JSL addr3 (65816) **/
      case 0x22 :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 4)
          break;
        current_codeline->data_size = 4;
        current_codeline->data_value = data[1] + 256*data[2] + 65536*data[3];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->data[3] = data[3];
        current_codeline->operand_address = current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE_LONG;
        strcpy(current_codeline->opcode,"JSL");
        break;

      /** AND sr,s (65816) **/
      case 0x23 :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_STACK_RELATIVE;
        strcpy(current_codeline->opcode,"AND");
        break;

      /** BIT dp (65816 65c02 6502) **/
      case 0x24 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE;
        strcpy(current_codeline->opcode,"BIT");
        break;

      /** AND dp (65816 65c02 6502) **/
      case 0x25 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE;
        strcpy(current_codeline->opcode,"AND");
        break;

      /** ROL dp (65816 65c02 6502) **/
      case 0x26 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE;
        strcpy(current_codeline->opcode,"ROL");
        break;

      /** AND [dp] (65816) **/
      case 0x27 :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDIRECT_LONG;
        strcpy(current_codeline->opcode,"AND");
        break;

      /** PLP (65816 65c02 6502) **/
      case 0x28 :
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"PLP");
        break;

      /** AND #const (65816 65c02 6502) **/
      case 0x29 :
        if(current_codeline->m == 1)
          {
            /* 8 Bit */
            if(data_size < 2)
              break;
            current_codeline->data_size = 2;
            current_codeline->data_value = data[1];
            current_codeline->data[1] = data[1];
            current_codeline->address_mode = AM_IMMEDIATE_8;
          }
        else
          {
            /* 16 Bit */
            if(data_size < 3)
              break;
            current_codeline->data_size = 3;
            current_codeline->data_value = data[1] + 256*data[2];
            current_codeline->data[1] = data[1];
            current_codeline->data[2] = data[2];
            current_codeline->address_mode = AM_IMMEDIATE_16;
          }
        strcpy(current_codeline->opcode,"AND");
        break;

      /** ROL (65816 65c02 6502) **/
      case 0x2A :
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"ROL");
        break;

      /** PLD (65816) **/
      case 0x2B :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"PLD");
        break;

      /** BIT addr2 (65816 65c02 6502) **/
      case 0x2C :
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE;
        strcpy(current_codeline->opcode,"BIT");
        break;

      /** AND addr2 (65816 65c02 6502) **/
      case 0x2D :
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE;
        strcpy(current_codeline->opcode,"AND");
        break;

      /** ROL addr2 (65816 65c02 6502) **/
      case 0x2E :
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE;
        strcpy(current_codeline->opcode,"ROL");
        break;

      /** AND addr3 (65816) **/
      case 0x2F :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 4)
          break;
        current_codeline->data_size = 4;
        current_codeline->data_value = data[1] + 256*data[2] + 65536*data[3];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->data[3] = data[3];
        current_codeline->operand_address = current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE_LONG;
        strcpy(current_codeline->opcode,"AND");
        break;

      /** BMI (65816 65c02 6502) **/
      case 0x30 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_PC_RELATIVE;
        strcpy(current_codeline->opcode,"BMI");
        break;

      /** AND (dp),y (65816 65c02 6502) **/
      case 0x31 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDIRECT_INDEXED_Y;
        strcpy(current_codeline->opcode,"AND");
        break;

      /** AND (dp) (65816 65c02) **/
      case 0x32 :
        if(processor == DISASSEMBLER_PROC_6502)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDIRECT;
        strcpy(current_codeline->opcode,"AND");
        break;

      /** AND (sr,s),y (65816) **/
      case 0x33 :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_STACK_RELATIVE_INDIRECT_INDEXED_Y;
        strcpy(current_codeline->opcode,"AND");      
        break;

      /** BIT dp,x (65816 65c02) **/
      case 0x34 :
        if(processor == DISASSEMBLER_PROC_6502)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDEXED_X;
        strcpy(current_codeline->opcode,"BIT");
        break;

      /** AND dp,x (65816 65c02 6502) **/
      case 0x35 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDEXED_X;
        strcpy(current_codeline->opcode,"AND");
        break;

      /** ROL dp,x (65816 65c02 6502) **/
      case 0x36 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDEXED_X;
        strcpy(current_codeline->opcode,"ROL");      
        break;

      /** AND [dp],y (65816) **/
      case 0x37 :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDIRECT_LONG_INDEXED_Y;
        strcpy(current_codeline->opcode,"AND");
        break;

      /** SEC (65816 65c02 6502) **/
      case 0x38 :
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"SEC");
        break;

      /** AND addr2,y (65816 65c02 6502) **/
      case 0x39 :
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE_INDEXED_Y;
        strcpy(current_codeline->opcode,"AND");
        break;

      /** DEC (65816 65c02) **/
      case 0x3A :
        if(processor == DISASSEMBLER_PROC_6502)
          break;
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"DEC");
        break;

      /** TSC (65816) **/
      case 0x3B :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"TSC");
        break;

      /** BIT addr2,X (65816 65c02) **/
      case 0x3C :
        if(processor == DISASSEMBLER_PROC_6502)
          break;
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE_INDEXED_X;
        strcpy(current_codeline->opcode,"BIT");
        break;

      /** AND addr2,X (65816 65c02 6502) **/
      case 0x3D :
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE_INDEXED_X;
        strcpy(current_codeline->opcode,"AND");
        break;

      /** ROL addr2,X (65816 65c02 6502) **/
      case 0x3E :
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE_INDEXED_X;
        strcpy(current_codeline->opcode,"ROL");
        break;

      /** AND addr3,X (65816) **/
      case 0x3F :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 4)
          break;
        current_codeline->data_size = 4;
        current_codeline->data_value = data[1] + 256*data[2] + 65536*data[3];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->data[3] = data[3];
        current_codeline->operand_address = current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE_LONG_INDEXED_X;
        strcpy(current_codeline->opcode,"ANDL");
        break;

      /** RTI (65816 65c02 6502) **/
      case 0x40 :
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"RTI");
        break;

      /** EOR (dp,x) (65816 65c02 6502) **/
      case 0x41 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDEXED_X_INDIRECT;
        strcpy(current_codeline->opcode,"EOR");
        break;

      /** WDM const_8 (65816) **/
      case 0x42 :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_IMMEDIATE_8;
        strcpy(current_codeline->opcode,"WDM");
        break;

      /** EOR sr,s (65816) **/
      case 0x43 :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_STACK_RELATIVE;
        strcpy(current_codeline->opcode,"EOR");      
        break;

      /** MVP Src,Dst (65816) **/
      case 0x44 :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1];
        current_codeline->aux_value = data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->address_mode = AM_BLOCK_MOVE;
        strcpy(current_codeline->opcode,"MVP");      
        break;

      /** EOR dp (65816 65c02 6502) **/
      case 0x45 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE;
        strcpy(current_codeline->opcode,"EOR");
        break;

      /** LSR dp (65816 65c02 6502) **/
      case 0x46 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE;
        strcpy(current_codeline->opcode,"LSR");
        break;

      /** EOR [dp] (65816) **/
      case 0x47 :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDIRECT_LONG;
        strcpy(current_codeline->opcode,"EOR");
        break;

      /** PHA (65816 65c02 6502) **/
      case 0x48 :
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"PHA");
        break;

      /** EOR #const (65816 65c02 6502) **/
      case 0x49 :
        if(current_codeline->m == 1)
          {
            /* 8 Bit */
            if(data_size < 2)
              break;
            current_codeline->data_size = 2;
            current_codeline->data_value = data[1];
            current_codeline->data[1] = data[1];
            current_codeline->address_mode = AM_IMMEDIATE_8;
          }
        else
          {
            /* 16 Bit */
            if(data_size < 3)
              break;
            current_codeline->data_size = 3;
            current_codeline->data_value = data[1] + 256*data[2];
            current_codeline->data[1] = data[1];
            current_codeline->data[2] = data[2];
            current_codeline->address_mode = AM_IMMEDIATE_16;
          }
        strcpy(current_codeline->opcode,"EOR");
        break;

      /** LSR (65816 65c02 6502) **/
      case 0x4A :
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"LSR");
        break;

      /** PHK (65816) **/
      case 0x4B :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"PHK");
        break;

      /** JMP addr2 (65816 65c02 6502) **/
      case 0x4C :
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (pc_address & 0x00FF0000) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE;
        strcpy(current_codeline->opcode,"JMP");
        break;

      /** EOR addr2 (65816 65c02 6502) **/
      case 0x4D :
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE;
        strcpy(current_codeline->opcode,"EOR");
        break;

      /** LSR addr2 (65816 65c02 6502) **/
      case 0x4E :
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE;
        strcpy(current_codeline->opcode,"LSR");
        break;

      /** EOR addr3 (65816) **/
      case 0x4F :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 4)
          break;
        current_codeline->data_size = 4;
        current_codeline->data_value = data[1] + 256*data[2] + 65536*data[3];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->data[3] = data[3];
        current_codeline->operand_address = current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE_LONG;
        strcpy(current_codeline->opcode,"EORL");
        break;

      /** BVC (65816 65c02 6502) **/
      case 0x50 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_PC_RELATIVE;
        strcpy(current_codeline->opcode,"BVC");
        break;

      /** EOR (dp),y (65816 65c02 6502) **/
      case 0x51 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDIRECT_INDEXED_Y;
        strcpy(current_codeline->opcode,"EOR");
        break;

      /** EOR (dp) (65816 65c02) **/
      case 0x52 :
        if(processor == DISASSEMBLER_PROC_6502)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDIRECT;
        strcpy(current_codeline->opcode,"EOR");
        break;

      /** EOR (sr,s),y (65816) **/
      case 0x53 :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_STACK_RELATIVE_INDIRECT_INDEXED_Y;
        strcpy(current_codeline->opcode,"EOR");      
        break;

      /** MVN Src,Dst (65816) **/
      case 0x54 :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1];
        current_codeline->aux_value = data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->address_mode = AM_BLOCK_MOVE;
        strcpy(current_codeline->opcode,"MVN");        
        break;

      /** EOR dp,x (65816 65c02 6502) **/
      case 0x55 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDEXED_X;
        strcpy(current_codeline->opcode,"EOR");      
        break;

      /** LSR dp,x (65816 65c02 6502) **/
      case 0x56 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDEXED_X;
        strcpy(current_codeline->opcode,"LSR");       
        break;

      /** EOR [dp],y (65816) **/
      case 0x57 :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDIRECT_LONG_INDEXED_Y;
        strcpy(current_codeline->opcode,"EOR");
        break;

      /** CLI (65816 65c02 6502) **/
      case 0x58 :
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"CLI");
        break;

      /** EOR addr2,y (65816 65c02 6502) **/
      case 0x59 :
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE_INDEXED_Y;
        strcpy(current_codeline->opcode,"EOR");
        break;

      /** PHY (65816 65c02) **/
      case 0x5A :
        if(processor == DISASSEMBLER_PROC_6502)
          break;
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"PHY");
        break;

      /** TCD (65816) **/
      case 0x5B :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"TCD");
        break;

      /** JMPL addr3 (65816) **/
      case 0x5C :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 4)
          break;
        current_codeline->data_size = 4;
        current_codeline->data_value = data[1] + 256*data[2] + 65536*data[3];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->data[3] = data[3];
        current_codeline->operand_address = current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE_LONG;
        strcpy(current_codeline->opcode,"JMPL");
        break;

      /** EOR addr2,X (65816 65c02 6502) **/
      case 0x5D :
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE_INDEXED_X;
        strcpy(current_codeline->opcode,"EOR");
        break;

      /** LSR addr2,X (65816 65c02 6502) **/
      case 0x5E :
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE_INDEXED_X;
        strcpy(current_codeline->opcode,"LSR");
        break;

      /** EOR addr3,X (65816) **/
      case 0x5F :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 4)
          break;
        current_codeline->data_size = 4;
        current_codeline->data_value = data[1] + 256*data[2] + 65536*data[3];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->data[3] = data[3];
        current_codeline->operand_address = current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE_LONG_INDEXED_X;
        strcpy(current_codeline->opcode,"EORL");
        break;

      /** RTS (65816 65c02 6502) **/
      case 0x60 :
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"RTS");
        break;

      /** ADC (dp,x) (65816 65c02 6502) **/
      case 0x61 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDEXED_X_INDIRECT;
        strcpy(current_codeline->opcode,"ADC");
        break;

      /** PER label => delta_16 (65816) **/
      case 0x62 :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->address_mode = AM_STACK_PC_RELATIVE_LONG;
        strcpy(current_codeline->opcode,"PER");      
        break;

      /** ADC sr,s (65816) **/
      case 0x63 :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_STACK_RELATIVE;
        strcpy(current_codeline->opcode,"ADC");      
        break;

      /** STZ dp (65816 65c02) **/
      case 0x64 :
        if(processor == DISASSEMBLER_PROC_6502)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE;
        strcpy(current_codeline->opcode,"STZ");
        break;

      /** ADC dp (65816 65c02 6502) **/
      case 0x65 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE;
        strcpy(current_codeline->opcode,"ADC");
        break;

      /** ROR dp (65816 65c02 6502) **/
      case 0x66 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE;
        strcpy(current_codeline->opcode,"ROR");
        break;

      /** ADC [dp] (65816) **/
      case 0x67 :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDIRECT_LONG;
        strcpy(current_codeline->opcode,"ADC");
        break;

      /** PLA (65816 65c02 6502) **/
      case 0x68 :
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"PLA");
        break;

      /** ADC #const (65816 65c02 6502) **/
      case 0x69 :
        if(current_codeline->m == 1)
          {
            /* 8 Bit */
            if(data_size < 2)
              break;
            current_codeline->data_size = 2;
            current_codeline->data_value = data[1];
            current_codeline->data[1] = data[1];
            current_codeline->address_mode = AM_IMMEDIATE_8;
          }
        else
          {
            /* 16 Bit */
            if(data_size < 3)
              break;
            current_codeline->data_size = 3;
            current_codeline->data_value = data[1] + 256*data[2];
            current_codeline->data[1] = data[1];
            current_codeline->data[2] = data[2];
            current_codeline->address_mode = AM_IMMEDIATE_16;
          }
        strcpy(current_codeline->opcode,"ADC");
        break;

      /** ROR (65816 65c02 6502) **/
      case 0x6A :
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"ROR");
        break;

      /** RTL (65816) **/
      case 0x6B :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"RTL");
        break;

      /** JMP (addr2) (65816 65c02 6502) **/
      case 0x6C :
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (pc_address & 0x00FF0000) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE_INDIRECT;
        strcpy(current_codeline->opcode,"JMP");      
        break;

      /** ADC addr2 (65816 65c02 6502) **/
      case 0x6D :
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE;
        strcpy(current_codeline->opcode,"ADC");
        break;

      /** ROR addr2 (65816 65c02 6502) **/
      case 0x6E :
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE;
        strcpy(current_codeline->opcode,"ROR");
        break;

      /** ADC addr3 (65816) **/
      case 0x6F :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 4)
          break;
        current_codeline->data_size = 4;
        current_codeline->data_value = data[1] + 256*data[2] + 65536*data[3];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->data[3] = data[3];
        current_codeline->operand_address = current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE_LONG;
        strcpy(current_codeline->opcode,"ADCL");
        break;

      /** BVS (65816 65c02 6502) **/
      case 0x70 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_PC_RELATIVE;
        strcpy(current_codeline->opcode,"BVS");
        break;

      /** ADC (dp),Y (65816 65c02 6502) **/
      case 0x71 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDIRECT_INDEXED_Y;
        strcpy(current_codeline->opcode,"ADC");
        break;

      /** ADC (dp) (65816 65c02) **/
      case 0x72 :
        if(processor == DISASSEMBLER_PROC_6502)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDIRECT;
        strcpy(current_codeline->opcode,"ADC");
        break;

      /** ADC (sr,s),y (65816) **/
      case 0x73 :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_STACK_RELATIVE_INDIRECT_INDEXED_Y;
        strcpy(current_codeline->opcode,"ADC");      
        break;

      /** STZ dp,x (65816 65c02) **/
      case 0x74 :
        if(processor == DISASSEMBLER_PROC_6502)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDEXED_X;
        strcpy(current_codeline->opcode,"STZ");      
        break;

      /** ADC dp,X (65816 65c02 6502) **/
      case 0x75 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDEXED_X;
        strcpy(current_codeline->opcode,"ADC");
        break;

      /** ROR dp,x (65816 65c02 6502) **/
      case 0x76 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDEXED_X;
        strcpy(current_codeline->opcode,"ROR");      
        break;

      /** ADC [dp],y (65816) **/
      case 0x77 :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDIRECT_LONG_INDEXED_Y;
        strcpy(current_codeline->opcode,"ADC");
        break;

      /** SEI (65816 65c02 6502) **/
      case 0x78 :
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"SEI");
        break;

      /** ADC addr2,y (65816 65c02 6502) **/
      case 0x79 :
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE_INDEXED_Y;
        strcpy(current_codeline->opcode,"ADC");
        break;

      /** PLY (65816 65c02) **/
      case 0x7A :
        if(processor == DISASSEMBLER_PROC_6502)
          break;
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"PLY");
        break;

      /** TDC (65816) **/
      case 0x7B :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"TDC");
        break;

      /** JMP (addr2,x) (65816 65c02) **/
      case 0x7C :
        if(processor == DISASSEMBLER_PROC_6502)
          break;
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (pc_address & 0x00FF0000) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE_INDEXED_X_INDIRECT;
        strcpy(current_codeline->opcode,"JMP");      
        break;

      /** ADC addr2,X (65816 65c02 6502) **/
      case 0x7D :
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE_INDEXED_X;
        strcpy(current_codeline->opcode,"ADC");
        break;

      /** ROR addr2,X (65816 65c02 6502) **/
      case 0x7E :
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE_INDEXED_X;
        strcpy(current_codeline->opcode,"ROR");
        break;

      /** ADC addr3,X (65816) **/
      case 0x7F :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 4)
          break;
        current_codeline->data_size = 4;
        current_codeline->data_value = data[1] + 256*data[2] + 65536*data[3];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->data[3] = data[3];
        current_codeline->operand_address = current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE_LONG_INDEXED_X;
        strcpy(current_codeline->opcode,"ADCL");
        break;

      /** BRA (65816 65c02) **/
      case 0x80 :
        if(processor == DISASSEMBLER_PROC_6502)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_PC_RELATIVE;
        strcpy(current_codeline->opcode,"BRA");
        break;

      /** STA (dp,x) (65816 65c02 6502) **/
      case 0x81 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDEXED_X_INDIRECT;
        strcpy(current_codeline->opcode,"STA");
        break;

      /** BRL (65816) **/
      case 0x82 :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->address_mode = AM_PC_RELATIVE_LONG;
        strcpy(current_codeline->opcode,"BRL");      
        break;

      /** STA sr,s (65816) **/
      case 0x83 :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_STACK_RELATIVE;
        strcpy(current_codeline->opcode,"STA");      
        break;

      /** STY dp (65816 65c02 6502) **/
      case 0x84 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE;
        strcpy(current_codeline->opcode,"STY");
        break;

      /** STA dp (65816 65c02 6502) **/
      case 0x85 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE;
        strcpy(current_codeline->opcode,"STA");
        break;

      /** STX dp (65816 65c02 6502) **/
      case 0x86 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE;
        strcpy(current_codeline->opcode,"STX");
        break;

      /** STA [dp] (65816) **/
      case 0x87 :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDIRECT_LONG;
        strcpy(current_codeline->opcode,"STA");
        break;

      /** DEY (65816 65c02 6502) **/
      case 0x88 :
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"DEY");
        break;

      /** BIT #const (65816 65c02) **/
      case 0x89 :
        if(processor == DISASSEMBLER_PROC_6502)
          break;
        if(current_codeline->m == 1)
          {
            /* 8 Bit */
            if(data_size < 2)
              break;
            current_codeline->data_size = 2;
            current_codeline->data_value = data[1];
            current_codeline->data[1] = data[1];
            current_codeline->address_mode = AM_IMMEDIATE_8;
          }
        else
          {
            /* 16 Bit */
            if(data_size < 3)
              break;
            current_codeline->data_size = 3;
            current_codeline->data_value = data[1] + 256*data[2];
            current_codeline->data[1] = data[1];
            current_codeline->data[2] = data[2];
            current_codeline->address_mode = AM_IMMEDIATE_16;
          }
        strcpy(current_codeline->opcode,"BIT");
        break;

      /** TXA (65816 65c02 6502) **/
      case 0x8A :
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"TXA");
        break;

      /** PHB (65816) **/
      case 0x8B :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"PHB");
        break;

      /** STY addr2 (65816 65c02 6502) **/
      case 0x8C :
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE;
        strcpy(current_codeline->opcode,"STY");
        break;

      /** STA addr2 (65816 65c02 6502) **/
      case 0x8D :
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE;
        strcpy(current_codeline->opcode,"STA");
        break;

      /** STX addr2 (65816 65c02 6502) **/
      case 0x8E :
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE;
        strcpy(current_codeline->opcode,"STX");
        break;

      /** STA addr3 (65816) **/
      case 0x8F :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 4)
          break;
        current_codeline->data_size = 4;
        current_codeline->data_value = data[1] + 256*data[2] + 65536*data[3];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->data[3] = data[3];
        current_codeline->operand_address = current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE_LONG;
        strcpy(current_codeline->opcode,"STAL");
        break;

      /** BCC (65816 65c02 6502) **/
      case 0x90 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_PC_RELATIVE;
        strcpy(current_codeline->opcode,"BCC");
        break;

      /** STA (dp),y (65816 65c02 6502) **/
      case 0x91 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDIRECT_INDEXED_Y;
        strcpy(current_codeline->opcode,"STA");
        break;

      /** STA (dp) (65816 65c02) **/
      case 0x92 :
        if(processor == DISASSEMBLER_PROC_6502)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDIRECT;
        strcpy(current_codeline->opcode,"STA");
        break;

      /** STA (sr,s),y (65816) **/
      case 0x93 :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_STACK_RELATIVE_INDIRECT_INDEXED_Y;
        strcpy(current_codeline->opcode,"STA");      
        break;

      /** STY dp,x (65816 65c02 6502) **/
      case 0x94 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDEXED_X;
        strcpy(current_codeline->opcode,"STY");      
        break;

      /** STA dp,x (65816 65c02 6502) **/
      case 0x95 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDEXED_X;
        strcpy(current_codeline->opcode,"STA");         
        break;

      /** STX dp,y (65816 65c02 6502) **/
      case 0x96 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDEXED_Y;
        strcpy(current_codeline->opcode,"STX");        
        break;

      /** STA [dp],y (65816) **/
      case 0x97 :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDIRECT_LONG_INDEXED_Y;
        strcpy(current_codeline->opcode,"STA");
        break;

      /** TYA (65816 65c02 6502) **/
      case 0x98 :
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"TYA");
        break;

      /** STA addr2,y (65816 65c02 6502) **/
      case 0x99 :
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE_INDEXED_Y;
        strcpy(current_codeline->opcode,"STA");
        break;

      /** TXS (65816 65c02 6502) **/
      case 0x9A :
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"TXS");
        break;

      /** TXY (65816) **/
      case 0x9B :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"TXY");
        break;

      /** STZ addr2 (65816 65c02) **/
      case 0x9C :
        if(processor == DISASSEMBLER_PROC_6502)
          break;
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE;
        strcpy(current_codeline->opcode,"STZ");
        break;

      /** STA addr2,X (65816 65c02 6502) **/
      case 0x9D :
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE_INDEXED_X;
        strcpy(current_codeline->opcode,"STA");
        break;

      /** STZ addr2,X (65816 65c02) **/
      case 0x9E :
        if(processor == DISASSEMBLER_PROC_6502)
          break;
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE_INDEXED_X;
        strcpy(current_codeline->opcode,"STZ");
        break;

      /** STA addr3,X (65816) **/
      case 0x9F :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 4)
          break;
        current_codeline->data_size = 4;
        current_codeline->data_value = data[1] + 256*data[2] + 65536*data[3];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->data[3] = data[3];
        current_codeline->operand_address = current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE_LONG_INDEXED_X;
        strcpy(current_codeline->opcode,"STAL");
        break;

      /** LDY #const (65816 65c02 6502) **/
      case 0xA0 :
        if(current_codeline->x == 1)
          {
            /* 8 Bit */
            if(data_size < 2)
              break;
            current_codeline->data_size = 2;
            current_codeline->data_value = data[1];
            current_codeline->data[1] = data[1];
            current_codeline->address_mode = AM_IMMEDIATE_8;
          }
        else
          {
            /* 16 Bit */
            if(data_size < 3)
              break;
            current_codeline->data_size = 3;
            current_codeline->data_value = data[1] + 256*data[2];
            current_codeline->data[1] = data[1];
            current_codeline->data[2] = data[2];
            current_codeline->address_mode = AM_IMMEDIATE_16;
          }
        strcpy(current_codeline->opcode,"LDY");
        break;

      /** LDA (dp,x) (65816 65c02 6502) **/
      case 0xA1 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDEXED_X_INDIRECT;
        strcpy(current_codeline->opcode,"LDA");
        break;

      /** LDX #const (65816 65c02 6502) **/
      case 0xA2 :
        if(current_codeline->x == 1)
          {
            /* 8 Bit */
            if(data_size < 2)
              break;
            current_codeline->data_size = 2;
            current_codeline->data_value = data[1];
            current_codeline->data[1] = data[1];
            current_codeline->address_mode = AM_IMMEDIATE_8;
          }
        else
          {
            /* 16 Bit */
            if(data_size < 3)
              break;
            current_codeline->data_size = 3;
            current_codeline->data_value = data[1] + 256*data[2];
            current_codeline->data[1] = data[1];
            current_codeline->data[2] = data[2];
            current_codeline->address_mode = AM_IMMEDIATE_16;
          }
        strcpy(current_codeline->opcode,"LDX");
        break;

      /** LDA sr,s (65816) **/
      case 0xA3 :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_STACK_RELATIVE;
        strcpy(current_codeline->opcode,"LDA");      
        break;

      /** LDY dp (65816 65c02 6502) **/
      case 0xA4 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE;
        strcpy(current_codeline->opcode,"LDY");
        break;

      /** LDA dp (65816 65c02 6502) **/
      case 0xA5 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE;
        strcpy(current_codeline->opcode,"LDA");
        break;

      /** LDX dp (65816 65c02 6502) **/
      case 0xA6 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE;
        strcpy(current_codeline->opcode,"LDX");
        break;

      /** LDA [dp] (65816) **/
      case 0xA7 :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDIRECT_LONG;
        strcpy(current_codeline->opcode,"LDA");
        break;

      /** TAY (65816 65c02 6502) **/
      case 0xA8 :
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"TAY");
        break;

      /** LDA #const (65816 65c02 6502) **/
      case 0xA9 :
        if(current_codeline->m == 1)
          {
            /* 8 Bit */
            if(data_size < 2)
              break;
            current_codeline->data_size = 2;
            current_codeline->data_value = data[1];
            current_codeline->data[1] = data[1];
            current_codeline->address_mode = AM_IMMEDIATE_8;
          }
        else
          {
            /* 16 Bit */
            if(data_size < 3)
              break;
            current_codeline->data_size = 3;
            current_codeline->data_value = data[1] + 256*data[2];
            current_codeline->data[1] = data[1];
            current_codeline->data[2] = data[2];
            current_codeline->address_mode = AM_IMMEDIATE_16;
          }
        strcpy(current_codeline->opcode,"LDA");
        break;

      /** TAX (65816 65c02 6502) **/
      case 0xAA :
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"TAX");
        break;

      /** PLB (65816) **/
      case 0xAB :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"PLB");
        break;

      /** LDY addr2 (65816 65c02 6502) **/
      case 0xAC :
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE;
        strcpy(current_codeline->opcode,"LDY");
        break;

      /** LDA addr2 (65816 65c02 6502) **/
      case 0xAD :
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE;
        strcpy(current_codeline->opcode,"LDA");
        break;

      /** LDX addr2 (65816 65c02 6502) **/
      case 0xAE :
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE;
        strcpy(current_codeline->opcode,"LDX");
        break;

      /** LDA addr3 (65816) **/
      case 0xAF :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 4)
          break;
        current_codeline->data_size = 4;
        current_codeline->data_value = data[1] + 256*data[2] + 65536*data[3];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->data[3] = data[3];
        current_codeline->operand_address = current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE_LONG;
        strcpy(current_codeline->opcode,"LDAL");
        break;

      /** BCS (65816 65c02 6502) **/
      case 0xB0 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_PC_RELATIVE;
        strcpy(current_codeline->opcode,"BCS");
        break;

      /** LDA (dp),y (65816 65c02 6502) **/
      case 0xB1 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDIRECT_INDEXED_Y;
        strcpy(current_codeline->opcode,"LDA");
        break;

      /** LDA (dp) (65816 65c02) **/
      case 0xB2 :
        if(processor == DISASSEMBLER_PROC_6502)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDIRECT;
        strcpy(current_codeline->opcode,"LDA");
        break;

      /** LDA (sr,s),y (65816) **/
      case 0xB3 :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_STACK_RELATIVE_INDIRECT_INDEXED_Y;
        strcpy(current_codeline->opcode,"LDA");      
        break;

      /** LDY dp,x (65816 65c02 6502) **/
      case 0xB4 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDEXED_X;
        strcpy(current_codeline->opcode,"LDY");      
        break;

      /** LDA dp,x (65816 65c02 6502) **/
      case 0xB5 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDEXED_X;
        strcpy(current_codeline->opcode,"LDA");        
        break;

      /** LDX (dp),y (65816 65c02 6502) **/
      case 0xB6 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDIRECT_INDEXED_Y;
        strcpy(current_codeline->opcode,"LDX");      
        break;

      /** LDA [dp],y (65816) **/
      case 0xB7 :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDIRECT_LONG_INDEXED_Y;
        strcpy(current_codeline->opcode,"LDA");
        break;

      /** CLV (65816 65c02 6502) **/
      case 0xB8 :
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"CLV");
        break;

      /** LDA addr2,y (65816 65c02 6502) **/
      case 0xB9 :
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE_INDEXED_Y;
        strcpy(current_codeline->opcode,"LDA");
        break;

      /** TSX (65816 65c02 6502) **/
      case 0xBA :
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"TSX");
        break;

      /** TYX (65816) **/
      case 0xBB :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"TYX");
        break;

      /** LDY addr2,X (65816 65c02 6502) **/
      case 0xBC :
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE_INDEXED_X;
        strcpy(current_codeline->opcode,"LDY");
        break;

      /** LDA addr2,X (65816 65c02 6502) **/
      case 0xBD :
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE_INDEXED_X;
        strcpy(current_codeline->opcode,"LDA");
        break;

      /** LDX addr2,y (65816 65c02 6502) **/
      case 0xBE :
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE_INDEXED_Y;
        strcpy(current_codeline->opcode,"LDX");
        break;

      /** LDA addr3,X (65816) **/
      case 0xBF :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 4)
          break;
        current_codeline->data_size = 4;
        current_codeline->data_value = data[1] + 256*data[2] + 65536*data[3];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->data[3] = data[3];
        current_codeline->operand_address = current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE_LONG_INDEXED_X;
        strcpy(current_codeline->opcode,"LDAL");
        break;

      /** CPY #const (65816 65c02 6502) **/
      case 0xC0 :
        if(current_codeline->x == 1)
          {
            /* 8 Bit */
            if(data_size < 2)
              break;
            current_codeline->data_size = 2;
            current_codeline->data_value = data[1];
            current_codeline->data[1] = data[1];
            current_codeline->address_mode = AM_IMMEDIATE_8;
          }
        else
          {
            /* 16 Bit */
            if(data_size < 3)
              break;
            current_codeline->data_size = 3;
            current_codeline->data_value = data[1] + 256*data[2];
            current_codeline->data[1] = data[1];
            current_codeline->data[2] = data[2];
            current_codeline->address_mode = AM_IMMEDIATE_16;
          }
        strcpy(current_codeline->opcode,"CPY");
        break;

      /** CMP (dp,x) (65816 65c02 6502) **/
      case 0xC1 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDEXED_X_INDIRECT;
        strcpy(current_codeline->opcode,"CMP");
        break;

      /** REP const_8 (65816) **/
      case 0xC2 :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_IMMEDIATE_8;
        strcpy(current_codeline->opcode,"REP");      
        break;

      /** CMP sr,s (65816) **/
      case 0xC3 :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_STACK_RELATIVE;
        strcpy(current_codeline->opcode,"CMP");      
        break;

      /** CPY dp (65816 65c02 6502) **/
      case 0xC4 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE;
        strcpy(current_codeline->opcode,"CPY");
        break;

      /** CMP dp (65816 65c02 6502) **/
      case 0xC5 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE;
        strcpy(current_codeline->opcode,"CMP");
        break;

      /** DEC dp (65816 65c02 6502) **/
      case 0xC6 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE;
        strcpy(current_codeline->opcode,"DEC");
        break;

      /** CMP [dp] (65816) **/
      case 0xC7 :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDIRECT_LONG;
        strcpy(current_codeline->opcode,"CMP");
        break;

      /** INY (65816 65c02 6502) **/
      case 0xC8 :
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"INY");
        break;

      /** CMP #const (65816 65c02 6502) **/
      case 0xC9 :
        if(current_codeline->m == 1)
          {
            /* 8 Bit */
            if(data_size < 2)
              break;
            current_codeline->data_size = 2;
            current_codeline->data_value = data[1];
            current_codeline->data[1] = data[1];
            current_codeline->address_mode = AM_IMMEDIATE_8;
          }
        else
          {
            /* 16 Bit */
            if(data_size < 3)
              break;
            current_codeline->data_size = 3;
            current_codeline->data_value = data[1] + 256*data[2];
            current_codeline->data[1] = data[1];
            current_codeline->data[2] = data[2];
            current_codeline->address_mode = AM_IMMEDIATE_16;
          }
        strcpy(current_codeline->opcode,"CMP");
        break;

      /** DEX (65816 65c02 6502) **/
      case 0xCA :
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"DEX");
        break;

      /** WAI (65816) **/
      case 0xCB :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"WAI");
        break;

      /** CPY addr2 (65816 65c02 6502) **/
      case 0xCC :
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE;
        strcpy(current_codeline->opcode,"CPY");
        break;

      /** CMP addr2 (65816 65c02 6502) **/
      case 0xCD :
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE;
        strcpy(current_codeline->opcode,"CMP");
        break;

      /** DEC addr2 (65816 65c02 6502) **/
      case 0xCE :
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE;
        strcpy(current_codeline->opcode,"DEC");
        break;

      /** CMP addr3 (65816) **/
      case 0xCF :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 4)
          break;
        current_codeline->data_size = 4;
        current_codeline->data_value = data[1] + 256*data[2] + 65536*data[3];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->data[3] = data[3];
        current_codeline->operand_address = current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE_LONG;
        strcpy(current_codeline->opcode,"CMPL");
        break;

      /** BNE (65816 65c02 6502) **/
      case 0xD0 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_PC_RELATIVE;
        strcpy(current_codeline->opcode,"BNE");
        break;

      /** CMP (dp),y (65816 65c02 6502) **/
      case 0xD1 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDIRECT_INDEXED_Y;
        strcpy(current_codeline->opcode,"CMP");
        break;

      /** CMP (dp) (65816 65c02) **/
      case 0xD2 :
        if(processor == DISASSEMBLER_PROC_6502)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDIRECT;
        strcpy(current_codeline->opcode,"CMP");
        break;

      /** CMP (sr,s),y (65816) **/
      case 0xD3 :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_STACK_RELATIVE_INDIRECT_INDEXED_Y;
        strcpy(current_codeline->opcode,"CMP");      
        break;

      /** PEI (dp) (65816) **/
      case 0xD4 :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDIRECT;
        strcpy(current_codeline->opcode,"PEI");      
        break;

      /** CMP dp,x (65816 65c02 6502) **/
      case 0xD5 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDEXED_X;
        strcpy(current_codeline->opcode,"CMP");
        break;

      /** DEC dp,x (65816 65c02 6502) **/
      case 0xD6 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDEXED_X;
        strcpy(current_codeline->opcode,"DEC");
        break;

      /** CMP [dp],y (65816) **/
      case 0xD7 :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDIRECT_LONG_INDEXED_Y;
        strcpy(current_codeline->opcode,"CMP");
        break;

      /** CLD (65816 65c02 6502) **/
      case 0xD8 :
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"CLD");
        break;

      /** CMP addr2,y (65816 65c02 6502) **/
      case 0xD9 :
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE_INDEXED_Y;
        strcpy(current_codeline->opcode,"CMP");
        break;

      /** PHX (65816 65c02) **/
      case 0xDA :
        if(processor == DISASSEMBLER_PROC_6502)
          break;
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"PHX");
        break;

      /** STP (65816) **/
      case 0xDB :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"STP");
        break;

      /** JMP (addr2) (65816) **/
      case 0xDC :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (pc_address & 0x00FF0000) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE_INDIRECT;
        strcpy(current_codeline->opcode,"JMP");      
        break;

      /** CMP addr2,X (65816 65c02 6502) **/
      case 0xDD :
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE_INDEXED_X;
        strcpy(current_codeline->opcode,"CMP");
        break;

      /** DEC addr2,X (65816 65c02 6502) **/
      case 0xDE :
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE_INDEXED_X;
        strcpy(current_codeline->opcode,"DEC");
        break;

      /** CMP addr3,X (65816) **/
      case 0xDF :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 4)
          break;
        current_codeline->data_size = 4;
        current_codeline->data_value = data[1] + 256*data[2] + 65536*data[3];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->data[3] = data[3];
        current_codeline->operand_address = current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE_LONG_INDEXED_X;
        strcpy(current_codeline->opcode,"CMPL");
        break;

      /** CPX #const (65816 65c02 6502) **/
      case 0xE0 :
        if(current_codeline->x == 1)
          {
            /* 8 Bit */
            if(data_size < 2)
              break;
            current_codeline->data_size = 2;
            current_codeline->data_value = data[1];
            current_codeline->data[1] = data[1];
            current_codeline->address_mode = AM_IMMEDIATE_8;
          }
        else
          {
            /* 16 Bit */
            if(data_size < 3)
              break;
            current_codeline->data_size = 3;
            current_codeline->data_value = data[1] + 256*data[2];
            current_codeline->data[1] = data[1];
            current_codeline->data[2] = data[2];
            current_codeline->address_mode = AM_IMMEDIATE_16;
          }
        strcpy(current_codeline->opcode,"CPX");
        break;

      /** SBC (dp,x) (65816 65c02 6502) **/
      case 0xE1 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDEXED_X_INDIRECT;
        strcpy(current_codeline->opcode,"SBC");
        break;

      /** SEP const_8 (65816) **/
      case 0xE2 :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_IMMEDIATE_8;
        strcpy(current_codeline->opcode,"SEP");      
        break;

      /** SBC sr,s (65816) **/
      case 0xE3 :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_STACK_RELATIVE;
        strcpy(current_codeline->opcode,"SBC");      
        break;

      /** CPX dp (65816 65c02 6502) **/
      case 0xE4 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE;
        strcpy(current_codeline->opcode,"CPX");
        break;

      /** SBC dp (65816 65c02 6502) **/
      case 0xE5 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE;
        strcpy(current_codeline->opcode,"SBC");
        break;

      /** INC dp (65816 65c02 6502) **/
      case 0xE6 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE;
        strcpy(current_codeline->opcode,"INC");
        break;

      /** SBC [dp] (65816) **/
      case 0xE7 :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDIRECT_LONG;
        strcpy(current_codeline->opcode,"SBC");
        break;

      /** INX (65816 65c02 6502) **/
      case 0xE8 :
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"INX");
        break;

      /** SBC #const (65816 65c02 6502) **/
      case 0xE9 :
        if(current_codeline->m == 1)
          {
            /* 8 Bit */
            if(data_size < 2)
              break;
            current_codeline->data_size = 2;
            current_codeline->data_value = data[1];
            current_codeline->data[1] = data[1];
            current_codeline->address_mode = AM_IMMEDIATE_8;
          }
        else
          {
            /* 16 Bit */
            if(data_size < 3)
              break;
            current_codeline->data_size = 3;
            current_codeline->data_value = data[1] + 256*data[2];
            current_codeline->data[1] = data[1];
            current_codeline->data[2] = data[2];
            current_codeline->address_mode = AM_IMMEDIATE_16;
          }
        strcpy(current_codeline->opcode,"SBC");
        break;

      /** NOP (65816 65c02 6502) **/
      case 0xEA :
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"NOP");
        break;

      /** XBA (65816) **/
      case 0xEB :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"XBA");
        break;

      /** CPX addr2 (65816 65c02 6502) **/
      case 0xEC :
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE;
        strcpy(current_codeline->opcode,"CPX");
        break;

      /** SBC addr2 (65816 65c02 6502) **/
      case 0xED :
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE;
        strcpy(current_codeline->opcode,"SBC");
        break;

      /** INC addr2 (65816 65c02 6502) **/
      case 0xEE :
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE;
        strcpy(current_codeline->opcode,"INC");
        break;

      /** SBC addr3 (65816) **/
      case 0xEF :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 4)
          break;
        current_codeline->data_size = 4;
        current_codeline->data_value = data[1] + 256*data[2] + 65536*data[3];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->data[3] = data[3];
        current_codeline->operand_address = current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE_LONG;
        strcpy(current_codeline->opcode,"SBCL");
        break;

      /** BEQ (65816 65c02 6502) **/
      case 0xF0 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_PC_RELATIVE;
        strcpy(current_codeline->opcode,"BEQ");
        break;

      /** SBC (dp),y (65816 65c02 6502) **/
      case 0xF1 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDIRECT_INDEXED_Y;
        strcpy(current_codeline->opcode,"SBC");
        break;

      /** SBC (dp) (65816 65c02) **/
      case 0xF2 :
        if(processor == DISASSEMBLER_PROC_6502)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDIRECT;
        strcpy(current_codeline->opcode,"SBC");
        break;

      /** SBC (sr,s),y (65816) **/
      case 0xF3 :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_STACK_RELATIVE_INDIRECT_INDEXED_Y;
        strcpy(current_codeline->opcode,"SBC");      
        break;

      /** PEA const_16 (65816) **/
      case 0xF4 :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->address_mode = AM_IMMEDIATE_16;
        strcpy(current_codeline->opcode,"PEA");      
        break;

      /** SBC dp,x (65816 65c02 6502) **/
      case 0xF5 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDEXED_X;
        strcpy(current_codeline->opcode,"SBC");      
        break;

      /** INC dp,x (65816 65c02 6502) **/
      case 0xF6 :
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDEXED_X;
        strcpy(current_codeline->opcode,"INC");        
        break;

      /** SBC [dp],y (65816) **/
      case 0xF7 :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 2)
          break;
        current_codeline->data_size = 2;
        current_codeline->data_value = data[1];
        current_codeline->data[1] = data[1];
        current_codeline->address_mode = AM_DIRECT_PAGE_INDIRECT_LONG_INDEXED_Y;
        strcpy(current_codeline->opcode,"SBC");
        break;

      /** SED (65816 65c02 6502) **/
      case 0xF8 :
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"SED");
        break;

      /** SBC addr2,y (65816 65c02 6502) **/
      case 0xF9 :
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE_INDEXED_Y;
        strcpy(current_codeline->opcode,"SBC");
        break;

      /** PLX (65816 65c02) **/
      case 0xFA :
        if(processor == DISASSEMBLER_PROC_6502)
          break;
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"PLX");
        break;

      /** XCE (65816) **/
      case 0xFB :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        current_codeline->data_size = 1;
        current_codeline->address_mode = AM_IMPLICIT;
        strcpy(current_codeline->opcode,"XCE");
        break;

      /** JSR (addr2,x) (65816) **/
      case 0xFC :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (pc_address & 0x00FF0000) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE_INDEXED_X_INDIRECT;
        strcpy(current_codeline->opcode,"JSR");      
        break;

      /** SBC addr2,X (65816 65c02 6502) **/
      case 0xFD :
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE_INDEXED_X;
        strcpy(current_codeline->opcode,"SBC");
        break;

      /** INC addr2,X (65816 65c02 6502) **/
      case 0xFE :
        if(data_size < 3)
          break;
        current_codeline->data_size = 3;
        current_codeline->data_value = data[1] + 256*data[2];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->operand_address = (data_bank << 16) + current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE_INDEXED_X;
        strcpy(current_codeline->opcode,"INC");
        break;

      /** SBC addr3,X (65816) **/
      case 0xFF :
        if(processor != DISASSEMBLER_PROC_65C816)
          break;
        if(data_size < 4)
          break;
        current_codeline->data_size = 4;
        current_codeline->data_value = data[1] + 256*data[2] + 65536*data[3];
        current_codeline->data[1] = data[1];
        current_codeline->data[2] = data[2];
        current_codeline->data[3] = data[3];
        current_codeline->operand_address = current_codeline->data_value;
        current_codeline->address_mode = AM_ABSOLUTE_LONG_INDEXED_X;
        strcpy(current_codeline->opcode,"SBCL");
        break;
    }

  /********************************************/
  /** Instruction inconnue => Passage en Hex **/
  if(current_codeline->data_size == 0)
    {
      current_codeline->data_size = 1;
      current_codeline->address_mode = AM_IMPLICIT;
      strcpy(current_codeline->opcode,"HEX");
      sprintf(current_codeline->operand,"%02X",data[0]);
      return(current_codeline);
    }

  /************************************************************/
  /** On va crer l'Oprande en fonction du mode d'adressage **/
  if(current_codeline->address_mode == AM_IMPLICIT)
    strcpy(current_codeline->operand,"");
  else if(current_codeline->address_mode == AM_ABSOLUTE)
    {
      sprintf(current_codeline->operand,"$%04X",current_codeline->data_value);

      /* On va rechercher un Symbol  l'adresse de cette Oprande */
      current_operand_symbol = GetSymbolByAddress(current_codeline->operand_address);
      if(current_operand_symbol)
        {
          sprintf(current_codeline->operand,"%s",current_operand_symbol->name);
          current_codeline->operand_symbol = 1;
        }
      else
        {
          /* Recherche un Symbol  une adresse prcdante */
          current_operand_symbol = GetSymbolByPreviousAddress(current_codeline->operand_address);
          if(current_operand_symbol)
            {
              /* Adresse Partielle */
              current_codeline->operand_symbol = 1;
              delta_address = current_codeline->operand_address - current_operand_symbol->symbol_relocated_address;
              sprintf(current_codeline->operand,"%s+%d",current_operand_symbol->name,delta_address);
            }
        }
    }
  else if(current_codeline->address_mode == AM_ABSOLUTE_INDEXED_X_INDIRECT)
    {
      sprintf(current_codeline->operand,"($%04X,X)",current_codeline->data_value);

      /* On va rechercher un Symbol  l'adresse de cette Oprande */
      current_operand_symbol = GetSymbolByAddress(current_codeline->operand_address);
      if(current_operand_symbol)
        {
          sprintf(current_codeline->operand,"(%s,X)",current_operand_symbol->name);
          current_codeline->operand_symbol = 1;
        }
      else
        {
          /* Recherche un Symbol  une adresse prcdante */
          current_operand_symbol = GetSymbolByPreviousAddress(current_codeline->operand_address);
          if(current_operand_symbol)
            {
              /* Adresse Partielle */
              current_codeline->operand_symbol = 1;
              delta_address = current_codeline->operand_address - current_operand_symbol->symbol_relocated_address;
              sprintf(current_codeline->operand,"(%s+%d,X)",current_operand_symbol->name,delta_address);
            }
        }
        
    }
  else if(current_codeline->address_mode == AM_ABSOLUTE_INDEXED_X)
    {
      sprintf(current_codeline->operand,"$%04X,X",current_codeline->data_value);

      /* On va rechercher un Symbol  l'adresse de cette Oprande */
      current_operand_symbol = GetSymbolByAddress(current_codeline->operand_address);
      if(current_operand_symbol)
        {
          sprintf(current_codeline->operand,"%s,X",current_operand_symbol->name);
          current_codeline->operand_symbol = 1;
        }
      else
        {
          /* Recherche un Symbol  une adresse prcdante */
          current_operand_symbol = GetSymbolByPreviousAddress(current_codeline->operand_address);
          if(current_operand_symbol)
            {
              /* Adresse Partielle */
              current_codeline->operand_symbol = 1;
              delta_address = current_codeline->operand_address - current_operand_symbol->symbol_relocated_address;
              sprintf(current_codeline->operand,"%s+%d,X",current_operand_symbol->name,delta_address);
            }
        }

    }
  else if(current_codeline->address_mode == AM_ABSOLUTE_INDEXED_Y)
    {
      sprintf(current_codeline->operand,"$%04X,Y",current_codeline->data_value);

      /* On va rechercher un Symbol  l'adresse de cette Oprande */
      current_operand_symbol = GetSymbolByAddress(current_codeline->operand_address);
      if(current_operand_symbol)
        {
          sprintf(current_codeline->operand,"%s,Y",current_operand_symbol->name);
          current_codeline->operand_symbol = 1;
        }
      else
        {
          /* Recherche un Symbol  une adresse prcdante */
          current_operand_symbol = GetSymbolByPreviousAddress(current_codeline->operand_address);
          if(current_operand_symbol)
            {
              /* Adresse Partielle */
              current_codeline->operand_symbol = 1;
              delta_address = current_codeline->operand_address - current_operand_symbol->symbol_relocated_address;
              sprintf(current_codeline->operand,"%s+%d,Y",current_operand_symbol->name,delta_address);
            }
        }
    }
  else if(current_codeline->address_mode == AM_ABSOLUTE_INDIRECT)
    sprintf(current_codeline->operand,"($%04X)",current_codeline->data_value);
  else if(current_codeline->address_mode == AM_ABSOLUTE_INDIRECT_LONG)
    sprintf(current_codeline->operand,"[$%04X]",current_codeline->data_value);
  else if(current_codeline->address_mode == AM_ABSOLUTE_LONG)
    {
      sprintf(current_codeline->operand,"$%06X",current_codeline->data_value);

      /* On va rechercher un Symbol  l'adresse de cette Oprande */
      current_operand_symbol = GetSymbolByAddress(current_codeline->operand_address);
      if(current_operand_symbol)
        {
          sprintf(current_codeline->operand,"%s",current_operand_symbol->name);
          current_codeline->operand_symbol = 1;
        }
      else
        {
          /* Recherche un Symbol  une adresse prcdante */
          current_operand_symbol = GetSymbolByPreviousAddress(current_codeline->operand_address);
          if(current_operand_symbol)
            {
              /* Adresse Partielle */
              current_codeline->operand_symbol = 1;
              delta_address = current_codeline->operand_address - current_operand_symbol->symbol_relocated_address;
              sprintf(current_codeline->operand,"%s+%d",current_operand_symbol->name,delta_address);
            }
        }        
    }
  else if(current_codeline->address_mode == AM_ABSOLUTE_LONG_INDEXED_X)
    {
      sprintf(current_codeline->operand,"$%06X,X",current_codeline->data_value);

      /* On va rechercher un Symbol  l'adresse de cette Oprande */
      current_operand_symbol = GetSymbolByAddress(current_codeline->operand_address);
      if(current_operand_symbol)
        {
          sprintf(current_codeline->operand,"%s,X",current_operand_symbol->name);
          current_codeline->operand_symbol = 1;
        }
      else
        {
          /* Recherche un Symbol  une adresse prcdante */
          current_operand_symbol = GetSymbolByPreviousAddress(current_codeline->operand_address);
          if(current_operand_symbol)
            {
              /* Adresse Partielle */
              current_codeline->operand_symbol = 1;
              delta_address = current_codeline->operand_address - current_operand_symbol->symbol_relocated_address;
              sprintf(current_codeline->operand,"%s+%d,X",current_operand_symbol->name,delta_address);
            }
        } 
    }
  else if(current_codeline->address_mode == AM_DIRECT_PAGE)
    sprintf(current_codeline->operand,"$%02X",current_codeline->data_value);
  else if(current_codeline->address_mode == AM_DIRECT_PAGE_INDEXED_X)
    sprintf(current_codeline->operand,"$%02X,X",current_codeline->data_value);
  else if(current_codeline->address_mode == AM_DIRECT_PAGE_INDEXED_Y)
    sprintf(current_codeline->operand,"$%02X,Y",current_codeline->data_value);
  else if(current_codeline->address_mode == AM_DIRECT_PAGE_INDIRECT)
    sprintf(current_codeline->operand,"($%02X)",current_codeline->data_value);
  else if(current_codeline->address_mode == AM_DIRECT_PAGE_INDIRECT_LONG)
    sprintf(current_codeline->operand,"[$%02X]",current_codeline->data_value);
  else if(current_codeline->address_mode == AM_DIRECT_PAGE_INDEXED_X_INDIRECT)
    sprintf(current_codeline->operand,"($%02X,X)",current_codeline->data_value);
  else if(current_codeline->address_mode == AM_DIRECT_PAGE_INDIRECT_INDEXED_Y)
    sprintf(current_codeline->operand,"($%02X),Y",current_codeline->data_value);
  else if(current_codeline->address_mode == AM_DIRECT_PAGE_INDIRECT_LONG_INDEXED_Y)
    sprintf(current_codeline->operand,"[$%02X],Y",current_codeline->data_value);  
  else if(current_codeline->address_mode == AM_IMMEDIATE_8)
    sprintf(current_codeline->operand,"#$%02X",current_codeline->data_value);
  else if(current_codeline->address_mode == AM_IMMEDIATE_16)
    {
      sprintf(current_codeline->operand,"#$%04X",current_codeline->data_value);

      /* On vrifie que cette ligne contient un Reloc */
      current_reloc = GetRelocByAddress(pc_address);
      if(current_reloc)
        {
          /* On va rechercher un Symbol  l'adresse de cette Oprande */
          current_operand_symbol = GetSymbolByAddress((data_bank << 16) + current_codeline->data_value);
          if(current_operand_symbol)
            {
              sprintf(current_codeline->operand,"#%s",current_operand_symbol->name);
              current_codeline->operand_symbol = 1;
            }
        }
    }
  else if(current_codeline->address_mode == AM_PC_RELATIVE)        /* BEQ BNE BPL BRA... */
    {
      /* Delta sur 8 bit */
      delta = ((current_codeline->data_value & 0x80) == 0x00) ? current_codeline->data_value : (-1)*(256-current_codeline->data_value); 
      address = (pc_address + 2) + delta;

      /* On doit rester dans le mme banc */
      if((address & 0xFF0000) != (pc_address & 0xFF0000))
        {
          current_codeline->wrap_address_bank = 1;
          address = (pc_address & 0xFF0000) | (address & 0x00FFFF);
        }
      sprintf(current_codeline->operand,"$%04X",address & 0xFFFF);

      /* Adresse d'arrive du saut relatif : On va rechercher un Symbol  l'adresse de cette Oprande */
      current_operand_symbol = GetSymbolByAddress(address);
      if(current_operand_symbol)
        {
          sprintf(current_codeline->operand,"%s",current_operand_symbol->name);
          current_codeline->operand_symbol = 1;
        }
    }
  else if(current_codeline->address_mode == AM_PC_RELATIVE_LONG)    /* BRL */
    {
      /* Delta sur 16 bit */
      delta = ((current_codeline->data_value & 0x8000) == 0x0000) ? current_codeline->data_value : (-1)*(65536-current_codeline->data_value);    
      address = (pc_address + 3) + delta;

      /* On doit rester dans le mme banc */
      if((address & 0xFF0000) != (pc_address & 0xFF0000))
        {
          current_codeline->wrap_address_bank = 1;
          address = (pc_address & 0xFF0000) | (address & 0x00FFFF);
        }
      sprintf(current_codeline->operand,"$%04X",address & 0xFFFF);

      /* Adresse d'arrive du saut relatif : On va rechercher un Symbol  l'adresse de cette Oprande */
      current_operand_symbol = GetSymbolByAddress(address);
      if(current_operand_symbol)
        {
          sprintf(current_codeline->operand,"%s",current_operand_symbol->name);
          current_codeline->operand_symbol = 1;
        }
    }
  else if(current_codeline->address_mode == AM_STACK_RELATIVE_INDIRECT_INDEXED_Y)
    sprintf(current_codeline->operand,"(#$%02X,S),Y",current_codeline->data_value);
  else if(current_codeline->address_mode == AM_STACK_PC_RELATIVE_LONG)   /* PER */
    {
      /* Delta sur 16 bit */
      delta = ((current_codeline->data_value & 0x8000) == 0x0000) ? current_codeline->data_value : (-1)*(65536-current_codeline->data_value);    
      address = (pc_address + 3) + delta;

      /* On doit rester dans le mme banc */
      if((address & 0xFF0000) != (pc_address & 0xFF0000))
        {
          current_codeline->wrap_address_bank = 1;
          address = (pc_address & 0xFF0000) | (address & 0x00FFFF);
        }

      /* On doit rester dans le mme banc */
      sprintf(current_codeline->operand,"$%04X",address & 0xFFFF);    
    }
  else if(current_codeline->address_mode == AM_STACK_RELATIVE)
    sprintf(current_codeline->operand,"#$%02X,S",current_codeline->data_value);
  else if(current_codeline->address_mode == AM_BLOCK_MOVE)
    sprintf(current_codeline->operand,"#$%02X,#$%02X",current_codeline->data_value,current_codeline->aux_value);  

  /** On indique la longueur des sauts relatifs **/
  if(current_codeline->address_mode == AM_PC_RELATIVE)              /* BEQ BNE BPL BRA... */
    {
      /* Delta sur 8 bit */
      delta = 2 + (((current_codeline->data_value & 0x80) == 0x00) ? current_codeline->data_value : (-1)*(256-current_codeline->data_value));
      sprintf(&current_codeline->operand[strlen(current_codeline->operand)],"  %s%d",(delta>0)?"+":"-",abs(delta));
    }
  if(current_codeline->address_mode == AM_PC_RELATIVE_LONG)    /* BRL */
    {
      /* Delta sur 16 bit */
      delta = 3 + (((current_codeline->data_value & 0x8000) == 0x0000) ? current_codeline->data_value : (-1)*(65536-current_codeline->data_value));
      sprintf(&current_codeline->operand[strlen(current_codeline->operand)],"  %s%d",(delta>0)?"+":"-",abs(delta));
    }

  /***************************************************************************************************/
  /** On va crer la Zone Commentaire en fonction des adresses reconnues (Softswitch, P8, Tools...) **/
  if(current_codeline->address_mode == AM_ABSOLUTE)
    {
      /* Softswitch / IWM */
      GetNameFromAddress((current_codeline->address & 0x00FF0000)|current_codeline->data_value,0x0000,buffer_name,buffer_type,buffer_application);
      if(strlen(buffer_name) > 0)
        sprintf(current_codeline->comment,"; %s",buffer_name);
    }
  if(current_codeline->address_mode == AM_ABSOLUTE_LONG)
    {
      /* Softswitch / IWM */
      GetNameFromAddress(current_codeline->data_value,0x0000,buffer_name,buffer_type,buffer_application);
      if(strlen(buffer_name) > 0)
        sprintf(current_codeline->comment,"; %s",buffer_name);
    }

  /* Zones connues */
  if(current_codeline->data_value == 0xE12000 || current_codeline->data_value == 0x012000)
    strcpy(current_codeline->comment,"; SHR Pixels");
  if(current_codeline->data_value == 0xE19D00 || current_codeline->data_value == 0x019D00)
    strcpy(current_codeline->comment,"; SHR SCBs");
  if(current_codeline->data_value == 0xE19E00 || current_codeline->data_value == 0x019E00)
    strcpy(current_codeline->comment,"; SHR Palettes");

  /** Points d'entres connus 16 bit **/
  if(!stricmp(current_codeline->opcode,"JSL") && current_codeline->data_value == 0xE10000)
    {
      /* Toolbox Entry Point */
      strcpy(current_codeline->comment,"; Toolbox Entry");
      
      /* On dispose d'un call_num */
      if(call_num != 0xFFFF)
        {
          /* ToolSet + Numro du Tool */
          GetNameFromAddress(current_codeline->data_value,call_num,buffer_name,buffer_type,buffer_application);
          if(strlen(buffer_name) > 0)
            sprintf(current_codeline->comment,"; %s",buffer_name);
        }
      else
        {
          /* On cherche un LDX #$ au dessus avec le numro du Tool */
          for(previous_codeline = last_codeline; previous_codeline; previous_codeline = previous_codeline->previous)
            {
              /** Certains opcodes sont liminatoires (branchements, modif de X, arrts...) **/
              if(!stricmp(previous_codeline->opcode,"JSR") || !stricmp(previous_codeline->opcode,"JSL") ||
                 !stricmp(previous_codeline->opcode,"JMP") || !stricmp(previous_codeline->opcode,"JMPL") ||
                 !stricmp(previous_codeline->opcode,"RTS") || !stricmp(previous_codeline->opcode,"RTL") || !stricmp(previous_codeline->opcode,"RTI") ||
                 previous_codeline->address_mode == AM_PC_RELATIVE || previous_codeline->address_mode == AM_PC_RELATIVE_LONG)
                break;
              if(!stricmp(previous_codeline->opcode,"TAX") || !stricmp(previous_codeline->opcode,"TYX") ||
                 !stricmp(previous_codeline->opcode,"INX") || !stricmp(previous_codeline->opcode,"DEX") ||
                 (!stricmp(previous_codeline->opcode,"LDX") && previous_codeline->address_mode != AM_IMMEDIATE_16))
                break;
              if(!stricmp(previous_codeline->opcode,"BRK") || !stricmp(previous_codeline->opcode,"COP") ||
                 !stricmp(previous_codeline->opcode,"WDM") || !stricmp(previous_codeline->opcode,"WAI") ||
                 !stricmp(previous_codeline->opcode,"STP"))
                break;

              /** On vient de tomber sur un LDX const_16 **/
              if(!stricmp(previous_codeline->opcode,"LDX") && previous_codeline->address_mode == AM_IMMEDIATE_16)
                {
                  /* ToolSet + Numro du Tool */
                  GetNameFromAddress(current_codeline->data_value,(previous_codeline->data[2]<<8)|previous_codeline->data[1],buffer_name,buffer_type,buffer_application);
                  if(strlen(buffer_name) > 0)
                    sprintf(current_codeline->comment,"; %s",buffer_name);
                  break;
                }
            }
        }
    }
  else if(!stricmp(current_codeline->opcode,"JSL") && current_codeline->data_value == 0xE100A8)
    {
      strcpy(current_codeline->comment,"; GS/OS");
      if(call_num != 0xFFFF)
        {
          /* GS/OS Call */
          GetNameFromAddress(current_codeline->data_value,call_num,buffer_name,buffer_type,buffer_application);
          if(strlen(buffer_name) > 0)
            sprintf(current_codeline->comment,"; %s",buffer_name);
        }
    }
  else if(!stricmp(current_codeline->opcode,"JSL") && current_codeline->data_value == 0xE100B0)
    {
      strcpy(current_codeline->comment,"; GS/OS");
      if(call_num != 0xFFFF)
        {
          /* GS/OS Call */
          GetNameFromAddress(current_codeline->data_value,call_num,buffer_name,buffer_type,buffer_application);
          if(strlen(buffer_name) > 0)
            sprintf(current_codeline->comment,"; %s",buffer_name);
        }
    }
  /** Banc 0 **/
  if((current_codeline->data_value >> 16) == 0x00 && !stricmp(current_codeline->opcode,"JSR"))
    {
      /* BFxx / F8xx / F9xx / FAxx / FBxx / FCxx / FDxx / FExx / FFxx */
      GetNameFromAddress(current_codeline->data_value,0x0000,buffer_name,buffer_type,buffer_application);
      if(strlen(buffer_name) > 0)
        sprintf(current_codeline->comment,"; %s",buffer_name);
    }

  /* Renvoi la ligne */
  return(current_codeline);
}


/**************************************************************************/
/*  ComputeDesassemblyQuality() :  Dtermine la qualit du dsassemblage. */
/**************************************************************************/
static int ComputeDesassemblyQuality(struct code_line *first_codeline)
{
  int nb_line, quality_coef;
  struct code_line *current_codeline;
  struct code_line *next_codeline;

  /* Init */
  nb_line = 0;
  quality_coef = 0;

  /** On va comptabiliser le nombre de ligne **/
  for(nb_line=0, current_codeline = first_codeline; current_codeline; current_codeline = current_codeline->next)
    nb_line++;

  /** On pondre les lements de ce qui a t dsassembl **/
  for(current_codeline = first_codeline; current_codeline; current_codeline = current_codeline->next)
    {
      /* Ligne en Erreur */
      if(!stricmp(current_codeline->opcode,"BRK") || !stricmp(current_codeline->opcode,"COP") ||
         !stricmp(current_codeline->opcode,"WDM") || !stricmp(current_codeline->opcode,"WAI") ||
         !stricmp(current_codeline->opcode,"STP"))
        {
          quality_coef -= 10;
          continue;
        }

      /* Ligne Data */
      if(!stricmp(current_codeline->opcode,"HEX"))
        {
          quality_coef -= 5;
          continue;
        }

      /* Ligne Directive qui tombe juste */
      if(current_codeline->is_directive == 1)
        {
          quality_coef += 5;
          continue;
        }

      /* Ligne de Data suite au appel Prodos ou GS/OS */
      if(!stricmp(current_codeline->opcode,"DFB") || !stricmp(current_codeline->opcode,"DA") || !stricmp(current_codeline->opcode,"ADRL"))
        {
          quality_coef += 5;
          continue;
        }

      /* Cette ligne a t correctement dsassemble */
      quality_coef += 10;
    }

  /** Bonus pour les lignes qui ont un comentaire **/
  for(current_codeline = first_codeline; current_codeline; current_codeline = current_codeline->next)
    if(strlen(current_codeline->comment) > 0)
      quality_coef += 5;

  /** Malus pour les lignes qui ont un saut wrappant le banc **/
  for(current_codeline = first_codeline; current_codeline; current_codeline = current_codeline->next)
    if(current_codeline->wrap_address_bank == 1)
      quality_coef -= 5;

  /** Malus pour les lignes qui se suivent et qui se ressemblent **/
  for(current_codeline = first_codeline; current_codeline; current_codeline = current_codeline->next)
    {
      /* On a fini ? */
      next_codeline = current_codeline->next;
      if(next_codeline == NULL)
        break;
    
      /* Il nous faut une ligne valide */
      if(IsValidCodeLine(current_codeline) == 0)
        continue;
        
      /** Certaines lignes peuvent tre dupliques **/
      if(!strcmp(current_codeline->opcode,"ASL") || !strcmp(current_codeline->opcode,"LSR") ||
         !strcmp(current_codeline->opcode,"ROL") || !strcmp(current_codeline->opcode,"ROR") ||
         !strcmp(current_codeline->opcode,"NOP") || !strcmp(current_codeline->opcode,"PHA") ||
         !strcmp(current_codeline->opcode,"PHX") || !strcmp(current_codeline->opcode,"PHY") ||
         !strcmp(current_codeline->opcode,"PEA") || !strcmp(current_codeline->opcode,"PEI") ||
         !strcmp(current_codeline->opcode,"PER") || !strcmp(current_codeline->opcode,"PHB") ||
         !strcmp(current_codeline->opcode,"PHD") || !strcmp(current_codeline->opcode,"PHK") ||
         !strcmp(current_codeline->opcode,"PHP"))
        continue;
        
      /** Pas deux LDx identiques qui se suivent **/
      if(!strcmp(current_codeline->opcode,"LDA") || !strcmp(current_codeline->opcode,"LDAL") ||
         !strcmp(current_codeline->opcode,"LDX") || !strcmp(current_codeline->opcode,"LDY"))
        {
          if(!strcmp(current_codeline->opcode,"LDA") && (!strcmp(next_codeline->opcode,"LDA") || !strcmp(next_codeline->opcode,"LDAL")))
            {
              quality_coef -= 10;
              continue;
            }
          else if(!strcmp(current_codeline->opcode,"LDX") && !strcmp(next_codeline->opcode,"LDX"))
            {
              quality_coef -= 10;
              continue;
            }
          else if(!strcmp(current_codeline->opcode,"LDY") && !strcmp(next_codeline->opcode,"LDY"))
            {
              quality_coef -= 10;
              continue;
            }
        }
        
      /** Pas deux instructions opposes qui se suivent **/
      if((!strcmp(current_codeline->opcode,"INC") && !strcmp(next_codeline->opcode,"DEC")) ||
         (!strcmp(current_codeline->opcode,"DEC") && !strcmp(next_codeline->opcode,"INC")) ||
         (!strcmp(current_codeline->opcode,"INX") && !strcmp(next_codeline->opcode,"DEX")) ||
         (!strcmp(current_codeline->opcode,"DEX") && !strcmp(next_codeline->opcode,"INX")) ||
         (!strcmp(current_codeline->opcode,"INY") && !strcmp(next_codeline->opcode,"DEY")) ||
         (!strcmp(current_codeline->opcode,"DEY") && !strcmp(next_codeline->opcode,"INY")) ||
         (!strcmp(current_codeline->opcode,"CLC") && !strcmp(next_codeline->opcode,"SEC")) ||
         (!strcmp(current_codeline->opcode,"SEC") && !strcmp(next_codeline->opcode,"CLC")) ||
         (!strcmp(current_codeline->opcode,"CLD") && !strcmp(next_codeline->opcode,"SED")) ||
         (!strcmp(current_codeline->opcode,"SED") && !strcmp(next_codeline->opcode,"CLD")) ||
         (!strcmp(current_codeline->opcode,"CLI") && !strcmp(next_codeline->opcode,"SEI")) ||
         (!strcmp(current_codeline->opcode,"SEI") && !strcmp(next_codeline->opcode,"CLI")))
        {
          quality_coef -= 5;
          continue;
        }
        
      /** Pas deux instructions identiques qui se suivent **/
      if(!strcmp(current_codeline->opcode,next_codeline->opcode) && 
         current_codeline->data_size == next_codeline->data_size &&
         !memcmp(current_codeline->data,next_codeline->data,current_codeline->data_size))
        {
          quality_coef -= 10;
          continue;
        }
    }

  /** Bonus pour les lignes qui se suivent et qui ont une Pattern connue **/
  for(current_codeline = first_codeline; current_codeline; current_codeline = current_codeline->next)
    {
      /* On a fini ? */
      next_codeline = current_codeline->next;
      if(next_codeline == NULL)
        break;
    
      /* Il nous faut une ligne valide */
      if(IsValidCodeLine(current_codeline) == 0)
        continue;
        
      /* PHK + PLB */
      if(!strcmp(current_codeline->opcode,"PHK") && !strcmp(next_codeline->opcode,"PLB"))
        {
          quality_coef += 5;
          continue;
        }
        
      /* CLC + ADC */
      if(!strcmp(current_codeline->opcode,"CLC") && (!strcmp(next_codeline->opcode,"ADC") || !strcmp(next_codeline->opcode,"ADCL")))
        {
          quality_coef += 5;
          continue;
        }

      /* SEC + SBC */
      if(!strcmp(current_codeline->opcode,"SEC") && (!strcmp(next_codeline->opcode,"SBC") || !strcmp(next_codeline->opcode,"SBCL")))
        {
          quality_coef += 5;
          continue;
        }

      /* REP / SEP */
      if((!strcmp(current_codeline->opcode,"REP") && current_codeline->data[1] == 0x30) ||
         (!strcmp(current_codeline->opcode,"SEP") && current_codeline->data[1] == 0x30))
        {
          quality_coef += 5;
          continue;
        }              
        
      /* CMP | CPX | CPY + Bxx  */
      if((!strcmp(current_codeline->opcode,"CMP") || !strcmp(current_codeline->opcode,"CMPL") ||
          !strcmp(current_codeline->opcode,"CPX") || !strcmp(current_codeline->opcode,"CPY")) &&
         (!strcmp(current_codeline->opcode,"BNE") || !strcmp(current_codeline->opcode,"BEQ") ||
          !strcmp(current_codeline->opcode,"BMI") || !strcmp(current_codeline->opcode,"BPL") ||
          !strcmp(current_codeline->opcode,"BCS") || !strcmp(current_codeline->opcode,"BCC") ||
          !strcmp(current_codeline->opcode,"BVS") || !strcmp(current_codeline->opcode,"BVC")))
        {
          quality_coef += 5;
          continue;
        }        
    }

  /* Niveau de qualit */
  return(quality_coef);
}


/********************************************************************************/
/*  IsValidCodeLine() :  Dtermine si cette ligne est une ligne de code valide. */
/********************************************************************************/
static int IsValidCodeLine(struct code_line *current_codeline)
{
  /* Ligne en Erreur */
  if(!stricmp(current_codeline->opcode,"BRK") || !stricmp(current_codeline->opcode,"COP") ||
     !stricmp(current_codeline->opcode,"WDM") || !stricmp(current_codeline->opcode,"WAI") ||
     !stricmp(current_codeline->opcode,"STP"))
    return(0);

  /* Ligne Data */
  if(!stricmp(current_codeline->opcode,"HEX"))
    return(0);

  /* Ligne Directive */
  if(current_codeline->is_directive == 1)
    return(0);

  /* Ligne de Data suite au appel Prodos ou GS/OS */
  if(!stricmp(current_codeline->opcode,"DFB") || !stricmp(current_codeline->opcode,"DA") || !stricmp(current_codeline->opcode,"ADRL"))
    return(0);

  /* Ligne de Code valide */
  return(1);
}


/*******************************************************************/
/*  mem_free_codeline() :  Libration mmoire structure code_line. */
/*******************************************************************/
void mem_free_codeline(struct code_line *current_codeline)
{
  if(current_codeline)
    {
      free(current_codeline);
    }
}


/*****************************************************************************/
/*  mem_free_directiveline() :  Libration mmoire structure directive_line. */
/*****************************************************************************/
void mem_free_directiveline(struct directive_line *current_directiveline)
{
  if(current_directiveline)
    {
      free(current_directiveline);
    }
}

/*******************************************************************/
