/************************************************************************/
/*                                                                      */
/*  Dc_IIgsInternals.c :  Module pour les structures internes du IIgs.  */
/*                                                                      */
/************************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe Software  *  Aot 2023   */
/************************************************************************/

#include <stdio.h>
#include <time.h>
#include <sys/timeb.h>

#include <windows.h>

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_MMU.h"
#include "Cyrene_Engine.h"
#include "Dc_IIgsInternals.h"

static DWORD GetMemoryValue32Bit(DWORD,struct mmu *);
static DWORD GetMemoryValue24Bit(DWORD,struct mmu *);
static WORD GetMemoryValue16Bit(DWORD,struct mmu *);
static struct file_segment *DecodeFileSegment7004(struct gs_handle *,struct gs_handle *,struct mmu *);
static struct file_segment *DecodeFileSegment7101(struct gs_handle *,struct mmu *);
static struct file_segment *mem_alloc_file_segment(char *,char *,int,DWORD,DWORD,DWORD,WORD,struct gs_handle *);
static int compare_filesegment(const void *,const void *);
static void GetInternalFileType(WORD,char *);
static int compare_init(const void *,const void *);
static int compare_driver(const void *,const void *);
static int compare_fst(const void *,const void *);

/**********************************************************************/
/*  BuildMemoryHandleList() :  Rcupre la liste dans Handle Mmoire. */
/**********************************************************************/
struct gs_handle *BuildHandleList(DWORD first_handle_address, int *nb_handle_rtn, int *size_alloc_rtn, char *buffer_error, struct mmu *current_mmu)
{
  int has_error, nb_handle, size_alloc;
  DWORD handle_address;
  struct gs_handle *first_handle;
  struct gs_handle *last_handle;
  struct gs_handle *current_handle;

  /* Init */
  first_handle = NULL;
  last_handle = NULL;
  nb_handle = 0;
  size_alloc = 0;
  *nb_handle_rtn = 0;
  *size_alloc_rtn = 0;
  strcpy(buffer_error,"");

  /* First Handle */
  handle_address = GetMemoryValue24Bit(first_handle_address,current_mmu);
  first_handle = GetMemoryHandle(handle_address,current_mmu);
  last_handle = first_handle;

  /* Problme mmoire */
  if(first_handle == NULL)
    {
      sprintf(buffer_error,"Error : Can't allocate memory to load Memory Handle %02X/%04X",handle_address >> 16,handle_address & 0xFFFF);
      return(NULL);
    }
  nb_handle++;
  size_alloc += first_handle->size;

  /* Vrifie la cohrence */
  if(first_handle->prev_handle != 0x000000)
    {
      sprintf(buffer_error,"Error : Wrong Previous Handle value in First Memory Handle %02X/%04X",handle_address >> 16,handle_address & 0xFFFF);
      *nb_handle_rtn = nb_handle;
      *size_alloc_rtn = size_alloc;
      return(first_handle);
    }
  has_error = CheckMemoryHandle(first_handle,buffer_error);
  if(has_error)
    {
      *nb_handle_rtn = nb_handle;
      *size_alloc_rtn = size_alloc;
      return(first_handle);
    }

  /** Boucle sur tous les Handles **/
  current_handle = first_handle;
  while(current_handle->next_handle != 0x000000)
    {
      /* Rcupre le Handle suivant */
      current_handle = GetMemoryHandle(current_handle->next_handle,current_mmu);
      if(current_handle == NULL)
        {
          *nb_handle_rtn = nb_handle;
          *size_alloc_rtn = size_alloc;
          sprintf(buffer_error,"Error : Can't allocate memory to load Memory Handle %02X/%04X",current_handle->next_handle >> 16,current_handle->next_handle & 0xFFFF);
          return(first_handle);
        }
      nb_handle++;
      size_alloc += first_handle->size;

      /* Ajoute  la liste */
      last_handle->next = current_handle;
      last_handle = current_handle;

      /* Vrifie le Handle */
      has_error = CheckMemoryHandle(current_handle,buffer_error);
      if(has_error)
        {
          *nb_handle_rtn = nb_handle;
          *size_alloc_rtn = size_alloc;
          return(first_handle);
        }
    }

  /* Renvoi la liste */
  *nb_handle_rtn = nb_handle;
  *size_alloc_rtn = size_alloc;
  return(first_handle);
}


/*****************************************************/
/*  GetMemoryHandle() :  Rcupre un Handle Mmoire. */
/*****************************************************/
struct gs_handle *GetMemoryHandle(DWORD handle_address, struct mmu *current_mmu)
{
  int error;
  struct gs_handle *current_handle;
  char buffer_error[1024];

  /* Allocation mmoire */
  current_handle = (struct gs_handle *) calloc(1,sizeof(struct gs_handle));
  if(current_handle == NULL)
    return(NULL);
  current_handle->is_valid = 1;

  /* Rcupre les membres */
  current_handle->handle      = handle_address;
  current_handle->address     = GetMemoryValue32Bit(handle_address+0,current_mmu);
  current_handle->attributes  = GetMemoryValue16Bit(handle_address+4,current_mmu);
  current_handle->id          = GetMemoryValue16Bit(handle_address+6,current_mmu);
  current_handle->size        = GetMemoryValue32Bit(handle_address+8,current_mmu);
  current_handle->prev_handle = GetMemoryValue24Bit(handle_address+12,current_mmu);
  current_handle->next_handle = GetMemoryValue24Bit(handle_address+16,current_mmu);

  /* Ce Handle est t'il valide ? */
  error = CheckMemoryHandle(current_handle,buffer_error);
  if(error)
    current_handle->is_valid = 0;

  /* Tronque l'adresse en 24 bit */
  current_handle->address &= GS_MAX_ADDRESS;

  /* Renvoi le Handle */
  return(current_handle);
}


/*********************************************************************/
/*  CheckMemoryHandle() :  Vrifie la cohrence d'un Handle Mmoire. */
/*********************************************************************/
int CheckMemoryHandle(struct gs_handle *current_handle, char *buffer_error)
{
  /* Handle Null ? */
  if(current_handle->prev_handle == 0x000000 && current_handle->next_handle == 0x000000)
    {
      sprintf(buffer_error,"Error : Wrong Prev+Next values in Memory Handle %02X/%04X",current_handle->handle >> 16,current_handle->handle & 0xFFFF);
      return(1);
    }

  /* Prev Handle */
  if(current_handle->prev_handle != 0x000000)
    if(((current_handle->prev_handle & 0xFF0000) != 0xE00000) && ((current_handle->prev_handle & 0xFF0000) != 0xE10000))
      {
        sprintf(buffer_error,"Error : Wrong Prev Handle value (%02X/%04X) in Memory Handle %02X/%04X",current_handle->prev_handle >> 16,current_handle->prev_handle & 0xFFFF,current_handle->handle >> 16,current_handle->handle & 0xFFFF);
        return(1);
      }

  /* Next Handle */
  if(current_handle->next_handle != 0x000000)
    if(((current_handle->next_handle & 0xFF0000) != 0xE00000) && ((current_handle->next_handle & 0xFF0000) != 0xE10000))
      {
        sprintf(buffer_error,"Error : Wrong Next Handle value (%02X/%04X) in Memory Handle %02X/%04X",current_handle->next_handle >> 16,current_handle->next_handle & 0xFFFF,current_handle->handle >> 16,current_handle->handle & 0xFFFF);
        return(1);
      }

  /* Taille */
  if(current_handle->id != 0x0000 && current_handle->size > GS_MAX_ALLOC_SIZE)
    {
      sprintf(buffer_error,"Error : Wrong Size value (%X) in Memory Handle %02X/%04X",current_handle->size,current_handle->handle >> 16,current_handle->handle & 0xFFFF);
      return(1);
    }

  /* Adresse begin */
  if(current_handle->address > GS_MAX_ADDRESS)
    {
      sprintf(buffer_error,"Error : Wrong Address value (%08X) in Memory Handle %02X/%04X",current_handle->address,current_handle->handle >> 16,current_handle->handle & 0xFFFF);
      return(1);
    }

  /* Adresse end */
  if((current_handle->address + current_handle->size) > GS_MAX_ADDRESS)
    {
      sprintf(buffer_error,"Error : Wrong Address value (%08X) + Size (%X) in Memory Handle %02X/%04X",current_handle->address,current_handle->size,current_handle->handle >> 16,current_handle->handle & 0xFFFF);
      return(1);
    }

  /* OK */
  return(0);
}


/**************************************************************/
/*  GetHandleIDType() :  Dtermine le type d'un ID de Handle. */
/**************************************************************/
void GetHandleIDType(WORD handle_id, char *buffer_type)
{
  if((handle_id & 0xF000) == 0x0000)
    strcpy(buffer_type,"Memory Manager");
  else if((handle_id & 0xF000) == 0x1000)
    strcpy(buffer_type,"Application");
  else if((handle_id & 0xF000) == 0x2000)
    strcpy(buffer_type,"Control Program");
  else if((handle_id & 0xF000) == 0x3000)
    strcpy(buffer_type,"ProDOS");
  else if((handle_id & 0xF000) == 0x4000)
    strcpy(buffer_type,"Tool Sets");
  else if((handle_id & 0xF000) == 0x5000)
    strcpy(buffer_type,"Desk Accessories");
  else if((handle_id & 0xF000) == 0x6000)
    strcpy(buffer_type,"Runtime Libraries");
  else if((handle_id & 0xF000) == 0x7000)
    strcpy(buffer_type,"System Loader");
  else if((handle_id & 0xF000) == 0x8000)
    strcpy(buffer_type,"Firmware");
  else if((handle_id & 0xF000) == 0x9000)
    strcpy(buffer_type,"Tool Locator");
  else if((handle_id & 0xF000) == 0xA000)
    strcpy(buffer_type,"Setup File");
  else
    strcpy(buffer_type,"Undefined");
}


/******************************************************************/
/*  GetHandleAttributes() :  Dtermine les attributs d'un Handle. */
/******************************************************************/
void GetHandleAttributes(WORD handle_attributes, char *buffer_attributes)
{
  strcpy(buffer_attributes,"");

  /* Bit #15 : attrLocked */
  if((handle_attributes & 0x8000) == 0x8000)
    sprintf(&buffer_attributes[strlen(buffer_attributes)],"%sLocked",strlen(buffer_attributes) > 0 ? " + " : "");

  /* Bit #14 : attrFixed */
  if((handle_attributes & 0x4000) == 0x4000)
    sprintf(&buffer_attributes[strlen(buffer_attributes)],"%sFixed",strlen(buffer_attributes) > 0 ? " + " : "");

  /* Bit #9-8 : attrPurge */
  if((handle_attributes & 0x0300) == 0x0100)
    sprintf(&buffer_attributes[strlen(buffer_attributes)],"%sPurgeLevel=1",strlen(buffer_attributes) > 0 ? " + " : "");
  else if((handle_attributes & 0x0300) == 0x0200)
    sprintf(&buffer_attributes[strlen(buffer_attributes)],"%sPurgeLevel=2",strlen(buffer_attributes) > 0 ? " + " : "");
  else if((handle_attributes & 0x0300) == 0x0300)
    sprintf(&buffer_attributes[strlen(buffer_attributes)],"%sPurgeLevel=3",strlen(buffer_attributes) > 0 ? " + " : "");

  /* Bit #4 : attrNoCross */
  if((handle_attributes & 0x0010) == 0x0010)
    sprintf(&buffer_attributes[strlen(buffer_attributes)],"%sNoCrossBank",strlen(buffer_attributes) > 0 ? " + " : "");

  /* Bit #3 : attrNoSpec */
  if((handle_attributes & 0x0008) == 0x0008)
    sprintf(&buffer_attributes[strlen(buffer_attributes)],"%sNoSpecialMem",strlen(buffer_attributes) > 0 ? " + " : "");

  /* Bit #2 : attrPage */
  if((handle_attributes & 0x0004) == 0x0004)
    sprintf(&buffer_attributes[strlen(buffer_attributes)],"%sPageAligned",strlen(buffer_attributes) > 0 ? " + " : "");

  /* Bit #1 : attrAddr */
  if((handle_attributes & 0x0002) == 0x0002)
    sprintf(&buffer_attributes[strlen(buffer_attributes)],"%sFixedAddress",strlen(buffer_attributes) > 0 ? " + " : "");

  /* Bit #0 : attrBank */
  if((handle_attributes & 0x0001) == 0x0001)
    sprintf(&buffer_attributes[strlen(buffer_attributes)],"%sFixedBank",strlen(buffer_attributes) > 0 ? " + " : "");
}


/***************************************************************************/
/*  GetHandleGroup() :  Rcupre le groupe (Used/Purged/Free) d'un Handle. */
/***************************************************************************/
void GetHandleGroup(DWORD handle, char *handle_group)
{
  struct gs_handle *current_handle;
  struct s65816 *current_65816;

  /* Rcupration du Engine */
  my_Memory(MEMORY_GET_S65816,&current_65816,NULL);

  /** Used **/
  for(current_handle = current_65816->list_used_handle; current_handle; current_handle = current_handle->next)
    if(current_handle->handle == handle)
      {
        strcpy(handle_group,"Used");
        return;
      }

  /** Purged **/
  for(current_handle = current_65816->list_purged_handle; current_handle; current_handle = current_handle->next)
    if(current_handle->handle == handle)
      {
        strcpy(handle_group,"Purged");
        return;
      }

  /** Free **/
  for(current_handle = current_65816->list_free_handle; current_handle; current_handle = current_handle->next)
    if(current_handle->handle == handle)
      {
        strcpy(handle_group,"Free");
        return;
      }

  /** Unknown **/
  strcpy(handle_group,"Unknown");
}


/****************************************************************************/
/*  GetHandleInfo() :  Rcupre les informations contextuelles d'un Handle. */
/****************************************************************************/
void GetHandleInfo(struct gs_handle *current_handle, char *information)
{
  int i, tool_number;
  BYTE b_length;
  WORD w_length, w_offset, id;
  DWORD handle;
  struct s65816 *current_65816;
  char *tab_tool[] = 
    {
      "",                /* $00 */
      "ToolLocator",     /* $01 Tool001 */  
      "MemoryManager",   /* $02 Tool002 */
      "Miscellaneous",   /* $03 Tool003 */
      "Quickdraw II",    /* $04 Tool004 */
      "DeskManager",     /* $05 Tool005 */
      "EventManager",    /* $06 Tool006 */
      "Scheduler",       /* $07 Tool007 */
      "SoundTool",       /* $08 Tool008 */
      "ADB",             /* $09 Tool009 */
      "SANE",            /* $0A Tool010 */
      "IntegerMath",     /* $0B Tool011 */
      "TextTool",        /* $0C Tool012 */
      "???",             /* $0D Tool013 */  
      "WindowManager",   /* $0E Tool014 */
      "MenuManager",     /* $0F Tool015 */
      "ControlManager",  /* $10 Tool016 */
      "SystemLoader",    /* $11 Tool017 */
      "QDIIAuxiliary",   /* $12 Tool018 */
      "PrintManager",    /* $13 Tool019 */
      "LineEdit",        /* $14 Tool020 */
      "DialogManager",   /* $15 Tool021 */
      "ScrapManager",    /* $16 Tool022 */
      "StandardFile",    /* $17 Tool023 */
      "???",             /* $18 Tool024 */
      "NoteSynthetizer", /* $19 Tool025 */
      "NoteSequencer",   /* $1A Tool026 */
      "FontManager",     /* $1B Tool027 */
      "ListManager",     /* $1C Tool028 */
      "ACE",             /* $1D Tool029 */
      "ResourceManager", /* $1E Tool030 */
      "???",             /* $1F Tool031 */
      "MIDITool",        /* $20 Tool032 */
      "VideoOverlay",    /* $21 Tool033 */
      "TextExit",        /* $22 Tool034 */
      "MIDISynth",       /* $23 Tool035 */
      "???",             /* $24 Tool036 */
      "???",             /* $25 Tool037 */
      "MediaControl"     /* $26 Tool038 */
    };
  char file_path[1024];
  char file_name[1024];
  char file_type[1024];
  unsigned char *data;

  /* Rcupration du Engine */
  my_Memory(MEMORY_GET_S65816,&current_65816,NULL);

  /* Init */
  strcpy(information,"");
  strcpy(file_name,"");

  /** RAM Disk **/
  if(current_handle->id == 0x0001)
    {
      strcpy(information,"RAM Disk : RAM5");
      return;
    }

  /** Bundle ID **/
  if((current_handle->id & 0xFF00) == 0x1700)
    {
      sprintf(information,"Finder : Bundle ID");
      return;
    }

  /** Icon List ID **/
  if((current_handle->id & 0xFF00) == 0x1800)
    {
      sprintf(information,"Finder : Icon List ID");
      return;
    }

  /** Finder en plus du 0x1001 **/
  if(current_handle->id == 0x1101)
    {
      sprintf(information,"Finder");
      return;
    }

  /** Finder Path ID **/
  if((current_handle->id & 0xFF00) == 0x1900 && current_handle->is_valid == 1)
    {
      data = LoadHandleData(current_handle,current_65816->current_mmu);
      if(data)
        {
          if(current_handle->size > 8)
            {
              memcpy(&w_offset,&data[0x02],sizeof(WORD));
              if(w_offset < current_handle->size)
                {
                  memcpy(&w_length,&data[w_offset],sizeof(WORD));

                  if((w_offset + w_length) <= (int) current_handle->size)
                    {
                      /* Chemin Complet */
                      memcpy(file_path,&data[w_offset+2],w_length);
                      file_path[w_length] = '\0';

                      /* Nom de fichier */
                      GetProdosFileName(file_path,file_name);

                      /* */
                      sprintf(information,"Finder Path ID : %s",file_name);
                    }
                }
            }
          else
            sprintf(information,"Finder Path ID");
          free(data);
        }
      else 
        sprintf(information,"Finder Path ID");
      return;
    }

  /** Tools **/
  if((current_handle->id & 0xFF00) == 0x4000)
    {
      tool_number = current_handle->id & 0x00FF;
      if(tool_number < 39)
        {
          sprintf(information,"Tool%03d / %s",tool_number,tab_tool[tool_number]);
          return;
        }
    }

  /** Tool Locator : 9001 **/
  if(current_handle->id == 0x9001 && current_handle->size > 8 && current_handle->is_valid == 1)
    {
      data = LoadHandleData(current_handle,current_65816->current_mmu);
      if(data)
        {
          b_length = data[0x08];
          if(b_length > 2 && b_length < 0x40 && (0x08 + b_length) < (int) current_handle->size)
            {
              memcpy(information,&data[0x0A],b_length-2);
              information[b_length-2] = '\0';
            }
          free(data);
        }
    }

  /** System Loader 7003 **/
  if(current_handle->id == 0x7003 && current_handle->size == 0x14 && current_handle->is_valid == 1)
    {
      data = LoadHandleData(current_handle,current_65816->current_mmu);
      if(data)
        {
          /* ID */
          memcpy(&id,&data[8],sizeof(WORD));

          /* Handle */
          memcpy(&handle,&data[10],sizeof(DWORD));

          /** Recherche via le User ID dans la File ID List 7004 **/
          if(current_65816->table_file_id != NULL)
            {
              for(i=0; i<current_65816->table_file_id->nb_file_7004; i++)
                {
                  if(current_65816->table_file_id->tab_id_file_7004[i]->id == id && strlen(current_65816->table_file_id->tab_id_file_7004[i]->pathname) > 0)
                    {
                      /* Chemin Complet */
                      strcpy(file_path,current_65816->table_file_id->tab_id_file_7004[i]->pathname);

                      /* Nom de fichier */
                      GetProdosFileName(file_path,file_name);

                      /* */
                      strcpy(information,file_name);

                      free(data);
                      return;
                    }
                }
            }
          free(data);
        }
    }

  /** System Loader 7004 **/
  if(current_handle->id == 0x7004 && current_handle->size > 0x20 && current_handle->is_valid == 1)
    {
      data = LoadHandleData(current_handle,current_65816->current_mmu);
      if(data)
        {
          memcpy(&w_length,&data[0x1E],sizeof(WORD));

          if((0x20 + w_length) == current_handle->size)
            {
              /* Chemin Complet */
              memcpy(file_path,&data[0x20],w_length);
              file_path[w_length] = '\0';

              /* Nom de fichier */
              GetProdosFileName(file_path,file_name);

              /* */
              strcpy(information,file_name);
            }

          free(data);
        }
      return;
    }

  /** System Loader 7101 (Multi Segment) **/
  if(current_handle->id == 0x7101 && current_handle->size > 4 && current_handle->is_valid == 1)
    {
      data = LoadHandleData(current_handle,current_65816->current_mmu);
      if(data)
        {
          /* Offset de la longueur du Path */
          memcpy(&w_offset,&data[0x02],sizeof(WORD));
          if((w_offset + 2) < (int) current_handle->size)
            {
              /* Longueur du Path */
              memcpy(&w_length,&data[w_offset],sizeof(WORD));
              if((w_offset + 2 + w_length) == current_handle->size)
                {
                  /* Chemin Complet */
                  memcpy(file_path,&data[w_offset+2],w_length);
                  file_path[w_length] = '\0';

                  /* Nom de fichier */
                  GetProdosFileName(file_path,file_name);

                  /* */
                  strcpy(information,file_name);
                }
            }
          free(data);
        }
      return;
    }

  /** SetupFile A410 **/
  if(current_handle->id == 0xA410 && current_handle->is_valid == 1)
    {
      data = LoadHandleData(current_handle,current_65816->current_mmu);
      if(data)
        {
          b_length = data[0];
          if(b_length == (current_handle->size-1))
            {
              memcpy(information,&data[1],b_length);
              information[b_length] = '\0';
            }
          free(data);
        }
      return;
    }

  /** Recherche dans la File ID List 7004 **/
  if(current_65816->table_file_id != NULL)
    {
      for(i=0; i<current_65816->table_file_id->nb_file_7004; i++)
        {
          if(current_65816->table_file_id->tab_id_file_7004[i]->id == current_handle->id && strlen(current_65816->table_file_id->tab_id_file_7004[i]->pathname) > 0)
            {
              /* Type de fichier */
              GetInternalFileType(current_handle->id,file_type);

              /* Chemin Complet */
              strcpy(file_path,current_65816->table_file_id->tab_id_file_7004[i]->pathname);

              /* Nom de fichier */
              GetProdosFileName(file_path,file_name);

              /* Information : Type + File Name */
              sprintf(information,"%s%s",file_type,file_name);

              return;
            }
        }
    }

  /** Recherche dans la File ID Table 7101 **/
  if(current_65816->table_file_id != NULL)
    {
      for(i=0; i<current_65816->table_file_id->nb_file_7101; i++)
        {
          if(current_65816->table_file_id->tab_id_file_7101[i]->file_id == current_handle->id && strlen(current_65816->table_file_id->tab_id_file_7101[i]->pathname) > 0)
            {
              /* Type de fichier */
              GetInternalFileType(current_handle->id,file_type);

              /* Chemin Complet */
              strcpy(file_path,current_65816->table_file_id->tab_id_file_7101[i]->pathname);

              /* Nom de fichier */
              GetProdosFileName(file_path,file_name);

              /* Information : Type + File Name */
              sprintf(information,"%s%s",file_type,file_name);

              return;
            }
        }
    }

  /** Prodos FST ? **/
  if((current_handle->id & 0xFF00) == 0x3300 && current_handle->is_valid == 1)
    {
      data = LoadHandleData(current_handle,current_65816->current_mmu);
      if(data)
        {
          for(i=0; i<(int)(current_handle->size-strlen("ProDOS FST")); i++)
            {
              if(!memcmp(&data[i],"ProDOS FST",strlen("ProDOS FST")))
                {
                  strcpy(information,"FST : Pro.FST");
                  free(data);
                  return;
                }
            }
          free(data);
        }
    }
}


/***********************************************************************/
/*  GetInternalFileType() :  Rcupre le type du fichier (NDA,CDA...). */
/***********************************************************************/
static void GetInternalFileType(WORD user_id, char *file_type_rtn)
{
  int i;
  struct s65816 *current_65816;

  /* Rcupration du Engine */
  my_Memory(MEMORY_GET_S65816,&current_65816,NULL);

  /* Init */
  strcpy(file_type_rtn,"");

  /** CDA **/
  for(i=0; i<current_65816->current_cda->nb_cda; i++)
    if(current_65816->current_cda->tab_cda[i].user_id == user_id)
      {
        strcpy(file_type_rtn,"CDA : ");
        return;
      }

  /** NDA **/
  for(i=0; i<current_65816->current_nda->nb_nda; i++)
    if(current_65816->current_nda->tab_nda[i].user_id == user_id)
      {
        strcpy(file_type_rtn,"NDA : ");
        return;
      }

  /** Inits **/
  for(i=0; i<current_65816->current_init->nb_init; i++)
    if(current_65816->current_init->tab_init[i]->user_id == user_id)
      {
        sprintf(file_type_rtn,"%s : ",current_65816->current_init->tab_init[i]->type);
        return;
      }

  /** Driver **/
  for(i=0; i<current_65816->current_driver->nb_driver; i++)
    if(current_65816->current_driver->tab_driver[i]->user_id == user_id)
      {
        strcpy(file_type_rtn,"Driver : ");
        return;
      }

  /** FST **/
  for(i=0; i<current_65816->current_fst->nb_fst; i++)
    if(current_65816->current_fst->tab_fst[i]->user_id == user_id)
      {
        strcpy(file_type_rtn,"FST : ");
        return;
      }
}


/********************************************************************/
/*  GetHandleFile() :  Rcupre le fichiers qui a allou le Handle. */
/********************************************************************/
void GetHandleFile(struct gs_handle *current_handle, char *file_name, char *file_path, int *segment_num, int *nb_segment, char *segment_name)
{
  int i, j;
  struct s65816 *current_65816;

  /* Rcupration du Engine */
  my_Memory(MEMORY_GET_S65816,&current_65816,NULL);

  /* Init */
  strcpy(file_name,"");
  strcpy(file_path,"");
  *segment_num = 1;
  *nb_segment = 1;
  strcpy(segment_name,"");

  /** Elments connus **/
  if(current_handle->id == 0x0001)
    {
      strcpy(file_name,"RAM Disk : RAM5");
      return;
    }

  /** Recherche dans la File ID Table 7101 **/
  if(current_65816->table_file_id != NULL)
    {
      for(i=0; i<current_65816->table_file_id->nb_file_7101; i++)
        {
          if(current_65816->table_file_id->tab_id_file_7101[i]->file_id == current_handle->id)
            {
              /* Chemin Complet */
              strcpy(file_path,current_65816->table_file_id->tab_id_file_7101[i]->pathname);

              /* Nom de fichier */
              strcpy(file_name,current_65816->table_file_id->tab_id_file_7101[i]->pathname);
              for(j=(int)strlen(file_path); j>=0; j--)
                if(file_path[j] == ':')
                  {
                    strcpy(file_name,&file_path[j+1]);
                    break;
                  }
              return;
            }
        }
    }
}


/*********************************************************/
/*  GetMemoryValue32Bit() :  Rcupre une valeur 32 bit. */
/*********************************************************/
static DWORD GetMemoryValue32Bit(DWORD ptr_address, struct mmu *current_mmu)
{
  DWORD value;

  /* Get 32 bit Value */
  value =  ReadMemoryByte(ptr_address+0,current_mmu)        | 
          (ReadMemoryByte(ptr_address+1,current_mmu) << 8)  |
          (ReadMemoryByte(ptr_address+2,current_mmu) << 16) |
          (ReadMemoryByte(ptr_address+3,current_mmu) << 24);

  /* Renvoi la valeur */
  return(value);
}


/*********************************************************/
/*  GetMemoryValue24Bit() :  Rcupre une valeur 24 bit. */
/*********************************************************/
static DWORD GetMemoryValue24Bit(DWORD ptr_address, struct mmu *current_mmu)
{
  DWORD value;

  /* Get 24 bit Value */
  value = ReadMemoryByte(ptr_address+0,current_mmu)       | 
         (ReadMemoryByte(ptr_address+1,current_mmu) << 8) |
         (ReadMemoryByte(ptr_address+2,current_mmu) << 16);

  /* Renvoi la valeur */
  return(value);
}


/*********************************************************/
/*  GetMemoryValue16Bit() :  Rcupre une valeur 16 bit. */
/*********************************************************/
static WORD GetMemoryValue16Bit(DWORD ptr_address, struct mmu *current_mmu)
{
  WORD value;

  /* Get 16 bit Value */
  value = ReadMemoryByte(ptr_address+0,current_mmu)       | 
         (ReadMemoryByte(ptr_address+1,current_mmu) << 8);

  /* Renvoi la valeur */
  return(value);
}


/****************************************/
/*  LoadIDTable() :  Charge l'ID Table. */
/****************************************/
struct file_id_table *LoadIDTable(struct gs_handle *first_handle, char *buffer_error, struct mmu *current_mmu)
{
  int i,  found;
  DWORD id_table_handle, express_handle;
  WORD path_name_offset, path_name_length;
  struct file_id_table *current_idtable;
  struct gs_handle *current_handle;
  unsigned char *data;

  /* Init */
  strcpy(buffer_error,"");

  /* Allocation mmoire */
  current_idtable = (struct file_id_table *) calloc(1,sizeof(struct file_id_table));
  if(current_idtable == NULL)
    {
      sprintf(buffer_error,"Error : Impossible to allocate memory for ID Table Structure");
      return(NULL);
    }

  /** Table des SystemLoader 7004 **/
  for(current_handle = first_handle; current_handle; current_handle = current_handle->next)
    if(current_handle->id == 0x7004)
      current_idtable->nb_file_7004++;

  /* Allocation mmoire Table */
  current_idtable->tab_id_file_7004 = (struct id_file_7004 **) calloc(current_idtable->nb_file_7004+1,sizeof(struct id_file_7004 *));
  if(current_idtable->tab_id_file_7004 == NULL)
    {
      sprintf(buffer_error,"Error : Memory Allocation Impossible");
      mem_free_fileidtable(current_idtable);
      return(NULL);
    }
  current_idtable->nb_file_7004 = 0;
  for(current_handle = first_handle; current_handle; current_handle = current_handle->next)
    if(current_handle->id == 0x7004 && current_handle->is_valid == 1)
      {
        /* Allocation mmoire */
        current_idtable->tab_id_file_7004[current_idtable->nb_file_7004] = (struct id_file_7004 *) calloc(1,sizeof(struct id_file_7004));
        if(current_idtable->tab_id_file_7004[current_idtable->nb_file_7004] == NULL)
          {
            sprintf(buffer_error,"Error : Memory Allocation Impossible");
            mem_free_fileidtable(current_idtable);
            return(NULL);
          }

        /* Remplissage */
        current_idtable->tab_id_file_7004[current_idtable->nb_file_7004]->current_handle = current_handle;
        current_idtable->tab_id_file_7004[current_idtable->nb_file_7004]->data = LoadHandleData(current_handle,current_mmu);
        if(current_idtable->tab_id_file_7004[current_idtable->nb_file_7004]->data == NULL)
          {
            sprintf(buffer_error,"Error : Memory Allocation Impossible");
            mem_free_fileidtable(current_idtable);
            return(NULL);
          }
        memcpy(&current_idtable->tab_id_file_7004[current_idtable->nb_file_7004]->id,&current_idtable->tab_id_file_7004[current_idtable->nb_file_7004]->data[8],sizeof(WORD));
        memcpy(&path_name_length,&current_idtable->tab_id_file_7004[current_idtable->nb_file_7004]->data[0x1E],sizeof(WORD));
        memcpy(current_idtable->tab_id_file_7004[current_idtable->nb_file_7004]->pathname,&current_idtable->tab_id_file_7004[current_idtable->nb_file_7004]->data[0x20],path_name_length);
        current_idtable->tab_id_file_7004[current_idtable->nb_file_7004]->pathname[path_name_length] = '\0';

        /* Une structure valide de plus */
        current_idtable->nb_file_7004++;
      }

  /** Recherche le Handle du 'System Loader' **/
  found = 0;
  for(current_handle = first_handle; current_handle; current_handle = current_handle->next)
    if(current_handle->id == GS_SYSTEM_LOADER_ID)
      {
        found = 1;
        break;
      }
  if(found == 0)
    {
      sprintf(buffer_error,"Error : Impossible to find 'System Loader' Memory Handle (ID=%X)",GS_SYSTEM_LOADER_ID);
      return(current_idtable);
    }

  /* Vrifie la taille de la Page Direct du System Loader */
  if(current_handle->size < (GS_TABLE_ID_HANDLE_OFFSET+3))
    {
      return(current_idtable);
    }
  /* Handle non valide */
  if(current_handle->is_valid == 0)
    return(current_idtable);

  /** Charge les donnes de la Page Direct du System Loader **/
  data = LoadHandleData(current_handle,current_mmu);
  if(data == NULL)
    {
      sprintf(buffer_error,"Error : Impossible to allocate Memory (%d bytes) to load 'System Loader' Direct Page data",current_handle->size);
      mem_free_fileidtable(current_idtable);
      return(current_idtable);
    }

  /* Rcupre le Handle de la ID Table */
  id_table_handle = data[GS_TABLE_ID_HANDLE_OFFSET+0]       | 
                   (data[GS_TABLE_ID_HANDLE_OFFSET+1] << 8) |
                   (data[GS_TABLE_ID_HANDLE_OFFSET+2] << 16);

  /* Libration mmoire */
  free(data);

  /* Rcupre le Handle de l'ID Table */
  current_idtable->table_id_handle_7201 = GetMemoryHandle(id_table_handle,current_mmu);
  if(current_idtable->table_id_handle_7201 == NULL)
    {
      mem_free_fileidtable(current_idtable);
      return(NULL);
    }

  /* Vrifie la taille des donnes (multiple de 16) */
  if((current_idtable->table_id_handle_7201->size & 0x0000000F) != 0x00000000)
    {
      return(current_idtable);
    }
  /* Handle non valide */
  if(current_idtable->table_id_handle_7201->is_valid == 0)
    return(current_idtable);

  /* Charge les donnes */
  current_idtable->table_id_data_7201 = LoadHandleData(current_idtable->table_id_handle_7201,current_mmu);
  if(current_idtable->table_id_data_7201 == NULL)
    {
      mem_free_fileidtable(current_idtable);
      return(NULL);
    }

  /* Allocation de la table */
  current_idtable->nb_file_7101 = current_idtable->table_id_handle_7201->size / 16;
  current_idtable->tab_id_file_7101 = (struct id_file_7101 **) calloc(current_idtable->nb_file_7101+1,sizeof(struct id_file_7101 *));
  if(current_idtable->tab_id_file_7101 == NULL)
    {
      mem_free_fileidtable(current_idtable);
      return(NULL);
    }

  /** On va analyser tous les fichiers possibles **/
  for(i=0; i<current_idtable->nb_file_7101; i++)
    {
      /* Allocation mmoire */
      current_idtable->tab_id_file_7101[i] = (struct id_file_7101 *) calloc(1,sizeof(struct id_file_7101));
      if(current_idtable->tab_id_file_7101[i] == NULL)
        {
          mem_free_fileidtable(current_idtable);
          return(NULL);
        }

      /* ID */
      memcpy(&current_idtable->tab_id_file_7101[i]->file_id,&current_idtable->table_id_data_7201[16*i+0],sizeof(WORD));
      if(current_idtable->tab_id_file_7101[i]->file_id != 0x0000)
        {
          /* Charge le ExpressHandle */
          memcpy(&express_handle,&current_idtable->table_id_data_7201[16*i+2],sizeof(DWORD));
          current_idtable->tab_id_file_7101[i]->express_handle = GetMemoryHandle(express_handle,current_mmu);
          if(current_idtable->tab_id_file_7101[i]->express_handle == NULL)
            {
              mem_free_fileidtable(current_idtable);
              return(NULL);
            }
          if(current_idtable->tab_id_file_7101[i]->express_handle->is_valid == 0)
            {
              mem_free_fileidtable(current_idtable);
              return(NULL);
            }

          /* Charge le Express Load Segment */
          current_idtable->tab_id_file_7101[i]->express_load_segment_data = LoadHandleData(current_idtable->tab_id_file_7101[i]->express_handle,current_mmu);
          if(current_idtable->tab_id_file_7101[i]->express_load_segment_data == NULL)
            {
              mem_free_fileidtable(current_idtable);
              return(NULL);
            }

          /* Rcupre le chemin du fichier */
          memcpy(&current_idtable->tab_id_file_7101[i]->ref_num,&current_idtable->tab_id_file_7101[i]->express_load_segment_data[0],sizeof(WORD));
          memcpy(&path_name_offset,&current_idtable->tab_id_file_7101[i]->express_load_segment_data[2],sizeof(WORD));
          if(path_name_offset < current_idtable->tab_id_file_7101[i]->express_handle->size)
            {
              memcpy(&path_name_length,&current_idtable->tab_id_file_7101[i]->express_load_segment_data[path_name_offset],sizeof(WORD));
              if(path_name_offset+2+path_name_length <= (int) current_idtable->tab_id_file_7101[i]->express_handle->size)
                {
                  memcpy(current_idtable->tab_id_file_7101[i]->pathname,&current_idtable->tab_id_file_7101[i]->express_load_segment_data[path_name_offset+2],path_name_length);
                  current_idtable->tab_id_file_7101[i]->pathname[path_name_length] = '\0';
                }
            }
        }
    }

  /* Renvoi la table */
  return(current_idtable);
}


/***********************************************************/
/*  GetCDATable() :  Rcupre la liste des CDA en mmoire. */
/***********************************************************/
struct cda_table *GetCDATable(char *buffer_error, struct gs_handle *first_used_handle, struct mmu *current_mmu)
{
  int i, j;
  DWORD cda_handle_addr, handle, cda_entry_addr;
  BYTE length;
  char cda_name[1024];
  struct cda_table *current_cdatable;
  struct gs_handle *current_handle;

  /* Init */
  strcpy(buffer_error,"");

  /* Allocation mmoire */
  current_cdatable = (struct cda_table *) calloc(1,sizeof(struct cda_table));
  if(current_cdatable == NULL)
    {
      sprintf(buffer_error,"Error : Impossible to allocate Memory for structure cda_table");
      return(NULL);
    }

  /* Nombre de CDA */
  current_cdatable->nb_cda = GetMemoryValue16Bit(GS_NB_CDA_ADDR,current_mmu);

  /* Allocation mmoire */
  current_cdatable->tab_cda = (struct cda *) calloc(current_cdatable->nb_cda,sizeof(struct cda));
  if(current_cdatable->tab_cda == NULL)
    {
      sprintf(buffer_error,"Error : Impossible to allocate Memory for cda table (%d entries)",current_cdatable->nb_cda);
      mem_free_cdatable(current_cdatable);
      return(NULL);
    }

  /* Aucun CDA disponible */
  if(current_cdatable->nb_cda == 0)
    return(current_cdatable);

  /* Adresse du Handle */
  cda_handle_addr = GetMemoryValue32Bit(GS_CDA_TABLE_ADDR,current_mmu);

  /* Charge le Handle */
  current_cdatable->cda_table_handle = GetMemoryHandle(cda_handle_addr,current_mmu);
  if(current_cdatable->cda_table_handle == NULL)
    {
      sprintf(buffer_error,"Error : Impossible to load Handle %02X/%04X",(cda_handle_addr>>16) & 0x00FF,cda_handle_addr & 0xFFFF);
      mem_free_cdatable(current_cdatable);
      return(NULL);
    }

  /* Handle non valide */
  if(current_cdatable->cda_table_handle->is_valid == 0)
    {
      sprintf(buffer_error,"Error : Invalid Memoy Handle %02X/%04X",(cda_handle_addr>>16) & 0x00FF,cda_handle_addr & 0xFFFF);
      mem_free_cdatable(current_cdatable);
      return(NULL);
    }

  /* Charge les donnes du Handle */
  current_cdatable->cda_table_data = LoadHandleData(current_cdatable->cda_table_handle,current_mmu);
  if(current_cdatable->cda_table_data == NULL)
    {
      sprintf(buffer_error,"Error : Impossible to allocate Memory (%d bytes) to load 'CDA Table' Handle data",current_cdatable->cda_table_handle->size);
      mem_free_cdatable(current_cdatable);
      return(NULL);
    }

  /** Rcupre les informations des CDA **/
  for(i=0; i<current_cdatable->nb_cda; i++)
    {
      /* Handle */
      memcpy(&handle,&current_cdatable->cda_table_data[i*sizeof(DWORD)],sizeof(DWORD));
      current_cdatable->tab_cda[i].handle = handle;

      /* +0 long pointer to CDA entry */
      cda_entry_addr = GetMemoryValue32Bit(handle,current_mmu);
      current_cdatable->tab_cda[i].cda_entry_address = cda_entry_addr;

      /* CDA entry : +0 [Byte] Longueur du Num */
      length = ReadMemoryByte(cda_entry_addr,current_mmu);

       /* CDA Name */
       ReadMemoryBlock(cda_entry_addr+1,length,(unsigned char *)&cda_name[0],current_mmu);
       cda_name[length] = '\0';

       /* Supprime le bit #7 de la chaine de caractre */
       for(j=0; j<length; j++)
         cda_name[j] &= 0x7F;
       current_cdatable->tab_cda[i].name = strdup(cda_name);
       if(current_cdatable->tab_cda[i].name == NULL)
         {
           sprintf(buffer_error,"Error : Impossible to allocate Memory to copy '%s' CDA Name",cda_name);
           mem_free_cdatable(current_cdatable);
           return(NULL);
         }

       /* Entry Address */
       current_cdatable->tab_cda[i].entry_addr = GetMemoryValue32Bit(cda_entry_addr+1+length,current_mmu);

       /* Exit Address */
       current_cdatable->tab_cda[i].exit_addr = GetMemoryValue32Bit(cda_entry_addr+1+length+sizeof(DWORD),current_mmu);

       /** Recherche l'ID **/
       for(current_handle = first_used_handle; current_handle; current_handle=current_handle->next)
         if(current_handle->handle == current_cdatable->tab_cda[i].handle)
           {
             current_cdatable->tab_cda[i].user_id = current_handle->id;
             break;
           }
    }

  /* Renvoi la table */
  return(current_cdatable);
}


/***********************************************************/
/*  GetNDATable() :  Rcupre la liste des NDA en mmoire. */
/***********************************************************/
struct nda_table *GetNDATable(char *buffer_error, struct mmu *current_mmu)
{
  int i, offset, name_length;
  WORD num_entries, active_flag, not_again;
  DWORD wap_table_addr, nda_list_addr, tick_address, task_queue_header;
  DWORD wap_handle_desk_manager;
  struct nda_table *current_ndatable;
  char nda_name[1024];

  /* Init */
  strcpy(buffer_error,"");

  /* Allocation mmoire */
  current_ndatable = (struct nda_table *) calloc(1,sizeof(struct nda_table));
  if(current_ndatable == NULL)
    {
      sprintf(buffer_error,"Error : Impossible to allocate Memory for structure nda_table");
      return(NULL);
    }

  /* On rcupre l'adresse de la Table WAP */
  wap_table_addr = GetMemoryValue32Bit(GS_WAP_TABLE_ADDR_PTR,current_mmu);

  /* Rcupre la 5me entre = Handle de la WAP du desk manager */
  wap_handle_desk_manager = GetMemoryValue32Bit(wap_table_addr+5*sizeof(DWORD),current_mmu);

  /* La premire entre est l'adresse le la liste des NDA */
  nda_list_addr = GetMemoryValue32Bit(wap_handle_desk_manager,current_mmu);
  if(nda_list_addr == 0x00000000)
    return(NULL);

  /* Rcupre les entres de la Global Data Structure (14 octets) */
  num_entries = GetMemoryValue16Bit(nda_list_addr+0,current_mmu);
  active_flag = GetMemoryValue16Bit(nda_list_addr+2,current_mmu);
  not_again = GetMemoryValue16Bit(nda_list_addr+4,current_mmu);
  tick_address = GetMemoryValue32Bit(nda_list_addr+6,current_mmu);
  task_queue_header = GetMemoryValue32Bit(nda_list_addr+10,current_mmu);

  /* Allocation mmoire */
  current_ndatable->nb_nda = num_entries;
  current_ndatable->tab_nda = (struct nda *) calloc(current_ndatable->nb_nda,sizeof(struct nda));
  if(current_ndatable->tab_nda == NULL)
    {
      sprintf(buffer_error,"Error : Impossible to allocate Memory for table tab_nda (%d entries)",current_ndatable->nb_nda);
      mem_free_ndatable(current_ndatable);
      return(NULL);
    }

  /** On boucle sur les NDA Record (36 octets) **/
  for(i=0; i<current_ndatable->nb_nda; i++)
    {
      offset = GS_SIZE_OF_GLOBAL_DATA + i*GS_SIZE_OF_NDA_RECORD;

      /* Rcupre les entres du NDA Record */
      current_ndatable->tab_nda[i].nda_status = GetMemoryValue16Bit(nda_list_addr+offset+0,current_mmu);      /* _NDAStatus */
      current_ndatable->tab_nda[i].open_addr = GetMemoryValue32Bit(nda_list_addr+offset+2,current_mmu);       /* _Open */
      current_ndatable->tab_nda[i].close_addr = GetMemoryValue32Bit(nda_list_addr+offset+6,current_mmu);      /* _Close */
      current_ndatable->tab_nda[i].action_addr = GetMemoryValue32Bit(nda_list_addr+offset+10,current_mmu);    /* _Action */
      current_ndatable->tab_nda[i].init_addr = GetMemoryValue32Bit(nda_list_addr+offset+14,current_mmu);      /* _Init */
      current_ndatable->tab_nda[i].period = GetMemoryValue16Bit(nda_list_addr+offset+18,current_mmu);         /* _Period */
      current_ndatable->tab_nda[i].event_mask = GetMemoryValue16Bit(nda_list_addr+offset+20,current_mmu);     /* _EventMask */
      current_ndatable->tab_nda[i].last_serviced = GetMemoryValue32Bit(nda_list_addr+offset+22,current_mmu);  /* _LastServiced */
      current_ndatable->tab_nda[i].window_ptr = GetMemoryValue32Bit(nda_list_addr+offset+26,current_mmu);     /* _WindowPtr */
      current_ndatable->tab_nda[i].handle = GetMemoryValue32Bit(nda_list_addr+offset+30,current_mmu);         /* _Handle */
      current_ndatable->tab_nda[i].user_id = GetMemoryValue16Bit(nda_list_addr+offset+34,current_mmu);        /* _UserID */

      /* Rcupre le Handle */
      current_ndatable->tab_nda[i].current_handle = GetMemoryHandle(current_ndatable->tab_nda[i].handle,current_mmu);
      if(current_ndatable->tab_nda[i].current_handle == NULL)
        {
          sprintf(buffer_error,"Error : Impossible to allocate Memory for Memory Handle : %02X/%04X",(current_ndatable->tab_nda[i].handle >> 16) & 0xFF,current_ndatable->tab_nda[i].handle & 0xFFFF);
          mem_free_ndatable(current_ndatable);
          return(NULL);
        }

      /* Handle non valide */
      if(current_ndatable->tab_nda[i].current_handle->is_valid == 0)
        {
          sprintf(buffer_error,"Error : Invalid Memory Handle : %02X/%04X",(current_ndatable->tab_nda[i].handle >> 16) & 0xFF,current_ndatable->tab_nda[i].handle & 0xFFFF);
          mem_free_ndatable(current_ndatable);
          return(NULL);
        }

      /* Donnes du Handle */
      current_ndatable->tab_nda[i].data = LoadHandleData(current_ndatable->tab_nda[i].current_handle,current_mmu);
      if(current_ndatable->tab_nda[i].data == NULL)
        {
          sprintf(buffer_error,"Error : Impossible to allocate Memory for Memory Handle Data : %02X/%04X / %d bytes",(current_ndatable->tab_nda[i].handle >> 16) & 0xFF,current_ndatable->tab_nda[i].handle & 0xFFFF,current_ndatable->tab_nda[i].current_handle->size);
          mem_free_ndatable(current_ndatable);
          return(NULL);
        }

      /* Extrait le nom du NDA dans les donnes */
      if(current_ndatable->tab_nda[i].current_handle->size < (GS_NDA_NAME_OFFSET+1))
        {
          sprintf(buffer_error,"Error : Wrong NDA Data length for NDA Hanlde : %02X/%04X / %d bytes",(current_ndatable->tab_nda[i].handle >> 16) & 0xFF,current_ndatable->tab_nda[i].handle & 0xFFFF,current_ndatable->tab_nda[i].current_handle->size);
          mem_free_ndatable(current_ndatable);
          return(NULL);
        }
      name_length = (int) current_ndatable->tab_nda[i].data[GS_NDA_NAME_OFFSET];
      memcpy(nda_name,&current_ndatable->tab_nda[i].data[GS_NDA_NAME_OFFSET+1],name_length);
      nda_name[name_length] = '\0';

      /* Allocation mmoire */
      current_ndatable->tab_nda[i].name = strdup(nda_name);
      if(current_ndatable->tab_nda[i].name == NULL)
        {
          sprintf(buffer_error,"Error : Impossible to allocate Memory for NDA Name : %s",nda_name);
          mem_free_ndatable(current_ndatable);
          return(NULL);
        }
    }

  /* Renvoi la table */
  return(current_ndatable);
}


/**************************************************************************************/
/*  GetQuickdrawVectorTable() :  Rcupre la liste des Vecteurs Quickdraw en mmoire. */
/**************************************************************************************/
struct quickdraw_vector_table *GetQuickdrawVectorTable(char *buffer_error, struct mmu *current_mmu)
{
  int i;
  struct quickdraw_vector_table *current_quickdraw_vector;
  char *tab_vector_name[] = 
    {
      "ToSTuff / ToSlab / ToSlice / ToStuffBig", "StdText", "StdLine", "StdRect", "StdRRect", "StdOval", "StdArc", "StdPoly",
      "StdRgn", "StdPixels", "StdComment", "StdTxMeas", "StdTxBnds", "StdGetPic", "StdPutPic", "PortLoc2ZP",
      "UserPat2ZP", "GetPenPReady", "GetBackPReady", "Pen2ZP", "Back2ZP", "RotatePat", "GetMaskReady", "Mask2ZP",
      "ExpandMask", "RotateMask", "RotateLong", "PreDraw", "XSetup", "DrawRectB", "DrawRect", "SliceSetup",
      "vShiftStay", "vXShiftStay", "vShiftOut", "vXShiftOut", "BMakeMinRect", "MakeMinRect", "ShieldCursor", "UnShieldCursor",
      "SetUpPortRegions", "SetUpScanPtrs", "FreePortRegions", "Free1Region", "Free2Regions", "Free3Regions", "GetFGS", "Init1Region",
      "Init2Regions", "Init3Regions", "Seek1Region", "Seek2Regions", "Seek3Regions", "InitRgn", "SeekRgn", "Join1Buffer",
      "Join2Buffers", "Join3Buffers", "SetSlabAdr", "SetSliceAdr", "SetStuffAdr", "BlowItUp", "PutPictStuff", "AddToPoly",
      "OpenPicture", "PicComment", "ClosePicture", "DrawPicture", "KillPicture", "GimmeVersion", "v_FrRgn", "v_PRgn",
      "v_ERgn", "v_IRgn", "v_FRgn", "v_FrRect", "v_PRect", "v_ERect", "v_IRect", "v_FRect",
      "DChar", "DText / DCString", "BChar", "BText / BCString", "ValidChar", "GetNextChar", "CalcDrawStatus", "TextAndMinRect",
      "CalcTextBounds", "SetUpFontInfo", "SetUpBufInfo", "GetTextHandle", "ClearBufPart", "PutChar", "BumpPen", "CleanUpBuffer",
      "PutLine", "PutRgn", "PutRect", "CullPoints", "SortPoints", "PackRgn", "RgnOp", "PutOval",
      "BumpOval", "InitOval", "SetTB", "ResetTB", "StyleText", "StyleExtents", "DoBold", "DoUnderLine",
      "StdConic", "DrawConic", "SetScaleAndMap", "DrawCursor", "UndrawCursor", "vAngleFromSlope", "vSlopeFromAngle", "RgnBlt",
      "SharedMRSetup", "ShareShieldStuff", "GetCharVals", "DoFntFlgMods", "AddExtras", "CalcSlop", "GetFntInf"
    };

  /* Init */
  strcpy(buffer_error,"");

  /* Allocation mmoire */
  current_quickdraw_vector = (struct quickdraw_vector_table *) calloc(1,sizeof(struct quickdraw_vector_table));
  if(current_quickdraw_vector == NULL)
    {
      sprintf(buffer_error,"Error : Impossible to allocate Memory for structure quickdraw_vector_table");
      return(NULL);
    }

  /** On va rcuprer les Vecteur Quickdraw **/
  current_quickdraw_vector->nb_quickdraw_vector = NB_QUICKDRAW_VECTOR;
  for(i=0; i<NB_QUICKDRAW_VECTOR; i++)
    {
      current_quickdraw_vector->tab_quickdraw_vector[i].number = (WORD) i;
      strcpy(current_quickdraw_vector->tab_quickdraw_vector[i].name,tab_vector_name[i]);
      current_quickdraw_vector->tab_quickdraw_vector[i].location = GS_QUICKDRAW_VECTOR_ADDR_RAM + 4*i;
      if(current_mmu->rom_version == ROM_APPLEIIGS_01)
        current_quickdraw_vector->tab_quickdraw_vector[i].vector_address_rom = GetMemoryValue32Bit(GS_QUICKDRAW_VECTOR_ADDR_ROM01 + 4*i,current_mmu);
      else
        current_quickdraw_vector->tab_quickdraw_vector[i].vector_address_rom = GetMemoryValue32Bit(GS_QUICKDRAW_VECTOR_ADDR_ROM03 + 4*i,current_mmu);
      current_quickdraw_vector->tab_quickdraw_vector[i].vector_address = GetMemoryValue32Bit(GS_QUICKDRAW_VECTOR_ADDR_RAM + 4*i,current_mmu);
    }

  /* Renvoi la table */
  return(current_quickdraw_vector);
}


/**************************************************************************************/
/*  GetInterruptVectorTable() :  Rcupre la liste des Vecteurs Interrupt en mmoire. */
/**************************************************************************************/
struct interrupt_vector_table *GetInterruptVectorTable(char *buffer_error, struct mmu *current_mmu)
{
  int i, j, is_error;
  struct interrupt_vector_table *current_interrupt_vector_table;
  struct interrupt_vector *current_interrupt_vector;
  char one_byte_txt[3] = "00";
  char *tab_vector_name[NB_INTERRUPT_VECTOR*7] = 
    {
                      /* Init Rom01  Init Rom03  Address    #Byte Format  Name       Information */
      /* $00/03F0 x2 */  "59FA0000", "59FA0000", "000003F0", "2", "ADR",  "BRKV",    "User BRK Vector",
      /* $00/03F2 x2 */  "00E00000", "00E00000", "000003F2", "2", "ADR",  "SOFTEV",  "User soft-entry Vector for RESET",
      /* $00/03F4 x1 */  "45000000", "45000000", "000003F4", "1", "BYTE", "PWREDUP", "EOR of High Byte of SOFTEV address",
      /* $00/03F5 x3 */  "4C03E000", "4C03E000", "000003F5", "3", "JMP", "AMPERV",  "Applesoft & JMP Vector",
      /* $00/03F8 x3 */  "4C65FF00", "4C65FF00", "000003F8", "3", "JMP", "USRADR",  "User Control-Y and Applesoft USR function JMP Vector",
      /* $00/03FB x3 */  "00000000", "00000000", "000003FB", "3", "JMP", "NMI",     "User NMI Vector",
      /* $00/03FE x2 */  "65FF0000", "65FF0000", "000003FE", "2", "ADR",  "IRQLOC",  "User IRQ Vector",

      /* $E1/0000 x4 JMP $07/6616 */  "5CAF00FE", "5CC500FE", "00E10000", "4", "JMPL", "DISPATCH1",    "Tool locator entry type 1",
      /* $E1/0004 x4 JMP $FE/00A3 */  "5CA300FE", "5CC200FE", "00E10004", "4", "JMPL", "DISPATCH2",    "Tool locator entry type 2",
      /* $E1/0008 x4 JMP $FE/0075 */  "5C7500FE", "5CA000FE", "00E10008", "4", "JMPL", "UDISPATCH1",   "User's Tool locator entry type 1",
      /* $E1/000C x4 JMP $FE/0069 */  "5C6900FE", "5C9D00FE", "00E1000C", "4", "JMPL", "UDISPATCH2",   "User's Tool locator entry type 2",
      /* $E1/0010 x4 JMP $07/424A */  "5CCCB7FF", "5C6CBCFF", "00E10010", "4", "JMPL", "INTMGRV",      "System Interrupt Manager",
      /* $E1/0014 x4 JMP $FF/B7BE */  "5CBEB7FF", "5C5EBCFF", "00E10014", "4", "JMPL", "COPMGRV",      "COP Manager",
      /* $E1/0018 x4 JMP $FF/B7BE */  "5CBEB7FF", "5C5EBCFF", "00E10018", "4", "JMPL", "ABORTMGRV",    "Abort Manager",
      /* $E1/001C x4 JMP $FF/A4E6 */  "5CE6A4FF", "5C33A5FF", "00E1001C", "4", "JMPL", "SYSDMGRV",     "System Failure Manager",
      /* $E1/0020 x4 JMP $FF/B5DE */  "5CDEB5FF", "5C18BAFF", "00E10020", "4", "JMPL", "IRQ.APTALK",   "AppleTalk Interrupt Handler",
      /* $E1/0024 x4 JMP $FF/B5DE */  "5CDEB5FF", "5C18BAFF", "00E10024", "4", "JMPL", "IRQ.SERIAL",   "Serial-port Interrupt Handler",
      /* $E1/0028 x4 JMP $04/38E7 */  "5CDEB5FF", "5C18BAFF", "00E10028", "4", "JMPL", "IRQ.SCAN",     "Scan-line Interrupt Handler",
      /* $E1/002C x4 JMP $FF/3FF4 */  "5CF43FFF", "5C62CDFC", "00E1002C", "4", "JMPL", "IRQ.SOUND",    "Sound Interrupt Handler",
      /* $E1/0030 x4 JMP $04/4304 */  "5CDEB5FF", "5C18BAFF", "00E10030", "4", "JMPL", "IRQ.VBL",      "VBL Handler",
      /* $E1/0034 x4 JMP $FE/C5C8 */  "5CDEB5FF", "5C18BAFF", "00E10034", "4", "JMPL", "IRQ.MOUSE",    "Mouse Interrupt Handler",
      /* $E1/0038 x4 JMP $FF/B5DE */  "5CDEB5FF", "5C18BAFF", "00E10038", "4", "JMPL", "IRQ.QTR",      "Quarter-second Interrupt Handler",
      /* $E1/003C x4 JMP $07/74E6 */  "5CDEB5FF", "5C18BAFF", "00E1003C", "4", "JMPL", "IRQ.KBD",      "Keyboard Interrupt Handler",
      /* $E1/0040 x4 JMP $FF/8485 */  "5C8584FF", "5C65DBFC", "00E10040", "4", "JMPL", "IRQ.RESPONSE", "ADB Response Interrupt Handler",
      /* $E1/0044 x4 JMP $FF/818A */  "5C8A81FF", "5C3AD8FC", "00E10044", "4", "JMPL", "IRQ.SRQ",      "ADB SRQ Interrupt Handler",
      /* $E1/0048 x4 JMP $FE/C686 */  "5CB9ADFE", "5C28CFFE", "00E10048", "4", "JMPL", "IRQ.DSKACC",   "Desk Accessory Interrupt Handler",
      /* $E1/004C x4 JMP $FE/C6A0 */  "5CDEB5FF", "5C18BAFF", "00E1004C", "4", "JMPL", "IRQ.FLUSH",    "Keyboard Flush Interrupt Handler",
      /* $E1/0050 x4 JMP $FF/B686 */  "5C86B6FF", "5CC0BAFF", "00E10050", "4", "JMPL", "IRQ.MICRO",    "Keyboard Micro Abort Interrupt Handler",
      /* $E1/0054 x4 JMP $FF/B5DE */  "5CDEB5FF", "5C18BAFF", "00E10054", "4", "JMPL", "IRQ.1SEC",     "1-second Interrupt Handler",
      /* $E1/0058 x4 JMP $FF/B5DE */  "5CDEB5FF", "5C18BAFF", "00E10058", "4", "JMPL", "IRQ.EXT",      "VGC External Interrupt Handler",
      /* $E1/005C x4 JMP $FF/B5DE */  "5CDEB5FF", "5C18BAFF", "00E1005C", "4", "JMPL", "IRQ.OTHER",    "Other Interrupt Handler",
      /* $E1/0060 x4 JMP $04/3891 */  "5CCE3FFE", "5CC052FE", "00E10060", "4", "JMPL", "CUPDATE",      "Cursor Update Vector",
      /* $E1/0064 x4 JMP $FF/C05C */  "5C5CC0FF", "5C07DDFC", "00E10064", "4", "JMPL", "INCBUSYFLG",   "Increment Busy Flag Vector",
      /* $E1/0068 x4 JMP $FF/BFEA */  "5CEABFFF", "5C94DCFC", "00E10068", "4", "JMPL", "DECBUSYFLG",   "Decrement Busy Flag Vector",
      /* $E1/006C x4 JMP $0B/BD2A */  "5CDEB5FF", "5C18BAFF", "00E1006C", "4", "JMPL", "BELLVECTOR",   "Monitor Bell Vector Interrupt routine",
      /* $E1/0070 x4 JMP $FF/B5DE */  "5CDEB5FF", "5C18BAFF", "00E10070", "4", "JMPL", "BREAKVECTOR",  "Break Vector",
      /* $E1/0074 x4 JMP $FF/B5DE */  "5CDEB5FF", "5C18BAFF", "00E10074", "4", "JMPL", "TRACEVECTOR",  "Trace Vector",
      /* $E1/0078 x4 JMP $FF/B5DE */  "5CDEB5FF", "5C18BAFF", "00E10078", "4", "JMPL", "STEPVECTOR",   "Step Vector",
      /* $E1/007C x4 JMP $FF/A684 */  "5C84A6FF", "5CF1A6FF", "00E1007C", "4", "JMPL", "ROMDISK",      "Install ROM  disk",
      /* $E1/0080 x4 JMP $FF/B51C */  "5C1CB5FF", "5C4EB9FF", "00E10080", "4", "JMPL", "TOWRITEBR",    "Write BatteryRam routine",
      /* $E1/0084 x4 JMP $FF/B53E */  "5C3EB5FF", "5C70B9FF", "00E10084", "4", "JMPL", "TOREADBR",     "Read BatteryRam routine",
      /* $E1/0088 x4 JMP $FF/B5E4 */  "5CE4B5FF", "5C1EBAFF", "00E10088", "4", "JMPL", "TOWRITETIME",  "Write Time routine",
      /* $E1/008C x4 JMP $FF/B5A0 */  "5CA0B5FF", "5CDAB9FF", "00E1008C", "4", "JMPL", "TOREADTIME",   "Read Time routine",
      /* $E1/0090 x4 JMP $FF/9283 */  "5C8392FF", "5CAC8FFF", "00E10090", "4", "JMPL", "TOCTRL.PANEL", "Show Control Panel",
      /* $E1/0094 x4 JMP $03/BFF3 */  "5CF4B6FF", "5C7ABBFF", "00E10094", "4", "JMPL", "TOBRAMSETUP",  "Setup System to BatteryRam parameters routine",
      /* $E1/0098 x4 JMP $FF/C4F3 */  "5CF3C4FF", "5CE7C4FF", "00E10098", "4", "JMPL", "TOPRINTMSG8",  "Print ASCII string designated by the 8-bit accumulator",
      /* $E1/009C x4 JMP $FF/C4F7 */  "5CF7C4FF", "5CEBC4FF", "00E1009C", "4", "JMPL", "TOPRINTMSG16", "Print ASCII string designated by the 16-bit accumulator",
      /* $E1/00A0 x4 JMP $00/FE19 */  "5C19FE00", "5C19FE00", "00E100A0", "4", "JMPL", "CTRLYVECTOR",  "User Control-Y Vector",
      /* $E1/00A4 x4 JMP $FF/91AC */  "5CAC91FF", "5C9E8EFF", "00E100A4", "4", "JMPL", "TOTEXTPG2DA",  "Point to Alternate Mode Desk Accessory",
      /* $E1/00A8 x4 JMP $08/24D8 */  "00000000", "00000000", "00E100A8", "4", "JMPL", "GSOS",         "GS/OS Calls Vector",
      /* $E1/00AC x4 RTL */           "00000000", "00000000", "00E100AC", "4", "JMPL", "OS",           "OS Vector",
      /* $E1/00B0 x4 JMP $08/2482 */  "00000000", "5CBBE6FC", "00E100B0", "4", "JMPL", "GSOS2",        "GS/OS Calls Vector #2",
      /* $E1/00B4 x4 RTL */           "00000000", "00000000", "00E100B4", "4", "JMPL", "OSP8Switch",   "Prodos 8 Switch Vector",
      /* $E1/00C0 x3 $FF/86BB */      "BB86FF00", "3084FF", "00E100C0", "3", "ADRL", "MSGPOINTER",   "Pointer to all strings used in Control Panel, Alt Display Mode and Monitor system messages",

      /* $FF/FFE4 x2 $C07B */  "7BC00000", "7BC00000", "00FFFFE4", "2", "ADR", "NCOP",    "Native-mode COP Vector",
      /* $FF/FFE6 x2 $C071 */  "71C00000", "71C00000", "00FFFFE6", "2", "ADR", "NBREAK",  "Native-mode BRK Vector",
      /* $FF/FFE8 x2 $C079 */  "79C00000", "79C00000", "00FFFFE8", "2", "ADR", "NABORT",  "Native-mode ABORT Vector",
      /* $FF/FFEA x2 $03FB */  "FB030000", "FB030000", "00FFFFEA", "2", "ADR", "NNMI",    "Native-mode NMI Vector",
      /* $FF/FFEC x2 $E2E6 */
      /* $FF/FFEE x2 $C074 */  "74C00000", "74C00000", "00FFFFEE", "2", "ADR", "NIRQ",    "Native-mode IRQ Vector",
      /* $FF/FFF0 x2 $144C */
      /* $FF/FFF2 x2 $00F9 */
      /* $FF/FFF4 x2 $C07B */  "7BC00000", "7BC00000", "00FFFFF4", "2", "ADR", "ECOP",    "Emulation-mode COP Vector",
      /* $FF/FFF6 x2 $67E9 */
      /* $FF/FFF8 x2 $C079 */  "79C00000", "79C00000", "00FFFFF8", "2", "ADR", "EABORT",  "Emulation-mode ABORT Vector",
      /* $FF/FFFA x2 $03FB */  "FB030000", "FB030000", "00FFFFFA", "2", "ADR", "ENMI",    "Emulation-mode NMI Vector",
      /* $FF/FFFC x2 $FA62 */  "62FA0000", "62FA0000", "00FFFFFC", "2", "ADR", "ERESET",  "RESET Vector",
      /* $FF/FFFE x2 $C074 */  "74C00000", "74C00000", "00FFFFFE", "2", "ADR", "EBRKIRQ", "Emulation-mode BRK/IRQ Vector"
    };

  /* Init */
  strcpy(buffer_error,"");

  /* Allocation mmoire */
  current_interrupt_vector_table = (struct interrupt_vector_table *) calloc(1,sizeof(struct interrupt_vector_table));
  if(current_interrupt_vector_table == NULL)
    {
      sprintf(buffer_error,"Error : Impossible to allocate Memory for structure interrupt_vector_table");
      return(NULL);
    }
  current_interrupt_vector_table->nb_interrupt_vector = 0;

  /** On va rcuprer les Vecteur Interrupt **/
  for(i=0; i<NB_INTERRUPT_VECTOR; i++)
    {
      current_interrupt_vector = &current_interrupt_vector_table->tab_interrupt_vector[current_interrupt_vector_table->nb_interrupt_vector];

      /* Location */
      sscanf(tab_vector_name[7*i+2],"%08X",&current_interrupt_vector->location);

      /* Nb Byte */
      current_interrupt_vector->nb_byte = atoi(tab_vector_name[7*i+3]);

      /* Format */
      if(!stricmp(tab_vector_name[7*i+4],"BYTE"))
        current_interrupt_vector->format = GS_INTERRUPT_FORMAT_BYTE;
      else if(!stricmp(tab_vector_name[7*i+4],"ADR"))
        current_interrupt_vector->format = GS_INTERRUPT_FORMAT_ADR;
      else if(!stricmp(tab_vector_name[7*i+4],"JMP"))
        current_interrupt_vector->format = GS_INTERRUPT_FORMAT_JMP;
      else if(!stricmp(tab_vector_name[7*i+4],"ADRL"))
        current_interrupt_vector->format = GS_INTERRUPT_FORMAT_ADRL;
      else if(!stricmp(tab_vector_name[7*i+4],"JMPL"))
        current_interrupt_vector->format = GS_INTERRUPT_FORMAT_JMPL;

      /* Name */
      strcpy(current_interrupt_vector->name,tab_vector_name[7*i+5]);

      /* Information */
      strcpy(current_interrupt_vector->information,tab_vector_name[7*i+6]);

      /* Donnes du Vecteur */
      for(j=0; j<current_interrupt_vector->nb_byte; j++)
        current_interrupt_vector->data[j] = ReadMemoryByte(current_interrupt_vector->location+j,current_mmu);

      /* Donnes Initiale en ROM */
      if(current_mmu->rom_version == ROM_APPLEIIGS_01)
        {
          for(j=0; j<current_interrupt_vector->nb_byte; j++)
            {
              memcpy(one_byte_txt,&tab_vector_name[7*i+0][2*j],2);
              current_interrupt_vector->data_rom[j] = EvaluateAsByte(one_byte_txt,&is_error);
            }
        }
      else
        {
          for(j=0; j<current_interrupt_vector->nb_byte; j++)
            {
              memcpy(one_byte_txt,&tab_vector_name[7*i+1][2*j],2);
              current_interrupt_vector->data_rom[j] = EvaluateAsByte(one_byte_txt,&is_error);
            }
        }

      /* Valeur du Vecteur */
      if(current_interrupt_vector->format == GS_INTERRUPT_FORMAT_BYTE)
        {
          current_interrupt_vector->vector_address = current_interrupt_vector->data[0];
          sprintf(current_interrupt_vector->vector_address_txt,"#$%02X",current_interrupt_vector->data[0]);

          current_interrupt_vector->vector_address_rom = current_interrupt_vector->data_rom[0];
          sprintf(current_interrupt_vector->vector_address_rom_txt,"#$%02X",current_interrupt_vector->data_rom[0]);
        }
      else if(current_interrupt_vector->format == GS_INTERRUPT_FORMAT_ADR)
        {
          current_interrupt_vector->vector_address = current_interrupt_vector->data[0] | (current_interrupt_vector->data[1] << 8);
          sprintf(current_interrupt_vector->vector_address_txt,"$%02X%02X",current_interrupt_vector->data[1],current_interrupt_vector->data[0]);

          current_interrupt_vector->vector_address_rom = current_interrupt_vector->data_rom[0] | (current_interrupt_vector->data_rom[1] << 8);
          sprintf(current_interrupt_vector->vector_address_rom_txt,"$%02X%02X",current_interrupt_vector->data_rom[1],current_interrupt_vector->data_rom[0]);
        }
      else if(current_interrupt_vector->format == GS_INTERRUPT_FORMAT_ADRL)
        {
          current_interrupt_vector->vector_address = current_interrupt_vector->data[0] | (current_interrupt_vector->data[1] << 8) | (current_interrupt_vector->data[2] << 16);
          sprintf(current_interrupt_vector->vector_address_txt,"$%02X/%02X%02X",current_interrupt_vector->data[2],current_interrupt_vector->data[1],current_interrupt_vector->data[0]);

          current_interrupt_vector->vector_address_rom = current_interrupt_vector->data_rom[0] | (current_interrupt_vector->data_rom[1] << 8) | (current_interrupt_vector->data_rom[2] << 16);
          sprintf(current_interrupt_vector->vector_address_rom_txt,"$%02X/%02X%02X",current_interrupt_vector->data_rom[2],current_interrupt_vector->data_rom[1],current_interrupt_vector->data_rom[0]);
        }
      else if(current_interrupt_vector->format == GS_INTERRUPT_FORMAT_JMP)
        {
          /* JMP $AABB */
          if(current_interrupt_vector->data[0] == 0x4C)
            {
              current_interrupt_vector->vector_address = current_interrupt_vector->data[1] | (current_interrupt_vector->data[2] << 8);
              sprintf(current_interrupt_vector->vector_address_txt,"JMP  $%02X%02X",current_interrupt_vector->data[2],current_interrupt_vector->data[1]);
            }
          /* BRK */
          else if(current_interrupt_vector->data[0] == 0x00)
            {
              current_interrupt_vector->vector_address = 0x0000;
              sprintf(current_interrupt_vector->vector_address_txt,"BRK  $#%02X",current_interrupt_vector->data[1]);
            }
          /* Other */
          else
            {
              current_interrupt_vector->vector_address = 0x0000;
              sprintf(current_interrupt_vector->vector_address_txt,"%02X %02X %02X",current_interrupt_vector->data[0],current_interrupt_vector->data[1],current_interrupt_vector->data[2]);
            }

          /* JMP $AABB */
          if(current_interrupt_vector->data_rom[0] == 0x4C)
            {
              current_interrupt_vector->vector_address_rom = current_interrupt_vector->data_rom[1] | (current_interrupt_vector->data_rom[2] << 8);
              sprintf(current_interrupt_vector->vector_address_rom_txt,"JMP  $%02X%02X",current_interrupt_vector->data_rom[2],current_interrupt_vector->data_rom[1]);
            }
          /* BRK */
          else if(current_interrupt_vector->data_rom[0] == 0x00)
            {
              current_interrupt_vector->vector_address_rom = 0x0000;
              sprintf(current_interrupt_vector->vector_address_rom_txt,"BRK  $#%02X",current_interrupt_vector->data_rom[1]);
            }
          /* Other */
          else
            {
              current_interrupt_vector->vector_address_rom = 0x0000;
              sprintf(current_interrupt_vector->vector_address_rom_txt,"%02X %02X %02X",current_interrupt_vector->data_rom[0],current_interrupt_vector->data[1],current_interrupt_vector->data_rom[2]);
            }
        }
      else if(current_interrupt_vector->format == GS_INTERRUPT_FORMAT_JMPL)
        {
          /* JMP $AA/BBCC */
          if(current_interrupt_vector->data[0] == 0x5C)
            {
              current_interrupt_vector->vector_address = current_interrupt_vector->data[1] | (current_interrupt_vector->data[2] << 8) | (current_interrupt_vector->data[3] << 16);
              sprintf(current_interrupt_vector->vector_address_txt,"JMP  $%02X/%02X%02X",current_interrupt_vector->data[3],current_interrupt_vector->data[2],current_interrupt_vector->data[1]);
            }
          /* RTL */
          else if(current_interrupt_vector->data[0] == 0x6B)
            {
              current_interrupt_vector->vector_address = 0x00;
              strcpy(current_interrupt_vector->vector_address_txt,"RTL");
            }
          /* Other */
          else
            {
              current_interrupt_vector->vector_address = 0x00;
              sprintf(current_interrupt_vector->vector_address_txt,"%02X %02X %02X %02X",current_interrupt_vector->data[0],current_interrupt_vector->data[1],current_interrupt_vector->data[2],current_interrupt_vector->data[3]);
            }

          /* JMP $AA/BBCC */
          if(current_interrupt_vector->data_rom[0] == 0x5C)
            {
              current_interrupt_vector->vector_address_rom = current_interrupt_vector->data_rom[1] | (current_interrupt_vector->data_rom[2] << 8) | (current_interrupt_vector->data_rom[3] << 16);
              sprintf(current_interrupt_vector->vector_address_rom_txt,"JMP  $%02X/%02X%02X",current_interrupt_vector->data_rom[3],current_interrupt_vector->data_rom[2],current_interrupt_vector->data_rom[1]);
            }
          /* RTL */
          else if(current_interrupt_vector->data_rom[0] == 0x6B)
            {
              current_interrupt_vector->vector_address_rom = 0x00;
              strcpy(current_interrupt_vector->vector_address_rom_txt,"RTL");
            }
          /* Other */
          else
            {
              current_interrupt_vector->vector_address_rom = 0x00;
              sprintf(current_interrupt_vector->vector_address_rom_txt,"%02X %02X %02X %02X",current_interrupt_vector->data_rom[0],current_interrupt_vector->data_rom[1],current_interrupt_vector->data_rom[2],current_interrupt_vector->data_rom[3]);
            }
        }

      /* Next Entry */
      current_interrupt_vector_table->nb_interrupt_vector++;
    }

  /* Renvoi la table */
  return(current_interrupt_vector_table);
}


/*******************************************************************/
/*  LoadHandleData() :  Charge en mmoire les donnes d'un Handle. */
/*******************************************************************/
unsigned char *LoadHandleData(struct gs_handle *current_handle, struct mmu *current_mmu)
{
  unsigned char *data;

  /* Allocation mmoire */
  data = (unsigned char *) calloc(current_handle->size,sizeof(unsigned char));
  if(data == NULL)
    return(NULL);

  /* Rcupration des donnes */
  ReadMemoryBlock(current_handle->address,current_handle->size,data,current_mmu);

  /* Renvoi les donnes */
  return(data);
}


/****************************************************************/
/*  GetProdosFileName() :  Extrait le nom de fichier du chemin. */
/****************************************************************/
void GetProdosFileName(char *file_path, char *file_name_rtn)
{
  int i;

  /* Init */
  strcpy(file_name_rtn,file_path);

  /* Nom de fichier */
  for(i=(int)strlen(file_path); i>=0; i--)
    if(file_path[i] == ':')
      {
        strcpy(file_name_rtn,&file_path[i+1]);
        break;
      }
}


/*****************************************************************************/
/*  GetHandleListInfo() :  Dtermine le nombre de handle et les allocations. */
/*****************************************************************************/
void GetHandleListInfo(struct gs_handle *first_handle, int *nb_handle_rtn, int *size_allocated_rtn)
{
  DWORD address_max;
  int nb_handle, size_allocated;
  struct gs_handle *current_handle;
  struct s65816 *current_65816;

  /* Rcupration du Engine */
  my_Memory(MEMORY_GET_S65816,&current_65816,NULL);

  /* Init */
  nb_handle = 0;
  size_allocated = 0;
  address_max = (current_65816->current_mmu->nb_extended_bank+2) << 16;

  /** On passe en revue tous les Handles **/
  for(current_handle = first_handle; current_handle; current_handle = current_handle->next)
    {
      nb_handle++;
      if((current_handle->size & 0xFF000000) != 0x00000000)
        continue;
      if((current_handle->address + current_handle->size) > address_max)
        continue;
      size_allocated += current_handle->size;
    }

  /* Renvoi les valeurs */
  *nb_handle_rtn = nb_handle;
  *size_allocated_rtn = size_allocated;
}


/************************************************************************************/
/*  GetFileSegmentList() :  Construit la liste des File/Segment prsent en mmoire. */
/************************************************************************************/
struct file_segment_table *GetFileSegmentList(struct gs_handle *first_gs_handle, struct mmu *current_mmu)
{
  int i, nb_file_segment, nb_sub_segment, nb_separator;
  struct file_segment *first_file_segment;
  struct file_segment *last_file_segment;
  struct file_segment *current_file_segment;
  struct file_segment *begin_file_segment;
  struct file_segment *end_file_segment;
  struct file_segment *next_file_segment;
  struct file_segment_table *current_filesegmenttable;
  struct gs_handle *current_gs_handle;
  struct gs_handle *current_prodosfst_handle;
  unsigned char *data;
  char file_path[1024];

  /* Init */
  nb_file_segment = 0;
  first_file_segment = NULL;
  last_file_segment = NULL;

  /*** Dcodage System Loader 7004 ***/
  for(current_gs_handle = first_gs_handle; current_gs_handle; current_gs_handle = current_gs_handle->next)
    if(current_gs_handle->id == 0x7004)
      {
        /* Dcode les informations */
        current_file_segment = DecodeFileSegment7004(current_gs_handle,first_gs_handle,current_mmu);

        /* Rattachement */
        if(current_file_segment != NULL)
          {
            if(first_file_segment == NULL)
              first_file_segment = current_file_segment;
            else
              last_file_segment->next = current_file_segment;
            last_file_segment = current_file_segment;
            nb_file_segment++;
          }
      }

  /*** Dcodage System Loader 7101 : Multi-Segment ***/
  for(current_gs_handle = first_gs_handle; current_gs_handle; current_gs_handle = current_gs_handle->next)
    if(current_gs_handle->id == 0x7101)
      {
        /* Dcode les informations */
        begin_file_segment = DecodeFileSegment7101(current_gs_handle,current_mmu);

        /* Rattachement */
        if(begin_file_segment != NULL)
          {
            /* Recherche le dernier */
            nb_sub_segment = 1;
            for(end_file_segment = begin_file_segment; end_file_segment->next != NULL; end_file_segment = end_file_segment->next)
              nb_sub_segment++;

            /* Ajoute */
            if(first_file_segment == NULL)
              first_file_segment = begin_file_segment;
            else
              last_file_segment->next = begin_file_segment;
            last_file_segment = end_file_segment;
            nb_file_segment += nb_sub_segment;
          }
      }

  /** Cration du fichier Prodos.FST **/
  current_prodosfst_handle = NULL;
  for(current_gs_handle = first_gs_handle; current_gs_handle; current_gs_handle = current_gs_handle->next)
    if((current_gs_handle->id & 0xFF00) == 0x3300 && current_gs_handle->is_valid == 1)
      {
        data = LoadHandleData(current_gs_handle,current_mmu);
        if(data)
          {
            for(i=0; i<(int)(current_gs_handle->size-strlen("ProDOS FST")); i++)
              {
                if(!memcmp(&data[i],"ProDOS FST",strlen("ProDOS FST")))
                  {
                    current_prodosfst_handle = current_gs_handle;
                    break;
                  }
              }
            free(data);
          }
        if(current_prodosfst_handle != NULL)
          break;
      }
  /* Allocation mmoire + Stockage */
  if(current_prodosfst_handle != NULL)
    {
      /* Rcupration du chemin : Utilise un Tool */
      strcpy(file_path,"");
      for(current_file_segment = first_file_segment; current_file_segment; current_file_segment = current_file_segment->next)
        if((current_file_segment->user_id & 0xFF00) == 0x4000)
          {
            strcpy(file_path,current_file_segment->file_path);
            break;
          }
      nb_separator = 0;
      for(i=(int)strlen(file_path); i>=0; i--)
        {
          if(file_path[i] == ':' && nb_separator == 0)
            nb_separator++;
          else if(file_path[i] == ':' && nb_separator == 1)
            {
              file_path[i] = '\0';
              break;
            }
        }
      strcat(file_path,":FSTS:Pro.FST");

      /* Allocation */
      current_file_segment = mem_alloc_file_segment(":D3:SYSTEM:FSTS:Pro.FST","main",2,current_prodosfst_handle->handle,current_prodosfst_handle->address,current_prodosfst_handle->size,current_prodosfst_handle->id,current_prodosfst_handle);
      if(current_file_segment != NULL)
        {
          if(first_file_segment == NULL)
            first_file_segment = current_file_segment;
          else
            last_file_segment->next = current_file_segment;
          last_file_segment = current_file_segment;
          nb_file_segment++;
        }
    }

  /** Allocation mmoire **/
  current_filesegmenttable = (struct file_segment_table *) calloc(1,sizeof(struct file_segment_table));
  if(current_filesegmenttable == NULL)
    {
      for(current_file_segment=first_file_segment; current_file_segment; current_file_segment=next_file_segment)
        {
          next_file_segment = current_file_segment->next;
          mem_free_filesegment(current_file_segment);
        }
      return(NULL);
    }

  /* Cration du tableau */
  current_filesegmenttable->tab_file_segment = (struct file_segment **) calloc(nb_file_segment,sizeof(struct file_segment *));
  if(current_filesegmenttable == NULL)
    {
      for(current_file_segment=first_file_segment; current_file_segment; current_file_segment=next_file_segment)
        {
          next_file_segment = current_file_segment->next;
          mem_free_filesegment(current_file_segment);
        }
      free(current_filesegmenttable);
      return(NULL);
    }
  
  /* Remplissage */
  current_filesegmenttable->nb_segment_table = nb_file_segment;
  for(i=0,current_file_segment=first_file_segment; current_file_segment; current_file_segment=current_file_segment->next,i++)
    current_filesegmenttable->tab_file_segment[i] = current_file_segment;

  /* Tri du tableau (par File Path) */
  if(current_filesegmenttable->nb_segment_table > 1)
    {
      qsort((void *)current_filesegmenttable->tab_file_segment,(size_t)current_filesegmenttable->nb_segment_table,sizeof(struct file_segment *),compare_filesegment);
      current_filesegmenttable->tab_file_segment[current_filesegmenttable->nb_segment_table-1]->next = NULL;
      if(current_filesegmenttable->nb_segment_table > 1)
        for(i=0; i<current_filesegmenttable->nb_segment_table-1; i++)
          current_filesegmenttable->tab_file_segment[i]->next = current_filesegmenttable->tab_file_segment[i+1];
    }

  /* Renvoi la table */
  return(current_filesegmenttable);
}


/**********************************************************************************/
/*  DecodeFileSegment7004() :  Dcode un File Segment pour un system Loader 7004. */
/**********************************************************************************/
static struct file_segment *DecodeFileSegment7004(struct gs_handle *current_gs_handle, struct gs_handle *first_gs_handle, struct mmu *current_mmu)
{
  unsigned char *handle_data;
  WORD name_length, user_id, user_7003_id;
  DWORD handle, address, size;
  struct gs_handle *current_handle;
  struct file_segment *current_file_segment;
  char file_path[1024] = "";

  /* Pas assez de donnes */
  if(current_gs_handle->size <= 0x20)
    return(NULL);
  /* Handle non valide */
  if(current_gs_handle->is_valid == 0)
    return(NULL);

  /** Rcupre les donnes du Handle **/
  handle_data = LoadHandleData(current_gs_handle,current_mmu);
  if(handle_data == NULL)
    return(NULL);

  /* User ID du fichier */
  memcpy(&user_id,&handle_data[0x08],sizeof(WORD));

  /* Rcupre le chemin du fichier */
  memcpy(&name_length,&handle_data[0x1E],sizeof(WORD));
  if((0x20 + name_length) == current_gs_handle->size)
    {
      /* Chemin Complet */
      memcpy(file_path,&handle_data[0x20],name_length);
      file_path[name_length] = '\0';
    }

  /* Libration mmoire */
  free(handle_data);

  /*** On va chercher le Handle dans un System Loader 7003 ***/
  handle = 0x00000000;
  for(current_handle = first_gs_handle; current_handle; current_handle = current_handle->next)
    if(current_handle->id == 0x7003 && current_handle->size == 0x14 && current_handle->is_valid == 1)
      {
        handle_data = LoadHandleData(current_handle,current_mmu);
        if(handle_data != NULL)
          {
            memcpy(&user_7003_id,&handle_data[8],sizeof(WORD));
            if(user_7003_id == user_id)
              memcpy(&handle,&handle_data[10],sizeof(DWORD));
            free(handle_data);
          }
        if(handle != 0x00000000)
          break;
      }

  /** Addresse et Size **/
  address = 0x00000000;
  size = 0x00000000;
  if(handle != 0x00000000)
    {
      current_handle = GetMemoryHandle(handle,current_mmu);
      if(current_handle != NULL)
        {
          address = current_handle->address;
          size = current_handle->size;
          free(current_handle);
        }
    }

  /* Alocation mmoire */
  current_file_segment = mem_alloc_file_segment(file_path,"",1,handle,address,size,user_id,current_gs_handle);

  /* Renvoi le segment */
  return(current_file_segment);
/*
E0/7554

33/DF5D   000000   2C 75 E0 00  -> System Loader ZipDA
                   B4 74 E0 00  -> System Loader Where
                   11 50        User ID = 5011
                   01 00 00 2B 0E 5E 13 0B 00 03 00 00 00 00.01 00 3F D0 08 00 
                  [1F 00]
                   3A 44 33 3A 53 59 53 54.45 4D 3A 44 45 53 4B 2E  :D3:SYSTEM:DESK.
                   41 43 43 53 3A 58 59 54.6F 53 63 72 65 65 6E     ACCS:XYToScreen 
Handle  : E0/7540
Address : 08/D03F
*/
}


/**********************************************************************************/
/*  DecodeFileSegment7101() :  Dcode un File Segment pour un system Loader 7101. */
/**********************************************************************************/
static struct file_segment *DecodeFileSegment7101(struct gs_handle *current_gs_handle, struct mmu *current_mmu)
{
  int i, offset_segment, label_length;
  DWORD handle, address, size;
  WORD offset_file_path, length_file_path, nb_segment, segment_num, user_id;
  BYTE length_segment;
  struct gs_handle *current_file_handle;
  unsigned char *handle_data;
  struct file_segment *current_file_segment;
  struct file_segment *first_file_segment;
  struct file_segment *last_file_segment;
  char file_path[1024] = "";
  char load_name[1024] = "";
  char segment_name[1024] = "";
  unsigned char zero_12[12] = {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};

  /* Init */
  first_file_segment = NULL;
  last_file_segment = NULL;

  /* Pas assez de donnes */
  if(current_gs_handle->size <= 4)
    return(NULL);
  /* Handle invalide */
  if(current_gs_handle->is_valid == 0)
    return(NULL);

  /** Rcupre les donnes du Handle **/
  handle_data = LoadHandleData(current_gs_handle,current_mmu);
  if(handle_data == NULL)
    return(NULL);

  /* Offset de la longueur du Path */
  memcpy(&offset_file_path,&handle_data[0x02],sizeof(WORD));
  if((DWORD)(offset_file_path + 2) >= current_gs_handle->size)
    return(NULL);
  memcpy(&length_file_path,&handle_data[offset_file_path],sizeof(WORD));
  
  /* Chemin Complet */
  memcpy(file_path,&handle_data[offset_file_path+2],length_file_path);
  file_path[length_file_path] = '\0';

  /** On va rcuprr les informations de tous les Segments **/
  memcpy(&nb_segment,&handle_data[0x04],sizeof(WORD));
  nb_segment++;
  offset_segment = 6 + 8*nb_segment + 2*nb_segment;
  for(i=0; i<nb_segment; i++)
    {
      /* Handle */
      memcpy(&handle,&handle_data[6+8*i+4],sizeof(DWORD));

      /* Address + Size */
      address = 0x00000000;
      size = 0x00000000;
      user_id = 0x0000;
      if(handle != 0x00000000)
        {
          current_file_handle = GetMemoryHandle(handle,current_mmu);
          if(current_file_handle != NULL)
            {
              address = current_file_handle->address;
              size = current_file_handle->size;
              user_id = current_file_handle->id;
              free(current_file_handle);
            }
        }

      /* Label Length */
      label_length = (int) handle_data[offset_segment+17];

      /* Segment Number */
      memcpy(&segment_num,&handle_data[offset_segment+38],sizeof(WORD));

      /* Load Name */
      if(memcmp(&handle_data[offset_segment+46],zero_12,12))   /* 12 Zero qui se suivent */
        {
          memcpy(load_name,&handle_data[offset_segment+48],10);
          load_name[10] = '\0';
        }
      else
        strcpy(load_name,"");

      /* Segment Name */
      if(label_length == 0)
        {
          /* Si Label Length est  0, 1 Byte code la longueur */
          memcpy(&length_segment,&handle_data[offset_segment+58],sizeof(BYTE));
          memcpy(segment_name,&handle_data[offset_segment+58+1],length_segment);
          segment_name[length_segment] = '\0';

          /* MAJ Offset */
          offset_segment += (58 + 1 + length_segment);
        }
      else
        {
          memcpy(segment_name,&handle_data[offset_segment+58],label_length);
          segment_name[label_length] = '\0';

          /* MAJ Offset */
          offset_segment += (58 + label_length);
        }

      /* Allocation mmoire */
      current_file_segment = mem_alloc_file_segment(file_path,segment_name,(int)segment_num,handle,address,size,user_id,current_gs_handle);
      if(current_file_segment)
        {
          if(first_file_segment == NULL)
            first_file_segment = current_file_segment;
          else
            last_file_segment->next = current_file_segment;
          last_file_segment = current_file_segment;
        }
    }

  /* On va uniformiser les User ID */
  user_id = 0x0000;
  for(current_file_segment=first_file_segment; current_file_segment; current_file_segment=current_file_segment->next)
    if(current_file_segment->user_id != 0x0000 && user_id == 0x0000)
      user_id = current_file_segment->user_id;
  for(current_file_segment=first_file_segment; current_file_segment; current_file_segment=current_file_segment->next)
    if(current_file_segment->user_id == 0x0000)
      current_file_segment->user_id = user_id;

  /* Renvoi la liste des Segments */
  return(first_file_segment);
}


/*********************************************************************************/
/*  mem_alloc_file_segment() :  Allocation mmoire de la structure file_segment. */
/*********************************************************************************/
static struct file_segment *mem_alloc_file_segment(char *file_path, char *segment_name, int segment_num, DWORD handle, DWORD address, DWORD size, WORD user_id, struct gs_handle *current_handle)
{
  char file_name[1024];
  struct file_segment *current_file_segment;

  /* Allocation mmoire */
  current_file_segment = (struct file_segment *) calloc(1,sizeof(struct file_segment));
  if(current_file_segment == NULL)
    return(NULL);

  /* Identifie le nom  la fin */
  GetProdosFileName(file_path,file_name);

  /* Remplissage */
  current_file_segment->file_path = strdup(file_path);
  current_file_segment->file_name = strdup(file_name);
  current_file_segment->segment_name = strdup(segment_name);
  current_file_segment->segment_num = segment_num;
  current_file_segment->handle = handle;
  current_file_segment->address = address;
  current_file_segment->size = size;
  current_file_segment->current_handle = current_handle;
  current_file_segment->user_id = user_id;

  /* Vrifie les allocations mmoires */
  if(current_file_segment->file_path == NULL || current_file_segment->file_name == NULL || current_file_segment->segment_name == NULL)
    {
      mem_free_filesegment(current_file_segment);
      return(NULL);
    }

  /* On compacte le nom du Segment */
  CleanBuffer(current_file_segment->segment_name);

  /* Renvoi la structure */
  return(current_file_segment);
}


/************************************************************************/
/*  compare_filesegment() : Fonction de comparaison pour le Quick Sort. */
/************************************************************************/
static int compare_filesegment(const void *data_1, const void *data_2)
{
  int result;
  struct file_segment *file_segment_1;
  struct file_segment *file_segment_2;

  /* Rcupration des paramtres */
  file_segment_1 = *((struct file_segment **) data_1);
  file_segment_2 = *((struct file_segment **) data_2);

  /* Comparaison des Chemins + Segment Num */
  result = stricmp(file_segment_1->file_path,file_segment_2->file_path);
  if(result == 0)
    {
      if(file_segment_1->segment_num < file_segment_2->segment_num)
        return(-1);
      else if(file_segment_1->segment_num > file_segment_2->segment_num)
        return(1);
      else
        return(0);
    }
  else
    return(result);
}


/***************************************************************/
/*  GetInitTable() :  Construit le tableau des Init (TIF+PIF). */
/***************************************************************/
struct init_table *GetInitTable(struct s65816 *current_s65816)
{
  int i, nb_init, already;
  struct init_table *current_inittable;
  struct init *current_init;
  struct gs_handle *current_used_handle;

  /* Nombre Max de Init */
  nb_init = 0;
  for(current_used_handle = current_s65816->list_used_handle; current_used_handle; current_used_handle = current_used_handle->next)
    if((current_used_handle->id & 0xFF00) == 0xA000)
      nb_init++;

  /* Allocation mmoire */
  current_inittable = (struct init_table *) calloc(1,sizeof(struct init_table));
  if(current_inittable == NULL)
    return(NULL);
  current_inittable->tab_init = (struct init **) calloc(nb_init,sizeof(struct init *));
  if(current_inittable->tab_init == NULL)
    {
      free(current_inittable);
      return(NULL);
    }

  /** Remplissage de la table **/
  nb_init = 0;
  for(current_used_handle = current_s65816->list_used_handle; current_used_handle; current_used_handle = current_used_handle->next)
    if((current_used_handle->id & 0xFF00) == 0xA000)
      {
        /* A t'on dj cet Init dans la liste ? */
        for(i=0,already=0; i<nb_init; i++)
          if(current_inittable->tab_init[i]->user_id == current_used_handle->id)
            {
              already = 1;
              break;
            }
        if(already == 1)
          continue;

        /** On ajoute cet Init dans la table **/
        /* Allocation mmoire */
        current_init = (struct init *) calloc(1,sizeof(struct init));
        if(current_init == NULL)
          {
            mem_free_inittable(current_inittable);
            return(NULL);
          }
        /* Remplissage */
        current_init->handle = current_used_handle->handle;
        current_init->address = current_used_handle->address;
        current_init->size = current_used_handle->size;
        current_init->user_id = current_used_handle->id;

        /* Cherche le chemin parmi les fichiers */
        for(i=0; i<current_s65816->current_file_segment->nb_segment_table; i++)
          if(current_s65816->current_file_segment->tab_file_segment[i]->user_id == current_init->user_id)
            {
              current_init->file_path = strdup(current_s65816->current_file_segment->tab_file_segment[i]->file_path);
              current_init->file_name = strdup(current_s65816->current_file_segment->tab_file_segment[i]->file_name);
              break;
            }
        if(current_init->file_path == NULL || current_init->file_name == NULL)
          {
            current_init->file_path = strdup("");
            current_init->file_name = strdup("");
            if(current_init->file_path == NULL || current_init->file_name == NULL)
              {
                mem_free_init(current_init);
                mem_free_inittable(current_inittable);
                return(NULL);
              }
          }

        /* Type */
        strcpy(current_init->type,"Init");
        if(!stricmp(current_init->file_name,"TS2") || !stricmp(current_init->file_name,"TS3"))
          strcpy(current_init->type,"ToolSetup");
        else if(my_stristr(current_init->file_path,":System.Setup:") != NULL)
          strcpy(current_init->type,"PIF / TIF");
        else if(my_stristr(current_init->file_path,":FinderExtras:") != NULL)
          strcpy(current_init->type,"FinderExtras");

        /* Ajout */
        current_inittable->tab_init[nb_init] = current_init;
        nb_init++;
      }
  current_inittable->nb_init = nb_init;

  /* Tri le tableau par ordre Alphabtique */
  qsort(current_inittable->tab_init,current_inittable->nb_init,sizeof(struct init *),compare_init);

  /* Renvoi la structure */
  return(current_inittable);
}


/*****************************************************************/
/*  compare_init() : Fonction de comparaison pour le Quick Sort. */
/*****************************************************************/
static int compare_init(const void *data_1, const void *data_2)
{
  struct init *init_1;
  struct init *init_2;

  /* Rcupration des paramtres */
  init_1 = *((struct init **) data_1);
  init_2 = *((struct init **) data_2);

  /* Comparaison des chemins */
  return(stricmp(init_1->file_path,init_2->file_path));
}


/**********************************************************/
/*  GetDriverTable() :  Construit le tableau des Drivers. */
/**********************************************************/
struct driver_table *GetDriverTable(struct s65816 *current_s65816)
{
  int i, nb_driver, already;
  struct driver_table *current_drivertable;
  struct driver *current_driver;
  struct gs_handle *current_used_handle;

  /* Nombre Max de Driver */
  nb_driver = 0;
  for(current_used_handle = current_s65816->list_used_handle; current_used_handle; current_used_handle = current_used_handle->next)
    if((current_used_handle->id & 0xFF00) == 0x3900 || (current_used_handle->id & 0xFF00) == 0x3A00)
      nb_driver++;

  /* Allocation mmoire */
  current_drivertable = (struct driver_table *) calloc(1,sizeof(struct driver_table));
  if(current_drivertable == NULL)
    return(NULL);
  current_drivertable->tab_driver = (struct driver **) calloc(nb_driver,sizeof(struct driver *));
  if(current_drivertable->tab_driver == NULL)
    {
      free(current_drivertable);
      return(NULL);
    }

  /** Remplissage de la table **/
  nb_driver = 0;
  for(current_used_handle = current_s65816->list_used_handle; current_used_handle; current_used_handle = current_used_handle->next)
    if((current_used_handle->id & 0xFF00) == 0x3900 || (current_used_handle->id & 0xFF00) == 0x3A00)
      {
        /* A t'on dj ce Driver dans la liste ? */
        for(i=0,already=0; i<nb_driver; i++)
          if(current_drivertable->tab_driver[i]->user_id == current_used_handle->id)
            {
              already = 1;
              break;
            }
        if(already == 1)
          continue;

        /** On ajoute ce Driver dans la table **/
        /* Allocation mmoire */
        current_driver = (struct driver *) calloc(1,sizeof(struct driver));
        if(current_driver == NULL)
          {
            mem_free_drivertable(current_drivertable);
            return(NULL);
          }
        /* Remplissage */
        current_driver->handle = current_used_handle->handle;
        current_driver->address = current_used_handle->address;
        current_driver->size = current_used_handle->size;
        current_driver->user_id = current_used_handle->id;

        /* Cherche le chemin parmi les fichiers */
        for(i=0; i<current_s65816->current_file_segment->nb_segment_table; i++)
          if(current_s65816->current_file_segment->tab_file_segment[i]->user_id == current_driver->user_id)
            {
              current_driver->file_path = strdup(current_s65816->current_file_segment->tab_file_segment[i]->file_path);
              current_driver->file_name = strdup(current_s65816->current_file_segment->tab_file_segment[i]->file_name);
              break;
            }
        if(current_driver->file_path == NULL || current_driver->file_name == NULL)
          {
            current_driver->file_path = strdup("");
            current_driver->file_name = strdup("");
            if(current_driver->file_path == NULL || current_driver->file_name == NULL)
              {
                mem_free_driver(current_driver);
                mem_free_drivertable(current_drivertable);
                return(NULL);
              }
          }

        /* Ajout */
        current_drivertable->tab_driver[nb_driver] = current_driver;
        nb_driver++;
      }
  current_drivertable->nb_driver = nb_driver;

  /* Tri le tableau par ordre Alphabtique */
  qsort(current_drivertable->tab_driver,current_drivertable->nb_driver,sizeof(struct driver *),compare_driver);

  /* Renvoi la structure */
  return(current_drivertable);
}


/*******************************************************************/
/*  compare_driver() : Fonction de comparaison pour le Quick Sort. */
/*******************************************************************/
static int compare_driver(const void *data_1, const void *data_2)
{
  struct driver *driver_1;
  struct driver *driver_2;

  /* Rcupration des paramtres */
  driver_1 = *((struct driver **) data_1);
  driver_2 = *((struct driver **) data_2);

  /* Comparaison des chemins */
  return(stricmp(driver_1->file_path,driver_2->file_path));
}


/***************************************************/
/*  GetFSTTable() :  Construit le tableau des FST. */
/***************************************************/
struct fst_table *GetFSTTable(struct s65816 *current_s65816)
{
  int i, nb_fst, already;
  struct fst_table *current_fsttable;
  struct fst *current_fst;
  struct gs_handle *current_used_handle;

  /* Nombre Max de FST */
  nb_fst = 0;
  for(current_used_handle = current_s65816->list_used_handle; current_used_handle; current_used_handle = current_used_handle->next)
    if((current_used_handle->id & 0xFF00) == 0x3300)
      nb_fst++;

  /* Allocation mmoire */
  current_fsttable = (struct fst_table *) calloc(1,sizeof(struct fst_table));
  if(current_fsttable == NULL)
    return(NULL);
  current_fsttable->tab_fst = (struct fst **) calloc(nb_fst,sizeof(struct fst *));
  if(current_fsttable->tab_fst == NULL)
    {
      free(current_fsttable);
      return(NULL);
    }

  /** Remplissage de la table **/
  nb_fst = 0;
  for(current_used_handle = current_s65816->list_used_handle; current_used_handle; current_used_handle = current_used_handle->next)
    if((current_used_handle->id & 0xFF00) == 0x3300)
      {
        /* A t'on dj ce FST dans la liste ? */
        for(i=0,already=0; i<nb_fst; i++)
          if(current_fsttable->tab_fst[i]->user_id == current_used_handle->id)
            {
              already = 1;
              break;
            }
        if(already == 1)
          continue;

        /** On ajoute ce FST dans la table **/
        /* Allocation mmoire */
        current_fst = (struct fst *) calloc(1,sizeof(struct fst));
        if(current_fst == NULL)
          {
            mem_free_fsttable(current_fsttable);
            return(NULL);
          }
        /* Remplissage */
        current_fst->handle = current_used_handle->handle;
        current_fst->address = current_used_handle->address;
        current_fst->size = current_used_handle->size;
        current_fst->user_id = current_used_handle->id;

        /* Cherche le chemin parmi les fichiers */
        for(i=0; i<current_s65816->current_file_segment->nb_segment_table; i++)
          if(current_s65816->current_file_segment->tab_file_segment[i]->user_id == current_fst->user_id)
            {
              current_fst->file_path = strdup(current_s65816->current_file_segment->tab_file_segment[i]->file_path);
              current_fst->file_name = strdup(current_s65816->current_file_segment->tab_file_segment[i]->file_name);
              break;
            }
        if(current_fst->file_path == NULL || current_fst->file_name == NULL)
          {
            current_fst->file_path = strdup("");
            current_fst->file_name = strdup("");
            if(current_fst->file_path == NULL || current_fst->file_name == NULL)
              {
                mem_free_fst(current_fst);
                mem_free_fsttable(current_fsttable);
                return(NULL);
              }
          }

        /* Ajout */
        current_fsttable->tab_fst[nb_fst] = current_fst;
        nb_fst++;
      }
  current_fsttable->nb_fst = nb_fst;

  /* Tri le tableau par ordre Alphabtique */
  qsort(current_fsttable->tab_fst,current_fsttable->nb_fst,sizeof(struct fst *),compare_fst);

  /* Renvoi la structure */
  return(current_fsttable);
}


/****************************************************************/
/*  compare_fst() : Fonction de comparaison pour le Quick Sort. */
/****************************************************************/
static int compare_fst(const void *data_1, const void *data_2)
{
  struct fst *fst_1;
  struct fst *fst_2;

  /* Rcupration des paramtres */
  fst_1 = *((struct fst **) data_1);
  fst_2 = *((struct fst **) data_2);

  /* Comparaison des chemins */
  return(stricmp(fst_1->file_path,fst_2->file_path));
}


/*********************************************************************************************/
/*  mem_free_handle_list() :  Libration mmoire de la liste chaine de structure gs_handle. */
/*********************************************************************************************/
void mem_free_handle_list(struct gs_handle *first_handle)
{
  struct gs_handle *current_handle;
  struct gs_handle *next_handle;

  /* Libration mmoire de la liste chaine */
  for(current_handle = first_handle; current_handle; current_handle = next_handle)
    {
      next_handle = current_handle->next;
      free(current_handle);
    }
}


/********************************************************************************/
/*  mem_free_fileidtable() :  Libration mmoire de la structure file_id_table. */
/********************************************************************************/
void mem_free_fileidtable(struct file_id_table *current_idtable)
{
  if(current_idtable)
    {
      if(current_idtable->table_id_handle_7201)
        free(current_idtable->table_id_handle_7201);

      if(current_idtable->table_id_data_7201)
        free(current_idtable->table_id_data_7201);

      free(current_idtable);
    }
}


/*************************************************************************/
/*  mem_free_cdatable() :  Libration mmoire de la structure cda_table. */
/*************************************************************************/
void mem_free_cdatable(struct cda_table *current_cdatable)
{
  int i;

  if(current_cdatable)
    {
      if(current_cdatable->tab_cda)
        {
          for(i=0; i<current_cdatable->nb_cda; i++)
             if(current_cdatable->tab_cda[i].name)
               free(current_cdatable->tab_cda[i].name);

          free(current_cdatable->tab_cda);
        }

      if(current_cdatable->cda_table_handle)
        free(current_cdatable->cda_table_handle);

      if(current_cdatable->cda_table_data)
        free(current_cdatable->cda_table_data);

      free(current_cdatable);
    }
}


/*************************************************************************/
/*  mem_free_ndatable() :  Libration mmoire de la structure nda_table. */
/*************************************************************************/
void mem_free_ndatable(struct nda_table *current_ndatable)
{
  int i;

  if(current_ndatable)
    {
      if(current_ndatable->tab_nda)
        {
          for(i=0; i<current_ndatable->nb_nda; i++)
            {
              if(current_ndatable->tab_nda[i].current_handle)
                free(current_ndatable->tab_nda[i].current_handle);

              if(current_ndatable->tab_nda[i].data)
                free(current_ndatable->tab_nda[i].data);

              if(current_ndatable->tab_nda[i].name)
                free(current_ndatable->tab_nda[i].name);
            }

          free(current_ndatable->tab_nda);
        }

      free(current_ndatable);
    }
}


/********************************************************************************************/
/*  mem_free_quickdrawvectortable() :  Libration mmoire de la structure quickdraw_vector. */
/********************************************************************************************/
void mem_free_quickdrawvectortable(struct quickdraw_vector_table *current_quickdraw_vector)
{
  if(current_quickdraw_vector)
    {
      free(current_quickdraw_vector);
    }
}


/********************************************************************************************/
/*  mem_free_interruptvectortable() :  Libration mmoire de la structure interrupt_vector. */
/********************************************************************************************/
void mem_free_interruptvectortable(struct interrupt_vector_table *current_interrupt_vector)
{
  if(current_interrupt_vector)
    {
      free(current_interrupt_vector);
    }
}


/**********************************************************************************/
/*  mem_free_filesegmenttable() :  Libration mmoire du tableau de file_segment. */
/**********************************************************************************/
void mem_free_filesegmenttable(struct file_segment_table *current_filesegmenttable)
{
  int i;

  if(current_filesegmenttable)
    {
      for(i=0; i<current_filesegmenttable->nb_segment_table; i++)
        mem_free_filesegment(current_filesegmenttable->tab_file_segment[i]);

      free(current_filesegmenttable);
    }
}


/*******************************************************************************/
/*  mem_free_filesegment() :  Libration mmoire d'une structure file_segment. */
/*******************************************************************************/
void mem_free_filesegment(struct file_segment *current_file_segment)
{
  if(current_file_segment)
    {
      if(current_file_segment->file_path != NULL)
        free(current_file_segment->file_path);

      if(current_file_segment->file_name != NULL)
        free(current_file_segment->file_name);

      if(current_file_segment->segment_name != NULL)
        free(current_file_segment->segment_name);

      free(current_file_segment);
    }
}


/***************************************************************************/
/*  mem_free_inittable() :  Libration mmoire de la structure init_table. */
/***************************************************************************/
void mem_free_inittable(struct init_table *current_inittable)
{
  int i;

  if(current_inittable)
    {
      if(current_inittable->tab_init)
        {
          for(i=0; i<current_inittable->nb_init; i++)
            mem_free_init(current_inittable->tab_init[i]);

          free(current_inittable->tab_init);
        }
      free(current_inittable);
    }
}


/****************************************************************/
/*  mem_free_init() :  Libration mmoire de la structure init. */
/****************************************************************/
void mem_free_init(struct init *current_init)
{
  if(current_init)
    {
      if(current_init->file_name)
        free(current_init->file_name);

      if(current_init->file_path)
        free(current_init->file_path); 

      free(current_init);
    }
}


/*******************************************************************************/
/*  mem_free_drivertable() :  Libration mmoire de la structure driver_table. */
/*******************************************************************************/
void mem_free_drivertable(struct driver_table *current_drivertable)
{
  int i;

  if(current_drivertable)
    {
      if(current_drivertable->tab_driver)
        {
          for(i=0; i<current_drivertable->nb_driver; i++)
            mem_free_driver(current_drivertable->tab_driver[i]);

          free(current_drivertable->tab_driver);
        }
      free(current_drivertable);
    }
}


/********************************************************************/
/*  mem_free_driver() :  Libration mmoire de la structure driver. */
/********************************************************************/
void mem_free_driver(struct driver *current_driver)
{
  if(current_driver)
    {
      if(current_driver->file_name)
        free(current_driver->file_name);

      if(current_driver->file_path)
        free(current_driver->file_path); 

      free(current_driver);
    }
}


/*************************************************************************/
/*  mem_free_fsttable() :  Libration mmoire de la structure fst_table. */
/*************************************************************************/
void mem_free_fsttable(struct fst_table *current_fsttable)
{
  int i;

  if(current_fsttable)
    {
      if(current_fsttable->tab_fst)
        {
          for(i=0; i<current_fsttable->nb_fst; i++)
            mem_free_fst(current_fsttable->tab_fst[i]);

          free(current_fsttable->tab_fst);
        }
      free(current_fsttable);
    }
}


/**************************************************************/
/*  mem_free_fst() :  Libration mmoire de la structure fst. */
/**************************************************************/
void mem_free_fst(struct fst *current_fst)
{
  if(current_fst)
    {
      if(current_fst->file_name)
        free(current_fst->file_name);

      if(current_fst->file_path)
        free(current_fst->file_path); 

      free(current_fst);
    }
}

/************************************************************************/
