/**************************************************************************/
/*                                                                        */
/*  Dc_MMU.c :  Module pour la bibliothque de gestion de la Mmoire.     */
/*                                                                        */
/**************************************************************************/
/*  Auteur : Olivier ZARDINI  *   Brutal Deluxe Software   *  Avril 2018  */
/**************************************************************************/

#include <stdio.h>
#include <stdlib.h>

#include <windows.h>

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_MMU.h"


/******************************************************/
/*  ReadMemoryByte() :  Rcupre un octet en mmoire. */
/******************************************************/
BYTE ReadMemoryByte(DWORD address, struct mmu *current_mmu)
{
  WORD base_address;
  BYTE bank_address, byte;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  
  /* Init */
  byte = 0x00;
  
  /* Banc mmoire */
  bank_address = (BYTE) ((address >> 16) & 0xFF);
  base_address = (WORD) (address & 0xFFFF);
  
  /* Accs unique  la zone mmoire */
  EnterCriticalSection(&param->cs_mmu_threadhandle);  
  
  /* Dispatch la lecture selon le banc */
  switch(bank_address)
    {
      /* Fast Ram */
      case 0x00 :
        byte = current_mmu->ram_00[base_address];
        break;

      case 0x01 :
        byte = current_mmu->ram_01[base_address];
        break;

      /* Slow Ram */
      case 0xE0 :
        byte = current_mmu->ram_e0[base_address];
        break;

      case 0xE1 :
        byte = current_mmu->ram_e1[base_address];
        break;

      /* ROM */
      case 0xFC :
        byte = current_mmu->rom_fc[base_address];
        break;
        
      case 0xFD :
        byte = current_mmu->rom_fd[base_address];
        break;
        
      case 0xFE :
        byte = current_mmu->rom_fe[base_address];
        break;                

      case 0xFF :
        byte = current_mmu->rom_ff[base_address];
        break;
        
      /* RAM Extended */
      default :
        /* Read Out of Memory Error */
        if(bank_address-2 >= current_mmu->nb_extended_bank)
          ;
        else
          byte = current_mmu->ram_extended[(address & 0xFFFFFF)-(2*64*1024)];
        break;
    }

  /* Fin de l'accs unique */
  LeaveCriticalSection(&param->cs_mmu_threadhandle);  

  /* Renvoi le Byte lu */
  return(byte);
}


/****************************************************/
/*  ReadMemoryBlock() :  Rcupre un block mmoire. */
/****************************************************/
void ReadMemoryBlock(DWORD address, int length, unsigned char *output_buffer, struct mmu *current_mmu)
{
  int data_address, data_length, output_offset, bank_data_length, end_address;
  int begin_ram_extended, end_ram_entended, begin_rom_extended, end_rom_extended;
  int begin_00, begin_01, begin_extended, begin_e0, begin_e1, begin_fc, begin_fd, begin_fe, begin_ff;
  int end_00, end_01, end_extended, end_e0, end_e1, end_fc, end_fd, end_fe, end_ff;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  
  /* Init */
  memset(output_buffer,0,length);
  end_address = address + length;
  data_address = address;
  data_length = length;
  output_offset = 0;
  begin_00 = begin_01 = begin_extended = begin_e0 = begin_e1 = begin_fc = begin_fd = begin_fe = begin_ff = -1;
  end_00 = end_01 = end_extended = end_e0 = end_e1 = end_fc = end_fd = end_fe = end_ff = -1;
    
  /* Vrification de la prsence de donnes */
  if(current_mmu == NULL)
    return;

  /** Dcoupe le block demand en zone de banc **/
  /* Bank 00 */
  if(data_address >= 0x000000 && data_address <= 0x00FFFF && data_length > 0)
    {
      /* Zone  copier depuis ce bank */
      begin_00 = (data_address & 0xFFFF);
      
      /* Longueur des donnes dans ce bank */
      if((data_address + data_length) > 0x00FFFF)
        end_00 = 0xFFFF;
      else
        end_00 = begin_00 + data_length - 1;

      /* Dcale */
      bank_data_length = (end_00 - begin_00) + 1;
      data_length -= bank_data_length;
      data_address += bank_data_length;
    }
    
  /* Bank 01 */
  if(data_address >= 0x010000 && data_address <= 0x01FFFF && data_length > 0)
    {
      /* Zone  copier depuis ce bank */
      begin_01 = (data_address & 0xFFFF);
      
      /* Longueur des donnes dans ce bank */
      if((data_address + data_length) > 0x01FFFF)
        end_01 = 0xFFFF;
      else
        end_01 = begin_01 + data_length - 1;

      /* Dcale */
      bank_data_length = (end_01 - begin_01) + 1;
      data_length -= bank_data_length;
      data_address += bank_data_length;
    }

  /* Extended RAM : Bank 02-DF */
  if(current_mmu->nb_extended_bank == 0)
    {
      /* Il n'y a pas de Ram Extended d'install */
      if(data_address >= 0x020000 && data_address <= 0xDFFFFF && data_length > 0)
        {
          if(data_length > (0xE0-0x02)*64*1024)
            {
              data_length -= (0xE0-0x02)*64*1024;
              data_address = 0xE00000;
            }
          else
            data_length = 0;    /* Fini */
        }
    }
  else
    {
      /* On a de la Ram Extended d'installe */
      begin_ram_extended = 0x020000;
      end_ram_entended = (begin_ram_extended + current_mmu->nb_extended_bank*64*1024) - 1;
      
      /* On veut piocher dedans */
      if(data_address >= begin_ram_extended && data_address <= end_ram_entended && data_length > 0)
        {
          /* Dbut */
          begin_extended = data_address;

          /* Fin */
          if(data_address + data_length > end_ram_entended)
            end_extended = end_ram_entended;
          else
            end_extended = (begin_extended + data_length - 1);     

          /* Longueur */
          data_length -= (end_extended - begin_extended) + 1;
          
          /* On veut un offset dans current_mmu->ram_extended */
          begin_extended -= 0x020000;
          end_extended -= 0x020000;
        }
    }
        
  /* Bank E0 */
  if(data_address >= 0xE00000 && data_address <= 0xE0FFFF && data_length > 0)
    {
      /* Zone  copier depuis ce bank */
      begin_e0 = (data_address & 0xFFFF);
      
      /* Longueur des donnes dans ce bank */
      if((data_address + data_length) > 0xE0FFFF)
        end_e0 = 0xFFFF;
      else
        end_e0 = begin_e0 + data_length - 1;

      /* Dcale */
      bank_data_length = (end_e0 - begin_e0) + 1;
      data_length -= bank_data_length;
      data_address += bank_data_length;
    }
    
  /* Bank E1 */
  if(data_address >= 0xE10000 && data_address <= 0xE1FFFF && data_length > 0)
    {
      /* Zone  copier depuis ce bank */
      begin_e1 = (data_address & 0xFFFF);
      
      /* Longueur des donnes dans ce bank */
      if((data_address + data_length) > 0xE1FFFF)
        end_e1 = 0xFFFF;
      else
        end_e1 = begin_e1 + data_length - 1;

      /* Dcale */
      bank_data_length = (end_e1 - begin_e1) + 1;
      data_length -= bank_data_length;
      data_address += bank_data_length;
    }

  /* Bank E2 -> FB : vides */
  if(data_address >= 0xE20000 && data_address <= 0xFBFFFF && data_length > 0)
    {
      /* Dbut */
      begin_rom_extended = data_address;

      /* Fin */
      if(data_address + data_length > 0xFBFFFF)
        end_rom_extended = 0xFBFFFF;
      else
        end_rom_extended = (begin_rom_extended + data_length - 1);

      /* Longueur */
      data_length -= (end_rom_extended - begin_rom_extended) + 1;
    }
    
  /* Bank FC */
  if(data_address >= 0xFC0000 && data_address <= 0xFCFFFF && data_length > 0)
    {
      /* Zone  copier depuis ce bank */
      begin_fc = (data_address & 0xFFFF);
      
      /* Longueur des donnes dans ce bank */
      if((data_address + data_length) > 0xFCFFFF)
        end_fc = 0xFFFF;
      else
        end_fc = begin_fc + data_length - 1;

      /* Dcale */
      bank_data_length = (end_fc - begin_fc) + 1;
      data_length -= bank_data_length;
      data_address += bank_data_length;
    }

  /* Bank FD */
  if(data_address >= 0xFD0000 && data_address <= 0xFDFFFF && data_length > 0)
    {
      /* Zone  copier depuis ce bank */
      begin_fd = (data_address & 0xFFFF);
      
      /* Longueur des donnes dans ce bank */
      if((data_address + data_length) > 0xFDFFFF)
        end_fd = 0xFFFF;
      else
        end_fd = begin_fd + data_length - 1;

      /* Dcale */
      bank_data_length = (end_fd - begin_fd) + 1;
      data_length -= bank_data_length;
      data_address += bank_data_length;
    }

  /* Bank FE */
  if(data_address >= 0xFE0000 && data_address <= 0xFEFFFF && data_length > 0)
    {
      /* Zone  copier depuis ce bank */
      begin_fe = (data_address & 0xFFFF);
      
      /* Longueur des donnes dans ce bank */
      if((data_address + data_length) > 0xFEFFFF)
        end_fe = 0xFFFF;
      else
        end_fe = begin_fe + data_length - 1;

      /* Dcale */
      bank_data_length = (end_fe - begin_fe) + 1;
      data_length -= bank_data_length;
      data_address += bank_data_length;
    }

  /* Bank FF */
  if(data_address >= 0xFF0000 && data_address <= 0xFFFFFF && data_length > 0)
    {
      /* Zone  copier depuis ce bank */
      begin_ff = (data_address & 0xFFFF);
      
      /* Longueur des donnes dans ce bank */
      if((data_address + data_length) > 0xFFFFFF)
        end_ff = 0xFFFF;
      else
        end_ff = begin_ff + data_length - 1;

      /* Dcale */
      bank_data_length = (end_ff - begin_ff) + 1;
      data_length -= bank_data_length;
      data_address += bank_data_length;
    }
    
  /* Accs unique  la zone mmoire */
  EnterCriticalSection(&param->cs_mmu_threadhandle);  
  
  /** Copie les donnes depuis les bancs mmoires concerns **/
  output_offset = 0;
  /* Fast RAM */
  if(begin_00 >= 0)
    {
      bank_data_length = end_00 - begin_00 + 1;
      memcpy(&output_buffer[output_offset],&current_mmu->ram_00[begin_00],bank_data_length);
      output_offset += bank_data_length;
    }
  if(begin_01 >= 0)
    {
      bank_data_length = end_01 - begin_01 + 1;
      memcpy(&output_buffer[output_offset],&current_mmu->ram_01[begin_01],bank_data_length);
      output_offset += bank_data_length;
    }
    
  /* Extended RAM */
  if(begin_extended >= 0)
    {
      bank_data_length = end_extended - begin_extended + 1;
      memcpy(&output_buffer[output_offset],&current_mmu->ram_extended[begin_extended],bank_data_length);
      output_offset += bank_data_length;
    }
    
  /* Slow Ram */
  if(begin_e0 >= 0)
    {
      bank_data_length = end_e0 - begin_e0 + 1;
      memcpy(&output_buffer[output_offset],&current_mmu->ram_e0[begin_e0],bank_data_length);
      output_offset += bank_data_length;
    }
  if(begin_e1 >= 0)
    {
      bank_data_length = end_e1 - begin_e1 + 1;
      memcpy(&output_buffer[output_offset],&current_mmu->ram_e1[begin_e1],bank_data_length);
      output_offset += bank_data_length;
    }

  /* Rom */
  if(begin_fc >= 0)
    {
      bank_data_length = (end_fc - begin_fc) + 1;
      memcpy(&output_buffer[output_offset],&current_mmu->rom_fc[begin_fc],bank_data_length);
      output_offset += bank_data_length;
    }
  if(begin_fd >= 0)
    {
      bank_data_length = (end_fd - begin_fd) + 1;
      memcpy(&output_buffer[output_offset],&current_mmu->rom_fd[begin_fd],bank_data_length);
      output_offset += bank_data_length;
    }
  if(begin_fe >= 0)
    {
      bank_data_length = (end_fe - begin_fe) + 1;
      memcpy(&output_buffer[output_offset],&current_mmu->rom_fe[begin_fe],bank_data_length);
      output_offset += bank_data_length;
    }
  if(begin_ff >= 0)
    {
      bank_data_length = (end_ff - begin_ff) + 1;
      memcpy(&output_buffer[output_offset],&current_mmu->rom_ff[begin_ff],bank_data_length);
      output_offset += bank_data_length;
    }

  /* Fin de l'accs unique */
  LeaveCriticalSection(&param->cs_mmu_threadhandle);  
}


/****************************************************/
/*  WriteMemoryByte() :  Ecrit un octet en mmoire. */
/****************************************************/
void WriteMemoryByte(DWORD address, BYTE value, struct mmu *current_mmu)
{
  WORD base_address;
  BYTE bank_address;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  
  /* Banc mmoire */
  bank_address = (BYTE) ((address >> 16) & 0xFF);
  base_address = (WORD) (address & 0xFFFF);
  
  /* Accs unique  la zone mmoire */
  EnterCriticalSection(&param->cs_mmu_threadhandle);  
  
  /* Dispatch la lecture selon le banc */
  switch(bank_address)
    {
      /* Fast Ram */
      case 0x00 :
        current_mmu->ram_00[base_address] = value;
        break;

      case 0x01 :
        current_mmu->ram_01[base_address] = value;
        break;

      /* Slow Ram */
      case 0xE0 :
        current_mmu->ram_e0[base_address] = value;
        break;

      case 0xE1 :
        current_mmu->ram_e1[base_address] = value;
        break;

      /* ROM */
      case 0xFC :
        current_mmu->rom_fc[base_address] = value;
        break;
        
      case 0xFD :
        current_mmu->rom_fd[base_address] = value;
        break;
        
      case 0xFE :
        current_mmu->rom_fe[base_address] = value;
        break;                

      case 0xFF :
        current_mmu->rom_ff[base_address] = value;
        break;
        
      /* RAM Extended */
      default :
        /* Read Out of Memory Error */
        if(bank_address-2 >= current_mmu->nb_extended_bank)
          ;
        else
          current_mmu->ram_extended[address-(2*64*1024)] = value;
        break;
    }

  /* Fin de l'accs unique */
  LeaveCriticalSection(&param->cs_mmu_threadhandle);  
}


/**********************************************************/
/*  ReadDOCRAMByte() :  Rcupre un octet en mmoire Son. */
/**********************************************************/
BYTE ReadDOCRAMByte(DWORD address, struct mmu *current_mmu)
{
  WORD base_address;
  BYTE byte;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  
  /* Init */
  byte = 0x00;
  
  /* Adresse mmoire */
  base_address = (WORD) (address & 0xFFFF);
  
  /* Accs unique  la zone mmoire */
  EnterCriticalSection(&param->cs_mmu_threadhandle);  
  
  /* Rcupre l'octet en mmoire Son */
  byte = current_mmu->ram_sound[base_address];

  /* Fin de l'accs unique */
  LeaveCriticalSection(&param->cs_mmu_threadhandle);  

  /* Renvoi le Byte lu */
  return(byte);
}


/********************************************************/
/*  ReadDOCRAMBlock() :  Rcupre un block mmoire Son. */
/********************************************************/
void ReadDOCRAMBlock(DWORD address, int length, unsigned char *output_buffer, struct mmu *current_mmu)
{
  int bank_data_length;
  int begin_00, end_00;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  
  /* Init */
  memset(output_buffer,0,length);
  bank_data_length = 0;

  /** Dcoupe le block demand en zone de banc **/
  if(address >= 0x0000 && address <= 0xFFFF && length > 0)
    {
      /* Zone  copier depuis ce bank de Ram Son */
      begin_00 = (address & 0xFFFF);
      
      /* Longueur des donnes dans ce bank */
      if((address+length) > 0x00FFFF)
        end_00 = 0xFFFF;
      else
        end_00 = begin_00 + length - 1;

      /* Longueur des donnes de ce Bank */
      bank_data_length = (end_00 - begin_00) + 1;
    }
    
  /* Accs unique  la zone mmoire */
  EnterCriticalSection(&param->cs_mmu_threadhandle);  
  
  /** Copie les donnes depuis le banc mmoire RAM Son **/
  if(bank_data_length > 0)
    memcpy(output_buffer,&current_mmu->ram_sound[begin_00],bank_data_length);
    
  /* Fin de l'accs unique */
  LeaveCriticalSection(&param->cs_mmu_threadhandle);  
}


/********************************************************/
/*  WriteDOCRAMByte() :  Ecrit un octet en mmoire Son. */
/********************************************************/
void WriteDOCRAMByte(DWORD address, BYTE value, struct mmu *current_mmu)
{
  WORD base_address;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  
  /* Adresse mmoire */
  base_address = (WORD) (address & 0xFFFF);
  
  /* Accs unique  la zone mmoire */
  EnterCriticalSection(&param->cs_mmu_threadhandle);  
  
  /* Ecrit l'octet en mmoire Son */
  current_mmu->ram_sound[base_address] = value;

  /* Fin de l'accs unique */
  LeaveCriticalSection(&param->cs_mmu_threadhandle);  
}


/*****************************************************************/
/*  ReadDOCRegisterByte() :  Rcupre un octet des DOC Register. */
/*****************************************************************/
BYTE ReadDOCRegisterByte(DWORD address, struct mmu *current_mmu)
{
  BYTE byte;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  
  /* Init */
  byte = 0x00;
  
  /* Vrification de l'adresse mmoire */
  if(address >= DOC_REGISTER_SIZE)
    return(byte);
  
  /* Accs unique  la zone mmoire */
  EnterCriticalSection(&param->cs_mmu_threadhandle);  
  
  /* Rcupre l'octet dans les DOC Register */
  byte = current_mmu->doc_register[address];

  /* Fin de l'accs unique */
  LeaveCriticalSection(&param->cs_mmu_threadhandle);  

  /* Renvoi le Byte lu */
  return(byte);
}


/**************************************************************************/
/*  ReadDOCRegisterBlock() :  Rcupre un block mmoire des DOC Register. */
/**************************************************************************/
void ReadDOCRegisterBlock(DWORD address, int length, unsigned char *output_buffer, struct mmu *current_mmu)
{
  int bank_data_length;
  int begin_00, end_00;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  
  /* Init */
  memset(output_buffer,0,length);
  bank_data_length = 0;

  /** Dcoupe le block demand en zone de banc **/
  if(address >= 0x0000 && address < DOC_REGISTER_SIZE && length > 0)
    {
      /* Zone  copier depuis les DOC Register */
      begin_00 = address;
      
      /* Longueur des donnes dans ce bank */
      if((address+length) > (DOC_REGISTER_SIZE-1))
        end_00 = (DOC_REGISTER_SIZE-1);
      else
        end_00 = begin_00 + length - 1;

      /* Longueur des donnes de ce Bank */
      bank_data_length = (end_00 - begin_00) + 1;
    }
    
  /* Accs unique  la zone mmoire */
  EnterCriticalSection(&param->cs_mmu_threadhandle);  
  
  /** Copie les donnes depuis les DOC Register **/
  if(bank_data_length > 0)
    memcpy(output_buffer,&current_mmu->doc_register[begin_00],bank_data_length);
    
  /* Fin de l'accs unique */
  LeaveCriticalSection(&param->cs_mmu_threadhandle);  
}


/********************************************************************/
/*  WriteDOCRegisterByte() :  Ecrit un octet dans les DOC Register. */
/********************************************************************/
void WriteDOCRegisterByte(DWORD address, BYTE value, struct mmu *current_mmu)
{
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  
  /* Vrification de l'adresse mmoire */
  if(address >= DOC_REGISTER_SIZE)
    return;
  
  /* Accs unique  la zone mmoire */
  EnterCriticalSection(&param->cs_mmu_threadhandle);  
  
  /* Ecrit l'octet en mmoire Son */
  current_mmu->doc_register[address] = value;

  /* Fin de l'accs unique */
  LeaveCriticalSection(&param->cs_mmu_threadhandle);  
}


/*****************************************************************/
/*  ReadBatteryRAMByte() :  Rcupre un octet de la Battery RAM. */
/*****************************************************************/
BYTE ReadBatteryRAMByte(DWORD address, struct mmu *current_mmu)
{
  BYTE byte;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  
  /* Init */
  byte = 0x00;
  
  /* Vrification de l'adresse mmoire */
  if(address >= BATTERY_RAM_SIZE)
    return(byte);
  
  /* Accs unique  la zone mmoire */
  EnterCriticalSection(&param->cs_mmu_threadhandle);  
  
  /* Rcupre l'octet dans la Battery RAM */
  byte = current_mmu->battery_ram[address];

  /* Fin de l'accs unique */
  LeaveCriticalSection(&param->cs_mmu_threadhandle);  

  /* Renvoi le Byte lu */
  return(byte);
}


/**************************************************************************/
/*  ReadBatteryRAMBlock() :  Rcupre un block mmoire de la Battery RAM. */
/**************************************************************************/
void ReadBatteryRAMBlock(DWORD address, int length, unsigned char *output_buffer, struct mmu *current_mmu)
{
  int bank_data_length;
  int begin_00, end_00;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  
  /* Init */
  memset(output_buffer,0,length);
  bank_data_length = 0;

  /** Dcoupe le block demand **/
  if(address >= 0x0000 && address < BATTERY_RAM_SIZE && length > 0)
    {
      /* Zone  copier depuis la Battery RAM */
      begin_00 = address;
      
      /* Longueur des donnes dans cette zone */
      if((address+length) > (BATTERY_RAM_SIZE-1))
        end_00 = (BATTERY_RAM_SIZE-1);
      else
        end_00 = begin_00 + length - 1;

      /* Longueur des donnes de cette zone */
      bank_data_length = (end_00 - begin_00) + 1;
    }
    
  /* Accs unique  la zone mmoire */
  EnterCriticalSection(&param->cs_mmu_threadhandle);  
  
  /** Copie les donnes depuis la Battery RAM **/
  if(bank_data_length > 0)
    memcpy(output_buffer,&current_mmu->battery_ram[begin_00],bank_data_length);
    
  /* Fin de l'accs unique */
  LeaveCriticalSection(&param->cs_mmu_threadhandle);  
}


/*****************************************************************/
/*  WriteBatteryRAMByte() :  Ecrit un octet dans la Battery RAM. */
/*****************************************************************/
void WriteBatteryRAMByte(DWORD address, BYTE value, struct mmu *current_mmu)
{
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  
  /* Vrification de l'adresse mmoire */
  if(address >= BATTERY_RAM_SIZE)
    return;
  
  /* Accs unique  la zone mmoire */
  EnterCriticalSection(&param->cs_mmu_threadhandle);  
  
  /* Ecrit l'octet en Battery RAM */
  current_mmu->battery_ram[address] = value;

  /* Fin de l'accs unique */
  LeaveCriticalSection(&param->cs_mmu_threadhandle);  
}


/**********************************************************************/
/*  GetRamRomPointer() :  Rcupre un pointeur vers une zone mmoire. */
/**********************************************************************/
unsigned char *GetRamRomPointer(DWORD bank_address, int is_doc_ram, struct mmu *current_mmu)
{
  int bank;
  WORD address;

  /* Dcoupage */
  bank = (bank_address >> 16) & 0x000000FF;
  address = bank_address & 0x0000FFFF;

  /** DOC Ram **/
  if(is_doc_ram == 1)
    return(&current_mmu->ram_sound[address]);

  /** Rcupre un pointeur vers l'adresse **/
  if(bank == 0x00)
    return(&current_mmu->ram_00[address]);
  else if(bank == 0x01)
    return(&current_mmu->ram_01[address]);
  else if((bank-1) <= current_mmu->nb_extended_bank)
    return(&current_mmu->ram_extended[bank_address - 2*RAM_BANK_SIZE]);
  else if(bank == 0xE0)
    return(&current_mmu->ram_e0[address]);
  else if(bank == 0xE1)
    return(&current_mmu->ram_e1[address]);
  else if(bank == 0xFC && current_mmu->rom_version == 3)
    return(&current_mmu->rom_fc[address]);
  else if(bank == 0xFD && current_mmu->rom_version == 3)
    return(&current_mmu->rom_fd[address]);
  else if(bank == 0xFE)
    return(&current_mmu->rom_fe[address]);
  else if(bank == 0xFF)
    return(&current_mmu->rom_ff[address]);

  /* Pas trouv */
  return(NULL);
}


/********************************************************************************/
/*  GetMapRamRomRWXPointer() :  Rcupre un pointeur vers une zone mmoire Map. */
/********************************************************************************/
unsigned char *GetMapRamRomRWXPointer(DWORD bank_address, int is_doc_ram, struct mmu *current_mmu)
{
  int bank;
  WORD address;

  /* Dcoupage */
  bank = (bank_address >> 16) & 0x000000FF;
  address = bank_address & 0x0000FFFF;

  /** DOC Ram **/
  if(is_doc_ram == 1)
    return(&current_mmu->map_ram_sound[address]);

  /** Rcupre un pointeur vers l'adresse **/
  if(bank == 0x00)
    return(&current_mmu->map_ram_00[address]);
  else if(bank == 0x01)
    return(&current_mmu->map_ram_01[address]);
  else if((bank-1) <= current_mmu->nb_extended_bank)
    return(&current_mmu->map_ram_extended[bank_address - 2*RAM_BANK_SIZE]);
  else if(bank == 0xE0)
    return(&current_mmu->map_ram_e0[address]);
  else if(bank == 0xE1)
    return(&current_mmu->map_ram_e1[address]);
  else if(bank == 0xFC && current_mmu->rom_version == 3)
    return(&current_mmu->map_rom_fc[address]);
  else if(bank == 0xFD && current_mmu->rom_version == 3)
    return(&current_mmu->map_rom_fd[address]);
  else if(bank == 0xFE)
    return(&current_mmu->map_rom_fe[address]);
  else if(bank == 0xFF)
    return(&current_mmu->map_rom_ff[address]);

  /* Pas trouv */
  return(NULL);
}

/**************************************************************************/
