/******************************************************************************/
/*                                                                            */
/*  Dc_Output.c :  Module de gestion des lignes Output de la Shared Memory.   */
/*                                                                            */
/******************************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe Software  *  Date : June 2023  */
/******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <process.h>
#include <malloc.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <io.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/timeb.h>
#include <direct.h>
#include <errno.h>

#include <windows.h>

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_MMU.h"
#include "Dc_Snapshot.h"
#include "Dc_Notification.h"
#include "Cyrene_Engine.h"
#include "Window_IPC.h"
#include "Window_QuickWatch.h"
#include "Window_Main.h"
#include "Window_Main_Menu.h"
#include "Window_MemoryMap.h"
#include "Window_Statistic.h"
#include "Window_Breakpoint.h"
#include "Window_Profile.h"
#include "Dc_Disassembler.h"
#include "Dc_Output.h"
#include "resource.h"

/* [00000000:413C273D][01/F27F][2010] o */
#define SYSTEMCALL_GOID_HIGH_OFFSET       1
#define SYSTEMCALL_GOID_LOW_OFFSET       10
#define SYSTEMCALL_ADDRESS_BANK_OFFSET   20
#define SYSTEMCALL_ADDRESS_ADDR_OFFSET   23
#define SYSTEMCALL_CALL_NUMBER_OFFSET    29
#define SYSTEMCALL_DATA_OFFSET           35

/* [00000000:3FEA9B2C][0E/9650][202D] => [3637] */
#define SYSTEMCALL_RESULT_VALUE_GOID_HIGH_OFFSET   1
#define SYSTEMCALL_RESULT_VALUE_GOID_LOW_OFFSET   10
#define SYSTEMCALL_RESULT_VALUE_ADDRESS_OFFSET    20
#define SYSTEMCALL_RESULT_VALUE_ARROW_OFFSET      35
#define SYSTEMCALL_RESULT_VALUE_CYCLE_OFFSET      39

static DWORD StartOutputThreadInit(void *);
static struct output *ExtractOutputFromSharedMemory(struct window_ipc *,struct parameter *);
static int GetReturnValueParam(char *,char *);
static BYTE GetSharedMemoryCircularValue(DWORD,unsigned char *);
static void SetSharedMemoryCircularValue(DWORD,unsigned char *,BYTE);


/******************************************************************/
/*  StartOutputManager() :  Lance le Thread en charge des Output. */
/******************************************************************/
int StartOutputManager(void)
{
  HANDLE thread_handle;
  DWORD thread_id;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /****************************************************/
  /*** On va lancer le Thread de Gestion des Output ***/
  param->output_thread_status = 1;

  /** Lancement du Thread **/
  thread_handle = CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)StartOutputThreadInit,NULL,0,&thread_id);
  if(thread_handle == NULL)
    {
      /* Log l'erreur */
      param->output_thread_status = 0;   
      return(1);
    }
  
  /* Conserve le Handle de ce Thread */
  param->output_thread_handle = thread_handle;
  
  /* OK */
  return(0);
}


/******************************************************************************/
/*  StartOutputThreadInit() :  Traite les donnes Output de la Shared Memory. */
/******************************************************************************/
static DWORD StartOutputThreadInit(void *nothing_here)
{
  FILE *fd;
  FILE *tab_fd[BREAKPOINT_NUMBER_MAX];
  DWORD tab_id[BREAKPOINT_NUMBER_MAX];
  int i, j, wait_delay, nb_print_output, include, exclude;
  int nb_file_output;
  struct output *first_file_output;
  struct output *last_file_output;
  struct output *current_output;
  struct output *next_output;
  struct output *call_output;
  struct cyrene_breakpoint *current_breakpoint;
  struct window_ipc *window_ipc;
  struct window_main *window_main;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Init */
  Sleep(500);                 /* On attend un peu histoire de laisser les fentres MAIN + IPC se crer */
  wait_delay = 0;
  my_Memory(MEMORY_GET_WINDOW_IPC,&window_ipc,NULL);
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);

  /*** On entre dans la boucle du traitement ***/
  while(1)
    {
      /** Y a t'il des Lignes Output en attente dans la Shared Memory ? **/
      nb_print_output = 0;
      nb_file_output = 0;
      first_file_output = NULL;
      last_file_output = NULL;
      while(1)
        {
          /** Rcupre une ligne output dans la Shared Memory **/
          current_output = ExtractOutputFromSharedMemory(window_ipc,param);
          if(current_output == NULL)
            break;

          /** On filtre certains appels systmes **/
          if((current_output->origin == TRACE_CALL_ORIGIN_P8   && param->trace_p8_calls == 0)   ||
             (current_output->origin == TRACE_CALL_ORIGIN_P16  && param->trace_p16_calls == 0)  ||
             (current_output->origin == TRACE_CALL_ORIGIN_GSOS && param->trace_gsos_calls == 0) ||
             (current_output->origin == TRACE_CALL_ORIGIN_TOOL && param->trace_toolbox_calls == 0))
            {
              mem_free_output(current_output);
              continue;
            }

          /** On filtre certains appels systmes [OS/TOOLBOX] **/
          if(current_output->type == BREAKPOINT_ACTION_TRACE_SYSTEM_CALL)
            {
              /** On filtre certains appels systmes [P8 / P16 / GS/OS] **/
              if(current_output->origin == TRACE_CALL_ORIGIN_P8 || current_output->origin == TRACE_CALL_ORIGIN_P16 || current_output->origin == TRACE_CALL_ORIGIN_GSOS)
                {
                  /* Exclusion OS (P8, P16, GS/OS) */
                  for(i=0, exclude=0; i<param->nb_ignore_os_call; i++)
                    if(param->tab_ignore_os_call[i] == current_output->call_number)
                      {
                        exclude = 1;
                        break;
                      }
                  if(exclude == 1)
                    {
                      mem_free_output(current_output);
                      continue;
                    }

                  /* Trace Only OS (P8, P16, GS/OS) */
                  if(param->nb_traceonly_os_call > 0)
                    {
                      for(i=0, include=0; i<param->nb_traceonly_os_call; i++)
                        if(param->tab_traceonly_os_call[i] == current_output->call_number)
                          {
                            include = 1;
                            break;
                          }
                      if(include == 0)
                        {
                          mem_free_output(current_output);
                          continue;
                        }
                    }
                }
              /** On filtre certains appels systmes [Toolbox] **/
              else if(current_output->origin == TRACE_CALL_ORIGIN_TOOL)
                {
                  /* Filtrage par ToolSet */
                  if((param->trace_toolbox_001 == 0 && (current_output->call_number & 0x00FF) == 1)   ||
                     (param->trace_toolbox_002 == 0 && (current_output->call_number & 0x00FF) == 2)   ||
                     (param->trace_toolbox_003 == 0 && (current_output->call_number & 0x00FF) == 3)   ||
                     (param->trace_toolbox_004 == 0 && (current_output->call_number & 0x00FF) == 4)   ||
                     (param->trace_toolbox_005 == 0 && (current_output->call_number & 0x00FF) == 5)   ||
                     (param->trace_toolbox_006 == 0 && (current_output->call_number & 0x00FF) == 6)   ||
                     (param->trace_toolbox_007 == 0 && (current_output->call_number & 0x00FF) == 7)   ||
                     (param->trace_toolbox_008 == 0 && (current_output->call_number & 0x00FF) == 8)   ||
                     (param->trace_toolbox_009 == 0 && (current_output->call_number & 0x00FF) == 9)   ||
                     (param->trace_toolbox_010 == 0 && (current_output->call_number & 0x00FF) == 10)  ||
                     (param->trace_toolbox_011 == 0 && (current_output->call_number & 0x00FF) == 11)  ||
                     (param->trace_toolbox_012 == 0 && (current_output->call_number & 0x00FF) == 12)  ||
                     (param->trace_toolbox_013 == 0 && (current_output->call_number & 0x00FF) == 13)  ||
                     (param->trace_toolbox_014 == 0 && (current_output->call_number & 0x00FF) == 14)  ||
                     (param->trace_toolbox_015 == 0 && (current_output->call_number & 0x00FF) == 15)  ||
                     (param->trace_toolbox_016 == 0 && (current_output->call_number & 0x00FF) == 16)  ||
                     (param->trace_toolbox_017 == 0 && (current_output->call_number & 0x00FF) == 17)  ||
                     (param->trace_toolbox_018 == 0 && (current_output->call_number & 0x00FF) == 18)  ||
                     (param->trace_toolbox_019 == 0 && (current_output->call_number & 0x00FF) == 19)  ||
                     (param->trace_toolbox_020 == 0 && (current_output->call_number & 0x00FF) == 20)  ||
                     (param->trace_toolbox_021 == 0 && (current_output->call_number & 0x00FF) == 21)  ||
                     (param->trace_toolbox_022 == 0 && (current_output->call_number & 0x00FF) == 22)  ||
                     (param->trace_toolbox_023 == 0 && (current_output->call_number & 0x00FF) == 23)  ||
                     (param->trace_toolbox_024 == 0 && (current_output->call_number & 0x00FF) == 24)  ||
                     (param->trace_toolbox_025 == 0 && (current_output->call_number & 0x00FF) == 25)  ||
                     (param->trace_toolbox_026 == 0 && (current_output->call_number & 0x00FF) == 26)  ||
                     (param->trace_toolbox_027 == 0 && (current_output->call_number & 0x00FF) == 27)  ||
                     (param->trace_toolbox_028 == 0 && (current_output->call_number & 0x00FF) == 28)  ||
                     (param->trace_toolbox_029 == 0 && (current_output->call_number & 0x00FF) == 29)  ||
                     (param->trace_toolbox_030 == 0 && (current_output->call_number & 0x00FF) == 30)  ||
                     (param->trace_toolbox_032 == 0 && (current_output->call_number & 0x00FF) == 32)  ||
                     (param->trace_toolbox_033 == 0 && (current_output->call_number & 0x00FF) == 33)  ||
                     (param->trace_toolbox_034 == 0 && (current_output->call_number & 0x00FF) == 34)  ||
                     (param->trace_toolbox_035 == 0 && (current_output->call_number & 0x00FF) == 35)  ||
                     (param->trace_toolbox_037 == 0 && (current_output->call_number & 0x00FF) == 37)  ||
                     (param->trace_toolbox_038 == 0 && (current_output->call_number & 0x00FF) == 38)  ||
                     (param->trace_toolbox_050 == 0 && (current_output->call_number & 0x00FF) == 50)  ||
                     (param->trace_toolbox_051 == 0 && (current_output->call_number & 0x00FF) == 51)  ||
                     (param->trace_toolbox_052 == 0 && (current_output->call_number & 0x00FF) == 52)  ||
                     (param->trace_toolbox_053 == 0 && (current_output->call_number & 0x00FF) == 53)  ||
                     (param->trace_toolbox_128 == 0 && (current_output->call_number & 0x00FF) == 128) ||
                     (param->trace_toolbox_129 == 0 && (current_output->call_number & 0x00FF) == 129) ||
                     (param->trace_toolbox_130 == 0 && (current_output->call_number & 0x00FF) == 130) ||
                     (param->trace_toolbox_131 == 0 && (current_output->call_number & 0x00FF) == 131) ||
                     (param->trace_toolbox_132 == 0 && (current_output->call_number & 0x00FF) == 132) ||
                     (param->trace_toolbox_219 == 0 && (current_output->call_number & 0x00FF) == 219) ||
                     (param->trace_toolbox_220 == 0 && (current_output->call_number & 0x00FF) == 220) ||
                     (param->trace_toolbox_221 == 0 && (current_output->call_number & 0x00FF) == 221) ||
                     (param->trace_toolbox_222 == 0 && (current_output->call_number & 0x00FF) == 222) ||
                     (param->trace_toolbox_230 == 0 && (current_output->call_number & 0x00FF) == 230) ||               
                     (param->trace_toolbox_255 == 0 && (current_output->call_number & 0x00FF) == 255) )
                    {
                      mem_free_output(current_output);
                      continue;
                    }

                  /* Exclusion Toolbox */
                  for(i=0, exclude=0; i<param->nb_ignore_toolbox_call; i++)
                    if(param->tab_ignore_toolbox_call[i] == current_output->call_number)
                      {
                        exclude = 1;
                        break;
                      }
                  if(exclude == 1)
                    {
                      mem_free_output(current_output);
                      continue;
                    }

                  /* Trace Only Toolbox */
                  if(param->nb_traceonly_toolbox_call > 0)
                    {
                      for(i=0, include=0; i<param->nb_traceonly_toolbox_call; i++)
                        if(param->tab_traceonly_toolbox_call[i] == current_output->call_number)
                          {
                            include = 1;
                            break;
                          }
                      if(include == 0)
                        {
                          mem_free_output(current_output);
                          continue;
                        }
                    }
                }
            }

          /** Stockage nouvelle entre **/
          if(current_output->type == BREAKPOINT_ACTION_PRINT || (current_output->type == BREAKPOINT_ACTION_TRACE_SYSTEM_CALL && current_output->is_return == 0))
            {
              /* Ajoute  la liste chaine de la fentre Main */
              EnterCriticalSection(&param->cs_output);
              if(window_main->first_output == NULL)
                {
                  window_main->first_output = current_output;
                  current_output->previous = NULL;
                }
              else
                {
                  window_main->last_output->next = current_output;
                  current_output->previous = window_main->last_output;
                }
              window_main->last_output = current_output;
              window_main->nb_output++;
              LeaveCriticalSection(&param->cs_output);

              /* Stats Internes */
              nb_print_output++;
            }
          else if(current_output->type == BREAKPOINT_ACTION_TRACE_SYSTEM_CALL && current_output->is_return == 1)
            {
              /** On va raccrocher les lignes Return Values **/
              /* On remonte dans la liste */
              EnterCriticalSection(&param->cs_output);
              for(call_output = window_main->last_output; call_output; call_output = call_output->previous)
                if(call_output->goid == current_output->goid)
                  {
                    /* # Cycle */
                    call_output->nb_cycle = current_output->nb_cycle;

                    /* Error Code */
                    call_output->error_code = current_output->error_code;

                    /* On transfre les valeurs des Param retour */
                    for(i=0; i<current_output->nb_param; i++)
                      for(j=0; j<call_output->nb_param; j++)
                        if(call_output->tab_param_value_utf8[j] == NULL && call_output->tab_param_type[j] == TRACE_CALL_PARAM_TYPE_OUT)
                          {
                            call_output->tab_param_value_utf8[j] = current_output->tab_param_value_utf8[i];
                            current_output->tab_param_value_utf8[i] = NULL;    /* No need to free */
                            break;
                          }

                    /* On a bien rcupr le retour */
                    call_output->wait_for_return_value = 0;
                    break;
                  }
               LeaveCriticalSection(&param->cs_output);

              /* Libration mmoire */
              mem_free_output(current_output);
            }
          else    /* BREAKPOINT_ACTION_FILE */
            {
              if(first_file_output == NULL)
                first_file_output = current_output;
              else
                last_file_output->next = current_output;
              last_file_output = current_output;
              nb_file_output++;
            }

          /* Pas plus de 100 lignes ouput */
          if(nb_print_output + nb_file_output > 100)
            break;
        }

      /** Ecriture des lignes Output File dans les fichiers **/
      if(first_file_output != NULL)
        {
          /* Init */
          for(i=0; i<BREAKPOINT_NUMBER_MAX; i++)
            {
              tab_id[i] = 0;
              tab_fd[i] = NULL;
            }

          /** Ouverture des Fichiers / Ajout des lignes Output dans les fichiers **/
          for(current_output = first_file_output; current_output; current_output = current_output->next)
            {
              /* Fichier dj ouvert ? */
              for(i = 0, fd = NULL; i < BREAKPOINT_NUMBER_MAX; i++)
                if(tab_id[i] == current_output->breakpoint_id)
                  {
                    fd = tab_fd[i];
                    break;
                  }

              /** Nouveau Breakpoint **/
              if(fd == NULL)
                {
                  /* Recherche le Breakpoint par son ID */
                  current_breakpoint = GetBreakpointByID(current_output->breakpoint_id);
                  if(current_breakpoint != NULL)
                    {
                      /* A t'on un chemin de fichier ? */
                      if(strlen(current_breakpoint->action_file_path) > 0)
                        {
                          /* Ouverture / Cration du fichier */
                          fd = fopen(current_breakpoint->action_file_path,"a+");
                          if(fd != NULL)
                            {
                              /* Stocke le File Descripteur dans la tableau */
                              for(i = 0; i < BREAKPOINT_NUMBER_MAX; i++)
                                if(tab_fd[i] == NULL)
                                  {
                                    tab_id[i] = current_output->breakpoint_id;
                                    tab_fd[i] = fd;
                                    break;
                                  }
                            }
                        }
                    }
                }
           
              /* Ecriture dans le fichier */
              if(fd != NULL)
                fprintf(fd,"%s\n",current_output->line);
            }

          /* Fermeture des fichiers */
          for(i=0; i<BREAKPOINT_NUMBER_MAX; i++)
            if(tab_fd[i] != NULL)
              fclose(tab_fd[i]);

          /* Libration mmoire */
          for(current_output = first_file_output; current_output; current_output = next_output)
            {
              next_output = current_output->next;
              mem_free_output(current_output);
            }
        }

      /* Une demande d'arrt vient d'arriver */
      if(param->stop_output_thread == 1)
        break;

      /* On va faire varier le temps d'attente 1-200 ms en fonction de la proximit de la dernire output */
      if((nb_print_output + nb_file_output) > 0)
        wait_delay = 0;
      wait_delay = (wait_delay >= 200) ? 200 : (wait_delay+1);

      /** On s'endort en attendant soit un changement de l'Event soit un dlai de 1-200 ms **/
      if(wait_delay > 1)
        Sleep(wait_delay);
    }

  /* Fin du Thread */
  param->output_thread_status = 0;
  ExitThread(0);

  /* OK */
  return(0);
}


/***************************************************************************************/
/*  ExtractOutputFromSharedMemory() :  On extrait une ligne Output de la SharedMemory. */ 
/***************************************************************************************/
static struct output *ExtractOutputFromSharedMemory(struct window_ipc *window_ipc, struct parameter *param)
{
  int i, length, length_cycle, length_error, length_name;
  BYTE value;
  DWORD offset_read, new_offset, goid_high, goid_low, dwd_value, address_bank, address_addr;
  char *begin_cycle;
  char *end_cycle;
  char *begin_error;
  char *end_error;
  char *begin_param;
  char *begin_name;
  char *end_name;
  char buffer_id[16];
  char buffer_address[16];
  char buffer_origin[16];
  char buffer_call_number[16];
  struct output *current_output;
  char buffer[2048];
  //char buffer_time[256];

  /* La Shared Memory est t'elle disponible ? */
  if(window_ipc->shared_memory_ptr == NULL)
    return(NULL);

  /* Offset de dpart pour les Read dans la Shared Memory */
  memcpy(&offset_read,&window_ipc->shared_memory_ptr[IPC_SHARED_MEMORY_OFFSET_OUTPUT_READ],sizeof(DWORD));

  /* Y a t'il une ligne Output en attente ? */
  value = GetSharedMemoryCircularValue(offset_read,window_ipc->shared_memory_ptr);
  if(value == 0x00)
    return(NULL);

  /* Calcule la longueur de la ligne Output */
  for(length=0; ; length++)
    {
      value = GetSharedMemoryCircularValue(offset_read+length,window_ipc->shared_memory_ptr);
      if(value == '\n')
        break;
    }
  length++;
  if(length < 9)
    return(NULL);

  /** Allocation mmoire **/
  current_output = (struct output *) calloc(1,sizeof(struct output));
  if(current_output == NULL)
    return(NULL);
  current_output->line = (char *) calloc(length+1,sizeof(char));
  if(current_output->line == NULL)
    {
      free(current_output);
      return(NULL);
    }
  current_output->offset_sm = offset_read;          /* Position dans la Shared Memory */
  current_output->length_sm = (DWORD) length;

  /** Remplissage **/
  /* Type */
  value = GetSharedMemoryCircularValue(offset_read+0,window_ipc->shared_memory_ptr);
  if(value == 'F')
    current_output->type = BREAKPOINT_ACTION_FILE;
  else if(value == 'P')
    current_output->type = BREAKPOINT_ACTION_PRINT;
  else if(value == 'T')
    current_output->type = BREAKPOINT_ACTION_TRACE_SYSTEM_CALL;

  /* Breakpoint ID */
  if(current_output->type == BREAKPOINT_ACTION_FILE || current_output->type == BREAKPOINT_ACTION_PRINT)
    {
      buffer_id[0] = value;
      for(i=0; i<8; i++)
        buffer_id[1+i] = GetSharedMemoryCircularValue(offset_read+1+i,window_ipc->shared_memory_ptr);
      buffer_id[1+8] = '\0';
      sscanf(&buffer_id[1],"%08X",&current_output->breakpoint_id);
    }
  else if(current_output->type == BREAKPOINT_ACTION_TRACE_SYSTEM_CALL)
    {
      buffer_origin[0] = value;
      /* Origin : GSOS | P16 | P8 | TOOL */
      for(i=0; i<4; i++)
        buffer_origin[1+i] = GetSharedMemoryCircularValue(offset_read+1+i,window_ipc->shared_memory_ptr);
      buffer_origin[1+4] = '\0';
      
      if(!stricmp(&buffer_origin[1],"GSOS"))
        current_output->origin = TRACE_CALL_ORIGIN_GSOS;
      else if(!stricmp(&buffer_origin[1],"P16 "))
        current_output->origin = TRACE_CALL_ORIGIN_P16;
      else if(!stricmp(&buffer_origin[1],"P8  "))
        current_output->origin = TRACE_CALL_ORIGIN_P8;
      else if(!stricmp(&buffer_origin[1],"TOOL"))
        current_output->origin = TRACE_CALL_ORIGIN_TOOL;
      else if(!stricmp(&buffer_origin[1],"RETU"))
        current_output->origin = TRACE_CALL_ORIGIN_RETURN;

      /* Call Number */
      for(i=0; i<4; i++)
        buffer_call_number[i] = GetSharedMemoryCircularValue(offset_read+5+i,window_ipc->shared_memory_ptr);
      buffer_call_number[4] = '\0';
      sscanf(buffer_call_number,"%04X",&current_output->call_number);
    }

  /* Line Data */
  for(i=0; i<(length-(1+8+1)); i++)
    current_output->line[i] = GetSharedMemoryCircularValue(offset_read+1+8+i,window_ipc->shared_memory_ptr);

  /** RAZ la Ligne Output dans la Shared Memory (on commence par la fin) **/
  for(i=length-1; i>=0; i--)
    SetSharedMemoryCircularValue(offset_read+i,window_ipc->shared_memory_ptr,0x00);

  /** MAJ de l'offset pour les Read dans la Shared Memory **/
  new_offset = offset_read + length;
  new_offset = (new_offset < IPC_SHARED_MEMORY_OUTPUT_SIZE) ? new_offset : (new_offset - IPC_SHARED_MEMORY_OUTPUT_SIZE);
  memcpy(&window_ipc->shared_memory_ptr[IPC_SHARED_MEMORY_OFFSET_OUTPUT_READ],&new_offset,sizeof(DWORD));

  /** Dcode le GOID **/
  memcpy(buffer_id,&current_output->line[SYSTEMCALL_RESULT_VALUE_GOID_HIGH_OFFSET],8);
  buffer_id[8] = '\0';
  sscanf(buffer_id,"%08X",&goid_high);
  memcpy(buffer_id,&current_output->line[SYSTEMCALL_RESULT_VALUE_GOID_LOW_OFFSET],8);
  buffer_id[8] = '\0';
  sscanf(buffer_id,"%08X",&goid_low);
  current_output->goid = (((QWORD)goid_high) << 32) | goid_low;

  /** Dcode l'adresse **/
  memcpy(buffer_address,&current_output->line[SYSTEMCALL_ADDRESS_BANK_OFFSET],2);
  buffer_address[2] = '\0';
  sscanf(buffer_address,"%02X",&address_bank);
  memcpy(buffer_address,&current_output->line[SYSTEMCALL_ADDRESS_ADDR_OFFSET],4);
  buffer_address[4] = '\0';
  sscanf(buffer_address,"%04X",&address_addr);
  current_output->address = (address_bank << 16) | address_addr;

  /* Est-ce un retour de valeurs ? */
  if(!strnicmp(&current_output->line[SYSTEMCALL_RESULT_VALUE_ARROW_OFFSET],"=>",2))
    current_output->is_return = 1;

  /** On va dcoder les paramtres du System Call lors de l'appel **/
  if(current_output->type == BREAKPOINT_ACTION_TRACE_SYSTEM_CALL && current_output->is_return == 0)
    {
      /* On va attendre le retour de la fonction pour complter : Cycle + Param Out + Code Erreur */
      current_output->wait_for_return_value = 1;

      /* Nom de la fonction */
      begin_name = strstr(current_output->line,"::");
      if(begin_name == NULL)
        ;
      begin_name += 2;
      end_name = strchr(begin_name,'(');
      if(end_name == NULL)
        ;
      length_name = (int) (end_name - begin_name);
      current_output->function_name = (char *) calloc(length_name+1,sizeof(char)); 
      if(current_output->function_name == NULL)
        ;
      memcpy(current_output->function_name,begin_name,length_name);

      if(!stricmp(current_output->function_name,"Destroy"))
        printf("");

      /** Paramtres de la fonction IN/OUT **/
      begin_param = strchr(current_output->line,'(');
      if(begin_param == NULL)
        ;
      begin_param++;
      length = GetReturnValueParam(begin_param,buffer);
      while(length != 0)
        {
          /* Conserve ce paramtre dans le tableau */
          if(!stricmp(buffer,"-"))
            {
              /* Paramtre OUT */
              current_output->tab_param_value_utf8[current_output->nb_param] = NULL;
              current_output->tab_param_type[current_output->nb_param] = TRACE_CALL_PARAM_TYPE_OUT;
              current_output->nb_param++;
            }
          else
            {
              /* Paramtre IN */
              current_output->tab_param_value_utf8[current_output->nb_param] = strdup(buffer);
              if(current_output->tab_param_value_utf8[current_output->nb_param] == NULL)
                break;
              current_output->tab_param_type[current_output->nb_param] = TRACE_CALL_PARAM_TYPE_IN;
              current_output->nb_param++;
            }

          /* Prochain paramtre */
          begin_param += length + 1;
          length = GetReturnValueParam(begin_param,buffer);
        }
    }

  /** On va dcoder le Nombre de cycle + Return Value + Error Code **/
  if(current_output->is_return == 1)
    {
      /** Nombre de Cycle **/
      begin_cycle = &current_output->line[SYSTEMCALL_RESULT_VALUE_CYCLE_OFFSET];
      end_cycle = strchr(begin_cycle,']');
      length_cycle = (int) (end_cycle - begin_cycle);
      memcpy(buffer,begin_cycle,length_cycle);
      buffer[length_cycle] = '\0';
      current_output->nb_cycle = (DWORD) atoi(buffer);

      /** Code d'Erreur **/
      begin_error = end_cycle + 1;
      if(!strnicmp(begin_error,"Error (",strlen("Error (")))
        {
          /* Decode Error Code */
          begin_error += strlen("Error (");
          end_error = strchr(begin_error,')');
          length_error = (int) (end_error - begin_error);
          memcpy(buffer,begin_error,length_error);
          buffer[length_error] = '\0';
          sscanf(buffer,"%X",&dwd_value);
          current_output->error_code = (WORD) dwd_value;
        }
      else
        {
          /** Paramtres en Retour **/
          begin_param = end_cycle + 2;
          length = GetReturnValueParam(begin_param,buffer);
          while(length != 0)
            {
              /* Conserve ce paramtre dans le tableau */
              current_output->tab_param_value_utf8[current_output->nb_param] = strdup(buffer);
              if(current_output->tab_param_value_utf8[current_output->nb_param] == NULL)
                break;
              current_output->tab_param_type[current_output->nb_param] = TRACE_CALL_PARAM_TYPE_OUT;
              current_output->nb_param++;

              /* Prochain paramtre */
              begin_param += length + 1;
              length = GetReturnValueParam(begin_param,buffer);
            }
        }
    }

  /* Debug Shared Memory I/O */
 /* sprintf(buffer_log,"%s  O=%08X  L=%X  O=%08X  '",GetCurrentTimeMS(buffer_time),offset_read,length,new_offset);
  if(current_output->type == BREAKPOINT_ACTION_FILE || current_output->type == BREAKPOINT_ACTION_PRINT)
    strcat(buffer_log,buffer_id);
  else
    {
      strcat(buffer_log,buffer_origin);
      strcat(buffer_log,buffer_call_number);
    }
  LogTextLineDirect("c:\\AppleIIgs\\Cyrene_SharedMemory.txt",buffer_log,0);
  LogTextLineDirect("c:\\AppleIIgs\\Cyrene_SharedMemory.txt",current_output->line,0);
  LogTextLineDirect("c:\\AppleIIgs\\Cyrene_SharedMemory.txt","'",1); */

  /* Renvoi la ligne Output */
  return(current_output);
}


/****************************************************************************/
/*  GetReturnValueParam() :  Rcupre un paramtre de retour du SystemCall. */
/****************************************************************************/
static int GetReturnValueParam(char *begin, char *buffer_rtn)
{
  int i, j, data_length, param_length;

  /* Init */
  param_length = 0;
  strcpy(buffer_rtn,"");

  /* Vide ? */
  if(begin[0] == ',' || begin[0] == ')' || begin[0] == '\0')
    return(param_length);

  /** Hexa ou Chaine de caractres ? **/
  for(i=0, j=0; ; i++, j++)
    {
      if(i > 512 || j > 512)
        printf("");

      /* Fin */
      if(begin[i]== ',' || begin[i]== ')')
        {
          buffer_rtn[j] = '\0';
          break;
        }

      /* Debut de chaine */
      if(begin[i] == '"')
        {
          /* Dcode la longueur */
          buffer_rtn[j] = '\0';
          data_length = (int) atoi(buffer_rtn);

          /* Copie la chaine */
          memcpy(buffer_rtn,&begin[i],data_length+2);
          buffer_rtn[data_length+2] = '\0';

          /* Longueur du Param */
          param_length += (data_length+2);
          break;
        }

      /* Caractres valide */
      buffer_rtn[j] = begin[i];
      param_length++;
    }

  /* Renvoie la longueur du paramtre */
  return(param_length);
}


/*******************************************************************************/
/*  GetSharedMemoryCircularValue() :  Rcupre un octet dans la Shared Memory. */
/*******************************************************************************/
static BYTE GetSharedMemoryCircularValue(DWORD offset, unsigned char *shared_memory)
{
  DWORD offset_circular;

  /* Offset dans un buffer circulaire */
  offset_circular = (offset < IPC_SHARED_MEMORY_OUTPUT_SIZE) ? offset : (offset - IPC_SHARED_MEMORY_OUTPUT_SIZE);

  /* Renvoie la valeur */
  return(shared_memory[IPC_SHARED_MEMORY_OFFSET_DATA_OUTPUT + offset_circular]);
}


/******************************************************************************/
/*  SetSharedMemoryCircularValue() :  Modifie un octet dans la Shared Memory. */
/******************************************************************************/
static void SetSharedMemoryCircularValue(DWORD offset, unsigned char *shared_memory, BYTE value)
{
  DWORD offset_circular;

  /* Offset dans un buffer circulaire */
  offset_circular = (offset < IPC_SHARED_MEMORY_OUTPUT_SIZE) ? offset : (offset - IPC_SHARED_MEMORY_OUTPUT_SIZE);

  /* Modifie la valeur */
  shared_memory[IPC_SHARED_MEMORY_OFFSET_DATA_OUTPUT + offset_circular] = value;
}


/********************************************************************/
/*  mem_free_output() :  Libration mmoire de la structure output. */
/********************************************************************/
void mem_free_output(struct output *current_output)
{
  int i;

  if(current_output)
    {
      if(current_output->line)
        free(current_output->line);

      if(current_output->function_name)
        free(current_output->function_name);

      for(i=0; i<current_output->nb_param; i++)
        if(current_output->tab_param_value_utf8[i])
          free(current_output->tab_param_value_utf8[i]);

      free(current_output);
    }
}

/******************************************************************************/
