/************************************************************************/
/*                                                                      */
/*  Dc_Project.c :  Module pour les fonctions de gestion des Projets.   */
/*                                                                      */
/************************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe Software  *  Avril 2018  */
/************************************************************************/

#include <stdio.h>
#include <time.h>

#include <windows.h>

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_Project.h"

#define TYPE_DATA_TEXT     1
#define TYPE_DATA_BANK     2
#define TYPE_DATA_ADDRESS  3
#define TYPE_DATA_WORD     4
#define TYPE_DATA_BYTE     5
#define TYPE_DATA_COLOR    5
#define TYPE_DATA_BOOL     6
#define TYPE_DATA_INTEGER  7
#define TYPE_DATA_IIGSROM  8

static void GetProjectFileValue(char *,char *,int,void *);
static struct file_path *mem_alloc_projectfilepath(void);

/*********************************************************************************/
/*  SaveProjectFile() :  Cration d'un fichier Texte avec les donnes du projet. */
/*********************************************************************************/
int SaveProjectFile(struct project *current_project, char *file_path)
{
  int i;
  FILE *fd;

  /* Cration du fichier Text */
  fd = fopen(file_path,"w");
  if(fd == NULL)
    return(0);

  /** Files 0-8 **/
  fprintf(fd,"###  File Data\n");
  for(i=0; i<9; i++)
    if(current_project->tab_file[i] != NULL)
      {
        fprintf(fd,"File%d_Path=%s\n",i,current_project->tab_file[i]->path);
        fprintf(fd,"File%d_Bank=%02X\n",i,current_project->tab_file_bank[i]);
        fprintf(fd,"File%d_Address=%04X\n",i,current_project->tab_file_address[i]);
      }
  fprintf(fd,"\n");

  /** File Sound **/
  if(current_project->file_sound != NULL)
    {
      fprintf(fd,"###  File Sound\n");
      fprintf(fd,"FileSound_Path=%s\n",current_project->file_sound->path);
      fprintf(fd,"FileSound_Address=%04X\n",current_project->file_sound_address);
      fprintf(fd,"\n");
    }

  /** Merlin 32 output files **/
  if(strlen(current_project->file_merlin32_path) > 0)
    {
      fprintf(fd,"###  Merlin 32 Files\n");
      fprintf(fd,"FileMerlin32_Path=%s\n",current_project->file_merlin32_path);
      fprintf(fd,"\n");
    }
  
  /* Processor */
  fprintf(fd,"###  Processor\n");
  fprintf(fd,"Proc.A=%04X\n",current_project->proc_a);
  fprintf(fd,"Proc.X=%04X\n",current_project->proc_x);
  fprintf(fd,"Proc.Y=%04X\n",current_project->proc_y);
  fprintf(fd,"Proc.DBR=%02X\n",current_project->proc_dbr);
  fprintf(fd,"Proc.PBR=%02X\n",current_project->proc_pbr);
  fprintf(fd,"Proc.PC=%04X\n",current_project->proc_pc);
  fprintf(fd,"Proc.STACK=%04X\n",current_project->proc_stack);
  fprintf(fd,"Proc.DIRECTPAGE=%04X\n",current_project->proc_directpage);
  fprintf(fd,"\n");

  /* Status Register */
  fprintf(fd,"###  Status Register\n");
  fprintf(fd,"Status.C=%d\n",current_project->status_c);
  fprintf(fd,"Status.Z=%d\n",current_project->status_z);
  fprintf(fd,"Status.I=%d\n",current_project->status_i);
  fprintf(fd,"Status.D=%d\n",current_project->status_d);
  fprintf(fd,"Status.X=%d\n",current_project->status_x);
  fprintf(fd,"Status.M=%d\n",current_project->status_m);
  fprintf(fd,"Status.V=%d\n",current_project->status_v);
  fprintf(fd,"Status.N=%d\n",current_project->status_n);
  fprintf(fd,"Status.E=%d\n",current_project->status_e);
  fprintf(fd,"Status.B=%d\n",current_project->status_b);
  fprintf(fd,"\n");

  /* RAM Size */
  fprintf(fd,"###  RAM Size\n");
  fprintf(fd,"RamSize=%d\n",current_project->ram_size);
  fprintf(fd,"\n");

  /* Apple II ROM */
  fprintf(fd,"###  Apple II ROM\n");
  fprintf(fd,"AppleIIRom=%d\n",current_project->appleii_rom);
  fprintf(fd,"\n");

  /* Apple IIgs ROM */
  fprintf(fd,"###  Apple IIgs ROM\n");
  fprintf(fd,"AppleIIgsRom=%d\n",current_project->appleiigs_rom);
  fprintf(fd,"\n");

  /* Display Preferences */
  fprintf(fd,"###  Display Preferences\n");
  fprintf(fd,"ColorText=%d\n",current_project->color_text);
  fprintf(fd,"ColorBackground=%d\n",current_project->color_background);
  fprintf(fd,"ColorBorder=%d\n",current_project->color_border);
  fprintf(fd,"\n");

  /* Fermeture du fichier */
  fclose(fd);

  /* OK */
  return(0);
}


/******************************************************************/
/*  LoadProjectFile() :  Charge en mmoire les donnes du projet. */
/******************************************************************/
struct project *LoadProjectFile(char *file_path, char *buffer_error_rtn)
{
  int i, file_size;
  char *file_data;
  struct project *current_project;
  char buffer_error[BUFFER_SIZE];

  /** Charge le contenu du fichier Text **/
  file_data = (char *) LoadTextFile(file_path,&file_size,&buffer_error[0]);
  if(file_data == NULL)
    {
      sprintf(buffer_error_rtn,"Impossible to load file '%s' in memory",file_path);
      return(NULL);
    }

  /* Allocation mmoire */
  current_project = (struct project *) calloc(1,sizeof(struct project));
  if(current_project == NULL)
    {
      strcpy(buffer_error_rtn,"Impossible to allocate memory for this project");
      free(file_data);
      return(NULL);
    }
  current_project->file_sound = mem_alloc_projectfilepath();
  if(current_project->file_sound == NULL)
    {
      strcpy(buffer_error_rtn,"Impossible to allocate memory for this project");
      mem_free_project(current_project);
      return(NULL);
    }
  for(i=0; i<9; i++)
    {
      current_project->tab_file[i] = mem_alloc_projectfilepath();
      if(current_project->tab_file[i] == NULL)
        {
          strcpy(buffer_error_rtn,"Impossible to allocate memory for this project");
          mem_free_project(current_project);
          return(NULL);
        }
    }

  /** Valeurs par dfaut **/
  current_project->proc_a = 0;
  current_project->proc_x = 0;
  current_project->proc_y = 0;
  current_project->proc_dbr = 0;
  current_project->proc_pbr = 0;
  current_project->proc_pc = 0x0300;
  current_project->proc_stack = 0x01FF;
  current_project->proc_directpage = 0;
  current_project->status_c = 0;
  current_project->status_z = 0;
  current_project->status_i = 1;
  current_project->status_d = 0;
  current_project->status_x = 1;
  current_project->status_m = 1;
  current_project->status_v = 0;
  current_project->status_n = 0;
  current_project->status_e = 1;
  current_project->status_b = 0;  
  current_project->ram_size = 16;
  current_project->appleii_rom = 1;
  current_project->appleiigs_rom = PROJECT_APPLEIIGS_ROM01;          /* PROJECT_APPLEIIGS_XXX */
  current_project->color_text = COLOR_DISPLAY_CODE_WHITE;
  current_project->color_background = COLOR_DISPLAY_CODE_MEDIUMBLUE;
  current_project->color_border = COLOR_DISPLAY_CODE_MEDIUMBLUE;

  /** Rcupration des valeurs du fichier **/
  /* File 0-8 */
  GetProjectFileValue(file_data,"File0_Path",TYPE_DATA_TEXT,&current_project->tab_file[0]->path[0]);
  GetProjectFileValue(file_data,"File0_Bank",TYPE_DATA_BANK,&current_project->tab_file_bank[0]);
  GetProjectFileValue(file_data,"File0_Address",TYPE_DATA_ADDRESS,&current_project->tab_file_address[0]);
  GetProjectFileValue(file_data,"File1_Path",TYPE_DATA_TEXT,&current_project->tab_file[1]->path[0]);
  GetProjectFileValue(file_data,"File1_Bank",TYPE_DATA_BANK,&current_project->tab_file_bank[1]);
  GetProjectFileValue(file_data,"File1_Address",TYPE_DATA_ADDRESS,&current_project->tab_file_address[1]);
  GetProjectFileValue(file_data,"File2_Path",TYPE_DATA_TEXT,&current_project->tab_file[2]->path[0]);
  GetProjectFileValue(file_data,"File2_Bank",TYPE_DATA_BANK,&current_project->tab_file_bank[2]);
  GetProjectFileValue(file_data,"File2_Address",TYPE_DATA_ADDRESS,&current_project->tab_file_address[2]);
  GetProjectFileValue(file_data,"File3_Path",TYPE_DATA_TEXT,&current_project->tab_file[3]->path[0]);
  GetProjectFileValue(file_data,"File3_Bank",TYPE_DATA_BANK,&current_project->tab_file_bank[3]);
  GetProjectFileValue(file_data,"File3_Address",TYPE_DATA_ADDRESS,&current_project->tab_file_address[3]);
  GetProjectFileValue(file_data,"File4_Path",TYPE_DATA_TEXT,&current_project->tab_file[4]->path[0]);
  GetProjectFileValue(file_data,"File4_Bank",TYPE_DATA_BANK,&current_project->tab_file_bank[4]);
  GetProjectFileValue(file_data,"File4_Address",TYPE_DATA_ADDRESS,&current_project->tab_file_address[4]);
  GetProjectFileValue(file_data,"File5_Path",TYPE_DATA_TEXT,&current_project->tab_file[5]->path[0]);
  GetProjectFileValue(file_data,"File5_Bank",TYPE_DATA_BANK,&current_project->tab_file_bank[5]);
  GetProjectFileValue(file_data,"File5_Address",TYPE_DATA_ADDRESS,&current_project->tab_file_address[5]);
  GetProjectFileValue(file_data,"File6_Path",TYPE_DATA_TEXT,&current_project->tab_file[6]->path[0]);
  GetProjectFileValue(file_data,"File6_Bank",TYPE_DATA_BANK,&current_project->tab_file_bank[6]);
  GetProjectFileValue(file_data,"File6_Address",TYPE_DATA_ADDRESS,&current_project->tab_file_address[6]);
  GetProjectFileValue(file_data,"File7_Path",TYPE_DATA_TEXT,&current_project->tab_file[7]->path[0]);
  GetProjectFileValue(file_data,"File7_Bank",TYPE_DATA_BANK,&current_project->tab_file_bank[7]);
  GetProjectFileValue(file_data,"File7_Address",TYPE_DATA_ADDRESS,&current_project->tab_file_address[7]);
  GetProjectFileValue(file_data,"File8_Path",TYPE_DATA_TEXT,&current_project->tab_file[8]->path[0]);
  GetProjectFileValue(file_data,"File8_Bank",TYPE_DATA_BANK,&current_project->tab_file_bank[8]);
  GetProjectFileValue(file_data,"File8_Address",TYPE_DATA_ADDRESS,&current_project->tab_file_address[8]);
  /* File Sound */
  GetProjectFileValue(file_data,"FileSound_Path",TYPE_DATA_TEXT,&current_project->file_sound->path[0]);
  GetProjectFileValue(file_data,"FileSound_Address",TYPE_DATA_ADDRESS,&current_project->file_sound_address);
  /* Merlin 32 output Files */
  GetProjectFileValue(file_data,"FileMerlin32_Path",TYPE_DATA_TEXT,&current_project->file_merlin32_path[0]);
  /* Processor */
  GetProjectFileValue(file_data,"Proc.A",TYPE_DATA_WORD,&current_project->proc_a);
  GetProjectFileValue(file_data,"Proc.X",TYPE_DATA_WORD,&current_project->proc_x);
  GetProjectFileValue(file_data,"Proc.Y",TYPE_DATA_WORD,&current_project->proc_y);
  GetProjectFileValue(file_data,"Proc.DBR",TYPE_DATA_BYTE,&current_project->proc_dbr);
  GetProjectFileValue(file_data,"Proc.PBR",TYPE_DATA_BYTE,&current_project->proc_pbr);
  GetProjectFileValue(file_data,"Proc.PC",TYPE_DATA_WORD,&current_project->proc_pc);
  GetProjectFileValue(file_data,"Proc.STACK",TYPE_DATA_WORD,&current_project->proc_stack);
  GetProjectFileValue(file_data,"Proc.DIRECTPAGE",TYPE_DATA_WORD,&current_project->proc_directpage);
  /* Status Register */
  GetProjectFileValue(file_data,"Status.C",TYPE_DATA_BOOL,&current_project->status_c);
  GetProjectFileValue(file_data,"Status.Z",TYPE_DATA_BOOL,&current_project->status_z);
  GetProjectFileValue(file_data,"Status.I",TYPE_DATA_BOOL,&current_project->status_i);
  GetProjectFileValue(file_data,"Status.D",TYPE_DATA_BOOL,&current_project->status_d);
  GetProjectFileValue(file_data,"Status.X",TYPE_DATA_BOOL,&current_project->status_x);
  GetProjectFileValue(file_data,"Status.M",TYPE_DATA_BOOL,&current_project->status_m);
  GetProjectFileValue(file_data,"Status.V",TYPE_DATA_BOOL,&current_project->status_v);
  GetProjectFileValue(file_data,"Status.N",TYPE_DATA_BOOL,&current_project->status_n);
  GetProjectFileValue(file_data,"Status.E",TYPE_DATA_BOOL,&current_project->status_e);
  GetProjectFileValue(file_data,"Status.B",TYPE_DATA_BOOL,&current_project->status_b);
  /* Ram Size */
  GetProjectFileValue(file_data,"RamSize",TYPE_DATA_INTEGER,&current_project->ram_size);
  /* Apple II Rom */
  GetProjectFileValue(file_data,"AppleIIRom",TYPE_DATA_BOOL,&current_project->appleii_rom);
  /* Apple IIgs Rom */
  GetProjectFileValue(file_data,"AppleIIgsRom",TYPE_DATA_IIGSROM,&current_project->appleiigs_rom);
  /* Display Preferences */
  GetProjectFileValue(file_data,"ColorText",TYPE_DATA_COLOR,&current_project->color_text);
  GetProjectFileValue(file_data,"ColorBackground",TYPE_DATA_COLOR,&current_project->color_background);
  GetProjectFileValue(file_data,"ColorBorder",TYPE_DATA_COLOR,&current_project->color_border);

  /* Chemin du fichier Projet */
  strcpy(current_project->project_file_path,file_path);

  /* Libration mmoire */
  free(file_data);

  /** Libration des structure file_path non utilises **/
  if(strlen(current_project->file_sound->path) == 0)
    {
      mem_free_filepath(current_project->file_sound);
      current_project->file_sound = NULL;
    }
  for(i=0; i<9; i++)
    if(strlen(current_project->tab_file[i]->path) == 0);
      {
        mem_free_filepath(current_project->tab_file[i]);
        current_project->tab_file[i] = NULL;
      }

  /* Renvoi le projet */
  return(current_project);
}


/**************************************************************************************/
/*  GetProjectFileValue() :  Rcupre une valeur dans les donnes du fichier Project. */
/**************************************************************************************/
static void GetProjectFileValue(char *file_data, char *item_name, int item_type, void *value_rtn)
{
  int line_length, name_length, value_length, is_error;
  char *begin_line;
  char *end_line;
  char *next_sep;
  char name[BUFFER_SIZE];
  char value[BUFFER_SIZE];
  BYTE value_byte;
  WORD value_word;
  int value_int;

  /** Recherche la valeur Text dans le fichier **/
  begin_line = file_data;
  while(begin_line)
    {
      /* Isole une ligne */
      end_line = strchr(begin_line,'\n');
      if(end_line == NULL)
        end_line = &file_data[strlen(file_data)];
      line_length = end_line - begin_line;

      /* Ligne de commentaire ? */
      if(begin_line[0] == '#' || begin_line == end_line)
        {
          /* Ligne suivante */
          if(end_line[0] == '\0')
            return;
          begin_line = end_line + 1;
          continue;
        }

      /* Sparateur Name=Value */
      next_sep = strchr(begin_line,'=');
      if(next_sep == NULL)
        return;
      if(next_sep > end_line)
        {
          /* Ligne suivante */
          if(end_line[0] == '\0')
            break;
          begin_line = end_line + 1;
          continue;
        }

      /* Isole le nom */
      name_length = next_sep - begin_line;
      if(name_length > BUFFER_SIZE)
        return;
      memcpy(name,begin_line,name_length);
      name[name_length]  = '\0';

      /* Isole la valeur */
      value_length = end_line - (next_sep+1);
      if(value_length > BUFFER_SIZE)
        return;
      memcpy(value,next_sep+1,value_length);
      value[value_length]  = '\0';

      /* Est-ce le nom attendu ? */
      CleanBuffer(name);
      if(!stricmp(name,item_name))
        break;

      /* Ligne suivante */
      if(end_line[0] == '\0')
        return;
      begin_line = end_line + 1;
      continue;
    }

  /** On a rcupr la valeur, on va la contextualiser selon le type **/
  CleanBuffer(value);
  if(item_type == TYPE_DATA_TEXT)
    {
      strcpy((char *)value_rtn,value);
    }
  else if(item_type == TYPE_DATA_BANK)
    {
      value_byte = EvaluateAsByte(value,&is_error);
      if(is_error == 0)
        *((BYTE *)value_rtn) = value_byte;
    }
  else if(item_type == TYPE_DATA_ADDRESS || item_type == TYPE_DATA_WORD)
    {
      value_word = EvaluateAsWord(value,&is_error);
      if(is_error == 0)
        *((WORD *)value_rtn) = value_word;
    }
  else if(item_type == TYPE_DATA_COLOR)
    {
      value_int = atoi(value);
      if(value_int >= 0 && value_int <= 15)
        *((int *)value_rtn) = value_int;
    }
  else if(item_type == TYPE_DATA_BOOL)
    {
      value_int = EvaluateAsBit(value,&is_error);
      if(is_error == 0)
        *((int *)value_rtn) = value_int;
    }
  else if(item_type == TYPE_DATA_INTEGER)
    {
      value_int = atoi(value);
      if(value_int >= 0)
        *((int *)value_rtn) = value_int;
    }
  else if(item_type == TYPE_DATA_IIGSROM)
    {
      value_int = atoi(value);
      if(value_int == 0 || value_int == 1 || value_int == 3)
        *((int *)value_rtn) = value_int;
    }
}


/**************************************************************************************/
/*  mem_alloc_projectfilepath() :  Allocation mmoire d'une structure file_path vide. */
/**************************************************************************************/
static struct file_path *mem_alloc_projectfilepath(void)
{
  struct file_path *current_filepath;

  /* Allocation mmoire */
  current_filepath = (struct file_path *) calloc(1,sizeof(struct file_path));
  if(current_filepath == NULL)
    return(NULL);
  current_filepath->path = (char *) calloc(1,BUFFER_SIZE);
  if(current_filepath->path == NULL)
    {
      free(current_filepath);
      return(NULL);
    }

  /* Renvoi la structure */
  return(current_filepath);
}


/**********************************************************************/
/*  mem_free_project() :  Libration mmoire de la structure project. */
/**********************************************************************/
void mem_free_project(struct project *current_project)
{
  int i;
  struct file_path *current_filepath;
  struct file_path *next_filepath;
  
  if(current_project)
    {
      /* Libration des donnes des fichiers */
      for(i=0; i<9; i++)
        mem_free_filepath(current_project->tab_file[i]);
      
      /* Sound File */
      mem_free_filepath(current_project->file_sound);    
    
      /* Liste des fichiers Merlin 32 */
      for(current_filepath=current_project->first_merlin32_file; current_filepath; current_filepath=next_filepath)
        {
          next_filepath = current_filepath->next;
          mem_free_filepath(current_filepath);
        }
        
      /* Libration structure */
      free(current_project);
    }
}

/************************************************************************/
