/*****************************************************************************
 *                                                                           *
 *  Dc_Request.h :  Module regroupant les fonctions des Requtes.            *
 *                                                                           *
 *****************************************************************************
 *  Auteur : Olivier ZARDINI  *  Brutal Deluxe Software  *  Date : May 2023  *
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <process.h>
#include <malloc.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <io.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/timeb.h>
#include <direct.h>
#include <errno.h>

#include <windows.h>

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_MMU.h"
#include "Dc_Snapshot.h"
#include "Dc_Notification.h"
#include "Cyrene_Engine.h"
#include "Window_IPC.h"
#include "Window_QuickWatch.h"
#include "Window_Main.h"
#include "Window_Main_Menu.h"
#include "Window_MemoryMap.h"
#include "Window_Statistic.h"
#include "Window_Profile.h"
#include "Window_Symbol.h"
#include "Window_Breakpoint.h"
#include "Dc_Disassembler.h"
#include "Dc_Request.h"
#include "resource.h"

static DWORD StartRequestThreadInit(void *);
static void ProcessOneRequest(struct request *);
static int SendRequestToKegs(struct request *);
static void BuildOperationData(struct request *,int,struct operation *);
static void UpdateMemoryMapRWX(DWORD,int,int,unsigned char,struct mmu *);


/********************************************************************/
/*  StartRequestManager() :  Lance le Thread en charge des Request. */
/********************************************************************/
int StartRequestManager()
{
  HANDLE thread_handle;
  DWORD thread_id;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /******************************************************/
  /*** On va lancer le Thread de Gestion des Requtes ***/
  param->request_thread_status = 1;
  
  /* Init : Reset Event */
  ResetEvent(param->thread_request_event);

  /** Lancement du Thread **/
  thread_handle = CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)StartRequestThreadInit,NULL,0,&thread_id);
  if(thread_handle == NULL)
    {
      /* Log l'erreur */
      param->request_thread_status = 0;   
      return(1);
    }
  
  /* Conserve le Handle de ce Thread */
  param->request_thread_handle = thread_handle;
  
  /* OK */
  return(0);
}


/*************************************************************************/
/*  StartRequestThreadInit() :  Traite les requtes via un Thread ddi. */
/*************************************************************************/
static DWORD StartRequestThreadInit(void *nothing_here)
{
  DWORD result_wait, exit_code;
  HANDLE process_handle;
  int has_request, wait_delay;
  struct request *current_request;
  struct window_main *window_main;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Init */
  wait_delay = 0;

  /*** On entre dans la boucle du traitement ***/
  while(1)
    {
      /* Y a t'il une requte en attente ? */
      has_request = 0;
      EnterCriticalSection(&param->cs_request);
      my_Memory(MEMORY_EXTRACT_REQUEST,&current_request,NULL);
      LeaveCriticalSection(&param->cs_request);
      if(current_request != NULL)
        {
          has_request = 1;

          /* Une demande d'arrt vient d'arriver */
          if(param->stop_request_thread == 1)
            break;

          /** Traitement de la Requte **/
          ProcessOneRequest(current_request);

          /* Libration mmoire de la Requte */
          mem_free_request(current_request);

          /* Passe l'Event en tat Non Signal */
          ResetEvent(param->thread_request_event);

          /* On vient de traiter une request, on vrifie qu'il n'y en a pas une juste derrire (sans passer par le Wait) */
          wait_delay = 0;
        }

      /** On va dtecter un crash de Kegs **/
      if(param->kegs_connection_status == CONNECTION_STATUS_OPEN)
        {
          /* Est-ce que le process de KEGS est en mmoire ? */
          process_handle = OpenProcess(PROCESS_QUERY_INFORMATION,FALSE,param->attach_pid);
          if(process_handle != NULL)
            {
              GetExitCodeProcess(process_handle,&exit_code);
              CloseHandle(process_handle);
            }
          else
            exit_code = 1;

          /* On se dconnecte de KEGS */
          if(exit_code != STILL_ACTIVE)
            {
              my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
              if(window_main != NULL)
                MenuFileDetachKegs(window_main);
            }
        }

      /* Une demande d'arrt vient d'arriver */
      if(param->stop_request_thread == 1)
        break;

      /* On va faire varier le temps d'attente 1-200 ms en fonction de la proximit de la dernire request */
      wait_delay = (wait_delay >= 200) ? 200 : (wait_delay+1);

      /** On s'endort en attendant soit un changement de l'Event soit un dlai de 1-200 ms **/
      if(wait_delay > 1)
        {
          result_wait = WaitForSingleObject(param->thread_request_event,wait_delay);
          if(result_wait == WAIT_OBJECT_0)
            ResetEvent(param->thread_request_event);    /* R-arme tout de suite l'Event pour tre re-sollicit */
        }
    }

  /* Fin du Thread */
  ExitThread(0);

  /* OK */
  return(0);
}


/*************************************************************************/
/*  ProcessOneRequest() :  Traitement d'une Requte par un Thread ddi. */
/*************************************************************************/
static void ProcessOneRequest(struct request *current_request)
{
  BOOL button_operation_status;
  DWORD begin_goid_0, begin_goid_1, end_goid_0, end_goid_1;
  int i, nb_operation, nb_display, has_error, length, end_offset;
  struct operation new_operation;
  char buffer_error[1024];
  char buffer_log[1024];
  char buffer_time[256];
  struct window_main *window_main;
  struct snapshot *current_snapshot;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Init */
  has_error = 0;

  /* Log */
  sprintf(buffer_log,"%s o Process Request Begin [%d]\n",GetCurrentTimeMS(&buffer_time[0]),current_request->request_data_size);
  memcpy(&begin_goid_0,current_request->request_data+OPERATION_DATA_OFFSET_GLOBALID_Q,sizeof(DWORD));
  memcpy(&begin_goid_1,current_request->request_data+OPERATION_DATA_OFFSET_GLOBALID_Q+sizeof(DWORD),sizeof(DWORD));
  end_offset = current_request->request_data_size - OPERATION_DATA_SIZE;
  memcpy(&end_goid_0,current_request->request_data+end_offset+OPERATION_DATA_OFFSET_GLOBALID_Q,sizeof(DWORD));
  memcpy(&end_goid_1,current_request->request_data+end_offset+OPERATION_DATA_OFFSET_GLOBALID_Q+sizeof(DWORD),sizeof(DWORD));

  /* Rcupre la fentre Main */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);

  /** IPC_REQUEST_TYPE_K2C_SEND_SNAPSHOT : Kegs vient d'envoyer les donnes d'un Snapshot **/
  if(current_request->request_type == IPC_REQUEST_TYPE_K2C_SEND_SNAPSHOT)
    {
      /* On est connect  Kegs */
      param->kegs_connection_status = CONNECTION_STATUS_OPEN;

      /** Rcupre les donnes au format Snapshot GS **/
      current_snapshot = BuildSnapshotFromKegs(current_request->request_data_size,current_request->request_data,buffer_error);
      if(current_snapshot)
        {
          /** On va initialiser/MAJ la mmoire avec les donnes du Snapshot **/
          Initialize65816Engine(current_snapshot);

          /* Libration du snapshot */
          mem_free_snapshot(current_snapshot);

          /** Rcupre les valeurs provenant de KEGS + MAJ de Param avec la prochaine Opration **/
          BuildOperationData(current_request,0,&new_operation);

          /** Dtermine la condition de l'arrt **/
          window_main->stop_condition = current_request->request_data[SNAPSHOT_DATA_OFFSET_STOP+0];
          if(window_main->stop_condition == STOP_CONDITION_BREAKPOINT)
            memcpy(&window_main->stop_breakpoint_id,&current_request->request_data[SNAPSHOT_DATA_OFFSET_STOP+1],sizeof(DWORD));

          /** Stocke l'operation (si demand) **/
          EnterCriticalSection(&param->cs_operation);
          if(window_main->operation_history_checkbox == 1)
            my_Memory(MEMORY_ADD_OPERATION,&new_operation,&has_error);
          else
            my_Memory(MEMORY_ADD_EXTRA_OPERATION,&new_operation,&has_error);
          LeaveCriticalSection(&param->cs_operation);

          /** On va mettre  jours les @ des Symbols **/
          UpdateSymbolAddress();

          /** On va mettre  jour les @ des Symbols dans les Breakpoint **/
          UpdateBreakpointSymbolAddress();

          /** On va mettre  jkour les @ des Symbols dans les QuickWtach **/
          UpdateQuickWatchSymbolAddress();

          /* On active/dsactive les boutons */
          my_Memory(MEMORY_GET_OPERATION_NB,&nb_operation,NULL);
          window_main->button_operation_status = (nb_operation == 0) ? FALSE : TRUE;
          EnableWindow(window_main->hwnd_button_operation_reset_operation,window_main->button_operation_status);
          EnableWindow(window_main->hwnd_button_operation_reset_cycle,window_main->button_operation_status);
          EnableWindow(window_main->hwnd_button_operation_save,window_main->button_operation_status);
          EnableWindow(window_main->hwnd_button_operation_clear,window_main->button_operation_status);

          /* Activation de tous les menus */
          UpdateWindowMainMenuItemStatus();

          /* MAJ de la Status Bar */
          UpdateWindowMainStatusBar();
        }
      else
        has_error = 1;

      /* On va demander  KEGS d'attendre */
      SendDataToKegs(IPC_REQUEST_TYPE_C2K_PAUSE,0,NULL);

      /** On demande la MAJ de la fentre Main et on positionne la scrollbar en bas **/
      my_Memory(MEMORY_GET_OPERATION_NB,&nb_operation,NULL);
      nb_display = (nb_operation >= window_main->operation_text_data_row) ? window_main->operation_text_data_row : nb_operation;     /* Nombre de lignes  afficher */
      window_main->operation_scroll_position = nb_operation - nb_display;

      /* Cyrene va attendre l'action de l'oprateur */
      window_main->code_status = CYRENE_STATUS_STOPPED_ON_OPERATION;

      /** On demande la MAJ de la fentre Main : Operation History + Code + CallStack + Output **/
      SendMessage(window_main->hwnd_main,HM_OPERATION_CODE_REFRESHWINDOW,0,0);

      /* On demande la MAJ de toutes les fentres VIEW */
      my_Notification(NOTIFICATION_BROADCAST,NULL,NULL);

      /* On va afficher une MsgBox Error */
      if(has_error == 1)
        {
          /* On se dconecte de Kegs */
          MainMenuToolbarCommand(window_main->hwnd_main,ID_MENU_FILE_DETACH_KEGS,NULL,0);

          /* On affiche la MsgBox d'erreur */
          MessageBox(window_main->hwnd_main,"Error, Impossible to store Operation Data !","Memory Allocation Error",MB_OK|MB_ICONWARNING);
        }
    }
  /** IPC_REQUEST_TYPE_K2C_SEND_OPCODE  : Kegs vient d'envoyer les informations d'une Opration **/
  else if(current_request->request_type == IPC_REQUEST_TYPE_K2C_SEND_OPERATION)
    {
     /* Log */
      sprintf(&buffer_log[strlen(buffer_log)],"%s   - Build Operation Data + ADD_OPERATION\n",GetCurrentTimeMS(&buffer_time[0]));

      /** Rcupre les Oprations Data provenant de KEGS + MAJ de la liste des Oprations ct Cyrene **/
      for(i=0; i<current_request->request_data_size; i += OPERATION_DATA_SIZE)
        {
          /* Prpare les donnes d'une Opration */
          BuildOperationData(current_request,i,&new_operation);

          /** Stocke l'operation (si demand) **/
          EnterCriticalSection(&param->cs_operation);
          if(window_main->operation_history_checkbox == 1)
            my_Memory(MEMORY_ADD_OPERATION,&new_operation,&has_error);
          else
            my_Memory(MEMORY_ADD_EXTRA_OPERATION,&new_operation,&has_error);
          LeaveCriticalSection(&param->cs_operation);
          if(has_error == 1)
            break;
        }

      /* Log */
      sprintf(&buffer_log[strlen(buffer_log)],"%s   - Update Operation History\n",GetCurrentTimeMS(&buffer_time[0]));

      /** On demande la MAJ de la fentre Main et on positionne la scrollbar en bas (5 fois / sec) **/
      if(GetCurrentTimeDiff(window_main->latest_refresh_operation_td) > 200)
        {
          my_Memory(MEMORY_GET_OPERATION_NB,&nb_operation,NULL);
          nb_display = (nb_operation >= window_main->operation_text_data_row) ? window_main->operation_text_data_row : nb_operation;     /* Nombre de lignes  afficher */
          window_main->operation_scroll_position = nb_operation - nb_display;

          /* Demande la MAJ de la fentre Main : Operation History */
          SendMessage(window_main->hwnd_main,HM_OPERATION_REFRESHWINDOW,0,0);

          /* Date de la dernire MAJ */
          window_main->latest_refresh_operation_td = GetCurrentTimeDwd();
        }

      /* On active/dsactive les boutons */
      button_operation_status = (nb_operation == 0) ? FALSE : TRUE;
      if(window_main->button_operation_status != button_operation_status)
        {
          window_main->button_operation_status = button_operation_status;
          EnableWindow(window_main->hwnd_button_operation_reset_operation,button_operation_status);
          EnableWindow(window_main->hwnd_button_operation_reset_cycle,button_operation_status);
          EnableWindow(window_main->hwnd_button_operation_save,button_operation_status);
          EnableWindow(window_main->hwnd_button_operation_clear,button_operation_status);
        }

      /* Log */
      sprintf(&buffer_log[strlen(buffer_log)],"%s   - Notification Broadcast\n",GetCurrentTimeMS(&buffer_time[0]));

      /* On demande la MAJ de toutes les fentres VIEW */
      my_Notification(NOTIFICATION_BROADCAST,NULL,NULL);

      /* On va afficher une MsgBox Error */
      if(has_error == 1)
        {
          /* On se dconecte de Kegs */
          MainMenuToolbarCommand(window_main->hwnd_main,ID_MENU_FILE_DETACH_KEGS,NULL,0);

          /* On affiche la MsgBox d'erreur */
          MessageBox(window_main->hwnd_main,"Error, Impossible to store Operation Data !","Memory Allocation Error",MB_OK|MB_ICONWARNING);
        }
    }

  /* Log */
  sprintf(&buffer_log[strlen(buffer_log)],"%s o Process Request End   [%08X:%08X]-[%08X:%08X]\n",GetCurrentTimeMS(&buffer_time[0]),begin_goid_1,begin_goid_0,end_goid_1,end_goid_0);
  length = (int) strlen(buffer_log);
  if(param->buffer_request_offset < 64000)
    {
      memcpy(&param->buffer_request_log[param->buffer_request_offset],buffer_log,length);
      param->buffer_request_offset += length;
    }
}


/*****************************************************************************************************************/
/*  BuildOperationData() :  Construit les donnes d'une Opration  partir des donnes envoyes par l'mulateur. */
/*****************************************************************************************************************/
static void BuildOperationData(struct request *current_request, int offset_start, struct operation *current_operation)
{
  int is_interrupt, length, bank_wrap;
  struct window_main *window_main;
  struct mmu *current_mmu;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Rcupre le MMU */
  my_Memory(MEMORY_GET_S65816_MMU,&current_mmu,NULL);

  /* Rcupre la fentre Main */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);

  /** Prparartion des Data de l'opration **/
  memset(current_operation,0,sizeof(struct operation));
  memcpy(&current_operation->register_A,&current_request->request_data[offset_start+OPERATION_DATA_OFFSET_A_W],sizeof(WORD));
  memcpy(&current_operation->register_X,&current_request->request_data[offset_start+OPERATION_DATA_OFFSET_X_W],sizeof(WORD));
  memcpy(&current_operation->register_Y,&current_request->request_data[offset_start+OPERATION_DATA_OFFSET_Y_W],sizeof(WORD));
  memcpy(&current_operation->register_PC,&current_request->request_data[offset_start+OPERATION_DATA_OFFSET_PC_W],sizeof(WORD));              /* Program Counter */
  current_operation->register_PB = current_request->request_data[offset_start+OPERATION_DATA_OFFSET_PBR_B];                                  /* Program Bank */
  current_operation->register_DB = current_request->request_data[offset_start+OPERATION_DATA_OFFSET_DBR_B];                                  /* Data Bank */
  memcpy(&current_operation->register_STACK,&current_request->request_data[offset_start+OPERATION_DATA_OFFSET_STACK_W],sizeof(WORD));        /* Stack */
  memcpy(&current_operation->register_DIRECT,&current_request->request_data[offset_start+OPERATION_DATA_OFFSET_DIRECT_W],sizeof(WORD));      /* Direct Page */
  memcpy(&current_operation->register_STATUS,&current_request->request_data[offset_start+OPERATION_DATA_OFFSET_STATUS_W],sizeof(WORD));      /* Status Regsiter + MX */
  current_operation->m_opcode     = current_request->request_data[offset_start+OPERATION_DATA_OFFSET_OPCODE_B];                                /* Opcode */
  current_operation->m_operand1   = current_request->request_data[offset_start+OPERATION_DATA_OFFSET_OPERAND1_B];                              /* Operand #1 */
  current_operation->m_operand2   = current_request->request_data[offset_start+OPERATION_DATA_OFFSET_OPERAND2_B];                              /* Operand #2 */
  current_operation->m_operand3   = current_request->request_data[offset_start+OPERATION_DATA_OFFSET_OPERAND3_B];                              /* Operand #3 */
  current_operation->data_size    = (current_request->request_data[offset_start+OPERATION_DATA_OFFSET_DATASIZE_CYCLE_B] >> 4) & 0x0F;           /* Data Size */
  current_operation->nb_cycle     = current_request->request_data[offset_start+OPERATION_DATA_OFFSET_DATASIZE_CYCLE_B] & 0x0F;                   /* # Cycle */
  current_operation->read_address = (current_request->request_data[offset_start+OPERATION_DATA_OFFSET_READ_ADDRESSB_B] << 16) |              /* Read Address (0xFFFFFF si inutilis) */
                                    (current_request->request_data[offset_start+OPERATION_DATA_OFFSET_READ_ADDRESSH_B] <<  8) |
                                     current_request->request_data[offset_start+OPERATION_DATA_OFFSET_READ_ADDRESSL_B];
  current_operation->write_address = (current_request->request_data[offset_start+OPERATION_DATA_OFFSET_WRITE_ADDRESSB_B] << 16) |            /* Write Address (0xFFFFFF si inutilis) */
                                     (current_request->request_data[offset_start+OPERATION_DATA_OFFSET_WRITE_ADDRESSH_B] <<  8) |
                                      current_request->request_data[offset_start+OPERATION_DATA_OFFSET_WRITE_ADDRESSL_B];
  current_operation->jump_address = (current_request->request_data[offset_start+OPERATION_DATA_OFFSET_JUMP_ADDRESSB_B] << 16) |              /* Jump Address (0xFFFFFF si inutilis) */
                                    (current_request->request_data[offset_start+OPERATION_DATA_OFFSET_JUMP_ADDRESSH_B] <<  8) |
                                     current_request->request_data[offset_start+OPERATION_DATA_OFFSET_JUMP_ADDRESSL_B];
  memcpy(&current_operation->read_value,&current_request->request_data[offset_start+OPERATION_DATA_OFFSET_READ_VALUE_W],sizeof(WORD));       /* Read Value */
  memcpy(&current_operation->write_value,&current_request->request_data[offset_start+OPERATION_DATA_OFFSET_WRITE_VALUE_W],sizeof(WORD));     /* Write Value */
  current_operation->flags = current_request->request_data[offset_start+OPERATION_DATA_OFFSET_FLAGS_B];                                      /* Flags */
  memcpy(&current_operation->call_number,&current_request->request_data[offset_start+OPERATION_DATA_OFFSET_CALL_NUMBER_W],sizeof(WORD));     /* Call Number */
  current_operation->h_rom  = current_request->request_data[offset_start+OPERATION_DATA_OFFSET_ROM_B];                                       /* ROM Version : 01 / 03 */
  current_operation->h_ram  = current_request->request_data[offset_start+OPERATION_DATA_OFFSET_RAM_B];                                       /* # Ram Bank (4-) */
  current_operation->s_C02E = current_request->request_data[offset_start+OPERATION_DATA_OFFSET_C02E_B];                                      /* Softswitch $C02E VertCnt */
  current_operation->s_C02F = current_request->request_data[offset_start+OPERATION_DATA_OFFSET_C02F_B];                                      /* Softswitch $C02F HorizCnt */
  current_operation->s_C035 = current_request->request_data[offset_start+OPERATION_DATA_OFFSET_C035_B];                                      /* Softswitch $C035 Shadowing */
  current_operation->s_C068 = current_request->request_data[offset_start+OPERATION_DATA_OFFSET_C068_B];                                      /* Softswitch $C068 StateReg */
  memcpy(&current_operation->goid,&current_request->request_data[offset_start+OPERATION_DATA_OFFSET_GLOBALID_Q],sizeof(QWORD));              /* Global Operation ID */
  memcpy(&current_operation->gcc,&current_request->request_data[offset_start+OPERATION_DATA_OFFSET_GLOBALCYCLE_Q],sizeof(QWORD));            /* Global Cycle Count */

  /*** Calcule les Next Operation Address ***/
  /** Adresse de la prochaine Opration excute **/
  if(current_operation->jump_address != 0x00FFFFFF)
    current_operation->next_op_addr = current_operation->jump_address;
  else
    {
      /* On doit grer les MVN / MVP spcifiquement */
      if((current_operation->m_opcode == 0x54 || current_operation->m_opcode == 0x44) && current_operation->register_A != 0x0000)
        current_operation->next_op_addr = (current_operation->register_PB << 16) | current_operation->register_PC;     /* Reste sur la ligne du MVN/MVP */
      else
        current_operation->next_op_addr = (current_operation->register_PB << 16) | ((current_operation->register_PC + 1 + current_operation->data_size) & 0x0000FFFF);
    }
  /** Adresse de la prochaine Opration de cette fonction **/
  if(current_operation->jump_address != 0x00FFFFFF && current_operation->m_opcode != 0x20 && current_operation->m_opcode != 0xFC && current_operation->m_opcode != 0x22)   /* JSR / JSL */
    current_operation->sub_next_op_addr = current_operation->jump_address;                       /* JMP BRA BRL BEQ BNE BCC BCS... */
  else
    {
      /* GS/OS : JSL $E100A8 + DW + ADRL [INLINE PARAM] */
      if(current_operation->m_opcode == 0x22 && current_operation->jump_address == 0x00E100A8)
        current_operation->sub_next_op_addr = (current_operation->register_PB << 16) | ((current_operation->register_PC + 1 + current_operation->data_size + 2 + 4) & 0x0000FFFF);
      /* Prodos 8 : JSR $BF00 + DB + ADR [INLINE PARAM] */
      else if((current_operation->m_opcode == 0x20 || current_operation->m_opcode == 0xFC) && current_operation->jump_address == 0x0000BF00)
        current_operation->sub_next_op_addr = (current_operation->register_PB << 16) | ((current_operation->register_PC + 1 + current_operation->data_size + 1 + 2) & 0x0000FFFF);
      else
        {
          /* On doit grer les MVN / MVP spcifiquement */
          if((current_operation->m_opcode == 0x54 || current_operation->m_opcode == 0x44) && current_operation->register_A != 0x0000)
            current_operation->sub_next_op_addr = (current_operation->register_PB << 16) | current_operation->register_PC;     /* Reste sur la ligne du MVN/MVP */
          else
            current_operation->sub_next_op_addr = (current_operation->register_PB << 16) | ((current_operation->register_PC + 1 + current_operation->data_size) & 0x0000FFFF);
        }
    }

  /** Calcule les Stat consolides **/
  param->operation_total_cycle += ((param->is_first_operation == 0) ? param->operation_nb_cycle : 0);
  param->operation_id++;
  current_operation->total_cycle  = param->operation_total_cycle;
  current_operation->operation_id = param->operation_id;

  /** Backup l'ancienne 'future' Opration = Dernire Opration excute **/
  if(param->is_first_operation == 0)
    {
      param->prev_operation_opcode           = param->operation_opcode;                      /* Opcode */
      param->prev_operation_operand1         = param->operation_operand1;                    /* Operand #1 */
      param->prev_operation_operand2         = param->operation_operand2;                    /* Operand #2 */
      param->prev_operation_operand3         = param->operation_operand3;                    /* Operand #3 */
      param->prev_operation_PC               = param->operation_PC;                          /* Program Counter */
      param->prev_operation_PB               = param->operation_PB;                          /* Program Bank */
      param->prev_operation_DB               = param->operation_DB;                          /* Data Bank */
      param->prev_operation_STACK            = param->operation_STACK;                       /* Stack */
      param->prev_operation_X                = param->operation_X;                           /* X Register */
      param->prev_operation_data_size        = param->operation_data_size;                   /* Operand Size */
      param->prev_operation_nb_cycle         = param->operation_nb_cycle;                    /* Nb Cycle */
      param->prev_operation_status_m         = param->operation_status_m;                    /* M */
      param->prev_operation_status_x         = param->operation_status_x;                    /* X */
      param->prev_operation_goid             = param->operation_goid;                        /* Global Operation ID */
      param->prev_operation_gcc              = param->operation_gcc;                         /* Global Cycle Count */
      param->prev_operation_C02E             = param->operation_C02E;                        /* Softswitch $C02E VertCnt */
      param->prev_operation_C02F             = param->operation_C02F;                        /* Softswitch $C02F HorizCnt */
      param->prev_operation_C035             = param->operation_C035;                        /* Softswitch $C035 Shadowing */
      param->prev_operation_C068             = param->operation_C068;                        /* Softswitch $C068 StateReg */
      param->prev_operation_read_address     = param->operation_read_address;                /* Read Address */
      param->prev_operation_write_address    = param->operation_write_address;               /* Write Address */
      param->prev_operation_jump_address     = param->operation_jump_address;                /* Jump Address */
      param->prev_operation_read_value       = param->operation_read_value;                  /* Read Value */
      param->prev_operation_write_value      = param->operation_write_value;                 /* Write Value */
      param->prev_operation_flags            = param->operation_flags;                       /* Flags (Interrupt / Size / Wrap) */
      param->prev_operation_call_number      = param->operation_call_number;                 /* Call Number */
      param->prev_operation_next_op_addr     = param->operation_next_op_addr;                /* Next operation address */
      param->prev_operation_sub_next_op_addr = param->operation_sub_next_op_addr;            /* Current Function next operation address */
    }

  /** Conserve les donnes de l'opration en cours **/
  param->operation_opcode           = current_operation->m_opcode;                       /* Opcode */
  param->operation_operand1         = current_operation->m_operand1;                     /* Operand #1 */
  param->operation_operand2         = current_operation->m_operand2;                     /* Operand #2 */
  param->operation_operand3         = current_operation->m_operand3;                     /* Operand #3 */
  param->operation_PC               = current_operation->register_PC;                    /* Program Counter */
  param->operation_PB               = current_operation->register_PB;                    /* Program Bank */
  param->operation_DB               = current_operation->register_DB;                    /* Data Bank */
  param->operation_STACK            = current_operation->register_STACK;                 /* Stack */
  param->operation_X                = current_operation->register_X;                     /* X Register */
  param->operation_data_size        = current_operation->data_size;                      /* Operand Size */
  param->operation_nb_cycle         = current_operation->nb_cycle;                       /* Nombre de Cycle */
  param->operation_status_m         = (current_operation->register_STATUS >> 5) & 0x01;  /* M */
  param->operation_status_x         = (current_operation->register_STATUS >> 4) & 0x01;  /* X */
  param->operation_goid             = current_operation->goid;                           /* Global Operation ID */
  param->operation_gcc              = current_operation->gcc;                            /* Global Cycle Count */
  param->operation_C02E             = current_operation->s_C02E;                         /* Softswitch $C02E VertCnt */
  param->operation_C02F             = current_operation->s_C02F;                         /* Softswitch $C02F HorizCnt */
  param->operation_C035             = current_operation->s_C035;                         /* Softswitch $C035 Shadowing */
  param->operation_C068             = current_operation->s_C068;                         /* Softswitch $C068 StateReg */
  param->operation_read_address     = current_operation->read_address;                   /* Read Address */
  param->operation_write_address    = current_operation->write_address;                  /* Write Address */
  param->operation_jump_address     = current_operation->jump_address;                   /* Jump Address */
  param->operation_read_value       = current_operation->read_value;                     /* Read Value */
  param->operation_write_value      = current_operation->write_value;                    /* Write Value */
  param->operation_flags            = current_operation->flags;                          /* Flags */
  param->operation_call_number      = current_operation->call_number;                    /* Call Number */
  param->operation_next_op_addr     = current_operation->next_op_addr;                   /* Next operation address */
  param->operation_sub_next_op_addr = current_operation->sub_next_op_addr;               /* Current Function next operation address */

  /** On va impacter la MemoryMap pour les RWX **/
  /* Read */
  length    = (current_operation->flags >> 3) & 0x03;
  bank_wrap = (current_operation->flags >> 1) & 0x01;
  UpdateMemoryMapRWX(param->operation_read_address,length,bank_wrap,MEMORYMAP_OPERATION_READ,current_mmu);
  /* Write */
  length    = (current_operation->flags >> 5) & 0x03;
  bank_wrap = (current_operation->flags >> 2) & 0x01;
  UpdateMemoryMapRWX(param->operation_write_address,length,bank_wrap,MEMORYMAP_OPERATION_WRITE,current_mmu);
  /* Execute */
  length    = 1 + param->operation_data_size;
  bank_wrap = 1;
  UpdateMemoryMapRWX((param->operation_PB<<16)|param->operation_PC,length,bank_wrap,MEMORYMAP_OPERATION_EXECUTE,current_mmu);

  /** On va dtecter l'entre dans une interruption **/
  is_interrupt = ((current_operation->flags & OPERATION_FLAG_INTERRUPT) == OPERATION_FLAG_INTERRUPT) ? 1 : 0;

  /** On met  jour la CallStack des fonctions **/
  ProcessCallStackOperation(window_main,is_interrupt);

  /** On met  jour la ListBox Output avec les nouvelles entres (5 fois / sec) **/
  if(GetCurrentTimeDiff(window_main->latest_refresh_output_td) > 200)
    {
      RefreshOutputListBox(window_main);
      window_main->latest_refresh_output_td = GetCurrentTimeDwd();
    }

  /** On met  jour les Statistiques des fonctions **/
  ProcessStatisticOperation(window_main,is_interrupt);

  /** On met  jour le Profile des fonctions **/
  ProcessProfileOperation(window_main,is_interrupt);

  /* On vient de stocker la premoire opration */
  param->is_first_operation = 0;
}


/***************************************************************************/
/*  UpdateMemoryMapRWX() :  Met  jour la MemoryMap pour les Adresses RWX. */
/***************************************************************************/
static void UpdateMemoryMapRWX(DWORD first_address, int length, int bank_wrap, unsigned char operation_type, struct mmu *current_mmu)
{
  int i;
  DWORD address; 
  unsigned char *ram_rom_ptr;

  /* Rien  faire */
  if(current_mmu == NULL)
    return;
  if(first_address == 0xFFFFFFFF)
    return;

  /** Addresse : +0 - +3 **/
  for(i=0; i<length; i++)
    {
      address = first_address + i;
      if(bank_wrap == 1 && (first_address & 0x00FF0000) != (address & 0x00FF0000))
        address = (first_address & 0x00FF0000) | (address & 0x0000FFFF);
      ram_rom_ptr = GetMapRamRomRWXPointer(address,0,current_mmu);
      if(ram_rom_ptr != NULL)
        *ram_rom_ptr |= operation_type;
    }
}


/**********************************************************************/
/*  AddRequest() :  Ajoute la Requte  liste des Requtes  traiter. */
/**********************************************************************/
void AddRequest(int request_type, int request_data_size, unsigned char *request_data)
{
  int length, end_offset;
  DWORD begin_goid_0, begin_goid_1, end_goid_0, end_goid_1;
  char buffer_time[256];
  char buffer_log[1024];
  struct request *current_request;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Log */
  sprintf(buffer_log,"%s o Add Request Begin [%d]\n",GetCurrentTimeMS(&buffer_time[0]),request_data_size);
  length = (int) strlen(buffer_log);

  /* Allocation mmoire */
  current_request = (struct request *) calloc(1,sizeof(struct request));
  if(current_request == NULL)
    return;
  current_request->request_type = request_type;
  current_request->request_data_size = request_data_size;
  current_request->request_data = (unsigned char *) calloc(current_request->request_data_size,sizeof(unsigned char));
  if(current_request->request_data == NULL)
    {
      mem_free_request(current_request);
      return;
    }
  memcpy(current_request->request_data,request_data,current_request->request_data_size);

  /** Ajoute la Requte  la liste des requtes  traiter **/
  EnterCriticalSection(&param->cs_request);
  my_Memory(MEMORY_ADD_REQUEST,current_request,NULL);
  LeaveCriticalSection(&param->cs_request);

  /** Signale au Thread Request qu'une Request vient d'tre dpose **/
  SetEvent(param->thread_request_event);

  /* Log */
  memcpy(&begin_goid_0,request_data+OPERATION_DATA_OFFSET_GLOBALID_Q,sizeof(DWORD));
  memcpy(&begin_goid_1,request_data+OPERATION_DATA_OFFSET_GLOBALID_Q+sizeof(DWORD),sizeof(DWORD));
  end_offset = request_data_size - OPERATION_DATA_SIZE;
  memcpy(&end_goid_0,request_data+end_offset+OPERATION_DATA_OFFSET_GLOBALID_Q,sizeof(DWORD));
  memcpy(&end_goid_1,request_data+end_offset+OPERATION_DATA_OFFSET_GLOBALID_Q+sizeof(DWORD),sizeof(DWORD));
  sprintf(&buffer_log[strlen(buffer_log)],"%s o Add Request End   [%08X:%08X]-[%08X:%08X]\n",GetCurrentTimeMS(&buffer_time[0]),begin_goid_1,begin_goid_0,end_goid_1,end_goid_0);
  length = (int) strlen(buffer_log);
  if(param->buffer_ipc_offset < 64000)
    {
      memcpy(&param->buffer_ipc_log[param->buffer_ipc_offset],buffer_log,length);
      param->buffer_ipc_offset += length;
    }
}


/**********************************************************************/
/*  mem_free_request() :  Libration mmoire de la structure Request. */
/**********************************************************************/
void mem_free_request(struct request *current_request)
{
  if(current_request)
    {
      if(current_request->request_data)
        free(current_request->request_data);

      free(current_request);
    }
}


/**************************************************************************************/
/*  WriteDataToEmulatorMemory() :  Ecrit des donnes dans la mmmoire de l'mulateur. */
/**************************************************************************************/
void WriteDataToEmulatorMemory(int ram_type, DWORD address, int data_length, unsigned char *data)
{
  DWORD address_kegs;
  int data_kegs_size;
  unsigned char *data_kegs;

  /* Allocation mmoire */
  data_kegs_size = 4 + data_length;
  data_kegs = (unsigned char *) calloc(data_kegs_size,sizeof(unsigned char));
  if(data_kegs == NULL)
    return;

  /* Cration du buffer : Adress:Data */
  address_kegs = (address & 0x00FFFFFF) | ((ram_type == RAMTYPE_RAM ) ? 0x00000000 : 0x80000000);
  memcpy(data_kegs+0,&address_kegs,sizeof(DWORD));
  memcpy(data_kegs+4,data,data_length);

  /** On transmet  KEGS les donnes  copier en mmoire **/
  SendDataToKegs(IPC_REQUEST_TYPE_C2K_WRITE_DATA,data_kegs_size,data_kegs);

  /* Libration mmoire */
  free(data_kegs);
}


/*******************************************************************/
/*  SendDataToKegs() :  Envoie une requte  Kegs pour traitement. */
/*******************************************************************/
int SendDataToKegs(int request_type, int request_data_size, unsigned char *request_operation_data)
{
  LRESULT result;
  COPYDATASTRUCT copy_data;
  struct window_ipc *window_ipc;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Rcupre les fentres */
  my_Memory(MEMORY_GET_WINDOW_IPC,&window_ipc,NULL);

  /** Fill the COPYDATA structure **/
  memset(&copy_data,0,sizeof(COPYDATASTRUCT));
  copy_data.dwData = (DWORD) request_type;                     /* Function identifier = IPC_REQUEST_TYPE_XXX */
  copy_data.cbData = (DWORD) request_data_size;                /* Size of data */
  copy_data.lpData = (unsigned char *) request_operation_data; /* Operation Data */

  /** Envoi la Requte  Kegs via un message **/
  result = SendMessage(param->hwnd_kegs_ipc,WM_COPYDATA,(WPARAM)window_ipc->hwnd_cyrene_ipc,(LPARAM)(LPVOID)&copy_data);

  /* Process SendMessage Errors */
  if(result == FALSE)
    return(1);

  /* OK */
  return(0);
}

/*************************************************************************/
