/************************************************************************/
/*                                                                      */
/*  Dc_Snapshot.c :  Module pour les fonction de gestion des Snapshot.  */
/*                                                                      */
/************************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe Software  *  July 2023   */
/************************************************************************/

#include <stdio.h>
#include <time.h>

#include <windows.h>

#include "Dc_Shared.h"
#include "Dc_MMU.h"
#include "Dc_CPU.h"
#include "Dc_Memory.h"
#include "Dc_Request.h"
#include "Cyrene_Engine.h"
#include "Window_QuickWatch.h"
#include "Window_Main.h"
#include "Dc_Snapshot.h"

/********************************************************************************************/
/*  BuildSnapshotFromKegs() :  Construit les donnes du Snapshot  partir des donnes KEGS. */
/********************************************************************************************/
struct snapshot *BuildSnapshotFromKegs(int data_length, unsigned char *data, char *buffer_error_rtn)
{
  WORD one_word;
  int rom_type, ext_bank_nb, expected_data_length, ext_ram_size;
  struct snapshot *current_snapshot;

  /* Init */
  strcpy(buffer_error_rtn,"");
  
  /** Les donnes sont celles d'un Snapshot GS : 1KB Header (Hard + Proc + DOC Reg=224+3b + BRAM=256b) + 4*64KB (RAM:00+01+E0+E1) + nb_extra_ram*64KB (Ext RAM) + 2|4*64KB (ROM) + 1*64KB (DOC RAM) **/
  if(data_length < SNAPSHOT_HEADER_SIZE)
    {
      strcpy(buffer_error_rtn,"Invalid Kegs Snapshot data Header size");
      return(NULL);
    }

  /** Rom Type (01 ou 03) **/
  if(data[OPERATION_DATA_OFFSET_ROM_B] == 0x01)
    rom_type = SNAPSHOT_APPLEIIGS_ROM01;
  else if (data[OPERATION_DATA_OFFSET_ROM_B] == 0x03)
    rom_type = SNAPSHOT_APPLEIIGS_ROM03;
  else
    {
      strcpy(buffer_error_rtn,"Invalid Kegs Snapshot data : Unknown Rom Version");
      return(NULL);
    }

  /** Nb de Bank de mmoire tendue (en plus des 256 KB de base) **/
  ext_ram_size = data_length - (SNAPSHOT_HEADER_SIZE + 4*RAM_BANK_SIZE + ((rom_type == SNAPSHOT_APPLEIIGS_ROM01) ? 2*RAM_BANK_SIZE : 4*RAM_BANK_SIZE) + DOC_RAM_SIZE);
  ext_bank_nb = ext_ram_size/RAM_BANK_SIZE;

  /* Vrifie la taille des donnes envoyes par KEGS */
  expected_data_length = SNAPSHOT_HEADER_SIZE + 4*64*1024 + ext_bank_nb*64*1024 + ((rom_type == SNAPSHOT_APPLEIIGS_ROM01) ? 2*64*1024 : 4*64*1024) + 1*64*1024;
  if(expected_data_length != data_length)
    {
      strcpy(buffer_error_rtn,"Invalid Kegs Snapshot data size");
      return(NULL);
    }

  /* Allocation mmoire */
  current_snapshot = (struct snapshot *) calloc(1,sizeof(struct snapshot));
  if(current_snapshot == NULL)
    {
      strcpy(buffer_error_rtn,"Impossible to allocate memory to process Kegs Snapshot data");
      return(NULL);
    }

  /** HEADER **/
  strcpy((char *)current_snapshot->snapshot_header,"GSm");
  memcpy(&current_snapshot->snapshot_header[SNAPSHOT_FILE_HEADER_OFFSET_A_W],&data[OPERATION_DATA_OFFSET_A_W],2);            /* A */
  memcpy(&current_snapshot->snapshot_header[SNAPSHOT_FILE_HEADER_OFFSET_X_W],&data[OPERATION_DATA_OFFSET_X_W],2);            /* X */
  memcpy(&current_snapshot->snapshot_header[SNAPSHOT_FILE_HEADER_OFFSET_Y_W],&data[OPERATION_DATA_OFFSET_Y_W],2);            /* Y */
  memcpy(&current_snapshot->snapshot_header[SNAPSHOT_FILE_HEADER_OFFSET_PC_W],&data[OPERATION_DATA_OFFSET_PC_W],2);          /* PC */
  memcpy(&current_snapshot->snapshot_header[SNAPSHOT_FILE_HEADER_OFFSET_PBR_B],&data[OPERATION_DATA_OFFSET_PBR_B],1);        /* PBR */
  memcpy(&current_snapshot->snapshot_header[SNAPSHOT_FILE_HEADER_OFFSET_DBR_B],&data[OPERATION_DATA_OFFSET_DBR_B],1);        /* DBR */
  memcpy(&current_snapshot->snapshot_header[SNAPSHOT_FILE_HEADER_OFFSET_STACK_W],&data[OPERATION_DATA_OFFSET_STACK_W],2);    /* STACK */
  memcpy(&current_snapshot->snapshot_header[SNAPSHOT_FILE_HEADER_OFFSET_DIRECT_W],&data[OPERATION_DATA_OFFSET_DIRECT_W],2);  /* DIRECT */
  memcpy(&current_snapshot->snapshot_header[SNAPSHOT_FILE_HEADER_OFFSET_STATUS_W],&data[OPERATION_DATA_OFFSET_STATUS_W],2);  /* STATUS */
  one_word = data[OPERATION_DATA_OFFSET_ROM_B];
  memcpy(&current_snapshot->snapshot_header[SNAPSHOT_FILE_HEADER_OFFSET_ROM_W],&one_word,2);                                 /* ROM Version */
  memcpy(&current_snapshot->snapshot_header[DOC_REGISTER_OFFSET],&data[DOC_REGISTER_OFFSET],DOC_REGISTER_SIZE);              /* DOC Registers */
  memcpy(&current_snapshot->snapshot_header[BATTERY_RAM_OFFSET],&data[BATTERY_RAM_OFFSET],BATTERY_RAM_SIZE);                 /* Battery RAM */

  /** Remplissage des membres de la structure **/
  current_snapshot->snapshot_type = SNAPSHOT_TYPE_GS;
  /* Processor */
  memcpy(&current_snapshot->proc_a,&data[OPERATION_DATA_OFFSET_A_W],2);
  memcpy(&current_snapshot->proc_x,&data[OPERATION_DATA_OFFSET_X_W],2);
  memcpy(&current_snapshot->proc_y,&data[OPERATION_DATA_OFFSET_Y_W],2);
  memcpy(&current_snapshot->proc_pbr,&data[OPERATION_DATA_OFFSET_PBR_B],1);
  memcpy(&current_snapshot->proc_dbr,&data[OPERATION_DATA_OFFSET_DBR_B],1);
  memcpy(&current_snapshot->proc_pc,&data[OPERATION_DATA_OFFSET_PC_W],2);
  memcpy(&current_snapshot->proc_stack,&data[OPERATION_DATA_OFFSET_STACK_W],2);
  memcpy(&current_snapshot->proc_directpage,&data[OPERATION_DATA_OFFSET_DIRECT_W],2);
  /* Status Register */
  memcpy(&one_word,&data[OPERATION_DATA_OFFSET_STATUS_W],2);
  current_snapshot->status_c = (one_word >> 0) & 0x01;     /* C  Carry */
  current_snapshot->status_z = (one_word >> 1) & 0x01;     /* Z  Zero */
  current_snapshot->status_i = (one_word >> 2) & 0x01;     /* I  Interrupt */
  current_snapshot->status_d = (one_word >> 3) & 0x01;     /* D  Decimal */
  current_snapshot->status_x = (one_word >> 4) & 0x01;     /* X  X/Y Register Size */
  current_snapshot->status_m = (one_word >> 5) & 0x01;     /* M  Accumulator Size */
  current_snapshot->status_v = (one_word >> 6) & 0x01;     /* V  Overflow */
  current_snapshot->status_n = (one_word >> 7) & 0x01;     /* N  Negative */
  current_snapshot->status_e = (one_word >> 8) & 0x01;     /* E  6502 Emulation */
  current_snapshot->status_b = (one_word >> 9) & 0x01;     /* B */
  /* RAM Size */
  current_snapshot->ram_size = ext_bank_nb;                /* Nombre de bank mmoire en plus des 256 KB de base */
  /* ROM Type */
  current_snapshot->rom_type = data[OPERATION_DATA_OFFSET_ROM_B];  /* ROM01 | ROM03 */
  /* DOC Registers : 224+3 bytes */
  memcpy(&current_snapshot->doc_register,&data[DOC_REGISTER_OFFSET],DOC_REGISTER_SIZE);
  /* Battery RAM : 256 bytes */
  memcpy(&current_snapshot->battery_ram,&data[BATTERY_RAM_OFFSET],BATTERY_RAM_SIZE);

  /** Allocation mmoire **/
  current_snapshot->snapshot_length = data_length - SNAPSHOT_HEADER_SIZE;
  current_snapshot->snapshot_data = (unsigned char *) calloc(current_snapshot->snapshot_length,sizeof(unsigned char));
  if(current_snapshot->snapshot_data == NULL)
    {
      strcpy(buffer_error_rtn,"Impossible to allocate memory to process Kegs Snapshot data");
      free(current_snapshot);
      return(NULL);
    }
  memcpy(current_snapshot->snapshot_data,&data[SNAPSHOT_HEADER_SIZE],current_snapshot->snapshot_length);
  
  /* Renvoi le Snapshot */
  return(current_snapshot);
}


/******************************************************************************/
/*  SaveSnapshotFile() :  Cration d'un fichier avec les donnes du Snapshot. */
/******************************************************************************/
int SaveSnapshotFile(int snapshot_type)
{
  BOOL is_OK;
  WORD one_word;
  FILE *fd;
  time_t clock;
  struct tm *time_date;
  int nb_write, offset, nb_data_bank;
  struct s65816 *current_65816;
  struct window_main *window_main;
  char file_path[BUFFER_SIZE];
  unsigned char header[SNAPSHOT_HEADER_SIZE];

  /* Rcupre la fentre Main */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);

  /* Rcupre les donnes de simulation */
  my_Memory(MEMORY_GET_S65816,&current_65816,NULL);

  /** Cration d'un nom de fichier **/
  /* Now */
  time(&clock);
  time_date = localtime(&clock);
  sprintf(file_path,"Snapshot_%4d%02d%02d_%02d%02d%02d",
          1900+time_date->tm_year,time_date->tm_mon+1,time_date->tm_mday,
          time_date->tm_hour,time_date->tm_min,time_date->tm_sec);

  /** Lancer le file dialog pour rcuprer le chemin  sauvegarder **/
  if(snapshot_type == SNAPSHOT_TYPE_II)
    {
      strcat(file_path,".IIm");
      is_OK = GetSaveFilePath(window_main->hwnd_main,"Select Snapshot file","All Files (*.*)\0*.*\0Apple II Snapshot (*.IIm)\0*.IIm\0\0",file_path);
    }
  else
    {
      strcat(file_path,".GSm");
      is_OK = GetSaveFilePath(window_main->hwnd_main,"Select Snapshot file","All Files (*.*)\0*.*\0Apple IIgs Snapshot (*.GSm)\0*.GSm\0\0",file_path);
    }

  /* Si l'utilisateur l'annule */
  if(!is_OK)
    return(0);

  /** Sauvegarde les donnes du Snapshot **/
  /* Cration du fichier */
  fd = fopen(file_path,"wb");
  if(fd == NULL)
    return(1);

  /** Cration du Header **/
  memset(&header[0],0x00,SNAPSHOT_HEADER_SIZE);
  offset = 0;
  memcpy(&header[offset],(snapshot_type == SNAPSHOT_TYPE_II) ? "IIm" : "GSm",4);
  offset += 4;
  memcpy(&header[offset],&current_65816->current_cpu->proc_a,2);           /* A */
  offset += 2;
  memcpy(&header[offset],&current_65816->current_cpu->proc_x,2);           /* X */
  offset += 2;
  memcpy(&header[offset],&current_65816->current_cpu->proc_y,2);           /* Y */
  offset += 2;
  memcpy(&header[offset],&current_65816->current_cpu->proc_pc,2);          /* PC */
  offset += 2;
  memcpy(&header[offset],&current_65816->current_cpu->proc_dbr,1);         /* DBR */
  offset++;
  memcpy(&header[offset],&current_65816->current_cpu->proc_pbr,1);         /* PBR */
  offset++;
  memcpy(&header[offset],&current_65816->current_cpu->proc_stack,2);       /* STACK */
  offset += 2;
  memcpy(&header[offset],&current_65816->current_cpu->proc_directpage,2);  /* DIRECT */
  offset += 2;
  one_word = 0x0000;
  one_word |= ((current_65816->current_cpu->status_c & 0x0001) << 0);      /* C */
  one_word |= ((current_65816->current_cpu->status_z & 0x0001) << 1);      /* Z */
  one_word |= ((current_65816->current_cpu->status_i & 0x0001) << 2);      /* I */
  one_word |= ((current_65816->current_cpu->status_d & 0x0001) << 3);      /* D */
  one_word |= ((current_65816->current_cpu->status_x & 0x0001) << 4);      /* X */
  one_word |= ((current_65816->current_cpu->status_m & 0x0001) << 5);      /* M */
  one_word |= ((current_65816->current_cpu->status_v & 0x0001) << 6);      /* V */
  one_word |= ((current_65816->current_cpu->status_n & 0x0001) << 7);      /* N */
  one_word |= ((current_65816->current_cpu->status_e & 0x0001) << 8);      /* E */
  one_word |= ((current_65816->current_cpu->status_b & 0x0001) << 9);      /* B */
  memcpy(&header[offset],&one_word,2);                                     /* STATUS */
  offset += 2;
  one_word = current_65816->current_mmu->rom_version;
  memcpy(&header[offset],&one_word,2);                                     /* ROM Version */
  offset += 2;
  /* DOC Register */
  memcpy(&header[DOC_REGISTER_OFFSET],current_65816->current_mmu->doc_register,DOC_REGISTER_SIZE);
  /* BATTERY RAM */
  memcpy(&header[BATTERY_RAM_OFFSET],current_65816->current_mmu->battery_ram,BATTERY_RAM_SIZE);

  /*** Donnes de la mmoire ***/
  if(snapshot_type == SNAPSHOT_TYPE_II)
    {
      /** Ecriture Data RAM : 128 KB **/
      nb_write = 0;
      nb_write += (int) fwrite(current_65816->current_mmu->ram_00,1,RAM_BANK_SIZE,fd);
      nb_write += (int) fwrite(current_65816->current_mmu->ram_01,1,RAM_BANK_SIZE,fd);
      if(nb_write != 2*RAM_BANK_SIZE)
        {
          fclose(fd);
          unlink(file_path);
          return(1);
        }
    }
  else
    {
      /** Ecriture Data RAM **/
      nb_write = 0;
      nb_write += (int) fwrite(current_65816->current_mmu->ram_00,1,RAM_BANK_SIZE,fd);
      nb_write += (int) fwrite(current_65816->current_mmu->ram_01,1,RAM_BANK_SIZE,fd);
      nb_write += (int) fwrite(current_65816->current_mmu->ram_extended,1,current_65816->current_mmu->nb_extended_bank*RAM_BANK_SIZE,fd);
      nb_write += (int) fwrite(current_65816->current_mmu->ram_e0,1,RAM_BANK_SIZE,fd);
      nb_write += (int) fwrite(current_65816->current_mmu->ram_e1,1,RAM_BANK_SIZE,fd);
      nb_data_bank = 2 + current_65816->current_mmu->nb_extended_bank + 2;

      /** Ecriture ROM **/
      if(current_65816->current_mmu->rom_version == ROM_APPLEIIGS_03)
        {
          nb_write += (int) fwrite(current_65816->current_mmu->rom_fc,1,RAM_BANK_SIZE,fd);
          nb_write += (int) fwrite(current_65816->current_mmu->rom_fd,1,RAM_BANK_SIZE,fd);
          nb_data_bank += 2;
        }
      nb_write += (int) fwrite(current_65816->current_mmu->rom_fe,1,RAM_BANK_SIZE,fd);
      nb_write += (int) fwrite(current_65816->current_mmu->rom_ff,1,RAM_BANK_SIZE,fd);
      nb_data_bank += 2;

      /** Ecriture Doc RAM **/
      nb_write += (int) fwrite(current_65816->current_mmu->ram_sound,1,RAM_BANK_SIZE,fd);
      nb_data_bank++;

      /* Vrification */
      if(nb_write != nb_data_bank*RAM_BANK_SIZE)
        {
          fclose(fd);
          unlink(file_path);
          return(1);
        }
    }

  /** Ecriture Header  la fin du fichier = Footer :-) **/
  nb_write = (int) fwrite(header,1,SNAPSHOT_HEADER_SIZE,fd);

  /* Fermeture du fichier */
  fclose(fd);

  /* Vrification de l'criture */
  if(nb_write != SNAPSHOT_HEADER_SIZE)
    {
      unlink(file_path);
      return(1);
    }

  /* OK */
  return(0);
}


/********************************************************************************/
/*  LoadSnapshotFile() :  Chargement d'un fichier avec les donnes du Snapshot. */
/********************************************************************************/
struct snapshot *LoadSnapshotFile(int snapshot_type)
{
  int nb_path, data_length;
  WORD one_word, rom_version;
  char **file_path_tab;
  char *file_path;
  unsigned char *file_data;
  char error_buffer[1024];
  unsigned char header[SNAPSHOT_HEADER_SIZE];
  struct window_main *window_main;
  struct snapshot *current_snapshot;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* init */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);

  /** Demande le chemin du fichier **/
  file_path_tab = GetFilePath(window_main->hwnd_main,"Select Snapshot file","All Files (*.*)\0*.*\0Apple IIgs Snapshot (*.GSm)\0*.GSm\0\0",&nb_path,0);
  if(file_path_tab == NULL)
    return(NULL);
  if(nb_path == 0)
    {
      mem_free_list(nb_path,file_path_tab);
      return(NULL);
    }
  file_path = file_path_tab[0];

  /** Charge les donnes du fichier **/
  file_data = LoadBinaryFile(file_path,&data_length,&error_buffer[0]);
  if(file_data == NULL)
    {
      mem_free_list(nb_path,file_path_tab);
      return(NULL);
    }

  /** Dcodage du fichier **/
  if(data_length < (5*64*1024 + SNAPSHOT_HEADER_SIZE))
    {
      mem_free_list(nb_path,file_path_tab);
      return(NULL);
    }
  memcpy(&header[0],&file_data[data_length-SNAPSHOT_HEADER_SIZE],SNAPSHOT_HEADER_SIZE);

  /* Apple IIgs Snapshot */
  if(memcmp(&header[0],"GSm",4))
    {
      mem_free_list(nb_path,file_path_tab);
      return(NULL);
    }

  /* Rom Size */
  memcpy(&rom_version,&header[20],sizeof(WORD));

  /* Allocation mmoire */
  current_snapshot = (struct snapshot *) calloc(1,sizeof(struct snapshot));
  if(current_snapshot == NULL)
    {
      mem_free_list(nb_path,file_path_tab);
      return(NULL);
    }
  current_snapshot->snapshot_type = SNAPSHOT_TYPE_GS;

  /* Header */
  memcpy(current_snapshot->snapshot_header,header,SNAPSHOT_HEADER_SIZE);

  /* Nom du fichier */
  strcpy(current_snapshot->snapshot_file_path,file_path);

  /* Snapshot Data */
  current_snapshot->snapshot_length = data_length - SNAPSHOT_HEADER_SIZE;
  current_snapshot->snapshot_data = (unsigned char *) calloc(current_snapshot->snapshot_length,sizeof(unsigned char));
  if(current_snapshot->snapshot_data == NULL)
    {
      free(current_snapshot);
      mem_free_list(nb_path,file_path_tab);
      return(NULL);
    }
  memcpy(current_snapshot->snapshot_data,file_data,current_snapshot->snapshot_length);

  /* Processor */
  memcpy(&current_snapshot->proc_a,&header[4],sizeof(WORD));
  memcpy(&current_snapshot->proc_x,&header[6],sizeof(WORD));
  memcpy(&current_snapshot->proc_y,&header[8],sizeof(WORD));
  current_snapshot->proc_dbr = header[12];
  current_snapshot->proc_pbr = header[13];
  memcpy(&current_snapshot->proc_pc,&header[10],sizeof(WORD));
  memcpy(&current_snapshot->proc_stack,&header[14],sizeof(WORD));
  memcpy(&current_snapshot->proc_directpage,&header[16],sizeof(WORD));

  /* Status Register */
  memcpy(&one_word,&header[18],sizeof(WORD));
  current_snapshot->status_c = (one_word >> 0) & 0x0001;
  current_snapshot->status_z = (one_word >> 1) & 0x0001;
  current_snapshot->status_i = (one_word >> 2) & 0x0001;
  current_snapshot->status_d = (one_word >> 3) & 0x0001;
  current_snapshot->status_x = (one_word >> 4) & 0x0001;
  current_snapshot->status_m = (one_word >> 5) & 0x0001;
  current_snapshot->status_v = (one_word >> 6) & 0x0001;
  current_snapshot->status_n = (one_word >> 7) & 0x0001;
  current_snapshot->status_e = (one_word >> 8) & 0x0001;
  current_snapshot->status_b = (one_word >> 9) & 0x0001;

  /* RAM Size : Nombre de bank mmoire en plus des 256 KB de base */
  current_snapshot->ram_size = (data_length-SNAPSHOT_HEADER_SIZE) / 65536;
  current_snapshot->ram_size--;       /* DOC RAM */
  current_snapshot->ram_size -= 2;    /* ROM 01 */
  if(rom_version == 0x0003)
    current_snapshot->ram_size -= 2;  /* ROM 03 */
  current_snapshot->ram_size -= 4;    /* RAM */

  /* ROM Type */
  if(rom_version == 0x0001)
    current_snapshot->rom_type = SNAPSHOT_APPLEIIGS_ROM01;
  else if(rom_version == 0x0003)
    current_snapshot->rom_type = SNAPSHOT_APPLEIIGS_ROM03;

  /* DOC Registers */
  memcpy(current_snapshot->doc_register,&header[512],32*7+3);

  /* Battery RAM */
  memcpy(current_snapshot->battery_ram,&header[768],256);

  /* Libration mmoire */
  mem_free_list(nb_path,file_path_tab);

  /* OK */
  return(current_snapshot);
}


/************************************************************************/
/*  mem_free_snapshot() :  Libration mmoire de la structure snapshot. */
/************************************************************************/
void mem_free_snapshot(struct snapshot *current_snapshot)
{  
  if(current_snapshot)
    {
      if(current_snapshot->snapshot_data)
        free(current_snapshot->snapshot_data);
        
      /* Libration structure */
      free(current_snapshot);
    }
}

/************************************************************************/
