/************************************************************************/
/*                                                                      */
/*  Dc_Snapshot.h :  Header pour les fonction de gestion des Snapshot.  */
/*                                                                      */
/************************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe Software  *  July 2023   */
/************************************************************************/

/* Types de ROM dans le Snapshot */
#define SNAPSHOT_APPLEII_ROM       0
#define SNAPSHOT_APPLEIIGS_ROM01   1
#define SNAPSHOT_APPLEIIGS_ROM03   3

#define SNAPSHOT_HEADER_SIZE    1024

#define DOC_REGISTER_OFFSET      512
#define BATTERY_RAM_OFFSET       768

#define SNAPSHOT_TYPE_II           0     /* Apple IIe */
#define SNAPSHOT_TYPE_GS           1     /* Apple IIgs */

#define SNAPSHOT_FILE_HEADER_OFFSET_A_W        4    /* A */
#define SNAPSHOT_FILE_HEADER_OFFSET_X_W        6    /* X */
#define SNAPSHOT_FILE_HEADER_OFFSET_Y_W        8    /* Y */
#define SNAPSHOT_FILE_HEADER_OFFSET_PC_W      10    /* PC */
#define SNAPSHOT_FILE_HEADER_OFFSET_PBR_B     12    /* Program Bank Register */
#define SNAPSHOT_FILE_HEADER_OFFSET_DBR_B     13    /* Data Bank Register */
#define SNAPSHOT_FILE_HEADER_OFFSET_STACK_W   14    /* Stack */
#define SNAPSHOT_FILE_HEADER_OFFSET_DIRECT_W  16    /* Direct */
#define SNAPSHOT_FILE_HEADER_OFFSET_STATUS_W  18    /* Status */
#define SNAPSHOT_FILE_HEADER_OFFSET_ROM_W     20    /* Rom Version */

/** Snapshot File Format **/
/*    RAM
           Bank 00             64 KB
           Bank 01             64 KB
          [Bank 02             64 KB]
          [...]
           Bank E0             64 KB
           Bank E1             64 KB
      ROM
          [Bank FC             64 KB]   ROM 03 only
          [Bank FD             64 KB]   ROM 03 only
           Bank FE             64 KB
           Bank FF             64 KB
      DOC
           DOC RAM             64 KB
      HEADER
      0    IIm\0 | GSm\0        4 Bytes
      4    A                   16 bit
      6    X                   16 bit
      8    Y                   16 bit
     10    PC                  16 bit
     12    DBR                  8 bit
     13    PBR                  8 bit
     14    STACK               16 bit
     16    DIRECT              16 bit
     18    STATUS              16 bit
     20    ROM VERSION         16 bit
  
    512    DOC Registers      227 bytes 32*7+3
    768    Battery RAM        256 bytes 
*/

/** KEGS -> Cyrene Snapshot Format **/
/*  NEXT OP
      0    A                   16 bit
      2    X                   16 bit
      4    Y                   16 bit
      6    PC                  16 bit
      8    PBR                  8 bit
      9    DBR                  8 bit
     10    STACK               16 bit
     12    DIRECT              16 bit
     14    STATUS              16 bit
     16    Opcode               8 bit
     17    Operand #1           8 bit
     18    Operand #2           8 bit
     19    Operand #3           8 bit
     20    Data Size / #Cycle   8 bit
     21    Read Address        24 bit
     24    Write Address       24 bit
     27    Jump Address        24 bit
     30    GS ROM Version       8 bit    01=Rom01 / 03=Rom03
     31    # RAM Bank           8 bit    Min=4 / Max=225
     32    $C02E                8 bit
     33    $C02F                8 bit
     34    $C035                8 bit
     35    $C068                8 bit
     36    Global ID           64 bit
     44    Global Cycle        64 bit

    512    DOC Registers      227 bytes  32*7+3
    768    Battery RAM        256 bytes
    RAM
           Bank 00             64 KB
           Bank 01             64 KB
          [Bank 02             64 KB]
          [...]
           Bank E0             64 KB
           Bank E1             64 KB
    ROM
          [Bank FC             64 KB]
          [Bank FD             64 KB]
           Bank FE             64 KB
           Bank FF             64 KB
    DOC
           DOC RAM             64 KB    
    */

struct snapshot
{
  /* Nom du fichier contenant les informations du snapshot */
  char snapshot_file_path[BUFFER_SIZE];
  
  /* Donnes du Snapshot */
  int snapshot_type;                                    /* SNAPSHOT_TYPE_XXX... */
  unsigned char snapshot_header[SNAPSHOT_HEADER_SIZE];  /* Header */
  int snapshot_length;
  unsigned char *snapshot_data;                         /* Data */
  
  /* Processor */
  WORD proc_a;
  WORD proc_x;
  WORD proc_y;
  BYTE proc_dbr;
  BYTE proc_pbr;
  WORD proc_pc;
  WORD proc_stack;
  WORD proc_directpage;

  /* Status Register */
  int status_c;
  int status_z;
  int status_i;
  int status_d;
  int status_x;
  int status_m;
  int status_v;
  int status_n;
  int status_e;
  int status_b;

  /* RAM Size */
  int ram_size;       /* Nombre de bank mmoire en plus des 256 KB de base */
  
  /* ROM Type */
  int rom_type;       /* SNAPSHOT_APPLEII_ROM | SNAPSHOT_APPLEIIGS_ROM01 | SNAPSHOT_APPLEIIGS_ROM03 */

  /* DOC Registers : 224+3 bytes */
  BYTE doc_register[DOC_REGISTER_SIZE];

  /* Battery RAM : 256 bytes */
  BYTE battery_ram[BATTERY_RAM_SIZE];
};

struct snapshot *BuildSnapshotFromKegs(int,unsigned char *,char *);
int SaveSnapshotFile(int);
struct snapshot *LoadSnapshotFile(int);
void mem_free_snapshot(struct snapshot *);

/************************************************************************/
