/*********************************************************************/
/*                                                                   */
/*  Window_Attach.c :  Module de gestion de la fentre Attach.       */
/*                                                                   */
/*********************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe  *  Date : Juin 2023  */
/*********************************************************************/

#include <stdio.h>
#include <time.h>

#include <windows.h>
#include <windowsx.h>
#include <Commctrl.h>
#include <Winuser.h>
#include <Richedit.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_MMU.h"
#include "Dc_Snapshot.h"
#include "Dc_Notification.h"
#include "Dc_Graphic.h"
#include "Dc_CPU.h"
#include "Dc_Request.h"
#include "Window_QuickWatch.h"
#include "Window_Main.h"
#include "Window_Main_Menu.h"
#include "Window_IPC.h"
#include "Window_Attach.h"

/* Kegs Executable IPC Window */
#define KEGS_IPC_WINDOW_CLASS     "KegsIPC"
#define KEGS_IPC_WINDOW_NAME      "Kegs_%d"

static DWORD BuildAttachWindowThread(void *);
LRESULT CALLBACK ProcHandlerAttachWindow(HWND,UINT,WPARAM,LPARAM);
static int CreateAttachChildWindows(HWND,struct window_attach *);
LRESULT CALLBACK subEditAttachProcControl(HWND,UINT,WPARAM,LPARAM);
static void SelectApplication(struct window_attach *);
static void RefreshApplicationList(struct window_attach *);
BOOL CALLBACK EnumApplicationWindowsProc(HWND,LPARAM) ;
static int compare_application(const void *,const void *);
static void mem_free_application(struct application *);
static void DeleteAttachChildWindows(struct window_attach *);


/**************************************************************************/
/*  BuildAttachWindow() :  Construction / Affichage de la fentre Attach. */
/**************************************************************************/
struct window_attach *BuildAttachWindow(HINSTANCE hInstance, int menu_id)
{
  struct window_attach *window_attach;

  /** Allocation de la structure window_attach **/
  window_attach = (struct window_attach *) calloc(1,sizeof(struct window_attach));
  if(window_attach == NULL)
    return(NULL);
  my_Memory(MEMORY_SET_WINDOW_ATTACH,window_attach,NULL);

  /* Lance un thread pour grer la fentre Attach */
  CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)BuildAttachWindowThread,window_attach,0,NULL);

  /* Renvoi la structure mmoire grant la fentre */
  return(window_attach);
}


/************************************************************************************/
/*  BuildAttachWindowThread() :  Cration de la fentre Attach via un Thread ddi. */
/************************************************************************************/
static DWORD BuildAttachWindowThread(void *window_attach_param)
{
  HWND hwndAttach;
  MSG msg;
  BOOL bRet;
  struct window_main *window_main;
  struct window_attach *window_attach;
  struct parameter *param;

  /* Init */
  window_attach = (struct window_attach *) window_attach_param;
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Cration de la fentre Attach **/
  hwndAttach = CreateDialog(param->hInstance,MAKEINTRESOURCE(IDD_WINDOW_ATTACH),window_main->hwnd_main,(DLGPROC)ProcHandlerAttachWindow);
  if(hwndAttach == NULL)
    return(1);

  /** Show Window + Paint Content **/
  ShowWindow(hwndAttach,SW_SHOWDEFAULT);
  UpdateWindow(hwndAttach);

  /** Message Loop pour toutes les fentres de ce Thread (fentre Attach...) **/
  while((bRet = GetMessage(&msg,NULL,0,0)) != 0)
    {
      if(bRet == -1)
        break;

      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }

  /* Fin du Thread */
  ExitThread(0);

  /* OK */
  return(0);
}


/***********************************************************************/
/*  ProcHandlerAttachWindow() :  Gestion des vnements de la fentre. */
/***********************************************************************/
LRESULT CALLBACK ProcHandlerAttachWindow(HWND hwndAttach, UINT umsg, WPARAM wParam, LPARAM lParam)
{
  struct window_attach *window_attach;
  struct parameter *param;
  
  /* Init */
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  my_Memory(MEMORY_GET_WINDOW_ATTACH,&window_attach,NULL);

  switch(umsg)
    {
      case WM_INITDIALOG :
        /* Cration du contenu de la DialogBox */
        CreateAttachChildWindows(hwndAttach,window_attach);
        return(TRUE);

      /****************************/
      /* Evnements de la fentre */
      /****************************/
      case WM_COMMAND :
        switch(LOWORD(wParam))
          {              
            /** Bouton pour rechercher des Fentres **/
            case IDC_BUTTON_REFRESH :
              RefreshApplicationList(window_attach);
              break;

            /** Bouton pour choisir la fenre slectionne **/
            case IDC_BUTTON_SELECT :
              SelectApplication(window_attach);
              break;
          }
        return(TRUE);

      /********************************************************************/
      /* Fermeture de window (le WM_CLOSE est appel avant le WM_DESTROY) */
      /********************************************************************/
      case WM_CLOSE :
        /* Fentre Attach */
        DestroyWindow(hwndAttach);
        return(0);

      case WM_DESTROY :
        /* Suppression des Fentres */
        DeleteAttachChildWindows(window_attach);
        CheckMainWindowMenuItem(ID_MENU_FILE_ATTACH_KEGS,FALSE);
        my_Memory(MEMORY_FREE_WINDOW_ATTACH,NULL,NULL);
        /* Demande la sortie de la boucle des messages */
        PostQuitMessage(0);
        return(0);
    }

  /* Retourner FALSE pour relayer le message non trait au systme */
  return(FALSE);
}


/***************************************************************************************************/
/*  CreateAttachChildWindows() : Crer les fentres enfants et remplir la structure window_attach. */
/***************************************************************************************************/
static int CreateAttachChildWindows(HWND hwndAttach, struct window_attach *window_attach)
{
  RECT client_rect, window_rect;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
        
  /** Window Handle **/
  window_attach->hwnd_attach = hwndAttach;

  /* Hauteur Titre */
  GetWindowRect(window_attach->hwnd_attach,&window_rect);
  GetClientRect(window_attach->hwnd_attach,&client_rect);
  window_attach->title_height = (window_rect.bottom - window_rect.top) - client_rect.bottom;

  /** Remplissage de la liste avec la liste des applications **/
  RefreshApplicationList(window_attach);

  /* OK */
  return(0);
}


/*************************************************************************/
/*  SelectApplication() :  Slectionne l'application KEGS dans la liste. */
/*************************************************************************/
static void SelectApplication(struct window_attach *window_attach)
{
  int i;
  char request_data[1024];
  struct window_main *window_main;
  struct window_ipc *window_ipc;
  struct callstack *current_callstack;
  struct callstack *next_callstack;
  struct application *current_application;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Rcupre la fentre Main */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);

  /* Rcupre la fentre IPC de Cyrene */
  my_Memory(MEMORY_GET_WINDOW_IPC,&window_ipc,NULL);

  /* Rcupre l'application slectionne */
  i = (int) SendMessage(GetDlgItem(window_attach->hwnd_attach,IDC_LIST_APPLICATION),LB_GETCURSEL,0,0);
  current_application = (i>=0) ? window_attach->tab_application[i] : NULL;
  if(current_application != NULL)
    {
      /* Conserve le PID de l'application KEGS */
      param->attach_pid = current_application->pid;

      /* Nom de la fentre Kegs IPC */
      sprintf(param->kegs_ipc_window_name,KEGS_IPC_WINDOW_NAME,param->attach_pid);

      /* Rcupre le Handle de la fentre IPC de Kegs */
      param->hwnd_kegs_ipc = FindWindow(KEGS_IPC_WINDOW_CLASS,param->kegs_ipc_window_name);

      /** Accs  la Shared Memory **/
      sprintf(window_ipc->shared_memory_path,"Local\\"CYRENE_IPC_WINDOW_NAME,param->attach_pid);
      window_ipc->hSharedMemory = OpenFileMapping(FILE_MAP_ALL_ACCESS,FALSE,window_ipc->shared_memory_path);
      if(window_ipc->hSharedMemory == NULL)
        {
          MessageBox(window_main->hwnd_main,"Error, Impossible to access Kegs/Cyrene Shared Memory !","Shared Memory Error",MB_OK|MB_ICONWARNING);
          return;
        }
      window_ipc->shared_memory_ptr = (unsigned char *) MapViewOfFile(window_ipc->hSharedMemory,FILE_MAP_ALL_ACCESS,0,0,IPC_SHARED_MEMORY_TOTAL_SIZE);
      if(window_ipc->shared_memory_ptr == NULL)
        {
          MessageBox(window_main->hwnd_main,"Error, Impossible to access Kegs/Cyrene Shared Memory !","Shared Memory Error",MB_OK|MB_ICONWARNING);
          if(window_ipc->hSharedMemory != 0)
            CloseHandle(window_ipc->hSharedMemory);
          window_ipc->hSharedMemory = NULL;
          return;
        }

      /* On va attendre la rponse de Kegs */
      param->kegs_connection_status = CONNECTION_STATUS_WAIT_FOR_ACCEPT;

      /* Reset la Call Stack */
      EnterCriticalSection(&param->cs_callstack);
      for(current_callstack = window_main->first_callstack; current_callstack; current_callstack = next_callstack)
        {
          next_callstack = current_callstack->next;
          free(current_callstack);
        }
      window_main->nb_callstack = 0;
      window_main->first_callstack = NULL;
      window_main->last_callstack = NULL;
      LeaveCriticalSection(&param->cs_callstack);

      /* On transmet le nom de la fentre IPC Cyrene  Kegs lors de la demande d'ouverture de session */
      sprintf(request_data,"CyreneIPCWindowName=%s,",window_ipc->cyrene_ipc_window_name);
      SendDataToKegs(IPC_REQUEST_TYPE_C2K_OPEN_CONNECTION,(int)strlen(request_data)+1,(unsigned char *)request_data);

      /* Ferme la fentre */
      SendMessage(window_attach->hwnd_attach,WM_CLOSE,0,0);

      /* MAJ de la Status Bar */
      UpdateWindowMainStatusBar();
    }
}


/*********************************************************************/
/*  RefreshApplicationList() : Rafraichit la liste des applications. */
/*********************************************************************/
static void RefreshApplicationList(struct window_attach *window_attach)
{
  int i, position;
  struct application *current_application;
  struct application *next_application;
  char application_name[1024];

  /** Libration mmoire **/
  for(current_application = window_attach->first_application; current_application; current_application = next_application)
    {
      next_application = current_application->next;
      mem_free_application(current_application);
    }
  if(window_attach->tab_application)
    free(window_attach->tab_application);
  window_attach->first_application = NULL;
  window_attach->last_application = NULL;
  window_attach->tab_application = NULL;
  window_attach->nb_application = 0;

  /** Recherche des applications **/
  EnumWindows(EnumApplicationWindowsProc,0);

  /** Allocation mmoire + Tri par ordre alphabtique de la liste chaine */
  window_attach->tab_application = (struct application **) calloc(window_attach->nb_application+1,sizeof(struct application *));
  if(window_attach->tab_application == NULL)
    return;
  for(i=0,current_application=window_attach->first_application; current_application; current_application=current_application->next)
    window_attach->tab_application[i++] = current_application;

  /* Tri du tableau (par ordre d'archivage) */
  qsort((void *)window_attach->tab_application,(size_t)window_attach->nb_application,sizeof(struct application *),compare_application);

  /* Remise en place des liens */
  if(window_attach->nb_application > 0)
    {
      window_attach->first_application = window_attach->tab_application[0];
      window_attach->last_application = window_attach->tab_application[window_attach->nb_application-1];
      for(i=0; i<window_attach->nb_application-1; i++)
        window_attach->tab_application[i]->next = window_attach->tab_application[i+1];
      window_attach->tab_application[window_attach->nb_application-1]->next = NULL;
    }

  /* Supprime tous les lments de la liste */
  SendMessage(GetDlgItem(window_attach->hwnd_attach,IDC_LIST_APPLICATION),LB_RESETCONTENT,0,(LPARAM)0);
  
  /** Ajoute le titre des Applications dans la liste **/
  for(i=0, current_application=window_attach->first_application; current_application; current_application=current_application->next, i++)
    {
      sprintf(application_name,"%s",current_application->title);
      position = (int) SendMessage(GetDlgItem(window_attach->hwnd_attach,IDC_LIST_APPLICATION),LB_ADDSTRING,0,(LPARAM)application_name);
      SendMessage(GetDlgItem(window_attach->hwnd_attach,IDC_LIST_APPLICATION),LB_SETITEMDATA,position,(LPARAM)i);
    }

  /* Focus dans la zone de slection */
  SetFocus(GetDlgItem(window_attach->hwnd_attach,IDC_LIST_APPLICATION));
}


/*****************************************************************************/
/*  EnumApplicationWindowsProc() :  Fonction de scan des fentres top-level. */
/*****************************************************************************/
BOOL CALLBACK EnumApplicationWindowsProc(HWND hWnd, LPARAM lParam) 
{
  BOOL result;
  WINDOWINFO window_info;
  char title[1024];
  char exe_path[1024];
  int delta_rc_window_height,delta_rc_client_height;
  int delta_rc_window_width,delta_rc_client_width;
  int length;
  struct application *current_application;
  struct window_attach *window_attach;

  /* Init */
  my_Memory(MEMORY_GET_WINDOW_ATTACH,&window_attach,NULL);

  /* Titre de la fentre et chemin du .exe */
  length = GetWindowText(hWnd,title,1024);
  title[length] = '\0';
  length = GetWindowModuleFileName(hWnd,exe_path,1024);
  exe_path[length] = '\0';
  
  /* On ne prend que les fentres visibles */
  result = IsWindowVisible(hWnd);
  if(!result)
    return(TRUE);
    
  /* On ne prend que les fentres utiles */
  if(strlen(title) == 0)
    return(TRUE);
    
  /* On limine les .dll */
  if(strlen(exe_path) > 4)
    if(!_stricmp(&exe_path[strlen(exe_path)-4],".dll"))
      return(TRUE);
   
  /* On ne prend que les fentre de taille minimum */
  memset(&window_info,0,sizeof(WINDOWINFO));
  result = GetWindowInfo(hWnd,&window_info);
  
  /* On fait le delta de la taille des fentre */
  delta_rc_window_height = window_info.rcWindow.bottom - window_info.rcWindow.top;
  delta_rc_window_width = window_info.rcWindow.right - window_info.rcWindow.left;
  
  /* On fait le delta de la taille de la zone client */
  delta_rc_client_height = window_info.rcClient.bottom - window_info.rcClient.top;
  delta_rc_client_width = window_info.rcClient.right - window_info.rcClient.left;
  if((delta_rc_window_height < 10  && delta_rc_window_width < 20) && (delta_rc_client_height < 5 && delta_rc_client_width < 10))
    return(TRUE);
  
  /* Allocation mmoire */
  current_application = (struct application *) calloc(1,sizeof(struct application));
  if(current_application == NULL)
    return(TRUE);

  /* Remplissage structure */
  current_application->hwnd_application = hWnd;
  memcpy(&current_application->window_info,&window_info,sizeof(WINDOWINFO));
  current_application->title = _strdup(title);
  current_application->exe_path = _strdup(exe_path);
  GetWindowThreadProcessId(current_application->hwnd_application,&current_application->pid); 

  /* Vrification des allocations */
  if(current_application->title == NULL || current_application->exe_path == NULL)
    {
      mem_free_application(current_application);
      return(TRUE);
    }

  /* Stockage de la structure */
  if(window_attach->first_application == NULL)
    window_attach->first_application = current_application;
  else
    window_attach->last_application->next = current_application;
  window_attach->last_application = current_application;
  window_attach->nb_application++;

  return(TRUE);
}


/************************************************************************/
/*  compare_application() : Fonction de comparaison pour le Quick Sort  */
/************************************************************************/
static int compare_application(const void *data_1, const void *data_2)
{
  struct application *application_1;
  struct application *application_2;

  /* Rcupration des paramtres */
  application_1 = *((struct application **) data_1);
  application_2 = *((struct application **) data_2);

  /* Comparaison des Titres */
  return(stricmp(application_1->title,application_2->title));
}


/**********************************************************************/
/*  mem_free_application() :  Libration de la structure application. */
/**********************************************************************/
static void mem_free_application(struct application *current_application)
{
  if(current_application)
    {
      if(current_application->title)
        free(current_application->title);

      if(current_application->exe_path)
        free(current_application->exe_path);

      free(current_application);
    }
}


/********************************************************************/
/*  DeleteAttachChildWindows() :  Destruction des fentres enfants. */
/********************************************************************/
static void DeleteAttachChildWindows(struct window_attach *window_attach)
{
  /** Libration des ressources **/  
}


/**********************************************************************/
/*  mem_free_window_attach() :  Libration mmoire pour la structure. */
/**********************************************************************/
void mem_free_window_attach(struct window_attach *window_attach)
{ 
  struct application *current_application;
  struct application *next_application;

  if(window_attach)
    {
      for(current_application = window_attach->first_application; current_application; current_application = next_application)
        {
          next_application = current_application->next;
          mem_free_application(current_application);
        }
      if(window_attach->tab_application)
        free(window_attach->tab_application);
           
      free(window_attach);
    }
}

/**********************************************************************/
