/*************************************************************************/
/*                                                                       */
/*  Window_BatteryRam.c :  Module de gestion de la fentre Battery Ram.  */
/*                                                                       */
/*************************************************************************/
/*  Auteur : Olivier ZARDINI   *   Brutal Deluxe   *   Date : Juin 2023  */
/*************************************************************************/

#include <stdio.h>
#include <time.h>

#include <windows.h>
#include <windowsx.h>
#include <Commctrl.h>
#include <Winuser.h>
#include <Richedit.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_Notification.h"
#include "Dc_Graphic.h"
#include "Dc_CPU.h"
#include "Dc_MMU.h"
#include "Window_QuickWatch.h"
#include "Window_Main.h"
#include "Window_Main_Menu.h"
#include "Window_BatteryRam.h"

static DWORD BuildBatteryRamWindowThread(void *);
LRESULT CALLBACK ProcHandlerBatteryRamWindow(HWND,UINT,WPARAM,LPARAM);
static int CreateBatteryRamChildWindows(HWND,struct window_batteryram *);
static void UpdateBatteryRamContextInformation(struct window_batteryram *,int,int);
static void UpdateBatteryRamRefreshMode(struct window_batteryram *,int);
static void RefreshBatteryRamWindowData(struct window_batteryram *);
static int IsBatteryRamDataModified(int,int,int,struct window_batteryram *);
static void ResizeBatteryRamWindow(struct window_batteryram *);
static void BuildBatteryRamFromData(struct window_batteryram *);
static DWORD CALLBACK BatteryRamRichEditStreamCallback(DWORD *,LPBYTE,LONG,LONG *);
static void DeleteBatteryRamChildWindows(struct window_batteryram *);


/**********************************************************************************/
/*  BuildBatteryRamWindow() :  Construction / Affichage de la fentre BatteryRam. */
/**********************************************************************************/
struct window_batteryram *BuildBatteryRamWindow(HINSTANCE hInstance, int menu_id)
{
  struct window_batteryram *window_batteryram;

  /** Allocation de la structure window_batteryram **/
  window_batteryram = (struct window_batteryram *) calloc(1,sizeof(struct window_batteryram));
  if(window_batteryram == NULL)
    return(NULL);
  my_Memory(MEMORY_SET_WINDOW_BATTERYRAM,window_batteryram,NULL);

  /* Lance un thread pour grer la fentre BatteryRam */
  CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)BuildBatteryRamWindowThread,window_batteryram,0,NULL);

  /* Renvoi la structure mmoire grant la fentre */
  return(window_batteryram);
}


/********************************************************************************************/
/*  BuildBatteryRamWindowThread() :  Cration de la fentre BatteryRam via un Thread ddi. */
/********************************************************************************************/
static DWORD BuildBatteryRamWindowThread(void *window_batteryram_param)
{
  HWND hwndBatteryRam;
  MSG msg;
  BOOL bRet;
  struct window_main *window_main;
  struct window_batteryram *window_batteryram;
  struct parameter *param;
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Init */
  window_batteryram = (struct window_batteryram *) window_batteryram_param;

  /** Cration de la fentre Memory Area **/
  hwndBatteryRam = CreateDialog(param->hInstance,MAKEINTRESOURCE(IDD_WINDOW_BATTERYRAM),window_main->hwnd_main,(DLGPROC)ProcHandlerBatteryRamWindow);
  if(hwndBatteryRam == NULL)
    return(1);

  /** Show Window + Paint Content **/
  ShowWindow(hwndBatteryRam,SW_SHOWDEFAULT);
  UpdateWindow(hwndBatteryRam);

  /** Message Loop pour toutes les fentres de ce Thread (fentre BatteryRam...) **/
  while((bRet = GetMessage(&msg,NULL,0,0)) != 0)
    {
      if(bRet == -1)
        break;

      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }

  /* Fin du Thread */
  ExitThread(0);
}


/***************************************************************************/
/*  ProcHandlerBatteryRamWindow() :  Gestion des vnements de la fentre. */
/***************************************************************************/
LRESULT CALLBACK ProcHandlerBatteryRamWindow(HWND hwndBatteryRam, UINT umsg, WPARAM wParam, LPARAM lParam)
{
  int mouse_x, mouse_y;
  MSGFILTER *message_filter;
  struct window_batteryram *window_batteryram;
  struct parameter *param;
  
  /* Init */
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  my_Memory(MEMORY_GET_WINDOW_BATTERYRAM,&window_batteryram,NULL);
  if(window_batteryram == NULL)
    return(TRUE);

  switch(umsg)
    {
      case WM_INITDIALOG :
        /* Cration du contenu de la DialogBox */
        CreateBatteryRamChildWindows(hwndBatteryRam,window_batteryram);
        return(TRUE);

      /***************************/
      /* Evnements de la souris */
      /***************************/
      case WM_MOUSEMOVE :
        SetWindowText(GetDlgItem(window_batteryram->hwnd_batteryram,IDC_STATIC_CONTEXT),"");
        return(TRUE);
        
      /****************************/
      /* Evnements de la fentre */
      /****************************/
      case WM_COMMAND :
        switch(LOWORD(wParam))
          {
            /** Bouton Radio : On change la mthode de rafraichissement de la fentre **/
            case IDC_RADIO_REFRESH_AUTOMATIC :
              if(HIWORD(wParam) == BN_CLICKED && window_batteryram->refresh_mode != BATTERYRAM_REFRESH_AUTOMATIC)
                UpdateBatteryRamRefreshMode(window_batteryram,BATTERYRAM_REFRESH_AUTOMATIC);
              break;

            case IDC_RADIO_REFRESH_MANUAL :
              if(HIWORD(wParam) == BN_CLICKED && window_batteryram->refresh_mode != BATTERYRAM_REFRESH_MANUAL)
                UpdateBatteryRamRefreshMode(window_batteryram,BATTERYRAM_REFRESH_MANUAL);
              break;

            /** Bouton de rafraichissement Manuel de la fentre **/
            case IDC_BUTTON_REFRESH :
              RefreshBatteryRamWindowData(window_batteryram);
              break;
          }
        return(TRUE);

      /**********************************************/
      /* Evnements des Controles  la fentre mre */
      /**********************************************/
      case WM_NOTIFY :
        message_filter = (MSGFILTER *) lParam;
        
        /** Evnement de la souris dans le Rich Edit transmis  la fentre mre **/
        if(message_filter && message_filter->nmhdr.hwndFrom == window_batteryram->hwnd_richedit_data)
          {
            mouse_x = GET_X_LPARAM(message_filter->lParam) / 7;
            mouse_y = GET_Y_LPARAM(message_filter->lParam) / BATTERYRAM_RICHEDIT_FONT_HEIGHT;
            UpdateBatteryRamContextInformation(window_batteryram,mouse_x,mouse_y);
          }
        return(FALSE);
        
      /*****************************************************************/
      /* Message extrieur demandant le rafraichissement de la fentre */
      /*****************************************************************/
      case HM_BATTERYRAM_REFRESHWINDOW :
        RefreshBatteryRamWindowData(window_batteryram);
        return(TRUE);

      /********************************************************************/
      /* Fermeture de window (le WM_CLOSE est appel avant le WM_DESTROY) */
      /********************************************************************/
      case WM_CLOSE :
        /* Fentre BatteryRam */
        DestroyWindow(hwndBatteryRam);
        return(0);

      case WM_DESTROY :
        /* Suppression des Fentres */
        DeleteBatteryRamChildWindows(window_batteryram);

        /* Dcoche le Menu */
        CheckMainWindowMenuItem(ID_MENU_VIEW_BATTERYRAM,FALSE);

        /* Supprime la structure mmoire de ce thread */
        my_Memory(MEMORY_FREE_WINDOW_BATTERYRAM,NULL,NULL);

        /* Demande la sortie de la boucle des messages */
        PostQuitMessage(0);
        return(0);
    }

  /* Retourner FALSE pour relayer le message non trait au systme */
  return(FALSE);
}


/***********************************************************************************************************/
/*  CreateBatteryRamChildWindows() : Crer les fentres enfants et remplir la structure window_batteryram. */
/***********************************************************************************************************/
static int CreateBatteryRamChildWindows(HWND hwndBatteryRam, struct window_batteryram *window_batteryram)
{
  HWND hwnd_group;
  DWORD dialog_background_color;
  RECT client_rect, window_rect, group_rect;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Window Handle **/
  window_batteryram->hwnd_batteryram = hwndBatteryRam;

  /** Adresse de la Zone de Data **/
  window_batteryram->batteryram_address = 0x0000; 

  /** Zone d'affichage des Battery Ram Data **/
  window_batteryram->nb_column = 16;
  window_batteryram->nb_row = 16; 

  /* Nombre caractres visibles sur la ligne : Address Data Ascii (on ne compte pas le \r\n final) */
  window_batteryram->text_data_width = (int) ((strlen("00   ") + window_batteryram->nb_column*3 - 1));   /* Enlve le dernier sparateur */

  /* Refresh Mode : Automatic */
  UpdateBatteryRamRefreshMode(window_batteryram,BATTERYRAM_REFRESH_AUTOMATIC);

  /* Hauteur Titre */
  GetWindowRect(window_batteryram->hwnd_batteryram,&window_rect);
  GetClientRect(window_batteryram->hwnd_batteryram,&client_rect);
  window_batteryram->title_height = (window_rect.bottom - window_rect.top) - client_rect.bottom;

  /* Position de la GroupBox Select + GroupBox Palette */
  hwnd_group = GetDlgItem(window_batteryram->hwnd_batteryram,IDC_GROUP_BATTERYRAM);     /* Rcupre le Handle du Control GroupBox li  IDC_GROUP_RAMSOUND */
  GetWindowRect(hwnd_group,&group_rect);
  window_batteryram->group_data_x = group_rect.left - window_rect.left;
  window_batteryram->group_data_y = (group_rect.top - window_rect.top) - window_batteryram->title_height;

  /* Position de la Zone Text Direct Page Data dans la fenetre */
  window_batteryram->batteryram_data_x = window_batteryram->group_data_x + WINDOW_GROUP_BORDER_LEFT_SIZE;
  window_batteryram->batteryram_data_y = window_batteryram->group_data_y + WINDOW_GROUP_BORDER_TOP_SIZE;

  /* Dimension en Pixel du Control Text Battery Ram Data */
  window_batteryram->batteryram_data_width = window_batteryram->text_data_width*7 + 2;
  window_batteryram->batteryram_data_height = window_batteryram->nb_row*BATTERYRAM_EDIT_FONT_HEIGHT;

  /* Dimension en Pixel du Control Group Box + Resize */  
  window_batteryram->group_data_width = WINDOW_GROUP_BORDER_LEFT_SIZE + window_batteryram->batteryram_data_width + WINDOW_GROUP_BORDER_RIGHT_SIZE;
  window_batteryram->group_data_height = WINDOW_GROUP_BORDER_TOP_SIZE + window_batteryram->batteryram_data_height + WINDOW_GROUP_BORDER_BOTTOM_SIZE;
  MoveWindow(hwnd_group,window_batteryram->group_data_x,window_batteryram->group_data_y,window_batteryram->group_data_width,window_batteryram->group_data_height,TRUE);

  /** Cration du Rich Edit **/
  window_batteryram->hwnd_richedit_data = CreateWindowEx(0,RICHEDIT_CLASS,"",WS_CHILD | WS_VISIBLE | ES_SAVESEL | ES_MULTILINE | ES_READONLY, 
                                                         window_batteryram->batteryram_data_x, window_batteryram->batteryram_data_y, window_batteryram->batteryram_data_width, window_batteryram->batteryram_data_height,   // set size in WM_SIZE message 
                                                         window_batteryram->hwnd_batteryram,
                                                         (HMENU)IDC_RICHEDIT_BATTERYRAM_DATA,
                                                         param->hInstance,
                                                         NULL);
  
  /* Couleur de Fond du control Rich Edit : Gris comme la fentre d'accueil */
  dialog_background_color = GetSysColor(COLOR_3DFACE);
  SendMessage(window_batteryram->hwnd_richedit_data,EM_SETBKGNDCOLOR,0,(LPARAM)dialog_background_color); 
  SendMessage(window_batteryram->hwnd_richedit_data,EM_SETEVENTMASK,0,(LPARAM)ENM_MOUSEEVENTS);

  /* Retaille la fentre */
  ResizeBatteryRamWindow(window_batteryram);

  /** Construction du Texte + Rafraichissement de la fentre **/
  RefreshBatteryRamWindowData(window_batteryram);
    
  /* OK */
  return(0);
}



/*******************************************************************************************************/
/*  UpdateBatteryRamContextInformation() :  MAJ des information de contexte de la fenetre BatteryRam. */
/*******************************************************************************************************/
static void UpdateBatteryRamContextInformation(struct window_batteryram *window_batteryram, int grid_x, int grid_y)
{
  int grid_batteryram_x, batteryram_data_address, grid_batteryram_offset, batteryram_data_offset;
  unsigned char batteryram_data;

  /* Rien  faire */
  if(window_batteryram == NULL)
    return;

  /* En dehors de la zone */
  if(grid_x < 5 || grid_x > (5+window_batteryram->nb_column*3-1))
    {
      SetWindowText(GetDlgItem(window_batteryram->hwnd_batteryram,IDC_STATIC_CONTEXT),"");
      return;
    }
  grid_batteryram_x = grid_x - 5;     /* Enlve les 5 caractres de gauche */
  if(((grid_batteryram_x+1) % 3) == 0)
    {
      SetWindowText(GetDlgItem(window_batteryram->hwnd_batteryram,IDC_STATIC_CONTEXT),"");
      return;
    }
  grid_batteryram_offset = (grid_batteryram_x+1) / 3;

  /* Rcupre l'adresse de la cellule */
  batteryram_data_address = window_batteryram->batteryram_address + grid_y*window_batteryram->nb_column;
  batteryram_data_address += grid_batteryram_offset;

  /* Rcupre la valeur de la cellule */
  batteryram_data_offset = batteryram_data_address - window_batteryram->batteryram_address;
  if(batteryram_data_offset > (window_batteryram->nb_column*window_batteryram->nb_row-1))
    {
      SetWindowText(GetDlgItem(window_batteryram->hwnd_batteryram,IDC_STATIC_CONTEXT),"");
      return;
    }
  batteryram_data = window_batteryram->batteryram_data[batteryram_data_offset];

  /* Coordonnes de la souris en coodonnes grid_x,grid_y */
  sprintf(window_batteryram->buffer_log,"BRAM  $%02X : %02X",batteryram_data_address&0xFFFF,batteryram_data);
  SetWindowText(GetDlgItem(window_batteryram->hwnd_batteryram,IDC_STATIC_CONTEXT),window_batteryram->buffer_log);
}


/************************************************************************************************/
/*  UpdateBatteryRamRefreshMode() :  MAJ du mode de rafraichissement de la fenetre BatteryRam. */
/************************************************************************************************/
static void UpdateBatteryRamRefreshMode(struct window_batteryram *window_batteryram, int refresh_mode)
{
  struct notification *current_notification;

  if(refresh_mode == BATTERYRAM_REFRESH_AUTOMATIC)
    {
      /* Visuel */
      window_batteryram->refresh_mode = BATTERYRAM_REFRESH_AUTOMATIC;
      SendDlgItemMessage(window_batteryram->hwnd_batteryram,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_batteryram->hwnd_batteryram,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,0,0);

      /* Cration d'une Notification */
      current_notification = (struct notification *) calloc(1,sizeof(struct notification));
      if(current_notification == NULL)
        return;
      current_notification->hwnd_window = window_batteryram->hwnd_batteryram;
      current_notification->message = HM_BATTERYRAM_REFRESHWINDOW;
      current_notification->type = NOTIFICATION_TYPE_VIEW;
      current_notification->sub_type = NOTIFICATION_SUBTYPE_BATTERYRAM;

      /* Ajout de la notification */
      window_batteryram->current_notification = current_notification;
      my_Notification(NOTIFICATION_ADD,current_notification,NULL);
    }
  else
    {
      /* Visuel */
      window_batteryram->refresh_mode = BATTERYRAM_REFRESH_MANUAL;
      SendDlgItemMessage(window_batteryram->hwnd_batteryram,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_batteryram->hwnd_batteryram,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,1,0);

      /* Supprime la Notification */
      my_Notification(NOTIFICATION_REMOVE,window_batteryram->current_notification,NULL);
      window_batteryram->current_notification = NULL;
    }
}


/******************************************************************************/
/*  RefreshBatteryRamWindowData() :  MAJ du contenu de la fentre BatteryRam. */
/******************************************************************************/
static void RefreshBatteryRamWindowData(struct window_batteryram *window_batteryram)
{
  int is_modified;
  int batteryram_address, nb_column, nb_row;

  /* Rcupre la valeur du Battery Ram Pointer */
  batteryram_address = 0x0000;
  nb_column = 16;
  nb_row = 16;

  /** Les donnes ont t'elles t modifies + Rcupration des nouvelles Data si besoin **/
  is_modified = IsBatteryRamDataModified(batteryram_address,nb_column,nb_row,window_batteryram);
  if(is_modified == 0)
    return;

  /** Cration de la Zone Text Edit  partir des nouvelles donnes **/
  BuildBatteryRamFromData(window_batteryram);

  /* Rafraichit la fentre */
  UpdateWindow(window_batteryram->hwnd_batteryram);
}


/*************************************************************************************************************/
/*  IsBatteryRamDataModified() :  Les donnes BatteryRam ont t'elles t modifies depuis la dernirer fois. */
/*************************************************************************************************************/
static int IsBatteryRamDataModified(int batteryram_address, int nb_column, int nb_row, struct window_batteryram *window_batteryram)
{
  int batteryram_data_address;
  unsigned char *batteryram_data;
  struct mmu *current_mmu;

  /* Rcupre le MMU */
  my_Memory(MEMORY_GET_S65816_MMU,&current_mmu,NULL);
  
  /* Allocation mmoire */
  batteryram_data = (unsigned char *) calloc(nb_column*nb_row+1,sizeof(unsigned char));
  if(batteryram_data == NULL)
    return(0);

  /* Rcupre les donnes */
  batteryram_data_address = batteryram_address;
  ReadBatteryRAMBlock(batteryram_data_address,nb_column*nb_row,batteryram_data,current_mmu);
  
  /* Compare les donnes */
  if(window_batteryram->batteryram_data != NULL)
    if(batteryram_address == window_batteryram->batteryram_address &&
       nb_column == window_batteryram->nb_column &&
       nb_row == window_batteryram->nb_row &&
       memcmp(&batteryram_data[0],&window_batteryram->batteryram_data[0],MY_MIN(nb_column*nb_row,window_batteryram->nb_column*window_batteryram->nb_row)) == 0)
      return(0);
    
  /* Les donnes sont diffrentes => MAJ */
  window_batteryram->batteryram_address = batteryram_address;
  window_batteryram->nb_column = nb_column;
  window_batteryram->nb_row = nb_row;
  if(window_batteryram->batteryram_data != NULL)
    free(window_batteryram->batteryram_data);
  window_batteryram->batteryram_data = batteryram_data;

  /* Donnes diffrentes */
  return(1);
}


/***********************************************************************/
/*  ResizeBatteryRamWindow() : Resize BatteryRam window by given size. */
/***********************************************************************/
static void ResizeBatteryRamWindow(struct window_batteryram *window_batteryram)
{
  RECT window_rect;
  int window_x, window_y;
  
  /* Largeur de la fentre */
  window_batteryram->window_width = window_batteryram->group_data_x + window_batteryram->group_data_width + WINDOW_CLIENT_RIGHT_BORDER_SIZE + WINDOW_BORDER_SIZE;

  /* Hauteur de la fentre */
  window_batteryram->window_height = window_batteryram->title_height + window_batteryram->group_data_y + window_batteryram->group_data_height + WINDOW_CLIENT_BOTTOM_BORDER_SIZE;

  /* Position actuelle de la fentre dans l'cran */
  GetWindowRect(window_batteryram->hwnd_batteryram,&window_rect);
  window_x = window_rect.left;
  window_y = window_rect.top;

  /* Resize window */
  MoveWindow(window_batteryram->hwnd_batteryram,window_x,window_y,window_batteryram->window_width,window_batteryram->window_height,TRUE);
}


/************************************************************************************************************/
/*  BuildBatteryRamFromData() :  Construire les donnes hxadcimal  afficher  partir des donnes brutes. */
/************************************************************************************************************/
static void BuildBatteryRamFromData(struct window_batteryram *window_batteryram)
{
  HWND hwnd_group;
  EDITSTREAM edit_stream;
  int i, j, offset, offset_rtf, nb_byte_to_process, address_data, text_data_width_min;

  /** On doit peut tre redimensionner la zone Text Edit dynamique **/
  if(window_batteryram->rtf_data_hexadecimal == NULL || 
     window_batteryram->text_data_column != window_batteryram->nb_column ||
     window_batteryram->text_data_row != window_batteryram->nb_row)
    {
      /* MAJ */
      window_batteryram->text_data_column = window_batteryram->nb_column;
      window_batteryram->text_data_row = window_batteryram->nb_row;

      /* Libration mmoire */
      if(window_batteryram->rtf_data_hexadecimal)
        free(window_batteryram->rtf_data_hexadecimal);
      window_batteryram->rtf_data_hexadecimal = NULL;

      /* Nombre caractres visibles sur la ligne : Address Data Ascii (on ne compte pas le \r\n final) */
      window_batteryram->text_data_width = (int) (strlen("00   ") + window_batteryram->nb_column*3 - 1);   /* Enlve le dernier sparateur */

      /* Allocation mmoire */
      window_batteryram->rtf_data_size = (int) ((window_batteryram->text_data_width + strlen("\r\n"))*window_batteryram->text_data_row);
      window_batteryram->rtf_data_hexadecimal = (char *) calloc(30*window_batteryram->rtf_data_size+1,sizeof(char));
      if(window_batteryram->rtf_data_hexadecimal == NULL)
        return;

      /* Dimension en Pixel du Control Text Battery Ram Data */
      text_data_width_min = (int) strlen("03   26 2F 00 00 00 00 00 00.00 00 00 00 00 00 ");
      window_batteryram->batteryram_data_width = MY_MAX(window_batteryram->text_data_width,text_data_width_min)*7 + 2;
      window_batteryram->batteryram_data_height = window_batteryram->nb_row*BATTERYRAM_EDIT_FONT_HEIGHT;
      MoveWindow(window_batteryram->hwnd_richedit_data,window_batteryram->batteryram_data_x,window_batteryram->batteryram_data_y,window_batteryram->batteryram_data_width,window_batteryram->batteryram_data_height,TRUE);

      /* Dimension en Pixel du Control Group Box + Resize */  
      hwnd_group = GetDlgItem(window_batteryram->hwnd_batteryram,IDC_GROUP_BATTERYRAM);     /* Rcupre le Handle du Control GroupBox li  IDC_GROUP_RAMSOUND */
      window_batteryram->group_data_width = WINDOW_GROUP_BORDER_LEFT_SIZE + window_batteryram->batteryram_data_width + WINDOW_GROUP_BORDER_RIGHT_SIZE;
      window_batteryram->group_data_height = WINDOW_GROUP_BORDER_TOP_SIZE + window_batteryram->batteryram_data_height + WINDOW_GROUP_BORDER_BOTTOM_SIZE;
      MoveWindow(hwnd_group,window_batteryram->group_data_x,window_batteryram->group_data_y,window_batteryram->group_data_width,window_batteryram->group_data_height,TRUE);

      /* Retaille la fentre */
      ResizeBatteryRamWindow(window_batteryram);
    }

  /* Adresse des donnes */
  address_data = window_batteryram->batteryram_address;

  /** Cration d'un buffer RTF rpentant les donnes **/
  window_batteryram->rtf_data_hexadecimal[0] = '\0';
  strcat(window_batteryram->rtf_data_hexadecimal,"{\\rtf1\\ansi\\ansicpg1252\\deff0\\deflang1036{\\fonttbl{\\f0\\fnil\\fcharset0 Courier New;}}\r\n");
  strcat(window_batteryram->rtf_data_hexadecimal,"{\\colortbl ;\\red51\\green153\\blue255;}\r\n");
  strcat(window_batteryram->rtf_data_hexadecimal,"{\\*\\generator Msftedit 5.41.21.2510;}\\viewkind4\\uc1\\pard\\sl240\\slmult1\\lang12\\f0\\fs");
  strcat(window_batteryram->rtf_data_hexadecimal,BATTERYRAM_RICHEDIT_FONT_HEIGHT_RTF);  /* Taille de la Police : Si 8 dans WordPad => 16 ici (x2)*/
  strcat(window_batteryram->rtf_data_hexadecimal," ");
  offset_rtf = (int) strlen(window_batteryram->rtf_data_hexadecimal);

  /*** Transformer les donnes BYTE en Hexa Ascii ***/
  nb_byte_to_process = window_batteryram->nb_column*window_batteryram->nb_row;
  for(i=0,offset=0; i<window_batteryram->nb_row; i++,offset+=window_batteryram->nb_column)
    {
      /* Dbut de ligne : Adresse */
      sprintf(&window_batteryram->rtf_data_hexadecimal[offset_rtf],"%02X   ",(offset+address_data) & 0xFFFF);
      offset_rtf += (int) strlen("00   ");

      /** Encode les Byte en Hexa **/
      for(j=0; j<window_batteryram->nb_column; j++)
        {
          /* On encode la valeur */
          if(i*window_batteryram->nb_column+j < nb_byte_to_process)
            {
              /* Byte en noir */
              sprintf(&window_batteryram->rtf_data_hexadecimal[offset_rtf],"%02X",window_batteryram->batteryram_data[offset+j]);
              offset_rtf += 2;
            }
          else
            {
              /* Aucune donne  afficher */
              strcpy(&window_batteryram->rtf_data_hexadecimal[offset_rtf],"  ");
              offset_rtf += 2;
            }

          /* Sparateur */
          if(j < (window_batteryram->nb_column-1))
            {
              window_batteryram->rtf_data_hexadecimal[offset_rtf] = (j==7 && (i*window_batteryram->nb_column+j < nb_byte_to_process)) ? '.' : ' ';
              offset_rtf++;
            }
        }

      /* Fin de ligne */
      strcpy(&window_batteryram->rtf_data_hexadecimal[offset_rtf],"\\par\r\n");
      offset_rtf += (int) strlen("\\par\r\n");
    }
    
  /** Fin du RTF **/
  strcpy(&window_batteryram->rtf_data_hexadecimal[offset_rtf],"}\r\n");
  offset_rtf += (int) strlen("}\r\n");
  window_batteryram->rtf_data_hexadecimal[offset_rtf] = '\0';
  
  /* Taille du RTF (le \0 final compte) */
  window_batteryram->rtf_data_size = (int) strlen(window_batteryram->rtf_data_hexadecimal) + 1;    
    
  /** Afficher le rsultat en RTF dans le control Rich Edit **/
  window_batteryram->rtf_write_offset = 0;
  memset(&edit_stream,0,sizeof(EDITSTREAM));
  edit_stream.pfnCallback = (EDITSTREAMCALLBACK) BatteryRamRichEditStreamCallback;
  edit_stream.dwCookie = (DWORD_PTR) window_batteryram;          /* Paramtre  passer  la fonction Callback */
  SendMessage(window_batteryram->hwnd_richedit_data,EM_STREAMIN,SF_RTF,(LPARAM)&edit_stream);
  
  /* Titre de la fentre */
  SetWindowText(window_batteryram->hwnd_batteryram,"Battery RAM : $00-$FF");
}


/****************************************************************************************/
/*  BatteryRamRichEditStreamCallback() :  Fonction Call back pour remplir le Rich Edit. */
/****************************************************************************************/
static DWORD CALLBACK BatteryRamRichEditStreamCallback(DWORD *dwCookie, LPBYTE output_buffer, LONG nb_write_max, LONG *nb_write_rtn)
{
  int nb_remain_byte, nb_copy_byte;
  struct window_batteryram *window_batteryram;

  /* Rcupration du paramtre */
  window_batteryram = (struct window_batteryram *) dwCookie;

  /* Taille des donnes encore  copier */
  nb_remain_byte = window_batteryram->rtf_data_size - window_batteryram->rtf_write_offset;
  if(nb_remain_byte == 0)
    {
      /* Plus rien  copier */
      *nb_write_rtn = 0;
      return(0);
    }

  /* Taille des donnes que l'on est autoris  copier */
  nb_copy_byte = (nb_write_max >= nb_remain_byte) ? nb_remain_byte : nb_write_max;

  /* Nombre de byte transfrs */
  *nb_write_rtn = nb_copy_byte;
		  
  /** Copie les Byte dans le output buffer **/
  memcpy(output_buffer,&window_batteryram->rtf_data_hexadecimal[window_batteryram->rtf_write_offset],nb_copy_byte);

  /* Dplace l'offset */
  window_batteryram->rtf_write_offset += nb_copy_byte;

  /* OK */
  return(0);
}


/************************************************************************/
/*  DeleteBatteryRamChildWindows() :  Destruction des fentres enfants. */
/************************************************************************/
static void DeleteBatteryRamChildWindows(struct window_batteryram *window_batteryram)
{
  /** Libration des ressources **/  
  if(window_batteryram->rtf_data_hexadecimal != NULL)
    free(window_batteryram->rtf_data_hexadecimal);
  window_batteryram->rtf_data_hexadecimal = NULL;

  if(window_batteryram->batteryram_data != NULL)
    free(window_batteryram->batteryram_data);
  window_batteryram->batteryram_data = NULL;
}


/**************************************************************************/
/*  mem_free_window_batteryram() :  Libration mmoire pour la structure. */
/**************************************************************************/
void mem_free_window_batteryram(struct window_batteryram *window_batteryram)
{
  if(window_batteryram)
    {
      /* Supprime la notification */
      my_Notification(NOTIFICATION_REMOVE,window_batteryram->current_notification,NULL);
      window_batteryram->current_notification = NULL;
      
      free(window_batteryram);
    }
}

/**********************************************************************/
