/********************************************************************/
/*                                                                  */
/*  Window_Compare.c :  Module de gestion de la fentre Compare.    */
/*                                                                  */
/********************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe  *  Date : Jan 2019  */
/********************************************************************/

#include <stdio.h>
#include <time.h>

#include <windows.h>
#include <windowsx.h>
#include <Commctrl.h>
#include <Winuser.h>
#include <Richedit.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_MMU.h"
#include "Dc_Snapshot.h"
#include "Dc_Notification.h"
#include "Dc_Graphic.h"
#include "Dc_CPU.h"
#include "Window_QuickWatch.h"
#include "Window_Main.h"
#include "Window_Main_Menu.h"
#include "Window_Compare.h"

static DWORD BuildCompareWindowThread(void *);
LRESULT CALLBACK ProcHandlerCompareWindow(HWND,UINT,WPARAM,LPARAM);
static int CreateCompareChildWindows(HWND,struct window_compare *);
LRESULT CALLBACK subEditCompareProcControl(HWND,UINT,WPARAM,LPARAM);
static void UpdateCompareRamType1(struct window_compare *,int);
static void UpdateCompareRamType2(struct window_compare *,int);
static void CompareData(struct window_compare *);
static void DisplayCompareResult(struct window_compare *,int);
static DWORD CALLBACK CompareRichEditStreamCallback(DWORD *,LPBYTE,LONG,LONG *);
static void DeleteCompareChildWindows(struct window_compare *);
static void BuildCompareFromData(struct window_compare *,struct compare_result *);
static void EmptyCompareResult(struct window_compare *);
static struct compare_result *mem_alloc_compareresult(int,int,int,int);
static void mem_free_all_compareresult(struct compare_result *);

/****************************************************************************/
/*  BuildCompareWindow() :  Construction / Affichage de la fentre Compare. */
/****************************************************************************/
struct window_compare *BuildCompareWindow(HINSTANCE hInstance, int menu_id)
{
  struct window_compare *window_compare;

  /** Allocation de la structure window_compare **/
  window_compare = (struct window_compare *) calloc(1,sizeof(struct window_compare));
  if(window_compare == NULL)
    return(NULL);
  my_Memory(MEMORY_ADD_WINDOW_COMPARE,window_compare,NULL);

  /* Lance un thread pour grer la fentre Compare */
  CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)BuildCompareWindowThread,window_compare,0,NULL);

  /* Renvoi la structure mmoire grant la fentre */
  return(window_compare);
}


/**************************************************************************************/
/*  BuildCompareWindowThread() :  Cration de la fentre Compare via un Thread ddi. */
/**************************************************************************************/
static DWORD BuildCompareWindowThread(void *window_compare_param)
{
  HWND hwndCompare;
  MSG msg;
  BOOL bRet;
  struct window_main *window_main;
  struct window_compare *window_compare;
  struct parameter *param;
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Init */
  window_compare = (struct window_compare *) window_compare_param;
  window_compare->thread_id = GetCurrentThreadId();         /* Le ThreadID sert  diffrencier les diffrentes fentres */

  /** Cration de la fentre Compare **/
  hwndCompare = CreateDialog(param->hInstance,MAKEINTRESOURCE(IDD_WINDOW_COMPARE),window_main->hwnd_main,(DLGPROC)ProcHandlerCompareWindow);
  if(hwndCompare == NULL)
    return(1);

  /** Show Window + Paint Content **/
  ShowWindow(hwndCompare,SW_SHOWDEFAULT);
  UpdateWindow(hwndCompare);

  /** Message Loop pour toutes les fentres de ce Thread (fentre Compare...) **/
  while((bRet = GetMessage(&msg,NULL,0,0)) != 0)
    {
      if(bRet == -1)
        break;

      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }

  /* Fin du Thread */
  ExitThread(0);
}


/************************************************************************/
/*  ProcHandlerCompareWindow() :  Gestion des vnements de la fentre. */
/************************************************************************/
LRESULT CALLBACK ProcHandlerCompareWindow(HWND hwndCompare, UINT umsg, WPARAM wParam, LPARAM lParam)
{
  struct window_compare *window_compare;
  struct parameter *param;
  
  /* Init */
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  my_Memory(MEMORY_GET_WINDOW_COMPARE,&window_compare,NULL);

  switch(umsg)
    {
      case WM_INITDIALOG :
        /* Cration du contenu de la DialogBox */
        CreateCompareChildWindows(hwndCompare,window_compare);
        return(TRUE);

      /****************************/
      /* Evnements de la fentre */
      /****************************/
      case WM_COMMAND :
        switch(LOWORD(wParam))
          {
            /** Bouton Radio : On change la zone #1 des donnes (Ram ou Doc Ram) **/
            case IDC_RADIO_RAM1 :
              if(HIWORD(wParam) == BN_CLICKED && window_compare->ram_type_1 != COMPARE_RAMTYPE_RAM)
                UpdateCompareRamType1(window_compare,COMPARE_RAMTYPE_RAM);
              break;

            case IDC_RADIO_DOCRAM1 :
              if(HIWORD(wParam) == BN_CLICKED && window_compare->ram_type_1 != COMPARE_RAMTYPE_DOCRAM)
                UpdateCompareRamType1(window_compare,COMPARE_RAMTYPE_DOCRAM);
              break;

            /** Bouton Radio : On change la zone #2 des donnes (Ram ou Doc Ram) **/
            case IDC_RADIO_RAM2 :
              if(HIWORD(wParam) == BN_CLICKED && window_compare->ram_type_2 != COMPARE_RAMTYPE_RAM)
                UpdateCompareRamType2(window_compare,COMPARE_RAMTYPE_RAM);
              break;

            case IDC_RADIO_DOCRAM2 :
              if(HIWORD(wParam) == BN_CLICKED && window_compare->ram_type_2 != COMPARE_RAMTYPE_DOCRAM)
                UpdateCompareRamType2(window_compare,COMPARE_RAMTYPE_DOCRAM);
              break;
              
            /** Bouton pour lancer la comparaison **/
            case IDC_BUTTON_COMPARE :
              CompareData(window_compare);
              break;

            /** Boutons pour faire dfiler les rsultats **/
            case IDC_BUTTON_PREVIOUS :
              DisplayCompareResult(window_compare,-1);
              break;
            case IDC_BUTTON_NEXT :
              DisplayCompareResult(window_compare,1);
              break;
          }
        return(TRUE);

      /********************************************************************/
      /* Fermeture de window (le WM_CLOSE est appel avant le WM_DESTROY) */
      /********************************************************************/
      case WM_CLOSE :
        /* Fentre Compare */
        DestroyWindow(hwndCompare);
        return(0);

      case WM_DESTROY :
        /* Suppression des Fentres */
        DeleteCompareChildWindows(window_compare);

        /* Supprime la structure mmoire de ce thread */
        my_Memory(MEMORY_DELETE_WINDOW_COMPARE,window_compare,NULL);

        /* Demande la sortie de la boucle des messages */
        PostQuitMessage(0);
        return(0);
    }

  /* Retourner FALSE pour relayer le message non trait au systme */
  return(FALSE);
}


/*****************************************************************************************************/
/*  CreateCompareChildWindows() : Crer les fentres enfants et remplir la structure window_compare. */
/*****************************************************************************************************/
static int CreateCompareChildWindows(HWND hwndCompare, struct window_compare *window_compare)
{
  HWND hwnd_group;
  DWORD dialog_background_color;
  RECT client_rect, window_rect, group_rect;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
    
  /** Surchage des Control Edit pour capter les RETURN lors des saisies **/
  window_compare->oldEditProcBank1 = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndCompare,IDC_EDIT_RAM_BANK1),GWLP_WNDPROC,(LONG_PTR)subEditCompareProcControl);
  window_compare->oldEditProcAddress1 = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndCompare,IDC_EDIT_RAM_ADDRESS1),GWLP_WNDPROC,(LONG_PTR)subEditCompareProcControl);
  window_compare->oldEditProcBank2 = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndCompare,IDC_EDIT_RAM_BANK2),GWLP_WNDPROC,(LONG_PTR)subEditCompareProcControl);
  window_compare->oldEditProcAddress2 = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndCompare,IDC_EDIT_RAM_ADDRESS2),GWLP_WNDPROC,(LONG_PTR)subEditCompareProcControl);
  window_compare->oldEditProcAreaLength = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndCompare,IDC_EDIT_AREA_LENGTH),GWLP_WNDPROC,(LONG_PTR)subEditCompareProcControl);

  /** Allocation Buffer mmoire Zone Texte **/
  window_compare->nb_line = 2; 
  window_compare->nb_column = 16;
  /* Nombre caractres visibles sur la ligne (on ne compte pas le \r\n final) */
  window_compare->text_data_width = (int) ((strlen("00/0000   ") + window_compare->nb_column*3 - 1));   /* Enlve le dernier sparateur */
  /* Nombre d'octet pour encoder la zone Texte (on compte les \r\n en fin de ligne) */
  window_compare->rtf_data_size = COMPARE_RICHEDIT_HEADER_SIZE;
  window_compare->rtf_data_size += (int) (((window_compare->text_data_width + strlen("\r\n"))*window_compare->nb_line)*COMPARE_RICHEDIT_MAX_CHAR_SIZE);
  window_compare->rtf_data_size += COMPARE_RICHEDIT_FOOTER_SIZE;
  window_compare->rtf_data_hexadecimal = (char *) calloc(window_compare->rtf_data_size+1,sizeof(char));
  if(window_compare->rtf_data_hexadecimal == NULL)
    return(1);

  /** Window Handle **/
  window_compare->hwnd_compare = hwndCompare;

  /* Hauteur Titre */
  GetWindowRect(window_compare->hwnd_compare,&window_rect);
  GetClientRect(window_compare->hwnd_compare,&client_rect);
  window_compare->title_height = (window_rect.bottom - window_rect.top) - client_rect.bottom;

  /* Position de la GroupBox Compare Result */
  hwnd_group = GetDlgItem(window_compare->hwnd_compare,IDC_GROUP_COMPARE);     /* Rcupre le Handle du Control GroupBox li  IDC_GROUP_COMPARE */
  GetWindowRect(hwnd_group,&group_rect);
  window_compare->group_data_x = group_rect.left - window_rect.left;
  window_compare->group_data_y = (group_rect.top - window_rect.top) - window_compare->title_height;

  /* Position de la Zone Text Direct Page Data dans la fentre */
  window_compare->compare_data_x = window_compare->group_data_x + WINDOW_GROUP_BORDER_LEFT_SIZE + 3;
  window_compare->compare_data_y = window_compare->group_data_y + WINDOW_GROUP_BORDER_TOP_SIZE + 34;

  /* Dimension en Pixel du Control Text Compare Data */
  window_compare->compare_data_width = window_compare->text_data_width*7 + 2;
  window_compare->compare_data_height = window_compare->nb_line*COMPARE_EDIT_FONT_HEIGHT;

  /** Cration du Rich Edit **/
  window_compare->hwnd_richedit_data = CreateWindowEx(0,RICHEDIT_CLASS,"",WS_CHILD | WS_VISIBLE | ES_SAVESEL | ES_MULTILINE | ES_READONLY, 
                                                     window_compare->compare_data_x, window_compare->compare_data_y, window_compare->compare_data_width, window_compare->compare_data_height,   // set size in WM_SIZE message 
                                                     window_compare->hwnd_compare,
                                                     (HMENU)IDC_RICHEDIT_COMPARE_DATA,
                                                     param->hInstance,
                                                     NULL);

  /* Couleur de Fond du control Rich Edit : Gris comme la fentre d'accueil */
  dialog_background_color = GetSysColor(COLOR_3DFACE);
  SendMessage(window_compare->hwnd_richedit_data,EM_SETBKGNDCOLOR,0,(LPARAM)dialog_background_color); 

  /* Ram Type #1 */
  window_compare->ram_type_1 = COMPARE_RAMTYPE_RAM;
  SendDlgItemMessage(window_compare->hwnd_compare,IDC_RADIO_RAM1,BM_SETCHECK,1,0);
  SendDlgItemMessage(window_compare->hwnd_compare,IDC_RADIO_DOCRAM1,BM_SETCHECK,0,0);

  /* Area Address #1 */
  window_compare->bank_1 = 0x01;
  window_compare->address_1 = 0x2000;
  SetWindowText(GetDlgItem(window_compare->hwnd_compare,IDC_EDIT_RAM_BANK1),"01");
  SetWindowText(GetDlgItem(window_compare->hwnd_compare,IDC_EDIT_RAM_ADDRESS1),"2000");

  /* Ram Type #2 */
  window_compare->ram_type_2 = COMPARE_RAMTYPE_RAM;
  SendDlgItemMessage(window_compare->hwnd_compare,IDC_RADIO_RAM2,BM_SETCHECK,1,0);
  SendDlgItemMessage(window_compare->hwnd_compare,IDC_RADIO_DOCRAM2,BM_SETCHECK,0,0);

  /* Area Address #2 */
  window_compare->bank_2 = 0xE1;
  window_compare->address_2 = 0x2000;
  SetWindowText(GetDlgItem(window_compare->hwnd_compare,IDC_EDIT_RAM_BANK2),"E1");
  SetWindowText(GetDlgItem(window_compare->hwnd_compare,IDC_EDIT_RAM_ADDRESS2),"2000");

  /* Area Length */
  window_compare->area_length = 32768;
  SetWindowText(GetDlgItem(window_compare->hwnd_compare,IDC_EDIT_AREA_LENGTH),"0x8000");
    
  /* OK */
  return(0);
}


/****************************************************************************************/
/*  subEditCompareProcControl() :  Rcupre le RETURN pour valider les donnes saisies. */
/****************************************************************************************/
LRESULT CALLBACK subEditCompareProcControl(HWND hwnd_control, UINT msg, WPARAM wParam, LPARAM lParam)
{
  int index;
  struct window_compare *window_compare;
  my_Memory(MEMORY_GET_WINDOW_COMPARE,&window_compare,NULL);

  /* Protection */
  if(window_compare == NULL)
    return(0);

  /** Identifie le Control **/
  if(GetDlgItem(window_compare->hwnd_compare,IDC_EDIT_RAM_BANK1) == hwnd_control)
    index = 1;
  else if(GetDlgItem(window_compare->hwnd_compare,IDC_EDIT_RAM_ADDRESS1) == hwnd_control)
    index = 2;
  else if(GetDlgItem(window_compare->hwnd_compare,IDC_EDIT_RAM_BANK2) == hwnd_control)
    index = 3;
  else if(GetDlgItem(window_compare->hwnd_compare,IDC_EDIT_RAM_ADDRESS2) == hwnd_control)
    index = 4;
  else if(GetDlgItem(window_compare->hwnd_compare,IDC_EDIT_AREA_LENGTH) == hwnd_control)
    index = 5;
  else
    return(0);      /* Pas trouv */

  /** Dtecte la touche RETURN pour valider la donne **/
  if(msg == WM_CHAR && wParam == VK_RETURN)
    {
      /* Enlve le focus de l'Edit */
      SetFocus(window_compare->hwnd_compare);
    }
  else if(index == 1)
    return(CallWindowProc(window_compare->oldEditProcBank1,hwnd_control,msg,wParam,lParam));
  else if(index == 2)
    return(CallWindowProc(window_compare->oldEditProcAddress1,hwnd_control,msg,wParam,lParam));
  else if(index == 3)
    return(CallWindowProc(window_compare->oldEditProcBank2,hwnd_control,msg,wParam,lParam));
  else if(index == 4)
    return(CallWindowProc(window_compare->oldEditProcAddress2,hwnd_control,msg,wParam,lParam));
  else if(index == 5)
    return(CallWindowProc(window_compare->oldEditProcAreaLength,hwnd_control,msg,wParam,lParam));

  /* OK */
  return(0);
}


/**********************************************************************************/
/*  UpdateCompareRamType1() :  Change le type de la RAM contenant les donnes #1. */
/**********************************************************************************/
static void UpdateCompareRamType1(struct window_compare *window_compare, int ram_type_1)
{
  if(ram_type_1 == COMPARE_RAMTYPE_RAM)
    {
      /* Visuel */
      window_compare->ram_type_1 = COMPARE_RAMTYPE_RAM;
      SendDlgItemMessage(window_compare->hwnd_compare,IDC_RADIO_RAM1,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_compare->hwnd_compare,IDC_RADIO_DOCRAM1,BM_SETCHECK,0,0);

      /* Enable BANK */
      EnableWindow(GetDlgItem(window_compare->hwnd_compare,IDC_EDIT_RAM_BANK1),TRUE);
    }
  else
    {
      /* Visuel */
      window_compare->ram_type_1 = COMPARE_RAMTYPE_DOCRAM;
      SendDlgItemMessage(window_compare->hwnd_compare,IDC_RADIO_RAM1,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_compare->hwnd_compare,IDC_RADIO_DOCRAM1,BM_SETCHECK,1,0);

      /* Disable BANK */
      EnableWindow(GetDlgItem(window_compare->hwnd_compare,IDC_EDIT_RAM_BANK1),FALSE);
    }
}


/**********************************************************************************/
/*  UpdateCompareRamType2() :  Change le type de la RAM contenant les donnes #2. */
/**********************************************************************************/
static void UpdateCompareRamType2(struct window_compare *window_compare, int ram_type_2)
{
  if(ram_type_2 == COMPARE_RAMTYPE_RAM)
    {
      /* Visuel */
      window_compare->ram_type_2 = COMPARE_RAMTYPE_RAM;
      SendDlgItemMessage(window_compare->hwnd_compare,IDC_RADIO_RAM2,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_compare->hwnd_compare,IDC_RADIO_DOCRAM2,BM_SETCHECK,0,0);

      /* Enable BANK */
      EnableWindow(GetDlgItem(window_compare->hwnd_compare,IDC_EDIT_RAM_BANK2),TRUE);
    }
  else
    {
      /* Visuel */
      window_compare->ram_type_2 = COMPARE_RAMTYPE_DOCRAM;
      SendDlgItemMessage(window_compare->hwnd_compare,IDC_RADIO_RAM2,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_compare->hwnd_compare,IDC_RADIO_DOCRAM2,BM_SETCHECK,1,0);

      /* Disable BANK */
      EnableWindow(GetDlgItem(window_compare->hwnd_compare,IDC_EDIT_RAM_BANK2),FALSE);
    }
}


/******************************************************************/
/*  CompareData() :  Compare les donnes dans les zones mmoires. */
/******************************************************************/
static void CompareData(struct window_compare *window_compare)
{
  unsigned char one_byte_1;
  unsigned char one_byte_2;
  int i, bank, address, bank_address1, bank_address2, is_error, area_length;
  char value_buffer[1024];
  struct compare_result *current_compareresult;
  struct mmu *current_mmu;
    
  /* Rcupre le MMU */
  my_Memory(MEMORY_GET_S65816_MMU,&current_mmu,NULL);
    
  /* Libration mmoire */
  mem_free_all_compareresult(window_compare->first_compareresult);
  window_compare->nb_found = 0;
  window_compare->current_result = 0;
  window_compare->first_compareresult = NULL;
  window_compare->last_compareresult = NULL;
  
  /* RAZ de l'affichage */
  DisplayCompareResult(window_compare,0);
    
  /** Rcupre les critres de comparaison **/
  /** Addresse Mmoire #1 **/
  if(window_compare->ram_type_1 == COMPARE_RAMTYPE_RAM)
    {
      /* Rcupre l'adresse cible */
      GetWindowText(GetDlgItem(window_compare->hwnd_compare,IDC_EDIT_RAM_BANK1),value_buffer,1024-1);
      bank = EvaluateAsByte(value_buffer,&is_error);
      GetWindowText(GetDlgItem(window_compare->hwnd_compare,IDC_EDIT_RAM_ADDRESS1),value_buffer,1024-1);
      address = EvaluateAsWord(value_buffer,&is_error);
      bank_address1 = (bank << 16) | address;
    }
  else   /* Doc RAM */
    {
      /* Rcupre l'adresse DOC cible */
      GetWindowText(GetDlgItem(window_compare->hwnd_compare,IDC_EDIT_RAM_ADDRESS1),value_buffer,1024-1);
      bank_address1 = EvaluateAsWord(value_buffer,&is_error);
    }

  /** Addresse Mmoire #2 **/
  if(window_compare->ram_type_2 == COMPARE_RAMTYPE_RAM)
    {
      /* Rcupre l'adresse cible */
      GetWindowText(GetDlgItem(window_compare->hwnd_compare,IDC_EDIT_RAM_BANK2),value_buffer,1024-1);
      bank = EvaluateAsByte(value_buffer,&is_error);
      GetWindowText(GetDlgItem(window_compare->hwnd_compare,IDC_EDIT_RAM_ADDRESS2),value_buffer,1024-1);
      address = EvaluateAsWord(value_buffer,&is_error);
      bank_address2 = (bank << 16) | address;
    }
  else   /* Doc RAM */
    {
      /* Rcupre l'adresse DOC cible */
      GetWindowText(GetDlgItem(window_compare->hwnd_compare,IDC_EDIT_RAM_ADDRESS2),value_buffer,1024-1);
      bank_address2 = EvaluateAsWord(value_buffer,&is_error);
    }

  /** Length de la Zone **/
  area_length = EvaluateAsOffsetOrLength(window_compare->hwnd_compare,IDC_EDIT_AREA_LENGTH);
  if(area_length == 0)
    {
      MessageBox(window_compare->hwnd_compare,"Error, The Area length is Zero !","Compare Data Error",MB_OK|MB_ICONWARNING);
      return;
    }
      
  /** Compare les octets dans la zone mmoire **/
  for(i=0; i<area_length; i++)
    {
      /** Rcupre les data en mmoire **/
      one_byte_1 = (window_compare->ram_type_1 == COMPARE_RAMTYPE_RAM) ? ReadMemoryByte(bank_address1+i,current_mmu) : ReadDOCRAMByte(bank_address1+i,current_mmu);
      one_byte_2 = (window_compare->ram_type_2 == COMPARE_RAMTYPE_RAM) ? ReadMemoryByte(bank_address2+i,current_mmu) : ReadDOCRAMByte(bank_address2+i,current_mmu);

      /* Comparaison */
      if(one_byte_1 != one_byte_2)
        {
          /* Allocation mmoire */
          current_compareresult = mem_alloc_compareresult(window_compare->ram_type_1,window_compare->ram_type_2,bank_address1+i,bank_address2+i);
          if(current_compareresult == NULL)
            continue;

          /* Attachement */
          if(window_compare->first_compareresult == NULL)
            window_compare->first_compareresult = current_compareresult;
          else
            window_compare->last_compareresult->next = current_compareresult;
          window_compare->last_compareresult = current_compareresult;
          window_compare->nb_found++;
        }
    }
       
  /** On met en place le premier rsultat de la comparaison **/
  DisplayCompareResult(window_compare,0);
}


/************************************************************************************/
/*  mem_alloc_compareresult() :  Allocation mmoire de la structure compare_result. */
/************************************************************************************/
static struct compare_result *mem_alloc_compareresult(int compare_area1, int compare_area2, int memory_address1, int memory_address2)
{
  struct compare_result *current_compareresult;
  
  /* Allocation mmoire */
  current_compareresult = (struct compare_result *) calloc(1,sizeof(struct compare_result));
  if(current_compareresult == NULL)
    return(NULL);
    
  /* Remplissage */
  current_compareresult->compare_area1 = compare_area1;
  current_compareresult->compare_area2 = compare_area2;

  current_compareresult->bank1 = memory_address1 >> 16;
  current_compareresult->address1 = memory_address1 & 0xFFFF;
  current_compareresult->bank2 = memory_address2 >> 16;
  current_compareresult->address2 = memory_address2 & 0xFFFF;

  /* Renvoi la structure */
  return(current_compareresult);
}


/***************************************************************************************/
/*  mem_free_all_compareresult() :  Libration mmoire de la structure compare_result. */
/***************************************************************************************/
static void mem_free_all_compareresult(struct compare_result *first_compareresult)
{
  struct compare_result *current_compareresult;
  struct compare_result *next_compareresult;
  
  for(current_compareresult = first_compareresult; current_compareresult; current_compareresult = next_compareresult)
    {
      next_compareresult = current_compareresult->next;
      mem_free_compareresult(current_compareresult);
    }
}


/***********************************************************************************/
/*  mem_free_compareresult() :  Libration mmoire de la structure compare_result. */
/***********************************************************************************/
void mem_free_compareresult(struct compare_result *current_compareresult)
{
  if(current_compareresult)
    {       
      free(current_compareresult);
    }
}


/******************************************************************/
/*  DisplayCompareResult() :  Affiche les rsultats de recherche. */
/******************************************************************/
static void DisplayCompareResult(struct window_compare *window_compare, int direction)
{
  int i;
  struct compare_result *current_compareresult;
  
  /** Premier affichage aprs une recherche **/
  if(direction == 0)
    {
      /* Rien trouv */
      if(window_compare->nb_found == 0)
        {
          /* Rien  afficher */
          window_compare->current_result = 0;
          
          /* Dsactive les 2 boutons Compare Results */
          EnableWindow(GetDlgItem(window_compare->hwnd_compare,IDC_BUTTON_PREVIOUS),FALSE);
          EnableWindow(GetDlgItem(window_compare->hwnd_compare,IDC_BUTTON_NEXT),FALSE);
          
          /* Construit les rsultats */
          SetWindowText(GetDlgItem(window_compare->hwnd_compare,IDC_STATIC_RESULT),"0");
          EmptyCompareResult(window_compare);
        }
      else if(window_compare->nb_found == 1)
        {
          /* Affiche la premire */
          window_compare->current_result = 1;
          
          /* Dsactive les 2 boutons Compare Results */
          EnableWindow(GetDlgItem(window_compare->hwnd_compare,IDC_BUTTON_PREVIOUS),FALSE);
          EnableWindow(GetDlgItem(window_compare->hwnd_compare,IDC_BUTTON_NEXT),FALSE);
          
          /* Construit les rsultats */
          BuildCompareFromData(window_compare,window_compare->first_compareresult);
        }
      else
        {
          /* Affiche la premire */
          window_compare->current_result = 1;

          /* Active les 2 boutons Compare Results */
          EnableWindow(GetDlgItem(window_compare->hwnd_compare,IDC_BUTTON_PREVIOUS),TRUE);
          EnableWindow(GetDlgItem(window_compare->hwnd_compare,IDC_BUTTON_NEXT),TRUE);
                    
          /* Construit les rsultats */
          BuildCompareFromData(window_compare,window_compare->first_compareresult);        
        }
    }
  else if(direction == -1)  /* Previous */
    {
      /* Borne infrieure */
      if(window_compare->current_result == 1)
        return;

      /* Affiche la precdente */
      window_compare->current_result -= 1;
                
      /* Construit les rsultats */
      for(i=1, current_compareresult=window_compare->first_compareresult; i<=window_compare->nb_found; i++, current_compareresult=current_compareresult->next)
        if(i == window_compare->current_result)
          break;
      BuildCompareFromData(window_compare,current_compareresult);          
    }
  else if(direction == 1)   /* Next */
    {
      /* Borne supprieure */
      if(window_compare->current_result == window_compare->nb_found)
        return;

      /* Affiche la suivante */
      window_compare->current_result += 1;
                
      /* Construit les rsultats */
      for(i=1, current_compareresult=window_compare->first_compareresult; i<=window_compare->nb_found; i++, current_compareresult=current_compareresult->next)
        if(i == window_compare->current_result)
          break;      
      BuildCompareFromData(window_compare,current_compareresult);      
    }
}


/*********************************************************************************************************/
/*  BuildCompareFromData() :  Construire les donnes hxadcimal  afficher  partir des donnes brutes. */
/*********************************************************************************************************/
static void BuildCompareFromData(struct window_compare *window_compare, struct compare_result *current_compareresult)
{
  EDITSTREAM edit_stream;
  int j, offset_rtf;
  int compare_data_address1, compare_data_address2, compare_data_value, compare_data_sep;
  char buffer[1024];
  unsigned char data1[256];
  unsigned char data2[256];
  struct mmu *current_mmu;
    
  /* Rcupre le MMU */
  my_Memory(MEMORY_GET_S65816_MMU,&current_mmu,NULL);
  
  /* Numro de la recherche */
  sprintf(buffer,"%d / %d",window_compare->current_result,window_compare->nb_found);
  SetWindowText(GetDlgItem(window_compare->hwnd_compare,IDC_STATIC_RESULT),buffer);

  /* Zone de recherche des donnes */
  compare_data_address1 = (current_compareresult->bank1 << 16) | current_compareresult->address1;
  compare_data_address2 = (current_compareresult->bank2 << 16) | current_compareresult->address2;

  /** Rcupre les donnes #1 **/
  if(current_compareresult->compare_area1 == COMPARE_RAMTYPE_DOCRAM)
    ReadDOCRAMBlock(compare_data_address1,16,data1,current_mmu);
  else
    ReadMemoryBlock(compare_data_address1,16,data1,current_mmu);
  data1[16] = '\0';

  /** Rcupre les donnes #2 **/
  if(current_compareresult->compare_area2 == COMPARE_RAMTYPE_DOCRAM)
    ReadDOCRAMBlock(compare_data_address2,16,data2,current_mmu);
  else
    ReadMemoryBlock(compare_data_address2,16,data2,current_mmu);
  data2[16] = '\0';

  /******************************************************/
  /*** Cration d'un buffer RTF rpentant les donnes ***/
  window_compare->rtf_data_hexadecimal[0] = '\0';
  strcat(window_compare->rtf_data_hexadecimal,"{\\rtf1\\ansi\\ansicpg1252\\deff0\\deflang1036{\\fonttbl{\\f0\\fnil\\fcharset0 Courier New;}}\r\n");
  strcat(window_compare->rtf_data_hexadecimal,"{\\colortbl ;\\red51\\green153\\blue255;\\red153\\green153\\blue153;\\red0\\green176\\blue80;\\red192\\green80\\blue77;\\red128\\green128\\blue128;}\r\n");
  strcat(window_compare->rtf_data_hexadecimal,"{\\*\\generator Msftedit 5.41.21.2510;}\\viewkind4\\uc1\\pard\\sl240\\slmult1\\lang12\\f0\\fs");
  strcat(window_compare->rtf_data_hexadecimal,COMPARE_RICHEDIT_FONT_HEIGHT_RTF);  /* Taille de la Police : Si 8 dans WordPad => 16 ici (x2)*/
  strcat(window_compare->rtf_data_hexadecimal," ");
  offset_rtf = (int) strlen(window_compare->rtf_data_hexadecimal);

  /*** Transformer les donnes BYTE en Hexa ***/
  /** Area #1 **/
  /* Adresse */
  if(current_compareresult->compare_area1 == COMPARE_RAMTYPE_DOCRAM)
    sprintf(&window_compare->rtf_data_hexadecimal[offset_rtf],"DOC/%04X  ",(compare_data_address1)&0xFFFF);      
  else
    sprintf(&window_compare->rtf_data_hexadecimal[offset_rtf],"%02X/%04X   ",(compare_data_address1)>>16,(compare_data_address1)&0xFFFF);
  offset_rtf += (int) strlen("00/0000   ");

  /** Data Hexa **/
  for(j=0; j<16; j++)
    {
      /* Les valeurs sont t'elles diffrentes ? */
      if(data1[j] == data2[j])
        compare_data_value = 0;
      else
        compare_data_value = ((j == 0) ? 1 : 2);
      compare_data_sep = (data1[j+1] == data2[j+1]) ? 0 : 1;

      /* Insre le code couleur de slection */
      if(compare_data_value == 1)
        {
          strcpy(&window_compare->rtf_data_hexadecimal[offset_rtf],"\\highlight1 ");
          offset_rtf += (int) strlen("\\highlight1 ");
        }
      else if(compare_data_value == 2)
        {
          strcpy(&window_compare->rtf_data_hexadecimal[offset_rtf],"\\highlight2 ");
          offset_rtf += (int) strlen("\\highlight2 ");
        }
        
      /* Byte Hexa Ascii */
      sprintf(&window_compare->rtf_data_hexadecimal[offset_rtf],"%02X",data1[j]);
      offset_rtf += 2;

      /* Supprime la couleur de slection */
      if(compare_data_value == 1 || compare_data_value == 2)
        {
          strcpy(&window_compare->rtf_data_hexadecimal[offset_rtf],"\\highlight0 ");
          offset_rtf += (int) strlen("\\highlight0 ");
        }

      /* Insre le code couleur de slection */
      if((compare_data_value == 1 || compare_data_value == 2) && compare_data_sep == 1)
        {
          strcpy(&window_compare->rtf_data_hexadecimal[offset_rtf],"\\highlight2 ");
          offset_rtf += (int) strlen("\\highlight2 ");
        }

      /* Sparateur */
      if(j < (16-1))
        {
          window_compare->rtf_data_hexadecimal[offset_rtf] = ((j==7) ? '.' : ' ');
          offset_rtf++;
        }

      /* Supprime la couleur de slection */
      if((compare_data_value == 1 || compare_data_value == 2) && compare_data_sep == 1)
        {
          strcpy(&window_compare->rtf_data_hexadecimal[offset_rtf],"\\highlight0 ");
          offset_rtf += (int) strlen("\\highlight0 ");
        }
    }

  /* Fin de ligne */
  strcpy(buffer,"\\par\r\n");
  strcpy(&window_compare->rtf_data_hexadecimal[offset_rtf],buffer);
  offset_rtf += (int) strlen(buffer);

  /** Area #2 **/
  /* Adresse */
  if(current_compareresult->compare_area2 == COMPARE_RAMTYPE_DOCRAM)
    sprintf(&window_compare->rtf_data_hexadecimal[offset_rtf],"DOC/%04X  ",(compare_data_address2)&0xFFFF);      
  else
    sprintf(&window_compare->rtf_data_hexadecimal[offset_rtf],"%02X/%04X   ",(compare_data_address2)>>16,(compare_data_address2)&0xFFFF);
  offset_rtf += (int) strlen("00/0000   ");

  /** Data Hexa **/
  for(j=0; j<16; j++)
    {
      /* Les valeurs sont t'elles diffrentes ? */
      if(data1[j] == data2[j])
        compare_data_value = 0;
      else
        compare_data_value = ((j == 0) ? 1 : 2);
      compare_data_sep = (data1[j+1] == data2[j+1]) ? 0 : 1;

      /* Insre le code couleur de slection */
      if(compare_data_value == 1)
        {
          strcpy(&window_compare->rtf_data_hexadecimal[offset_rtf],"\\highlight1 ");
          offset_rtf += (int) strlen("\\highlight1 ");
        }
      else if(compare_data_value == 2)
        {
          strcpy(&window_compare->rtf_data_hexadecimal[offset_rtf],"\\highlight2 ");
          offset_rtf += (int) strlen("\\highlight2 ");
        }
        
      /* Byte Hexa Ascii */
      sprintf(&window_compare->rtf_data_hexadecimal[offset_rtf],"%02X",data2[j]);
      offset_rtf += 2;

      /* Supprime la couleur de slection */
      if(compare_data_value == 1 || compare_data_value == 2)
        {
          strcpy(&window_compare->rtf_data_hexadecimal[offset_rtf],"\\highlight0 ");
          offset_rtf += (int) strlen("\\highlight0 ");
        }

      /* Insre le code couleur de slection */
      if((compare_data_value == 1 || compare_data_value == 2) && compare_data_sep == 1)
        {
          strcpy(&window_compare->rtf_data_hexadecimal[offset_rtf],"\\highlight2 ");
          offset_rtf += (int) strlen("\\highlight2 ");
        }

      /* Sparateur */
      if(j < (16-1))
        {
          window_compare->rtf_data_hexadecimal[offset_rtf] = ((j==7) ? '.' : ' ');
          offset_rtf++;
        }

      /* Supprime la couleur de slection */
      if((compare_data_value == 1 || compare_data_value == 2) && compare_data_sep == 1)
        {
          strcpy(&window_compare->rtf_data_hexadecimal[offset_rtf],"\\highlight0 ");
          offset_rtf += (int) strlen("\\highlight0 ");
        }
    }

  /** Fin du RTF **/
  strcpy(&window_compare->rtf_data_hexadecimal[offset_rtf],"}\r\n");
  offset_rtf += (int) strlen("}\r\n");
  window_compare->rtf_data_hexadecimal[offset_rtf] = '\0';

  /* Taille du RTF (le \0 final compte) */
  window_compare->rtf_data_size = (int) strlen(window_compare->rtf_data_hexadecimal) + 1;

  /** Afficher le rsultat en RTF dans le control Rich Edit **/
  window_compare->rtf_write_offset = 0;
  memset(&edit_stream,0,sizeof(EDITSTREAM));
  edit_stream.pfnCallback = (EDITSTREAMCALLBACK) CompareRichEditStreamCallback;
  edit_stream.dwCookie = (DWORD_PTR) window_compare;          /* Paramtre  passer  la fonction Callback */
  SendMessage(window_compare->hwnd_richedit_data,EM_STREAMIN,SF_RTF,(LPARAM)&edit_stream);
}


/**********************************************************************************/
/*  EmptyCompareResult() :  Vide la zone Rich Text des rsultats de la recherche. */
/**********************************************************************************/
static void EmptyCompareResult(struct window_compare *window_compare)
{
  EDITSTREAM edit_stream;
  int offset_rtf;
  
  /******************************************************/
  /*** Cration d'un buffer RTF rpentant les donnes ***/
  window_compare->rtf_data_hexadecimal[0] = '\0';
  strcat(window_compare->rtf_data_hexadecimal,"{\\rtf1\\ansi\\ansicpg1252\\deff0\\deflang1036{\\fonttbl{\\f0\\fnil\\fcharset0 Courier New;}}\r\n");
  strcat(window_compare->rtf_data_hexadecimal,"{\\colortbl ;\\red255\\green0\\blue0;\\red0\\green77\\blue187;\\red0\\green176\\blue80;\\red192\\green80\\blue77;\\red128\\green128\\blue128;}\r\n");
  strcat(window_compare->rtf_data_hexadecimal,"{\\*\\generator Msftedit 5.41.21.2510;}\\viewkind4\\uc1\\pard\\sl240\\slmult1\\lang12\\f0\\fs");
  strcat(window_compare->rtf_data_hexadecimal,COMPARE_RICHEDIT_FONT_HEIGHT_RTF);  /* Taille de la Police : Si 8 dans WordPad => 16 ici (x2)*/
  strcat(window_compare->rtf_data_hexadecimal," ");
  offset_rtf = (int) strlen(window_compare->rtf_data_hexadecimal);  
  
  /** Fin du RTF **/
  strcpy(&window_compare->rtf_data_hexadecimal[offset_rtf],"}\r\n");
  offset_rtf += (int) strlen("}\r\n");
  window_compare->rtf_data_hexadecimal[offset_rtf] = '\0';

  /* Taille du RTF (le \0 final compte) */
  window_compare->rtf_data_size = (int) strlen(window_compare->rtf_data_hexadecimal) + 1;

  /** Afficher le rsultat en RTF dans le control Rich Edit **/
  window_compare->rtf_write_offset = 0;
  memset(&edit_stream,0,sizeof(EDITSTREAM));
  edit_stream.pfnCallback = (EDITSTREAMCALLBACK) CompareRichEditStreamCallback;
  edit_stream.dwCookie = (DWORD_PTR) window_compare;          /* Paramtre  passer  la fonction Callback */
  SendMessage(window_compare->hwnd_richedit_data,EM_STREAMIN,SF_RTF,(LPARAM)&edit_stream);  
}


/*************************************************************************************/
/*  CompareRichEditStreamCallback() :  Fonction Call back pour remplir le Rich Edit. */
/*************************************************************************************/
static DWORD CALLBACK CompareRichEditStreamCallback(DWORD *dwCookie, LPBYTE output_buffer, LONG nb_write_max, LONG *nb_write_rtn)
{
  int nb_remain_byte, nb_copy_byte;
  struct window_compare *window_compare;

  /* Rcupration du paramtre */
  window_compare = (struct window_compare *) dwCookie;

  /* Taille des donnes encore  copier */
  nb_remain_byte = window_compare->rtf_data_size - window_compare->rtf_write_offset;
  if(nb_remain_byte == 0)
    {
      /* Plus rien  copier */
      *nb_write_rtn = 0;
      return(0);
    }

  /* Taille des donnes que l'on est autoris  copier */
  nb_copy_byte = (nb_write_max >= nb_remain_byte) ? nb_remain_byte : nb_write_max;

  /* Nombre de byte transfrs */
  *nb_write_rtn = nb_copy_byte;
		  
  /** Copie les Byte dans le output buffer **/
  memcpy(output_buffer,&window_compare->rtf_data_hexadecimal[window_compare->rtf_write_offset],nb_copy_byte);

  /* Dplace l'offset */
  window_compare->rtf_write_offset += nb_copy_byte;

  /* OK */
  return(0);
}


/*********************************************************************/
/*  DeleteCompareChildWindows() :  Destruction des fentres enfants. */
/*********************************************************************/
static void DeleteCompareChildWindows(struct window_compare *window_compare)
{
  /** Libration des ressources **/  
  if(window_compare->rtf_data_hexadecimal != NULL)
    free(window_compare->rtf_data_hexadecimal);
  window_compare->rtf_data_hexadecimal = NULL;
}


/***********************************************************************/
/*  mem_free_window_compare() :  Libration mmoire pour la structure. */
/***********************************************************************/
void mem_free_window_compare(struct window_compare *window_compare)
{ 
  if(window_compare)
    {
      mem_free_all_compareresult(window_compare->first_compareresult);
           
      free(window_compare);
    }
}

/**********************************************************************/
