/**************************************************************************/
/*                                                                        */
/*  Window_DOCRegister.c :  Module de gestion de la fentre DOCRegister.  */
/*                                                                        */
/**************************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe  *  Date : Septembre 2018  */
/**************************************************************************/

#include <stdio.h>
#include <time.h>

#include <windows.h>
#include <windowsx.h>
#include <Commctrl.h>
#include <Winuser.h>
#include <Richedit.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_Notification.h"
#include "Dc_Graphic.h"
#include "Dc_CPU.h"
#include "Dc_MMU.h"
#include "Window_QuickWatch.h"
#include "Window_Main.h"
#include "Window_Main_Menu.h"
#include "Window_DOCRegister.h"

static DWORD BuildDOCRegisterWindowThread(void *);
LRESULT CALLBACK ProcHandlerDOCRegisterWindow(HWND,UINT,WPARAM,LPARAM);
static int CreateDOCRegisterChildWindows(HWND,struct window_docregister *);
static void UpdateDOCRegisterRefreshMode(struct window_docregister *,int);
static void RefreshDOCRegisterWindowData(struct window_docregister *,int);
static int IsDOCRegisterDataModified(struct window_docregister *);
static void BuildDOCRegisterFromData(struct window_docregister *,int);
static void DeleteDOCRegisterChildWindows(struct window_docregister *);


/************************************************************************************/
/*  BuildDOCRegisterWindow() :  Construction / Affichage de la fentre DOCRegister. */
/************************************************************************************/
struct window_docregister *BuildDOCRegisterWindow(HINSTANCE hInstance, int menu_id)
{
  struct window_docregister *window_docregister;

  /** Allocation de la structure window_docregister **/
  window_docregister = (struct window_docregister *) calloc(1,sizeof(struct window_docregister));
  if(window_docregister == NULL)
    return(NULL);
  my_Memory(MEMORY_SET_WINDOW_DOCREGISTER,window_docregister,NULL);

  /* Lance un thread pour grer la fentre DOCRegister */
  CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)BuildDOCRegisterWindowThread,window_docregister,0,NULL);

  /* Renvoi la structure mmoire grant la fentre */
  return(window_docregister);
}


/**********************************************************************************************/
/*  BuildDOCRegisterWindowThread() :  Cration de la fentre DOCRegister via un Thread ddi. */
/**********************************************************************************************/
static DWORD BuildDOCRegisterWindowThread(void *window_docregister_param)
{
  HWND hwndDOCRegister;
  MSG msg;
  BOOL bRet;
  struct window_main *window_main;
  struct window_docregister *window_docregister;
  struct parameter *param;
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Init */
  window_docregister = (struct window_docregister *) window_docregister_param;

  /** Cration de la fentre Direct Page **/
  hwndDOCRegister = CreateDialog(param->hInstance,MAKEINTRESOURCE(IDD_WINDOW_DOCREGISTER),window_main->hwnd_main,(DLGPROC)ProcHandlerDOCRegisterWindow);
  if(hwndDOCRegister == NULL)
    return(1);

  /** Show Window + Paint Content **/
  ShowWindow(hwndDOCRegister,SW_SHOWDEFAULT);
  UpdateWindow(hwndDOCRegister);

  /** Message Loop pour toutes les fentres de ce Thread (fentre DOCRegister...) **/
  while((bRet = GetMessage(&msg,NULL,0,0)) != 0)
    {
      if(bRet == -1)
        break;

      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }

  /* Fin du Thread */
  ExitThread(0);
}


/****************************************************************************/
/*  ProcHandlerDOCRegisterWindow() :  Gestion des vnements de la fentre. */
/****************************************************************************/
LRESULT CALLBACK ProcHandlerDOCRegisterWindow(HWND hwndDOCRegister, UINT umsg, WPARAM wParam, LPARAM lParam)
{
  struct window_docregister *window_docregister;
  struct parameter *param;
  
  /* Init */
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  my_Memory(MEMORY_GET_WINDOW_DOCREGISTER,&window_docregister,NULL);

  switch(umsg)
    {
      case WM_INITDIALOG :
        /* Cration du contenu de la DialogBox */
        CreateDOCRegisterChildWindows(hwndDOCRegister,window_docregister);
        return(TRUE);
        
      /****************************/
      /* Evnements de la fentre */
      /****************************/
      case WM_COMMAND :
        switch(LOWORD(wParam))
          {
            /** Bouton Radio : On change la mthode de rafraichissement de la fentre **/
            case IDC_RADIO_REFRESH_AUTOMATIC :
              if(HIWORD(wParam) == BN_CLICKED && window_docregister->refresh_mode != DOCREGISTER_REFRESH_AUTOMATIC)
                UpdateDOCRegisterRefreshMode(window_docregister,DOCREGISTER_REFRESH_AUTOMATIC);
              break;

            case IDC_RADIO_REFRESH_MANUAL :
              if(HIWORD(wParam) == BN_CLICKED && window_docregister->refresh_mode != DOCREGISTER_REFRESH_MANUAL)
                UpdateDOCRegisterRefreshMode(window_docregister,DOCREGISTER_REFRESH_MANUAL);
              break;

            /** Bouton de rafraichissement Manuel de la fentre **/
            case IDC_BUTTON_REFRESH :
              RefreshDOCRegisterWindowData(window_docregister,1);
              break;
          }
        return(TRUE);

      /*****************************************************************/
      /* Message extrieur demandant le rafraichissement de la fentre */
      /*****************************************************************/
      case HM_DOCREGISTER_REFRESHWINDOW :
        RefreshDOCRegisterWindowData(window_docregister,0);
        return(TRUE);

      /********************************************************************/
      /* Fermeture de window (le WM_CLOSE est appel avant le WM_DESTROY) */
      /********************************************************************/
      case WM_CLOSE :
        /* Fentre DOCRegister */
        DestroyWindow(hwndDOCRegister);
        return(0);

      case WM_DESTROY :
        /* Suppression des Fentres */
        DeleteDOCRegisterChildWindows(window_docregister);
        CheckMainWindowMenuItem(ID_MENU_VIEW_DOCREGISTER,FALSE);
        my_Memory(MEMORY_FREE_WINDOW_DOCREGISTER,NULL,NULL);
        /* Demande la sortie de la boucle des messages */
        PostQuitMessage(0);
        return(0);
    }

  /* Retourner FALSE pour relayer le message non trait au systme */
  return(FALSE);
}


/*************************************************************************************************************/
/*  CreateDOCRegisterChildWindows() : Crer les fentres enfants et remplir la structure window_docregister. */
/*************************************************************************************************************/
static int CreateDOCRegisterChildWindows(HWND hwndDOCRegister, struct window_docregister *window_docregister)
{
  RECT client_rect, window_rect;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Window Handle **/
  window_docregister->hwnd_docregister = hwndDOCRegister;

  /* Refresh Mode : Automatic */
  UpdateDOCRegisterRefreshMode(window_docregister,DOCREGISTER_REFRESH_AUTOMATIC);

  /* Hauteur Titre */
  GetWindowRect(window_docregister->hwnd_docregister,&window_rect);
  GetClientRect(window_docregister->hwnd_docregister,&client_rect);
  window_docregister->title_height = (window_rect.bottom - window_rect.top) - client_rect.bottom;

  /** Construction du Texte + Rafraichissement de la fentre **/
  RefreshDOCRegisterWindowData(window_docregister,1);
    
  /* OK */
  return(0);
}


/**************************************************************************************************/
/*  UpdateDOCRegisterRefreshMode() :  MAJ du mode de rafraichissement de la fenetre DOCRegister. */
/**************************************************************************************************/
static void UpdateDOCRegisterRefreshMode(struct window_docregister *window_docregister, int refresh_mode)
{
  struct notification *current_notification;

  if(refresh_mode == DOCREGISTER_REFRESH_AUTOMATIC)
    {
      /* Visuel */
      window_docregister->refresh_mode = DOCREGISTER_REFRESH_AUTOMATIC;
      SendDlgItemMessage(window_docregister->hwnd_docregister,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_docregister->hwnd_docregister,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,0,0);

      /* Cration d'une Notification */
      current_notification = (struct notification *) calloc(1,sizeof(struct notification));
      if(current_notification == NULL)
        return;
      current_notification->hwnd_window = window_docregister->hwnd_docregister;
      current_notification->message = HM_DOCREGISTER_REFRESHWINDOW;
      current_notification->type = NOTIFICATION_TYPE_VIEW;
      current_notification->sub_type = NOTIFICATION_SUBTYPE_DOCREGISTER;

      /* Ajout de la notification */
      window_docregister->current_notification = current_notification;
      my_Notification(NOTIFICATION_ADD,current_notification,NULL);
    }
  else
    {
      /* Visuel */
      window_docregister->refresh_mode = DOCREGISTER_REFRESH_MANUAL;
      SendDlgItemMessage(window_docregister->hwnd_docregister,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_docregister->hwnd_docregister,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,1,0);

      /* Supprime la Notification */
      my_Notification(NOTIFICATION_REMOVE,window_docregister->current_notification,NULL);
      window_docregister->current_notification = NULL;
    }
}


/********************************************************************************/
/*  RefreshDOCRegisterWindowData() :  MAJ du contenu de la fentre DOCRegister. */
/********************************************************************************/
static void RefreshDOCRegisterWindowData(struct window_docregister *window_docregister, int force)
{
  int is_modified;
    
  /** Les donnes ont t'elles t modifies + Rcupration des nouvelles Data si besoin **/
  is_modified = IsDOCRegisterDataModified(window_docregister);
  if(is_modified == 0 && force == 0)
    return;
    
  /** Cration de la Zone des DOC Registers de la fentre partir des nouvelles donnes **/
  BuildDOCRegisterFromData(window_docregister,force);

  /* Rafraichit la fentre */
  UpdateWindow(window_docregister->hwnd_docregister);
}


/***************************************************************************************************************/
/*  IsDOCRegisterDataModified() :  Les donnes DOCRegister ont t'elles t modifies depuis la dernirer fois. */
/***************************************************************************************************************/
static int IsDOCRegisterDataModified(struct window_docregister *window_docregister)
{
  unsigned char docregister_data[DOC_REGISTER_SIZE];
  struct mmu *current_mmu;

  /* Rcupre le MMU */
  my_Memory(MEMORY_GET_S65816_MMU,&current_mmu,NULL);
  
  /* Rcupre les donnes des DOC Register */
  ReadDOCRegisterBlock(0,DOC_REGISTER_SIZE,&docregister_data[0],current_mmu);
  
  /* Compare les donnes */
  if(memcmp(&docregister_data[0],&window_docregister->docregister_data[0],DOC_REGISTER_SIZE) == 0)
    return(0);
    
  /* Les donnes sont diffrentes */
  memcpy(&window_docregister->docregister_data[0],&docregister_data[0],DOC_REGISTER_SIZE);
  return(1);
}


/*************************************************************************************************************/
/*  BuildDOCRegisterFromData() :  Construire les donnes hexadcimal  afficher  partir des donnes brutes. */
/*************************************************************************************************************/
static void BuildDOCRegisterFromData(struct window_docregister *window_docregister, int force)
{ 
  /** Met  jour les 227 DOC Registers **/
  /* 00-1F */
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_00,window_docregister->docregister_data[0x00],window_docregister->docregister_data_old[0x00],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_01,window_docregister->docregister_data[0x01],window_docregister->docregister_data_old[0x01],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_02,window_docregister->docregister_data[0x02],window_docregister->docregister_data_old[0x02],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_03,window_docregister->docregister_data[0x03],window_docregister->docregister_data_old[0x03],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_04,window_docregister->docregister_data[0x04],window_docregister->docregister_data_old[0x04],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_05,window_docregister->docregister_data[0x05],window_docregister->docregister_data_old[0x05],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_06,window_docregister->docregister_data[0x06],window_docregister->docregister_data_old[0x06],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_07,window_docregister->docregister_data[0x07],window_docregister->docregister_data_old[0x07],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_08,window_docregister->docregister_data[0x08],window_docregister->docregister_data_old[0x08],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_09,window_docregister->docregister_data[0x09],window_docregister->docregister_data_old[0x09],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_0A,window_docregister->docregister_data[0x0A],window_docregister->docregister_data_old[0x0A],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_0B,window_docregister->docregister_data[0x0B],window_docregister->docregister_data_old[0x0B],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_0C,window_docregister->docregister_data[0x0C],window_docregister->docregister_data_old[0x0C],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_0D,window_docregister->docregister_data[0x0D],window_docregister->docregister_data_old[0x0D],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_0E,window_docregister->docregister_data[0x0E],window_docregister->docregister_data_old[0x0E],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_0F,window_docregister->docregister_data[0x0F],window_docregister->docregister_data_old[0x0F],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_10,window_docregister->docregister_data[0x10],window_docregister->docregister_data_old[0x10],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_11,window_docregister->docregister_data[0x11],window_docregister->docregister_data_old[0x11],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_12,window_docregister->docregister_data[0x12],window_docregister->docregister_data_old[0x12],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_13,window_docregister->docregister_data[0x13],window_docregister->docregister_data_old[0x13],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_14,window_docregister->docregister_data[0x14],window_docregister->docregister_data_old[0x14],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_15,window_docregister->docregister_data[0x15],window_docregister->docregister_data_old[0x15],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_16,window_docregister->docregister_data[0x16],window_docregister->docregister_data_old[0x16],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_17,window_docregister->docregister_data[0x17],window_docregister->docregister_data_old[0x17],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_18,window_docregister->docregister_data[0x18],window_docregister->docregister_data_old[0x18],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_19,window_docregister->docregister_data[0x19],window_docregister->docregister_data_old[0x19],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_1A,window_docregister->docregister_data[0x1A],window_docregister->docregister_data_old[0x1A],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_1B,window_docregister->docregister_data[0x1B],window_docregister->docregister_data_old[0x1B],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_1C,window_docregister->docregister_data[0x1C],window_docregister->docregister_data_old[0x1C],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_1D,window_docregister->docregister_data[0x1D],window_docregister->docregister_data_old[0x1D],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_1E,window_docregister->docregister_data[0x1E],window_docregister->docregister_data_old[0x1E],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_1F,window_docregister->docregister_data[0x1F],window_docregister->docregister_data_old[0x1F],force);
  /* 20-3F */
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_20,window_docregister->docregister_data[0x20],window_docregister->docregister_data_old[0x20],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_21,window_docregister->docregister_data[0x21],window_docregister->docregister_data_old[0x21],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_22,window_docregister->docregister_data[0x22],window_docregister->docregister_data_old[0x22],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_23,window_docregister->docregister_data[0x23],window_docregister->docregister_data_old[0x23],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_24,window_docregister->docregister_data[0x24],window_docregister->docregister_data_old[0x24],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_25,window_docregister->docregister_data[0x25],window_docregister->docregister_data_old[0x25],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_26,window_docregister->docregister_data[0x26],window_docregister->docregister_data_old[0x26],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_27,window_docregister->docregister_data[0x27],window_docregister->docregister_data_old[0x27],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_28,window_docregister->docregister_data[0x28],window_docregister->docregister_data_old[0x28],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_29,window_docregister->docregister_data[0x29],window_docregister->docregister_data_old[0x29],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_2A,window_docregister->docregister_data[0x2A],window_docregister->docregister_data_old[0x2A],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_2B,window_docregister->docregister_data[0x2B],window_docregister->docregister_data_old[0x2B],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_2C,window_docregister->docregister_data[0x2C],window_docregister->docregister_data_old[0x2C],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_2D,window_docregister->docregister_data[0x2D],window_docregister->docregister_data_old[0x2D],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_2E,window_docregister->docregister_data[0x2E],window_docregister->docregister_data_old[0x2E],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_2F,window_docregister->docregister_data[0x2F],window_docregister->docregister_data_old[0x2F],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_30,window_docregister->docregister_data[0x30],window_docregister->docregister_data_old[0x30],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_31,window_docregister->docregister_data[0x31],window_docregister->docregister_data_old[0x31],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_32,window_docregister->docregister_data[0x32],window_docregister->docregister_data_old[0x32],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_33,window_docregister->docregister_data[0x33],window_docregister->docregister_data_old[0x33],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_34,window_docregister->docregister_data[0x34],window_docregister->docregister_data_old[0x34],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_35,window_docregister->docregister_data[0x35],window_docregister->docregister_data_old[0x35],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_36,window_docregister->docregister_data[0x36],window_docregister->docregister_data_old[0x36],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_37,window_docregister->docregister_data[0x37],window_docregister->docregister_data_old[0x37],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_38,window_docregister->docregister_data[0x38],window_docregister->docregister_data_old[0x38],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_39,window_docregister->docregister_data[0x39],window_docregister->docregister_data_old[0x39],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_3A,window_docregister->docregister_data[0x3A],window_docregister->docregister_data_old[0x3A],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_3B,window_docregister->docregister_data[0x3B],window_docregister->docregister_data_old[0x3B],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_3C,window_docregister->docregister_data[0x3C],window_docregister->docregister_data_old[0x3C],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_3D,window_docregister->docregister_data[0x3D],window_docregister->docregister_data_old[0x3D],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_3E,window_docregister->docregister_data[0x3E],window_docregister->docregister_data_old[0x3E],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_3F,window_docregister->docregister_data[0x3F],window_docregister->docregister_data_old[0x3F],force);
  /* 40-5F */  
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_40,window_docregister->docregister_data[0x40],window_docregister->docregister_data_old[0x40],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_41,window_docregister->docregister_data[0x41],window_docregister->docregister_data_old[0x41],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_42,window_docregister->docregister_data[0x42],window_docregister->docregister_data_old[0x42],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_43,window_docregister->docregister_data[0x43],window_docregister->docregister_data_old[0x43],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_44,window_docregister->docregister_data[0x44],window_docregister->docregister_data_old[0x44],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_45,window_docregister->docregister_data[0x45],window_docregister->docregister_data_old[0x45],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_46,window_docregister->docregister_data[0x46],window_docregister->docregister_data_old[0x46],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_47,window_docregister->docregister_data[0x47],window_docregister->docregister_data_old[0x47],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_48,window_docregister->docregister_data[0x48],window_docregister->docregister_data_old[0x48],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_49,window_docregister->docregister_data[0x49],window_docregister->docregister_data_old[0x49],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_4A,window_docregister->docregister_data[0x4A],window_docregister->docregister_data_old[0x4A],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_4B,window_docregister->docregister_data[0x4B],window_docregister->docregister_data_old[0x4B],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_4C,window_docregister->docregister_data[0x4C],window_docregister->docregister_data_old[0x4C],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_4D,window_docregister->docregister_data[0x4D],window_docregister->docregister_data_old[0x4D],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_4E,window_docregister->docregister_data[0x4E],window_docregister->docregister_data_old[0x4E],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_4F,window_docregister->docregister_data[0x4F],window_docregister->docregister_data_old[0x4F],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_50,window_docregister->docregister_data[0x50],window_docregister->docregister_data_old[0x50],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_51,window_docregister->docregister_data[0x51],window_docregister->docregister_data_old[0x51],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_52,window_docregister->docregister_data[0x52],window_docregister->docregister_data_old[0x52],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_53,window_docregister->docregister_data[0x53],window_docregister->docregister_data_old[0x53],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_54,window_docregister->docregister_data[0x54],window_docregister->docregister_data_old[0x54],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_55,window_docregister->docregister_data[0x55],window_docregister->docregister_data_old[0x55],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_56,window_docregister->docregister_data[0x56],window_docregister->docregister_data_old[0x56],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_57,window_docregister->docregister_data[0x57],window_docregister->docregister_data_old[0x57],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_58,window_docregister->docregister_data[0x58],window_docregister->docregister_data_old[0x58],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_59,window_docregister->docregister_data[0x59],window_docregister->docregister_data_old[0x59],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_5A,window_docregister->docregister_data[0x5A],window_docregister->docregister_data_old[0x5A],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_5B,window_docregister->docregister_data[0x5B],window_docregister->docregister_data_old[0x5B],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_5C,window_docregister->docregister_data[0x5C],window_docregister->docregister_data_old[0x5C],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_5D,window_docregister->docregister_data[0x5D],window_docregister->docregister_data_old[0x5D],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_5E,window_docregister->docregister_data[0x5E],window_docregister->docregister_data_old[0x5E],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_5F,window_docregister->docregister_data[0x5F],window_docregister->docregister_data_old[0x5F],force);
  /* 60-7F */
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_60,window_docregister->docregister_data[0x60],window_docregister->docregister_data_old[0x60],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_61,window_docregister->docregister_data[0x61],window_docregister->docregister_data_old[0x61],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_62,window_docregister->docregister_data[0x62],window_docregister->docregister_data_old[0x62],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_63,window_docregister->docregister_data[0x63],window_docregister->docregister_data_old[0x63],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_64,window_docregister->docregister_data[0x64],window_docregister->docregister_data_old[0x64],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_65,window_docregister->docregister_data[0x65],window_docregister->docregister_data_old[0x65],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_66,window_docregister->docregister_data[0x66],window_docregister->docregister_data_old[0x66],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_67,window_docregister->docregister_data[0x67],window_docregister->docregister_data_old[0x67],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_68,window_docregister->docregister_data[0x68],window_docregister->docregister_data_old[0x68],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_69,window_docregister->docregister_data[0x69],window_docregister->docregister_data_old[0x69],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_6A,window_docregister->docregister_data[0x6A],window_docregister->docregister_data_old[0x6A],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_6B,window_docregister->docregister_data[0x6B],window_docregister->docregister_data_old[0x6B],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_6C,window_docregister->docregister_data[0x6C],window_docregister->docregister_data_old[0x6C],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_6D,window_docregister->docregister_data[0x6D],window_docregister->docregister_data_old[0x6D],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_6E,window_docregister->docregister_data[0x6E],window_docregister->docregister_data_old[0x6E],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_6F,window_docregister->docregister_data[0x6F],window_docregister->docregister_data_old[0x6F],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_70,window_docregister->docregister_data[0x70],window_docregister->docregister_data_old[0x70],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_71,window_docregister->docregister_data[0x71],window_docregister->docregister_data_old[0x71],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_72,window_docregister->docregister_data[0x72],window_docregister->docregister_data_old[0x72],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_73,window_docregister->docregister_data[0x73],window_docregister->docregister_data_old[0x73],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_74,window_docregister->docregister_data[0x74],window_docregister->docregister_data_old[0x74],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_75,window_docregister->docregister_data[0x75],window_docregister->docregister_data_old[0x75],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_76,window_docregister->docregister_data[0x76],window_docregister->docregister_data_old[0x76],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_77,window_docregister->docregister_data[0x77],window_docregister->docregister_data_old[0x77],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_78,window_docregister->docregister_data[0x78],window_docregister->docregister_data_old[0x78],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_79,window_docregister->docregister_data[0x79],window_docregister->docregister_data_old[0x79],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_7A,window_docregister->docregister_data[0x7A],window_docregister->docregister_data_old[0x7A],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_7B,window_docregister->docregister_data[0x7B],window_docregister->docregister_data_old[0x7B],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_7C,window_docregister->docregister_data[0x7C],window_docregister->docregister_data_old[0x7C],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_7D,window_docregister->docregister_data[0x7D],window_docregister->docregister_data_old[0x7D],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_7E,window_docregister->docregister_data[0x7E],window_docregister->docregister_data_old[0x7E],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_7F,window_docregister->docregister_data[0x7F],window_docregister->docregister_data_old[0x7F],force);
  /* 80-9F */
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_80,window_docregister->docregister_data[0x80],window_docregister->docregister_data_old[0x80],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_81,window_docregister->docregister_data[0x81],window_docregister->docregister_data_old[0x81],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_82,window_docregister->docregister_data[0x82],window_docregister->docregister_data_old[0x82],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_83,window_docregister->docregister_data[0x83],window_docregister->docregister_data_old[0x83],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_84,window_docregister->docregister_data[0x84],window_docregister->docregister_data_old[0x84],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_85,window_docregister->docregister_data[0x85],window_docregister->docregister_data_old[0x85],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_86,window_docregister->docregister_data[0x86],window_docregister->docregister_data_old[0x86],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_87,window_docregister->docregister_data[0x87],window_docregister->docregister_data_old[0x87],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_88,window_docregister->docregister_data[0x88],window_docregister->docregister_data_old[0x88],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_89,window_docregister->docregister_data[0x89],window_docregister->docregister_data_old[0x89],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_8A,window_docregister->docregister_data[0x8A],window_docregister->docregister_data_old[0x8A],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_8B,window_docregister->docregister_data[0x8B],window_docregister->docregister_data_old[0x8B],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_8C,window_docregister->docregister_data[0x8C],window_docregister->docregister_data_old[0x8C],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_8D,window_docregister->docregister_data[0x8D],window_docregister->docregister_data_old[0x8D],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_8E,window_docregister->docregister_data[0x8E],window_docregister->docregister_data_old[0x8E],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_8F,window_docregister->docregister_data[0x8F],window_docregister->docregister_data_old[0x8F],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_90,window_docregister->docregister_data[0x90],window_docregister->docregister_data_old[0x90],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_91,window_docregister->docregister_data[0x91],window_docregister->docregister_data_old[0x91],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_92,window_docregister->docregister_data[0x92],window_docregister->docregister_data_old[0x92],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_93,window_docregister->docregister_data[0x93],window_docregister->docregister_data_old[0x93],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_94,window_docregister->docregister_data[0x94],window_docregister->docregister_data_old[0x94],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_95,window_docregister->docregister_data[0x95],window_docregister->docregister_data_old[0x95],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_96,window_docregister->docregister_data[0x96],window_docregister->docregister_data_old[0x96],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_97,window_docregister->docregister_data[0x97],window_docregister->docregister_data_old[0x97],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_98,window_docregister->docregister_data[0x98],window_docregister->docregister_data_old[0x98],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_99,window_docregister->docregister_data[0x99],window_docregister->docregister_data_old[0x99],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_9A,window_docregister->docregister_data[0x9A],window_docregister->docregister_data_old[0x9A],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_9B,window_docregister->docregister_data[0x9B],window_docregister->docregister_data_old[0x9B],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_9C,window_docregister->docregister_data[0x9C],window_docregister->docregister_data_old[0x9C],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_9D,window_docregister->docregister_data[0x9D],window_docregister->docregister_data_old[0x9D],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_9E,window_docregister->docregister_data[0x9E],window_docregister->docregister_data_old[0x9E],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_9F,window_docregister->docregister_data[0x9F],window_docregister->docregister_data_old[0x9F],force);
  /* A0-BF */
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_A0,window_docregister->docregister_data[0xA0],window_docregister->docregister_data_old[0xA0],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_A1,window_docregister->docregister_data[0xA1],window_docregister->docregister_data_old[0xA1],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_A2,window_docregister->docregister_data[0xA2],window_docregister->docregister_data_old[0xA2],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_A3,window_docregister->docregister_data[0xA3],window_docregister->docregister_data_old[0xA3],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_A4,window_docregister->docregister_data[0xA4],window_docregister->docregister_data_old[0xA4],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_A5,window_docregister->docregister_data[0xA5],window_docregister->docregister_data_old[0xA5],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_A6,window_docregister->docregister_data[0xA6],window_docregister->docregister_data_old[0xA6],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_A7,window_docregister->docregister_data[0xA7],window_docregister->docregister_data_old[0xA7],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_A8,window_docregister->docregister_data[0xA8],window_docregister->docregister_data_old[0xA8],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_A9,window_docregister->docregister_data[0xA9],window_docregister->docregister_data_old[0xA9],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_AA,window_docregister->docregister_data[0xAA],window_docregister->docregister_data_old[0xAA],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_AB,window_docregister->docregister_data[0xAB],window_docregister->docregister_data_old[0xAB],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_AC,window_docregister->docregister_data[0xAC],window_docregister->docregister_data_old[0xAC],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_AD,window_docregister->docregister_data[0xAD],window_docregister->docregister_data_old[0xAD],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_AE,window_docregister->docregister_data[0xAE],window_docregister->docregister_data_old[0xAE],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_AF,window_docregister->docregister_data[0xAF],window_docregister->docregister_data_old[0xAF],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_B0,window_docregister->docregister_data[0xB0],window_docregister->docregister_data_old[0xB0],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_B1,window_docregister->docregister_data[0xB1],window_docregister->docregister_data_old[0xB1],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_B2,window_docregister->docregister_data[0xB2],window_docregister->docregister_data_old[0xB2],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_B3,window_docregister->docregister_data[0xB3],window_docregister->docregister_data_old[0xB3],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_B4,window_docregister->docregister_data[0xB4],window_docregister->docregister_data_old[0xB4],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_B5,window_docregister->docregister_data[0xB5],window_docregister->docregister_data_old[0xB5],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_B6,window_docregister->docregister_data[0xB6],window_docregister->docregister_data_old[0xB6],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_B7,window_docregister->docregister_data[0xB7],window_docregister->docregister_data_old[0xB7],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_B8,window_docregister->docregister_data[0xB8],window_docregister->docregister_data_old[0xB8],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_B9,window_docregister->docregister_data[0xB9],window_docregister->docregister_data_old[0xB9],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_BA,window_docregister->docregister_data[0xBA],window_docregister->docregister_data_old[0xBA],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_BB,window_docregister->docregister_data[0xBB],window_docregister->docregister_data_old[0xBB],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_BC,window_docregister->docregister_data[0xBC],window_docregister->docregister_data_old[0xBC],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_BD,window_docregister->docregister_data[0xBD],window_docregister->docregister_data_old[0xBD],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_BE,window_docregister->docregister_data[0xBE],window_docregister->docregister_data_old[0xBE],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_BF,window_docregister->docregister_data[0xBF],window_docregister->docregister_data_old[0xBF],force);
  /* C0-DF */
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_C0,window_docregister->docregister_data[0xC0],window_docregister->docregister_data_old[0xC0],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_C1,window_docregister->docregister_data[0xC1],window_docregister->docregister_data_old[0xC1],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_C2,window_docregister->docregister_data[0xC2],window_docregister->docregister_data_old[0xC2],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_C3,window_docregister->docregister_data[0xC3],window_docregister->docregister_data_old[0xC3],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_C4,window_docregister->docregister_data[0xC4],window_docregister->docregister_data_old[0xC4],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_C5,window_docregister->docregister_data[0xC5],window_docregister->docregister_data_old[0xC5],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_C6,window_docregister->docregister_data[0xC6],window_docregister->docregister_data_old[0xC6],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_C7,window_docregister->docregister_data[0xC7],window_docregister->docregister_data_old[0xC7],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_C8,window_docregister->docregister_data[0xC8],window_docregister->docregister_data_old[0xC8],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_C9,window_docregister->docregister_data[0xC9],window_docregister->docregister_data_old[0xC9],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_CA,window_docregister->docregister_data[0xCA],window_docregister->docregister_data_old[0xCA],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_CB,window_docregister->docregister_data[0xCB],window_docregister->docregister_data_old[0xCB],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_CC,window_docregister->docregister_data[0xCC],window_docregister->docregister_data_old[0xCC],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_CD,window_docregister->docregister_data[0xCD],window_docregister->docregister_data_old[0xCD],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_CE,window_docregister->docregister_data[0xCE],window_docregister->docregister_data_old[0xCE],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_CF,window_docregister->docregister_data[0xCF],window_docregister->docregister_data_old[0xCF],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_D0,window_docregister->docregister_data[0xD0],window_docregister->docregister_data_old[0xD0],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_D1,window_docregister->docregister_data[0xD1],window_docregister->docregister_data_old[0xD1],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_D2,window_docregister->docregister_data[0xD2],window_docregister->docregister_data_old[0xD2],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_D3,window_docregister->docregister_data[0xD3],window_docregister->docregister_data_old[0xD3],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_D4,window_docregister->docregister_data[0xD4],window_docregister->docregister_data_old[0xD4],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_D5,window_docregister->docregister_data[0xD5],window_docregister->docregister_data_old[0xD5],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_D6,window_docregister->docregister_data[0xD6],window_docregister->docregister_data_old[0xD6],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_D7,window_docregister->docregister_data[0xD7],window_docregister->docregister_data_old[0xD7],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_D8,window_docregister->docregister_data[0xD8],window_docregister->docregister_data_old[0xD8],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_D9,window_docregister->docregister_data[0xD9],window_docregister->docregister_data_old[0xD9],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_DA,window_docregister->docregister_data[0xDA],window_docregister->docregister_data_old[0xDA],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_DB,window_docregister->docregister_data[0xDB],window_docregister->docregister_data_old[0xDB],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_DC,window_docregister->docregister_data[0xDC],window_docregister->docregister_data_old[0xDC],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_DD,window_docregister->docregister_data[0xDD],window_docregister->docregister_data_old[0xDD],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_DE,window_docregister->docregister_data[0xDE],window_docregister->docregister_data_old[0xDE],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_DF,window_docregister->docregister_data[0xDF],window_docregister->docregister_data_old[0xDF],force);

  /* E0 E1 E2 */
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_E0,window_docregister->docregister_data[0xE0],window_docregister->docregister_data_old[0xE0],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_E1,window_docregister->docregister_data[0xE1],window_docregister->docregister_data_old[0xE1],force);
  SetWindowTextHexaByte(window_docregister->hwnd_docregister,IDC_STATIC_E2,window_docregister->docregister_data[0xE2],window_docregister->docregister_data_old[0xE2],force);  
  
  /* Recopie */
  memcpy(&window_docregister->docregister_data_old[0],&window_docregister->docregister_data[0],DOC_REGISTER_SIZE);
}


/*************************************************************************/
/*  DeleteDOCRegisterChildWindows() :  Destruction des fentres enfants. */
/*************************************************************************/
static void DeleteDOCRegisterChildWindows(struct window_docregister *window_docregister)
{
  /** Libration des ressources **/  
}


/***************************************************************************/
/*  mem_free_window_docregister() :  Libration mmoire pour la structure. */
/***************************************************************************/
void mem_free_window_docregister(struct window_docregister *window_docregister)
{
  if(window_docregister)
    {
      /* Supprime la notification */
      my_Notification(NOTIFICATION_REMOVE,window_docregister->current_notification,NULL);
      window_docregister->current_notification = NULL;
      
      free(window_docregister);
    }
}

/**************************************************************************/
