/************************************************************************/
/*                                                                      */
/*  Window_DirectPage.c :  Module de gestion de la fentre DirectPage.  */
/*                                                                      */
/************************************************************************/
/*  Auteur : Olivier ZARDINI  *   Brutal Deluxe   *   Date : Juin 2018  */
/************************************************************************/

#include <stdio.h>
#include <time.h>

#include <windows.h>
#include <windowsx.h>
#include <Commctrl.h>
#include <Winuser.h>
#include <Richedit.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_Notification.h"
#include "Dc_Graphic.h"
#include "Dc_CPU.h"
#include "Dc_MMU.h"
#include "Window_QuickWatch.h"
#include "Window_Main.h"
#include "Window_Main_Menu.h"
#include "Window_DirectPage.h"

static DWORD BuildDirectPageWindowThread(void *);
LRESULT CALLBACK ProcHandlerDirectPageWindow(HWND,UINT,WPARAM,LPARAM);
static int CreateDirectPageChildWindows(HWND,struct window_directpage *);
LRESULT CALLBACK subEditDirectPageProcControl(HWND,UINT,WPARAM,LPARAM);
static void UpdateDirectPageContextInformation(struct window_directpage *,int,int);
static void UpdateDirectPageRefreshMode(struct window_directpage *,int);
static void RefreshDirectPageWindowData(struct window_directpage *);
static int IsDirectPageDataModified(int,int,struct window_directpage *);
static void ResizeDirectPageWindow(struct window_directpage *);
static void BuildDirectPageFromData(struct window_directpage *);
static void DeleteDirectPageChildWindows(struct window_directpage *);


/**********************************************************************************/
/*  BuildDirectPageWindow() :  Construction / Affichage de la fentre DirectPage. */
/**********************************************************************************/
struct window_directpage *BuildDirectPageWindow(HINSTANCE hInstance, int menu_id)
{
  struct window_directpage *window_directpage;

  /** Allocation de la structure window_directpage **/
  window_directpage = (struct window_directpage *) calloc(1,sizeof(struct window_directpage));
  if(window_directpage == NULL)
    return(NULL);
  my_Memory(MEMORY_SET_WINDOW_DIRECTPAGE,window_directpage,NULL);

  /* Lance un thread pour grer la fentre DirectPage */
  CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)BuildDirectPageWindowThread,window_directpage,0,NULL);

  /* Renvoi la structure mmoire grant la fentre */
  return(window_directpage);
}


/********************************************************************************************/
/*  BuildDirectPageWindowThread() :  Cration de la fentre DirectPage via un Thread ddi. */
/********************************************************************************************/
static DWORD BuildDirectPageWindowThread(void *window_directpage_param)
{
  HWND hwndDirectPage;
  MSG msg;
  BOOL bRet;
  struct window_main *window_main;
  struct window_directpage *window_directpage;
  struct parameter *param;
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Init */
  window_directpage = (struct window_directpage *) window_directpage_param;

  /** Cration de la fentre Direct Page **/
  hwndDirectPage = CreateDialog(param->hInstance,MAKEINTRESOURCE(IDD_WINDOW_DIRECTPAGE),window_main->hwnd_main,(DLGPROC)ProcHandlerDirectPageWindow);
  if(hwndDirectPage == NULL)
    return(1);

  /** Show Window + Paint Content **/
  ShowWindow(hwndDirectPage,SW_SHOWDEFAULT);
  UpdateWindow(hwndDirectPage);

  /** Message Loop pour toutes les fentres de ce Thread (fentre DirectPage...) **/
  while((bRet = GetMessage(&msg,NULL,0,0)) != 0)
    {
      if(bRet == -1)
        break;

      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }

  /* Fin du Thread */
  ExitThread(0);
}


/***************************************************************************/
/*  ProcHandlerDirectPageWindow() :  Gestion des vnements de la fentre. */
/***************************************************************************/
LRESULT CALLBACK ProcHandlerDirectPageWindow(HWND hwndDirectPage, UINT umsg, WPARAM wParam, LPARAM lParam)
{
  struct window_directpage *window_directpage;
  struct parameter *param;
  
  /* Init */
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  my_Memory(MEMORY_GET_WINDOW_DIRECTPAGE,&window_directpage,NULL);

  switch(umsg)
    {
      case WM_INITDIALOG :
        /* Cration du contenu de la DialogBox */
        CreateDirectPageChildWindows(hwndDirectPage,window_directpage);
        return(TRUE);

      /***************************/
      /* Evnements de la souris */
      /***************************/
      case WM_MOUSEMOVE :
        SetWindowText(GetDlgItem(window_directpage->hwnd_directpage,IDC_STATIC_CONTEXT),"");
        return(TRUE);
        
      /****************************/
      /* Evnements de la fentre */
      /****************************/
      case WM_COMMAND :
        switch(LOWORD(wParam))
          {
            /** Bouton Radio : On change la mthode de rafraichissement de la fentre **/
            case IDC_RADIO_REFRESH_AUTOMATIC :
              if(HIWORD(wParam) == BN_CLICKED && window_directpage->refresh_mode != DIRECTPAGE_REFRESH_AUTOMATIC)
                UpdateDirectPageRefreshMode(window_directpage,DIRECTPAGE_REFRESH_AUTOMATIC);
              break;

            case IDC_RADIO_REFRESH_MANUAL :
              if(HIWORD(wParam) == BN_CLICKED && window_directpage->refresh_mode != DIRECTPAGE_REFRESH_MANUAL)
                UpdateDirectPageRefreshMode(window_directpage,DIRECTPAGE_REFRESH_MANUAL);
              break;

            /** Bouton de rafraichissement Manuel de la fentre **/
            case IDC_BUTTON_REFRESH :
              RefreshDirectPageWindowData(window_directpage);
              break;
          }
        return(TRUE);

      /*****************************************************************/
      /* Message extrieur demandant le rafraichissement de la fentre */
      /*****************************************************************/
      case HM_DIRECTPAGE_REFRESHWINDOW :
        RefreshDirectPageWindowData(window_directpage);
        return(TRUE);

      /********************************************************************/
      /* Fermeture de window (le WM_CLOSE est appel avant le WM_DESTROY) */
      /********************************************************************/
      case WM_CLOSE :
        /* Fentre DirectPage */
        DestroyWindow(hwndDirectPage);
        return(0);

      case WM_DESTROY :
        /* Suppression des Fentres */
        DeleteDirectPageChildWindows(window_directpage);
        CheckMainWindowMenuItem(ID_MENU_VIEW_DIRECTPAGE,FALSE);
        my_Memory(MEMORY_FREE_WINDOW_DIRECTPAGE,NULL,NULL);
        /* Demande la sortie de la boucle des messages */
        PostQuitMessage(0);
        return(0);
    }

  /* Retourner FALSE pour relayer le message non trait au systme */
  return(FALSE);
}


/***********************************************************************************************************/
/*  CreateDirectPageChildWindows() : Crer les fentres enfants et remplir la structure window_directpage. */
/***********************************************************************************************************/
static int CreateDirectPageChildWindows(HWND hwndDirectPage, struct window_directpage *window_directpage)
{
  HWND hwnd_group;
  RECT client_rect, window_rect, group_rect;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Allocation Buffer mmoire Zone Texte : 256 Bytes de raw data **/
  window_directpage->nb_line = 16; 
  window_directpage->nb_column = 16;
  /* Nombre caractres visibles sur la ligne (on ne compte pas le \r\n final) */
  window_directpage->text_data_width = (int) (strlen("00/0000   ") + window_directpage->nb_column*3 - 1);   /* Enlve le dernier sparateur */
  /* Nombre d'octet pour encoder la zone Texte (on compte les \r\n en fin de ligne) */
  window_directpage->text_data_size = window_directpage->nb_line*(window_directpage->text_data_width+2);  /* +2 pour */
  window_directpage->text_data_hexadecimal = (char *) calloc(window_directpage->text_data_size+1,sizeof(char));
  if(window_directpage->text_data_hexadecimal == NULL)
    return(1);

  /** Window Handle **/
  window_directpage->hwnd_directpage = hwndDirectPage;

  /* Refresh Mode : Automatic */
  UpdateDirectPageRefreshMode(window_directpage,DIRECTPAGE_REFRESH_AUTOMATIC);

  /* Force la MAJ du contenu de la fentre */
  window_directpage->directpage_address = -1;

  /* Hauteur Titre */
  GetWindowRect(window_directpage->hwnd_directpage,&window_rect);
  GetClientRect(window_directpage->hwnd_directpage,&client_rect);
  window_directpage->title_height = (window_rect.bottom - window_rect.top) - client_rect.bottom;

  /* Position de la GroupBox Select + GroupBox Palette */
  hwnd_group = GetDlgItem(window_directpage->hwnd_directpage,IDC_GROUP_DIRECTPAGE);     /* Rcupre le Handle du Control GroupBox li  IDC_GROUP_DIRECTPAGE */
  GetWindowRect(hwnd_group,&group_rect);
  window_directpage->group_data_x = group_rect.left - window_rect.left;
  window_directpage->group_data_y = (group_rect.top - window_rect.top) - window_directpage->title_height;

  /* Position de la Zone Text Direct Page Data dans la fenetre */
  window_directpage->directpage_data_x = window_directpage->group_data_x + WINDOW_GROUP_BORDER_LEFT_SIZE;
  window_directpage->directpage_data_y = window_directpage->group_data_y + WINDOW_GROUP_BORDER_TOP_SIZE;

  /* Dimension en Pixel du Control Text DirectPage Data */
  window_directpage->directpage_data_width = window_directpage->text_data_width*7 + 2;
  window_directpage->directpage_data_height = window_directpage->nb_line*DIRECTPAGE_EDIT_FONT_HEIGHT;

  /* Dimension en Pixel du Control Group Box + Resize */  
  window_directpage->group_data_width = WINDOW_GROUP_BORDER_LEFT_SIZE + window_directpage->directpage_data_width + WINDOW_GROUP_BORDER_RIGHT_SIZE;
  window_directpage->group_data_height = WINDOW_GROUP_BORDER_TOP_SIZE + window_directpage->directpage_data_height + WINDOW_GROUP_BORDER_BOTTOM_SIZE;
  MoveWindow(hwnd_group,window_directpage->group_data_x,window_directpage->group_data_y,window_directpage->group_data_width,window_directpage->group_data_height,TRUE);

  /** Cration du Control Text Edit Multi ligne ReadOnly **/
  window_directpage->hwnd_edit_data = CreateWindowExA(0,"EDIT",NULL,WS_CHILD | WS_VISIBLE | ES_LEFT | ES_MULTILINE | ES_READONLY, 
                                                      window_directpage->directpage_data_x, window_directpage->directpage_data_y, window_directpage->directpage_data_width, window_directpage->directpage_data_height,   // set size in WM_SIZE message 
                                                      window_directpage->hwnd_directpage,
                                                      (HMENU)IDC_EDIT_DIRECTPAGE_DATA,            // edit control ID
                                                      param->hInstance,
                                                      NULL);

  /** Surchage des Control Edit pour capter les vnement **/
  window_directpage->oldEditProcDirectPageData = (WNDPROC) SetWindowLongPtr(window_directpage->hwnd_edit_data,GWLP_WNDPROC,(LONG_PTR)subEditDirectPageProcControl);

  /** Cration de la Police de caractre + Affectation de la police au Control Edit **/
  window_directpage->hFont = CreateFont(DIRECTPAGE_EDIT_FONT_HEIGHT,0,0,0,FW_DONTCARE,FALSE,FALSE,FALSE,ANSI_CHARSET,OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY,DEFAULT_PITCH|FF_SWISS,TEXT("Courier New"));
  SendMessage(window_directpage->hwnd_edit_data,WM_SETFONT,(WPARAM)window_directpage->hFont,MAKELPARAM(FALSE,0));  

  /* Retaille la fentre */
  ResizeDirectPageWindow(window_directpage);

  /** Construction du Texte + Rafraichissement de la fentre **/
  RefreshDirectPageWindowData(window_directpage);
    
  /* OK */
  return(0);
}

/*************************************************************************************************/
/*  subEditDirectPageProcControl() :  Rcupre les vnements qui arrivent dans le Control Edit. */
/*************************************************************************************************/
LRESULT CALLBACK subEditDirectPageProcControl(HWND hwnd_control, UINT msg, WPARAM wParam, LPARAM lParam)
{
  int index, mouse_x, mouse_y;
  struct window_directpage *window_directpage;
  my_Memory(MEMORY_GET_WINDOW_DIRECTPAGE,&window_directpage,NULL);

  /* Protection */
  if(window_directpage == NULL)
    return(0);

  /** Identifie le Control **/
  /* Src */
  if(window_directpage->hwnd_edit_data == hwnd_control)
    index = 1;
  else
    return(0);      /* Pas trouv */

  /** Dtecte l'vnement de la souris dans l'Edit **/
  if(msg == WM_MOUSEMOVE)
    {
      /* Corrdonnes de la souris dans l'Edit */
      mouse_x = GET_X_LPARAM(lParam) / 7;
      mouse_y = GET_Y_LPARAM(lParam) / DIRECTPAGE_EDIT_FONT_HEIGHT;
      UpdateDirectPageContextInformation(window_directpage,mouse_x,mouse_y);
      
      /* Transmet l'vnement au Control Text */
      return(CallWindowProc(window_directpage->oldEditProcDirectPageData,hwnd_control,msg,wParam,lParam)); 
    }
  else if(index == 1)
    return(CallWindowProc(window_directpage->oldEditProcDirectPageData,hwnd_control,msg,wParam,lParam)); 
         
  /* OK */
  return(0);
} 


/*******************************************************************************************************/
/*  UpdateDirectPageContextInformation() :  MAJ des information de contexte de la fenetre DirectPage. */
/*******************************************************************************************************/
static void UpdateDirectPageContextInformation(struct window_directpage *window_directpage, int grid_x, int grid_y)
{
  int grid_directpage_x, directpage_data_address, grid_directpage_offset, directpage_data_offset;
  unsigned char directpage_data;
  
  /* Rien  faire */
  if(window_directpage == NULL)
    return;
    
  /* En dehors de la zone */
  if(grid_x < 10)
    {
      SetWindowText(GetDlgItem(window_directpage->hwnd_directpage,IDC_STATIC_CONTEXT),"");
      return;
    }
  grid_directpage_x = grid_x - 10;     /* Enlve les 10 caractres de gauche */
  if(((grid_directpage_x+1) % 3) == 0)
    {
      SetWindowText(GetDlgItem(window_directpage->hwnd_directpage,IDC_STATIC_CONTEXT),"");
      return;
    }
  grid_directpage_offset = (grid_directpage_x+1) / 3;
  
  /* Rcupre l'adresse de la cellule */
  directpage_data_address = window_directpage->directpage_address + grid_y*16;
  directpage_data_address += grid_directpage_offset;
  
  /* Rcupre la valeur de la cellule */
  directpage_data_offset = directpage_data_address - window_directpage->directpage_address;
  if(directpage_data_offset > 255)
    {
      SetWindowText(GetDlgItem(window_directpage->hwnd_directpage,IDC_STATIC_CONTEXT),"");
      return;
    }
  directpage_data = window_directpage->directpage_data[directpage_data_offset];
  
  /* Coordonnes de la souris en coodonnes grid_x,grid_y */
  sprintf(window_directpage->buffer_log,"Direct Page Data  %02X/%04X : %02X",directpage_data_address>>16,directpage_data_address&0xFFFF,directpage_data);
  SetWindowText(GetDlgItem(window_directpage->hwnd_directpage,IDC_STATIC_CONTEXT),window_directpage->buffer_log);
}


/************************************************************************************************/
/*  UpdateDirectPageRefreshMode() :  MAJ du mode de rafraichissement de la fenetre DirectPage. */
/************************************************************************************************/
static void UpdateDirectPageRefreshMode(struct window_directpage *window_directpage, int refresh_mode)
{
  struct notification *current_notification;

  if(refresh_mode == DIRECTPAGE_REFRESH_AUTOMATIC)
    {
      /* Visuel */
      window_directpage->refresh_mode = DIRECTPAGE_REFRESH_AUTOMATIC;
      SendDlgItemMessage(window_directpage->hwnd_directpage,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_directpage->hwnd_directpage,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,0,0);

      /* Cration d'une Notification */
      current_notification = (struct notification *) calloc(1,sizeof(struct notification));
      if(current_notification == NULL)
        return;
      current_notification->hwnd_window = window_directpage->hwnd_directpage;
      current_notification->message = HM_DIRECTPAGE_REFRESHWINDOW;
      current_notification->type = NOTIFICATION_TYPE_VIEW;
      current_notification->sub_type = NOTIFICATION_SUBTYPE_DIRECTPAGE;

      /* Ajout de la notification */
      window_directpage->current_notification = current_notification;
      my_Notification(NOTIFICATION_ADD,current_notification,NULL);
    }
  else
    {
      /* Visuel */
      window_directpage->refresh_mode = DIRECTPAGE_REFRESH_MANUAL;
      SendDlgItemMessage(window_directpage->hwnd_directpage,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_directpage->hwnd_directpage,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,1,0);

      /* Supprime la Notification */
      my_Notification(NOTIFICATION_REMOVE,window_directpage->current_notification,NULL);
      window_directpage->current_notification = NULL;
    }
}


/******************************************************************************/
/*  RefreshDirectPageWindowData() :  MAJ du contenu de la fentre DirectPage. */
/******************************************************************************/
static void RefreshDirectPageWindowData(struct window_directpage *window_directpage)
{
  int directpage_address, directpage_data_base_address, is_modified;
  struct cpu *current_cpu;

  /* Rcupre le CPU */
  my_Memory(MEMORY_GET_S65816_CPU,&current_cpu,NULL);

  /* Rcupre la valeur du DirectPage Pointer */
  directpage_address = GetCPUData(CPU_ADDRESS_DIRECTPAGE,current_cpu);
  if((directpage_address & 0x0000F0) <= 0x000070)
    directpage_data_base_address = (directpage_address & 0xFFFF00);
  else
    directpage_data_base_address = (directpage_address & 0xFFFF00) + 0x80;
    
  /** Les donnes ont t'elles t modifies + Rcupration des nouvelles Data si besoin **/
  is_modified = IsDirectPageDataModified(directpage_address,directpage_data_base_address,window_directpage);
  if(is_modified == 0)
    return;
    
  /** Cration de la Zone Rich Edit (RTF)  partir des nouvelles donnes **/
  BuildDirectPageFromData(window_directpage);

  /* Direct Page Pointer */
  sprintf(window_directpage->buffer_log,"%02X/%04X",directpage_address>>16,directpage_address&0xFFFF);
  SetWindowText(GetDlgItem(window_directpage->hwnd_directpage,IDC_STATIC_DIRECTPAGEPOINTER),window_directpage->buffer_log);

  /* Rafraichit la fentre */
  UpdateWindow(window_directpage->hwnd_directpage);
}


/*************************************************************************************************************/
/*  IsDirectPageDataModified() :  Les donnes DirectPage ont t'elles t modifies depuis la dernirer fois. */
/*************************************************************************************************************/
static int IsDirectPageDataModified(int directpage_address, int directpage_data_base_address, struct window_directpage *window_directpage)
{
  unsigned char directpage_data[256];
  struct mmu *current_mmu;

  /* Rcupre le MMU */
  my_Memory(MEMORY_GET_S65816_MMU,&current_mmu,NULL);
  
  /* Rcupre les donnes */
  ReadMemoryBlock(directpage_data_base_address,256,&directpage_data[0],current_mmu);
  
  /* Compare les donnes */
  if(directpage_address == window_directpage->directpage_address &&
     memcmp(&directpage_data[0],&window_directpage->directpage_data[0],256) == 0)
    return(0);
    
  /* Les donnes sont diffrentes */
  window_directpage->directpage_address = directpage_address;
  memcpy(&window_directpage->directpage_data[0],&directpage_data[0],256);
  return(1);
}


/***********************************************************************/
/*  ResizeDirectPageWindow() : Resize DirectPage window by given size. */
/***********************************************************************/
static void ResizeDirectPageWindow(struct window_directpage *window_directpage)
{
  RECT window_rect;
  int window_x, window_y;
  
  /* Largeur de la fentre */
  window_directpage->window_width = window_directpage->group_data_x + window_directpage->group_data_width + WINDOW_CLIENT_RIGHT_BORDER_SIZE + WINDOW_BORDER_SIZE;

  /* Hauteur de la fentre */
  window_directpage->window_height = window_directpage->title_height + window_directpage->group_data_y + window_directpage->group_data_height + WINDOW_CLIENT_BOTTOM_BORDER_SIZE;

  /* Position actuelle de la fentre dans l'cran */
  GetWindowRect(window_directpage->hwnd_directpage,&window_rect);
  window_x = window_rect.left;
  window_y = window_rect.top;

  /* Resize window */
  MoveWindow(window_directpage->hwnd_directpage,window_x,window_y,window_directpage->window_width,window_directpage->window_height,TRUE);
}


/************************************************************************************************************/
/*  BuildDirectPageFromData() :  Construire les donnes hxadcimal  afficher  partir des donnes brutes. */
/************************************************************************************************************/
static void BuildDirectPageFromData(struct window_directpage *window_directpage)
{
  int i, j, offset, offset_text, nb_byte_to_process;

  /*** Transformer les donnes BYTE en Hexa ***/
  nb_byte_to_process = window_directpage->nb_column*window_directpage->nb_line;
  for(i=0,offset=0,offset_text=0; i<window_directpage->nb_line; i++,offset+=16)
    {
      /* Adresse */
      sprintf(&window_directpage->text_data_hexadecimal[offset_text],"%02X/%04X   ",(offset+window_directpage->directpage_address)>>16,(offset+window_directpage->directpage_address) & 0xFFFF);
      offset_text += (int) strlen("00/0000   ");

      /* Hexa */
      for(j=0; j<window_directpage->nb_column; j++)
        {
          if(i*window_directpage->nb_column+j < nb_byte_to_process)
            sprintf(&window_directpage->text_data_hexadecimal[offset_text],"%02X",window_directpage->directpage_data[offset+j]);
          else
            strcpy(&window_directpage->text_data_hexadecimal[offset_text],"  ");
          offset_text += 2;

          /* Sparateur */
          if(j < (window_directpage->nb_column-1))
            {
              window_directpage->text_data_hexadecimal[offset_text] = (j==7 && (i*window_directpage->nb_column+j < nb_byte_to_process)) ? '.' : ' ';
              offset_text++;
            }
        }

      /* Fin de ligne */
      strcpy(&window_directpage->text_data_hexadecimal[offset_text],"\r\n");
      offset_text += 2;
    }
    
  /** Efface le contenu de la fentre **/
  SetWindowText(window_directpage->hwnd_edit_data,"");

  /** Afficher le rsultat dans le control **/
  SetWindowText(window_directpage->hwnd_edit_data,window_directpage->text_data_hexadecimal);
}


/************************************************************************/
/*  DeleteDirectPageChildWindows() :  Destruction des fentres enfants. */
/************************************************************************/
static void DeleteDirectPageChildWindows(struct window_directpage *window_directpage)
{
  /** Libration des ressources **/  
  if(window_directpage->text_data_hexadecimal != NULL)
    free(window_directpage->text_data_hexadecimal);
  window_directpage->text_data_hexadecimal = NULL;
}


/**************************************************************************/
/*  mem_free_window_directpage() :  Libration mmoire pour la structure. */
/**************************************************************************/
void mem_free_window_directpage(struct window_directpage *window_directpage)
{
  if(window_directpage)
    {
      /* Supprime la notification */
      my_Notification(NOTIFICATION_REMOVE,window_directpage->current_notification,NULL);
      window_directpage->current_notification = NULL;
      
      free(window_directpage);
    }
}

/**********************************************************************/
