/**********************************************************************/
/*                                                                    */
/*   Window_Stack.c :  Module de gestion de la fentre Stack.         */
/*                                                                    */
/**********************************************************************/
/*  Auteur : Olivier ZARDINI  *   Brutal Deluxe   *  Date : Mai 2018  */
/**********************************************************************/

#include <stdio.h>
#include <time.h>

#include <windows.h>
#include <windowsx.h>
#include <Commctrl.h>
#include <Winuser.h>
#include <Richedit.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_Notification.h"
#include "Dc_Graphic.h"
#include "Dc_CPU.h"
#include "Dc_MMU.h"
#include "Window_Main.h"
#include "Window_Main_Menu.h"
#include "Window_Stack.h"

static DWORD BuildStackWindowThread(void *);
LRESULT CALLBACK ProcHandlerStackWindow(HWND,UINT,WPARAM,LPARAM);
static int CreateStackChildWindows(HWND,struct window_stack *);
static void UpdateStackRefreshMode(struct window_stack *,int);
static void RefreshStackWindowData(struct window_stack *);
static int IsStackDataModified(int,int,struct window_stack *);
static void ResizeStackWindow(struct window_stack *);
static void BuildStackFromData(struct window_stack *);
static void DeleteStackChildWindows(struct window_stack *);


/************************************************************************/
/*  BuildStackWindow() :  Construction / Affichage de la fentre Stack. */
/************************************************************************/
struct window_stack *BuildStackWindow(HINSTANCE hInstance, int menu_id)
{
  struct window_stack *window_stack;

  /** Allocation de la structure window_stack **/
  window_stack = (struct window_stack *) calloc(1,sizeof(struct window_stack));
  if(window_stack == NULL)
    return(NULL);
  my_Memory(MEMORY_SET_WINDOW_STACK,window_stack,NULL);

  /* Lance un thread pour grer la fentre Stack */
  CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)BuildStackWindowThread,window_stack,0,NULL);

  /* Renvoi la structure mmoire grant la fentre */
  return(window_stack);
}


/**********************************************************************************/
/*  BuildStackWindowThread() :  Cration de la fentre Stack via un Thread ddi. */
/**********************************************************************************/
static DWORD BuildStackWindowThread(void *window_stack_param)
{
  HWND hwndStack;
  MSG msg;
  BOOL bRet;
  struct window_main *window_main;
  struct window_stack *window_stack;
  struct parameter *param;

  /* Init */
  window_stack = (struct window_stack *) window_stack_param;
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Cration de la fentre Stack **/
  hwndStack = CreateDialog(param->hInstance,MAKEINTRESOURCE(IDD_WINDOW_STACK),window_stack->hwnd_stack,(DLGPROC)ProcHandlerStackWindow);
  if(hwndStack == NULL)
    return(1);

  /** Show Window + Paint Content **/
  ShowWindow(hwndStack,SW_SHOWDEFAULT);
  UpdateWindow(hwndStack);

  /** Message Loop pour la fentre Stack **/
  while((bRet = GetMessage(&msg,hwndStack,0,0)) != 0)
    {
      if(bRet == -1)
        break;

      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }

  /* Fin du Thread */
  ExitThread(0);
}


/**********************************************************************/
/*  ProcHandlerStackWindow() :  Gestion des vnements de la fentre. */
/**********************************************************************/
LRESULT CALLBACK ProcHandlerStackWindow(HWND hwndStack, UINT umsg, WPARAM wParam, LPARAM lParam)
{
  HGDIOBJ bitmap_stack_pointer;
  struct window_stack *window_stack;
  struct parameter *param;
  
  /* Init */
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  my_Memory(MEMORY_GET_WINDOW_STACK,&window_stack,NULL);

  switch(umsg)
    {
      case WM_INITDIALOG :
        /* Cration du contenu de la DialogBox */
        CreateStackChildWindows(hwndStack,window_stack);
        return(TRUE);

      /**********************************/
      /* Rafrachissement de la fentre */
      /**********************************/
      case WM_PAINT :
        if(window_stack->hBitmap_stack_pointer != NULL)
          {
            /** Bipmap dsignant le Stack Pointer **/
            bitmap_stack_pointer = SelectObject(window_stack->hCompatibleDC,window_stack->hBitmap_stack_pointer);
            StretchBlt(window_stack->hDC,50,5,STACK_SELECT_WIDTH,STACK_SELECT_HEIGHT,
                       window_stack->hCompatibleDC,0,0,STACK_SELECT_WIDTH,STACK_SELECT_HEIGHT,SRCCOPY);
            SelectObject(window_stack->hCompatibleDC,bitmap_stack_pointer);
          }
        break;

      /****************************/
      /* Evnements de la fentre */
      /****************************/
      case WM_COMMAND :
        switch(LOWORD(wParam))
          {
            /** Bouton Radio : On change la mthode de rafraichissement de la fentre **/
            case IDC_RADIO_REFRESH_AUTOMATIC :
              if(HIWORD(wParam) == BN_CLICKED && window_stack->refresh_mode != STACK_REFRESH_AUTOMATIC)
                UpdateStackRefreshMode(window_stack,STACK_REFRESH_AUTOMATIC);
              break;

            case IDC_RADIO_REFRESH_MANUAL :
              if(HIWORD(wParam) == BN_CLICKED && window_stack->refresh_mode != STACK_REFRESH_MANUAL)
                UpdateStackRefreshMode(window_stack,STACK_REFRESH_MANUAL);
              break;

            /** Bouton de rafraichissement Manuel de la fentre **/
            case IDC_BUTTON_REFRESH :
              RefreshStackWindowData(window_stack);
              break;
          }
        return(TRUE);

      /**************************************************************/
      /* Message extrieur demandant le rafraissiment de la fentre */
      /**************************************************************/
      case HM_STACK_REFRESHWINDOW :
        RefreshStackWindowData(window_stack);
        return(TRUE);

      /********************************************************************/
      /* Fermeture de window (le WM_CLOSE est appel avant le WM_DESTROY) */
      /********************************************************************/
      case WM_CLOSE :
        /* Fentre Stack */
        DestroyWindow(hwndStack);
        return(0);

      case WM_DESTROY :
        /* Suppression des Fentres */
        DeleteStackChildWindows(window_stack);
        CheckMainWindowMenuItem(ID_MENU_VIEW_STACK,FALSE);
        my_Memory(MEMORY_FREE_WINDOW_STACK,NULL,NULL);
        return(0);
    }

  /* Retourner FALSE pour relayer le message non trait au systme */
  return(FALSE);
}


/*************************************************************************************************/
/*  CreateStackChildWindows() : Crer les fentres enfants et remplir la structure window_stack. */
/*************************************************************************************************/
static int CreateStackChildWindows(HWND hwndStack, struct window_stack *window_stack)
{
  int i;
  HWND hwndItem;
  RECT client_rect, window_rect, group_rect;
  BITMAPINFO bitmap_info;
  CHARFORMAT2 char_format;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Window Handle **/
  window_stack->hwnd_stack = hwndStack;

  /** Device Context **/
  window_stack->hDC = GetDC(window_stack->hwnd_stack);
  window_stack->hCompatibleDC = CreateCompatibleDC(window_stack->hDC);

  /* Refresh Mode */
  window_stack->refresh_mode = STACK_REFRESH_AUTOMATIC;
  SendDlgItemMessage(window_stack->hwnd_stack,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,1,0);
  SendDlgItemMessage(window_stack->hwnd_stack,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,0,0);

  /* Hauteur Titre */
  GetWindowRect(window_stack->hwnd_stack,&window_rect);
  GetClientRect(window_stack->hwnd_stack,&client_rect);
  window_stack->title_height = (window_rect.bottom - window_rect.top) - client_rect.bottom;

  /* Position de la GroupBox Select + GroupBox Palette */
  hwndItem = GetDlgItem(window_stack->hwnd_stack,IDC_GROUP_STACK);     /* Rcupre le Handle du Control GroupBox li  IDC_GROUP_STACK */
  GetWindowRect(hwndItem,&group_rect);

  /* Position des Bitmap dans la fenetre */
  window_stack->stack_data_x = (group_rect.left - window_rect.left) + 3;
  window_stack->stack_data_y = (group_rect.top - window_rect.top) - window_stack->title_height + 18;

  /** Allocation Edit Control  **/
  /* Taille de la Zone en Byte : 256 Bytes */
  window_stack->column = 16;
  window_stack->line = 16;

  /** Allocation Buffer mmoire Zone Texte **/
  window_stack->text_data_width = (strlen("00/0000   ") + window_stack->column*3 - 1);   /* Enlve le dernier sparateur */
  window_stack->text_data_size = window_stack->line*window_stack->text_data_width;
  window_stack->text_data_hexadecimal = (char *) calloc(window_stack->text_data_size+1,sizeof(char));
  if(window_stack->text_data_hexadecimal == NULL)
    return(1);
    
  /** Cration du Control Text Edit Multi ligne ReadOnly **/
  window_stack->text_area_width = window_stack->text_data_width*7 + 2;
  window_stack->text_area_height = window_stack->line*14;
  window_stack->hwnd_edit = CreateWindowExA(0,"EDIT",NULL,WS_CHILD | WS_VISIBLE | ES_LEFT | ES_MULTILINE | ES_READONLY, 
                                            window_stack->stack_data_x, window_stack->stack_data_y, window_stack->text_area_width, window_stack->text_area_height,   // set size in WM_SIZE message 
                                            window_stack->hwnd_stack,
                                            (HMENU)IDC_EDIT_STACK_DATA,            // edit control ID
                                            param->hInstance,
                                            NULL);

  /** Cration de la Police de caractre + Affectation de la police au Control Edit **/
  window_stack->hFont = CreateFont(14,0,0,0,FW_DONTCARE,FALSE,FALSE,FALSE,ANSI_CHARSET,OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY,DEFAULT_PITCH|FF_SWISS,TEXT("Courier New"));
  SendMessage(window_stack->hwnd_edit,WM_SETFONT,(WPARAM)window_stack->hFont,MAKELPARAM(FALSE,0));  

  /* Bitmap 14*1 pour les couleurs slectionnes */
  memset(&bitmap_info,0,sizeof(BITMAPINFO));
  bitmap_info.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
  bitmap_info.bmiHeader.biWidth = STACK_SELECT_WIDTH;
  bitmap_info.bmiHeader.biHeight = (-1)*STACK_SELECT_HEIGHT;
  bitmap_info.bmiHeader.biPlanes = 1;
  bitmap_info.bmiHeader.biBitCount = 32;
  bitmap_info.bmiHeader.biCompression = BI_RGB;
  window_stack->hBitmap_stack_pointer = CreateDIBSection(window_stack->hDC,&bitmap_info,DIB_RGB_COLORS,(VOID **)&(window_stack->bitmap_stack_pointer),NULL,0);
  for(i=0; i<STACK_SELECT_WIDTH*STACK_SELECT_HEIGHT; i++)
    window_stack->bitmap_stack_pointer[i] = STACK_POINTER_COLOR;

  /** Cration du Rich Text **/
  window_stack->hwnd_richedit = CreateWindowEx(0,RICHEDIT_CLASS,"",WS_CHILD | WS_VISIBLE | ES_SAVESEL | ES_MULTILINE | ES_AUTOVSCROLL | WS_VSCROLL | ES_AUTOHSCROLL | WS_HSCROLL | ES_AUTOHSCROLL | ES_READONLY, 
                                               170, 40, 250, 30, 
                                               window_stack->hwnd_stack,
                                               (HMENU)IDC_RICHEDIT_STACK_DATA,
                                               param->hInstance,
                                               NULL);
  
  /* Couleur de Fond du control */
  //SendMessage(window_stack->hwnd_richedit, EM_SETBKGNDCOLOR, 0, (LPARAM)RGB(255,255,255)); 
  
  /** Police Courier New **/
  char_format.cbSize = sizeof(char_format); 
  char_format.dwMask = CFM_BOLD | CFM_COLOR | CFM_SIZE | CFM_ITALIC | CFM_UNDERLINE | CFM_FACE;
  char_format.crTextColor = RGB(255,0,0);
  char_format.yHeight = 160;   /* 8*20 */
  char_format.dwEffects = 0;
  char_format.bCharSet = ANSI_CHARSET;
  char_format.bPitchAndFamily = DEFAULT_PITCH;
  lstrcpy(char_format.szFaceName,"Courier New");
  SendMessage(window_stack->hwnd_richedit,EM_SETCHARFORMAT,SCF_ALL,(DWORD)&char_format);

  /* Retaille la fentre */
  ResizeStackWindow(window_stack);

  /** Construction du Texte + Retaille la fentre + Rafraichissement de la fentre **/
  RefreshStackWindowData(window_stack);
    
  /* OK */
  return(0);
}


/**************************************************************************************/
/*  UpdateStackRefreshMode() :  MAJ du mode de rafraichissement de la fenetre Stack. */
/**************************************************************************************/
static void UpdateStackRefreshMode(struct window_stack *window_stack, int refresh_mode)
{
  struct notification *current_notification;

  if(refresh_mode == STACK_REFRESH_AUTOMATIC)
    {
      /* Visuel */
      window_stack->refresh_mode = STACK_REFRESH_AUTOMATIC;
      SendDlgItemMessage(window_stack->hwnd_stack,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_stack->hwnd_stack,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,0,0);

      /* Cration d'une Notification */
      current_notification = (struct notification *) calloc(1,sizeof(struct notification));
      if(current_notification == NULL)
        return;
      current_notification->hwnd_window = window_stack->hwnd_stack;
      current_notification->message = HM_STACK_REFRESHWINDOW;
      current_notification->type = NOTIFICATION_TYPE_VIEW;
      current_notification->sub_type = NOTIFICATION_SUBTYPE_STACK;

      /* Ajout de la notification */
      window_stack->current_notification = current_notification;
      my_Notification(NOTIFICATION_ADD,current_notification,NULL);
    }
  else
    {
      /* Visuel */
      window_stack->refresh_mode = STACK_REFRESH_MANUAL;
      SendDlgItemMessage(window_stack->hwnd_stack,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_stack->hwnd_stack,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,1,0);

      /* Supprime la Notification */
      my_Notification(NOTIFICATION_REMOVE,window_stack->current_notification,NULL);
      window_stack->current_notification = NULL;
    }
}


/********************************************************************/
/*  RefreshStackWindowData() :  MAJ du contenu de la fentre Stack. */
/********************************************************************/
static void RefreshStackWindowData(struct window_stack *window_stack)
{
  int stack_address, stack_data_base_address, is_modified;
  struct cpu *current_cpu;

  /* Rcupre le CPU */
  my_Memory(MEMORY_GET_S65816_CPU,&current_cpu,NULL);

  /* Rcupre la valeur du Stack Pointer */
  stack_address = GetCPUData(CPU_ADDRESS_STACK,current_cpu);
  if((stack_address & 0x0000F0) <= 0x000070)
    stack_data_base_address = (stack_address & 0xFFFF0F);
  else
    stack_data_base_address = (stack_address & 0xFFFF0F) + 0x80;
    
  /** Les donnes ont t'elles t modifies + Rcupration des nouvelles Data si besoin **/
  is_modified = IsStackDataModified(stack_address,stack_data_base_address,window_stack);
  if(is_modified == 0)
    return;
    
  /** Cration de la Zone Texte  partir des nouvelles donnes **/
  BuildStackFromData(window_stack);

  /* Stack Pointer */
  sprintf(window_stack->buffer_log,"%02X/%04X",stack_address>>16,stack_address&0xFFFF);
  SetWindowText(GetDlgItem(window_stack->hwnd_stack,IDC_STATIC_STACKPOINTER),window_stack->buffer_log);

  /* Raifrachit la fentre */
  UpdateWindow(window_stack->hwnd_stack);
}


/***************************************************************************************************/
/*  IsStackDataModified() :  Les donnes Stack ont t'elles t modifies depuis la dernirer fois. */
/***************************************************************************************************/
static int IsStackDataModified(int stack_address, int stack_data_base_address, struct window_stack *window_stack)
{
  unsigned char stack_data[256];
  struct mmu *current_mmu;

  /* Rcupre le MMU */
  my_Memory(MEMORY_GET_S65816_MMU,&current_mmu,NULL);
  
  /* Rcupre les donnes */
  ReadMemoryBlock(stack_data_base_address,256,&stack_data[0],current_mmu);
  
  /* Compare les donnes */
  if(stack_address == window_stack->stack_address &&
     stack_data_base_address == window_stack->stack_data_base_address &&
     memcmp(&stack_data[0],&window_stack->stack_data[0],256) == 0)
    return(0);
    
  /* Les donnes sont diffrentes */
  window_stack->stack_address = stack_address;
  window_stack->stack_data_base_address = stack_data_base_address;
  memcpy(&window_stack->stack_data[0],&stack_data[0],256);
  return(1);
}


/*************************************************************/
/*  ResizeStackWindow() : Resize Stack window by given size. */
/*************************************************************/
static void ResizeStackWindow(struct window_stack *window_stack)
{
  RECT window_rect;
  int window_x, window_y;
  
  /* Largeur de la fentre */
  window_stack->window_width = WINDOW_BORDER_SIZE + window_stack->stack_data_x + window_stack->text_area_width + WINDOW_GROUP_BORDER_SIZE + WINDOW_CLIENT_BORDER_SIZE + WINDOW_BORDER_SIZE;

  /* Hauteur de la fentre */
  window_stack->window_height = window_stack->title_height + window_stack->stack_data_y + window_stack->text_area_height + WINDOW_GROUP_BORDER_SIZE + WINDOW_CLIENT_BORDER_SIZE;

  /* Position actuelle de la fentre dans l'cran */
  GetWindowRect(window_stack->hwnd_stack,&window_rect);
  window_x = window_rect.left;
  window_y = window_rect.top;

  /* Resize window */
  MoveWindow(window_stack->hwnd_stack,window_x,window_y,window_stack->window_width,window_stack->window_height,TRUE);
}


/*******************************************************************************************************/
/*  BuildStackFromData() :  Construire les donnes hxadcimal  afficher  partir des donnes brutes. */
/*******************************************************************************************************/
static void BuildStackFromData(struct window_stack *window_stack)
{
  int i, j, offset, offset_text, nb_byte_to_process;

  /*** Transformer les donnes BYTE en Hexa ***/
  nb_byte_to_process = window_stack->column*window_stack->line;
  for(i=0,offset=0,offset_text=0; i<window_stack->line; i++,offset+=16)
    {
      /* Adresse */
      sprintf(&window_stack->text_data_hexadecimal[offset_text],"%02X/%04X   ",(offset+window_stack->stack_data_base_address)>>16,(offset+window_stack->stack_data_base_address) & 0xFFFF);
      offset_text += strlen("00/0000   ");

      /* Hexa */
      for(j=0; j<window_stack->column; j++)
        {
          if(i*window_stack->column+j < nb_byte_to_process)
            sprintf(&window_stack->text_data_hexadecimal[offset_text],"%02X",window_stack->stack_data[offset+j]);
          else
            strcpy(&window_stack->text_data_hexadecimal[offset_text],"  ");
          offset_text += 2;

          /* Sparateur */
          if(j < (window_stack->column-1))
            {
              window_stack->text_data_hexadecimal[offset_text] = (j==7 && (i*window_stack->column+j < nb_byte_to_process)) ? '.' : ' ';
              offset_text++;
            }
        }

      /* Fin de ligne */
      strcpy(&window_stack->text_data_hexadecimal[offset_text],"\r\n");
      offset_text += 2;
    }
    
  /** Efface le contenu de la fentre **/
  SetWindowText(window_stack->hwnd_edit,"");

  /** Afficher le rsultat dans le control **/
  SetWindowText(window_stack->hwnd_edit,window_stack->text_data_hexadecimal);
}


/*******************************************************************/
/*  DeleteStackChildWindows() :  Destruction des fentres enfants. */
/*******************************************************************/
static void DeleteStackChildWindows(struct window_stack *window_stack)
{
  /** Libration des ressources **/
  DeleteObject(window_stack->hBitmap_stack_pointer);
  ReleaseDC(window_stack->hwnd_stack,window_stack->hDC);
  DeleteDC(window_stack->hCompatibleDC);  
  
  if(window_stack->text_data_hexadecimal != NULL)
    free(window_stack->text_data_hexadecimal);
  window_stack->text_data_hexadecimal = NULL;
}


/*********************************************************************/
/*  mem_free_window_stack() :  Libration mmoire pour la structure. */
/*********************************************************************/
void mem_free_window_stack(struct window_stack *window_stack)
{
  if(window_stack)
    {
      /* Supprime la notification */
      my_Notification(NOTIFICATION_REMOVE,window_stack->current_notification,NULL);
      window_stack->current_notification = NULL;
      
      free(window_stack);
    }
}

/**********************************************************************/
