/****************************************************************************/
/*                                                                          */
/*  Window_Disassembler.c :  Module de gestion de la fentre Disassembler.  */
/*                                                                          */
/****************************************************************************/
/*   Auteur : Olivier ZARDINI   *   Brutal Deluxe   *    Date : Aout 2018   */
/****************************************************************************/

#include <stdio.h>
#include <time.h>
#include <sys/timeb.h>

#include <windows.h>
#include <windowsx.h>
#include <Commctrl.h>
#include <Winuser.h>
#include <Richedit.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_Disassembler.h"
#include "Dc_Notification.h"
#include "Dc_Graphic.h"
#include "Dc_CPU.h"
#include "Dc_MMU.h"
#include "Cyrene_Engine.h"
#include "Window_QuickWatch.h"
#include "Window_Main.h"
#include "Window_Main_Menu.h"
#include "Window_Symbol.h"
#include "Window_SymbolPickup.h"
#include "Window_Disassembler.h"

static DWORD BuildDisassemblerWindowThread(void *);
LRESULT CALLBACK ProcHandlerDisassemblerWindow(HWND,UINT,WPARAM,LPARAM);
static int CreateDisassemblerChildWindows(HWND,struct window_disassembler *);
LRESULT CALLBACK subEditDisassemblerProcControl(HWND,UINT,WPARAM,LPARAM);
static int ComputePreviousDisassemblerAddress(struct window_disassembler *,int);
static void UpdateDisassemblerAddress(struct window_disassembler *,int);
static void UpdateDisassemblerRefreshMode(struct window_disassembler *,int);
static void UpdateDisassemblerProcessor(struct window_disassembler *,int);
static void UpdateDisassemblerRegister(struct window_disassembler *,int);
static void InsertRemoveDisassemblerDirective(struct window_disassembler *,int);
static void RefreshDisassemblerWindowData(struct window_disassembler *,int);
static int IsDisassemblerDataModified(int,int,int,struct window_disassembler *);
static void ResizeDisassemblerWindow(struct window_disassembler *);
static void BuildDisassemblerFromData(struct window_disassembler *);
static DWORD CALLBACK DisassemblerRichEditStreamCallback(DWORD *,LPBYTE,LONG,LONG *);
static void UserResizeDisassemblerWindow(struct window_disassembler *,int,int);
static void DeleteDisassemblerChildWindows(struct window_disassembler *);


/**************************************************************************************/
/*  BuildDisassemblerWindow() :  Construction / Affichage de la fentre Disassembler. */
/**************************************************************************************/
struct window_disassembler *BuildDisassemblerWindow(HINSTANCE hInstance, int menu_id, DWORD init_address)
{
  struct window_disassembler *window_disassembler;

  /** Allocation de la structure window_disassembler **/
  window_disassembler = (struct window_disassembler *) calloc(1,sizeof(struct window_disassembler));
  if(window_disassembler == NULL)
    return(NULL);
  window_disassembler->init_address = init_address;
  my_Memory(MEMORY_ADD_WINDOW_DISASSEMBLER,window_disassembler,NULL);

  /* Lance un thread pour grer la fentre Disassembler */
  CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)BuildDisassemblerWindowThread,window_disassembler,0,NULL);

  /* Renvoi la structure mmoire grant la fentre */
  return(window_disassembler);
}


/************************************************************************************************/
/*  BuildDisassemblerWindowThread() :  Cration de la fentre Disassembler via un Thread ddi. */
/************************************************************************************************/
static DWORD BuildDisassemblerWindowThread(void *window_disassembler_param)
{
  HWND hwndDisassembler;
  MSG msg;
  BOOL bRet;
  struct window_main *window_main;
  struct window_disassembler *window_disassembler;
  struct parameter *param;
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Init */
  window_disassembler = (struct window_disassembler *) window_disassembler_param;
  window_disassembler->thread_id = GetCurrentThreadId();         /* Le ThreadID sert  diffrencier les diffrentes fentres */

  /** Cration de la fentre Disassembler **/
  hwndDisassembler = CreateDialog(param->hInstance,MAKEINTRESOURCE(IDD_WINDOW_DISASSEMBLER),window_main->hwnd_main,(DLGPROC)ProcHandlerDisassemblerWindow);
  if(hwndDisassembler == NULL)
    return(1);

  /** Show Window + Paint Content **/
  ShowWindow(hwndDisassembler,SW_SHOWDEFAULT);
  UpdateWindow(hwndDisassembler);

  /** Message Loop pour toutes les fentres de ce Thread (fentre Disassembler + ComboBox + ...) **/
  while((bRet = GetMessage(&msg,NULL,0,0)) != 0)
    {
      if(bRet == -1)
        break;

      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }

  /* Fin du Thread */
  ExitThread(0);
}


/*****************************************************************************/
/*  ProcHandlerDisassemblerWindow() :  Gestion des vnements de la fentre. */
/*****************************************************************************/
LRESULT CALLBACK ProcHandlerDisassemblerWindow(HWND hwndDisassembler, UINT umsg, WPARAM wParam, LPARAM lParam)
{
  POINT pt;
  HMENU hMenuPopup;
  UINT size_type;
  WORD window_width, window_height;
  int i, mouse_x, mouse_y, is_directive, previous_address, current_address, mouse_wheel;
  MSGFILTER *message_filter;
  int first_line_size, current_page_size, previous_line_size, previous_page_size;
  struct code_line *current_code_line;
  struct window_disassembler *window_disassembler;
  struct parameter *param;
  
  /* Init */
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  my_Memory(MEMORY_GET_WINDOW_DISASSEMBLER,&window_disassembler,NULL);
  if(window_disassembler == NULL)
    return(TRUE);

  switch(umsg)
    {
      case WM_INITDIALOG :
        /* Cration du contenu de la DialogBox */
        CreateDisassemblerChildWindows(hwndDisassembler,window_disassembler);
        return(TRUE);

      /**********************************************/
      /* Evnements des Controles  la fentre mre */
      /**********************************************/
      case WM_NOTIFY :
        message_filter = (MSGFILTER *) lParam;
        
        /** Evnement de la souris dans le Rich Edit transmis  la fentre mre **/
        if(message_filter && message_filter->nmhdr.hwndFrom == window_disassembler->hwnd_richedit_data)
          {
            if(message_filter->msg == WM_RBUTTONDOWN)
              {
                /* Coodonnes de la souris dans la grille des caractres du Control Edit */
                mouse_x = GET_X_LPARAM(message_filter->lParam) / 7;
                mouse_y = GET_Y_LPARAM(message_filter->lParam) / DISASSEMBLER_RICHEDIT_FONT_HEIGHT;

                /* Adresse de la ligne Y */
                is_directive = 0;
                window_disassembler->context_directive_address = -1;
                for(i=0, current_code_line=window_disassembler->first_code_line; current_code_line; current_code_line=current_code_line->next, i++)
                  if(i == mouse_y)
                    {
                      window_disassembler->context_directive_address = current_code_line->address;
                      if(current_code_line->is_directive == 1)
                        is_directive = 1;
                      break;
                    }

                /* Coordonnes de la souris dans la fentre */
                pt.x= GET_X_LPARAM(message_filter->lParam) + window_disassembler->disassembler_data_x;
                pt.y= GET_Y_LPARAM(message_filter->lParam) + window_disassembler->disassembler_data_y;
                ClientToScreen(hwndDisassembler,&pt);

                /** Affiche le Menu contextuel **/
                hMenuPopup = CreatePopupMenu();
                AppendMenu(hMenuPopup,MF_STRING,IDM_CONTEXT_DISASSEMBLER_00,"Insert MX %00 directive");
                AppendMenu(hMenuPopup,MF_STRING,IDM_CONTEXT_DISASSEMBLER_01,"Insert MX %01 directive");
                AppendMenu(hMenuPopup,MF_STRING,IDM_CONTEXT_DISASSEMBLER_10,"Insert MX %10 directive");
                AppendMenu(hMenuPopup,MF_STRING,IDM_CONTEXT_DISASSEMBLER_11,"Insert MX %11 directive");
                if(is_directive == 1)
                  {
                    /* On ajoute la suppression */
                    AppendMenu(hMenuPopup,MF_SEPARATOR,0,NULL);
                    AppendMenu(hMenuPopup,MF_STRING,IDM_CONTEXT_DISASSEMBLER_CLEAR,"Delete MX directive");
                  }
                TrackPopupMenu(hMenuPopup,TPM_LEFTALIGN|TPM_RIGHTBUTTON,pt.x,pt.y,0,hwndDisassembler, NULL);
                DestroyMenu(hMenuPopup);
              }
            /** Roulette Souris => +/- Ligne **/
            else if(message_filter->msg == WM_MOUSEWHEEL)
              {
                mouse_wheel = GET_WHEEL_DELTA_WPARAM(message_filter->wParam);
                if(mouse_wheel < 0)
                  SendMessage(GetDlgItem(window_disassembler->hwnd_disassembler,IDC_BUTTON_DOWN),BM_CLICK,0,0);
                else if(mouse_wheel > 0)
                  SendMessage(GetDlgItem(window_disassembler->hwnd_disassembler,IDC_BUTTON_UP),BM_CLICK,0,0);
              }
            }
        break;
        return(FALSE);

      /***************************/
      /* Evnements de la souris */
      /***************************/     
      case WM_MOUSEWHEEL :
        mouse_wheel = GET_WHEEL_DELTA_WPARAM(wParam);   /* +/- 120 */
        if(mouse_wheel < 0)
          SendMessage(GetDlgItem(window_disassembler->hwnd_disassembler,IDC_BUTTON_DOWN),BM_CLICK,0,0);
        else if(mouse_wheel > 0)
          SendMessage(GetDlgItem(window_disassembler->hwnd_disassembler,IDC_BUTTON_UP),BM_CLICK,0,0);
        return(0);

      /****************************/
      /* Evnements de la fentre */
      /****************************/
      case WM_COMMAND :
        switch(LOWORD(wParam))
          {
            /** Bouton Radio : On change la mthode de rafraichissement de la fentre **/
            case IDC_RADIO_REFRESH_AUTOMATIC :
              if(HIWORD(wParam) == BN_CLICKED && window_disassembler->refresh_mode != DISASSEMBLER_REFRESH_AUTOMATIC)
                UpdateDisassemblerRefreshMode(window_disassembler,DISASSEMBLER_REFRESH_AUTOMATIC);
              break;

            case IDC_RADIO_REFRESH_MANUAL :
              if(HIWORD(wParam) == BN_CLICKED && window_disassembler->refresh_mode != DISASSEMBLER_REFRESH_MANUAL)
                UpdateDisassemblerRefreshMode(window_disassembler,DISASSEMBLER_REFRESH_MANUAL);
              break;

            /** Bouton Radio : On change le Processeur **/
            case IDC_RADIO_PROC_6502 :
              if(HIWORD(wParam) == BN_CLICKED && window_disassembler->processor != DISASSEMBLER_PROC_6502)
                UpdateDisassemblerProcessor(window_disassembler,DISASSEMBLER_PROC_6502);
              break;

            case IDC_RADIO_PROC_65C02 :
              if(HIWORD(wParam) == BN_CLICKED && window_disassembler->processor != DISASSEMBLER_PROC_65C02)
                UpdateDisassemblerProcessor(window_disassembler,DISASSEMBLER_PROC_65C02);
              break;

            case IDC_RADIO_PROC_65C816 :
              if(HIWORD(wParam) == BN_CLICKED && window_disassembler->processor != DISASSEMBLER_PROC_65C816)
                UpdateDisassemblerProcessor(window_disassembler,DISASSEMBLER_PROC_65C816);
              break;

            /** Bouton Radio : On change la taille des Registres **/
            case IDC_RADIO_REGISTER_00 :
              if(HIWORD(wParam) == BN_CLICKED && window_disassembler->radio_register_mx != DISASSEMBLER_REGISTER_00)
                UpdateDisassemblerRegister(window_disassembler,DISASSEMBLER_REGISTER_00);
              break;

            case IDC_RADIO_REGISTER_01 :
              if(HIWORD(wParam) == BN_CLICKED && window_disassembler->radio_register_mx != DISASSEMBLER_REGISTER_01)
                UpdateDisassemblerRegister(window_disassembler,DISASSEMBLER_REGISTER_01);
              break;

            case IDC_RADIO_REGISTER_10 :
              if(HIWORD(wParam) == BN_CLICKED && window_disassembler->radio_register_mx != DISASSEMBLER_REGISTER_10)
                UpdateDisassemblerRegister(window_disassembler,DISASSEMBLER_REGISTER_10);
              break;

            case IDC_RADIO_REGISTER_11 :
              if(HIWORD(wParam) == BN_CLICKED && window_disassembler->radio_register_mx != DISASSEMBLER_REGISTER_11)
                UpdateDisassemblerRegister(window_disassembler,DISASSEMBLER_REGISTER_11);
              break;

            /** Bouton de slection de l'adresse via Symbol **/
            case IDC_BUTTON_SYMBOL :
              BuildSymbolPickupWindow(param->hInstance,window_disassembler->hwnd_disassembler,HM_DISASSEMBLER_REFRESHSYMBOL);
              break;

            /** Bouton de dplacement de l'adresse **/
            case IDC_BUTTON_LEFT :
              window_disassembler->current_register_mx = window_disassembler->radio_register_mx;
              UpdateDisassemblerAddress(window_disassembler,-1);
              break;

            case IDC_BUTTON_RIGHT :
              window_disassembler->current_register_mx = window_disassembler->radio_register_mx;
              UpdateDisassemblerAddress(window_disassembler,1);
              break;

            case IDC_BUTTON_UP :
              if(window_disassembler->first_code_line != NULL)
                {
                  /* Calcul l'adresse */
                  current_address = (window_disassembler->disassembler_bank << 16) | window_disassembler->disassembler_address;
                  if(current_address == 0)
                    break;
                  previous_address = ComputePreviousDisassemblerAddress(window_disassembler,IDC_BUTTON_UP);
                  
                  /* RAZ RegisterMX */
                  window_disassembler->current_register_mx = window_disassembler->radio_register_mx;
                    
                  /* On dplace l'adresse d'une Ligne */ 
                  previous_line_size = current_address - previous_address;
                  UpdateDisassemblerAddress(window_disassembler,(-1)*previous_line_size);
                }
              break;

            case IDC_BUTTON_DOWN :
              if(window_disassembler->first_code_line != NULL)
                {
                  first_line_size = window_disassembler->first_code_line->data_size;
                  if(window_disassembler->first_code_line->next != NULL)
                    window_disassembler->current_register_mx = window_disassembler->first_code_line->next->m*2 + window_disassembler->first_code_line->next->x;
                  UpdateDisassemblerAddress(window_disassembler,first_line_size);
                }
              break;

            case IDC_BUTTON_PAGEUP :
              if(window_disassembler->first_code_line != NULL)
                {
                  /* Calcul l'adresse */
                  current_address = (window_disassembler->disassembler_bank << 16) | window_disassembler->disassembler_address;
                  if(current_address == 0)
                    break;
                  previous_address = ComputePreviousDisassemblerAddress(window_disassembler,IDC_BUTTON_PAGEUP);

                  /* RAZ RegisterMX */
                  window_disassembler->current_register_mx = window_disassembler->radio_register_mx;
                    
                  /* On dplace l'adresse d'une Page */
                  previous_page_size = current_address - previous_address;
                  UpdateDisassemblerAddress(window_disassembler,(-1)*previous_page_size);
                }
              break;

            case IDC_BUTTON_PAGEDOWN :
              if(window_disassembler->first_code_line != NULL)
                {
                  current_page_size = 0;
                  for(current_code_line = window_disassembler->first_code_line; current_code_line; current_code_line = current_code_line->next)
                    current_page_size += current_code_line->data_size;
                  UpdateDisassemblerAddress(window_disassembler,current_page_size);
                }
              break;

            /** Bouton de rafraichissement Manuel de la fentre **/
            case IDC_BUTTON_REFRESH :
              window_disassembler->current_register_mx = window_disassembler->radio_register_mx;
              RefreshDisassemblerWindowData(window_disassembler,1);
              break;              

            /** Choix dans le menu contextuel : Ajout ou Suppression d'une Directive **/
            case IDM_CONTEXT_DISASSEMBLER_00 :
            case IDM_CONTEXT_DISASSEMBLER_01 :
            case IDM_CONTEXT_DISASSEMBLER_10 :
            case IDM_CONTEXT_DISASSEMBLER_11 :
            case IDM_CONTEXT_DISASSEMBLER_CLEAR :
              if(window_disassembler->context_directive_address != -1)
                InsertRemoveDisassemblerDirective(window_disassembler,(int)LOWORD(wParam));
              break;
          }
        return(TRUE);

      /**************************************/
      /*  On vient de retailler la fentre  */
      /**************************************/
      case WM_SIZE :
        /* Nouvelle taille de la fentre */
        size_type = (UINT) wParam;
        window_width = LOWORD(lParam);
        window_height = HIWORD(lParam);        
      
        /* On va tendre la fentre de Dsassemblage */
        UserResizeDisassemblerWindow(window_disassembler,window_width,window_height);
        return(TRUE);

      /*****************************************************************/
      /* Message extrieur demandant le rafraichissement de la fentre */
      /*****************************************************************/
      case HM_DISASSEMBLER_REFRESHWINDOW :
        RefreshDisassemblerWindowData(window_disassembler,0);
        return(TRUE);

      /** On vient de demander la MAJ de l'adresse via un Symbol **/
      case HM_DISASSEMBLER_REFRESHSYMBOL :
        if(param->symbol_is_available == 1)
          {
            window_disassembler->disassembler_bank = (param->symbol_address >> 16) & 0x000000FF;
            window_disassembler->disassembler_address = param->symbol_address & 0x0000FFFF;
            UpdateDisassemblerAddress(window_disassembler,0);
          }
        return(TRUE);

      /********************************************************************/
      /* Fermeture de window (le WM_CLOSE est appel avant le WM_DESTROY) */
      /********************************************************************/
      case WM_CLOSE :
        /* Fentre Disassembler */
        DestroyWindow(hwndDisassembler);
        return(0);

      case WM_DESTROY :
        /* Suppression des Fentres */
        DeleteDisassemblerChildWindows(window_disassembler);

        /* Supprime la structure mmoire de ce thread */
        my_Memory(MEMORY_DELETE_WINDOW_DISASSEMBLER,window_disassembler,NULL);

        /* Demande la sortie de la boucle des messages */
        PostQuitMessage(0);
        return(0);
    }

  /* Retourner FALSE pour relayer le message non trait au systme */
  return(FALSE);
}


/***************************************************************************************************************/
/*  CreateDisassemblerChildWindows() : Crer les fentres enfants et remplir la structure window_disassembler. */
/***************************************************************************************************************/
static int CreateDisassemblerChildWindows(HWND hwndDisassembler, struct window_disassembler *window_disassembler)
{
  HWND hwnd_group;
  DWORD dialog_background_color;
  RECT client_rect, window_rect, group_rect;
  char buffer[BUFFER_SIZE];
  struct s65816 *current_65816;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Rcupration des structures */
  my_Memory(MEMORY_GET_S65816,&current_65816,NULL);

  /** Window Handle **/
  window_disassembler->hwnd_disassembler = hwndDisassembler;

  /** Surchage des Control Edit pour capter les RETURN lors des saisies **/
  window_disassembler->oldEditProcBank     = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndDisassembler,IDC_EDIT_BANK),GWLP_WNDPROC,(LONG_PTR)subEditDisassemblerProcControl);
  window_disassembler->oldEditProcAddress  = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndDisassembler,IDC_EDIT_ADDRESS),GWLP_WNDPROC,(LONG_PTR)subEditDisassemblerProcControl);
  window_disassembler->oldEditProcRow      = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndDisassembler,IDC_EDIT_NBLINE),GWLP_WNDPROC,(LONG_PTR)subEditDisassemblerProcControl);
  window_disassembler->oldEditProcDataBank = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndDisassembler,IDC_EDIT_DATA_BANK),GWLP_WNDPROC,(LONG_PTR)subEditDisassemblerProcControl);

  /** Ajoute les images sur les boutons **/
  SendMessage(GetDlgItem(hwndDisassembler,IDC_BUTTON_SYMBOL),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_SYMBOL)));
  SendMessage(GetDlgItem(hwndDisassembler,IDC_BUTTON_LEFT),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_LEFT)));
  SendMessage(GetDlgItem(hwndDisassembler,IDC_BUTTON_RIGHT),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_RIGHT)));
  SendMessage(GetDlgItem(hwndDisassembler,IDC_BUTTON_UP),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_UP)));
  SendMessage(GetDlgItem(hwndDisassembler,IDC_BUTTON_DOWN),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_DOWN)));
  SendMessage(GetDlgItem(hwndDisassembler,IDC_BUTTON_PAGEUP),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_PAGE_UP)));
  SendMessage(GetDlgItem(hwndDisassembler,IDC_BUTTON_PAGEDOWN),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_PAGE_DOWN)));

  /** Adresse de la Zone de Data **/
  if(window_disassembler->init_address != 0xFFFFFFFF)
    {
      window_disassembler->disassembler_bank = (window_disassembler->init_address & 0x00FF0000) >> 16;
      window_disassembler->disassembler_address = window_disassembler->init_address & 0x0000FFFF;
      window_disassembler->disassembler_data_bank = window_disassembler->disassembler_bank;
    }
  else
    {
      window_disassembler->disassembler_bank = current_65816->current_cpu->proc_pbr;
      window_disassembler->disassembler_address = current_65816->current_cpu->proc_pc;  /* Adresse de dpart du dsassemblage */
      window_disassembler->disassembler_data_bank = window_disassembler->disassembler_bank;
    }
  sprintf(buffer,"%02X",window_disassembler->disassembler_bank);
  SetDlgItemText(window_disassembler->hwnd_disassembler,IDC_EDIT_BANK,buffer);
  sprintf(buffer,"%04X",window_disassembler->disassembler_address);
  SetDlgItemText(window_disassembler->hwnd_disassembler,IDC_EDIT_ADDRESS,buffer);
  sprintf(buffer,"%02X",window_disassembler->disassembler_data_bank);
  SetDlgItemText(window_disassembler->hwnd_disassembler,IDC_EDIT_DATA_BANK,buffer);
  
  /** Processor : 65c816 / 65c02 / 6502 **/
  window_disassembler->processor = DISASSEMBLER_PROC_65C816;
  if(window_disassembler->processor == DISASSEMBLER_PROC_65C816)
    {
      SendDlgItemMessage(window_disassembler->hwnd_disassembler,IDC_RADIO_PROC_6502,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_disassembler->hwnd_disassembler,IDC_RADIO_PROC_65C02,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_disassembler->hwnd_disassembler,IDC_RADIO_PROC_65C816,BM_SETCHECK,1,0);
    }
  else
    {
      SendDlgItemMessage(window_disassembler->hwnd_disassembler,IDC_RADIO_PROC_6502,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_disassembler->hwnd_disassembler,IDC_RADIO_PROC_65C02,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_disassembler->hwnd_disassembler,IDC_RADIO_PROC_65C816,BM_SETCHECK,0,0);    
    }
    
  /** Register : 00 -> 11 **/
  window_disassembler->radio_register_mx = (current_65816->current_cpu->status_m << 1) | current_65816->current_cpu->status_x;
  if(window_disassembler->processor == DISASSEMBLER_PROC_6502 || window_disassembler->processor == DISASSEMBLER_PROC_65C02)
    window_disassembler->radio_register_mx = DISASSEMBLER_REGISTER_11;
  /* Affichage */
  SendDlgItemMessage(window_disassembler->hwnd_disassembler,IDC_RADIO_REGISTER_00,BM_SETCHECK,0,0);
  SendDlgItemMessage(window_disassembler->hwnd_disassembler,IDC_RADIO_REGISTER_01,BM_SETCHECK,0,0);
  SendDlgItemMessage(window_disassembler->hwnd_disassembler,IDC_RADIO_REGISTER_10,BM_SETCHECK,0,0);
  SendDlgItemMessage(window_disassembler->hwnd_disassembler,IDC_RADIO_REGISTER_11,BM_SETCHECK,0,0);
  if(window_disassembler->radio_register_mx == DISASSEMBLER_REGISTER_00)
    SendDlgItemMessage(window_disassembler->hwnd_disassembler,IDC_RADIO_REGISTER_00,BM_SETCHECK,1,0);
  else if(window_disassembler->radio_register_mx == DISASSEMBLER_REGISTER_01)
    SendDlgItemMessage(window_disassembler->hwnd_disassembler,IDC_RADIO_REGISTER_01,BM_SETCHECK,1,0);
  else if(window_disassembler->radio_register_mx == DISASSEMBLER_REGISTER_10)
    SendDlgItemMessage(window_disassembler->hwnd_disassembler,IDC_RADIO_REGISTER_10,BM_SETCHECK,1,0);
  else if(window_disassembler->radio_register_mx == DISASSEMBLER_REGISTER_11)
    SendDlgItemMessage(window_disassembler->hwnd_disassembler,IDC_RADIO_REGISTER_11,BM_SETCHECK,1,0);

  /** Zone d'affichage des Disassembler **/
  window_disassembler->text_min_row = 5;
  window_disassembler->nb_row = 24;
  SetDlgItemInt(window_disassembler->hwnd_disassembler,IDC_EDIT_NBLINE,window_disassembler->nb_row,FALSE);

  /* Nombre caractres visibles sur la ligne : Address Data Ascii (on ne compte pas le \r\n final) */
  window_disassembler->text_data_width = (int) (strlen(DISASSEMBLER_AREA_MX) + 
                                                strlen(DISASSEMBLER_AREA_ADDRESS) + 
                                                strlen(DISASSEMBLER_AREA_DATA) + 
                                                strlen(DISASSEMBLER_AREA_LABEL) + 
                                                strlen(DISASSEMBLER_AREA_OPCODE) + 
                                                strlen(DISASSEMBLER_AREA_OPERAND) + 
                                                strlen(DISASSEMBLER_AREA_COMMENT));

  /* Hauteur Titre */
  window_disassembler->window_border_size = GetSystemMetrics(SM_CXSIZEFRAME);   /* Win 7 = 4, Win 10 = 8 */
  GetWindowRect(window_disassembler->hwnd_disassembler,&window_rect);
  GetClientRect(window_disassembler->hwnd_disassembler,&client_rect);
  window_disassembler->title_height = (window_rect.bottom - window_rect.top) - client_rect.bottom - window_disassembler->window_border_size; /* Win 7 = 23 */

  /* Position de la GroupBox Select + GroupBox Palette */
  hwnd_group = GetDlgItem(window_disassembler->hwnd_disassembler,IDC_GROUP_DISASSEMBLER);     /* Rcupre le Handle du Control GroupBox li  IDC_GROUP_DISASSEMBLER */
  GetWindowRect(hwnd_group,&group_rect);
  window_disassembler->group_data_x = group_rect.left - window_rect.left;
  window_disassembler->group_data_y = (group_rect.top - window_rect.top) - window_disassembler->title_height;

  /* Position de la Zone Text Direct Page Data dans la fentre */
  window_disassembler->disassembler_data_x = window_disassembler->group_data_x + WINDOW_GROUP_BORDER_LEFT_SIZE + 2;
  window_disassembler->disassembler_data_y = window_disassembler->group_data_y + WINDOW_GROUP_BORDER_TOP_SIZE;

  /* Dimension en Pixel du Control RichEdit MemoryArea Data */
  window_disassembler->disassembler_data_width = window_disassembler->text_data_width*7 + 2;
  window_disassembler->disassembler_data_height = window_disassembler->nb_row*DISASSEMBLER_RICHEDIT_FONT_HEIGHT;

  /* Dimension en Pixel du Control Group Box + Resize */  
  window_disassembler->group_data_width = WINDOW_GROUP_BORDER_LEFT_SIZE + window_disassembler->disassembler_data_width + WINDOW_GROUP_BORDER_RIGHT_SIZE;
  window_disassembler->group_data_height = WINDOW_GROUP_BORDER_TOP_SIZE + window_disassembler->disassembler_data_height + WINDOW_GROUP_BORDER_BOTTOM_SIZE;
  MoveWindow(hwnd_group,window_disassembler->group_data_x,window_disassembler->group_data_y,window_disassembler->group_data_width,window_disassembler->group_data_height,FALSE);

  /** Cration du Rich Edit **/
  window_disassembler->hwnd_richedit_data = CreateWindowEx(0,RICHEDIT_CLASS,"",WS_CHILD | WS_VISIBLE | ES_SAVESEL | ES_MULTILINE | ES_READONLY, 
                                                           window_disassembler->disassembler_data_x, window_disassembler->disassembler_data_y, window_disassembler->disassembler_data_width, window_disassembler->disassembler_data_height,   // set size in WM_SIZE message 
                                                           window_disassembler->hwnd_disassembler,
                                                           (HMENU)IDC_RICHEDIT_DISASSEMBLER_DATA,
                                                           param->hInstance,
                                                           NULL);

  /* Couleur de Fond du control Rich Edit : Gris comme la fentre d'accueil */
  dialog_background_color = GetSysColor(COLOR_3DFACE);
  SendMessage(window_disassembler->hwnd_richedit_data,EM_SETBKGNDCOLOR,0,(LPARAM)dialog_background_color); 

  /* On demance  ce que les Event Souris du RichEdit soient envoys  la fentre principale : WM_NOTIFY */
  SendMessage(window_disassembler->hwnd_richedit_data,EM_SETEVENTMASK,0,(LPARAM)ENM_MOUSEEVENTS);

  /* Pour l'instant, aucune ligne de code n'a t pointe */
  window_disassembler->context_directive_address = -1;

  /* Retaille la fentre */
  ResizeDisassemblerWindow(window_disassembler);

  /** Construction du Texte + Rafraichissement de la fentre **/
  window_disassembler->current_register_mx = window_disassembler->radio_register_mx;
  RefreshDisassemblerWindowData(window_disassembler,1);
  
  /* Refresh Mode : Automatic [Ajoute la Notification] */
  UpdateDisassemblerRefreshMode(window_disassembler,DISASSEMBLER_REFRESH_AUTOMATIC);

  /* OK */
  return(0);
}


/*********************************************************************************************/
/*  subEditDisassemblerProcControl() :  Rcupre le RETURN pour valider les donnes saisies. */
/*********************************************************************************************/
LRESULT CALLBACK subEditDisassemblerProcControl(HWND hwnd_control, UINT msg, WPARAM wParam, LPARAM lParam)
{
  int index;
  struct window_disassembler *window_disassembler;
  my_Memory(MEMORY_GET_WINDOW_DISASSEMBLER,&window_disassembler,NULL);

  /* Protection */
  if(window_disassembler == NULL)
      return(0);

  /** Identifie le Control **/
  /* Src */
  if(GetDlgItem(window_disassembler->hwnd_disassembler,IDC_EDIT_BANK) == hwnd_control)
    index = 1;
  else if(GetDlgItem(window_disassembler->hwnd_disassembler,IDC_EDIT_ADDRESS) == hwnd_control)
    index = 2;
  else if(GetDlgItem(window_disassembler->hwnd_disassembler,IDC_EDIT_NBLINE) == hwnd_control)
    index = 3;
  else if(GetDlgItem(window_disassembler->hwnd_disassembler,IDC_EDIT_DATA_BANK) == hwnd_control)
    index = 4;
  else
    return(0);      /* Pas trouv */

  /** Dtecte la touche RETURN pour valider la donne **/
  if(msg == WM_CHAR && wParam == VK_RETURN)
    {
      /* Enlve le focus de l'Edit */
      SetFocus(window_disassembler->hwnd_disassembler);

      /* On force la MAJ de la fentre */
      window_disassembler->current_register_mx = window_disassembler->radio_register_mx;
      RefreshDisassemblerWindowData(window_disassembler,1); 
    }
  else if(index == 1)
    return(CallWindowProc(window_disassembler->oldEditProcBank,hwnd_control,msg,wParam,lParam)); 
  else if(index == 2)
    return(CallWindowProc(window_disassembler->oldEditProcAddress,hwnd_control,msg,wParam,lParam)); 
  else if(index == 3)
    return(CallWindowProc(window_disassembler->oldEditProcRow,hwnd_control,msg,wParam,lParam));
  else if(index == 4)
    return(CallWindowProc(window_disassembler->oldEditProcDataBank,hwnd_control,msg,wParam,lParam));
      
  /* OK */
  return(0);
}


/******************************************************************************************************/
/*  ComputePreviousDisassemblerAddress() :  Calcul l'adresse prcdente pour la fentre Disassembler. */
/******************************************************************************************************/
static int ComputePreviousDisassemblerAddress(struct window_disassembler *window_disassembler, int direction)
{
  int i, base_address, previous_address, current_address, found, current_register_mx, data_bank;
  int quality_coef_0, quality_coef_1, quality_coef_2, quality_coef_3, quality_coef_best;
  struct code_line *first_codeline_0;
  struct code_line *first_codeline_1;
  struct code_line *first_codeline_2;
  struct code_line *first_codeline_3;
  struct code_line *previous_codeline;  
  struct code_line *current_codeline;
  struct code_line *next_codeline;
  struct code_line *best_codeline;
  unsigned char data[1024];
  struct mmu *current_mmu;

  /* Rcupre le MMU */
  my_Memory(MEMORY_GET_S65816_MMU,&current_mmu,NULL);
  
  /* Adresse actuelle */
  current_address = (window_disassembler->disassembler_bank << 16) | window_disassembler->disassembler_address;
  base_address = current_address - (window_disassembler->nb_row+1)*4;    /* Max 4 bytes / Lignes */
  if(base_address < 0)
    base_address = 0;
  data_bank = window_disassembler->disassembler_data_bank;

  /* Rcupre 1024 bytes */
  ReadMemoryBlock(base_address,1024,&data[0],current_mmu);
  
  /** On va dsassembler les 4 offsets possibles **/
  current_register_mx = window_disassembler->radio_register_mx;
  first_codeline_0 = DisassembleDataBlock(1024,&data[0],0,current_address,base_address+0,data_bank,window_disassembler->processor,&current_register_mx,window_disassembler->first_directive_line,&quality_coef_0);
  current_register_mx = window_disassembler->radio_register_mx;
  first_codeline_1 = DisassembleDataBlock(1024,&data[1],0,current_address,base_address+1,data_bank,window_disassembler->processor,&current_register_mx,window_disassembler->first_directive_line,&quality_coef_1);
  current_register_mx = window_disassembler->radio_register_mx;
  first_codeline_2 = DisassembleDataBlock(1024,&data[2],0,current_address,base_address+2,data_bank,window_disassembler->processor,&current_register_mx,window_disassembler->first_directive_line,&quality_coef_2);
  current_register_mx = window_disassembler->radio_register_mx;
  first_codeline_3 = DisassembleDataBlock(1024,&data[3],0,current_address,base_address+3,data_bank,window_disassembler->processor,&current_register_mx,window_disassembler->first_directive_line,&quality_coef_3);

  /** On va liminer ceux qui ne retombe pas sur current_address **/
  for(found=0, current_codeline = first_codeline_0; current_codeline; current_codeline = current_codeline->next)
    if(current_codeline->address == current_address)
      {
        found = 1;
        break;
      }
  if(found == 0)
    {
      for(current_codeline = first_codeline_0; current_codeline; current_codeline = next_codeline)
        {
          next_codeline = current_codeline->next;
          mem_free_codeline(current_codeline);
        }
      first_codeline_0 = NULL;
    }
  for(found=0, current_codeline = first_codeline_1; current_codeline; current_codeline = current_codeline->next)
    if(current_codeline->address == current_address)
      {
        found = 1;
        break;
      }
  if(found == 0)
    {
      for(current_codeline = first_codeline_1; current_codeline; current_codeline = next_codeline)
        {
          next_codeline = current_codeline->next;
          mem_free_codeline(current_codeline);
        }
      first_codeline_1 = NULL;
    }  
  for(found=0, current_codeline = first_codeline_2; current_codeline; current_codeline = current_codeline->next)
    if(current_codeline->address == current_address)
      {
        found = 1;
        break;
      }
  if(found == 0)
    {
      for(current_codeline = first_codeline_2; current_codeline; current_codeline = next_codeline)
        {
          next_codeline = current_codeline->next;
          mem_free_codeline(current_codeline);
        }
      first_codeline_2 = NULL;
    } 
  for(found=0, current_codeline = first_codeline_3; current_codeline; current_codeline = current_codeline->next)
    if(current_codeline->address == current_address)
      {
        found = 1;
        break;
      }
  if(found == 0)
    {
      for(current_codeline = first_codeline_3; current_codeline; current_codeline = next_codeline)
        {
          next_codeline = current_codeline->next;
          mem_free_codeline(current_codeline);
        }
      first_codeline_3 = NULL;
    } 
  
  /** On va slectionner le meilleur qui reste **/
  quality_coef_best = -1000;
  best_codeline = NULL;
  if(first_codeline_0 != NULL && quality_coef_0 > quality_coef_best)
    {
      best_codeline = first_codeline_0;
      quality_coef_best = quality_coef_0;
    }
  if(first_codeline_1 != NULL && quality_coef_1 > quality_coef_best)
    {
      best_codeline = first_codeline_1;
      quality_coef_best = quality_coef_1;
    }
  if(first_codeline_2 != NULL && quality_coef_2 > quality_coef_best)
    {
      best_codeline = first_codeline_2;
      quality_coef_best = quality_coef_2;
    }
  if(first_codeline_3 != NULL && quality_coef_3 > quality_coef_best)
    {
      best_codeline = first_codeline_3;
      quality_coef_best = quality_coef_3;
    }

  /** Libration mmoire des dsassemblages inutiles **/
  if(first_codeline_0 != NULL && best_codeline != first_codeline_0)
    {
      for(current_codeline = first_codeline_0; current_codeline; current_codeline = next_codeline)
        {
          next_codeline = current_codeline->next;
          mem_free_codeline(current_codeline);
        }
      first_codeline_0 = NULL;
    } 
  if(first_codeline_1 != NULL && best_codeline != first_codeline_1)
    {
      for(current_codeline = first_codeline_1; current_codeline; current_codeline = next_codeline)
        {
          next_codeline = current_codeline->next;
          mem_free_codeline(current_codeline);
        }
      first_codeline_1 = NULL;
    } 
  if(first_codeline_2 != NULL && best_codeline != first_codeline_2)
    {
      for(current_codeline = first_codeline_2; current_codeline; current_codeline = next_codeline)
        {
          next_codeline = current_codeline->next;
          mem_free_codeline(current_codeline);
        }
      first_codeline_2 = NULL;
    } 
  if(first_codeline_3 != NULL && best_codeline != first_codeline_3)
    {
      for(current_codeline = first_codeline_3; current_codeline; current_codeline = next_codeline)
        {
          next_codeline = current_codeline->next;
          mem_free_codeline(current_codeline);
        }
      first_codeline_3 = NULL;
    } 
  
  /** On n'a aucun dsassemblage valide : Au Pif **/
  if(best_codeline == NULL)
    {
      if(direction == IDC_BUTTON_PAGEUP)
        {
          previous_address = (current_address - window_disassembler->nb_row);
          return((previous_address<0)?0:previous_address);
        }
      if(direction == IDC_BUTTON_UP)
        return(current_address-1);
    }
  
  /** On va se mettre sur la bonne ligne **/
  for(current_codeline = best_codeline; current_codeline; current_codeline = current_codeline->next)
    if(current_codeline->address == current_address)
      {
        /* Une Ligne au dessus */
        if(direction == IDC_BUTTON_UP)
          {
            previous_address = current_codeline->previous->address;
            break;
          }
          
        /* Une Page au dessus */
        if(direction == IDC_BUTTON_PAGEUP)
          {
            previous_codeline = current_codeline->previous;
            for(i=0; i<window_disassembler->nb_row; i++)
              {
                previous_address = previous_codeline->address;
                previous_codeline = previous_codeline->previous;
                if(previous_codeline == NULL)
                  break;
              }
            break;
          }
      }
  
  /* Renvoi cette adresse */
  return(previous_address);
}


/***********************************************************************************/
/*  UpdateDisassemblerAddress() :  Dplace l'adresse pour la fentre Disassembler. */
/***********************************************************************************/
static void UpdateDisassemblerAddress(struct window_disassembler *window_disassembler, int delta)
{
  int current_address, new_address, disassembler_bank, disassembler_address;
  char buffer[BUFFER_SIZE];

  /* Adresse actuelle */
  current_address = (window_disassembler->disassembler_bank << 16) | window_disassembler->disassembler_address;
  if((current_address + delta) < 0)
    return;
  if((current_address + delta) > 0xFFFFFF)
    return;

  /* Nouvelle Addresse */
  new_address = current_address + delta;
  disassembler_bank = (new_address >> 16) & 0xFF;
  disassembler_address = new_address & 0xFFFF;

  /* Replace les valeurs dans les Control pour montrer ce qui a t compris */
  sprintf(buffer,"%02X",disassembler_bank);
  SetDlgItemText(window_disassembler->hwnd_disassembler,IDC_EDIT_BANK,buffer);
  sprintf(buffer,"%04X",disassembler_address);
  SetDlgItemText(window_disassembler->hwnd_disassembler,IDC_EDIT_ADDRESS,buffer);

  /* MAJ de la fentre */
  RefreshDisassemblerWindowData(window_disassembler,1);
}


/****************************************************************************************************/
/*  UpdateDisassemblerRefreshMode() :  MAJ du mode de rafraichissement de la fenetre Disassembler. */
/****************************************************************************************************/
static void UpdateDisassemblerRefreshMode(struct window_disassembler *window_disassembler, int refresh_mode)
{
  struct notification *current_notification;

  if(refresh_mode == DISASSEMBLER_REFRESH_AUTOMATIC)
    {
      /* Visuel */
      window_disassembler->refresh_mode = DISASSEMBLER_REFRESH_AUTOMATIC;
      SendDlgItemMessage(window_disassembler->hwnd_disassembler,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_disassembler->hwnd_disassembler,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,0,0);

      /* Cration d'une Notification */
      current_notification = (struct notification *) calloc(1,sizeof(struct notification));
      if(current_notification == NULL)
        return;
      current_notification->hwnd_window = window_disassembler->hwnd_disassembler;
      current_notification->message = HM_DISASSEMBLER_REFRESHWINDOW;
      current_notification->type = NOTIFICATION_TYPE_VIEW;
      current_notification->sub_type = NOTIFICATION_SUBTYPE_DISASSEMBLER;

      /* Ajout de la notification */
      window_disassembler->current_notification = current_notification;
      my_Notification(NOTIFICATION_ADD,current_notification,NULL);
    }
  else
    {
      /* Visuel */
      window_disassembler->refresh_mode = DISASSEMBLER_REFRESH_MANUAL;
      SendDlgItemMessage(window_disassembler->hwnd_disassembler,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_disassembler->hwnd_disassembler,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,1,0);

      /* Supprime la Notification */
      my_Notification(NOTIFICATION_REMOVE,window_disassembler->current_notification,NULL);
      window_disassembler->current_notification = NULL;
    }
}


/**********************************************************************************/
/*  UpdateDisassemblerProcessor() :  MAJ du Processor de la fentre Disassembler. */
/**********************************************************************************/
static void UpdateDisassemblerProcessor(struct window_disassembler *window_disassembler, int processor)
{
  if(processor == DISASSEMBLER_PROC_6502)
    {
      /* Visuel */
      window_disassembler->processor = DISASSEMBLER_PROC_6502;
      SendDlgItemMessage(window_disassembler->hwnd_disassembler,IDC_RADIO_PROC_6502,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_disassembler->hwnd_disassembler,IDC_RADIO_PROC_65C02,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_disassembler->hwnd_disassembler,IDC_RADIO_PROC_65C816,BM_SETCHECK,0,0);
    }
  else if(processor == DISASSEMBLER_PROC_65C02)
    {
      /* Visuel */
      window_disassembler->processor = DISASSEMBLER_PROC_65C02;
      SendDlgItemMessage(window_disassembler->hwnd_disassembler,IDC_RADIO_PROC_6502,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_disassembler->hwnd_disassembler,IDC_RADIO_PROC_65C02,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_disassembler->hwnd_disassembler,IDC_RADIO_PROC_65C816,BM_SETCHECK,0,0);
    }
  else if(processor == DISASSEMBLER_PROC_65C816)
    {
      /* Visuel */
      window_disassembler->processor = DISASSEMBLER_PROC_65C816;
      SendDlgItemMessage(window_disassembler->hwnd_disassembler,IDC_RADIO_PROC_6502,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_disassembler->hwnd_disassembler,IDC_RADIO_PROC_65C02,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_disassembler->hwnd_disassembler,IDC_RADIO_PROC_65C816,BM_SETCHECK,1,0);
    }
  
  /* On force Register Size  11 */
  if(processor == DISASSEMBLER_PROC_6502 || processor == DISASSEMBLER_PROC_65C02)
    {
      window_disassembler->radio_register_mx = DISASSEMBLER_REGISTER_11;
      SendDlgItemMessage(window_disassembler->hwnd_disassembler,IDC_RADIO_REGISTER_00,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_disassembler->hwnd_disassembler,IDC_RADIO_REGISTER_01,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_disassembler->hwnd_disassembler,IDC_RADIO_REGISTER_10,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_disassembler->hwnd_disassembler,IDC_RADIO_REGISTER_11,BM_SETCHECK,1,0);
    }

  /* On force la MAJ de la fentre */
  window_disassembler->current_register_mx = window_disassembler->radio_register_mx;
  RefreshDisassemblerWindowData(window_disassembler,1);    
}


/**************************************************************************/
/*  UpdateDisassemblerRegister() :  MAJ du MX de la fentre Disassembler. */
/**************************************************************************/
static void UpdateDisassemblerRegister(struct window_disassembler *window_disassembler, int register_mx)
{
  /* On ne peut pas changer la taille des registres en 6502 */
  if(window_disassembler->processor == DISASSEMBLER_PROC_6502 || window_disassembler->processor == DISASSEMBLER_PROC_65C02)
    return;

  if(register_mx == DISASSEMBLER_REGISTER_00)
    {
      /* Visuel */
      window_disassembler->radio_register_mx = DISASSEMBLER_REGISTER_00;
      SendDlgItemMessage(window_disassembler->hwnd_disassembler,IDC_RADIO_REGISTER_00,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_disassembler->hwnd_disassembler,IDC_RADIO_REGISTER_01,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_disassembler->hwnd_disassembler,IDC_RADIO_REGISTER_10,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_disassembler->hwnd_disassembler,IDC_RADIO_REGISTER_11,BM_SETCHECK,0,0);
    }
  else if(register_mx == DISASSEMBLER_REGISTER_01)
    {
      /* Visuel */
      window_disassembler->radio_register_mx = DISASSEMBLER_REGISTER_01;
      SendDlgItemMessage(window_disassembler->hwnd_disassembler,IDC_RADIO_REGISTER_00,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_disassembler->hwnd_disassembler,IDC_RADIO_REGISTER_01,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_disassembler->hwnd_disassembler,IDC_RADIO_REGISTER_10,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_disassembler->hwnd_disassembler,IDC_RADIO_REGISTER_11,BM_SETCHECK,0,0);
    }
  else if(register_mx == DISASSEMBLER_REGISTER_10)
    {
      /* Visuel */
      window_disassembler->radio_register_mx = DISASSEMBLER_REGISTER_10;
      SendDlgItemMessage(window_disassembler->hwnd_disassembler,IDC_RADIO_REGISTER_00,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_disassembler->hwnd_disassembler,IDC_RADIO_REGISTER_01,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_disassembler->hwnd_disassembler,IDC_RADIO_REGISTER_10,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_disassembler->hwnd_disassembler,IDC_RADIO_REGISTER_11,BM_SETCHECK,0,0);
    }
  else if(register_mx == DISASSEMBLER_REGISTER_11)
    {
      /* Visuel */
      window_disassembler->radio_register_mx = DISASSEMBLER_REGISTER_11;
      SendDlgItemMessage(window_disassembler->hwnd_disassembler,IDC_RADIO_REGISTER_00,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_disassembler->hwnd_disassembler,IDC_RADIO_REGISTER_01,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_disassembler->hwnd_disassembler,IDC_RADIO_REGISTER_10,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_disassembler->hwnd_disassembler,IDC_RADIO_REGISTER_11,BM_SETCHECK,1,0);
    }
  
  /* On force la MAJ de la fentre */  
  window_disassembler->current_register_mx = window_disassembler->radio_register_mx;
  RefreshDisassemblerWindowData(window_disassembler,1);    
}


/******************************************************************************************/
/*  InsertRemoveDisassemblerDirective() :  Ajoute ou supprime une Directive d'assemblage. */
/******************************************************************************************/
static void InsertRemoveDisassemblerDirective(struct window_disassembler *window_disassembler, int action)
{
  int modified, found;
  struct directive_line *previous_directiveline;
  struct directive_line *current_directiveline;

  /* Init */
  modified = 0;

  /** Suppression **/
  if(action == IDM_CONTEXT_DISASSEMBLER_CLEAR)
    {
      previous_directiveline = NULL;
      for(current_directiveline = window_disassembler->first_directive_line; current_directiveline; previous_directiveline = current_directiveline, current_directiveline = current_directiveline->next)
        if(current_directiveline->address == window_disassembler->context_directive_address)
          {
            /* Unique Directive */
            if(window_disassembler->first_directive_line == window_disassembler->last_directive_line)
              {
                window_disassembler->first_directive_line = NULL;
                window_disassembler->last_directive_line = NULL;
                mem_free_directiveline(current_directiveline);
                modified = 1;
                break;
              }
            /* Premire Directive */
            else if(window_disassembler->first_directive_line == current_directiveline)
              {
                window_disassembler->first_directive_line = current_directiveline->next;
                mem_free_directiveline(current_directiveline);
                modified = 1;
                break;
              }
            /* Dernire Directive */
            else if(window_disassembler->last_directive_line == current_directiveline)
              {
                window_disassembler->last_directive_line = previous_directiveline;
                previous_directiveline->next = NULL;
                mem_free_directiveline(current_directiveline);
                modified = 1;
                break;
              }
            /* Au milieu */
            else
              {
                previous_directiveline->next = current_directiveline->next;
                mem_free_directiveline(current_directiveline);
                modified = 1;
                break;
              }
          }
    }
  else
    {
      /** Modification d'une directive existante **/
      found = 0;
      for(current_directiveline = window_disassembler->first_directive_line; current_directiveline; current_directiveline = current_directiveline->next)
        if(current_directiveline->address == window_disassembler->context_directive_address)
          {
            found = 1;
            if(action == IDM_CONTEXT_DISASSEMBLER_00)
              {
                modified = (current_directiveline->m != 0 || current_directiveline->x != 0) ? 1 : 0;
                current_directiveline->m = 0;
                current_directiveline->x = 0;
              }
            else if(action == IDM_CONTEXT_DISASSEMBLER_01)
              {
                modified = (current_directiveline->m != 0 || current_directiveline->x != 1) ? 1 : 0;
                current_directiveline->m = 0;
                current_directiveline->x = 1;
              }
            else if(action == IDM_CONTEXT_DISASSEMBLER_10)
              {
                modified = (current_directiveline->m != 1 || current_directiveline->x != 0) ? 1 : 0;
                current_directiveline->m = 1;
                current_directiveline->x = 0;
              }
            else if(action == IDM_CONTEXT_DISASSEMBLER_11)
              {
                modified = (current_directiveline->m != 1 || current_directiveline->x != 1) ? 1 : 0;
                current_directiveline->m = 1;
                current_directiveline->x = 1;
              }
          }

      /** Cration d'une nouvelle Directive **/
      if(found == 0)
        {
          /* Allocation mmoire */
          current_directiveline = (struct directive_line *) calloc(1,sizeof(struct directive_line));
          if(current_directiveline == NULL)
            return;
          current_directiveline->address = window_disassembler->context_directive_address;
          current_directiveline->type = DIRECTIVE_TYPE_MX;
          if(action == IDM_CONTEXT_DISASSEMBLER_00)
            {
              current_directiveline->m = 0;
              current_directiveline->x = 0;
            }
          else if(action == IDM_CONTEXT_DISASSEMBLER_01)
            {
              current_directiveline->m = 0;
              current_directiveline->x = 1;
            }
          else if(action == IDM_CONTEXT_DISASSEMBLER_10)
            {
              current_directiveline->m = 1;
              current_directiveline->x = 0;
            }
          else if(action == IDM_CONTEXT_DISASSEMBLER_11)
            {
              current_directiveline->m = 1;
              current_directiveline->x = 1;
            }

          /* Rattache la structure */
          if(window_disassembler->first_directive_line == NULL)
            window_disassembler->first_directive_line = current_directiveline;
          else
            window_disassembler->last_directive_line->next = current_directiveline;
          window_disassembler->last_directive_line = current_directiveline;
          
          /* On dispose d'une nouvelle Directive */
          modified = 1;
        }
    }
    
  /* On force la MAJ de la fentre */  
  if(modified == 1)
    RefreshDisassemblerWindowData(window_disassembler,1);       
}


/**********************************************************************************/
/*  RefreshDisassemblerWindowData() :  MAJ du contenu de la fentre Disassembler. */
/**********************************************************************************/
static void RefreshDisassemblerWindowData(struct window_disassembler *window_disassembler, int force)
{
  int length, is_modified;
  int disassembler_bank, disassembler_address, disassembler_data_bank, nb_row;
  char buffer[1024];

  /* Rcupre la valeur du Disassembler Pointer */
  length = GetDlgItemText(window_disassembler->hwnd_disassembler,IDC_EDIT_BANK,buffer,1024);
  sscanf(buffer,"%X",&disassembler_bank);
  length = GetDlgItemText(window_disassembler->hwnd_disassembler,IDC_EDIT_ADDRESS,buffer,1024);
  sscanf(buffer,"%X",&disassembler_address);
  length = GetDlgItemText(window_disassembler->hwnd_disassembler,IDC_EDIT_DATA_BANK,buffer,1024);
  sscanf(buffer,"%X",&disassembler_data_bank);
  nb_row = GetDlgItemInt(window_disassembler->hwnd_disassembler,IDC_EDIT_NBLINE,NULL,FALSE);
  if(nb_row < window_disassembler->text_min_row)
    nb_row = window_disassembler->text_min_row;

  /* Replace les valeurs dans les Control pour montrer ce qui a t compris */
  sprintf(buffer,"%02X",disassembler_bank);
  SetDlgItemText(window_disassembler->hwnd_disassembler,IDC_EDIT_BANK,buffer);
  sprintf(buffer,"%04X",disassembler_address);
  SetDlgItemText(window_disassembler->hwnd_disassembler,IDC_EDIT_ADDRESS,buffer);
  sprintf(buffer,"%02X",disassembler_data_bank);
  SetDlgItemText(window_disassembler->hwnd_disassembler,IDC_EDIT_DATA_BANK,buffer);
  SetDlgItemInt(window_disassembler->hwnd_disassembler,IDC_EDIT_NBLINE,nb_row,FALSE);

  /** Les donnes ont t'elles t modifies + Rcupration des nouvelles Data si besoin **/
  is_modified = IsDisassemblerDataModified(disassembler_bank,disassembler_address,nb_row,window_disassembler);
  if(is_modified == 0 && force == 0)
    return;

  /** Cration de la Zone Text Code  partir des nouvelles donnes **/
  BuildDisassemblerFromData(window_disassembler);

  /* Rafraichit la fentre */
  InvalidateRect(window_disassembler->hwnd_disassembler,NULL,TRUE);
  UpdateWindow(window_disassembler->hwnd_disassembler);
}


/*****************************************************************************************************************/
/*  IsDisassemblerDataModified() :  Les donnes Disassembler ont t'elles t modifies depuis la dernirer fois. */
/*****************************************************************************************************************/
static int IsDisassemblerDataModified(int disassembler_bank, int disassembler_address, int nb_row, struct window_disassembler *window_disassembler)
{
  int disassembler_data_address;
  unsigned char disassembler_data[1024];
  struct mmu *current_mmu;

  /* Rcupre le MMU */
  my_Memory(MEMORY_GET_S65816_MMU,&current_mmu,NULL);

  /* Rcupre les donnes brutes : nb_row * 4 bytes */
  disassembler_data_address = (disassembler_bank << 16) | disassembler_address;
  ReadMemoryBlock(disassembler_data_address,nb_row*4,disassembler_data,current_mmu);

  /* Compare les donnes */
  if(window_disassembler->disassembler_data != NULL)
    if(disassembler_bank == window_disassembler->disassembler_bank &&
       disassembler_address == window_disassembler->disassembler_address &&
       nb_row == window_disassembler->nb_row &&
       memcmp(&disassembler_data[0],&window_disassembler->disassembler_data[0],nb_row*4) == 0)
      return(0);

  /* Les donnes sont diffrentes => MAJ */
  window_disassembler->disassembler_bank = disassembler_bank;
  window_disassembler->disassembler_address = disassembler_address;
  window_disassembler->nb_row = nb_row;
  memcpy(window_disassembler->disassembler_data,disassembler_data,window_disassembler->nb_row*4);

  /* Donnes diffrentes */
  return(1);
}


/***************************************************************************/
/*  ResizeDisassemblerWindow() : Resize Disassembler window by given size. */
/***************************************************************************/
static void ResizeDisassemblerWindow(struct window_disassembler *window_disassembler)
{
  RECT window_rect;
  int window_x, window_y;
  
  /* Largeur de la fentre */
  window_disassembler->window_width = window_disassembler->group_data_x + window_disassembler->group_data_width + WINDOW_CLIENT_RIGHT_BORDER_SIZE + WINDOW_BORDER_SIZE + window_disassembler->window_border_size;

  /* Hauteur de la fentre */
  window_disassembler->window_height = window_disassembler->title_height + window_disassembler->group_data_y + window_disassembler->group_data_height + WINDOW_CLIENT_BOTTOM_BORDER_SIZE + window_disassembler->window_border_size;

  /* Position actuelle de la fentre dans l'cran */
  GetWindowRect(window_disassembler->hwnd_disassembler,&window_rect);
  window_x = window_rect.left;
  window_y = window_rect.top;

  /* Resize window */
  MoveWindow(window_disassembler->hwnd_disassembler,window_x,window_y,window_disassembler->window_width,window_disassembler->window_height,FALSE);
}


/****************************************************************************/
/*  UserResizeDisassemblerWindow() :  Retaille la fentre de dsassemblage. */
/****************************************************************************/
static void UserResizeDisassemblerWindow(struct window_disassembler *window_disassembler, int new_width, int new_height)
{
  RECT window_rect;
  int window_height, group_data_height, codesource_data_height;
  int text_data_row;

  /* Position actuelle de la fentre dans l'cran */
  GetWindowRect(window_disassembler->hwnd_disassembler,&window_rect);

  /* Dimension de la fentre (avec la barre de titre et les bordures) */
  window_height = window_rect.bottom - window_rect.top;

  /** On va calculer le nb de lignes pour le Rich Edit stock dans le Group **/
  /* Taille du Group (en pixel) */
  group_data_height = window_height - (window_disassembler->title_height + window_disassembler->group_data_y + WINDOW_CLIENT_BOTTOM_BORDER_SIZE + window_disassembler->window_border_size);

  /* Taille du Rich Edit en pixel */
  codesource_data_height = group_data_height - (WINDOW_GROUP_BORDER_TOP_SIZE + WINDOW_GROUP_BORDER_BOTTOM_SIZE);

  /* Taille du Rich Edit en Row / Column */
  text_data_row = codesource_data_height/DISASSEMBLER_RICHEDIT_FONT_HEIGHT;

  /** Teste les valeurs minimales autorises **/
  if(text_data_row < window_disassembler->text_min_row)
    text_data_row = window_disassembler->text_min_row;

  /* MAJ du nombre de lignes affiche */
  window_disassembler->nb_row = text_data_row;
  SetDlgItemInt(window_disassembler->hwnd_disassembler,IDC_EDIT_NBLINE,text_data_row,FALSE);

  /** Reconstruit les lements de la fentre avec les nouvelles valeurs du Rich Edit **/
  RefreshDisassemblerWindowData(window_disassembler,1);

  /* Retaille la fentre pour viter les tailles entre deux */
  ResizeDisassemblerWindow(window_disassembler);
}


/**************************************************************************************************************/
/*  BuildDisassemblerFromData() :  Construire les donnes hexadcimal  afficher  partir des donnes brutes. */
/**************************************************************************************************************/
static void BuildDisassemblerFromData(struct window_disassembler *window_disassembler)
{
  HWND hwnd_group;
  EDITSTREAM edit_stream;
  char *next_sep;
  int i, offset_text, offset_rtf, address_data, color, quality_coef, data_bank;
  struct code_line *first_code_line;
  struct code_line *last_code_line;
  struct code_line *current_code_line;
  struct code_line *next_code_line;
  struct symbol *current_symbol;
  char *tab_opcode_padding[] = {"     ","    ","   ","  "," ",""};
  char *tab_operand_padding[] = {"             ","            ","           ","          ","         ","        ","       ","      ","     ","    ","   ","  "," ",""};
  char buffer[1024];
  char buffer_title[256];
  char buffer_color[256];

  /** On doit peut tre redimensionner la zone Text Edit dynamique **/
  if(window_disassembler->rtf_data_hexadecimal == NULL || 
     window_disassembler->text_data_row != window_disassembler->nb_row)
    {
      /* MAJ */
      window_disassembler->text_data_row = window_disassembler->nb_row;

      /* Libration mmoire */
      if(window_disassembler->rtf_data_hexadecimal)
        free(window_disassembler->rtf_data_hexadecimal);
      window_disassembler->rtf_data_hexadecimal = NULL;

      /* Allocation mmoire */
      window_disassembler->rtf_data_size = DISASSEMBLER_RICHEDIT_HEADER_SIZE;
      window_disassembler->rtf_data_size += ((window_disassembler->text_data_width + (int)strlen("\r\n"))*window_disassembler->text_data_row)*DISASSEMBLER_RICHEDIT_MAX_CHAR_SIZE;
      window_disassembler->rtf_data_size += DISASSEMBLER_RICHEDIT_FOOTER_SIZE;
      window_disassembler->rtf_data_hexadecimal = (char *) calloc(window_disassembler->rtf_data_size+1,sizeof(char));
      if(window_disassembler->rtf_data_hexadecimal == NULL)
        return;

      /* Dimension en Pixel du Control Text MemoryArea Data */
      window_disassembler->disassembler_data_width = window_disassembler->text_data_width*7 + 2;
      window_disassembler->disassembler_data_height = window_disassembler->nb_row*DISASSEMBLER_RICHEDIT_FONT_HEIGHT;
      MoveWindow(window_disassembler->hwnd_richedit_data,window_disassembler->disassembler_data_x,window_disassembler->disassembler_data_y,window_disassembler->disassembler_data_width,window_disassembler->disassembler_data_height,FALSE);

      /* Dimension en Pixel du Control Group Box + Resize */  
      hwnd_group = GetDlgItem(window_disassembler->hwnd_disassembler,IDC_GROUP_DISASSEMBLER);     /* Rcupre le Handle du Control GroupBox li  IDC_GROUP_DISASSEMBLER */
      window_disassembler->group_data_width = WINDOW_GROUP_BORDER_LEFT_SIZE + window_disassembler->disassembler_data_width + WINDOW_GROUP_BORDER_RIGHT_SIZE;
      window_disassembler->group_data_height = WINDOW_GROUP_BORDER_TOP_SIZE + window_disassembler->disassembler_data_height + WINDOW_GROUP_BORDER_BOTTOM_SIZE;
      MoveWindow(hwnd_group,window_disassembler->group_data_x,window_disassembler->group_data_y,window_disassembler->group_data_width,window_disassembler->group_data_height,FALSE);

      /* Retaille la fentre */
      ResizeDisassemblerWindow(window_disassembler);
    }

  /* Adresse des donnes */
  address_data = (window_disassembler->disassembler_bank << 16) + window_disassembler->disassembler_address;
  data_bank = window_disassembler->disassembler_data_bank;

  /** On va dsassembler window_disassembler->nb_row lignes **/
  first_code_line = DisassembleDataBlock(1024,window_disassembler->disassembler_data,window_disassembler->nb_row,-1,address_data,data_bank,window_disassembler->processor,&window_disassembler->current_register_mx,window_disassembler->first_directive_line,&quality_coef);
  if(first_code_line == NULL)
    return;
  for(current_code_line = first_code_line; current_code_line; current_code_line = current_code_line->next)
    last_code_line = current_code_line;

  /* Libration des lignes prcdentes */
  for(current_code_line = window_disassembler->first_code_line; current_code_line; current_code_line = next_code_line)
    {
      next_code_line = current_code_line->next;
      mem_free_codeline(current_code_line);
    } 

  /* Rattache les nouvelles lignes */
  window_disassembler->first_code_line = first_code_line;
  window_disassembler->last_code_line = last_code_line;

  /******************************************************/
  /*** Cration d'un buffer RTF rpentant les donnes ***/
  window_disassembler->rtf_data_hexadecimal[0] = '\0';
  strcat(window_disassembler->rtf_data_hexadecimal,"{\\rtf1\\ansi\\ansicpg1252\\deff0\\deflang1036{\\fonttbl{\\f0\\fnil\\fcharset0 Courier New;}}\r\n");
  strcat(window_disassembler->rtf_data_hexadecimal,"{\\colortbl ;\\red255\\green0\\blue0;\\red0\\green77\\blue187;\\red0\\green176\\blue80;\\red192\\green80\\blue77;\\red128\\green128\\blue128;}\r\n");
  strcat(window_disassembler->rtf_data_hexadecimal,"{\\*\\generator Msftedit 5.41.21.2510;}\\viewkind4\\uc1\\pard\\sl240\\slmult1\\lang12\\f0\\fs");
  strcat(window_disassembler->rtf_data_hexadecimal,DISASSEMBLER_RICHEDIT_FONT_HEIGHT_RTF);  /* Taille de la Police : Si 8 dans WordPad => 16 ici (x2)*/
  strcat(window_disassembler->rtf_data_hexadecimal," ");
  offset_rtf = (int) strlen(window_disassembler->rtf_data_hexadecimal);

  /*** Transformer les donnes BYTE en Rich Edit ***/
  for(current_code_line = window_disassembler->first_code_line,offset_text=0; current_code_line; current_code_line=current_code_line->next)
    {
      /*** Cas particulier des lignes Directives ***/
      if(current_code_line->is_directive == 1)
        {
          /** MX + Adresse Data = Vide **/
          for(i=0; i<(int)(strlen(DISASSEMBLER_AREA_MX)+strlen(DISASSEMBLER_AREA_ADDRESS)+strlen(DISASSEMBLER_AREA_DATA)); i++)
            buffer[i] = ' ';
          buffer[strlen(DISASSEMBLER_AREA_MX)+strlen(DISASSEMBLER_AREA_ADDRESS)+strlen(DISASSEMBLER_AREA_DATA)] = '\0';
          strcpy(&window_disassembler->rtf_data_hexadecimal[offset_rtf],buffer);
          offset_rtf += (int) strlen(buffer);
      
          /** Opcode **/
          /* Couleur */
          sprintf(buffer_color,"\\cf%d ",DISASSEMBLER_RICHEDIT_COLOR_BLUE);
          strcpy(&window_disassembler->rtf_data_hexadecimal[offset_rtf],buffer_color);
          offset_rtf += (int) strlen(buffer_color);
          /* Data */
          sprintf(buffer,"%s%s",current_code_line->opcode,tab_opcode_padding[strlen(current_code_line->opcode)]);
          strcpy(&window_disassembler->rtf_data_hexadecimal[offset_rtf],buffer);
          offset_rtf += (int) strlen(buffer);
          /* Couleur */
          sprintf(buffer_color,"\\cf%d ",DISASSEMBLER_RICHEDIT_COLOR_BLACK);
          strcpy(&window_disassembler->rtf_data_hexadecimal[offset_rtf],buffer_color);
          offset_rtf += (int) strlen(buffer_color);
          
          /** Operand **/
          /* Couleur */
          sprintf(buffer_color,"\\cf%d ",DISASSEMBLER_RICHEDIT_COLOR_BLUE);
          strcpy(&window_disassembler->rtf_data_hexadecimal[offset_rtf],buffer_color);
          offset_rtf += (int) strlen(buffer_color);
          /* Data */
          sprintf(buffer,"%s%s",current_code_line->operand,tab_operand_padding[strlen(current_code_line->operand)]);
          strcpy(&window_disassembler->rtf_data_hexadecimal[offset_rtf],buffer);
          offset_rtf += (int) strlen(buffer);
          /* Couleur */
          sprintf(buffer_color,"\\cf%d ",DISASSEMBLER_RICHEDIT_COLOR_BLACK);
          strcpy(&window_disassembler->rtf_data_hexadecimal[offset_rtf],buffer_color);
          offset_rtf += (int) strlen(buffer_color);

          /** Fin de ligne **/
          if(current_code_line->next != NULL)
            {
              strcpy(buffer,"\\par\r\n");
              strcpy(&window_disassembler->rtf_data_hexadecimal[offset_rtf],buffer);
              offset_rtf += (int) strlen(buffer);
            }          
          continue;
        }
    
      /** MX **/
      sprintf(buffer,"%d%d ",current_code_line->m,current_code_line->x);
      strcpy(&window_disassembler->rtf_data_hexadecimal[offset_rtf],buffer);
      offset_rtf += (int) strlen(buffer);

      /** Adresse **/
      /* Couleur */
      sprintf(buffer_color,"\\cf%d ",(current_code_line->line_symbol == 1) ? DISASSEMBLER_RICHEDIT_COLOR_BLUE : DISASSEMBLER_RICHEDIT_COLOR_BLACK);
      strcpy(&window_disassembler->rtf_data_hexadecimal[offset_rtf],buffer_color);
      offset_rtf += (int) strlen(buffer_color);
      sprintf(buffer,"%02X/%04X: ",current_code_line->address>>16,current_code_line->address & 0xFFFF);
      strcpy(&window_disassembler->rtf_data_hexadecimal[offset_rtf],buffer);
      offset_rtf += (int) strlen(buffer);
      /* Couleur */
      sprintf(buffer_color,"\\cf%d ",DISASSEMBLER_RICHEDIT_COLOR_BLACK);
      strcpy(&window_disassembler->rtf_data_hexadecimal[offset_rtf],buffer_color);
      offset_rtf += (int) strlen(buffer_color);

      /** Data **/
      if(current_code_line->data_size == 1)
        sprintf(buffer,"%02X           ",current_code_line->data[0]);
      else if(current_code_line->data_size == 2)
        sprintf(buffer,"%02X %02X        ",current_code_line->data[0],current_code_line->data[1]);
      else if(current_code_line->data_size == 3)
        sprintf(buffer,"%02X %02X %02X     ",current_code_line->data[0],current_code_line->data[1],current_code_line->data[2]);
      else if(current_code_line->data_size == 4)
        sprintf(buffer,"%02X %02X %02X %02X  ",current_code_line->data[0],current_code_line->data[1],current_code_line->data[2],current_code_line->data[3]);
      strcpy(&window_disassembler->rtf_data_hexadecimal[offset_rtf],buffer);
      offset_rtf += (int) strlen(buffer);

      /** Label **/
      /* Couleur */
      sprintf(buffer_color,"\\cf%d ",DISASSEMBLER_RICHEDIT_COLOR_BLUE);
      strcpy(&window_disassembler->rtf_data_hexadecimal[offset_rtf],buffer_color);
      offset_rtf += (int) strlen(buffer_color);
      /* Label */
      strcpy(buffer,DISASSEMBLER_AREA_LABEL);
      current_symbol = GetSymbolByAddress(current_code_line->address);
      if(strlen(current_code_line->label) < strlen(DISASSEMBLER_AREA_LABEL))
        memcpy(buffer,current_code_line->label,strlen(current_code_line->label));
      else
        memcpy(buffer,current_code_line->label,strlen(DISASSEMBLER_AREA_LABEL));
      strcpy(&window_disassembler->rtf_data_hexadecimal[offset_rtf],buffer);
      offset_rtf += (int) strlen(buffer);
      /* Couleur */
      sprintf(buffer_color,"\\cf%d ",DISASSEMBLER_RICHEDIT_COLOR_BLACK);
      strcpy(&window_disassembler->rtf_data_hexadecimal[offset_rtf],buffer_color);
      offset_rtf += (int) strlen(buffer_color);

      /*** Opcode ***/
      color = DISASSEMBLER_RICHEDIT_COLOR_BLACK;
      if(!stricmp(current_code_line->opcode,"BRK") || !stricmp(current_code_line->opcode,"COP") ||
         !stricmp(current_code_line->opcode,"WDM") || !stricmp(current_code_line->opcode,"WAI") ||
         !stricmp(current_code_line->opcode,"STP"))
        color = DISASSEMBLER_RICHEDIT_COLOR_RED;
      else if(current_code_line->data_symbol == 1)
        color = DISASSEMBLER_RICHEDIT_COLOR_BLUE;
      else if(!stricmp(current_code_line->opcode,"HEX"))
        color = DISASSEMBLER_RICHEDIT_COLOR_DARKRED;
      else if(!stricmp(current_code_line->opcode,"DFB") || !stricmp(current_code_line->opcode,"DA") || !stricmp(current_code_line->opcode,"ADRL"))
        color = DISASSEMBLER_RICHEDIT_COLOR_BLUE;
      else if(current_code_line->wrap_address_bank == 1)    /* WARNING : Le saut utilise le wrapping dans le banc */
        color = DISASSEMBLER_RICHEDIT_COLOR_DARKRED;
      /* Couleur */
      sprintf(buffer_color,"\\cf%d ",color);
      strcpy(&window_disassembler->rtf_data_hexadecimal[offset_rtf],buffer_color);
      offset_rtf += (int) strlen(buffer_color);
      /* Data */
      sprintf(buffer,"%s%s",current_code_line->opcode,tab_opcode_padding[strlen(current_code_line->opcode)]);
      strcpy(&window_disassembler->rtf_data_hexadecimal[offset_rtf],buffer);
      offset_rtf += (int) strlen(buffer);
      /* Couleur */
      sprintf(buffer_color,"\\cf%d ",DISASSEMBLER_RICHEDIT_COLOR_BLACK);
      strcpy(&window_disassembler->rtf_data_hexadecimal[offset_rtf],buffer_color);
      offset_rtf += (int) strlen(buffer_color);

      /** Operand **/
      if(current_code_line->operand_symbol == 1)
        {
          /* Couleur */
          sprintf(buffer_color,"\\cf%d ",DISASSEMBLER_RICHEDIT_COLOR_BLUE);
          strcpy(&window_disassembler->rtf_data_hexadecimal[offset_rtf],buffer_color);
          offset_rtf += (int) strlen(buffer_color);

          /* Operand (No Comment) */
          strcpy(buffer,current_code_line->operand);
          if(strstr(current_code_line->operand," +") != NULL || strstr(current_code_line->operand," -") != NULL)
            {
              /* Opcode contenant un +/-offet => colorise en gris le +- */
              next_sep = strchr(buffer,' ');
              *next_sep = '\0';
              /* Adresse du Saut */
              strcpy(&window_disassembler->rtf_data_hexadecimal[offset_rtf],buffer);
              offset_rtf += (int) strlen(buffer);
              /* Offset du saut */
              *next_sep = ' ';
              /* Couleur Gris Fonc */
              sprintf(buffer_color,"\\cf%d ",DISASSEMBLER_RICHEDIT_COLOR_DARKGREY);
              strcpy(&window_disassembler->rtf_data_hexadecimal[offset_rtf],buffer_color);
              offset_rtf += (int) strlen(buffer_color);
              strcpy(&window_disassembler->rtf_data_hexadecimal[offset_rtf],next_sep);
              offset_rtf += (int) strlen(next_sep);
            }
          else
            {
              strcpy(&window_disassembler->rtf_data_hexadecimal[offset_rtf],buffer);
              offset_rtf += (int) strlen(buffer);
            }

          /* Couleur */
          sprintf(buffer_color,"\\cf%d",DISASSEMBLER_RICHEDIT_COLOR_BLACK);
          strcpy(&window_disassembler->rtf_data_hexadecimal[offset_rtf],buffer_color);
          offset_rtf += (int) strlen(buffer_color);
        }
      else if(current_code_line->data_symbol == 1)
        {
          /* Couleur */
          sprintf(buffer_color,"\\cf%d ",DISASSEMBLER_RICHEDIT_COLOR_BLUE);
          strcpy(&window_disassembler->rtf_data_hexadecimal[offset_rtf],buffer_color);
          offset_rtf += (int) strlen(buffer_color);

          /* Data */
          strcpy(&window_disassembler->rtf_data_hexadecimal[offset_rtf],current_code_line->operand);
          offset_rtf += (int) strlen(current_code_line->operand);

          /* Couleur */
          sprintf(buffer_color,"\\cf%d",DISASSEMBLER_RICHEDIT_COLOR_BLACK);
          strcpy(&window_disassembler->rtf_data_hexadecimal[offset_rtf],buffer_color);
          offset_rtf += (int) strlen(buffer_color);
        }
      else
        {
          /* Couleur */
          sprintf(buffer_color,"\\cf%d ",color);
          strcpy(&window_disassembler->rtf_data_hexadecimal[offset_rtf],buffer_color);
          offset_rtf += (int) strlen(buffer_color);
          /* Data */
          sprintf(buffer,"%s%s",current_code_line->operand,tab_operand_padding[strlen(current_code_line->operand)]);
          if(strstr(current_code_line->operand," +") != NULL || strstr(current_code_line->operand," -") != NULL)
            {
              /* Opcode contenant un +/-offet => colorise en gris le +- */
              next_sep = strchr(buffer,' ');
              *next_sep = '\0';
              /* Adresse du Saut */
              strcpy(&window_disassembler->rtf_data_hexadecimal[offset_rtf],buffer);
              offset_rtf += (int) strlen(buffer);
              /* Offset du saut */
              *next_sep = ' ';
              /* Couleur Gris Fonc */
              sprintf(buffer_color,"\\cf%d ",DISASSEMBLER_RICHEDIT_COLOR_DARKGREY);
              strcpy(&window_disassembler->rtf_data_hexadecimal[offset_rtf],buffer_color);
              offset_rtf += (int) strlen(buffer_color);
              strcpy(&window_disassembler->rtf_data_hexadecimal[offset_rtf],next_sep);
              offset_rtf += (int) strlen(next_sep);
            }
          else
            {
              strcpy(&window_disassembler->rtf_data_hexadecimal[offset_rtf],buffer);
              offset_rtf += (int) strlen(buffer);
            }
          /* Couleur */
          sprintf(buffer_color,"\\cf%d ",DISASSEMBLER_RICHEDIT_COLOR_BLACK);
          strcpy(&window_disassembler->rtf_data_hexadecimal[offset_rtf],buffer_color);
          offset_rtf += (int) strlen(buffer_color);

          /** Comment (troncature) **/
          /* Couleur */
          sprintf(buffer_color,"\\cf%d ",DISASSEMBLER_RICHEDIT_COLOR_GREEN);
          strcpy(&window_disassembler->rtf_data_hexadecimal[offset_rtf],buffer_color);
          offset_rtf += (int) strlen(buffer_color);
          /* Data */
          strcpy(buffer,current_code_line->comment);
          buffer[strlen(DISASSEMBLER_AREA_COMMENT)] = '\0';
          strcpy(&window_disassembler->rtf_data_hexadecimal[offset_rtf],buffer);
          offset_rtf += (int) strlen(buffer);
          /* Couleur */
          sprintf(buffer_color,"\\cf%d",DISASSEMBLER_RICHEDIT_COLOR_BLACK);
          strcpy(&window_disassembler->rtf_data_hexadecimal[offset_rtf],buffer_color);
          offset_rtf += (int) strlen(buffer_color);
        }

      /** Fin de ligne **/
      if(current_code_line->next != NULL)
        {
          strcpy(buffer,"\\par\r\n");
          strcpy(&window_disassembler->rtf_data_hexadecimal[offset_rtf],buffer);
          offset_rtf += (int) strlen(buffer);
        }
    }

  /** Fin du RTF **/
  strcpy(&window_disassembler->rtf_data_hexadecimal[offset_rtf],"}\r\n");
  offset_rtf += (int) strlen("}\r\n");
  window_disassembler->rtf_data_hexadecimal[offset_rtf] = '\0';

  /* Taille du RTF (le \0 final compte) */
  window_disassembler->rtf_data_size = (int) strlen(window_disassembler->rtf_data_hexadecimal) + 1;

  /** Afficher le rsultat en RTF dans le control Rich Edit **/
  window_disassembler->rtf_write_offset = 0;
  memset(&edit_stream,0,sizeof(EDITSTREAM));
  edit_stream.pfnCallback = (EDITSTREAMCALLBACK) DisassemblerRichEditStreamCallback;
  edit_stream.dwCookie = (DWORD_PTR) window_disassembler;          /* Paramtre  passer  la fonction Callback */
  SendMessage(window_disassembler->hwnd_richedit_data,EM_STREAMIN,SF_RTF,(LPARAM)&edit_stream);

  /* Titre de la fentre */
  sprintf(buffer_title,"Code Disassembler : %02X/%04X",window_disassembler->disassembler_bank,window_disassembler->disassembler_address);
  SetWindowText(window_disassembler->hwnd_disassembler,buffer_title);
}


/******************************************************************************************/
/*  DisassemblerRichEditStreamCallback() :  Fonction Call back pour remplir le Rich Edit. */
/******************************************************************************************/
static DWORD CALLBACK DisassemblerRichEditStreamCallback(DWORD *dwCookie, LPBYTE output_buffer, LONG nb_write_max, LONG *nb_write_rtn)
{
  int nb_remain_byte, nb_copy_byte;
  struct window_disassembler *window_disassembler;

  /* Rcupration du paramtre */
  window_disassembler = (struct window_disassembler *) dwCookie;

  /* Taille des donnes encore  copier */
  nb_remain_byte = window_disassembler->rtf_data_size - window_disassembler->rtf_write_offset;
  if(nb_remain_byte == 0)
    {
      /* Plus rien  copier */
      *nb_write_rtn = 0;
      return(0);
    }

  /* Taille des donnes que l'on est autoris  copier */
  nb_copy_byte = (nb_write_max >= nb_remain_byte) ? nb_remain_byte : nb_write_max;

  /* Nombre de byte transfrs */
  *nb_write_rtn = nb_copy_byte;
		  
  /** Copie les Byte dans le output buffer **/
  memcpy(output_buffer,&window_disassembler->rtf_data_hexadecimal[window_disassembler->rtf_write_offset],nb_copy_byte);

  /* Dplace l'offset */
  window_disassembler->rtf_write_offset += nb_copy_byte;

  /* OK */
  return(0);
}


/**************************************************************************/
/*  DeleteDisassemblerChildWindows() :  Destruction des fentres enfants. */
/**************************************************************************/
static void DeleteDisassemblerChildWindows(struct window_disassembler *window_disassembler)
{
  struct code_line *current_code_line;
  struct code_line *next_code_line;
  struct directive_line *current_directive_line;
  struct directive_line *next_directive_line;
  
  if(window_disassembler)
    {      
      /* Libration mmoire */
      if(window_disassembler->rtf_data_hexadecimal)
        free(window_disassembler->rtf_data_hexadecimal);
      window_disassembler->rtf_data_hexadecimal = NULL;
        
      /* Lignes de Code */
      for(current_code_line = window_disassembler->first_code_line; current_code_line; current_code_line = next_code_line)
        {
          next_code_line = current_code_line->next;
          mem_free_codeline(current_code_line);
        }
      window_disassembler->first_code_line = NULL;
      window_disassembler->last_code_line = NULL;   

      /* Lignes de Directive */
      for(current_directive_line = window_disassembler->first_directive_line; current_directive_line; current_directive_line = next_directive_line)
        {
          next_directive_line = current_directive_line->next;
          mem_free_directiveline(current_directive_line);
        }
      window_disassembler->first_directive_line = NULL;
      window_disassembler->last_directive_line = NULL;
    }
}


/****************************************************************************/
/*  mem_free_window_disassembler() :  Libration mmoire pour la structure. */
/****************************************************************************/
void mem_free_window_disassembler(struct window_disassembler *window_disassembler)
{
  if(window_disassembler)
    {
      /* Supprime la notification */
      my_Notification(NOTIFICATION_REMOVE,window_disassembler->current_notification,NULL);
      window_disassembler->current_notification = NULL;

      /* Libre la structure */ 
      free(window_disassembler);
    }
}

/**********************************************************************/
