/************************************************************************/
/*                                                                      */
/*  Window_GraphicSHR.c :  Module de gestion de la fentre GraphicSHR.  */
/*                                                                      */
/************************************************************************/
/*  Auteur : Olivier ZARDINI   *   Brutal Deluxe  *   Date : Aout 2018  */
/************************************************************************/

#include <stdio.h>
#include <time.h>
#include <sys/timeb.h>

#include <windows.h>
#include <windowsx.h>
#include <Commctrl.h>
#include <Winuser.h>
#include <Richedit.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_Notification.h"
#include "Dc_Graphic.h"
#include "Dc_CPU.h"
#include "Dc_MMU.h"
#include "Dc_Gif.h"
#include "Window_QuickWatch.h"
#include "Window_Main.h"
#include "Window_Main_Menu.h"
#include "Window_GraphicSHR.h"

static DWORD BuildGraphicSHRWindowThread(void *);
LRESULT CALLBACK ProcHandlerGraphicSHRWindow(HWND,UINT,WPARAM,LPARAM);
static int CreateGraphicSHRChildWindows(HWND,struct window_graphicshr *);
static void ChangePaletteNumber(int,struct window_graphicshr *);
LRESULT CALLBACK subEditGraphicSHRProcControl(HWND,UINT,WPARAM,LPARAM);
static void UpdateDoubleHeight(struct window_graphicshr *);
static void UpdateDisplayBorder(struct window_graphicshr *);
static void UpdateDisplaySCB(struct window_graphicshr *);
static void UpdateDisplayVideoSpot(struct window_graphicshr *);
static void UpdateFillMode(struct window_graphicshr *);
void UpdateGraphicSHRClientAllArea(HDC,struct window_graphicshr *);
static void UpdateGraphicSHRClientBitmapArea(HDC,struct window_graphicshr *);
static void UpdateGraphicSHRContextInformation(struct window_graphicshr *,int,int);
static void UpdateGraphicSHRAddress(struct window_graphicshr *,int);
static void UpdateGraphicSHRRefreshMode(struct window_graphicshr *,int);
static void UpdateGraphicSHRZoomLevel(struct window_graphicshr *,int);
static void RefreshGraphicSHRWindowData(struct window_graphicshr *,int);
static int IsGraphicSHRDataModified(int,int,int,int,struct window_graphicshr *);
static void LoadPalette(struct window_graphicshr *);
static void BuildGraphicSHRPalette(struct window_graphicshr *);
static void BuildPicturePalette(struct window_graphicshr *);
static void ResizeGraphicSHRWindow(struct window_graphicshr *);
static void BuildGraphicSHRFromData(struct window_graphicshr *);
static int BuildPaletteWindowAllBitmap(struct window_graphicshr *);
static int BuildPaletteWindowPaletteBitmap(struct window_graphicshr *);
static void UpdatePaletteClientPaletteArea(HDC,struct window_graphicshr *);
static void SaveGraphicPicture(struct window_graphicshr *);
static void DeleteGraphicSHRChildWindows(struct window_graphicshr *);


/**********************************************************************************/
/*  BuildGraphicSHRWindow() :  Construction / Affichage de la fentre GraphicSHR. */
/**********************************************************************************/
struct window_graphicshr *BuildGraphicSHRWindow(HINSTANCE hInstance, int menu_id)
{
  struct window_graphicshr *window_graphicshr;

  /** Allocation de la structure window_graphicshr **/
  window_graphicshr = (struct window_graphicshr *) calloc(1,sizeof(struct window_graphicshr));
  if(window_graphicshr == NULL)
    return(NULL);
  my_Memory(MEMORY_ADD_WINDOW_GRAPHICSHR,window_graphicshr,NULL);

  /* Lance un thread pour grer la fentre GraphicSHR */
  CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)BuildGraphicSHRWindowThread,window_graphicshr,0,NULL);

  /* Renvoi la structure mmoire grant la fentre */
  return(window_graphicshr);
}


/********************************************************************************************/
/*  BuildGraphicSHRWindowThread() :  Cration de la fentre GraphicSHR via un Thread ddi. */
/********************************************************************************************/
static DWORD BuildGraphicSHRWindowThread(void *window_graphicshr_param)
{
  HWND hwndGraphicSHR;
  MSG msg;
  BOOL bRet;
  struct window_main *window_main;
  struct window_graphicshr *window_graphicshr;
  struct parameter *param;
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Init */
  window_graphicshr = (struct window_graphicshr *) window_graphicshr_param;
  window_graphicshr->thread_id = GetCurrentThreadId();         /* Le ThreaID sert  diffrencier les diffrentes fentres */

  /** Cration de la fentre Graphic SHR **/
  hwndGraphicSHR = CreateDialog(param->hInstance,MAKEINTRESOURCE(IDD_WINDOW_GRAPHICSHR),window_main->hwnd_main,(DLGPROC)ProcHandlerGraphicSHRWindow);
  if(hwndGraphicSHR == NULL)
    return(1);

  /** Show Window + Paint Content **/
  ShowWindow(hwndGraphicSHR,SW_SHOWDEFAULT);
  UpdateWindow(hwndGraphicSHR);

  /** Message Loop pour toutes les fentres de ce Thread (fentre GraphicSHR + ComboBox + ...) **/
  while((bRet = GetMessage(&msg,NULL,0,0)) != 0)
    {
      if(bRet == -1)
        break;

      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }

  /* Fin du Thread */
  ExitThread(0);
}


/***************************************************************************/
/*  ProcHandlerGraphicSHRWindow() :  Gestion des vnements de la fentre. */
/***************************************************************************/
LRESULT CALLBACK ProcHandlerGraphicSHRWindow(HWND hwndGraphicSHR, UINT umsg, WPARAM wParam, LPARAM lParam)
{
  HDC hDC;
  PAINTSTRUCT ps;
  LRESULT result;
  int mouse_x, mouse_y;
  struct window_graphicshr *window_graphicshr;
  struct parameter *param;
  
  /* Init */
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  my_Memory(MEMORY_GET_WINDOW_GRAPHICSHR,&window_graphicshr,NULL);
  if(window_graphicshr == NULL)
    return(TRUE);

  switch(umsg)
    {
      case WM_INITDIALOG :
        /* Cration du contenu de la DialogBox */
        CreateGraphicSHRChildWindows(hwndGraphicSHR,window_graphicshr);
        return(TRUE);

      /***************************/
      /* Evnements de la souris */
      /***************************/
      case WM_MOUSEMOVE :
        mouse_x = GET_X_LPARAM(lParam);
        mouse_y = GET_Y_LPARAM(lParam);
        UpdateGraphicSHRContextInformation(window_graphicshr,mouse_x,mouse_y);
        return(TRUE);

      /**********************************/
      /* Rafrachissement de la fentre */
      /**********************************/
      case WM_PAINT:
        if(window_graphicshr->hBitmap_graphicshr_320_200 != NULL && window_graphicshr->hBitmap_graphicshr_640_200 != NULL &&
           window_graphicshr->hBitmap_graphicshr_320_200_BORDER != NULL && window_graphicshr->hBitmap_graphicshr_640_200_BORDER != NULL &&
           window_graphicshr->hBitmap_graphicshr_640_400 != NULL && window_graphicshr->hBitmap_graphicshr_640_400_BORDER != NULL)
          {
            /* Begin Paint */
            hDC = BeginPaint(hwndGraphicSHR,&ps);

            /** MAJ Bitmap **/
            UpdateGraphicSHRClientAllArea(hDC,window_graphicshr);

            /* End Paint */
            EndPaint(hwndGraphicSHR,&ps);
          }
        break;
        
      /****************************/
      /* Evnements de la fentre */
      /****************************/
      case WM_COMMAND :
        switch(LOWORD(wParam))
          {
            /** Bouton Radio : On change la mthode de rafraichissement de la fentre **/
            case IDC_RADIO_REFRESH_AUTOMATIC :
              if(HIWORD(wParam) == BN_CLICKED && window_graphicshr->refresh_mode != GRAPHICSHR_REFRESH_AUTOMATIC)
                UpdateGraphicSHRRefreshMode(window_graphicshr,GRAPHICSHR_REFRESH_AUTOMATIC);
              break;

            case IDC_RADIO_REFRESH_MANUAL :
              if(HIWORD(wParam) == BN_CLICKED && window_graphicshr->refresh_mode != GRAPHICSHR_REFRESH_MANUAL)
                UpdateGraphicSHRRefreshMode(window_graphicshr,GRAPHICSHR_REFRESH_MANUAL);
              break;

            /** Bouton Radio : On change le Zoom de la fentre **/
            case IDC_RADIO_ZOOM_X1 :
              if(HIWORD(wParam) == BN_CLICKED && window_graphicshr->zoom_level != GRAPHICSHR_ZOOM_X1)
                UpdateGraphicSHRZoomLevel(window_graphicshr,GRAPHICSHR_ZOOM_X1);
              break;
            case IDC_RADIO_ZOOM_X2 :
              if(HIWORD(wParam) == BN_CLICKED && window_graphicshr->zoom_level != GRAPHICSHR_ZOOM_X2)
                UpdateGraphicSHRZoomLevel(window_graphicshr,GRAPHICSHR_ZOOM_X2);
              break;
            case IDC_RADIO_ZOOM_X3 :
              if(HIWORD(wParam) == BN_CLICKED && window_graphicshr->zoom_level != GRAPHICSHR_ZOOM_X3)
                UpdateGraphicSHRZoomLevel(window_graphicshr,GRAPHICSHR_ZOOM_X3);
              break;
            case IDC_RADIO_ZOOM_X4 :
              if(HIWORD(wParam) == BN_CLICKED && window_graphicshr->zoom_level != GRAPHICSHR_ZOOM_X4)
                UpdateGraphicSHRZoomLevel(window_graphicshr,GRAPHICSHR_ZOOM_X4);
              break;

            /** Bouton de dplacement de l'adresse **/
            case IDC_BUTTON_LEFT :
              UpdateGraphicSHRAddress(window_graphicshr,-1);
              break;

            case IDC_BUTTON_RIGHT :
              UpdateGraphicSHRAddress(window_graphicshr,1);
              break;

            case IDC_BUTTON_UP :
              UpdateGraphicSHRAddress(window_graphicshr,-160);
              break;

            case IDC_BUTTON_DOWN :
              UpdateGraphicSHRAddress(window_graphicshr,160);
              break;

            case IDC_BUTTON_PAGEUP :
              UpdateGraphicSHRAddress(window_graphicshr,-32768);
              break;

            case IDC_BUTTON_PAGEDOWN :
              UpdateGraphicSHRAddress(window_graphicshr,32768);
              break;

            /** Activate Fill Mode **/
            case IDC_CHECK_FILLMODE :
              if(HIWORD(wParam) == BN_CLICKED)
                UpdateFillMode(window_graphicshr);
              break;

            /** Double Height **/
            case IDC_CHECK_DOUBLE_HEIGHT :
              if(HIWORD(wParam) == BN_CLICKED)
                UpdateDoubleHeight(window_graphicshr);
              break;

            /** Display Border **/
            case IDC_CHECK_DISPLAY_BORDER :
              if(HIWORD(wParam) == BN_CLICKED)
                UpdateDisplayBorder(window_graphicshr);
              break;

            /** Display Spot **/
            case IDC_CHECK_DISPLAY_SCB :
              if(HIWORD(wParam) == BN_CLICKED)
                UpdateDisplaySCB(window_graphicshr);
              break;

            /** Display Spot **/
            case IDC_CHECK_DISPLAY_SPOT :
              if(HIWORD(wParam) == BN_CLICKED)
                UpdateDisplayVideoSpot(window_graphicshr);
              break;

            /** Slection dans la ComboBox de la Palette **/
            case IDC_COMBOBOX_SCB_PALETTE :
              if(HIWORD(wParam) == CBN_CLOSEUP)
                {
                  /* Rcupre l'index de ce qui est slectionn */
                  result = SendMessage(window_graphicshr->hwnd_combo_scb_palette,CB_GETCURSEL,0,0);
                  if(result != CB_ERR)
                    if(result != window_graphicshr->scb_palette)
                      {
                        window_graphicshr->scb_palette = (int) result;
                        RefreshGraphicSHRWindowData(window_graphicshr,1);
                      }
                }
              break;

            /** Slection dans la ComboBox de la Couleur de la Bordure **/
            case IDC_COMBOBOX_BORDER_COLOR_BACKGROUND :
              if(HIWORD(wParam) == CBN_CLOSEUP)
                {
                  /* Rcupre l'index de ce qui est slectionn */
                  result = SendMessage(window_graphicshr->hwnd_combo_color_background,CB_GETCURSEL,0,0);
                  if(result != CB_ERR)
                    if(result != window_graphicshr->border_color)
                      {
                        window_graphicshr->border_color = (int) result;
                        RefreshGraphicSHRWindowData(window_graphicshr,1);
                      }
                }
              break; 

            /** On appuie sur un des boutons de changement de numro de palette **/
            case IDC_BUTTON_PALETTE_MINUS :
              ChangePaletteNumber(-1,window_graphicshr);
              break;

            case IDC_BUTTON_PALETTE_PLUS :
              ChangePaletteNumber(1,window_graphicshr);
              break;

            /** On appuie sur le bouton 'Load Palette' **/
            case IDC_BUTTON_PALETTE_LOAD :
              LoadPalette(window_graphicshr);
              break;

            /** Bouton de rafraichissement Manuel de la fentre **/
            case IDC_BUTTON_REFRESH :
              RefreshGraphicSHRWindowData(window_graphicshr,1);
              break;

            /** Bouton de Sauvegarde de l'image au format GIF **/
            case IDC_BUTTON_SAVEAS :
              SaveGraphicPicture(window_graphicshr);
              break;
          }
        return(TRUE);

      /*****************************************************************/
      /* Message extrieur demandant le rafraichissement de la fentre */
      /*****************************************************************/
      case HM_GRAPHICSHR_REFRESHWINDOW :
        RefreshGraphicSHRWindowData(window_graphicshr,0);
        return(TRUE);

      /********************************************************************/
      /* Fermeture de window (le WM_CLOSE est appel avant le WM_DESTROY) */
      /********************************************************************/
      case WM_CLOSE :
        /* Fentre GraphicSHR */
        DestroyWindow(hwndGraphicSHR);
        return(0);

      case WM_DESTROY :
        /* Suppression des Fentres */
        DeleteGraphicSHRChildWindows(window_graphicshr);

        /* Supprime la structure mmoire de ce thread */
        my_Memory(MEMORY_DELETE_WINDOW_GRAPHICSHR,window_graphicshr,NULL);

        /* Demande la sortie de la boucle des messages */
        PostQuitMessage(0);
        return(0);
    }

  /* Retourner FALSE pour relayer le message non trait au systme */
  return(FALSE);
}


/***********************************************************************************************************/
/*  CreateGraphicSHRChildWindows() : Crer les fentres enfants et remplir la structure window_graphicshr. */
/***********************************************************************************************************/
static int CreateGraphicSHRChildWindows(HWND hwndGraphicSHR, struct window_graphicshr *window_graphicshr)
{
  BITMAPINFO bitmap_info;
  HWND hwndItem;
  int i, title_height;
  RECT rect_palette, rect_window, rect_client; 
  char *scb_palette_name[16] = {"From Current Area","From E1/2000","From 01/2000","Fixed Palette"};
  char *color_name[17] = {"Dynamic","Black","Deep Red","Dark Blue","Purple","Dark Green","Dark Gray","Medium Blue","Light Blue","Brown","Orange","Light Gray","Pink","Light Green","Yellow","Aquamarine","White"};
  char buffer[BUFFER_SIZE];
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Couleurs par dfaut de la palette : Celles du IIgs **/
  BuildGraphicSHRPalette(window_graphicshr);

  /** Window Handle **/
  window_graphicshr->hwnd_graphicshr = hwndGraphicSHR;

  /** Device Context **/
  window_graphicshr->hDC = GetDC(window_graphicshr->hwnd_graphicshr);
  window_graphicshr->hCompatibleDC = CreateCompatibleDC(window_graphicshr->hDC);

  /** Surchage des Control Edit pour capter les vnements **/
  window_graphicshr->oldEditProcBank    = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndGraphicSHR,IDC_EDIT_BANK),GWLP_WNDPROC,(LONG_PTR)subEditGraphicSHRProcControl);
  window_graphicshr->oldEditProcAddress = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndGraphicSHR,IDC_EDIT_ADDRESS),GWLP_WNDPROC,(LONG_PTR)subEditGraphicSHRProcControl);

  /** Ajoute les images sur les boutons **/
  SendMessage(GetDlgItem(hwndGraphicSHR,IDC_BUTTON_LEFT),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_LEFT)));
  SendMessage(GetDlgItem(hwndGraphicSHR,IDC_BUTTON_RIGHT),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_RIGHT)));
  SendMessage(GetDlgItem(hwndGraphicSHR,IDC_BUTTON_UP),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_UP)));
  SendMessage(GetDlgItem(hwndGraphicSHR,IDC_BUTTON_DOWN),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_DOWN)));
  SendMessage(GetDlgItem(hwndGraphicSHR,IDC_BUTTON_PAGEUP),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_PAGE_UP)));
  SendMessage(GetDlgItem(hwndGraphicSHR,IDC_BUTTON_PAGEDOWN),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_PAGE_DOWN)));
  SendMessage(GetDlgItem(hwndGraphicSHR,IDC_BUTTON_PALETTE_MINUS),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_MINUS)));
  SendMessage(GetDlgItem(hwndGraphicSHR,IDC_BUTTON_PALETTE_PLUS),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_PLUS)));

  /** Cration de la ComboBox **/
  window_graphicshr->hwnd_combo_scb_palette = CreateWindowEx(0,"COMBOBOX", 0, CBS_DROPDOWNLIST | WS_CHILD | WS_VISIBLE,
                                                             254,72,180,250,hwndGraphicSHR,(HMENU)IDC_COMBOBOX_SCB_PALETTE,param->hInstance,0);
  for(i=0; i<4; i++)
    SendMessage(window_graphicshr->hwnd_combo_scb_palette,CB_ADDSTRING,0,(LPARAM)scb_palette_name[i]);
  /* 1re valeur */
  window_graphicshr->scb_palette = GRAPHICSHR_SCBPALETTE_CURRENT;
  SendMessage(window_graphicshr->hwnd_combo_scb_palette,CB_SETCURSEL,window_graphicshr->scb_palette,0);

  /* Border Color */
  window_graphicshr->hwnd_combo_color_background = CreateWindowEx(0,"COMBOBOX", 0, CBS_DROPDOWNLIST | WS_CHILD | WS_VISIBLE, 
                                                                  254,102,180,270,hwndGraphicSHR,(HMENU)IDC_COMBOBOX_BORDER_COLOR_BACKGROUND,param->hInstance,0);
  for(i=0; i<17; i++)
    SendMessage(window_graphicshr->hwnd_combo_color_background,CB_ADDSTRING,0,(LPARAM)color_name[i]);
  /* 1re valeur */
  window_graphicshr->border_color = GRAPHICSHR_COLOR_DYNAMIC;
  SendMessage(window_graphicshr->hwnd_combo_color_background,CB_SETCURSEL,window_graphicshr->border_color,0);

  /** Cration de la Police de caractre + Affectation de la police au Control Edit **/
  window_graphicshr->hFont = CreateFont(14,0,0,0,FW_DONTCARE,FALSE,FALSE,FALSE,ANSI_CHARSET,OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY,DEFAULT_PITCH|FF_SWISS,"Arial");
  SendMessage(window_graphicshr->hwnd_combo_scb_palette,WM_SETFONT,(WPARAM)window_graphicshr->hFont,TRUE);
  SendMessage(window_graphicshr->hwnd_combo_color_background,WM_SETFONT,(WPARAM)window_graphicshr->hFont,TRUE);

  /** Adresse de la Zone de Data **/
  window_graphicshr->graphicshr_bank = 0xE1;
  window_graphicshr->graphicshr_address = 0x2000;       /* Adresse SHR Page #1 */
  sprintf(buffer,"%02X",window_graphicshr->graphicshr_bank);
  SetDlgItemText(window_graphicshr->hwnd_graphicshr,IDC_EDIT_BANK,buffer);
  sprintf(buffer,"%04X",window_graphicshr->graphicshr_address);
  SetDlgItemText(window_graphicshr->hwnd_graphicshr,IDC_EDIT_ADDRESS,buffer);

  /* Refresh Mode : Automatic */
  UpdateGraphicSHRRefreshMode(window_graphicshr,GRAPHICSHR_REFRESH_AUTOMATIC);

  /* Zoom Level : 1 */
  window_graphicshr->zoom_level = GRAPHICSHR_ZOOM_X1;
  SendDlgItemMessage(window_graphicshr->hwnd_graphicshr,IDC_RADIO_ZOOM_X1,BM_SETCHECK,1,0);
  SendDlgItemMessage(window_graphicshr->hwnd_graphicshr,IDC_RADIO_ZOOM_X2,BM_SETCHECK,0,0);
  SendDlgItemMessage(window_graphicshr->hwnd_graphicshr,IDC_RADIO_ZOOM_X3,BM_SETCHECK,0,0);
  SendDlgItemMessage(window_graphicshr->hwnd_graphicshr,IDC_RADIO_ZOOM_X4,BM_SETCHECK,0,0);

  /* On ne double pas la hauteur */
  window_graphicshr->double_height = 0;

  /** Hauteur et Largeur des Bitmap couleurs **/
  /* Palette */
  window_graphicshr->bitmap_palette_width = PALETTE_COLOR_BORDER + 16*(PALETTE_COLOR_SIZE_WIDTH + PALETTE_COLOR_BORDER);   /* 1 ligne de 16 couleurs */
  window_graphicshr->bitmap_palette_height = PALETTE_COLOR_BORDER + PALETTE_COLOR_SIZE_HEIGHT + PALETTE_COLOR_BORDER;

  /* Position de la GroupBox Select + GroupBox Palette */
  GetWindowRect(window_graphicshr->hwnd_graphicshr,&rect_window);
  GetClientRect(window_graphicshr->hwnd_graphicshr,&rect_client);
  title_height = (rect_window.bottom - rect_window.top) - rect_client.bottom;     /* Hauteur du titre */
  hwndItem = GetDlgItem(window_graphicshr->hwnd_graphicshr,IDC_GROUP_PALETTE);    /* Rcupre le Handle du Control GroupBox li  IDC_GROUP_PALETTE */
  GetWindowRect(hwndItem,&rect_palette);

  /* Position des Bitmap dans la fenetre */
  window_graphicshr->bitmap_palette_x = (rect_palette.left - rect_window.left) + 5;
  window_graphicshr->bitmap_palette_y = (rect_palette.top - rect_window.top) - title_height + 18;
  
  /** Bitmap affichant les couleurs de la palette **/
  memset(&bitmap_info,0,sizeof(BITMAPINFO));
  bitmap_info.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
  bitmap_info.bmiHeader.biWidth = window_graphicshr->bitmap_palette_width;
  bitmap_info.bmiHeader.biHeight = (-1)*window_graphicshr->bitmap_palette_height;
  bitmap_info.bmiHeader.biPlanes = 1;
  bitmap_info.bmiHeader.biBitCount = 32;
  bitmap_info.bmiHeader.biCompression = BI_RGB;
  window_graphicshr->hBitmap_palette = CreateDIBSection(window_graphicshr->hDC,&bitmap_info,DIB_RGB_COLORS,(VOID **)&(window_graphicshr->bitmap_data_palette),NULL,0);
  for(i=0; i<window_graphicshr->bitmap_palette_width*window_graphicshr->bitmap_palette_height; i++)
    window_graphicshr->bitmap_data_palette[i] = BITMAP_BACKGROUND_COLOR;  

  /* Dimension en Pixel de la Bitmap GraphicSHR Data */
  window_graphicshr->bitmap_graphicshr_width_320         = 320;
  window_graphicshr->bitmap_graphicshr_width_320_BORDER  = 320 + GRAPHICSHR_BORDER_SIZE_LEFT + GRAPHICSHR_BORDER_SIZE_RIGHT;
  window_graphicshr->bitmap_graphicshr_width_640         = 640;
  window_graphicshr->bitmap_graphicshr_width_640_BORDER  = 640 + GRAPHICSHR_BORDER_SIZE_LEFT*2 + GRAPHICSHR_BORDER_SIZE_RIGHT*2;
  window_graphicshr->bitmap_graphicshr_height_200        = 200;
  window_graphicshr->bitmap_graphicshr_height_200_BORDER = 200 + GRAPHICSHR_BORDER_SIZE_TOP + GRAPHICSHR_BORDER_SIZE_BOTTOM;
  window_graphicshr->bitmap_graphicshr_height_400        = 400;
  window_graphicshr->bitmap_graphicshr_height_400_BORDER = 400 + GRAPHICSHR_BORDER_SIZE_TOP*2 + GRAPHICSHR_BORDER_SIZE_BOTTOM*2;
  
  /** Bitmap affichant la zone de slection : 320*200 **/
  memset(&bitmap_info,0,sizeof(BITMAPINFO));
  bitmap_info.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
  bitmap_info.bmiHeader.biWidth = window_graphicshr->bitmap_graphicshr_width_320;
  bitmap_info.bmiHeader.biHeight = (-1)*window_graphicshr->bitmap_graphicshr_height_200;
  bitmap_info.bmiHeader.biPlanes = 1;
  bitmap_info.bmiHeader.biBitCount = 32;
  bitmap_info.bmiHeader.biCompression = BI_RGB;
  window_graphicshr->hBitmap_graphicshr_320_200 = CreateDIBSection(window_graphicshr->hDC,&bitmap_info,DIB_RGB_COLORS,(VOID **)&(window_graphicshr->bitmap_data_graphicshr_320_200),NULL,0);
  for(i=0; i<window_graphicshr->bitmap_graphicshr_width_320*window_graphicshr->bitmap_graphicshr_height_200; i++)
    window_graphicshr->bitmap_data_graphicshr_320_200[i] = BITMAP_BACKGROUND_COLOR;

  /** Bitmap affichant la zone de slection : 320*200 + BORDER **/
  memset(&bitmap_info,0,sizeof(BITMAPINFO));
  bitmap_info.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
  bitmap_info.bmiHeader.biWidth = window_graphicshr->bitmap_graphicshr_width_320_BORDER;
  bitmap_info.bmiHeader.biHeight = (-1)*window_graphicshr->bitmap_graphicshr_height_200_BORDER;
  bitmap_info.bmiHeader.biPlanes = 1;
  bitmap_info.bmiHeader.biBitCount = 32;
  bitmap_info.bmiHeader.biCompression = BI_RGB;
  window_graphicshr->hBitmap_graphicshr_320_200_BORDER = CreateDIBSection(window_graphicshr->hDC,&bitmap_info,DIB_RGB_COLORS,(VOID **)&(window_graphicshr->bitmap_data_graphicshr_320_200_BORDER),NULL,0);
  for(i=0; i<window_graphicshr->bitmap_graphicshr_width_320_BORDER*window_graphicshr->bitmap_graphicshr_height_200_BORDER; i++)
    window_graphicshr->bitmap_data_graphicshr_320_200_BORDER[i] = BITMAP_BACKGROUND_COLOR;

  /** Bitmap affichant la zone de slection : 640*200 **/
  memset(&bitmap_info,0,sizeof(BITMAPINFO));
  bitmap_info.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
  bitmap_info.bmiHeader.biWidth = window_graphicshr->bitmap_graphicshr_width_640;
  bitmap_info.bmiHeader.biHeight = (-1)*window_graphicshr->bitmap_graphicshr_height_200;
  bitmap_info.bmiHeader.biPlanes = 1;
  bitmap_info.bmiHeader.biBitCount = 32;
  bitmap_info.bmiHeader.biCompression = BI_RGB;
  window_graphicshr->hBitmap_graphicshr_640_200 = CreateDIBSection(window_graphicshr->hDC,&bitmap_info,DIB_RGB_COLORS,(VOID **)&(window_graphicshr->bitmap_data_graphicshr_640_200),NULL,0);
  for(i=0; i<window_graphicshr->bitmap_graphicshr_width_640*window_graphicshr->bitmap_graphicshr_height_200; i++)
    window_graphicshr->bitmap_data_graphicshr_640_200[i] = BITMAP_BACKGROUND_COLOR;

  /** Bitmap affichant la zone de slection : 640*200 + BORDER **/
  memset(&bitmap_info,0,sizeof(BITMAPINFO));
  bitmap_info.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
  bitmap_info.bmiHeader.biWidth = window_graphicshr->bitmap_graphicshr_width_640_BORDER;
  bitmap_info.bmiHeader.biHeight = (-1)*window_graphicshr->bitmap_graphicshr_height_200_BORDER;
  bitmap_info.bmiHeader.biPlanes = 1;
  bitmap_info.bmiHeader.biBitCount = 32;
  bitmap_info.bmiHeader.biCompression = BI_RGB;
  window_graphicshr->hBitmap_graphicshr_640_200_BORDER = CreateDIBSection(window_graphicshr->hDC,&bitmap_info,DIB_RGB_COLORS,(VOID **)&(window_graphicshr->bitmap_data_graphicshr_640_200_BORDER),NULL,0);
  for(i=0; i<window_graphicshr->bitmap_graphicshr_width_640_BORDER*window_graphicshr->bitmap_graphicshr_height_200_BORDER; i++)
    window_graphicshr->bitmap_data_graphicshr_640_200_BORDER[i] = BITMAP_BACKGROUND_COLOR;

  /** Bitmap affichant la zone de slection : 640*400 **/
  memset(&bitmap_info,0,sizeof(BITMAPINFO));
  bitmap_info.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
  bitmap_info.bmiHeader.biWidth = window_graphicshr->bitmap_graphicshr_width_640;
  bitmap_info.bmiHeader.biHeight = (-1)*window_graphicshr->bitmap_graphicshr_height_400;
  bitmap_info.bmiHeader.biPlanes = 1;
  bitmap_info.bmiHeader.biBitCount = 32;
  bitmap_info.bmiHeader.biCompression = BI_RGB;
  window_graphicshr->hBitmap_graphicshr_640_400 = CreateDIBSection(window_graphicshr->hDC,&bitmap_info,DIB_RGB_COLORS,(VOID **)&(window_graphicshr->bitmap_data_graphicshr_640_400),NULL,0);
  for(i=0; i<window_graphicshr->bitmap_graphicshr_width_640*window_graphicshr->bitmap_graphicshr_height_400; i++)
    window_graphicshr->bitmap_data_graphicshr_640_400[i] = BITMAP_BACKGROUND_COLOR;

  /** Bitmap affichant la zone de slection : 640*400 + BORDER **/
  memset(&bitmap_info,0,sizeof(BITMAPINFO));
  bitmap_info.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
  bitmap_info.bmiHeader.biWidth = window_graphicshr->bitmap_graphicshr_width_640_BORDER;
  bitmap_info.bmiHeader.biHeight = (-1)*window_graphicshr->bitmap_graphicshr_height_400_BORDER;
  bitmap_info.bmiHeader.biPlanes = 1;
  bitmap_info.bmiHeader.biBitCount = 32;
  bitmap_info.bmiHeader.biCompression = BI_RGB;
  window_graphicshr->hBitmap_graphicshr_640_400_BORDER = CreateDIBSection(window_graphicshr->hDC,&bitmap_info,DIB_RGB_COLORS,(VOID **)&(window_graphicshr->bitmap_data_graphicshr_640_400_BORDER),NULL,0);
  for(i=0; i<window_graphicshr->bitmap_graphicshr_width_640_BORDER*window_graphicshr->bitmap_graphicshr_height_400_BORDER; i++)
    window_graphicshr->bitmap_data_graphicshr_640_400_BORDER[i] = BITMAP_BACKGROUND_COLOR;

  /** Construction de la Page Graphique + Rafraichissement de la fentre **/
  RefreshGraphicSHRWindowData(window_graphicshr,1);

  /** Remplissage /Affichage de la Bitpmap Palette **/
  window_graphicshr->num_palette = 1;
  ChangePaletteNumber(0,window_graphicshr);

  /* OK */
  return(0);
}


/*************************************************************************************************/
/*  subEditGraphicSHRProcControl() :  Rcupre les vnements qui arrivent dans le Control Edit. */
/*************************************************************************************************/
LRESULT CALLBACK subEditGraphicSHRProcControl(HWND hwnd_control, UINT msg, WPARAM wParam, LPARAM lParam)
{
  int index;
  struct window_graphicshr *window_graphicshr;
  my_Memory(MEMORY_GET_WINDOW_GRAPHICSHR,&window_graphicshr,NULL);

  /* Rien  faire */
  if(window_graphicshr == NULL)
    return(0);

  /** Identifie le Control **/
  /* Src */
  if(GetDlgItem(window_graphicshr->hwnd_graphicshr,IDC_EDIT_BANK) == hwnd_control)
    index = 1;
  else if(GetDlgItem(window_graphicshr->hwnd_graphicshr,IDC_EDIT_ADDRESS) == hwnd_control)
    index = 2;
  else
    return(0);      /* Pas trouv */

  /** Dtecte l'vnement du Return dans l'Edit **/
  if(msg == WM_CHAR && wParam == VK_RETURN)
    {
      /* Enlve le focus de l'Edit */
      SetFocus(window_graphicshr->hwnd_graphicshr);

      /* On force la MAJ de la fentre */
      RefreshGraphicSHRWindowData(window_graphicshr,1);
    }
  else if(index == 1)
    return(CallWindowProc(window_graphicshr->oldEditProcBank,hwnd_control,msg,wParam,lParam)); 
  else if(index == 2)
    return(CallWindowProc(window_graphicshr->oldEditProcAddress,hwnd_control,msg,wParam,lParam)); 
         
  /* OK */
  return(0);
}


/**********************************************************************/
/*  ChangePaletteNumber() :  Change le numro de la palette affiche. */
/**********************************************************************/
static void ChangePaletteNumber(int increment, struct window_graphicshr *window_graphicshr)
{
  int i;
  char buffer[1024];
  struct palette *current_palette;

  /* Vrification des limites */
  if(increment == 1 && window_graphicshr->num_palette == window_graphicshr->nb_palette)
    return;
  if(increment == -1 && window_graphicshr->num_palette == 1)
    return;

  /* Recherche la palette */
  window_graphicshr->num_palette += increment;
  for(i=1, current_palette=window_graphicshr->first_palette; current_palette; current_palette=current_palette->next,i++)
    if(i == window_graphicshr->num_palette)
      break;

  /* Transfert des couleurs et du nom */
  window_graphicshr->nb_color = current_palette->nb_color;
  for(i=0; i<current_palette->nb_color; i++)
    {
      window_graphicshr->palette_red[i] = current_palette->palette_red[i];
      window_graphicshr->palette_green[i] = current_palette->palette_green[i];
      window_graphicshr->palette_blue[i] = current_palette->palette_blue[i];
    }
  strcpy(window_graphicshr->palette_name,current_palette->name);

  /** MAJ de la Pixmap affichant les couleurs **/
  BuildPaletteWindowAllBitmap(window_graphicshr);

  /* MAJ de la Group Box */
  sprintf(buffer,"Palette #%d/%d : %d Colors - %s",window_graphicshr->num_palette,window_graphicshr->nb_palette,current_palette->nb_color,window_graphicshr->palette_name);
  SetWindowText(GetDlgItem(window_graphicshr->hwnd_graphicshr,IDC_GROUP_PALETTE),buffer);

  /** MAJ de la Pixmap Palette de la fentre Palette **/
  InvalidateRect(window_graphicshr->hwnd_graphicshr,NULL,TRUE);

  /** Rafrachit la fentre avec l'image en cours **/
  if(window_graphicshr->scb_palette == GRAPHICSHR_SCBPALETTE_FIXED)
    RefreshGraphicSHRWindowData(window_graphicshr,1);
}


/****************************************************************/
/*  UpdateFillMode() :  Active le Mode Fill sur les images 320. */
/****************************************************************/
static void UpdateFillMode(struct window_graphicshr *window_graphicshr)
{
  /* Inverse la valeur */
  if(window_graphicshr->fill_mode == 0)
    window_graphicshr->fill_mode = 1;
  else
    window_graphicshr->fill_mode = 0;

  /** MAJ fentre graphique **/
  RefreshGraphicSHRWindowData(window_graphicshr,1);
}


/******************************************************************************/
/*  UpdateDoubleHeight() :  Double la hauteur des Pixels pour les images 640. */
/******************************************************************************/
static void UpdateDoubleHeight(struct window_graphicshr *window_graphicshr)
{
  /* Inverse la valeur */
  if(window_graphicshr->double_height == 0)
    window_graphicshr->double_height = 1;
  else
    window_graphicshr->double_height = 0;

  /** MAJ fentre graphique **/
  RefreshGraphicSHRWindowData(window_graphicshr,1);
}


/*************************************************************************/
/*  UpdateDisplayBorder() :  Active l'affichage des bordures de l'image. */
/*************************************************************************/
static void UpdateDisplayBorder(struct window_graphicshr *window_graphicshr)
{
  /* Inverse la valeur */
  if(window_graphicshr->display_border == 0)
    window_graphicshr->display_border = 1;
  else
    window_graphicshr->display_border = 0;

  /** MAJ fentre graphique **/
  RefreshGraphicSHRWindowData(window_graphicshr,1);
}


/*********************************************************************************/
/*  UpdateDisplaySCB() :  Active l'affichage des SCB dans la bordure de l'image. */
/*********************************************************************************/
static void UpdateDisplaySCB(struct window_graphicshr *window_graphicshr)
{
  /* Inverse la valeur */
  if(window_graphicshr->display_scb == 0)
    window_graphicshr->display_scb = 1;
  else
    window_graphicshr->display_scb = 0;

  /** MAJ fentre graphique **/
  RefreshGraphicSHRWindowData(window_graphicshr,1);
}


/*******************************************************************************/
/*  UpdateDisplayVideoSpot() :  Active l'affichage du spot video dans l'image. */
/*******************************************************************************/
static void UpdateDisplayVideoSpot(struct window_graphicshr *window_graphicshr)
{
  /* Inverse la valeur */
  if(window_graphicshr->display_video_spot == 0)
    window_graphicshr->display_video_spot = 1;
  else
    window_graphicshr->display_video_spot = 0;

  /** MAJ fentre graphique **/
  RefreshGraphicSHRWindowData(window_graphicshr,1);
}


/**************************************************************************************/
/*  UpdateGraphicSHRClientAllArea() :  Affichage des Bitmap de la fentre GraphicSHR. */
/**************************************************************************************/
void UpdateGraphicSHRClientAllArea(HDC hDC, struct window_graphicshr *window_graphicshr)
{
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Palette **/
  UpdatePaletteClientPaletteArea(hDC,window_graphicshr);

  /** Bitmap **/
  UpdateGraphicSHRClientBitmapArea(hDC,window_graphicshr);
}


/*****************************************************************************/
/*  UpdateGraphicSHRClientBitmapArea() :  Affichage de la Bitmap GraphicSHR. */
/*****************************************************************************/
static void UpdateGraphicSHRClientBitmapArea(HDC hDC, struct window_graphicshr *window_graphicshr)
{
  HGDIOBJ bitmap_old;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Bitmap avec la zone mmoire et la zone slectionne **/
  if(window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_320_200)
    {
      bitmap_old = SelectObject(window_graphicshr->hCompatibleDC,window_graphicshr->hBitmap_graphicshr_320_200);
      StretchBlt(hDC,window_graphicshr->bitmap_graphicshr_x,window_graphicshr->bitmap_graphicshr_y,
                 window_graphicshr->bitmap_graphicshr_width_320*window_graphicshr->zoom_level,
                 window_graphicshr->bitmap_graphicshr_height_200*window_graphicshr->zoom_level,
                 window_graphicshr->hCompatibleDC,0,0,window_graphicshr->bitmap_graphicshr_width_320,window_graphicshr->bitmap_graphicshr_height_200,SRCCOPY);
      SelectObject(window_graphicshr->hCompatibleDC,bitmap_old);
    }
  else if(window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_320_200_BORDER)
    {
      bitmap_old = SelectObject(window_graphicshr->hCompatibleDC,window_graphicshr->hBitmap_graphicshr_320_200_BORDER);
      StretchBlt(hDC,window_graphicshr->bitmap_graphicshr_x,window_graphicshr->bitmap_graphicshr_y,
                 window_graphicshr->bitmap_graphicshr_width_320_BORDER*window_graphicshr->zoom_level,
                 window_graphicshr->bitmap_graphicshr_height_200_BORDER*window_graphicshr->zoom_level,
                 window_graphicshr->hCompatibleDC,0,0,window_graphicshr->bitmap_graphicshr_width_320_BORDER,window_graphicshr->bitmap_graphicshr_height_200_BORDER,SRCCOPY);
      SelectObject(window_graphicshr->hCompatibleDC,bitmap_old);
    }
  else if(window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_200)
    {
      bitmap_old = SelectObject(window_graphicshr->hCompatibleDC,window_graphicshr->hBitmap_graphicshr_640_200);
      StretchBlt(hDC,window_graphicshr->bitmap_graphicshr_x,window_graphicshr->bitmap_graphicshr_y,
                 window_graphicshr->bitmap_graphicshr_width_640*window_graphicshr->zoom_level,
                 window_graphicshr->bitmap_graphicshr_height_200*window_graphicshr->zoom_level,
                 window_graphicshr->hCompatibleDC,0,0,window_graphicshr->bitmap_graphicshr_width_640,window_graphicshr->bitmap_graphicshr_height_200,SRCCOPY);
      SelectObject(window_graphicshr->hCompatibleDC,bitmap_old);
    }
  else if(window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_200_BORDER)
    {
      bitmap_old = SelectObject(window_graphicshr->hCompatibleDC,window_graphicshr->hBitmap_graphicshr_640_200_BORDER);
      StretchBlt(hDC,window_graphicshr->bitmap_graphicshr_x,window_graphicshr->bitmap_graphicshr_y,
                 window_graphicshr->bitmap_graphicshr_width_640_BORDER*window_graphicshr->zoom_level,
                 window_graphicshr->bitmap_graphicshr_height_200_BORDER*window_graphicshr->zoom_level,
                 window_graphicshr->hCompatibleDC,0,0,window_graphicshr->bitmap_graphicshr_width_640_BORDER,window_graphicshr->bitmap_graphicshr_height_200_BORDER,SRCCOPY);
      SelectObject(window_graphicshr->hCompatibleDC,bitmap_old);
    }
  else if(window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_400)
    {
      bitmap_old = SelectObject(window_graphicshr->hCompatibleDC,window_graphicshr->hBitmap_graphicshr_640_400);
      StretchBlt(hDC,window_graphicshr->bitmap_graphicshr_x,window_graphicshr->bitmap_graphicshr_y,
                 window_graphicshr->bitmap_graphicshr_width_640*window_graphicshr->zoom_level,
                 window_graphicshr->bitmap_graphicshr_height_400*window_graphicshr->zoom_level,
                 window_graphicshr->hCompatibleDC,0,0,window_graphicshr->bitmap_graphicshr_width_640,window_graphicshr->bitmap_graphicshr_height_400,SRCCOPY);
      SelectObject(window_graphicshr->hCompatibleDC,bitmap_old);
    }
  else if(window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_400_BORDER)
    {
      bitmap_old = SelectObject(window_graphicshr->hCompatibleDC,window_graphicshr->hBitmap_graphicshr_640_400_BORDER);
      StretchBlt(hDC,window_graphicshr->bitmap_graphicshr_x,window_graphicshr->bitmap_graphicshr_y,
                 window_graphicshr->bitmap_graphicshr_width_640_BORDER*window_graphicshr->zoom_level,
                 window_graphicshr->bitmap_graphicshr_height_400_BORDER*window_graphicshr->zoom_level,
                 window_graphicshr->hCompatibleDC,0,0,window_graphicshr->bitmap_graphicshr_width_640_BORDER,window_graphicshr->bitmap_graphicshr_height_400_BORDER,SRCCOPY);
      SelectObject(window_graphicshr->hCompatibleDC,bitmap_old);
    }
}


/*******************************************************************************/
/*  UpdateGraphicSHRAddress() :  Dplace l'adresse pour la fentre GraphicSHR. */
/*******************************************************************************/
static void UpdateGraphicSHRAddress(struct window_graphicshr *window_graphicshr, int delta)
{
  int current_address, new_address, graphicshr_bank, graphicshr_address;
  char buffer[BUFFER_SIZE];

  /* Adresse actuelle */
  current_address = (window_graphicshr->graphicshr_bank << 16) | window_graphicshr->graphicshr_address;
  if((current_address + delta) < 0)
    return;
  if((current_address + delta) > 0xFFFFFF)
    return;

  /* Nouvelle Addresse */
  new_address = current_address + delta;
  graphicshr_bank = (new_address >> 16) & 0xFF;
  graphicshr_address = new_address & 0xFFFF;

  /* Replace les valeurs dans les Control pour montrer ce qui a t compris */
  sprintf(buffer,"%02X",graphicshr_bank);
  SetDlgItemText(window_graphicshr->hwnd_graphicshr,IDC_EDIT_BANK,buffer);
  sprintf(buffer,"%04X",graphicshr_address);
  SetDlgItemText(window_graphicshr->hwnd_graphicshr,IDC_EDIT_ADDRESS,buffer);

  /* MAJ de la fentre */
  RefreshGraphicSHRWindowData(window_graphicshr,1);
}


/*******************************************************************************************************/
/*  UpdateGraphicSHRContextInformation() :  MAJ des information de contexte de la fenetre GraphicSHR. */
/*******************************************************************************************************/
static void UpdateGraphicSHRContextInformation(struct window_graphicshr *window_graphicshr, int mouse_x, int mouse_y)
{
  int pixmap_x, pixmap_y, grid_x, grid_y, graphicshr_data_offset;
  DWORD graphicshr_data_address;
  unsigned char graphicshr_data;
  char buffer_title[1024];

  /* Rien  faire */
  if(window_graphicshr == NULL)
    return;

  /** Est t'on dans l'image ? **/
  if(window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_320_200)
    {
      if(mouse_x < window_graphicshr->bitmap_graphicshr_x || mouse_x >= (window_graphicshr->bitmap_graphicshr_x + window_graphicshr->bitmap_graphicshr_width_320*window_graphicshr->zoom_level) ||
         mouse_y < window_graphicshr->bitmap_graphicshr_y || mouse_y >= (window_graphicshr->bitmap_graphicshr_y + window_graphicshr->bitmap_graphicshr_height_200*window_graphicshr->zoom_level))
        {
          sprintf(buffer_title,"Graphic SHR : %02X/%04X",window_graphicshr->graphicshr_bank,window_graphicshr->graphicshr_address);
          SetWindowText(window_graphicshr->hwnd_graphicshr,buffer_title);
          return;
        }
    }
  else if(window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_320_200_BORDER)
    {
      if(mouse_x < window_graphicshr->bitmap_graphicshr_x || mouse_x >= (window_graphicshr->bitmap_graphicshr_x + window_graphicshr->bitmap_graphicshr_width_320_BORDER*window_graphicshr->zoom_level) ||
         mouse_y < window_graphicshr->bitmap_graphicshr_y || mouse_y >= (window_graphicshr->bitmap_graphicshr_y + window_graphicshr->bitmap_graphicshr_height_200_BORDER*window_graphicshr->zoom_level))
        {
          sprintf(buffer_title,"Graphic SHR : %02X/%04X",window_graphicshr->graphicshr_bank,window_graphicshr->graphicshr_address);
          SetWindowText(window_graphicshr->hwnd_graphicshr,buffer_title);
          return;
        }
    }
  else if(window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_200)
    {
      if(mouse_x < window_graphicshr->bitmap_graphicshr_x || mouse_x >= (window_graphicshr->bitmap_graphicshr_x + window_graphicshr->bitmap_graphicshr_width_640*window_graphicshr->zoom_level) ||
         mouse_y < window_graphicshr->bitmap_graphicshr_y || mouse_y >= (window_graphicshr->bitmap_graphicshr_y + window_graphicshr->bitmap_graphicshr_height_200*window_graphicshr->zoom_level))
        {
          sprintf(buffer_title,"Graphic SHR : %02X/%04X",window_graphicshr->graphicshr_bank,window_graphicshr->graphicshr_address);
          SetWindowText(window_graphicshr->hwnd_graphicshr,buffer_title);
          return;
        }
    }
  else if(window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_200_BORDER)
    {
      if(mouse_x < window_graphicshr->bitmap_graphicshr_x || mouse_x >= (window_graphicshr->bitmap_graphicshr_x + window_graphicshr->bitmap_graphicshr_width_640_BORDER*window_graphicshr->zoom_level) ||
         mouse_y < window_graphicshr->bitmap_graphicshr_y || mouse_y >= (window_graphicshr->bitmap_graphicshr_y + window_graphicshr->bitmap_graphicshr_height_200_BORDER*window_graphicshr->zoom_level))
        {
          sprintf(buffer_title,"Graphic SHR : %02X/%04X",window_graphicshr->graphicshr_bank,window_graphicshr->graphicshr_address);
          SetWindowText(window_graphicshr->hwnd_graphicshr,buffer_title);
          return;
        }
    }
  else if(window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_400)
    {
      if(mouse_x < window_graphicshr->bitmap_graphicshr_x || mouse_x >= (window_graphicshr->bitmap_graphicshr_x + window_graphicshr->bitmap_graphicshr_width_640*window_graphicshr->zoom_level) ||
         mouse_y < window_graphicshr->bitmap_graphicshr_y || mouse_y >= (window_graphicshr->bitmap_graphicshr_y + window_graphicshr->bitmap_graphicshr_height_400*window_graphicshr->zoom_level))
        {
          sprintf(buffer_title,"Graphic SHR : %02X/%04X",window_graphicshr->graphicshr_bank,window_graphicshr->graphicshr_address);
          SetWindowText(window_graphicshr->hwnd_graphicshr,buffer_title);
          return;
        }
    }
  else if(window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_400_BORDER)
    {
      if(mouse_x < window_graphicshr->bitmap_graphicshr_x || mouse_x >= (window_graphicshr->bitmap_graphicshr_x + window_graphicshr->bitmap_graphicshr_width_640_BORDER*window_graphicshr->zoom_level) ||
         mouse_y < window_graphicshr->bitmap_graphicshr_y || mouse_y >= (window_graphicshr->bitmap_graphicshr_y + window_graphicshr->bitmap_graphicshr_height_400_BORDER*window_graphicshr->zoom_level))
        {
          sprintf(buffer_title,"Graphic SHR : %02X/%04X",window_graphicshr->graphicshr_bank,window_graphicshr->graphicshr_address);
          SetWindowText(window_graphicshr->hwnd_graphicshr,buffer_title);
          return;
        }
    }

  /* Coordonnes dans la Pixmap */
  pixmap_x = (mouse_x - window_graphicshr->bitmap_graphicshr_x)/window_graphicshr->zoom_level;
  pixmap_y = (mouse_y - window_graphicshr->bitmap_graphicshr_y)/window_graphicshr->zoom_level;

  /* Coordonnes dans la grille 640x200 */
  grid_x = pixmap_x;     /* 0-319 ou 0-639 */
  grid_y = pixmap_y;     /* 0-199 ou 0-399 */
  if(window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_400 || window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_400_BORDER)
    grid_y /= 2;         /* 0-199 */
  if(window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_320_200_BORDER)
    {
      grid_x -= GRAPHICSHR_BORDER_SIZE_LEFT;
      grid_y -= GRAPHICSHR_BORDER_SIZE_TOP;
      if(grid_x < 0 || grid_y < 0 || grid_x >= 320 || grid_y >= 200)
        {
          sprintf(buffer_title,"Graphic SHR : %02X/%04X",window_graphicshr->graphicshr_bank,window_graphicshr->graphicshr_address);
          SetWindowText(window_graphicshr->hwnd_graphicshr,buffer_title);
          return;
        }
    }
  else if(window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_200_BORDER)
    {
      grid_x -= GRAPHICSHR_BORDER_SIZE_LEFT*2;
      grid_y -= GRAPHICSHR_BORDER_SIZE_TOP;
      if(grid_x < 0 || grid_y < 0 || grid_x >= 640 || grid_y >= 200)
        {
          sprintf(buffer_title,"Graphic SHR : %02X/%04X",window_graphicshr->graphicshr_bank,window_graphicshr->graphicshr_address);
          SetWindowText(window_graphicshr->hwnd_graphicshr,buffer_title);
          return;
        }
    }
  else if(window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_400_BORDER)
    {
      grid_x -= GRAPHICSHR_BORDER_SIZE_LEFT*2;
      grid_y -= GRAPHICSHR_BORDER_SIZE_TOP;
      if(grid_x < 0 || grid_y < 0 || grid_x >= 640 || grid_y >= 200)
        {
          sprintf(buffer_title,"Graphic SHR : %02X/%04X",window_graphicshr->graphicshr_bank,window_graphicshr->graphicshr_address);
          SetWindowText(window_graphicshr->hwnd_graphicshr,buffer_title);
          return;
        }
    }

  /* Adresse */
  if(window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_320_200 || window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_320_200_BORDER)
    {
      graphicshr_data_offset = grid_x/2;
      graphicshr_data_offset += grid_y*160;
    }
  else if(window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_200 || window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_200_BORDER)
    {
      graphicshr_data_offset = grid_x/4;
      graphicshr_data_offset += grid_y*160;
    }
  else if(window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_400 || window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_400_BORDER)
    {
      graphicshr_data_offset = grid_x/4;
      graphicshr_data_offset += grid_y*160;
    }
  graphicshr_data_address = (window_graphicshr->graphicshr_bank << 16) + window_graphicshr->graphicshr_address + graphicshr_data_offset;

  /* Valeur */
  graphicshr_data = window_graphicshr->graphicshr_data[graphicshr_data_offset];

  /* Titre de la fentre + Message */
  sprintf(buffer_title,"Graphic SHR : %02X/%04X / Memory Data  %02X/%04X : %02X  %d,%d",window_graphicshr->graphicshr_bank,window_graphicshr->graphicshr_address,
                                                 graphicshr_data_address>>16,graphicshr_data_address&0xFFFF,graphicshr_data,grid_x,grid_y);
  SetWindowText(window_graphicshr->hwnd_graphicshr,buffer_title);
}


/************************************************************************************************/
/*  UpdateGraphicSHRRefreshMode() :  MAJ du mode de rafraichissement de la fenetre GraphicSHR. */
/************************************************************************************************/
static void UpdateGraphicSHRRefreshMode(struct window_graphicshr *window_graphicshr, int refresh_mode)
{
  struct notification *current_notification;

  if(refresh_mode == GRAPHICSHR_REFRESH_AUTOMATIC)
    {
      /* Visuel */
      window_graphicshr->refresh_mode = GRAPHICSHR_REFRESH_AUTOMATIC;
      SendDlgItemMessage(window_graphicshr->hwnd_graphicshr,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_graphicshr->hwnd_graphicshr,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,0,0);

      /* Cration d'une Notification */
      current_notification = (struct notification *) calloc(1,sizeof(struct notification));
      if(current_notification == NULL)
        return;
      current_notification->hwnd_window = window_graphicshr->hwnd_graphicshr;
      current_notification->message = HM_GRAPHICSHR_REFRESHWINDOW;
      current_notification->type = NOTIFICATION_TYPE_VIEW;
      current_notification->sub_type = NOTIFICATION_SUBTYPE_GRAPHICSHR;

      /* Ajout de la notification */
      window_graphicshr->current_notification = current_notification;
      my_Notification(NOTIFICATION_ADD,current_notification,NULL);
    }
  else
    {
      /* Visuel */
      window_graphicshr->refresh_mode = GRAPHICSHR_REFRESH_MANUAL;
      SendDlgItemMessage(window_graphicshr->hwnd_graphicshr,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_graphicshr->hwnd_graphicshr,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,1,0);

      /* Supprime la Notification */
      my_Notification(NOTIFICATION_REMOVE,window_graphicshr->current_notification,NULL);
      window_graphicshr->current_notification = NULL;
    }
}


/***********************************************************************************/
/*  UpdateGraphicSHRZoomLevel() :  MAJ du nieau de Zomm de la fenetre GraphicSHR. */
/***********************************************************************************/
static void UpdateGraphicSHRZoomLevel(struct window_graphicshr *window_graphicshr, int zoom_level)
{
  if(zoom_level == GRAPHICSHR_ZOOM_X1)
    {
      /* Visuel */
      window_graphicshr->zoom_level = GRAPHICSHR_ZOOM_X1;
      SendDlgItemMessage(window_graphicshr->hwnd_graphicshr,IDC_RADIO_ZOOM_X1,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_graphicshr->hwnd_graphicshr,IDC_RADIO_ZOOM_X2,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_graphicshr->hwnd_graphicshr,IDC_RADIO_ZOOM_X3,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_graphicshr->hwnd_graphicshr,IDC_RADIO_ZOOM_X4,BM_SETCHECK,0,0);
    }
  else if(zoom_level == GRAPHICSHR_ZOOM_X2)
    {
      /* Visuel */
      window_graphicshr->zoom_level = GRAPHICSHR_ZOOM_X2;
      SendDlgItemMessage(window_graphicshr->hwnd_graphicshr,IDC_RADIO_ZOOM_X1,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_graphicshr->hwnd_graphicshr,IDC_RADIO_ZOOM_X2,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_graphicshr->hwnd_graphicshr,IDC_RADIO_ZOOM_X3,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_graphicshr->hwnd_graphicshr,IDC_RADIO_ZOOM_X4,BM_SETCHECK,0,0);
    }
  else if(zoom_level == GRAPHICSHR_ZOOM_X3)
    {
      /* Visuel */
      window_graphicshr->zoom_level = GRAPHICSHR_ZOOM_X3;
      SendDlgItemMessage(window_graphicshr->hwnd_graphicshr,IDC_RADIO_ZOOM_X1,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_graphicshr->hwnd_graphicshr,IDC_RADIO_ZOOM_X2,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_graphicshr->hwnd_graphicshr,IDC_RADIO_ZOOM_X3,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_graphicshr->hwnd_graphicshr,IDC_RADIO_ZOOM_X4,BM_SETCHECK,0,0);
    }
  else if(zoom_level == GRAPHICSHR_ZOOM_X4)
    {
      /* Visuel */
      window_graphicshr->zoom_level = GRAPHICSHR_ZOOM_X4;
      SendDlgItemMessage(window_graphicshr->hwnd_graphicshr,IDC_RADIO_ZOOM_X1,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_graphicshr->hwnd_graphicshr,IDC_RADIO_ZOOM_X2,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_graphicshr->hwnd_graphicshr,IDC_RADIO_ZOOM_X3,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_graphicshr->hwnd_graphicshr,IDC_RADIO_ZOOM_X4,BM_SETCHECK,1,0);
    } 
  
  /* On force la MAJ de la fentre */  
  RefreshGraphicSHRWindowData(window_graphicshr,1);    
}


/******************************************************************************/
/*  RefreshGraphicSHRWindowData() :  MAJ du contenu de la fentre GraphicSHR. */
/******************************************************************************/
static void RefreshGraphicSHRWindowData(struct window_graphicshr *window_graphicshr, int force)
{
  int length, is_modified;
  int graphicshr_bank, graphicshr_address, nb_column, nb_row;
  char buffer[1024];

  /* Rcupre la valeur du GraphicSHR Pointer */
  length = GetDlgItemText(window_graphicshr->hwnd_graphicshr,IDC_EDIT_BANK,buffer,1024);
  sscanf(buffer,"%X",&graphicshr_bank);
  length = GetDlgItemText(window_graphicshr->hwnd_graphicshr,IDC_EDIT_ADDRESS,buffer,1024);
  sscanf(buffer,"%X",&graphicshr_address);
  nb_column = GetDlgItemInt(window_graphicshr->hwnd_graphicshr,IDC_EDIT_COLUMN,NULL,FALSE);
  if(nb_column < 1)
    nb_column = 1;
  nb_row = GetDlgItemInt(window_graphicshr->hwnd_graphicshr,IDC_EDIT_ROW,NULL,FALSE);
  if(nb_row < 1)
    nb_row = 1;

  /* Replace les valeurs dans les Control pour montrer ce qui a t compris */
  sprintf(buffer,"%02X",graphicshr_bank);
  SetDlgItemText(window_graphicshr->hwnd_graphicshr,IDC_EDIT_BANK,buffer);
  sprintf(buffer,"%04X",graphicshr_address);
  SetDlgItemText(window_graphicshr->hwnd_graphicshr,IDC_EDIT_ADDRESS,buffer);
  SetDlgItemInt(window_graphicshr->hwnd_graphicshr,IDC_EDIT_COLUMN,nb_column,FALSE);
  SetDlgItemInt(window_graphicshr->hwnd_graphicshr,IDC_EDIT_ROW,nb_row,FALSE);

  /** Les donnes ont t'elles t modifies + Rcupration des nouvelles Data si besoin **/
  is_modified = IsGraphicSHRDataModified(graphicshr_bank,graphicshr_address,nb_column,nb_row,window_graphicshr);
  if(is_modified == 0 && force == 0)
    {
      /* Retaille la fentre */
      ResizeGraphicSHRWindow(window_graphicshr);
      return;
    }
   
  /** Cration de la Zone Graphique  partir des nouvelles donnes **/
  BuildGraphicSHRFromData(window_graphicshr);

  /** Cration des 16 Palettes de l'images **/
  BuildPicturePalette(window_graphicshr);

  /* Retaille la fentre */
  ResizeGraphicSHRWindow(window_graphicshr);

  /* Rafraichit la fentre */
  InvalidateRect(window_graphicshr->hwnd_graphicshr,NULL,TRUE);
  UpdateWindow(window_graphicshr->hwnd_graphicshr);
}


/*************************************************************************************************************/
/*  IsGraphicSHRDataModified() :  Les donnes GraphicSHR ont t'elles t modifies depuis la dernirer fois. */
/*************************************************************************************************************/
static int IsGraphicSHRDataModified(int graphicshr_bank, int graphicshr_address, int nb_column, int nb_row, struct window_graphicshr *window_graphicshr)
{
  int i, graphicshr_data_address, picture_format, border_color;
  DWORD border_color_rgb;
  unsigned char col_0r;
  unsigned char col_gb;
  unsigned char graphicshr_data[32768];   /* 32 KB */
  struct mmu *current_mmu;
  DWORD tab_rgb_color_border[16] = 
    {
      /* 16 couleurs de la Bordure / Fond Ecran / Text */
      MAKECOLOR(0,0,0), MAKECOLOR(221,0,51), MAKECOLOR(0,0,153), MAKECOLOR(221,34,221),
      MAKECOLOR(0,119,34), MAKECOLOR(85,85,85), MAKECOLOR(34,34,255), MAKECOLOR(102,170,255),
      MAKECOLOR(136,85,0), MAKECOLOR(255,102,0), MAKECOLOR(170,170,170), MAKECOLOR(255,153,136),
      MAKECOLOR(17,221,0), MAKECOLOR(255,255,0), MAKECOLOR(68,255,153), MAKECOLOR(255,255,255)
    };
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Rcupre le MMU */
  my_Memory(MEMORY_GET_S65816_MMU,&current_mmu,NULL);

  /* Rcupre les donnes brutes : 32768 bytes */
  graphicshr_data_address = (graphicshr_bank << 16) | graphicshr_address;
  ReadMemoryBlock(graphicshr_data_address,32768,graphicshr_data,current_mmu);

  /* SCB + Palette */
  if(window_graphicshr->scb_palette == GRAPHICSHR_SCBPALETTE_CURRENT)
    ;
  else if(window_graphicshr->scb_palette == GRAPHICSHR_SCBPALETTE_E12000)
    ReadMemoryBlock(0xE19D00,768,graphicshr_data+0x7D00,current_mmu);
  else if(window_graphicshr->scb_palette == GRAPHICSHR_SCBPALETTE_012000)
    ReadMemoryBlock(0x019D00,768,graphicshr_data+0x7D00,current_mmu);
  else if(window_graphicshr->scb_palette == GRAPHICSHR_SCBPALETTE_FIXED)
    {
      /* SCB  0 */
      memset(graphicshr_data+0x7D00,0x00,256);

      /* Palette en cours */
      for(i=0; i<16; i++)
        {
          col_0r = (window_graphicshr->palette_red[i] >> 4) & 0x0F;
          col_gb = ((window_graphicshr->palette_green[i] >> 4) << 4) | (window_graphicshr->palette_blue[i] >> 4);
          graphicshr_data[0x7D00 + 0x0100 + 2*i + 0] = col_gb;
          graphicshr_data[0x7D00 + 0x0100 + 2*i + 1] = col_0r;
        }
    }

  /** Format de l'image : 320 ou 640 **/
  picture_format = GRAPHICSHR_PICTURE_FORMAT_320_200;
  for(i=0; i<200; i++)
    if((graphicshr_data[GRAPHICSHR_SCB_OFFSET+i] & 0x80) == 0x80)
      {
        picture_format = GRAPHICSHR_PICTURE_FORMAT_640_200;
        break;
      }
  /* Affiche la Bordure */
  if(window_graphicshr->display_border == 1 || window_graphicshr->display_scb == 1 || window_graphicshr->display_video_spot == 1)
    {
      if(picture_format == GRAPHICSHR_PICTURE_FORMAT_320_200)
        picture_format = GRAPHICSHR_PICTURE_FORMAT_320_200_BORDER;
      else if(picture_format == GRAPHICSHR_PICTURE_FORMAT_640_200)
        picture_format = GRAPHICSHR_PICTURE_FORMAT_640_200_BORDER;
    }
  /* Aspect Ratio de l'image */
  if(window_graphicshr->double_height == 1)
    {
      if(picture_format == GRAPHICSHR_PICTURE_FORMAT_640_200)
        picture_format = GRAPHICSHR_PICTURE_FORMAT_640_400;
      else if(picture_format == GRAPHICSHR_PICTURE_FORMAT_640_200_BORDER)
        picture_format = GRAPHICSHR_PICTURE_FORMAT_640_400_BORDER;
    }

  /* Couleur de la bordure */
  if(window_graphicshr->border_color == GRAPHICSHR_COLOR_DYNAMIC)
    border_color = ReadMemoryByte(0x00C034,current_mmu) & 0x0F;
  else
    border_color = window_graphicshr->border_color - 1;
  border_color_rgb = tab_rgb_color_border[border_color];

  /** Compare les donnes **/
  if(window_graphicshr->graphicshr_data != NULL)
    if(graphicshr_bank == window_graphicshr->graphicshr_bank &&
       graphicshr_address == window_graphicshr->graphicshr_address &&
       window_graphicshr->picture_format == picture_format &&
       window_graphicshr->border_color_rgb == border_color_rgb &&
       window_graphicshr->C02E == param->operation_C02E &&
       window_graphicshr->C02F == param->operation_C02F &&
       memcmp(&graphicshr_data[0],&window_graphicshr->graphicshr_data[0],32768) == 0)
      return(0);

  /* Les donnes sont diffrentes => MAJ */
  window_graphicshr->graphicshr_bank = graphicshr_bank;
  window_graphicshr->graphicshr_address = graphicshr_address;
  memcpy(window_graphicshr->graphicshr_data,graphicshr_data,32768);
  window_graphicshr->picture_format = picture_format;
  window_graphicshr->border_color_rgb = border_color_rgb;
  window_graphicshr->C02E = param->operation_C02E;
  window_graphicshr->C02F = param->operation_C02F;

  /** Masque les controles **/
  /* Couleur de la bordure */
  if(picture_format == GRAPHICSHR_PICTURE_FORMAT_320_200 || 
     picture_format == GRAPHICSHR_PICTURE_FORMAT_640_200 ||
     picture_format == GRAPHICSHR_PICTURE_FORMAT_640_400)
    EnableWindow(window_graphicshr->hwnd_combo_color_background,FALSE);
  else
    EnableWindow(window_graphicshr->hwnd_combo_color_background,TRUE);
  /* Double la hauteur de l'image */
  if(picture_format == GRAPHICSHR_PICTURE_FORMAT_320_200 || 
     picture_format == GRAPHICSHR_PICTURE_FORMAT_320_200_BORDER)
    EnableWindow(GetDlgItem(window_graphicshr->hwnd_graphicshr,IDC_CHECK_DOUBLE_HEIGHT),FALSE);
  else
    EnableWindow(GetDlgItem(window_graphicshr->hwnd_graphicshr,IDC_CHECK_DOUBLE_HEIGHT),TRUE);

  /* Donnes diffrentes */
  return(1);
}


/**********************************************************************************/
/*  LoadPalette() :  Charge une/plusieurs palette(s) en mmoire depuis le disque. */
/**********************************************************************************/
static void LoadPalette(struct window_graphicshr *window_graphicshr)
{
  int i, j, nb_path, palette_size, nb_color;
  char **file_path_tab;
  char file_name[1024];
  unsigned char *palette_data;
  struct palette *new_palette;

  /** Rcupre le chemin des fichiers **/
  file_path_tab = GetFilePath(window_graphicshr->hwnd_graphicshr,"Select Palette file","All Files (*.*)\0*.*\0Palette (*.pal)\0*.pal\0\0\0",&nb_path,1);
  if(file_path_tab == NULL)
    return;
  if(nb_path == 0)
    {
      mem_free_list(nb_path,file_path_tab);
      return;
    }

  /** Charge tous les fichiers Palette en mmoire **/
  for(i=0; i<nb_path; i++)
    {
      /* Charge les donnes */
      palette_data = LoadFileAsBinary(file_path_tab[i],&palette_size);
      if(palette_data == NULL)
        {
          MessageBox(window_graphicshr->hwnd_graphicshr, "Impossible to load Palette file", file_path_tab[i], MB_OK|MB_ICONWARNING);
          continue;
        }

      /* Vrifie le contenu */
      nb_color = palette_size / 3;
      if(nb_color*3 != palette_size || nb_color < 16)
        {
          MessageBox(window_graphicshr->hwnd_graphicshr, "This file is not a valid 16 Colors Palette file", file_path_tab[i], MB_OK|MB_ICONWARNING);
          free(palette_data);
          continue;
        }

      /* Extrait le nom du fichier  partir du chemin */
      Extract_filename(file_path_tab[i],&file_name[0]);
  
      /* Allocation mmoire Palette */
      new_palette = (struct palette *) calloc(1,sizeof(struct palette));
      if(new_palette == NULL)
        {
          MessageBox(window_graphicshr->hwnd_graphicshr,"Impossible to allocate memory to load Palette file", file_path_tab[i], MB_OK|MB_ICONWARNING);
          free(palette_data);
          continue;
        }
      strcpy(new_palette->path,file_path_tab[i]);
      strcpy(new_palette->name,file_name);

      /* Couleurs */
      new_palette->nb_color = nb_color;
      for(j=0; j<nb_color; j++)
        {
          new_palette->palette_red[j] = palette_data[3*j+0];
          new_palette->palette_green[j] = palette_data[3*j+1];
          new_palette->palette_blue[j] = palette_data[3*j+2];
        }

      /* Rattache la palette */
      window_graphicshr->nb_palette++;
      if(window_graphicshr->first_palette == NULL)
        window_graphicshr->first_palette = new_palette;
      else
        window_graphicshr->last_palette->next = new_palette;
      window_graphicshr->last_palette = new_palette;

      /* Libration mmoire */
      free(palette_data);
    }

  /* Libration mmoire */
  mem_free_list(nb_path,file_path_tab);

  /** Affiche la dernire Palette **/
  window_graphicshr->num_palette = window_graphicshr->nb_palette;
  ChangePaletteNumber(0,window_graphicshr);
}


/*******************************************************************/
/*  BuildGraphicSHRPalette() :  Cration des Palettes 16 couleurs. */
/*******************************************************************/
static void BuildGraphicSHRPalette(struct window_graphicshr *window_graphicshr)
{
  int i;
  WORD apple_standard[16] = {0x000,0x777,0x841,0x72C,0x00F,0x080,0xF70,0xD00,0xFA9,0xFF0,0x0E0,0x4DF,0xDAF,0x78F,0xCCC,0xFFF};
  WORD border_color[16] =   {0x000,0xD03,0x009,0xD2D,0x072,0x555,0x22F,0x6AF,0x850,0xF60,0xAAA,0xF98,0x0D0,0xFF0,0x5F9,0xFFF};
  WORD greyscale[16] =      {0x000,0x111,0x222,0x333,0x444,0x555,0x666,0x777,0x888,0x999,0xAAA,0xBBB,0xCCC,0xDDD,0xEEE,0xFFF};
  unsigned char palette_gr_red[16] =   {0,221,  0,221,  0,85, 34,102,136,255,170,255,  0,255,  0,255};      /* GR */
  unsigned char palette_gr_green[16] = {0,  0,  0,  0,119,85, 34,170, 85,102,170,153,221,255,255,255};
  unsigned char palette_gr_blue[16] =  {0, 51,153,221,  0,85,255,255, 34,  0,170,136,  0,  0,153,255};
  unsigned char palette_dhgr_red[16] =   {0,  0,  0, 34,136,170,  0,  0,221,221, 85,102,255,255,255,255};   /* Couleurs entrelace par rapport au GR */
  unsigned char palette_dhgr_green[16] = {0,  0,119, 34, 85,170,221,255,  0,  0, 85,170,102,153,255,255};
  unsigned char palette_dhgr_blue[16] =  {0,153,  0,255, 34,170,  0,153, 51,221, 85,255,  0,136,  0,255};
  unsigned char palette_ega_red[16] =   {0,  0,  0,  0,170,170,170,170, 85, 85, 85, 85,255,255,255,255};   /* Couleurs EGA */
  unsigned char palette_ega_green[16] = {0,  0,170,170,  0,  0, 85,170, 85, 85,255,255, 85, 85,255,255};
  unsigned char palette_ega_blue[16] =  {0,170,  0,170,  0,170,  0,170, 85,255, 85,255, 85,255, 85,255};
  unsigned char palette_arne_red[16] =   {0, 33, 57, 82,165, 82,165,239,247, 57, 57,181,189,222,165,247};   /* Couleurs ARNE */
  unsigned char palette_arne_green[16] = {0, 49, 82,148,206, 66,115,148,231, 66,165,222, 49,123,165,247};
  unsigned char palette_arne_blue[16] =  {0, 57, 82, 33, 49, 49, 41, 57,115,189,247,239, 66,148,165,231};
  unsigned char palette_atari_red[16] =   {255,255,  0,255,  0,255,  0,170, 85,255, 85,255, 85,255, 85,  0};   /* Couleurs Atari ST */
  unsigned char palette_atari_green[16] = {255,  0,255,255,  0,  0,255,170, 85, 85,255,255, 85, 85,255,  0};
  unsigned char palette_atari_blue[16] =  {255,  0,  0,  0,255,255,255,170, 85, 85, 85, 85,255,255,255,  0};
  int tone_translation_16[16] = {0x00,0x11,0x22,0x33,0x44,0x55,0x66,0x77,0x88,0x99,0xAA,0xBB,0xCC,0xDD,0xEE,0xFF};
  struct palette *current_palette;

  /***************************/
  /** Palette Apple Default **/
  /***************************/
  current_palette = (struct palette *) calloc(1,sizeof(struct palette));
  if(current_palette == NULL)
    return;
  strcpy(current_palette->name,"Apple IIgs Standard ");
  current_palette->nb_color = 16;
  for(i=0; i<16; i++)
    {
      current_palette->palette_red[i]   = tone_translation_16[apple_standard[i] >> 8];
      current_palette->palette_green[i] = tone_translation_16[(apple_standard[i] >> 4) & 0x0F];
      current_palette->palette_blue[i]  = tone_translation_16[apple_standard[i] & 0x0F];
    }
  /* Attachement */
  window_graphicshr->nb_palette++;
  if(window_graphicshr->first_palette == NULL)
    window_graphicshr->first_palette = current_palette;
  else
    window_graphicshr->last_palette->next = current_palette;
  window_graphicshr->last_palette = current_palette;

  /***************************/
  /** Palette Border Colors **/
  /***************************/
  current_palette = (struct palette *) calloc(1,sizeof(struct palette));
  if(current_palette == NULL)
    return;
  strcpy(current_palette->name,"Apple IIgs Border Colors ");
  current_palette->nb_color = 16;
  for(i=0; i<16; i++)
    {
      current_palette->palette_red[i]   = tone_translation_16[border_color[i] >> 8];
      current_palette->palette_green[i] = tone_translation_16[(border_color[i] >> 4) & 0x0F];
      current_palette->palette_blue[i]  = tone_translation_16[border_color[i] & 0x0F];
    }
  /* Attachement */
  window_graphicshr->nb_palette++;
  if(window_graphicshr->first_palette == NULL)
    window_graphicshr->first_palette = current_palette;
  else
    window_graphicshr->last_palette->next = current_palette;
  window_graphicshr->last_palette = current_palette;

  /***********************/
  /** Palette GR Colors **/
  /***********************/
  current_palette = (struct palette *) calloc(1,sizeof(struct palette));
  if(current_palette == NULL)
    return;
  strcpy(current_palette->name,"Apple II GR Colors ");
  current_palette->nb_color = 16;
  for(i=0; i<16; i++)
    {
      current_palette->palette_red[i]   = palette_gr_red[i];
      current_palette->palette_green[i] = palette_gr_green[i];
      current_palette->palette_blue[i]  = palette_gr_blue[i];
    }
  /* Attachement */
  window_graphicshr->nb_palette++;
  if(window_graphicshr->first_palette == NULL)
    window_graphicshr->first_palette = current_palette;
  else
    window_graphicshr->last_palette->next = current_palette;
  window_graphicshr->last_palette = current_palette;


  /*************************/
  /** Palette DHGR Colors **/
  /*************************/
  current_palette = (struct palette *) calloc(1,sizeof(struct palette));
  if(current_palette == NULL)
    return;
  strcpy(current_palette->name,"Appe II DHGR Colors ");
  current_palette->nb_color = 16;
  for(i=0; i<16; i++)
    {
      current_palette->palette_red[i]   = palette_dhgr_red[i];
      current_palette->palette_green[i] = palette_dhgr_green[i];
      current_palette->palette_blue[i]  = palette_dhgr_blue[i];
    }
  /* Attachement */
  window_graphicshr->nb_palette++;
  if(window_graphicshr->first_palette == NULL)
    window_graphicshr->first_palette = current_palette;
  else
    window_graphicshr->last_palette->next = current_palette;
  window_graphicshr->last_palette = current_palette;

  /*****************/
  /** Palette EGA **/
  /*****************/
  current_palette = (struct palette *) calloc(1,sizeof(struct palette));
  if(current_palette == NULL)
    return;
  strcpy(current_palette->name,"EGA 16 Colors ");
  current_palette->nb_color = 16;
  for(i=0; i<16; i++)
    {
      current_palette->palette_red[i]   = palette_ega_red[i];
      current_palette->palette_green[i] = palette_ega_green[i];
      current_palette->palette_blue[i]  = palette_ega_blue[i];
    }
  /* Attachement */
  window_graphicshr->nb_palette++;
  if(window_graphicshr->first_palette == NULL)
    window_graphicshr->first_palette = current_palette;
  else
    window_graphicshr->last_palette->next = current_palette;
  window_graphicshr->last_palette = current_palette;

  /******************/
  /** Palette ARNE **/
  /******************/
  current_palette = (struct palette *) calloc(1,sizeof(struct palette));
  if(current_palette == NULL)
    return;
  strcpy(current_palette->name,"ARNE Palette ");
  current_palette->nb_color = 16;
  for(i=0; i<16; i++)
    {
      current_palette->palette_red[i]   = palette_arne_red[i];
      current_palette->palette_green[i] = palette_arne_green[i];
      current_palette->palette_blue[i]  = palette_arne_blue[i];
    }
  /* Attachement */
  window_graphicshr->nb_palette++;
  if(window_graphicshr->first_palette == NULL)
    window_graphicshr->first_palette = current_palette;
  else
    window_graphicshr->last_palette->next = current_palette;
  window_graphicshr->last_palette = current_palette;

  /**********************/
  /** Palette Atari ST **/
  /**********************/
  current_palette = (struct palette *) calloc(1,sizeof(struct palette));
  if(current_palette == NULL)
    return;
  strcpy(current_palette->name,"Atari ST Standard ");
  current_palette->nb_color = 16;
  for(i=0; i<16; i++)
    {
      current_palette->palette_red[i]   = palette_atari_red[i];
      current_palette->palette_green[i] = palette_atari_green[i];
      current_palette->palette_blue[i]  = palette_atari_blue[i];
    }
  /* Attachement */
  window_graphicshr->nb_palette++;
  if(window_graphicshr->first_palette == NULL)
    window_graphicshr->first_palette = current_palette;
  else
    window_graphicshr->last_palette->next = current_palette;
  window_graphicshr->last_palette = current_palette;

  /***********************/
  /** Palette Greyscale **/
  /***********************/
  current_palette = (struct palette *) calloc(1,sizeof(struct palette));
  if(current_palette == NULL)
    return;
  strcpy(current_palette->name,"Greyscale 16 ");
  current_palette->nb_color = 16;
  for(i=0; i<16; i++)
    {
      current_palette->palette_red[i]   = tone_translation_16[greyscale[i] >> 8];
      current_palette->palette_green[i] = tone_translation_16[(greyscale[i] >> 4) & 0x0F];
      current_palette->palette_blue[i]  = tone_translation_16[greyscale[i] & 0x0F];
    }
  /* Attachement */
  window_graphicshr->nb_palette++;
  if(window_graphicshr->first_palette == NULL)
    window_graphicshr->first_palette = current_palette;
  else
    window_graphicshr->last_palette->next = current_palette;
  window_graphicshr->last_palette = current_palette;
}


/*******************************************************************/
/*  BuildPicturePalette() :  Construit les 16 palettes de l'image. */
/*******************************************************************/
static void BuildPicturePalette(struct window_graphicshr *window_graphicshr)
{
  int i, j;
  unsigned char color_0R;
  unsigned char color_GB;
  struct palette *previous_palette;
  struct palette *current_palette;
  struct palette *next_palette;

  /** Suppression des palettes Pictures **/
  previous_palette = NULL;
  for(current_palette = window_graphicshr->first_palette; current_palette; current_palette=next_palette)
    {
      if(!strnicmp(current_palette->name,"Picture",strlen("Picture")))
        {
          next_palette = current_palette->next;
          /* MAJ Listre chaine */
          previous_palette->next = current_palette->next;
          if(window_graphicshr->last_palette == current_palette)
            window_graphicshr->last_palette = previous_palette;
          free(current_palette);
          window_graphicshr->nb_palette--;
        }
      else
        {
          previous_palette = current_palette;
          next_palette = current_palette->next;
        }
    }

  /* Rien  faire */
  if(window_graphicshr->scb_palette == GRAPHICSHR_SCBPALETTE_FIXED)
    return;

  /** On construit les palettes **/
  for(j=0; j<16; j++)
    {
      current_palette = (struct palette *) calloc(1,sizeof(struct palette));
      if(current_palette == NULL)
        return;
      sprintf(current_palette->name,"Picture Palette #%0X",j);
      current_palette->nb_color = 16;
      for(i=0; i<16; i++)
        {
          color_0R = window_graphicshr->graphicshr_data[GRAPHICSHR_PALETTE_OFFSET + j*32 + 2*i + 1] & 0x0F;
          current_palette->palette_red[i] = (color_0R << 4) | color_0R;
          color_GB = window_graphicshr->graphicshr_data[GRAPHICSHR_PALETTE_OFFSET + j*32 + 2*i + 0];
          current_palette->palette_green[i] = (color_GB & 0xF0) | ((color_GB >> 4) & 0x0F);
          current_palette->palette_blue[i]  = (color_GB & 0x0F) | ((color_GB << 4) & 0xF0);
        }

      /* Attachement */
      window_graphicshr->nb_palette++;
      if(window_graphicshr->first_palette == NULL)
        window_graphicshr->first_palette = current_palette;
      else
        window_graphicshr->last_palette->next = current_palette;
      window_graphicshr->last_palette = current_palette;
    }
}


/***********************************************************************/
/*  ResizeGraphicSHRWindow() : Resize GraphicSHR window by given size. */
/***********************************************************************/
static void ResizeGraphicSHRWindow(struct window_graphicshr *window_graphicshr)
{
  HWND hwnd_group;
  RECT window_rect, client_rect, group_rect;
  int window_x, window_y;
  
  /** On va ajuster la fentre par rapport au Zoom demand **/  
  /* Position actuelle de la fentre dans l'cran */
  GetWindowRect(window_graphicshr->hwnd_graphicshr,&window_rect);
  window_x = window_rect.left;
  window_y = window_rect.top;
    
  /* Hauteur Titre */
  GetClientRect(window_graphicshr->hwnd_graphicshr,&client_rect);
  window_graphicshr->title_height = (window_rect.bottom - window_rect.top) - client_rect.bottom;

  /* Position de la GroupBox Select */
  hwnd_group = GetDlgItem(window_graphicshr->hwnd_graphicshr,IDC_GROUP_GRAPHICSHR);     /* Rcupre le Handle du Control GroupBox li  IDC_GROUP_GRAPHICSHR */
  GetWindowRect(hwnd_group,&group_rect);
  window_graphicshr->group_data_x = (group_rect.left - window_rect.left) - 3;
  window_graphicshr->group_data_y = ((group_rect.top - window_rect.top) - window_graphicshr->title_height) + 3;

  /* Position de la Zone BitMap dans la fentre */
  window_graphicshr->bitmap_graphicshr_x = window_graphicshr->group_data_x + WINDOW_GROUP_BORDER_LEFT_SIZE + 4;
  window_graphicshr->bitmap_graphicshr_y = window_graphicshr->group_data_y + WINDOW_GROUP_BORDER_TOP_SIZE;

  /** Dimension en Pixel du Control Group Box + Resize **/
  if(window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_320_200)
    {
      window_graphicshr->group_data_width = WINDOW_GROUP_BORDER_LEFT_SIZE + 4 + window_graphicshr->bitmap_graphicshr_width_320*window_graphicshr->zoom_level + WINDOW_GROUP_BORDER_RIGHT_SIZE + 4;
      window_graphicshr->group_data_height = WINDOW_GROUP_BORDER_TOP_SIZE + window_graphicshr->bitmap_graphicshr_height_200*window_graphicshr->zoom_level + WINDOW_GROUP_BORDER_BOTTOM_SIZE + 5;
    }
  else if(window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_320_200_BORDER)
    {
      window_graphicshr->group_data_width = WINDOW_GROUP_BORDER_LEFT_SIZE + 4 + window_graphicshr->bitmap_graphicshr_width_320_BORDER*window_graphicshr->zoom_level + WINDOW_GROUP_BORDER_RIGHT_SIZE + 4;
      window_graphicshr->group_data_height = WINDOW_GROUP_BORDER_TOP_SIZE + window_graphicshr->bitmap_graphicshr_height_200_BORDER*window_graphicshr->zoom_level + WINDOW_GROUP_BORDER_BOTTOM_SIZE + 5;
    }
  else if(window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_200)
    {
      window_graphicshr->group_data_width = WINDOW_GROUP_BORDER_LEFT_SIZE + 4 + window_graphicshr->bitmap_graphicshr_width_640*window_graphicshr->zoom_level + WINDOW_GROUP_BORDER_RIGHT_SIZE + 4;
      window_graphicshr->group_data_height = WINDOW_GROUP_BORDER_TOP_SIZE + window_graphicshr->bitmap_graphicshr_height_200*window_graphicshr->zoom_level + WINDOW_GROUP_BORDER_BOTTOM_SIZE + 5;
    }
  else if(window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_200_BORDER)
    {
      window_graphicshr->group_data_width = WINDOW_GROUP_BORDER_LEFT_SIZE + 4 + window_graphicshr->bitmap_graphicshr_width_640_BORDER*window_graphicshr->zoom_level + WINDOW_GROUP_BORDER_RIGHT_SIZE + 4;
      window_graphicshr->group_data_height = WINDOW_GROUP_BORDER_TOP_SIZE + window_graphicshr->bitmap_graphicshr_height_200_BORDER*window_graphicshr->zoom_level + WINDOW_GROUP_BORDER_BOTTOM_SIZE + 5;
    }
  else if(window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_400)
    {
      window_graphicshr->group_data_width = WINDOW_GROUP_BORDER_LEFT_SIZE + 4 + window_graphicshr->bitmap_graphicshr_width_640*window_graphicshr->zoom_level + WINDOW_GROUP_BORDER_RIGHT_SIZE + 4;
      window_graphicshr->group_data_height = WINDOW_GROUP_BORDER_TOP_SIZE + window_graphicshr->bitmap_graphicshr_height_400*window_graphicshr->zoom_level + WINDOW_GROUP_BORDER_BOTTOM_SIZE + 5;
    }
  else if(window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_400_BORDER)
    {
      window_graphicshr->group_data_width = WINDOW_GROUP_BORDER_LEFT_SIZE + 4 + window_graphicshr->bitmap_graphicshr_width_640_BORDER*window_graphicshr->zoom_level + WINDOW_GROUP_BORDER_RIGHT_SIZE + 4;
      window_graphicshr->group_data_height = WINDOW_GROUP_BORDER_TOP_SIZE + window_graphicshr->bitmap_graphicshr_height_400_BORDER*window_graphicshr->zoom_level + WINDOW_GROUP_BORDER_BOTTOM_SIZE + 5;
    }
  MoveWindow(hwnd_group,window_graphicshr->group_data_x,window_graphicshr->group_data_y,window_graphicshr->group_data_width,window_graphicshr->group_data_height,TRUE);

  /* Largeur de la fentre */
  window_graphicshr->window_width = window_graphicshr->group_data_x + window_graphicshr->group_data_width + WINDOW_CLIENT_RIGHT_BORDER_SIZE + WINDOW_BORDER_SIZE;

  /* On doit garantir une largeur minimale de la fentre */
  if(window_graphicshr->window_width < GRAPHICSHR_DIALOGBOX_WIDTH)
    window_graphicshr->window_width = GRAPHICSHR_DIALOGBOX_WIDTH;

  /* Hauteur de la fentre */
  window_graphicshr->window_height = window_graphicshr->title_height + window_graphicshr->group_data_y + window_graphicshr->group_data_height + WINDOW_CLIENT_BOTTOM_BORDER_SIZE;

  /* Resize window */
  MoveWindow(window_graphicshr->hwnd_graphicshr,window_x,window_y,window_graphicshr->window_width,window_graphicshr->window_height,TRUE);
}


/***********************************************************************************************/
/*  BuildGraphicSHRFromData() :  Construire la page graphique SHR  partir des donnes brutes. */
/***********************************************************************************************/
static void BuildGraphicSHRFromData(struct window_graphicshr *window_graphicshr)
{
  unsigned char one_byte;
  unsigned char one_scb_byte;
  unsigned char one_0R_byte;
  unsigned char one_GB_byte;
  DWORD color_rgb;
  int i, offset_line, palette_number, x, y, line_width, is_fill_mode, color_index, spot_x, spot_y, spot_bitmap_x, spot_bitmap_y, area_size, spot_step, spot_size;
  char buffer_title[256];
  char buffer_spot[256];
  DWORD tab_rgb_color[16];
  DWORD *bitmap_data_graphicshr;
  DWORD border_color[16] = {0x00000000,0x00DD0033,0x00000099,0x00DD22DD,0x00007722,0x00555555,0x002222FF,0x0066AAFF,0x00885500,0x00FF6600,0x00AAAAAA,0x00FF9988,0x0000DD00,0x00FFFF00,0x0055FF99,0x00FFFFFF};
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Rien  faire */
  if(window_graphicshr->bitmap_data_graphicshr_320_200 == NULL || window_graphicshr->bitmap_data_graphicshr_320_200_BORDER == NULL ||
     window_graphicshr->bitmap_data_graphicshr_640_200 == NULL || window_graphicshr->bitmap_data_graphicshr_640_200_BORDER == NULL ||
     window_graphicshr->bitmap_data_graphicshr_640_400 == NULL || window_graphicshr->bitmap_data_graphicshr_640_400_BORDER == NULL)
    return;    

  /** Spot Video : X[0,127], Y[0,251] **/
  spot_x = window_graphicshr->C02F & 0x7F;
  spot_y = ((window_graphicshr->C02E << 1) | (window_graphicshr->C02F >> 7)) - 250;
  /* Corrige les valeurs fantaisistes provenant des Snapshot mmoire */
  if(spot_x < 0 || spot_x > 127 || spot_y < 0 || spot_y > 251)
    {
      spot_x = 0;
      spot_y = 0;
    }

  /*** Cration de la bitmap 320*200 / 320*200 BORDER correspondant au SHR ***/
  if(window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_320_200 || window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_320_200_BORDER)
    {
      /** Couleur de Fond **/
      if(window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_320_200_BORDER)
        {
          bitmap_data_graphicshr = window_graphicshr->bitmap_data_graphicshr_320_200_BORDER;
          for(i=0; i<window_graphicshr->bitmap_graphicshr_width_320_BORDER*window_graphicshr->bitmap_graphicshr_height_200_BORDER; i++)
            bitmap_data_graphicshr[i] = window_graphicshr->border_color_rgb;
        }
      else
        bitmap_data_graphicshr = window_graphicshr->bitmap_data_graphicshr_320_200;

      /** Bitmap de la page graphique **/
      for(y=0; y<window_graphicshr->bitmap_graphicshr_height_200; y++)
        {
          /* Palette 16 couleurs de cette ligne */
          one_scb_byte = window_graphicshr->graphicshr_data[GRAPHICSHR_SCB_OFFSET + y];
          palette_number = (int) (one_scb_byte & 0x0F);
          for(i=0; i<16; i++)
            {
              one_GB_byte = window_graphicshr->graphicshr_data[GRAPHICSHR_PALETTE_OFFSET + palette_number*32 + 2*i + 0];
              one_0R_byte = window_graphicshr->graphicshr_data[GRAPHICSHR_PALETTE_OFFSET + palette_number*32 + 2*i + 1];
              tab_rgb_color[i] = ((one_0R_byte*17) << 16) | (((one_GB_byte>>4)*17)<<8) | ((one_GB_byte & 0x0F)*17);
            }

          /* Fill Mode */
          is_fill_mode = ((one_scb_byte & 0x20) == 0x20) ? 1 : 0;

          /* Offset Line */
          if(window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_320_200)
            offset_line = y*window_graphicshr->bitmap_graphicshr_width_320;
          else if(window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_320_200_BORDER)
            offset_line = (y+GRAPHICSHR_BORDER_SIZE_TOP)*window_graphicshr->bitmap_graphicshr_width_320_BORDER + GRAPHICSHR_BORDER_SIZE_LEFT;

          /** Pixels du SCB de la ligne **/
          if(window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_320_200_BORDER && window_graphicshr->display_scb == 1)
            for(x=0; x<GRAPHICSHR_BORDER_SIZE_LEFT; x++)
              bitmap_data_graphicshr[offset_line - GRAPHICSHR_BORDER_SIZE_LEFT + x] = border_color[(one_scb_byte & 0x0F)];

          /** Pixels de la ligne **/
          for(x=0; x<160; x++)
            {
              /* 1 Byte = 2 Pixels ou 4 Pixels */
              one_byte = window_graphicshr->graphicshr_data[GRAPHICSHR_PIXEL_OFFSET + y*160+x];

              /* Ligne 320 dans une image 320 (en cas de FillMode on remplace la couleur 0 par un vert clair) */
              color_index = (one_byte & 0xF0) >> 4;
              color_rgb = (color_index == 0x00 && is_fill_mode == 1) ? FILLMODE_BACKGROUND_COLOR : tab_rgb_color[color_index];
              bitmap_data_graphicshr[offset_line + 2*x + 0] = color_rgb;
              color_index = one_byte & 0x0F;
              color_rgb = (color_index == 0x00 && is_fill_mode == 1) ? FILLMODE_BACKGROUND_COLOR : tab_rgb_color[color_index];
              bitmap_data_graphicshr[offset_line + 2*x + 1] = color_rgb;
            }

          /** Mode Fill de la ligne 320 **/
          if(is_fill_mode == 1 && window_graphicshr->fill_mode == 1)
            {
              /** Remplace les FILLMODE_BACKGROUND_COLOR par la couleur prcdente **/
              color_rgb = bitmap_data_graphicshr[offset_line];
              for(x=0; x<320; x++)
                {
                  if(bitmap_data_graphicshr[offset_line + x] == FILLMODE_BACKGROUND_COLOR)
                    bitmap_data_graphicshr[offset_line + x] = color_rgb;
                  else
                    color_rgb = bitmap_data_graphicshr[offset_line + x];
                }
            }
        }
    } 
  else if(window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_200 || window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_200_BORDER)
    {
      /** Couleur de Fond **/
      if(window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_200_BORDER)
        {
          bitmap_data_graphicshr = window_graphicshr->bitmap_data_graphicshr_640_200_BORDER;
          for(i=0; i<window_graphicshr->bitmap_graphicshr_width_640_BORDER*window_graphicshr->bitmap_graphicshr_height_200_BORDER; i++)
            bitmap_data_graphicshr[i] = window_graphicshr->border_color_rgb;
        }
      else
        bitmap_data_graphicshr = window_graphicshr->bitmap_data_graphicshr_640_200;

      /** Bitmap de la page graphique **/
      for(y=0; y<window_graphicshr->bitmap_graphicshr_height_200; y++)
        {
          /* Palette 16 couleurs de cette ligne */
          one_scb_byte = window_graphicshr->graphicshr_data[GRAPHICSHR_SCB_OFFSET + y];
          palette_number = (int) (one_scb_byte & 0x0F);
          for(i=0; i<16; i++)
            {
              one_GB_byte = window_graphicshr->graphicshr_data[GRAPHICSHR_PALETTE_OFFSET + palette_number*32 + 2*i + 0];
              one_0R_byte = window_graphicshr->graphicshr_data[GRAPHICSHR_PALETTE_OFFSET + palette_number*32 + 2*i + 1];
              tab_rgb_color[i] = ((one_0R_byte*17) << 16) | (((one_GB_byte>>4)*17)<<8) | ((one_GB_byte & 0x0F)*17);
            }

          /* Fill Mode */
          is_fill_mode = ((one_scb_byte & 0x20) == 0x20) ? 1 : 0;

          /* Rsolution */
          line_width = ((one_scb_byte & 0x80) == 0x00) ? 320 : 640;

          /* Offset Line */
          if(window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_200)
            offset_line = y*window_graphicshr->bitmap_graphicshr_width_640;
          else if(window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_200_BORDER)
            offset_line = (y+GRAPHICSHR_BORDER_SIZE_TOP)*window_graphicshr->bitmap_graphicshr_width_640_BORDER + GRAPHICSHR_BORDER_SIZE_LEFT*2;

          /** Pixels du SCB de la ligne **/
          if(window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_200_BORDER && window_graphicshr->display_scb == 1)
            for(x=0; x<GRAPHICSHR_BORDER_SIZE_LEFT*2; x++)
              bitmap_data_graphicshr[offset_line - GRAPHICSHR_BORDER_SIZE_LEFT*2 + x] = border_color[(one_scb_byte & 0x0F)];

          /** Pixels de la ligne **/
          for(x=0; x<160; x++)
            {
              /* 1 Byte = 2 Pixels ou 4 Pixels */
              one_byte = window_graphicshr->graphicshr_data[GRAPHICSHR_PIXEL_OFFSET + y*160 + x];

              /* Ligne 320 dans une image 640 (4 pixels) */
              if(line_width == 320)
                {
                  color_index = (one_byte & 0xF0) >> 4;
                  color_rgb = (color_index == 0x00 && is_fill_mode == 1) ? FILLMODE_BACKGROUND_COLOR : tab_rgb_color[color_index];
                  bitmap_data_graphicshr[offset_line + 4*x + 0] = color_rgb;
                  bitmap_data_graphicshr[offset_line + 4*x + 1] = color_rgb;
                  color_index = one_byte & 0x0F;
                  color_rgb = (color_index == 0x00 && is_fill_mode == 1) ? FILLMODE_BACKGROUND_COLOR : tab_rgb_color[color_index];
                  bitmap_data_graphicshr[offset_line + 4*x + 2] = color_rgb;
                  bitmap_data_graphicshr[offset_line + 4*x + 3] = color_rgb;
                }
              else
                {
                  /* Couleur du point #1, Couleurs 0-3 de la palette */
                  bitmap_data_graphicshr[offset_line + 4*x + 0] = tab_rgb_color[(unsigned char) (one_byte >> 6)];
                  /* Couleur du point #2, Couleurs 4-7 de la palette */
                  bitmap_data_graphicshr[offset_line + 4*x + 1] = tab_rgb_color[(unsigned char) (((one_byte >> 4) & 0x03)+4)];
                  /* Couleur du point #3, Couleurs 8-11 de la palette */
                  bitmap_data_graphicshr[offset_line + 4*x + 2] = tab_rgb_color[(unsigned char) (((one_byte >> 2) & 0x03)+8)];
                  /* Couleur du point #4, Couleurs 12-15 de la palette */
                  bitmap_data_graphicshr[offset_line + 4*x + 3] = tab_rgb_color[(unsigned char) ((one_byte & 0x03)+12)];
                }
            }

          /** Mode Fill de la ligne 320 **/
          if(line_width == 320 && is_fill_mode == 1 && window_graphicshr->fill_mode == 1)
            {
              /** Remplace les FILLMODE_BACKGROUND_COLOR par la couleur prcdente **/
              color_rgb = bitmap_data_graphicshr[offset_line];
              for(x=0; x<640; x++)
                {
                  if(bitmap_data_graphicshr[offset_line + x] == FILLMODE_BACKGROUND_COLOR)
                    bitmap_data_graphicshr[offset_line + x] = color_rgb;
                  else
                    color_rgb = bitmap_data_graphicshr[offset_line + x];
                }
            }
        }
    }
  else if(window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_400 || window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_400_BORDER)
    {
      /** Couleur de Fond **/
      if(window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_400_BORDER)
        {
          bitmap_data_graphicshr = window_graphicshr->bitmap_data_graphicshr_640_400_BORDER;
          for(i=0; i<window_graphicshr->bitmap_graphicshr_width_640_BORDER*window_graphicshr->bitmap_graphicshr_height_400_BORDER; i++)
            bitmap_data_graphicshr[i] = window_graphicshr->border_color_rgb;
        }
      else
        bitmap_data_graphicshr = window_graphicshr->bitmap_data_graphicshr_640_400;

      /** Bitmap de la page graphique **/
      for(y=0; y<window_graphicshr->bitmap_graphicshr_height_400; y++)
        {
          /* Palette 16 couleurs de cette ligne */
          one_scb_byte = window_graphicshr->graphicshr_data[GRAPHICSHR_SCB_OFFSET + y/2];
          palette_number = (int) (one_scb_byte & 0x0F);
          for(i=0; i<16; i++)
            {
              one_GB_byte = window_graphicshr->graphicshr_data[GRAPHICSHR_PALETTE_OFFSET + palette_number*32 + 2*i + 0];
              one_0R_byte = window_graphicshr->graphicshr_data[GRAPHICSHR_PALETTE_OFFSET + palette_number*32 + 2*i + 1];
              tab_rgb_color[i] = ((one_0R_byte*17) << 16) | (((one_GB_byte>>4)*17)<<8) | ((one_GB_byte & 0x0F)*17);
            }

          /* Fill Mode */
          is_fill_mode = ((one_scb_byte & 0x20) == 0x20) ? 1 : 0;

          /* Rsolution */
          line_width = ((one_scb_byte & 0x80) == 0x00) ? 320 : 640;

          /* Offset Line */
          if(window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_400)
            offset_line = y*window_graphicshr->bitmap_graphicshr_width_640;
          else if(window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_400_BORDER)
            offset_line = (y+GRAPHICSHR_BORDER_SIZE_TOP*2)*window_graphicshr->bitmap_graphicshr_width_640_BORDER + GRAPHICSHR_BORDER_SIZE_LEFT*2;

          /** Pixels du SCB de la ligne **/
          if(window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_400_BORDER && window_graphicshr->display_scb == 1)
            for(x=0; x<GRAPHICSHR_BORDER_SIZE_LEFT*2; x++)
              bitmap_data_graphicshr[offset_line - GRAPHICSHR_BORDER_SIZE_LEFT*2 + x] = border_color[(one_scb_byte & 0x0F)];

          /** Pixels de la ligne **/
          for(x=0; x<160; x++)
            {
              /* 1 Byte = 2 Pixels ou 4 Pixels */
              one_byte = window_graphicshr->graphicshr_data[GRAPHICSHR_PIXEL_OFFSET + (y/2)*160+x];

              /* Ligne 320 dans une image 640 (4 pixels) */
              if(line_width == 320)
                {
                  color_index = (one_byte & 0xF0) >> 4;
                  color_rgb = (color_index == 0x00 && is_fill_mode == 1) ? FILLMODE_BACKGROUND_COLOR : tab_rgb_color[color_index];
                  bitmap_data_graphicshr[offset_line + 4*x + 0] = color_rgb;
                  bitmap_data_graphicshr[offset_line + 4*x + 1] = color_rgb;
                  color_index = one_byte & 0x0F;
                  color_rgb = (color_index == 0x00 && is_fill_mode == 1) ? FILLMODE_BACKGROUND_COLOR : tab_rgb_color[color_index];
                  bitmap_data_graphicshr[offset_line + 4*x + 2] = color_rgb;
                  bitmap_data_graphicshr[offset_line + 4*x + 3] = color_rgb;
                }
              else
                {
                  /* Couleur du point #1, Couleurs 0-3 de la palette */
                  bitmap_data_graphicshr[offset_line + 4*x + 0] = tab_rgb_color[(unsigned char) (one_byte >> 6)];
                  /* Couleur du point #2, Couleurs 4-7 de la palette */
                  bitmap_data_graphicshr[offset_line + 4*x + 1] = tab_rgb_color[(unsigned char) (((one_byte >> 4) & 0x03)+4)];
                  /* Couleur du point #3, Couleurs 8-11 de la palette */
                  bitmap_data_graphicshr[offset_line + 4*x + 2] = tab_rgb_color[(unsigned char) (((one_byte >> 2) & 0x03)+8)];
                  /* Couleur du point #4, Couleurs 12-15 de la palette */
                  bitmap_data_graphicshr[offset_line + 4*x + 3] = tab_rgb_color[(unsigned char) ((one_byte & 0x03)+12)];
                }
            }

          /** Mode Fill de la ligne 320 **/
          if(line_width == 320 && is_fill_mode == 1 && window_graphicshr->fill_mode == 1)
            {
              /** Remplace les FILLMODE_BACKGROUND_COLOR par la couleur prcdente **/
              color_rgb = bitmap_data_graphicshr[offset_line];
              for(x=0; x<640; x++)
                {
                  if(bitmap_data_graphicshr[offset_line + x] == FILLMODE_BACKGROUND_COLOR)
                    bitmap_data_graphicshr[offset_line + x] = color_rgb;
                  else
                    color_rgb = bitmap_data_graphicshr[offset_line + x];
                }
            }
        }
    }

  /* Titre de la fentre */
  sprintf(buffer_title,"Graphic SHR : %02X/%04X",window_graphicshr->graphicshr_bank,window_graphicshr->graphicshr_address);
  SetWindowText(window_graphicshr->hwnd_graphicshr,buffer_title);

  /** Position du Spot Video : Text **/
  /* Spot X */
  if(spot_x >= 0x46 && spot_x <= 0x56)
    strcpy(buffer_spot,"X=HBL");
  else
    sprintf(buffer_spot,"X=%02X",spot_x);
  SetWindowText(GetDlgItem(window_graphicshr->hwnd_graphicshr,IDC_STATIC_SPOT_X),buffer_spot);
  /* Spot Y */
  if(spot_y >= 0 && spot_y <= 199)
    sprintf(buffer_spot,"Y=SHR (%d/200)",spot_y+1);
  else if(spot_y >= 200 && spot_y <= 220)
    sprintf(buffer_spot,"Y=BOTTOM (%d/21)",spot_y-199);
  else if(spot_y >= 221 && spot_y <= 242)
    sprintf(buffer_spot,"Y=VBL (%d/22)",spot_y-220);
  else if(spot_y >= 243)
    sprintf(buffer_spot,"Y=TOP (%d/19)",spot_y-242);
  SetWindowText(GetDlgItem(window_graphicshr->hwnd_graphicshr,IDC_STATIC_SPOT_Y),buffer_spot);

  /** Position du Spot : Graphique **/
  if(window_graphicshr->display_video_spot == 1 && 
     (window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_320_200_BORDER ||
      window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_200_BORDER ||
      window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_400_BORDER))
    {
      /** Spot X **/
      /* On passe en coordonnes Bitmap */
      if(spot_x == 0x00 || spot_x >= 0x40 && spot_x <= 0x45)    /* Right Border */
        {
          area_size = (window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_320_200_BORDER) ? GRAPHICSHR_BORDER_SIZE_RIGHT : 2*GRAPHICSHR_BORDER_SIZE_RIGHT;
          spot_step = 1 + (0x45 -  0x40) + 1;
          spot_size = area_size/spot_step;
          spot_bitmap_x = (window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_320_200_BORDER) ? GRAPHICSHR_BORDER_SIZE_LEFT + 320 : 2*GRAPHICSHR_BORDER_SIZE_LEFT + 640;
          if(spot_x != 0x00)
            spot_bitmap_x += ((spot_x - 0x40 + 1)*area_size)/spot_step;
        }
      else if(spot_x >= 0x46 && spot_x <= 0x56)              /* HBL */
        {
          spot_bitmap_x = -1;
        }
      else if(spot_x >= 0x57 && spot_x <= 0x5A)              /* Left Border */
        {
          area_size = (window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_320_200_BORDER) ? GRAPHICSHR_BORDER_SIZE_LEFT : 2*GRAPHICSHR_BORDER_SIZE_LEFT;
          spot_step = (0x5A - 0x57) + 1;
          spot_size = area_size/spot_step;
          spot_bitmap_x = ((spot_x - 0x57)*area_size)/spot_step;
        }
      else if(spot_x >= 0x5B && spot_x <= 0x7F)              /* SHR */
        {
          area_size = (window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_320_200_BORDER) ? 320 : 640;
          spot_step = (0x7F - 0x5B) + 1;
          spot_size = area_size/spot_step;
          spot_bitmap_x = (window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_320_200_BORDER) ? GRAPHICSHR_BORDER_SIZE_LEFT : 2*GRAPHICSHR_BORDER_SIZE_LEFT;
          spot_bitmap_x += ((spot_x - 0x5B)*area_size)/spot_step;
        }

      /* On passe sur la colonne */
      if(spot_bitmap_x != -1)
        {
          if(window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_320_200_BORDER)
            {
              for(y=0; y<window_graphicshr->bitmap_graphicshr_height_200_BORDER; y++)
                for(x=0; x<spot_size; x++)
                  bitmap_data_graphicshr[y*window_graphicshr->bitmap_graphicshr_width_320_BORDER + spot_bitmap_x + x] |= 0x00FF0000;    /* Force le Rouge */
            }
          else if(window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_200_BORDER)
            {
              for(y=0; y<window_graphicshr->bitmap_graphicshr_height_200_BORDER; y++)
                for(x=0; x<spot_size; x++)
                  bitmap_data_graphicshr[y*window_graphicshr->bitmap_graphicshr_width_640_BORDER + spot_bitmap_x  + x] |= 0x00FF0000;    /* Force le Rouge */
            }
          else if(window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_400_BORDER)
            {
              for(y=0; y<window_graphicshr->bitmap_graphicshr_height_400_BORDER; y++)
                for(x=0; x<spot_size; x++)
                  bitmap_data_graphicshr[y*window_graphicshr->bitmap_graphicshr_width_640_BORDER + spot_bitmap_x + x] |= 0x00FF0000;    /* Force le Rouge */
            }
        }
      /** Spot Y **/
      /* On passe en coordonnes Bitmap */
      if(spot_y >= 0 && spot_y <= 220)
        spot_bitmap_y = GRAPHICSHR_BORDER_SIZE_TOP + spot_y;
      else if(spot_y >= 221 && spot_y <= 242)
        spot_bitmap_y = -1;    /* VBL */
      else if(spot_y >= 243 && spot_y <= 261)
        spot_bitmap_y = spot_y - 243;    /* VBL */
      /* On passe repasse sur la ligne */
      if(spot_bitmap_y != -1)
        {
          if(window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_320_200_BORDER)
            {
              offset_line = spot_bitmap_y*window_graphicshr->bitmap_graphicshr_width_320_BORDER;
              for(x=0; x<window_graphicshr->bitmap_graphicshr_width_320_BORDER; x++)
                bitmap_data_graphicshr[offset_line + x] |= 0x00FF0000;    /* Force le Rouge */
            }
          else if(window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_200_BORDER)
            {
              offset_line = spot_bitmap_y*window_graphicshr->bitmap_graphicshr_width_640_BORDER;
              for(x=0; x<window_graphicshr->bitmap_graphicshr_width_640_BORDER; x++)
                bitmap_data_graphicshr[offset_line + x] |= 0x00FF0000;    /* Force le Rouge */
            }
          else if(window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_400_BORDER)
            {
              offset_line = 2*spot_bitmap_y*window_graphicshr->bitmap_graphicshr_width_640_BORDER;
              for(x=0; x<window_graphicshr->bitmap_graphicshr_width_640_BORDER; x++)
                {
                  bitmap_data_graphicshr[offset_line + x] |= 0x00FF0000;    /* Force le Rouge */
                  bitmap_data_graphicshr[offset_line + window_graphicshr->bitmap_graphicshr_width_640_BORDER + x] |= 0x00FF0000;    /* Force le Rouge */
                }
            }
        }
    }
}


/************************************************************************************/
/*  BuildPaletteWindowAllBitmap() :  Construction des Bitmap de la fentre Palette. */
/************************************************************************************/
static int BuildPaletteWindowAllBitmap(struct window_graphicshr *window_graphicshr)
{
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Bitmap Palette **/
  BuildPaletteWindowPaletteBitmap(window_graphicshr);

  /* OK */
  return(0);
}


/**************************************************************************************************/
/*  BuildPaletteWindowPaletteBitmap() :  Construction de la Bitmap Palette de la fentre Palette. */
/**************************************************************************************************/
static int BuildPaletteWindowPaletteBitmap(struct window_graphicshr *window_graphicshr)
{
  int i, x, y, x0, y0;
  DWORD color_true;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Init coordonnes */
  for(i=0; i<256; i++)
    {
      window_graphicshr->palette_x0[i] = 0;
      window_graphicshr->palette_y0[i] = 0;
      window_graphicshr->palette_x1[i] = 0;
      window_graphicshr->palette_y1[i] = 0;
    }

  /** Couleur de fond **/
  color_true = BITMAP_BACKGROUND_COLOR;
  for(i=0; i<(window_graphicshr->bitmap_palette_width*window_graphicshr->bitmap_palette_height); i++)
    window_graphicshr->bitmap_data_palette[i] = color_true;

  /** Place les 16 couleurs **/
  for(i=0; i<16; i++)
    {
      /* Couleur */
      color_true = MAKECOLOR(window_graphicshr->palette_red[i],window_graphicshr->palette_green[i],window_graphicshr->palette_blue[i]);

      /* 16 couleurs : Carr */
      for(y=0; y<PALETTE_COLOR_SIZE_HEIGHT; y++)
        for(x=0; x<PALETTE_COLOR_SIZE_WIDTH; x++)
          {
            x0 = PALETTE_COLOR_BORDER + i*(PALETTE_COLOR_SIZE_WIDTH + PALETTE_COLOR_BORDER) + x;
            y0 = PALETTE_COLOR_BORDER + y;
            window_graphicshr->bitmap_data_palette[x0 + y0*(window_graphicshr->bitmap_palette_width)] = color_true;
          }

      /* Coordonnes de la couleur */
      window_graphicshr->palette_x0[i] = window_graphicshr->bitmap_palette_x + PALETTE_COLOR_BORDER + i*(PALETTE_COLOR_SIZE_WIDTH + PALETTE_COLOR_BORDER);
      window_graphicshr->palette_y0[i] = window_graphicshr->bitmap_palette_y + PALETTE_COLOR_BORDER;
      window_graphicshr->palette_x1[i] = window_graphicshr->palette_x0[i] + PALETTE_COLOR_SIZE_WIDTH;
      window_graphicshr->palette_y1[i] = window_graphicshr->palette_y0[i] + PALETTE_COLOR_SIZE_HEIGHT;
    }

  /* OK */
  return(0);
}


/************************************************************************/
/*  UpdatePaletteClientPaletteArea() :  Affichage de la Bitmap Palette. */
/************************************************************************/
static void UpdatePaletteClientPaletteArea(HDC hDC, struct window_graphicshr *window_graphicshr)
{
  HGDIOBJ bitmap_old;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Bitmap avec les couleurs de la Palette **/
  bitmap_old = SelectObject(window_graphicshr->hCompatibleDC,window_graphicshr->hBitmap_palette);
  StretchBlt(hDC,window_graphicshr->bitmap_palette_x,window_graphicshr->bitmap_palette_y,window_graphicshr->bitmap_palette_width,window_graphicshr->bitmap_palette_height,
             window_graphicshr->hCompatibleDC,0,0,window_graphicshr->bitmap_palette_width,window_graphicshr->bitmap_palette_height,SRCCOPY);
  SelectObject(window_graphicshr->hCompatibleDC,bitmap_old);
}


/************************************************************************/
/*  SaveGraphicPicture() :  Enregistre l'image de la zone slectionne. */
/************************************************************************/
static void SaveGraphicPicture(struct window_graphicshr *window_graphicshr)
{
  BOOL is_OK;
  int i, x, y, error, picture_width, picture_height, line_width, scb_color_index;
  unsigned char one_byte;
  unsigned char color_fill_index;
  char gif_file_name[1024];
  char gif_file_path[1024];
  struct picture_256 *current_256_picture;

  /* 640*200 ou 320*200 */
  if(window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_320_200 || window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_320_200_BORDER)
    picture_width = 320;
  else
    picture_width = 640;
  picture_height = 200;

  /* Construit le nom du fichier GIF  partir du format de l'image */
  sprintf(gif_file_name,"%02X.%04X_Graphic_%dx%d.gif",window_graphicshr->graphicshr_bank,window_graphicshr->graphicshr_address,picture_width,picture_height);

  /* Demande le nom de l'image */
  strcpy(gif_file_path,gif_file_name);
      
  /** Fentre de slection de chemin de fichier **/
  is_OK = GetSaveFilePath(window_graphicshr->hwnd_graphicshr,"Select GIF file","All Files (*.*)\0*.*\0\0\0",gif_file_path);

  /* Si l'utilisateur l'annule */
  if(!is_OK)
    return;

  /** Cration d'une image 256 couleurs **/
  current_256_picture = (struct picture_256 *) calloc(1,sizeof(struct picture_256));
  if(current_256_picture == NULL)
    {
      MessageBox(NULL,"Impossible to save Picture on Disk (not enough memory)","Save Picture Error",MB_OK|MB_ICONWARNING);
      return;
    }
  current_256_picture->width = picture_width;
  current_256_picture->height = picture_height;
  current_256_picture->data = (unsigned char *) calloc(current_256_picture->width*current_256_picture->height,1);
  if(current_256_picture->data == NULL)
    {
      MessageBox(NULL,"Impossible to save Picture on Disk (not enough memory)","Save Picture Error",MB_OK|MB_ICONWARNING);    
      free(current_256_picture);
      return;
    }

  /** Construction de l'image **/
  for(y=0; y<200; y++)
    {
      /* La ligne peut tre en 320 dans une image 640 => On double les pixels */
      line_width = ((window_graphicshr->graphicshr_data[GRAPHICSHR_SCB_OFFSET+y] & 0x80) == 0x80) ? 640 : 320;

      /** 1 Ligne de Pixels **/
      for(x=0; x<160; x++)
        {
          /* 1 Byte = 2 Pixels ou 4 Pixels */
          one_byte = window_graphicshr->graphicshr_data[GRAPHICSHR_PIXEL_OFFSET + y*160 + x];

          /* Image en 640 */
          if((window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_200 ||
              window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_200_BORDER ||
              window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_400 ||
              window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_400_BORDER) && 
             line_width == 640)
            {
              /* 1 Byte = 4 Pixels */
              scb_color_index = window_graphicshr->graphicshr_data[GRAPHICSHR_SCB_OFFSET+y] & 0x0F;
              current_256_picture->data[picture_width*y + 4*x + 0] = 16*scb_color_index + (((one_byte >> 6) & 0x03) + 0);
              current_256_picture->data[picture_width*y + 4*x + 1] = 16*scb_color_index + (((one_byte >> 4) & 0x03) + 4);
              current_256_picture->data[picture_width*y + 4*x + 2] = 16*scb_color_index + (((one_byte >> 2) & 0x03) + 8);
              current_256_picture->data[picture_width*y + 4*x + 3] = 16*scb_color_index + ((one_byte & 0x03) + 12);
            }
          else if((window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_200 ||
                   window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_200_BORDER ||
                   window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_400 ||
                   window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_400_BORDER) && 
                  line_width == 320)
            {
              /* 1 Byte = 2*2 Pixels */
              scb_color_index = window_graphicshr->graphicshr_data[GRAPHICSHR_SCB_OFFSET+y] & 0x0F;
              current_256_picture->data[picture_width*y + 4*x + 0] = 16*scb_color_index + ((one_byte >> 4) & 0x0F);
              current_256_picture->data[picture_width*y + 4*x + 1] = 16*scb_color_index + ((one_byte >> 4) & 0x0F);
              current_256_picture->data[picture_width*y + 4*x + 2] = 16*scb_color_index + (one_byte & 0x0F);
              current_256_picture->data[picture_width*y + 4*x + 3] = 16*scb_color_index + (one_byte & 0x0F);
            }
          else if(window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_320_200 || 
                  window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_320_200_BORDER)
            {
              /* 1 Byte = 2 Pixels */
              scb_color_index = window_graphicshr->graphicshr_data[GRAPHICSHR_SCB_OFFSET+y] & 0x0F;
              current_256_picture->data[picture_width*y + 2*x + 0] = 16*scb_color_index + ((one_byte >> 4) & 0x0F);
              current_256_picture->data[picture_width*y + 2*x + 1] = 16*scb_color_index + (one_byte & 0x0F);
            }
        }

      /* Fill Mode sur la ligne */
      if(line_width == 320 && window_graphicshr->fill_mode == 1)
        {
          /* Ligne en 320 dans une image 640 */
          if(window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_200 ||
             window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_200_BORDER ||
             window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_400 ||
             window_graphicshr->picture_format == GRAPHICSHR_PICTURE_FORMAT_640_400_BORDER)
            {
              color_fill_index = current_256_picture->data[picture_width*y + 0];
              for(x=0; x<640; x+=2)
                {
                  if((current_256_picture->data[picture_width*y + x] & 0x0F) == 0x00)
                    {
                      current_256_picture->data[picture_width*y + x + 0] = color_fill_index;
                      current_256_picture->data[picture_width*y + x + 1] = color_fill_index;
                    }
                  else
                    color_fill_index = current_256_picture->data[picture_width*y + x];
                }
            }
          else
            {
              color_fill_index = current_256_picture->data[picture_width*y + 0];
              for(x=0; x<320; x++)
                {
                  if((current_256_picture->data[picture_width*y + x] & 0x0F) == 0x00)
                    current_256_picture->data[picture_width*y + x] = color_fill_index;
                  else
                    color_fill_index = current_256_picture->data[picture_width*y + x];
                }
            }
        }
    }
  /* Couleurs */
  current_256_picture->nb_color = 256;
  for(i=0; i<current_256_picture->nb_color; i++)
    {
      /* GB 0R */
      current_256_picture->palette_red[i]   = (window_graphicshr->graphicshr_data[GRAPHICSHR_PALETTE_OFFSET+2*i+1] & 0x0F) | ((window_graphicshr->graphicshr_data[GRAPHICSHR_PALETTE_OFFSET+2*i+1] & 0x0F) << 4);
      current_256_picture->palette_green[i] = (window_graphicshr->graphicshr_data[GRAPHICSHR_PALETTE_OFFSET+2*i+0] & 0xF0) | ((window_graphicshr->graphicshr_data[GRAPHICSHR_PALETTE_OFFSET+2*i+0] & 0xF0) >> 4);
      current_256_picture->palette_blue[i]  = (window_graphicshr->graphicshr_data[GRAPHICSHR_PALETTE_OFFSET+2*i+0] & 0x0F) | ((window_graphicshr->graphicshr_data[GRAPHICSHR_PALETTE_OFFSET+2*i+0] & 0x0F) << 4);
    }

  /** Conversion de la Pixmap en Image GIF **/
  error = GIFWriteFileFrom256Color(gif_file_path,current_256_picture);  
  if(error)
    MessageBox(NULL,"Impossible to save GIF Picture on Disk","Save Picture Error",MB_OK|MB_ICONWARNING);

  /* Libration mmoire */
  mem_free_picture_256(current_256_picture);
}


/************************************************************************/
/*  DeleteGraphicSHRChildWindows() :  Destruction des fentres enfants. */
/************************************************************************/
static void DeleteGraphicSHRChildWindows(struct window_graphicshr *window_graphicshr)
{
  DeleteObject(window_graphicshr->hBitmap_palette);
  DeleteObject(window_graphicshr->hBitmap_graphicshr_320_200);
  DeleteObject(window_graphicshr->hBitmap_graphicshr_320_200_BORDER);
  DeleteObject(window_graphicshr->hBitmap_graphicshr_640_200);
  DeleteObject(window_graphicshr->hBitmap_graphicshr_640_200_BORDER);
  DeleteObject(window_graphicshr->hBitmap_graphicshr_640_400);
  DeleteObject(window_graphicshr->hBitmap_graphicshr_640_400_BORDER);
  ReleaseDC(window_graphicshr->hwnd_graphicshr,window_graphicshr->hDC);
  DeleteDC(window_graphicshr->hCompatibleDC);
}


/**************************************************************************/
/*  mem_free_window_graphicshr() :  Libration mmoire pour la structure. */
/**************************************************************************/
void mem_free_window_graphicshr(struct window_graphicshr *window_graphicshr)
{
  struct palette *current_palette;
  struct palette *next_palette;

  if(window_graphicshr)
    {
      /* Supprime la notification */
      my_Notification(NOTIFICATION_REMOVE,window_graphicshr->current_notification,NULL);
      window_graphicshr->current_notification = NULL;
      
      /* Libration des palettes */
      for(current_palette = window_graphicshr->first_palette; current_palette; current_palette = next_palette)
        {
          next_palette = current_palette->next;
          mem_free_palette(current_palette);
        }

      free(window_graphicshr);
    }
}

/****************************************************************************/
