/******************************************************************************/
/*                                                                            */
/*  Window_GraphicScreen.c :  Module de gestion de la fentre GraphicScreen.  */
/*                                                                            */
/******************************************************************************/
/*  Auteur : Olivier ZARDINI   *    Brutal Deluxe   *    Date : Janvier 2019  */
/******************************************************************************/

#include <stdio.h>
#include <time.h>
#include <sys/timeb.h>

#include <windows.h>
#include <windowsx.h>
#include <Commctrl.h>
#include <Winuser.h>
#include <Richedit.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_Notification.h"
#include "Dc_Graphic.h"
#include "Dc_CPU.h"
#include "Dc_MMU.h"
#include "Window_QuickWatch.h"
#include "Window_Main.h"
#include "Window_Main_Menu.h"
#include "Window_GraphicScreen.h"

static DWORD BuildGraphicScreenWindowThread(void *);
LRESULT CALLBACK ProcHandlerGraphicScreenWindow(HWND,UINT,WPARAM,LPARAM);
static int CreateGraphicScreenChildWindows(HWND,struct window_graphicscreen *);
LRESULT CALLBACK subEditGraphicScreenProcControl(HWND,UINT,WPARAM,LPARAM);
void UpdateGraphicScreenClientAllArea(HDC,struct window_graphicscreen *);
static void UpdateGraphicScreenClientBitmapArea(HDC,struct window_graphicscreen *);
static void UpdateGraphicScreenContextInformation(struct window_graphicscreen *,int,int);
static void RefreshGraphicScreenWindowData(struct window_graphicscreen *,int);
static int IsGraphicScreenDataModified(int,int,int,int,struct window_graphicscreen *);
static void ResizeGraphicScreenWindow(struct window_graphicscreen *);
static void BuildGraphicScreenFromData(struct window_graphicscreen *);
static void DeleteGraphicScreenChildWindows(struct window_graphicscreen *);


/****************************************************************************************/
/*  BuildGraphicScreenWindow() :  Construction / Affichage de la fentre GraphicScreen. */
/****************************************************************************************/
struct window_graphicscreen *BuildGraphicScreenWindow(HINSTANCE hInstance, int menu_id)
{
  struct window_graphicscreen *window_graphicscreen;

  /** Allocation de la structure window_graphicscreen **/
  window_graphicscreen = (struct window_graphicscreen *) calloc(1,sizeof(struct window_graphicscreen));
  if(window_graphicscreen == NULL)
    return(NULL);
  my_Memory(MEMORY_SET_WINDOW_GRAPHICSCREEN,window_graphicscreen,NULL);

  /* Lance un thread pour grer la fentre GraphicScreen */
  CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)BuildGraphicScreenWindowThread,window_graphicscreen,0,NULL);

  /* Renvoi la structure mmoire grant la fentre */
  return(window_graphicscreen);
}


/**************************************************************************************************/
/*  BuildGraphicScreenWindowThread() :  Cration de la fentre GraphicScreen via un Thread ddi. */
/**************************************************************************************************/
static DWORD BuildGraphicScreenWindowThread(void *window_graphicscreen_param)
{
  HWND hwndGraphicScreen;
  MSG msg;
  BOOL bRet;
  struct window_main *window_main;
  struct window_graphicscreen *window_graphicscreen;
  struct parameter *param;
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Init */
  window_graphicscreen = (struct window_graphicscreen *) window_graphicscreen_param;

  /** Cration de la fentre Graphic SHR **/
  hwndGraphicScreen = CreateDialog(param->hInstance,MAKEINTRESOURCE(IDD_WINDOW_GRAPHICSCREEN),window_main->hwnd_main,(DLGPROC)ProcHandlerGraphicScreenWindow);
  if(hwndGraphicScreen == NULL)
    return(1);

  /** Show Window + Paint Content **/
  ShowWindow(hwndGraphicScreen,SW_SHOWDEFAULT);
  UpdateWindow(hwndGraphicScreen);

  /** Message Loop pour toutes les fentres de ce Thread (fentre GraphicScreen + ComboBox + ...) **/
  while((bRet = GetMessage(&msg,NULL,0,0)) != 0)
    {
      if(bRet == -1)
        break;

      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }

  /* Fin du Thread */
  ExitThread(0);
}


/******************************************************************************/
/*  ProcHandlerGraphicScreenWindow() :  Gestion des vnements de la fentre. */
/******************************************************************************/
LRESULT CALLBACK ProcHandlerGraphicScreenWindow(HWND hwndGraphicScreen, UINT umsg, WPARAM wParam, LPARAM lParam)
{
  HDC hDC;
  PAINTSTRUCT ps;
  int mouse_x, mouse_y;
  struct window_graphicscreen *window_graphicscreen;
  struct parameter *param;
  
  /* Init */
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  my_Memory(MEMORY_GET_WINDOW_GRAPHICSCREEN,&window_graphicscreen,NULL);
  if(window_graphicscreen == NULL)
    return(TRUE);

  switch(umsg)
    {
      case WM_INITDIALOG :
        /* Cration du contenu de la DialogBox */
        CreateGraphicScreenChildWindows(hwndGraphicScreen,window_graphicscreen);
        return(TRUE);

      /***************************/
      /* Evnements de la souris */
      /***************************/
      case WM_MOUSEMOVE :
        mouse_x = GET_X_LPARAM(lParam);
        mouse_y = GET_Y_LPARAM(lParam);
        UpdateGraphicScreenContextInformation(window_graphicscreen,mouse_x,mouse_y);
        return(TRUE);

      /**********************************/
      /* Rafrachissement de la fentre */
      /**********************************/
      case WM_PAINT:
        if(window_graphicscreen->hBitmap_graphicscreen != NULL)
          {
            /* Begin Paint */
            hDC = BeginPaint(hwndGraphicScreen,&ps);

            /** MAJ Bitmap **/
            UpdateGraphicScreenClientAllArea(hDC,window_graphicscreen);

            /* End Paint */
            EndPaint(hwndGraphicScreen,&ps);
          }
        break;
        
      /****************************/
      /* Evnements de la fentre */
      /****************************/
      case WM_COMMAND :
        switch(LOWORD(wParam))
          {
          }
        return(TRUE);

      /*****************************************************************/
      /* Message extrieur demandant le rafraichissement de la fentre */
      /*****************************************************************/
      case HM_GRAPHICSCREEN_REFRESHWINDOW :
        RefreshGraphicScreenWindowData(window_graphicscreen,0);
        return(TRUE);

      /********************************************************************/
      /* Fermeture de window (le WM_CLOSE est appel avant le WM_DESTROY) */
      /********************************************************************/
      case WM_CLOSE :
        /* Fentre GraphicScreen */
        DestroyWindow(hwndGraphicScreen);
        return(0);

      case WM_DESTROY :
        /* Suppression des Fentres */
        DeleteGraphicScreenChildWindows(window_graphicscreen);
        CheckMainWindowMenuItem(ID_MENU_VIEW_GRAPHICSCREEN,FALSE);
        my_Memory(MEMORY_FREE_WINDOW_GRAPHICSCREEN,NULL,NULL);
        /* Demande la sortie de la boucle des messages */
        PostQuitMessage(0);
        return(0);
    }

  /* Retourner FALSE pour relayer le message non trait au systme */
  return(FALSE);
}


/*****************************************************************************************************************/
/*  CreateGraphicScreenChildWindows() : Crer les fentres enfants et remplir la structure window_graphicscreen. */
/*****************************************************************************************************************/
static int CreateGraphicScreenChildWindows(HWND hwndGraphicScreen, struct window_graphicscreen *window_graphicscreen)
{
  BITMAPINFO bitmap_info;
  int i;
  struct notification *current_notification;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Window Handle **/
  window_graphicscreen->hwnd_graphicscreen = hwndGraphicScreen;

  /** Device Context **/
  window_graphicscreen->hDC = GetDC(window_graphicscreen->hwnd_graphicscreen);
  window_graphicscreen->hCompatibleDC = CreateCompatibleDC(window_graphicscreen->hDC);

  /** Hauteur et Largeur des Bitmap couleurs **/
  /* Dimension en Pixel de la Bitmap GraphicScreen Data */
  window_graphicscreen->bitmap_graphicscreen_width = 640;
  window_graphicscreen->bitmap_graphicscreen_height = 400;
  
  /** Bitmap affichant la zone de slection **/
  memset(&bitmap_info,0,sizeof(BITMAPINFO));
  bitmap_info.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
  bitmap_info.bmiHeader.biWidth = window_graphicscreen->bitmap_graphicscreen_width;
  bitmap_info.bmiHeader.biHeight = (-1)*window_graphicscreen->bitmap_graphicscreen_height;
  bitmap_info.bmiHeader.biPlanes = 1;
  bitmap_info.bmiHeader.biBitCount = 32;
  bitmap_info.bmiHeader.biCompression = BI_RGB;
  window_graphicscreen->hBitmap_graphicscreen = CreateDIBSection(window_graphicscreen->hDC,&bitmap_info,DIB_RGB_COLORS,(VOID **)&(window_graphicscreen->bitmap_data_graphicscreen),NULL,0);
  for(i=0; i<window_graphicscreen->bitmap_graphicscreen_width*window_graphicscreen->bitmap_graphicscreen_height; i++)
    window_graphicscreen->bitmap_data_graphicscreen[i] = BITMAP_BACKGROUND_COLOR;

  /* Retaille la fentre */
  ResizeGraphicScreenWindow(window_graphicscreen);

  /** Construction de la Page Texte + Rafraichissement de la fentre **/
  RefreshGraphicScreenWindowData(window_graphicscreen,1);

  /* Cration d'une Notification */
  current_notification = (struct notification *) calloc(1,sizeof(struct notification));
  if(current_notification == NULL)
    return(0);
  current_notification->hwnd_window = window_graphicscreen->hwnd_graphicscreen;
  current_notification->message = HM_GRAPHICSCREEN_REFRESHWINDOW;
  current_notification->type = NOTIFICATION_TYPE_VIEW;
  current_notification->sub_type = NOTIFICATION_SUBTYPE_GRAPHICSCREEN;

  /* Ajout de la notification */
  window_graphicscreen->current_notification = current_notification;
  my_Notification(NOTIFICATION_ADD,current_notification,NULL);

  /* OK */
  return(0);
}


/********************************************************************************************/
/*  UpdateGraphicScreenClientAllArea() :  Affichage des Bitmap de la fentre GraphicScreen. */
/********************************************************************************************/
void UpdateGraphicScreenClientAllArea(HDC hDC, struct window_graphicscreen *window_graphicscreen)
{
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Bitmap **/
  UpdateGraphicScreenClientBitmapArea(hDC,window_graphicscreen);
}


/***********************************************************************************/
/*  UpdateGraphicScreenClientBitmapArea() :  Affichage de la Bitmap GraphicScreen. */
/***********************************************************************************/
static void UpdateGraphicScreenClientBitmapArea(HDC hDC, struct window_graphicscreen *window_graphicscreen)
{
  HGDIOBJ bitmap_old;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Bitmap avec la zone mmoire et la zone slectionne **/
  bitmap_old = SelectObject(window_graphicscreen->hCompatibleDC,window_graphicscreen->hBitmap_graphicscreen);
  StretchBlt(hDC,window_graphicscreen->bitmap_graphicscreen_x,window_graphicscreen->bitmap_graphicscreen_y,
             window_graphicscreen->bitmap_graphicscreen_width,window_graphicscreen->bitmap_graphicscreen_height,
             window_graphicscreen->hCompatibleDC,0,0,window_graphicscreen->bitmap_graphicscreen_width,window_graphicscreen->bitmap_graphicscreen_height,SRCCOPY);
  SelectObject(window_graphicscreen->hCompatibleDC,bitmap_old);
}


/*************************************************************************************************************/
/*  UpdateGraphicScreenContextInformation() :  MAJ des information de contexte de la fenetre GraphicScreen. */
/*************************************************************************************************************/
static void UpdateGraphicScreenContextInformation(struct window_graphicscreen *window_graphicscreen, int mouse_x, int mouse_y)
{
  int pixmap_x, pixmap_y, grid_x, grid_y, is_640;

  /* Rien  faire */
  if(window_graphicscreen == NULL)
    return;

  /* Est t'on dans l'image ? */
  if(mouse_x < window_graphicscreen->bitmap_graphicscreen_x || mouse_x >= (window_graphicscreen->bitmap_graphicscreen_x + window_graphicscreen->bitmap_graphicscreen_width) ||
     mouse_y < window_graphicscreen->bitmap_graphicscreen_y || mouse_y >= (window_graphicscreen->bitmap_graphicscreen_y + window_graphicscreen->bitmap_graphicscreen_height))
    {
      SetWindowText(GetDlgItem(window_graphicscreen->hwnd_graphicscreen,IDC_STATIC_CONTEXT),"");
      return;
    }

  /* Coordonnes dans la Pixmap */
  pixmap_x = (mouse_x - window_graphicscreen->bitmap_graphicscreen_x);
  pixmap_y = (mouse_y - window_graphicscreen->bitmap_graphicscreen_y);

  /* Coordonnes dans la grille 640x200 */
  grid_x = pixmap_x;       /* 0-319 ou 0-639 */
  grid_y = pixmap_y/2;     /* 0-199 */

  /* 640 ou 320 ? */
  is_640 = 1;
}


/************************************************************************************/
/*  RefreshGraphicScreenWindowData() :  MAJ du contenu de la fentre GraphicScreen. */
/************************************************************************************/
static void RefreshGraphicScreenWindowData(struct window_graphicscreen *window_graphicscreen, int force)
{
  /** Cration de la Zone Graphique  partir des nouvelles donnes **/
  BuildGraphicScreenFromData(window_graphicscreen);

  /* Rafraichit la fentre */
  InvalidateRect(window_graphicscreen->hwnd_graphicscreen,NULL,TRUE);
  UpdateWindow(window_graphicscreen->hwnd_graphicscreen);
}


/*****************************************************************************/
/*  ResizeGraphicScreenWindow() : Resize GraphicScreen window by given size. */
/*****************************************************************************/
static void ResizeGraphicScreenWindow(struct window_graphicscreen *window_graphicscreen)
{
  HWND hwnd_group;
  RECT window_rect, client_rect, group_rect;
  int window_x, window_y;
  
  /** On va ajuster la fentre par rapport au Zoom demand **/  
  /* Position actuelle de la fentre dans l'cran */
  GetWindowRect(window_graphicscreen->hwnd_graphicscreen,&window_rect);
  window_x = window_rect.left;
  window_y = window_rect.top;
    
  /* Hauteur Titre */
  GetClientRect(window_graphicscreen->hwnd_graphicscreen,&client_rect);
  window_graphicscreen->title_height = (window_rect.bottom - window_rect.top) - client_rect.bottom;

  /* Position de la GroupBox Select */
  hwnd_group = GetDlgItem(window_graphicscreen->hwnd_graphicscreen,IDC_GROUP_GRAPHICSCREEN);     /* Rcupre le Handle du Control GroupBox li  IDC_GROUP_GRAPHICSCREEN */
  GetWindowRect(hwnd_group,&group_rect);
  window_graphicscreen->group_data_x = (group_rect.left - window_rect.left) - 3;
  window_graphicscreen->group_data_y = ((group_rect.top - window_rect.top) - window_graphicscreen->title_height) + 3;

  /* Position de la Zone BitMap dans la fentre */
  window_graphicscreen->bitmap_graphicscreen_x = window_graphicscreen->group_data_x + WINDOW_GROUP_BORDER_LEFT_SIZE;
  window_graphicscreen->bitmap_graphicscreen_y = window_graphicscreen->group_data_y + WINDOW_GROUP_BORDER_TOP_SIZE;

  /* Dimension en Pixel du Control Group Box + Resize */  
  window_graphicscreen->group_data_width = WINDOW_GROUP_BORDER_LEFT_SIZE + window_graphicscreen->bitmap_graphicscreen_width + WINDOW_GROUP_BORDER_RIGHT_SIZE;
  window_graphicscreen->group_data_height = WINDOW_GROUP_BORDER_TOP_SIZE + window_graphicscreen->bitmap_graphicscreen_height + WINDOW_GROUP_BORDER_BOTTOM_SIZE;
  MoveWindow(hwnd_group,window_graphicscreen->group_data_x,window_graphicscreen->group_data_y,window_graphicscreen->group_data_width,window_graphicscreen->group_data_height,TRUE);

  /* Largeur de la fentre */
  window_graphicscreen->window_width = window_graphicscreen->group_data_x + window_graphicscreen->group_data_width + WINDOW_CLIENT_RIGHT_BORDER_SIZE + WINDOW_BORDER_SIZE;

  /* On doit garantir une largeur minimale de la fentre */
  if(window_graphicscreen->window_width < GRAPHICSCREEN_DIALOGBOX_WIDTH)
    window_graphicscreen->window_width = GRAPHICSCREEN_DIALOGBOX_WIDTH;

  /* Hauteur de la fentre */
  window_graphicscreen->window_height = window_graphicscreen->title_height + window_graphicscreen->group_data_y + window_graphicscreen->group_data_height + WINDOW_CLIENT_BOTTOM_BORDER_SIZE;

  /* Resize window */
  MoveWindow(window_graphicscreen->hwnd_graphicscreen,window_x,window_y,window_graphicscreen->window_width,window_graphicscreen->window_height,TRUE);
}


/*************************************************************************************************************/
/*  BuildGraphicScreenFromData() :  Construire les donnes Graphique  afficher  partir des donnes brutes. */
/*************************************************************************************************************/
static void BuildGraphicScreenFromData(struct window_graphicscreen *window_graphicscreen)
{
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Rien  faire */
  if(window_graphicscreen->bitmap_data_graphicscreen == NULL)
    return;
}


/***************************************************************************/
/*  DeleteGraphicScreenChildWindows() :  Destruction des fentres enfants. */
/***************************************************************************/
static void DeleteGraphicScreenChildWindows(struct window_graphicscreen *window_graphicscreen)
{
  DeleteObject(window_graphicscreen->hBitmap_graphicscreen);
  ReleaseDC(window_graphicscreen->hwnd_graphicscreen,window_graphicscreen->hDC);
  DeleteDC(window_graphicscreen->hCompatibleDC);
}


/**************************************************************************/
/*  mem_free_window_graphicscreen() :  Libration mmoire pour la structure. */
/**************************************************************************/
void mem_free_window_graphicscreen(struct window_graphicscreen *window_graphicscreen)
{
  if(window_graphicscreen)
    {
      /* Supprime la notification */
      my_Notification(NOTIFICATION_REMOVE,window_graphicscreen->current_notification,NULL);
      window_graphicscreen->current_notification = NULL;
      
      free(window_graphicscreen);
    }
}

/****************************************************************************/
