/*******************************************************************************/
/*                                                                             */
/*  Window_IPC.c :  Module de gestion de la fentre IPC.                       */
/*                                                                             */
/*******************************************************************************/
/*  Auteur : Olivier ZARDINI   *  Brutal Deluxe Software  *   Date : May 2023  */
/*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <process.h>
#include <malloc.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <io.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/timeb.h>
#include <direct.h>
#include <errno.h>

#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>
#include <Commdlg.h>

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_Request.h"
#include "Window_Breakpoint.h"
#include "Window_QuickWatch.h"
#include "Window_Symbol.h"
#include "Window_Main.h"
#include "Window_IPC.h"

static DWORD StartWindowIPCThreadInit(void *);
static int BuildIPCWindow(HINSTANCE);
LRESULT CALLBACK ProcHandlerIPCWindow(HWND,UINT,WPARAM,LPARAM);
static int CreateIPCStaticChildWindows(HWND);
static int DeleteIPCChildWindows(void);

/***************************************************************************/
/*  StartWindowIPCThread() :  Lance un thread pour traiter la fentre IPC. */
/***************************************************************************/
int StartWindowIPCThread(void)
{
  HANDLE thread_handle;
  DWORD thread_id;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Cherche une place libre **/
  param->ipc_thread_status = 1;
  
  /** Lancement du Thread **/
  thread_handle = CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)StartWindowIPCThreadInit,NULL,0,&thread_id);
  if(thread_handle == NULL)
    {
      param->ipc_thread_status = 0;   
      return(1);
    }
  
  /* Conserve le Handle de ce Thread */
  param->ipc_thread_handle = thread_handle;

  /* OK */
  return(0);
}


/***********************************************************************************************/
/*  StartWindowIPCThreadInit() :  Traite la rception des messages de KEGS via 1 Thread ddi. */
/***********************************************************************************************/
static DWORD StartWindowIPCThreadInit(void *thread_nothing)
{
  MSG msg;
  int error;
  struct window_ipc *window_ipc;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /*******************************************************/
  /** Cration de la fentre IPC, mais ne l'affiche pas **/
  error = BuildIPCWindow(param->hInstance);
  if(error)
    {
      return(1);
    }
  my_Memory(MEMORY_GET_WINDOW_IPC,&window_ipc,NULL);

  /**************************************************************************************/
  /** Rentre dans la boucle de message de la fentre => Rception des Messages de KEGS **/
  msg.wParam = 0;
  /** On rcupre/traite les messages des fentres de ce Thread = window_ipc **/
  while(GetMessage(&msg,NULL,0,0))
    {
      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }
  my_Memory(MEMORY_FREE_WINDOW_IPC,NULL,NULL);

  /* Fin du Thread */
  ExitThread(0);

  /* OK */
  return(0);
}


/********************************************************************/
/*  BuildIPCWindow() :  Construction / Affichage de la fentre IPC. */
/********************************************************************/
static int BuildIPCWindow(HINSTANCE hInstance)
{
  WNDCLASS wndclass;
  HWND hwndIPC;
  RECT client_rect, window_rect;
  struct window_ipc *window_ipc;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  
  /** Allocation de la structure Window_IPC **/
  window_ipc = (struct window_ipc *) calloc(1,sizeof(struct window_ipc));
  if(window_ipc == NULL)
    {
      return(1);
    }
  my_Memory(MEMORY_SET_WINDOW_IPC,window_ipc,NULL);

  /* Nom de la fentre IPC Cyrene */
  sprintf(window_ipc->cyrene_ipc_window_name,CYRENE_IPC_WINDOW_NAME,param->cyrene_pid);

  /** Enregistrement de la Classe de la Fentre ipc **/
  if(param->register_class_ipc == 0)
    {
      memset(&wndclass,0,sizeof(WNDCLASS));
      wndclass.style = 0;
      wndclass.lpfnWndProc = (WNDPROC)ProcHandlerIPCWindow;
      wndclass.cbClsExtra = 0;
      wndclass.cbWndExtra = 0;
      wndclass.hInstance = hInstance;
      wndclass.hIcon = LoadIcon(NULL,IDI_APPLICATION);
      wndclass.hCursor = LoadCursor((HINSTANCE)NULL,IDC_ARROW);
      wndclass.hbrBackground = (HBRUSH) COLOR_BACKGROUND;       /* Couleur de Fond */
      wndclass.lpszClassName = CYRENE_IPC_WINDOW_CLASS;
      if(!RegisterClass(&wndclass))
        {
          return(1);
        }
        
      /* La classe de fentre est enregistre */
      param->register_class_ipc = 1;
    }
    
  /** Create Window IPC **/
  hwndIPC = CreateWindowEx(0,CYRENE_IPC_WINDOW_CLASS,window_ipc->cyrene_ipc_window_name,WS_CAPTION|WS_SYSMENU|WS_MINIMIZEBOX,CW_USEDEFAULT,CW_USEDEFAULT,
                           IPC_WINDOW_MIN_WIDTH,IPC_WINDOW_MIN_HEIGHT,NULL,NULL,hInstance,NULL);
  if(hwndIPC == NULL)
    {
      return(1);
    }
  window_ipc->hwnd_cyrene_ipc = hwndIPC;

  /* Affiche la fentre */
  ShowWindow(window_ipc->hwnd_cyrene_ipc,SW_HIDE);

  /* Hauteur Titre + Menu */
  GetWindowRect(hwndIPC,&window_rect);
  GetClientRect(hwndIPC,&client_rect);
  window_ipc->title_height = (window_rect.bottom - window_rect.top) - client_rect.bottom;

  /* OK */
  return(0);
}


/********************************************************************/
/*  ProcHandlerIPCWindow() :  Gestion des vnements de la fentre. */
/********************************************************************/
LRESULT CALLBACK ProcHandlerIPCWindow(HWND hwndIPC, UINT umsg, WPARAM wParam, LPARAM lParam)
{
  COPYDATASTRUCT *kegs_copy_data;
  WORD low_word_param, high_word_param;
  struct window_ipc *window_ipc;

  /* Rcupre les fentres */
  my_Memory(MEMORY_GET_WINDOW_IPC,&window_ipc,NULL);

  switch(umsg)
    {
      /****************************/
      /* Initialization de window */
      /****************************/
      case WM_CREATE:
        CreateIPCStaticChildWindows(hwndIPC);
        return(0);

      /*********************************************/
      /* Redimension et rafrachissement de window */
      /*********************************************/
      case WM_SIZE:
        return(0);

      case WM_PAINT:
        break;
      
      /****************************************************/
      /* Inter Process Communication entre Kegs et Cyrene */
      /****************************************************/
      case WM_COPYDATA :
        kegs_copy_data = (COPYDATASTRUCT *) lParam;
        AddRequest((int)kegs_copy_data->dwData,(int)kegs_copy_data->cbData,(unsigned char *)kegs_copy_data->lpData);
        return(1);         /* On renvoi 1 pour indiquer  KEGS.exe que le message a bien t pris en compte */

      /************************/
      /* Evnements de Bouton */
      /************************/
      case WM_COMMAND :
        low_word_param = LOWORD(wParam);
        high_word_param = HIWORD(wParam);
        return(0);
        
      /***********************/
      /* Fermeture de window */
      /***********************/
      case WM_CLOSE :
        /** On demande la fermeture de l'application **/
        /* Fentre Principale */
        DestroyWindow(hwndIPC);
        return(0);

      case WM_DESTROY :
        DeleteIPCChildWindows();
        /* Demande la sortie de la boucle des messages */
        PostQuitMessage(0);
        return(0);
    }

  /* Renvoi  la fentre appelante */
  return(DefWindowProc(hwndIPC,umsg,wParam,lParam));
}


/***************************************************************************************************/
/*  CreateIPCStaticChildWindows() : Crer les fentres enfants et remplir la structure window_ipc. */
/***************************************************************************************************/
static int CreateIPCStaticChildWindows(HWND hwndIPC)
{
  struct window_ipc *window_ipc;
  my_Memory(MEMORY_GET_WINDOW_IPC,&window_ipc,NULL);

  /* Fill the structure member ipc window */
  window_ipc->hwnd_cyrene_ipc = hwndIPC;

  /** Device Context **/
  window_ipc->hDC = GetDC(hwndIPC);
  window_ipc->hCompatibleDC = CreateCompatibleDC(window_ipc->hDC);
  
  /* OK */
  return(0);
}


/*******************************************************************************************/
/*  UpdateBreapointTableInSharedMemory() :  Transfert les Breakpoint dans la SharedMemory. */
/*******************************************************************************************/
void UpdateBreapointTableInSharedMemory(void)
{
  int offset, nb_breakpoint, status;
  char breakpoint_line_txt[1024];
  struct cyrene_breakpoint *current_breakpoint;
  struct symbol *current_symbol;
  struct window_main *window_main;
  struct window_ipc *window_ipc;

  /* Rcupre la fentre Main + IPC */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  my_Memory(MEMORY_GET_WINDOW_IPC,&window_ipc,NULL);
  if(window_main == NULL || window_ipc == NULL)
    return;
  if(window_ipc->shared_memory_ptr == NULL)
    return;

  /** Pas de Breakpoint disponibles **/
  if(window_main->current_breakpoint_table == NULL)
    {
      /* Peut t'on crire ? */
      memcpy(&status,&window_ipc->shared_memory_ptr[IPC_SHARED_MEMORY_OFFSET_STATUS_BREAKPOINT],sizeof(int));
      if(status == 1)
        return;

      /* Termine la liste avec un \0 */
      window_ipc->shared_memory_ptr[IPC_SHARED_MEMORY_OFFSET_DATA_BREAKPOINT] = '\0';

      /* Nombre de Breapoint */
      nb_breakpoint = 0;
      memcpy(&window_ipc->shared_memory_ptr[IPC_SHARED_MEMORY_OFFSET_NB_BREAKPOINT],&nb_breakpoint,sizeof(int));

      /* Les donnes sont disponibles */
      status = 1;
      memcpy(&window_ipc->shared_memory_ptr[IPC_SHARED_MEMORY_OFFSET_STATUS_BREAKPOINT],&status,sizeof(int));
      return;
    }

  /* Peut t'on crire ? */
  memcpy(&status,&window_ipc->shared_memory_ptr[IPC_SHARED_MEMORY_OFFSET_STATUS_BREAKPOINT],sizeof(int));
  if(status == 1)
    return;

  /*** Passe en revue tous les breakpoints Enable ***/
  offset = 0;
  nb_breakpoint = 0;
  for(current_breakpoint = window_main->current_breakpoint_table->first_breakpoint; current_breakpoint; current_breakpoint=current_breakpoint->next)
    if(current_breakpoint->status == 1)
      {
        /** On ne va pas transmettre les breakpoints Symbols pour les Symbols non rsolus **/
        if(current_breakpoint->source == BREAKPOINT_SOURCE_SYMBOL)
          {
            /* Recherche le Symbol */
            current_symbol = GetSymbolByNameSegmentFile(current_breakpoint->source_symbol_name,current_breakpoint->source_symbol_segment,current_breakpoint->source_symbol_program);
            if(current_symbol == NULL)
              continue;
            if(current_symbol->is_valid == 0)
              continue;
          }

        /** On construit la ligne du Breakpoint au format Texte (on ajoute le Status mme si a ne sert  rien / Symbol en Adresse) **/
        BuildBreakpointTextLine(current_breakpoint,BREAKPOINT_LINE_TARGET_KEGS,breakpoint_line_txt);

        /* A t'on de la place pour placer le Breakpoint dans la SharedMemory ? */
        if((offset + strlen(breakpoint_line_txt) + 1) >= IPC_SHARED_MEMORY_BREAKPOINT_SIZE)
          break;

        /* Ajoute la ligne dans la mmoire */
        memcpy(&window_ipc->shared_memory_ptr[IPC_SHARED_MEMORY_OFFSET_DATA_BREAKPOINT+offset],breakpoint_line_txt,strlen(breakpoint_line_txt));
        offset += (int) strlen(breakpoint_line_txt);

        /* Un Breapoint valide de plus */
        nb_breakpoint++;
      }

  /* Termine la liste avec un \0 */
  window_ipc->shared_memory_ptr[IPC_SHARED_MEMORY_OFFSET_DATA_BREAKPOINT+offset] = '\0';

  /* Nombre de Breapoint */
  memcpy(&window_ipc->shared_memory_ptr[IPC_SHARED_MEMORY_OFFSET_NB_BREAKPOINT],&nb_breakpoint,sizeof(int));

  /* Les donnes sont disponibles */
  status = 1;
  memcpy(&window_ipc->shared_memory_ptr[IPC_SHARED_MEMORY_OFFSET_STATUS_BREAKPOINT],&status,sizeof(int));
}


/**************************************************************************************************/
/*  UpdateTraceSystemCallInSharedMemory() :  Transfert le Trace System Call dans la SharedMemory. */
/**************************************************************************************************/
void UpdateTraceSystemCallInSharedMemory(void)
{
  BYTE trace_system_call;
  struct window_main *window_main;
  struct window_ipc *window_ipc;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Rcupre la fentre Main + IPC */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  my_Memory(MEMORY_GET_WINDOW_IPC,&window_ipc,NULL);
  if(window_main == NULL || window_ipc == NULL)
    return;
  if(window_ipc->shared_memory_ptr == NULL)
    return;

  /** Dfinir quels appels sont tracs **/
  trace_system_call = 0x00;
  if(window_main->output_trace_syscall == 1)
    {
      trace_system_call |= ((param->trace_p8_calls      == 1) ? TRACE_SYSTEM_CALL_P8      : 0);
      trace_system_call |= ((param->trace_p16_calls     == 1) ? TRACE_SYSTEM_CALL_P16     : 0);
      trace_system_call |= ((param->trace_gsos_calls    == 1) ? TRACE_SYSTEM_CALL_GSOS    : 0);
      trace_system_call |= ((param->trace_toolbox_calls == 1) ? TRACE_SYSTEM_CALL_TOOLBOX : 0);
    }

  /** MAJ du Status **/
  window_ipc->shared_memory_ptr[IPC_SHARED_MEMORY_OFFSET_OUTPUT_SYSCALL] = trace_system_call;
}


/****************************************************************/
/*  DeleteIPCChildWindows() : Destruction des fentres enfants. */
/****************************************************************/
static int DeleteIPCChildWindows(void)
{
  struct window_ipc *window_ipc;
  my_Memory(MEMORY_GET_WINDOW_IPC,&window_ipc,NULL);

  /* OK */
  return(0);
}


/*******************************************************************/
/*  mem_free_window_ipc() :  Libration mmoire pour la structure. */
/*******************************************************************/
void mem_free_window_ipc(struct window_ipc *window_ipc)
{
  if(window_ipc)
    {
      /* Shared Memory */
      if(window_ipc->shared_memory_ptr != NULL)
         UnmapViewOfFile(window_ipc->shared_memory_ptr);
      if(window_ipc->hSharedMemory != 0)
        CloseHandle(window_ipc->hSharedMemory);

      free(window_ipc);
    }
}

/*********************************************************************/
