/*********************************************************************************/
/*                                                                               */
/*  Window_Internals.c :  Module de gestion de la fentre Apple IIgs Internals.  */
/*                                                                               */
/*********************************************************************************/
/*  Auteur : Olivier ZARDINI     *     Brutal Deluxe     *     Date : Aot 2023  */
/*********************************************************************************/

#include <stdio.h>
#include <time.h>

#include <windows.h>
#include <windowsx.h>
#include <Commctrl.h>
#include <Winuser.h>
#include <Richedit.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_Notification.h"
#include "Dc_Graphic.h"
#include "Dc_CPU.h"
#include "Dc_MMU.h"
#include "Dc_IIgsInternals.h"
#include "Cyrene_Engine.h"
#include "Window_QuickWatch.h"
#include "Window_Main.h"
#include "Window_Main_Menu.h"
#include "Window_Disassembler.h"
#include "Window_Internals.h"

static DWORD BuildInternalsWindowThread(void *);
LRESULT CALLBACK ProcHandlerInternalsWindow(HWND,UINT,WPARAM,LPARAM);
static int CreateInternalsChildWindows(HWND,struct window_internals *);
static void RefreshInternalsWindow(void);
static void BuildInternalsFileListView(struct list_view *,HFONT,int,struct window_internals *);
static void BuildInternalsCDAListView(struct list_view *,HFONT,int,struct window_internals *);
static void BuildInternalsNDAListView(struct list_view *,HFONT,int,struct window_internals *);
static void BuildInternalsInitListView(struct list_view *,HFONT,int,struct window_internals *);
static void BuildInternalsDriverListView(struct list_view *,HFONT,int,struct window_internals *);
static void BuildInternalsFSTListView(struct list_view *,HFONT,int,struct window_internals *);
static void BuildInternalsQuickdrawListView(struct list_view *,HFONT,int,struct window_internals *);
static void BuildInternalsInterruptListView(struct list_view *,HFONT,int,struct window_internals *);
static void BuildInternalsHandleListView(struct list_view *,HFONT,int,struct window_internals *);
static void FillInternalsInterruptListView(void);
static void FillInternalsQuickdrawListView(void);
static void FillInternalsFileListView(void);
static void FillInternalsCDAListView(void);
static void FillInternalsNDAListView(void);
static void FillInternalsInitListView(void);
static void FillInternalsDriverListView(void);
static void FillInternalsFSTListView(void);
static void FillInternalsHandleListView(int);
static void ShowOneTabControl(int,struct window_internals *);
LRESULT CALLBACK subEditInternalsProcControl(HWND,UINT,WPARAM,LPARAM);
static void UpdateInternalsGlanceTab(struct window_internals *,int,int);
static void UpdateInternalsGlanceOffset(struct window_internals *,int);
static void UpdateInternalsRefreshMode(struct window_internals *,int);
static void RefreshInternalsGlanceWindowData(struct window_internals *);
static int IsInternalsDataModified(int,int,int,int,struct window_internals *);
static void BuildInternalsFromHandleData(struct window_internals *);
static void DeleteInternalsChildWindows(struct window_internals *);
static void SaveInternalsGlanceData(struct window_internals *);
static void ExportInternalsGlanceHandle(struct window_internals *);


/********************************************************************************/
/*  BuildInternalsWindow() :  Construction / Affichage de la fentre Internals. */
/********************************************************************************/
struct window_internals *BuildInternalsWindow(HINSTANCE hInstance, int menu_id)
{
  struct window_internals *window_internals;

  /** Allocation de la structure window_internals **/
  window_internals = (struct window_internals *) calloc(1,sizeof(struct window_internals));
  if(window_internals == NULL)
    return(NULL);
  my_Memory(MEMORY_SET_WINDOW_INTERNALS,window_internals,NULL);

  /* Lance un thread pour grer la fentre Internals */
  CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)BuildInternalsWindowThread,window_internals,0,NULL);

  /* Renvoi la structure mmoire grant la fentre */
  return(window_internals);
}


/******************************************************************************************/
/*  BuildInternalsWindowThread() :  Cration de la fentre Internals via un Thread ddi. */
/******************************************************************************************/
static DWORD BuildInternalsWindowThread(void *window_internals_param)
{
  HWND hwndInternals;
  MSG msg;
  BOOL bRet;
  struct window_main *window_main;
  struct window_internals *window_internals;
  struct parameter *param;
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Init */
  window_internals = (struct window_internals *) window_internals_param;

  /** Cration de la fentre Apple IIgs Internals **/
  hwndInternals = CreateDialog(param->hInstance,MAKEINTRESOURCE(IDD_WINDOW_INTERNALS),window_main->hwnd_main,(DLGPROC)ProcHandlerInternalsWindow);
  if(hwndInternals == NULL)
    return(1);

  /** Show Window + Paint Content **/
  ShowWindow(hwndInternals,SW_SHOWDEFAULT);
  UpdateWindow(hwndInternals);

  /** Message Loop pour toutes les fentres de ce Thread (fentre Internals...) **/
  while((bRet = GetMessage(&msg,NULL,0,0)) != 0)
    {
      if(bRet == -1)
        break;

      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }

  /* Fin du Thread */
  ExitThread(0);
}


/**************************************************************************/
/*  ProcHandlerInternalsWindow() :  Gestion des vnements de la fentre. */
/**************************************************************************/
LRESULT CALLBACK ProcHandlerInternalsWindow(HWND hwndInternals, UINT umsg, WPARAM wParam, LPARAM lParam)
{
  int i, tab_index, select_id, is_error;
  LPDRAWITEMSTRUCT lpdis;
  TCITEM tie;
  UINT code, code_beginlabeledit, code_endlabeledit, code_getdispinfo, code_setdispinfo;
  BOOL result;
  NMLVDISPINFO *plvdi;
  LVITEM LvItem;
  NMLISTVIEW *pListView;
  DWORD handle_address;
  struct list_view *current_listview;
  char szTabText[256];
  char buffer_handle[256];
  char buffer_address[256];
  struct window_internals *window_internals;
  struct parameter *param;
  
  /* Init */
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  my_Memory(MEMORY_GET_WINDOW_INTERNALS,&window_internals,NULL);
  if(window_internals == NULL)
    return(TRUE);

  switch(umsg)
    {
      case WM_INITDIALOG :
        /* Cration du contenu de la DialogBox */
        CreateInternalsChildWindows(hwndInternals,window_internals);
        return(TRUE);
        
      /****************************/
      /* Evnements de la fentre */
      /****************************/
      case WM_COMMAND :
        switch(LOWORD(wParam))
          {
            /** Bouton Radio : On change la mthode de rafraichissement de la fentre **/
            case IDC_RADIO_REFRESH_AUTOMATIC :
              if(HIWORD(wParam) == BN_CLICKED && window_internals->refresh_mode != INTERNALS_REFRESH_AUTOMATIC)
                UpdateInternalsRefreshMode(window_internals,INTERNALS_REFRESH_AUTOMATIC);
              break;

            case IDC_RADIO_REFRESH_MANUAL :
              if(HIWORD(wParam) == BN_CLICKED && window_internals->refresh_mode != INTERNALS_REFRESH_MANUAL)
                UpdateInternalsRefreshMode(window_internals,INTERNALS_REFRESH_MANUAL);
              break;

            /** Bouton de rafraichissement Manuel de la fentre **/
            case IDC_BUTTON_REFRESH :
              RefreshInternalsWindow();
              RefreshInternalsGlanceWindowData(window_internals);
              break;

            /****************/
            /**  [HANDLE]  **/
            /****************/
            /* Export de tous les handles dans un fichier text */
            case IDC_BUTTON_EXPORT_HANDLE_DATA :
              ExportInternalsGlanceHandle(window_internals);
              break;

            /****************/
            /**  [GLANCE]  **/
            /****************/
            /** Boutons de MAJ de l'onglet [GLANCE] **/
            case IDC_GLANCE_BUTTON_DISASSEMBLE :
              /* Ouvre une fentre de dsassemblage */
              GetDlgItemText(window_internals->hwnd_internals,IDC_GLANCE_EDIT_ADDRESS,buffer_address,256);
              if(strlen(buffer_address) == 0)
                break;
              handle_address = EvaluateAsAddress(buffer_address,&is_error);
              BuildDisassemblerWindow(param->hInstance,ID_MENU_VIEW_CODEDISASSEMBLER,handle_address);
              break;
            case IDC_GLANCE_BUTTON_HANDLE_EXAMINE :
              UpdateInternalsGlanceTab(window_internals,0,1);   /* Current + Reset Offset */
              break;
            case IDC_GLANCE_BUTTON_EXAMINE_PREV :
              UpdateInternalsGlanceTab(window_internals,-1,1);  /* Previous + Reset Offset */
              break;
            case IDC_GLANCE_BUTTON_EXAMINE_NEXT :
              UpdateInternalsGlanceTab(window_internals,1,1);   /* Next + Reset Offset */
              break;

            /** Bouton de dplacement de l'offset **/
            case IDC_GLANCE_BUTTON_LEFT :
              UpdateInternalsGlanceOffset(window_internals,-1);
              break;

            case IDC_GLANCE_BUTTON_RIGHT :
              UpdateInternalsGlanceOffset(window_internals,1);
              break;

            case IDC_GLANCE_BUTTON_UP :
              UpdateInternalsGlanceOffset(window_internals,(-1)*window_internals->nb_column);
              break;

            case IDC_GLANCE_BUTTON_DOWN :
              UpdateInternalsGlanceOffset(window_internals,window_internals->nb_column);
              break;

            case IDC_GLANCE_BUTTON_PAGEUP :
              UpdateInternalsGlanceOffset(window_internals,(-1)*window_internals->nb_column*window_internals->nb_row);
              break;

            case IDC_GLANCE_BUTTON_PAGEDOWN :
              UpdateInternalsGlanceOffset(window_internals,window_internals->nb_column*window_internals->nb_row);
              break;

            /* Sauvegarde des donnes d'un Handle en format binaire */
            case IDC_GLANCE_BUTTON_SAVE_DATA :
              SaveInternalsGlanceData(window_internals);
              break;
          }
        return(TRUE);

      /***********************************/
      /* Evnements de changement de Tab */
      /***********************************/
      case WM_NOTIFY :
        code = ((LPNMHDR)lParam)->code;
        code_beginlabeledit = LVN_BEGINLABELEDIT;
        code_endlabeledit   = LVN_ENDLABELEDIT;
        code_getdispinfo    = LVN_GETDISPINFO;
        code_setdispinfo    = LVN_SETDISPINFO;

        /*************************************/
        /** Evnements de changement de Tab **/
        /*************************************/
        if(code == TCN_SELCHANGING)
          return(FALSE);                    /* Return FALSE to allow the selection to change. */
        else if(code == TCN_SELCHANGE)
          {
            /* Rcupre l'index du Tab slectionn */
            tab_index = (int) SendMessage(window_internals->hwnd_tabcontrol,TCM_GETCURSEL,0,0);
            ShowOneTabControl(tab_index+1,window_internals);
          }
          
        /***************************************************************/
        /**  Click sur une ligne d'une ListView pour la slectionner  **/
        /***************************************************************/
        if(code == NM_CLICK)
          {
            /* Rcupre la ListView (Interrupt / Quickdraw / File / CDA / NDA / Init / Driver / FST / Used / Purged / Free) */
            if(LOWORD(wParam) == window_internals->listview_interrupt.idc_listview)
              current_listview = (struct list_view *) &window_internals->listview_interrupt;
            else if(LOWORD(wParam) == window_internals->listview_quickdraw.idc_listview)
              current_listview = (struct list_view *) &window_internals->listview_quickdraw;
            else if(LOWORD(wParam) == window_internals->listview_file.idc_listview)
              current_listview = (struct list_view *) &window_internals->listview_file;
            else if(LOWORD(wParam) == window_internals->listview_cda.idc_listview)
              current_listview = (struct list_view *) &window_internals->listview_cda;
            else if(LOWORD(wParam) == window_internals->listview_nda.idc_listview)
              current_listview = (struct list_view *) &window_internals->listview_nda;
            else if(LOWORD(wParam) == window_internals->listview_init.idc_listview)
              current_listview = (struct list_view *) &window_internals->listview_init;
            else if(LOWORD(wParam) == window_internals->listview_driver.idc_listview)
              current_listview = (struct list_view *) &window_internals->listview_driver;
            else if(LOWORD(wParam) == window_internals->listview_fst.idc_listview)
              current_listview = (struct list_view *) &window_internals->listview_fst;
            else if(LOWORD(wParam) == window_internals->listview_used.idc_listview)
              current_listview = (struct list_view *) &window_internals->listview_used;
            else if(LOWORD(wParam) == window_internals->listview_purged.idc_listview)
              current_listview = (struct list_view *) &window_internals->listview_purged;
            else if(LOWORD(wParam) == window_internals->listview_free.idc_listview)
              current_listview = (struct list_view *) &window_internals->listview_free;
            else
              break;

            /* Identifie la ligne clicke (0-N) */
            select_id = (int) SendMessage(current_listview->hwnd_listview,LVM_GETNEXTITEM,-1,LVNI_FOCUSED); // return item selected
            if(select_id == -1)
              return(TRUE);
          }

        /*************************************************************************/
        /**  Double-click sur une ligne d'une ListView pour activer une action  **/
        /*************************************************************************/
        if(code == NM_DBLCLK)
          {
            /* Rcupre la ListView (File / CDA / NDA / Init / Driver / FST / Used / Purged / Free) */
            if(LOWORD(wParam) == window_internals->listview_file.idc_listview)
              current_listview = (struct list_view *) &window_internals->listview_file;
            else if(LOWORD(wParam) == window_internals->listview_cda.idc_listview)
              current_listview = (struct list_view *) &window_internals->listview_cda;
            else if(LOWORD(wParam) == window_internals->listview_nda.idc_listview)
              current_listview = (struct list_view *) &window_internals->listview_nda;
            else if(LOWORD(wParam) == window_internals->listview_init.idc_listview)
              current_listview = (struct list_view *) &window_internals->listview_init;
            else if(LOWORD(wParam) == window_internals->listview_driver.idc_listview)
              current_listview = (struct list_view *) &window_internals->listview_driver;
            else if(LOWORD(wParam) == window_internals->listview_fst.idc_listview)
              current_listview = (struct list_view *) &window_internals->listview_fst;
            else if(LOWORD(wParam) == window_internals->listview_used.idc_listview)
              current_listview = (struct list_view *) &window_internals->listview_used;
            else if(LOWORD(wParam) == window_internals->listview_purged.idc_listview)
              current_listview = (struct list_view *) &window_internals->listview_purged;
            else if(LOWORD(wParam) == window_internals->listview_free.idc_listview)
              current_listview = (struct list_view *) &window_internals->listview_free;
            else
              break;

            /* Identifie la ligne clicke (0-N) */
            select_id = (int) SendMessage(current_listview->hwnd_listview,LVM_GETNEXTITEM,-1,LVNI_FOCUSED); // return item selected
            if(select_id == -1)
              return(TRUE);

            /* Rcupre le Handle sur la Colonne #0 */
            memset(&LvItem,0,sizeof(LVITEM));
            LvItem.mask = LVIF_TEXT;
            if(LOWORD(wParam) == window_internals->listview_file.idc_listview)
              LvItem.iSubItem = 5;            /* Colonne #5 pour les Files */
            else
              LvItem.iSubItem = 0;            /* Colonne #0 */
            LvItem.pszText = buffer_handle;
            LvItem.cchTextMax = 256;
            LvItem.iItem = select_id;
            SendMessage(current_listview->hwnd_listview,LVM_GETITEMTEXT,select_id,(LPARAM)&LvItem);

            /** On va ouvrir l'onglet [GLANCE] pour afficher les dtail du Handle slectionn **/
            if(strlen(buffer_handle) == strlen("E1/2000"))
              {
                handle_address = (DWORD) EvaluateAsAddress(buffer_handle,&is_error);
                if(is_error == 0)
                  {
                    /* Nouvel Handle */
                    sprintf(buffer_handle,"%02X",(handle_address >> 16) & 0xFF);
                    SetDlgItemText(window_internals->hwnd_internals,IDC_GLANCE_EDIT_HANDLE_BANK,buffer_handle);
                    sprintf(buffer_handle,"%04X",handle_address & 0xFFFF);
                    SetDlgItemText(window_internals->hwnd_internals,IDC_GLANCE_EDIT_HANDLE_ADDRESS,buffer_handle);

                    /* Construit les donnes de l'onglet [Glance] */
                    UpdateInternalsGlanceTab(window_internals,0,1);    /* Current + Reset Data Offset */

                    /* Affiche l'onglet [Glance] (1-N) */
                    ShowOneTabControl(INTERNALS_TAB_GLANCE_INDEX,window_internals);

                    /* Slectionne l'onglet [Glance] (0-N) */
                    TabCtrl_SetCurSel(window_internals->hwnd_tabcontrol,INTERNALS_TAB_GLANCE_INDEX-1);
                  }
              }
          }
       
        /***************************************************************/
        /**  Click sur les titres des colonnes pour activer les tris  **/
        /***************************************************************/
        if(code == LVN_COLUMNCLICK)
          {
            pListView = (NMLISTVIEW *) lParam;
            select_id = (int) pListView->iSubItem;

            /* Rcupre la ListView (Interrupt / Quickdraw / File / CDA / NDA / Init / Driver / FST / Used / Purged / Free) */
            if(LOWORD(wParam) == window_internals->listview_interrupt.idc_listview)
              current_listview = (struct list_view *) &window_internals->listview_interrupt;
            else if(LOWORD(wParam) == window_internals->listview_quickdraw.idc_listview)
              current_listview = (struct list_view *) &window_internals->listview_quickdraw;
            else if(LOWORD(wParam) == window_internals->listview_file.idc_listview)
              current_listview = (struct list_view *) &window_internals->listview_file;
            else if(LOWORD(wParam) == window_internals->listview_cda.idc_listview)
              current_listview = (struct list_view *) &window_internals->listview_cda;
            else if(LOWORD(wParam) == window_internals->listview_nda.idc_listview)
              current_listview = (struct list_view *) &window_internals->listview_nda;
            else if(LOWORD(wParam) == window_internals->listview_init.idc_listview)
              current_listview = (struct list_view *) &window_internals->listview_init;
            else if(LOWORD(wParam) == window_internals->listview_driver.idc_listview)
              current_listview = (struct list_view *) &window_internals->listview_driver;
            else if(LOWORD(wParam) == window_internals->listview_fst.idc_listview)
              current_listview = (struct list_view *) &window_internals->listview_fst;
            else if(LOWORD(wParam) == window_internals->listview_used.idc_listview)
              current_listview = (struct list_view *) &window_internals->listview_used;
            else if(LOWORD(wParam) == window_internals->listview_purged.idc_listview)
              current_listview = (struct list_view *) &window_internals->listview_purged;
            else if(LOWORD(wParam) == window_internals->listview_free.idc_listview)
              current_listview = (struct list_view *) &window_internals->listview_free;
            else
              break;

            /* On fait le choix du type de tri */
            for(i=0; i<current_listview->nb_column; i++)
              {
                if(i == select_id)
                  {
                    if(current_listview->tab_column[i].sort == INTERNALS_COLUMN_SORT_NONE)
                      current_listview->tab_column[i].sort = INTERNALS_COLUMN_SORT_UP;
                    else if(current_listview->tab_column[i].sort == INTERNALS_COLUMN_SORT_UP)
                      current_listview->tab_column[i].sort = INTERNALS_COLUMN_SORT_DOWN;
                    else
                      current_listview->tab_column[i].sort = INTERNALS_COLUMN_SORT_NONE;
                  }
               else
                 current_listview->tab_column[i].sort = INTERNALS_COLUMN_SORT_NONE;
              }
             
            /* A t'on une colonne trie ? */
            current_listview->has_column_sort = INTERNALS_COLUMN_SORT_NONE;
            for(i=0; i<current_listview->nb_column; i++)
              if(current_listview->tab_column[i].sort != INTERNALS_COLUMN_SORT_NONE)
                current_listview->has_column_sort = current_listview->tab_column[i].sort;
             
            /* On va trier selon le tri mis en place */
            result = ListView_SortItems(current_listview->hwnd_listview,ProcessListViewCompareProc,current_listview);
           
            /* MAJ des images ^ / v des Tris des colonnes */
            setListViewSortIcon(current_listview);
          }
       
        /*********************************************************************************/
        /**  Affichage des Item de la fentre : Remplissage des Item dans les colonnes  **/
        /*********************************************************************************/
        if(code == LVN_GETDISPINFO)
          {
            plvdi = (NMLVDISPINFO *) lParam;

            /* Identifie la ListView : Interrupt / Quickdraw / File / CDA / NDA / Init / Driver / FST / Used / Purged / Free */
            if(LOWORD(wParam) == window_internals->listview_interrupt.idc_listview)
              current_listview = (struct list_view *) &window_internals->listview_interrupt;
            else if(LOWORD(wParam) == window_internals->listview_quickdraw.idc_listview)
              current_listview = (struct list_view *) &window_internals->listview_quickdraw;
            else if(LOWORD(wParam) == window_internals->listview_file.idc_listview)
              current_listview = (struct list_view *) &window_internals->listview_file;
            else if(LOWORD(wParam) == window_internals->listview_cda.idc_listview)
              current_listview = (struct list_view *) &window_internals->listview_cda;
            else if(LOWORD(wParam) == window_internals->listview_nda.idc_listview)
              current_listview = (struct list_view *) &window_internals->listview_nda;
            else if(LOWORD(wParam) == window_internals->listview_init.idc_listview)
              current_listview = (struct list_view *) &window_internals->listview_init;
            else if(LOWORD(wParam) == window_internals->listview_driver.idc_listview)
              current_listview = (struct list_view *) &window_internals->listview_driver;
            else if(LOWORD(wParam) == window_internals->listview_fst.idc_listview)
              current_listview = (struct list_view *) &window_internals->listview_fst;
            else if(LOWORD(wParam) == window_internals->listview_used.idc_listview)
              current_listview = (struct list_view *) &window_internals->listview_used;
            else if(LOWORD(wParam) == window_internals->listview_purged.idc_listview)
              current_listview = (struct list_view *) &window_internals->listview_purged;
            else if(LOWORD(wParam) == window_internals->listview_free.idc_listview)
              current_listview = (struct list_view *) &window_internals->listview_free;
            else
              break;

            /* Valide le numro de la colonne */
            if(plvdi->item.iSubItem < current_listview->nb_column)
              {
                plvdi->item.pszText = current_listview->tab_column[plvdi->item.iSubItem].tab_value_txt[(current_listview->has_column_sort == INTERNALS_COLUMN_SORT_NONE) ? plvdi->item.iItem : plvdi->item.lParam];
                break;
              }
          }

        /***********************************************************************************/
        /**  Gestion de l'affichage des Item pour dfinir une couleur de fond de cellule  **/
        /***********************************************************************************/
        else if(code == NM_CUSTOMDRAW)
          {
            /* Dans une DialogBox, on doit renvoyer TRUE */
            SetWindowLongPtr(window_internals->hwnd_internals,DWLP_MSGRESULT,(LONG_PTR)ProcessListViewWindowCustomDraw(lParam));
            return(TRUE);
          }

        return(TRUE);
        break;

      /**************************/
      /* Couleur du Tab Control */
      /**************************/
      case WM_DRAWITEM :
        lpdis = (LPDRAWITEMSTRUCT) lParam;                          /* item drawing information */
        if(lpdis->hwndItem == window_internals->hwnd_tabcontrol)    /* is this the tab control? */
          {
            /* Init */
            memset(szTabText,'\0',sizeof(szTabText));
            tie.mask = TCIF_TEXT;
            tie.pszText = szTabText;
            tie.cchTextMax = sizeof(szTabText)-1;

            /* Rcupre le Tab Control */
            TabCtrl_GetItem(window_internals->hwnd_tabcontrol,lpdis->itemID,&tie);
            
            /* Dessine le fond du Tab Control */
            FillRect(lpdis->hDC,&lpdis->rcItem,window_internals->hBrush);
            SetBkColor(lpdis->hDC,GetSysColor(COLOR_3DFACE));
            
            /* Remet le Texte */
            TextOut(lpdis->hDC,lpdis->rcItem.left+4,lpdis->rcItem.top+4,tie.pszText,lstrlen(tie.pszText));
          }
        break;

      /*****************************************************************/
      /* Message extrieur demandant le rafraichissement de la fentre */
      /*****************************************************************/
      case HM_INTERNALS_REFRESHWINDOW :
        RefreshInternalsWindow();
        RefreshInternalsGlanceWindowData(window_internals);
        return(TRUE);

      /********************************************************************/
      /* Fermeture de window (le WM_CLOSE est appel avant le WM_DESTROY) */
      /********************************************************************/
      case WM_CLOSE :
        /* Fentre Internals */
        DestroyWindow(hwndInternals);
        return(0);

      case WM_DESTROY :
        /* Suppression des Fentres */
        DeleteInternalsChildWindows(window_internals);

        /* Dcoche le Menu */
        CheckMainWindowMenuItem(ID_MENU_VIEW_INTERNALS,FALSE);

        /* Supprime la structure mmoire de ce thread */
        my_Memory(MEMORY_FREE_WINDOW_INTERNALS,NULL,NULL);

        /* Demande la sortie de la boucle des messages */
        PostQuitMessage(0);
        return(0);
    }

  /* Retourner FALSE pour relayer le message non trait au systme */
  return(FALSE);
}


/*********************************************************************************************************/
/*  CreateInternalsChildWindows() : Crer les fentres enfants et remplir la structure window_internals. */
/*********************************************************************************************************/
static int CreateInternalsChildWindows(HWND hwndInternals, struct window_internals *window_internals)
{
  int i;
  HWND hwnd_group;
  RECT client_rect, window_rect, group_rect;
  int glance_group_data_x, glance_group_data_y, glance_data_x, glance_data_y, glance_data_width, glance_data_height;
  TCITEM tie;
  char *tab_tabcontrol_name[INTERNALS_TAB_COUNT+1] = {" Interrupt Vectors "," QuickDraw II Vectors ", " Files ", " CDA "," NDA "," Inits "," Drivers "," FST "," Used Handles "," Purged Handles "," Free Handles "," Handle at a Glance ",NULL};
  char buffer[BUFFER_SIZE];
  struct s65816 *current_65816;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Rcupration du  */
  my_Memory(MEMORY_GET_S65816,&current_65816,NULL);

  /** Window Handle **/
  window_internals->hwnd_internals = hwndInternals;

  /* Brush */
  window_internals->hBrush = CreateSolidBrush(GetSysColor(COLOR_3DFACE));

  /** Cration du Tableau  onglets **/
  window_internals->hwnd_tabcontrol = CreateWindowA(WC_TABCONTROL,"",WS_CHILD | WS_CLIPSIBLINGS | WS_VISIBLE | TCS_OWNERDRAWFIXED,      /* TCS_OWNERDRAWFIXED : Pour changer la couleur de fond */
                                                    10, 38, 870, 658, window_internals->hwnd_internals,(HMENU)IDC_INTERNALS_TABCONTROL,param->hInstance,0); 
  for(i=0; i<INTERNALS_TAB_COUNT; i++)
    {
      /* On ajoute les Tab */
      tie.mask = TCIF_TEXT | TCIF_IMAGE; 
      tie.iImage = -1;
      tie.pszText = tab_tabcontrol_name[i];
      SendMessage(window_internals->hwnd_tabcontrol,TCM_INSERTITEM,(WPARAM)i,(LPARAM)&tie);
    }

  /* Police de caractre de la fentre (14 points) */
  window_internals->hFont = CreateFontA(LISTVIEW_FONT_HEIGHT,0,0,0,FW_DONTCARE,FALSE,FALSE,FALSE,ANSI_CHARSET,
                                        OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY,DEFAULT_PITCH|FF_SWISS,"Arial");
  SendMessage(window_internals->hwnd_tabcontrol,WM_SETFONT,(WPARAM)window_internals->hFont,TRUE);

  /*******************/
  /**  [INTERRUPT]  **/
  /*******************/
  BuildInternalsInterruptListView(&window_internals->listview_interrupt,window_internals->hFont,IDC_INTERNALS_LISTVIEW_INTERRUPT,window_internals);

  /*******************/
  /**  [QUICKDRAW]  **/
  /*******************/
  BuildInternalsQuickdrawListView(&window_internals->listview_quickdraw,window_internals->hFont,IDC_INTERNALS_LISTVIEW_QUICKDRAW,window_internals);

  /**************/
  /**  [FILE]  **/
  /**************/
  BuildInternalsFileListView(&window_internals->listview_file,window_internals->hFont,IDC_INTERNALS_LISTVIEW_FILE,window_internals);

  /**************/
  /**  [CDA]  **/
  /**************/
  BuildInternalsCDAListView(&window_internals->listview_cda,window_internals->hFont,IDC_INTERNALS_LISTVIEW_CDA,window_internals);

  /**************/
  /**  [NDA]  **/
  /**************/
  BuildInternalsNDAListView(&window_internals->listview_nda,window_internals->hFont,IDC_INTERNALS_LISTVIEW_NDA,window_internals);

  /**************/
  /**  [Init]  **/
  /**************/
  BuildInternalsInitListView(&window_internals->listview_init,window_internals->hFont,IDC_INTERNALS_LISTVIEW_INIT,window_internals);

  /****************/
  /**  [Driver]  **/
  /****************/
  BuildInternalsDriverListView(&window_internals->listview_driver,window_internals->hFont,IDC_INTERNALS_LISTVIEW_DRIVER,window_internals);

  /*************/
  /**  [FST]  **/
  /*************/
  BuildInternalsFSTListView(&window_internals->listview_fst,window_internals->hFont,IDC_INTERNALS_LISTVIEW_FST,window_internals);

  /**************/
  /**  [Used]  **/
  /**************/
  BuildInternalsHandleListView(&window_internals->listview_used,window_internals->hFont,IDC_INTERNALS_LISTVIEW_USED,window_internals);

  /****************/
  /**  [Purged]  **/
  /****************/
  BuildInternalsHandleListView(&window_internals->listview_purged,window_internals->hFont,IDC_INTERNALS_LISTVIEW_PURGED,window_internals);

  /**************/
  /**  [Free]  **/
  /**************/
  BuildInternalsHandleListView(&window_internals->listview_free,window_internals->hFont,IDC_INTERNALS_LISTVIEW_FREE,window_internals);

  /****************/
  /**  [Glance]  **/
  /****************/

  /** Ajoute les images sur les boutons **/
  SendMessage(GetDlgItem(hwndInternals,IDC_GLANCE_BUTTON_LEFT),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_LEFT)));
  SendMessage(GetDlgItem(hwndInternals,IDC_GLANCE_BUTTON_RIGHT),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_RIGHT)));
  SendMessage(GetDlgItem(hwndInternals,IDC_GLANCE_BUTTON_UP),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_UP)));
  SendMessage(GetDlgItem(hwndInternals,IDC_GLANCE_BUTTON_DOWN),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_DOWN)));
  SendMessage(GetDlgItem(hwndInternals,IDC_GLANCE_BUTTON_PAGEUP),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_PAGE_UP)));
  SendMessage(GetDlgItem(hwndInternals,IDC_GLANCE_BUTTON_PAGEDOWN),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_PAGE_DOWN)));

  /** Adresse de la Zone de Data **/
  window_internals->data_offset = 0x000000; 
  sprintf(buffer,"%06X",window_internals->data_offset);
  SetDlgItemText(window_internals->hwnd_internals,IDC_GLANCE_EDIT_DATA_OFFSET,buffer);

  /** Zone d'affichage des Memory Data **/
  window_internals->nb_column = 16;
  window_internals->nb_row = 16; 

  /* Nombre caractres visibles sur la ligne : Address Data Ascii (on ne compte pas le \r\n final) */
  window_internals->text_data_width = (int) ((strlen("00/0000   000000   ") + window_internals->nb_column*3 - 1 + strlen("  ") + window_internals->nb_column));   /* Enlve le dernier sparateur */

  /* Refresh Mode : Automatic */
  UpdateInternalsRefreshMode(window_internals,INTERNALS_REFRESH_AUTOMATIC);

  /* Hauteur Titre */
  GetWindowRect(window_internals->hwnd_internals,&window_rect);
  GetClientRect(window_internals->hwnd_internals,&client_rect);
  window_internals->title_height = (window_rect.bottom - window_rect.top) - client_rect.bottom;

  /* Position de la GroupBox Select + GroupBox Palette */
  hwnd_group = GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_GROUP_HANDLE_DATA);     /* Rcupre le Handle du Control GroupBox li  IDC_GROUP_INTERNALS */
  GetWindowRect(hwnd_group,&group_rect);
  glance_group_data_x = group_rect.left - window_rect.left;
  glance_group_data_y = (group_rect.top - window_rect.top) - window_internals->title_height;

  /* Position de la Zone Text Handle Data dans la fenetre */
  glance_data_x = glance_group_data_x + WINDOW_GROUP_BORDER_LEFT_SIZE;
  glance_data_y = glance_group_data_y + 37 + WINDOW_GROUP_BORDER_TOP_SIZE;

  /* Dimension en Pixel du Control Text Handle Data */
  glance_data_width = window_internals->text_data_width*7 + 2;
  glance_data_height = window_internals->nb_row*INTERNALS_EDIT_FONT_HEIGHT;

  /** Cration du Control Text Edit Multi ligne ReadOnly **/
  window_internals->hwnd_edit_data = CreateWindowExA(0,"EDIT",NULL,WS_CHILD | WS_VISIBLE | ES_LEFT | ES_MULTILINE | ES_READONLY, 
                                                     glance_data_x, glance_data_y, glance_data_width, glance_data_height,   // set size in WM_SIZE message 
                                                     window_internals->hwnd_internals,
                                                     (HMENU)IDC_GLANCE_EDIT_DATA,            // edit control ID
                                                     param->hInstance,
                                                     NULL);

  /** Surchage des Control Edit pour capter les vnement **/
  window_internals->oldEditProcInternalsData = (WNDPROC) SetWindowLongPtr(window_internals->hwnd_edit_data,GWLP_WNDPROC,(LONG_PTR)subEditInternalsProcControl);

  /** Cration de la Police de caractre Courrier + Affectation de la police au Control Edit **/
  window_internals->hFont_courrier = CreateFont(INTERNALS_EDIT_FONT_HEIGHT,0,0,0,FW_DONTCARE,FALSE,FALSE,FALSE,ANSI_CHARSET,OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY,DEFAULT_PITCH|FF_SWISS,TEXT("Courier New"));
  SendMessage(window_internals->hwnd_edit_data,WM_SETFONT,(WPARAM)window_internals->hFont_courrier,MAKELPARAM(FALSE,0));

  /* On se positionne sur l'onglet 1 du tableau */
  window_internals->tab_index = 1;
  ShowOneTabControl(1,window_internals);

  /* Masque les boutons de l'onglet [Glance] */
  EnableWindow(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_BUTTON_EXAMINE_PREV),FALSE);
  EnableWindow(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_BUTTON_EXAMINE_NEXT),FALSE);
  EnableWindow(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_BUTTON_LEFT),FALSE);
  EnableWindow(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_BUTTON_RIGHT),FALSE);
  EnableWindow(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_BUTTON_UP),FALSE);
  EnableWindow(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_BUTTON_DOWN),FALSE);
  EnableWindow(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_BUTTON_PAGEUP),FALSE);
  EnableWindow(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_BUTTON_PAGEDOWN),FALSE);
  EnableWindow(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_BUTTON_SAVE_DATA),FALSE);

  /** Vidage + Remplissage des ListView **/
  RefreshInternalsWindow();

  /* OK */
  return(0);
}


/***************************************************************************************/
/*  RefreshInternalsWindow() :  On va vider le contenu des ListView et les re-Remplir. */
/***************************************************************************************/
static void RefreshInternalsWindow(void)
{
  /** Vidage + Remplissage des ListView **/
  FillInternalsInterruptListView();
  FillInternalsQuickdrawListView();
  FillInternalsFileListView();
  FillInternalsCDAListView();
  FillInternalsNDAListView();
  FillInternalsInitListView();
  FillInternalsDriverListView();
  FillInternalsFSTListView();
  FillInternalsHandleListView(TYPE_HANDLE_USED);
  FillInternalsHandleListView(TYPE_HANDLE_PURGED);
  FillInternalsHandleListView(TYPE_HANDLE_FREE);
}


/*****************************************************************************************/
/*  BuildInternalsInterruptListView() :  Construction d'une ListView pour les Interrupt. */
/*****************************************************************************************/
static void BuildInternalsInterruptListView(struct list_view *current_listview, HFONT hFont, int idc_internals_listview, struct window_internals *window_internals)
{
  /** Init de la structure List_view **/
  current_listview->idc_listview = idc_internals_listview;

  /** Liste des Colonnes **/
  current_listview->nb_column = 0;

  /* Col 0 : Address */
  current_listview->tab_column[0].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[0].align = LVCFMT_LEFT;
  current_listview->tab_column[0].length = 80;
  strcpy(current_listview->tab_column[0].title,"Address");
  current_listview->nb_column++;

  /* Col 1 : Current Value */
  current_listview->tab_column[1].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[1].align = LVCFMT_LEFT;
  current_listview->tab_column[1].length = 120;
  strcpy(current_listview->tab_column[1].title,"Current Value");
  current_listview->nb_column++;

  /* Col 2 : Init Value */
  current_listview->tab_column[2].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[2].align = LVCFMT_LEFT;
  current_listview->tab_column[2].length = 120;
  strcpy(current_listview->tab_column[2].title,"Init Value");
  current_listview->nb_column++;

  /* Col 3 : Vector Name */
  current_listview->tab_column[3].type = INTERNALS_COLUMN_TYPE_TXT;
  current_listview->tab_column[3].align = LVCFMT_LEFT;
  current_listview->tab_column[3].length = 100;
  strcpy(current_listview->tab_column[3].title,"Name");
  current_listview->nb_column++;

  /* Col 4 : Vector Information */
  current_listview->tab_column[4].type = INTERNALS_COLUMN_TYPE_TXT;
  current_listview->tab_column[4].align = LVCFMT_LEFT;
  current_listview->tab_column[4].length = 400;
  strcpy(current_listview->tab_column[4].title,"Information");
  current_listview->nb_column++;

  /** Cration du ListView **/
  BuildListView(window_internals->hwnd_internals,hFont,25,90,INTERNALS_LISTVIEW_LINE_COUNT,current_listview);
}


/***************************************************************************************/
/*  FillInternalsInterruptListView() :  Remplissage de la ListView pour les Interrupt. */
/***************************************************************************************/
static void FillInternalsInterruptListView(void)
{
  int i, result;
  LVITEM listview_item;
  struct s65816 *current_65816;
  struct list_view *current_listview;
  struct window_internals *window_internals;
  struct interrupt_vector_table *current_interrupt_vector;
  char buffer[BUFFER_SIZE];

  /* Rcupre la fentre Internals */
  my_Memory(MEMORY_GET_WINDOW_INTERNALS,&window_internals,NULL);
  if(window_internals == NULL)
    return;
  current_listview = &window_internals->listview_interrupt;

  /** Vide la List View **/
  ListView_DeleteAllItems(window_internals->listview_interrupt.hwnd_listview);

  /* Aucun Vecteur Interrupt ? */
  my_Memory(MEMORY_GET_S65816,&current_65816,NULL);
  if(current_65816 == NULL)
    return;
  current_interrupt_vector = current_65816->current_interrupt_vector;
  if(current_interrupt_vector == NULL)
    return;
  if(current_interrupt_vector->nb_interrupt_vector == 0)
    return;

  /** Remplissage avec les valeurs dans la structure mmoire **/
  current_listview->nb_row = current_interrupt_vector->nb_interrupt_vector;
  for(i=0; i<current_listview->nb_column; i++)
    {
      /* Allocation des tableaux */
      current_listview->tab_column[i].tab_value_num = (DWORD *) calloc(current_listview->nb_row,sizeof(DWORD));
      current_listview->tab_column[i].tab_value_txt = (char **) calloc(current_listview->nb_row,sizeof(char *));
    }
  /** Remplissage des tableaux **/
  for(i=0; i<current_listview->nb_row; i++)
    {
      /* Colonne 0 : Address */
      current_listview->tab_column[0].tab_value_num[i] = current_interrupt_vector->tab_interrupt_vector[i].location;
      sprintf(buffer,"%02X/%04X",(current_interrupt_vector->tab_interrupt_vector[i].location >> 16) & 0xFF,current_interrupt_vector->tab_interrupt_vector[i].location & 0xFFFF);
      current_listview->tab_column[0].tab_value_txt[i] = strdup(buffer);

      /* Colonne 1 : Current Value */
      current_listview->tab_column[1].tab_value_num[i] = current_interrupt_vector->tab_interrupt_vector[i].vector_address;
      current_listview->tab_column[1].tab_value_txt[i] = strdup(current_interrupt_vector->tab_interrupt_vector[i].vector_address_txt);

      /* Colonne 2 : Init Value */
      current_listview->tab_column[2].tab_value_num[i] = current_interrupt_vector->tab_interrupt_vector[i].vector_address_rom;
      current_listview->tab_column[2].tab_value_txt[i] = strdup(current_interrupt_vector->tab_interrupt_vector[i].vector_address_rom_txt);

      /* Colonne 3 : Name */
      current_listview->tab_column[3].tab_value_num[i] = i;
      current_listview->tab_column[3].tab_value_txt[i] = strdup(current_interrupt_vector->tab_interrupt_vector[i].name);

      /* Colonne 4 : Information */
      current_listview->tab_column[4].tab_value_num[i] = i;
      current_listview->tab_column[4].tab_value_txt[i] = strdup(current_interrupt_vector->tab_interrupt_vector[i].information);
    }

  /** Transfert des donnes dans la List View via un Msg LVN_GETDISPINFO **/
  /** Ajout des Item dans le ListView **/
  listview_item.pszText   = LPSTR_TEXTCALLBACK;           /* Sends an LVN_GETDISPINFO message to fill the item */
  listview_item.mask      = LVIF_TEXT | LVIF_IMAGE | LVIF_STATE | LVIF_PARAM;
  listview_item.stateMask = 0;
  listview_item.iSubItem  = 0;
  listview_item.state     = 0;

  /** Initialize LVITEM members that are different for each item **/
  for(i=0; i<current_listview->nb_row; i++)
    {
      listview_item.iItem  = i;
      listview_item.iImage = -1;         /* Identifiant de l'image dans la liste */
      listview_item.lParam = i;          /* Identifiant renvoy lors du tri */
      result = ListView_InsertItem(current_listview->hwnd_listview,&listview_item);
    }
}


/*****************************************************************************************/
/*  BuildInternalsQuickdrawListView() :  Construction d'une ListView pour les Quickdraw. */
/*****************************************************************************************/
static void BuildInternalsQuickdrawListView(struct list_view *current_listview, HFONT hFont, int idc_internals_listview, struct window_internals *window_internals)
{
  /** Init de la structure List_view **/
  current_listview->idc_listview = idc_internals_listview;

  /** Liste des Colonnes **/
  current_listview->nb_column = 0;

  /* Col 0 : Vector Number */
  current_listview->tab_column[0].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[0].align = LVCFMT_LEFT;
  current_listview->tab_column[0].length = 150;
  strcpy(current_listview->tab_column[0].title,"Vector Number");
  current_listview->nb_column++;

  /* Col 1 : Vector Name */
  current_listview->tab_column[1].type = INTERNALS_COLUMN_TYPE_TXT;
  current_listview->tab_column[1].align = LVCFMT_LEFT;
  current_listview->tab_column[1].length = 200;
  strcpy(current_listview->tab_column[1].title,"Vector Name");
  current_listview->nb_column++;

  /* Col 2 : Vector Address */
  current_listview->tab_column[2].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[2].align = LVCFMT_LEFT;
  current_listview->tab_column[2].length = 150;
  strcpy(current_listview->tab_column[2].title,"Vector Location");
  current_listview->nb_column++;

  /* Col 3 : Vector Current Address */
  current_listview->tab_column[3].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[3].align = LVCFMT_LEFT;
  current_listview->tab_column[3].length = 150;
  strcpy(current_listview->tab_column[3].title,"Vector Current Value");
  current_listview->nb_column++;

  /* Col 4 : Vector Init Address */
  current_listview->tab_column[4].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[4].align = LVCFMT_LEFT;
  current_listview->tab_column[4].length = 150;
  strcpy(current_listview->tab_column[4].title,"Vector Init Value");
  current_listview->nb_column++;

  /** Cration du ListView **/
  BuildListView(window_internals->hwnd_internals,hFont,25,90,INTERNALS_LISTVIEW_LINE_COUNT,current_listview);
}


/***************************************************************************************/
/*  FillInternalsQuickdrawListView() :  Remplissage de la ListView pour les Quickdraw. */
/***************************************************************************************/
static void FillInternalsQuickdrawListView(void)
{
  int i, result;
  LVITEM listview_item;
  struct s65816 *current_65816;
  struct list_view *current_listview;
  struct window_internals *window_internals;
  struct quickdraw_vector_table *current_quickdraw_vector;
  char buffer[BUFFER_SIZE];

  /* Rcupre la fentre Internals */
  my_Memory(MEMORY_GET_WINDOW_INTERNALS,&window_internals,NULL);
  if(window_internals == NULL)
    return;
  current_listview = &window_internals->listview_quickdraw;

  /** Vide la List View **/
  ListView_DeleteAllItems(window_internals->listview_quickdraw.hwnd_listview);

  /* Aucun Vecteur Quickdraw ? */
  my_Memory(MEMORY_GET_S65816,&current_65816,NULL);
  if(current_65816 == NULL)
    return;
  current_quickdraw_vector = current_65816->current_quickdraw_vector;
  if(current_quickdraw_vector == NULL)
    return;
  if(current_quickdraw_vector->nb_quickdraw_vector == 0)
    return;

  /** Remplissage avec les valeurs dans la structure mmoire **/
  current_listview->nb_row = current_quickdraw_vector->nb_quickdraw_vector;
  for(i=0; i<current_listview->nb_column; i++)
    {
      /* Allocation des tableaux */
      current_listview->tab_column[i].tab_value_num = (DWORD *) calloc(current_listview->nb_row,sizeof(DWORD));
      current_listview->tab_column[i].tab_value_txt = (char **) calloc(current_listview->nb_row,sizeof(char *));
    }
  /** Remplissage des tableaux **/
  for(i=0; i<current_listview->nb_row; i++)
    {
      /* Colonne 0 : Vector Number */
      current_listview->tab_column[0].tab_value_num[i] = current_quickdraw_vector->tab_quickdraw_vector[i].number;
      sprintf(buffer,"%04X",current_quickdraw_vector->tab_quickdraw_vector[i].number);
      current_listview->tab_column[0].tab_value_txt[i] = strdup(buffer);

      /* Colonne 1 : Vector Name */
      current_listview->tab_column[1].tab_value_num[i] = i;
      current_listview->tab_column[1].tab_value_txt[i] = strdup(current_quickdraw_vector->tab_quickdraw_vector[i].name);

      /* Colonne 2 : Vector Location */
      current_listview->tab_column[2].tab_value_num[i] = current_quickdraw_vector->tab_quickdraw_vector[i].location;
      sprintf(buffer,"%02X/%04X",(current_quickdraw_vector->tab_quickdraw_vector[i].location >> 16) & 0xFF,current_quickdraw_vector->tab_quickdraw_vector[i].location & 0xFFFF);
      current_listview->tab_column[2].tab_value_txt[i] = strdup(buffer);

      /* Colonne 3 : Vector Current Address */
      current_listview->tab_column[3].tab_value_num[i] = current_quickdraw_vector->tab_quickdraw_vector[i].vector_address;
      sprintf(buffer,"JMP  $%02X/%04X",(current_quickdraw_vector->tab_quickdraw_vector[i].vector_address >> 24) & 0x00FF,(current_quickdraw_vector->tab_quickdraw_vector[i].vector_address >> 8) & 0xFFFF);
      current_listview->tab_column[3].tab_value_txt[i] = strdup(buffer);

      /* Colonne 4 : Vector Init Address */
      current_listview->tab_column[4].tab_value_num[i] = current_quickdraw_vector->tab_quickdraw_vector[i].vector_address_rom;
      sprintf(buffer,"JMP  $%02X/%04X",(current_quickdraw_vector->tab_quickdraw_vector[i].vector_address_rom >> 24) & 0x00FF,(current_quickdraw_vector->tab_quickdraw_vector[i].vector_address_rom >> 8) & 0xFFFF);
      current_listview->tab_column[4].tab_value_txt[i] = strdup(buffer);
    }

  /** Transfert des donnes dans la List View via un Msg LVN_GETDISPINFO **/
  /** Ajout des Item dans le ListView **/
  listview_item.pszText   = LPSTR_TEXTCALLBACK;           /* Sends an LVN_GETDISPINFO message to fill the item */
  listview_item.mask      = LVIF_TEXT | LVIF_IMAGE | LVIF_STATE | LVIF_PARAM;
  listview_item.stateMask = 0;
  listview_item.iSubItem  = 0;
  listview_item.state     = 0;

  /** Initialize LVITEM members that are different for each item **/
  for(i=0; i<current_listview->nb_row; i++)
    {
      listview_item.iItem  = i;
      listview_item.iImage = -1;         /* Identifiant de l'image dans la liste */
      listview_item.lParam = i;          /* Identifiant renvoy lors du tri */
      result = ListView_InsertItem(current_listview->hwnd_listview,&listview_item);
    }
}


/*******************************************************************************/
/*  BuildInternalsFileListView() :  Construction d'une ListView pour les File. */
/*******************************************************************************/
static void BuildInternalsFileListView(struct list_view *current_listview, HFONT hFont, int idc_internals_listview, struct window_internals *window_internals)
{
  /** Init de la structure List_view **/
  current_listview->idc_listview = idc_internals_listview;

  /** Liste des Colonnes **/
  current_listview->nb_column = 0;

  /* Col 0 : File Path */
  current_listview->tab_column[0].type = INTERNALS_COLUMN_TYPE_TXT;
  current_listview->tab_column[0].align = LVCFMT_LEFT;
  current_listview->tab_column[0].length = 240;
  strcpy(current_listview->tab_column[0].title,"File Path");
  current_listview->nb_column++;

  /* Col 1 : File Name */
  current_listview->tab_column[1].type = INTERNALS_COLUMN_TYPE_TXT;
  current_listview->tab_column[1].align = LVCFMT_LEFT;
  current_listview->tab_column[1].length = 110;
  strcpy(current_listview->tab_column[1].title,"File Name");
  current_listview->nb_column++;

  /* Col 2 : User ID */
  current_listview->tab_column[2].type = INTERNALS_COLUMN_TYPE_TXT;
  current_listview->tab_column[2].align = LVCFMT_LEFT;
  current_listview->tab_column[2].length = 75;
  strcpy(current_listview->tab_column[2].title,"User ID");
  current_listview->nb_column++;

  /* Col 3 : Segment Num */
  current_listview->tab_column[3].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[3].align = LVCFMT_LEFT;
  current_listview->tab_column[3].length = 85;
  strcpy(current_listview->tab_column[3].title,"Seg Num");
  current_listview->nb_column++;

  /* Col 4 : Segment Name */
  current_listview->tab_column[4].type = INTERNALS_COLUMN_TYPE_TXT;
  current_listview->tab_column[4].align = LVCFMT_LEFT;
  current_listview->tab_column[4].length = 100;
  strcpy(current_listview->tab_column[4].title,"Seg Name");
  current_listview->nb_column++;

  /* Col 5 : Handle */
  current_listview->tab_column[5].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[5].align = LVCFMT_LEFT;
  current_listview->tab_column[5].length = 80;
  strcpy(current_listview->tab_column[5].title,"Handle");
  current_listview->nb_column++;

  /* Col 6 : Address */
  current_listview->tab_column[6].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[6].align = LVCFMT_LEFT;
  current_listview->tab_column[6].length = 80;
  strcpy(current_listview->tab_column[6].title,"Address");
  current_listview->nb_column++;

  /* Col 7 : Size */
  current_listview->tab_column[7].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[7].align = LVCFMT_LEFT;
  current_listview->tab_column[7].length = 60;
  strcpy(current_listview->tab_column[7].title,"Size");
  current_listview->nb_column++;

  /** Cration du ListView **/
  BuildListView(window_internals->hwnd_internals,hFont,25,90,INTERNALS_LISTVIEW_LINE_COUNT,current_listview);
}


/*****************************************************************************/
/*  FillInternalsFileListView() :  Remplissage de la ListView pour les File. */
/*****************************************************************************/
static void FillInternalsFileListView(void)
{
  int i, result;
  LVITEM listview_item;
  struct s65816 *current_65816;
  struct list_view *current_listview;
  struct window_internals *window_internals;
  struct file_segment_table *current_filesegmenttable;
  char buffer[BUFFER_SIZE];
  
  /* Rcupre la fentre Internals */
  my_Memory(MEMORY_GET_WINDOW_INTERNALS,&window_internals,NULL);
  if(window_internals == NULL)
    return;
  current_listview = &window_internals->listview_file;

  /** Vide la List View **/
  ListView_DeleteAllItems(window_internals->listview_file.hwnd_listview);

  /* Aucun CDA ? */
  my_Memory(MEMORY_GET_S65816,&current_65816,NULL);
  if(current_65816 == NULL)
    return;
  current_filesegmenttable = current_65816->current_file_segment;
  if(current_filesegmenttable == NULL)
    return;
  if(current_filesegmenttable->nb_segment_table == 0)
    return;

  /** Remplissage avec les valeurs dans la structure mmoire **/
  current_listview->nb_row = current_filesegmenttable->nb_segment_table;
  for(i=0; i<current_listview->nb_column; i++)
    {
      /* Allocation des tableaux */
      current_listview->tab_column[i].tab_value_num = (DWORD *) calloc(current_listview->nb_row,sizeof(DWORD));
      current_listview->tab_column[i].tab_value_txt = (char **) calloc(current_listview->nb_row,sizeof(char *));
    }
  /** Remplissage des tableaux **/
  for(i=0; i<current_listview->nb_row; i++)
    {
      /* Colonne 0 : File Path */
      current_listview->tab_column[0].tab_value_num[i] = i;
      current_listview->tab_column[0].tab_value_txt[i] = strdup(current_filesegmenttable->tab_file_segment[i]->file_path);

      /* Colonne 1 : File Name */
      current_listview->tab_column[1].tab_value_num[i] = i;
      current_listview->tab_column[1].tab_value_txt[i] = strdup(current_filesegmenttable->tab_file_segment[i]->file_name);

      /* Colonne 2 : User ID */
      current_listview->tab_column[2].tab_value_num[i] = current_filesegmenttable->tab_file_segment[i]->user_id;
      sprintf(buffer,"%04X",current_filesegmenttable->tab_file_segment[i]->user_id);
      current_listview->tab_column[2].tab_value_txt[i] = strdup(buffer);

      /* Colonne 3 : Segment Name */
      current_listview->tab_column[3].tab_value_num[i] = current_filesegmenttable->tab_file_segment[i]->segment_num;
      sprintf(buffer,"%d",current_filesegmenttable->tab_file_segment[i]->segment_num);
      current_listview->tab_column[3].tab_value_txt[i] = strdup(buffer);

      /* Colonne 4 : Segment Name */
      current_listview->tab_column[4].tab_value_num[i] = i;
      current_listview->tab_column[4].tab_value_txt[i] = strdup(current_filesegmenttable->tab_file_segment[i]->segment_name);

      /* Colonne 5 : Handle */
      current_listview->tab_column[5].tab_value_num[i] = current_filesegmenttable->tab_file_segment[i]->handle;
      sprintf(buffer,"%02X/%04X",(current_filesegmenttable->tab_file_segment[i]->handle >> 16) & 0xFF,current_filesegmenttable->tab_file_segment[i]->handle & 0xFFFF);
      current_listview->tab_column[5].tab_value_txt[i] = strdup(buffer);

      /* Colonne 6 : Address */
      current_listview->tab_column[6].tab_value_num[i] = current_filesegmenttable->tab_file_segment[i]->address;
      sprintf(buffer,"%02X/%04X",(current_filesegmenttable->tab_file_segment[i]->address >> 16) & 0xFF,current_filesegmenttable->tab_file_segment[i]->address & 0xFFFF);
      current_listview->tab_column[6].tab_value_txt[i] = strdup(buffer); 

       /* Colonne 7 : Size */
      current_listview->tab_column[7].tab_value_num[i] = current_filesegmenttable->tab_file_segment[i]->size;
      sprintf(buffer,"%d",current_filesegmenttable->tab_file_segment[i]->size);
      current_listview->tab_column[7].tab_value_txt[i] = strdup(buffer);  
    }

  /** Transfert des donnes dans la List View via un Msg LVN_GETDISPINFO **/
  /** Ajout des Item dans le ListView **/
  listview_item.pszText   = LPSTR_TEXTCALLBACK;           /* Sends an LVN_GETDISPINFO message to fill the item */
  listview_item.mask      = LVIF_TEXT | LVIF_IMAGE | LVIF_STATE | LVIF_PARAM;
  listview_item.stateMask = 0;
  listview_item.iSubItem  = 0;
  listview_item.state     = 0;

  /** Initialize LVITEM members that are different for each item **/
  for(i=0; i<current_listview->nb_row; i++)
    {
      listview_item.iItem  = i;
      listview_item.iImage = -1;         /* Identifiant de l'image dans la liste */
      listview_item.lParam = i;          /* Identifiant renvoy lors du tri */
      result = ListView_InsertItem(current_listview->hwnd_listview,&listview_item);
    }
}


/*****************************************************************************/
/*  BuildInternalsCDAListView() :  Construction d'une ListView pour les CDA. */
/*****************************************************************************/
static void BuildInternalsCDAListView(struct list_view *current_listview, HFONT hFont, int idc_internals_listview, struct window_internals *window_internals)
{
  /** Init de la structure List_view **/
  current_listview->idc_listview = idc_internals_listview;

  /** Liste des Colonnes **/
  current_listview->nb_column = 0;

  /* Col 0 : Handle */
  current_listview->tab_column[0].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[0].align = LVCFMT_LEFT;
  current_listview->tab_column[0].length = 70;
  strcpy(current_listview->tab_column[0].title,"Handle");
  current_listview->nb_column++;

  /* Col 1 : CDA Entry Address */
  current_listview->tab_column[1].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[1].align = LVCFMT_LEFT;
  current_listview->tab_column[1].length = 130;
  strcpy(current_listview->tab_column[1].title,"CDA Entry Address");
  current_listview->nb_column++;

  /* Col 2 : CDA Name */
  current_listview->tab_column[2].type = INTERNALS_COLUMN_TYPE_TXT;
  current_listview->tab_column[2].align = LVCFMT_LEFT;
  current_listview->tab_column[2].length = 130;
  strcpy(current_listview->tab_column[2].title,"CDA Name");
  current_listview->nb_column++;

  /* Col 3 : Entry Address */
  current_listview->tab_column[3].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[3].align = LVCFMT_LEFT;
  current_listview->tab_column[3].length = 110;
  strcpy(current_listview->tab_column[3].title,"Entry Address");
  current_listview->nb_column++;

  /* Col 4 : Exit Address */
  current_listview->tab_column[4].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[4].align = LVCFMT_LEFT;
  current_listview->tab_column[4].length = 100;
  strcpy(current_listview->tab_column[4].title,"Exit Address");
  current_listview->nb_column++;

  /* Col 5 : User ID */
  current_listview->tab_column[5].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[5].align = LVCFMT_LEFT;
  current_listview->tab_column[5].length = 70;
  strcpy(current_listview->tab_column[5].title,"User ID");
  current_listview->nb_column++;

  /* Col 6 : File Path */
  current_listview->tab_column[6].type = INTERNALS_COLUMN_TYPE_TXT;
  current_listview->tab_column[6].align = LVCFMT_LEFT;
  current_listview->tab_column[6].length = 210;
  strcpy(current_listview->tab_column[6].title,"File Path");
  current_listview->nb_column++;

  /** Cration du ListView **/
  BuildListView(window_internals->hwnd_internals,hFont,25,90,INTERNALS_LISTVIEW_LINE_COUNT,current_listview);
}


/***************************************************************************/
/*  FillInternalsCDAListView() :  Remplissage de la ListView pour les CDA. */
/***************************************************************************/
static void FillInternalsCDAListView(void)
{
  int i, j, result;
  LVITEM listview_item;
  struct s65816 *current_65816;
  struct list_view *current_listview;
  struct window_internals *window_internals;
  struct cda_table *current_cdatable;
  char buffer[BUFFER_SIZE];
  char *file_path;

  /* Rcupre la fentre Internals */
  my_Memory(MEMORY_GET_WINDOW_INTERNALS,&window_internals,NULL);
  if(window_internals == NULL)
    return;
  current_listview = &window_internals->listview_cda;

  /** Vide la List View **/
  ListView_DeleteAllItems(window_internals->listview_cda.hwnd_listview);

  /* Aucun CDA ? */
  my_Memory(MEMORY_GET_S65816,&current_65816,NULL);
  if(current_65816 == NULL)
    return;
  current_cdatable = current_65816->current_cda;
  if(current_cdatable == NULL)
    return;
  if(current_cdatable->nb_cda == 0)
    return;

  /** Remplissage avec les valeurs dans la structure mmoire **/
  current_listview->nb_row = current_cdatable->nb_cda;
  for(i=0; i<current_listview->nb_column; i++)
    {
      /* Allocation des tableaux */
      current_listview->tab_column[i].tab_value_num = (DWORD *) calloc(current_listview->nb_row,sizeof(DWORD));
      current_listview->tab_column[i].tab_value_txt = (char **) calloc(current_listview->nb_row,sizeof(char *));
    }
  /** Remplissage des tableaux **/
  for(i=0; i<current_listview->nb_row; i++)
    {
      /* Colonne 0 : Handle */
      current_listview->tab_column[0].tab_value_num[i] = current_cdatable->tab_cda[i].handle;
      sprintf(buffer,"%02X/%04X",(current_cdatable->tab_cda[i].handle >> 16) & 0xFF,current_cdatable->tab_cda[i].handle & 0xFFFF);
      current_listview->tab_column[0].tab_value_txt[i] = strdup(buffer);

      /* Colonne 1 : CDA Entry Address */
      current_listview->tab_column[1].tab_value_num[i] = current_cdatable->tab_cda[i].cda_entry_address;
      sprintf(buffer,"%02X/%04X",(current_cdatable->tab_cda[i].cda_entry_address >> 16) & 0xFF,current_cdatable->tab_cda[i].cda_entry_address & 0xFFFF);
      current_listview->tab_column[1].tab_value_txt[i] = strdup(buffer);

      /* Colonne 2 : Name */
      current_listview->tab_column[2].tab_value_num[i] = i;
      current_listview->tab_column[2].tab_value_txt[i] = strdup(current_cdatable->tab_cda[i].name);

      /* Colonne 3 : Entry Address */
      current_listview->tab_column[3].tab_value_num[i] = current_cdatable->tab_cda[i].entry_addr;
      sprintf(buffer,"%02X/%04X",(current_cdatable->tab_cda[i].entry_addr >> 16) & 0xFF,current_cdatable->tab_cda[i].entry_addr & 0xFFFF);
      current_listview->tab_column[3].tab_value_txt[i] = strdup(buffer);

      /* Colonne 4 : Exit Address */
      current_listview->tab_column[4].tab_value_num[i] = current_cdatable->tab_cda[i].exit_addr;
      sprintf(buffer,"%02X/%04X",(current_cdatable->tab_cda[i].exit_addr >> 16) & 0xFF ,current_cdatable->tab_cda[i].exit_addr & 0xFFFF);
      current_listview->tab_column[4].tab_value_txt[i] = strdup(buffer);

      /* Colonne 5 : User ID */
      current_listview->tab_column[5].tab_value_num[i] = current_cdatable->tab_cda[i].user_id;
      sprintf(buffer,"%04X",current_cdatable->tab_cda[i].user_id);
      current_listview->tab_column[5].tab_value_txt[i] = strdup(buffer);

      /* Colonne 6 : File Path */
      file_path = NULL;
      current_listview->tab_column[6].tab_value_num[i] = i;
      for(j=0; j<current_65816->current_file_segment->nb_segment_table; j++)
        if(current_65816->current_file_segment->tab_file_segment[j]->handle == current_cdatable->tab_cda[i].handle)
          {
            file_path = current_65816->current_file_segment->tab_file_segment[j]->file_path;
            break;
          }
      current_listview->tab_column[6].tab_value_txt[i] = strdup((file_path == NULL) ? "" : file_path); 
    }

  /** Transfert des donnes dans la List View via un Msg LVN_GETDISPINFO **/
  /** Ajout des Item dans le ListView **/
  listview_item.pszText   = LPSTR_TEXTCALLBACK;           /* Sends an LVN_GETDISPINFO message to fill the item */
  listview_item.mask      = LVIF_TEXT | LVIF_IMAGE | LVIF_STATE | LVIF_PARAM;
  listview_item.stateMask = 0;
  listview_item.iSubItem  = 0;
  listview_item.state     = 0;

  /** Initialize LVITEM members that are different for each item **/
  for(i=0; i<current_listview->nb_row; i++)
    {
      listview_item.iItem  = i;
      listview_item.iImage = -1;         /* Identifiant de l'image dans la liste */
      listview_item.lParam = i;          /* Identifiant renvoy lors du tri */
      result = ListView_InsertItem(current_listview->hwnd_listview,&listview_item);
    }
}


/*****************************************************************************/
/*  BuildInternalsNDAListView() :  Construction d'une ListView pour les NDA. */
/*****************************************************************************/
static void BuildInternalsNDAListView(struct list_view *current_listview, HFONT hFont, int idc_internals_listview, struct window_internals *window_internals)
{
  /** Init de la structure List_view **/
  current_listview->idc_listview = idc_internals_listview;

  /** Liste des Colonnes **/
  current_listview->nb_column = 0;

  /* Col 0 : Handle */
  current_listview->tab_column[0].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[0].align = LVCFMT_LEFT;
  current_listview->tab_column[0].length = 75;
  strcpy(current_listview->tab_column[0].title,"Handle");
  current_listview->nb_column++;

  /* Col 1 : User ID */
  current_listview->tab_column[1].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[1].align = LVCFMT_LEFT;
  current_listview->tab_column[1].length = 75;
  strcpy(current_listview->tab_column[1].title,"User ID");
  current_listview->nb_column++;

  /* Col 2 : Status */
  current_listview->tab_column[2].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[2].align = LVCFMT_LEFT;
  current_listview->tab_column[2].length = 70;
  strcpy(current_listview->tab_column[2].title,"Status");
  current_listview->nb_column++;

  /* Col 3 : CDA Name */
  current_listview->tab_column[3].type = INTERNALS_COLUMN_TYPE_TXT;
  current_listview->tab_column[3].align = LVCFMT_LEFT;
  current_listview->tab_column[3].length = 110;
  strcpy(current_listview->tab_column[3].title,"NDA Name");
  current_listview->nb_column++;

  /* Col 4 : Open */
  current_listview->tab_column[4].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[4].align = LVCFMT_LEFT;
  current_listview->tab_column[4].length = 70;
  strcpy(current_listview->tab_column[4].title,"Open");
  current_listview->nb_column++;

  /* Col 5 : Close */
  current_listview->tab_column[5].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[5].align = LVCFMT_LEFT;
  current_listview->tab_column[5].length = 70;
  strcpy(current_listview->tab_column[5].title,"Close");
  current_listview->nb_column++;

  /* Col 6 : Action */
  current_listview->tab_column[6].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[6].align = LVCFMT_LEFT;
  current_listview->tab_column[6].length = 70;
  strcpy(current_listview->tab_column[6].title,"Action");
  current_listview->nb_column++;

  /* Col 7 : Init */
  current_listview->tab_column[7].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[7].align = LVCFMT_LEFT;
  current_listview->tab_column[7].length = 65;
  strcpy(current_listview->tab_column[7].title,"Init");
  current_listview->nb_column++;

  /* Col 8 : Period */
  current_listview->tab_column[8].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[8].align = LVCFMT_LEFT;
  current_listview->tab_column[8].length = 70;
  strcpy(current_listview->tab_column[8].title,"Period");
  current_listview->nb_column++;

  /* Col 9 : Mask */
  current_listview->tab_column[9].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[9].align = LVCFMT_LEFT;
  current_listview->tab_column[9].length = 65;
  strcpy(current_listview->tab_column[9].title,"Mask");
  current_listview->nb_column++;
  
  /* Col 10 : Init */
  current_listview->tab_column[10].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[10].align = LVCFMT_LEFT;
  current_listview->tab_column[10].length = 80;
  strcpy(current_listview->tab_column[10].title,"Window");
  current_listview->nb_column++;

  /** Cration du ListView **/
  BuildListView(window_internals->hwnd_internals,hFont,25,90,INTERNALS_LISTVIEW_LINE_COUNT,current_listview);
}


/***************************************************************************/
/*  FillInternalsNDAListView() :  Remplissage de la ListView pour les NDA. */
/***************************************************************************/
static void FillInternalsNDAListView(void)
{
  int i, result;
  LVITEM listview_item;
  struct s65816 *current_65816;
  struct list_view *current_listview;
  struct window_internals *window_internals;
  struct nda_table *current_ndatable;
  char buffer[BUFFER_SIZE];

  /* Rcupre la fentre Internals */
  my_Memory(MEMORY_GET_WINDOW_INTERNALS,&window_internals,NULL);
  if(window_internals == NULL)
    return;
  current_listview = &window_internals->listview_nda;

  /** Vide la List View **/
  ListView_DeleteAllItems(window_internals->listview_nda.hwnd_listview);

  /* Aucun NDA ? */
  my_Memory(MEMORY_GET_S65816,&current_65816,NULL);
  if(current_65816 == NULL)
    return;
  current_ndatable = current_65816->current_nda;
  if(current_ndatable == NULL)
    return;
  if(current_ndatable->nb_nda == 0)
    return;

  /** Remplissage avec les valeurs dans la structure mmoire **/
  current_listview->nb_row = current_ndatable->nb_nda;
  for(i=0; i<current_listview->nb_column; i++)
    {
      /* Allocation des tableaux */
      current_listview->tab_column[i].tab_value_num = (DWORD *) calloc(current_listview->nb_row,sizeof(DWORD));
      current_listview->tab_column[i].tab_value_txt = (char **) calloc(current_listview->nb_row,sizeof(char *));
    }
  /** Remplissage des tableaux **/
  for(i=0; i<current_listview->nb_row; i++)
    {
      /* Colonne 0 : Handle */
      current_listview->tab_column[0].tab_value_num[i] = current_ndatable->tab_nda[i].handle;
      sprintf(buffer,"%02X/%04X",(current_ndatable->tab_nda[i].handle >> 16) & 0xFF,current_ndatable->tab_nda[i].handle & 0xFFFF);
      current_listview->tab_column[0].tab_value_txt[i] = strdup(buffer);

      /* Colonne 1 : User ID */
      current_listview->tab_column[1].tab_value_num[i] = current_ndatable->tab_nda[i].user_id;
      sprintf(buffer,"%04X",current_ndatable->tab_nda[i].user_id);
      current_listview->tab_column[1].tab_value_txt[i] = strdup(buffer);

      /* Colonne 2 : Status */
      current_listview->tab_column[2].tab_value_num[i] = current_ndatable->tab_nda[i].nda_status;
      sprintf(buffer,"%04X",current_ndatable->tab_nda[i].nda_status);
      current_listview->tab_column[2].tab_value_txt[i] = strdup(buffer);

      /* Colonne 3 : Name */
      current_listview->tab_column[3].tab_value_num[i] = i;
      current_listview->tab_column[3].tab_value_txt[i] = strdup(current_ndatable->tab_nda[i].name);

      /* Colonne 4 : Open */
      current_listview->tab_column[4].tab_value_num[i] = current_ndatable->tab_nda[i].open_addr;
      sprintf(buffer,"%02X/%04X",(current_ndatable->tab_nda[i].open_addr >> 16) & 0xFF,current_ndatable->tab_nda[i].open_addr & 0xFFFF);
      current_listview->tab_column[4].tab_value_txt[i] = strdup(buffer);

      /* Colonne 5 : Close */
      current_listview->tab_column[5].tab_value_num[i] = current_ndatable->tab_nda[i].close_addr;
      sprintf(buffer,"%02X/%04X",(current_ndatable->tab_nda[i].close_addr >> 16) & 0xFF ,current_ndatable->tab_nda[i].close_addr & 0xFFFF);
      current_listview->tab_column[5].tab_value_txt[i] = strdup(buffer);

      /* Colonne 6 : Action */
      current_listview->tab_column[6].tab_value_num[i] = current_ndatable->tab_nda[i].action_addr;
      sprintf(buffer,"%02X/%04X",(current_ndatable->tab_nda[i].action_addr >> 16) & 0xFF ,current_ndatable->tab_nda[i].action_addr & 0xFFFF);
      current_listview->tab_column[6].tab_value_txt[i] = strdup(buffer);

      /* Colonne 7 : Init */
      current_listview->tab_column[7].tab_value_num[i] = current_ndatable->tab_nda[i].init_addr;
      sprintf(buffer,"%02X/%04X",(current_ndatable->tab_nda[i].init_addr >> 16) & 0xFF ,current_ndatable->tab_nda[i].init_addr & 0xFFFF);
      current_listview->tab_column[7].tab_value_txt[i] = strdup(buffer);

      /* Colonne 8 : Period */
      current_listview->tab_column[8].tab_value_num[i] = current_ndatable->tab_nda[i].period;
      sprintf(buffer,"%04X",current_ndatable->tab_nda[i].period);
      current_listview->tab_column[8].tab_value_txt[i] = strdup(buffer);

      /* Colonne 9 : Mask */
      current_listview->tab_column[9].tab_value_num[i] = current_ndatable->tab_nda[i].event_mask;
      sprintf(buffer,"%04X",current_ndatable->tab_nda[i].event_mask);
      current_listview->tab_column[9].tab_value_txt[i] = strdup(buffer);

      /* Colonne 10 : Window */
      current_listview->tab_column[10].tab_value_num[i] = current_ndatable->tab_nda[i].window_ptr;
      sprintf(buffer,"%02X/%04X",(current_ndatable->tab_nda[i].window_ptr >> 16) & 0xFF ,current_ndatable->tab_nda[i].window_ptr & 0xFFFF);
      current_listview->tab_column[10].tab_value_txt[i] = strdup(buffer);
  }

  /** Transfert des donnes dans la List View via un Msg LVN_GETDISPINFO **/
  /** Ajout des Item dans le ListView **/
  listview_item.pszText   = LPSTR_TEXTCALLBACK;           /* Sends an LVN_GETDISPINFO message to fill the item */
  listview_item.mask      = LVIF_TEXT | LVIF_IMAGE | LVIF_STATE | LVIF_PARAM;
  listview_item.stateMask = 0;
  listview_item.iSubItem  = 0;
  listview_item.state     = 0;

  /** Initialize LVITEM members that are different for each item **/
  for(i=0; i<current_listview->nb_row; i++)
    {
      listview_item.iItem  = i;
      listview_item.iImage = -1;         /* Identifiant de l'image dans la liste */
      listview_item.lParam = i;          /* Identifiant renvoy lors du tri */
      result = ListView_InsertItem(current_listview->hwnd_listview,&listview_item);
    }
}


/*******************************************************************************/
/*  BuildInternalsInitListView() :  Construction d'une ListView pour les Init. */
/*******************************************************************************/
static void BuildInternalsInitListView(struct list_view *current_listview, HFONT hFont, int idc_internals_listview, struct window_internals *window_internals)
{
  /** Init de la structure List_view **/
  current_listview->idc_listview = idc_internals_listview;

  /** Liste des Colonnes **/
  current_listview->nb_column = 0;

  /* Col 0 : Handle */
  current_listview->tab_column[0].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[0].align = LVCFMT_LEFT;
  current_listview->tab_column[0].length = 70;
  strcpy(current_listview->tab_column[0].title,"Handle");
  current_listview->nb_column++;

  /* Col 1 : File Path */
  current_listview->tab_column[1].type = INTERNALS_COLUMN_TYPE_TXT;
  current_listview->tab_column[1].align = LVCFMT_LEFT;
  current_listview->tab_column[1].length = 270;
  strcpy(current_listview->tab_column[1].title,"File Path");
  current_listview->nb_column++;

  /* Col 2 : File Name */
  current_listview->tab_column[2].type = INTERNALS_COLUMN_TYPE_TXT;
  current_listview->tab_column[2].align = LVCFMT_LEFT;
  current_listview->tab_column[2].length = 130;
  strcpy(current_listview->tab_column[2].title,"Name");
  current_listview->nb_column++;

  /* Col 3 : Type */
  current_listview->tab_column[3].type = INTERNALS_COLUMN_TYPE_TXT;
  current_listview->tab_column[3].align = LVCFMT_LEFT;
  current_listview->tab_column[3].length = 80;
  strcpy(current_listview->tab_column[3].title,"Type");
  current_listview->nb_column++;

  /* Col 4 : User ID */
  current_listview->tab_column[4].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[4].align = LVCFMT_LEFT;
  current_listview->tab_column[4].length = 70;
  strcpy(current_listview->tab_column[4].title,"User ID");
  current_listview->nb_column++;

  /* Col 5 : Address */
  current_listview->tab_column[5].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[5].align = LVCFMT_LEFT;
  current_listview->tab_column[5].length = 100;
  strcpy(current_listview->tab_column[5].title,"Address");
  current_listview->nb_column++;

  /* Col 6 : Size */
  current_listview->tab_column[6].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[6].align = LVCFMT_LEFT;
  current_listview->tab_column[6].length = 100;
  strcpy(current_listview->tab_column[6].title,"Size");
  current_listview->nb_column++;

  /** Cration du ListView **/
  BuildListView(window_internals->hwnd_internals,hFont,25,90,INTERNALS_LISTVIEW_LINE_COUNT,current_listview);
}


/*****************************************************************************/
/*  FillInternalsInitListView() :  Remplissage de la ListView pour les Init. */
/*****************************************************************************/
static void FillInternalsInitListView(void)
{
  int i, result;
  LVITEM listview_item;
  struct s65816 *current_65816;
  struct list_view *current_listview;
  struct window_internals *window_internals;
  struct init_table *current_inittable;
  char buffer[BUFFER_SIZE];
  char *file_path;
  
  /* Rcupre la fentre Internals */
  my_Memory(MEMORY_GET_WINDOW_INTERNALS,&window_internals,NULL);
  if(window_internals == NULL)
    return;
  current_listview = &window_internals->listview_init;

  /** Vide la List View **/
  ListView_DeleteAllItems(window_internals->listview_init.hwnd_listview);

  /* Aucun Init ? */
  my_Memory(MEMORY_GET_S65816,&current_65816,NULL);
  if(current_65816 == NULL)
    return;
  current_inittable = current_65816->current_init;
  if(current_inittable == NULL)
    return;
  if(current_inittable->nb_init == 0)
    return;

  /** Remplissage avec les valeurs dans la structure mmoire **/
  current_listview->nb_row = current_inittable->nb_init;
  for(i=0; i<current_listview->nb_column; i++)
    {
      /* Allocation des tableaux */
      current_listview->tab_column[i].tab_value_num = (DWORD *) calloc(current_listview->nb_row,sizeof(DWORD));
      current_listview->tab_column[i].tab_value_txt = (char **) calloc(current_listview->nb_row,sizeof(char *));
    }
  /** Remplissage des tableaux **/
  for(i=0; i<current_listview->nb_row; i++)
    {
      /* Colonne 0 : Handle */
      current_listview->tab_column[0].tab_value_num[i] = current_inittable->tab_init[i]->handle;
      sprintf(buffer,"%02X/%04X",(current_inittable->tab_init[i]->handle >> 16) & 0xFF,current_inittable->tab_init[i]->handle & 0xFFFF);
      current_listview->tab_column[0].tab_value_txt[i] = strdup(buffer);

      /* Colonne 1 : File Path */
      file_path = NULL;
      current_listview->tab_column[1].tab_value_num[i] = i;
      current_listview->tab_column[1].tab_value_txt[i] = current_inittable->tab_init[i]->file_path; 

      /* Colonne 2 : File Name */
      current_listview->tab_column[2].tab_value_num[i] = i;
      current_listview->tab_column[2].tab_value_txt[i] = current_inittable->tab_init[i]->file_name;

      /* Colonne 3 : Type */
      current_listview->tab_column[3].tab_value_num[i] = i;
      current_listview->tab_column[3].tab_value_txt[i] = strdup(current_inittable->tab_init[i]->type);

      /* Colonne 4 : User ID */
      current_listview->tab_column[4].tab_value_num[i] = current_inittable->tab_init[i]->user_id;
      sprintf(buffer,"%04X",current_inittable->tab_init[i]->user_id);
      current_listview->tab_column[4].tab_value_txt[i] = strdup(buffer);

      /* Colonne 5 : Address */
      current_listview->tab_column[5].tab_value_num[i] = current_inittable->tab_init[i]->address;
      sprintf(buffer,"%02X/%04X",(current_inittable->tab_init[i]->address >> 16) & 0xFF,current_inittable->tab_init[i]->address & 0xFFFF);
      current_listview->tab_column[5].tab_value_txt[i] = strdup(buffer);

      /* Colonne 6 : Size */
      current_listview->tab_column[6].tab_value_num[i] = current_inittable->tab_init[i]->size;
      sprintf(buffer,"%08X",current_inittable->tab_init[i]->size);
      current_listview->tab_column[6].tab_value_txt[i] = strdup(buffer);
    }

  /** Transfert des donnes dans la List View via un Msg LVN_GETDISPINFO **/
  /** Ajout des Item dans le ListView **/
  listview_item.pszText   = LPSTR_TEXTCALLBACK;           /* Sends an LVN_GETDISPINFO message to fill the item */
  listview_item.mask      = LVIF_TEXT | LVIF_IMAGE | LVIF_STATE | LVIF_PARAM;
  listview_item.stateMask = 0;
  listview_item.iSubItem  = 0;
  listview_item.state     = 0;

  /** Initialize LVITEM members that are different for each item **/
  for(i=0; i<current_listview->nb_row; i++)
    {
      listview_item.iItem  = i;
      listview_item.iImage = -1;         /* Identifiant de l'image dans la liste */
      listview_item.lParam = i;          /* Identifiant renvoy lors du tri */
      result = ListView_InsertItem(current_listview->hwnd_listview,&listview_item);
    }
}


/***********************************************************************************/
/*  BuildInternalsDriverListView() :  Construction d'une ListView pour les Driver. */
/***********************************************************************************/
static void BuildInternalsDriverListView(struct list_view *current_listview, HFONT hFont, int idc_internals_listview, struct window_internals *window_internals)
{
  /** Init de la structure List_view **/
  current_listview->idc_listview = idc_internals_listview;

  /** Liste des Colonnes **/
  current_listview->nb_column = 0;

  /* Col 0 : Handle */
  current_listview->tab_column[0].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[0].align = LVCFMT_LEFT;
  current_listview->tab_column[0].length = 70;
  strcpy(current_listview->tab_column[0].title,"Handle");
  current_listview->nb_column++;

  /* Col 1 : File Path */
  current_listview->tab_column[1].type = INTERNALS_COLUMN_TYPE_TXT;
  current_listview->tab_column[1].align = LVCFMT_LEFT;
  current_listview->tab_column[1].length = 350;
  strcpy(current_listview->tab_column[1].title,"File Path");
  current_listview->nb_column++;

  /* Col 2 : File Name */
  current_listview->tab_column[2].type = INTERNALS_COLUMN_TYPE_TXT;
  current_listview->tab_column[2].align = LVCFMT_LEFT;
  current_listview->tab_column[2].length = 130;
  strcpy(current_listview->tab_column[2].title,"Name");
  current_listview->nb_column++;

  /* Col 3 : User ID */
  current_listview->tab_column[3].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[3].align = LVCFMT_LEFT;
  current_listview->tab_column[3].length = 70;
  strcpy(current_listview->tab_column[3].title,"User ID");
  current_listview->nb_column++;

  /* Col 4 : Address */
  current_listview->tab_column[4].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[4].align = LVCFMT_LEFT;
  current_listview->tab_column[4].length = 100;
  strcpy(current_listview->tab_column[4].title,"Address");
  current_listview->nb_column++;

  /* Col 5 : Size */
  current_listview->tab_column[5].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[5].align = LVCFMT_LEFT;
  current_listview->tab_column[5].length = 100;
  strcpy(current_listview->tab_column[5].title,"Size");
  current_listview->nb_column++;

  /** Cration du ListView **/
  BuildListView(window_internals->hwnd_internals,hFont,25,90,INTERNALS_LISTVIEW_LINE_COUNT,current_listview);
}


/*********************************************************************************/
/*  FillInternalsDriverListView() :  Remplissage de la ListView pour les Driver. */
/*********************************************************************************/
static void FillInternalsDriverListView(void)
{
  int i, result;
  LVITEM listview_item;
  struct s65816 *current_65816;
  struct list_view *current_listview;
  struct window_internals *window_internals;
  struct driver_table *current_drivertable;
  char buffer[BUFFER_SIZE];
  char *file_path;
  
  /* Rcupre la fentre Internals */
  my_Memory(MEMORY_GET_WINDOW_INTERNALS,&window_internals,NULL);
  if(window_internals == NULL)
    return;
  current_listview = &window_internals->listview_driver;

  /** Vide la List View **/
  ListView_DeleteAllItems(window_internals->listview_driver.hwnd_listview);

  /* Aucun Init ? */
  my_Memory(MEMORY_GET_S65816,&current_65816,NULL);
  if(current_65816 == NULL)
    return;
  current_drivertable = current_65816->current_driver;
  if(current_drivertable == NULL)
    return;
  if(current_drivertable->nb_driver == 0)
    return;

  /** Remplissage avec les valeurs dans la structure mmoire **/
  current_listview->nb_row = current_drivertable->nb_driver;
  for(i=0; i<current_listview->nb_column; i++)
    {
      /* Allocation des tableaux */
      current_listview->tab_column[i].tab_value_num = (DWORD *) calloc(current_listview->nb_row,sizeof(DWORD));
      current_listview->tab_column[i].tab_value_txt = (char **) calloc(current_listview->nb_row,sizeof(char *));
    }
  /** Remplissage des tableaux **/
  for(i=0; i<current_listview->nb_row; i++)
    {
      /* Colonne 0 : Handle */
      current_listview->tab_column[0].tab_value_num[i] = current_drivertable->tab_driver[i]->handle;
      sprintf(buffer,"%02X/%04X",(current_drivertable->tab_driver[i]->handle >> 16) & 0xFF,current_drivertable->tab_driver[i]->handle & 0xFFFF);
      current_listview->tab_column[0].tab_value_txt[i] = strdup(buffer);

      /* Colonne 1 : File Path */
      file_path = NULL;
      current_listview->tab_column[1].tab_value_num[i] = i;
      current_listview->tab_column[1].tab_value_txt[i] = current_drivertable->tab_driver[i]->file_path; 

      /* Colonne 2 : File Name */
      current_listview->tab_column[2].tab_value_num[i] = i;
      current_listview->tab_column[2].tab_value_txt[i] = current_drivertable->tab_driver[i]->file_name;

      /* Colonne 3 : User ID */
      current_listview->tab_column[3].tab_value_num[i] = current_drivertable->tab_driver[i]->user_id;
      sprintf(buffer,"%04X",current_drivertable->tab_driver[i]->user_id);
      current_listview->tab_column[3].tab_value_txt[i] = strdup(buffer);

      /* Colonne 4 : Address */
      current_listview->tab_column[4].tab_value_num[i] = current_drivertable->tab_driver[i]->address;
      sprintf(buffer,"%02X/%04X",(current_drivertable->tab_driver[i]->address >> 16) & 0xFF,current_drivertable->tab_driver[i]->address & 0xFFFF);
      current_listview->tab_column[4].tab_value_txt[i] = strdup(buffer);

      /* Colonne 5 : Size */
      current_listview->tab_column[5].tab_value_num[i] = current_drivertable->tab_driver[i]->size;
      sprintf(buffer,"%08X",current_drivertable->tab_driver[i]->size);
      current_listview->tab_column[5].tab_value_txt[i] = strdup(buffer);
    }

  /** Transfert des donnes dans la List View via un Msg LVN_GETDISPINFO **/
  /** Ajout des Item dans le ListView **/
  listview_item.pszText   = LPSTR_TEXTCALLBACK;           /* Sends an LVN_GETDISPINFO message to fill the item */
  listview_item.mask      = LVIF_TEXT | LVIF_IMAGE | LVIF_STATE | LVIF_PARAM;
  listview_item.stateMask = 0;
  listview_item.iSubItem  = 0;
  listview_item.state     = 0;

  /** Initialize LVITEM members that are different for each item **/
  for(i=0; i<current_listview->nb_row; i++)
    {
      listview_item.iItem  = i;
      listview_item.iImage = -1;         /* Identifiant de l'image dans la liste */
      listview_item.lParam = i;          /* Identifiant renvoy lors du tri */
      result = ListView_InsertItem(current_listview->hwnd_listview,&listview_item);
    }
}


/*****************************************************************************/
/*  BuildInternalsFSTListView() :  Construction d'une ListView pour les FST. */
/*****************************************************************************/
static void BuildInternalsFSTListView(struct list_view *current_listview, HFONT hFont, int idc_internals_listview, struct window_internals *window_internals)
{
  /** Init de la structure List_view **/
  current_listview->idc_listview = idc_internals_listview;

  /** Liste des Colonnes **/
  current_listview->nb_column = 0;

  /* Col 0 : Handle */
  current_listview->tab_column[0].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[0].align = LVCFMT_LEFT;
  current_listview->tab_column[0].length = 70;
  strcpy(current_listview->tab_column[0].title,"Handle");
  current_listview->nb_column++;

  /* Col 1 : File Path */
  current_listview->tab_column[1].type = INTERNALS_COLUMN_TYPE_TXT;
  current_listview->tab_column[1].align = LVCFMT_LEFT;
  current_listview->tab_column[1].length = 350;
  strcpy(current_listview->tab_column[1].title,"File Path");
  current_listview->nb_column++;

  /* Col 2 : File Name */
  current_listview->tab_column[2].type = INTERNALS_COLUMN_TYPE_TXT;
  current_listview->tab_column[2].align = LVCFMT_LEFT;
  current_listview->tab_column[2].length = 130;
  strcpy(current_listview->tab_column[2].title,"Name");
  current_listview->nb_column++;

  /* Col 3 : User ID */
  current_listview->tab_column[3].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[3].align = LVCFMT_LEFT;
  current_listview->tab_column[3].length = 70;
  strcpy(current_listview->tab_column[3].title,"User ID");
  current_listview->nb_column++;

  /* Col 4 : Address */
  current_listview->tab_column[4].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[4].align = LVCFMT_LEFT;
  current_listview->tab_column[4].length = 100;
  strcpy(current_listview->tab_column[4].title,"Address");
  current_listview->nb_column++;

  /* Col 5 : Size */
  current_listview->tab_column[5].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[5].align = LVCFMT_LEFT;
  current_listview->tab_column[5].length = 100;
  strcpy(current_listview->tab_column[5].title,"Size");
  current_listview->nb_column++;

  /** Cration du ListView **/
  BuildListView(window_internals->hwnd_internals,hFont,25,90,INTERNALS_LISTVIEW_LINE_COUNT,current_listview);
}


/*****************************************************************************/
/*  FillInternalsFSTListView() :  Construction d'une ListView pour les FST. */
/*****************************************************************************/
static void FillInternalsFSTListView(void)
{
  int i, result;
  LVITEM listview_item;
  struct s65816 *current_65816;
  struct list_view *current_listview;
  struct window_internals *window_internals;
  struct fst_table *current_fsttable;
  char buffer[BUFFER_SIZE];
  char *file_path;  

  /* Rcupre la fentre Internals */
  my_Memory(MEMORY_GET_WINDOW_INTERNALS,&window_internals,NULL);
  if(window_internals == NULL)
    return;
  current_listview = &window_internals->listview_fst;

  /** Vide la List View **/
  ListView_DeleteAllItems(window_internals->listview_fst.hwnd_listview);

  /* Aucun FST ? */
  my_Memory(MEMORY_GET_S65816,&current_65816,NULL);
  if(current_65816 == NULL)
    return;
  current_fsttable = current_65816->current_fst;
  if(current_fsttable == NULL)
    return;
  if(current_fsttable->nb_fst == 0)
    return;

  /** Remplissage avec les valeurs dans la structure mmoire **/
  current_listview->nb_row = current_fsttable->nb_fst;
  for(i=0; i<current_listview->nb_column; i++)
    {
      /* Allocation des tableaux */
      current_listview->tab_column[i].tab_value_num = (DWORD *) calloc(current_listview->nb_row,sizeof(DWORD));
      current_listview->tab_column[i].tab_value_txt = (char **) calloc(current_listview->nb_row,sizeof(char *));
    }
  /** Remplissage des tableaux **/
  for(i=0; i<current_listview->nb_row; i++)
    {
      /* Colonne 0 : Handle */
      current_listview->tab_column[0].tab_value_num[i] = current_fsttable->tab_fst[i]->handle;
      sprintf(buffer,"%02X/%04X",(current_fsttable->tab_fst[i]->handle >> 16) & 0xFF,current_fsttable->tab_fst[i]->handle & 0xFFFF);
      current_listview->tab_column[0].tab_value_txt[i] = strdup(buffer);

      /* Colonne 1 : File Path */
      file_path = NULL;
      current_listview->tab_column[1].tab_value_num[i] = i;
      current_listview->tab_column[1].tab_value_txt[i] = current_fsttable->tab_fst[i]->file_path; 

      /* Colonne 2 : File Name */
      current_listview->tab_column[2].tab_value_num[i] = i;
      current_listview->tab_column[2].tab_value_txt[i] = current_fsttable->tab_fst[i]->file_name;

      /* Colonne 3 : User ID */
      current_listview->tab_column[3].tab_value_num[i] = current_fsttable->tab_fst[i]->user_id;
      sprintf(buffer,"%04X",current_fsttable->tab_fst[i]->user_id);
      current_listview->tab_column[3].tab_value_txt[i] = strdup(buffer);

      /* Colonne 4 : Address */
      current_listview->tab_column[4].tab_value_num[i] = current_fsttable->tab_fst[i]->address;
      sprintf(buffer,"%02X/%04X",(current_fsttable->tab_fst[i]->address >> 16) & 0xFF,current_fsttable->tab_fst[i]->address & 0xFFFF);
      current_listview->tab_column[4].tab_value_txt[i] = strdup(buffer);

      /* Colonne 5 : Size */
      current_listview->tab_column[5].tab_value_num[i] = current_fsttable->tab_fst[i]->size;
      sprintf(buffer,"%08X",current_fsttable->tab_fst[i]->size);
      current_listview->tab_column[5].tab_value_txt[i] = strdup(buffer);
    }

  /** Transfert des donnes dans la List View via un Msg LVN_GETDISPINFO **/
  /** Ajout des Item dans le ListView **/
  listview_item.pszText   = LPSTR_TEXTCALLBACK;           /* Sends an LVN_GETDISPINFO message to fill the item */
  listview_item.mask      = LVIF_TEXT | LVIF_IMAGE | LVIF_STATE | LVIF_PARAM;
  listview_item.stateMask = 0;
  listview_item.iSubItem  = 0;
  listview_item.state     = 0;

  /** Initialize LVITEM members that are different for each item **/
  for(i=0; i<current_listview->nb_row; i++)
    {
      listview_item.iItem  = i;
      listview_item.iImage = -1;         /* Identifiant de l'image dans la liste */
      listview_item.lParam = i;          /* Identifiant renvoy lors du tri */
      result = ListView_InsertItem(current_listview->hwnd_listview,&listview_item);
    }
}


/******************************************************************************************************/
/*  BuildInternalsHandleListView() :  Construction d'une ListView pour les Handle (Used/Purged/Free). */
/******************************************************************************************************/
static void BuildInternalsHandleListView(struct list_view *current_listview, HFONT hFont, int idc_internals_listview, struct window_internals *window_internals)
{
  /** Init de la structure List_view **/
  current_listview->idc_listview = idc_internals_listview;

  /** Liste des Colonnes **/
  current_listview->nb_column = 0;

  /* Col 0 : Handle */
  current_listview->tab_column[0].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[0].align = LVCFMT_LEFT;
  current_listview->tab_column[0].length = 75;
  strcpy(current_listview->tab_column[0].title,"Handle");
  current_listview->nb_column++;

  /* Col 1 : Address */
  current_listview->tab_column[1].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[1].align = LVCFMT_LEFT;
  current_listview->tab_column[1].length = 80;
  strcpy(current_listview->tab_column[1].title,"Address");
  current_listview->nb_column++;

  /* Col 2 : Attributes */
  current_listview->tab_column[2].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[2].align = LVCFMT_LEFT;
  current_listview->tab_column[2].length = 90;
  strcpy(current_listview->tab_column[2].title,"Attributes");
  current_listview->nb_column++;

  /* Col 3 : User ID */
  current_listview->tab_column[3].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[3].align = LVCFMT_LEFT;
  current_listview->tab_column[3].length = 75;
  strcpy(current_listview->tab_column[3].title,"User ID");
  current_listview->nb_column++;

  /* Col 4 : Size */
  current_listview->tab_column[4].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[4].align = LVCFMT_LEFT;
  current_listview->tab_column[4].length = 65;
  strcpy(current_listview->tab_column[4].title,"Size");
  current_listview->nb_column++;

  /* Col 5 : Previous */
  current_listview->tab_column[5].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[5].align = LVCFMT_LEFT;
  current_listview->tab_column[5].length = 80;
  strcpy(current_listview->tab_column[5].title,"Previous");
  current_listview->nb_column++;

  /* Col 6 : Next */
  current_listview->tab_column[6].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[6].align = LVCFMT_LEFT;
  current_listview->tab_column[6].length = 65;
  strcpy(current_listview->tab_column[6].title,"Next");
  current_listview->nb_column++;

  /* Col 7 : Type */
  current_listview->tab_column[7].type = INTERNALS_COLUMN_TYPE_TXT;
  current_listview->tab_column[7].align = LVCFMT_LEFT;
  current_listview->tab_column[7].length = 105;
  strcpy(current_listview->tab_column[7].title,"Type");
  current_listview->nb_column++;

  /* Col 8 : File Name */
  current_listview->tab_column[8].type = INTERNALS_COLUMN_TYPE_TXT;
  current_listview->tab_column[8].align = LVCFMT_LEFT;
  current_listview->tab_column[8].length = 185;
  strcpy(current_listview->tab_column[8].title,"Information");
  current_listview->nb_column++;

  /** Cration du ListView **/
  BuildListView(window_internals->hwnd_internals,hFont,25,90,INTERNALS_LISTVIEW_LINE_COUNT,current_listview);
}


/**********************************************************************************************/
/*  FillInternalsHandleListView() :  Remplissage ListView pour les Handle (Used/Purged/Free). */
/**********************************************************************************************/
static void FillInternalsHandleListView(int type_handle)
{
  int i, result;
  LVITEM listview_item;
  struct s65816 *current_65816;
  struct list_view *current_listview;
  struct window_internals *window_internals;
  struct gs_handle *current_gs_handle;
  char buffer[BUFFER_SIZE];
  int nb_gs_handle;
  struct gs_handle *list_gs_handle;

  /* Rcupre la fentre Internals */
  my_Memory(MEMORY_GET_WINDOW_INTERNALS,&window_internals,NULL);
  if(window_internals == NULL)
    return;
  if(type_handle == TYPE_HANDLE_USED)
    current_listview = &window_internals->listview_used;
  else if(type_handle == TYPE_HANDLE_PURGED)
    current_listview = &window_internals->listview_purged;
  else if(type_handle == TYPE_HANDLE_FREE)
    current_listview = &window_internals->listview_free;

  /** Vide la List View **/
  ListView_DeleteAllItems(current_listview->hwnd_listview);

  /* Aucun Handle ? */
  my_Memory(MEMORY_GET_S65816,&current_65816,NULL);
  if(current_65816 == NULL)
    return;
  if(type_handle == TYPE_HANDLE_USED)
    {
      nb_gs_handle = current_65816->nb_used_handle;
      list_gs_handle = current_65816->list_used_handle;
    }
  else if(type_handle == TYPE_HANDLE_PURGED)
    {
      nb_gs_handle = current_65816->nb_purged_handle;
      list_gs_handle = current_65816->list_purged_handle;
    }
  else if(type_handle == TYPE_HANDLE_FREE)
    {
      nb_gs_handle = current_65816->nb_free_handle;
      list_gs_handle = current_65816->list_free_handle;
    }
  if(list_gs_handle == NULL)
    return;
  if(nb_gs_handle == 0)
    return;

  /** Remplissage avec les valeurs dans la structure mmoire **/
  current_listview->nb_row = nb_gs_handle;
  for(i=0; i<current_listview->nb_column; i++)
    {
      /* Allocation des tableaux */
      current_listview->tab_column[i].tab_value_num = (DWORD *) calloc(current_listview->nb_row,sizeof(DWORD));
      current_listview->tab_column[i].tab_value_txt = (char **) calloc(current_listview->nb_row,sizeof(char *));
    }
  /** Remplissage des tableaux **/
  for(i=0, current_gs_handle=list_gs_handle; i<current_listview->nb_row; i++, current_gs_handle=current_gs_handle->next)
    {
      /* Colonne 0 : Handle */
      current_listview->tab_column[0].tab_value_num[i] = current_gs_handle->handle;
      sprintf(buffer,"%02X/%04X",current_gs_handle->handle >> 16,current_gs_handle->handle & 0xFFFF);
      current_listview->tab_column[0].tab_value_txt[i] = strdup(buffer);

      /* Colonne 1 : Address */
      current_listview->tab_column[1].tab_value_num[i] = current_gs_handle->address;
      sprintf(buffer,"%02X/%04X",current_gs_handle->address >> 16,current_gs_handle->address & 0xFFFF);
      current_listview->tab_column[1].tab_value_txt[i] = strdup(buffer);

      /* Colonne 2 : Attributes */
      current_listview->tab_column[2].tab_value_num[i] = current_gs_handle->attributes;
      sprintf(buffer,"%04X",current_gs_handle->attributes);
      current_listview->tab_column[2].tab_value_txt[i] = strdup(buffer);

      /* Colonne 3 : User ID */
      current_listview->tab_column[3].tab_value_num[i] = current_gs_handle->id;
      sprintf(buffer,"%04X",current_gs_handle->id);
      current_listview->tab_column[3].tab_value_txt[i] = strdup(buffer);

      /* Colonne 4 : Size */
      current_listview->tab_column[4].tab_value_num[i] = current_gs_handle->size;
      sprintf(buffer,"%08X",current_gs_handle->size);
      current_listview->tab_column[4].tab_value_txt[i] = strdup(buffer);

      /* Colonne 5 : Previous Handle */
      current_listview->tab_column[5].tab_value_num[i] = current_gs_handle->prev_handle;
      sprintf(buffer,"%02X/%04X",current_gs_handle->prev_handle >> 16,current_gs_handle->prev_handle & 0xFFFF);
      current_listview->tab_column[5].tab_value_txt[i] = strdup(buffer);

      /* Colonne 6 : Next Handle */
      current_listview->tab_column[6].tab_value_num[i] = current_gs_handle->next_handle;
      sprintf(buffer,"%02X/%04X",current_gs_handle->next_handle >> 16,current_gs_handle->next_handle & 0xFFFF);
      current_listview->tab_column[6].tab_value_txt[i] = strdup(buffer);

      /* Colonne 7 : Type */
      current_listview->tab_column[7].tab_value_num[i] = current_gs_handle->id;
      GetHandleIDType(current_gs_handle->id,buffer);
      current_listview->tab_column[7].tab_value_txt[i] = strdup(buffer);

      /* Colonne 8 : Information */
      current_listview->tab_column[8].tab_value_num[i] = current_gs_handle->id;
      GetHandleInfo(current_gs_handle,buffer);
      current_listview->tab_column[8].tab_value_txt[i] = strdup(buffer);
    }

  /** Transfert des donnes dans la List View via un Msg LVN_GETDISPINFO **/
  /** Ajout des Item dans le ListView **/
  listview_item.pszText   = LPSTR_TEXTCALLBACK;           /* Sends an LVN_GETDISPINFO message to fill the item */
  listview_item.mask      = LVIF_TEXT | LVIF_IMAGE | LVIF_STATE | LVIF_PARAM;
  listview_item.stateMask = 0;
  listview_item.iSubItem  = 0;
  listview_item.state     = 0;

  /** Initialize LVITEM members that are different for each item **/
  for(i=0; i<current_listview->nb_row; i++)
    {
      listview_item.iItem  = i;
      listview_item.iImage = -1;         /* Identifiant de l'image dans la liste */
      listview_item.lParam = i;          /* Identifiant renvoy lors du tri */
      result = ListView_InsertItem(current_listview->hwnd_listview,&listview_item);
    }
}


/*****************************************************************/
/*  ShowOneTabControl() :  On va afficher les Controls d'un Tab. */
/*****************************************************************/
static void ShowOneTabControl(int tab_type, struct window_internals *window_internals)
{
  int i, nb_handle, size_allocated;
  int tab_control_interrupt[] = 
    {
      IDC_GLANCE_STATIC_NB_HANDLE,
      IDC_GLANCE_STATIC_NB_HANDLE_VAL,
      IDC_INTERNALS_LISTVIEW_INTERRUPT,
      0
    };
  int tab_control_quickdraw[] = 
    {
      IDC_GLANCE_STATIC_NB_HANDLE,
      IDC_GLANCE_STATIC_NB_HANDLE_VAL,
      IDC_INTERNALS_LISTVIEW_QUICKDRAW,
      0
    };
  int tab_control_file[] = 
    {
      IDC_GLANCE_STATIC_NB_HANDLE,
      IDC_GLANCE_STATIC_NB_HANDLE_VAL,
      IDC_INTERNALS_LISTVIEW_FILE,
      0
    };
  int tab_control_cda[] = 
    {
      IDC_GLANCE_STATIC_NB_HANDLE,
      IDC_GLANCE_STATIC_NB_HANDLE_VAL,
      IDC_INTERNALS_LISTVIEW_CDA,
      0
    };
  int tab_control_nda[] = 
    {
      IDC_GLANCE_STATIC_NB_HANDLE,
      IDC_GLANCE_STATIC_NB_HANDLE_VAL,
      IDC_INTERNALS_LISTVIEW_NDA,
      0
    };
  int tab_control_init[] = 
    {
      IDC_GLANCE_STATIC_NB_HANDLE,
      IDC_GLANCE_STATIC_NB_HANDLE_VAL,
      IDC_INTERNALS_LISTVIEW_INIT,
      0
    };
  int tab_control_driver[] = 
    {
      IDC_GLANCE_STATIC_NB_HANDLE,
      IDC_GLANCE_STATIC_NB_HANDLE_VAL,
      IDC_INTERNALS_LISTVIEW_DRIVER,
      0
    };
  int tab_control_fst[] = 
    {
      IDC_GLANCE_STATIC_NB_HANDLE,
      IDC_GLANCE_STATIC_NB_HANDLE_VAL,
      IDC_INTERNALS_LISTVIEW_FST,
      0
    };
  int tab_control_used[] = 
    {
      IDC_GLANCE_STATIC_NB_HANDLE,
      IDC_GLANCE_STATIC_NB_HANDLE_VAL,
      IDC_GLANCE_STATIC_NB_BYTE,
      IDC_GLANCE_STATIC_NB_BYTE_VAL,
      IDC_BUTTON_EXPORT_HANDLE_DATA,
      IDC_INTERNALS_LISTVIEW_USED,
      0
    };
  int tab_control_purged[] = 
    {
      IDC_GLANCE_STATIC_NB_HANDLE,
      IDC_GLANCE_STATIC_NB_HANDLE_VAL,
      IDC_GLANCE_STATIC_NB_BYTE,
      IDC_GLANCE_STATIC_NB_BYTE_VAL,
      IDC_BUTTON_EXPORT_HANDLE_DATA,
      IDC_INTERNALS_LISTVIEW_PURGED,
      0
    };
  int tab_control_free[] = 
    {
      IDC_GLANCE_STATIC_NB_HANDLE,
      IDC_GLANCE_STATIC_NB_HANDLE_VAL,
      IDC_GLANCE_STATIC_NB_BYTE,
      IDC_GLANCE_STATIC_NB_BYTE_VAL,
      IDC_BUTTON_EXPORT_HANDLE_DATA,
      IDC_INTERNALS_LISTVIEW_FREE,
      0
    };
  int tab_control_glance[] = 
    {
      IDC_GLANCE_STATIC_HANDLE,
      IDC_GLANCE_EDIT_HANDLE_BANK,
      IDC_GLANCE_STATIC_HANDLE_SEP,
      IDC_GLANCE_EDIT_HANDLE_ADDRESS,
      IDC_GLANCE_BUTTON_DISASSEMBLE,
      IDC_GLANCE_BUTTON_HANDLE_EXAMINE,
      IDC_GLANCE_GROUP_HANDLE_FIELDS,
      IDC_GLANCE_STATIC_GROUP,
      IDC_GLANCE_EDIT_GROUP,
      IDC_GLANCE_STATIC_ADDRESS,
      IDC_GLANCE_EDIT_ADDRESS,
      IDC_GLANCE_STATIC_ATTRIBUTES,
      IDC_GLANCE_EDIT_ATTRIBUTES,
      IDC_GLANCE_STATIC_ATTRIBUTES_INFO,
      IDC_GLANCE_STATIC_USERID,
      IDC_GLANCE_EDIT_USERID,
      IDC_GLANCE_STATIC_USERID_INFO,
      IDC_GLANCE_STATIC_SIZE,
      IDC_GLANCE_EDIT_SIZE,
      IDC_GLANCE_STATIC_SIZE_INFO,
      IDC_GLANCE_STATIC_PREVIOUS,
      IDC_GLANCE_EDIT_PREVIOUS,
      IDC_GLANCE_BUTTON_EXAMINE_PREV,
      IDC_GLANCE_STATIC_NEXT,
      IDC_GLANCE_EDIT_NEXT,
      IDC_GLANCE_BUTTON_EXAMINE_NEXT,
      IDC_GLANCE_GROUP_HANDLE_DATA,
      IDC_GLANCE_STATIC_DATA_OFFSET,
      IDC_GLANCE_EDIT_DATA_OFFSET,
      IDC_GLANCE_EDIT_DATA,
      IDC_GLANCE_STATIC_OFFSET_MAX,
      IDC_GLANCE_BUTTON_LEFT,
      IDC_GLANCE_BUTTON_RIGHT,
      IDC_GLANCE_BUTTON_UP,
      IDC_GLANCE_BUTTON_DOWN,
      IDC_GLANCE_BUTTON_PAGEUP,
      IDC_GLANCE_BUTTON_PAGEDOWN,
      IDC_GLANCE_BUTTON_SAVE_DATA,
      IDC_GLANCE_GROUP_HANDLE_INFORMATION,
      IDC_GLANCE_STATIC_SUMMARY,
      IDC_GLANCE_STATIC_FILEPATH,
      IDC_GLANCE_EDIT_SUMMARY,
      IDC_GLANCE_EDIT_FILEPATH,
      0};
  char buffer[1024];
  struct s65816 *current_65816;

  /* Rcupration du Engine */
  my_Memory(MEMORY_GET_S65816,&current_65816,NULL);

  /**************************/
  /** Masque Les controles **/
  if(tab_type != INTERNALS_TAB_INTERRUPT)
    {
      /* Onglet Interrupt */
      for(i=0; tab_control_interrupt[i] != 0; i++)
        ShowWindow(GetDlgItem(window_internals->hwnd_internals,tab_control_interrupt[i]),SW_HIDE);
    }
  if(tab_type != INTERNALS_TAB_QUICKDRAW)
    {
      /* Onglet Quickdraw */
      for(i=0; tab_control_quickdraw[i] != 0; i++)
        ShowWindow(GetDlgItem(window_internals->hwnd_internals,tab_control_quickdraw[i]),SW_HIDE);
    }
  if(tab_type != INTERNALS_TAB_FILE)
    {
      /* Onglet FILE */
      for(i=0; tab_control_file[i] != 0; i++)
        ShowWindow(GetDlgItem(window_internals->hwnd_internals,tab_control_file[i]),SW_HIDE);
    }
  if(tab_type != INTERNALS_TAB_CDA)
    {
      /* Onglet CDA */
      for(i=0; tab_control_cda[i] != 0; i++)
        ShowWindow(GetDlgItem(window_internals->hwnd_internals,tab_control_cda[i]),SW_HIDE);
    }
  if(tab_type != INTERNALS_TAB_NDA)
    {
      /* Onglet NDA */
      for(i=0; tab_control_nda[i] != 0; i++)
        ShowWindow(GetDlgItem(window_internals->hwnd_internals,tab_control_nda[i]),SW_HIDE);
    }
  if(tab_type != INTERNALS_TAB_INIT)
    {
      /* Onglet Init */
      for(i=0; tab_control_init[i] != 0; i++)
        ShowWindow(GetDlgItem(window_internals->hwnd_internals,tab_control_init[i]),SW_HIDE);
    }
  if(tab_type != INTERNALS_TAB_DRIVER)
    {
      /* Onglet Driver */
      for(i=0; tab_control_driver[i] != 0; i++)
        ShowWindow(GetDlgItem(window_internals->hwnd_internals,tab_control_driver[i]),SW_HIDE);
    }
  if(tab_type != INTERNALS_TAB_FST)
    {
      /* Onglet FST */
      for(i=0; tab_control_fst[i] != 0; i++)
        ShowWindow(GetDlgItem(window_internals->hwnd_internals,tab_control_fst[i]),SW_HIDE);
    }
  if(tab_type != INTERNALS_TAB_USED)
    {
      /* Onglet Used */
      for(i=0; tab_control_used[i] != 0; i++)
        ShowWindow(GetDlgItem(window_internals->hwnd_internals,tab_control_used[i]),SW_HIDE);
    }
  if(tab_type != INTERNALS_TAB_PURGED)
    {
      /* Onglet Purged */  
      for(i=0; tab_control_purged[i] != 0; i++)
        ShowWindow(GetDlgItem(window_internals->hwnd_internals,tab_control_purged[i]),SW_HIDE);
    }
  if(tab_type != INTERNALS_TAB_FREE)
    {
      /* Onglet Free */
      for(i=0; tab_control_free[i] != 0; i++)
        ShowWindow(GetDlgItem(window_internals->hwnd_internals,tab_control_free[i]),SW_HIDE);
    }
  if(tab_type != INTERNALS_TAB_GLANCE)
    {
      /* Onglet Glance */
      for(i=0; tab_control_glance[i] != 0; i++)
        ShowWindow(GetDlgItem(window_internals->hwnd_internals,tab_control_glance[i]),SW_HIDE);
    }

  /***************************/
  /** Affiche Les controles **/
  if(tab_type == INTERNALS_TAB_INTERRUPT)
    {
      /* Affiche Interrupt */
      for(i=0; tab_control_interrupt[i] != 0; i++)
        ShowWindow(GetDlgItem(window_internals->hwnd_internals,tab_control_interrupt[i]),SW_SHOW);

      /* # Interrupt Vectors */
      SetWindowText(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_STATIC_NB_HANDLE),"Number of Vectors :");
      sprintf(buffer,"%d",current_65816->current_interrupt_vector->nb_interrupt_vector);
      SetWindowText(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_STATIC_NB_HANDLE_VAL),buffer);
    }
  else if(tab_type == INTERNALS_TAB_QUICKDRAW)
    {
      /* Affiche Quickdraw */
      for(i=0; tab_control_quickdraw[i] != 0; i++)
        ShowWindow(GetDlgItem(window_internals->hwnd_internals,tab_control_quickdraw[i]),SW_SHOW);

      /* # Quickdraw Vectors */
      SetWindowText(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_STATIC_NB_HANDLE),"Number of Vectors :");
      sprintf(buffer,"%d",current_65816->current_quickdraw_vector->nb_quickdraw_vector);
      SetWindowText(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_STATIC_NB_HANDLE_VAL),buffer);
    }
  else if(tab_type == INTERNALS_TAB_FILE)
    {
      /* Affiche file */
      for(i=0; tab_control_file[i] != 0; i++)
        ShowWindow(GetDlgItem(window_internals->hwnd_internals,tab_control_file[i]),SW_SHOW);

      /* # FILE */
      SetWindowText(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_STATIC_NB_HANDLE),"Number of Files :");
      sprintf(buffer,"%d",current_65816->current_file_segment->nb_segment_table);
      SetWindowText(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_STATIC_NB_HANDLE_VAL),buffer);
    }
  else if(tab_type == INTERNALS_TAB_CDA)
    {
      /* Affiche cda */
      for(i=0; tab_control_cda[i] != 0; i++)
        ShowWindow(GetDlgItem(window_internals->hwnd_internals,tab_control_cda[i]),SW_SHOW);

      /* # CDA */
      SetWindowText(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_STATIC_NB_HANDLE),"Number of CDAs :");
      sprintf(buffer,"%d",current_65816->current_cda->nb_cda);
      SetWindowText(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_STATIC_NB_HANDLE_VAL),buffer);
    }
  else if(tab_type == INTERNALS_TAB_NDA)
    {
      /* Affiche nda */
      for(i=0; tab_control_nda[i] != 0; i++)
        ShowWindow(GetDlgItem(window_internals->hwnd_internals,tab_control_nda[i]),SW_SHOW);

      /* # NDA */
      SetWindowText(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_STATIC_NB_HANDLE),"Number of NDAs :");
      sprintf(buffer,"%d",(current_65816->current_nda == NULL) ? 0 : current_65816->current_nda->nb_nda);
      SetWindowText(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_STATIC_NB_HANDLE_VAL),buffer);
    }
  else if(tab_type == INTERNALS_TAB_INIT)
    {
      /* Affiche init */
      for(i=0; tab_control_init[i] != 0; i++)
        ShowWindow(GetDlgItem(window_internals->hwnd_internals,tab_control_init[i]),SW_SHOW);

      /* # Init */
      SetWindowText(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_STATIC_NB_HANDLE),"Number of Inits :");
      sprintf(buffer,"%d",current_65816->current_init->nb_init);
      SetWindowText(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_STATIC_NB_HANDLE_VAL),buffer);
    }
  else if(tab_type == INTERNALS_TAB_DRIVER)
    {
      /* Affiche driver */
      for(i=0; tab_control_driver[i] != 0; i++)
        ShowWindow(GetDlgItem(window_internals->hwnd_internals,tab_control_driver[i]),SW_SHOW);

      /* # Driver */
      SetWindowText(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_STATIC_NB_HANDLE),"Number of Drivers :");
      sprintf(buffer,"%d",current_65816->current_driver->nb_driver);
      SetWindowText(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_STATIC_NB_HANDLE_VAL),buffer);
    }
  else if(tab_type == INTERNALS_TAB_FST)
    {
      /* Affiche Fst */
      for(i=0; tab_control_fst[i] != 0; i++)
        ShowWindow(GetDlgItem(window_internals->hwnd_internals,tab_control_fst[i]),SW_SHOW);

      /* # Fst */
      SetWindowText(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_STATIC_NB_HANDLE),"Number of FSTs :");
      sprintf(buffer,"%d",current_65816->current_fst->nb_fst);
      SetWindowText(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_STATIC_NB_HANDLE_VAL),buffer);
    }
  else if(tab_type == INTERNALS_TAB_USED)
    {
      /* Affiche Used */
      for(i=0; tab_control_used[i] != 0; i++)
        ShowWindow(GetDlgItem(window_internals->hwnd_internals,tab_control_used[i]),SW_SHOW);

      /* Handle List Info */
      SetWindowText(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_STATIC_NB_HANDLE),"Number of Handles :");
      GetHandleListInfo(current_65816->list_used_handle,&nb_handle,&size_allocated);
      sprintf(buffer,"%d",nb_handle);
      SetWindowText(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_STATIC_NB_HANDLE_VAL),buffer);
      sprintf(buffer,"%d",size_allocated);
      SetWindowText(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_STATIC_NB_BYTE_VAL),buffer);
    }
  else if(tab_type == INTERNALS_TAB_PURGED)
    {
      /* Affiche Purged */
      for(i=0; tab_control_purged[i] != 0; i++)
        ShowWindow(GetDlgItem(window_internals->hwnd_internals,tab_control_purged[i]),SW_SHOW);

      /* Handle List Info */
      SetWindowText(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_STATIC_NB_HANDLE),"Number of Handles :");
      GetHandleListInfo(current_65816->list_purged_handle,&nb_handle,&size_allocated);
      sprintf(buffer,"%d",nb_handle);
      SetWindowText(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_STATIC_NB_HANDLE_VAL),buffer);
      sprintf(buffer,"%d",size_allocated);
      SetWindowText(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_STATIC_NB_BYTE_VAL),buffer);
    }
  else if(tab_type == INTERNALS_TAB_FREE)
    {
      /* Affiche Free */
      for(i=0; tab_control_free[i] != 0; i++)
        ShowWindow(GetDlgItem(window_internals->hwnd_internals,tab_control_free[i]),SW_SHOW);

      /* Handle List Info */
      SetWindowText(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_STATIC_NB_HANDLE),"Number of Handles :");
      GetHandleListInfo(current_65816->list_free_handle,&nb_handle,&size_allocated);
      sprintf(buffer,"%d",nb_handle);
      SetWindowText(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_STATIC_NB_HANDLE_VAL),buffer);
      sprintf(buffer,"%d",size_allocated);
      SetWindowText(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_STATIC_NB_BYTE_VAL),buffer);
    }
  else if(tab_type == INTERNALS_TAB_GLANCE)
    {
      /* Affiche Glance */
      for(i=0; tab_control_glance[i] != 0; i++)
        ShowWindow(GetDlgItem(window_internals->hwnd_internals,tab_control_glance[i]),SW_SHOW);
    }
}


/************************************************************************************************/
/*  subEditInternalsProcControl() :  Rcupre les vnements qui arrivent dans le Control Edit. */
/************************************************************************************************/
LRESULT CALLBACK subEditInternalsProcControl(HWND hwnd_control, UINT msg, WPARAM wParam, LPARAM lParam)
{
  int index;
  struct window_internals *window_internals;
  my_Memory(MEMORY_GET_WINDOW_INTERNALS,&window_internals,NULL);

  /* Rien  faire */
  if(window_internals == NULL)
    return(0);

  /** Identifie le Control **/
  /* Src */
  if(window_internals->hwnd_edit_data == hwnd_control)
    index = 1;
  else
    return(0);      /* Pas trouv */

  /** Dtecte l'vnement de la souris dans l'Edit **/
  if(index == 1)
    return(CallWindowProc(window_internals->oldEditProcInternalsData,hwnd_control,msg,wParam,lParam)); 
         
  /* OK */
  return(0);
} 


/***********************************************************************/
/*  UpdateInternalsGlanceTab() :  MAJ du contenu de l'onglet [Glance]. */
/***********************************************************************/
static void UpdateInternalsGlanceTab(struct window_internals *window_internals, int which_handle, int reset_data_offset)
{
  int i, length, has_error;
  int handle_bank, handle_address, handle_full_address, data_offset;
  struct s65816 *current_65816;
  char buffer[1024];
  char buffer_error[1024];

  /* Rcupration du Engine */
  my_Memory(MEMORY_GET_S65816,&current_65816,NULL);

  /* Rcupre la valeur du Handle */
  if(which_handle == 0)
    {
      length = GetDlgItemText(window_internals->hwnd_internals,IDC_GLANCE_EDIT_HANDLE_BANK,buffer,1024);
      sscanf(buffer,"%X",&handle_bank);
      length = GetDlgItemText(window_internals->hwnd_internals,IDC_GLANCE_EDIT_HANDLE_ADDRESS,buffer,1024);
      sscanf(buffer,"%X",&handle_address);
    }
  else if(which_handle == -1)   /* Previous */
    {
      length = GetDlgItemText(window_internals->hwnd_internals,IDC_GLANCE_EDIT_PREVIOUS,buffer,1024);
      if(strlen(buffer) == strlen("E1/2000"))
        sscanf(buffer,"%02X/%04X",&handle_bank,&handle_address);
      else
        {
          handle_bank = 0;
          handle_address = 0;
        }
    }
  else if(which_handle == 1)    /* Next */
    {
      length = GetDlgItemText(window_internals->hwnd_internals,IDC_GLANCE_EDIT_NEXT,buffer,1024);
      if(strlen(buffer) == strlen("E1/2000"))
        sscanf(buffer,"%02X/%04X",&handle_bank,&handle_address);
      else
        {
          handle_bank = 0;
          handle_address = 0;
        }
    }
  handle_full_address = (handle_bank << 16) | handle_address;
  if(reset_data_offset == 1)
    data_offset = 0;
  else
    {
      length = GetDlgItemText(window_internals->hwnd_internals,IDC_GLANCE_EDIT_DATA_OFFSET,buffer,1024);
      sscanf(buffer,"%X",&data_offset);
    }

  /* Replace les valeurs dans les Control pour montrer ce qui a t compris */
  sprintf(buffer,"%02X",(handle_full_address >> 16) & 0xFF);
  SetDlgItemText(window_internals->hwnd_internals,IDC_GLANCE_EDIT_HANDLE_BANK,buffer);
  sprintf(buffer,"%04X",handle_full_address & 0xFFFF);
  SetDlgItemText(window_internals->hwnd_internals,IDC_GLANCE_EDIT_HANDLE_ADDRESS,buffer);
  sprintf(buffer,"%06X",data_offset);
  SetDlgItemText(window_internals->hwnd_internals,IDC_GLANCE_EDIT_DATA_OFFSET,buffer);

  /** Charge le Handle **/
  if(window_internals->current_handle)
    free(window_internals->current_handle);
  window_internals->current_handle = GetMemoryHandle(handle_full_address,current_65816->current_mmu);

  /* Nettoyage des informations du Handle */
  if(window_internals->current_handle == NULL)
    {
      /* RAZ */
      SetDlgItemText(window_internals->hwnd_internals,IDC_GLANCE_EDIT_ADDRESS,"");
      SetDlgItemText(window_internals->hwnd_internals,IDC_GLANCE_EDIT_ATTRIBUTES,"");
      SetDlgItemText(window_internals->hwnd_internals,IDC_GLANCE_STATIC_ATTRIBUTES_INFO,"");
      SetDlgItemText(window_internals->hwnd_internals,IDC_GLANCE_EDIT_USERID,"");
      SetDlgItemText(window_internals->hwnd_internals,IDC_GLANCE_STATIC_USERID_INFO,"");
      SetDlgItemText(window_internals->hwnd_internals,IDC_GLANCE_EDIT_SIZE,"");
      SetDlgItemText(window_internals->hwnd_internals,IDC_GLANCE_STATIC_SIZE_INFO,"");
      SetDlgItemText(window_internals->hwnd_internals,IDC_GLANCE_EDIT_PREVIOUS,"");
      SetDlgItemText(window_internals->hwnd_internals,IDC_GLANCE_EDIT_NEXT,"");

      SetDlgItemText(window_internals->hwnd_internals,IDC_GLANCE_EDIT_SUMMARY,buffer);
      SetDlgItemText(window_internals->hwnd_internals,IDC_GLANCE_EDIT_FILEPATH,buffer);

      /* Dsactive le bouton Dsassemble */
      EnableWindow(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_BUTTON_DISASSEMBLE),FALSE);

      /* Dsactive les boutons Examine */
      EnableWindow(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_BUTTON_EXAMINE_PREV),FALSE);
      EnableWindow(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_BUTTON_EXAMINE_NEXT),FALSE);

      /* Offset Max */
      EnableWindow(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_STATIC_OFFSET_MAX),FALSE);

      /* Dsactive les boutons Data */
      EnableWindow(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_BUTTON_LEFT),FALSE);
      EnableWindow(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_BUTTON_RIGHT),FALSE);
      EnableWindow(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_BUTTON_UP),FALSE);
      EnableWindow(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_BUTTON_DOWN),FALSE);
      EnableWindow(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_BUTTON_PAGEUP),FALSE);
      EnableWindow(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_BUTTON_PAGEDOWN),FALSE);
      EnableWindow(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_BUTTON_SAVE_DATA),FALSE);

      /* RAZ de l'onglet [Glance] Data */
      RefreshInternalsGlanceWindowData(window_internals);
      return;
    }

  /** Affichage des informations du Handle (mme si invalide) **/
  /* Group */
  GetHandleGroup(window_internals->current_handle->handle,buffer);
  SetDlgItemText(window_internals->hwnd_internals,IDC_GLANCE_EDIT_GROUP,buffer);

  /* Address */
  sprintf(buffer,"%02X/%04X",(window_internals->current_handle->address >> 16) & 0xFF,window_internals->current_handle->address & 0xFFFF);
  SetDlgItemText(window_internals->hwnd_internals,IDC_GLANCE_EDIT_ADDRESS,buffer);

  /* Attributes */
  sprintf(buffer,"%04X",window_internals->current_handle->attributes);
  SetDlgItemText(window_internals->hwnd_internals,IDC_GLANCE_EDIT_ATTRIBUTES,buffer);

  /* Attributes Info */
  GetHandleAttributes(window_internals->current_handle->attributes,buffer);
  SetDlgItemText(window_internals->hwnd_internals,IDC_GLANCE_STATIC_ATTRIBUTES_INFO,buffer);

  /* User ID */
  sprintf(buffer,"%04X",window_internals->current_handle->id);
  SetDlgItemText(window_internals->hwnd_internals,IDC_GLANCE_EDIT_USERID,buffer);

  /* User Info */
  strcpy(buffer,"Type=");
  GetHandleIDType(window_internals->current_handle->id,&buffer[strlen(buffer)]);
  sprintf(&buffer[strlen(buffer)],", auxID=%1X",(window_internals->current_handle->id >>8)&0x0F);
  sprintf(&buffer[strlen(buffer)],", mainID=%02X",(window_internals->current_handle->id) & 0xFF);
  SetDlgItemText(window_internals->hwnd_internals,IDC_GLANCE_STATIC_USERID_INFO,buffer);

  /* Size */
  sprintf(buffer,"%08X",window_internals->current_handle->size);
  SetDlgItemText(window_internals->hwnd_internals,IDC_GLANCE_EDIT_SIZE,buffer);

  /* Size Info */
  sprintf(buffer,"%d bytes",window_internals->current_handle->size);
  SetDlgItemText(window_internals->hwnd_internals,IDC_GLANCE_STATIC_SIZE_INFO,buffer);

  /* Offset Max */
  sprintf(buffer,"/  %06X",window_internals->current_handle->size);
  SetDlgItemText(window_internals->hwnd_internals,IDC_GLANCE_STATIC_OFFSET_MAX,buffer);

  /* Prev Handle */
  sprintf(buffer,"%02X/%04X",(window_internals->current_handle->prev_handle >> 16) & 0xFF,window_internals->current_handle->prev_handle & 0xFFFF);
  SetDlgItemText(window_internals->hwnd_internals,IDC_GLANCE_EDIT_PREVIOUS,buffer);

  /* Next Handle */
  sprintf(buffer,"%02X/%04X",(window_internals->current_handle->next_handle >> 16) & 0xFF,window_internals->current_handle->next_handle & 0xFFFF);
  SetDlgItemText(window_internals->hwnd_internals,IDC_GLANCE_EDIT_NEXT,buffer);

  /* Summary */
  GetHandleInfo(window_internals->current_handle,buffer);
  SetDlgItemText(window_internals->hwnd_internals,IDC_GLANCE_EDIT_SUMMARY,buffer);

  /* File Path */
  SetDlgItemText(window_internals->hwnd_internals,IDC_GLANCE_EDIT_FILEPATH,"");
  for(i=0; i<current_65816->current_file_segment->nb_segment_table; i++)
    if(current_65816->current_file_segment->tab_file_segment[i]->user_id == window_internals->current_handle->id)
      {
        SetDlgItemText(window_internals->hwnd_internals,IDC_GLANCE_EDIT_FILEPATH,current_65816->current_file_segment->tab_file_segment[i]->file_path);
        break;
      }

  /* Active le bouton Dsassemble */
  EnableWindow(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_BUTTON_DISASSEMBLE),TRUE);

  /* Active / Dsactive les boutons Examine */
  EnableWindow(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_BUTTON_EXAMINE_PREV),window_internals->current_handle->prev_handle == 0 ? FALSE : TRUE);
  EnableWindow(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_BUTTON_EXAMINE_NEXT),window_internals->current_handle->next_handle == 0 ? FALSE : TRUE);

  /* Vrifie les donnes du Handle */
  has_error = CheckMemoryHandle(window_internals->current_handle,buffer_error);

  /** Libration des donnes du Handle **/
  if(window_internals->handle_data)
    free(window_internals->handle_data);
  window_internals->handle_data = NULL;

  /** Charge les donnes du Handle (on va grer le dernier Handle avec une longueur FF144000) **/
  if(window_internals->current_handle->is_valid == 1 && window_internals->current_handle->size < GS_MAX_ALLOC_SIZE)
    window_internals->handle_data = LoadHandleData(window_internals->current_handle,current_65816->current_mmu);

  /** Donnes du Handle **/
  if(window_internals->handle_data != NULL)
    {
      /* Active / Dsactive les boutons Data */
      EnableWindow(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_BUTTON_LEFT),window_internals->current_handle->size == 0 ? FALSE : TRUE);
      EnableWindow(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_BUTTON_RIGHT),window_internals->current_handle->size == 0 ? FALSE : TRUE);
      EnableWindow(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_BUTTON_UP),window_internals->current_handle->size == 0 ? FALSE : TRUE);
      EnableWindow(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_BUTTON_DOWN),window_internals->current_handle->size == 0 ? FALSE : TRUE);
      EnableWindow(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_BUTTON_PAGEUP),window_internals->current_handle->size == 0 ? FALSE : TRUE);
      EnableWindow(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_BUTTON_PAGEDOWN),window_internals->current_handle->size == 0 ? FALSE : TRUE);
      EnableWindow(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_BUTTON_SAVE_DATA),window_internals->current_handle->size == 0 ? FALSE : TRUE);

      /* MAJ de l'onglet [Glance] Data */
      RefreshInternalsGlanceWindowData(window_internals);
    }
  else
    {
      /* Offset */
      EnableWindow(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_EDIT_DATA_OFFSET),FALSE);

      /* Offset Max */
      EnableWindow(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_STATIC_OFFSET_MAX),FALSE);

      /* Dsactive les boutons Data */
      EnableWindow(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_BUTTON_LEFT),FALSE);
      EnableWindow(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_BUTTON_RIGHT),FALSE);
      EnableWindow(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_BUTTON_UP),FALSE);
      EnableWindow(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_BUTTON_DOWN),FALSE);
      EnableWindow(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_BUTTON_PAGEUP),FALSE);
      EnableWindow(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_BUTTON_PAGEDOWN),FALSE);
      EnableWindow(GetDlgItem(window_internals->hwnd_internals,IDC_GLANCE_BUTTON_SAVE_DATA),FALSE);

      /* RAZ de l'onglet [Glance] Data */
      RefreshInternalsGlanceWindowData(window_internals);
    }
}


/****************************************************************************************/
/*  UpdateInternalsGlanceOffset() :  Dplace l'offset pour la fentre Internals Glance. */
/****************************************************************************************/
static void UpdateInternalsGlanceOffset(struct window_internals *window_internals, int delta)
{
  int new_offset;
  char buffer[BUFFER_SIZE];

  /* Pas de Handle ? */
  if(window_internals->current_handle == NULL)
    return;
  /* Pas de Data ? */
  if(window_internals->handle_data == NULL)
    return;

  /* On vrifie les limites du dplacement */
  new_offset = window_internals->data_offset + delta;
  if(new_offset < 0)
    new_offset = 0;
  if(new_offset >=  (int) window_internals->current_handle->size)
    return;

  /* Replace les valeurs dans les Control pour montrer ce qui a t compris */
  sprintf(buffer,"%06X",new_offset);
  SetDlgItemText(window_internals->hwnd_internals,IDC_GLANCE_EDIT_DATA_OFFSET,buffer);
  window_internals->data_offset = new_offset;

  /* MAJ de l'onglet [Glance] */
  RefreshInternalsGlanceWindowData(window_internals);
}


/*********************************************************************************************/
/*  UpdateInternalsRefreshMode() :  MAJ du mode de rafraichissement de la fentre Internals. */
/*********************************************************************************************/
static void UpdateInternalsRefreshMode(struct window_internals *window_internals, int refresh_mode)
{
  struct notification *current_notification;

  if(refresh_mode == INTERNALS_REFRESH_AUTOMATIC)
    {
      /* Visuel */
      window_internals->refresh_mode = INTERNALS_REFRESH_AUTOMATIC;
      SendDlgItemMessage(window_internals->hwnd_internals,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_internals->hwnd_internals,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,0,0);

      /* Cration d'une Notification */
      current_notification = (struct notification *) calloc(1,sizeof(struct notification));
      if(current_notification == NULL)
        return;
      current_notification->hwnd_window = window_internals->hwnd_internals;
      current_notification->message = HM_INTERNALS_REFRESHWINDOW;
      current_notification->type = NOTIFICATION_TYPE_VIEW;
      current_notification->sub_type = NOTIFICATION_SUBTYPE_INTERNALS;

      /* Ajout de la notification */
      window_internals->current_notification = current_notification;
      my_Notification(NOTIFICATION_ADD,current_notification,NULL);
    }
  else
    {
      /* Visuel */
      window_internals->refresh_mode = INTERNALS_REFRESH_MANUAL;
      SendDlgItemMessage(window_internals->hwnd_internals,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_internals->hwnd_internals,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,1,0);

      /* Supprime la Notification */
      my_Notification(NOTIFICATION_REMOVE,window_internals->current_notification,NULL);
      window_internals->current_notification = NULL;
    }
}


/***************************************************************************************/
/*  RefreshInternalsGlanceWindowData() :  MAJ du contenu de l'onglet Internals Glance. */
/***************************************************************************************/
static void RefreshInternalsGlanceWindowData(struct window_internals *window_internals)
{
  int length;
  int data_offset;
  char buffer[1024];

  /* Rcupre la valeur du Handle Data offset */
  length = GetDlgItemText(window_internals->hwnd_internals,IDC_GLANCE_EDIT_DATA_OFFSET,buffer,1024);
  sscanf(buffer,"%X",&data_offset);
  window_internals->data_offset = data_offset;

  /* Replace la valeur dans les Control pour montrer ce qui a t compris */
  sprintf(buffer,"%06X",data_offset);
  SetDlgItemText(window_internals->hwnd_internals,IDC_GLANCE_EDIT_DATA_OFFSET,buffer);
    
  /** Cration de la Zone Text Edit  partir des nouvelles donnes **/
  BuildInternalsFromHandleData(window_internals);

  /* Rafraichit la fentre */
  UpdateWindow(window_internals->hwnd_internals);
}


/*****************************************************************************************************************/
/*  BuildInternalsFromHandleData() :  Construire les donnes hexadcimal  afficher  partir des donnes brutes. */
/*****************************************************************************************************************/
static void BuildInternalsFromHandleData(struct window_internals *window_internals)
{
  int i, j, offset, offset_text, nb_byte_to_process, address_data, offset_data;

  /* Rien  afficher */
  if(window_internals->current_handle == NULL || window_internals->handle_data == NULL)
    {
      /* Efface le contenu de la fentre */
      SetWindowText(window_internals->hwnd_edit_data,"");
      return;
    }

  /* Adresse des donnes */
  address_data = window_internals->current_handle->address;

  /* Offset des donnes */
  offset_data = window_internals->data_offset;
  if(offset_data > (int) window_internals->current_handle->size)
    offset_data = window_internals->current_handle->size - 1;

  /* Taille des donnes */
  nb_byte_to_process = window_internals->current_handle->size - offset_data;
  if(nb_byte_to_process > 256)
    nb_byte_to_process = 256;

  /* Rcupre les donnes */
  memcpy(window_internals->memoryarea_data,&window_internals->handle_data[offset_data],nb_byte_to_process);

  /*** Transformer les donnes BYTE en Hexa ***/
  for(i=0,offset=0,offset_text=0; i<16; i++,offset+=16)
    {
      /* Donnes valides ? */
      if(i*16 >= nb_byte_to_process)
        continue;

      /** Adresse **/
      sprintf(&window_internals->text_data_hexadecimal[offset_text],"%02X/%04X   ",(address_data+offset_data+offset)>>16,(address_data+offset_data+offset) & 0xFFFF);
      offset_text += (int) strlen("00/0000   ");

      /** Data Offset **/
      sprintf(&window_internals->text_data_hexadecimal[offset_text],"%06X   ",offset_data+offset);
      offset_text += (int) strlen("000000   ");

      /** Hexa **/
      for(j=0; j<16; j++)
        {
          if(i*16+j < nb_byte_to_process)
            sprintf(&window_internals->text_data_hexadecimal[offset_text],"%02X",window_internals->memoryarea_data[offset+j]);
          else
            strcpy(&window_internals->text_data_hexadecimal[offset_text],"  ");
          offset_text += 2;

          /* Sparateur */
          if(j < (16-1))
            {
              window_internals->text_data_hexadecimal[offset_text] = ((j > 0 && (((j+1)%8)==0)) && (i*16+j < nb_byte_to_process)) ? '.' : ' ';
              offset_text++;
            }
        }

      /* Sparateur */
      strcpy(&window_internals->text_data_hexadecimal[offset_text],"  ");
      offset_text += 2;

      /** Ascii **/
      for(j=0; j<16; j++)
        {
          if(i*16+j < nb_byte_to_process)
            window_internals->text_data_hexadecimal[offset_text] = GETASCIICODE(window_internals->memoryarea_data[offset+j]);
          else
            window_internals->text_data_hexadecimal[offset_text] = ' ';
          offset_text++;
        }

      /* Fin de ligne */
      strcpy(&window_internals->text_data_hexadecimal[offset_text],"\r\n");
      offset_text += 2;
    }
    
  /** Efface le contenu de la fentre **/
  SetWindowText(window_internals->hwnd_edit_data,"");

  /** Afficher le rsultat dans le control **/
  SetWindowText(window_internals->hwnd_edit_data,window_internals->text_data_hexadecimal);
}


/**************************************************************************************************/
/*  SaveInternalsGlanceData() :  On va enregistrer les donnes du Handle dans un fichier binaire. */
/**************************************************************************************************/
static void SaveInternalsGlanceData(struct window_internals *window_internals)
{
  BOOL is_OK;
  struct window_main *window_main;
  char file_path[1024];
  char buffer_error[1024];

  /* Rcupre la fentre Main */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);

  /* Rien  faire */
  if(window_internals->current_handle == NULL)
    return;
  if(window_internals->current_handle->size == 0 || window_internals->handle_data == NULL)
    return;

  /* Handle Data File Name */
  sprintf(file_path,"Handle_%02X_%4X_Data_%02X_%4X.bin",
          window_internals->current_handle->handle>>16,window_internals->current_handle->handle&0xFFFF,
          window_internals->current_handle->address>>16,window_internals->current_handle->address&0xFFFF);

  /* Demande un chemin pour le fichier */
  is_OK = GetSaveFilePath(window_main->hwnd_main,"Select Data file","All Files (*.*)\0*.*\0\0",file_path);
  if(is_OK = FALSE)
    return;

  /* Cration du fichier */
  SaveBinaryFile(file_path,window_internals->current_handle->size,window_internals->handle_data,buffer_error);
}


/**************************************************************************************************/
/*  ExportInternalsGlanceHandle() :  On va exporter les donnes des Handle dans un fichier texte. */
/**************************************************************************************************/
static void ExportInternalsGlanceHandle(struct window_internals *window_internals)
{
  BOOL is_OK;
  FILE *fd;
  DWORD address_data;
  int j, offset, offset_text, offset_data, nb_byte_to_process;
  struct window_main *window_main;
  struct gs_handle *first_handle;
  struct gs_handle *current_handle;
  struct s65816 *current_65816;
  unsigned char *data;
  char file_path[1024];
  char buffer[1024];

  /* Rcupre la fentre Main */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);

  /* Rcupration du Engine */
  my_Memory(MEMORY_GET_S65816,&current_65816,NULL);

  /* Handle List File Name */
  if(window_internals->tab_index == 1)
    {
      sprintf(file_path,"Handle_Used_List.txt");
      first_handle = current_65816->list_used_handle;
    }
  else if(window_internals->tab_index == 2)
    {
      sprintf(file_path,"Handle_Purged_List.txt");
      first_handle = current_65816->list_purged_handle;
    }
  else if(window_internals->tab_index == 3)
    {
      sprintf(file_path,"Handle_Free_List.txt");
      first_handle = current_65816->list_free_handle;
    }

  /* Demande un chemin pour le fichier */
  is_OK = GetSaveFilePath(window_main->hwnd_main,"Select Handle Data file","All Files (*.txt)\0*.txt\0\0",file_path);
  if(is_OK = FALSE)
    return;

  /* Cration du fichier */
  fd = fopen(file_path,"w");
  if(fd == NULL)
    {
      MessageBox(window_main->hwnd_main,"Impossible to create File","Error",MB_OK|MB_ICONWARNING);
      return;
    }
  
  /** Passe en revue tous les Handles **/
  for(current_handle = first_handle; current_handle; current_handle=current_handle->next)
    {
      /* Sparation */
      fprintf(fd,"------------------------------------------------------------------------------------\n\n");

      /* Information du Handle */
      fprintf(fd,"Handle          : %02X/%04X\n",current_handle->handle >> 16,current_handle->handle & 0xFFFF);
      fprintf(fd,"   Address      : %02X/%04X\n",current_handle->address >> 16,current_handle->address & 0xFFFF);
      GetHandleAttributes(current_handle->attributes,buffer);
      fprintf(fd,"   Attributes   : %04X       %s\n",current_handle->attributes,buffer);
      strcpy(buffer,"Type=");
      GetHandleIDType(current_handle->id,&buffer[strlen(buffer)]);
      sprintf(&buffer[strlen(buffer)],", auxID=%1X",(current_handle->id >>8)&0x0F);
      sprintf(&buffer[strlen(buffer)],", mainID=%02X",(current_handle->id) & 0xFF);
      fprintf(fd,"   User ID      : %04X       %s\n",current_handle->id,buffer);
      fprintf(fd,"   Size         : %08X   %d bytes\n",current_handle->size,current_handle->size);
      fprintf(fd,"   Prev Handle  : %02X/%04X\n",current_handle->prev_handle >> 16,current_handle->prev_handle & 0xFFFF);
      fprintf(fd,"   Next Handle  : %02X/%04X\n\n",current_handle->next_handle >> 16,current_handle->next_handle & 0xFFFF);

      /* Informations Complmentaires */
      GetHandleInfo(current_handle,buffer);
      fprintf(fd,"   Information  : %s\n\n",buffer);

      /** Donnes du Handle **/
      fprintf(fd,"Data\n");
      if(current_handle->size == 0)
        continue;
      if(current_handle->is_valid == 0)
        continue;
      data = LoadHandleData(current_handle,current_65816->current_mmu);
      if(data == NULL)
        continue;

      /** Dump les donnes en Hexa Ascii **/
      address_data = current_handle->address;
      offset_data = 0;
      for(offset=0; offset<(int)current_handle->size; offset+=16)
        {
          /* Nombre de Bytes dans cette ligne */
          nb_byte_to_process = ((offset+16) < (int) current_handle->size) ? 16 : (current_handle->size - offset);

          /** Adresse **/
          offset_text = 0;
          sprintf(&buffer[offset_text],"%02X/%04X   ",(address_data+offset_data+offset)>>16,(address_data+offset_data+offset) & 0xFFFF);
          offset_text += (int) strlen("00/0000   ");

          /** Data Offset **/
          sprintf(&buffer[offset_text],"%06X   ",offset_data+offset);
          offset_text += (int) strlen("000000   ");

          /** Hexa **/
          for(j=0; j<16; j++)
            {
              if(j < nb_byte_to_process)
                sprintf(&buffer[offset_text],"%02X",data[offset+j]);
              else
                strcpy(&buffer[offset_text],"  ");
              offset_text += 2;

              /* Sparateur */
              if(j < (16-1))
                {
                  buffer[offset_text] = ((j > 0 && (((j+1)%8)==0)) && (offset+j < nb_byte_to_process)) ? '.' : ' ';
                  offset_text++;
                }
            }

          /* Sparateur */
          strcpy(&buffer[offset_text],"  ");
          offset_text += 2;

          /** Ascii **/
          for(j=0; j<16; j++)
            {
              if(j < nb_byte_to_process)
                buffer[offset_text] = GETASCIICODE(data[offset+j]);
              else
                buffer[offset_text] = ' ';
              offset_text++;
            }

          /* Fin de ligne */
          strcpy(&buffer[offset_text],"\n");
          offset_text += 1;

          /* Dump dans le fichier */
          fprintf(fd,"   %s",buffer);
        }

      /* Libration mmoire */
      free(data);
    }

  /* Fermeture du fichier */
  fclose(fd);
}


/***********************************************************************/
/*  DeleteInternalsChildWindows() :  Destruction des fentres enfants. */
/***********************************************************************/
static void DeleteInternalsChildWindows(struct window_internals *window_internals)
{
  /** Libration des ressources **/

}


/*************************************************************************/
/*  mem_free_window_internals() :  Libration mmoire pour la structure. */
/*************************************************************************/
void mem_free_window_internals(struct window_internals *window_internals)
{
  if(window_internals)
    {
      /* Supprime la notification */
      my_Notification(NOTIFICATION_REMOVE,window_internals->current_notification,NULL);
      window_internals->current_notification = NULL;
      
      free(window_internals);
    }
}

/**********************************************************************/
