/**********************************************************************/
/*                                                                    */
/*   Window_Main.c :  Module de gestion de la fentre Main.           */
/*                                                                    */
/**********************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe  *  Date : Avril 2018  */
/**********************************************************************/

#include <stdio.h>
#include <stdlib.h>

#include <windows.h>
#include <windowsx.h>
#include <Richedit.h>
#include <commctrl.h>
#include <Commdlg.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_MMU.h"
#include "Dc_Request.h"
#include "Dc_Notification.h"
#include "Dc_Disassembler.h"
#include "Dc_Output.h"
#include "Dc_Name.h"
#include "Cyrene_Engine.h"
#include "Window_ProgressBar.h"
#include "Window_Disassembler.h"
#include "Window_IPC.h"
#include "Window_Symbol.h"
#include "Window_Breakpoint.h"
#include "Window_Statistic.h"
#include "Window_QuickWatch.h"
#include "Window_Main_Menu.h"
#include "Window_Main.h"

LRESULT CALLBACK ProcHandlerMainWindow(HWND,UINT,WPARAM,LPARAM);
static int CreateMainChildWindows(HWND,struct window_main *);
static DWORD CALLBACK CodeRichEditStreamCallback(DWORD *,LPBYTE,LONG,LONG *);
static void UpdateOperationStatus(struct window_main *,int);
static void UpdateTraceSyscallStatus(struct window_main *,int);
static void UpdateOperationTextEdit(struct window_main *,int);
static void BuildOperationLine(struct operation *,int,int,char *,int);
static void BuildCodeRichEdit(struct window_main *,int,int,int,int,struct mmu *);
static void DisplayCodeRichEdit(struct window_main *);
static void ClearCodeRichEdit(struct window_main *);
static void RefreshLastOperationWindowData(struct window_main *);
static void RefreshCodeWindowData(struct window_main *);
static void SaveOperationList(struct window_main *);
static DWORD SaveOperationListThread(void *);
static void CallStackListDisassemble(struct window_main *);
static void CallStackListClear(struct window_main *);
static void OutputListSave(struct window_main *);
static void BuildSystemCallOutputLine(struct output *,int,char *);
static void OutputListClear(struct window_main *);
static void OutputListRefresh(struct window_main *);
static void RefreshCallStackListBox(struct window_main *);
static void UpdateCodeButtonStatus(struct window_main *);
static void BuildCallStackLine(int,DWORD,DWORD,DWORD,WORD,BYTE,int,int,char *,int,struct window_main *);
static BOOL CreateMainToolBarWindow(struct window_main *);
static BOOL CreateMainStatusBarWindow(struct window_main *);
static void UserResizeMainWindow(struct window_main *);
static void MoveMainWindowControls(int,int,struct window_main *);
static int DeleteMainChildWindows(struct window_main *);
static void SaveOperationStackList(struct window_main *);


/**********************************************************************/
/*  BuildMainWindow() :  Construction / Affichage de la fentre Main. */
/**********************************************************************/
int BuildMainWindow(HINSTANCE hInstance)
{
  WNDCLASS wndclass;
  HWND hwndMain;
  RECT client_rect, window_rect;
  struct window_main *window_main;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  
  /** Allocation de la structure Window_Main **/
  window_main = (struct window_main *) calloc(1,sizeof(struct window_main));
  if(window_main == NULL)
    return(1);
  my_Memory(MEMORY_SET_WINDOW_MAIN,window_main,NULL);

  /* Brush */
  window_main->hBrush = CreateSolidBrush(GetSysColor(COLOR_3DFACE));

  /** Enregistrement de la Classe de la Fentre principale **/
  if(param->register_class_main == 0)
    {
      memset(&wndclass,0,sizeof(WNDCLASS));
      wndclass.style = 0;
      wndclass.lpfnWndProc = (WNDPROC)ProcHandlerMainWindow;
      wndclass.cbClsExtra = 0;
      wndclass.cbWndExtra = 0;
      wndclass.hInstance = hInstance;
      wndclass.hIcon = LoadIcon(hInstance,MAKEINTRESOURCE(IDI_ICON_APPLE));
      wndclass.hCursor = LoadCursor((HINSTANCE)NULL,IDC_ARROW);
      wndclass.hbrBackground = window_main->hBrush;   /* Couleur de Fond */
      wndclass.lpszMenuName = MAKEINTRESOURCE(IDR_MENU_MAIN);
      wndclass.lpszClassName = "Cyrene";
      if(!RegisterClass(&wndclass))
        return(1);
        
      /* La classe de fentre est enregistre */
      param->register_class_main = 1;
    }
    
  /** Create Window Main **/
  hwndMain = CreateWindowEx(0,"Cyrene","Cyrene - v 1.0 - The Apple IIgs Remote Debugger", WS_CAPTION | WS_SYSMENU | WS_MINIMIZEBOX | WS_THICKFRAME,CW_USEDEFAULT,CW_USEDEFAULT,
                            MAIN_WINDOW_MIN_WIDTH, MAIN_WINDOW_MIN_HEIGHT, NULL, NULL, hInstance, NULL);
  if(hwndMain == NULL)
    return(1);
  window_main->hwnd_main = hwndMain;

  /* Hauteur Titre + Menu */
  GetWindowRect(hwndMain,&window_rect);
  GetClientRect(hwndMain,&client_rect);
  window_main->title_menu_height = (window_rect.bottom - window_rect.top) - client_rect.bottom;

  /** Show Window + Paint Content **/
  ShowWindow(hwndMain,SW_SHOWDEFAULT);
  UpdateWindow(hwndMain);

  /** Met les Menus en Gris car aucun projet n'est charg **/
  UpdateWindowMainMenuItemStatus();

  /* MAJ de la Status Bar */
  UpdateWindowMainStatusBar();

  /* OK */
  return(0);
}


/*********************************************************************/
/*  ProcHandlerMainWindow() :  Gestion des vnements de la fentre. */
/*********************************************************************/
LRESULT CALLBACK ProcHandlerMainWindow(HWND hwndMain, UINT umsg, WPARAM wParam, LPARAM lParam)
{
  int id, timer_id, nb_operation, nb_display, scroll_position_max, mouse_x, mouse_y;
  UINT code_notify;
  HWND g_wnd_ctl, hwnd;
  HDC hdc;
  WORD scroll_position;
  SCROLLINFO scroll_info;
  COLORREF background_color;
  MSGFILTER *message_filter;
  struct notification *current_notification;
  struct parameter *param;
  struct window_main *window_main;

  /* Rcupre les fentres */
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);

  switch(umsg)
    {
      /******************************/
      /*  Initialization de window  */
      /******************************/
      case WM_CREATE:
        /* Cration du contenu de la Fenetre Main */
        CreateMainChildWindows(hwndMain,window_main);
        return(0);

      /**************************************/
      /*  On vient de retailler la fentre  */
      /**************************************/
      case WM_SIZE :
        /* On va tendre la fentre de donne mmoire si c'est un Resize User */
        if(window_main->user_resize == 1)
          UserResizeMainWindow(window_main);
        window_main->user_resize = 1;
        return(TRUE);

      case WM_PAINT:
        break;

      /** Couleur d'arrire plan pour les Text Edit **/
      case WM_CTLCOLORSTATIC:
        /* Rcupration des paramtres */
        hdc = (HDC) wParam;
        hwnd = (HWND) lParam; 
        background_color = RGB(251,251,251);

        /* Couleur d'arrire plan du Text Edit Operation */
        if(hwnd == window_main->hwnd_edit_operation)
          {
            /* Renvoi un DCBRUSH avec la couleur */
            SetBkColor(hdc, background_color);
            SetDCBrushColor(hdc, background_color);
            return((LRESULT) GetStockObject(DC_BRUSH));
          }
        else
         return(DefWindowProc(hwndMain,umsg,wParam,lParam));
        break;

      /*****************************/
      /*  Evnements de la souris  */
      /*****************************/
      case WM_MOUSEMOVE:
      case WM_LBUTTONDOWN:
      case WM_LBUTTONUP:  
      case WM_RBUTTONDOWN:
      case WM_RBUTTONUP:
        break;

      /************************/
      /*  Evnements de menu  */
      /************************/
      case WM_COMMAND :
        /** Evnements provenant du Menu **/
        id = LOWORD(wParam);
        code_notify = HIWORD(wParam);
        g_wnd_ctl = (HWND) lParam;
        MainMenuToolbarCommand(hwndMain,id,g_wnd_ctl,code_notify);

        /** Gestion des boutons **/
        switch(LOWORD(wParam))
          {
            /** CheckBox Operation : On active/dsactive l'enregistrement des oprations **/
            case IDC_CHECKBOX_OPERATION :
              if(HIWORD(wParam) == BN_CLICKED)
                {
                  if(SendDlgItemMessage(hwndMain,IDC_CHECKBOX_OPERATION,BM_GETCHECK,0,0))
                    UpdateOperationStatus(window_main,1);
                  else
                    UpdateOperationStatus(window_main,0);
                }
              break;

            /** Bouton Operation : Clear Operation **/
            case IDC_BUTTON_OPERATION_CLEAR :
              if(HIWORD(wParam) == BN_CLICKED)
                {
                  /* Supprime toutes les Oprations */
                  EnterCriticalSection(&param->cs_operation);
                  my_Memory(MEMORY_FREE_OPERATION,NULL,NULL);
                  LeaveCriticalSection(&param->cs_operation);
                  /* MAJ de l'Operation History */
                  window_main->operation_scroll_position = 0;
                  param->operation_total_cycle = 0;
                  param->operation_id = 0;
                  UpdateOperationTextEdit(window_main,0);
                  /* MAJ Last Operation */
                  RefreshLastOperationWindowData(window_main);
                  /* On active/dsactive les boutons */
                  window_main->button_operation_status = FALSE;
                  EnableWindow(window_main->hwnd_button_operation_reset_operation,window_main->button_operation_status);
                  EnableWindow(window_main->hwnd_button_operation_reset_cycle,window_main->button_operation_status);
                  EnableWindow(window_main->hwnd_button_operation_save,window_main->button_operation_status);
                  EnableWindow(window_main->hwnd_button_operation_clear,window_main->button_operation_status);
                  /* On va stocker la premire opration */
                  param->is_first_operation = 1;
                }
              break;

            /** Bouton Operation : Reset Cycle **/
            case IDC_BUTTON_OPERATION_RESET_CYCLE :
              if(HIWORD(wParam) == BN_CLICKED)
                {
                  param->operation_total_cycle = 0;
                  RefreshLastOperationWindowData(window_main);
                }
              break;

            /** Bouton Operation : Reset Operation Count **/
            case IDC_BUTTON_OPERATION_RESET_OPERATION :
              if(HIWORD(wParam) == BN_CLICKED)
                {
                  param->operation_id = 0;
                  RefreshLastOperationWindowData(window_main);
                }
              break;

            /** Bouton Operation : Save Operation **/
            case IDC_BUTTON_OPERATION_SAVE :
              if(HIWORD(wParam) == BN_CLICKED)
                SaveOperationList(window_main);
              break;

            /** Bouton Code : Run/Stop **/
            case ID_ACCELERATOR_RUN :
              if(window_main->acc_code_run == TRUE)
                ProcessOperationButton(window_main,IDC_BUTTON_CODE_RUN);
              break;
            case IDC_BUTTON_CODE_RUN :
              if(HIWORD(wParam) == BN_CLICKED)
                ProcessOperationButton(window_main,IDC_BUTTON_CODE_RUN);
              break;

            /** Bouton Code : Step In **/
            case ID_ACCELERATOR_STEP_IN :
              if(window_main->acc_code_next == TRUE)
                ProcessOperationButton(window_main,IDC_BUTTON_CODE_IN);
              break;
            case IDC_BUTTON_CODE_IN :
              if(HIWORD(wParam) == BN_CLICKED)
                ProcessOperationButton(window_main,IDC_BUTTON_CODE_IN);
              break;

            /** Bouton Code : Step Over **/
            case ID_ACCELERATOR_STEP_OVER :
              if(window_main->acc_code_over == TRUE)
                ProcessOperationButton(window_main,IDC_BUTTON_CODE_OVER);
              break;
            case IDC_BUTTON_CODE_OVER :
              if(HIWORD(wParam) == BN_CLICKED)
                ProcessOperationButton(window_main,IDC_BUTTON_CODE_OVER);
              break;

            /** Bouton Code : Next 10 **/
            case IDC_BUTTON_CODE_10X :
              if(HIWORD(wParam) == BN_CLICKED)
                ProcessOperationButton(window_main,IDC_BUTTON_CODE_10X);
              break;

            /** Bouton Code : Next 100 **/
            case IDC_BUTTON_CODE_100X :
              if(HIWORD(wParam) == BN_CLICKED)
                ProcessOperationButton(window_main,IDC_BUTTON_CODE_100X);
              break;

            /** Bouton Code : Next 1000 **/
            case IDC_BUTTON_CODE_1000X :
              if(HIWORD(wParam) == BN_CLICKED)
                ProcessOperationButton(window_main,IDC_BUTTON_CODE_1000X);
              break;

            /** Bouton Code : Next 10000 **/
            case IDC_BUTTON_CODE_10000X :
              if(HIWORD(wParam) == BN_CLICKED)
                ProcessOperationButton(window_main,IDC_BUTTON_CODE_10000X);
              break;

            /** Bouton Code : Next JSR/JSL **/
            case IDC_BUTTON_CODE_JSR :
              if(HIWORD(wParam) == BN_CLICKED)
                ProcessOperationButton(window_main,IDC_BUTTON_CODE_JSR);
              break;

            /** Bouton Code : Next RTS **/
            case IDC_BUTTON_CODE_RTS :
              if(HIWORD(wParam) == BN_CLICKED)
                ProcessOperationButton(window_main,IDC_BUTTON_CODE_RTS);
              break;

            /** Bouton Code : Next JMP/Branch **/
            case IDC_BUTTON_CODE_JMP :
              if(HIWORD(wParam) == BN_CLICKED)
                ProcessOperationButton(window_main,IDC_BUTTON_CODE_JMP);
              break;

            /** Bouton Code : Next VBL **/
            case IDC_BUTTON_CODE_VBL :
              if(HIWORD(wParam) == BN_CLICKED)
                ProcessOperationButton(window_main,IDC_BUTTON_CODE_VBL);
              break;

            /** Bouton Code : Next Interrupt **/
            case IDC_BUTTON_CODE_INTERRUPT :
              if(HIWORD(wParam) == BN_CLICKED)
                ProcessOperationButton(window_main,IDC_BUTTON_CODE_INTERRUPT);
              break;

            /** Bouton Code : Next Toolbox **/
            case IDC_BUTTON_CODE_TOOLBOX :
              if(HIWORD(wParam) == BN_CLICKED)
                ProcessOperationButton(window_main,IDC_BUTTON_CODE_TOOLBOX);
              break;

            /** Bouton CallStack : Next GS/OS **/
            case IDC_BUTTON_CODE_GSOS :
              if(HIWORD(wParam) == BN_CLICKED)
                ProcessOperationButton(window_main,IDC_BUTTON_CODE_GSOS);
              break;

            /** Bouton Code : Next Highlight **/
            case IDC_BUTTON_CODE_HIGHLIGHT :
              if(HIWORD(wParam) == BN_CLICKED)
                ProcessOperationButton(window_main,IDC_BUTTON_CODE_HIGHLIGHT);
              break;

            /** Bouton CallStack : Disassemble **/
            case IDC_BUTTON_CALLSTACK_DISASSEMBLE :
              if(HIWORD(wParam) == BN_CLICKED)
                CallStackListDisassemble(window_main);
              break;

            /** Bouton CallStack : Clear **/
            case IDC_BUTTON_CALLSTACK_CLEAR :
              if(HIWORD(wParam) == BN_CLICKED)
                CallStackListClear(window_main);
              break;

            /** Bouton Output : Save **/
            case IDC_BUTTON_OUTPUT_SAVE :
              if(HIWORD(wParam) == BN_CLICKED)
                OutputListSave(window_main);
              break;

            /** Bouton Output : Clear **/
            case IDC_BUTTON_OUTPUT_CLEAR :
              if(HIWORD(wParam) == BN_CLICKED)
                OutputListClear(window_main);
              break;

            /** Bouton Output : Refresh **/
            case IDC_BUTTON_OUTPUT_REFRESH :
              if(HIWORD(wParam) == BN_CLICKED)
                OutputListRefresh(window_main);
              break;

            /** CheckBox Trace Syscall : On active/dsactive la trace des appels systmes **/
            case IDC_CHECKBOX_TRACE_SYSCALL :
              if(HIWORD(wParam) == BN_CLICKED)
                {
                  if(SendDlgItemMessage(hwndMain,IDC_CHECKBOX_TRACE_SYSCALL,BM_GETCHECK,0,0))
                    UpdateTraceSyscallStatus(window_main,1);
                  else
                    UpdateTraceSyscallStatus(window_main,0);
                }
              break;

            /** F1 : About **/
            case ID_ACCELERATOR_ABOUT :
              MenuAbout(window_main);
              break;
          }
        return(0);

      /****************/
      /*  Scroll Bar  */
      /****************/
      case WM_VSCROLL :
        {
          /* Rcupre les informations de la scroll bar */
          scroll_info.cbSize = sizeof(scroll_info);
          scroll_info.fMask  = SIF_ALL;
          GetScrollInfo(window_main->hwnd_scrollbar_operation,SB_CTL,&scroll_info);  /* SB_CTL : La fentre est une scrollbar */

          switch((int)LOWORD(wParam))
            {
              /* On clique sur la flche haut : Une ligne vers le haut */
              case SB_LINEUP :
                if(window_main->operation_scroll_position > 0)
                  {
                    window_main->operation_scroll_position--;
                    UpdateOperationTextEdit(window_main,0);
                  }
                break;

              /* On clique sur la flche bas : Une ligne vers le bas */
              case SB_LINEDOWN :
                my_Memory(MEMORY_GET_OPERATION_NB,&nb_operation,NULL);
                nb_display = (nb_operation >= window_main->operation_text_data_row) ? window_main->operation_text_data_row : nb_operation;
                scroll_position_max = nb_operation - nb_display;
                if(window_main->operation_scroll_position < scroll_position_max)
                  {
                    window_main->operation_scroll_position++;
                    UpdateOperationTextEdit(window_main,0);
                  }
                break;

              /* On clique sur la zone cliquable haute : Une page vers le haut */
              case SB_PAGEUP :
                if(window_main->operation_scroll_position > 0)
                  {
                    window_main->operation_scroll_position -= window_main->operation_text_data_row;
                    if(window_main->operation_scroll_position < 0)
                      window_main->operation_scroll_position = 0;
                    UpdateOperationTextEdit(window_main,0);
                  }
                break;

              /* On clique sur la zone cliquable basse : Une page vers le bas */
              case SB_PAGEDOWN :
                my_Memory(MEMORY_GET_OPERATION_NB,&nb_operation,NULL);
                nb_display = (nb_operation >= window_main->operation_text_data_row) ? window_main->operation_text_data_row : nb_operation;
                scroll_position_max = nb_operation - nb_display;
                if(window_main->operation_scroll_position < scroll_position_max)
                  {
                    window_main->operation_scroll_position += window_main->operation_text_data_row;;
                    if(window_main->operation_scroll_position > scroll_position_max)
                      window_main->operation_scroll_position = scroll_position_max;
                    UpdateOperationTextEdit(window_main,0);
                  }
                break;

              /* On vient de dplacer le curseur et le positionner sur une nouvelle position *
              case SB_THUMBPOSITION :
                scroll_position = HIWORD(wParam);
                window_main->operation_scroll_position = scroll_position;
                UpdateOperationTextEdit(window_main);
                break; */

              /* On est entrain de dplacer le curseur */
              case SB_THUMBTRACK :
                scroll_position = HIWORD(wParam);
                window_main->operation_scroll_position = scroll_info.nTrackPos;   /* scroll_position est limit  65K entres,on prend scroll_info */
                UpdateOperationTextEdit(window_main,0);
                break;
            }
          return(0);
        }
      break;

      /**********************************************/
      /* Evnements des Controles  la fentre mre */
      /**********************************************/
      case WM_NOTIFY :
        message_filter = (MSGFILTER *) lParam;
        
        /** Evnement de la souris dans le Rich Edit transmis  la fentre mre **/
        if(message_filter && message_filter->nmhdr.hwndFrom == window_main->hwnd_richedit_code && window_main->code_status == CYRENE_STATUS_STOPPED_ON_OPERATION)
          {
            if(message_filter->msg == WM_LBUTTONDOWN)
              {
                /* Coodonnes de la souris dans la grille des caractres du Control Edit */
                mouse_x = GET_X_LPARAM(message_filter->lParam) / 7;
                mouse_y = GET_Y_LPARAM(message_filter->lParam) / DISASSEMBLER_RICHEDIT_FONT_HEIGHT;

                /* La ligne Y est slectionne (0-N) */
                window_main->code_selected_line = mouse_y;

                /* On n'autorise pas la slection de la ligne courante */
                if(window_main->code_selected_line == window_main->code_current_line)
                  window_main->code_selected_line = -1;

                /* MAJ de la zone Code */
                DisplayCodeRichEdit(window_main);

                /* MAJ des Boutons de la Zone Code pour activer le bouton 'Highlight' */
                UpdateCodeButtonStatus(window_main);
              }
          }
        break;

      /**************************************/
      /*  Cration / Suppression des Timer  */
      /**************************************/
      case HM_NOTIFICATION_TIMER_CREATE :
        /* Rcupration de la Notification */
        current_notification = (struct notification *) lParam;
        if(current_notification == NULL)
          return(0);
          
        /* Positionne le Timer */
        SetTimer(hwndMain,current_notification->timer_id,current_notification->timer_milliseconds,(TIMERPROC) NULL); 
        return(0);

      case HM_NOTIFICATION_TIMER_DELETE :
        /* Rcupration de la Notification */
        current_notification = (struct notification *) lParam;
        if(current_notification == NULL)
          return(0);

        /* Supprime le Timer */   
        KillTimer(hwndMain,current_notification->timer_id);
        return(0);

      /****************************/
      /*  Notification des Timer  */
      /****************************/
      case WM_TIMER :
        /* On rcupre la notification via le timer_id */
        timer_id = (int) wParam;
        my_Notification(NOTIFICATION_GET_BY_ID,&timer_id,&current_notification);
        if(current_notification == NULL)
          return(0);
        
        /* On envoie le message  la fentre */
        SendMessage(current_notification->hwnd_window,current_notification->message,0,0);
        return(0);

      /*****************************************************************/
      /* Message extrieur demandant le rafraichissement de la fentre */
      /*****************************************************************/
      case HM_OPERATION_REFRESHWINDOW :
        /* Operation History */
        RefreshLastOperationWindowData(window_main);
        return(TRUE);

      case HM_OPERATION_CODE_REFRESHWINDOW :
        /* Operation History */
        RefreshLastOperationWindowData(window_main);
        /* Code */
        RefreshCodeWindowData(window_main);
        /* CallStack */
        RefreshCallStackListBox(window_main);
        /* Output */
        RefreshOutputListBox(window_main);
        return(TRUE);

      case HM_WINDOW_MAIN_RESET :
        /** Supprime toutes les Oprations + Reset Cycle + Reset Operation **/
        EnterCriticalSection(&param->cs_operation);
        my_Memory(MEMORY_FREE_OPERATION,NULL,NULL);
        LeaveCriticalSection(&param->cs_operation);
        /* MAJ de l'Operation History */
        window_main->operation_scroll_position = 0;
        param->operation_total_cycle = 0;
        param->operation_id = 0;
        UpdateOperationTextEdit(window_main,0);
        /* MAJ Last Operation */
        RefreshLastOperationWindowData(window_main);
        /* On active/dsactive les boutons */
        window_main->button_operation_status = FALSE;
        EnableWindow(window_main->hwnd_button_operation_reset_operation,window_main->button_operation_status);
        EnableWindow(window_main->hwnd_button_operation_reset_cycle,window_main->button_operation_status);
        EnableWindow(window_main->hwnd_button_operation_save,window_main->button_operation_status);
        EnableWindow(window_main->hwnd_button_operation_clear,window_main->button_operation_status);
        /* On va stocker la premire opration */
        param->is_first_operation = 1;
        /** Clear CallStack **/
        CallStackListClear(window_main);
        /** Clear Output **/
        OutputListClear(window_main);
        /** Reset la zone Code **/
        ClearCodeRichEdit(window_main);
        /** Operation Data **/
        SetWindowText(window_main->hwnd_edit_operation_data,"00/0000 : 00");
        /** GOID **/
        param->operation_goid = 0;
        UpdateWindowMainStatusBar();
        break;

      /*************************/
      /*  Fermeture de window  */
      /*************************/
      case WM_CLOSE :
        /** On demande la fermeture de l'application **/
        /* Fermeture de toutes les autres fentres */
        my_Memory(MEMORY_CLOSE_WINDOW_ALL,NULL,NULL);

        /* Fentre Principale */
        DestroyWindow(hwndMain);
        return(0);

      case WM_DESTROY :
        DeleteMainChildWindows(window_main);
        PostQuitMessage(0);
        return(0);
    }

  /* Evnements non traits */
  return(DefWindowProc(hwndMain,umsg,wParam,lParam));
}


/***********************************************************************************************/
/*  CreateMainChildWindows() : Crer les fentres enfants et remplir la structure window_main  */
/***********************************************************************************************/
static int CreateMainChildWindows(HWND hwndMain, struct window_main *window_main)
{
  BOOL result;
  RECT rect_edit, window_rect;
  int i, x0, y0, width, height;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Fill the structure member main window */
  window_main->hwnd_main = hwndMain;

  /** Rcupre le Handle du Menu **/
  window_main->hmenu_menu = GetMenu(window_main->hwnd_main);
  if(window_main->hmenu_menu == NULL)
    return(FALSE);

  /** Create ToolBar **/
  result = CreateMainToolBarWindow(window_main);
  if(result == FALSE)
    return(FALSE);

  /** Create StatusBar **/
  result = CreateMainStatusBarWindow(window_main);
  if(result == FALSE)
    return(FALSE);

  /* Init All Default Values */
  window_main->window_main_width = MAIN_WINDOW_MIN_WIDTH;
  window_main->window_main_height = MAIN_WINDOW_MIN_HEIGHT;

  /*************************/
  /*** Operation History ***/
  /*************************/
  window_main->operation_history_checkbox = 0;    /* On n'historise pas */

  /* Group Box Operation */
  window_main->hwnd_groupbox_operation = CreateWindowExA(0,"BUTTON"," Operation History ", WS_CHILD | WS_VISIBLE | BS_GROUPBOX | WS_GROUP,
                                                         10, 35, 1298, 217, hwndMain, (HMENU)IDC_GROUPBOX_OPERATION,param->hInstance,0);

  /* Libells Operation */
  window_main->hwnd_static_operation = CreateWindowExA(0,"STATIC","Operations              A         X         Y      Stack   Direct DBR PBR   PC            Status       M  X    C02E-2F  C035 C068          Cycles                 Address       Data                       Label                                          Opcode  Operand                                 Comment",WS_VISIBLE|WS_CHILD|SS_LEFT,23,52,1200,16,hwndMain,(HMENU)IDC_STATIC_OPERATION,param->hInstance,NULL);

  /** Cration du Control Text Edit Multi ligne ReadOnly **/
  window_main->operation_text_data_column = MAIN_OPERATION_NB_COLUMN;    /* 179 */
  window_main->operation_text_data_row = MAIN_OPERATION_NB_ROW;
  window_main->operation_text_area_width = window_main->operation_text_data_column*7;
  window_main->operation_text_area_height = window_main->operation_text_data_row*14;
  window_main->hwnd_edit_operation = CreateWindowExA(0,"EDIT",NULL,WS_CHILD | WS_VISIBLE | ES_LEFT | ES_MULTILINE | ES_READONLY, 
                                                     20, 68, window_main->operation_text_area_width, window_main->operation_text_area_height,   // set size in WM_SIZE message 
                                                     hwndMain,(HMENU)IDC_EDIT_OPERATION,param->hInstance,NULL);

  /** Cration du Scroll bar vertical le long du Control Text Edit **/
  GetClientRect(window_main->hwnd_edit_operation,&rect_edit);
  window_main->hwnd_scrollbar_operation = CreateWindowEx(0,"SCROLLBAR",(PTSTR) NULL,WS_CHILD | WS_VISIBLE | SBS_VERT,         // horizontal scroll bar style 
                                                         28 + window_main->operation_text_area_width, 68,
                                                         15, rect_edit.bottom - rect_edit.top,
                                                         hwndMain,(HMENU)NULL,param->hInstance,NULL);

  /** Cration des boutons Oprations **/
  x0 = 788;
  y0 = 218;
  window_main->hwnd_button_operation_reset_operation = CreateWindow("BUTTON","Reset # Operations",WS_TABSTOP|WS_VISIBLE|WS_CHILD|BS_DEFPUSHBUTTON,x0,y0,110,22,hwndMain,(HMENU)IDC_BUTTON_OPERATION_RESET_OPERATION,param->hInstance,NULL);
  x0 += 125;
  window_main->hwnd_button_operation_reset_cycle     = CreateWindow("BUTTON","Reset # Cycles",    WS_TABSTOP|WS_VISIBLE|WS_CHILD|BS_DEFPUSHBUTTON,x0,y0,110,22,hwndMain,(HMENU)IDC_BUTTON_OPERATION_RESET_CYCLE,param->hInstance,NULL);
  x0 += 125;
  window_main->hwnd_button_operation_save            = CreateWindow("BUTTON","Save Operations", WS_TABSTOP|WS_VISIBLE|WS_CHILD|BS_DEFPUSHBUTTON,x0,y0,110,22,hwndMain,(HMENU)IDC_BUTTON_OPERATION_SAVE,param->hInstance,NULL);
  x0 += 125;
  window_main->hwnd_button_operation_clear           = CreateWindow("BUTTON","Clear History",   WS_TABSTOP|WS_VISIBLE|WS_CHILD|BS_DEFPUSHBUTTON,x0,y0,110,22,hwndMain,(HMENU)IDC_BUTTON_OPERATION_CLEAR,param->hInstance,NULL);

  /* Cration de la Check Box Operation History */
  window_main->hwnd_checkbox_history_operation = CreateWindowEx(0,"BUTTON","Record Apple IIgs Emulator Operations",WS_VISIBLE|WS_CHILD|BS_AUTOCHECKBOX,20,218,220,20,hwndMain,(HMENU)IDC_CHECKBOX_OPERATION,param->hInstance,NULL); 

  /* Statistiques */
  window_main->hwnd_static_operation_count = CreateWindowExA(0,"STATIC","# Operations :",  WS_VISIBLE|WS_CHILD|SS_LEFT,240,220,70,16,hwndMain,(HMENU)IDC_STATIC_NB_OPERATION,param->hInstance,NULL);
  window_main->hwnd_edit_operation_count   = CreateWindowExA(0,"EDIT",NULL,                WS_CHILD | WS_VISIBLE | ES_LEFT,310,220,70,14,hwndMain,(HMENU)IDC_EDIT_NB_OPERATION,param->hInstance,NULL);
  window_main->hwnd_static_cycle_count     = CreateWindowExA(0,"STATIC","# Cycles :",      WS_VISIBLE|WS_CHILD|SS_LEFT,395,220,50,16,hwndMain,(HMENU)IDC_STATIC_NB_CYCLE,param->hInstance,NULL);
  window_main->hwnd_edit_cycle_count       = CreateWindowExA(0,"EDIT",NULL,                WS_CHILD | WS_VISIBLE | ES_LEFT,445,220,75,14,hwndMain,(HMENU)IDC_EDIT_NB_CYCLE,param->hInstance,NULL);
  window_main->hwnd_static_operation_data  = CreateWindowExA(0,"STATIC","Operation Data :",WS_VISIBLE|WS_CHILD|SS_LEFT,525,220,80,16,hwndMain,(HMENU)IDC_STATIC_CURRENT_OPERATION_DATA,param->hInstance,NULL);
  window_main->hwnd_edit_operation_data    = CreateWindowExA(0,"EDIT",NULL,                WS_CHILD | WS_VISIBLE | ES_LEFT,605,220,120,14,hwndMain,(HMENU)IDC_EDIT_CURRENT_OPERATION_DATA,param->hInstance,NULL);
  EnableWindow(window_main->hwnd_edit_operation_count,FALSE);
  EnableWindow(window_main->hwnd_edit_cycle_count,FALSE);
  EnableWindow(window_main->hwnd_edit_operation_data,FALSE);

  /** Cration de la Police de caractre + Affectation de la police au Control Edit **/
  window_main->hFont_courrier = CreateFont(14,0,0,0,FW_DONTCARE,FALSE,FALSE,FALSE,ANSI_CHARSET,OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY,DEFAULT_PITCH|FF_SWISS,TEXT("Courier New"));
  SendMessage(window_main->hwnd_edit_operation,WM_SETFONT,(WPARAM)window_main->hFont_courrier,MAKELPARAM(FALSE,0));

  /* On dsactive les boutons */
  window_main->button_operation_status = FALSE;
  EnableWindow(window_main->hwnd_button_operation_reset_operation,window_main->button_operation_status);
  EnableWindow(window_main->hwnd_button_operation_reset_cycle,window_main->button_operation_status);
  EnableWindow(window_main->hwnd_button_operation_save,window_main->button_operation_status);
  EnableWindow(window_main->hwnd_button_operation_clear,window_main->button_operation_status);

  /************************/
  /*** Code Disassembly ***/
  /************************/
  window_main->code_x0 = 10;
  window_main->code_y0 = 255;

  /* Group Box Code */
  window_main->hwnd_groupbox_code = CreateWindowExA(0,"BUTTON"," Code ", WS_CHILD | WS_VISIBLE | BS_GROUPBOX | WS_GROUP,
                                                    window_main->code_x0, window_main->code_y0, 811, 385+30+3+8+60, hwndMain, (HMENU)IDC_GROUPBOX_CODE,param->hInstance,0);

  /* Libells Code */
  x0 = window_main->code_x0 + 10;
  y0 = window_main->code_y0 + 17;
  window_main->hwnd_static_code = CreateWindowExA(0,"STATIC","MX   Address       Data                       Label                                                   Opcode  Operand            Comment",WS_VISIBLE|WS_CHILD|SS_LEFT,
                                                  x0,y0,540,16,hwndMain,(HMENU)IDC_STATIC_CODE,param->hInstance,NULL);

  /** Cration du Rich Edit pour code **/
  window_main->nb_code_line = 31;
  x0 = window_main->code_x0 + 10;
  y0 = window_main->code_y0 + 37;
  width = 690;
  height = window_main->nb_code_line*14 + 2;
  window_main->hwnd_richedit_code = CreateWindowEx(0,RICHEDIT_CLASS,"",WS_CHILD | WS_VISIBLE | ES_SAVESEL | ES_MULTILINE | ES_READONLY, 
                                                   x0,y0,width,height,hwndMain,(HMENU)IDC_RICHEDIT_CODE,param->hInstance,NULL);

  /* Couleur de Fond du control Rich Edit : Gris comme la fentre Opration */
  SendMessage(window_main->hwnd_richedit_code,EM_SETBKGNDCOLOR,0,(LPARAM)RGB(251,251,251)); 

  /* On demance  ce que les Event Souris (click+scroll) du RichEdit soient envoys  la fentre principale : WM_NOTIFY */
  SendMessage(window_main->hwnd_richedit_code,EM_SETEVENTMASK,0,(LPARAM)(ENM_MOUSEEVENTS|ENM_SCROLLEVENTS));

  /** Cration des boutons Code **/
  x0 = window_main->code_x0 + 712;
  y0 = window_main->code_y0 + 15 + 3;
  window_main->hwnd_button_code_run = CreateWindow("BUTTON","Run",WS_TABSTOP|WS_VISIBLE|WS_CHILD|BS_DEFPUSHBUTTON,x0,y0,85,22,hwndMain,(HMENU)IDC_BUTTON_CODE_RUN,param->hInstance,NULL);
  y0 += CODE_INTERSPACE_LARGE;
  window_main->hwnd_button_code_in = CreateWindow("BUTTON","> | Step In",WS_TABSTOP|WS_VISIBLE|WS_CHILD|BS_DEFPUSHBUTTON,x0,y0,85,22,hwndMain,(HMENU)IDC_BUTTON_CODE_IN,param->hInstance,NULL);
  y0 += CODE_INTERSPACE_SMALL;
  window_main->hwnd_button_code_over = CreateWindow("BUTTON",">> | Step Over",WS_TABSTOP|WS_VISIBLE|WS_CHILD|BS_DEFPUSHBUTTON,x0,y0,85,22,hwndMain,(HMENU)IDC_BUTTON_CODE_OVER,param->hInstance,NULL);
  y0 += CODE_INTERSPACE_LARGE;
  window_main->hwnd_button_code_10x = CreateWindow("BUTTON",">> | +10",WS_TABSTOP|WS_VISIBLE|WS_CHILD|BS_DEFPUSHBUTTON,x0,y0,85,22,hwndMain,(HMENU)IDC_BUTTON_CODE_10X,param->hInstance,NULL);
  y0 += CODE_INTERSPACE_SMALL;
  window_main->hwnd_button_code_100x = CreateWindow("BUTTON",">> | +100",WS_TABSTOP|WS_VISIBLE|WS_CHILD|BS_DEFPUSHBUTTON,x0,y0,85,22,hwndMain,(HMENU)IDC_BUTTON_CODE_100X,param->hInstance,NULL);
  y0 += CODE_INTERSPACE_SMALL;
  window_main->hwnd_button_code_1000x = CreateWindow("BUTTON",">> | +1000",WS_TABSTOP|WS_VISIBLE|WS_CHILD|BS_DEFPUSHBUTTON,x0,y0,85,22,hwndMain,(HMENU)IDC_BUTTON_CODE_1000X,param->hInstance,NULL);
  y0 += CODE_INTERSPACE_SMALL;
  window_main->hwnd_button_code_10000x = CreateWindow("BUTTON",">> | +10000",WS_TABSTOP|WS_VISIBLE|WS_CHILD|BS_DEFPUSHBUTTON,x0,y0,85,22,hwndMain,(HMENU)IDC_BUTTON_CODE_10000X,param->hInstance,NULL);
  y0 += CODE_INTERSPACE_LARGE;
  window_main->hwnd_button_code_jmp = CreateWindow("BUTTON",">> | JMP / Bxx",WS_TABSTOP|WS_VISIBLE|WS_CHILD|BS_DEFPUSHBUTTON,x0,y0,85,22,hwndMain,(HMENU)IDC_BUTTON_CODE_JMP,param->hInstance,NULL);
  y0 += CODE_INTERSPACE_SMALL;
  window_main->hwnd_button_code_jsr = CreateWindow("BUTTON",">> | JSR / JSL",WS_TABSTOP|WS_VISIBLE|WS_CHILD|BS_DEFPUSHBUTTON,x0,y0,85,22,hwndMain,(HMENU)IDC_BUTTON_CODE_JSR,param->hInstance,NULL);
  y0 += CODE_INTERSPACE_SMALL;
  window_main->hwnd_button_code_rts = CreateWindow("BUTTON",">> | RTS / RTL",WS_TABSTOP|WS_VISIBLE|WS_CHILD|BS_DEFPUSHBUTTON,x0,y0,85,22,hwndMain,(HMENU)IDC_BUTTON_CODE_RTS,param->hInstance,NULL);
  y0 += CODE_INTERSPACE_LARGE;
  window_main->hwnd_button_code_vbl = CreateWindow("BUTTON",">> | VBL",WS_TABSTOP|WS_VISIBLE|WS_CHILD|BS_DEFPUSHBUTTON,x0,y0,85,22,hwndMain,(HMENU)IDC_BUTTON_CODE_VBL,param->hInstance,NULL);
  y0 += CODE_INTERSPACE_SMALL;
  window_main->hwnd_button_code_interrupt = CreateWindow("BUTTON",">> | Interrupt",WS_TABSTOP|WS_VISIBLE|WS_CHILD|BS_DEFPUSHBUTTON,x0,y0,85,22,hwndMain,(HMENU)IDC_BUTTON_CODE_INTERRUPT,param->hInstance,NULL);
  y0 += CODE_INTERSPACE_LARGE;
  window_main->hwnd_button_code_toolbox = CreateWindow("BUTTON",">> | Toolbox",WS_TABSTOP|WS_VISIBLE|WS_CHILD|BS_DEFPUSHBUTTON,x0,y0,85,22,hwndMain,(HMENU)IDC_BUTTON_CODE_TOOLBOX,param->hInstance,NULL);
  y0 += CODE_INTERSPACE_SMALL;
  window_main->hwnd_button_code_gsos = CreateWindow("BUTTON",">> | OS",WS_TABSTOP|WS_VISIBLE|WS_CHILD|BS_DEFPUSHBUTTON,x0,y0,85,22,hwndMain,(HMENU)IDC_BUTTON_CODE_GSOS,param->hInstance,NULL);
  y0 += CODE_INTERSPACE_LARGE;
  window_main->hwnd_button_code_highlight = CreateWindow("BUTTON",">> | Highlight",WS_TABSTOP|WS_VISIBLE|WS_CHILD|BS_DEFPUSHBUTTON,x0,y0,85,22,hwndMain,(HMENU)IDC_BUTTON_CODE_HIGHLIGHT,param->hInstance,NULL);

  /******************/
  /*** Call Stack ***/
  /******************/
  window_main->callstack_x0 = 831;
  window_main->callstack_y0 = 255;

  /* Group Box Code */
  window_main->hwnd_groupbox_callstack = CreateWindowExA(0,"BUTTON"," Call Stack ",WS_CHILD | WS_VISIBLE | BS_GROUPBOX | WS_GROUP,
                                                         window_main->callstack_x0,window_main->callstack_y0,477,190+14+28+1,hwndMain,(HMENU)IDC_GROUPBOX_CALLSTACK,param->hInstance,0);

  /* List Box */
  window_main->hwnd_listbox_callstack = CreateWindowEx(0,"LISTBOX", 0, WS_CHILD | WS_VISIBLE | WS_VSCROLL,
                                                       window_main->callstack_x0+10,window_main->callstack_y0+20,455,140+14+28,hwndMain,(HMENU)IDC_LISTBOX_CALLSTACK,param->hInstance,0);
  SendMessage(window_main->hwnd_listbox_callstack,WM_SETFONT,(WPARAM)window_main->hFont_courrier,TRUE);

  /* Boutons */
  x0 = window_main->callstack_x0 + 265;
  y0 = window_main->callstack_y0 + 140 + 14 + 28 + 16 + 1;
  window_main->hwnd_button_callstack_disassemble = CreateWindow("BUTTON","Disassemble",WS_TABSTOP|WS_VISIBLE|WS_CHILD|BS_DEFPUSHBUTTON,x0,y0,85,22,hwndMain,(HMENU)IDC_BUTTON_CALLSTACK_DISASSEMBLE,param->hInstance,NULL);
  x0 += 85 + 15;
  window_main->hwnd_button_callstack_clear = CreateWindow("BUTTON","Clear",WS_TABSTOP|WS_VISIBLE|WS_CHILD|BS_DEFPUSHBUTTON,x0,y0,85,22,hwndMain,(HMENU)IDC_BUTTON_CALLSTACK_CLEAR,param->hInstance,NULL);

  /* Libells */
  x0 = window_main->callstack_x0 + 10;
  y0 = window_main->callstack_y0 + 140 + 14 + 28 + 16 + 3 + 1;
  window_main->hwnd_static_callstack_entry = CreateWindowExA(0,"STATIC","# Entries :",WS_VISIBLE|WS_CHILD|SS_LEFT,x0,y0,70,16,hwndMain,(HMENU)IDC_STATIC_CALLSTACK_ENTRY,param->hInstance,NULL);
  x0 += 60;
  window_main->hwnd_static_callstack_count = CreateWindowExA(0,"STATIC","0",WS_VISIBLE|WS_CHILD|SS_LEFT,x0,y0,70,16,hwndMain,(HMENU)IDC_STATIC_CALLSTACK_COUNT,param->hInstance,NULL);

  /* On dsactive les boutons */
  EnableWindow(window_main->hwnd_button_callstack_disassemble,FALSE);
  EnableWindow(window_main->hwnd_button_callstack_clear,FALSE);


  /**************/
  /*** Output ***/
  /**************/
  window_main->output_x0 = 831;
  window_main->output_y0 = 255 + 190  + 14 + 28 + 4 + 3;

  /* Group Box Code */
  window_main->hwnd_groupbox_output = CreateWindowExA(0,"BUTTON"," Breakpoint / System Calls Output ", WS_CHILD | WS_VISIBLE | BS_GROUPBOX | WS_GROUP,
                                                      window_main->output_x0, window_main->output_y0, 477, 190 + 14 + 14 + 28 + 1, hwndMain, (HMENU)IDC_GROUPBOX_OUTPUT,param->hInstance,0);

  /* List Box */
  window_main->hwnd_listbox_output = CreateWindowEx(0,"LISTBOX", 0, WS_CHILD | WS_VISIBLE | WS_VSCROLL,
                                                    window_main->output_x0+10,window_main->output_y0+20,455,140 + 14 + 14 + 28,hwndMain,(HMENU)IDC_LISTBOX_OUTPUT,param->hInstance,0);
  SendMessage(window_main->hwnd_listbox_output,WM_SETFONT,(WPARAM)window_main->hFont_courrier,TRUE);

  /* Boutons */
  x0 = window_main->output_x0 + 265;
  y0 = window_main->output_y0 + 140 + 14 + 14 + 16 + 28 + 1;
  window_main->hwnd_button_output_save = CreateWindow("BUTTON","Save",WS_TABSTOP|WS_VISIBLE|WS_CHILD|BS_DEFPUSHBUTTON,x0,y0,55,22,hwndMain,(HMENU)IDC_BUTTON_OUTPUT_SAVE,param->hInstance,NULL);
  x0 += 55 + 15;
  window_main->hwnd_button_output_clear = CreateWindow("BUTTON","Clear",WS_TABSTOP|WS_VISIBLE|WS_CHILD|BS_DEFPUSHBUTTON,x0,y0,55,22,hwndMain,(HMENU)IDC_BUTTON_OUTPUT_CLEAR,param->hInstance,NULL);
  x0 += 55 + 15;
  window_main->hwnd_button_output_refresh = CreateWindow("BUTTON","Refresh",WS_TABSTOP|WS_VISIBLE|WS_CHILD|BS_DEFPUSHBUTTON,x0,y0,55,22,hwndMain,(HMENU)IDC_BUTTON_OUTPUT_REFRESH,param->hInstance,NULL);

  /* Cration de la Check Box Trace Syscall */
  x0 = window_main->output_x0 + 140;
  y0 = window_main->output_y0 + 140 + 14 + 14 + 16 + 1 + 28 + 1;
  window_main->hwnd_checkbox_trace_syscall = CreateWindowEx(0,"BUTTON","Trace System Calls",WS_VISIBLE|WS_CHILD|BS_AUTOCHECKBOX,x0,y0,120,20,hwndMain,(HMENU)IDC_CHECKBOX_TRACE_SYSCALL,param->hInstance,NULL); 

  /* Libells */
  x0 = window_main->output_x0 + 10;
  y0 = window_main->output_y0 + 140 + 14 + 14 + 16 + 3 + 28 + 1;
  window_main->hwnd_static_output_entry = CreateWindowExA(0,"STATIC","# Entries :",WS_VISIBLE|WS_CHILD|SS_LEFT,x0,y0,70,16,hwndMain,(HMENU)IDC_STATIC_OUTPUT_ENTRY,param->hInstance,NULL);
  x0 += 60;
  window_main->hwnd_static_output_count = CreateWindowExA(0,"STATIC","0",WS_VISIBLE|WS_CHILD|SS_LEFT,x0,y0,70,16,hwndMain,(HMENU)IDC_STATIC_OUTPUT_COUNT,param->hInstance,NULL);

  /* On dsactive les boutons */
  EnableWindow(window_main->hwnd_button_output_save,FALSE);
  EnableWindow(window_main->hwnd_button_output_clear,FALSE);
  EnableWindow(window_main->hwnd_button_output_refresh,FALSE);

  /*** Police ***/
  /* Police de caractre de la fentre */
  window_main->hFont = CreateFontA(14,0,0,0,FW_DONTCARE,FALSE,FALSE,FALSE,ANSI_CHARSET,
                                   OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY,DEFAULT_PITCH|FF_SWISS,"Arial");

  /* Operation */
  SendMessage(window_main->hwnd_groupbox_operation,WM_SETFONT,(WPARAM)window_main->hFont,TRUE);
  SendMessage(window_main->hwnd_static_operation,WM_SETFONT,(WPARAM)window_main->hFont,TRUE);
  SendMessage(window_main->hwnd_button_operation_clear,WM_SETFONT,(WPARAM)window_main->hFont,TRUE);
  SendMessage(window_main->hwnd_button_operation_reset_cycle,WM_SETFONT,(WPARAM)window_main->hFont,TRUE);
  SendMessage(window_main->hwnd_button_operation_reset_operation,WM_SETFONT,(WPARAM)window_main->hFont,TRUE);
  SendMessage(window_main->hwnd_button_operation_save,WM_SETFONT,(WPARAM)window_main->hFont,TRUE);
  SendMessage(window_main->hwnd_checkbox_history_operation,WM_SETFONT,(WPARAM)window_main->hFont,TRUE);
  SendMessage(window_main->hwnd_static_operation_count,WM_SETFONT,(WPARAM)window_main->hFont,TRUE);
  SendMessage(window_main->hwnd_edit_operation_count,WM_SETFONT,(WPARAM)window_main->hFont,TRUE);
  SendMessage(window_main->hwnd_static_cycle_count,WM_SETFONT,(WPARAM)window_main->hFont,TRUE);
  SendMessage(window_main->hwnd_edit_cycle_count,WM_SETFONT,(WPARAM)window_main->hFont,TRUE);
  SendMessage(window_main->hwnd_static_operation_data,WM_SETFONT,(WPARAM)window_main->hFont,TRUE);
  SendMessage(window_main->hwnd_edit_operation_data,WM_SETFONT,(WPARAM)window_main->hFont,TRUE);
  /* Code */
  SendMessage(window_main->hwnd_groupbox_code,WM_SETFONT,(WPARAM)window_main->hFont,TRUE);  
  SendMessage(window_main->hwnd_static_code,WM_SETFONT,(WPARAM)window_main->hFont,TRUE);
  SendMessage(window_main->hwnd_button_code_run,WM_SETFONT,(WPARAM)window_main->hFont,TRUE);
  SendMessage(window_main->hwnd_button_code_in,WM_SETFONT,(WPARAM)window_main->hFont,TRUE);
  SendMessage(window_main->hwnd_button_code_over,WM_SETFONT,(WPARAM)window_main->hFont,TRUE);
  SendMessage(window_main->hwnd_button_code_10x,WM_SETFONT,(WPARAM)window_main->hFont,TRUE);
  SendMessage(window_main->hwnd_button_code_100x,WM_SETFONT,(WPARAM)window_main->hFont,TRUE);
  SendMessage(window_main->hwnd_button_code_1000x,WM_SETFONT,(WPARAM)window_main->hFont,TRUE);
  SendMessage(window_main->hwnd_button_code_10000x,WM_SETFONT,(WPARAM)window_main->hFont,TRUE);
  SendMessage(window_main->hwnd_button_code_rts,WM_SETFONT,(WPARAM)window_main->hFont,TRUE);
  SendMessage(window_main->hwnd_button_code_jsr,WM_SETFONT,(WPARAM)window_main->hFont,TRUE);
  SendMessage(window_main->hwnd_button_code_jmp,WM_SETFONT,(WPARAM)window_main->hFont,TRUE);
  SendMessage(window_main->hwnd_button_code_vbl,WM_SETFONT,(WPARAM)window_main->hFont,TRUE);
  SendMessage(window_main->hwnd_button_code_interrupt,WM_SETFONT,(WPARAM)window_main->hFont,TRUE);
  SendMessage(window_main->hwnd_button_code_toolbox,WM_SETFONT,(WPARAM)window_main->hFont,TRUE);
  SendMessage(window_main->hwnd_button_code_gsos,WM_SETFONT,(WPARAM)window_main->hFont,TRUE);
  SendMessage(window_main->hwnd_button_code_highlight,WM_SETFONT,(WPARAM)window_main->hFont,TRUE);
  /* Call Stack */
  SendMessage(window_main->hwnd_groupbox_callstack,WM_SETFONT,(WPARAM)window_main->hFont,TRUE);  
  SendMessage(window_main->hwnd_button_callstack_disassemble,WM_SETFONT,(WPARAM)window_main->hFont,TRUE);  
  SendMessage(window_main->hwnd_button_callstack_clear,WM_SETFONT,(WPARAM)window_main->hFont,TRUE);
  SendMessage(window_main->hwnd_static_callstack_entry,WM_SETFONT,(WPARAM)window_main->hFont,TRUE);  
  SendMessage(window_main->hwnd_static_callstack_count,WM_SETFONT,(WPARAM)window_main->hFont,TRUE);
  /* Output */
  SendMessage(window_main->hwnd_groupbox_output,WM_SETFONT,(WPARAM)window_main->hFont,TRUE);
  //SendMessage(window_main->hwnd_listbox_output,WM_SETFONT,(WPARAM)window_main->hFont,TRUE);
  SendMessage(window_main->hwnd_button_output_save,WM_SETFONT,(WPARAM)window_main->hFont,TRUE);
  SendMessage(window_main->hwnd_button_output_clear,WM_SETFONT,(WPARAM)window_main->hFont,TRUE);
  SendMessage(window_main->hwnd_button_output_refresh,WM_SETFONT,(WPARAM)window_main->hFont,TRUE);
  SendMessage(window_main->hwnd_static_output_entry,WM_SETFONT,(WPARAM)window_main->hFont,TRUE);
  SendMessage(window_main->hwnd_static_output_count,WM_SETFONT,(WPARAM)window_main->hFont,TRUE);
  SendMessage(window_main->hwnd_checkbox_trace_syscall,WM_SETFONT,(WPARAM)window_main->hFont,TRUE);

  /** Dplace / Agrandit une fentre **/
  //SetWindowPos(window_main->hwnd_button_operation_save,NULL,975,218+100,0,0,SWP_NOZORDER|SWP_NOSIZE);   /* Dplace */
  //SetWindowPos(window_main->hwnd_groupbox_operation,NULL,0,0,1225,217+100,SWP_NOZORDER|SWP_NOMOVE);     /* Agrandit */

  /* Aucune ligne slectionne */
  window_main->code_selected_line = -1;

  /* Cyrene n'est pas connect  Kegs */
  window_main->code_status = CYRENE_STATUS_NOT_CONNECTED;

  /* MAJ de la fentre Main : Operation History + Code + Call Stack + Output */
  SendMessage(window_main->hwnd_main,HM_OPERATION_CODE_REFRESHWINDOW,0,0);

  /** On initialise les valeur de la QuickWatch **/
  for(i=0; i<10; i++)
    {
      window_main->tab_quickwatch_data[i].ram_type               = QUICKWATCH_RAMTYPE_RAM;
      window_main->tab_quickwatch_data[i].combo_length_alignment = QUICKWATCH_COMBO_8;
      window_main->tab_quickwatch_data[i].data_length            = QUICKWATCH_LENGTH_8;
      window_main->tab_quickwatch_data[i].data_alignment         = QUICKWATCH_ALIGNMENT_INTEL;
      window_main->tab_quickwatch_data[i].is_data_error          = 1;     /* Aucune Adresse de slectionne */
      window_main->tab_quickwatch_data[i].bank                   = 0x00;
      window_main->tab_quickwatch_data[i].address                = 0x0000;
    }

  /** Taille de la fentre au dmarrage ((avec la barre de titre et les bordures)) **/
  GetWindowRect(window_main->hwnd_main,&window_rect);
  window_main->window_init_width  = window_rect.right  - window_rect.left;
  window_main->window_init_height = window_rect.bottom - window_rect.top;

  /* Nombre de ligne dans la ListView au dmarrage de la fentre */
  window_main->nb_init_data_row    = MAIN_OPERATION_NB_ROW;
  window_main->nb_init_data_column = MAIN_OPERATION_NB_COLUMN;

  /* Nombre de lignes + Nombre de colonnes actuellement affich dans la ListView */
  window_main->nb_row    = window_main->nb_init_data_row;
  window_main->nb_column = window_main->nb_init_data_column;

  /* OK */
  return(0);
}


/******************************************************************************************/
/*  CodeRichEditStreamCallback() :  Fonction Call back pour remplir le Rich Edit du Code. */
/******************************************************************************************/
static DWORD CALLBACK CodeRichEditStreamCallback(DWORD *dwCookie, LPBYTE output_buffer, LONG nb_write_max, LONG *nb_write_rtn)
{
  int nb_remain_byte, nb_copy_byte;
  struct window_main *window_main;

  /* Rcupration du paramtre */
  window_main = (struct window_main *) dwCookie;

  /* Taille des donnes encore  copier */
  nb_remain_byte = window_main->rtf_data_code_size - window_main->rtf_write_code_offset;
  if(nb_remain_byte == 0)
    {
      /* Plus rien  copier */
      *nb_write_rtn = 0;
      return(0);
    }

  /* Taille des donnes que l'on est autoris  copier */
  nb_copy_byte = (nb_write_max >= nb_remain_byte) ? nb_remain_byte : nb_write_max;

  /* Nombre de byte transfrs */
  *nb_write_rtn = nb_copy_byte;
		  
  /** Copie les Byte dans le output buffer **/
  memcpy(output_buffer,&window_main->rtf_data_code[window_main->rtf_write_code_offset],nb_copy_byte);

  /* Dplace l'offset */
  window_main->rtf_write_code_offset += nb_copy_byte;

  /* OK */
  return(0);
}


/***********************************************************************************/
/*  UpdateOperationStatus() :  On vient de cliquer sur la case  cocher Operation. */
/***********************************************************************************/
static void UpdateOperationStatus(struct window_main *window_main, int status)
{
  int nb_operation;

  /** On vient de dcocher la case : On ne veut plus historiser **/
  if(status == 0)
    {
      /* On n'historise plus les oprations */
      window_main->operation_history_checkbox = 0;

      /* On supprime les valeurs historises (on garde juste quelques lignes de valeurs) */
      my_Memory(MEMORY_FREE_EXTRA_OPERATION,&window_main->operation_text_data_row,NULL);

      /* On remet en haut le scoll */
      window_main->operation_scroll_position = 0;
    }
  else 
    {
      /* On historise les oprations */
      window_main->operation_history_checkbox = 1;
    }

  /* On active/dsactive les boutons */
  my_Memory(MEMORY_GET_OPERATION_NB,&nb_operation,NULL);
  window_main->button_operation_status = (nb_operation == 0) ? FALSE : TRUE;
  EnableWindow(window_main->hwnd_button_operation_reset_operation,window_main->button_operation_status);
  EnableWindow(window_main->hwnd_button_operation_reset_cycle,window_main->button_operation_status);
  EnableWindow(window_main->hwnd_button_operation_save,window_main->button_operation_status);
  EnableWindow(window_main->hwnd_button_operation_clear,window_main->button_operation_status);

  /** On met  jour la Zone Text Edit et l'Ascenseur  **/
  UpdateOperationTextEdit(window_main,0);
}


/******************************************************************************************/
/*  UpdateTraceSyscallStatus() :  On vient de cliquer sur la case  cocher Trace Syscall. */
/******************************************************************************************/
static void UpdateTraceSyscallStatus(struct window_main *window_main, int status)
{
  /** On vient de dcocher la case : On ne veut plus tracer les System Call **/
  if(status == 0)
    {
      /* On ne trace plus les appels systmes */
      window_main->output_trace_syscall = 0;
    }
  else 
    {
      /* On trace les appels systmes */
      window_main->output_trace_syscall = 1;
    }
}


/*******************************************************************************************/
/*  UpdateOperationTextEdit() :  MAJ du contenu de la Zone Text Edit de Operation History. */
/*******************************************************************************************/
static void UpdateOperationTextEdit(struct window_main *window_main, int align_down)
{
  int i, offset, nb_operation, nb_display, first_operation_index, last_operation_index, scroll_position_max, extra_length;
  SCROLLINFO scrollinfo_edit;
  char operation_line[1024];
  char operation_tab[10240];
  struct operation *current_operation;

  /* Calcul de la taille Extra disponible */
  extra_length = window_main->nb_column - window_main->nb_init_data_column;

  /* Rcupre le nombre d'oprations historises */
  my_Memory(MEMORY_GET_OPERATION_NB,&nb_operation,NULL);

  /* Nombre de lignes max qu'il est possible d'afficher */
  nb_display = (nb_operation >= window_main->operation_text_data_row) ? window_main->operation_text_data_row : nb_operation;

  /* Position Max  da la ScrollBar [0-N] */
  scroll_position_max = nb_operation - nb_display;

  /* On aligne la ScrollBar sur le bas [0-N] */
  if(align_down == 1)
    window_main->operation_scroll_position = scroll_position_max;

  /** On dfinie Min / Max + Taille de la Page + Position **/
  scrollinfo_edit.cbSize = sizeof(SCROLLINFO);
  scrollinfo_edit.fMask = SIF_PAGE | SIF_RANGE | SIF_POS;
  scrollinfo_edit.nMin = 0;
  scrollinfo_edit.nMax = nb_operation-1;
  scrollinfo_edit.nPage = (nb_operation <= window_main->operation_text_data_row) ? nb_operation : window_main->operation_text_data_row;
  scrollinfo_edit.nPos = window_main->operation_scroll_position;
  SetScrollInfo(window_main->hwnd_scrollbar_operation,SB_CTL,&scrollinfo_edit,TRUE);

  /* Init */
  strcpy(operation_tab,"");

  /* Premire ligne  afficher [1-N] */
  first_operation_index = window_main->operation_scroll_position + 1;

  /* Dernire ligne  afficher [1-N] */
  last_operation_index = first_operation_index + nb_display - 1;

  /*** Construction du tableau ***/
  for(offset=0,i=first_operation_index; i<=last_operation_index && nb_display > 0; i++)
    {
      /* Rcupre les donnes d'une opration */
      my_Memory(MEMORY_GET_OPERATION,&i,&current_operation);
      if(current_operation == NULL)
        continue;

      /* Construit la ligne */
      BuildOperationLine(current_operation,i,last_operation_index,operation_line,extra_length);

      /* Ajoute la ligne au tableau */
      strcpy(&operation_tab[offset],operation_line);
      offset += (int) strlen(operation_line);
    }
      
  /* Rempli le contenu du Text Edit */
  SetWindowText(window_main->hwnd_edit_operation,operation_tab);
}


/****************************************************************/
/*  BuildOperationLine() :  Construit la ligne d'une Opration. */
/****************************************************************/
static void BuildOperationLine(struct operation *current_operation, int current_operation_index, int last_operation_index, char *operation_line, int extra_length)
{
  int status_c, status_z, status_i, status_d, status_x, status_m, status_v, status_n, status_e, status_b, length;
  char operand_1[16];
  char operand_2[16];
  char operand_3[16];
  char buffer_operand_comment[1024];
  char buffer_label[1024];
  char buffer_operation_id[256];
  char buffer_total_cycle[256];
  char buffer_comment_padding[1024];
  unsigned char operation_data[256];
  struct code_line *current_codeline;

  /* Status Register + MX */
  status_c = (current_operation->register_STATUS >> 0) & 0x01;     /* C */
  status_z = (current_operation->register_STATUS >> 1) & 0x01;     /* Z */
  status_i = (current_operation->register_STATUS >> 2) & 0x01;     /* I */
  status_d = (current_operation->register_STATUS >> 3) & 0x01;     /* D */
  status_x = (current_operation->register_STATUS >> 4) & 0x01;     /* X */
  status_m = (current_operation->register_STATUS >> 5) & 0x01;     /* M */
  status_v = (current_operation->register_STATUS >> 6) & 0x01;     /* V */
  status_n = (current_operation->register_STATUS >> 7) & 0x01;     /* N */
  status_e = (current_operation->register_STATUS >> 8) & 0x01;     /* E */
  status_b = (current_operation->register_STATUS >> 9) & 0x01;     /* B */

  /* Operand Hexa Ascii */
  sprintf(operand_1,"%02X",current_operation->m_operand1);
  sprintf(operand_2,"%02X",current_operation->m_operand2);
  sprintf(operand_3,"%02X",current_operation->m_operand3);

  /* Dsassemble la ligne */
  operation_data[0] = current_operation->m_opcode;
  operation_data[1] = current_operation->m_operand1;
  operation_data[2] = current_operation->m_operand2;
  operation_data[3] = current_operation->m_operand3;
  current_codeline = DisassembleOneLine(4,operation_data,(current_operation->register_PB << 16)+current_operation->register_PC,current_operation->register_DB,DISASSEMBLER_PROC_65C816,status_m,status_x,current_operation->call_number,NULL);

  /* Cration de la Partie Label (Coupe  20) */
  strcpy(buffer_label,current_codeline->label);
  strcat(buffer_label,"                    ");
  buffer_label[20] = '\0';

  /* Taille Max du Comment : 2+[11+ExtraLength] */
  memset(buffer_comment_padding,' ',2+11+extra_length);
  buffer_comment_padding[2+11+extra_length] = '\0';

  /** Cration de la partie Operand + Commentaire (l'Oprande peut dborder sur le commentaire) **/
  strcpy(buffer_operand_comment,current_codeline->operand);
  if(strlen(buffer_operand_comment) < 20)
    {
      /* On coupe  l'Operand  20 */
      strcat(buffer_operand_comment,"                    ");   /* Operand : 20 */
      buffer_operand_comment[20] = '\0';

      /* Ajoute le Comment */
      strcat(buffer_operand_comment,current_codeline->comment);
      strcat(buffer_operand_comment,buffer_comment_padding);   /* 2+[11+ExtraLength] */
      buffer_operand_comment[20+2+11+extra_length] = '\0';
    }
  else
    {
      /* On n'affiche que l'Oprande : On coupe  20+2+[11+ExtraLength] */
      strcat(buffer_operand_comment,"                    ");  /* Operand : 20 */
      strcat(buffer_operand_comment,buffer_comment_padding);  /* 2+[11+ExtraLength] */
      buffer_operand_comment[20+2+11+extra_length] = '\0';
    }

  /* Operation ID */
  sprintf(buffer_operation_id,"%09d",current_operation->operation_id);
  length = (int) strlen(buffer_operation_id);
  if(length > 9)
    memmove(&buffer_operation_id[0],&buffer_operation_id[length-9],strlen(&buffer_operation_id[length-9])+1);

  /* Total Cycle */
  sprintf(buffer_total_cycle,"%09d",current_operation->total_cycle);
  length = (int) strlen(buffer_total_cycle);
  if(length > 9)
    memmove(&buffer_total_cycle[0],&buffer_total_cycle[length-9],strlen(&buffer_total_cycle[length-9])+1);

  /* Cration d'une ligne */
  sprintf(operation_line,"%s | %04X %04X %04X %04X %04X %02X %02X %04X  %c%c%c%c%c%c.%c%c %d %d  %02X %02X  %02X  %02X | %s %d | %02X/%04X  %02X %s %s %s  %s  %-4s  %s%s",
                         buffer_operation_id,
                         current_operation->register_A,
                         current_operation->register_X,
                         current_operation->register_Y,
                         current_operation->register_STACK,
                         current_operation->register_DIRECT,
                         current_operation->register_DB,
                         current_operation->register_PB,
                         current_operation->register_PC,
                         (status_n == 1) ? 'N':'n',
                         (status_v == 1) ? 'V':'v',
                         (status_d == 1) ? 'D':'d',
                         (status_i == 1) ? 'I':'i',
                         (status_z == 1) ? 'Z':'z',
                         (status_c == 1) ? 'C':'c',
                         (status_e == 1) ? 'E':'e',
                         (status_b == 1) ? 'B':'b',
                         status_m,
                         status_x,
                         current_operation->s_C02E,
                         current_operation->s_C02F,
                         current_operation->s_C035,
                         current_operation->s_C068,
                         buffer_total_cycle,
                         current_operation->nb_cycle,
                         current_operation->register_PB,
                         current_operation->register_PC,
                         current_operation->m_opcode,
                         (current_codeline->data_size > 1) ? operand_1 : "  ",
                         (current_codeline->data_size > 2) ? operand_2 : "  ",
                         (current_codeline->data_size > 3) ? operand_3 : "  ",
                         buffer_label,
                         current_codeline->opcode,
                         buffer_operand_comment,
                         (current_operation_index==last_operation_index) ? "" : "\n");

  /* Libration mmoire */
  mem_free_codeline(current_codeline);
}


/******************************************************************/
/*  BuildCodeRichEdit() :  Construit la zone de code dsassembl. */
/******************************************************************/
static void BuildCodeRichEdit(struct window_main *window_main, int pc_address_param, int data_bank, int status_m_param, int status_x_param, struct mmu *current_mmu)
{
  int i, pc_address, status_m, status_x, bit_m, bit_x;
  char *next_sep;
  struct code_line *current_codeline;
  unsigned char operation_data[4];
  char operand_1[4];
  char operand_2[4];
  char operand_3[4];
  char buffer_operand_ascii[1024];
  char buffer_operand_color[1024];
  char buffer_comment_ascii[1024];
  char buffer_comment_color[1024];
  char buffer_operand_comment_ascii[1024];
  char buffer_operand_comment_color[1024];
  char buffer_label_ascii[1024];
  char buffer_label_color[1024];
  char buffer_address_ascii[256];
  char buffer_address_color[256];
  char code_line_ascii[1024];
  char code_line_color[1024];
  char buffer_color[256];

  /** Cration des Lignes dsassembles : Ascii + Color **/
  pc_address = pc_address_param;
  status_m = status_m_param;
  status_x = status_x_param;
  for(i=0; i<window_main->nb_code_line+1; i++)   /* On prend une ligne de plus pour grer le StepOver de la dernire ligne */
    {
      /* Dsassemble une ligne */
      operation_data[0] = ReadMemoryByte(pc_address+0,current_mmu);
      operation_data[1] = ReadMemoryByte(pc_address+1,current_mmu);
      operation_data[2] = ReadMemoryByte(pc_address+2,current_mmu);
      operation_data[3] = ReadMemoryByte(pc_address+3,current_mmu);
      current_codeline = DisassembleOneLine(4,operation_data,pc_address,data_bank,DISASSEMBLER_PROC_65C816,status_m,status_x,0xFFFF,NULL);   /* Pas de call_num */

      /* Teste si la ligne courrante est un JSR/JSR */
      if(pc_address == pc_address_param)
        {
          if(!stricmp(current_codeline->opcode,"JSR") || !stricmp(current_codeline->opcode,"JSL"))
            window_main->code_current_line_is_jsr = 1;
          else
            window_main->code_current_line_is_jsr = 0;
        }

      /* Adresse */
      sprintf(buffer_address_ascii,"%02X/%04X",(current_codeline->address >> 16) & 0x000000FF,current_codeline->address & 0x0000FFFF);
      sprintf(buffer_address_color,"\\cf%d %s\\cf%d ",
                                   (current_codeline->line_symbol == 1) ? CODE_RICHEDIT_COLOR_BLUE : CODE_RICHEDIT_COLOR_BLACK,
                                   buffer_address_ascii,
                                   CODE_RICHEDIT_COLOR_BLACK);

      /* Cration de la Partie Label (Coupe  24) */
      strcpy(buffer_label_ascii,current_codeline->label);
      strcat(buffer_label_ascii,"                        ");
      buffer_label_ascii[24] = '\0';
      sprintf(buffer_label_color,"\\cf%d %s\\cf%d ",
                                 (current_codeline->line_symbol == 1) ? CODE_RICHEDIT_COLOR_BLUE : CODE_RICHEDIT_COLOR_BLACK,
                                 buffer_label_ascii,
                                 CODE_RICHEDIT_COLOR_BLACK);

      /** Cration de la partie Operand + Commentaire (l'Oprande peut dborder sur le commentaire) **/
      strcpy(buffer_operand_ascii,current_codeline->operand);
      if(strlen(buffer_operand_ascii) < 11)
        {
          /* Operand : on coupe  11 */
          strcat(buffer_operand_ascii,"           ");
          buffer_operand_ascii[11] = '\0';

          /* Comment : on coupe  2+28 */
          strcpy(buffer_comment_ascii,current_codeline->comment);
          strcat(buffer_comment_ascii,"                              ");
          buffer_comment_ascii[2+28] = '\0';

          /* Operand Couleur */
          strcpy(buffer_operand_color,buffer_operand_ascii);
          if(strstr(buffer_operand_color," +") != NULL || strstr(buffer_operand_color," -") != NULL)
            {
              /* Couleur Gris Fonc */
              sprintf(buffer_color,"\\cf%d ",DISASSEMBLER_RICHEDIT_COLOR_DARKGREY);

              /* Opcode contenant un +/-offet => colorise en gris le +- */
              next_sep = strchr(buffer_operand_color,' ');
              next_sep++;
              memmove(next_sep+strlen(buffer_color),next_sep,strlen(next_sep)+1);
              memcpy(next_sep,buffer_color,strlen(buffer_color));
              next_sep += strlen(buffer_color);

              /* Couleur Noir */
              sprintf(buffer_color,"\\cf%d ",DISASSEMBLER_RICHEDIT_COLOR_BLACK);
              next_sep = strchr(buffer_operand_color,' ');

              next_sep = strchr(buffer_operand_color,' ');
              memmove(next_sep+strlen(buffer_color),next_sep,strlen(next_sep)+1);
              memcpy(next_sep,buffer_color,strlen(buffer_color));
            }
          else
            {
              /* Operand Couleur */
              sprintf(buffer_operand_color,"\\cf%d %s\\cf%d ",(current_codeline->operand_symbol == 1)?CODE_RICHEDIT_COLOR_BLUE:CODE_RICHEDIT_COLOR_BLACK,
                                                              buffer_operand_ascii,
                                                              CODE_RICHEDIT_COLOR_BLACK);
            }

          /* Comment Couleur = Vert */
          sprintf(buffer_comment_color,"\\cf%d %s\\cf%d ",CODE_RICHEDIT_COLOR_GREEN,buffer_comment_ascii,CODE_RICHEDIT_COLOR_BLACK);

          /* Buffer */
          sprintf(buffer_operand_comment_ascii,"%s%s",buffer_operand_ascii,buffer_comment_ascii);
          sprintf(buffer_operand_comment_color,"%s%s",buffer_operand_color,buffer_comment_color);
        }
      else
        {
          /* On coupe  11+2+28 */
          strcpy(buffer_operand_comment_ascii,buffer_operand_ascii);
          strcat(buffer_operand_comment_ascii,"                                         ");
          buffer_operand_comment_ascii[11+2+28] = '\0';

          /* Couleur Bleu */
          sprintf(buffer_operand_comment_color,"\\cf%d %s\\cf%d ",CODE_RICHEDIT_COLOR_BLUE,buffer_operand_comment_ascii,CODE_RICHEDIT_COLOR_BLACK);
        }

      /* Operand Data */
      strcpy(operand_1,"  ");
      strcpy(operand_2,"  ");
      strcpy(operand_3,"  ");
      if(current_codeline->data_size > 1)
        sprintf(operand_1,"%02X",current_codeline->data[1]);
      if(current_codeline->data_size > 2)
        sprintf(operand_2,"%02X",current_codeline->data[2]);
      if(current_codeline->data_size > 3)
        sprintf(operand_3,"%02X",current_codeline->data[3]);

      /** Cration de la ligne Texte **/
      sprintf(code_line_ascii,"%d%d %s  %02X %s %s %s  %s  %-4s  %s",
                              status_m,
                              status_x,
                              buffer_address_ascii,
                              current_codeline->data[0],     /* Opcode Byte */
                              operand_1,                     /* Operand Byte #1 */
                              operand_2,                     /* Operand Byte #2 */
                              operand_3,                     /* Operand Byte #3 */
                              buffer_label_ascii,            /* Label */
                              current_codeline->opcode,      /* Opcode */
                              buffer_operand_comment_ascii); /* Operand + Comment */

      /** Cration de la ligne en couleur **/
      sprintf(code_line_color,"%d%d %s  %02X %s %s %s  %s  %-4s  %s",
                              status_m,
                              status_x,
                              buffer_address_color,
                              current_codeline->data[0],     /* Opcode Byte */
                              operand_1,                     /* Operand Byte #1 */
                              operand_2,                     /* Operand Byte #2 */
                              operand_3,                     /* Operand Byte #3 */
                              buffer_label_color,            /* Label */
                              current_codeline->opcode,      /* Opcode */
                              buffer_operand_comment_color); /* Operand + Comment */

      /** Conserve les donne de cette ligne **/
      window_main->tab_code_line_address[i] = current_codeline->address;
      window_main->tab_code_line_m[i] = status_m;
      window_main->tab_code_line_x[i] = status_x;
      window_main->tab_code_line_ascii[i] = strdup(code_line_ascii);
      window_main->tab_code_line_color[i] = strdup(code_line_color);
      if(window_main->tab_code_line_ascii[i] == NULL || window_main->tab_code_line_color[i] == NULL)
        ;

      /** Adresse de la ligne suivante **/
      pc_address += current_codeline->data_size;

      /** On va grer les SEP et les REP pour influencer les MX **/
      if(!stricmp(current_codeline->opcode,"SEP"))
        {
          bit_m = (current_codeline->data[1] & 0x20) >> 5;
          if(bit_m == 1)
            status_m = 1;
          bit_x = (current_codeline->data[1] & 0x10) >> 4;
          if(bit_x == 1)
            status_x = 1;
        }
      if(!stricmp(current_codeline->opcode,"REP"))
        {
          bit_m = (current_codeline->data[1] & 0x20) >> 5;
          if(bit_m == 1)
            status_m = 0;
          bit_x = (current_codeline->data[1] & 0x10) >> 4;
          if(bit_x == 1)
            status_x = 0;
        }

      /* Libration mmoire */
      mem_free_codeline(current_codeline);
    }
}


/**********************************************************************************/
/*  DisplayCodeRichEdit() :  Affiche les lignes dans la zone de code dsassembl. */
/**********************************************************************************/
static void DisplayCodeRichEdit(struct window_main *window_main)
{
  int i, offset_rtf;
  EDITSTREAM edit_stream;
  char buffer_color[256];

  /*** Cration d'un buffer RTF rpentant les lignes dsassembles ***/
  window_main->rtf_data_code[0] = '\0';
  strcat(window_main->rtf_data_code,"{\\rtf1\\ansi\\ansicpg1252\\deff0\\deflang1036{\\fonttbl{\\f0\\fnil\\fcharset0 Courier New;}}\r\n");
  strcat(window_main->rtf_data_code,"{\\colortbl ;");
  /* Couleurs */
  strcat(window_main->rtf_data_code,"\\red255\\green0\\blue0;");      /* SNAPSHOT_CODE65C816_RICHEDIT_COLOR_RED */
  strcat(window_main->rtf_data_code,"\\red0\\green77\\blue187;");     /* SNAPSHOT_CODE65C816_RICHEDIT_COLOR_BLUE */
  strcat(window_main->rtf_data_code,"\\red0\\green128\\blue0;");      /* SNAPSHOT_CODE65C816_RICHEDIT_COLOR_GREEN */
  strcat(window_main->rtf_data_code,"\\red192\\green80\\blue77;");    /* SNAPSHOT_CODE65C816_RICHEDIT_COLOR_DARKRED */
  strcat(window_main->rtf_data_code,"\\red128\\green128\\blue128;");  /* SNAPSHOT_CODE65C816_RICHEDIT_COLOR_DARKGREY */
  strcat(window_main->rtf_data_code,"\\red163\\green73\\blue164;");   /* SNAPSHOT_CODE65C816_RICHEDIT_COLOR_PURPLE */
  strcat(window_main->rtf_data_code,"\\red0\\green120\\blue215;");    /* SNAPSHOT_CODE65C816_RICHEDIT_COLOR_PETROL */
  strcat(window_main->rtf_data_code,"\\red255\\green255\\blue255;");  /* SNAPSHOT_CODE65C816_RICHEDIT_COLOR_WHITE */
  strcat(window_main->rtf_data_code,"\\red192\\green192\\blue192;");  /* SNAPSHOT_CODE65C816_RICHEDIT_COLOR_LIGHTGREY */
  strcat(window_main->rtf_data_code,"}\r\n");
  strcat(window_main->rtf_data_code,"{\\*\\generator Msftedit 5.41.21.2510;}\\viewkind4\\uc1\\pard\\sl240\\slmult1\\lang12\\f0\\fs");
  strcat(window_main->rtf_data_code,CODE_RICHEDIT_FONT_HEIGHT_RTF);   /* Taille de la Police : Si 8 dans WordPad => 16 ici (x2)*/
  strcat(window_main->rtf_data_code," ");
  offset_rtf = (int) strlen(window_main->rtf_data_code);

  /** Ajoute les lignes dans le buffer **/
  for(i=0; i<window_main->nb_code_line; i++)
    {
      /* Aucune ligne dsassemble */
      if(window_main->tab_code_line_ascii[i] == NULL || window_main->tab_code_line_color[i] == NULL)
        break;

      /** Ajoute au buffer Rich Edit **/
      if(i== 0)                                       /** Ligne en cours **/
        {
          /* Couleur de Fond : Bleu Ptrol */
          sprintf(buffer_color,"\\highlight%d ",CODE_RICHEDIT_COLOR_PETROL);
          strcpy(&window_main->rtf_data_code[offset_rtf],buffer_color);
          offset_rtf += (int) strlen(buffer_color);
          /* Couleur du Texte : Blanc */
          sprintf(buffer_color,"\\cf%d ",CODE_RICHEDIT_COLOR_WHITE);
          strcpy(&window_main->rtf_data_code[offset_rtf],buffer_color);
          offset_rtf += (int) strlen(buffer_color);

          /* Donnes de la ligne */
          strcpy(&window_main->rtf_data_code[offset_rtf],window_main->tab_code_line_ascii[i]);
          offset_rtf += (int) strlen(window_main->tab_code_line_ascii[i]);

          /* Couleur du Texte : Noir */
          sprintf(buffer_color,"\\cf%d ",DISASSEMBLER_RICHEDIT_COLOR_BLACK);
          strcpy(&window_main->rtf_data_code[offset_rtf],buffer_color);
          offset_rtf += (int) strlen(buffer_color);
          /* Couleur de fond */
          strcat(&window_main->rtf_data_code[offset_rtf],"\\highlight0 ");
          offset_rtf += (int) strlen("\\highlight0 ");
        }
      else if(i == window_main->code_selected_line)          /** Ligne slectionne **/
        {
          /* Couleur de Fond : Bleu Ptrol */
          sprintf(buffer_color,"\\highlight%d ",CODE_RICHEDIT_COLOR_LIGHTGREY);
          strcpy(&window_main->rtf_data_code[offset_rtf],buffer_color);
          offset_rtf += (int) strlen(buffer_color);
          /* Couleur du Texte : Blanc */
          sprintf(buffer_color,"\\cf%d ",CODE_RICHEDIT_COLOR_BLACK);
          strcpy(&window_main->rtf_data_code[offset_rtf],buffer_color);
          offset_rtf += (int) strlen(buffer_color);

          /* Donnes de la ligne */
          strcpy(&window_main->rtf_data_code[offset_rtf],window_main->tab_code_line_ascii[i]);
          offset_rtf += (int) strlen(window_main->tab_code_line_ascii[i]);

          /* Couleur du Texte : Noir */
          sprintf(buffer_color,"\\cf%d ",DISASSEMBLER_RICHEDIT_COLOR_BLACK);
          strcpy(&window_main->rtf_data_code[offset_rtf],buffer_color);
          offset_rtf += (int) strlen(buffer_color);
          /* Couleur de fond */
          strcat(&window_main->rtf_data_code[offset_rtf],"\\highlight0 ");
          offset_rtf += (int) strlen("\\highlight0 ");
        }
      else
        {
          /* Donnes de la ligne couleur */
          strcpy(&window_main->rtf_data_code[offset_rtf],window_main->tab_code_line_color[i]);
          offset_rtf += (int) strlen(window_main->tab_code_line_color[i]);
        }

      /* Fin de la ligne (sauf la dernire) */
      if(i != (window_main->nb_code_line-1))
        {
          strcpy(&window_main->rtf_data_code[offset_rtf],"\\par\r\n");
          offset_rtf += (int) strlen("\\par\r\n");
        }
    }

  /** Fin du RTF **/
  strcpy(&window_main->rtf_data_code[offset_rtf],"}\r\n");
  offset_rtf += (int) strlen("}\r\n");
  window_main->rtf_data_code[offset_rtf] = '\0';

  /* Taille du RTF (le \0 final compte) */
  window_main->rtf_data_code_size = (int) strlen(window_main->rtf_data_code) + 1;

  /** Afficher le rsultat en RTF dans le control Rich Edit **/
  window_main->rtf_write_code_offset = 0;
  memset(&edit_stream,0,sizeof(EDITSTREAM));
  edit_stream.pfnCallback = (EDITSTREAMCALLBACK) CodeRichEditStreamCallback;
  edit_stream.dwCookie = (DWORD_PTR) window_main;          /* Paramtre  passer  la fonction Callback */
  SendMessage(window_main->hwnd_richedit_code,EM_STREAMIN,SF_RTF,(LPARAM)&edit_stream);
}


/**********************************************************************************/
/*  ClearCodeRichEdit() :  Nettoyage des lignes dans la zone de code dsassembl. */
/**********************************************************************************/
static void ClearCodeRichEdit(struct window_main *window_main)
{
  int offset_rtf;
  EDITSTREAM edit_stream;

  /*** Cration d'un buffer RTF rpentant les lignes dsassembles ***/
  window_main->rtf_data_code[0] = '\0';
  strcat(window_main->rtf_data_code,"{\\rtf1\\ansi\\ansicpg1252\\deff0\\deflang1036{\\fonttbl{\\f0\\fnil\\fcharset0 Courier New;}}\r\n");
  strcat(window_main->rtf_data_code,"{\\colortbl ;");
  /* Couleurs */
  strcat(window_main->rtf_data_code,"\\red255\\green0\\blue0;");      /* SNAPSHOT_CODE65C816_RICHEDIT_COLOR_RED */
  strcat(window_main->rtf_data_code,"\\red0\\green77\\blue187;");     /* SNAPSHOT_CODE65C816_RICHEDIT_COLOR_BLUE */
  strcat(window_main->rtf_data_code,"\\red0\\green128\\blue0;");      /* SNAPSHOT_CODE65C816_RICHEDIT_COLOR_GREEN */
  strcat(window_main->rtf_data_code,"\\red192\\green80\\blue77;");    /* SNAPSHOT_CODE65C816_RICHEDIT_COLOR_DARKRED */
  strcat(window_main->rtf_data_code,"\\red128\\green128\\blue128;");  /* SNAPSHOT_CODE65C816_RICHEDIT_COLOR_DARKGREY */
  strcat(window_main->rtf_data_code,"\\red163\\green73\\blue164;");   /* SNAPSHOT_CODE65C816_RICHEDIT_COLOR_PURPLE */
  strcat(window_main->rtf_data_code,"\\red0\\green120\\blue215;");    /* SNAPSHOT_CODE65C816_RICHEDIT_COLOR_PETROL */
  strcat(window_main->rtf_data_code,"\\red255\\green255\\blue255;");  /* SNAPSHOT_CODE65C816_RICHEDIT_COLOR_WHITE */
  strcat(window_main->rtf_data_code,"\\red192\\green192\\blue192;");  /* SNAPSHOT_CODE65C816_RICHEDIT_COLOR_LIGHTGREY */
  strcat(window_main->rtf_data_code,"}\r\n");
  strcat(window_main->rtf_data_code,"{\\*\\generator Msftedit 5.41.21.2510;}\\viewkind4\\uc1\\pard\\sl240\\slmult1\\lang12\\f0\\fs");
  strcat(window_main->rtf_data_code,CODE_RICHEDIT_FONT_HEIGHT_RTF);   /* Taille de la Police : Si 8 dans WordPad => 16 ici (x2)*/
  strcat(window_main->rtf_data_code," ");
  offset_rtf = (int) strlen(window_main->rtf_data_code);

  /** Fin du RTF **/
  strcpy(&window_main->rtf_data_code[offset_rtf],"}\r\n");
  offset_rtf += (int) strlen("}\r\n");
  window_main->rtf_data_code[offset_rtf] = '\0';

  /* Taille du RTF (le \0 final compte) */
  window_main->rtf_data_code_size = (int) strlen(window_main->rtf_data_code) + 1;

  /** Afficher le rsultat en RTF dans le control Rich Edit **/
  window_main->rtf_write_code_offset = 0;
  memset(&edit_stream,0,sizeof(EDITSTREAM));
  edit_stream.pfnCallback = (EDITSTREAMCALLBACK) CodeRichEditStreamCallback;
  edit_stream.dwCookie = (DWORD_PTR) window_main;          /* Paramtre  passer  la fonction Callback */
  SendMessage(window_main->hwnd_richedit_code,EM_STREAMIN,SF_RTF,(LPARAM)&edit_stream);
}


/*******************************************************************************/
/*  ProcessOperationButton() :  Traite les actions des boutons des Operations. */
/*******************************************************************************/
int ProcessOperationButton(struct window_main *window_main, int button_id)
{
  int next_address;
  char buffer_stop[256];

  /** Bouton Run / Stop **/
  if(button_id == IDC_BUTTON_CODE_RUN)
    {
      /** Bouton Run / Stop **/
      if(window_main->code_status == CYRENE_STATUS_WAIT_FOR_DATA)       /** STOP **/
        {
          /* On demande  KEGS de s'arrter et d'envoyer les Data Snapshot */
          SendDataToKegs(IPC_REQUEST_TYPE_C2K_GET_SNAPSHOT,0,NULL);

          /* On attend que Kegs envoie les dernire donnes et vide la queue */
          window_main->code_status = CYRENE_STATUS_WAIT_FOR_STOP;
        }
      else if(window_main->code_status == CYRENE_STATUS_STOPPED_ON_OPERATION)  /** RUN **/
        {
          /* MAJ des Breakpoint dans la Shared Memory */
          UpdateBreapointTableInSharedMemory();

          /* MAJ de la Trace des Appels Systmes dans l'Output */
          UpdateTraceSystemCallInSharedMemory();

          /* On demande  KEGS d'envoyer les Operations Data (sans arrt planifi autre que les Breakpoints) */
          SendDataToKegs(IPC_REQUEST_TYPE_C2K_GET_OPERATION,(int)strlen("")+1,(unsigned char *)"");;

          /* On attend que Kegs envoie les donnes */
          window_main->code_status = CYRENE_STATUS_WAIT_FOR_DATA;
        }
    }
  /** Step In **/
  else if(button_id == IDC_BUTTON_CODE_IN)
    {
      /* MAJ des Breakpoint dans la Shared Memory */
      UpdateBreapointTableInSharedMemory();

      /* MAJ de la Trace des Appels Systmes dans l'Output */
      UpdateTraceSystemCallInSharedMemory();

      /* On va demander  KEGS d'envoyer 1 Operation et le Data Snapshot */
      SendDataToKegs(IPC_REQUEST_TYPE_C2K_GET_OPERATION,(int)strlen("Stop=x1")+1,(unsigned char *)"Stop=x1");

      /* On attend que Kegs envoie les donnes */
      window_main->code_status = CYRENE_STATUS_WAIT_FOR_DATA;
    }
  /** Step Over **/
  else if(button_id == IDC_BUTTON_CODE_OVER)
    {
      /* MAJ des Breakpoint dans la Shared Memory */
      UpdateBreapointTableInSharedMemory();

      /* MAJ de la Trace des Appels Systmes dans l'Output */
      UpdateTraceSystemCallInSharedMemory();

      /** On va demander  KEGS d'envoyer les Operations jusqu' une @ et le Data Snapshot **/
      next_address = window_main->tab_code_line_address[window_main->code_current_line+1];
      sprintf(buffer_stop,"Stop=@%06X",next_address);
      SendDataToKegs(IPC_REQUEST_TYPE_C2K_GET_OPERATION,(int)strlen(buffer_stop)+1,(unsigned char *)buffer_stop);

      /* On attend que Kegs envoie les donnes */
      window_main->code_status = CYRENE_STATUS_WAIT_FOR_DATA;
    }
  /** Next 10 **/
  else if(button_id == IDC_BUTTON_CODE_10X)
    {
      /* MAJ des Breakpoint dans la Shared Memory */
      UpdateBreapointTableInSharedMemory();

      /* MAJ de la Trace des Appels Systmes dans l'Output */
      UpdateTraceSystemCallInSharedMemory();

      /* On va demander  KEGS d'envoyer 10 Operations et le Data Snapshot */
      SendDataToKegs(IPC_REQUEST_TYPE_C2K_GET_OPERATION,(int)strlen("Stop=x10")+1,(unsigned char *)"Stop=x10");

      /* On attend que Kegs envoie les donnes */
      window_main->code_status = CYRENE_STATUS_WAIT_FOR_DATA;
    }
  /** Next 100 **/
  else if(button_id == IDC_BUTTON_CODE_100X)
    {
      /* MAJ des Breakpoint dans la Shared Memory */
      UpdateBreapointTableInSharedMemory();

      /* MAJ de la Trace des Appels Systmes dans l'Output */
      UpdateTraceSystemCallInSharedMemory();

      /* On va demander  KEGS d'envoyer 100 Operations et le Data Snapshot */
      SendDataToKegs(IPC_REQUEST_TYPE_C2K_GET_OPERATION,(int)strlen("Stop=x100")+1,(unsigned char *)"Stop=x100");

      /* On attend que Kegs envoie les donnes */
      window_main->code_status = CYRENE_STATUS_WAIT_FOR_DATA;
    }
  /** Next 1000 **/
  else if(button_id == IDC_BUTTON_CODE_1000X)
    {
      /* MAJ des Breakpoint dans la Shared Memory */
      UpdateBreapointTableInSharedMemory();

      /* MAJ de la Trace des Appels Systmes dans l'Output */
      UpdateTraceSystemCallInSharedMemory();

      /* On va demander  KEGS d'envoyer 1000 Operations et le Data Snapshot */
      SendDataToKegs(IPC_REQUEST_TYPE_C2K_GET_OPERATION,(int)strlen("Stop=x1000")+1,(unsigned char *)"Stop=x1000");

      /* On attend que Kegs envoie les donnes */
      window_main->code_status = CYRENE_STATUS_WAIT_FOR_DATA;
    }
  /** Next 10000 **/
  else if(button_id == IDC_BUTTON_CODE_10000X)
    {
      /* MAJ des Breakpoint dans la Shared Memory */
      UpdateBreapointTableInSharedMemory();

      /* MAJ de la Trace des Appels Systmes dans l'Output */
      UpdateTraceSystemCallInSharedMemory();

      /* On va demander  KEGS d'envoyer 10000 Operations et le Data Snapshot */
      SendDataToKegs(IPC_REQUEST_TYPE_C2K_GET_OPERATION,(int)strlen("Stop=x10000")+1,(unsigned char *)"Stop=x10000");

      /* On attend que Kegs envoie les donnes */
      window_main->code_status = CYRENE_STATUS_WAIT_FOR_DATA;
    }
  /** Next JSR/JSL **/
  else if(button_id == IDC_BUTTON_CODE_JSR)
    {
      /* MAJ des Breakpoint dans la Shared Memory */
      UpdateBreapointTableInSharedMemory();

      /* MAJ de la Trace des Appels Systmes dans l'Output */
      UpdateTraceSystemCallInSharedMemory();

      /* On va demander  KEGS d'envoyer les Operations jusqu' un JSR et le Data Snapshot */
      SendDataToKegs(IPC_REQUEST_TYPE_C2K_GET_OPERATION,(int)strlen("Stop=JSR")+1,(unsigned char *)"Stop=JSR");

      /* On attend que Kegs envoie les donnes */
      window_main->code_status = CYRENE_STATUS_WAIT_FOR_DATA;
    }
  /** Next RTS **/
  else if(button_id == IDC_BUTTON_CODE_RTS)
    {
      /* MAJ des Breakpoint dans la Shared Memory */
      UpdateBreapointTableInSharedMemory();

      /* MAJ de la Trace des Appels Systmes dans l'Output */
      UpdateTraceSystemCallInSharedMemory();

      /* On va demander  KEGS d'envoyer les Operations jusqu' un RTS et le Data Snapshot */
      SendDataToKegs(IPC_REQUEST_TYPE_C2K_GET_OPERATION,(int)strlen("Stop=RTS")+1,(unsigned char *)"Stop=RTS");

      /* On attend que Kegs envoie les donnes */
      window_main->code_status = CYRENE_STATUS_WAIT_FOR_DATA;
    }
  /** Next JMP/Branch **/
  else if(button_id == IDC_BUTTON_CODE_JMP)
    {
      /* MAJ des Breakpoint dans la Shared Memory */
      UpdateBreapointTableInSharedMemory();

      /* MAJ de la Trace des Appels Systmes dans l'Output */
      UpdateTraceSystemCallInSharedMemory();

      /* On va demander  KEGS d'envoyer les Operations jusqu' un JMP et le Data Snapshot */
      SendDataToKegs(IPC_REQUEST_TYPE_C2K_GET_OPERATION,(int)strlen("Stop=JMP")+1,(unsigned char *)"Stop=JMP");

      /* On attend que Kegs envoie les donnes */
      window_main->code_status = CYRENE_STATUS_WAIT_FOR_DATA;
    }
  /** Next VBL **/
  else if(button_id == IDC_BUTTON_CODE_VBL)
    {
      /* MAJ des Breakpoint dans la Shared Memory */
      UpdateBreapointTableInSharedMemory();

      /* MAJ de la Trace des Appels Systmes dans l'Output */
      UpdateTraceSystemCallInSharedMemory();

      /* On va demander  KEGS d'envoyer les Operations jusqu' une VBL et le Data Snapshot */
      SendDataToKegs(IPC_REQUEST_TYPE_C2K_GET_OPERATION,(int)strlen("Stop=VBL")+1,(unsigned char *)"Stop=VBL");

      /* On attend que Kegs envoie les donnes */
      window_main->code_status = CYRENE_STATUS_WAIT_FOR_DATA;
    }
  /** Next Interrupt **/
  else if(button_id == IDC_BUTTON_CODE_INTERRUPT)
    {
      /* MAJ des Breakpoint dans la Shared Memory */
      UpdateBreapointTableInSharedMemory();

      /* MAJ de la Trace des Appels Systmes dans l'Output */
      UpdateTraceSystemCallInSharedMemory();

      /* On va demander  KEGS d'envoyer les Operations jusqu' une Interruption et le Data Snapshot */
      SendDataToKegs(IPC_REQUEST_TYPE_C2K_GET_OPERATION,(int)strlen("Stop=INTERRUPT")+1,(unsigned char *)"Stop=INTERRUPT");

      /* On attend que Kegs envoie les donnes */
      window_main->code_status = CYRENE_STATUS_WAIT_FOR_DATA;
    }
  /** Next Toolbox **/
  else if(button_id == IDC_BUTTON_CODE_TOOLBOX)
    {
      /* MAJ des Breakpoint dans la Shared Memory */
      UpdateBreapointTableInSharedMemory();

      /* MAJ de la Trace des Appels Systmes dans l'Output */
      UpdateTraceSystemCallInSharedMemory();

      /* On va demander  KEGS d'envoyer les Operations jusqu' un appel  la Toolbox et le Data Snapshot */
      SendDataToKegs(IPC_REQUEST_TYPE_C2K_GET_OPERATION,(int)strlen("Stop=TOOLBOX")+1,(unsigned char *)"Stop=TOOLBOX");

      /* On attend que Kegs envoie les donnes */
      window_main->code_status = CYRENE_STATUS_WAIT_FOR_DATA;
    }
  /** Next P8 / P16 / GS/OS **/
  else if(button_id == IDC_BUTTON_CODE_GSOS)
    {
      /* MAJ des Breakpoint dans la Shared Memory */
      UpdateBreapointTableInSharedMemory();

      /* MAJ de la Trace des Appels Systmes dans l'Output */
      UpdateTraceSystemCallInSharedMemory();

      /* On va demander  KEGS d'envoyer les Operations jusqu' un appel  P8 / P16 / GS/OS et le Data Snapshot */
      SendDataToKegs(IPC_REQUEST_TYPE_C2K_GET_OPERATION,(int)strlen("Stop=OS")+1,(unsigned char *)"Stop=OS");

      /* On attend que Kegs envoie les donnes */
      window_main->code_status = CYRENE_STATUS_WAIT_FOR_DATA;
    }
  /** Next Highlight **/
  else if(button_id == IDC_BUTTON_CODE_HIGHLIGHT)
    {
      /* MAJ des Breakpoint dans la Shared Memory */
      UpdateBreapointTableInSharedMemory();

      /* MAJ de la Trace des Appels Systmes dans l'Output */
      UpdateTraceSystemCallInSharedMemory();

      /** On va demander  KEGS d'envoyer les Operations jusqu' une @ et le Data Snapshot **/
      next_address = window_main->tab_code_line_address[window_main->code_selected_line];
      sprintf(buffer_stop,"Stop=@%06X",next_address);
      SendDataToKegs(IPC_REQUEST_TYPE_C2K_GET_OPERATION,(int)strlen(buffer_stop)+1,(unsigned char *)buffer_stop);

      /* On attend que Kegs envoie les donnes */
      window_main->code_status = CYRENE_STATUS_WAIT_FOR_DATA;
    }

  /* On d-slectionne la ligne */
  window_main->code_selected_line = -1;

  /* On met  jour les boutons que la partie Code + Operation*/
  UpdateCodeButtonStatus(window_main);

  /* OK */
  return(0);
}


/*****************************************************************************/
/*  UpdateCodeButtonStatus() :  On met  jour le Status des Boutons du code. */
/*****************************************************************************/
static void UpdateCodeButtonStatus(struct window_main *window_main)
{
  /** Cyrene affiche la dernire Operation et attend l'ordre de l'oprateur **/
  if(window_main->code_status == CYRENE_STATUS_STOPPED_ON_OPERATION)
    {
      /* Le bouton passe  'Run' */
      SetWindowText(window_main->hwnd_button_code_run,"Run");
      EnableWindow(window_main->hwnd_button_code_run,TRUE);

      /* On active les boutons Operation */
      EnableWindow(window_main->hwnd_button_operation_reset_operation,TRUE);
      EnableWindow(window_main->hwnd_button_operation_reset_cycle,TRUE);
      EnableWindow(window_main->hwnd_button_operation_save,TRUE);
      EnableWindow(window_main->hwnd_button_operation_clear,TRUE);

      /* On active les boutons Code */
      EnableWindow(window_main->hwnd_button_code_in,TRUE);
      EnableWindow(window_main->hwnd_button_code_10x,TRUE);
      EnableWindow(window_main->hwnd_button_code_100x,TRUE);
      EnableWindow(window_main->hwnd_button_code_1000x,TRUE);
      EnableWindow(window_main->hwnd_button_code_10000x,TRUE);
      EnableWindow(window_main->hwnd_button_code_jmp,TRUE);
      EnableWindow(window_main->hwnd_button_code_jsr,TRUE);
      EnableWindow(window_main->hwnd_button_code_rts,TRUE);
      EnableWindow(window_main->hwnd_button_code_vbl,TRUE);
      EnableWindow(window_main->hwnd_button_code_interrupt,TRUE);
      EnableWindow(window_main->hwnd_button_code_toolbox,TRUE);
      EnableWindow(window_main->hwnd_button_code_gsos,TRUE);

      /* Le bouton 'Step Over' passe  actif en cas de JSR/JSL */
      if(window_main->code_current_line_is_jsr == 1)
        EnableWindow(window_main->hwnd_button_code_over,TRUE);
      else
        EnableWindow(window_main->hwnd_button_code_over,FALSE);

      /* Le bouton Highlight reste inactif tant qu'il n'y a pas de slection */
      if(window_main->code_selected_line >= 0)
        EnableWindow(window_main->hwnd_button_code_highlight,TRUE);
      else
        EnableWindow(window_main->hwnd_button_code_highlight,FALSE);

      /* On active/dsactive les touches Accelerator */
      window_main->acc_code_run = TRUE;
      window_main->acc_code_next = TRUE;
      window_main->acc_code_over = (window_main->code_current_line_is_jsr == 1) ? TRUE : FALSE;
      window_main->acc_code_highlight = (window_main->code_selected_line >= 0) ? TRUE : FALSE;
    }
  /** On est attente des donnes venant de Kegs **/
  else if(window_main->code_status == CYRENE_STATUS_WAIT_FOR_DATA)
    {
      /* Seul le bouton 'Stop' est actif */
      SetWindowText(window_main->hwnd_button_code_run,"Stop");
      EnableWindow(window_main->hwnd_button_code_run,TRUE);

      /* On dsactive les boutons Operation */
      EnableWindow(window_main->hwnd_button_operation_reset_operation,FALSE);
      EnableWindow(window_main->hwnd_button_operation_reset_cycle,FALSE);
      EnableWindow(window_main->hwnd_button_operation_save,FALSE);
      EnableWindow(window_main->hwnd_button_operation_clear,FALSE);

      /* On dsactive les boutons Code */
      EnableWindow(window_main->hwnd_button_code_in,FALSE);
      EnableWindow(window_main->hwnd_button_code_over,FALSE);
      EnableWindow(window_main->hwnd_button_code_10x,FALSE);
      EnableWindow(window_main->hwnd_button_code_100x,FALSE);
      EnableWindow(window_main->hwnd_button_code_1000x,FALSE);
      EnableWindow(window_main->hwnd_button_code_10000x,FALSE);
      EnableWindow(window_main->hwnd_button_code_jmp,FALSE);
      EnableWindow(window_main->hwnd_button_code_jsr,FALSE);
      EnableWindow(window_main->hwnd_button_code_rts,FALSE);
      EnableWindow(window_main->hwnd_button_code_vbl,FALSE);
      EnableWindow(window_main->hwnd_button_code_interrupt,FALSE);
      EnableWindow(window_main->hwnd_button_code_toolbox,FALSE);
      EnableWindow(window_main->hwnd_button_code_gsos,FALSE);
      EnableWindow(window_main->hwnd_button_code_highlight,FALSE);

      /* On active/dsactive les touches Accelerator */
      window_main->acc_code_run = TRUE;
      window_main->acc_code_next = FALSE;
      window_main->acc_code_over = FALSE;
      window_main->acc_code_highlight = FALSE;
    }
  /** On est attente de la fin des donnes venant de Kegs (queue) **/
  else if(window_main->code_status == CYRENE_STATUS_WAIT_FOR_STOP)
    {
      /* Le bouton 'Stop' actif passe en inactif */
      EnableWindow(window_main->hwnd_button_code_run,FALSE);

      /* On active/dsactive les touches Accelerator */
      window_main->acc_code_run = FALSE;
    }
  /** Cyrene n'est pas connect  Kegs **/
  else if(window_main->code_status == CYRENE_STATUS_NOT_CONNECTED)
    {
      /* Le bouton passe  'Run'*/
      SetWindowText(window_main->hwnd_button_code_run,"Run");

      /* On active les boutons Operation */
      EnableWindow(window_main->hwnd_button_operation_reset_operation,TRUE);
      EnableWindow(window_main->hwnd_button_operation_reset_cycle,TRUE);
      EnableWindow(window_main->hwnd_button_operation_save,TRUE);
      EnableWindow(window_main->hwnd_button_operation_clear,TRUE);

      /* On dsactive tous les boutons */
      EnableWindow(window_main->hwnd_button_code_run,FALSE);
      EnableWindow(window_main->hwnd_button_code_in,FALSE);
      EnableWindow(window_main->hwnd_button_code_over,FALSE);
      EnableWindow(window_main->hwnd_button_code_10x,FALSE);
      EnableWindow(window_main->hwnd_button_code_100x,FALSE);
      EnableWindow(window_main->hwnd_button_code_1000x,FALSE);
      EnableWindow(window_main->hwnd_button_code_10000x,FALSE);
      EnableWindow(window_main->hwnd_button_code_jmp,FALSE);
      EnableWindow(window_main->hwnd_button_code_jsr,FALSE);
      EnableWindow(window_main->hwnd_button_code_rts,FALSE);
      EnableWindow(window_main->hwnd_button_code_vbl,FALSE);
      EnableWindow(window_main->hwnd_button_code_interrupt,FALSE);
      EnableWindow(window_main->hwnd_button_code_toolbox,FALSE);
      EnableWindow(window_main->hwnd_button_code_gsos,FALSE);
      EnableWindow(window_main->hwnd_button_code_highlight,FALSE);

      /* On active/dsactive les touches Accelerator */
      window_main->acc_code_run = FALSE;
      window_main->acc_code_next = FALSE;
      window_main->acc_code_over = FALSE;
      window_main->acc_code_highlight = FALSE;
    }
}


/***************************************************************************************/
/*  RefreshLastOperationWindowData() :  MAJ des informations de la dernire opration. */
/***************************************************************************************/
static void RefreshLastOperationWindowData(struct window_main *window_main)
{
  char buffer[256];
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* MAJ des informations sur les oprations */
  sprintf(buffer,"%d",param->operation_id);
  PunctuateAtThoudand(buffer);
  SetWindowText(window_main->hwnd_edit_operation_count,buffer);
  sprintf(buffer,"%d",param->operation_total_cycle);
  PunctuateAtThoudand(buffer);
  SetWindowText(window_main->hwnd_edit_cycle_count,buffer);

  /* Dernire Opration */
  sprintf(buffer,"%02X/%04X : %02X",param->operation_PB,param->operation_PC,param->operation_opcode);
  if(param->operation_data_size > 0)
    sprintf(&buffer[strlen(buffer)]," %02X",param->operation_operand1);
  else
    strcpy(&buffer[strlen(buffer)],"   ");
  if(param->operation_data_size > 1)
    sprintf(&buffer[strlen(buffer)]," %02X",param->operation_operand2);
  else
    strcpy(&buffer[strlen(buffer)],"   ");
  if(param->operation_data_size > 2)
    sprintf(&buffer[strlen(buffer)]," %02X",param->operation_operand3);
  else
    strcpy(&buffer[strlen(buffer)],"   ");
  SetWindowText(window_main->hwnd_edit_operation_data,buffer);

  /** MAJ de l'historique des dernires Oprations **/
  UpdateOperationTextEdit(window_main,0);

  /** MAJ de la Status Bar qui affiche la taille des Operations Data **/
  UpdateWindowMainStatusBar();
}


/************************************************************************/
/*  RefreshCodeWindowData() :  MAJ des informations de la zone de Code. */
/************************************************************************/
static void RefreshCodeWindowData(struct window_main *window_main)
{
  struct s65816 *current_65816;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Rcupration du Engine */
  my_Memory(MEMORY_GET_S65816,&current_65816,NULL);

  /** Dsassemblage **/
  if(current_65816 != NULL)
    if(current_65816->current_mmu != NULL)
      {
        /** Construction des lignes dsassembls **/
        BuildCodeRichEdit(window_main,(param->operation_PB<<16)|param->operation_PC,param->operation_DB,param->operation_status_m,param->operation_status_x,current_65816->current_mmu);

        /** Affiche les lignes de code dans le Rich Edit **/
        DisplayCodeRichEdit(window_main);
      }

  /** MAJ des Boutons de la Zone Code **/
  UpdateCodeButtonStatus(window_main);
}


/***************************************************************************************/
/*  SaveOperationList() :  Sauvegarde sur disque les oprations conserves en mmoire. */
/***************************************************************************************/
static void SaveOperationList(struct window_main *window_main)
{
  BOOL is_OK;
  int i, result, nb_operation;
  char file_path[1024];

  /* Rcupre le nombre d'oprations historises */
  my_Memory(MEMORY_GET_OPERATION_NB,&nb_operation,NULL);
  if(nb_operation == 0)
    return;

  /* Demande le chemin du fichier  */
  sprintf(file_path,"Operation_List.txt");
  is_OK = GetSaveFilePath(window_main->hwnd_main,"Select Operation file","All Files (*.txt)\0*.txt\0\0",file_path);
  if(is_OK = FALSE)
    return;
  for(i=(int)strlen(file_path); i>=0; i--)
    if(file_path[i] == '.')
      {
        file_path[i] = '\0';
        break;
      }

  /* Lancement du Thread de cration des fichiers */
  CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)SaveOperationListThread,(char *)file_path,0,NULL);

  /* Cration de la fentre de ProgressBar */
  result = ProgressBarWindow(window_main->hwnd_main,"Save Operation List","Write Operation Data into files :",nb_operation/1000);
}


/*******************************************************************************************************/
/*  SaveOperationListThread() :  Thread ddi  la cration des fichiers contenant les Operation Data. */
/*******************************************************************************************************/
static DWORD SaveOperationListThread(void *file_path)
{
  FILE *fd;
  DWORD end_of_interrupt_address;
  int i, nb_operation, is_interrupt, file_id;
  int status_c, status_z, status_i, status_d, status_x, status_m, status_v, status_n, status_e, status_b;
  char buffer_a[16];
  char buffer_x[16];
  char buffer_y[16];
  char operand_1[16];
  char operand_2[16];
  char operand_3[16];
  char buffer_operand_comment[1024];
  char buffer_label[1024];
  unsigned char operation_data[256];
  char operation_line[1024];
  char file_path_full[1024];
  char file_name[1024];
  char file_name_full[1024];
  char buffer[1024];
  char buffer_read_address[16];
  char buffer_write_address[16];
  char buffer_jump_address[16];
  char buffer_read_value[16];
  char buffer_write_value[16];
  char buffer_call_number[16];
  char buffer_interrupt[256] = "--------------------------------------------------------------------------------------------------------------";
  struct code_line *current_codeline;
  struct operation *previous_operation;
  struct operation *current_operation;
  struct window_progressbar *window_progressbar;
  struct parameter *param;
  struct window_main *window_main;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);

  /* Init */
  fd = NULL;
  is_interrupt = 0;
  previous_operation = NULL;

  /* Extrait le nom du fichier */
  strcpy(file_name,(char *)file_path);
  for(i=(int)strlen((char *)file_path); i>=0; i--)
    if(((char *)file_path)[i] == '\\' || ((char *)file_path)[i] == '/')
      {
        strcpy(file_name,&((char *)file_path)[i+1]);
        break;
      }

  /* On va attendre la cration de la fentre ProgressBar */
  while(1)
    {
      Sleep(1);
      my_Memory(MEMORY_GET_WINDOW_PROGRESSBAR,&window_progressbar,NULL);
      if(window_progressbar != NULL)
        break;
    }

  /* Rcupre le nombre d'oprations historises */
  my_Memory(MEMORY_GET_OPERATION_NB,&nb_operation,NULL);

  /** Dump toutes les lignes **/
  for(i=1,file_id=1; i<=nb_operation; i++)
    {
      /* On va faire avancer la ProgressBar */
      if((i % 1000) == 1)
        {
          EnterCriticalSection(&param->cs_progress_bar);
          my_Memory(MEMORY_GET_WINDOW_PROGRESSBAR,&window_progressbar,NULL);
          if(window_progressbar != NULL)
            SendMessage(window_progressbar->hwnd_progressbar,HM_PROGRESSBAR_BUTTON_INCREMENT,0,0);
          LeaveCriticalSection(&param->cs_progress_bar);
        }

      /* Rcupre les donnes d'une opration */
      my_Memory(MEMORY_GET_OPERATION,&i,&current_operation);

      /** On va crer les fichiers Operation : 1 M de lignes / fichier **/
      if((i % 1000000) == 1)
        {
          /* Fermeture du fichier prcdent */
          if(fd != NULL)
            fclose(fd);
          fd = NULL;

          /* Nom du fichier */
          if(nb_operation < 1000000)
            {
              sprintf(file_path_full,"%s.txt",(char *)file_path);
              sprintf(file_name_full,"%s.txt",(char *)file_name);
            }
          else
            {
              sprintf(file_path_full,"%s_%04d.txt",(char *)file_path,file_id);
              sprintf(file_name_full,"%s_%04d.txt",(char *)file_name,file_id);
            }
          file_id++;

          /* Cration du fichier */
          fd = fopen(file_path_full,"w");
          if(fd == NULL)
            {
              MessageBox(window_main->hwnd_main,"Impossible to create File","Error",MB_OK|MB_ICONWARNING);

              /* Fermeture de la fentre ProgressBar */
              EnterCriticalSection(&param->cs_progress_bar);
              my_Memory(MEMORY_GET_WINDOW_PROGRESSBAR,&window_progressbar,NULL);
              if(window_progressbar != NULL)
                SendMessage(window_progressbar->hwnd_progressbar,HM_PROGRESSBAR_BUTTON_CANCEL,0,0);
              LeaveCriticalSection(&param->cs_progress_bar);

              /* Fin du Thread */
              ExitThread(0);
              return(0);
            }

          /* Met  jour le Prompt de la fentre Progress */
          sprintf(buffer,"Write Operation Data into file '%s' :",file_name_full);
          EnterCriticalSection(&param->cs_progress_bar);
          my_Memory(MEMORY_GET_WINDOW_PROGRESSBAR,&window_progressbar,NULL);
          if(window_progressbar != NULL)
            SendMessage(window_progressbar->hwnd_progressbar,HM_PROGRESSBAR_UPDATE_PROMPT,0,(LPARAM)&buffer[0]);
          LeaveCriticalSection(&param->cs_progress_bar);

          /* Ligne entte */
          fprintf(fd,"Global Operation ID  Global Cycle Count  Operation      A      X      Y     Stack   Direct   DB PB  PC    Status   M X   Read         Write        Jump  Flags Call Rom Ram C02E C02F C035 C068     Cycle       Address  Data         Label                    Opcode  Operand    Comment\n");
        }

      /* Entre dans une Interruption */
      if(previous_operation != NULL && is_interrupt == 0)
        {
          /* Interruption */
          if((current_operation->flags & OPERATION_FLAG_INTERRUPT) == OPERATION_FLAG_INTERRUPT)
            {
              fprintf(fd,"%s Begin Interruption %s\n",buffer_interrupt,buffer_interrupt);
              is_interrupt = 1;
              end_of_interrupt_address = previous_operation->next_op_addr;
            }
        }
      /* Fin d'une Interruption */
      else if(is_interrupt == 1 && end_of_interrupt_address == ((current_operation->register_PB << 16) | current_operation->register_PC))
        {
          is_interrupt = 0;
          fprintf(fd,"%s  End Interruption  %s\n",buffer_interrupt,buffer_interrupt);
        }

      /* Global Operation ID */
      sprintf(buffer," %08X:%08X | ",(DWORD)(current_operation->goid>>32),(DWORD)(current_operation->goid & 0x00000000FFFFFFFF));
      fwrite(buffer,strlen(buffer),1,fd);

      /* Global Cycle Count */
      sprintf(buffer,"%08X:%08X | ",(DWORD)(current_operation->gcc>>32),(DWORD)(current_operation->gcc & 0x00000000FFFFFFFF));
      fwrite(buffer,strlen(buffer),1,fd);

      /* Status Register + MX */
      status_c = (current_operation->register_STATUS >> 0) & 0x01;     /* C */
      status_z = (current_operation->register_STATUS >> 1) & 0x01;     /* Z */
      status_i = (current_operation->register_STATUS >> 2) & 0x01;     /* I */
      status_d = (current_operation->register_STATUS >> 3) & 0x01;     /* D */
      status_x = (current_operation->register_STATUS >> 4) & 0x01;     /* X */
      status_m = (current_operation->register_STATUS >> 5) & 0x01;     /* M */
      status_v = (current_operation->register_STATUS >> 6) & 0x01;     /* V */
      status_n = (current_operation->register_STATUS >> 7) & 0x01;     /* N */
      status_e = (current_operation->register_STATUS >> 8) & 0x01;     /* E */
      status_b = (current_operation->register_STATUS >> 9) & 0x01;     /* B */

      /* Operand Hexa Ascii */
      sprintf(operand_1,"%02X",current_operation->m_operand1);
      sprintf(operand_2,"%02X",current_operation->m_operand2);
      sprintf(operand_3,"%02X",current_operation->m_operand3);

      /* Dsassemble la ligne */
      operation_data[0] = current_operation->m_opcode;
      operation_data[1] = current_operation->m_operand1;
      operation_data[2] = current_operation->m_operand2;
      operation_data[3] = current_operation->m_operand3;
      current_codeline = DisassembleOneLine(4,operation_data,(current_operation->register_PB << 16)+current_operation->register_PC,current_operation->register_DB,DISASSEMBLER_PROC_65C816,status_m,status_x,current_operation->call_number,NULL);

      /* Cration de la Partie Label (Coupe  25) */
      strcpy(buffer_label,current_codeline->label);
      strcat(buffer_label,"                         ");     /* 25 */
      buffer_label[25] = '\0';

      /** Cration de la partie Operand + Commentaire (l'Oprande peut dborder sur le commentaire) **/
      strcpy(buffer_operand_comment,current_codeline->operand);
      if(strlen(buffer_operand_comment) < 15)
        {
          /* On coupe  15 */
          strcat(buffer_operand_comment,"               ");           /* 15 */
          buffer_operand_comment[15] = '\0';

          /* Ajoute le Comment */
          strcat(buffer_operand_comment,current_codeline->comment);
          strcat(buffer_operand_comment,"                        ");  /* 2+22 */
          buffer_operand_comment[15+2+22] = '\0';
        }
      else
        {
          /* On coupe  15+2+22 */
          strcat(buffer_operand_comment,"                                       ");   /* 15+2+22 */
          buffer_operand_comment[15+2+22] = '\0';
        }

      /** Buffer Read / Write / Jump Address **/
      /* Read */
      if(current_operation->read_address == 0x00FFFFFF)
        strcpy(buffer_read_address,"- - - -");
      else
        sprintf(buffer_read_address,"%02X/%04X",(current_operation->read_address >> 16) & 0xFF,current_operation->read_address & 0xFFFF);
      /* Write */
      if(current_operation->write_address == 0x00FFFFFF)
        strcpy(buffer_write_address,"- - - -");
      else
        sprintf(buffer_write_address,"%02X/%04X",(current_operation->write_address >> 16) & 0xFF,current_operation->write_address & 0xFFFF);
      /* Jump */
      if(current_operation->jump_address == 0x00FFFFFF)
        strcpy(buffer_jump_address,"- - - -");
      else
        sprintf(buffer_jump_address,"%02X/%04X",(current_operation->jump_address >> 16) & 0xFF,current_operation->jump_address & 0xFFFF);

      /** Buffer Read / Write Value **/
      /* Read */
      if(((current_operation->flags >> 3) & 0x03) == 0x00)
        strcpy(buffer_read_value," - - ");
      else if(((current_operation->flags >> 3) & 0x03) == 0x01)
        sprintf(buffer_read_value,":%02X  ",current_operation->read_value);
      else if(((current_operation->flags >> 3) & 0x03) == 0x02)
        sprintf(buffer_read_value,":%04X",current_operation->read_value);
      /* Write */
      if(((current_operation->flags >> 5) & 0x03) == 0x00)
        strcpy(buffer_write_value," - - ");
      else if(((current_operation->flags >> 5) & 0x03) == 0x01)
        sprintf(buffer_write_value,":%02X  ",current_operation->write_value);
      else if(((current_operation->flags >> 5) & 0x03) == 0x02)
        sprintf(buffer_write_value,":%04X",current_operation->write_value);

      /* Call Number */
      if(current_operation->call_number == 0xFFFF)
        strcpy(buffer_call_number,"----");
      else
        sprintf(buffer_call_number,"%04X",current_operation->call_number);

      /* A */
      if(status_m == 0)
        sprintf(buffer_a,"[%04X]",current_operation->register_A);
      else
        sprintf(buffer_a,"%02X[%02X]",(current_operation->register_A>>8) & 0x00FF,current_operation->register_A & 0x00FF);

      /* X + Y */
      if(status_m == 0)
        {
          sprintf(buffer_x,"[%04X]",current_operation->register_X);
          sprintf(buffer_y,"[%04X]",current_operation->register_Y);
        }
      else
        {
          sprintf(buffer_x,"%02X[%02X]",(current_operation->register_X>>8) & 0x00FF,current_operation->register_X & 0x00FF);
          sprintf(buffer_y,"%02X[%02X]",(current_operation->register_Y>>8) & 0x00FF,current_operation->register_Y & 0x00FF);
        }

      /* Cration d'une ligne */
      sprintf(operation_line,"%09d | %s %s %s  %02X/%04X  %02X/%04X  %02X %02X %04X  %c%c%c%c%c%c.%c%c %d %d  %s%s %s%s %s  %02X  %s  %02X  %02X  %02X   %02X   %02X   %02X  | %09d %d | %02X/%04X  %02X %s %s %s  %s  %-4s  %s",
                             current_operation->operation_id,
                             buffer_a,
                             buffer_x,
                             buffer_y,
                             ((current_operation->s_C068&0x30)==0x30)?0x01:0x00,current_operation->register_STACK,
                             ((current_operation->s_C068&0x30)==0x30)?0x01:0x00,current_operation->register_DIRECT,
                             current_operation->register_DB,
                             current_operation->register_PB,
                             current_operation->register_PC,
                             (status_n == 1) ? 'N':'n',
                             (status_v == 1) ? 'V':'v',
                             (status_d == 1) ? 'D':'d',
                             (status_i == 1) ? 'I':'i',
                             (status_z == 1) ? 'Z':'z',
                             (status_c == 1) ? 'C':'c',
                             (status_e == 1) ? 'E':'e',
                             (status_b == 1) ? 'B':'b',
                             status_m,
                             status_x,
                             buffer_read_address,
                             buffer_read_value,
                             buffer_write_address,
                             buffer_write_value,
                             buffer_jump_address,
                             current_operation->flags,
                             buffer_call_number,
                             current_operation->h_rom,
                             current_operation->h_ram,
                             current_operation->s_C02E,
                             current_operation->s_C02F,
                             current_operation->s_C035,
                             current_operation->s_C068,
                             current_operation->total_cycle,
                             current_operation->nb_cycle,
                             current_operation->register_PB,
                             current_operation->register_PC,
                             current_operation->m_opcode,
                             (current_codeline->data_size > 1) ? operand_1 : "  ",
                             (current_codeline->data_size > 2) ? operand_2 : "  ",
                             (current_codeline->data_size > 3) ? operand_3 : "  ",
                             buffer_label,
                             current_codeline->opcode,
                             buffer_operand_comment);

      /* Libration mmoire */
      mem_free_codeline(current_codeline);

      /* Dump la ligne */
      fwrite(operation_line,strlen(operation_line),1,fd);

      /* Fin de la ligne */
      fwrite("\n",strlen("\n"),1,fd);

      /** Operation Smartport Magic **/
      if(((current_operation->register_PB << 16) | current_operation->register_PC) == 0x0000C70D && current_operation->jump_address != 0x00FFFFFF)
        fprintf(fd,"%s Smartport Magic %s\n",buffer_interrupt,buffer_interrupt);

      /* Opration suivante */
      previous_operation = current_operation;

      /* La ProgressBar a t'elle t ferme ? */
      my_Memory(MEMORY_GET_WINDOW_PROGRESSBAR,&window_progressbar,NULL);
      if(window_progressbar == NULL)
        break;
    }

  /* Fermeture du fichier */
  if(fd != NULL)
    fclose(fd);
  fd = NULL;
  //SaveOperationStackList(window_main);

  /* Fermeture de la fentre ProgressBar */
  EnterCriticalSection(&param->cs_progress_bar);
  my_Memory(MEMORY_GET_WINDOW_PROGRESSBAR,&window_progressbar,NULL);
  if(window_progressbar != NULL)
    SendMessage(window_progressbar->hwnd_progressbar,HM_PROGRESSBAR_BUTTON_CANCEL,0,0);
  LeaveCriticalSection(&param->cs_progress_bar);

  /* Fin du Thread */
  ExitThread(0);
  return(0);
}

struct sub_address
{
  DWORD sub_begin_address;     /* Premire adresse de la Fonction (ou du point d'entre de l'interruption) */
  DWORD out_of_sub_address;    /* Addresse indiquant qu'on est sortit de la fonction (adresse de l'instruction suivant le JSR/JSL) */

  QWORD sub_begin_gcc;         /* #Cycle au dbut de la fonction */

  int is_interrupt_entry_point;
  int is_under_interrupt;
};

/********************************************************************************************/
/*  SaveOperationStackList() :  Sauvegarde sur disque les oprations conserves en mmoire. */
/********************************************************************************************/
static void SaveOperationStackList(struct window_main *window_main)
{
  FILE *fd;
  DWORD current_operation_address, next_operation_address, current_sub_next_address;
  int i, j, offset, nb_operation, is_interrupt, end_of_sub, end_of_sub_index;
  int status_c, status_z, status_i, status_d, status_x, status_m, status_v, status_n, status_e, status_b;
  int nb_sub_address;
  struct sub_address tab_sub_address[1024];
  unsigned char operation_data[256];
  char operation_line[1024];
  char buffer[1024];
  char buffer_jump_address[1024];
  char buffer_next_address[1024];
  char buffer_offset[1024];
  char buffer_interrupt[256] = "-------------------------------------------------";
  struct code_line *current_codeline;
  struct operation *previous_operation;
  struct operation *current_operation;

  /* Rcupre le nombre d'oprations historises */
  my_Memory(MEMORY_GET_OPERATION_NB,&nb_operation,NULL);
  if(nb_operation == 0)
    return;

  /* Cration du fichier */
  fd = fopen("c:\\AppleIIgs\\Operation_List_Stack.txt","w");
  if(fd == NULL)
    return;

  /** Dump toutes les lignes **/
  nb_sub_address = 0;
  offset = 10;
  is_interrupt = 0;
  previous_operation = NULL;
  for(i=1; i<=nb_operation; i++)
    {
      /* Rcupre les donnes d'une opration */
      my_Memory(MEMORY_GET_OPERATION,&i,&current_operation);

      /** Gestion de la Stack Call **/
      if(previous_operation != NULL)
        {
          /* Current Operation Address */
          current_operation_address = (current_operation->register_PB << 16) | current_operation->register_PC;

          /* Current Sub Next Operation Address */
          if(previous_operation->jump_address == 0x00FFFFFF)
            current_sub_next_address = (previous_operation->register_PB << 16) | ((previous_operation->register_PC + 1 + previous_operation->data_size) & 0x0000FFFF);
          else if(previous_operation->m_opcode == 0x20 || previous_operation->m_opcode == 0xFC || previous_operation->m_opcode == 0x22)      /* JSR / JSL */
            current_sub_next_address = (previous_operation->register_PB << 16) | ((previous_operation->register_PC + 1 + previous_operation->data_size) & 0x0000FFFF);
          else
            current_sub_next_address = previous_operation->jump_address;

          /** Fin d'une Sub ? **/
          end_of_sub = 0;
          for(j=nb_sub_address-1; j>=0; j--)
            if(tab_sub_address[j].out_of_sub_address == current_operation_address)
              {
                end_of_sub = 1;
                end_of_sub_index = j;
                break;
              }

          /** On vient de sortir d'une Sub/Interruption **/
          if(end_of_sub == 1)
            {
              nb_sub_address = end_of_sub_index;
              fprintf(fd,"%s End Sub %s\n",buffer_interrupt,buffer_interrupt);
            }
          /** On vient de rentrer dans une fonction via un JSR / JSL **/
          else if((previous_operation->m_opcode == 0x20 || previous_operation->m_opcode == 0xFC || previous_operation->m_opcode == 0x22) && 
                  current_operation_address == previous_operation->jump_address)
            {
              tab_sub_address[nb_sub_address].sub_begin_address = current_operation_address;     /* Adresse du dbut de la Sub */
              tab_sub_address[nb_sub_address].out_of_sub_address = current_sub_next_address;    /* 1re Adresse indiquant la fin de la Sub */
              tab_sub_address[nb_sub_address].sub_begin_gcc = current_operation->gcc;
              tab_sub_address[nb_sub_address].is_interrupt_entry_point = 0;
              tab_sub_address[nb_sub_address].is_under_interrupt = (nb_sub_address == 0) ? 0 : tab_sub_address[nb_sub_address-1].is_under_interrupt;
              nb_sub_address++;
              fprintf(fd,"%s Begin Sub %s\n",buffer_interrupt,buffer_interrupt);
            }
          /** On vient de rentrer dans une interruption **/
          else if(current_operation_address != current_sub_next_address)
            {
              tab_sub_address[nb_sub_address].sub_begin_address = current_operation_address;     /* Adresse du dbut de la Sub */
              tab_sub_address[nb_sub_address].out_of_sub_address = current_sub_next_address;     /* 1re Adresse indiquant la fin de la Sub */
              tab_sub_address[nb_sub_address].sub_begin_gcc = current_operation->gcc;
              tab_sub_address[nb_sub_address].is_interrupt_entry_point = 1;
              tab_sub_address[nb_sub_address].is_under_interrupt = 1;
              nb_sub_address++;
              fprintf(fd,"%s Begin Interruption %s\n",buffer_interrupt,buffer_interrupt);
            }
        }

      /* Global Operation ID */
      sprintf(buffer," %08X:%08X | ",(DWORD)(current_operation->goid>>32),(DWORD)(current_operation->goid & 0x00000000FFFFFFFF));
      fwrite(buffer,strlen(buffer),1,fd);

      /** Buffer Jump Address **/
      next_operation_address = (current_operation->register_PB << 16) | ((current_operation->register_PC + 1 + current_operation->data_size) & 0x0000FFFF);
      if(current_operation->jump_address == 0x00FFFFFF)
        strcpy(buffer_jump_address,"       ");
      else
        {
          sprintf(buffer_jump_address,"%02X/%04X",(current_operation->jump_address >> 16) & 0xFF,current_operation->jump_address & 0xFFFF);

          /* JSR / JSL */
          if(!(current_operation->m_opcode == 0x20 || current_operation->m_opcode == 0xFC || current_operation->m_opcode == 0x22))
            next_operation_address = current_operation->jump_address;
        }
      sprintf(buffer_next_address,"%02X/%04X",(next_operation_address >> 16) & 0xFF,next_operation_address & 0xFFFF);

      /* Status Register + MX */
      status_c = (current_operation->register_STATUS >> 0) & 0x01;     /* C */
      status_z = (current_operation->register_STATUS >> 1) & 0x01;     /* Z */
      status_i = (current_operation->register_STATUS >> 2) & 0x01;     /* I */
      status_d = (current_operation->register_STATUS >> 3) & 0x01;     /* D */
      status_x = (current_operation->register_STATUS >> 4) & 0x01;     /* X */
      status_m = (current_operation->register_STATUS >> 5) & 0x01;     /* M */
      status_v = (current_operation->register_STATUS >> 6) & 0x01;     /* V */
      status_n = (current_operation->register_STATUS >> 7) & 0x01;     /* N */
      status_e = (current_operation->register_STATUS >> 8) & 0x01;     /* E */
      status_b = (current_operation->register_STATUS >> 9) & 0x01;     /* B */

      /* Dsassemble la ligne */
      operation_data[0] = current_operation->m_opcode;
      operation_data[1] = current_operation->m_operand1;
      operation_data[2] = current_operation->m_operand2;
      operation_data[3] = current_operation->m_operand3;
      current_codeline = DisassembleOneLine(4,operation_data,(current_operation->register_PB << 16)+current_operation->register_PC,current_operation->register_DB,DISASSEMBLER_PROC_65C816,status_m,status_x,current_operation->call_number,NULL);

      /* Offset */
      offset = 10 + nb_sub_address;
      sprintf(buffer_offset,"%s%3d  ",(offset>=0)?" ":"-",offset);
      for(j=0; j<offset; j++)
        strcat(buffer_offset,"  ");

      /* Cration d'une ligne */
      sprintf(operation_line,"%02X/%04X  N=%s  J=%s   %s  %s  %s",
                             current_operation->register_PB,
                             current_operation->register_PC,
                             buffer_next_address,
                             buffer_jump_address,
                             buffer_offset,
                             current_codeline->opcode,
                             current_codeline->operand);

      /* Libration mmoire */
      mem_free_codeline(current_codeline);

      /* Dump la ligne */
      fwrite(operation_line,strlen(operation_line),1,fd);

      /* Fin de la ligne */
      fwrite("\n",strlen("\n"),1,fd);

      /* Opration suivante */
      previous_operation = current_operation;
    }

  /* Fermeture du fichier */
  fclose(fd);
}


/*****************************************************************************/
/*  ProcessCallStackOperation() :  MAJ de la CallStack avec cette opration. */
/*****************************************************************************/
void ProcessCallStackOperation(struct window_main *window_main, int is_interrupt)
{
  DWORD curr_op_address;
  int prev_op_is_jsr_jsl, nb_unstack, nb_exit, root_callstack;
  struct callstack *current_callstack;
  struct callstack *next_callstack;
  struct callstack *eof_callstack;
  struct callstack *previous_eof_callstack;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Init */
  prev_op_is_jsr_jsl = (param->prev_operation_opcode == 0x20 || param->prev_operation_opcode == 0xFC || param->prev_operation_opcode == 0x22) ? 1 : 0;
  curr_op_address    = (param->operation_PB << 16) | param->operation_PC;

  /* Protection ON */
  EnterCriticalSection(&param->cs_callstack);

  /** Allocation de la premire entre dans la Call Stack **/
  if(window_main->first_callstack == NULL)
    {
      /** Nouvelle entre dans la Call Stack **/
      /* Allocation mmoire */
      current_callstack = (struct callstack *) calloc(1,sizeof(struct callstack));
      if(current_callstack == NULL)
        {
          LeaveCriticalSection(&param->cs_callstack);
          MessageBox(window_main->hwnd_main,"Error, Impossible to store Callback !","Memory Allocation Error",MB_OK|MB_ICONWARNING);
          return;
        }
      /* Remplissage CallStack Root */
      current_callstack->function_address         = curr_op_address;
      current_callstack->call_number              = 0x0000;
      current_callstack->gcc                      = param->operation_gcc;
      current_callstack->goid                     = param->operation_goid;
      current_callstack->function_end_address     = 0xFFFFFFFF;
      current_callstack->is_interrupt             = 0;
      current_callstack->depth                    = 0;
      current_callstack->function_under_interrupt = 0;

      /* Attache */
      window_main->first_callstack = current_callstack;
      window_main->last_callstack  = current_callstack;
      window_main->nb_callstack    = 1;

      /* On vient de crer la Root Callstack */
      root_callstack = 1;
    }
  else
    root_callstack = 0;

  /** Vient t'on de sortir d'une fonction ? **/
  for(eof_callstack = window_main->last_callstack; eof_callstack; eof_callstack = eof_callstack->previous)
    if(eof_callstack->function_end_address == curr_op_address)
      break;

  /** Cas 1 : On vient de rentrer dans une fonction JSR/JSL **/
  if(root_callstack == 0 && prev_op_is_jsr_jsl == 1 && curr_op_address == param->prev_operation_next_op_addr)
    {
      /** Nouvelle entre dans la Call Stack **/
      /* Allocation mmoire */
      current_callstack = (struct callstack *) calloc(1,sizeof(struct callstack));
      if(current_callstack == NULL)
        {
          LeaveCriticalSection(&param->cs_callstack);
          MessageBox(window_main->hwnd_main,"Error, Impossible to store Callback !","Memory Allocation Error",MB_OK|MB_ICONWARNING);
          return;
        }
      /* Remplissage */
      current_callstack->function_address         = curr_op_address;
      current_callstack->call_number              = param->prev_operation_call_number;
      current_callstack->gcc                      = param->operation_gcc;
      current_callstack->goid                     = param->operation_goid;
      current_callstack->function_end_address     = param->prev_operation_sub_next_op_addr;
      current_callstack->is_interrupt             = 0;
      current_callstack->depth                    = window_main->last_callstack->depth + 1;
      current_callstack->function_under_interrupt = window_main->last_callstack->function_under_interrupt;

      /* Ajoute  la fin de la liste doublement chaine */
      current_callstack->previous       = window_main->last_callstack;
      window_main->last_callstack->next = current_callstack;
      window_main->last_callstack       = current_callstack;
      window_main->nb_callstack++;
    }
  /** Cas 2 : On vient de rentrer dans une Interruption **/
  else if(root_callstack == 0 && curr_op_address != param->prev_operation_next_op_addr && param->is_first_operation == 0)
    {
      /** Nouvelle entre dans la Call Stack **/
      /* Allocation mmoire */
      current_callstack = (struct callstack *) calloc(1,sizeof(struct callstack));
      if(current_callstack == NULL)
        {
          LeaveCriticalSection(&param->cs_callstack);
          MessageBox(window_main->hwnd_main,"Error, Impossible to store Callback !","Memory Allocation Error",MB_OK|MB_ICONWARNING);
          return;
        }
      /* Remplissage */
      current_callstack->function_address         = curr_op_address;
      current_callstack->call_number              = param->prev_operation_call_number;
      current_callstack->gcc                      = param->operation_gcc;
      current_callstack->goid                     = param->operation_goid;
      current_callstack->function_end_address     = param->prev_operation_sub_next_op_addr;
      current_callstack->is_interrupt             = 1;
      current_callstack->depth                    = window_main->last_callstack->depth + 1;
      current_callstack->function_under_interrupt = 1;

      /* Ajoute  la fin de la liste doublement chaine */
      current_callstack->previous       = window_main->last_callstack;
      window_main->last_callstack->next = current_callstack;
      window_main->last_callstack       = current_callstack;
      window_main->nb_callstack++;
    }
  /** Cas 3 : On vient de sortir d'une fonction **/
  else if(root_callstack == 0 && eof_callstack != NULL)
    {
      /** On va peut tre devoir dpiler plusieurs appels sur la mme opration (Interruption + Fonction) **/
      nb_exit = 1;
      while(eof_callstack->function_end_address == curr_op_address)
        {
          /* On conserve un pointeur sur la fonction prcdente */
          previous_eof_callstack = eof_callstack->previous;

          /** On va dpiler la Call Stack (jamais la premire entre) **/
          nb_unstack = 0;
          eof_callstack->previous->next = NULL;
          window_main->last_callstack = eof_callstack->previous;
          for(current_callstack = eof_callstack; current_callstack; current_callstack = next_callstack)
            {
              next_callstack = current_callstack->next;
              free(current_callstack);
              window_main->nb_callstack--;
              nb_unstack++;
            }

          /* On peut encore dpiler si on est sur la mme @ de retour */
          nb_exit++;
          if(previous_eof_callstack == NULL)
            break;
          eof_callstack = previous_eof_callstack;
        }
    }
  /** Cas 4 : C'est une opration neutre **/

  /** On met  jour les informations de l'opration en cours dans la fonction actuelle (Last) **/
  window_main->last_callstack->operation_gcc          = param->operation_gcc;                               /* # Cycle au moment de cycle instruction */
  window_main->last_callstack->operation_goid         = param->operation_goid;
  window_main->last_callstack->operation_pc_address   = (param->operation_PB << 16) | param->operation_PC;  /* Adresse de l'instruction en cours = Adresse d'o l'on saute si Saut/Interruption */
  window_main->last_callstack->operation_opcode       = param->operation_opcode;                            
  window_main->last_callstack->operation_operand_size = param->operation_data_size;
  window_main->last_callstack->operation_operand[0]   = param->operation_operand1;
  window_main->last_callstack->operation_operand[1]   = param->operation_operand2;
  window_main->last_callstack->operation_operand[2]   = param->operation_operand3;
  window_main->last_callstack->operation_jump_address = param->operation_jump_address;                      /* Adresse o on va sauter (si JSR/JSL) */
  window_main->last_callstack->operation_call_number  = param->operation_call_number;                       /* Information complmentaire pour nommer la fonction */

  /* Protection OFF */
  LeaveCriticalSection(&param->cs_callstack);
}


/*****************************************************************************/
/*  RefreshCallStackListBox() :  MAJ de l'affichage de la ListBox CallStack. */
/*****************************************************************************/
static void RefreshCallStackListBox(struct window_main *window_main)
{
  int next_is_interrupt, extra_length;
  char buffer[1024];
  struct callstack *current_callstack;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Supprime toutes les entres de la ListBox **/
  SendMessage(window_main->hwnd_listbox_callstack,LB_RESETCONTENT,0,0);

  /* Calcul de la taille Extra disponible */
  extra_length = window_main->nb_column - window_main->nb_init_data_column;

  /* Protection ON */
  EnterCriticalSection(&param->cs_callstack);

  /* Aucune information disponible */
  if(window_main->nb_callstack == 0)
    {
      LeaveCriticalSection(&param->cs_callstack);
      return;
    }

  /*** Construit la nouvelle ListBox ***/
  /** Ligne Actuelle + Adresses de la Pile **/
  for(current_callstack = window_main->last_callstack; current_callstack; current_callstack = current_callstack->previous)
    {
      /* Entre dans une interuption ? */
      next_is_interrupt = 0;
      if(current_callstack->next != NULL)
        if(current_callstack->next->is_interrupt == 1 && current_callstack->is_interrupt == 0)
          next_is_interrupt = 1;

      /** Cration de la ligne **/
      BuildCallStackLine(current_callstack->depth,current_callstack->function_address,current_callstack->operation_jump_address,
                         current_callstack->operation_pc_address,current_callstack->operation_call_number,current_callstack->operation_opcode,
                         current_callstack->function_under_interrupt,next_is_interrupt,buffer,extra_length,window_main);

      /* Ajoute cette entre en tte de la ListBox */
      SendMessage(window_main->hwnd_listbox_callstack,LB_ADDSTRING,0,(LPARAM)buffer);
    }

  /* MAJ du Count */
  sprintf(buffer,"%d",window_main->nb_callstack);
  SetWindowText(window_main->hwnd_static_callstack_count,buffer);

  /* Protection OFF */
  LeaveCriticalSection(&param->cs_callstack);

  /** On active les boutons **/
  EnableWindow(window_main->hwnd_button_callstack_disassemble,TRUE);
  EnableWindow(window_main->hwnd_button_callstack_clear,TRUE);
}


/******************************************************************/
/*  BuildCallStackLine() :  Construit une ligne de la Call Stack. */
/******************************************************************/
static void BuildCallStackLine(int level, DWORD base_address, DWORD jump_address, DWORD pc_address, WORD call_number, BYTE opcode, int under_interrupt, int next_is_interrupt, char *buffer_line_rtn, int extra_length, struct window_main *window_main)
{
  int function_name_length;
  char buffer_name_padding[1024];
  char buffer_name[1024];
  char buffer_type[1024];
  char buffer_application[1024];
  char buffer_delta[256];

  /** Nom de la fonction li  l'adresse : Appel de Tools / GS/OS / Prodos16 / Rom /Symbol **/
  GetNameFromAddress(base_address,call_number,buffer_name,buffer_type,buffer_application);

  /*** Construction de la ligne : "## --/---- FunctionName-------- +----- = --/---- : JSR $--/----" ***/
  /** Numro dans la Pile : 0-N **/
  sprintf(buffer_line_rtn,"%02d ",level);

  /* Interrupt */
  strcpy(&buffer_line_rtn[strlen(buffer_line_rtn)],(under_interrupt == 1) ? "I " : "  ");

  /** Adresse de base **/
  sprintf(&buffer_line_rtn[strlen(buffer_line_rtn)],"%02X/%04X ",(base_address >> 16) & 0xFF,base_address & 0xFFFF);

  /** Nom de la Fonction de Base : 18 char + ExtraLength **/
  function_name_length = 18 + extra_length;
  memset(buffer_name_padding,' ',function_name_length);
  buffer_name_padding[function_name_length] = '\0';
  if(strlen(buffer_name) == 0)
    strcpy(buffer_name,buffer_name_padding);
  else
    {
      strcat(buffer_name,buffer_name_padding);
      buffer_name[function_name_length] = '\0';
    }
  sprintf(&buffer_line_rtn[strlen(buffer_line_rtn)],"%s ",buffer_name);

  /** Delta Adresse de base / Adresse Actuelle : 6 char **/
  if((pc_address & 0x00FF0000) != (base_address & 0x00FF0000))
    strcpy(buffer_delta,"...");
  else if(pc_address > base_address)
    sprintf(buffer_delta,"+%d",pc_address-base_address);
  else if(pc_address < base_address)
    sprintf(buffer_delta,"-%d",base_address-pc_address);
  else
    strcpy(buffer_delta,"");
  strcat(buffer_delta,"      ");
  buffer_delta[strlen("      ")] = '\0';
  sprintf(&buffer_line_rtn[strlen(buffer_line_rtn)],"%s ",buffer_delta);

  /** Adresse Actuelle **/
  sprintf(&buffer_line_rtn[strlen(buffer_line_rtn)],"= %02X/%04X: ",(pc_address >> 16) & 0xFF,pc_address & 0xFFFF);

  /** Fonction JSR / JSL + Adresse Cible **/
  if(opcode == 0x20 || opcode == 0xFC)
    sprintf(&buffer_line_rtn[strlen(buffer_line_rtn)],"JSR $%02X/%04X",(jump_address >> 16) & 0xFF,jump_address & 0xFFFF);
  else if(opcode == 0x22)
    sprintf(&buffer_line_rtn[strlen(buffer_line_rtn)],"JSL $%02X/%04X",(jump_address >> 16) & 0xFF,jump_address & 0xFFFF);
  else if(under_interrupt == 0 && next_is_interrupt == 1)
    strcat(buffer_line_rtn,">> INTERRUPT");
  else
    strcat(buffer_line_rtn,"            ");
}


/***********************************************************************/
/*  RefreshOutputListBox() :  MAJ de l'affichage de la ListBox Output. */
/***********************************************************************/
void RefreshOutputListBox(struct window_main *window_main)
{
  int nb_added;
  char buffer[2048];
  WCHAR buffer_w[2048];
  //char buffer_sm[256];
  struct output *current_output;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Publie dans la ListBox toutes les lignes Output non publies **/
  nb_added = 0;
  EnterCriticalSection(&param->cs_output);
  for(current_output = window_main->first_output; current_output; current_output = current_output->next)
    if(current_output->published == 0)
      {
        /** Construit la ligne System Call / [GOID][GCC][ADDRESS][CALL NUMBER] : <origin>::<function>(<param1>,<param2>...) => Error(<error_code>) **/
        BuildSystemCallOutputLine(current_output,0,buffer);

        /** Conversion en WCHAR (UTF8 si TraceCall / Win si Breakpoint) **/
        if(current_output->type == BREAKPOINT_ACTION_TRACE_SYSTEM_CALL)
          MultiByteToWideChar(CP_UTF8,0,(LPCSTR)buffer,-1,buffer_w,2048);    /* UTF8 -> WCHAR */
        else
          MultiByteToWideChar(CP_ACP,0,(LPCSTR)buffer,-1,buffer_w,2048);     /* Win -> WCHAR */

        /* Shared Memory Debug */
        //sprintf(buffer_sm,"O=%06X,L=%03d",current_output->offset_sm,current_output->length_sm);
        //memcpy(&buffer[offset],buffer_sm,(int) strlen(buffer_sm));
        //offset += (int) strlen(buffer_sm);

        /** Ajoute  la fin de la ListBox **/
        SendMessageW(window_main->hwnd_listbox_output,LB_ADDSTRING,0,(LPARAM)buffer_w);
        current_output->published = 1;
        nb_added++;
      }
  LeaveCriticalSection(&param->cs_output);

  /* Rien  MAJ */
  if(nb_added == 0)
    return;

  /* MAJ du Count */
  sprintf(buffer,"%d",window_main->nb_output);
  SetWindowText(window_main->hwnd_static_output_count,buffer);

  /** On active les boutons **/
  if(window_main->nb_output > 0)
    {
      EnableWindow(window_main->hwnd_button_output_save,TRUE);
      EnableWindow(window_main->hwnd_button_output_clear,TRUE);
      EnableWindow(window_main->hwnd_button_output_refresh,TRUE);
    }

  /* On slectionne la dernire entre */
  SendMessageA(window_main->hwnd_listbox_output,LB_SETCURSEL,window_main->nb_output-1,(LPARAM)NULL); 
}


/**********************************************************************************/
/*  CallStackListDisassemble() :  Dsassemble  partir de l'adresse slectionne. */
/**********************************************************************************/
static void CallStackListDisassemble(struct window_main *window_main)
{
  LRESULT entry_id, current_id;
  DWORD address;
  struct callstack *current_callstack;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Rien  faire */
  if(window_main->nb_callstack == 0)
    return;

  /** Rcupre l'adresse de la ligne slectionne dans la ListBox **/
  address = 0xFFFFFFFF;
  entry_id = SendMessage(window_main->hwnd_listbox_callstack,LB_GETCURSEL,0,0); 
  if(entry_id == LB_ERR)
    return;

  /* Protection ON */
  EnterCriticalSection(&param->cs_callstack);

  /** Rcupre l'adresse **/
  for(current_id=0, current_callstack = window_main->last_callstack; current_callstack; current_callstack = current_callstack->previous, current_id++)
    if(current_id == entry_id)
      {
        address = current_callstack->operation_pc_address;
        break;
      }

  /* Protection OFF */
  LeaveCriticalSection(&param->cs_callstack);

  /** Affiche une fentre avec le code dsassembl **/
  BuildDisassemblerWindow(param->hInstance,ID_MENU_VIEW_CODEDISASSEMBLER,address);
}


/*******************************************************************/
/*  CallStackListClear() :  Supprime les entres de la Call Stack. */
/*******************************************************************/
static void CallStackListClear(struct window_main *window_main)
{
  struct callstack *current_callstack;
  struct callstack *next_callstack;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Supprime toutes les entres de la ListBox **/
  SendMessage(window_main->hwnd_listbox_callstack,LB_RESETCONTENT,0,0);

  /* Protection ON */
  EnterCriticalSection(&param->cs_callstack);

  /** Supprime la liste chaine de structure **/
  for(current_callstack = window_main->first_callstack; current_callstack; current_callstack = next_callstack)
    {
      next_callstack = current_callstack->next;
      free(current_callstack);
    }
  window_main->first_callstack = NULL;
  window_main->last_callstack  = NULL;
  window_main->nb_callstack    = 0;

  /* Protection OFF */
  LeaveCriticalSection(&param->cs_callstack);

  /* MAJ du Count */
  SetWindowText(window_main->hwnd_static_callstack_count,"0");

  /* On dsactive les boutons */
  EnableWindow(window_main->hwnd_button_callstack_disassemble,FALSE);
  EnableWindow(window_main->hwnd_button_callstack_clear,FALSE);

  /* MAJ du contenu de la fentre Call Stack */
  RefreshCallStackListBox(window_main);
}


/******************************************************************************************/
/*  OutputListSave() :  Enregistre les entres de la ListBox Output dans un fichier UTF8. */
/******************************************************************************************/
static void OutputListSave(struct window_main *window_main)
{
  BOOL is_OK;
  FILE *fd;
  char buffer_line[4096];
  char file_path[BUFFER_SIZE];
  unsigned char header_utf8[] = {0xEF,0xBB,0xBF};
  struct output *current_output;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Pas de ligne Output */
  if(window_main->first_output == NULL)
    return;

  /** Demande le nom du fichier **/
  sprintf(file_path,"Breakpoint_SystemCall_Output.txt");
  is_OK = GetSaveFilePath(window_main->hwnd_main,"Select Breakpoint/ System Call Output file","All Files (*.txt)\0*.txt\0\0",file_path);
  if(is_OK = FALSE)
    return;

  /* Cration du fichier */
  fd = fopen(file_path,"w");
  if(fd == NULL)
    {
      MessageBox(window_main->hwnd_main,"Impossible to create File","Error",MB_OK|MB_ICONWARNING);
      return;
    }

  /* Encodage UTF8 */
  fwrite(header_utf8,1,3,fd);

  /* Entte */
  fprintf(fd,"Global Operation ID  #Cycle  Address  Call  Origin  Function\n");

  /** Passe en revue toutes les lignes  **/
  EnterCriticalSection(&param->cs_output);
  for(current_output = window_main->first_output; current_output; current_output = current_output->next)
    {
      /** Ligne Output [] **/
      if(current_output->type == BREAKPOINT_ACTION_TRACE_SYSTEM_CALL)
        {
          /** Construit la ligne System Call / [GOID][GCC][ADDRESS][CALL NUMBER] : <origin>::<function>(<param1>,<param2>...) => Error(<error_code>) **/
          BuildSystemCallOutputLine(current_output,1,buffer_line);

          /* Ecriture de la ligne */
          fprintf(fd,"%s\n",buffer_line);
        }
      else
        /** Ligne Output [Breakpoint] **/
        fprintf(fd,"%s\n",current_output->line);
    }
  LeaveCriticalSection(&param->cs_output);

  /* Fermeture du fichier */
  fclose(fd);
}


/********************************************************************************************/
/*  BuildSystemCallOutputLine() :  Cration d'une ligne du System Call / Breakpoint Output. */
/********************************************************************************************/
static void BuildSystemCallOutputLine(struct output *current_output, int target_file, char *buffer_line_rtn)
{
  int i;
  char buffer_gcc[16];
  char buffer_utf8[2048];
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Init */
  strcpy(buffer_line_rtn,"");

  /** Trace System Calls **/
  if(current_output->origin == TRACE_CALL_ORIGIN_P8 || current_output->origin == TRACE_CALL_ORIGIN_P16 || current_output->origin == TRACE_CALL_ORIGIN_GSOS || current_output->origin == TRACE_CALL_ORIGIN_TOOL)
    {
      /* GOID */
      if(target_file == 1 || param->trace_display_goid == 1)
        sprintf(&buffer_line_rtn[strlen(buffer_line_rtn)],"[%08X:%08X]",(DWORD)(current_output->goid>>32),(DWORD)(current_output->goid & 0x00000000FFFFFFFF));

      /* Cycle */
      if(target_file == 1 || param->trace_display_gcc == 1)
        {
          sprintf(buffer_gcc,"[%7d]",current_output->nb_cycle);
          if(strlen(buffer_gcc) != TRACE_CALL_GCC_LENGTH)
            strcpy(buffer_gcc,"[*******]");
          strcat(buffer_line_rtn,buffer_gcc);
        }

      /* Address */
      if(target_file == 1 || param->trace_display_pc_address == 1)
        sprintf(&buffer_line_rtn[strlen(buffer_line_rtn)],"[%02X/%04X]",(BYTE)((current_output->address>>16) & 0x000000FF),(WORD)(current_output->address & 0x0000FFFF));

      /* Call Number */
      if(target_file == 1 || param->trace_display_call_number == 1)
        sprintf(&buffer_line_rtn[strlen(buffer_line_rtn)],"[%04X]",current_output->call_number);

      /* Origin */
      if(current_output->origin == TRACE_CALL_ORIGIN_P8)
        strcat(buffer_line_rtn," o   P8::");
      else if(current_output->origin == TRACE_CALL_ORIGIN_P16)
        strcat(buffer_line_rtn," o  P16::");
      else if(current_output->origin == TRACE_CALL_ORIGIN_GSOS)
        strcat(buffer_line_rtn," o GSOS::");
      else if(current_output->origin == TRACE_CALL_ORIGIN_TOOL)
        sprintf(&buffer_line_rtn[strlen(buffer_line_rtn)]," o T%03d::",current_output->call_number & 0x00FF);

      /* Fonction Name */
      sprintf(&buffer_line_rtn[strlen(buffer_line_rtn)],"%s(",current_output->function_name);

      /** Parameters **/
      for(i=0; i<current_output->nb_param; i++)
        {
          /* Sparator */
          if(i != 0)
            strcat(buffer_line_rtn,",");

          /* Return Value marker */
          if(current_output->tab_param_type[i] == TRACE_CALL_PARAM_TYPE_OUT && current_output->tab_param_value_utf8[i] != NULL)
            strcat(buffer_line_rtn,"[");

          /** Value **/
          if(current_output->tab_param_value_utf8[i] == NULL)
            strcat(buffer_line_rtn,"-");
          else if(current_output->tab_param_value_utf8[i][0] == '"')
            {
              /* Chaine Hexa Ascii Apple IIgs -> UTF8 */
              ConvertAppleIIgsHexaAsciiAsUTF8(current_output->tab_param_value_utf8[i],buffer_utf8);

              /* Ajoute la chaine */
              strcpy(&buffer_line_rtn[strlen(buffer_line_rtn)],buffer_utf8);
            }
          else
            strcpy(&buffer_line_rtn[strlen(buffer_line_rtn)],current_output->tab_param_value_utf8[i]);

          /* Return Value marker */
          if(current_output->tab_param_type[i] == TRACE_CALL_PARAM_TYPE_OUT && current_output->tab_param_value_utf8[i] != NULL)
            strcat(buffer_line_rtn,"]");
        }

      /* End of parameters */
      strcat(buffer_line_rtn,")");

      /* Error Code */
      if(current_output->error_code != 0x0000)
        sprintf(&buffer_line_rtn[strlen(buffer_line_rtn)]," => Error (%04X)",current_output->error_code);
    }
  /** Breakpoints **/
  else
    {
      /* GOID */
      if(target_file == 1)
        sprintf(&buffer_line_rtn[strlen(buffer_line_rtn)],"[%08X:%08X]",(DWORD)(current_output->goid>>32),(DWORD)(current_output->goid & 0x00000000FFFFFFFF));

      /* Address */
      if(target_file == 1)
        sprintf(&buffer_line_rtn[strlen(buffer_line_rtn)],"[%02X/%04X]",(BYTE)((current_output->address>>16) & 0x000000FF),(WORD)(current_output->address & 0x0000FFFF));

      /** Line **/
      sprintf(&buffer_line_rtn[strlen(buffer_line_rtn)]," %s",current_output->line);
    }
}


/***********************************************************/
/*  OutputListClear() :  Supprime les entres de l'Output. */
/***********************************************************/
static void OutputListClear(struct window_main *window_main)
{
  struct output *current_output;
  struct output *next_output;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Supprime toutes les entres de la ListBox **/
  SendMessage(window_main->hwnd_listbox_output,LB_RESETCONTENT,0,0);

  /** Supprime la liste chaine de structure **/
  EnterCriticalSection(&param->cs_output);
  for(current_output = window_main->first_output; current_output; current_output = next_output)
    {
      next_output = current_output->next;
      mem_free_output(current_output);
    }
  window_main->first_output = NULL;
  window_main->last_output = NULL;
  window_main->nb_output = 0;
  LeaveCriticalSection(&param->cs_output);

  /* MAJ du Count */
  SetWindowText(window_main->hwnd_static_output_count,"0");

  /* On dsactive les boutons */
  EnableWindow(window_main->hwnd_button_output_save,FALSE);
  EnableWindow(window_main->hwnd_button_output_clear,FALSE);
  EnableWindow(window_main->hwnd_button_output_refresh,FALSE);
}


/***************************************************************/
/*  OutputListRefresh() :  Rafrachit les entres de l'Output. */
/***************************************************************/
static void OutputListRefresh(struct window_main *window_main)
{
  struct output *current_output;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Supprime toutes les entres de la ListBox **/
  SendMessage(window_main->hwnd_listbox_output,LB_RESETCONTENT,0,0);

  /** Reset des Flags de la liste chaine **/
  EnterCriticalSection(&param->cs_output);
  for(current_output = window_main->first_output; current_output; current_output = current_output->next)
    current_output->published = 0;
  LeaveCriticalSection(&param->cs_output);

  /** Remplissage de la ListBox **/
  RefreshOutputListBox(window_main);
}


/************************************************************************************/
/*  CreateMainToolBarWindow() :  Cration de la fentre ToolBar de la fentre Main. */
/************************************************************************************/
static BOOL CreateMainToolBarWindow(struct window_main *window_main)
{
  int i, j;
  TBADDBITMAP tbab;
  TBBUTTON tbb[18];
  RECT toolbar_rect;
  int idCmd[18] = 
    {
      IDH_BUTTON_ATTACH,        IDH_BUTTON_DETACH,          -1,
      IDH_BUTTON_LOAD_SNAPSHOT, IDH_BUTTON_SAVE_SNAPSHOT,   -1,
      IDH_BUTTON_LOAD_SYMBOL,   IDH_BUTTON_LOAD_BREAKPOINT, -1,
      IDH_BUTTON_VIEW_SHR,      IDH_BUTTON_VIEW_CODE,       IDH_BUTTON_VIEW_INTERNALS, IDH_BUTTON_VIEW_MEMORYMAP, -1,
      IDH_BUTTON_STATISTIC,     IDH_BUTTON_PROFILE,         -1,
      IDH_BUTTON_RESET
    };
  char *iString[18] = 
    {
      "Attach to Apple IIgs Emulator",   "Detach from Apple IIgs Emulator", "",
      "Load Apple IIgs Memory Snapshot", "Save Apple IIgs Memory Snapshot", "",
      "Load Symbols file",               "Load Breakpoints file",           "",
      "View Graphic SHR",                "View Code Disassembler",          "View Internals",  "View Memory Map", "",
      "Function Statistics",             "Function Profiler",               "",
      "Clear"
    };
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Create ToolBar **/
  window_main->hwnd_toolbar = CreateWindowEx(0,TOOLBARCLASSNAME,NULL, WS_CHILD | WS_VISIBLE | CCS_ADJUSTABLE | TBSTYLE_TOOLTIPS,
                                             0,0,0,0,window_main->hwnd_main,(HMENU)IDR_TOOLBAR_MAIN,param->hInstance,NULL);
  if(window_main->hwnd_toolbar == NULL)
    return(FALSE);

  /** ToolBar buttons + bitmap **/
  SendMessage(window_main->hwnd_toolbar,TB_BUTTONSTRUCTSIZE,(WPARAM)sizeof(TBBUTTON),0);
  memset(tbb,0,sizeof(tbb));
  for(i=0,j=0; i<sizeof(tbb)/sizeof(TBBUTTON); i++)
    {
      if(idCmd[i] <0) 
        tbb[i].fsStyle = TBSTYLE_SEP;
      else
        {
          tbb[i].iBitmap = j;
          tbb[i].idCommand = idCmd[i];
          tbb[i].fsState = TBSTATE_ENABLED;
          tbb[i].fsStyle = TBSTYLE_BUTTON;
          tbb[i].iString = SendMessage(window_main->hwnd_toolbar,TB_ADDSTRING,0,(LPARAM)iString[i]);
          SendMessage(window_main->hwnd_toolbar,TB_SETMAXTEXTROWS,0,0);     /* Le texte des boutons apparait en Tooltip */
          j++;
        }
    }
  tbab.hInst = param->hInstance;
  tbab.nID = IDR_TOOLBAR_MAIN;
  SendMessage(window_main->hwnd_toolbar,TB_ADDBITMAP,j,(LPARAM)&tbab);
  SendMessage(window_main->hwnd_toolbar,TB_ADDBUTTONS,i,(LPARAM)&tbb);

  /* Hauteur de la Toollbar */
  GetWindowRect(window_main->hwnd_toolbar,&toolbar_rect);
  window_main->toolbar_height = toolbar_rect.bottom - toolbar_rect.top;

  /* OK */
  return(TRUE);
}


/****************************************************************************************/
/*  CreateMainStatusBarWindow() :  Cration de la fentre StatusBar de la fentre Main. */
/****************************************************************************************/
static BOOL CreateMainStatusBarWindow(struct window_main *window_main)
{
  RECT status_rect;
  int iStatusWidths[] = {STATUSBAR_MAIN_0_WIDTH,STATUSBAR_MAIN_1_WIDTH,STATUSBAR_MAIN_2_WIDTH,STATUSBAR_MAIN_3_WIDTH,STATUSBAR_MAIN_4_WIDTH,STATUSBAR_MAIN_5_WIDTH,STATUSBAR_MAIN_6_WIDTH,STATUSBAR_MAIN_7_WIDTH,STATUSBAR_MAIN_8_WIDTH};
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Create StatusBar **/
  window_main->hwnd_statusbar = CreateWindowEx(0,STATUSCLASSNAME,NULL,WS_CHILD|WS_VISIBLE|SBARS_SIZEGRIP,0,0,0,0,window_main->hwnd_main,(HMENU)IDR_STATUSBAR_MAIN,param->hInstance,NULL);
  if(window_main->hwnd_statusbar == NULL)
    return(FALSE);

  /* StatusBar parameters */
  SendMessage(window_main->hwnd_statusbar,SB_SETPARTS,9,(LPARAM)iStatusWidths);
  SendMessage(window_main->hwnd_statusbar,TB_AUTOSIZE,0,0);
  SendMessage(window_main->hwnd_statusbar,WM_SIZE,0,0);

  /* Hauteur de la Status Bar */
  GetWindowRect(window_main->hwnd_statusbar,&status_rect);
  window_main->statusbar_height = status_rect.bottom - status_rect.top;

  /* OK */
  return(TRUE);
}


/*****************************************************************************************/
/*  UpdateWindowMainStatusBar() :  Mise  jour des donnes affiches dans la Status Bar. */
/*****************************************************************************************/
void UpdateWindowMainStatusBar(void)
{
  __int64 operation_size, kegs_goid;
  int memory_size, nb_symbol, nb_breakpoint, nb_operation, operation_size_mb, queue_size_k;
  char buffer_connected[BUFFER_SIZE] = " ";
  char buffer_rom[BUFFER_SIZE] = "";
  char buffer_ramsize[BUFFER_SIZE] = "";
  char buffer_symbol[BUFFER_SIZE] = "";
  char buffer_breakpoint[BUFFER_SIZE] = "";
  char buffer_operation_size[BUFFER_SIZE] = "";
  char buffer_operation_goid[256] = "";
  char buffer_operation_queue[256] = "";
  char buffer_status[1024] = "";
  struct cyrene_breakpoint *current_breakpoint;
  struct s65816 *current_65816;
  struct window_main *window_main;
  struct window_ipc *window_ipc;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Init */
  strcpy(buffer_connected,"");
  strcpy(buffer_rom,"");
  strcpy(buffer_ramsize,"");

  /* Rcupration des structures */
  my_Memory(MEMORY_GET_S65816,&current_65816,NULL);
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  my_Memory(MEMORY_GET_WINDOW_IPC,&window_ipc,NULL);

  /** Etat de la Connexion **/
  if(param->kegs_connection_status == CONNECTION_STATUS_CLOSED)
    strcpy(buffer_connected," Disconnected");
  else if(param->kegs_connection_status == CONNECTION_STATUS_WAIT_FOR_ACCEPT)
    strcpy(buffer_connected," Connecting...");
  else if(param->kegs_connection_status == CONNECTION_STATUS_OPEN)
    sprintf(buffer_connected," Connected to KEGS (PID=%d)",param->attach_pid);

  /* Prparation des buffers Mmoire */
  if(current_65816 != NULL)
    {
      /* Type de la Rom */
      if(current_65816->current_mmu->rom_version == ROM_APPLEIIGS_01)
        strcpy(buffer_rom," ROM 01");
      else if(current_65816->current_mmu->rom_version == ROM_APPLEIIGS_03)
        strcpy(buffer_rom," ROM 03");

      /* Taille de la mmoire */
      memory_size = 256 + 64*current_65816->current_mmu->nb_extended_bank;
      sprintf(buffer_ramsize," %d KB",memory_size);        
    }

  /** Symbols **/
  nb_symbol = (window_main->current_symbol_table == NULL) ? 0 : window_main->current_symbol_table->nb_symbol;
  sprintf(buffer_symbol," Symbol : %d",nb_symbol);

  /** Breakpoints **/
  nb_breakpoint = (window_main->current_breakpoint_table == NULL) ? 0 : window_main->current_breakpoint_table->nb_breakpoint;
  sprintf(buffer_breakpoint," Breakpoint : %d",nb_breakpoint);

  /** Taille des Operation Data **/
  my_Memory(MEMORY_GET_OPERATION_NB,&nb_operation,NULL);
  operation_size = nb_operation*sizeof(struct operation) + (1+(nb_operation/MAX_OPERATION))*sizeof(struct operation_set) + nb_operation*sizeof(struct operation *);
  operation_size_mb = (int) (operation_size / (1024*1024));
  sprintf(buffer_operation_size," Operation size : %d MB",operation_size_mb);

  /** Global Operation ID **/
  sprintf(buffer_operation_goid," GOID : %08X:%08X",(DWORD)(param->operation_goid>>32),(DWORD)(param->operation_goid & 0x00000000FFFFFFFF));

  /** Delta GOID **/
  kegs_goid = 0;
  queue_size_k = 0;
  if(window_ipc != NULL)
    if(window_ipc->shared_memory_ptr != NULL)
      memcpy(&kegs_goid,&window_ipc->shared_memory_ptr[IPC_SHARED_MEMORY_OFFSET_GOID],sizeof(__int64));
  if(kegs_goid != 0)
    {
      queue_size_k = (DWORD)(kegs_goid - param->operation_goid) / 1000;
      sprintf(buffer_operation_queue," Queue : %d K",queue_size_k);
    }

  /** Status **/
  if(queue_size_k == 0 && window_main->stop_condition == STOP_CONDITION_BREAKPOINT)
    {
      strcpy(buffer_status,"  >>>  Stop Breakpoint");
      if(window_main->current_breakpoint_table != NULL)
        {
          for(current_breakpoint = window_main->current_breakpoint_table->first_breakpoint; current_breakpoint; current_breakpoint=current_breakpoint->next)
            if(current_breakpoint->id == window_main->stop_breakpoint_id)
              {
                BuildBreakpointTextLine(current_breakpoint,BREAKPOINT_LINE_TARGET_STATUS,buffer_status);
                break;
              }
        }
    }
  else if(queue_size_k == 0 || ((queue_size_k/100) % 10) > 4)
    strcpy(buffer_status,"");
  else
    strcpy(buffer_status,"  >>>  Receiving Operation data from emulator  <<<");

  /** Mise  jour des valeurs affches dans la Status Bar **/
  SendMessage(window_main->hwnd_statusbar,SB_SETTEXT,STATUSBAR_MAIN_CONNECTED,(LPARAM)buffer_connected);
  SendMessage(window_main->hwnd_statusbar,SB_SETTEXT,STATUSBAR_MAIN_APPLEIIGS_ROM,(LPARAM)buffer_rom);
  SendMessage(window_main->hwnd_statusbar,SB_SETTEXT,STATUSBAR_MAIN_MEMORY_SIZE,(LPARAM)buffer_ramsize);
  SendMessage(window_main->hwnd_statusbar,SB_SETTEXT,STATUSBAR_MAIN_SYMBOL,(LPARAM)buffer_symbol);
  SendMessage(window_main->hwnd_statusbar,SB_SETTEXT,STATUSBAR_MAIN_BREAKPOINT,(LPARAM)buffer_breakpoint);
  SendMessage(window_main->hwnd_statusbar,SB_SETTEXT,STATUSBAR_MAIN_OPERATION_SIZE,(LPARAM)buffer_operation_size);
  SendMessage(window_main->hwnd_statusbar,SB_SETTEXT,STATUSBAR_MAIN_OPERATION_GOID,(LPARAM)buffer_operation_goid);
  SendMessage(window_main->hwnd_statusbar,SB_SETTEXT,STATUSBAR_MAIN_OPERATION_QUEUE,(LPARAM)buffer_operation_queue);
  SendMessage(window_main->hwnd_statusbar,SB_SETTEXT,STATUSBAR_MAIN_STATUS,(LPARAM)buffer_status);
}


/********************************************************/
/*  UserResizeMainWindow() :  Retaille la fentre Main. */
/********************************************************/
static void UserResizeMainWindow(struct window_main *window_main)
{
  RECT window_rect;
  int window_x, window_y, window_new_width, window_new_height, minimum_window_width, minimum_window_height;
  int listview_pixel_width, listview_pixel_height, delta_pixel_width, delta_pixel_height;
  int nb_listview_data_column, nb_listview_data_row, delta_listview_nb_column, delta_listview_nb_row;

  /* Position + Dimension actuelle de la fentre retaille dans l'cran */
  GetWindowRect(window_main->hwnd_main,&window_rect);

  /* Nouvelle Dimension de la fentre (avec la barre de titre et les bordures) */
  window_x = window_rect.left;
  window_y = window_rect.top;
  window_new_width = window_rect.right - window_rect.left;
  window_new_height = window_rect.bottom - window_rect.top;

  /* Nombre de Pixels restant pour la ListView */
  minimum_window_width = window_main->window_init_width - window_main->nb_init_data_column*MAIN_OPERATION_COLUMN_WIDTH;
  listview_pixel_width = window_new_width - minimum_window_width;
  minimum_window_height = window_main->window_init_height - window_main->nb_init_data_row*MAIN_OPERATION_ROW_HEIGHT;
  listview_pixel_height = window_new_height - minimum_window_height;

  /* Nombre de Colonne+lignes de la ListView */
  nb_listview_data_column = listview_pixel_width / MAIN_OPERATION_COLUMN_WIDTH;
  nb_listview_data_row = listview_pixel_height / MAIN_OPERATION_ROW_HEIGHT;

  /** Teste les valeurs minimales autorises **/
  if(nb_listview_data_column < MAIN_OPERATION_MIN_NB_COLUMN)
    nb_listview_data_column = MAIN_OPERATION_MIN_NB_COLUMN;
  if(nb_listview_data_row < MAIN_OPERATION_MIN_NB_ROW)
    nb_listview_data_row = MAIN_OPERATION_MIN_NB_ROW;

  /* Delta de la ListView */
  delta_listview_nb_column = nb_listview_data_column - window_main->nb_column;
  delta_pixel_width = delta_listview_nb_column*MAIN_OPERATION_COLUMN_WIDTH;
  window_main->nb_column = nb_listview_data_column;
  delta_listview_nb_row = nb_listview_data_row - window_main->nb_row;
  delta_pixel_height = delta_listview_nb_row*MAIN_OPERATION_ROW_HEIGHT;
  window_main->nb_row = nb_listview_data_row;

  /* Nouvelle Largeur/hauteur de la fentre (on ajuste pour tomber sur un nombre extact de colonne/ligne dans la ListView) */
  window_new_width = minimum_window_width + nb_listview_data_column*MAIN_OPERATION_COLUMN_WIDTH;
  window_new_height = minimum_window_height + nb_listview_data_row*MAIN_OPERATION_ROW_HEIGHT;

  /* Retaille la fentre */
  window_main->user_resize = 0;
  MoveWindow(window_main->hwnd_main,window_x,window_y,window_new_width,window_new_height,FALSE);

  /* Dplace les Contrles */
  MoveMainWindowControls(delta_pixel_width,delta_pixel_height,window_main);

  /* Rafraichit la fentre */
  InvalidateRect(window_main->hwnd_main,NULL,TRUE);
  UpdateWindow(window_main->hwnd_main);
}


/***************************************************************************************/
/*  MoveMainWindowControls() :  On va dplacer les diffrents controles de la fentre. */
/***************************************************************************************/
static void MoveMainWindowControls(int delta_x, int delta_y, struct window_main *window_main)
{
  HWND hwnd_control, hwnd_operation, hwnd_callstack, hwnd_output, hwnd_statusbar;
  int i, control_x, control_y, control_width, control_height;
  int operation_x, operation_y, operation_width, operation_height;
  int callstack_x, callstack_y, callstack_width, callstack_height;
  int output_x, output_y, output_width, output_height;
  int statusbar_x, statusbar_y, statusbar_width, statusbar_height;
  int tab_control_vertical_id[] = 
    {
      IDC_CHECKBOX_OPERATION,      IDC_STATIC_NB_OPERATION,           IDC_EDIT_NB_OPERATION,           IDC_STATIC_NB_CYCLE, 
      IDC_EDIT_NB_CYCLE,           IDC_STATIC_CURRENT_OPERATION_DATA, IDC_EDIT_CURRENT_OPERATION_DATA, IDC_STATIC_CODE,
      IDC_GROUPBOX_CODE,           IDC_RICHEDIT_CODE,
      IDC_BUTTON_CODE_RUN,         IDC_BUTTON_CODE_IN,                IDC_BUTTON_CODE_OVER,            IDC_BUTTON_CODE_10X,
      IDC_BUTTON_CODE_100X,        IDC_BUTTON_CODE_1000X,             IDC_BUTTON_CODE_10000X,          IDC_BUTTON_CODE_JSR,
      IDC_BUTTON_CODE_RTS,         IDC_BUTTON_CODE_JMP,               IDC_BUTTON_CODE_VBL,             IDC_BUTTON_CODE_INTERRUPT,
      IDC_BUTTON_CODE_TOOLBOX,     IDC_BUTTON_CODE_GSOS,              IDC_BUTTON_CODE_HIGHLIGHT,       IDC_STATIC_CALLSTACK_ENTRY,  
      IDC_STATIC_CALLSTACK_COUNT,  IDC_STATIC_OUTPUT_ENTRY,           IDC_STATIC_OUTPUT_COUNT,         IDC_CHECKBOX_TRACE_SYSCALL,
      0
    };
  int tab_control_full_id[] = 
    {
      IDC_BUTTON_OPERATION_CLEAR,       IDC_BUTTON_OPERATION_RESET_CYCLE,  IDC_BUTTON_OPERATION_RESET_OPERATION, IDC_BUTTON_OPERATION_SAVE,
      IDC_BUTTON_CALLSTACK_DISASSEMBLE, IDC_BUTTON_CALLSTACK_CLEAR,
      IDC_BUTTON_OUTPUT_SAVE,           IDC_BUTTON_OUTPUT_CLEAR,           IDC_BUTTON_OUTPUT_REFRESH,
      0
    };

  /** On va modifier la Hauteur+Largeur de la Groupbox Operation **/
  /* Handle du Controle */
  hwnd_operation = window_main->hwnd_groupbox_operation;
  /* Position + Taille actuelle */
  GetWindowControlInfo(window_main->hwnd_main,hwnd_operation,&operation_x,&operation_y,&operation_width,&operation_height);
  /* Change la hauteur du controle */
  MoveWindow(hwnd_operation,operation_x,operation_y,operation_width+delta_x,operation_height+delta_y,FALSE);

  /** On va modifier la Hauteur+Largeur de la Zone Operation Data **/
  /* Handle du Controle */
  hwnd_operation = window_main->hwnd_edit_operation;
  /* Position + Taille actuelle */
  GetWindowControlInfo(window_main->hwnd_main,hwnd_operation,&operation_x,&operation_y,&operation_width,&operation_height);
  /* Change la hauteur du controle */
  MoveWindow(hwnd_operation,operation_x,operation_y,operation_width+delta_x,operation_height+delta_y,FALSE);

  /** On va modifier la hauteur de la ScrollBar Operation + Dplacement Horizontal **/
  /* Handle du Controle */
  hwnd_operation = window_main->hwnd_scrollbar_operation;
  /* Position + Taille actuelle */
  GetWindowControlInfo(window_main->hwnd_main,hwnd_operation,&operation_x,&operation_y,&operation_width,&operation_height);
  /* Change la hauteur du controle */
  MoveWindow(hwnd_operation,operation_x+delta_x,operation_y,operation_width,operation_height+delta_y,FALSE);

  /** On va modifier la Largeur de la Groupbox CallStack + Dplacement Vertical **/
  /* Handle du Controle */
  hwnd_callstack = window_main->hwnd_groupbox_callstack;
  /* Position + Taille actuelle */
  GetWindowControlInfo(window_main->hwnd_main,hwnd_callstack,&callstack_x,&callstack_y,&callstack_width,&callstack_height);
  /* Change la largeur du controle */
  MoveWindow(hwnd_callstack,callstack_x,callstack_y+delta_y,callstack_width+delta_x,callstack_height,FALSE);

  /** On va modifier la Largeur de la ListView CallStack Data **/
  /* Handle du Controle */
  hwnd_callstack = window_main->hwnd_listbox_callstack;
  /* Position + Taille actuelle */
  GetWindowControlInfo(window_main->hwnd_main,hwnd_callstack,&callstack_x,&callstack_y,&callstack_width,&callstack_height);
  /* Change la hauteur du controle */
  MoveWindow(hwnd_callstack,callstack_x,callstack_y+delta_y,callstack_width+delta_x,callstack_height,FALSE);

  /** On va modifier la Largeur de la Groupbox Output + Dplacement Vertical **/
  /* Handle du Controle */
  hwnd_output = window_main->hwnd_groupbox_output;
  /* Position + Taille actuelle */
  GetWindowControlInfo(window_main->hwnd_main,hwnd_output,&output_x,&output_y,&output_width,&output_height);
  /* Change la largeur du controle */
  MoveWindow(hwnd_output,output_x,output_y+delta_y,output_width+delta_x,output_height,FALSE);

  /** On va modifier la Largeur de la ListView Output Data **/
  /* Handle du Controle */
  hwnd_output = window_main->hwnd_listbox_output;
  /* Position + Taille actuelle */
  GetWindowControlInfo(window_main->hwnd_main,hwnd_output,&output_x,&output_y,&output_width,&output_height);
  /* Change la hauteur du controle */
  MoveWindow(hwnd_output,output_x,output_y+delta_y,output_width+delta_x,output_height,FALSE);

  /** On va modifier la Largeur de la StatusBar + Dplacement Vertical **/
  /* Handle du Controle */
  hwnd_statusbar = window_main->hwnd_statusbar;
  /* Position + Taille actuelle */
  GetWindowControlInfo(window_main->hwnd_main,hwnd_statusbar,&statusbar_x,&statusbar_y,&statusbar_width,&statusbar_height);
  /* Change la Largeur du controle */
  MoveWindow(hwnd_statusbar,statusbar_x,statusbar_y+delta_y,statusbar_width+delta_x,statusbar_height,FALSE);

  /*******************************************************************/
  /** On va dplacer certains controles exclusivement verticalement **/
  for(i=0; tab_control_vertical_id[i] != 0; i++)
    {
      /* Handle du Controle */
      hwnd_control = GetDlgItem(window_main->hwnd_main,tab_control_vertical_id[i]);

      /* Position + Taille actuelle */
      GetWindowControlInfo(window_main->hwnd_main,hwnd_control,&control_x,&control_y,&control_width,&control_height);

      /* Dplace le controle verticalement */
      MoveWindow(hwnd_control,control_x,control_y+delta_y,control_width,control_height,FALSE);
    }

  /***********************************************************************/
  /** On va dplacer certains controles horizontalement + verticalement **/
  for(i=0; tab_control_full_id[i] != 0; i++)
    {
      /* Handle du Controle */
      hwnd_control = GetDlgItem(window_main->hwnd_main,tab_control_full_id[i]);

      /* Position + Taille actuelle */
      GetWindowControlInfo(window_main->hwnd_main,hwnd_control,&control_x,&control_y,&control_width,&control_height);

      /* Dplace le controle verticalement */
      MoveWindow(hwnd_control,control_x+delta_x,control_y+delta_y,control_width,control_height,FALSE);
    }

  /* Reconstruit les donnes affiches dans Opration (pour tendre le nombre de lignes ou largeur du Comment) */
  window_main->operation_text_data_column = window_main->nb_column;
  window_main->operation_text_data_row    = window_main->nb_row;
  UpdateOperationTextEdit(window_main,1);

  /* Reconstruit les donnes affiches dans CallStack pour largir la colonne Function Name */
  if(delta_x != 0)
    RefreshCallStackListBox(window_main);

  /* Pas besoin de reconstruire les donnes affiches dans Output (la ListView gre elle mme son affichage) */
}


/*****************************************************************/
/*  DeleteMainChildWindows() : Destruction des fentres enfants. */
/*****************************************************************/
static int DeleteMainChildWindows(struct window_main *window_main)
{
  /** Fermeture des fentres **/
  DestroyWindow(window_main->hwnd_toolbar);
  DestroyWindow(window_main->hwnd_statusbar);

  /* OK */
  return(0);
}


/***********************************************************************/
/*  mem_free_window_main() :  Libration mmoire structure windo_main. */
/***********************************************************************/
void mem_free_window_main(struct window_main *window_main)
{
  struct callstack *current_callstack;
  struct callstack *next_callstack;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  if(window_main)
    {
      /* Libration de la callstack */
      EnterCriticalSection(&param->cs_callstack);
      for(current_callstack = window_main->first_callstack; current_callstack; current_callstack = next_callstack)
        {
          next_callstack = current_callstack->next;
          free(current_callstack);
        }
      window_main->nb_callstack    = 0;
      window_main->first_callstack = NULL;
      window_main->last_callstack  = NULL;
      LeaveCriticalSection(&param->cs_callstack);

      free(window_main);
    }
}

/**********************************************************************/
