/************************************************************************/
/*                                                                      */
/*  Window_MemoryArea.c :  Module de gestion de la fentre MemoryArea.  */
/*                                                                      */
/************************************************************************/
/*  Auteur : Olivier ZARDINI  *   Brutal Deluxe   *   Date : Juin 2018  */
/************************************************************************/

#include <stdio.h>
#include <time.h>

#include <windows.h>
#include <windowsx.h>
#include <Commctrl.h>
#include <Winuser.h>
#include <Richedit.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_Notification.h"
#include "Dc_Graphic.h"
#include "Dc_CPU.h"
#include "Dc_MMU.h"
#include "Window_QuickWatch.h"
#include "Window_Main.h"
#include "Window_Main_Menu.h"
#include "Window_SymbolPickup.h"
#include "Window_MemoryArea.h"

static DWORD BuildMemoryAreaWindowThread(void *);
LRESULT CALLBACK ProcHandlerMemoryAreaWindow(HWND,UINT,WPARAM,LPARAM);
static int CreateMemoryAreaChildWindows(HWND,struct window_memoryarea *);
LRESULT CALLBACK subEditMemoryAreaProcControl(HWND,UINT,WPARAM,LPARAM);
static void UpdateMemoryAreaContextInformation(struct window_memoryarea *,int,int);
static void UpdateMemoryAreaAddress(struct window_memoryarea *,int);
static void UpdateMemoryAreaRefreshMode(struct window_memoryarea *,int);
static void RefreshMemoryAreaWindowData(struct window_memoryarea *);
static int IsMemoryAreaDataModified(int,int,int,int,struct window_memoryarea *);
static void UserResizeMemoryAreaWindow(struct window_memoryarea *);
static void ResizeMemoryAreaWindow(struct window_memoryarea *);
static void BuildMemoryAreaFromData(struct window_memoryarea *);
static void DeleteMemoryAreaChildWindows(struct window_memoryarea *);


/**********************************************************************************/
/*  BuildMemoryAreaWindow() :  Construction / Affichage de la fentre MemoryArea. */
/**********************************************************************************/
struct window_memoryarea *BuildMemoryAreaWindow(HINSTANCE hInstance, int menu_id, DWORD init_address)
{
  struct window_memoryarea *window_memoryarea;

  /** Allocation de la structure window_memoryarea **/
  window_memoryarea = (struct window_memoryarea *) calloc(1,sizeof(struct window_memoryarea));
  if(window_memoryarea == NULL)
    return(NULL);
  window_memoryarea->init_address = init_address;
  my_Memory(MEMORY_ADD_WINDOW_MEMORYAREA,window_memoryarea,NULL);

  /* Lance un thread pour grer la fentre MemoryArea */
  CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)BuildMemoryAreaWindowThread,window_memoryarea,0,NULL);

  /* Renvoi la structure mmoire grant la fentre */
  return(window_memoryarea);
}


/********************************************************************************************/
/*  BuildMemoryAreaWindowThread() :  Cration de la fentre MemoryArea via un Thread ddi. */
/********************************************************************************************/
static DWORD BuildMemoryAreaWindowThread(void *window_memoryarea_param)
{
  HWND hwndMemoryArea;
  MSG msg;
  BOOL bRet;
  struct window_main *window_main;
  struct window_memoryarea *window_memoryarea;
  struct parameter *param;
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Init */
  window_memoryarea = (struct window_memoryarea *) window_memoryarea_param;
  window_memoryarea->thread_id = GetCurrentThreadId();         /* Le ThreaID sert  diffrencier les diffrentes fentres */

  /** Cration de la fentre Memory Area **/
  hwndMemoryArea = CreateDialog(param->hInstance,MAKEINTRESOURCE(IDD_WINDOW_MEMORYAREA),window_main->hwnd_main,(DLGPROC)ProcHandlerMemoryAreaWindow);
  if(hwndMemoryArea == NULL)
    return(1);

  /** Show Window + Paint Content **/
  ShowWindow(hwndMemoryArea,SW_SHOWDEFAULT);
  UpdateWindow(hwndMemoryArea);

  /** Message Loop pour toutes les fentres de ce Thread (fentre MemoryArea...) **/
  while((bRet = GetMessage(&msg,NULL,0,0)) != 0)
    {
      if(bRet == -1)
        break;

      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }

  /* Fin du Thread */
  ExitThread(0);
}


/***************************************************************************/
/*  ProcHandlerMemoryAreaWindow() :  Gestion des vnements de la fentre. */
/***************************************************************************/
LRESULT CALLBACK ProcHandlerMemoryAreaWindow(HWND hwndMemoryArea, UINT umsg, WPARAM wParam, LPARAM lParam)
{
  char buffer[1024];
  struct window_memoryarea *window_memoryarea;
  struct parameter *param;
  
  /* Init */
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  my_Memory(MEMORY_GET_WINDOW_MEMORYAREA,&window_memoryarea,NULL);
  if(window_memoryarea == NULL)
    return(TRUE);

  switch(umsg)
    {
      case WM_INITDIALOG :
        /* Cration du contenu de la DialogBox */
        CreateMemoryAreaChildWindows(hwndMemoryArea,window_memoryarea);
        return(TRUE);

      /***************************/
      /* Evnements de la souris */
      /***************************/
      case WM_MOUSEMOVE :
        SetWindowText(GetDlgItem(window_memoryarea->hwnd_memoryarea,IDC_STATIC_CONTEXT),"");
        return(TRUE);
        
      /****************************/
      /* Evnements de la fentre */
      /****************************/
      case WM_COMMAND :
        switch(LOWORD(wParam))
          {
            /** Bouton Radio : On change la mthode de rafraichissement de la fentre **/
            case IDC_RADIO_REFRESH_AUTOMATIC :
              if(HIWORD(wParam) == BN_CLICKED && window_memoryarea->refresh_mode != MEMORYAREA_REFRESH_AUTOMATIC)
                UpdateMemoryAreaRefreshMode(window_memoryarea,MEMORYAREA_REFRESH_AUTOMATIC);
              break;

            case IDC_RADIO_REFRESH_MANUAL :
              if(HIWORD(wParam) == BN_CLICKED && window_memoryarea->refresh_mode != MEMORYAREA_REFRESH_MANUAL)
                UpdateMemoryAreaRefreshMode(window_memoryarea,MEMORYAREA_REFRESH_MANUAL);
              break;

            /** Bouton de slection de l'adresse via Symbol **/
            case IDC_BUTTON_SYMBOL :
              BuildSymbolPickupWindow(param->hInstance,window_memoryarea->hwnd_memoryarea,HM_MEMORYAREA_REFRESHSYMBOL);
              break;

            /** Bouton de dplacement de l'adresse **/
            case IDC_BUTTON_LEFT :
              UpdateMemoryAreaAddress(window_memoryarea,-1);
              break;

            case IDC_BUTTON_RIGHT :
              UpdateMemoryAreaAddress(window_memoryarea,1);
              break;

            case IDC_BUTTON_UP :
              UpdateMemoryAreaAddress(window_memoryarea,(-1)*window_memoryarea->nb_column);
              break;

            case IDC_BUTTON_DOWN :
              UpdateMemoryAreaAddress(window_memoryarea,window_memoryarea->nb_column);
              break;

            case IDC_BUTTON_PAGEUP :
              UpdateMemoryAreaAddress(window_memoryarea,(-1)*window_memoryarea->nb_column*window_memoryarea->nb_row);
              break;

            case IDC_BUTTON_PAGEDOWN :
              UpdateMemoryAreaAddress(window_memoryarea,window_memoryarea->nb_column*window_memoryarea->nb_row);
              break;

            /** Bouton de rafraichissement Manuel de la fentre **/
            case IDC_BUTTON_REFRESH :
              RefreshMemoryAreaWindowData(window_memoryarea);
              break;
          }
        return(TRUE);

      /**************************************/
      /*  On vient de retailler la fentre  */
      /**************************************/
      case WM_SIZE :
        /* On va tendre la fentre de donne mmoire si c'est un Resize User */
        if(window_memoryarea->user_resize == 1)
          UserResizeMemoryAreaWindow(window_memoryarea);
        window_memoryarea->user_resize = 1;
        return(TRUE);

      /*****************************************************************/
      /* Message extrieur demandant le rafraichissement de la fentre */
      /*****************************************************************/
      case HM_MEMORYAREA_REFRESHWINDOW :
        RefreshMemoryAreaWindowData(window_memoryarea);
        return(TRUE);

      /** On vient de demander la MAJ de l'adresse via un Symbol **/
      case HM_MEMORYAREA_REFRESHSYMBOL :
        if(param->symbol_is_available == 1)
          {
            /* Conserve les donnes du Symbol */
            window_memoryarea->has_symbol = 1;
            strcpy(window_memoryarea->symbol_name,param->symbol_name);
            strcpy(window_memoryarea->symbol_segment,param->symbol_segment);
            strcpy(window_memoryarea->symbol_file,param->symbol_file);
            window_memoryarea->symbol_address = param->symbol_address;
            /* MAJ de l'affichage */
            sprintf(buffer,"%02X",(param->symbol_address >> 16) & 0x000000FF);
            SetDlgItemText(window_memoryarea->hwnd_memoryarea,IDC_EDIT_BANK,buffer);
            sprintf(buffer,"%04X",param->symbol_address & 0x0000FFFF);
            SetDlgItemText(window_memoryarea->hwnd_memoryarea,IDC_EDIT_ADDRESS,buffer);
            RefreshMemoryAreaWindowData(window_memoryarea);
          }
        return(TRUE);

      /********************************************************************/
      /* Fermeture de window (le WM_CLOSE est appel avant le WM_DESTROY) */
      /********************************************************************/
      case WM_CLOSE :
        /* Fentre MemoryArea */
        DestroyWindow(hwndMemoryArea);
        return(0);

      case WM_DESTROY :
        /* Suppression des Fentres */
        DeleteMemoryAreaChildWindows(window_memoryarea);

        /* Supprime la structure mmoire de ce thread */
        my_Memory(MEMORY_DELETE_WINDOW_MEMORYAREA,window_memoryarea,NULL);

        /* Demande la sortie de la boucle des messages */
        PostQuitMessage(0);
        return(0);
    }

  /* Retourner FALSE pour relayer le message non trait au systme */
  return(FALSE);
}


/***********************************************************************************************************/
/*  CreateMemoryAreaChildWindows() : Crer les fentres enfants et remplir la structure window_memoryarea. */
/***********************************************************************************************************/
static int CreateMemoryAreaChildWindows(HWND hwndMemoryArea, struct window_memoryarea *window_memoryarea)
{
  HWND hwnd_group;
  int group_width, group_height;
  RECT client_rect, window_rect;
  char buffer[BUFFER_SIZE];
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Window Handle **/
  window_memoryarea->hwnd_memoryarea = hwndMemoryArea;

  /** Ajoute les images sur les boutons **/
  SendMessage(GetDlgItem(hwndMemoryArea,IDC_BUTTON_SYMBOL),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_SYMBOL)));
  SendMessage(GetDlgItem(hwndMemoryArea,IDC_BUTTON_LEFT),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_LEFT)));
  SendMessage(GetDlgItem(hwndMemoryArea,IDC_BUTTON_RIGHT),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_RIGHT)));
  SendMessage(GetDlgItem(hwndMemoryArea,IDC_BUTTON_UP),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_UP)));
  SendMessage(GetDlgItem(hwndMemoryArea,IDC_BUTTON_DOWN),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_DOWN)));
  SendMessage(GetDlgItem(hwndMemoryArea,IDC_BUTTON_PAGEUP),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_PAGE_UP)));
  SendMessage(GetDlgItem(hwndMemoryArea,IDC_BUTTON_PAGEDOWN),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_PAGE_DOWN)));

  /** Adresse de la Zone de Data **/
  if(window_memoryarea->init_address != 0xFFFFFFFF)
    {
      window_memoryarea->memoryarea_bank = (window_memoryarea->init_address & 0x00FF0000) >> 16;
      window_memoryarea->memoryarea_address = window_memoryarea->init_address & 0x0000FFFF; 
    }
  else
    {
      window_memoryarea->memoryarea_bank = 0x00;
      window_memoryarea->memoryarea_address = 0x0300; 
    }
  sprintf(buffer,"%02X",window_memoryarea->memoryarea_bank);
  SetDlgItemText(window_memoryarea->hwnd_memoryarea,IDC_EDIT_BANK,buffer);
  sprintf(buffer,"%04X",window_memoryarea->memoryarea_address);
  SetDlgItemText(window_memoryarea->hwnd_memoryarea,IDC_EDIT_ADDRESS,buffer);

  /** Zone d'affichage des Memory Data **/
  /* Taille minimale */
  window_memoryarea->text_min_data_column = 1;
  window_memoryarea->text_min_data_row    = 1;
  /* Taille par dfaut */
  window_memoryarea->nb_column = 16;
  window_memoryarea->nb_row    = 16; 
  SetDlgItemInt(window_memoryarea->hwnd_memoryarea,IDC_EDIT_COLUMN,window_memoryarea->nb_column,FALSE);
  SetDlgItemInt(window_memoryarea->hwnd_memoryarea,IDC_EDIT_ROW,window_memoryarea->nb_row,FALSE);

  /* Nombre caractres visibles sur la ligne : Address Data Ascii (on ne compte pas le \r\n final) */
  window_memoryarea->text_data_width = (int)(strlen("00/0000   ") + window_memoryarea->nb_column*3 - 1 + strlen("  ") + window_memoryarea->nb_column);   /* Enlve le dernier sparateur */

  /* Refresh Mode : Automatic */
  UpdateMemoryAreaRefreshMode(window_memoryarea,MEMORYAREA_REFRESH_AUTOMATIC);

  /* Hauteur Titre */
  GetWindowRect(window_memoryarea->hwnd_memoryarea,&window_rect);
  GetClientRect(window_memoryarea->hwnd_memoryarea,&client_rect);
  window_memoryarea->title_height = (window_rect.bottom - window_rect.top) - client_rect.bottom;

  /* Position X,Y de la GroupBox Memory Area */
  hwnd_group = GetDlgItem(window_memoryarea->hwnd_memoryarea,IDC_GROUP_MEMORYAREA);     /* Rcupre le Handle du Control GroupBox li  IDC_GROUP_MEMORYAREA */
  GetWindowControlInfo(window_memoryarea->hwnd_memoryarea,hwnd_group,&window_memoryarea->group_data_x,&window_memoryarea->group_data_y,&group_width,&group_height);

  /* Position de la Zone Text Memory Area dans la fenetre */
  window_memoryarea->memoryarea_data_x = window_memoryarea->group_data_x + WINDOW_GROUP_BORDER_LEFT_SIZE;
  window_memoryarea->memoryarea_data_y = window_memoryarea->group_data_y + WINDOW_GROUP_BORDER_TOP_SIZE;

  /* Dimension en Pixel du Control Text MemoryArea Data */
  window_memoryarea->memoryarea_data_width = window_memoryarea->text_data_width*MEMORYAREA_EDIT_FONT_WIDTH + 2;
  window_memoryarea->memoryarea_data_height = window_memoryarea->nb_row*MEMORYAREA_EDIT_FONT_HEIGHT;

  /* Dimension en Pixel du Control Group Box + Resize */  
  window_memoryarea->group_data_width = WINDOW_GROUP_BORDER_LEFT_SIZE + window_memoryarea->memoryarea_data_width + WINDOW_GROUP_BORDER_RIGHT_SIZE;
  window_memoryarea->group_data_height = WINDOW_GROUP_BORDER_TOP_SIZE + window_memoryarea->memoryarea_data_height + WINDOW_GROUP_BORDER_BOTTOM_SIZE;
  MoveWindow(hwnd_group,window_memoryarea->group_data_x,window_memoryarea->group_data_y,window_memoryarea->group_data_width,window_memoryarea->group_data_height,FALSE);

  /** Cration du Control Text Edit Multi ligne ReadOnly **/
  window_memoryarea->hwnd_edit_data = CreateWindowExA(0,"EDIT",NULL,WS_CHILD | WS_VISIBLE | ES_LEFT | ES_MULTILINE | ES_READONLY, 
                                                      window_memoryarea->memoryarea_data_x, window_memoryarea->memoryarea_data_y, window_memoryarea->memoryarea_data_width, window_memoryarea->memoryarea_data_height,   // set size in WM_SIZE message 
                                                      window_memoryarea->hwnd_memoryarea,
                                                      (HMENU)IDC_EDIT_MEMORYAREA_DATA,            // edit control ID
                                                      param->hInstance,
                                                      NULL);

  /** Surchage des Control Edit pour capter les vnements **/
  window_memoryarea->oldEditProcMemoryAreaData = (WNDPROC) SetWindowLongPtr(window_memoryarea->hwnd_edit_data,GWLP_WNDPROC,(LONG_PTR)subEditMemoryAreaProcControl);
  window_memoryarea->oldEditProcBank    = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndMemoryArea,IDC_EDIT_BANK),GWLP_WNDPROC,(LONG_PTR)subEditMemoryAreaProcControl);
  window_memoryarea->oldEditProcAddress = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndMemoryArea,IDC_EDIT_ADDRESS),GWLP_WNDPROC,(LONG_PTR)subEditMemoryAreaProcControl);
  window_memoryarea->oldEditProcColumn  = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndMemoryArea,IDC_EDIT_COLUMN),GWLP_WNDPROC,(LONG_PTR)subEditMemoryAreaProcControl);
  window_memoryarea->oldEditProcRow     = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndMemoryArea,IDC_EDIT_ROW),GWLP_WNDPROC,(LONG_PTR)subEditMemoryAreaProcControl);

  /** Cration de la Police de caractre + Affectation de la police au Control Edit **/
  window_memoryarea->hFont = CreateFont(MEMORYAREA_EDIT_FONT_HEIGHT,0,0,0,FW_DONTCARE,FALSE,FALSE,FALSE,ANSI_CHARSET,OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY,DEFAULT_PITCH|FF_SWISS,TEXT("Courier New"));
  SendMessage(window_memoryarea->hwnd_edit_data,WM_SETFONT,(WPARAM)window_memoryarea->hFont,MAKELPARAM(FALSE,0));  

  /* Retaille la fentre */
  ResizeMemoryAreaWindow(window_memoryarea);

  /** Construction du Texte + Rafraichissement de la fentre **/
  RefreshMemoryAreaWindowData(window_memoryarea);
    
  /* OK */
  return(0);
}


/*************************************************************************************************/
/*  subEditMemoryAreaProcControl() :  Rcupre les vnements qui arrivent dans le Control Edit. */
/*************************************************************************************************/
LRESULT CALLBACK subEditMemoryAreaProcControl(HWND hwnd_control, UINT msg, WPARAM wParam, LPARAM lParam)
{
  int index, mouse_x, mouse_y;
  struct window_memoryarea *window_memoryarea;
  my_Memory(MEMORY_GET_WINDOW_MEMORYAREA,&window_memoryarea,NULL);

  /* Rien  faire */
  if(window_memoryarea == NULL)
    return(0);

  /** Identifie le Control **/
  /* Src */
  if(window_memoryarea->hwnd_edit_data == hwnd_control)
    index = 1;
  else if(GetDlgItem(window_memoryarea->hwnd_memoryarea,IDC_EDIT_BANK) == hwnd_control)
    index = 2;
  else if(GetDlgItem(window_memoryarea->hwnd_memoryarea,IDC_EDIT_ADDRESS) == hwnd_control)
    index = 3;
  else if(GetDlgItem(window_memoryarea->hwnd_memoryarea,IDC_EDIT_COLUMN) == hwnd_control)
    index = 4;
  else if(GetDlgItem(window_memoryarea->hwnd_memoryarea,IDC_EDIT_ROW) == hwnd_control)
    index = 5;
  else
    return(0);      /* Pas trouv */

  /** Dtecte l'vnement de la souris dans l'Edit **/
  if(msg == WM_MOUSEMOVE)
    {
      /* Corrdonnes de la souris dans l'Edit */
      mouse_x = GET_X_LPARAM(lParam) / MEMORYAREA_EDIT_FONT_WIDTH;
      mouse_y = GET_Y_LPARAM(lParam) / MEMORYAREA_EDIT_FONT_HEIGHT;
      UpdateMemoryAreaContextInformation(window_memoryarea,mouse_x,mouse_y);
      
      /* Transmet l'vnement au Control Text */
      return(CallWindowProc(window_memoryarea->oldEditProcMemoryAreaData,hwnd_control,msg,wParam,lParam)); 
    }
  else if(msg == WM_CHAR && wParam == VK_RETURN)
    {
      /* Enlve le focus de l'Edit */
      SetFocus(window_memoryarea->hwnd_memoryarea);

      /* On force la MAJ de la fentre */
      RefreshMemoryAreaWindowData(window_memoryarea);
    }
  else if(index == 1)
    return(CallWindowProc(window_memoryarea->oldEditProcMemoryAreaData,hwnd_control,msg,wParam,lParam)); 
  else if(index == 2)
    return(CallWindowProc(window_memoryarea->oldEditProcBank,hwnd_control,msg,wParam,lParam)); 
  else if(index == 3)
    return(CallWindowProc(window_memoryarea->oldEditProcAddress,hwnd_control,msg,wParam,lParam)); 
  else if(index == 4)
    return(CallWindowProc(window_memoryarea->oldEditProcColumn,hwnd_control,msg,wParam,lParam));
  else if(index == 5)
    return(CallWindowProc(window_memoryarea->oldEditProcRow,hwnd_control,msg,wParam,lParam));
         
  /* OK */
  return(0);
} 


/*******************************************************************************/
/*  UpdateMemoryAreaAddress() :  Dplace l'adresse pour la fentre MemoryArea. */
/*******************************************************************************/
static void UpdateMemoryAreaAddress(struct window_memoryarea *window_memoryarea, int delta)
{
  int current_address, new_address, memoryarea_bank, memoryarea_address;
  char buffer[BUFFER_SIZE];

  /* Adresse actuelle */
  current_address = (window_memoryarea->memoryarea_bank << 16) | window_memoryarea->memoryarea_address;
  if((current_address + delta) < 0)
    return;
  if((current_address + delta) > 0xFFFFFF)
    return;

  /* Nouvelle Addresse */
  new_address = current_address + delta;
  memoryarea_bank = (new_address >> 16) & 0xFF;
  memoryarea_address = new_address & 0xFFFF;

  /* Replace les valeurs dans les Control pour montrer ce qui a t compris */
  sprintf(buffer,"%02X",memoryarea_bank);
  SetDlgItemText(window_memoryarea->hwnd_memoryarea,IDC_EDIT_BANK,buffer);
  sprintf(buffer,"%04X",memoryarea_address);
  SetDlgItemText(window_memoryarea->hwnd_memoryarea,IDC_EDIT_ADDRESS,buffer);

  /* MAJ de la fentre */
  RefreshMemoryAreaWindowData(window_memoryarea);
}


/*******************************************************************************************************/
/*  UpdateMemoryAreaContextInformation() :  MAJ des information de contexte de la fenetre MemoryArea. */
/*******************************************************************************************************/
static void UpdateMemoryAreaContextInformation(struct window_memoryarea *window_memoryarea, int grid_x, int grid_y)
{
  int grid_memoryarea_x, memoryarea_data_address, grid_memoryarea_offset, memoryarea_data_offset;
  unsigned char memoryarea_data;
  
  /* Rien  faire */
  if(window_memoryarea == NULL)
    return;
    
  /* En dehors de la zone */
  if(grid_x < 10 || grid_x > (10+window_memoryarea->nb_column*3-1))
    {
      SetWindowText(GetDlgItem(window_memoryarea->hwnd_memoryarea,IDC_STATIC_CONTEXT),"");
      return;
    }
  grid_memoryarea_x = grid_x - 10;     /* Enlve les 10 caractres de gauche */
  if(((grid_memoryarea_x+1) % 3) == 0)
    {
      SetWindowText(GetDlgItem(window_memoryarea->hwnd_memoryarea,IDC_STATIC_CONTEXT),"");
      return;
    }
  grid_memoryarea_offset = (grid_memoryarea_x+1) / 3;
  
  /* Rcupre l'adresse de la cellule */
  memoryarea_data_address = window_memoryarea->memoryarea_address + grid_y*window_memoryarea->nb_column;
  memoryarea_data_address += grid_memoryarea_offset;
  
  /* Rcupre la valeur de la cellule */
  memoryarea_data_offset = memoryarea_data_address - window_memoryarea->memoryarea_address;
  if(memoryarea_data_offset > (window_memoryarea->nb_column*window_memoryarea->nb_row-1))
    {
      SetWindowText(GetDlgItem(window_memoryarea->hwnd_memoryarea,IDC_STATIC_CONTEXT),"");
      return;
    }
  memoryarea_data = window_memoryarea->memoryarea_data[memoryarea_data_offset];
  
  /* Coordonnes de la souris en coodonnes grid_x,grid_y */
  sprintf(window_memoryarea->buffer_log,"Memory Data  %02X/%04X : %02X",memoryarea_data_address>>16,memoryarea_data_address&0xFFFF,memoryarea_data);
  SetWindowText(GetDlgItem(window_memoryarea->hwnd_memoryarea,IDC_STATIC_CONTEXT),window_memoryarea->buffer_log);
}


/************************************************************************************************/
/*  UpdateMemoryAreaRefreshMode() :  MAJ du mode de rafrachissement de la fenetre MemoryArea. */
/************************************************************************************************/
static void UpdateMemoryAreaRefreshMode(struct window_memoryarea *window_memoryarea, int refresh_mode)
{
  struct notification *current_notification;

  if(refresh_mode == MEMORYAREA_REFRESH_AUTOMATIC)
    {
      /* Visuel */
      window_memoryarea->refresh_mode = MEMORYAREA_REFRESH_AUTOMATIC;
      SendDlgItemMessage(window_memoryarea->hwnd_memoryarea,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_memoryarea->hwnd_memoryarea,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,0,0);

      /* Cration d'une Notification */
      current_notification = (struct notification *) calloc(1,sizeof(struct notification));
      if(current_notification == NULL)
        return;
      current_notification->hwnd_window = window_memoryarea->hwnd_memoryarea;
      current_notification->message = HM_MEMORYAREA_REFRESHWINDOW;
      current_notification->type = NOTIFICATION_TYPE_VIEW;
      current_notification->sub_type = NOTIFICATION_SUBTYPE_MEMORYAREA;

      /* Ajout de la notification */
      window_memoryarea->current_notification = current_notification;
      my_Notification(NOTIFICATION_ADD,current_notification,NULL);
    }
  else
    {
      /* Visuel */
      window_memoryarea->refresh_mode = MEMORYAREA_REFRESH_MANUAL;
      SendDlgItemMessage(window_memoryarea->hwnd_memoryarea,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_memoryarea->hwnd_memoryarea,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,1,0);

      /* Supprime la Notification */
      my_Notification(NOTIFICATION_REMOVE,window_memoryarea->current_notification,NULL);
      window_memoryarea->current_notification = NULL;
    }
}


/*************************************************************************/
/*  UserResizeMemoryAreaWindow() :  Retaille la fentre de zone mmoire. */
/*************************************************************************/
static void UserResizeMemoryAreaWindow(struct window_memoryarea *window_memoryarea)
{
  RECT window_rect;
  int window_new_width, window_new_height, delta_pixel_width, delta_pixel_height, delta_char_width, delta_char_height;
  int nb_new_text_column, nb_new_text_row, nb_new_data_column, nb_new_data_row;

  /* Position + Dimension actuelle de la fentre retaille dans l'cran */
  GetWindowRect(window_memoryarea->hwnd_memoryarea,&window_rect);

  /* Nouvelle Dimension de la fentre (avec la barre de titre et les bordures) */
  window_new_width  = window_rect.right  - window_rect.left;
  window_new_height = window_rect.bottom - window_rect.top;

  /* Calcule le Delta en Pixel par rapport  la taille prcdente */
  delta_pixel_width  = window_new_width  - window_memoryarea->window_metric_width;
  delta_pixel_height = window_new_height - window_memoryarea->window_metric_height;

  /* Calcule le Delta en Charactres par rapport  la taille prcdente */
  delta_char_width  = delta_pixel_width / MEMORYAREA_EDIT_FONT_WIDTH;
  delta_char_height = delta_pixel_height / MEMORYAREA_EDIT_FONT_HEIGHT;

  /* Nouvelle dimension de la zone Texte (Adresse + Data Hexa + Data Ascii) */
  nb_new_text_column = (window_memoryarea->window_metric_column*4)+11 + delta_char_width;
  nb_new_text_row    = window_memoryarea->window_metric_row           + delta_char_height;

  /* On dtermine la nouvelle dimension de la zone Data (Data Hexa) */
  nb_new_data_column = (nb_new_text_column - 11) / 4;   /* 4 bytes par valeur */
  nb_new_data_row    = nb_new_text_row;

  /** Teste les valeurs minimales autorises **/
  if(nb_new_data_column < window_memoryarea->text_min_data_column)
    nb_new_data_column = window_memoryarea->text_min_data_column;
  if(nb_new_data_row < window_memoryarea->text_min_data_row)
    nb_new_data_row = window_memoryarea->text_min_data_row;

  /* MAJ du nombre de Data colonnes + lignes affiches */
  SetDlgItemInt(window_memoryarea->hwnd_memoryarea,IDC_EDIT_COLUMN,nb_new_data_column,FALSE);
  SetDlgItemInt(window_memoryarea->hwnd_memoryarea,IDC_EDIT_ROW,nb_new_data_row,FALSE);

  /** Reconstruit les lements de la fentre avec les nouvelles valeurs du Rich Edit **/
  RefreshMemoryAreaWindowData(window_memoryarea);

  /* Retaille la fentre pour viter les tailles entre deux */
  ResizeMemoryAreaWindow(window_memoryarea);
}


/******************************************************************************/
/*  RefreshMemoryAreaWindowData() :  MAJ du contenu de la fentre MemoryArea. */
/******************************************************************************/
static void RefreshMemoryAreaWindowData(struct window_memoryarea *window_memoryarea)
{
  int length, is_modified;
  int memoryarea_bank, memoryarea_address, nb_column, nb_row;
  char buffer[1024];

  /* Rcupre la valeur du MemoryArea Pointer */
  length = GetDlgItemText(window_memoryarea->hwnd_memoryarea,IDC_EDIT_BANK,buffer,1024);
  sscanf(buffer,"%X",&memoryarea_bank);
  length = GetDlgItemText(window_memoryarea->hwnd_memoryarea,IDC_EDIT_ADDRESS,buffer,1024);
  sscanf(buffer,"%X",&memoryarea_address);
  nb_column = GetDlgItemInt(window_memoryarea->hwnd_memoryarea,IDC_EDIT_COLUMN,NULL,FALSE);
  if(nb_column < window_memoryarea->text_min_data_column)
    nb_column = window_memoryarea->text_min_data_column;
  nb_row = GetDlgItemInt(window_memoryarea->hwnd_memoryarea,IDC_EDIT_ROW,NULL,FALSE);
  if(nb_row < window_memoryarea->text_min_data_row)
    nb_row = window_memoryarea->text_min_data_row;

  /* Replace les valeurs dans les Control pour montrer ce qui a t compris */
  sprintf(buffer,"%02X",memoryarea_bank);
  SetDlgItemText(window_memoryarea->hwnd_memoryarea,IDC_EDIT_BANK,buffer);
  sprintf(buffer,"%04X",memoryarea_address);
  SetDlgItemText(window_memoryarea->hwnd_memoryarea,IDC_EDIT_ADDRESS,buffer);
  SetDlgItemInt(window_memoryarea->hwnd_memoryarea,IDC_EDIT_COLUMN,nb_column,FALSE);
  SetDlgItemInt(window_memoryarea->hwnd_memoryarea,IDC_EDIT_ROW,nb_row,FALSE);

  /** Les donnes ont t'elles t modifies + Rcupration des nouvelles Data si besoin **/
  is_modified = IsMemoryAreaDataModified(memoryarea_bank,memoryarea_address,nb_column,nb_row,window_memoryarea);
  if(is_modified == 0)
    return;
    
  /** Cration de la Zone Text Edit  partir des nouvelles donnes **/
  BuildMemoryAreaFromData(window_memoryarea);

  /* Rafraichit la fentre */
  InvalidateRect(window_memoryarea->hwnd_memoryarea,NULL,TRUE);
  UpdateWindow(window_memoryarea->hwnd_memoryarea);
}


/*************************************************************************************************************/
/*  IsMemoryAreaDataModified() :  Les donnes MemoryArea ont t'elles t modifies depuis la dernirer fois. */
/*************************************************************************************************************/
static int IsMemoryAreaDataModified(int memoryarea_bank, int memoryarea_address, int nb_column, int nb_row, struct window_memoryarea *window_memoryarea)
{
  int memoryarea_data_address;
  unsigned char *memoryarea_data;
  struct mmu *current_mmu;

  /* Rcupre le MMU */
  my_Memory(MEMORY_GET_S65816_MMU,&current_mmu,NULL);
  
  /* Allocation mmoire */
  memoryarea_data = (unsigned char *) calloc(nb_column*nb_row+1,sizeof(unsigned char));
  if(memoryarea_data == NULL)
    return(0);

  /* Rcupre les donnes */
  memoryarea_data_address = (memoryarea_bank << 16) | memoryarea_address;
  ReadMemoryBlock(memoryarea_data_address,nb_column*nb_row,memoryarea_data,current_mmu);
  
  /* Compare les donnes */
  if(window_memoryarea->memoryarea_data != NULL)
    if(memoryarea_bank == window_memoryarea->memoryarea_bank &&
       memoryarea_address == window_memoryarea->memoryarea_address &&
       nb_column == window_memoryarea->nb_column &&
       nb_row == window_memoryarea->nb_row &&
       memcmp(&memoryarea_data[0],&window_memoryarea->memoryarea_data[0],MY_MIN(nb_column*nb_row,window_memoryarea->nb_column*window_memoryarea->nb_row)) == 0)
      return(0);
    
  /* Les donnes sont diffrentes => MAJ */
  window_memoryarea->memoryarea_bank = memoryarea_bank;
  window_memoryarea->memoryarea_address = memoryarea_address;
  window_memoryarea->nb_column = nb_column;
  window_memoryarea->nb_row = nb_row;
  if(window_memoryarea->memoryarea_data != NULL)
    free(window_memoryarea->memoryarea_data);
  window_memoryarea->memoryarea_data = memoryarea_data;

  /* Donnes diffrentes */
  return(1);
}


/***********************************************************************/
/*  ResizeMemoryAreaWindow() : Resize MemoryArea window by given size. */
/***********************************************************************/
static void ResizeMemoryAreaWindow(struct window_memoryarea *window_memoryarea)
{
  RECT window_rect;
  int window_x, window_y;
  
  /* Largeur de la partie Client de la fentre */
  window_memoryarea->window_width = window_memoryarea->group_data_x + window_memoryarea->group_data_width + WINDOW_CLIENT_RIGHT_BORDER_SIZE + WINDOW_EXTRA_RESIZE_RIGHT_BORDER_SIZE;

  /* Hauteur der la partie Client de la fentre */
  window_memoryarea->window_height = window_memoryarea->group_data_y + window_memoryarea->group_data_height + WINDOW_CLIENT_BOTTOM_BORDER_SIZE;

  /* Position actuelle de la fentre dans l'cran */
  GetWindowRect(window_memoryarea->hwnd_memoryarea,&window_rect);
  window_x = window_rect.left;
  window_y = window_rect.top;

  /* Resize window */
  window_memoryarea->user_resize = 0;
  MoveWindow(window_memoryarea->hwnd_memoryarea,window_x,window_y,window_memoryarea->window_width,window_memoryarea->title_height+window_memoryarea->window_height,FALSE);

  /* On va conserver les mtriques de la fentre actuellement affiche */
  window_memoryarea->window_metric_width  = window_memoryarea->window_width;
  window_memoryarea->window_metric_height = window_memoryarea->title_height + window_memoryarea->window_height;
  window_memoryarea->window_metric_column = window_memoryarea->nb_column;
  window_memoryarea->window_metric_row    = window_memoryarea->nb_row;
}


/************************************************************************************************************/
/*  BuildMemoryAreaFromData() :  Construire les donnes hexadcimal  afficher  partir des donnes brutes. */
/************************************************************************************************************/
static void BuildMemoryAreaFromData(struct window_memoryarea *window_memoryarea)
{
  HWND hwnd_group;
  int i, j, offset, offset_text, nb_byte_to_process, address_data, text_data_width_min;
  char buffer_title[256];

  /** On doit peut tre redimensionner la zone Text Edit dynamique **/
  if(window_memoryarea->text_data_hexadecimal == NULL || 
     window_memoryarea->text_data_column != window_memoryarea->nb_column ||
     window_memoryarea->text_data_row != window_memoryarea->nb_row)
    {
      /* MAJ */
      window_memoryarea->text_data_column = window_memoryarea->nb_column;
      window_memoryarea->text_data_row = window_memoryarea->nb_row;

      /* Libration mmoire */
      if(window_memoryarea->text_data_hexadecimal)
        free(window_memoryarea->text_data_hexadecimal);
      window_memoryarea->text_data_hexadecimal = NULL;

      /* Nombre caractres visibles sur la ligne : Address Data Ascii (on ne compte pas le \r\n final) */
      window_memoryarea->text_data_width = (int)(strlen("00/0000   ") + window_memoryarea->nb_column*3 - 1 + strlen("  ") + window_memoryarea->nb_column);   /* Enlve le dernier sparateur */

      /* Allocation mmoire */
      window_memoryarea->text_data_size = (int) ((window_memoryarea->text_data_width + strlen("\r\n"))*window_memoryarea->text_data_row);
      window_memoryarea->text_data_hexadecimal = (char *) calloc(window_memoryarea->text_data_size+1,sizeof(char));
      if(window_memoryarea->text_data_hexadecimal == NULL)
        return;

      /* Dimension en Pixel du Control Text MemoryArea Data */
      text_data_width_min = (int) strlen("00/0300   26 2F 00 00 00 00 00 00.00 00 00 00  &/.........");
      window_memoryarea->memoryarea_data_width = MY_MAX(window_memoryarea->text_data_width,text_data_width_min)*MEMORYAREA_EDIT_FONT_WIDTH + 2;
      window_memoryarea->memoryarea_data_height = window_memoryarea->nb_row*MEMORYAREA_EDIT_FONT_HEIGHT;
      MoveWindow(window_memoryarea->hwnd_edit_data,window_memoryarea->memoryarea_data_x,window_memoryarea->memoryarea_data_y,window_memoryarea->memoryarea_data_width,window_memoryarea->memoryarea_data_height,FALSE);

      /* Dimension en Pixel du Control Group Box + Resize */  
      hwnd_group = GetDlgItem(window_memoryarea->hwnd_memoryarea,IDC_GROUP_MEMORYAREA);     /* Rcupre le Handle du Control GroupBox li  IDC_GROUP_MEMORYAREA */
      window_memoryarea->group_data_width = WINDOW_GROUP_BORDER_LEFT_SIZE + window_memoryarea->memoryarea_data_width + WINDOW_GROUP_BORDER_RIGHT_SIZE;
      window_memoryarea->group_data_height = WINDOW_GROUP_BORDER_TOP_SIZE + window_memoryarea->memoryarea_data_height + WINDOW_GROUP_BORDER_BOTTOM_SIZE;
      MoveWindow(hwnd_group,window_memoryarea->group_data_x,window_memoryarea->group_data_y,window_memoryarea->group_data_width,window_memoryarea->group_data_height,FALSE);

      /* Retaille la fentre */
      ResizeMemoryAreaWindow(window_memoryarea);
    }

  /* Adresse des donnes */
  address_data = (window_memoryarea->memoryarea_bank << 16) + window_memoryarea->memoryarea_address;

  /*** Transformer les donnes BYTE en Hexa ***/
  nb_byte_to_process = window_memoryarea->nb_column*window_memoryarea->nb_row;
  for(i=0,offset=0,offset_text=0; i<window_memoryarea->nb_row; i++,offset+=window_memoryarea->nb_column)
    {
      /* Adresse */
      sprintf(&window_memoryarea->text_data_hexadecimal[offset_text],"%02X/%04X   ",(offset+address_data)>>16,(offset+address_data) & 0xFFFF);
      offset_text += (int) strlen("00/0000   ");

      /** Hexa **/
      for(j=0; j<window_memoryarea->nb_column; j++)
        {
          if(i*window_memoryarea->nb_column+j < nb_byte_to_process)
            sprintf(&window_memoryarea->text_data_hexadecimal[offset_text],"%02X",window_memoryarea->memoryarea_data[offset+j]);
          else
            strcpy(&window_memoryarea->text_data_hexadecimal[offset_text],"  ");
          offset_text += 2;

          /* Sparateur */
          if(j < (window_memoryarea->nb_column-1))
            {
              window_memoryarea->text_data_hexadecimal[offset_text] = ((j > 0 && (((j+1)%8)==0)) && (i*window_memoryarea->nb_column+j < nb_byte_to_process)) ? '.' : ' ';
              offset_text++;
            }
        }

      /* Sparateur */
      strcpy(&window_memoryarea->text_data_hexadecimal[offset_text],"  ");
      offset_text += 2;

      /** Ascii **/
      for(j=0; j<window_memoryarea->nb_column; j++)
        {
          if(i*window_memoryarea->nb_column+j < nb_byte_to_process)
            window_memoryarea->text_data_hexadecimal[offset_text] = GETASCIICODE(window_memoryarea->memoryarea_data[offset+j]);
          else
            window_memoryarea->text_data_hexadecimal[offset_text] = ' ';
          offset_text++;
        }

      /* Fin de ligne */
      strcpy(&window_memoryarea->text_data_hexadecimal[offset_text],"\r\n");
      offset_text += 2;
    }
    
  /** Efface le contenu de la fentre **/
  SetWindowText(window_memoryarea->hwnd_edit_data,"");

  /** Afficher le rsultat dans le control **/
  SetWindowText(window_memoryarea->hwnd_edit_data,window_memoryarea->text_data_hexadecimal);
  
  /* Titre de la fentre */
  sprintf(buffer_title,"Memory Area : %02X/%04X",window_memoryarea->memoryarea_bank,window_memoryarea->memoryarea_address);
  SetWindowText(window_memoryarea->hwnd_memoryarea,buffer_title);
}


/************************************************************************/
/*  DeleteMemoryAreaChildWindows() :  Destruction des fentres enfants. */
/************************************************************************/
static void DeleteMemoryAreaChildWindows(struct window_memoryarea *window_memoryarea)
{
  /** Libration des ressources **/
  DeleteObject(window_memoryarea->hFont);

  if(window_memoryarea->text_data_hexadecimal != NULL)
    free(window_memoryarea->text_data_hexadecimal);
  window_memoryarea->text_data_hexadecimal = NULL;

  if(window_memoryarea->memoryarea_data != NULL)
    free(window_memoryarea->memoryarea_data);
  window_memoryarea->memoryarea_data = NULL;
}


/**************************************************************************/
/*  mem_free_window_memoryarea() :  Libration mmoire pour la structure. */
/**************************************************************************/
void mem_free_window_memoryarea(struct window_memoryarea *window_memoryarea)
{
  if(window_memoryarea)
    {
      /* Supprime la notification */
      my_Notification(NOTIFICATION_REMOVE,window_memoryarea->current_notification,NULL);
      window_memoryarea->current_notification = NULL;
      
      free(window_memoryarea);
    }
}

/*************************************************************************/
