/**************************************************************************/
/*                                                                        */
/*  Window_MemoryMap.c :  Module de gestion de la fentre MemoryMap.      */
/*                                                                        */
/**************************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe  *  Date : Septembre 2018  */
/**************************************************************************/

#include <stdio.h>
#include <time.h>
#include <sys/timeb.h>

#include <windows.h>
#include <windowsx.h>
#include <Commctrl.h>
#include <Winuser.h>
#include <Richedit.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_MMU.h"
#include "Cyrene_Engine.h"
#include "Dc_Notification.h"
#include "Dc_Graphic.h"
#include "Dc_CPU.h"
#include "Dc_IIgsInternals.h"
#include "Window_QuickWatch.h"
#include "Window_Main.h"
#include "Window_Main_Menu.h"
#include "Window_MemoryMap.h"

static DWORD BuildMemoryMapWindowThread(void *);
LRESULT CALLBACK ProcHandlerMemoryMapWindow(HWND,UINT,WPARAM,LPARAM);
static int CreateMemoryMapChildWindows(HWND,struct window_memorymap *);
LRESULT CALLBACK subEditMemoryMapProcControl(HWND,UINT,WPARAM,LPARAM);
static void ResetMemoryMapBitmap(struct window_memorymap *);
static void UpdateMemoryMapFastRamAddress(struct window_memorymap *,int);
static void UpdateMemoryMapContextInformation(struct window_memorymap *,int,int);
void UpdateMemoryMapClientAllArea(HDC,struct window_memorymap *);
static void UpdateMemoryMapClientBitmapArea(HDC,struct window_memorymap *);
static void BuildMemoryMapBitmapRWX(struct window_memorymap *);
static void MarkMemoryBitmapSelectedHandle(struct window_memorymap *);
static DWORD *GetMapColorPointer(DWORD,int,struct window_memorymap *);
static BYTE GetRamValue(DWORD,int,struct mmu *);
static BYTE GetMapRamRWX(DWORD,int,struct mmu *);
static int compare_handle(const void *,const void *);
static void CreateFileMemoryMapRWX(struct window_memorymap *);
static void DeleteMemoryMapChildWindows(struct window_memorymap *);


/********************************************************************************/
/*  BuildMemoryMapWindow() :  Construction / Affichage de la fentre MemoryMap. */
/********************************************************************************/
struct window_memorymap *BuildMemoryMapWindow(HINSTANCE hInstance, int menu_id)
{
  struct window_memorymap *window_memorymap;

  /** Allocation de la structure window_memorymap **/
  window_memorymap = (struct window_memorymap *) calloc(1,sizeof(struct window_memorymap));
  if(window_memorymap == NULL)
    return(NULL);
  my_Memory(MEMORY_SET_WINDOW_MEMORYMAP,window_memorymap,NULL);

  /* Lance un thread pour grer la fentre MemoryMap */
  CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)BuildMemoryMapWindowThread,window_memorymap,0,NULL);

  /* Renvoi la structure mmoire grant la fentre */
  return(window_memorymap);
}


/******************************************************************************************/
/*  BuildMemoryMapWindowThread() :  Cration de la fentre MemoryMap via un Thread ddi. */
/******************************************************************************************/
static DWORD BuildMemoryMapWindowThread(void *window_memorymap_param)
{
  HWND hwndMemoryMap;
  MSG msg;
  BOOL bRet;
  struct window_main *window_main;
  struct window_memorymap *window_memorymap;
  struct parameter *param;
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Init */
  window_memorymap = (struct window_memorymap *) window_memorymap_param;
  window_memorymap->thread_id = GetCurrentThreadId();         /* Le ThreadID sert  diffrencier les diffrentes fentres */

  /** Cration de la fentre Memory Map **/
  hwndMemoryMap = CreateDialog(param->hInstance,MAKEINTRESOURCE(IDD_WINDOW_MEMORYMAP),window_main->hwnd_main,(DLGPROC)ProcHandlerMemoryMapWindow);
  if(hwndMemoryMap == NULL)
    return(1);

  /** Show Window + Paint Content **/
  ShowWindow(hwndMemoryMap,SW_SHOWDEFAULT);
  UpdateWindow(hwndMemoryMap);

  /** Message Loop pour toutes les fentres de ce Thread (fentre MemoryMap + ComboBox + ...) **/
  while((bRet = GetMessage(&msg,NULL,0,0)) != 0)
    {
      if(bRet == -1)
        break;

      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }

  /* Fin du Thread */
  ExitThread(0);
}


/**************************************************************************/
/*  ProcHandlerMemoryMapWindow() :  Gestion des vnements de la fentre. */
/**************************************************************************/
LRESULT CALLBACK ProcHandlerMemoryMapWindow(HWND hwndMemoryMap, UINT umsg, WPARAM wParam, LPARAM lParam)
{
  HDC hDC;
  LRESULT result;
  PAINTSTRUCT ps;
  int mouse_x, mouse_y;
  struct window_memorymap *window_memorymap;
  struct parameter *param;
  
  /* Init */
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  my_Memory(MEMORY_GET_WINDOW_MEMORYMAP,&window_memorymap,NULL);
  if(window_memorymap == NULL)
    return(TRUE);

  switch(umsg)
    {
      case WM_INITDIALOG :
        /* Cration du contenu de la DialogBox */
        CreateMemoryMapChildWindows(hwndMemoryMap,window_memorymap);
        return(TRUE);

      /***************************/
      /* Evnements de la souris */
      /***************************/
      case WM_MOUSEMOVE :
        mouse_x = GET_X_LPARAM(lParam);
        mouse_y = GET_Y_LPARAM(lParam);
        UpdateMemoryMapContextInformation(window_memorymap,mouse_x,mouse_y);
        return(TRUE);

      /**********************************/
      /* Rafrachissement de la fentre */
      /**********************************/
      case WM_PAINT:
        if(window_memorymap->bitmap_data_fastram[0] != NULL)
          {
            /* Begin Paint */
            hDC = BeginPaint(hwndMemoryMap,&ps);

            /** MAJ Bitmap **/
            UpdateMemoryMapClientAllArea(hDC,window_memorymap);

            /* End Paint */
            EndPaint(hwndMemoryMap,&ps);
          }
        break;
        
      /****************************/
      /* Evnements de la fentre */
      /****************************/
      case WM_COMMAND :
        switch(LOWORD(wParam))
          {
            /** Bouton Radio : On change la mthode de rafraichissement de la fentre **/
            case IDC_RADIO_REFRESH_AUTOMATIC :
              if(HIWORD(wParam) == BN_CLICKED && window_memorymap->refresh_mode != MEMORYMAP_REFRESH_AUTOMATIC)
                {
                  window_memorymap->refresh_mode = MEMORYMAP_REFRESH_AUTOMATIC;
                  SendDlgItemMessage(window_memorymap->hwnd_memorymap,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,1,0);
                  SendDlgItemMessage(window_memorymap->hwnd_memorymap,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,0,0);
                }
              break;
            case IDC_RADIO_REFRESH_MANUAL :
              if(HIWORD(wParam) == BN_CLICKED && window_memorymap->refresh_mode != MEMORYMAP_REFRESH_MANUAL)
                {
                  window_memorymap->refresh_mode = MEMORYMAP_REFRESH_MANUAL;
                  SendDlgItemMessage(window_memorymap->hwnd_memorymap,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,0,0);
                  SendDlgItemMessage(window_memorymap->hwnd_memorymap,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,1,0);                  
                }
              break;
              
            /** Bouton de dplacement de l'adresse FastRam **/
            case IDC_BUTTON_LEFT_FAST :
              UpdateMemoryMapFastRamAddress(window_memorymap,-16);
              break;
            case IDC_BUTTON_LEFT :
              UpdateMemoryMapFastRamAddress(window_memorymap,-1);
              break;
            case IDC_BUTTON_RIGHT :
              UpdateMemoryMapFastRamAddress(window_memorymap,1);
              break;
            case IDC_BUTTON_RIGHT_FAST :
              UpdateMemoryMapFastRamAddress(window_memorymap,16);
              break;              

            /** Slection dans la ComboBox : Source Symbol Name **/
            case IDC_COMBOBOX_HANDLE :
              if(HIWORD(wParam) == CBN_CLOSEUP)
                {
                  /* Rcupre l'index de ce qui est slectionn */
                  result = SendMessage(window_memorymap->hwnd_combo_handle,CB_GETCURSEL,0,0);
                  if(result != CB_ERR)
                    {
                      /* Numro du Handle dans son tableau */
                      window_memorymap->selected_handle = (int) result;
                      if(window_memorymap->selected_handle != 0)
                        {
                          /** Marque en noir le Handle slectionn **/
                          MarkMemoryBitmapSelectedHandle(window_memorymap);

                          /** Cration des Bitmaps **/
                          BuildMemoryMapBitmapRWX(window_memorymap);

                          /* MAJ de la fentre */
                          InvalidateRect(window_memorymap->hwnd_memorymap,NULL,TRUE);
                          UpdateWindow(window_memorymap->hwnd_memorymap);
                        }
                    }
                }
              break;

            /** Bouton de Reset des graphiques de la fentre **/
            case IDC_BUTTON_RESET :
              /** RAZ RWX + Construction de l'image grise des zones occupes **/
              ResetMemoryMapBitmap(window_memorymap);
              
              /* MAJ de la fentre */
              InvalidateRect(window_memorymap->hwnd_memorymap,NULL,TRUE);
              UpdateWindow(window_memorymap->hwnd_memorymap);
              break;

            /** Bouton de rafraichissement Manuel de la fentre **/
            case IDC_BUTTON_REFRESH :
              /** Cration des Bitmaps **/
              BuildMemoryMapBitmapRWX(window_memorymap);

              /* MAJ de la fentre */
              InvalidateRect(window_memorymap->hwnd_memorymap,NULL,TRUE);
              UpdateWindow(window_memorymap->hwnd_memorymap);
              break;

            /** Bouton de Sauvegarde des donnes RWX **/
            case IDC_BUTTON_SAVE :
              /** Cration du fichier **/
              CreateFileMemoryMapRWX(window_memorymap);
              break;

          }
        return(TRUE);

      /*****************************************************************/
      /* Message extrieur demandant le rafraichissement de la fentre */
      /*****************************************************************/
      case HM_MEMORYMAP_REFRESHWINDOW :
        /** MAJ des Bitmaps RWX **/
        BuildMemoryMapBitmapRWX(window_memorymap);

        /* MAJ de la fentre */
        InvalidateRect(window_memorymap->hwnd_memorymap,NULL,TRUE);
        UpdateWindow(window_memorymap->hwnd_memorymap);
        return(TRUE);

      /********************************************************************/
      /* Fermeture de window (le WM_CLOSE est appel avant le WM_DESTROY) */
      /********************************************************************/
      case WM_CLOSE :
        /* Fentre MemoryMap */
        DestroyWindow(hwndMemoryMap);
        return(0);

      case WM_DESTROY :
        /* Suppression des Fentres */
        DeleteMemoryMapChildWindows(window_memorymap);
   
        /* Enlve la coche du menu */
        CheckMainWindowMenuItem(ID_MENU_VIEW_MEMORYMAP,FALSE);

        /* Supprime la structure mmoire de ce thread */
        my_Memory(MEMORY_FREE_WINDOW_MEMORYMAP,window_memorymap,NULL);

        /* Demande la sortie de la boucle des messages */
        PostQuitMessage(0);
        return(0);
    }

  /* Retourner FALSE pour relayer le message non trait au systme */
  return(FALSE);
}


/*********************************************************************************************************/
/*  CreateMemoryMapChildWindows() : Crer les fentres enfants et remplir la structure window_memorymap. */
/*********************************************************************************************************/
static int CreateMemoryMapChildWindows(HWND hwndMemoryMap, struct window_memorymap *window_memorymap)
{
  BITMAPINFO bitmap_info;
  HWND hwnd_group, hwnd_label, hwnd_button;
  RECT window_rect, client_rect, group_rect, button_rect;
  int window_x, window_y, label_x, label_y, button_x, button_y;
  int i, j, nb_display_bank, all_group_box_width, offset_rom, extra_length;
  DWORD tab_rgb_color[7] =    /* 7 couleurs pour les libells */
    {
      MAKECOLOR(0,255,0),       /* Read                   : Vert */
      MAKECOLOR(255,0,0),       /* Write                  : Rouge */
      MAKECOLOR(0,0,255),       /* Execute                : Bleu */
      MAKECOLOR(255,255,0),     /* Read + Write           : Jaune */
      MAKECOLOR(0,255,255),     /* Execute + Read         : Cian */
      MAKECOLOR(255,0,255),     /* Execute + Write        : Violet */
      MAKECOLOR(255,255,255)    /* Execute + Read + Write : Blanc */
    };  
  int tab_label_id[7] = {IDC_STATIC_LABEL1, IDC_STATIC_LABEL2, IDC_STATIC_LABEL3, IDC_STATIC_LABEL4, IDC_STATIC_LABEL5, IDC_STATIC_LABEL6, IDC_STATIC_LABEL7};
  char buffer[2048];
  char buffer_title[256];
  char buffer_handle_type[1024];
  char buffer_handle_information[1024];
  struct memorymap_handle *current_memorymap_handle;
  struct memorymap_handle *first_memorymap_handle = NULL;
  struct memorymap_handle *last_memorymap_handle = NULL;
  struct gs_handle *current_gs_handle;
  struct notification *current_notification;  
  struct s65816 *current_s65816;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Rcupre le modle mmoire */
  my_Memory(MEMORY_GET_S65816,&current_s65816,NULL);
  if(current_s65816 == NULL)
    return(0);

  /** Window Handle **/
  window_memorymap->hwnd_memorymap = hwndMemoryMap;

  /** Device Context **/
  window_memorymap->hDC = GetDC(window_memorymap->hwnd_memorymap);
  window_memorymap->hCompatibleDC = CreateCompatibleDC(window_memorymap->hDC);

  /** Cration des Polices de caractre **/
  window_memorymap->hFont = CreateFont(14,0,0,0,FW_DONTCARE,FALSE,FALSE,FALSE,ANSI_CHARSET,OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY,DEFAULT_PITCH|FF_SWISS,"Arial");
  window_memorymap->hFont_courier = CreateFont(MEMORYMAP_EDIT_FONT_HEIGHT,0,0,0,FW_DONTCARE,FALSE,FALSE,FALSE,ANSI_CHARSET,OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY,DEFAULT_PITCH|FF_SWISS,TEXT("Courier New"));
  SendMessage(GetDlgItem(window_memorymap->hwnd_memorymap,IDC_STATIC_CONTEXT),WM_SETFONT,(WPARAM)window_memorymap->hFont_courier,MAKELPARAM(FALSE,0));
  SendMessage(GetDlgItem(window_memorymap->hwnd_memorymap,IDC_STATIC_CONTEXT_RWX),WM_SETFONT,(WPARAM)window_memorymap->hFont_courier,MAKELPARAM(FALSE,0));
  
  /** Ajoute les images sur les boutons <<  <  >  >> **/
  SendMessage(GetDlgItem(window_memorymap->hwnd_memorymap,IDC_BUTTON_LEFT),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_LEFT)));
  SendMessage(GetDlgItem(window_memorymap->hwnd_memorymap,IDC_BUTTON_RIGHT),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_RIGHT)));
  SendMessage(GetDlgItem(window_memorymap->hwnd_memorymap,IDC_BUTTON_LEFT_FAST),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_LEFT_FAST)));
  SendMessage(GetDlgItem(window_memorymap->hwnd_memorymap,IDC_BUTTON_RIGHT_FAST),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_ARROW_RIGHT_FAST)));

  /** Refresh Mode : Automatic **/
  /* Visuel */
  window_memorymap->refresh_mode = MEMORYMAP_REFRESH_AUTOMATIC;
  SendDlgItemMessage(window_memorymap->hwnd_memorymap,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,1,0);
  SendDlgItemMessage(window_memorymap->hwnd_memorymap,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,0,0);

  /** Rcupration **/
  for(current_gs_handle=current_s65816->list_used_handle; current_gs_handle; current_gs_handle=current_gs_handle->next)
    {
      /* Rcupration des informations */
      GetHandleIDType(current_gs_handle->id,buffer_handle_type);
      GetHandleInfo(current_gs_handle,buffer_handle_information);
      
      /* Allocation mmoire */
      current_memorymap_handle = (struct memorymap_handle *) calloc(1,sizeof(struct memorymap_handle));
      if(current_memorymap_handle != NULL)
        {
          /* Remplissage */
          current_memorymap_handle->address = current_gs_handle->address;
          current_memorymap_handle->length = current_gs_handle->size;
          current_memorymap_handle->id = current_gs_handle->id;
          strcpy(current_memorymap_handle->type,buffer_handle_type);
          strcpy(current_memorymap_handle->information,buffer_handle_information);

          /* Attache */
          window_memorymap->nb_handle++;
          if(first_memorymap_handle == NULL)
            first_memorymap_handle = current_memorymap_handle;
          else
            last_memorymap_handle->next = current_memorymap_handle;
          last_memorymap_handle = current_memorymap_handle;
        }
    }
  /* Tableau des Handle */
  window_memorymap->tab_handle = (struct memorymap_handle **) calloc(window_memorymap->nb_handle,sizeof(struct memorymap_handle *));
  if(window_memorymap->tab_handle == NULL)
    return(0);
  for(i=0, current_memorymap_handle=first_memorymap_handle; current_memorymap_handle; current_memorymap_handle=current_memorymap_handle->next, i++)
    window_memorymap->tab_handle[i] = current_memorymap_handle;
  /* Tri du tableau par adresse */
  qsort(window_memorymap->tab_handle,window_memorymap->nb_handle,sizeof(struct memorymap_handle *),compare_handle);

  /** Cration ComboBox Handle **/
  extra_length = (current_s65816->current_mmu->rom_version == ROM_APPLEIIGS_01) ? 0 : 74;
  window_memorymap->hwnd_combo_handle = CreateWindowEx(0,"COMBOBOX", 0, CBS_DROPDOWNLIST | WS_CHILD | WS_VISIBLE | WS_VSCROLL,
                                                       652,6,215+extra_length,300,window_memorymap->hwnd_memorymap,(HMENU)IDC_COMBOBOX_HANDLE,param->hInstance,0);
  SendMessage(window_memorymap->hwnd_combo_handle,CB_ADDSTRING,0,(LPARAM)"- Nothing -");
  for(i=0; i<window_memorymap->nb_handle; i++)
    {
      current_memorymap_handle = window_memorymap->tab_handle[i];
      sprintf(buffer,"%02X/%04X : %s%s%s",(current_memorymap_handle->address>>16) & 0xFF,current_memorymap_handle->address & 0xFFFF,
                     current_memorymap_handle->type,strlen(current_memorymap_handle->information)==0?"":" / ",current_memorymap_handle->information);
      SendMessage(window_memorymap->hwnd_combo_handle,CB_ADDSTRING,0,(LPARAM)buffer);
    }
  SendMessage(window_memorymap->hwnd_combo_handle,WM_SETFONT,(WPARAM)window_memorymap->hFont,TRUE);

  /* Slectionne la 1re entre */
  SendMessage(window_memorymap->hwnd_combo_handle,CB_SETCURSEL,0,0);

  /* Cration d'une Notification */
  current_notification = (struct notification *) calloc(1,sizeof(struct notification));
  if(current_notification != NULL)
    {
      current_notification->hwnd_window = window_memorymap->hwnd_memorymap;
      current_notification->message = HM_MEMORYMAP_REFRESHWINDOW;
      current_notification->type = NOTIFICATION_TYPE_VIEW;
      current_notification->sub_type = NOTIFICATION_SUBTYPE_MEMORYMAP;

      /* Ajout de la notification */
      window_memorymap->current_notification = current_notification;
      my_Notification(NOTIFICATION_ADD,current_notification,NULL);
    }
  
  /** Modle mmoire **/
  /* Fast RAM */
  window_memorymap->nb_fastram_bank = current_s65816->current_mmu->nb_extended_bank + 2;

  /* ROM */
  if(current_s65816->current_mmu->rom_version == ROM_APPLEIIGS_01)
    window_memorymap->nb_rom_bank = 2;
  else
    window_memorymap->nb_rom_bank = 4;
  
  /* Nombre de Bank Fast Ram affich */
  window_memorymap->nb_fastram_bank_display = (window_memorymap->nb_fastram_bank>16) ? 16 : window_memorymap->nb_fastram_bank;

  /* Premier Bank Fast Ram */
  window_memorymap->current_fastram_bank = 0;

  /* Rend inactif les boutons pour le 8 bit ou les espaces mmoire limits */
  if(window_memorymap->nb_rom_bank == 0 || window_memorymap->nb_fastram_bank <= 16)
    {
      EnableWindow(GetDlgItem(window_memorymap->hwnd_memorymap,IDC_BUTTON_LEFT),FALSE);
      EnableWindow(GetDlgItem(window_memorymap->hwnd_memorymap,IDC_BUTTON_RIGHT),FALSE);
      EnableWindow(GetDlgItem(window_memorymap->hwnd_memorymap,IDC_BUTTON_LEFT_FAST),FALSE);
      EnableWindow(GetDlgItem(window_memorymap->hwnd_memorymap,IDC_BUTTON_RIGHT_FAST),FALSE);
    }

  /** Bitmap affichant la mmoire **/
  /* Fast Ram */
  nb_display_bank = (window_memorymap->nb_fastram_bank>16) ? 16 : window_memorymap->nb_fastram_bank;
  for(j=0; j<window_memorymap->nb_fastram_bank; j++)
    {
      /* On va crer les Bitmap mme si on n'en affiche que 16 */
      memset(&bitmap_info,0,sizeof(BITMAPINFO));
      bitmap_info.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
      bitmap_info.bmiHeader.biWidth = ((window_memorymap->nb_rom_bank == 0) ? MEMORYMAP_BITMAP_WIDTH_8BIT : MEMORYMAP_BITMAP_WIDTH);
      bitmap_info.bmiHeader.biHeight = (-1)*MEMORYMAP_BITMAP_HEIGHT;
      bitmap_info.bmiHeader.biPlanes = 1;
      bitmap_info.bmiHeader.biBitCount = 32;
      bitmap_info.bmiHeader.biCompression = BI_RGB;
      window_memorymap->hBitmap_fastram[j] = CreateDIBSection(window_memorymap->hDC,&bitmap_info,DIB_RGB_COLORS,(VOID **)&(window_memorymap->bitmap_data_fastram[j]),NULL,0);
      for(i=0; i<(bitmap_info.bmiHeader.biWidth*MEMORYMAP_BITMAP_HEIGHT); i++)
        window_memorymap->bitmap_data_fastram[j][i] = MEMORYMAP_EMPTY_COLOR;
    }
  /* Slow Ram : Apple IIgs only */
  if(window_memorymap->nb_rom_bank > 0)
    for(j=0; j<2; j++)
      {
        memset(&bitmap_info,0,sizeof(BITMAPINFO));
        bitmap_info.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
        bitmap_info.bmiHeader.biWidth = MEMORYMAP_BITMAP_WIDTH;
        bitmap_info.bmiHeader.biHeight = (-1)*MEMORYMAP_BITMAP_HEIGHT;
        bitmap_info.bmiHeader.biPlanes = 1;
        bitmap_info.bmiHeader.biBitCount = 32;
        bitmap_info.bmiHeader.biCompression = BI_RGB;
        window_memorymap->hBitmap_slowram[j] = CreateDIBSection(window_memorymap->hDC,&bitmap_info,DIB_RGB_COLORS,(VOID **)&(window_memorymap->bitmap_data_slowram[j]),NULL,0);
        for(i=0; i<MEMORYMAP_BITMAP_WIDTH*MEMORYMAP_BITMAP_HEIGHT; i++)
          window_memorymap->bitmap_data_slowram[j][i] = MEMORYMAP_EMPTY_COLOR;
      }
  /* Rom */
  for(j=0; j<4; j++)
    {
      memset(&bitmap_info,0,sizeof(BITMAPINFO));
      bitmap_info.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
      bitmap_info.bmiHeader.biWidth = MEMORYMAP_BITMAP_WIDTH;
      bitmap_info.bmiHeader.biHeight = (-1)*MEMORYMAP_BITMAP_HEIGHT;
      bitmap_info.bmiHeader.biPlanes = 1;
      bitmap_info.bmiHeader.biBitCount = 32;
      bitmap_info.bmiHeader.biCompression = BI_RGB;
      window_memorymap->hBitmap_rom[j] = CreateDIBSection(window_memorymap->hDC,&bitmap_info,DIB_RGB_COLORS,(VOID **)&(window_memorymap->bitmap_data_rom[j]),NULL,0);
      for(i=0; i<MEMORYMAP_BITMAP_WIDTH*MEMORYMAP_BITMAP_HEIGHT; i++)
        window_memorymap->bitmap_data_rom[j][i] = MEMORYMAP_EMPTY_COLOR;
    }
  /* DOC Ram : Apple IIgs only */
  if(window_memorymap->nb_rom_bank > 0)
    {
      memset(&bitmap_info,0,sizeof(BITMAPINFO));
      bitmap_info.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
      bitmap_info.bmiHeader.biWidth = MEMORYMAP_BITMAP_WIDTH;
      bitmap_info.bmiHeader.biHeight = (-1)*MEMORYMAP_BITMAP_HEIGHT;
      bitmap_info.bmiHeader.biPlanes = 1;
      bitmap_info.bmiHeader.biBitCount = 32;
      bitmap_info.bmiHeader.biCompression = BI_RGB;
      window_memorymap->hBitmap_docram = CreateDIBSection(window_memorymap->hDC,&bitmap_info,DIB_RGB_COLORS,(VOID **)&(window_memorymap->bitmap_data_docram),NULL,0);
      for(i=0; i<MEMORYMAP_BITMAP_WIDTH*MEMORYMAP_BITMAP_HEIGHT; i++)
        window_memorymap->bitmap_data_docram[i] = MEMORYMAP_EMPTY_COLOR;
    }
  /* Color */
  for(j=0; j<7; j++)
    {
      memset(&bitmap_info,0,sizeof(BITMAPINFO));
      bitmap_info.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
      bitmap_info.bmiHeader.biWidth = MEMORYMAP_BITMAP_COLOR_SIZE;
      bitmap_info.bmiHeader.biHeight = (-1)*MEMORYMAP_BITMAP_COLOR_SIZE;
      bitmap_info.bmiHeader.biPlanes = 1;
      bitmap_info.bmiHeader.biBitCount = 32;
      bitmap_info.bmiHeader.biCompression = BI_RGB;
      window_memorymap->hBitmap_color[j] = CreateDIBSection(window_memorymap->hDC,&bitmap_info,DIB_RGB_COLORS,(VOID **)&(window_memorymap->bitmap_data_color[j]),NULL,0);
      for(i=0; i<MEMORYMAP_BITMAP_COLOR_SIZE*MEMORYMAP_BITMAP_COLOR_SIZE; i++)
        window_memorymap->bitmap_data_color[j][i] = tab_rgb_color[j];
    }
  
  /* Charge l'image des 3 couleurs */
  window_memorymap->color_circle_picture = GetPictureFromResource(IDB_BITMAP_COLOR);
  if(window_memorymap->color_circle_picture != NULL)
    {
      window_memorymap->bitmap_color_circle_width = window_memorymap->color_circle_picture->width;
      window_memorymap->bitmap_color_circle_height = window_memorymap->color_circle_picture->height;
    }

  /** Bitmap affichant les 3 couleurs en cercle **/
  memset(&bitmap_info,0,sizeof(BITMAPINFO));
  bitmap_info.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
  bitmap_info.bmiHeader.biWidth = window_memorymap->bitmap_color_circle_width;
  bitmap_info.bmiHeader.biHeight = (-1)*window_memorymap->bitmap_color_circle_height;
  bitmap_info.bmiHeader.biPlanes = 1;
  bitmap_info.bmiHeader.biBitCount = 32;
  bitmap_info.bmiHeader.biCompression = BI_RGB;
  window_memorymap->hBitmap_color_circle = CreateDIBSection(window_memorymap->hDC,&bitmap_info,DIB_RGB_COLORS,(VOID **)&(window_memorymap->bitmap_data_color_circle),NULL,0);
  if(window_memorymap->color_circle_picture != NULL)
    memcpy(window_memorymap->bitmap_data_color_circle,window_memorymap->color_circle_picture->data_dwd,window_memorymap->bitmap_color_circle_width*window_memorymap->bitmap_color_circle_height*sizeof(DWORD));
  else
    {
      /* Couleur Unie */
      for(i=0; i<window_memorymap->bitmap_color_circle_width*window_memorymap->bitmap_color_circle_height; i++)
        window_memorymap->bitmap_data_color_circle[i] = BITMAP_BACKGROUND_COLOR;
    }

  /*** On va ajuster la fentre par rapport au Zone mmoire affiche ***/
  /* Position actuelle de la fentre dans l'cran */
  GetWindowRect(window_memorymap->hwnd_memorymap,&window_rect);
  window_x = window_rect.left;
  window_y = window_rect.top;
    
  /* Hauteur Titre */
  GetClientRect(window_memorymap->hwnd_memorymap,&client_rect);
  window_memorymap->title_height = (window_rect.bottom - window_rect.top) - client_rect.bottom;

  /** Position de la GroupBox FastRam **/
  hwnd_group = GetDlgItem(window_memorymap->hwnd_memorymap,IDC_GROUP_FASTRAM);     /* Rcupre le Handle du Control GroupBox */
  GetWindowRect(hwnd_group,&group_rect);
  window_memorymap->group_fastram_x = (group_rect.left - window_rect.left) - 3;
  window_memorymap->group_fastram_y = ((group_rect.top - window_rect.top) - window_memorymap->title_height) + 3;

  /** Position des Zones BitMap de la FastRam dans la fentre **/
  for(i=0; i<window_memorymap->nb_fastram_bank_display; i++)
    {
      /* Bitmap */
      window_memorymap->bitmap_fastram_x[i] = (window_memorymap->group_fastram_x + WINDOW_GROUP_BORDER_LEFT_SIZE + MEMORYMAP_GROUP_MARGIN) + i*(((window_memorymap->nb_rom_bank == 0) ? MEMORYMAP_BITMAP_WIDTH_8BIT : MEMORYMAP_BITMAP_WIDTH) + WINDOW_GROUP_BORDER_LEFT_SIZE);
      window_memorymap->bitmap_fastram_y[i] = window_memorymap->group_fastram_y + WINDOW_GROUP_BORDER_TOP_SIZE;

      /* Libell */
      sprintf(buffer,"%02X",i);
      label_x = window_memorymap->bitmap_fastram_x[i] + ((window_memorymap->nb_rom_bank == 0) ? 122 : 10);
      label_y = window_memorymap->bitmap_fastram_y[i] + 2 + MEMORYMAP_BITMAP_HEIGHT;
      window_memorymap->hwnd_fastram_label[i] = CreateWindowEx(0,"STATIC",buffer,WS_CHILD | WS_VISIBLE,
		                                                       label_x,label_y,20,15,window_memorymap->hwnd_memorymap,NULL,param->hInstance,NULL);
      SendMessage(window_memorymap->hwnd_fastram_label[i],WM_SETFONT,(WPARAM)window_memorymap->hFont,TRUE);
    }

  /* Dimension en Pixel du Control Group Box FASTRAM + Resize */  
  window_memorymap->group_fastram_width = WINDOW_GROUP_BORDER_LEFT_SIZE + MEMORYMAP_GROUP_MARGIN + window_memorymap->nb_fastram_bank_display*(((window_memorymap->nb_rom_bank == 0) ? MEMORYMAP_BITMAP_WIDTH_8BIT : MEMORYMAP_BITMAP_WIDTH) + WINDOW_GROUP_BORDER_LEFT_SIZE) + MEMORYMAP_GROUP_MARGIN; 
  window_memorymap->group_fastram_height = WINDOW_GROUP_BORDER_TOP_SIZE + MEMORYMAP_BITMAP_HEIGHT + MEMORYMAP_TEXT_BORDER_HEIGHT + MEMORYMAP_TEXT_HEIGHT + MEMORYMAP_TEXT_BORDER_HEIGHT + WINDOW_GROUP_BORDER_BOTTOM_SIZE;
  MoveWindow(hwnd_group,window_memorymap->group_fastram_x,window_memorymap->group_fastram_y,window_memorymap->group_fastram_width,window_memorymap->group_fastram_height,TRUE);

  /*** Slow Ram + ROM + DOC Ram : Apple IIgs ***/
  if(window_memorymap->nb_rom_bank > 0)
    {
      /*** Position de la GroupBox SlowRam ***/
      hwnd_group = GetDlgItem(window_memorymap->hwnd_memorymap,IDC_GROUP_SLOWRAM);     /* Rcupre le Handle du Control GroupBox */
      GetWindowRect(hwnd_group,&group_rect);
      window_memorymap->group_slowram_x = window_memorymap->group_fastram_x +  window_memorymap->group_fastram_width + MEMORYMAP_GROUP_BORDER;
      window_memorymap->group_slowram_y = window_memorymap->group_fastram_y;

      /* Position des Zones BitMap de la SlowRam dans la fentre */
      for(i=0; i<2; i++)
        {
          /* Bitmap */
          window_memorymap->bitmap_slowram_x[i] = (window_memorymap->group_slowram_x + WINDOW_GROUP_BORDER_LEFT_SIZE + MEMORYMAP_GROUP_MARGIN) + i*(MEMORYMAP_BITMAP_WIDTH + WINDOW_GROUP_BORDER_LEFT_SIZE);
          window_memorymap->bitmap_slowram_y[i] = window_memorymap->group_slowram_y + WINDOW_GROUP_BORDER_TOP_SIZE;

          /* Libell */
          sprintf(buffer,"%02X",0xE0 + i);
          label_x = window_memorymap->bitmap_slowram_x[i] + 10;
          label_y = window_memorymap->bitmap_slowram_y[i] + 2 + MEMORYMAP_BITMAP_HEIGHT;
          window_memorymap->hwnd_slowram_label[i] = CreateWindowEx(0,"STATIC",buffer,WS_CHILD | WS_VISIBLE,
                                                                   label_x,label_y,20,15,window_memorymap->hwnd_memorymap,NULL,param->hInstance,NULL);
          SendMessage(window_memorymap->hwnd_slowram_label[i],WM_SETFONT,(WPARAM)window_memorymap->hFont,TRUE);
        }      

      /* Dimension en Pixel du Control Group Box SLOWRAM + Resize */  
      window_memorymap->group_slowram_width = WINDOW_GROUP_BORDER_LEFT_SIZE + MEMORYMAP_GROUP_MARGIN + 2*(MEMORYMAP_BITMAP_WIDTH + WINDOW_GROUP_BORDER_LEFT_SIZE) + MEMORYMAP_GROUP_MARGIN;
      window_memorymap->group_slowram_height = window_memorymap->group_fastram_height;
      MoveWindow(hwnd_group,window_memorymap->group_slowram_x,window_memorymap->group_slowram_y,window_memorymap->group_slowram_width,window_memorymap->group_slowram_height,TRUE);
      
      /*** Position de la GroupBox Rom ***/
      hwnd_group = GetDlgItem(window_memorymap->hwnd_memorymap,IDC_GROUP_ROM);     /* Rcupre le Handle du Control GroupBox */
      window_memorymap->group_rom_x = window_memorymap->group_slowram_x + window_memorymap->group_slowram_width + MEMORYMAP_GROUP_BORDER;
      window_memorymap->group_rom_y = window_memorymap->group_fastram_y;

      /* Position des Zones BitMap de la Rom dans la fentre */
      offset_rom = (window_memorymap->nb_rom_bank == 2) ? 2 : 0;
      for(i=0; i<window_memorymap->nb_rom_bank; i++)
        {
          /* Bitmap */
          window_memorymap->bitmap_rom_x[offset_rom+i] = (window_memorymap->group_rom_x + WINDOW_GROUP_BORDER_LEFT_SIZE + MEMORYMAP_GROUP_MARGIN) + i*(MEMORYMAP_BITMAP_WIDTH + WINDOW_GROUP_BORDER_LEFT_SIZE);
          window_memorymap->bitmap_rom_y[offset_rom+i] = window_memorymap->group_rom_y + WINDOW_GROUP_BORDER_TOP_SIZE;

          /* Libell */
          sprintf(buffer,"%02X",0xFC + offset_rom + i);
          label_x = window_memorymap->bitmap_rom_x[offset_rom+i] + 10;
          label_y = window_memorymap->bitmap_rom_y[offset_rom+i] + 2 + MEMORYMAP_BITMAP_HEIGHT;
          window_memorymap->hwnd_rom_label[offset_rom+i] = CreateWindowEx(0,"STATIC",buffer,WS_CHILD | WS_VISIBLE,
	                                                                      label_x,label_y,20,15,window_memorymap->hwnd_memorymap,NULL,param->hInstance,NULL);
          SendMessage(window_memorymap->hwnd_rom_label[offset_rom+i],WM_SETFONT,(WPARAM)window_memorymap->hFont,TRUE);
        }      

      /* Dimension en Pixel du Control Group Box ROM + Resize */
      window_memorymap->group_rom_width = WINDOW_GROUP_BORDER_LEFT_SIZE + MEMORYMAP_GROUP_MARGIN + window_memorymap->nb_rom_bank*(MEMORYMAP_BITMAP_WIDTH + WINDOW_GROUP_BORDER_LEFT_SIZE) + MEMORYMAP_GROUP_MARGIN;
      window_memorymap->group_rom_height = window_memorymap->group_fastram_height;
      MoveWindow(hwnd_group,window_memorymap->group_rom_x,window_memorymap->group_rom_y,window_memorymap->group_rom_width,window_memorymap->group_rom_height,TRUE);

      /*** Position de la GroupBox DOC Ram ***/
      hwnd_group = GetDlgItem(window_memorymap->hwnd_memorymap,IDC_GROUP_DOCRAM);     /* Rcupre le Handle du Control GroupBox */
      window_memorymap->group_docram_x = window_memorymap->group_rom_x + window_memorymap->group_rom_width + MEMORYMAP_GROUP_BORDER;
      window_memorymap->group_docram_y = window_memorymap->group_fastram_y;

      /* Position de la Zone BitMap de la DOC Ram dans la fentre */
      window_memorymap->bitmap_docram_x = window_memorymap->group_docram_x + WINDOW_GROUP_BORDER_LEFT_SIZE + MEMORYMAP_GROUP_MARGIN;
      window_memorymap->bitmap_docram_y = window_memorymap->group_docram_y + WINDOW_GROUP_BORDER_TOP_SIZE;

      /* Libell */
      label_x = window_memorymap->bitmap_docram_x + 10;
      label_y = window_memorymap->bitmap_docram_y + 2 + MEMORYMAP_BITMAP_HEIGHT;
      window_memorymap->hwnd_docram_label = CreateWindowEx(0,"STATIC","00",WS_CHILD | WS_VISIBLE,
                                                           label_x,label_y,20,15,window_memorymap->hwnd_memorymap,NULL,param->hInstance,NULL);
      SendMessage(window_memorymap->hwnd_docram_label,WM_SETFONT,(WPARAM)window_memorymap->hFont,TRUE);

      /* Dimension en Pixel du Control Group Box DOC RAM + Resize */
      window_memorymap->group_docram_width = WINDOW_GROUP_BORDER_LEFT_SIZE + MEMORYMAP_GROUP_MARGIN + MEMORYMAP_BITMAP_WIDTH + WINDOW_GROUP_BORDER_LEFT_SIZE + MEMORYMAP_GROUP_MARGIN;
      window_memorymap->group_docram_height = window_memorymap->group_fastram_height;
      MoveWindow(hwnd_group,window_memorymap->group_docram_x,window_memorymap->group_docram_y,window_memorymap->group_docram_width,window_memorymap->group_docram_height,TRUE);
    }

  /* Largeur de toutes les Group Box */
  if(window_memorymap->nb_rom_bank > 0)
    all_group_box_width = window_memorymap->group_fastram_width + MEMORYMAP_GROUP_BORDER + window_memorymap->group_slowram_width + MEMORYMAP_GROUP_BORDER + window_memorymap->group_rom_width + MEMORYMAP_GROUP_BORDER + window_memorymap->group_docram_width;
  else
    all_group_box_width = window_memorymap->group_fastram_width;
  
  /*** Position des Libells / Couleur ***/
  for(i=0; i<7; i++)
    {
      /* Bitpmap Couleur */
      window_memorymap->bitmap_color_x[i] = window_memorymap->group_fastram_x + all_group_box_width + MEMORYMAP_GROUP_BORDER;
      window_memorymap->bitmap_color_y[i] = window_memorymap->bitmap_fastram_y[0] + window_memorymap->bitmap_color_circle_height - 2 + i*(MEMORYMAP_BITMAP_COLOR_SIZE + MEMORYMAP_COLOR_BORDER_SIZE);
      
      /* Libell */
      hwnd_label = GetDlgItem(window_memorymap->hwnd_memorymap,tab_label_id[i]);     /* Rcupre le Handle du Control Label */
      label_x = window_memorymap->bitmap_color_x[i] + MEMORYMAP_BITMAP_COLOR_SIZE + MEMORYMAP_BITMAP_BORDER + 3;
      label_y = window_memorymap->bitmap_color_y[i] + 1;
      MoveWindow(hwnd_label,label_x,label_y,MEMORYMAP_LABEL_WIDTH,MEMORYMAP_TEXT_HEIGHT+5,TRUE);
    }
    
  /** Position de l'image des 3 couleurs **/
  window_memorymap->bitmap_color_circle_x = window_memorymap->group_fastram_x + all_group_box_width + MEMORYMAP_GROUP_BORDER;
  window_memorymap->bitmap_color_circle_y = window_memorymap->bitmap_fastram_y[0] - 10;

  /** Position des boutons Reset / Refresh / Save **/
  /* Reset */
  hwnd_button = GetDlgItem(window_memorymap->hwnd_memorymap,IDC_BUTTON_RESET);     /* Rcupre le Handle du Bouton */
  GetWindowRect(hwnd_button,&button_rect);
  button_x = window_memorymap->bitmap_color_circle_x - 24;                         /* Position X,Y relative du bouton dans la fentre Memory Map */
  button_y = (button_rect.top - window_rect.top) - window_memorymap->title_height + 1;
  MoveWindow(hwnd_button,button_x,button_y,button_rect.right-button_rect.left,button_rect.bottom-button_rect.top,TRUE);
  /* Refresh */
  hwnd_button = GetDlgItem(window_memorymap->hwnd_memorymap,IDC_BUTTON_REFRESH);   /* Rcupre le Handle du Bouton */
  GetWindowRect(hwnd_button,&button_rect);
  button_x = window_memorymap->bitmap_color_circle_x + 33;                         /* Position X,Y relative du bouton dans la fentre Memory Map */
  MoveWindow(hwnd_button,button_x,button_y,button_rect.right-button_rect.left,button_rect.bottom-button_rect.top,TRUE);
  /* Save */
  hwnd_button = GetDlgItem(window_memorymap->hwnd_memorymap,IDC_BUTTON_SAVE);     /* Rcupre le Handle du Bouton */
  GetWindowRect(hwnd_button,&button_rect);
  button_x = window_memorymap->bitmap_color_circle_x + 90;                         /* Position X,Y relative du bouton dans la fentre Memory Map */
  MoveWindow(hwnd_button,button_x,button_y,button_rect.right-button_rect.left,button_rect.bottom-button_rect.top,TRUE);

  /** Largeur / Hauteur de la fentre **/
  window_memorymap->window_width = WINDOW_CLIENT_RIGHT_BORDER_SIZE + window_memorymap->group_fastram_x + all_group_box_width + MEMORYMAP_LABEL_WIDTH + WINDOW_BORDER_SIZE;

  /* On doit garantir une largeur minimale de la fentre */
  if(window_memorymap->window_width < MEMORYMAP_DIALOGBOX_WIDTH)
    window_memorymap->window_width = MEMORYMAP_DIALOGBOX_WIDTH;

  /** Hauteur de la fentre **/
  window_memorymap->window_height = window_memorymap->title_height + window_memorymap->group_fastram_y + window_memorymap->group_fastram_height + WINDOW_CLIENT_BOTTOM_BORDER_SIZE;

  /* Resize window */
  MoveWindow(window_memorymap->hwnd_memorymap,window_x,window_y,window_memorymap->window_width,window_memorymap->window_height,TRUE);

  /** Construction des Bitmap RWX **/
  BuildMemoryMapBitmapRWX(window_memorymap);

  /* Titre de la fentre */
  sprintf(buffer_title,"Memory Map : $00/0000 - $%02X/FFFF",window_memorymap->nb_fastram_bank-1);
  SetWindowText(window_memorymap->hwnd_memorymap,buffer_title);

  /* Rafraichit la fentre */
  InvalidateRect(window_memorymap->hwnd_memorymap,NULL,TRUE);
  UpdateWindow(window_memorymap->hwnd_memorymap);
  
  /* OK */
  return(0);
}


/*****************************************************************************************************/
/*  UpdateMemoryMapContextInformation() :  MAJ des information de contexte de la fentre Memory Map. */
/*****************************************************************************************************/
static void UpdateMemoryMapContextInformation(struct window_memorymap *window_memorymap, int mouse_x, int mouse_y)
{
  BYTE map_ram_rwx;
  int i, pixmap_x, pixmap_y, is_doc_ram, offset_rom, index_handle;
  DWORD memorymap_data_address;
  unsigned char one_byte;
  unsigned char memorymap_data[8] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
  char buffer_rwx[256];
  struct mmu *current_mmu;
  
  /* Rien  faire */
  if(window_memorymap == NULL)
    return;

  /* Init */
  pixmap_x = -1;
  pixmap_y = -1;
  is_doc_ram = 0;

  /*** Est t'on dans l'image ? ***/
  /* Fast Ram */
  for(i=0; i<window_memorymap->nb_fastram_bank_display; i++)
    if(mouse_x >= window_memorymap->bitmap_fastram_x[i] && mouse_x < (window_memorymap->bitmap_fastram_x[i] + ((window_memorymap->nb_rom_bank == 0) ? MEMORYMAP_BITMAP_WIDTH_8BIT : MEMORYMAP_BITMAP_WIDTH)) &&
       mouse_y >= window_memorymap->bitmap_fastram_y[i] && mouse_y < (window_memorymap->bitmap_fastram_y[i] + MEMORYMAP_BITMAP_HEIGHT))
      {
        pixmap_x = mouse_x - window_memorymap->bitmap_fastram_x[i];
        pixmap_y = mouse_y - window_memorymap->bitmap_fastram_y[i];
        if(window_memorymap->nb_rom_bank == 0)
          memorymap_data_address = i*64*1024 + pixmap_y*256 + pixmap_x;
        else
          memorymap_data_address = (window_memorymap->current_fastram_bank+i)*64*1024 + pixmap_y*256 + pixmap_x*8;
        break;
      }
  /* Slow Ram */
  if(window_memorymap->nb_rom_bank > 0 && pixmap_x == -1)
    for(i=0; i<2; i++)
      if(mouse_x >= window_memorymap->bitmap_slowram_x[i] && mouse_x < (window_memorymap->bitmap_slowram_x[i] + MEMORYMAP_BITMAP_WIDTH) &&
         mouse_y >= window_memorymap->bitmap_slowram_y[i] && mouse_y < (window_memorymap->bitmap_slowram_y[i] + MEMORYMAP_BITMAP_HEIGHT))
        {
          pixmap_x = mouse_x - window_memorymap->bitmap_slowram_x[i];
          pixmap_y = mouse_y - window_memorymap->bitmap_slowram_y[i];
          memorymap_data_address = (0xE0+i)*64*1024 + pixmap_y*256 + pixmap_x*8;
          break;
        }
  /* Rom */
  if(window_memorymap->nb_rom_bank > 0 && pixmap_x == -1)
    {
      offset_rom = (window_memorymap->nb_rom_bank == 2) ? 2 : 0;
      for(i=0; i<window_memorymap->nb_rom_bank; i++)
        if(mouse_x >= window_memorymap->bitmap_rom_x[offset_rom+i] && mouse_x < (window_memorymap->bitmap_rom_x[offset_rom+i] + MEMORYMAP_BITMAP_WIDTH) &&
           mouse_y >= window_memorymap->bitmap_rom_y[offset_rom+i] && mouse_y < (window_memorymap->bitmap_rom_y[offset_rom+i] + MEMORYMAP_BITMAP_HEIGHT))
          {
            pixmap_x = mouse_x - window_memorymap->bitmap_rom_x[offset_rom+i];
            pixmap_y = mouse_y - window_memorymap->bitmap_rom_y[offset_rom+i];
            memorymap_data_address = (0xFC + offset_rom + i)*64*1024 + pixmap_y*256 + pixmap_x*8;
            break;
          }
    }
  /* Doc Ram */
  if(window_memorymap->nb_rom_bank > 0 && pixmap_x == -1)
    if(mouse_x >= window_memorymap->bitmap_docram_x && mouse_x < (window_memorymap->bitmap_docram_x + MEMORYMAP_BITMAP_WIDTH) &&
       mouse_y >= window_memorymap->bitmap_docram_y && mouse_y < (window_memorymap->bitmap_docram_y + MEMORYMAP_BITMAP_HEIGHT))
      {
        pixmap_x = mouse_x - window_memorymap->bitmap_docram_x;
        pixmap_y = mouse_y - window_memorymap->bitmap_docram_y;
        memorymap_data_address = pixmap_y*256 + pixmap_x*8;
        is_doc_ram = 1;
      }

  /* En dehors des Zones Graphiques */
  if(pixmap_x == -1)
    {
      SetWindowText(GetDlgItem(window_memorymap->hwnd_memorymap,IDC_STATIC_CONTEXT),"");
      return;
    }

  /* Rcupre le MMU */
  my_Memory(MEMORY_GET_S65816_MMU,&current_mmu,NULL);

  /** Message **/
  if(window_memorymap->nb_rom_bank == 0)
    {
      /* Apple II */
      one_byte = ReadMemoryByte(memorymap_data_address,current_mmu);
      sprintf(window_memorymap->buffer_log," %02X/%04X : %02X ",memorymap_data_address>>16,memorymap_data_address&0xFFFF,one_byte);

      /* RWX */
      strcpy(buffer_rwx,"           ");
      for(i=0; i<8; i++)
        {
          map_ram_rwx = GetMapRamRWX(memorymap_data_address+i,0,current_mmu);
          sprintf(&buffer_rwx[strlen(buffer_rwx)],"%s%s%s ",((map_ram_rwx & MEMORYMAP_OPERATION_READ) == MEMORYMAP_OPERATION_READ) ? "R" : " ",
                                                            ((map_ram_rwx & MEMORYMAP_OPERATION_WRITE) == MEMORYMAP_OPERATION_WRITE) ? "W" : " ",
                                                            ((map_ram_rwx & MEMORYMAP_OPERATION_EXECUTE) == MEMORYMAP_OPERATION_EXECUTE) ? "X" : " ");
        }
    }
  else if(is_doc_ram == 0)
    {
      /* Apple IIgs */
      ReadMemoryBlock(memorymap_data_address,8,&memorymap_data[0],current_mmu);
      sprintf(window_memorymap->buffer_log," %02X/%04X : %02X  %02X  %02X  %02X  %02X  %02X  %02X  %02X ",memorymap_data_address>>16,memorymap_data_address&0xFFFF,
                                           memorymap_data[0],memorymap_data[1],memorymap_data[2],memorymap_data[3],
                                           memorymap_data[4],memorymap_data[5],memorymap_data[6],memorymap_data[7]);

      /* RWX */
      strcpy(buffer_rwx,"           ");
      for(i=0; i<8; i++)
        {
          map_ram_rwx = GetMapRamRWX(memorymap_data_address+i,0,current_mmu);
          sprintf(&buffer_rwx[strlen(buffer_rwx)],"%s%s%s ",((map_ram_rwx & MEMORYMAP_OPERATION_READ) == MEMORYMAP_OPERATION_READ) ? "R" : " ",
                                                            ((map_ram_rwx & MEMORYMAP_OPERATION_WRITE) == MEMORYMAP_OPERATION_WRITE) ? "W" : " ",
                                                            ((map_ram_rwx & MEMORYMAP_OPERATION_EXECUTE) == MEMORYMAP_OPERATION_EXECUTE) ? "X" : " ");
        }

      /** Handle Mmoire **/
      index_handle = 0;
      for(i=0; i<window_memorymap->nb_handle; i++)
        {
          if(memorymap_data_address >= window_memorymap->tab_handle[i]->address && memorymap_data_address < (window_memorymap->tab_handle[i]->address + window_memorymap->tab_handle[i]->length))
            {
              index_handle = i+1;
              break;
            }
          else if(memorymap_data_address < (window_memorymap->tab_handle[i]->address + window_memorymap->tab_handle[i]->length))
            break;
        }
      SendMessage(window_memorymap->hwnd_combo_handle,CB_SETCURSEL,index_handle,0);
    }
  else
    {
      /* Apple IIgs Sound Ram */
      ReadDOCRAMBlock(memorymap_data_address,8,&memorymap_data[0],current_mmu);
      sprintf(window_memorymap->buffer_log,"DOC/%04X : %02X  %02X  %02X  %02X  %02X  %02X  %02X  %02X ",memorymap_data_address&0xFFFF,
                                           memorymap_data[0],memorymap_data[1],memorymap_data[2],memorymap_data[3],
                                           memorymap_data[4],memorymap_data[5],memorymap_data[6],memorymap_data[7]);

      /* RWX */
      strcpy(buffer_rwx,"           ");
      for(i=0; i<8; i++)
        {
          map_ram_rwx = GetMapRamRWX(memorymap_data_address+i,1,current_mmu);
          sprintf(&buffer_rwx[strlen(buffer_rwx)],"%s%s%s ",((map_ram_rwx & MEMORYMAP_OPERATION_READ) == MEMORYMAP_OPERATION_READ) ? "R" : " ",
                                                            ((map_ram_rwx & MEMORYMAP_OPERATION_WRITE) == MEMORYMAP_OPERATION_WRITE) ? "W" : " ",
                                                            ((map_ram_rwx & MEMORYMAP_OPERATION_EXECUTE) == MEMORYMAP_OPERATION_EXECUTE) ? "X" : " ");
        }
    }
  SetWindowText(GetDlgItem(window_memorymap->hwnd_memorymap,IDC_STATIC_CONTEXT),window_memorymap->buffer_log);
  SetWindowText(GetDlgItem(window_memorymap->hwnd_memorymap,IDC_STATIC_CONTEXT_RWX),buffer_rwx);
}


/****************************************************************************************/
/*  UpdateMemoryMapFastRamAddress() :  Dplacement de l'adresse Fast Ram de Memory Map. */
/****************************************************************************************/
static void UpdateMemoryMapFastRamAddress(struct window_memorymap *window_memorymap, int delta)
{
  int i;
  char buffer[1024];
  
  /* Rien  dplacer ? */
  if(window_memorymap->nb_fastram_bank <= 16)
    return;
  if(window_memorymap->current_fastram_bank == 0 && delta < 0)
    return;
  if(window_memorymap->current_fastram_bank == (window_memorymap->nb_fastram_bank - 16) && delta > 0)
    return;

  /** Changement de l'adresse Fast Ram **/
  window_memorymap->current_fastram_bank += delta;
  if(window_memorymap->current_fastram_bank < 0)
    window_memorymap->current_fastram_bank = 0;
  if(window_memorymap->current_fastram_bank > (window_memorymap->nb_fastram_bank - 16))
    window_memorymap->current_fastram_bank = window_memorymap->nb_fastram_bank - 16;
  
  /** On indique les numros de banc **/
  for(i=0; i<window_memorymap->nb_fastram_bank_display; i++)
    {
      /* Libell */
      sprintf(buffer,"%02X",window_memorymap->current_fastram_bank+i);
      SetWindowText(window_memorymap->hwnd_fastram_label[i],buffer);
    }  
    
  /* Rafraichit les Bitmap de la fentre */
  InvalidateRect(window_memorymap->hwnd_memorymap,NULL,TRUE);
  UpdateWindow(window_memorymap->hwnd_memorymap);    
}


/************************************************************************************/
/*  UpdateMemoryMapClientAllArea() :  Affichage des Bitmap de la fentre MemoryMap. */
/************************************************************************************/
void UpdateMemoryMapClientAllArea(HDC hDC, struct window_memorymap *window_memorymap)
{
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Bitmap **/
  UpdateMemoryMapClientBitmapArea(hDC,window_memorymap);
}


/***************************************************************************/
/*  UpdateMemoryMapClientBitmapArea() :  Affichage de la Bitmap MemoryMap. */
/***************************************************************************/
static void UpdateMemoryMapClientBitmapArea(HDC hDC, struct window_memorymap *window_memorymap)
{
  int i, offset_rom;
  HGDIOBJ bitmap_old;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Fast Ram **/
  for(i=0; i<window_memorymap->nb_fastram_bank_display; i++)
    if(window_memorymap->bitmap_data_fastram[window_memorymap->current_fastram_bank + i] != NULL)
      {
        bitmap_old = SelectObject(window_memorymap->hCompatibleDC,window_memorymap->hBitmap_fastram[window_memorymap->current_fastram_bank + i]);
        StretchBlt(hDC,window_memorymap->bitmap_fastram_x[i],window_memorymap->bitmap_fastram_y[i],
                   ((window_memorymap->nb_rom_bank == 0) ? MEMORYMAP_BITMAP_WIDTH_8BIT : MEMORYMAP_BITMAP_WIDTH),MEMORYMAP_BITMAP_HEIGHT,
                   window_memorymap->hCompatibleDC,0,0,((window_memorymap->nb_rom_bank == 0) ? MEMORYMAP_BITMAP_WIDTH_8BIT : MEMORYMAP_BITMAP_WIDTH),MEMORYMAP_BITMAP_HEIGHT,SRCCOPY);
        SelectObject(window_memorymap->hCompatibleDC,bitmap_old);
      }
  
  /** Slow Ram **/
  for(i=0; i<2; i++)
    if(window_memorymap->bitmap_data_slowram[i] != NULL)
      {
        bitmap_old = SelectObject(window_memorymap->hCompatibleDC,window_memorymap->hBitmap_slowram[i]);
        StretchBlt(hDC,window_memorymap->bitmap_slowram_x[i],window_memorymap->bitmap_slowram_y[i],
                   MEMORYMAP_BITMAP_WIDTH,MEMORYMAP_BITMAP_HEIGHT,
                   window_memorymap->hCompatibleDC,0,0,MEMORYMAP_BITMAP_WIDTH,MEMORYMAP_BITMAP_HEIGHT,SRCCOPY);
        SelectObject(window_memorymap->hCompatibleDC,bitmap_old);
      }

  /** Rom **/
  offset_rom = (window_memorymap->nb_rom_bank == 2) ? 2 : 0;
  for(i=0; i<window_memorymap->nb_rom_bank; i++)
    if(window_memorymap->bitmap_data_rom[i+offset_rom] != NULL)
      {
        bitmap_old = SelectObject(window_memorymap->hCompatibleDC,window_memorymap->hBitmap_rom[i+offset_rom]);
        StretchBlt(hDC,window_memorymap->bitmap_rom_x[i+offset_rom],window_memorymap->bitmap_rom_y[i+offset_rom],
                   MEMORYMAP_BITMAP_WIDTH,MEMORYMAP_BITMAP_HEIGHT,
                   window_memorymap->hCompatibleDC,0,0,MEMORYMAP_BITMAP_WIDTH,MEMORYMAP_BITMAP_HEIGHT,SRCCOPY);
        SelectObject(window_memorymap->hCompatibleDC,bitmap_old);
      }

  /** DOC Ram **/
  if(window_memorymap->bitmap_data_docram != NULL)
    {
      bitmap_old = SelectObject(window_memorymap->hCompatibleDC,window_memorymap->hBitmap_docram);
      StretchBlt(hDC,window_memorymap->bitmap_docram_x,window_memorymap->bitmap_docram_y,
                 MEMORYMAP_BITMAP_WIDTH,MEMORYMAP_BITMAP_HEIGHT,
                 window_memorymap->hCompatibleDC,0,0,MEMORYMAP_BITMAP_WIDTH,MEMORYMAP_BITMAP_HEIGHT,SRCCOPY);
      SelectObject(window_memorymap->hCompatibleDC,bitmap_old);
    }

  /** 7 Couleurs pour les libells **/
  for(i=0; i<7; i++)
    if(window_memorymap->bitmap_data_color[i] != NULL)
      {
        bitmap_old = SelectObject(window_memorymap->hCompatibleDC,window_memorymap->hBitmap_color[i]);
        StretchBlt(hDC,window_memorymap->bitmap_color_x[i],window_memorymap->bitmap_color_y[i],
                   MEMORYMAP_BITMAP_COLOR_SIZE,MEMORYMAP_BITMAP_COLOR_SIZE,
                   window_memorymap->hCompatibleDC,0,0,MEMORYMAP_BITMAP_COLOR_SIZE,MEMORYMAP_BITMAP_COLOR_SIZE,SRCCOPY);
        SelectObject(window_memorymap->hCompatibleDC,bitmap_old);
      } 

  /** Images des 3 couleurs **/
  bitmap_old = SelectObject(window_memorymap->hCompatibleDC,window_memorymap->hBitmap_color_circle);
  StretchBlt(hDC,window_memorymap->bitmap_color_circle_x,window_memorymap->bitmap_color_circle_y,
              window_memorymap->bitmap_color_circle_width,window_memorymap->bitmap_color_circle_height,
              window_memorymap->hCompatibleDC,0,0,window_memorymap->bitmap_color_circle_width,window_memorymap->bitmap_color_circle_height,SRCCOPY);
  SelectObject(window_memorymap->hCompatibleDC,bitmap_old);
}


/*****************************************************************************************/
/*  ResetMemoryMapBitmap() :  Calcule les zones grises pour l'utilisation de la mmoire. */
/*****************************************************************************************/
static void ResetMemoryMapBitmap(struct window_memorymap *window_memorymap)
{
  struct mmu *current_mmu;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  
  /* Rcupre le MMU */
  my_Memory(MEMORY_GET_S65816_MMU,&current_mmu,NULL);
  
  /* Accs unique  la zone mmoire */
  EnterCriticalSection(&param->cs_mmu_threadhandle);    

  /** Reset MemoryMap RWX **/
  memset(current_mmu->map_ram_00,0x00,RAM_BANK_SIZE);
  memset(current_mmu->map_ram_01,0x00,RAM_BANK_SIZE);
  memset(current_mmu->map_ram_extended,0x00,current_mmu->nb_extended_bank*RAM_BANK_SIZE);
  memset(current_mmu->map_ram_e0,0x00,RAM_BANK_SIZE);
  memset(current_mmu->map_ram_e1,0x00,RAM_BANK_SIZE);
  memset(current_mmu->map_rom_fc,0x00,RAM_BANK_SIZE);
  memset(current_mmu->map_rom_fd,0x00,RAM_BANK_SIZE);
  memset(current_mmu->map_rom_fe,0x00,RAM_BANK_SIZE);
  memset(current_mmu->map_rom_ff,0x00,RAM_BANK_SIZE);
  memset(current_mmu->map_ram_sound,0x00,RAM_BANK_SIZE);

  /** MAJ des couleurs dans les Bitmap mmoires RWX **/
  BuildMemoryMapBitmapRWX(window_memorymap);

  /* Fin de l'accs unique */
  LeaveCriticalSection(&param->cs_mmu_threadhandle);
}


/********************************************************************************/
/*  BuildMemoryMapBitmapRWX() :  MAJ des couleurs dans les Bitmap mmoires RWX. */
/********************************************************************************/
static void BuildMemoryMapBitmapRWX(struct window_memorymap *window_memorymap)
{
  DWORD color, address;
  BYTE map_ram_rwx, ram_value;
  int i, bank, bitmap_x, bitmap_y, bitmap_offset, bank_offset;
  struct mmu *current_mmu;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Rcupre le MMU */
  my_Memory(MEMORY_GET_S65816_MMU,&current_mmu,NULL);

  /* Rien  faire */
  if(window_memorymap->bitmap_data_fastram[0] == NULL)
    return;    

  /*** Cration des Bitmap affichant les RWX  ***/
  /** Fast Ram : 00,01,02 - XX **/
  for(bank=0; bank<window_memorymap->nb_fastram_bank; bank++)
    if(window_memorymap->bitmap_data_fastram[bank] != NULL)
      for(bitmap_y=0; bitmap_y<MEMORYMAP_BITMAP_HEIGHT; bitmap_y++)
        {
          /** En version AppleII, un banc fait 256 pixels de large (1 pixel = 1 byte), sinon 32 pixels (1 pixel = 8 bytes) **/
          bitmap_offset = bitmap_y*((window_memorymap->nb_rom_bank == 0) ? MEMORYMAP_BITMAP_WIDTH_8BIT : MEMORYMAP_BITMAP_WIDTH);
          for(bitmap_x=0; bitmap_x<((window_memorymap->nb_rom_bank == 0) ? MEMORYMAP_BITMAP_WIDTH_8BIT : MEMORYMAP_BITMAP_WIDTH); bitmap_x++)
            {
              /* On cumule les RWX sur les bytes */
              if(window_memorymap->nb_rom_bank == 0)
                {
                  /* 1 pixel = 1 byte */
                  address = (bank << 16) | (bitmap_y << 8) | bitmap_x;
                  ram_value = GetRamValue(address,0,current_mmu);
                  map_ram_rwx = GetMapRamRWX(address,0,current_mmu);
                }
              else
                {
                  /* 1 pixel = 8 bytes */
                  ram_value = 0x00;
                  map_ram_rwx = 0x00;
                  for(i=0; i<8; i++)
                    {
                      address = (bank << 16) | (bitmap_y << 8) | (bitmap_x*8 + i);
                      ram_value |= GetRamValue(address,0,current_mmu);
                      map_ram_rwx |= GetMapRamRWX(address,0,current_mmu);
                    }
                }

              /** Couleur **/
              if(map_ram_rwx == 0x00 && ram_value == 0x00)
                color = MEMORYMAP_EMPTY_COLOR;
              else if(map_ram_rwx == 0x00 && ram_value != 0x00)
                color = MEMORYMAP_DATA_COLOR;
              else if(map_ram_rwx == MEMORYMAP_OPERATION_SELECTED)
                color = MEMORYMAP_SELECTED_COLOR;
              else if(map_ram_rwx == MEMORYMAP_OPERATION_FRIEND)
                color = MEMORYMAP_FRIEND_COLOR;
              else
                {
                  /* RWX */
                  color =  ((map_ram_rwx & MEMORYMAP_OPERATION_READ) == MEMORYMAP_OPERATION_READ)       ? 0x0000FF00 : 0x00000000;
                  color |= ((map_ram_rwx & MEMORYMAP_OPERATION_WRITE) == MEMORYMAP_OPERATION_WRITE)     ? 0x00FF0000 : 0x00000000;
                  color |= ((map_ram_rwx & MEMORYMAP_OPERATION_EXECUTE) == MEMORYMAP_OPERATION_EXECUTE) ? 0x000000FF : 0x00000000;
                }
              window_memorymap->bitmap_data_fastram[bank][bitmap_offset + bitmap_x] = color;
            }
        }

  /** Slow Ram : E0 - E1 **/
  for(bank=0; bank<2; bank++)
    if(window_memorymap->bitmap_data_slowram[bank] != NULL)
      for(bitmap_y=0; bitmap_y<MEMORYMAP_BITMAP_HEIGHT; bitmap_y++)
        {
          bitmap_offset = bitmap_y*MEMORYMAP_BITMAP_WIDTH;
          for(bitmap_x=0; bitmap_x<MEMORYMAP_BITMAP_WIDTH; bitmap_x++)
            {
              /* On cumule les RWX sur les bytes (1 pixel = 8 bytes) */
              ram_value = 0x00;
              map_ram_rwx = 0x00;
              for(i=0; i<8; i++)
                {
                  address = ((0xE0+bank) << 16) | (bitmap_y << 8) | (bitmap_x*8 + i);
                  ram_value |= GetRamValue(address,0,current_mmu);
                  map_ram_rwx |= GetMapRamRWX(address,0,current_mmu);
                }

              /** Couleur **/
              if(map_ram_rwx == 0x00 && ram_value == 0x00)
                color = MEMORYMAP_EMPTY_COLOR;
              else if(map_ram_rwx == 0x00 && ram_value != 0x00)
                color = MEMORYMAP_DATA_COLOR;
              else if(map_ram_rwx == MEMORYMAP_OPERATION_SELECTED)
                color = MEMORYMAP_SELECTED_COLOR;
              else if(map_ram_rwx == MEMORYMAP_OPERATION_FRIEND)
                color = MEMORYMAP_FRIEND_COLOR;
              else
                {
                  /* RWX */
                  color =  ((map_ram_rwx & MEMORYMAP_OPERATION_READ) == MEMORYMAP_OPERATION_READ)       ? 0x0000FF00 : 0x00000000;
                  color |= ((map_ram_rwx & MEMORYMAP_OPERATION_WRITE) == MEMORYMAP_OPERATION_WRITE)     ? 0x00FF0000 : 0x00000000;
                  color |= ((map_ram_rwx & MEMORYMAP_OPERATION_EXECUTE) == MEMORYMAP_OPERATION_EXECUTE) ? 0x000000FF : 0x00000000;
                }
              window_memorymap->bitmap_data_slowram[bank][bitmap_offset + bitmap_x] = color;
            }
        }

  /** Rom : FC - FF **/
  bank_offset = (window_memorymap->nb_rom_bank==2) ? 2 : 0;
  for(bank=0; bank<window_memorymap->nb_rom_bank; bank++)
    if(window_memorymap->bitmap_data_rom[bank+bank_offset] != NULL)
      for(bitmap_y=0; bitmap_y<MEMORYMAP_BITMAP_HEIGHT; bitmap_y++)
        {
          bitmap_offset = bitmap_y*MEMORYMAP_BITMAP_WIDTH;
          for(bitmap_x=0; bitmap_x<MEMORYMAP_BITMAP_WIDTH; bitmap_x++)
            {
              /* On cumule les RWX sur les bytes (1 pixel = 8 bytes) */
              ram_value = 0x00;
              map_ram_rwx = 0x00;
              for(i=0; i<8; i++)
                {
                  address = ((0xFC+bank_offset+bank) << 16) | (bitmap_y << 8) | (bitmap_x*8 + i);
                  ram_value |= GetRamValue(address,0,current_mmu);
                  map_ram_rwx |= GetMapRamRWX(address,0,current_mmu);
                }

              /** Couleur **/
              if(map_ram_rwx == 0x00 && ram_value == 0x00)
                color = MEMORYMAP_EMPTY_COLOR;
              else if(map_ram_rwx == 0x00 && ram_value != 0x00)
                color = MEMORYMAP_DATA_COLOR;
              else if(map_ram_rwx == MEMORYMAP_OPERATION_SELECTED)
                color = MEMORYMAP_SELECTED_COLOR;
              else if(map_ram_rwx == MEMORYMAP_OPERATION_FRIEND)
                color = MEMORYMAP_FRIEND_COLOR;
              else
                {
                  /* RWX */
                  color =  ((map_ram_rwx & MEMORYMAP_OPERATION_READ) == MEMORYMAP_OPERATION_READ)       ? 0x0000FF00 : 0x00000000;
                  color |= ((map_ram_rwx & MEMORYMAP_OPERATION_WRITE) == MEMORYMAP_OPERATION_WRITE)     ? 0x00FF0000 : 0x00000000;
                  color |= ((map_ram_rwx & MEMORYMAP_OPERATION_EXECUTE) == MEMORYMAP_OPERATION_EXECUTE) ? 0x000000FF : 0x00000000;
                }
              window_memorymap->bitmap_data_rom[bank+bank_offset][bitmap_offset + bitmap_x] = color;
            }
        }

  /** DOC Ram **/
  if(window_memorymap->bitmap_data_docram != NULL)
    for(bitmap_y=0; bitmap_y<MEMORYMAP_BITMAP_HEIGHT; bitmap_y++)
      {
        bitmap_offset = bitmap_y*MEMORYMAP_BITMAP_WIDTH;
        for(bitmap_x=0; bitmap_x<MEMORYMAP_BITMAP_WIDTH; bitmap_x++)
          {
            /* On cumule les RWX sur les bytes (1 pixel = 8 bytes) */
            ram_value = 0x00;
            map_ram_rwx = 0x00;
            for(i=0; i<8; i++)
              {
                address = (bitmap_y << 8) | (bitmap_x*8 + i);
                ram_value |= GetRamValue(address,1,current_mmu);
                map_ram_rwx |= GetMapRamRWX(address,1,current_mmu);
              }

            /** Couleur **/
            if(map_ram_rwx == 0x00 && ram_value == 0x00)
              color = MEMORYMAP_EMPTY_COLOR;
            else if(map_ram_rwx == 0x00 && ram_value != 0x00)
              color = MEMORYMAP_DATA_COLOR;
            else if(map_ram_rwx == MEMORYMAP_OPERATION_SELECTED)
              color = MEMORYMAP_SELECTED_COLOR;
            else if(map_ram_rwx == MEMORYMAP_OPERATION_FRIEND)
              color = MEMORYMAP_FRIEND_COLOR;
            else
              {
                /* RWX */
                color = 0x00000000;
                color |= ((map_ram_rwx & MEMORYMAP_OPERATION_READ) == MEMORYMAP_OPERATION_READ)       ? 0x0000FF00 : 0x00000000;
                color |= ((map_ram_rwx & MEMORYMAP_OPERATION_WRITE) == MEMORYMAP_OPERATION_WRITE)     ? 0x00FF0000 : 0x00000000;
                color |= ((map_ram_rwx & MEMORYMAP_OPERATION_EXECUTE) == MEMORYMAP_OPERATION_EXECUTE) ? 0x000000FF : 0x00000000;
              }
            window_memorymap->bitmap_data_docram[bitmap_offset + bitmap_x] = color;
          }
      }
}


/************************************************************************************************/
/*  MarkMemoryBitmapSelectedHandle() : On va souligner les zones mmoire du Handle slectionn. */
/************************************************************************************************/
static void MarkMemoryBitmapSelectedHandle(struct window_memorymap *window_memorymap)
{
  int i;
  WORD id;
  DWORD address, address_begin, address_end;
  BYTE *map_ram_rwx_ptr;
  struct mmu *current_mmu;

  /* Rcupre le MMU */
  my_Memory(MEMORY_GET_S65816_MMU,&current_mmu,NULL);

  /** On marque les Handle 'amis' du Handle slectionn **/
  id = window_memorymap->tab_handle[window_memorymap->selected_handle-1]->id;
  for(i=0; i<window_memorymap->nb_handle; i++)
    if(window_memorymap->tab_handle[i]->id == id && (i != window_memorymap->selected_handle-1))
      {
        address_begin = window_memorymap->tab_handle[i]->address;
        address_end = address_begin + window_memorymap->tab_handle[i]->length;
        for(address = address_begin; address < address_end; address++)
          {
            map_ram_rwx_ptr = GetMapRamRomRWXPointer(address,0,current_mmu);
            if(map_ram_rwx_ptr != NULL)
              *map_ram_rwx_ptr |= MEMORYMAP_OPERATION_FRIEND;
          }
      }

  /** On marque le Handle slectionn **/
  address_begin = window_memorymap->tab_handle[window_memorymap->selected_handle-1]->address;
  address_end = address_begin + window_memorymap->tab_handle[window_memorymap->selected_handle-1]->length;
  for(address = address_begin; address < address_end; address++)
    {
      map_ram_rwx_ptr = GetMapRamRomRWXPointer(address,0,current_mmu);
      if(map_ram_rwx_ptr != NULL)
        *map_ram_rwx_ptr |= MEMORYMAP_OPERATION_SELECTED;
    }
}


/***********************************************************************/
/*  GetMapColorPointer() :  Rcupre un pointeur dans la Bitmap Color. */
/***********************************************************************/
static DWORD *GetMapColorPointer(DWORD bank_address, int is_doc_ram, struct window_memorymap *window_memorymap)
{
  BYTE bank;
  WORD address;
  int bitmap_x, bitmap_y, bank_offset, bitmap_offset;

  /** Rien  faire **/
  if(bank_address == 0xFFFFFFFF)
    return(NULL);

  /* Rom */
  bank_offset = (window_memorymap->nb_rom_bank==2) ? 2 : 0;

  /* Isole l'adresse */
  bank = (bank_address >> 16) & 0xFF;
  address = bank_address & 0x0000FFFF;
  bitmap_x = (address & 0x00FF)/8;
  bitmap_y = (address >> 8);
  bitmap_offset = bitmap_y*MEMORYMAP_BITMAP_WIDTH + bitmap_x;

  /** DOC Ram **/
  if(is_doc_ram)
    return(&window_memorymap->bitmap_data_docram[bitmap_offset]);

  /** Fast RAM **/
  if(bank >= 0x00 && bank < window_memorymap->nb_fastram_bank)
    return(&window_memorymap->bitmap_data_fastram[bank][bitmap_offset]);

  /** Slow RAM **/
  if(bank == 0xE0 || bank == 0xE1)
    return(&window_memorymap->bitmap_data_slowram[(bank-0xE0)][bitmap_offset]);
    
  /** ROM **/
  if(((bank == 0xFC || bank == 0xFD) && window_memorymap->nb_rom_bank == 4) || bank == 0xFE || bank == 0xFF)
    return(&window_memorymap->bitmap_data_rom[(bank-0xFC)+bank_offset][bitmap_offset]);

  /* Pas trouv */
  return(NULL);
}


/******************************************************************/
/*  GetRamValue() :  Rcupre les informations RWX de la mmoire. */
/******************************************************************/
static BYTE GetRamValue(DWORD address, int is_doc_ram, struct mmu *current_mmu)
{
  BYTE ram_value;
  unsigned char *ram_value_ptr;

  /* Adresse en Ram */
  ram_value_ptr = GetRamRomPointer(address,is_doc_ram,current_mmu);
  if(ram_value_ptr == NULL)
    return(0x00);
  ram_value = *ram_value_ptr;

  /* Renvoi la valeur de cette adresse */
  return(ram_value);
}


/*******************************************************************/
/*  GetMapRamRWX() :  Rcupre les informations RWX de la mmoire. */
/*******************************************************************/
static BYTE GetMapRamRWX(DWORD address, int is_doc_ram, struct mmu *current_mmu)
{
  BYTE map_ram_rwx;
  unsigned char *map_ram_rwx_ptr;

  /* Adresse en Ram Map */
  map_ram_rwx_ptr = GetMapRamRomRWXPointer(address,is_doc_ram,current_mmu);
  if(map_ram_rwx_ptr == NULL)
    return(0x00);
  map_ram_rwx = *map_ram_rwx_ptr;

  /* Renvoi les infos RWX de cette adresse */
  return(map_ram_rwx);
}


/*****************************************************************************/
/*  CreateFileMemoryMapRWX() :  Cration du fichier contenant les infos RWX. */
/*****************************************************************************/
static void CreateFileMemoryMapRWX(struct window_memorymap *window_memorymap)
{
  BOOL is_OK;
  DWORD address;
  FILE *fd;
  int bank, i;
  BYTE ram_rom_data[16];
  BYTE rwx_data[16];
  struct mmu *current_mmu;
  char rwx_file_path[1024];
  char buffer_line[1024];

  /* Rcupre le MMU */
  my_Memory(MEMORY_GET_S65816_MMU,&current_mmu,NULL);

  /* Demande le nom de l'image */
  strcpy(rwx_file_path,"MemoryMapRWX.txt");
      
  /** Fentre de slection de chemin de fichier **/
  is_OK = GetSaveFilePath(window_memorymap->hwnd_memorymap,"Select MemoryMap file","All Files (*.*)\0*.*\0\0\0",rwx_file_path);

  /* Si l'utilisateur l'annule */
  if(!is_OK)
    return;

  /* Cration du fichier */
  fd = fopen(rwx_file_path,"w");
  if(fd == NULL)
    {
      MessageBox(NULL,"Impossible to create Memory Map RWX file","Create Memory Map RWX Error",MB_OK|MB_ICONWARNING);    
      return;
    }

  /*** On va dumper la mmoire et les informations RWX ***/
  /* Fast RAM : 00-XX */
  for(bank=0; bank<window_memorymap->nb_fastram_bank; bank++)
    {
      /* Nouveau Banc */
      fprintf(fd,"\n---------- Fast RAM Bank %02X ------------------------------+-0-+-1-+-2-+-3-+-4-+-5-+-6-+-7-+-8-+-9-+-A-+-B-+-C-+-D-+-E-+-F-+\n",bank);

      /** On Dump par tranche de 16 octets **/
      for(address=0; address<0x00010000; address+=16)
        {
          /* Rcupre les donnes RAM ROM + RWX */
          for(i=0; i<16; i++)
            {
              ram_rom_data[i] = GetRamValue((bank<<16)|address+i,0,current_mmu);
              rwx_data[i] = GetMapRamRWX((bank<<16)|address+i,0,current_mmu);
            }

          /* Construit la ligne */
          sprintf(buffer_line,"%02X/%04X : ",bank,address);
          for(i=0; i<16; i++)
            sprintf(&buffer_line[strlen(buffer_line)],"%02X ",ram_rom_data[i]);
          strcat(buffer_line,"|");
          for(i=0; i<16; i++)
            sprintf(&buffer_line[strlen(buffer_line)],"%s%s%s|",
                                                      ((rwx_data[i] & MEMORYMAP_OPERATION_READ) == MEMORYMAP_OPERATION_READ)?"R":" ",
                                                      ((rwx_data[i] & MEMORYMAP_OPERATION_WRITE) == MEMORYMAP_OPERATION_WRITE)?"W":" ",
                                                      ((rwx_data[i] & MEMORYMAP_OPERATION_EXECUTE) == MEMORYMAP_OPERATION_EXECUTE)?"X":" ");
          strcat(buffer_line,"\n");

          /* Ajoute de la ligne dans le fichier */
          fwrite(buffer_line,strlen(buffer_line),1,fd);
        }
    }

  /* Slow RAM : E0-E1 */
  for(bank=0xE0; bank<=0xE1; bank++)
    {
      /* Nouveau Banc */
      fprintf(fd,"\n---------- Slow RAM Bank %02X ------------------------------+-0-+-1-+-2-+-3-+-4-+-5-+-6-+-7-+-8-+-9-+-A-+-B-+-C-+-D-+-E-+-F-+\n",bank);

      /** On Dump par tranche de 16 octets **/
      for(address=0; address<0x00010000; address+=16)
        {
          /* Rcupre les donnes RAM ROM + RWX */
          for(i=0; i<16; i++)
            {
              ram_rom_data[i] = GetRamValue((bank<<16)|address+i,0,current_mmu);
              rwx_data[i] = GetMapRamRWX((bank<<16)|address+i,0,current_mmu);
            }

          /* Construit la ligne */
          sprintf(buffer_line,"%02X/%04X : ",bank,address);
          for(i=0; i<16; i++)
            sprintf(&buffer_line[strlen(buffer_line)],"%02X ",ram_rom_data[i]);
          strcat(buffer_line,"|");
          for(i=0; i<16; i++)
            sprintf(&buffer_line[strlen(buffer_line)],"%s%s%s|",
                                                      ((rwx_data[i] & MEMORYMAP_OPERATION_READ) == MEMORYMAP_OPERATION_READ)?"R":" ",
                                                      ((rwx_data[i] & MEMORYMAP_OPERATION_WRITE) == MEMORYMAP_OPERATION_WRITE)?"W":" ",
                                                      ((rwx_data[i] & MEMORYMAP_OPERATION_EXECUTE) == MEMORYMAP_OPERATION_EXECUTE)?"X":" ");
          strcat(buffer_line,"\n");

          /* Ajoute de la ligne dans le fichier */
          fwrite(buffer_line,strlen(buffer_line),1,fd);
        }
    }

  /* ROM : FC-FF */
  for(bank=(window_memorymap->nb_rom_bank == 4)?0xFC:0xFE; bank<=0xFF; bank++)
    {
      /* Nouveau Banc */
      fprintf(fd,"\n---------- ROM Bank %02X ------------------------------+-0-+-1-+-2-+-3-+-4-+-5-+-6-+-7-+-8-+-9-+-A-+-B-+-C-+-D-+-E-+-F-+\n",bank);

      /** On Dump par tranche de 16 octets **/
      for(address=0; address<0x00010000; address+=16)
        {
          /* Rcupre les donnes RAM ROM + RWX */
          for(i=0; i<16; i++)
            {
              ram_rom_data[i] = GetRamValue((bank<<16)|address+i,0,current_mmu);
              rwx_data[i] = GetMapRamRWX((bank<<16)|address+i,0,current_mmu);
            }

          /* Construit la ligne */
          sprintf(buffer_line,"%02X/%04X : ",bank,address);
          for(i=0; i<16; i++)
            sprintf(&buffer_line[strlen(buffer_line)],"%02X ",ram_rom_data[i]);
          strcat(buffer_line,"|");
          for(i=0; i<16; i++)
            sprintf(&buffer_line[strlen(buffer_line)],"%s%s%s|",
                                                      ((rwx_data[i] & MEMORYMAP_OPERATION_READ) == MEMORYMAP_OPERATION_READ)?"R":" ",
                                                      ((rwx_data[i] & MEMORYMAP_OPERATION_WRITE) == MEMORYMAP_OPERATION_WRITE)?"W":" ",
                                                      ((rwx_data[i] & MEMORYMAP_OPERATION_EXECUTE) == MEMORYMAP_OPERATION_EXECUTE)?"X":" ");
          strcat(buffer_line,"\n");

          /* Ajoute de la ligne dans le fichier */
          fwrite(buffer_line,strlen(buffer_line),1,fd);
        }
    }

  /* Doc RAM */
  bank = 0;
  fprintf(fd,"\n---------- DOC RAM Bank ------------------------------+-0-+-1-+-2-+-3-+-4-+-5-+-6-+-7-+-8-+-9-+-A-+-B-+-C-+-D-+-E-+-F-+\n",bank);
  /** On Dump par tranche de 16 octets **/
  for(address=0; address<0x00010000; address+=16)
    {
      /* Rcupre les donnes RAM ROM + RWX */
      for(i=0; i<16; i++)
        {
          ram_rom_data[i] = GetRamValue((bank<<16)|address+i,1,current_mmu);
          rwx_data[i] = GetMapRamRWX((bank<<16)|address+i,1,current_mmu);
        }

      /* Construit la ligne */
      sprintf(buffer_line,"%02X/%04X : ",bank,address);
      for(i=0; i<16; i++)
        sprintf(&buffer_line[strlen(buffer_line)],"%02X ",ram_rom_data[i]);
      strcat(buffer_line,"|");
      for(i=0; i<16; i++)
        sprintf(&buffer_line[strlen(buffer_line)],"%s%s%s|",
                                                  ((rwx_data[i] & MEMORYMAP_OPERATION_READ) == MEMORYMAP_OPERATION_READ)?"R":" ",
                                                  ((rwx_data[i] & MEMORYMAP_OPERATION_WRITE) == MEMORYMAP_OPERATION_WRITE)?"W":" ",
                                                  ((rwx_data[i] & MEMORYMAP_OPERATION_EXECUTE) == MEMORYMAP_OPERATION_EXECUTE)?"X":" ");
      strcat(buffer_line,"\n");

      /* Ajoute de la ligne dans le fichier */
      fwrite(buffer_line,strlen(buffer_line),1,fd);
    }

  /* Fin du fichier */
  fprintf(fd,"----------------------------------------------------------+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+\n");

  /* Fermeture */
  fclose(fd);
}


/*******************************************************************/
/*  compare_handle() : Fonction de comparaison pour le Quick Sort. */
/*******************************************************************/
static int compare_handle(const void *data_1, const void *data_2)
{
  struct memorymap_handle *handle_1;
  struct memorymap_handle *handle_2;

  /* Rcupration des paramtres */
  handle_1 = *((struct memorymap_handle **) data_1);
  handle_2 = *((struct memorymap_handle **) data_2);

  /* Comparaison des adresses */
  if(handle_1->address < handle_2->address)
    return(-1);
  else if(handle_1->address > handle_2->address)
    return(1);
  else
    return(0);
}


/***********************************************************************/
/*  DeleteMemoryMapChildWindows() :  Destruction des fentres enfants. */
/***********************************************************************/
static void DeleteMemoryMapChildWindows(struct window_memorymap *window_memorymap)
{
  int i;

  /** Libration des ressources **/
  DeleteObject(window_memorymap->hFont);
  DeleteObject(window_memorymap->hFont_courier);

  /* Fast Ram */
  for(i=0; i<64; i++)
    if(window_memorymap->bitmap_data_fastram[i] != NULL)
      DeleteObject(window_memorymap->hBitmap_fastram[i]);

  /* Slow Ram */
  for(i=0; i<2; i++)
    if(window_memorymap->bitmap_data_slowram[i] != NULL)
      DeleteObject(window_memorymap->hBitmap_slowram[i]);  

  /* Rom */
  for(i=0; i<4; i++)
    if(window_memorymap->bitmap_data_rom[i] != NULL)
      DeleteObject(window_memorymap->hBitmap_rom[i]);    

  /* DOC Ram */
  if(window_memorymap->bitmap_data_docram != NULL)
    DeleteObject(window_memorymap->hBitmap_docram); 

  /* Color */
  for(i=0; i<7; i++)
    if(window_memorymap->bitmap_data_color[i] != NULL)
      DeleteObject(window_memorymap->hBitmap_color[i]);

  /* Color Circle */
  DeleteObject(window_memorymap->hBitmap_color_circle);

  ReleaseDC(window_memorymap->hwnd_memorymap,window_memorymap->hDC);
  DeleteDC(window_memorymap->hCompatibleDC);
}


/*************************************************************************/
/*  mem_free_window_memorymap() :  Libration mmoire pour la structure. */
/*************************************************************************/
void mem_free_window_memorymap(struct window_memorymap *window_memorymap)
{
  if(window_memorymap)
    {
      if(window_memorymap->color_circle_picture != NULL)
        mem_free_picture_true(window_memorymap->color_circle_picture);

      /* Supprime la notification */
      my_Notification(NOTIFICATION_REMOVE,window_memorymap->current_notification,NULL);
      window_memorymap->current_notification = NULL;
      
      free(window_memorymap);
    }
}

/**************************************************************************/
