/*********************************************************************/
/*                                                                   */
/*  Window_Move.c :  Module de gestion de la fentre Move.           */
/*                                                                   */
/*********************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe  *  Date : Janv 2019  */
/*********************************************************************/

#include <stdio.h>
#include <time.h>

#include <windows.h>
#include <windowsx.h>
#include <Commctrl.h>
#include <Winuser.h>
#include <Richedit.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_Notification.h"
#include "Dc_Graphic.h"
#include "Dc_CPU.h"
#include "Dc_MMU.h"
#include "Dc_Request.h"
#include "Window_QuickWatch.h"
#include "Window_Main.h"
#include "Window_Main_Menu.h"
#include "Window_Move.h"

static DWORD BuildMoveWindowThread(void *);
LRESULT CALLBACK ProcHandlerMoveWindow(HWND,UINT,WPARAM,LPARAM);
static int CreateMoveChildWindows(HWND,struct window_move *);
LRESULT CALLBACK subEditMoveProcControl(HWND,UINT,WPARAM,LPARAM);
static void UpdateMoveRamType(struct window_move *,int,int);
static void MoveMemoryData(struct window_move *);
static void DeleteMoveChildWindows(struct window_move *);


/**********************************************************************/
/*  BuildMoveWindow() :  Construction / Affichage de la fentre Move. */
/**********************************************************************/
struct window_move *BuildMoveWindow(HINSTANCE hInstance, int menu_id)
{
  struct window_move *window_move;

  /** Allocation de la structure window_move **/
  window_move = (struct window_move *) calloc(1,sizeof(struct window_move));
  if(window_move == NULL)
    return(NULL);
  my_Memory(MEMORY_ADD_WINDOW_MOVE,window_move,NULL);

  /* Lance un thread pour grer la fentre Move */
  CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)BuildMoveWindowThread,window_move,0,NULL);

  /* Renvoi la structure mmoire grant la fentre */
  return(window_move);
}


/********************************************************************************/
/*  BuildMoveWindowThread() :  Cration de la fentre Move via un Thread ddi. */
/********************************************************************************/
static DWORD BuildMoveWindowThread(void *window_move_param)
{
  HWND hwndMove;
  MSG msg;
  BOOL bRet;
  struct window_main *window_main;
  struct window_move *window_move;
  struct parameter *param;
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Init */
  window_move = (struct window_move *) window_move_param;
  window_move->thread_id = GetCurrentThreadId();         /* Le ThreadID sert  diffrencier les diffrentes fentres */

  /** Cration de la fentre Move **/
  hwndMove = CreateDialog(param->hInstance,MAKEINTRESOURCE(IDD_WINDOW_MOVE),window_main->hwnd_main,(DLGPROC)ProcHandlerMoveWindow);
  if(hwndMove == NULL)
    return(1);

  /** Show Window + Paint Content **/
  ShowWindow(hwndMove,SW_SHOWDEFAULT);
  UpdateWindow(hwndMove);

  /** Message Loop pour toutes les fentres de ce Thread (fentre Move...) **/
  while((bRet = GetMessage(&msg,NULL,0,0)) != 0)
    {
      if(bRet == -1)
        break;

      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }

  /* Fin du Thread */
  ExitThread(0);
}


/*********************************************************************/
/*  ProcHandlerMoveWindow() :  Gestion des vnements de la fentre. */
/*********************************************************************/
LRESULT CALLBACK ProcHandlerMoveWindow(HWND hwndMove, UINT umsg, WPARAM wParam, LPARAM lParam)
{
  struct window_move *window_move;
  struct parameter *param;
  
  /* Init */
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  my_Memory(MEMORY_GET_WINDOW_MOVE,&window_move,NULL);

  switch(umsg)
    {
      case WM_INITDIALOG :
        /* Cration du contenu de la DialogBox */
        CreateMoveChildWindows(hwndMove,window_move);
        return(TRUE);
        
      /****************************/
      /* Evnements de la fentre */
      /****************************/
      case WM_COMMAND :
        switch(LOWORD(wParam))
          {
            /** Bouton Radio : On change le type de la Ram Source **/
            case IDC_RADIO_RAM1 :
              if(HIWORD(wParam) == BN_CLICKED && window_move->ram_type1 != MOVE_RAMTYPE_RAM)
                UpdateMoveRamType(window_move,1,MOVE_RAMTYPE_RAM);
              break;
            case IDC_RADIO_DOCRAM1 :
              if(HIWORD(wParam) == BN_CLICKED && window_move->ram_type1 != MOVE_RAMTYPE_DOCRAM)
                UpdateMoveRamType(window_move,1,MOVE_RAMTYPE_DOCRAM);
              break;

            /** Bouton Radio : On change le type de la Ram Destination **/
            case IDC_RADIO_RAM2 :
              if(HIWORD(wParam) == BN_CLICKED && window_move->ram_type2 != MOVE_RAMTYPE_RAM)
                UpdateMoveRamType(window_move,2,MOVE_RAMTYPE_RAM);
              break;
            case IDC_RADIO_DOCRAM2 :
              if(HIWORD(wParam) == BN_CLICKED && window_move->ram_type2 != MOVE_RAMTYPE_DOCRAM)
                UpdateMoveRamType(window_move,2,MOVE_RAMTYPE_DOCRAM);
              break;

            /** Bouton de move de la mmoire **/
            case IDC_BUTTON_MOVE :
              MoveMemoryData(window_move);
              break;
          }
        return(TRUE);

      /********************************************************************/
      /* Fermeture de window (le WM_CLOSE est appel avant le WM_DESTROY) */
      /********************************************************************/
      case WM_CLOSE :
        /* Fentre Move */
        DestroyWindow(hwndMove);
        return(0);

      case WM_DESTROY :
        /* Suppression des Fentres */
        DeleteMoveChildWindows(window_move);

        /* Supprime la structure mmoire de ce thread */
        my_Memory(MEMORY_DELETE_WINDOW_MOVE,window_move,NULL);

        /* Demande la sortie de la boucle des messages */
        PostQuitMessage(0);
        return(0);
    }

  /* Retourner FALSE pour relayer le message non trait au systme */
  return(FALSE);
}


/***********************************************************************************************/
/*  CreateMoveChildWindows() : Crer les fentres enfants et remplir la structure window_move. */
/***********************************************************************************************/
static int CreateMoveChildWindows(HWND hwndMove, struct window_move *window_move)
{
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Window Handle **/
  window_move->hwnd_move = hwndMove;

  /** Surchage des Control Edit pour capter les RETURN lors des saisies **/
  window_move->oldEditProcBank1 = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndMove,IDC_EDIT_RAM_BANK1),GWLP_WNDPROC,(LONG_PTR)subEditMoveProcControl);
  window_move->oldEditProcAddress1 = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndMove,IDC_EDIT_RAM_ADDRESS1),GWLP_WNDPROC,(LONG_PTR)subEditMoveProcControl);
  window_move->oldEditProcBank2 = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndMove,IDC_EDIT_RAM_BANK2),GWLP_WNDPROC,(LONG_PTR)subEditMoveProcControl);
  window_move->oldEditProcAddress2 = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndMove,IDC_EDIT_RAM_ADDRESS2),GWLP_WNDPROC,(LONG_PTR)subEditMoveProcControl);
  window_move->oldEditProcLength = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndMove,IDC_EDIT_LENGTH),GWLP_WNDPROC,(LONG_PTR)subEditMoveProcControl);

  /* Ram Type : RAM */
  UpdateMoveRamType(window_move,1,MOVE_RAMTYPE_RAM);
  UpdateMoveRamType(window_move,2,MOVE_RAMTYPE_RAM);

  /* OK */
  return(0);
}


/*************************************************************************************/
/*  subEditMoveProcControl() :  Rcupre le RETURN pour valider les donnes saisies. */
/*************************************************************************************/
LRESULT CALLBACK subEditMoveProcControl(HWND hwnd_control, UINT msg, WPARAM wParam, LPARAM lParam)
{
  int index;
  struct window_move *window_move;
  my_Memory(MEMORY_GET_WINDOW_MOVE,&window_move,NULL);

  /* Protection */
  if(window_move == NULL)
    return(0);

  /** Identifie le Control **/
  if(GetDlgItem(window_move->hwnd_move,IDC_EDIT_RAM_BANK1) == hwnd_control)
    index = 1;
  else if(GetDlgItem(window_move->hwnd_move,IDC_EDIT_RAM_ADDRESS1) == hwnd_control)
    index = 2;
  else if(GetDlgItem(window_move->hwnd_move,IDC_EDIT_RAM_BANK2) == hwnd_control)
    index = 3;
  else if(GetDlgItem(window_move->hwnd_move,IDC_EDIT_RAM_ADDRESS2) == hwnd_control)
    index = 4;
  else if(GetDlgItem(window_move->hwnd_move,IDC_EDIT_LENGTH) == hwnd_control)
    index = 5;
  else
    return(0);      /* Pas trouv */

  /** Dtecte la touche RETURN pour valider la donne **/
  if(msg == WM_CHAR && wParam == VK_RETURN)
    {
      /* Enlve le focus de l'Edit */
      SetFocus(window_move->hwnd_move);
    }
  else if(index == 1)
    return(CallWindowProc(window_move->oldEditProcBank1,hwnd_control,msg,wParam,lParam));
  else if(index == 2)
    return(CallWindowProc(window_move->oldEditProcAddress1,hwnd_control,msg,wParam,lParam));
  else if(index == 3)
    return(CallWindowProc(window_move->oldEditProcBank2,hwnd_control,msg,wParam,lParam));
  else if(index == 4)
    return(CallWindowProc(window_move->oldEditProcAddress2,hwnd_control,msg,wParam,lParam));
  else if(index == 5)
    return(CallWindowProc(window_move->oldEditProcLength,hwnd_control,msg,wParam,lParam));

  /* OK */
  return(0);
}


/***************************************************************************/
/*  UpdateMoveRamType() :  MAJ du Radio Bouton RamType de la fentre Move. */
/***************************************************************************/
static void UpdateMoveRamType(struct window_move *window_move, int src_dst, int ram_type)
{
  /** Source **/
  if(src_dst == 1)
    {
      window_move->ram_type1 = ram_type;
      if(ram_type == MOVE_RAMTYPE_RAM)
        {
          /* Visuel */
          SendDlgItemMessage(window_move->hwnd_move,IDC_RADIO_RAM1,BM_SETCHECK,1,0);
          SendDlgItemMessage(window_move->hwnd_move,IDC_RADIO_DOCRAM1,BM_SETCHECK,0,0);

          /* Enable BANK */
          EnableWindow(GetDlgItem(window_move->hwnd_move,IDC_EDIT_RAM_BANK1),TRUE);
        }
      else
        {
          /* Visuel */
          SendDlgItemMessage(window_move->hwnd_move,IDC_RADIO_RAM1,BM_SETCHECK,0,0);
          SendDlgItemMessage(window_move->hwnd_move,IDC_RADIO_DOCRAM1,BM_SETCHECK,1,0);

          /* Disable BANK */
          EnableWindow(GetDlgItem(window_move->hwnd_move,IDC_EDIT_RAM_BANK1),FALSE);
        }
    }
  else
    {
      window_move->ram_type2 = ram_type;
      if(ram_type == MOVE_RAMTYPE_RAM)
        {
          /* Visuel */
          SendDlgItemMessage(window_move->hwnd_move,IDC_RADIO_RAM2,BM_SETCHECK,1,0);
          SendDlgItemMessage(window_move->hwnd_move,IDC_RADIO_DOCRAM2,BM_SETCHECK,0,0);

          /* Enable BANK */
          EnableWindow(GetDlgItem(window_move->hwnd_move,IDC_EDIT_RAM_BANK2),TRUE);
        }
      else
        {
          /* Visuel */
          SendDlgItemMessage(window_move->hwnd_move,IDC_RADIO_RAM2,BM_SETCHECK,0,0);
          SendDlgItemMessage(window_move->hwnd_move,IDC_RADIO_DOCRAM2,BM_SETCHECK,1,0);

          /* Disable BANK */
          EnableWindow(GetDlgItem(window_move->hwnd_move,IDC_EDIT_RAM_BANK2),FALSE);
        }
    }
}


/***************************************************************************/
/*  MoveMemoryData() :  Modifie la mmoire avec les donnes de la fentre. */
/***************************************************************************/
static void MoveMemoryData(struct window_move *window_move)
{
  int i, bank, address, bank_address1, bank_address2, data_length, is_error;
  char value_buffer[1024];
  unsigned char *data;
  struct mmu *current_mmu;

  /* Rcupre le MMU */
  my_Memory(MEMORY_GET_S65816_MMU,&current_mmu,NULL);

  /** Rcupre les critres de Move **/
  /** Addresse Mmoire #1 **/
  if(window_move->ram_type1 == MOVE_RAMTYPE_RAM)
    {
      /* Rcupre l'adresse cible */
      GetWindowText(GetDlgItem(window_move->hwnd_move,IDC_EDIT_RAM_BANK1),value_buffer,1024-1);
      if(strlen(value_buffer) == 0)
        {
          MessageBox(window_move->hwnd_move,"Error, The Source Bank is Empty !","Move Data Error",MB_OK|MB_ICONWARNING);
          return;
        }
      bank = EvaluateAsByte(value_buffer,&is_error);
      GetWindowText(GetDlgItem(window_move->hwnd_move,IDC_EDIT_RAM_ADDRESS1),value_buffer,1024-1);
      if(strlen(value_buffer) == 0)
        {
          MessageBox(window_move->hwnd_move,"Error, The Source Address is Empty !","Move Data Error",MB_OK|MB_ICONWARNING);
          return;
        }
      address = EvaluateAsWord(value_buffer,&is_error);
      bank_address1 = (bank << 16) | address;
    }
  else   /* Doc RAM */
    {
      /* Rcupre l'adresse DOC cible */
      GetWindowText(GetDlgItem(window_move->hwnd_move,IDC_EDIT_RAM_ADDRESS1),value_buffer,1024-1);
      if(strlen(value_buffer) == 0)
        {
          MessageBox(window_move->hwnd_move,"Error, The Source Address is Empty !","Move Data Error",MB_OK|MB_ICONWARNING);
          return;
        }
      bank_address1 = EvaluateAsWord(value_buffer,&is_error);
    }

  /** Addresse Mmoire #2 **/
  if(window_move->ram_type2 == MOVE_RAMTYPE_RAM)
    {
      /* Rcupre l'adresse cible */
      GetWindowText(GetDlgItem(window_move->hwnd_move,IDC_EDIT_RAM_BANK2),value_buffer,1024-1);
      if(strlen(value_buffer) == 0)
        {
          MessageBox(window_move->hwnd_move,"Error, The Destination Bank is Empty !","Move Data Error",MB_OK|MB_ICONWARNING);
          return;
        }
      bank = EvaluateAsByte(value_buffer,&is_error);
      GetWindowText(GetDlgItem(window_move->hwnd_move,IDC_EDIT_RAM_ADDRESS2),value_buffer,1024-1);
      if(strlen(value_buffer) == 0)
        {
          MessageBox(window_move->hwnd_move,"Error, The Destination Address is Empty !","Move Data Error",MB_OK|MB_ICONWARNING);
          return;
        }
      address = EvaluateAsWord(value_buffer,&is_error);
      bank_address2 = (bank << 16) | address;
    }
  else   /* Doc RAM */
    {
      /* Rcupre l'adresse DOC cible */
      GetWindowText(GetDlgItem(window_move->hwnd_move,IDC_EDIT_RAM_ADDRESS2),value_buffer,1024-1);
      if(strlen(value_buffer) == 0)
        {
          MessageBox(window_move->hwnd_move,"Error, The Destination Address is Empty !","Move Data Error",MB_OK|MB_ICONWARNING);
          return;
        }
      bank_address2 = EvaluateAsWord(value_buffer,&is_error);
    }

  /** Length des donnes **/
  data_length = EvaluateAsOffsetOrLength(window_move->hwnd_move,IDC_EDIT_LENGTH);
  if(data_length == 0)
    {
      MessageBox(window_move->hwnd_move,"Error, The Data length is Zero !","Move Data Error",MB_OK|MB_ICONWARNING);
      return;
    }

  /* Allocation mmoire */
  data = (unsigned char *) calloc(data_length,sizeof(unsigned char));
  if(data == NULL)
    {
      MessageBox(window_move->hwnd_move,"Error, Can't allocate memory to move the data !","Move Data Error",MB_OK|MB_ICONWARNING);
      return;
    }

  /** Rcupre les octets Source dans la zone mmoire **/
  if(window_move->ram_type1 == MOVE_RAMTYPE_RAM)
    ReadMemoryBlock(bank_address1,data_length,data,current_mmu);
  else
    ReadDOCRAMBlock(bank_address1,data_length,data,current_mmu);

  /** Recopie les octets Destination */
  for(i=0; i<data_length; i++)
    {
      if(window_move->ram_type2 == MOVE_RAMTYPE_RAM)
        WriteMemoryByte(bank_address2+i,data[i],current_mmu);
      else
        WriteDOCRAMByte(bank_address2+i,data[i],current_mmu);
    }

  /** Transfre les donnes dans la mmoire de l'mulateur **/
  if(window_move->ram_type2 == MOVE_RAMTYPE_RAM)
    WriteDataToEmulatorMemory(RAMTYPE_RAM,bank_address2,data_length,data);
  else
    WriteDataToEmulatorMemory(RAMTYPE_DOC,bank_address2,data_length,data);

  /* Libration mmoire */
  free(data);
}


/******************************************************************/
/*  DeleteMoveChildWindows() :  Destruction des fentres enfants. */
/******************************************************************/
static void DeleteMoveChildWindows(struct window_move *window_move)
{
}


/********************************************************************/
/*  mem_free_window_move() :  Libration mmoire pour la structure. */
/********************************************************************/
void mem_free_window_move(struct window_move *window_move)
{
  if(window_move)
    {      
      free(window_move);
    }
}

/**********************************************************************/
