/*************************************************************************/
/*                                                                       */
/*   Window_Preference.c :  Module de gestion de la fentre Preference.  */
/*                                                                       */
/*************************************************************************/
/*  Auteur : Olivier ZARDINI   *   Brutal Deluxe   *   Date : Juin 2024  */
/*************************************************************************/

#include <stdio.h>
#include <time.h>

#include <windows.h>
#include <windowsx.h>
#include <Commctrl.h>
#include <Winuser.h>
#include <Richedit.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_Notification.h"
#include "Dc_Graphic.h"
#include "Dc_CPU.h"
#include "Dc_MMU.h"
#include "Window_Symbol.h"
#include "Window_Breakpoint.h"
#include "Window_QuickWatch.h"
#include "Window_Main.h"
#include "Window_Main_Menu.h"
#include "Window_Preference.h"

#define TAB_CONTROL_WIDTH      592
#define TAB_CONTROL_HEIGHT_1   251
#define TAB_CONTROL_HEIGHT_2   495
#define IDC_TABCONTROL        2600

/* Onglets */
#define TAB_CONFIGURATION_FILES     1
#define TAB_TRACE_SYSTEM_CALLS      2

static DWORD BuildPreferenceWindowThread(void *);
LRESULT CALLBACK ProcHandlerPreferenceWindow(HWND,UINT,WPARAM,LPARAM);
LRESULT CALLBACK subEditPreferenceProcControl(HWND,UINT,WPARAM,LPARAM);
static int CreatePreferenceChildWindows(HWND,struct window_preference *);
static void ShowOnePreferenceTabControl(int,struct window_preference *);
static void LoadPreferenceFile(struct window_preference *);
static void UpdatePreferenceFile(struct window_preference *);
static int DecodeTraceCallList(char *,WORD *);
static void UpdatePreferenceControlStatus(struct window_preference *);
static void DeletePreferenceChildWindows(struct window_preference *);


/**********************************************************************************/
/*  BuildPreferenceWindow() :  Construction / Affichage de la fentre Preference. */
/**********************************************************************************/
struct window_preference *BuildPreferenceWindow(HINSTANCE hInstance, int menu_id)
{
  struct window_preference *window_preference;

  /** Allocation de la structure window_preference **/
  window_preference = (struct window_preference *) calloc(1,sizeof(struct window_preference));
  if(window_preference == NULL)
    return(NULL);
  my_Memory(MEMORY_SET_WINDOW_PREFERENCE,window_preference,NULL);

  /* Lance un thread pour grer la fentre Preference */
  CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)BuildPreferenceWindowThread,window_preference,0,NULL);

  /* Renvoi la structure mmoire grant la fentre */
  return(window_preference);
}


/********************************************************************************************/
/*  BuildPreferenceWindowThread() :  Cration de la fentre Preference via un Thread ddi. */
/********************************************************************************************/
static DWORD BuildPreferenceWindowThread(void *window_preference_param)
{
  HWND hwndPreference;
  MSG msg;
  BOOL bRet;
  struct window_main *window_main;
  struct window_preference *window_preference;
  struct parameter *param;

  /* Init */
  window_preference = (struct window_preference *) window_preference_param;
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Cration de la fentre Preference **/
  hwndPreference = CreateDialog(param->hInstance,MAKEINTRESOURCE(IDD_WINDOW_PREFERENCE),window_main->hwnd_main,(DLGPROC)ProcHandlerPreferenceWindow);
  if(hwndPreference == NULL)
    return(1);

  /** Show Window + Paint Content **/
  ShowWindow(hwndPreference,SW_SHOWDEFAULT);
  UpdateWindow(hwndPreference);

  /** Message Loop pour toutes les fentres de ce Thread (fentre Preference...) **/
  while((bRet = GetMessage(&msg,NULL,0,0)) != 0)
    {
      if(bRet == -1)
        break;

      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }

  /* Fin du Thread */
  ExitThread(0);
}


/***************************************************************************/
/*  ProcHandlerPreferenceWindow() :  Gestion des vnements de la fentre. */
/***************************************************************************/
LRESULT CALLBACK ProcHandlerPreferenceWindow(HWND hwndPreference, UINT umsg, WPARAM wParam, LPARAM lParam)
{
  int tab_index;
  UINT tag_code;
  LPDRAWITEMSTRUCT lpdis;
  TCITEM tie;
  char szTabText[256];
  char file_path[1024];
  struct window_preference *window_preference;
  struct parameter *param;
  
  /* Init */
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  my_Memory(MEMORY_GET_WINDOW_PREFERENCE,&window_preference,NULL);

  switch(umsg)
    {
      case WM_INITDIALOG :
        /* Cration du contenu de la DialogBox */
        CreatePreferenceChildWindows(hwndPreference,window_preference);
        return(TRUE);
        
      /***********************************/
      /* Evnements de changement de Tab */
      /***********************************/
      case WM_NOTIFY :
        tag_code = ((LPNMHDR)lParam)->code;
        if(tag_code == TCN_SELCHANGING)
          return(FALSE);                    /* Return FALSE to allow the selection to change. */
        else if(tag_code == TCN_SELCHANGE)
          {
            /* Rcupre l'index du Tab slectionn */
            tab_index = (int) SendMessageA(window_preference->hwnd_tabcontrol,TCM_GETCURSEL,0,0);
            ShowOnePreferenceTabControl(tab_index+1,window_preference);
          }
        return(TRUE);
        break;

      /**************************/
      /* Couleur du Tab Control */
      /**************************/
      case WM_DRAWITEM :
        lpdis = (LPDRAWITEMSTRUCT) lParam;                           /* item drawing information */
        if(lpdis->hwndItem == window_preference->hwnd_tabcontrol)    /* is this the tab control? */
          {
            /* Init */
            memset(szTabText,'\0',sizeof(szTabText));
            tie.mask = TCIF_TEXT;
            tie.pszText = szTabText;
            tie.cchTextMax = sizeof(szTabText)-1;

            /* Rcupre le Tab Control */
            TabCtrl_GetItem(window_preference->hwnd_tabcontrol,lpdis->itemID,&tie);
            
            /* Dessine le fond du Tab Control */
            FillRect(lpdis->hDC,&lpdis->rcItem,window_preference->hBrush);
            //SetBkColor(lpdis->hDC,MAIN_BACKGROUND_COLOR);
            
            /* Remet le Texte */
            TextOut(lpdis->hDC,lpdis->rcItem.left+4,lpdis->rcItem.top+4,tie.pszText,lstrlen(tie.pszText));          
          }
        break;

      /****************************/
      /* Evnements de la fentre */
      /****************************/
      case WM_COMMAND :
        switch(LOWORD(wParam))
          {
            /* Case  cocher pour le chargement des fichiers */
            case IDC_CHECK_LOAD_FILE_STARTUP :
              if(HIWORD(wParam) == BN_CLICKED)
                {
                  /* Inverse la valeur */
                  if(window_preference->reload_at_startup == 0)
                    window_preference->reload_at_startup = 1;
                  else
                    window_preference->reload_at_startup = 0;
                }
              break;

            /** Rcupre le chemin des fichiers **/
            case IDC_BUTTON_FILE_GET_PATH_BREAKPOINT :
              GetOneFilePath(window_preference->hwnd_preference,"Select Breakpoint file","All Files (*.*)\0*.*\0\0",window_preference->breakpoint_file_path,file_path);
              if(strlen(file_path) > 0)
                {
                  /* Chemin du fichier */
                  strcpy(window_preference->breakpoint_file_path,file_path);
                  SetWindowText(GetDlgItem(window_preference->hwnd_preference,IDC_EDIT_BREAKPOINT_FILE_PATH),window_preference->breakpoint_file_path);
                }
              break;
            case IDC_BUTTON_FILE_GET_PATH_QUICKWATCH :
              GetOneFilePath(window_preference->hwnd_preference,"Select Quickwatch file","All Files (*.*)\0*.*\0\0",window_preference->quickwatch_file_path,file_path);
              if(strlen(file_path) > 0)
                {
                  /* Chemin du fichier */
                  strcpy(window_preference->quickwatch_file_path,file_path);
                  SetWindowText(GetDlgItem(window_preference->hwnd_preference,IDC_EDIT_QUICKWATCH_FILE_PATH),window_preference->quickwatch_file_path);
                }
              break;
            case IDC_BUTTON_FILE_GET_PATH_SYMBOL1 :
              GetOneFilePath(window_preference->hwnd_preference,"Select Symbol file #1","All Files (*.*)\0*.*\0\0",window_preference->symbol1_file_path,file_path);
              if(strlen(file_path) > 0)
                {
                  /* Chemin du fichier */
                  strcpy(window_preference->symbol1_file_path,file_path);
                  SetWindowText(GetDlgItem(window_preference->hwnd_preference,IDC_EDIT_SYMBOL_FILE_PATH1),window_preference->symbol1_file_path);
                }
              break;
            case IDC_BUTTON_FILE_GET_PATH_SYMBOL2 :
              GetOneFilePath(window_preference->hwnd_preference,"Select Symbol file #2","All Files (*.*)\0*.*\0\0",window_preference->symbol2_file_path,file_path);
              if(strlen(file_path) > 0)
                {
                  /* Chemin du fichier */
                  strcpy(window_preference->symbol2_file_path,file_path);
                  SetWindowText(GetDlgItem(window_preference->hwnd_preference,IDC_EDIT_SYMBOL_FILE_PATH2),window_preference->symbol2_file_path);
                }
              break;
            case IDC_BUTTON_FILE_GET_PATH_SYMBOL3 :
              GetOneFilePath(window_preference->hwnd_preference,"Select Symbol file #3","All Files (*.*)\0*.*\0\0",window_preference->symbol3_file_path,file_path);
              if(strlen(file_path) > 0)
                {
                  /* Chemin du fichier */
                  strcpy(window_preference->symbol3_file_path,file_path);
                  SetWindowText(GetDlgItem(window_preference->hwnd_preference,IDC_EDIT_SYMBOL_FILE_PATH3),window_preference->symbol3_file_path);
                }
              break;
            case IDC_BUTTON_FILE_GET_PATH_SYMBOL4 :
              GetOneFilePath(window_preference->hwnd_preference,"Select Symbol file #4","All Files (*.*)\0*.*\0\0",window_preference->symbol4_file_path,file_path);
              if(strlen(file_path) > 0)
                {
                  /* Chemin du fichier */
                  strcpy(window_preference->symbol4_file_path,file_path);
                  SetWindowText(GetDlgItem(window_preference->hwnd_preference,IDC_EDIT_SYMBOL_FILE_PATH4),window_preference->symbol4_file_path);
                }
              break;
            case IDC_BUTTON_FILE_GET_PATH_SYMBOL5 :
              GetOneFilePath(window_preference->hwnd_preference,"Select Symbol file #5","All Files (*.*)\0*.*\0\0",window_preference->symbol5_file_path,file_path);
              if(strlen(file_path) > 0)
                {
                  /* Chemin du fichier */
                  strcpy(window_preference->symbol5_file_path,file_path);
                  SetWindowText(GetDlgItem(window_preference->hwnd_preference,IDC_EDIT_SYMBOL_FILE_PATH5),window_preference->symbol5_file_path);
                }
              break;

            /** Case  cocher pour la trace des appels systmes (GS/OS + Toolbox) **/
            case IDC_CHECK_CALL_P8 :
              if(HIWORD(wParam) == BN_CLICKED)
                {
                  param->trace_p8_calls = (param->trace_p8_calls == 0) ? 1 : 0;
                  UpdatePreferenceControlStatus(window_preference);
                }
              break;
            case IDC_CHECK_CALL_P16 :
              if(HIWORD(wParam) == BN_CLICKED)
                {
                  param->trace_p16_calls = (param->trace_p16_calls == 0) ? 1 : 0;
                  UpdatePreferenceControlStatus(window_preference);
                }
              break;
            case IDC_CHECK_CALL_GSOS :
              if(HIWORD(wParam) == BN_CLICKED)
                {
                  param->trace_gsos_calls = (param->trace_gsos_calls == 0) ? 1 : 0;
                  UpdatePreferenceControlStatus(window_preference);
                }
              break;
            case IDC_CHECK_CALL_TOOLBOX :
              if(HIWORD(wParam) == BN_CLICKED)
                {
                  param->trace_toolbox_calls = (param->trace_toolbox_calls == 0) ? 1 : 0;
                  UpdatePreferenceControlStatus(window_preference);
                }
              break;

            case IDC_CHECK_TOOLBOX_001 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_001 = (param->trace_toolbox_001 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_002 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_002 = (param->trace_toolbox_002 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_003 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_003 = (param->trace_toolbox_003 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_004 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_004 = (param->trace_toolbox_004 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_005 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_005 = (param->trace_toolbox_005 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_006 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_006 = (param->trace_toolbox_006 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_007 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_007 = (param->trace_toolbox_007 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_008 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_008 = (param->trace_toolbox_008 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_009 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_009 = (param->trace_toolbox_009 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_010 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_010 = (param->trace_toolbox_010 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_011 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_011 = (param->trace_toolbox_011 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_012 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_012 = (param->trace_toolbox_012 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_013 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_013 = (param->trace_toolbox_013 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_014 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_014 = (param->trace_toolbox_014 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_015 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_015 = (param->trace_toolbox_015 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_016 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_016 = (param->trace_toolbox_016 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_017 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_017 = (param->trace_toolbox_017 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_018 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_018 = (param->trace_toolbox_018 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_019 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_019 = (param->trace_toolbox_019 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_020 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_020 = (param->trace_toolbox_020 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_021 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_021 = (param->trace_toolbox_021 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_022 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_022 = (param->trace_toolbox_022 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_023 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_023 = (param->trace_toolbox_023 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_024 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_024 = (param->trace_toolbox_024 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_025 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_025 = (param->trace_toolbox_025 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_026 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_026 = (param->trace_toolbox_026 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_027 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_027 = (param->trace_toolbox_027 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_028 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_028 = (param->trace_toolbox_028 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_029 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_029 = (param->trace_toolbox_029 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_030 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_030 = (param->trace_toolbox_030 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_032 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_032 = (param->trace_toolbox_032 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_033 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_033 = (param->trace_toolbox_033 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_034 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_034 = (param->trace_toolbox_034 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_035 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_035 = (param->trace_toolbox_035 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_037 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_037 = (param->trace_toolbox_037 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_038 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_038 = (param->trace_toolbox_038 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_050 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_050 = (param->trace_toolbox_050 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_051 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_051 = (param->trace_toolbox_051 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_052 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_052 = (param->trace_toolbox_052 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_053 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_053 = (param->trace_toolbox_053 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_128 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_128 = (param->trace_toolbox_128 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_129 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_129 = (param->trace_toolbox_129 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_130 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_130 = (param->trace_toolbox_130 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_131 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_131 = (param->trace_toolbox_131 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_132 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_132 = (param->trace_toolbox_132 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_219 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_219 = (param->trace_toolbox_219 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_220 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_220 = (param->trace_toolbox_220 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_221 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_221 = (param->trace_toolbox_221 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_222 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_222 = (param->trace_toolbox_222 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_230 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_230 = (param->trace_toolbox_230 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_TOOLBOX_255 :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_toolbox_255 = (param->trace_toolbox_255 == 0) ? 1 : 0;
              break;
            case IDC_CHECK_GOID :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_display_goid = (param->trace_display_goid == 0) ? 1 : 0;
              break;
            case IDC_CHECK_CYCLE :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_display_gcc = (param->trace_display_gcc == 0) ? 1 : 0;
              break;
            case IDC_CHECK_PC_ADDRESS :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_display_pc_address = (param->trace_display_pc_address == 0) ? 1 : 0;
              break;
            case IDC_CHECK_CALL_NUMBER :
              if(HIWORD(wParam) == BN_CLICKED)
                param->trace_display_call_number = (param->trace_display_call_number == 0) ? 1 : 0;
              break;

            /** Bouton de Sauvegarde des Preferences **/
            case IDC_BUTTON_UPDATE :
              UpdatePreferenceFile(window_preference);
              break;

            /** Bouton de fermeture de la fentre **/
            case IDC_BUTTON_CLOSE :
              SendMessage(window_preference->hwnd_preference,WM_CLOSE,0,0);
              break;
          }
        return(TRUE);

      /********************************************************************/
      /* Fermeture de window (le WM_CLOSE est appel avant le WM_DESTROY) */
      /********************************************************************/
      case WM_CLOSE :
        /* Fentre Preference */
        DestroyWindow(hwndPreference);
        return(0);

      case WM_DESTROY :
        /* Suppression des Fentres */
        DeletePreferenceChildWindows(window_preference);
        CheckMainWindowMenuItem(ID_MENU_EDIT_PREFERENCES,FALSE);
        my_Memory(MEMORY_FREE_WINDOW_PREFERENCE,NULL,NULL);
        /* Demande la sortie de la boucle des messages */
        PostQuitMessage(0);
        return(0);
    }

  /* Retourner FALSE pour relayer le message non trait au systme */
  return(FALSE);
}


/***********************************************************************************************************/
/*  CreatePreferenceChildWindows() : Crer les fentres enfants et remplir la structure window_preference. */
/***********************************************************************************************************/
static int CreatePreferenceChildWindows(HWND hwndPreference, struct window_preference *window_preference)
{
  int i;
  TCITEM tie;
  char *tab_tabcontrol_name[] = {" Cyrene Configuration files ", " Trace System Calls ", NULL};
  char buffer[BUFFER_SIZE];
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Window Handle **/
  window_preference->hwnd_preference = hwndPreference;

  /* Brush */
  window_preference->hBrush = CreateSolidBrush(GetSysColor(COLOR_3DFACE));

  /* Cration de la Police de caractre */
  window_preference->hFont = CreateFont(14,0,0,0,FW_DONTCARE,FALSE,FALSE,FALSE,ANSI_CHARSET,OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY,DEFAULT_PITCH|FF_SWISS,"Arial");

  /** Tableau  Onglets **/
  window_preference->hwnd_tabcontrol = CreateWindow(WC_TABCONTROL,"",WS_CHILD | WS_CLIPSIBLINGS | WS_VISIBLE | TCS_OWNERDRAWFIXED,      /* TCS_OWNERDRAWFIXED : Pour changer la couleur de fond */
                                                    8, 8, TAB_CONTROL_WIDTH, TAB_CONTROL_HEIGHT_1, window_preference->hwnd_preference,(HMENU)IDC_TABCONTROL,param->hInstance,0); 
  for(i=0; tab_tabcontrol_name[i] != NULL; i++)
    {
      /* On ajoute les Tab */
      tie.mask = TCIF_TEXT | TCIF_IMAGE; 
      tie.iImage = -1;
      tie.pszText = tab_tabcontrol_name[i];
      SendMessage(window_preference->hwnd_tabcontrol,TCM_INSERTITEM,(WPARAM)i,(LPARAM)&tie);
    }
  SendMessage(window_preference->hwnd_tabcontrol,WM_SETFONT,(WPARAM)window_preference->hFont,MAKELPARAM(FALSE,0));  

  /** Surchage des Control Edit pour capter les vnements **/
  window_preference->oldEditProcBreakpoint        = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndPreference,IDC_EDIT_BREAKPOINT_FILE_PATH),GWLP_WNDPROC,(LONG_PTR)subEditPreferenceProcControl);
  window_preference->oldEditProcQuickwatch        = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndPreference,IDC_EDIT_QUICKWATCH_FILE_PATH),GWLP_WNDPROC,(LONG_PTR)subEditPreferenceProcControl);
  window_preference->oldEditProcSymbol1           = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndPreference,IDC_EDIT_SYMBOL_FILE_PATH1),GWLP_WNDPROC,(LONG_PTR)subEditPreferenceProcControl);
  window_preference->oldEditProcSymbol2           = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndPreference,IDC_EDIT_SYMBOL_FILE_PATH2),GWLP_WNDPROC,(LONG_PTR)subEditPreferenceProcControl);
  window_preference->oldEditProcSymbol3           = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndPreference,IDC_EDIT_SYMBOL_FILE_PATH3),GWLP_WNDPROC,(LONG_PTR)subEditPreferenceProcControl);
  window_preference->oldEditProcSymbol4           = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndPreference,IDC_EDIT_SYMBOL_FILE_PATH4),GWLP_WNDPROC,(LONG_PTR)subEditPreferenceProcControl);
  window_preference->oldEditProcSymbol5           = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndPreference,IDC_EDIT_SYMBOL_FILE_PATH5),GWLP_WNDPROC,(LONG_PTR)subEditPreferenceProcControl);
  window_preference->oldEditProcIgnoreOsCall      = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndPreference,IDC_EDIT_IGNORE_OS_CALLS),GWLP_WNDPROC,(LONG_PTR)subEditPreferenceProcControl);
  window_preference->oldEditProcTraceOsCall       = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndPreference,IDC_EDIT_TRACE_OS_CALLS),GWLP_WNDPROC,(LONG_PTR)subEditPreferenceProcControl);
  window_preference->oldEditProcIgnoreToolboxCall = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndPreference,IDC_EDIT_IGNORE_TOOLBOX_CALLS),GWLP_WNDPROC,(LONG_PTR)subEditPreferenceProcControl);
  window_preference->oldEditProcTraceToolboxCall  = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndPreference,IDC_EDIT_TRACE_TOOLBOX_CALLS),GWLP_WNDPROC,(LONG_PTR)subEditPreferenceProcControl);

  /** Charge le fichier Cyrene.ini en mmoire **/
  LoadPreferenceFile(window_preference);

  /** Mise  jour des controles **/
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_LOAD_FILE_STARTUP,BM_SETCHECK,window_preference->reload_at_startup,0);
  SetDlgItemText(window_preference->hwnd_preference,IDC_EDIT_BREAKPOINT_FILE_PATH,window_preference->breakpoint_file_path);
  SetDlgItemText(window_preference->hwnd_preference,IDC_EDIT_QUICKWATCH_FILE_PATH,window_preference->quickwatch_file_path);
  SetDlgItemText(window_preference->hwnd_preference,IDC_EDIT_SYMBOL_FILE_PATH1,window_preference->symbol1_file_path);
  SetDlgItemText(window_preference->hwnd_preference,IDC_EDIT_SYMBOL_FILE_PATH2,window_preference->symbol2_file_path);
  SetDlgItemText(window_preference->hwnd_preference,IDC_EDIT_SYMBOL_FILE_PATH3,window_preference->symbol3_file_path);
  SetDlgItemText(window_preference->hwnd_preference,IDC_EDIT_SYMBOL_FILE_PATH4,window_preference->symbol4_file_path);
  SetDlgItemText(window_preference->hwnd_preference,IDC_EDIT_SYMBOL_FILE_PATH5,window_preference->symbol5_file_path);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_CALL_P8,BM_SETCHECK,param->trace_p8_calls,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_CALL_P16,BM_SETCHECK,param->trace_p16_calls,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_CALL_GSOS,BM_SETCHECK,param->trace_gsos_calls,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_CALL_TOOLBOX,BM_SETCHECK,param->trace_toolbox_calls,0);

  buffer[0] = '\0';
  for(i=0; i<param->nb_ignore_os_call; i++)
    sprintf(&buffer[strlen(buffer)],"%X ",param->tab_ignore_os_call[i]);
  SetDlgItemText(window_preference->hwnd_preference,IDC_EDIT_IGNORE_OS_CALLS,buffer);
  buffer[0] = '\0';
  for(i=0; i<param->nb_traceonly_os_call; i++)
    sprintf(&buffer[strlen(buffer)],"%X ",param->tab_traceonly_os_call[i]);
  SetDlgItemText(window_preference->hwnd_preference,IDC_EDIT_TRACE_OS_CALLS,buffer);
  buffer[0] = '\0';
  for(i=0; i<param->nb_ignore_toolbox_call; i++)
    sprintf(&buffer[strlen(buffer)],"%X ",param->tab_ignore_toolbox_call[i]);
  SetDlgItemText(window_preference->hwnd_preference,IDC_EDIT_IGNORE_TOOLBOX_CALLS,buffer);
  buffer[0] = '\0';
  for(i=0; i<param->nb_traceonly_toolbox_call; i++)
    sprintf(&buffer[strlen(buffer)],"%X ",param->tab_traceonly_toolbox_call[i]);
  SetDlgItemText(window_preference->hwnd_preference,IDC_EDIT_TRACE_TOOLBOX_CALLS,buffer);

  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_001,BM_SETCHECK,param->trace_toolbox_001,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_002,BM_SETCHECK,param->trace_toolbox_002,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_003,BM_SETCHECK,param->trace_toolbox_003,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_004,BM_SETCHECK,param->trace_toolbox_004,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_005,BM_SETCHECK,param->trace_toolbox_005,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_006,BM_SETCHECK,param->trace_toolbox_006,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_007,BM_SETCHECK,param->trace_toolbox_007,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_008,BM_SETCHECK,param->trace_toolbox_008,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_009,BM_SETCHECK,param->trace_toolbox_009,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_010,BM_SETCHECK,param->trace_toolbox_010,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_011,BM_SETCHECK,param->trace_toolbox_011,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_012,BM_SETCHECK,param->trace_toolbox_012,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_013,BM_SETCHECK,param->trace_toolbox_013,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_014,BM_SETCHECK,param->trace_toolbox_014,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_015,BM_SETCHECK,param->trace_toolbox_015,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_016,BM_SETCHECK,param->trace_toolbox_016,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_017,BM_SETCHECK,param->trace_toolbox_017,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_018,BM_SETCHECK,param->trace_toolbox_018,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_019,BM_SETCHECK,param->trace_toolbox_019,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_020,BM_SETCHECK,param->trace_toolbox_020,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_021,BM_SETCHECK,param->trace_toolbox_021,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_022,BM_SETCHECK,param->trace_toolbox_022,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_023,BM_SETCHECK,param->trace_toolbox_023,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_024,BM_SETCHECK,param->trace_toolbox_024,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_025,BM_SETCHECK,param->trace_toolbox_025,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_026,BM_SETCHECK,param->trace_toolbox_026,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_027,BM_SETCHECK,param->trace_toolbox_027,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_028,BM_SETCHECK,param->trace_toolbox_028,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_029,BM_SETCHECK,param->trace_toolbox_029,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_030,BM_SETCHECK,param->trace_toolbox_030,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_032,BM_SETCHECK,param->trace_toolbox_032,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_033,BM_SETCHECK,param->trace_toolbox_033,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_034,BM_SETCHECK,param->trace_toolbox_034,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_035,BM_SETCHECK,param->trace_toolbox_035,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_037,BM_SETCHECK,param->trace_toolbox_037,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_038,BM_SETCHECK,param->trace_toolbox_038,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_050,BM_SETCHECK,param->trace_toolbox_050,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_051,BM_SETCHECK,param->trace_toolbox_051,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_052,BM_SETCHECK,param->trace_toolbox_052,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_053,BM_SETCHECK,param->trace_toolbox_053,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_128,BM_SETCHECK,param->trace_toolbox_128,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_129,BM_SETCHECK,param->trace_toolbox_129,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_130,BM_SETCHECK,param->trace_toolbox_130,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_131,BM_SETCHECK,param->trace_toolbox_131,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_132,BM_SETCHECK,param->trace_toolbox_132,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_219,BM_SETCHECK,param->trace_toolbox_219,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_220,BM_SETCHECK,param->trace_toolbox_220,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_221,BM_SETCHECK,param->trace_toolbox_221,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_222,BM_SETCHECK,param->trace_toolbox_222,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_230,BM_SETCHECK,param->trace_toolbox_230,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_TOOLBOX_255,BM_SETCHECK,param->trace_toolbox_255,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_GOID,BM_SETCHECK,param->trace_display_goid,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_CYCLE,BM_SETCHECK,param->trace_display_gcc,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_PC_ADDRESS,BM_SETCHECK,param->trace_display_pc_address,0);
  SendDlgItemMessage(window_preference->hwnd_preference,IDC_CHECK_CALL_NUMBER,BM_SETCHECK,param->trace_display_call_number,0);

  /** MAJ du Statut des Contrles **/
  UpdatePreferenceControlStatus(window_preference);

  /** On affiche l'onglet Configuration Files **/
  ShowOnePreferenceTabControl(TAB_CONFIGURATION_FILES,window_preference);

  /* OK */
  return(0);
}


/*************************************************************************************************/
/*  subEditPreferenceProcControl() :  Rcupre les vnements qui arrivent dans le Control Edit. */
/*************************************************************************************************/
LRESULT CALLBACK subEditPreferenceProcControl(HWND hwnd_control, UINT msg, WPARAM wParam, LPARAM lParam)
{
  int index;
  struct window_preference *window_preference;
  my_Memory(MEMORY_GET_WINDOW_PREFERENCE,&window_preference,NULL);

  /* Rien  faire */
  if(window_preference == NULL)
    return(0);

  /** Identifie le Control **/
  /* Src */
  if(GetDlgItem(window_preference->hwnd_preference,IDC_EDIT_BREAKPOINT_FILE_PATH) == hwnd_control)
    index = 1;
  else if(GetDlgItem(window_preference->hwnd_preference,IDC_EDIT_QUICKWATCH_FILE_PATH) == hwnd_control)
    index = 2;
  else if(GetDlgItem(window_preference->hwnd_preference,IDC_EDIT_SYMBOL_FILE_PATH1) == hwnd_control)
    index = 3;
  else if(GetDlgItem(window_preference->hwnd_preference,IDC_EDIT_SYMBOL_FILE_PATH2) == hwnd_control)
    index = 4;
  else if(GetDlgItem(window_preference->hwnd_preference,IDC_EDIT_SYMBOL_FILE_PATH3) == hwnd_control)
    index = 5;
  else if(GetDlgItem(window_preference->hwnd_preference,IDC_EDIT_SYMBOL_FILE_PATH4) == hwnd_control)
    index = 6;
  else if(GetDlgItem(window_preference->hwnd_preference,IDC_EDIT_SYMBOL_FILE_PATH5) == hwnd_control)
    index = 7;
  else if(GetDlgItem(window_preference->hwnd_preference,IDC_EDIT_IGNORE_OS_CALLS) == hwnd_control)
    index = 8;
  else if(GetDlgItem(window_preference->hwnd_preference,IDC_EDIT_TRACE_OS_CALLS) == hwnd_control)
    index = 9;
  else if(GetDlgItem(window_preference->hwnd_preference,IDC_EDIT_IGNORE_TOOLBOX_CALLS) == hwnd_control)
    index = 10;
  else if(GetDlgItem(window_preference->hwnd_preference,IDC_EDIT_TRACE_TOOLBOX_CALLS) == hwnd_control)
    index = 11;
  else
    return(0);      /* Pas trouv */

  /** Dtecte l'vnement de la souris/clavier dans l'Edit **/
  if(msg == WM_CHAR && wParam == VK_RETURN)
    {
      /* Enlve le focus de l'Edit */
      SetFocus(window_preference->hwnd_preference);
    }
  else if(index == 1)
    return(CallWindowProc(window_preference->oldEditProcBreakpoint,hwnd_control,msg,wParam,lParam)); 
  else if(index == 2)
    return(CallWindowProc(window_preference->oldEditProcQuickwatch,hwnd_control,msg,wParam,lParam)); 
  else if(index == 3)
    return(CallWindowProc(window_preference->oldEditProcSymbol1,hwnd_control,msg,wParam,lParam)); 
  else if(index == 4)
    return(CallWindowProc(window_preference->oldEditProcSymbol2,hwnd_control,msg,wParam,lParam));
  else if(index == 5)
    return(CallWindowProc(window_preference->oldEditProcSymbol3,hwnd_control,msg,wParam,lParam));
  else if(index == 6)
    return(CallWindowProc(window_preference->oldEditProcSymbol4,hwnd_control,msg,wParam,lParam));
  else if(index == 7)
    return(CallWindowProc(window_preference->oldEditProcSymbol5,hwnd_control,msg,wParam,lParam));
  else if(index == 8)
    return(CallWindowProc(window_preference->oldEditProcIgnoreOsCall,hwnd_control,msg,wParam,lParam));
  else if(index == 9)
    return(CallWindowProc(window_preference->oldEditProcTraceOsCall,hwnd_control,msg,wParam,lParam));
  else if(index == 10)
    return(CallWindowProc(window_preference->oldEditProcIgnoreToolboxCall,hwnd_control,msg,wParam,lParam));
  else if(index == 11)
    return(CallWindowProc(window_preference->oldEditProcTraceToolboxCall,hwnd_control,msg,wParam,lParam));

  /* OK */
  return(0);
}


/********************************************************************************/
/*  ShowOnePreferenceTabControl() :  Affiche le contenu d'un onglet du tableau. */
/********************************************************************************/
static void ShowOnePreferenceTabControl(int tab_index, struct window_preference *window_preference)
{
  int i, button1_width, button1_height, button2_width, button2_height, button_y, window_width, window_height, title_height;
  RECT tab_rect, button1_rect, button2_rect, window_rect, client_rect;
  int tab_control_configuration_files[] = 
    {
      IDC_STATIC_CYRENE_INIT,           IDC_STATIC_BREAKPOINT_FILE,          IDC_STATIC_QUICKWATCH_FILE,
      IDC_STATIC_SYMBOL_FILE_1,         IDC_STATIC_SYMBOL_FILE_2,            IDC_STATIC_SYMBOL_FILE_3,
      IDC_STATIC_SYMBOL_FILE_4,         IDC_STATIC_SYMBOL_FILE_5,            IDC_CHECK_LOAD_FILE_STARTUP,
      IDC_EDIT_BREAKPOINT_FILE_PATH,    IDC_EDIT_QUICKWATCH_FILE_PATH,       IDC_EDIT_SYMBOL_FILE_PATH1,
      IDC_EDIT_SYMBOL_FILE_PATH2,       IDC_EDIT_SYMBOL_FILE_PATH3,          IDC_EDIT_SYMBOL_FILE_PATH4,
      IDC_EDIT_SYMBOL_FILE_PATH5,       IDC_BUTTON_FILE_GET_PATH_BREAKPOINT, IDC_BUTTON_FILE_GET_PATH_QUICKWATCH,
      IDC_BUTTON_FILE_GET_PATH_SYMBOL1, IDC_BUTTON_FILE_GET_PATH_SYMBOL2,    IDC_BUTTON_FILE_GET_PATH_SYMBOL3,
      IDC_BUTTON_FILE_GET_PATH_SYMBOL4, IDC_BUTTON_FILE_GET_PATH_SYMBOL5,    0
    };
  int tab_control_trace_system_calls[] = 
    {
      IDC_STATIC_TRACE_SYSTEM_CALLS, IDC_STATIC_DISPLAY_INFORMATION,  IDC_STATIC_TOOLBOX_CALLS,
      IDC_CHECK_CALL_P8,             IDC_CHECK_CALL_P16,              IDC_CHECK_CALL_GSOS,
      IDC_CHECK_CALL_TOOLBOX,        IDC_CHECK_GOID,                  IDC_CHECK_CYCLE,
      IDC_CHECK_PC_ADDRESS,          IDC_CHECK_CALL_NUMBER,           IDC_CHECK_TOOLBOX_001,
      IDC_CHECK_TOOLBOX_002,         IDC_CHECK_TOOLBOX_003,           IDC_CHECK_TOOLBOX_004,
      IDC_CHECK_TOOLBOX_005,         IDC_CHECK_TOOLBOX_006,           IDC_CHECK_TOOLBOX_007,
      IDC_CHECK_TOOLBOX_008,         IDC_CHECK_TOOLBOX_009,           IDC_CHECK_TOOLBOX_010,
      IDC_CHECK_TOOLBOX_011,         IDC_CHECK_TOOLBOX_012,           IDC_CHECK_TOOLBOX_013,
      IDC_CHECK_TOOLBOX_014,         IDC_CHECK_TOOLBOX_015,           IDC_CHECK_TOOLBOX_016,
      IDC_CHECK_TOOLBOX_017,         IDC_CHECK_TOOLBOX_018,           IDC_CHECK_TOOLBOX_019,
      IDC_CHECK_TOOLBOX_020,         IDC_CHECK_TOOLBOX_021,           IDC_CHECK_TOOLBOX_022,
      IDC_CHECK_TOOLBOX_023,         IDC_CHECK_TOOLBOX_024,           IDC_CHECK_TOOLBOX_025,
      IDC_CHECK_TOOLBOX_026,         IDC_CHECK_TOOLBOX_027,           IDC_CHECK_TOOLBOX_028,
      IDC_CHECK_TOOLBOX_029,         IDC_CHECK_TOOLBOX_030,           IDC_CHECK_TOOLBOX_032,
      IDC_CHECK_TOOLBOX_033,         IDC_CHECK_TOOLBOX_034,           IDC_CHECK_TOOLBOX_035,
      IDC_CHECK_TOOLBOX_037,         IDC_CHECK_TOOLBOX_038,           IDC_CHECK_TOOLBOX_050,
      IDC_CHECK_TOOLBOX_051,         IDC_CHECK_TOOLBOX_052,           IDC_CHECK_TOOLBOX_053,
      IDC_CHECK_TOOLBOX_128,         IDC_CHECK_TOOLBOX_129,           IDC_CHECK_TOOLBOX_130,
      IDC_CHECK_TOOLBOX_131,         IDC_CHECK_TOOLBOX_132,           IDC_CHECK_TOOLBOX_219,
      IDC_CHECK_TOOLBOX_220,         IDC_CHECK_TOOLBOX_221,           IDC_CHECK_TOOLBOX_222,
      IDC_CHECK_TOOLBOX_230,         IDC_CHECK_TOOLBOX_255,           IDC_STATIC_IGNORE_OS_CALLS,
      IDC_STATIC_TRACE_OS_CALLS,     IDC_STATIC_IGNORE_TOOLBOX_CALLS, IDC_STATIC_TRACE_TOOLBOX_CALLS,
      IDC_EDIT_IGNORE_OS_CALLS,      IDC_EDIT_TRACE_OS_CALLS,         IDC_EDIT_IGNORE_TOOLBOX_CALLS,
      IDC_EDIT_TRACE_TOOLBOX_CALLS,
      0
    };

  /* Position du Tableau */
  GetWindowRect(window_preference->hwnd_tabcontrol,&tab_rect);
  MapWindowPoints(HWND_DESKTOP,window_preference->hwnd_preference,(LPPOINT)&tab_rect,2);

  /* Taille des boutons 1 & 2 */
  GetWindowRect(GetDlgItem(window_preference->hwnd_preference,IDC_BUTTON_CLOSE),&button1_rect);
  button1_width = (button1_rect.right - button1_rect.left);
  button1_height = (button1_rect.bottom - button1_rect.top);
  GetWindowRect(GetDlgItem(window_preference->hwnd_preference,IDC_BUTTON_UPDATE),&button2_rect);
  button2_width = (button2_rect.right - button2_rect.left);
  button2_height = (button2_rect.bottom - button2_rect.top);

  /* Position des Boutons 1 & 2 */
  GetWindowRect(GetDlgItem(window_preference->hwnd_preference,IDC_BUTTON_CLOSE),&button1_rect);
  MapWindowPoints(HWND_DESKTOP,window_preference->hwnd_preference,(LPPOINT)&button1_rect,2);
  GetWindowRect(GetDlgItem(window_preference->hwnd_preference,IDC_BUTTON_UPDATE),&button2_rect);
  MapWindowPoints(HWND_DESKTOP,window_preference->hwnd_preference,(LPPOINT)&button2_rect,2);

  /* Position de la fentre */
  GetWindowRect(window_preference->hwnd_preference,&window_rect);
  window_width = (window_rect.right - window_rect.left);

  /* Hauteur Titre */
  GetWindowRect(window_preference->hwnd_preference,&window_rect);
  GetClientRect(window_preference->hwnd_preference,&client_rect);
  title_height = (window_rect.bottom - window_rect.top) - client_rect.bottom;

  /** Affiche / Cache les controles **/
  if(tab_index == TAB_CONFIGURATION_FILES)
    {
      /* Masque */
      for(i=0; tab_control_trace_system_calls[i] != 0; i++)
        ShowWindow(GetDlgItem(window_preference->hwnd_preference,tab_control_trace_system_calls[i]),SW_HIDE);

      /* Affiche */
      for(i=0; tab_control_configuration_files[i] != 0; i++)
        ShowWindow(GetDlgItem(window_preference->hwnd_preference,tab_control_configuration_files[i]),SW_SHOW);

      /* Hauteur de Tableau  onglet */
      MoveWindow(window_preference->hwnd_tabcontrol,tab_rect.left,tab_rect.top,TAB_CONTROL_WIDTH,TAB_CONTROL_HEIGHT_1,TRUE);

      /* Dplace les Boutons */
      button_y = tab_rect.top + TAB_CONTROL_HEIGHT_1 + 10;
      MoveWindow(GetDlgItem(window_preference->hwnd_preference,IDC_BUTTON_CLOSE) ,button1_rect.left,button_y,button1_width,button1_height,TRUE);
      MoveWindow(GetDlgItem(window_preference->hwnd_preference,IDC_BUTTON_UPDATE),button2_rect.left,button_y,button2_width,button2_height,TRUE);

      /* Retaille la fentre */
      window_height = title_height + TAB_CONTROL_HEIGHT_1 + 10 + button1_height + 17;
      MoveWindow(window_preference->hwnd_preference,window_rect.left,window_rect.top,window_width,window_height,TRUE);
    }
  else
    {
      /* Masque */
      for(i=0; tab_control_configuration_files[i] != 0; i++)
        ShowWindow(GetDlgItem(window_preference->hwnd_preference,tab_control_configuration_files[i]),SW_HIDE);

      /* Affiche */
      for(i=0; tab_control_trace_system_calls[i] != 0; i++)
        ShowWindow(GetDlgItem(window_preference->hwnd_preference,tab_control_trace_system_calls[i]),SW_SHOW);

      /* Hauteur de Tableau  onglet */
      MoveWindow(window_preference->hwnd_tabcontrol,tab_rect.left,tab_rect.top,TAB_CONTROL_WIDTH,TAB_CONTROL_HEIGHT_2,TRUE);

      /* Dplace les Boutons */
      button_y = tab_rect.top + TAB_CONTROL_HEIGHT_2 + 10;
      MoveWindow(GetDlgItem(window_preference->hwnd_preference,IDC_BUTTON_CLOSE) ,button1_rect.left,button_y,button1_width,button1_height,TRUE);
      MoveWindow(GetDlgItem(window_preference->hwnd_preference,IDC_BUTTON_UPDATE),button2_rect.left,button_y,button2_width,button2_height,TRUE);

      /* Retaille la fentre */
      window_height = title_height + TAB_CONTROL_HEIGHT_2 + 10 + button1_height + 17;
      MoveWindow(window_preference->hwnd_preference,window_rect.left,window_rect.top,window_width,window_height,TRUE);
    }
}


/*********************************************************************/
/*  LoadPreferenceFile() :  Charge le fichier Cyrene.ini en mmoire. */
/*********************************************************************/
static void LoadPreferenceFile(struct window_preference *window_preference)
{
  int i, file_size;
  char *data;
  char error_buffer[1024];
  char ini_file_path[1024];
  char value[1024];
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Chemin du fichier Cyrene.ini */
  GetModuleFileName(NULL,ini_file_path,MAX_PATH);
  for(i=(int)strlen(ini_file_path); i>=0; i--)
    if(ini_file_path[i] == '.')
      {
        strcpy(&ini_file_path[i],".ini");
        break;
      }

  /* Cration du contenu fichier */
  data = (char *) LoadTextFile(ini_file_path,&file_size,error_buffer);
  if(data == NULL)
    return;

  /** Rcupration des Valeurs **/
  GetTextValue(data,"ReloadFilesAtStartup=",value);
  window_preference->reload_at_startup = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"BreakpointFilePath=",window_preference->breakpoint_file_path);
  GetTextValue(data,"QuickWatchFilePath=",window_preference->quickwatch_file_path);
  GetTextValue(data,"Symbol1FilePath=",window_preference->symbol1_file_path);
  GetTextValue(data,"Symbol2FilePath=",window_preference->symbol2_file_path);
  GetTextValue(data,"Symbol3FilePath=",window_preference->symbol3_file_path);
  GetTextValue(data,"Symbol4FilePath=",window_preference->symbol4_file_path);
  GetTextValue(data,"Symbol5FilePath=",window_preference->symbol5_file_path);
  GetTextValue(data,"TraceP8Calls=",value);
  param->trace_p8_calls = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceP16Calls=",value);
  param->trace_p16_calls = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceGSOSCalls=",value);
  param->trace_gsos_calls = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolboxCalls=",value);
  param->trace_toolbox_calls = (!stricmp(value,"1")) ? 1 : 0;

  GetTextValue(data,"IgnoreOSCalls=",value);
  param->nb_ignore_os_call = DecodeTraceCallList(value,&param->tab_ignore_os_call[0]);
  GetTextValue(data,"TraceOnlyOSCalls=",value);
  param->nb_traceonly_os_call = DecodeTraceCallList(value,&param->tab_traceonly_os_call[0]);
  GetTextValue(data,"IgnoreToolboxCalls=",value);
  param->nb_ignore_toolbox_call = DecodeTraceCallList(value,&param->tab_ignore_toolbox_call[0]);
  GetTextValue(data,"TraceOnlyToolboxCalls=",value);
  param->nb_traceonly_toolbox_call = DecodeTraceCallList(value,&param->tab_traceonly_toolbox_call[0]);

  GetTextValue(data,"TraceToolbox001Calls=",value);
  param->trace_toolbox_001 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox002Calls=",value);
  param->trace_toolbox_002 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox003Calls=",value);
  param->trace_toolbox_003 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox004Calls=",value);
  param->trace_toolbox_004 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox005Calls=",value);
  param->trace_toolbox_005 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox006Calls=",value);
  param->trace_toolbox_006 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox007Calls=",value);
  param->trace_toolbox_007 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox008Calls=",value);
  param->trace_toolbox_008 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox009Calls=",value);
  param->trace_toolbox_009 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox010Calls=",value);
  param->trace_toolbox_010 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox011Calls=",value);
  param->trace_toolbox_011 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox012Calls=",value);
  param->trace_toolbox_012 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox013Calls=",value);
  param->trace_toolbox_013 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox014Calls=",value);
  param->trace_toolbox_014 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox015Calls=",value);
  param->trace_toolbox_015 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox016Calls=",value);
  param->trace_toolbox_016 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox017Calls=",value);
  param->trace_toolbox_017 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox018Calls=",value);
  param->trace_toolbox_018 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox019Calls=",value);
  param->trace_toolbox_019 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox020Calls=",value);
  param->trace_toolbox_020 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox021Calls=",value);
  param->trace_toolbox_021 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox022Calls=",value);
  param->trace_toolbox_022 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox023Calls=",value);
  param->trace_toolbox_023 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox024Calls=",value);
  param->trace_toolbox_024 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox025Calls=",value);
  param->trace_toolbox_025 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox026Calls=",value);
  param->trace_toolbox_026 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox027Calls=",value);
  param->trace_toolbox_027 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox028Calls=",value);
  param->trace_toolbox_028 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox029Calls=",value);
  param->trace_toolbox_029 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox030Calls=",value);
  param->trace_toolbox_030 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox032Calls=",value);
  param->trace_toolbox_032 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox033Calls=",value);
  param->trace_toolbox_033 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox034Calls=",value);
  param->trace_toolbox_034 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox035Calls=",value);
  param->trace_toolbox_035 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox037Calls=",value);
  param->trace_toolbox_037 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox038Calls=",value);
  param->trace_toolbox_038 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox050Calls=",value);
  param->trace_toolbox_050 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox051Calls=",value);
  param->trace_toolbox_051 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox052Calls=",value);
  param->trace_toolbox_052 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox053Calls=",value);
  param->trace_toolbox_053 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox128Calls=",value);
  param->trace_toolbox_128 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox129Calls=",value);
  param->trace_toolbox_129 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox130Calls=",value);
  param->trace_toolbox_130 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox131Calls=",value);
  param->trace_toolbox_131 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox132Calls=",value);
  param->trace_toolbox_132 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox219Calls=",value);
  param->trace_toolbox_219 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox220Calls=",value);
  param->trace_toolbox_220 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox221Calls=",value);
  param->trace_toolbox_221 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox222Calls=",value);
  param->trace_toolbox_222 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox230Calls=",value);
  param->trace_toolbox_230 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox255Calls=",value);
  param->trace_toolbox_255 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceDisplayGOID=",value);
  param->trace_display_goid = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceDisplayGCC=",value);
  param->trace_display_gcc = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceDisplayPCAddress=",value);
  param->trace_display_pc_address = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceDisplayCallNumber=",value);
  param->trace_display_call_number = (!stricmp(value,"1")) ? 1 : 0;

  /* Libration mmoire */
  free(data);
}


/*************************************************************************************/
/*  UpdatePreferenceFile() :  Enregistre dans le fichier Cyrene.ini les prfrences. */
/*************************************************************************************/
static void UpdatePreferenceFile(struct window_preference *window_preference)
{
  int i;
  FILE *fd;
  char ini_file_path[1024];
  char buffer[BUFFER_SIZE];
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Chemin du fichier Cyrene.ini */
  GetModuleFileName(NULL,ini_file_path,MAX_PATH);
  for(i=(int)strlen(ini_file_path); i>=0; i--)
    if(ini_file_path[i] == '.')
      {
        strcpy(&ini_file_path[i],".ini");
        break;
      }

  /* Cration du contenu fichier */
  fd = fopen(ini_file_path,"w");
  if(fd == NULL)
    return;

  /** Rcupre les valeurs des contrles **/
  GetDlgItemText(window_preference->hwnd_preference,IDC_EDIT_BREAKPOINT_FILE_PATH,window_preference->breakpoint_file_path,1024);
  GetDlgItemText(window_preference->hwnd_preference,IDC_EDIT_QUICKWATCH_FILE_PATH,window_preference->quickwatch_file_path,1024);
  GetDlgItemText(window_preference->hwnd_preference,IDC_EDIT_SYMBOL_FILE_PATH1,window_preference->symbol1_file_path,1024);
  GetDlgItemText(window_preference->hwnd_preference,IDC_EDIT_SYMBOL_FILE_PATH2,window_preference->symbol2_file_path,1024);
  GetDlgItemText(window_preference->hwnd_preference,IDC_EDIT_SYMBOL_FILE_PATH3,window_preference->symbol3_file_path,1024);
  GetDlgItemText(window_preference->hwnd_preference,IDC_EDIT_SYMBOL_FILE_PATH4,window_preference->symbol4_file_path,1024);
  GetDlgItemText(window_preference->hwnd_preference,IDC_EDIT_SYMBOL_FILE_PATH5,window_preference->symbol5_file_path,1024);

  /** Ecriture des donnes **/
  fprintf(fd,"ReloadFilesAtStartup=%d\n",(window_preference->reload_at_startup==1)?1:0);
  fprintf(fd,"BreakpointFilePath=%s\n",window_preference->breakpoint_file_path);
  fprintf(fd,"QuickWatchFilePath=%s\n",window_preference->quickwatch_file_path);
  fprintf(fd,"Symbol1FilePath=%s\n",window_preference->symbol1_file_path);
  fprintf(fd,"Symbol2FilePath=%s\n",window_preference->symbol2_file_path);
  fprintf(fd,"Symbol3FilePath=%s\n",window_preference->symbol3_file_path);
  fprintf(fd,"Symbol4FilePath=%s\n",window_preference->symbol4_file_path);
  fprintf(fd,"Symbol5FilePath=%s\n",window_preference->symbol5_file_path);
  fprintf(fd,"TraceP8Calls=%d\n",(param->trace_p8_calls==1)?1:0);
  fprintf(fd,"TraceP16Calls=%d\n",(param->trace_p16_calls==1)?1:0);
  fprintf(fd,"TraceGSOSCalls=%d\n",(param->trace_gsos_calls==1)?1:0);
  fprintf(fd,"TraceToolboxCalls=%d\n",(param->trace_toolbox_calls==1)?1:0);

  GetDlgItemText(window_preference->hwnd_preference,IDC_EDIT_IGNORE_OS_CALLS,buffer,1024);
  param->nb_ignore_os_call = DecodeTraceCallList(buffer,&param->tab_ignore_os_call[0]);
  fprintf(fd,"IgnoreOSCalls=");
  for(i=0; i<param->nb_ignore_os_call; i++)
    fprintf(fd,"%X ",param->tab_ignore_os_call[i]);
  fprintf(fd,"\n");

  GetDlgItemText(window_preference->hwnd_preference,IDC_EDIT_TRACE_OS_CALLS,buffer,1024);
  param->nb_traceonly_os_call = DecodeTraceCallList(buffer,&param->tab_traceonly_os_call[0]);
  fprintf(fd,"TraceOnlyOSCalls=");
  for(i=0; i<param->nb_traceonly_os_call; i++)
    fprintf(fd,"%X ",param->tab_traceonly_os_call[i]);
  fprintf(fd,"\n");

  GetDlgItemText(window_preference->hwnd_preference,IDC_EDIT_IGNORE_TOOLBOX_CALLS,buffer,1024);
  param->nb_ignore_toolbox_call = DecodeTraceCallList(buffer,&param->tab_ignore_toolbox_call[0]);
  fprintf(fd,"IgnoreToolboxCalls=");
  for(i=0; i<param->nb_ignore_toolbox_call; i++)
    fprintf(fd,"%X ",param->tab_ignore_toolbox_call[i]);
  fprintf(fd,"\n");

  GetDlgItemText(window_preference->hwnd_preference,IDC_EDIT_TRACE_TOOLBOX_CALLS,buffer,1024);
  param->nb_traceonly_toolbox_call = DecodeTraceCallList(buffer,&param->tab_traceonly_toolbox_call[0]);
  fprintf(fd,"TraceOnlyToolboxCalls=");
  for(i=0; i<param->nb_traceonly_toolbox_call; i++)
    fprintf(fd,"%X ",param->tab_traceonly_toolbox_call[i]);
  fprintf(fd,"\n");

  fprintf(fd,"TraceToolbox001Calls=%d\n",(param->trace_toolbox_001==1)?1:0);
  fprintf(fd,"TraceToolbox002Calls=%d\n",(param->trace_toolbox_002==1)?1:0);
  fprintf(fd,"TraceToolbox003Calls=%d\n",(param->trace_toolbox_003==1)?1:0);
  fprintf(fd,"TraceToolbox004Calls=%d\n",(param->trace_toolbox_004==1)?1:0);
  fprintf(fd,"TraceToolbox005Calls=%d\n",(param->trace_toolbox_005==1)?1:0);
  fprintf(fd,"TraceToolbox006Calls=%d\n",(param->trace_toolbox_006==1)?1:0);
  fprintf(fd,"TraceToolbox007Calls=%d\n",(param->trace_toolbox_007==1)?1:0);
  fprintf(fd,"TraceToolbox008Calls=%d\n",(param->trace_toolbox_008==1)?1:0);
  fprintf(fd,"TraceToolbox009Calls=%d\n",(param->trace_toolbox_009==1)?1:0);
  fprintf(fd,"TraceToolbox010Calls=%d\n",(param->trace_toolbox_010==1)?1:0);
  fprintf(fd,"TraceToolbox011Calls=%d\n",(param->trace_toolbox_011==1)?1:0);
  fprintf(fd,"TraceToolbox012Calls=%d\n",(param->trace_toolbox_012==1)?1:0);
  fprintf(fd,"TraceToolbox013Calls=%d\n",(param->trace_toolbox_013==1)?1:0);
  fprintf(fd,"TraceToolbox014Calls=%d\n",(param->trace_toolbox_014==1)?1:0);
  fprintf(fd,"TraceToolbox015Calls=%d\n",(param->trace_toolbox_015==1)?1:0);
  fprintf(fd,"TraceToolbox016Calls=%d\n",(param->trace_toolbox_016==1)?1:0);
  fprintf(fd,"TraceToolbox017Calls=%d\n",(param->trace_toolbox_017==1)?1:0);
  fprintf(fd,"TraceToolbox018Calls=%d\n",(param->trace_toolbox_018==1)?1:0);
  fprintf(fd,"TraceToolbox019Calls=%d\n",(param->trace_toolbox_019==1)?1:0);
  fprintf(fd,"TraceToolbox020Calls=%d\n",(param->trace_toolbox_020==1)?1:0);
  fprintf(fd,"TraceToolbox021Calls=%d\n",(param->trace_toolbox_021==1)?1:0);
  fprintf(fd,"TraceToolbox022Calls=%d\n",(param->trace_toolbox_022==1)?1:0);
  fprintf(fd,"TraceToolbox023Calls=%d\n",(param->trace_toolbox_023==1)?1:0);
  fprintf(fd,"TraceToolbox024Calls=%d\n",(param->trace_toolbox_024==1)?1:0);
  fprintf(fd,"TraceToolbox025Calls=%d\n",(param->trace_toolbox_025==1)?1:0);
  fprintf(fd,"TraceToolbox026Calls=%d\n",(param->trace_toolbox_026==1)?1:0);
  fprintf(fd,"TraceToolbox027Calls=%d\n",(param->trace_toolbox_027==1)?1:0);
  fprintf(fd,"TraceToolbox028Calls=%d\n",(param->trace_toolbox_028==1)?1:0);
  fprintf(fd,"TraceToolbox029Calls=%d\n",(param->trace_toolbox_029==1)?1:0);
  fprintf(fd,"TraceToolbox030Calls=%d\n",(param->trace_toolbox_030==1)?1:0);
  fprintf(fd,"TraceToolbox032Calls=%d\n",(param->trace_toolbox_032==1)?1:0);
  fprintf(fd,"TraceToolbox033Calls=%d\n",(param->trace_toolbox_033==1)?1:0);
  fprintf(fd,"TraceToolbox034Calls=%d\n",(param->trace_toolbox_034==1)?1:0);
  fprintf(fd,"TraceToolbox035Calls=%d\n",(param->trace_toolbox_035==1)?1:0);
  fprintf(fd,"TraceToolbox037Calls=%d\n",(param->trace_toolbox_037==1)?1:0);
  fprintf(fd,"TraceToolbox038Calls=%d\n",(param->trace_toolbox_038==1)?1:0);
  fprintf(fd,"TraceToolbox050Calls=%d\n",(param->trace_toolbox_050==1)?1:0);
  fprintf(fd,"TraceToolbox051Calls=%d\n",(param->trace_toolbox_051==1)?1:0);
  fprintf(fd,"TraceToolbox052Calls=%d\n",(param->trace_toolbox_052==1)?1:0);
  fprintf(fd,"TraceToolbox053Calls=%d\n",(param->trace_toolbox_053==1)?1:0);
  fprintf(fd,"TraceToolbox128Calls=%d\n",(param->trace_toolbox_128==1)?1:0);
  fprintf(fd,"TraceToolbox129Calls=%d\n",(param->trace_toolbox_129==1)?1:0);
  fprintf(fd,"TraceToolbox130Calls=%d\n",(param->trace_toolbox_130==1)?1:0);
  fprintf(fd,"TraceToolbox131Calls=%d\n",(param->trace_toolbox_131==1)?1:0);
  fprintf(fd,"TraceToolbox132Calls=%d\n",(param->trace_toolbox_132==1)?1:0);
  fprintf(fd,"TraceToolbox219Calls=%d\n",(param->trace_toolbox_219==1)?1:0);
  fprintf(fd,"TraceToolbox220Calls=%d\n",(param->trace_toolbox_220==1)?1:0);
  fprintf(fd,"TraceToolbox221Calls=%d\n",(param->trace_toolbox_221==1)?1:0);
  fprintf(fd,"TraceToolbox222Calls=%d\n",(param->trace_toolbox_222==1)?1:0);
  fprintf(fd,"TraceToolbox230Calls=%d\n",(param->trace_toolbox_230==1)?1:0);
  fprintf(fd,"TraceToolbox255Calls=%d\n",(param->trace_toolbox_255==1)?1:0);
  fprintf(fd,"TraceDisplayGOID=%d\n",(param->trace_display_goid==1)?1:0);
  fprintf(fd,"TraceDisplayGCC=%d\n",(param->trace_display_gcc==1)?1:0);
  fprintf(fd,"TraceDisplayPCAddress=%d\n",(param->trace_display_pc_address==1)?1:0);
  fprintf(fd,"TraceDisplayCallNumber=%d\n",(param->trace_display_call_number==1)?1:0);

  /* Fermeture du fichier */
  fclose(fd);
}


/***********************************************************************************/
/*  LoadInitPreferenceFiles() :  On va charger en mmoire les fichiers Prfrence. */
/***********************************************************************************/
void LoadInitPreferenceFiles(void)
{
  int i, file_size, reload_at_startup;
  char *data;
  char error_buffer[1024];
  char ini_file_path[1024];
  char value[1024];
  char breakpoint_file_path[1024];
  char quickwatch_file_path[1024];
  char symbol1_file_path[1024];
  char symbol2_file_path[1024];
  char symbol3_file_path[1024];
  char symbol4_file_path[1024];
  char symbol5_file_path[1024];
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Chemin du fichier Cyrene.ini */
  GetModuleFileName(NULL,ini_file_path,MAX_PATH);
  for(i=(int)strlen(ini_file_path); i>=0; i--)
    if(ini_file_path[i] == '.')
      {
        strcpy(&ini_file_path[i],".ini");
        break;
      }

  /* Cration du contenu fichier */
  data = (char *) LoadTextFile(ini_file_path,&file_size,error_buffer);
  if(data == NULL)
    {
      /* Trace System Calls : Default Values */
      param->trace_p8_calls = 1;
      param->trace_p16_calls = 1;
      param->trace_gsos_calls = 1;
      param->trace_toolbox_calls = 1;
      param->trace_toolbox_001 = 1;    /* Tool Locator */
      param->trace_toolbox_002 = 1;    /* Memory Manager */
      param->trace_toolbox_003 = 1;    /* Miscellaneous */
      param->trace_toolbox_007 = 1;    /* Scheduler */
      param->trace_toolbox_017 = 1;    /* Load Manager */
      return;
    }

  /** Rcupration des Valeurs **/
  GetTextValue(data,"ReloadFilesAtStartup=",value);
  reload_at_startup = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"BreakpointFilePath=",breakpoint_file_path);
  GetTextValue(data,"QuickWatchFilePath=",quickwatch_file_path);
  GetTextValue(data,"Symbol1FilePath=",symbol1_file_path);
  GetTextValue(data,"Symbol2FilePath=",symbol2_file_path);
  GetTextValue(data,"Symbol3FilePath=",symbol3_file_path);
  GetTextValue(data,"Symbol4FilePath=",symbol4_file_path);
  GetTextValue(data,"Symbol5FilePath=",symbol5_file_path);
  GetTextValue(data,"TraceP8Calls=",value);
  param->trace_p8_calls = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceP16Calls=",value);
  param->trace_p16_calls = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceGSOSCalls=",value);
  param->trace_gsos_calls = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolboxCalls=",value);
  param->trace_toolbox_calls = (!stricmp(value,"1")) ? 1 : 0;

  GetTextValue(data,"IgnoreOSCalls=",value);
  param->nb_ignore_os_call = DecodeTraceCallList(value,&param->tab_ignore_os_call[0]);
  GetTextValue(data,"TraceOnlyOSCalls=",value);
  param->nb_traceonly_os_call = DecodeTraceCallList(value,&param->tab_traceonly_os_call[0]);
  GetTextValue(data,"IgnoreToolboxCalls=",value);
  param->nb_ignore_toolbox_call = DecodeTraceCallList(value,&param->tab_ignore_toolbox_call[0]);
  GetTextValue(data,"TraceOnlyToolboxCalls=",value);
  param->nb_traceonly_toolbox_call = DecodeTraceCallList(value,&param->tab_traceonly_toolbox_call[0]);

  GetTextValue(data,"TraceToolbox001Calls=",value);
  param->trace_toolbox_001 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox002Calls=",value);
  param->trace_toolbox_002 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox003Calls=",value);
  param->trace_toolbox_003 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox004Calls=",value);
  param->trace_toolbox_004 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox005Calls=",value);
  param->trace_toolbox_005 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox006Calls=",value);
  param->trace_toolbox_006 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox007Calls=",value);
  param->trace_toolbox_007 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox008Calls=",value);
  param->trace_toolbox_008 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox009Calls=",value);
  param->trace_toolbox_009 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox010Calls=",value);
  param->trace_toolbox_010 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox011Calls=",value);
  param->trace_toolbox_011 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox012Calls=",value);
  param->trace_toolbox_012 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox013Calls=",value);
  param->trace_toolbox_013 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox014Calls=",value);
  param->trace_toolbox_014 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox015Calls=",value);
  param->trace_toolbox_015 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox016Calls=",value);
  param->trace_toolbox_016 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox017Calls=",value);
  param->trace_toolbox_017 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox018Calls=",value);
  param->trace_toolbox_018 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox019Calls=",value);
  param->trace_toolbox_019 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox020Calls=",value);
  param->trace_toolbox_020 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox021Calls=",value);
  param->trace_toolbox_021 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox022Calls=",value);
  param->trace_toolbox_022 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox023Calls=",value);
  param->trace_toolbox_023 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox024Calls=",value);
  param->trace_toolbox_024 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox025Calls=",value);
  param->trace_toolbox_025 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox026Calls=",value);
  param->trace_toolbox_026 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox027Calls=",value);
  param->trace_toolbox_027 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox028Calls=",value);
  param->trace_toolbox_028 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox029Calls=",value);
  param->trace_toolbox_029 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox030Calls=",value);
  param->trace_toolbox_030 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox032Calls=",value);
  param->trace_toolbox_032 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox033Calls=",value);
  param->trace_toolbox_033 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox034Calls=",value);
  param->trace_toolbox_034 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox035Calls=",value);
  param->trace_toolbox_035 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox037Calls=",value);
  param->trace_toolbox_037 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox038Calls=",value);
  param->trace_toolbox_038 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox050Calls=",value);
  param->trace_toolbox_050 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox051Calls=",value);
  param->trace_toolbox_051 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox052Calls=",value);
  param->trace_toolbox_052 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox053Calls=",value);
  param->trace_toolbox_053 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox128Calls=",value);
  param->trace_toolbox_128 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox129Calls=",value);
  param->trace_toolbox_129 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox130Calls=",value);
  param->trace_toolbox_130 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox131Calls=",value);
  param->trace_toolbox_131 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox132Calls=",value);
  param->trace_toolbox_132 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox219Calls=",value);
  param->trace_toolbox_219 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox220Calls=",value);
  param->trace_toolbox_220 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox221Calls=",value);
  param->trace_toolbox_221 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox222Calls=",value);
  param->trace_toolbox_222 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox230Calls=",value);
  param->trace_toolbox_230 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceToolbox255Calls=",value);
  param->trace_toolbox_255 = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceDisplayGOID=",value);
  param->trace_display_goid = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceDisplayGCC=",value);
  param->trace_display_gcc = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceDisplayPCAddress=",value);
  param->trace_display_pc_address = (!stricmp(value,"1")) ? 1 : 0;
  GetTextValue(data,"TraceDisplayCallNumber=",value);
  param->trace_display_call_number = (!stricmp(value,"1")) ? 1 : 0;

  /* Libration mmoire */
  free(data);

  /* Rien  charger */
  if(reload_at_startup == 0)
    return;

  /** Charge les fichiers de valeurs **/
  /* Symbol */
  if(strlen(symbol1_file_path) > 0)
    LoadAppendSymbolFile(symbol1_file_path,NULL);
  if(strlen(symbol2_file_path) > 0)
    LoadAppendSymbolFile(symbol2_file_path,NULL);
  if(strlen(symbol3_file_path) > 0)
    LoadAppendSymbolFile(symbol3_file_path,NULL);
  if(strlen(symbol4_file_path) > 0)
    LoadAppendSymbolFile(symbol4_file_path,NULL);
  if(strlen(symbol5_file_path) > 0)
    LoadAppendSymbolFile(symbol5_file_path,NULL);

  /* Breakpoint */
  if(strlen(breakpoint_file_path) > 0)
    LoadAppendBreakpointFile(breakpoint_file_path,NULL);

  /* QuickWatch */
  if(strlen(quickwatch_file_path) > 0)
    LoadQuickWatchDataFile(quickwatch_file_path,NULL);

  /* MAJ de la Status Bar */
  UpdateWindowMainStatusBar();
}


/******************************************************************/
/*  DecodeTraceCallList() :  Dcode la liste des Appels Systmes. */
/******************************************************************/
static int DecodeTraceCallList(char *value_list, WORD *tab_call)
{
  int i, j, nb_call, length;
  DWORD call_number;
  char buffer[1024];

  /* Init */
  nb_call = 0;

  /* Passe en revue toutes les valeurs */
  length = (int) strlen(value_list);
  for(i=0, j=0; i<length; i++)
    {
      if(IsHexaDecimalChar(value_list[i]))
        buffer[j++] = value_list[i];
      else
        {
          /* Termine la valeur */
          buffer[j] = '\0';

          /* Valeur valide */
          if(strlen(buffer) == 2 || strlen(buffer) == 4)
            {
              sscanf(buffer,"%X",&call_number);
              tab_call[nb_call++] = (WORD) call_number;
            }

          /* Init */
          j = 0;
        }
    }

  /* Dernire Valeur ? */
  if(j > 0)
    {
      /* Termine la valeur */
      buffer[j] = '\0';

      /* Valeur valide */
      if(strlen(buffer) == 2 || strlen(buffer) == 4)
        {
          sscanf(buffer,"%X",&call_number);
          tab_call[nb_call++] = (WORD) call_number;
        }
    }

  /* Renvoi le nombre d'appels systmes du tableau */
  return(nb_call);
}


/*********************************************************************************/
/*  UpdatePreferenceControlStatus() :  MAJ du Statut des Controles System Calls. */
/*********************************************************************************/
static void UpdatePreferenceControlStatus(struct window_preference *window_preference)
{
  int i, trace_os_call;
  int tab_toolbox_id[] = 
    {
      IDC_CHECK_TOOLBOX_001, IDC_CHECK_TOOLBOX_002, IDC_CHECK_TOOLBOX_003, IDC_CHECK_TOOLBOX_004, IDC_CHECK_TOOLBOX_005,
      IDC_CHECK_TOOLBOX_006, IDC_CHECK_TOOLBOX_007, IDC_CHECK_TOOLBOX_008, IDC_CHECK_TOOLBOX_009, IDC_CHECK_TOOLBOX_010,
      IDC_CHECK_TOOLBOX_011, IDC_CHECK_TOOLBOX_012, IDC_CHECK_TOOLBOX_013, IDC_CHECK_TOOLBOX_014, IDC_CHECK_TOOLBOX_015, 
      IDC_CHECK_TOOLBOX_016, IDC_CHECK_TOOLBOX_017, IDC_CHECK_TOOLBOX_018, IDC_CHECK_TOOLBOX_019, IDC_CHECK_TOOLBOX_020, 
      IDC_CHECK_TOOLBOX_021, IDC_CHECK_TOOLBOX_022, IDC_CHECK_TOOLBOX_023, IDC_CHECK_TOOLBOX_024, IDC_CHECK_TOOLBOX_025, 
      IDC_CHECK_TOOLBOX_026, IDC_CHECK_TOOLBOX_027, IDC_CHECK_TOOLBOX_028, IDC_CHECK_TOOLBOX_029, IDC_CHECK_TOOLBOX_030, 
      IDC_CHECK_TOOLBOX_032, IDC_CHECK_TOOLBOX_033, IDC_CHECK_TOOLBOX_034, IDC_CHECK_TOOLBOX_035, IDC_CHECK_TOOLBOX_037, 
      IDC_CHECK_TOOLBOX_038, IDC_CHECK_TOOLBOX_050, IDC_CHECK_TOOLBOX_051, IDC_CHECK_TOOLBOX_052, IDC_CHECK_TOOLBOX_053,
      IDC_CHECK_TOOLBOX_128, IDC_CHECK_TOOLBOX_129, IDC_CHECK_TOOLBOX_130, IDC_CHECK_TOOLBOX_131, IDC_CHECK_TOOLBOX_132,
      IDC_CHECK_TOOLBOX_219, IDC_CHECK_TOOLBOX_220, IDC_CHECK_TOOLBOX_221, IDC_CHECK_TOOLBOX_222, IDC_CHECK_TOOLBOX_230, 
      IDC_CHECK_TOOLBOX_255, 0
    };
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Trace GS/OS Calls **/
  trace_os_call = param->trace_p8_calls + param->trace_p16_calls + param->trace_gsos_calls;
  EnableWindow(GetDlgItem(window_preference->hwnd_preference,IDC_STATIC_IGNORE_OS_CALLS),(trace_os_call == 0)?FALSE:TRUE);
  EnableWindow(GetDlgItem(window_preference->hwnd_preference,IDC_EDIT_IGNORE_OS_CALLS),(trace_os_call == 0)?FALSE:TRUE);
  EnableWindow(GetDlgItem(window_preference->hwnd_preference,IDC_EDIT_TRACE_OS_CALLS),(trace_os_call == 0)?FALSE:TRUE);
  EnableWindow(GetDlgItem(window_preference->hwnd_preference,IDC_STATIC_TRACE_OS_CALLS),(trace_os_call == 0)?FALSE:TRUE);

  /** Trace ToolBox Calls **/
  EnableWindow(GetDlgItem(window_preference->hwnd_preference,IDC_STATIC_TOOLBOX_CALLS),(param->trace_toolbox_calls == 0)?FALSE:TRUE);
  EnableWindow(GetDlgItem(window_preference->hwnd_preference,IDC_STATIC_IGNORE_TOOLBOX_CALLS),(param->trace_toolbox_calls == 0)?FALSE:TRUE);
  EnableWindow(GetDlgItem(window_preference->hwnd_preference,IDC_STATIC_TRACE_TOOLBOX_CALLS),(param->trace_toolbox_calls == 0)?FALSE:TRUE);
  EnableWindow(GetDlgItem(window_preference->hwnd_preference,IDC_EDIT_IGNORE_TOOLBOX_CALLS),(param->trace_toolbox_calls == 0)?FALSE:TRUE);
  EnableWindow(GetDlgItem(window_preference->hwnd_preference,IDC_EDIT_TRACE_TOOLBOX_CALLS),(param->trace_toolbox_calls == 0)?FALSE:TRUE);
  for(i=0; tab_toolbox_id[i] != 0; i++)
    EnableWindow(GetDlgItem(window_preference->hwnd_preference,tab_toolbox_id[i]),(param->trace_toolbox_calls == 0)?FALSE:TRUE);
}


/************************************************************************/
/*  DeletePreferenceChildWindows() :  Destruction des fentres enfants. */
/************************************************************************/
static void DeletePreferenceChildWindows(struct window_preference *window_preference)
{
}


/**************************************************************************/
/*  mem_free_window_preference() :  Libration mmoire pour la structure. */
/**************************************************************************/
void mem_free_window_preference(struct window_preference *window_preference)
{
  if(window_preference)
    {      
      free(window_preference);
    }
}

/************************************************************************/
