/************************************************************************/
/*                                                                      */
/*  Window_Processor.c :  Module de gestion de la fentre Processor.    */
/*                                                                      */
/************************************************************************/
/*  Auteur : Olivier ZARDINI  *   Brutal Deluxe   *   Date : Juin 2018  */
/************************************************************************/

#include <stdio.h>
#include <time.h>

#include <windows.h>
#include <windowsx.h>
#include <Commctrl.h>
#include <Winuser.h>
#include <Richedit.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_Notification.h"
#include "Dc_Graphic.h"
#include "Dc_CPU.h"
#include "Dc_MMU.h"
#include "Window_QuickWatch.h"
#include "Window_Main.h"
#include "Window_Main_Menu.h"
#include "Window_Processor.h"

static DWORD BuildProcessorWindowThread(void *);
LRESULT CALLBACK ProcHandlerProcessorWindow(HWND,UINT,WPARAM,LPARAM);
static int CreateProcessorChildWindows(HWND,struct window_processor *);
static void UpdateProcessorRefreshMode(struct window_processor *,int);
static void RefreshProcessorWindowData(struct window_processor *);
static int IsProcessorDataModified(struct window_processor *);
static void BuildProcessorFromData(struct window_processor *);
static void FormatWordAsTextInfo(WORD,int,char *);
static void DeleteProcessorChildWindows(struct window_processor *);


/********************************************************************************/
/*  BuildProcessorWindow() :  Construction / Affichage de la fentre Processor. */
/********************************************************************************/
struct window_processor *BuildProcessorWindow(HINSTANCE hInstance, int menu_id)
{
  struct window_processor *window_processor;

  /** Allocation de la structure window_processor **/
  window_processor = (struct window_processor *) calloc(1,sizeof(struct window_processor));
  if(window_processor == NULL)
    return(NULL);
  my_Memory(MEMORY_SET_WINDOW_PROCESSOR,window_processor,NULL);

  /* Lance un thread pour grer la fentre Processor */
  CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)BuildProcessorWindowThread,window_processor,0,NULL);

  /* Renvoi la structure mmoire grant la fentre */
  return(window_processor);
}


/******************************************************************************************/
/*  BuildProcessorWindowThread() :  Cration de la fentre Processor via un Thread ddi. */
/******************************************************************************************/
static DWORD BuildProcessorWindowThread(void *window_processor_param)
{
  HWND hwndProcessor;
  MSG msg;
  BOOL bRet;
  struct window_main *window_main;
  struct window_processor *window_processor;
  struct parameter *param;

  /* Init */
  window_processor = (struct window_processor *) window_processor_param;
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Cration de la fentre Processor **/
  hwndProcessor = CreateDialog(param->hInstance,MAKEINTRESOURCE(IDD_WINDOW_PROCESSOR),window_main->hwnd_main,(DLGPROC)ProcHandlerProcessorWindow);
  if(hwndProcessor == NULL)
    return(1);

  /** Show Window + Paint Content **/
  ShowWindow(hwndProcessor,SW_SHOWDEFAULT);
  UpdateWindow(hwndProcessor);

  /** Message Loop pour toutes les fentres de ce Thread (fentre Processor...) **/
  while((bRet = GetMessage(&msg,NULL,0,0)) != 0)
    {
      if(bRet == -1)
        break;

      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }

  /* Fin du Thread */
  ExitThread(0);
}


/**************************************************************************/
/*  ProcHandlerProcessorWindow() :  Gestion des vnements de la fentre. */
/**************************************************************************/
LRESULT CALLBACK ProcHandlerProcessorWindow(HWND hwndProcessor, UINT umsg, WPARAM wParam, LPARAM lParam)
{
  struct window_processor *window_processor;
  struct parameter *param;
  
  /* Init */
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  my_Memory(MEMORY_GET_WINDOW_PROCESSOR,&window_processor,NULL);

  switch(umsg)
    {
      case WM_INITDIALOG :
        /* Cration du contenu de la DialogBox */
        CreateProcessorChildWindows(hwndProcessor,window_processor);
        return(TRUE);
        
      /****************************/
      /* Evnements de la fentre */
      /****************************/
      case WM_COMMAND :
        switch(LOWORD(wParam))
          {
            /** Bouton Radio : On change la mthode de rafraichissement de la fentre **/
            case IDC_RADIO_REFRESH_AUTOMATIC :
              if(HIWORD(wParam) == BN_CLICKED && window_processor->refresh_mode != PROCESSOR_REFRESH_AUTOMATIC)
                UpdateProcessorRefreshMode(window_processor,PROCESSOR_REFRESH_AUTOMATIC);
              break;

            case IDC_RADIO_REFRESH_MANUAL :
              if(HIWORD(wParam) == BN_CLICKED && window_processor->refresh_mode != PROCESSOR_REFRESH_MANUAL)
                UpdateProcessorRefreshMode(window_processor,PROCESSOR_REFRESH_MANUAL);
              break;

            /** Bouton de rafraichissement Manuel de la fentre **/
            case IDC_BUTTON_REFRESH :
              RefreshProcessorWindowData(window_processor);
              break;
          }
        return(TRUE);

      /*****************************************************************/
      /* Message extrieur demandant le rafraichissement de la fentre */
      /*****************************************************************/
      case HM_PROCESSOR_REFRESHWINDOW :
        RefreshProcessorWindowData(window_processor);
        return(TRUE);

      /********************************************************************/
      /* Fermeture de window (le WM_CLOSE est appel avant le WM_DESTROY) */
      /********************************************************************/
      case WM_CLOSE :
        /* Fentre Processor */
        DestroyWindow(hwndProcessor);
        return(0);

      case WM_DESTROY :
        /* Suppression des Fentres */
        DeleteProcessorChildWindows(window_processor);
        CheckMainWindowMenuItem(ID_MENU_VIEW_PROCESSOR,FALSE);
        my_Memory(MEMORY_FREE_WINDOW_PROCESSOR,NULL,NULL);
        /* Demande la sortie de la boucle des messages */
        PostQuitMessage(0);
        return(0);
    }

  /* Retourner FALSE pour relayer le message non trait au systme */
  return(FALSE);
}


/*********************************************************************************************************/
/*  CreateProcessorChildWindows() : Crer les fentres enfants et remplir la structure window_processor. */
/*********************************************************************************************************/
static int CreateProcessorChildWindows(HWND hwndProcessor, struct window_processor *window_processor)
{
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Window Handle **/
  window_processor->hwnd_processor = hwndProcessor;

  /* Refresh Mode : Automatic */
  UpdateProcessorRefreshMode(window_processor,PROCESSOR_REFRESH_AUTOMATIC);

  /** Cration de la Police de caractre + Affectation de la police au Control Edit **/
  window_processor->hFont = CreateFont(PROCESSOR_EDIT_FONT_HEIGHT,0,0,0,FW_DONTCARE,FALSE,FALSE,FALSE,ANSI_CHARSET,OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY,DEFAULT_PITCH|FF_SWISS,TEXT("Courier New"));
  SendMessage(GetDlgItem(hwndProcessor,IDC_EDIT_PROC_A),WM_SETFONT,(WPARAM)window_processor->hFont,MAKELPARAM(FALSE,0));
  SendMessage(GetDlgItem(hwndProcessor,IDC_EDIT_PROC_X),WM_SETFONT,(WPARAM)window_processor->hFont,MAKELPARAM(FALSE,0));
  SendMessage(GetDlgItem(hwndProcessor,IDC_EDIT_PROC_Y),WM_SETFONT,(WPARAM)window_processor->hFont,MAKELPARAM(FALSE,0));
  SendMessage(GetDlgItem(hwndProcessor,IDC_EDIT_PROC_DBR),WM_SETFONT,(WPARAM)window_processor->hFont,MAKELPARAM(FALSE,0));
  SendMessage(GetDlgItem(hwndProcessor,IDC_EDIT_PROC_PC),WM_SETFONT,(WPARAM)window_processor->hFont,MAKELPARAM(FALSE,0));
  SendMessage(GetDlgItem(hwndProcessor,IDC_EDIT_PROC_STACK),WM_SETFONT,(WPARAM)window_processor->hFont,MAKELPARAM(FALSE,0));
  SendMessage(GetDlgItem(hwndProcessor,IDC_EDIT_PROC_DPAGE),WM_SETFONT,(WPARAM)window_processor->hFont,MAKELPARAM(FALSE,0));
  SendMessage(GetDlgItem(hwndProcessor,IDC_EDIT_STATUS_C),WM_SETFONT,(WPARAM)window_processor->hFont,MAKELPARAM(FALSE,0));
  SendMessage(GetDlgItem(hwndProcessor,IDC_EDIT_STATUS_Z),WM_SETFONT,(WPARAM)window_processor->hFont,MAKELPARAM(FALSE,0));
  SendMessage(GetDlgItem(hwndProcessor,IDC_EDIT_STATUS_I),WM_SETFONT,(WPARAM)window_processor->hFont,MAKELPARAM(FALSE,0));
  SendMessage(GetDlgItem(hwndProcessor,IDC_EDIT_STATUS_D),WM_SETFONT,(WPARAM)window_processor->hFont,MAKELPARAM(FALSE,0));
  SendMessage(GetDlgItem(hwndProcessor,IDC_EDIT_STATUS_X),WM_SETFONT,(WPARAM)window_processor->hFont,MAKELPARAM(FALSE,0));
  SendMessage(GetDlgItem(hwndProcessor,IDC_EDIT_STATUS_M),WM_SETFONT,(WPARAM)window_processor->hFont,MAKELPARAM(FALSE,0));
  SendMessage(GetDlgItem(hwndProcessor,IDC_EDIT_STATUS_V),WM_SETFONT,(WPARAM)window_processor->hFont,MAKELPARAM(FALSE,0));
  SendMessage(GetDlgItem(hwndProcessor,IDC_EDIT_STATUS_N),WM_SETFONT,(WPARAM)window_processor->hFont,MAKELPARAM(FALSE,0));
  SendMessage(GetDlgItem(hwndProcessor,IDC_EDIT_STATUS_E),WM_SETFONT,(WPARAM)window_processor->hFont,MAKELPARAM(FALSE,0));
  SendMessage(GetDlgItem(hwndProcessor,IDC_EDIT_STATUS_B),WM_SETFONT,(WPARAM)window_processor->hFont,MAKELPARAM(FALSE,0));

  /** Construction des Data + Rafraichissement de la fentre **/
  RefreshProcessorWindowData(window_processor);

  /* OK */
  return(0);
}


/**********************************************************************************************/
/*  UpdateProcessorRefreshMode() :  MAJ du mode de rafraichissement de la fenetre Processor. */
/**********************************************************************************************/
static void UpdateProcessorRefreshMode(struct window_processor *window_processor, int refresh_mode)
{
  struct notification *current_notification;

  if(refresh_mode == PROCESSOR_REFRESH_AUTOMATIC)
    {
      /* Visuel */
      window_processor->refresh_mode = PROCESSOR_REFRESH_AUTOMATIC;
      SendDlgItemMessage(window_processor->hwnd_processor,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_processor->hwnd_processor,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,0,0);

      /* Cration d'une Notification */
      current_notification = (struct notification *) calloc(1,sizeof(struct notification));
      if(current_notification == NULL)
        return;
      current_notification->hwnd_window = window_processor->hwnd_processor;
      current_notification->message = HM_PROCESSOR_REFRESHWINDOW;
      current_notification->type = NOTIFICATION_TYPE_VIEW;
      current_notification->sub_type = NOTIFICATION_SUBTYPE_PROCESSOR;

      /* Ajout de la notification */
      window_processor->current_notification = current_notification;
      my_Notification(NOTIFICATION_ADD,current_notification,NULL);
    }
  else
    {
      /* Visuel */
      window_processor->refresh_mode = PROCESSOR_REFRESH_MANUAL;
      SendDlgItemMessage(window_processor->hwnd_processor,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_processor->hwnd_processor,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,1,0);

      /* Supprime la Notification */
      my_Notification(NOTIFICATION_REMOVE,window_processor->current_notification,NULL);
      window_processor->current_notification = NULL;
    }
}


/****************************************************************************/
/*  RefreshProcessorWindowData() :  MAJ du contenu de la fentre Processor. */
/****************************************************************************/
static void RefreshProcessorWindowData(struct window_processor *window_processor)
{
  int is_modified;

  /** Les donnes ont t'elles t modifies + Rcupration des nouvelles Data si besoin **/
  is_modified = IsProcessorDataModified(window_processor);
  if(is_modified == 0)
    return;

  /** MAJ des Control Edit **/
  BuildProcessorFromData(window_processor);
}


/***********************************************************************************************************/
/*  IsProcessorDataModified() :  Les donnes Processor ont t'elles t modifies depuis la dernirer fois. */
/***********************************************************************************************************/
static int IsProcessorDataModified(struct window_processor *window_processor)
{
  WORD proc_a, proc_x, proc_y;
  BYTE proc_dbr, proc_pbr;
  WORD proc_pc, proc_stack, proc_directpage;
  int status_c, status_z, status_i, status_d, status_x, status_m, status_v, status_n;
  int status_e, status_b;
  BYTE directpage_stack_bank;    /* $00/ ou $01/ */  
  struct cpu *current_cpu;

  /* Rcupre le CPU */
  my_Memory(MEMORY_GET_S65816_CPU,&current_cpu,NULL);
  
  /** Rcupre les donnes **/
  proc_a = current_cpu->proc_a;
  proc_x = current_cpu->proc_x;
  proc_y = current_cpu->proc_y;
  proc_dbr = current_cpu->proc_dbr;
  proc_pbr = current_cpu->proc_pbr;
  proc_pc = current_cpu->proc_pc;
  proc_stack = current_cpu->proc_stack;
  proc_directpage = current_cpu->proc_directpage;
  status_c = current_cpu->status_c;
  status_z = current_cpu->status_z;
  status_i = current_cpu->status_i;
  status_d = current_cpu->status_d;
  status_x = current_cpu->status_x;
  status_m = current_cpu->status_m;
  status_v = current_cpu->status_v;
  status_n = current_cpu->status_n;
  status_e = current_cpu->status_e;
  status_b = current_cpu->status_b;
  directpage_stack_bank = current_cpu->directpage_stack_bank;
  
  /** Compare les donnes **/
  if(proc_a == window_processor->proc_a &&
     proc_x == window_processor->proc_x &&
     proc_y == window_processor->proc_y &&
     proc_dbr == window_processor->proc_dbr &&
     proc_pbr == window_processor->proc_pbr &&
     proc_pc == window_processor->proc_pc &&
     proc_stack == window_processor->proc_stack &&
     proc_directpage == window_processor->proc_directpage &&
     status_c == window_processor->status_c &&
     status_z == window_processor->status_z &&
     status_i == window_processor->status_i &&
     status_d == window_processor->status_d &&
     status_x == window_processor->status_x &&
     status_m == window_processor->status_m &&
     status_v == window_processor->status_v &&
     status_n == window_processor->status_n &&
     status_e == window_processor->status_e &&
     status_b == window_processor->status_b &&
     directpage_stack_bank == window_processor->directpage_stack_bank)
    return(0);    /* Donnes non modifies */
    
  /** Les donnes sont diffrentes **/
  window_processor->proc_a = proc_a;
  window_processor->proc_x = proc_x;
  window_processor->proc_y = proc_y;
  window_processor->proc_dbr = proc_dbr;
  window_processor->proc_pbr = proc_pbr;
  window_processor->proc_pc = proc_pc;
  window_processor->proc_stack = proc_stack;
  window_processor->proc_directpage = proc_directpage;
  window_processor->status_c = status_c;
  window_processor->status_z = status_z;
  window_processor->status_i = status_i;
  window_processor->status_d = status_d;
  window_processor->status_x = status_x;
  window_processor->status_m = status_m;
  window_processor->status_v = status_v;
  window_processor->status_n = status_n;
  window_processor->status_e = status_e;
  window_processor->status_b = status_b;
  window_processor->directpage_stack_bank = directpage_stack_bank;

  /* Donnes modifies */
  return(1);
}


/***********************************************************************************************************/
/*  BuildProcessorFromData() :  Construire les donnes hxadcimal  afficher  partir des donnes brutes. */
/***********************************************************************************************************/
static void BuildProcessorFromData(struct window_processor *window_processor)
{
  char buffer[BUFFER_SIZE];

  /** Afficher les Data dans le control **/
  /* A */
  FormatWordAsTextInfo(window_processor->proc_a,window_processor->status_m,buffer);
  SetWindowText(GetDlgItem(window_processor->hwnd_processor,IDC_EDIT_PROC_A),buffer);
  /* X */
  FormatWordAsTextInfo(window_processor->proc_x,window_processor->status_x,buffer);
  SetWindowText(GetDlgItem(window_processor->hwnd_processor,IDC_EDIT_PROC_X),buffer);
  /* Y */
  FormatWordAsTextInfo(window_processor->proc_y,window_processor->status_x,buffer);
  SetWindowText(GetDlgItem(window_processor->hwnd_processor,IDC_EDIT_PROC_Y),buffer);
  /* DBR */
  sprintf(buffer,"%02X",window_processor->proc_dbr);
  SetWindowText(GetDlgItem(window_processor->hwnd_processor,IDC_EDIT_PROC_DBR),buffer);
  /* PBR + PC */
  sprintf(buffer,"%02X/%04X",window_processor->proc_pbr,window_processor->proc_pc);
  SetWindowText(GetDlgItem(window_processor->hwnd_processor,IDC_EDIT_PROC_PC),buffer);
  /* Stack */
  sprintf(buffer,"%02X/%04X",window_processor->directpage_stack_bank,window_processor->proc_stack);
  SetWindowText(GetDlgItem(window_processor->hwnd_processor,IDC_EDIT_PROC_STACK),buffer);
  /* Direct Page */
  sprintf(buffer,"%02X/%04X",window_processor->directpage_stack_bank,window_processor->proc_directpage);
  SetWindowText(GetDlgItem(window_processor->hwnd_processor,IDC_EDIT_PROC_DPAGE),buffer);
    
  /* C */
  sprintf(buffer,"%d",window_processor->status_c);
  SetWindowText(GetDlgItem(window_processor->hwnd_processor,IDC_EDIT_STATUS_C),buffer);
  /* Z */
  sprintf(buffer,"%d",window_processor->status_z);
  SetWindowText(GetDlgItem(window_processor->hwnd_processor,IDC_EDIT_STATUS_Z),buffer);
  /* I */
  sprintf(buffer,"%d",window_processor->status_i);
  SetWindowText(GetDlgItem(window_processor->hwnd_processor,IDC_EDIT_STATUS_I),buffer);
  /* D */
  sprintf(buffer,"%d",window_processor->status_d);
  SetWindowText(GetDlgItem(window_processor->hwnd_processor,IDC_EDIT_STATUS_D),buffer);
  /* X */
  sprintf(buffer,"%d",window_processor->status_x);
  SetWindowText(GetDlgItem(window_processor->hwnd_processor,IDC_EDIT_STATUS_X),buffer);
  /* M */
  sprintf(buffer,"%d",window_processor->status_m);
  SetWindowText(GetDlgItem(window_processor->hwnd_processor,IDC_EDIT_STATUS_M),buffer);
  /* V */
  sprintf(buffer,"%d",window_processor->status_v);
  SetWindowText(GetDlgItem(window_processor->hwnd_processor,IDC_EDIT_STATUS_V),buffer);
  /* N */
  sprintf(buffer,"%d",window_processor->status_n);
  SetWindowText(GetDlgItem(window_processor->hwnd_processor,IDC_EDIT_STATUS_N),buffer);
  /* E */
  sprintf(buffer,"%d",window_processor->status_e);
  SetWindowText(GetDlgItem(window_processor->hwnd_processor,IDC_EDIT_STATUS_E),buffer);
  /* B */
  sprintf(buffer,"%d",window_processor->status_b);
  SetWindowText(GetDlgItem(window_processor->hwnd_processor,IDC_EDIT_STATUS_B),buffer);
}


/***********************************************************************/
/*  FormatWordAsTextInfo() :  Convetit un WORD en buffer Text complet. */
/***********************************************************************/
static void FormatWordAsTextInfo(WORD one_word, int register_size, char *output_buffer)
{
  BYTE one_byte, extra_byte;
  int i, value, offset;
  char buffer_hexa[16];
  char buffer_binary[32];
  char buffer_uint[16];
  char buffer_sint[16];

  /** 8 / 16 bits **/
  if(register_size == 0)
    {
      /* Hexa */
      sprintf(buffer_hexa,"%04X",one_word);

      /* Binary */
      strcpy(buffer_binary,"0000.0000.0000.0000");
      for(i=0,offset=18; i<16; i++)
        {
          /* Le 0 est dj en place => one ne et que les 1 */
          if(((one_word >> i) & 0x0001) == 0x0001)
            buffer_binary[offset] = '1';

          /* Dcale */
          offset--;

          /* Gre les espaces */
          if(i == 3 || i == 7 || i == 11)
            offset--;
        }

      /* Unsigned Integer */
      sprintf(buffer_uint,"%5d",one_word);

      /* Signed Integer */
      if((one_word & 0x8000) == 0x8000)
        {
          value = (0x10000 - one_word);
          sprintf(buffer_sint,"%6d",(-1)*value);
        }
      else
        sprintf(buffer_sint,"%6d",one_word);

      /** Buffer Final **/
      sprintf(output_buffer,"   %s   %s   %s  %s",buffer_hexa,buffer_binary,buffer_uint,buffer_sint);
    }
  else    /* 8 bits */
    {
      /* Byte */
      one_byte = (BYTE) (one_word & 0x00FF);
      extra_byte = (BYTE) ((one_word >> 8) & 0x00FF);

      /* Hexa */
      sprintf(buffer_hexa,"[%02X] %02X",extra_byte,one_byte);

      /* Binary */
      strcpy(buffer_binary,"0000.0000");
      for(i=0,offset=8; i<8; i++)
        {
          /* Le 0 est dj en place => one ne et que les 1 */
          if(((one_byte >> i) & 0x01) == 0x01)
            buffer_binary[offset] = '1';

          /* Dcale */
          offset--;

          /* Gre les espaces */
          if(i == 3)
            offset--;
        }

      /* Unsigned Integer */
      sprintf(buffer_uint,"%3d",one_byte);

      /* Signed Integer */
      if((one_byte & 0x80) == 0x80)
        {
          value = (0x100 - one_byte);
          sprintf(buffer_sint,"%4d",(-1)*value);
        }
      else
        sprintf(buffer_sint,"%4d",one_byte);

      /** Buffer Final **/
      sprintf(output_buffer,"%s             %s     %s    %s",buffer_hexa,buffer_binary,buffer_uint,buffer_sint);
    }
}


/***********************************************************************/
/*  DeleteProcessorChildWindows() :  Destruction des fentres enfants. */
/***********************************************************************/
static void DeleteProcessorChildWindows(struct window_processor *window_processor)
{
}


/*************************************************************************/
/*  mem_free_window_processor() :  Libration mmoire pour la structure. */
/*************************************************************************/
void mem_free_window_processor(struct window_processor *window_processor)
{
  if(window_processor)
    {
      /* Supprime la notification */
      my_Notification(NOTIFICATION_REMOVE,window_processor->current_notification,NULL);
      window_processor->current_notification = NULL;
      
      free(window_processor);
    }
}

/**********************************************************************/
