/***********************************************************************/
/*                                                                     */
/*  Window_Profile.c :  Module de gestion de la fentre Profile.       */
/*                                                                     */
/***********************************************************************/
/*  Auteur : Olivier ZARDINI  *   Brutal Deluxe   *   Date : May 2024  */
/***********************************************************************/

#include <stdio.h>
#include <time.h>

#include <windows.h>
#include <windowsx.h>
#include <Commctrl.h>
#include <Winuser.h>
#include <Richedit.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_Notification.h"
#include "Dc_Graphic.h"
#include "Dc_CPU.h"
#include "Dc_MMU.h"
#include "Dc_IIgsInternals.h"
#include "Dc_Disassembler.h"
#include "Dc_Name.h"
#include "Cyrene_Engine.h"
#include "Window_QuickWatch.h"
#include "Window_Main.h"
#include "Window_Main_Menu.h"
#include "Window_Disassembler.h"
#include "Window_MemoryArea.h"
#include "Window_Symbol.h"
#include "Window_Profile.h"

#define PROFILER_EXTRA_SIZE    8         /* Taille  ajouter au dbut de chaque colonne */

static DWORD BuildProfileWindowThread(void *);
LRESULT CALLBACK ProcHandlerProfileWindow(HWND,UINT,WPARAM,LPARAM);
static int CreateProfileChildWindows(HWND,struct window_profile *);
static void UpdateProfileRefreshMode(struct window_profile *,int);
static void BuildProfileListView(struct list_view *,HFONT,int,struct window_profile *);
static void FillProfileListView(struct list_view *,struct profile_table *);
static void RefreshProfileWindow(struct window_profile *);
static void DeleteProfileFunction(struct window_profile *);
static void AddProfileFirstFunction(void);
static void AddProfileFunction(int);
static void ExitProfileFunction(struct profile_stack *);
static struct profile_function *mem_alloc_profile_function(struct profile_stack *,int,int);
static int IsSameProfileFunction(struct profile_function *,DWORD,WORD);
static void UpdateProfileHideInterrupt(struct window_profile *);
static void BuildProfileFunctionCycleCount(void);
static int ClearProfileFunctionCycleCount(struct profile_function *);
static void FillProfileFunctionCycleCountTab(struct profile_function *,int,struct profile_function **);
static void BuildProfileFunctionCurrentPath(struct window_profile *);
static void ClearProfileFunctionCurrentPath(struct profile_function *);
static void SaveProfilerFile(struct window_profile *);
static void ComputeProfileFileCoordinate(struct profile_function *,int,int,int,struct profile_table *,struct window_profile *);
static void ComputeProfileFileWidth(struct profile_function *,int,struct profile_table *);
static void ComputeProfileFileFill(struct profile_function *,int,int,char *,int,struct profile_table *);
static void UserResizeProfileWindow(struct window_profile *);
static void MoveProfileWindowControls(int,struct window_profile *);
static void DeleteProfileChildWindows(struct window_profile *);
static int compare_function(const void *,const void *);
static void mem_free_profile_function(struct profile_function *);


/****************************************************************************/
/*  BuildProfileWindow() :  Construction / Affichage de la fentre Profile. */
/****************************************************************************/
struct window_profile *BuildProfileWindow(HINSTANCE hInstance, int menu_id)
{
  struct window_profile *window_profile;

  /** Allocation de la structure window_profile **/
  window_profile = (struct window_profile *) calloc(1,sizeof(struct window_profile));
  if(window_profile == NULL)
    return(NULL);
  my_Memory(MEMORY_SET_WINDOW_PROFILE,window_profile,NULL);

  /* Lance un thread pour grer la fentre Profile */
  CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)BuildProfileWindowThread,window_profile,0,NULL);

  /* Renvoi la structure mmoire grant la fentre */
  return(window_profile);
}


/**************************************************************************************/
/*  BuildProfileWindowThread() :  Cration de la fentre Profile via un Thread ddi. */
/**************************************************************************************/
static DWORD BuildProfileWindowThread(void *window_profile_param)
{
  HWND hwndProfile;
  MSG msg;
  BOOL bRet;
  struct window_main *window_main;
  struct window_profile *window_profile;
  struct parameter *param;

  /* Init */
  window_profile = (struct window_profile *) window_profile_param;
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Cration de la fentre Profile **/
  hwndProfile = CreateDialog(param->hInstance,MAKEINTRESOURCE(IDD_WINDOW_PROFILE),window_main->hwnd_main,(DLGPROC)ProcHandlerProfileWindow);
  if(hwndProfile == NULL)
    return(1);

  /** Show Window + Paint Content **/
  ShowWindow(hwndProfile,SW_SHOWDEFAULT);
  UpdateWindow(hwndProfile);

  /** Message Loop pour toutes les fentres de ce Thread (fentre Profile...) **/
  while((bRet = GetMessage(&msg,NULL,0,0)) != 0)
    {
      if(bRet == -1)
        break;

      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }

  /* Fin du Thread */
  ExitThread(0);
}


/************************************************************************/
/*  ProcHandlerProfileWindow() :  Gestion des vnements de la fentre. */
/************************************************************************/
LRESULT CALLBACK ProcHandlerProfileWindow(HWND hwndProfile, UINT umsg, WPARAM wParam, LPARAM lParam)
{
  int i, is_error, select_id;
  UINT code, code_beginlabeledit, code_endlabeledit, code_getdispinfo, code_setdispinfo;
  DWORD profile_address;
  BOOL result;
  NMLVDISPINFO *plvdi;
  LVITEM LvItem;
  NMLISTVIEW *pListView;
  char buffer_address[1024];
  struct profile_function *current_profile_function;
  struct list_view *current_listview;
  struct window_profile *window_profile;
  struct window_main *window_main;
  struct parameter *param;
  
  /* Init */
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  my_Memory(MEMORY_GET_WINDOW_PROFILE,&window_profile,NULL);
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);

  switch(umsg)
    {
      case WM_INITDIALOG :
        /* Cration du contenu de la DialogBox */
        CreateProfileChildWindows(hwndProfile,window_profile);
        return(TRUE);
        
      /****************************/
      /* Evnements de la fentre */
      /****************************/
      case WM_COMMAND :
        switch(LOWORD(wParam))
          {
            /** Bouton Radio : On change la mthode de rafraichissement de la fentre **/
            case IDC_RADIO_REFRESH_AUTOMATIC :
              if(HIWORD(wParam) == BN_CLICKED && window_profile->refresh_mode != PROFILE_REFRESH_AUTOMATIC)
                UpdateProfileRefreshMode(window_profile,PROFILE_REFRESH_AUTOMATIC);
              break;

            case IDC_RADIO_REFRESH_MANUAL :
              if(HIWORD(wParam) == BN_CLICKED && window_profile->refresh_mode != PROFILE_REFRESH_MANUAL)
                UpdateProfileRefreshMode(window_profile,PROFILE_REFRESH_MANUAL);
              break;

            /** Bouton : Vide la table des Profile **/
            case IDC_BUTTON_PROFILE_CLEAR :
              if(HIWORD(wParam) == BN_CLICKED)
                ClearProfileTable(window_profile);
              break;

            /** Bouton : Backward << **/
            case IDC_BUTTON_PROFILE_BACKWARD :
              if(HIWORD(wParam) == BN_CLICKED)
                if(window_profile->current_profile_function != NULL)
                  if(window_profile->current_profile_function->parent != NULL)
                    {
                      /* Remonte d'un cran */
                      window_profile->current_profile_function = window_profile->current_profile_function->parent;
                      RefreshProfileWindow(window_profile);
                    }
              break;

            /** Bouton : Forward >> **/
            case IDC_BUTTON_PROFILE_FORWARD :
              if(HIWORD(wParam) == BN_CLICKED)
                if(window_profile->selected_profile_function != NULL)
                  if(window_profile->selected_profile_function != window_profile->current_profile_function)
                    {
                      /* Accde  la Sous fonction */
                      window_profile->current_profile_function = window_profile->selected_profile_function;
                      RefreshProfileWindow(window_profile);
                    }
              break;

            /** Bouton de rafraichissement Manuel de la fentre **/
            case IDC_BUTTON_REFRESH :
              RefreshProfileWindow(window_profile);
              break;

            /** On va supprimer la Sous-Fonction slectionner **/
            case IDC_BUTTON_DELETE_FUNCTION :
              DeleteProfileFunction(window_profile);
              break;

            /** On va crer le fichier Profiler **/
            case IDC_BUTTON_PROFILE_SAVE :
              if(HIWORD(wParam) == BN_CLICKED)
                SaveProfilerFile(window_profile);
              break;

            /** On va masquer les Interruptions **/
            case IDC_CHECK_HIDE_INTERRUPT :
              if(HIWORD(wParam) == BN_CLICKED)
                UpdateProfileHideInterrupt(window_profile);
              break;
          }
        return(TRUE);

      /*****************************************************************/
      /* Message extrieur demandant le rafraichissement de la fentre */
      /*****************************************************************/
      case HM_PROFILE_REFRESHWINDOW :
        RefreshProfileWindow(window_profile);
        return(TRUE);

      /********************************************/
      /* Evnements des CommonControls (ListView) */
      /********************************************/
      case WM_NOTIFY :
        code = ((LPNMHDR)lParam)->code;
        code_beginlabeledit = LVN_BEGINLABELEDIT;
        code_endlabeledit   = LVN_ENDLABELEDIT;
        code_getdispinfo    = LVN_GETDISPINFO;
        code_setdispinfo    = LVN_SETDISPINFO;

        /*****************************************************************************************************/
        /**  Affichage des Item de la fentre : Remplissage des Item dans les colonnes des lignes affiches **/
        /*****************************************************************************************************/
        if(code == LVN_GETDISPINFO)
          {
            plvdi = (NMLVDISPINFO *) lParam;

            /* Identifie la ListView : Profiles */
            if(LOWORD(wParam) == window_profile->listview_profile.idc_listview)
              current_listview = (struct list_view *) &window_profile->listview_profile;
            else
              break;

            /* Valide le numro de la colonne */
            if(plvdi->item.iSubItem < current_listview->nb_column)
              {
                plvdi->item.pszText = current_listview->tab_column[plvdi->item.iSubItem].tab_value_txt[(current_listview->has_column_sort == INTERNALS_COLUMN_SORT_NONE) ? plvdi->item.iItem : plvdi->item.lParam];
              }
          }
     
        /***************************************************************/
        /**  Click sur une ligne d'une ListView pour la slectionner  **/
        /***************************************************************/
        if(code == NM_CLICK)
          {
            /* Rcupre la ListView (Profile) */
            if(LOWORD(wParam) == window_profile->listview_profile.idc_listview)
              current_listview = (struct list_view *) &window_profile->listview_profile;
            else
              break;

            /* Identifie la ligne clicke (0-N) */
            select_id = (int) SendMessage(current_listview->hwnd_listview,LVM_GETNEXTITEM,-1,LVNI_FOCUSED); // return item selected
            if(select_id == -1)
              return(TRUE);

            /* Rcupre l'adresse de la Fonction Profile dans la Colonne #0 */
            memset(&LvItem,0,sizeof(LVITEM));
            LvItem.mask = LVIF_TEXT;
            LvItem.iSubItem = 0;                             /* Colonne #0 */
            LvItem.pszText = buffer_address;
            LvItem.cchTextMax = 256;
            LvItem.iItem = select_id;
            SendMessage(current_listview->hwnd_listview,LVM_GETITEMTEXT,select_id,(LPARAM)&LvItem);
            buffer_address[strlen("00/0000")] = '\0';        /* Supprime le * du current path */
            window_profile->selected_profile_address = EvaluateAsAddress(buffer_address,&is_error);

            /* Rcupre le Call Number de la Fonction Profile dans la Colonne #1 */
            memset(&LvItem,0,sizeof(LVITEM));
            LvItem.mask = LVIF_TEXT;
            LvItem.iSubItem = 1;            /* Colonne #1 */
            LvItem.pszText = buffer_address;
            LvItem.cchTextMax = 256;
            LvItem.iItem = select_id;
            SendMessage(current_listview->hwnd_listview,LVM_GETITEMTEXT,select_id,(LPARAM)&LvItem);
            window_profile->selected_profile_call_number = EvaluateAsWord(buffer_address,&is_error);
            if(is_error == 1)
              window_profile->selected_profile_call_number = 0xFFFF;     /* Indisponible */

            /** Rcupre un pointeur vers la Profile Function slectionne **/
            window_profile->selected_profile_function = NULL;
            if(IsSameProfileFunction(window_profile->current_profile_function,window_profile->selected_profile_address,window_profile->selected_profile_call_number))
              window_profile->selected_profile_function = window_profile->current_profile_function;
            else
              {
                for(current_profile_function = window_profile->current_profile_function->first_sub_function; current_profile_function; current_profile_function = current_profile_function->next)
                  if(IsSameProfileFunction(current_profile_function,window_profile->selected_profile_address,window_profile->selected_profile_call_number))
                    {
                      window_profile->selected_profile_function = current_profile_function;
                      break;
                    }
              }

            /** MAJ Boutons << >> **/
            if(window_profile->selected_profile_function != NULL)
              {
                /* << */
                if(window_profile->current_profile_function->depth == 0)
                  EnableWindow(GetDlgItem(window_profile->hwnd_profile,IDC_BUTTON_PROFILE_BACKWARD),FALSE);
                else
                  EnableWindow(GetDlgItem(window_profile->hwnd_profile,IDC_BUTTON_PROFILE_BACKWARD),TRUE);

                /* >> */
                if(window_profile->selected_profile_function == window_profile->current_profile_function)
                  EnableWindow(GetDlgItem(window_profile->hwnd_profile,IDC_BUTTON_PROFILE_FORWARD),FALSE);
                else
                  EnableWindow(GetDlgItem(window_profile->hwnd_profile,IDC_BUTTON_PROFILE_FORWARD),TRUE);

                /* MAJ Bouton Delete */
                if(window_profile->selected_profile_function == window_profile->current_profile_function)
                  EnableWindow(GetDlgItem(window_profile->hwnd_profile,IDC_BUTTON_DELETE_FUNCTION),FALSE);
                else
                  EnableWindow(GetDlgItem(window_profile->hwnd_profile,IDC_BUTTON_DELETE_FUNCTION),TRUE);
              }
          }

        /*************************************************************************/
        /**  Double-click sur une ligne d'une ListView pour activer une action  **/
        /*************************************************************************/
        if(code == NM_DBLCLK)
          {
            /* Rcupre la ListView (Profile) */
            if(LOWORD(wParam) == window_profile->listview_profile.idc_listview)
              current_listview = (struct list_view *) &window_profile->listview_profile;
            else
              break;

            /* Identifie la ligne clicke (0-N) */
            select_id = (int) SendMessage(current_listview->hwnd_listview,LVM_GETNEXTITEM,-1,LVNI_FOCUSED); // return item selected
            if(select_id == -1)
              return(TRUE);

            /* Rcupre l'adresse du Profile dans la Colonne #0 */
            memset(&LvItem,0,sizeof(LVITEM));
            LvItem.mask = LVIF_TEXT;
            LvItem.iSubItem = 0;            /* Colonne #0 */
            LvItem.pszText = buffer_address;
            LvItem.cchTextMax = 256;
            LvItem.iItem = select_id;
            SendMessage(current_listview->hwnd_listview,LVM_GETITEMTEXT,select_id,(LPARAM)&LvItem);
            buffer_address[strlen("00/0000")] = '\0';        /* Supprime le * du current path */
            profile_address = EvaluateAsAddress(buffer_address,&is_error);

            /** On va ouvrir une fentre de dsassemblage **/
            BuildDisassemblerWindow(param->hInstance,ID_MENU_VIEW_CODEDISASSEMBLER,profile_address);
          }

        /***************************************************************/
        /**  Click sur les titres des colonnes pour activer les tris  **/
        /***************************************************************/
        if(code == LVN_COLUMNCLICK)
          {
            pListView = (NMLISTVIEW *) lParam;
            select_id = (int) pListView->iSubItem;

            /* Rcupre la ListView (Profile) */
            if(LOWORD(wParam) == window_profile->listview_profile.idc_listview)
              current_listview = (struct list_view *) &window_profile->listview_profile;
            else
              break;

            /* On fait le choix du type de tri */
            for(i=0; i<current_listview->nb_column; i++)
              {
                if(i == select_id)
                  {
                    if(current_listview->tab_column[i].sort == INTERNALS_COLUMN_SORT_NONE)
                      current_listview->tab_column[i].sort = INTERNALS_COLUMN_SORT_UP;
                    else if(current_listview->tab_column[i].sort == INTERNALS_COLUMN_SORT_UP)
                      current_listview->tab_column[i].sort = INTERNALS_COLUMN_SORT_DOWN;
                    else
                      current_listview->tab_column[i].sort = INTERNALS_COLUMN_SORT_NONE;
                  }
               else
                 current_listview->tab_column[i].sort = INTERNALS_COLUMN_SORT_NONE;
              }
             
            /* A t'on une colonne trie ? */
            current_listview->has_column_sort = INTERNALS_COLUMN_SORT_NONE;
            for(i=0; i<current_listview->nb_column; i++)
              if(current_listview->tab_column[i].sort != INTERNALS_COLUMN_SORT_NONE)
                current_listview->has_column_sort = current_listview->tab_column[i].sort;
             
            /* On va trier selon le tri mis en place */
            result = ListView_SortItems(current_listview->hwnd_listview,ProcessListViewCompareProc,current_listview);
           
            /* MAJ des images ^ / v des Tris des colonnes */
            setListViewSortIcon(current_listview);
          }

        /***********************************************************************************/
        /**  Gestion de l'affichage des Item pour dfinir une couleur de fond de cellule  **/
        /***********************************************************************************/
        if(code == NM_CUSTOMDRAW)
          {
            SetWindowLongPtr(window_profile->hwnd_profile,DWLP_MSGRESULT,(LONG_PTR)ProcessListViewWindowCustomDraw(lParam));
            return(CDRF_NOTIFYITEMDRAW);
          }
        return(TRUE);
        break;

      /**************************************/
      /*  On vient de retailler la fentre  */
      /**************************************/
      case WM_SIZE :
        /* On va tendre la fentre de donne mmoire si c'est un Resize User */
        if(window_profile->user_resize == 1)
          UserResizeProfileWindow(window_profile);
        window_profile->user_resize = 1;
        return(TRUE);

      /********************************************************************/
      /* Fermeture de window (le WM_CLOSE est appel avant le WM_DESTROY) */
      /********************************************************************/
      case WM_CLOSE :
        /* Fentre Profile */
        DestroyWindow(hwndProfile);
        return(0);

      case WM_DESTROY :
        /* Suppression des Fentres */
        DeleteProfileChildWindows(window_profile);
        CheckMainWindowMenuItem(ID_MENU_EXTRA_PROFILE,FALSE);
        my_Memory(MEMORY_FREE_WINDOW_PROFILE,NULL,NULL);
        /* Demande la sortie de la boucle des messages */
        PostQuitMessage(0);
        return(0);
    }

  /* Retourner FALSE pour relayer le message non trait au systme */
  return(FALSE);
}


/*****************************************************************************************************/
/*  CreateProfileChildWindows() : Crer les fentres enfants et remplir la structure window_profile. */
/*****************************************************************************************************/
static int CreateProfileChildWindows(HWND hwndProfile, struct window_profile *window_profile)
{
  RECT window_rect;
  struct window_main *window_main;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Rcupre la fentre Main */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);

  /** Window Handle **/
  window_profile->hwnd_profile = hwndProfile;

  /* Police de caractre de la fentre (14 points) */
  window_profile->hFont = CreateFontA(LISTVIEW_FONT_HEIGHT,0,0,0,FW_DONTCARE,FALSE,FALSE,FALSE,ANSI_CHARSET,
                                      OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY,DEFAULT_PITCH|FF_SWISS,"Arial");

  /* Refresh Mode : Automatic */
  UpdateProfileRefreshMode(window_profile,PROFILE_REFRESH_AUTOMATIC);

  /***************/
  /*** Profile ***/
  /***************/
  window_profile->profile_x0 = 7;
  window_profile->profile_y0 = 7;

  /** Cration de la List View **/
  BuildProfileListView(&window_profile->listview_profile,window_profile->hFont,IDC_LISTVIEW_PROFILE,window_profile);

  /** Mise jour des entres de la fentre Profile **/
  RefreshProfileWindow(window_profile);

  /** Remplissage des entres de la fentre Profile au niveau 0 **/
  window_profile->current_profile_function = NULL;
  RefreshProfileWindow(window_profile);

  /** Taille de la fentre au dmarrage ((avec la barre de titre et les bordures)) **/
  GetWindowRect(window_profile->hwnd_profile,&window_rect);
  window_profile->window_init_width  = window_rect.right  - window_rect.left;
  window_profile->window_init_height = window_rect.bottom - window_rect.top;

  /* Nombre de ligne dans la ListView au dmarrage de la fentre */
  window_profile->nb_init_data_row = PROFILE_LISTVIEW_NB_ROW;

  /* Nombre de lignes qu'il est possible d'afficher dans la ListView */
  window_profile->nb_row = window_profile->nb_init_data_row;

  /* OK */
  return(0);
}


/*****************************************************************************************/
/*  UpdateProfileRefreshMode() :  MAJ du mode de rafraichissement de la fentre Profile. */
/*****************************************************************************************/
static void UpdateProfileRefreshMode(struct window_profile *window_profile, int refresh_mode)
{
  struct notification *current_notification;

  if(refresh_mode == PROFILE_REFRESH_AUTOMATIC)
    {
      /* Visuel */
      window_profile->refresh_mode = PROFILE_REFRESH_AUTOMATIC;
      SendDlgItemMessage(window_profile->hwnd_profile,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_profile->hwnd_profile,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,0,0);

      /* Cration d'une Notification */
      current_notification = (struct notification *) calloc(1,sizeof(struct notification));
      if(current_notification == NULL)
        return;
      current_notification->hwnd_window = window_profile->hwnd_profile;
      current_notification->message = HM_PROFILE_REFRESHWINDOW;
      current_notification->type = NOTIFICATION_TYPE_VIEW;
      current_notification->sub_type = NOTIFICATION_SUBTYPE_PROFILE;

      /* Ajout de la notification */
      window_profile->current_notification = current_notification;
      my_Notification(NOTIFICATION_ADD,current_notification,NULL);
    }
  else
    {
      /* Visuel */
      window_profile->refresh_mode = PROFILE_REFRESH_MANUAL;
      SendDlgItemMessage(window_profile->hwnd_profile,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_profile->hwnd_profile,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,1,0);

      /* Supprime la Notification */
      my_Notification(NOTIFICATION_REMOVE,window_profile->current_notification,NULL);
      window_profile->current_notification = NULL;
    }
}


/****************************************************************************/
/*  BuildProfileListView() :  Construction d'une ListView pour les Profile. */
/****************************************************************************/
static void BuildProfileListView(struct list_view *current_listview, HFONT hFont, int idc_listview, struct window_profile *window_profile)
{
  /** Init de la structure List_view **/
  current_listview->idc_listview = idc_listview;

  /** Liste des Colonnes **/
  current_listview->nb_column = 0;

  /* Col 0 : Address */
  current_listview->tab_column[0].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[0].align = LVCFMT_RIGHT;
  current_listview->tab_column[0].length = 80;
  strcpy(current_listview->tab_column[0].title,"Address");
  current_listview->nb_column++;

  /* Col 1 : Call Number */
  current_listview->tab_column[1].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[1].align = LVCFMT_RIGHT;
  current_listview->tab_column[1].length = 60;
  strcpy(current_listview->tab_column[1].title,"Num");
  current_listview->nb_column++;

  /* Col 2 : Name */
  current_listview->tab_column[2].type = INTERNALS_COLUMN_TYPE_TXT;
  current_listview->tab_column[2].align = LVCFMT_LEFT;
  current_listview->tab_column[2].length = 140;
  strcpy(current_listview->tab_column[2].title,"Name");
  current_listview->nb_column++;

  /* Col 3 : Application */
  current_listview->tab_column[3].type = INTERNALS_COLUMN_TYPE_TXT;
  current_listview->tab_column[3].align = LVCFMT_LEFT;
  current_listview->tab_column[3].length = 220;
  strcpy(current_listview->tab_column[3].title,"Application");
  current_listview->nb_column++;

  /* Col 4 : Interrupt */
  current_listview->tab_column[4].type = INTERNALS_COLUMN_TYPE_TXT;
  current_listview->tab_column[4].align = LVCFMT_CENTER;
  current_listview->tab_column[4].length = 80;
  strcpy(current_listview->tab_column[4].title,"Interrupt");
  current_listview->nb_column++;

  /* Col 5 : Depth */
  current_listview->tab_column[5].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[5].align = LVCFMT_RIGHT;
  current_listview->tab_column[5].length = 65;
  strcpy(current_listview->tab_column[5].title,"Depth");
  current_listview->nb_column++;

  /* Col 6 : # Sub */
  current_listview->tab_column[6].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[6].align = LVCFMT_RIGHT;
  current_listview->tab_column[6].length = 65;
  strcpy(current_listview->tab_column[6].title,"# Sub");
  current_listview->nb_column++;

  /* Col 7 : # Call */
  current_listview->tab_column[7].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[7].align = LVCFMT_RIGHT;
  current_listview->tab_column[7].length = 65;
  strcpy(current_listview->tab_column[7].title,"# Call");
  current_listview->nb_column++;

  /* Col 8 : Total Cycle */
  current_listview->tab_column[8].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[8].align = LVCFMT_RIGHT;
  current_listview->tab_column[8].length = 90;
  strcpy(current_listview->tab_column[8].title,"Total Cycle");
  current_listview->nb_column++;

  /* Col 9 : % */
  current_listview->tab_column[9].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[9].align = LVCFMT_RIGHT;
  current_listview->tab_column[9].length = 50;
  strcpy(current_listview->tab_column[9].title,"%");
  current_listview->nb_column++;

  /* Col 10 : Avg Cycle */
  current_listview->tab_column[10].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[10].align = LVCFMT_RIGHT;
  current_listview->tab_column[10].length = 90;
  strcpy(current_listview->tab_column[10].title,"Avg Cycle");
  current_listview->nb_column++;

  /** Cration du ListView (20 lignes) **/
  BuildListView(window_profile->hwnd_profile,hFont,window_profile->profile_x0,window_profile->profile_y0,PROFILE_LISTVIEW_NB_ROW,current_listview);
}


/**************************************************************************/
/*  FillProfileListView() :  Remplissage d'une ListView pour les Profile. */
/**************************************************************************/
static void FillProfileListView(struct list_view *current_listview, struct profile_table *current_profile_table)
{
  int i, j, result;
  QWORD root_total_cycle_count, root_total_cycle_sub, root_total_cycle_local;
  QWORD sub_total_cycle_count, sub_total_cycle_sub, sub_total_cycle_local, percent;
  double percent_dbl;
  LVITEM listview_item;
  char buffer[1024];
  char buffer_name[1024];
  char buffer_type[1024];
  char buffer_application[1024];
  struct window_main *window_main;
  struct window_profile *window_profile;
  struct profile_function *current_profile_function;
  struct profile_function *current_sub_function;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Libration mmoire des donnes affiches dans les colonnes de la ListView */
  mem_free_listviewcolumn(current_listview);

  /* Init */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  my_Memory(MEMORY_GET_WINDOW_PROFILE,&window_profile,NULL);
  if(window_main == NULL || window_profile == NULL)
    return;

  /* Aucune Profile Fonction */
  if(window_main->current_profile_table == NULL)
    return;
  if(window_profile->current_profile_function == NULL)
    window_profile->current_profile_function = window_main->current_profile_table->first_profile_function;
  current_profile_function = window_profile->current_profile_function;
  if(current_profile_function == NULL)
    return;

  /* Rien  afficher car la fonction ROOT est sous Interrupt et on ne veut pas afficher les Interrupt */
  if(current_profile_function->is_interrupt == 1 && window_profile->hide_interrupt == 1)
    return;

  /* Compte le nombre de ligne  afficher */
  current_listview->nb_row = 1;
  for(current_sub_function = current_profile_function->first_sub_function; current_sub_function; current_sub_function = current_sub_function->next)
    if(current_sub_function->is_interrupt == 0 || (current_sub_function->is_interrupt == 1 && window_profile->hide_interrupt == 0))
      current_listview->nb_row++;

  /** Remplissage avec les valeurs d'un niveau  **/
  current_listview->nb_row =  1 + current_profile_function->nb_sub_function;
  for(i=0; i<current_listview->nb_column; i++)
    {
      /* Allocation des tableaux */
      current_listview->tab_column[i].tab_value_num = (DWORD *) calloc(current_listview->nb_row,sizeof(DWORD));
      current_listview->tab_column[i].tab_value_txt = (char **) calloc(current_listview->nb_row,sizeof(char *));
    }

  /* [ROOT] : Nombre de cycle pour les oprations de la fonction */
  root_total_cycle_local = (window_profile->hide_interrupt == 0) ? current_profile_function->total_cycle_local : current_profile_function->total_cycle_local_hide_interrupt;

  /* [ROOT] : Nombre Total de cycle pour l'ensemble des sous-fonctions  afficher (filtre selon les interrupt) */
  root_total_cycle_sub = (window_profile->hide_interrupt == 0) ? current_profile_function->total_cycle_sub : current_profile_function->total_cycle_sub_hide_interrupt;

  /** [ROOT] : Total Cycle Count = Local + Sub **/
  root_total_cycle_count = root_total_cycle_local + root_total_cycle_sub;
  if(root_total_cycle_count == 0)
    root_total_cycle_count = 1;

  /** 1re ligne : La ligne de la fonction en cours **/
  j = 0;

  /* Rcupre les informations lies  cette adresse */
  GetNameFromAddress(current_profile_function->address,current_profile_function->call_number,buffer_name,buffer_type,buffer_application);

  /* Colonne 0 : Address */
  current_listview->tab_column[0].tab_value_num[j] = current_profile_function->address;
  sprintf(buffer,"%02X/%04X",(current_profile_function->address >> 16) & 0xFF,current_profile_function->address & 0xFFFF);
  strcat(buffer,(current_profile_function->is_current_path == 1) ? "  *" : "");      /* Marque le Current Path */
  current_listview->tab_column[0].tab_value_txt[j] = strdup(buffer);

  /* Colonne 1 : Call Number */
  current_listview->tab_column[1].tab_value_num[j] = current_profile_function->call_number;
  if(current_profile_function->call_number == 0x0000 || current_profile_function->call_number == 0xFFFF)
    strcpy(buffer,"");
  else
    sprintf(buffer,"%04X",current_profile_function->call_number);
  current_listview->tab_column[1].tab_value_txt[j] = strdup(buffer);

  /* Colonne 2 : Name */
  if(current_profile_function->depth == 0 && strlen(buffer_name) == 0)
    strcpy(buffer_name,"[ROOT]");
  else if(strlen(buffer_name) > 0)
    {
      strcat(buffer_name,"]");
      memmove(&buffer_name[1],&buffer_name[0],strlen(&buffer_name[0])+1);
      buffer_name[0] = '[';
    }
  else
    sprintf(buffer_name,"[%02X/%04X]",(current_profile_function->address >> 16) & 0xFF,current_profile_function->address & 0xFFFF);
  current_listview->tab_column[2].tab_value_num[j] = j;
  current_listview->tab_column[2].tab_value_txt[j] = strdup(buffer_name);

  /* Colonne 3 : Application */
  current_listview->tab_column[3].tab_value_num[j] = j;
  if(strlen(buffer_type) > 0 && strlen(buffer_application) > 0)
    sprintf(buffer,"%s : %s",buffer_type,buffer_application);
  else if(strlen(buffer_type) > 0 && strlen(buffer_application) == 0)
    strcpy(buffer,buffer_type);
  else if(strlen(buffer_type) == 0 && strlen(buffer_application) > 0)
    strcpy(buffer,buffer_application);
  else
    strcpy(buffer,"");
  current_listview->tab_column[3].tab_value_txt[j] = strdup(buffer);

  /* Colonne 4 : Interrupt */
  current_listview->tab_column[4].tab_value_num[j] = j;
  current_listview->tab_column[4].tab_value_txt[j] = strdup((current_profile_function->is_interrupt == 1) ? "Yes" : "");

  /* Colonne 5 : Depth */
  current_listview->tab_column[5].tab_value_num[j] = current_profile_function->depth;
  sprintf(buffer,"%d",current_profile_function->depth);
  current_listview->tab_column[5].tab_value_txt[j] = strdup(buffer);

  /* Colonne 6 : # Sub */
  current_listview->tab_column[6].tab_value_num[j] = current_profile_function->nb_sub_function;
  sprintf(buffer,"%d",current_profile_function->nb_sub_function);
  current_listview->tab_column[6].tab_value_txt[j] = strdup(buffer);

  /* Colonne 7 : # Call */
  current_listview->tab_column[7].tab_value_num[j] = current_profile_function->nb_call;
  sprintf(buffer,"%d",current_profile_function->nb_call);
  PunctuateAtThoudand(buffer);
  current_listview->tab_column[7].tab_value_txt[j] = strdup(buffer);

  /* Colonne 8 : Total Cycle Local */
  current_listview->tab_column[8].tab_value_num[j] = (DWORD) root_total_cycle_local;
  sprintf(buffer,"%d",(DWORD) root_total_cycle_local);
  PunctuateAtThoudand(buffer);
  current_listview->tab_column[8].tab_value_txt[j] = strdup(buffer);

  /* Colonne 9 : % */
  percent = (root_total_cycle_local*1000)/root_total_cycle_count;
  current_listview->tab_column[9].tab_value_num[j] = (DWORD) percent;
  percent_dbl = (double) percent;
  sprintf(buffer,"%.1f",percent_dbl/10.0);
  current_listview->tab_column[9].tab_value_txt[j] = strdup(buffer);

  /* Colonne 10 : Average Cycle Local */
  current_listview->tab_column[10].tab_value_num[j] = (DWORD) (root_total_cycle_local / current_profile_function->nb_call);
  sprintf(buffer,"%d",(DWORD) (root_total_cycle_local / current_profile_function->nb_call));
  PunctuateAtThoudand(buffer);
  current_listview->tab_column[10].tab_value_txt[j] = strdup(buffer);

  /* Ligne valide */
  j++;

  /** Nime ligne : La liste des Sous-Fonctions  **/
  for(current_sub_function=current_profile_function->first_sub_function; current_sub_function; current_sub_function=current_sub_function->next)
    {
      /* On n'affiche pas cette ligne */
      if(current_sub_function->is_interrupt == 1 && window_profile->hide_interrupt == 1)
        continue;

      /* [SUB] : Nombre de cycle pour les oprations de la fonction */
      sub_total_cycle_local = (window_profile->hide_interrupt == 0) ? current_sub_function->total_cycle_local : current_sub_function->total_cycle_local_hide_interrupt;

      /* [SUB] : Nombre Total de cycle pour l'ensemble des sous-fonctions  afficher (filtre selon les interrupt) */
      sub_total_cycle_sub = (window_profile->hide_interrupt == 0) ? current_sub_function->total_cycle_sub : current_sub_function->total_cycle_sub_hide_interrupt;

      /** [SUB] : Total Cycle Count = Local + Sub **/
      sub_total_cycle_count = sub_total_cycle_local + sub_total_cycle_sub;

      /** Rcupre les informations lies  cette adresse **/
      GetNameFromAddress(current_sub_function->address,current_sub_function->call_number,buffer_name,buffer_type,buffer_application);

      /* Colonne 0 : Address */
      current_listview->tab_column[0].tab_value_num[j] = current_sub_function->address;
      sprintf(buffer,"%02X/%04X",(current_sub_function->address >> 16) & 0xFF,current_sub_function->address & 0xFFFF);
      strcat(buffer,(current_sub_function->is_current_path == 1) ? "  =>" : "");      /* Marque le Current Path */
      current_listview->tab_column[0].tab_value_txt[j] = strdup(buffer);

      /* Colonne 1 : Call Number */
      current_listview->tab_column[1].tab_value_num[j] = current_sub_function->call_number;
      if(current_sub_function->call_number == 0x0000 || current_sub_function->call_number == 0xFFFF)
        strcpy(buffer,"");
      else
        sprintf(buffer,"%04X",current_sub_function->call_number);
      current_listview->tab_column[1].tab_value_txt[j] = strdup(buffer);

      /* Colonne 2 : Name */
      current_listview->tab_column[2].tab_value_num[j] = j;
      current_listview->tab_column[2].tab_value_txt[j] = strdup(buffer_name);

      /* Colonne 3 : Application */
      current_listview->tab_column[3].tab_value_num[j] = j;
      if(strlen(buffer_type) > 0 && strlen(buffer_application) > 0)
        sprintf(buffer,"%s : %s",buffer_type,buffer_application);
      else if(strlen(buffer_type) > 0 && strlen(buffer_application) == 0)
        strcpy(buffer,buffer_type);
      else if(strlen(buffer_type) == 0 && strlen(buffer_application) > 0)
        strcpy(buffer,buffer_application);
      else
        strcpy(buffer,"");
      current_listview->tab_column[3].tab_value_txt[j] = strdup(buffer);

      /* Colonne 4 : Interrupt */
      current_listview->tab_column[4].tab_value_num[j] = j;
      current_listview->tab_column[4].tab_value_txt[j] = strdup((current_sub_function->is_interrupt == 1) ? "Yes" : "");

      /* Colonne 5 : Depth */
      current_listview->tab_column[5].tab_value_num[j] = current_sub_function->depth;
      sprintf(buffer,"%d",current_sub_function->depth);
      current_listview->tab_column[5].tab_value_txt[j] = strdup(buffer);

      /* Colonne 6 : # Sub */
      current_listview->tab_column[6].tab_value_num[j] = current_sub_function->nb_sub_function;
      sprintf(buffer,"%d",current_sub_function->nb_sub_function);
      current_listview->tab_column[6].tab_value_txt[j] = strdup(buffer);

      /* Colonne 7 : # Call */
      current_listview->tab_column[7].tab_value_num[j] = current_sub_function->nb_call;
      sprintf(buffer,"%d",current_sub_function->nb_call);
      PunctuateAtThoudand(buffer);
      current_listview->tab_column[7].tab_value_txt[j] = strdup(buffer);

      /* Colonne 8 : Total Cycle de cette Sous-Fonction */
      current_listview->tab_column[8].tab_value_num[j] = (DWORD) sub_total_cycle_count;
      sprintf(buffer,"%d",(DWORD) sub_total_cycle_count);
      PunctuateAtThoudand(buffer);
      current_listview->tab_column[8].tab_value_txt[j] = strdup(buffer);

      /* Colonne 9 : % */
      percent = (sub_total_cycle_count*1000)/root_total_cycle_count;
      current_listview->tab_column[9].tab_value_num[j] = (DWORD) percent;
      percent_dbl = (double) percent;
      sprintf(buffer,"%.1f",percent_dbl/10.0);
      current_listview->tab_column[9].tab_value_txt[j] = strdup(buffer);

      /* Colonne 10 : Average Cycle */
      current_listview->tab_column[10].tab_value_num[j] = (DWORD) (sub_total_cycle_count / current_sub_function->nb_call);
      sprintf(buffer,"%d",(DWORD) (sub_total_cycle_count / current_sub_function->nb_call));
      PunctuateAtThoudand(buffer);
      current_listview->tab_column[10].tab_value_txt[j] = strdup(buffer);

      /* Une ligne valide de plus */
      j++;
    }

  /** Transfert des donnes dans la List View via un Msg LVN_GETDISPINFO **/
  /** Ajout des Item dans le ListView **/
  listview_item.pszText   = LPSTR_TEXTCALLBACK;           /* Sends an LVN_GETDISPINFO message to fill the item */
  listview_item.mask      = LVIF_TEXT | LVIF_IMAGE | LVIF_STATE | LVIF_PARAM;
  listview_item.stateMask = 0;
  listview_item.iSubItem  = 0;
  listview_item.state     = 0;

  /** Initialize LVITEM members that are different for each item **/
  for(j=0; j<current_listview->nb_row; j++)
    {
      listview_item.iItem  = j;
      listview_item.iImage = -1;         /* Identifiant de l'image dans la liste */
      listview_item.lParam = j;          /* Identifiant renvoy lors du tri */
      result = ListView_InsertItem(current_listview->hwnd_listview,&listview_item);
    }
}


/**********************************************************************/
/*  ProcessProfileOperation() :  MAJ du Profile avec cette opration. */
/**********************************************************************/
void ProcessProfileOperation(struct window_main *window_main, int is_interrupt)
{
  DWORD curr_op_address;
  int prev_op_is_jsr_jsl, nb_unstack, nb_exit, root_profile_stack;
  struct profile_stack *current_profile_stack;
  struct profile_stack *next_profile_stack;
  struct profile_stack *eof_profile_stack;
  struct profile_stack *previous_eof_profile_stack;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Init */
  prev_op_is_jsr_jsl = (param->prev_operation_opcode == 0x20 || param->prev_operation_opcode == 0xFC || param->prev_operation_opcode == 0x22) ? 1 : 0;
  curr_op_address    = (param->operation_PB << 16) | param->operation_PC;

  /* Protection ON */
  EnterCriticalSection(&param->cs_profile_stack);

  /** Allocation de la premire entre dans la Profile Stack **/
  if(window_main->current_profile_table == NULL)
    {
      /** Allocation mmoire **/
      window_main->current_profile_table = (struct profile_table *) calloc(1,sizeof(struct profile_table));
      if(window_main->current_profile_table == NULL)
        {
          LeaveCriticalSection(&param->cs_profile_stack);
          MessageBox(window_main->hwnd_main,"Error, Impossible to allocate Profile Table !","Memory Allocation Error",MB_OK|MB_ICONWARNING);
          return;
        }

      /** Nouvelle entre dans la Call Stack **/
      /* Allocation mmoire */
      current_profile_stack = (struct profile_stack *) calloc(1,sizeof(struct profile_stack));
      if(current_profile_stack == NULL)
        {
          LeaveCriticalSection(&param->cs_profile_stack);
          MessageBox(window_main->hwnd_main,"Error, Impossible to allocate Profile Stack !","Memory Allocation Error",MB_OK|MB_ICONWARNING);
          return;
        }
      /* Remplissage Profile Stack Root */
      current_profile_stack->function_address         = curr_op_address;
      current_profile_stack->call_number              = 0x0000;
      current_profile_stack->gcc                      = param->operation_gcc;
      current_profile_stack->goid                     = param->operation_goid;
      current_profile_stack->function_end_address     = 0xFFFFFFFF;
      current_profile_stack->is_interrupt             = 0;
      current_profile_stack->depth                    = 0;
      current_profile_stack->function_under_interrupt = 0;

      /* Attache */
      window_main->current_profile_table->first_profile_stack = current_profile_stack;
      window_main->current_profile_table->last_profile_stack  = current_profile_stack;
      window_main->current_profile_table->nb_profile_stack    = 1;

      /* On vient de crer la Root Profile Stack */
      root_profile_stack = 1;

      /* On dclare la premire fonction dans les Profile */
      AddProfileFirstFunction();
    }
  else
    root_profile_stack = 0;

  /** Vient t'on de sortir d'une fonction ? **/
  for(eof_profile_stack = window_main->current_profile_table->last_profile_stack; eof_profile_stack; eof_profile_stack = eof_profile_stack->previous)
    if(eof_profile_stack->function_end_address == curr_op_address)
      break;

  /** Cas 1 : On vient de rentrer dans une fonction JSR/JSL **/
  if(root_profile_stack == 0 && prev_op_is_jsr_jsl == 1 && curr_op_address == param->prev_operation_next_op_addr)
    {
      /** Nouvelle entre dans la Call Stack **/
      /* Allocation mmoire */
      current_profile_stack = (struct profile_stack *) calloc(1,sizeof(struct profile_stack));
      if(current_profile_stack == NULL)
        {
          LeaveCriticalSection(&param->cs_profile_stack);
          MessageBox(window_main->hwnd_main,"Error, Impossible to store Profile Stack !","Memory Allocation Error",MB_OK|MB_ICONWARNING);
          return;
        }
      /* Remplissage */
      current_profile_stack->function_address         = curr_op_address;
      current_profile_stack->call_number              = param->prev_operation_call_number;
      current_profile_stack->gcc                      = param->operation_gcc;
      current_profile_stack->goid                     = param->operation_goid;
      current_profile_stack->function_end_address     = param->prev_operation_sub_next_op_addr;
      current_profile_stack->is_interrupt             = 0;
      current_profile_stack->depth                    = window_main->current_profile_table->last_profile_stack->depth + 1;
      current_profile_stack->function_under_interrupt = window_main->current_profile_table->last_profile_stack->function_under_interrupt;

      /* Ajoute  la fin de la liste doublement chaine */
      current_profile_stack->previous                              = window_main->current_profile_table->last_profile_stack;
      window_main->current_profile_table->last_profile_stack->next = current_profile_stack;
      window_main->current_profile_table->last_profile_stack       = current_profile_stack;
      window_main->current_profile_table->nb_profile_stack++;

      /** On dclare cette fonction dans les Profile **/
      AddProfileFunction(current_profile_stack->function_under_interrupt);
    }
  /** Cas 2 : On vient de rentrer dans une Interruption **/
  else if(root_profile_stack == 0 && curr_op_address != param->prev_operation_next_op_addr && param->is_first_operation == 0)
    {
      /** Nouvelle entre dans la Call Stack **/
      /* Allocation mmoire */
      current_profile_stack = (struct profile_stack *) calloc(1,sizeof(struct profile_stack));
      if(current_profile_stack == NULL)
        {
          LeaveCriticalSection(&param->cs_profile_stack);
          MessageBox(window_main->hwnd_main,"Error, Impossible to store Profile Stack !","Memory Allocation Error",MB_OK|MB_ICONWARNING);
          return;
        }
      /* Remplissage */
      current_profile_stack->function_address         = curr_op_address;
      current_profile_stack->call_number              = param->prev_operation_call_number;
      current_profile_stack->gcc                      = param->operation_gcc;
      current_profile_stack->goid                     = param->operation_goid;
      current_profile_stack->function_end_address     = param->prev_operation_sub_next_op_addr;
      current_profile_stack->is_interrupt             = 1;
      current_profile_stack->depth                    = window_main->last_callstack->depth + 1;
      current_profile_stack->function_under_interrupt = 1;

      /* Ajoute  la fin de la liste doublement chaine */
      current_profile_stack->previous                              = window_main->current_profile_table->last_profile_stack;
      window_main->current_profile_table->last_profile_stack->next = current_profile_stack;
      window_main->current_profile_table->last_profile_stack       = current_profile_stack;
      window_main->current_profile_table->nb_profile_stack++;

      /** On dclare cette fonction dans les Profile **/
      AddProfileFunction(1);
    }
  /** Cas 3 : On vient de sortir d'une fonction **/
  else if(root_profile_stack == 0 && eof_profile_stack != NULL)
    {
      /** On va peut tre devoir dpiler plusieurs appels sur la mme opration (Interruption + Fonction) **/
      nb_exit = 1;
      while(eof_profile_stack->function_end_address == curr_op_address)
        {
          /* On conserve un pointeur sur la fonction prcdente */
          previous_eof_profile_stack = eof_profile_stack->previous;

          /** On vient de quitter une fonction **/
          ExitProfileFunction(eof_profile_stack);

          /** On va dpiler la Profile Stack (jamais la premire entre) **/
          nb_unstack = 0;
          eof_profile_stack->previous->next = NULL;
          window_main->current_profile_table->last_profile_stack = eof_profile_stack->previous;
          for(current_profile_stack = eof_profile_stack; current_profile_stack; current_profile_stack = next_profile_stack)
            {
              /* Suppression la CallStack */
              next_profile_stack = current_profile_stack->next;
              free(current_profile_stack);
              window_main->current_profile_table->nb_profile_stack--;
              nb_unstack++;
            }

          /* On peut encore dpiler si on est sur la mme @ de retour */
          nb_exit++;
          if(previous_eof_profile_stack == NULL)
            break;
          eof_profile_stack = previous_eof_profile_stack;
        }
    }
  /** Cas 4 : C'est une opration neutre **/
  else
    {
      /** MAJ du nombre de cycle de la Fonction courrante **/
      window_main->current_profile_table->current_profile_function->total_cycle_local += param->operation_nb_cycle;
      if(window_main->current_profile_table->current_profile_function->is_interrupt == 0)
        window_main->current_profile_table->current_profile_function->total_cycle_local_hide_interrupt += param->operation_nb_cycle;
    }

  /* Protection OFF */
  LeaveCriticalSection(&param->cs_profile_stack);
}


/**************************************************************************/
/*  AddProfileFirstFunction() :  On dclare la premire fonction Profile. */
/**************************************************************************/
static void AddProfileFirstFunction(void)
{
  struct profile_stack *current_profile_stack;
  struct profile_function *current_profile_function;
  struct window_main *window_main;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Rcupre la fentre Main */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  if(window_main == NULL)
    return;

  /* Rcupre la Profile Stack */
  current_profile_stack = window_main->current_profile_table->first_profile_stack;

  /* Allocation mmoire de la premire Profile Function (on la considre hors interruption) */
  current_profile_function = mem_alloc_profile_function(current_profile_stack,0,0);
  if(current_profile_function == NULL)
    {
      MessageBox(window_main->hwnd_main,"Impossible to allocate Memory (current_profile_function).","Error",MB_OK|MB_ICONWARNING);
      return;
    }

  /* Attachement */
  window_main->current_profile_table->first_profile_function = current_profile_function;

  /** On dplace le pointeur de la fonction courrante **/
  window_main->current_profile_table->current_profile_function = current_profile_function;
}


/********************************************************************/
/*  AddProfileFunction() :  On dclare un nouvel appel de fonction. */
/********************************************************************/
static void AddProfileFunction(int is_interrupt)
{
  struct profile_stack *first_profile_stack;
  struct profile_stack *current_profile_stack;
  struct profile_function *current_profile_function;
  struct profile_function *current_profile_sub_function;
  struct window_main *window_main;

  /* Rcupre la fentre Main */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  if(window_main == NULL)
    return;

  /** On va se positionner au niveau de la Sub Fonction demande **/
  current_profile_function = window_main->current_profile_table->first_profile_function;
  first_profile_stack      = window_main->current_profile_table->first_profile_stack;
  for(current_profile_stack = first_profile_stack; current_profile_stack; current_profile_stack = current_profile_stack->next)
    {
      /** On est arriv au dernier niveau de la CallStack : C'est une sous-fonction dj visit avant **/
      if(current_profile_stack->next == NULL && current_profile_stack->function_address == current_profile_function->address && current_profile_stack->call_number == current_profile_function->call_number)
        {
          /* Mise  jour de cette Sub Fonction */
          current_profile_sub_function->nb_call++;
          current_profile_sub_function->goid_init = current_profile_stack->goid;
          current_profile_sub_function->gcc_init  = current_profile_stack->gcc;

          /** On dplace le pointeur de la fonction courrante **/
          window_main->current_profile_table->current_profile_function = current_profile_sub_function;
          return;
        }

      /** Recherche la Sous fonction qui correspond au niveau Suivant de la CallBack **/
      for(current_profile_sub_function = current_profile_function->first_sub_function; current_profile_sub_function; current_profile_sub_function = current_profile_sub_function->next)
        if(current_profile_stack->next->function_address == current_profile_sub_function->address && current_profile_stack->next->call_number == current_profile_sub_function->call_number)
          break;

      /** On va allouer une Profile Sub Function  ce niveau **/
      if(current_profile_sub_function == NULL)
        {
          /* Allocation mmoire */
          current_profile_sub_function = mem_alloc_profile_function(current_profile_stack->next,current_profile_function->depth+1,is_interrupt);
          if(current_profile_sub_function == NULL)
            {
              return;
            }
          current_profile_sub_function->parent = current_profile_function;
          /* Attachement  la liste chaine */
          if(current_profile_function->first_sub_function == NULL)
            current_profile_function->first_sub_function = current_profile_sub_function;
          else
            current_profile_function->last_sub_function->next = current_profile_sub_function;
          current_profile_function->last_sub_function = current_profile_sub_function;
          current_profile_function->nb_sub_function++;

          /** On est au bout de la Call Stack ? **/
          if(current_profile_stack->next->next == NULL)
            {
              /** On dplace le pointeur de la fonction courrante **/
              window_main->current_profile_table->current_profile_function = current_profile_sub_function;
              return;
            }
        }

      /* Cette Sub Profile Function devient la Profile Function */
      current_profile_function = current_profile_sub_function;
    }
}


/****************************************************************/
/*  ExitProfileFunction() :  On vient de sortir d'une fonction. */
/****************************************************************/
static void ExitProfileFunction(struct profile_stack *eof_profile_stack)
{
  struct profile_stack *first_profile_stack;
  struct profile_stack *current_profile_stack;
  struct profile_function *current_profile_function;
  struct profile_function *current_profile_sub_function;
  struct window_main *window_main;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Rcupre la fentre Main */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  if(window_main == NULL)
    return;

  /* Rien  faire ? */
  if(window_main->current_profile_table == NULL)
    return;

  /** On va se positionner au niveau de la Sub Fonction demande **/
  current_profile_function = window_main->current_profile_table->first_profile_function;
  first_profile_stack      = window_main->current_profile_table->first_profile_stack;
  for(current_profile_stack = first_profile_stack; current_profile_stack; current_profile_stack = current_profile_stack->next)
    {
      /** On est arriv au dernier niveau de la CallStack **/
      if(current_profile_stack->next == NULL && current_profile_stack->function_address == current_profile_function->address && current_profile_stack->call_number == current_profile_function->call_number)
        {
          /** On dplace le pointeur de la fonction courrante vers le Parent**/
          window_main->current_profile_table->current_profile_function = current_profile_function->parent;
          return;
        }

      /** Recherche la Sous fonction qui correspond au niveau Suivant de la CallBack **/
      for(current_profile_sub_function = current_profile_function->first_sub_function; current_profile_sub_function; current_profile_sub_function = current_profile_sub_function->next)
        if(current_profile_stack->next->function_address == current_profile_sub_function->address && current_profile_stack->next->call_number == current_profile_sub_function->call_number)
          break;

      /** On n'a pas trouv la Profile Sub Function  ce niveau => On Sort **/
      if(current_profile_sub_function == NULL)
        return;

      /* Cette Sub Profile Function devient la Profile Function */
      current_profile_function = current_profile_sub_function;
    }
}


/*********************************************************************************************/
/*  BuildProfileFunctionCycleCount() :  Construction du nombre de Cycles pour les fonctions. */
/*********************************************************************************************/
static void BuildProfileFunctionCycleCount(void)
{
  int i, nb_function, current_depth, max_depth;
  struct profile_function **tab_function;
  struct window_main *window_main;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Rcupre la fentre Main */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  if(window_main == NULL)
    return;
  if(window_main->current_profile_table == NULL)
    return;

  /** RAZ Total Sub cycle (tous les total_cycle_sub / total_cycle_sub_hide_interrupt  0) **/
  nb_function = ClearProfileFunctionCycleCount(window_main->current_profile_table->first_profile_function);     /* Arbre Complet */

  /** Cration d'un tableau de Fonction **/
  tab_function = (struct profile_function **) calloc(nb_function,sizeof(struct profile_function *));
  if(tab_function == NULL)
    return;

  /** Remplissage du Tableau avec toutes les fonctions de l'arbre **/
  FillProfileFunctionCycleCountTab(window_main->current_profile_table->first_profile_function,nb_function,tab_function);

  /* Tri du tableau par niveau de profondeur */
  qsort(tab_function,nb_function,sizeof(struct profile_function *),compare_function);

  /** On va rpartir les cycles de chaque fonction sur les parents ([ROOT] = Depth #0) **/
  max_depth = tab_function[0]->depth;
  for(current_depth=max_depth; current_depth>0 ; current_depth--)
    {
      /* Passe en revue toutes les entres */
      for(i=0; i<nb_function; i++)
        if(tab_function[i]->depth == current_depth)
          {
            /** On va faire remonter les cycles au Pre **/
            tab_function[i]->parent->total_cycle_sub                += (tab_function[i]->total_cycle_local                + tab_function[i]->total_cycle_sub);
            tab_function[i]->parent->total_cycle_sub_hide_interrupt += (tab_function[i]->total_cycle_local_hide_interrupt + tab_function[i]->total_cycle_sub_hide_interrupt);
          }
    }

  /* Libration mmoire */
  free(tab_function);
}


/************************************************************************************/
/*  ClearProfileFunctionCycleCount() :  Nettoyage du nombre de cycle des fonctions. */
/************************************************************************************/
static int ClearProfileFunctionCycleCount(struct profile_function *current_function)
{
  int nb_function;
  struct profile_function *current_sub_function;

  /* Init */
  nb_function = 1;

  /* Fonction */
  current_function->total_cycle_sub                = 0;
  current_function->total_cycle_sub_hide_interrupt = 0;

  /** Sous Fonctions (rcursivit) **/
  for(current_sub_function = current_function->first_sub_function; current_sub_function; current_sub_function=current_sub_function->next)
    nb_function += ClearProfileFunctionCycleCount(current_sub_function);

  /* Renvoie le nombre de Fonction */
  return(nb_function);
}


/***************************************************************************************/
/*  FillProfileFunctionCycleCountTab() :  On va ajouter tous les Fonctions au tableau. */
/***************************************************************************************/
static void FillProfileFunctionCycleCountTab(struct profile_function *current_function, int nb_function, struct profile_function **tab_function)
{
  int i;
  struct profile_function *current_sub_function;

  /** Cherche une place libre dans le tableau pour cette fonction **/
  for(i=0; i<nb_function; i++)
    if(tab_function[i] == NULL)
      {
        tab_function[i] = current_function;
        break;
      }

  /** Sous Fonctions (rcursivit) **/
  for(current_sub_function = current_function->first_sub_function; current_sub_function; current_sub_function = current_sub_function->next)
    FillProfileFunctionCycleCountTab(current_sub_function,nb_function,tab_function);
}


/*****************************************************************************************/
/*  BuildProfileFunctionCurrentPath() :  Construction du chemin de la fonction actuelle. */
/*****************************************************************************************/
static void BuildProfileFunctionCurrentPath(struct window_profile *window_profile)
{
  struct profile_stack *first_profile_stack;
  struct profile_stack *current_profile_stack;
  struct profile_function *current_profile_function;
  struct profile_function *current_profile_sub_function;
  struct window_main *window_main;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Rcupre la fentre Main */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  if(window_main == NULL)
    return;

  /* Rien  faire ? */
  if(window_main->current_profile_table == NULL)
    return;

  /** Nettoyage pralable **/
  ClearProfileFunctionCurrentPath(window_main->current_profile_table->first_profile_function);  /* Premire Fonction de l'Arbre */

  /* Init : Premire fonction de l'arbre */
  current_profile_function = window_main->current_profile_table->first_profile_function;   /* Arbre Complet */
  first_profile_stack      = window_main->current_profile_table->first_profile_stack;      /* Liste des Fonctions */

  /** On va marquer toutes les fonctions qui sont sur le chemin **/
  current_profile_function->is_current_path = 1;
  for(current_profile_stack = first_profile_stack; current_profile_stack; current_profile_stack = current_profile_stack->next)
    {
      /** On est arriv au dernier niveau de la CallStack **/
      if(current_profile_stack->next == NULL && current_profile_stack->function_address == current_profile_function->address && current_profile_stack->call_number == current_profile_function->call_number)
        return;

      /** Recherche la Sous fonction qui correspond au niveau Suivant de la CallBack **/
      for(current_profile_sub_function = current_profile_function->first_sub_function; current_profile_sub_function; current_profile_sub_function = current_profile_sub_function->next)
        if(current_profile_stack->next->function_address == current_profile_sub_function->address && current_profile_stack->next->call_number == current_profile_sub_function->call_number)
          break;

      /** On n'a pas trouv la Profile Sub Function  ce niveau => On Sort **/
      if(current_profile_sub_function == NULL)
        return;

      /* Cette Sub Profile Function devient la Profile Function */
      current_profile_function = current_profile_sub_function;

      /* Cette sous-fonction est dans le chemin */
      current_profile_function->is_current_path = 1;
    }
}


/**************************************************************************************/
/*  ClearProfileFunctionCurrentPath() :  Nettoyage du chemin de la fonction actuelle. */
/**************************************************************************************/
static void ClearProfileFunctionCurrentPath(struct profile_function *current_function)
{
  struct profile_function *current_sub_function;

  /* Fonction */
  current_function->is_current_path = 0;

  /* Sous Fonctions (rcursivit) */
  for(current_sub_function = current_function->first_sub_function; current_sub_function; current_sub_function=current_sub_function->next)
    ClearProfileFunctionCurrentPath(current_sub_function);
}


/*****************************************************************************************/
/*  mem_alloc_profile_function() :  Allocation mmoire d'une structure profile_function. */
/*****************************************************************************************/
static struct profile_function *mem_alloc_profile_function(struct profile_stack *current_profile_stack, int depth, int is_interrupt)
{
  struct profile_function *current_profile_function;

  /* Allocation mmoire */
  current_profile_function = (struct profile_function *) calloc(1,sizeof(struct profile_function));
  if(current_profile_function == NULL)
    return(NULL);

  /* Remplissage */
  current_profile_function->address      = current_profile_stack->function_address;
  current_profile_function->call_number  = current_profile_stack->call_number;
  current_profile_function->depth        = depth;
  current_profile_function->is_interrupt = is_interrupt;
  current_profile_function->goid_init    = current_profile_stack->goid;
  current_profile_function->gcc_init     = current_profile_stack->gcc;
  current_profile_function->nb_call      = 1;

  /* Renvoi la structure */
  return(current_profile_function);
}


/******************************************************************************/
/*  RefreshProfileWindow() :  Mise  jour des entres de la fentre  Profile. */
/******************************************************************************/
static void RefreshProfileWindow(struct window_profile *window_profile)
{
  QWORD root_total_cycle_count, sub_total_cycle_count, total_cycle_count;
  char buffer_name[BUFFER_SIZE];
  char buffer_type[BUFFER_SIZE];
  char buffer_application[BUFFER_SIZE];
  char buffer_temp[BUFFER_SIZE];
  char buffer_path[BUFFER_SIZE];
  char window_title[BUFFER_SIZE];
  struct profile_function *current_sub_function;
  struct window_main *window_main;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Rcupre la fentre Main */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  if(window_main == NULL)
    return;

  /** Marque les fonctions qui sont dans le chemin courrant (met  1 is_current_path dans l'arbre des fonctions) **/
  BuildProfileFunctionCurrentPath(window_profile);

  /** Construit le nombre cycles des sous-fonctions **/
  BuildProfileFunctionCycleCount();

  /** Supprime toutes les entres du Listview **/
  if(window_profile != NULL)
    ListView_DeleteAllItems(window_profile->listview_profile.hwnd_listview);

  /** On refait le Listview avec les Profile Function de la Table **/
  if(window_profile != NULL)
    FillProfileListView(&window_profile->listview_profile,window_main->current_profile_table);

  /** Cration du Titre de la fentre **/
  strcpy(buffer_path,"");
  for(current_sub_function = window_profile->current_profile_function; current_sub_function; current_sub_function = current_sub_function->parent)
    {
      /** Rcupre les informations lies  cette adresse **/
      GetNameFromAddress(current_sub_function->address,current_sub_function->call_number,buffer_name,buffer_type,buffer_application);
      if(current_sub_function->depth == 0 && strlen(buffer_name) == 0)
        strcpy(buffer_name,"[ROOT]");
      if(strlen(buffer_name) == 0)
        sprintf(buffer_name,"[%02X/%02X]",(current_sub_function->address >>16) & 0xFF,current_sub_function->address & 0xFFFF);

      /* Ajoute au dbut */
      if(strlen(buffer_path) == 0)
        sprintf(buffer_temp," %s ",buffer_name);
      else
        sprintf(buffer_temp," %s / %s",buffer_name,buffer_path);
      strcpy(buffer_path,buffer_temp);
    }
  sprintf(window_title," Function Profiler : %s",buffer_path);
  SetWindowText(window_profile->hwnd_profile,window_title);

  /** MAJ du nombre de cycle **/
  if(window_profile->current_profile_function != NULL)
    {
      /** Total du nombre de cycles = Nombre de Cycle de la fonction actuelle + Nombre de Cycle des Sous Fonctions **/
      root_total_cycle_count = (window_profile->hide_interrupt == 0) ? window_profile->current_profile_function->total_cycle_local : window_profile->current_profile_function->total_cycle_local_hide_interrupt;
      sub_total_cycle_count  = (window_profile->hide_interrupt == 0) ? window_profile->current_profile_function->total_cycle_sub   : window_profile->current_profile_function->total_cycle_sub_hide_interrupt;
      total_cycle_count = root_total_cycle_count + sub_total_cycle_count;
    }
  else
    total_cycle_count = 0;
  sprintf(buffer_temp,"%d",total_cycle_count);
  PunctuateAtThoudand(buffer_temp);
  SetWindowText(GetDlgItem(window_profile->hwnd_profile,IDC_STATIC_TOTAL_CYCLE),buffer_temp);

  /** MAJ Boutons << >> **/
  if(window_profile->current_profile_function == NULL)
    EnableWindow(GetDlgItem(window_profile->hwnd_profile,IDC_BUTTON_PROFILE_BACKWARD),FALSE);
  else if(window_profile->current_profile_function->depth == 0)
    EnableWindow(GetDlgItem(window_profile->hwnd_profile,IDC_BUTTON_PROFILE_BACKWARD),FALSE);
  else
    EnableWindow(GetDlgItem(window_profile->hwnd_profile,IDC_BUTTON_PROFILE_BACKWARD),TRUE);
  EnableWindow(GetDlgItem(window_profile->hwnd_profile,IDC_BUTTON_PROFILE_FORWARD),FALSE);

  /* MAJ Bouton Delete */
  EnableWindow(GetDlgItem(window_profile->hwnd_profile,IDC_BUTTON_DELETE_FUNCTION),FALSE);

  /* Aucune Fonction slectionne dans la ListView */
  window_profile->selected_profile_function    = NULL;
  window_profile->selected_profile_address     = 0x00000000;
  window_profile->selected_profile_call_number = 0x0000;
}


/******************************************************************************/
/*  DeleteProfileFunction() :  On va supprimer la sous-fonction slectionne. */
/******************************************************************************/
static void DeleteProfileFunction(struct window_profile *window_profile)
{
  struct profile_function *parent_function;
  struct profile_function *current_sub_function;

  /* Rien  supprimer ? */
  if(window_profile->selected_profile_function == NULL)
    return;

  /* Trouve le Pre */
  parent_function = window_profile->selected_profile_function->parent;

  /** Parcours les Sous-Fonctions pour supprimer la sous-fonction slectionne **/
  if(parent_function->first_sub_function == window_profile->selected_profile_function && parent_function->nb_sub_function == 1)
    {
      /* 1 Seule sous-fonction */
      parent_function->nb_sub_function    = 0;
      parent_function->first_sub_function = NULL;
      parent_function->last_sub_function  = NULL;
    }
  if(parent_function->first_sub_function == window_profile->selected_profile_function)
    {
      /* Supprime la premire */
      parent_function->first_sub_function = parent_function->first_sub_function->next;
      parent_function->nb_sub_function--;
    }
  else if(parent_function->last_sub_function == window_profile->selected_profile_function)
    {
      /* Supprime la dernire */
      for(current_sub_function=parent_function->first_sub_function; current_sub_function; current_sub_function=current_sub_function->next)
        if(current_sub_function->next == window_profile->selected_profile_function)
          {
            current_sub_function->next         = NULL;
            parent_function->last_sub_function = current_sub_function;
            parent_function->nb_sub_function--;
            break;
          }
    }
  else
    {
      /** Supprime au milieu **/
      for(current_sub_function=parent_function->first_sub_function; current_sub_function; current_sub_function=current_sub_function->next)
        if(current_sub_function->next == window_profile->selected_profile_function)
          {
            current_sub_function->next = current_sub_function->next->next;
            parent_function->nb_sub_function--;
            break;
          }
    }

  /** Supprime la Sous-Fonction et toutes les Sous-Fonctions en dessous **/
  mem_free_profile_function(window_profile->selected_profile_function);

  /** On va reconstruire la ListView **/
  RefreshProfileWindow(window_profile);
}


/******************************************************/
/*  ClearProfileTable() :  Vide la table des Profile. */
/******************************************************/
void ClearProfileTable(struct window_profile *window_profile)
{
  struct window_main *window_main;

  /* Rcupre la fentre Main */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  if(window_main == NULL)
    return;

  /* Rien  faire */
  if(window_main->current_profile_table == NULL)
    return;

  /* Libration mmoire */
  mem_free_profile_table(window_main->current_profile_table);
  window_main->current_profile_table = NULL;
  window_profile->current_profile_function = NULL;

  /** Supprime toutes les entres du Listview **/
  ListView_DeleteAllItems(window_profile->listview_profile.hwnd_listview);

  /* MAJ du titre de la fentre avec le nombre de Profile */
  SetWindowText(window_profile->hwnd_profile," Function Profiler :");

  /* MAJ du nombre de total de cycle */
  SetWindowText(GetDlgItem(window_profile->hwnd_profile,IDC_STATIC_TOTAL_CYCLE),"0");
}


/************************************************************************/
/*  IsSameProfileFunction() :  Vrifie s'il s'agit de la mme fonction. */
/************************************************************************/
static int IsSameProfileFunction(struct profile_function *function_1, DWORD address_2, WORD call_number_2)
{
  int has_valid_1, has_valid_2;

  /* Adresse */
  if(function_1->address != address_2)
    return(0);

  /** Call Number (invalide si 0x0000 ou 0xFFFF **/
  has_valid_1 = (function_1->call_number == 0x0000 || function_1->call_number == 0xFFFF) ? 0 : 1;
  has_valid_2 = (call_number_2 == 0x0000 || call_number_2 == 0xFFFF) ? 0 : 1;
  if(has_valid_1 != has_valid_2)
    return(0);
  if(has_valid_1 == 1 && has_valid_2 == 1 && function_1->call_number != call_number_2)
    return(0);

  /* Identique */
  return(1);
}


/************************************************************************************/
/*  UpdateProfileHideInterrupt() :  MAJ de l'affichage avec/sans les interruptions. */
/************************************************************************************/
static void UpdateProfileHideInterrupt(struct window_profile *window_profile)
{
  /* Inverse la valeur */
  if(window_profile->hide_interrupt == 0)
    window_profile->hide_interrupt = 1;
  else
    window_profile->hide_interrupt = 0;

  /* MAJ de l'affichage */
  RefreshProfileWindow(window_profile);
}


/********************************************************/
/*  SaveProfilerFile() :  Cration du fichier Profiler. */
/********************************************************/
static void SaveProfilerFile(struct window_profile *window_profile)
{
  BOOL is_OK;
  int i, file_width, file_height, nb_write, hide_interrupt;
  FILE *fd_profiler;
  char *profile_file_data;
  char file_path[BUFFER_SIZE];
  struct window_main *window_main;
  struct profile_function *root_profile_function;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Rcupre la fentre Main */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  if(window_main == NULL)
    return;
  if(window_main->current_profile_table == NULL)
    return;

  /* Fonction de dmarrage de l'arbre */
  if(window_profile->current_profile_function != NULL)
    root_profile_function = window_profile->current_profile_function;
  else
    root_profile_function = window_main->current_profile_table->first_profile_function;

  /* Affichage des Interruptions */
  hide_interrupt = window_profile->hide_interrupt;
  if(hide_interrupt == 1 && root_profile_function->is_interrupt == 1)
    return;

  /** On va parcourir l'arbre pour dterminer la position de chaque function + MaxX / MaxY +  Nom de chaque fonction **/
  window_main->current_profile_table->file_max_x = 0;
  window_main->current_profile_table->file_max_y = 0;
  ComputeProfileFileCoordinate(root_profile_function,1,1,hide_interrupt,window_main->current_profile_table,window_profile);

  /* Allocation Tableau de la largeur Max de chaque colonne + RAZ */
  if(window_main->current_profile_table->tab_max_width)
    free(window_main->current_profile_table->tab_max_width);
  window_main->current_profile_table->tab_max_width = (int *) calloc(window_main->current_profile_table->file_max_x,sizeof(int));
  if(window_main->current_profile_table->tab_max_width == NULL)
    {
      MessageBox(window_profile->hwnd_profile,"Impossible to allocate Memory (window_main->current_profile_table->tab_max_width).","Error",MB_OK|MB_ICONWARNING);
      return;
    }

  /** On calcul la largeur Max de chaque colonne **/
  ComputeProfileFileWidth(root_profile_function,hide_interrupt,window_main->current_profile_table);

  /** Allocation de la zone Texte **/
  file_height = 2*window_main->current_profile_table->file_max_y;
  file_width  = 1;
  for(i=0; i<window_main->current_profile_table->file_max_x; i++)
    file_width += (PROFILER_EXTRA_SIZE + window_main->current_profile_table->tab_max_width[i]);
  profile_file_data = (char *) calloc(file_width*file_height+1,sizeof(char));
  if(profile_file_data == NULL)
    {
      MessageBox(window_profile->hwnd_profile,"Impossible to allocate Memory (profile_file_data).","Error",MB_OK|MB_ICONWARNING);
      return;
    }
  memset(profile_file_data,' ',file_width*file_height);

  /** Remplissage du fichier avec les noms des fonctions + Liens **/
  ComputeProfileFileFill(root_profile_function,file_width,file_height,profile_file_data,hide_interrupt,window_main->current_profile_table);

  /* Termine les lignes avec un \n */
  for(i=0; i<file_height; i++)
    profile_file_data[(i+1)*file_width-1] = '\n';

  /** Demande le nom du fichier Profiler **/
  sprintf(file_path,"Functions_Profiler.txt");
  is_OK = GetSaveFilePath(window_profile->hwnd_profile,"Select Functions Profiler file","All Files (*.txt)\0*.txt\0\0",file_path);
  if(is_OK = FALSE)
    return;

  /* Cration du fichier sur disque */
  fd_profiler = fopen(file_path,"w");
  if(fd_profiler == NULL)
    {
      MessageBox(window_profile->hwnd_profile,"Impossible to create File","Error",MB_OK|MB_ICONWARNING);
      return;
    }

  /** Ecriture des donnes du fichier **/
  nb_write = (int) fwrite(profile_file_data,strlen(profile_file_data),1,fd_profiler);

  /* Fermeture du fichier */
  fclose(fd_profiler);

  /* Libration mmoire */
  free(profile_file_data);
}


/********************************************************************************************/
/*  ComputeProfileFileCoordinate() :  Calcul les positions de toutes les Profile Fonctions. */
/********************************************************************************************/
static void ComputeProfileFileCoordinate(struct profile_function *current_function, int x, int y, int hide_interrupt, struct profile_table *current_profile_table, struct window_profile *window_profile)
{
  QWORD local_total_cycle_count, sub_total_cycle_count, total_cycle_count;
  int first_sub_function;
  char buffer_name[BUFFER_SIZE];
  char buffer_type[BUFFER_SIZE];
  char buffer_application[BUFFER_SIZE];
  struct profile_function *current_sub_function;

  /* On n'affiche pas les Interrupt */
  if(hide_interrupt == 1 && current_function->is_interrupt == 1)
    return;

  /* On vient de rentrer dans une nouvelle fonction */
  current_function->file_x = x;
  current_function->file_y = y;

  /* Max */
  if(current_profile_table->file_max_x < x)
    current_profile_table->file_max_x = x;
  if(current_profile_table->file_max_y < y)
    current_profile_table->file_max_y = y;

  /** Rcupre les informations lies  cette adresse **/
  GetNameFromAddress(current_function->address,current_function->call_number,buffer_name,buffer_type,buffer_application);
  sprintf(current_function->full_name,"%02X/%04X",(current_function->address >> 16) & 0xFF,current_function->address & 0xFFFF);
  if(current_function->call_number != 0x0000 && current_function->call_number != 0xFFFF)
    sprintf(&current_function->full_name[strlen(current_function->full_name)]," [%04X]",current_function->call_number);
  if(strlen(buffer_name) > 0)
    sprintf(&current_function->full_name[strlen(current_function->full_name)]," %s",buffer_name);  

  /** Total du nombre de cycles = Nombre de Cycle de la fonction actuelle + Nombre de Cycle des Sous-Fonctions **/
  local_total_cycle_count = (window_profile->hide_interrupt == 0) ? current_function->total_cycle_local : current_function->total_cycle_local_hide_interrupt;
  sub_total_cycle_count  = (window_profile->hide_interrupt == 0) ? current_function->total_cycle_sub   : current_function->total_cycle_sub_hide_interrupt;
  total_cycle_count      = local_total_cycle_count + sub_total_cycle_count;

  /* On ajoute le nombre d'appel */
  sprintf(&current_function->full_name[strlen(current_function->full_name)]," (x%d, L=%d, S=%d)",current_function->nb_call,(DWORD)local_total_cycle_count,(DWORD)sub_total_cycle_count);

  /* Largeur de cette fonction */
  current_function->width = (int) strlen(current_function->full_name);

  /** Traite toutes les sous-fonctions **/
  first_sub_function = 1;
  for(current_sub_function = current_function->first_sub_function; current_sub_function; current_sub_function = current_sub_function->next)
    {
      /* On n'affiche pas les Interrupt */
      if(hide_interrupt == 1 && current_sub_function->is_interrupt == 1)
        continue;

      /* La premire sous-fonction reste sur la mme ligne que la fonction (rcursivit) */
      if(first_sub_function == 1)
        ComputeProfileFileCoordinate(current_sub_function,x+1,y,hide_interrupt,current_profile_table,window_profile);
      else
        /* Pour les autres sous-fonctions, on prend en compte la bonne hauteur (rcursivit) */
        ComputeProfileFileCoordinate(current_sub_function,x+1,current_profile_table->file_max_y+1,hide_interrupt,current_profile_table,window_profile);

      /* La premire a t gre */
      first_sub_function = 0;
    }
}


/***************************************************************************/
/*  ComputeProfileFileWidth() :  Calcule la largeur Max de chaque colonne. */
/***************************************************************************/
static void ComputeProfileFileWidth(struct profile_function *current_function, int hide_interrupt, struct profile_table *current_profile_table)
{
  struct profile_function *current_sub_function;

  /* On n'affiche pas les Interrupt */
  if(hide_interrupt == 1 && current_function->is_interrupt == 1)
    return;

  /* Largeur Max de cette colonne */
  if(current_profile_table->tab_max_width[current_function->file_x-1] < current_function->width)
    current_profile_table->tab_max_width[current_function->file_x-1] = current_function->width;

  /** On va descendre dans les Sous Fonctions (rcursivit) **/
  for(current_sub_function = current_function->first_sub_function; current_sub_function; current_sub_function = current_sub_function->next)
    ComputeProfileFileWidth(current_sub_function,hide_interrupt,current_profile_table);
}


/***********************************************************************************/
/*  ComputeProfileFileFill() :  Remplissage du ficher avec les noms des fonctions. */
/***********************************************************************************/
static void ComputeProfileFileFill(struct profile_function *current_function, int file_width, int file_height, char *file_data, int hide_interrupt, struct profile_table *current_profile_table)
{
  int i, file_x_offset, file_y_offset, is_first_sub;
  struct profile_function *first_sub_function;
  struct profile_function *current_sub_function;

  /* On n'affiche pas les Interrupt */
  if(hide_interrupt == 1 && current_function->is_interrupt == 1)
    return;

  /* Est-ce la premire sous-fonction */
  if(current_function->parent == NULL)
    is_first_sub = 0;
  else
    {
      /* Dtermine la premire sous-fonction affiche */
      first_sub_function = NULL;
      for(current_sub_function = current_function->parent->first_sub_function; current_sub_function; current_sub_function = current_sub_function->next)
        {
          if(current_sub_function->is_interrupt == 1 && hide_interrupt == 1)
            continue;
          else
            {
              first_sub_function = current_sub_function;
              break;
            }
        }

      /* Est-ce la premire sous-fonction affiche ? */
      is_first_sub = (first_sub_function == current_function) ? 1 : 0;
    }

  /* Positionne le nom de la fonction en X,Y */
  file_x_offset = PROFILER_EXTRA_SIZE;
  for(i=0; i<current_function->file_x-1; i++)
    file_x_offset += (PROFILER_EXTRA_SIZE + current_profile_table->tab_max_width[i]);
  file_y_offset = file_width*(2*current_function->file_y-1);
  memcpy(&file_data[file_y_offset + file_x_offset],current_function->full_name,strlen(current_function->full_name));

  /* On dessine le lien vers le pre */
  if(current_function->file_x == 1 && current_function->file_y == 1)
    memcpy(&file_data[file_y_offset + file_x_offset - PROFILER_EXTRA_SIZE]," -----o ",PROFILER_EXTRA_SIZE);    /* ROOT */
  else if(is_first_sub == 1)
    {
      /* La premire sous fonction est au mme niveau que le pre */
      file_data[file_y_offset + file_x_offset - 2] = 'o';
      for(i=0; file_data[file_y_offset + file_x_offset - 4 - i] == ' '; i++)
        file_data[file_y_offset + file_x_offset - 3 - i] = '-';
    }
  else
    {
      /* Trait horizontal */
      memcpy(&file_data[file_y_offset + file_x_offset - 1 - strlen("+--o")],"+--o",strlen("+--o"));

      /* Trait vertical */
      for(i=0; file_data[file_y_offset + file_x_offset - strlen("+--o") - 1 - (1+i)*file_width] == ' '; i++)
        file_data[file_y_offset + file_x_offset - strlen("+--o") - 1 - (1+i)*file_width] = '|';

      /* Transforme le - en + */
      if(file_data[file_y_offset + file_x_offset - strlen("+--o") - 1 - (1+i)*file_width] == '-')
        file_data[file_y_offset + file_x_offset - strlen("+--o") - 1 - (1+i)*file_width] = '+';
    }

  /** On va descendre dans les Sous Fonctions (rcursivit) **/
  for(current_sub_function = current_function->first_sub_function; current_sub_function; current_sub_function = current_sub_function->next)
    ComputeProfileFileFill(current_sub_function,file_width,file_height,file_data,hide_interrupt,current_profile_table);
}


/******************************************************************/
/*  UserResizeProfileWindow() :  Retaille la fentre des Profile. */
/******************************************************************/
static void UserResizeProfileWindow(struct window_profile *window_profile)
{
  RECT window_rect;
  int window_x, window_y, window_new_height, minimum_window_height;
  int listview_pixel_height, nb_listview_data_row, delta_listview_nb_row, delta_pixel_height;

  /* Position + Dimension actuelle de la fentre retaille dans l'cran */
  GetWindowRect(window_profile->hwnd_profile,&window_rect);

  /* Nouvelle Dimension de la fentre (avec la barre de titre et les bordures) */
  window_x = window_rect.left;
  window_y = window_rect.top;
  window_new_height = window_rect.bottom - window_rect.top;

  /* Nombre de Pixels restant pour la ListView */
  minimum_window_height = window_profile->window_init_height - window_profile->nb_init_data_row*SYMBOL_LISTVIEW_ROW_HEIGHT;
  listview_pixel_height = window_new_height - minimum_window_height;

  /* Nombre de lignes de la ListView */
  nb_listview_data_row = listview_pixel_height / SYMBOL_LISTVIEW_ROW_HEIGHT;

  /** Teste les valeurs minimales autorises **/
  if(nb_listview_data_row < SYMBOL_LISTVIEW_MIN_NB_ROW)
    nb_listview_data_row = SYMBOL_LISTVIEW_MIN_NB_ROW;

  /* Delta de vla ListView */
  delta_listview_nb_row = nb_listview_data_row - window_profile->nb_row;
  delta_pixel_height = delta_listview_nb_row*SYMBOL_LISTVIEW_ROW_HEIGHT;
  window_profile->nb_row = nb_listview_data_row;

  /* Nouvelle hauteur de la fentre (on ajuste pour tomber sur un nombre extact de ligne dans la ListView) */
  window_new_height = minimum_window_height + nb_listview_data_row*SYMBOL_LISTVIEW_ROW_HEIGHT;

  /* Retaille la fentre */
  window_profile->user_resize = 0;
  MoveWindow(window_profile->hwnd_profile,window_x,window_y,window_profile->window_init_width,window_new_height,FALSE);

  /* Dplace les Contrles */
  MoveProfileWindowControls(delta_pixel_height,window_profile);

  /* Rafraichit la fentre */
  InvalidateRect(window_profile->hwnd_profile,NULL,TRUE);
  UpdateWindow(window_profile->hwnd_profile);
}


/******************************************************************************************/
/*  MoveProfileWindowControls() :  On va dplacer les diffrents controles de la fentre. */
/******************************************************************************************/
static void MoveProfileWindowControls(int delta_y, struct window_profile *window_profile)
{
  HWND hwnd_control, hwnd_listview;
  int i, control_x, control_y, control_width, control_height;
  int listview_x, listview_y, listview_width, listview_height;
  int tab_control_id[] = 
    {
      IDC_STATIC_REFRESH_MODE,  IDC_RADIO_REFRESH_AUTOMATIC, IDC_RADIO_REFRESH_MANUAL,    IDC_BUTTON_REFRESH,
      IDC_BUTTON_PROFILE_CLEAR, IDC_BUTTON_PROFILE_SAVE,     IDC_BUTTON_DELETE_FUNCTION,  IDC_CHECK_HIDE_INTERRUPT,
      IDC_STATIC_TOTAL,         IDC_STATIC_TOTAL_CYCLE,      IDC_BUTTON_PROFILE_BACKWARD, IDC_BUTTON_PROFILE_FORWARD,
      0
    };

  /** On va modifier la hauteur de la ListBox **/
  /* Handle du Controle */
  hwnd_listview = window_profile->listview_profile.hwnd_listview;
  /* Position + Taille actuelle */
  GetWindowControlInfo(window_profile->hwnd_profile,hwnd_listview,&listview_x,&listview_y,&listview_width,&listview_height);
  /* Change la taille le controle verticalement */
  MoveWindow(hwnd_listview,listview_x,listview_y,listview_width,listview_height+delta_y,FALSE);

  /** On va dplacer tous les controles verticalement **/
  for(i=0; tab_control_id[i] != 0; i++)
    {
      /* Handle du Controle */
      hwnd_control = GetDlgItem(window_profile->hwnd_profile,tab_control_id[i]);

      /* Position + Taille actuelle */
      GetWindowControlInfo(window_profile->hwnd_profile,hwnd_control,&control_x,&control_y,&control_width,&control_height);

      /* Dplace le controle verticalement */
      MoveWindow(hwnd_control,control_x,control_y+delta_y,control_width,control_height,FALSE);
    }
}


/*********************************************************************/
/*  DeleteProfileChildWindows() :  Destruction des fentres enfants. */
/*********************************************************************/
static void DeleteProfileChildWindows(struct window_profile *window_profile)
{
}


/*********************************************************************/
/*  compare_function() : Fonction de comparaison pour le Quick Sort. */
/*********************************************************************/
static int compare_function(const void *data_1, const void *data_2)
{
  struct profile_function *function_1;
  struct profile_function *function_2;

  /* Rcupration des paramtres */
  function_1 = *((struct profile_function **) data_1);
  function_2 = *((struct profile_function **) data_2);

  /* Comparaison des Profondeurs */
  if(function_1->depth < function_2->depth)
    return(1);
  else if(function_1->depth > function_2->depth)
    return(-1);
  else
    return(0);
}


/**********************************************************************************/
/*  mem_free_profile_table() :  Libration mmoire de la structure profile_table. */
/**********************************************************************************/
void mem_free_profile_table(struct profile_table *current_profile_table)
{
  struct profile_stack *current_profile_stack;
  struct profile_stack *next_profile_stack;

  if(current_profile_table)
    {
      /* Libration de la liste chaine Profile Function */
      mem_free_profile_function(current_profile_table->first_profile_function);

      /* Libration de la liste chaine Profile Stack */
      for(current_profile_stack = current_profile_table->first_profile_stack; current_profile_stack; current_profile_stack = next_profile_stack)
        {
          next_profile_stack = current_profile_stack->next;
          free(current_profile_stack);
        }

      free(current_profile_table);
    }
}


/****************************************************************************************/
/*  mem_free_profile_function() :  Libration mmoire de la structure profile_function. */
/****************************************************************************************/
static void mem_free_profile_function(struct profile_function *current_profile_function)
{
  struct profile_function *current_sub_function;
  struct profile_function *next_sub_function;

  if(current_profile_function)
    {
      /* Libration de la liste chaine Profile Function des Sous Fonctions (Rcursivit) */
      for(current_sub_function = current_profile_function->first_sub_function; current_sub_function; current_sub_function = next_sub_function)
        {
          next_sub_function = current_sub_function->next;

          /* Appel Rcursif */
          mem_free_profile_function(current_sub_function);
        }

      free(current_profile_function);
    }
}


/***********************************************************************/
/*  mem_free_window_profile() :  Libration mmoire pour la structure. */
/***********************************************************************/
void mem_free_window_profile(struct window_profile *window_profile)
{
  if(window_profile)
    {
      /* Supprime la notification */
      my_Notification(NOTIFICATION_REMOVE,window_profile->current_notification,NULL);
      window_profile->current_notification = NULL;

      free(window_profile);
    }
}

/**********************************************************************/
