/**************************************************************************/
/*                                                                        */
/*  Window_ProgressBar.c :  Module de gestion de la fentre ProgressBar.  */
/*                                                                        */
/**************************************************************************/
/*  Auteur : Olivier ZARDINI   *    Brutal Deluxe    *   Date : Sep 2024  */
/**************************************************************************/

#include <stdio.h>
#include <time.h>

#include <windows.h>
#include <windowsx.h>
#include <Commctrl.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Window_QuickWatch.h"
#include "Window_Main.h"
#include "Window_ProgressBar.h"

static BOOL CALLBACK ProgressBarWindowProc(HWND,UINT,WPARAM,LPARAM);
static void CreateProgressBarChildWindows(HWND,struct window_progressbar *);
LRESULT CALLBACK subEditProgressBarProcControl(HWND,UINT,WPARAM,LPARAM);

/*************************************************************************************/
/*  ProgressBarWindow() :  Cration d'une fentre ProgressBar Modale pour la saisie. */
/*************************************************************************************/
int ProgressBarWindow(HWND hwndParent, char *title, char *prompt, int max_value)
{
  int result;
  struct window_progressbar *window_progressbar;
  struct window_main *window_main;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);

  /* On passe par Param pour valeurs */
  window_progressbar = (struct window_progressbar *) calloc(1,sizeof(struct window_progressbar));
  if(window_progressbar == NULL)
    return(1);
  strcpy(window_progressbar->title,title);
  strcpy(window_progressbar->prompt,prompt);
  window_progressbar->max_value = max_value;
  my_Memory(MEMORY_SET_WINDOW_PROGRESSBAR,window_progressbar,NULL);

  /** Cration de la fentre DialogBox Modale **/
  result = (int) DialogBoxParamA(param->hInstance,MAKEINTRESOURCE(IDD_WINDOW_PROGRESSBAR),hwndParent,(DLGPROC)ProgressBarWindowProc,(LPARAM)window_progressbar);

  /* Libration mmoire */
  EnterCriticalSection(&param->cs_progress_bar);
  my_Memory(MEMORY_FREE_WINDOW_PROGRESSBAR,NULL,NULL);
  LeaveCriticalSection(&param->cs_progress_bar);

  /* OK */
  return(0);
}


/*********************************************************************/
/*  ProgressBarWindowProc() :  Gestion des vnements de la fentre. */
/*********************************************************************/
static BOOL CALLBACK ProgressBarWindowProc(HWND hwndProgressBar, UINT message, WPARAM wParam, LPARAM lParam) 
{
  struct window_progressbar *window_progressbar;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  my_Memory(MEMORY_GET_WINDOW_PROGRESSBAR,&window_progressbar,NULL);

  switch(message) 
    { 
      /****************************/
      /* Initialization de window */
      /****************************/
      case WM_INITDIALOG :
        window_progressbar = (struct window_progressbar *) lParam;
        CreateProgressBarChildWindows(hwndProgressBar,window_progressbar);
        return(TRUE);

      case HM_PROGRESSBAR_BUTTON_INCREMENT :
        /* Increment Progress Bar */
        SendMessage(GetDlgItem(window_progressbar->hwnd_progressbar,IDC_PROGRESS_BAR),PBM_STEPIT,0L,0L);
        return(TRUE);

      case HM_PROGRESSBAR_UPDATE_PROMPT :
        /* Prompt de la fentre */
        SetWindowText(GetDlgItem(window_progressbar->hwnd_progressbar,IDC_STATIC_TEXT),(char *)lParam);
        return(TRUE);

      case HM_PROGRESSBAR_BUTTON_CANCEL :
        /* Fermeture de la DialogBox */
        EndDialog(hwndProgressBar,IDC_BUTTON_CANCEL); 
        return(TRUE);

      case WM_COMMAND: 
        switch(LOWORD(wParam)) 
          { 
            case IDC_BUTTON_CANCEL :
              /* Fermeture de la DialogBox */
              EndDialog(hwndProgressBar,IDC_BUTTON_CANCEL); 
              return(TRUE);              
          } 
    } 

  /* Le Message n'a pas t trait, on l'envoie  la fentre pre */
  return(FALSE);
} 


/*******************************************************************/
/*  CreateProgressBarChildWindows() :  Crer les fentres enfants. */
/*******************************************************************/
static void CreateProgressBarChildWindows(HWND hwndProgressBar, struct window_progressbar *window_progressbar)
{
  RECT window_main_rect, window_progressbar_rect;
  int window_main_width, window_main_height, window_progressbar_width, window_progressbar_height, window_progressbar_x, window_progressbar_y;
  struct window_main *window_main;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Fentre principale */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);

  /* Init */
  window_progressbar->hwnd_progressbar = hwndProgressBar;

  /* Titre de le fentre */
  SetWindowText(hwndProgressBar,window_progressbar->title);

  /* Prompt de la fentre */
  SetWindowText(GetDlgItem(window_progressbar->hwnd_progressbar,IDC_STATIC_TEXT),window_progressbar->prompt);

  /** Valeur Max de la Progress Bar **/
  SendMessage(GetDlgItem(window_progressbar->hwnd_progressbar,IDC_PROGRESS_BAR),PBM_SETRANGE32,0L,window_progressbar->max_value);
  SendMessage(GetDlgItem(window_progressbar->hwnd_progressbar,IDC_PROGRESS_BAR),PBM_SETSTEP,1,0L);
  SendMessage(GetDlgItem(window_progressbar->hwnd_progressbar,IDC_PROGRESS_BAR),PBM_SETPOS,0,0L);    /* Remet  zero */  

  /** Centre la fentre par rapport  la fentre parent **/
  /* Position / Taille de la fentre Main */
  GetWindowRect(window_main->hwnd_main,&window_main_rect);
  window_main_width = (window_main_rect.right - window_main_rect.left);
  window_main_height = (window_main_rect.bottom - window_main_rect.top);
  /* Position / Taille de la fentre ProgressBar */
  GetWindowRect(window_progressbar->hwnd_progressbar,&window_progressbar_rect);
  window_progressbar_width = (window_progressbar_rect.right - window_progressbar_rect.left);
  window_progressbar_height = (window_progressbar_rect.bottom - window_progressbar_rect.top);
  /* Nouvelle position */
  window_progressbar_x = window_main_rect.left + window_main_width/2 - window_progressbar_width/2;
  window_progressbar_y = window_main_rect.top + window_main_height/2 - window_progressbar_height/2;
  /* Recentre la fentre */
  MoveWindow(window_progressbar->hwnd_progressbar,window_progressbar_x,window_progressbar_y,window_progressbar_width,window_progressbar_height,FALSE);
}


/********************************************************************************************/
/*  subEditProgressBarProcControl() :  Rcupre le RETURN pour valider les donnes saisies. */
/********************************************************************************************/
void mem_free_window_progressbar(struct window_progressbar *window_progressbar)
{
  if(window_progressbar)
    free(window_progressbar);
}

/****************************************************************************/
