/************************************************************************/
/*                                                                      */
/*  Window_QuickWatch.c :  Module de gestion de la fentre QuickWatch.  */
/*                                                                      */
/************************************************************************/
/*  Auteur : Olivier ZARDINI  *   Brutal Deluxe   *   Date : Janv 2019  */
/************************************************************************/

#include <stdio.h>
#include <time.h>

#include <windows.h>
#include <windowsx.h>
#include <Commctrl.h>
#include <Winuser.h>
#include <Richedit.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_Notification.h"
#include "Dc_Graphic.h"
#include "Dc_CPU.h"
#include "Dc_MMU.h"
#include "Window_QuickWatch.h"
#include "Window_Main.h"
#include "Window_Main_Menu.h"
#include "Window_Symbol.h"
#include "Window_SymbolPickup.h"

static DWORD BuildQuickWatchWindowThread(void *);
LRESULT CALLBACK ProcHandlerQuickWatchWindow(HWND,UINT,WPARAM,LPARAM);
LRESULT CALLBACK subEditQuickWatchProcControl(HWND,UINT,WPARAM,LPARAM);
static int CreateQuickWatchChildWindows(HWND,struct window_quickwatch *);
static void UpdateQuickWatchRefreshMode(struct window_quickwatch *,int);
static void UpdateQuickWatchDataLengthAlignment(struct window_quickwatch *,int,int);
static void BuildQuickWatchFromData(struct window_quickwatch *);
static void FormatWordAsTextInfo(WORD,int,char *);
static void SaveQuickWatchDataFile(struct window_quickwatch *);
static void DeleteQuickWatchChildWindows(struct window_quickwatch *);


/**********************************************************************************/
/*  BuildQuickWatchWindow() :  Construction / Affichage de la fentre QuickWatch. */
/**********************************************************************************/
struct window_quickwatch *BuildQuickWatchWindow(HINSTANCE hInstance, int menu_id)
{
  struct window_quickwatch *window_quickwatch;

  /** Allocation de la structure window_quickwatch **/
  window_quickwatch = (struct window_quickwatch *) calloc(1,sizeof(struct window_quickwatch));
  if(window_quickwatch == NULL)
    return(NULL);
  my_Memory(MEMORY_SET_WINDOW_QUICKWATCH,window_quickwatch,NULL);

  /* Lance un thread pour grer la fentre QuickWatch */
  CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)BuildQuickWatchWindowThread,window_quickwatch,0,NULL);

  /* Renvoi la structure mmoire grant la fentre */
  return(window_quickwatch);
}


/********************************************************************************************/
/*  BuildQuickWatchWindowThread() :  Cration de la fentre QuickWatch via un Thread ddi. */
/********************************************************************************************/
static DWORD BuildQuickWatchWindowThread(void *window_quickwatch_param)
{
  HWND hwndQuickWatch;
  MSG msg;
  BOOL bRet;
  struct window_main *window_main;
  struct window_quickwatch *window_quickwatch;
  struct parameter *param;
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Init */
  window_quickwatch = (struct window_quickwatch *) window_quickwatch_param;

  /** Cration de la fentre QuickWatch **/
  hwndQuickWatch = CreateDialog(param->hInstance,MAKEINTRESOURCE(IDD_WINDOW_QUICKWATCH),window_main->hwnd_main,(DLGPROC)ProcHandlerQuickWatchWindow);
  if(hwndQuickWatch == NULL)
    return(1);

  /** Show Window + Paint Content **/
  ShowWindow(hwndQuickWatch,SW_SHOWDEFAULT);
  UpdateWindow(hwndQuickWatch);

  /** Message Loop pour toutes les fentres de ce Thread (fentre QuickWatch...) **/
  while((bRet = GetMessage(&msg,NULL,0,0)) != 0)
    {
      if(bRet == -1)
        break;

      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }

  /* Fin du Thread */
  ExitThread(0);
}


/***************************************************************************/
/*  ProcHandlerQuickWatchWindow() :  Gestion des vnements de la fentre. */
/***************************************************************************/
LRESULT CALLBACK ProcHandlerQuickWatchWindow(HWND hwndQuickWatch, UINT umsg, WPARAM wParam, LPARAM lParam)
{
  LRESULT result;
  char buffer[1024];
  struct window_main *window_main;
  struct window_quickwatch *window_quickwatch;
  struct parameter *param;
  
  /* Init */
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  my_Memory(MEMORY_GET_WINDOW_QUICKWATCH,&window_quickwatch,NULL);

  switch(umsg)
    {
      case WM_INITDIALOG :
        /* Cration du contenu de la DialogBox */
        CreateQuickWatchChildWindows(hwndQuickWatch,window_quickwatch);
        return(TRUE);
        
      /****************************/
      /* Evnements de la fentre */
      /****************************/
      case WM_COMMAND :
        switch(LOWORD(wParam))
          {
            /** Bouton Radio : On change la mthode de rafraichissement de la fentre **/
            case IDC_RADIO_REFRESH_AUTOMATIC :
              if(HIWORD(wParam) == BN_CLICKED && window_quickwatch->refresh_mode != QUICKWATCH_REFRESH_AUTOMATIC)
                UpdateQuickWatchRefreshMode(window_quickwatch,QUICKWATCH_REFRESH_AUTOMATIC);
              break;

            case IDC_RADIO_REFRESH_MANUAL :
              if(HIWORD(wParam) == BN_CLICKED && window_quickwatch->refresh_mode != QUICKWATCH_REFRESH_MANUAL)
                UpdateQuickWatchRefreshMode(window_quickwatch,QUICKWATCH_REFRESH_MANUAL);
              break;

            /** Bouton de rafraichissement Manuel de la fentre **/
            case IDC_BUTTON_REFRESH :
              RefreshQuickWatchWindowData(window_quickwatch);
              break;

            /** Bouton de Chargement / Sauvegarde des QuickWatch **/
            case IDC_BUTTON_LOAD_QUICKWATCH :
              LoadQuickWatchDataFile(NULL,window_quickwatch);
              break;
            case IDC_BUTTON_SAVE_QUICKWATCH :
              SaveQuickWatchDataFile(window_quickwatch);
              break;

            /** Bouton Radio : On change le type de la Ram **/
            case IDC_RADIO_RAM1 :
              if(HIWORD(wParam) == BN_CLICKED && window_main->tab_quickwatch_data[0].ram_type != QUICKWATCH_RAMTYPE_RAM)
                UpdateQuickWatchRamType(window_quickwatch,0,QUICKWATCH_RAMTYPE_RAM,1);
              break;
            case IDC_RADIO_DOCRAM1 :
              if(HIWORD(wParam) == BN_CLICKED && window_main->tab_quickwatch_data[0].ram_type != QUICKWATCH_RAMTYPE_DOCRAM)
                UpdateQuickWatchRamType(window_quickwatch,0,QUICKWATCH_RAMTYPE_DOCRAM,1);
              break;

            case IDC_RADIO_RAM2 :
              if(HIWORD(wParam) == BN_CLICKED && window_main->tab_quickwatch_data[1].ram_type != QUICKWATCH_RAMTYPE_RAM)
                UpdateQuickWatchRamType(window_quickwatch,1,QUICKWATCH_RAMTYPE_RAM,1);
              break;
            case IDC_RADIO_DOCRAM2 :
              if(HIWORD(wParam) == BN_CLICKED && window_main->tab_quickwatch_data[1].ram_type != QUICKWATCH_RAMTYPE_DOCRAM)
                UpdateQuickWatchRamType(window_quickwatch,1,QUICKWATCH_RAMTYPE_DOCRAM,1);
              break;

            case IDC_RADIO_RAM3 :
              if(HIWORD(wParam) == BN_CLICKED && window_main->tab_quickwatch_data[2].ram_type != QUICKWATCH_RAMTYPE_RAM)
                UpdateQuickWatchRamType(window_quickwatch,2,QUICKWATCH_RAMTYPE_RAM,1);
              break;
            case IDC_RADIO_DOCRAM3 :
              if(HIWORD(wParam) == BN_CLICKED && window_main->tab_quickwatch_data[2].ram_type != QUICKWATCH_RAMTYPE_DOCRAM)
                UpdateQuickWatchRamType(window_quickwatch,2,QUICKWATCH_RAMTYPE_DOCRAM,1);
              break;

            case IDC_RADIO_RAM4 :
              if(HIWORD(wParam) == BN_CLICKED && window_main->tab_quickwatch_data[3].ram_type != QUICKWATCH_RAMTYPE_RAM)
                UpdateQuickWatchRamType(window_quickwatch,3,QUICKWATCH_RAMTYPE_RAM,1);
              break;
            case IDC_RADIO_DOCRAM4 :
              if(HIWORD(wParam) == BN_CLICKED && window_main->tab_quickwatch_data[3].ram_type != QUICKWATCH_RAMTYPE_DOCRAM)
                UpdateQuickWatchRamType(window_quickwatch,3,QUICKWATCH_RAMTYPE_DOCRAM,1);
              break;

            case IDC_RADIO_RAM5 :
              if(HIWORD(wParam) == BN_CLICKED && window_main->tab_quickwatch_data[4].ram_type != QUICKWATCH_RAMTYPE_RAM)
                UpdateQuickWatchRamType(window_quickwatch,4,QUICKWATCH_RAMTYPE_RAM,1);
              break;
            case IDC_RADIO_DOCRAM5 :
              if(HIWORD(wParam) == BN_CLICKED && window_main->tab_quickwatch_data[4].ram_type != QUICKWATCH_RAMTYPE_DOCRAM)
                UpdateQuickWatchRamType(window_quickwatch,4,QUICKWATCH_RAMTYPE_DOCRAM,1);
              break;

            case IDC_RADIO_RAM6 :
              if(HIWORD(wParam) == BN_CLICKED && window_main->tab_quickwatch_data[5].ram_type != QUICKWATCH_RAMTYPE_RAM)
                UpdateQuickWatchRamType(window_quickwatch,5,QUICKWATCH_RAMTYPE_RAM,1);
              break;
            case IDC_RADIO_DOCRAM6 :
              if(HIWORD(wParam) == BN_CLICKED && window_main->tab_quickwatch_data[5].ram_type != QUICKWATCH_RAMTYPE_DOCRAM)
                UpdateQuickWatchRamType(window_quickwatch,5,QUICKWATCH_RAMTYPE_DOCRAM,1);
              break;

            case IDC_RADIO_RAM7 :
              if(HIWORD(wParam) == BN_CLICKED && window_main->tab_quickwatch_data[6].ram_type != QUICKWATCH_RAMTYPE_RAM)
                UpdateQuickWatchRamType(window_quickwatch,6,QUICKWATCH_RAMTYPE_RAM,1);
              break;
            case IDC_RADIO_DOCRAM7 :
              if(HIWORD(wParam) == BN_CLICKED && window_main->tab_quickwatch_data[6].ram_type != QUICKWATCH_RAMTYPE_DOCRAM)
                UpdateQuickWatchRamType(window_quickwatch,6,QUICKWATCH_RAMTYPE_DOCRAM,1);
              break;

            case IDC_RADIO_RAM8 :
              if(HIWORD(wParam) == BN_CLICKED && window_main->tab_quickwatch_data[7].ram_type != QUICKWATCH_RAMTYPE_RAM)
                UpdateQuickWatchRamType(window_quickwatch,7,QUICKWATCH_RAMTYPE_RAM,1);
              break;
            case IDC_RADIO_DOCRAM8 :
              if(HIWORD(wParam) == BN_CLICKED && window_main->tab_quickwatch_data[7].ram_type != QUICKWATCH_RAMTYPE_DOCRAM)
                UpdateQuickWatchRamType(window_quickwatch,7,QUICKWATCH_RAMTYPE_DOCRAM,1);
              break;

            case IDC_RADIO_RAM9 :
              if(HIWORD(wParam) == BN_CLICKED && window_main->tab_quickwatch_data[8].ram_type != QUICKWATCH_RAMTYPE_RAM)
                UpdateQuickWatchRamType(window_quickwatch,8,QUICKWATCH_RAMTYPE_RAM,1);
              break;
            case IDC_RADIO_DOCRAM9 :
              if(HIWORD(wParam) == BN_CLICKED && window_main->tab_quickwatch_data[8].ram_type != QUICKWATCH_RAMTYPE_DOCRAM)
                UpdateQuickWatchRamType(window_quickwatch,8,QUICKWATCH_RAMTYPE_DOCRAM,1);
              break;

            case IDC_RADIO_RAM10 :
              if(HIWORD(wParam) == BN_CLICKED && window_main->tab_quickwatch_data[9].ram_type != QUICKWATCH_RAMTYPE_RAM)
                UpdateQuickWatchRamType(window_quickwatch,9,QUICKWATCH_RAMTYPE_RAM,1);
              break;
            case IDC_RADIO_DOCRAM10 :
              if(HIWORD(wParam) == BN_CLICKED && window_main->tab_quickwatch_data[9].ram_type != QUICKWATCH_RAMTYPE_DOCRAM)
                UpdateQuickWatchRamType(window_quickwatch,9,QUICKWATCH_RAMTYPE_DOCRAM,1);
              break;

            /** Bouton de slection de l'adresse via Symbol **/
            case IDC_BUTTON_SYMBOL1 :
              BuildSymbolPickupWindow(param->hInstance,window_quickwatch->hwnd_quickwatch,HM_QUICKWATCH_REFRESHSYMBOL1);
              break;
            case IDC_BUTTON_SYMBOL2 :
              BuildSymbolPickupWindow(param->hInstance,window_quickwatch->hwnd_quickwatch,HM_QUICKWATCH_REFRESHSYMBOL2);
              break;
            case IDC_BUTTON_SYMBOL3 :
              BuildSymbolPickupWindow(param->hInstance,window_quickwatch->hwnd_quickwatch,HM_QUICKWATCH_REFRESHSYMBOL3);
              break;
            case IDC_BUTTON_SYMBOL4 :
              BuildSymbolPickupWindow(param->hInstance,window_quickwatch->hwnd_quickwatch,HM_QUICKWATCH_REFRESHSYMBOL4);
              break;
            case IDC_BUTTON_SYMBOL5 :
              BuildSymbolPickupWindow(param->hInstance,window_quickwatch->hwnd_quickwatch,HM_QUICKWATCH_REFRESHSYMBOL5);
              break;
            case IDC_BUTTON_SYMBOL6 :
              BuildSymbolPickupWindow(param->hInstance,window_quickwatch->hwnd_quickwatch,HM_QUICKWATCH_REFRESHSYMBOL6);
              break;
            case IDC_BUTTON_SYMBOL7 :
              BuildSymbolPickupWindow(param->hInstance,window_quickwatch->hwnd_quickwatch,HM_QUICKWATCH_REFRESHSYMBOL7);
              break;
            case IDC_BUTTON_SYMBOL8 :
              BuildSymbolPickupWindow(param->hInstance,window_quickwatch->hwnd_quickwatch,HM_QUICKWATCH_REFRESHSYMBOL8);
              break;
            case IDC_BUTTON_SYMBOL9 :
              BuildSymbolPickupWindow(param->hInstance,window_quickwatch->hwnd_quickwatch,HM_QUICKWATCH_REFRESHSYMBOL9);
              break;
            case IDC_BUTTON_SYMBOL10 :
              BuildSymbolPickupWindow(param->hInstance,window_quickwatch->hwnd_quickwatch,HM_QUICKWATCH_REFRESHSYMBOL10);
              break;

            /** Slection dans les ComboBox Data Length Alignment **/
            case IDC_COMBOBOX_QUICKWATCH_DATALENGTH_1 :
              if(HIWORD(wParam) == CBN_CLOSEUP)
                {
                  /* Rcupre l'index de ce qui est slectionn */
                  result = SendMessage(window_quickwatch->tab_hwnd_combo_length_alignment[0],CB_GETCURSEL,0,0);
                  if(result != CB_ERR)
                    if(result != window_main->tab_quickwatch_data[0].combo_length_alignment)
                      UpdateQuickWatchDataLengthAlignment(window_quickwatch,1,(int)result);
                }
              break;

            case IDC_COMBOBOX_QUICKWATCH_DATALENGTH_2 :
              if(HIWORD(wParam) == CBN_CLOSEUP)
                {
                  /* Rcupre l'index de ce qui est slectionn */
                  result = SendMessage(window_quickwatch->tab_hwnd_combo_length_alignment[1],CB_GETCURSEL,0,0);
                  if(result != CB_ERR)
                    if(result != window_main->tab_quickwatch_data[1].combo_length_alignment)
                      UpdateQuickWatchDataLengthAlignment(window_quickwatch,2,(int)result);
                }
              break;

            case IDC_COMBOBOX_QUICKWATCH_DATALENGTH_3 :
              if(HIWORD(wParam) == CBN_CLOSEUP)
                {
                  /* Rcupre l'index de ce qui est slectionn */
                  result = SendMessage(window_quickwatch->tab_hwnd_combo_length_alignment[2],CB_GETCURSEL,0,0);
                  if(result != CB_ERR)
                    if(result != window_main->tab_quickwatch_data[2].combo_length_alignment)
                      UpdateQuickWatchDataLengthAlignment(window_quickwatch,3,(int)result);
                }
              break;

            case IDC_COMBOBOX_QUICKWATCH_DATALENGTH_4 :
              if(HIWORD(wParam) == CBN_CLOSEUP)
                {
                  /* Rcupre l'index de ce qui est slectionn */
                  result = SendMessage(window_quickwatch->tab_hwnd_combo_length_alignment[3],CB_GETCURSEL,0,0);
                  if(result != CB_ERR)
                    if(result != window_main->tab_quickwatch_data[3].combo_length_alignment)
                      UpdateQuickWatchDataLengthAlignment(window_quickwatch,4,(int)result);
                }
              break;

            case IDC_COMBOBOX_QUICKWATCH_DATALENGTH_5 :
              if(HIWORD(wParam) == CBN_CLOSEUP)
                {
                  /* Rcupre l'index de ce qui est slectionn */
                  result = SendMessage(window_quickwatch->tab_hwnd_combo_length_alignment[4],CB_GETCURSEL,0,0);
                  if(result != CB_ERR)
                    if(result != window_main->tab_quickwatch_data[4].combo_length_alignment)
                      UpdateQuickWatchDataLengthAlignment(window_quickwatch,5,(int)result);
                }
              break;

            case IDC_COMBOBOX_QUICKWATCH_DATALENGTH_6 :
              if(HIWORD(wParam) == CBN_CLOSEUP)
                {
                  /* Rcupre l'index de ce qui est slectionn */
                  result = SendMessage(window_quickwatch->tab_hwnd_combo_length_alignment[5],CB_GETCURSEL,0,0);
                  if(result != CB_ERR)
                    if(result != window_main->tab_quickwatch_data[5].combo_length_alignment)
                      UpdateQuickWatchDataLengthAlignment(window_quickwatch,6,(int)result);
                }
              break;

            case IDC_COMBOBOX_QUICKWATCH_DATALENGTH_7 :
              if(HIWORD(wParam) == CBN_CLOSEUP)
                {
                  /* Rcupre l'index de ce qui est slectionn */
                  result = SendMessage(window_quickwatch->tab_hwnd_combo_length_alignment[6],CB_GETCURSEL,0,0);
                  if(result != CB_ERR)
                    if(result != window_main->tab_quickwatch_data[6].combo_length_alignment)
                      UpdateQuickWatchDataLengthAlignment(window_quickwatch,7,(int)result);
                }
              break;

            case IDC_COMBOBOX_QUICKWATCH_DATALENGTH_8 :
              if(HIWORD(wParam) == CBN_CLOSEUP)
                {
                  /* Rcupre l'index de ce qui est slectionn */
                  result = SendMessage(window_quickwatch->tab_hwnd_combo_length_alignment[7],CB_GETCURSEL,0,0);
                  if(result != CB_ERR)
                    if(result != window_main->tab_quickwatch_data[7].combo_length_alignment)
                      UpdateQuickWatchDataLengthAlignment(window_quickwatch,8,(int)result);
                }
              break;

            case IDC_COMBOBOX_QUICKWATCH_DATALENGTH_9 :
              if(HIWORD(wParam) == CBN_CLOSEUP)
                {
                  /* Rcupre l'index de ce qui est slectionn */
                  result = SendMessage(window_quickwatch->tab_hwnd_combo_length_alignment[8],CB_GETCURSEL,0,0);
                  if(result != CB_ERR)
                    if(result != window_main->tab_quickwatch_data[8].combo_length_alignment)
                      UpdateQuickWatchDataLengthAlignment(window_quickwatch,9,(int)result);
                }
              break;

            case IDC_COMBOBOX_QUICKWATCH_DATALENGTH_10 :
              if(HIWORD(wParam) == CBN_CLOSEUP)
                {
                  /* Rcupre l'index de ce qui est slectionn */
                  result = SendMessage(window_quickwatch->tab_hwnd_combo_length_alignment[9],CB_GETCURSEL,0,0);
                  if(result != CB_ERR)
                    if(result != window_main->tab_quickwatch_data[9].combo_length_alignment)
                      UpdateQuickWatchDataLengthAlignment(window_quickwatch,10,(int)result);
                }
              break;
          }
        return(TRUE);

      /*****************************************************************/
      /* Message extrieur demandant le rafraichissement de la fentre */
      /*****************************************************************/
      case HM_QUICKWATCH_REFRESHWINDOW :
        RefreshQuickWatchWindowData(window_quickwatch);
        return(TRUE);

      /** On vient de demander la MAJ de l'adresse via un Symbol **/
      case HM_QUICKWATCH_REFRESHSYMBOL1 :
        if(param->symbol_is_available == 1)
          {
            /* Conserve le Symbol */
            window_main->tab_quickwatch_data[0].has_symbol = 1;
            window_main->tab_quickwatch_data[0].symbol_address = param->symbol_address;
            strcpy(window_main->tab_quickwatch_data[0].symbol_name,param->symbol_name);
            strcpy(window_main->tab_quickwatch_data[0].symbol_segment,param->symbol_segment);
            strcpy(window_main->tab_quickwatch_data[0].symbol_file,param->symbol_file);
            strcpy(window_main->tab_quickwatch_data[0].name,param->symbol_name);
            /* MAJ L'affichage */
            window_main->tab_quickwatch_data[0].bank = (BYTE) ((param->symbol_address >> 16) & 0x000000FF);
            window_main->tab_quickwatch_data[0].address = (WORD) (param->symbol_address & 0x0000FFFF);
            sprintf(buffer,"%02X",window_main->tab_quickwatch_data[0].bank);
            SetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_BANK1),buffer);
            sprintf(buffer,"%04X",window_main->tab_quickwatch_data[0].address);
            SetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_ADDRESS1),buffer);
            SetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_LABEL1),param->symbol_name);
            RefreshQuickWatchWindowData(window_quickwatch);
          }
        return(TRUE);
      case HM_QUICKWATCH_REFRESHSYMBOL2 :
        if(param->symbol_is_available == 1)
          {
            /* Conserve le Symbol */
            window_main->tab_quickwatch_data[1].has_symbol = 1;
            window_main->tab_quickwatch_data[1].symbol_address = param->symbol_address;
            strcpy(window_main->tab_quickwatch_data[1].symbol_name,param->symbol_name);
            strcpy(window_main->tab_quickwatch_data[1].symbol_segment,param->symbol_segment);
            strcpy(window_main->tab_quickwatch_data[1].symbol_file,param->symbol_file);
            strcpy(window_main->tab_quickwatch_data[1].name,param->symbol_name);
            /* MAJ L'affichage */
            window_main->tab_quickwatch_data[1].bank = (BYTE) ((param->symbol_address >> 16) & 0x000000FF);
            window_main->tab_quickwatch_data[1].address = (WORD) (param->symbol_address & 0x0000FFFF);
            sprintf(buffer,"%02X",window_main->tab_quickwatch_data[1].bank);
            SetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_BANK2),buffer);
            sprintf(buffer,"%04X",window_main->tab_quickwatch_data[1].address);
            SetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_ADDRESS2),buffer);
            SetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_LABEL2),param->symbol_name);
            RefreshQuickWatchWindowData(window_quickwatch);
          }
        return(TRUE);
      case HM_QUICKWATCH_REFRESHSYMBOL3 :
        if(param->symbol_is_available == 1)
          {
            /* Conserve le Symbol */
            window_main->tab_quickwatch_data[2].has_symbol = 1;
            window_main->tab_quickwatch_data[2].symbol_address = param->symbol_address;
            strcpy(window_main->tab_quickwatch_data[2].symbol_name,param->symbol_name);
            strcpy(window_main->tab_quickwatch_data[2].symbol_segment,param->symbol_segment);
            strcpy(window_main->tab_quickwatch_data[2].symbol_file,param->symbol_file);
            strcpy(window_main->tab_quickwatch_data[2].name,param->symbol_name);
            /* MAJ L'affichage */
            window_main->tab_quickwatch_data[2].bank = (BYTE) ((param->symbol_address >> 16) & 0x000000FF);
            window_main->tab_quickwatch_data[2].address = (WORD) (param->symbol_address & 0x0000FFFF);
            sprintf(buffer,"%02X",window_main->tab_quickwatch_data[2].bank);
            SetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_BANK3),buffer);
            sprintf(buffer,"%04X",window_main->tab_quickwatch_data[2].address);
            SetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_ADDRESS3),buffer);
            SetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_LABEL3),param->symbol_name);
            RefreshQuickWatchWindowData(window_quickwatch);
          }
        return(TRUE);
      case HM_QUICKWATCH_REFRESHSYMBOL4 :
        if(param->symbol_is_available == 1)
          {
            /* Conserve le Symbol */
            window_main->tab_quickwatch_data[3].has_symbol = 1;
            window_main->tab_quickwatch_data[3].symbol_address = param->symbol_address;
            strcpy(window_main->tab_quickwatch_data[3].symbol_name,param->symbol_name);
            strcpy(window_main->tab_quickwatch_data[3].symbol_segment,param->symbol_segment);
            strcpy(window_main->tab_quickwatch_data[3].symbol_file,param->symbol_file);
            strcpy(window_main->tab_quickwatch_data[3].name,param->symbol_name);
            /* MAJ L'affichage */
            window_main->tab_quickwatch_data[3].bank = (BYTE) ((param->symbol_address >> 16) & 0x000000FF);
            window_main->tab_quickwatch_data[3].address = (WORD) (param->symbol_address & 0x0000FFFF);
            sprintf(buffer,"%02X",window_main->tab_quickwatch_data[3].bank);
            SetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_BANK4),buffer);
            sprintf(buffer,"%04X",window_main->tab_quickwatch_data[3].address);
            SetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_ADDRESS4),buffer);
            SetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_LABEL4),param->symbol_name);
            RefreshQuickWatchWindowData(window_quickwatch);
          }
        return(TRUE);
      case HM_QUICKWATCH_REFRESHSYMBOL5 :
        if(param->symbol_is_available == 1)
          {
            /* Conserve le Symbol */
            window_main->tab_quickwatch_data[4].has_symbol = 1;
            window_main->tab_quickwatch_data[4].symbol_address = param->symbol_address;
            strcpy(window_main->tab_quickwatch_data[4].symbol_name,param->symbol_name);
            strcpy(window_main->tab_quickwatch_data[4].symbol_segment,param->symbol_segment);
            strcpy(window_main->tab_quickwatch_data[4].symbol_file,param->symbol_file);
            strcpy(window_main->tab_quickwatch_data[4].name,param->symbol_name);
            /* MAJ L'affichage */
            window_main->tab_quickwatch_data[4].bank = (BYTE) ((param->symbol_address >> 16) & 0x000000FF);
            window_main->tab_quickwatch_data[4].address = (WORD) (param->symbol_address & 0x0000FFFF);
            sprintf(buffer,"%02X",window_main->tab_quickwatch_data[4].bank);
            SetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_BANK5),buffer);
            sprintf(buffer,"%04X",window_main->tab_quickwatch_data[4].address);
            SetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_ADDRESS5),buffer);
            SetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_LABEL5),param->symbol_name);
            RefreshQuickWatchWindowData(window_quickwatch);
          }
        return(TRUE);
      case HM_QUICKWATCH_REFRESHSYMBOL6 :
        if(param->symbol_is_available == 1)
          {
            /* Conserve le Symbol */
            window_main->tab_quickwatch_data[5].has_symbol = 1;
            window_main->tab_quickwatch_data[5].symbol_address = param->symbol_address;
            strcpy(window_main->tab_quickwatch_data[5].symbol_name,param->symbol_name);
            strcpy(window_main->tab_quickwatch_data[5].symbol_segment,param->symbol_segment);
            strcpy(window_main->tab_quickwatch_data[5].symbol_file,param->symbol_file);
            strcpy(window_main->tab_quickwatch_data[5].name,param->symbol_name);
            /* MAJ L'affichage */
            window_main->tab_quickwatch_data[5].bank = (BYTE) ((param->symbol_address >> 16) & 0x000000FF);
            window_main->tab_quickwatch_data[5].address = (WORD) (param->symbol_address & 0x0000FFFF);
            sprintf(buffer,"%02X",window_main->tab_quickwatch_data[5].bank);
            SetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_BANK6),buffer);
            sprintf(buffer,"%04X",window_main->tab_quickwatch_data[5].address);
            SetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_ADDRESS6),buffer);
            SetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_LABEL6),param->symbol_name);
            RefreshQuickWatchWindowData(window_quickwatch);
          }
        return(TRUE);
      case HM_QUICKWATCH_REFRESHSYMBOL7 :
        if(param->symbol_is_available == 1)
          {
            /* Conserve le Symbol */
            window_main->tab_quickwatch_data[6].has_symbol = 1;
            window_main->tab_quickwatch_data[6].symbol_address = param->symbol_address;
            strcpy(window_main->tab_quickwatch_data[6].symbol_name,param->symbol_name);
            strcpy(window_main->tab_quickwatch_data[6].symbol_segment,param->symbol_segment);
            strcpy(window_main->tab_quickwatch_data[6].symbol_file,param->symbol_file);
            strcpy(window_main->tab_quickwatch_data[6].name,param->symbol_name);
            /* MAJ L'affichage */
            window_main->tab_quickwatch_data[6].bank = (BYTE) ((param->symbol_address >> 16) & 0x000000FF);
            window_main->tab_quickwatch_data[6].address = (WORD) (param->symbol_address & 0x0000FFFF);
            sprintf(buffer,"%02X",window_main->tab_quickwatch_data[6].bank);
            SetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_BANK7),buffer);
            sprintf(buffer,"%04X",window_main->tab_quickwatch_data[6].address);
            SetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_ADDRESS7),buffer);
            SetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_LABEL7),param->symbol_name);
            RefreshQuickWatchWindowData(window_quickwatch);
          }
        return(TRUE);
      case HM_QUICKWATCH_REFRESHSYMBOL8 :
        if(param->symbol_is_available == 1)
          {
            /* Conserve le Symbol */
            window_main->tab_quickwatch_data[7].has_symbol = 1;
            window_main->tab_quickwatch_data[7].symbol_address = param->symbol_address;
            strcpy(window_main->tab_quickwatch_data[7].symbol_name,param->symbol_name);
            strcpy(window_main->tab_quickwatch_data[7].symbol_segment,param->symbol_segment);
            strcpy(window_main->tab_quickwatch_data[7].symbol_file,param->symbol_file);
            strcpy(window_main->tab_quickwatch_data[7].name,param->symbol_name);
            /* MAJ L'affichage */
            window_main->tab_quickwatch_data[7].bank = (BYTE) ((param->symbol_address >> 16) & 0x000000FF);
            window_main->tab_quickwatch_data[7].address = (WORD) (param->symbol_address & 0x0000FFFF);
            sprintf(buffer,"%02X",window_main->tab_quickwatch_data[7].bank);
            SetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_BANK8),buffer);
            sprintf(buffer,"%04X",window_main->tab_quickwatch_data[7].address);
            SetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_ADDRESS8),buffer);
            SetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_LABEL8),param->symbol_name);
            RefreshQuickWatchWindowData(window_quickwatch);
          }
        return(TRUE);
      case HM_QUICKWATCH_REFRESHSYMBOL9 :
        if(param->symbol_is_available == 1)
          {
            /* Conserve le Symbol */
            window_main->tab_quickwatch_data[8].has_symbol = 1;
            window_main->tab_quickwatch_data[8].symbol_address = param->symbol_address;
            strcpy(window_main->tab_quickwatch_data[8].symbol_name,param->symbol_name);
            strcpy(window_main->tab_quickwatch_data[8].symbol_segment,param->symbol_segment);
            strcpy(window_main->tab_quickwatch_data[8].symbol_file,param->symbol_file);
            strcpy(window_main->tab_quickwatch_data[8].name,param->symbol_name);
            /* MAJ L'affichage */
            window_main->tab_quickwatch_data[8].bank = (BYTE) ((param->symbol_address >> 16) & 0x000000FF);
            window_main->tab_quickwatch_data[8].address = (WORD) (param->symbol_address & 0x0000FFFF);
            sprintf(buffer,"%02X",window_main->tab_quickwatch_data[8].bank);
            SetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_BANK9),buffer);
            sprintf(buffer,"%04X",window_main->tab_quickwatch_data[8].address);
            SetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_ADDRESS9),buffer);
            SetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_LABEL9),param->symbol_name);
            RefreshQuickWatchWindowData(window_quickwatch);
          }
        return(TRUE);
      case HM_QUICKWATCH_REFRESHSYMBOL10 :
        if(param->symbol_is_available == 1)
          {
            /* Conserve le Symbol */
            window_main->tab_quickwatch_data[9].has_symbol = 1;
            window_main->tab_quickwatch_data[9].symbol_address = param->symbol_address;
            strcpy(window_main->tab_quickwatch_data[9].symbol_name,param->symbol_name);
            strcpy(window_main->tab_quickwatch_data[9].symbol_segment,param->symbol_segment);
            strcpy(window_main->tab_quickwatch_data[9].symbol_file,param->symbol_file);
            strcpy(window_main->tab_quickwatch_data[9].name,param->symbol_name);
            /* MAJ L'affichage */
            window_main->tab_quickwatch_data[9].bank = (BYTE) ((param->symbol_address >> 16) & 0x000000FF);
            window_main->tab_quickwatch_data[9].address = (WORD) (param->symbol_address & 0x0000FFFF);
            sprintf(buffer,"%02X",window_main->tab_quickwatch_data[9].bank);
            SetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_BANK10),buffer);
            sprintf(buffer,"%04X",window_main->tab_quickwatch_data[9].address);
            SetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_ADDRESS10),buffer);
            SetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_LABEL10),param->symbol_name);
            RefreshQuickWatchWindowData(window_quickwatch);
          }
        return(TRUE);

      /********************************************************************/
      /* Fermeture de window (le WM_CLOSE est appel avant le WM_DESTROY) */
      /********************************************************************/
      case WM_CLOSE :
        /* On force une dernire Refresh pour rcuprer ce qui a t saisi */
        RefreshQuickWatchWindowData(window_quickwatch);
        /* Fentre QuickWatch */
        DestroyWindow(hwndQuickWatch);
        return(0);

      case WM_DESTROY :
        /* Suppression des Fentres */
        DeleteQuickWatchChildWindows(window_quickwatch);
        CheckMainWindowMenuItem(ID_MENU_VIEW_QUICKWATCH,FALSE);
        my_Memory(MEMORY_FREE_WINDOW_QUICKWATCH,NULL,NULL);
        /* Demande la sortie de la boucle des messages */
        PostQuitMessage(0);
        return(0);
    }

  /* Retourner FALSE pour relayer le message non trait au systme */
  return(FALSE);
}


/***********************************************************************************************************/
/*  CreateQuickWatchChildWindows() : Crer les fentres enfants et remplir la structure window_quickwatch. */
/***********************************************************************************************************/
static int CreateQuickWatchChildWindows(HWND hwndQuickWatch, struct window_quickwatch *window_quickwatch)
{
  int i;
  struct window_main *window_main;
  char *data_length_alignment[7] = {"8 Bit","16 Bit / Intel","24 Bit / Intel","32 Bit / Intel","16 Bit / Motorola","24 Bit / Motorola","32 Bit / Motorola"};
  int tab_bank_id[10]    = {IDC_EDIT_RAM_BANK1,IDC_EDIT_RAM_BANK2,IDC_EDIT_RAM_BANK3,IDC_EDIT_RAM_BANK4,IDC_EDIT_RAM_BANK5,IDC_EDIT_RAM_BANK6,IDC_EDIT_RAM_BANK7,IDC_EDIT_RAM_BANK8,IDC_EDIT_RAM_BANK9,IDC_EDIT_RAM_BANK10};
  int tab_address_id[10] = {IDC_EDIT_RAM_ADDRESS1,IDC_EDIT_RAM_ADDRESS2,IDC_EDIT_RAM_ADDRESS3,IDC_EDIT_RAM_ADDRESS4,IDC_EDIT_RAM_ADDRESS5,IDC_EDIT_RAM_ADDRESS6,IDC_EDIT_RAM_ADDRESS7,IDC_EDIT_RAM_ADDRESS8,IDC_EDIT_RAM_ADDRESS9,IDC_EDIT_RAM_ADDRESS10};
  int tab_name_id[10]    = {IDC_EDIT_LABEL1,IDC_EDIT_LABEL2,IDC_EDIT_LABEL3,IDC_EDIT_LABEL4,IDC_EDIT_LABEL5,IDC_EDIT_LABEL6,IDC_EDIT_LABEL7,IDC_EDIT_LABEL8,IDC_EDIT_LABEL9,IDC_EDIT_LABEL10};
  char buffer[BUFFER_SIZE];
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Rcupre le handle de la fentre Main */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);

  /** Window Handle **/
  window_quickwatch->hwnd_quickwatch = hwndQuickWatch;

  /** Surchage des Control Edit pour capter les vnements **/
  window_quickwatch->oldEditProcBank1  = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndQuickWatch,IDC_EDIT_RAM_BANK1),GWLP_WNDPROC,(LONG_PTR)subEditQuickWatchProcControl);
  window_quickwatch->oldEditProcBank2  = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndQuickWatch,IDC_EDIT_RAM_BANK2),GWLP_WNDPROC,(LONG_PTR)subEditQuickWatchProcControl);
  window_quickwatch->oldEditProcBank3  = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndQuickWatch,IDC_EDIT_RAM_BANK3),GWLP_WNDPROC,(LONG_PTR)subEditQuickWatchProcControl);
  window_quickwatch->oldEditProcBank4  = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndQuickWatch,IDC_EDIT_RAM_BANK4),GWLP_WNDPROC,(LONG_PTR)subEditQuickWatchProcControl);
  window_quickwatch->oldEditProcBank5  = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndQuickWatch,IDC_EDIT_RAM_BANK5),GWLP_WNDPROC,(LONG_PTR)subEditQuickWatchProcControl);
  window_quickwatch->oldEditProcBank6  = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndQuickWatch,IDC_EDIT_RAM_BANK6),GWLP_WNDPROC,(LONG_PTR)subEditQuickWatchProcControl);
  window_quickwatch->oldEditProcBank7  = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndQuickWatch,IDC_EDIT_RAM_BANK7),GWLP_WNDPROC,(LONG_PTR)subEditQuickWatchProcControl);
  window_quickwatch->oldEditProcBank8  = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndQuickWatch,IDC_EDIT_RAM_BANK8),GWLP_WNDPROC,(LONG_PTR)subEditQuickWatchProcControl);
  window_quickwatch->oldEditProcBank9  = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndQuickWatch,IDC_EDIT_RAM_BANK9),GWLP_WNDPROC,(LONG_PTR)subEditQuickWatchProcControl);
  window_quickwatch->oldEditProcBank10 = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndQuickWatch,IDC_EDIT_RAM_BANK10),GWLP_WNDPROC,(LONG_PTR)subEditQuickWatchProcControl);
  window_quickwatch->oldEditProcAddress1  = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndQuickWatch,IDC_EDIT_RAM_ADDRESS1),GWLP_WNDPROC,(LONG_PTR)subEditQuickWatchProcControl);
  window_quickwatch->oldEditProcAddress2  = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndQuickWatch,IDC_EDIT_RAM_ADDRESS2),GWLP_WNDPROC,(LONG_PTR)subEditQuickWatchProcControl);
  window_quickwatch->oldEditProcAddress3  = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndQuickWatch,IDC_EDIT_RAM_ADDRESS3),GWLP_WNDPROC,(LONG_PTR)subEditQuickWatchProcControl);
  window_quickwatch->oldEditProcAddress4  = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndQuickWatch,IDC_EDIT_RAM_ADDRESS4),GWLP_WNDPROC,(LONG_PTR)subEditQuickWatchProcControl);
  window_quickwatch->oldEditProcAddress5  = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndQuickWatch,IDC_EDIT_RAM_ADDRESS5),GWLP_WNDPROC,(LONG_PTR)subEditQuickWatchProcControl);
  window_quickwatch->oldEditProcAddress6  = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndQuickWatch,IDC_EDIT_RAM_ADDRESS6),GWLP_WNDPROC,(LONG_PTR)subEditQuickWatchProcControl);
  window_quickwatch->oldEditProcAddress7  = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndQuickWatch,IDC_EDIT_RAM_ADDRESS7),GWLP_WNDPROC,(LONG_PTR)subEditQuickWatchProcControl);
  window_quickwatch->oldEditProcAddress8  = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndQuickWatch,IDC_EDIT_RAM_ADDRESS8),GWLP_WNDPROC,(LONG_PTR)subEditQuickWatchProcControl);
  window_quickwatch->oldEditProcAddress9  = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndQuickWatch,IDC_EDIT_RAM_ADDRESS9),GWLP_WNDPROC,(LONG_PTR)subEditQuickWatchProcControl);
  window_quickwatch->oldEditProcAddress10 = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndQuickWatch,IDC_EDIT_RAM_ADDRESS10),GWLP_WNDPROC,(LONG_PTR)subEditQuickWatchProcControl);
  window_quickwatch->oldEditProcLabel1  = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndQuickWatch,IDC_EDIT_LABEL1),GWLP_WNDPROC,(LONG_PTR)subEditQuickWatchProcControl);
  window_quickwatch->oldEditProcLabel2  = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndQuickWatch,IDC_EDIT_LABEL2),GWLP_WNDPROC,(LONG_PTR)subEditQuickWatchProcControl);
  window_quickwatch->oldEditProcLabel3  = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndQuickWatch,IDC_EDIT_LABEL3),GWLP_WNDPROC,(LONG_PTR)subEditQuickWatchProcControl);
  window_quickwatch->oldEditProcLabel4  = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndQuickWatch,IDC_EDIT_LABEL4),GWLP_WNDPROC,(LONG_PTR)subEditQuickWatchProcControl);
  window_quickwatch->oldEditProcLabel5  = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndQuickWatch,IDC_EDIT_LABEL5),GWLP_WNDPROC,(LONG_PTR)subEditQuickWatchProcControl);
  window_quickwatch->oldEditProcLabel6  = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndQuickWatch,IDC_EDIT_LABEL6),GWLP_WNDPROC,(LONG_PTR)subEditQuickWatchProcControl);
  window_quickwatch->oldEditProcLabel7  = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndQuickWatch,IDC_EDIT_LABEL7),GWLP_WNDPROC,(LONG_PTR)subEditQuickWatchProcControl);
  window_quickwatch->oldEditProcLabel8  = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndQuickWatch,IDC_EDIT_LABEL8),GWLP_WNDPROC,(LONG_PTR)subEditQuickWatchProcControl);
  window_quickwatch->oldEditProcLabel9  = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndQuickWatch,IDC_EDIT_LABEL9),GWLP_WNDPROC,(LONG_PTR)subEditQuickWatchProcControl);
  window_quickwatch->oldEditProcLabel10 = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndQuickWatch,IDC_EDIT_LABEL10),GWLP_WNDPROC,(LONG_PTR)subEditQuickWatchProcControl);

  /** Ajoute les images sur les boutons **/
  SendMessage(GetDlgItem(hwndQuickWatch,IDC_BUTTON_SYMBOL1),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_SYMBOL)));
  SendMessage(GetDlgItem(hwndQuickWatch,IDC_BUTTON_SYMBOL2),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_SYMBOL)));
  SendMessage(GetDlgItem(hwndQuickWatch,IDC_BUTTON_SYMBOL3),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_SYMBOL)));
  SendMessage(GetDlgItem(hwndQuickWatch,IDC_BUTTON_SYMBOL4),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_SYMBOL)));
  SendMessage(GetDlgItem(hwndQuickWatch,IDC_BUTTON_SYMBOL5),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_SYMBOL)));
  SendMessage(GetDlgItem(hwndQuickWatch,IDC_BUTTON_SYMBOL6),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_SYMBOL)));
  SendMessage(GetDlgItem(hwndQuickWatch,IDC_BUTTON_SYMBOL7),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_SYMBOL)));
  SendMessage(GetDlgItem(hwndQuickWatch,IDC_BUTTON_SYMBOL8),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_SYMBOL)));
  SendMessage(GetDlgItem(hwndQuickWatch,IDC_BUTTON_SYMBOL9),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_SYMBOL)));
  SendMessage(GetDlgItem(hwndQuickWatch,IDC_BUTTON_SYMBOL10),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_SYMBOL)));

  /* Refresh Mode : Automatic */
  UpdateQuickWatchRefreshMode(window_quickwatch,QUICKWATCH_REFRESH_AUTOMATIC);

  /** Cration de la Police de caractre + Affectation de la police aux Control Edit Values **/
  window_quickwatch->hFont_value = CreateFont(QUICKWATCH_EDIT_FONT_HEIGHT,0,0,0,FW_DONTCARE,FALSE,FALSE,FALSE,ANSI_CHARSET,OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY,DEFAULT_PITCH|FF_SWISS,TEXT("Courier New"));
  SendMessage(GetDlgItem(hwndQuickWatch,IDC_EDIT_VALUE1),WM_SETFONT,(WPARAM)window_quickwatch->hFont_value,MAKELPARAM(FALSE,0));
  SendMessage(GetDlgItem(hwndQuickWatch,IDC_EDIT_VALUE2),WM_SETFONT,(WPARAM)window_quickwatch->hFont_value,MAKELPARAM(FALSE,0));
  SendMessage(GetDlgItem(hwndQuickWatch,IDC_EDIT_VALUE3),WM_SETFONT,(WPARAM)window_quickwatch->hFont_value,MAKELPARAM(FALSE,0));
  SendMessage(GetDlgItem(hwndQuickWatch,IDC_EDIT_VALUE4),WM_SETFONT,(WPARAM)window_quickwatch->hFont_value,MAKELPARAM(FALSE,0));
  SendMessage(GetDlgItem(hwndQuickWatch,IDC_EDIT_VALUE5),WM_SETFONT,(WPARAM)window_quickwatch->hFont_value,MAKELPARAM(FALSE,0));
  SendMessage(GetDlgItem(hwndQuickWatch,IDC_EDIT_VALUE6),WM_SETFONT,(WPARAM)window_quickwatch->hFont_value,MAKELPARAM(FALSE,0));
  SendMessage(GetDlgItem(hwndQuickWatch,IDC_EDIT_VALUE7),WM_SETFONT,(WPARAM)window_quickwatch->hFont_value,MAKELPARAM(FALSE,0));
  SendMessage(GetDlgItem(hwndQuickWatch,IDC_EDIT_VALUE8),WM_SETFONT,(WPARAM)window_quickwatch->hFont_value,MAKELPARAM(FALSE,0));
  SendMessage(GetDlgItem(hwndQuickWatch,IDC_EDIT_VALUE9),WM_SETFONT,(WPARAM)window_quickwatch->hFont_value,MAKELPARAM(FALSE,0));
  SendMessage(GetDlgItem(hwndQuickWatch,IDC_EDIT_VALUE10),WM_SETFONT,(WPARAM)window_quickwatch->hFont_value,MAKELPARAM(FALSE,0));

  /** Cration des ComboBox Data Length Alignment **/
  window_quickwatch->tab_hwnd_combo_length_alignment[0] = CreateWindowEx(0,"COMBOBOX", 0, CBS_DROPDOWNLIST | WS_CHILD | WS_VISIBLE,
                                                                         272,65,120,250,hwndQuickWatch,(HMENU)IDC_COMBOBOX_QUICKWATCH_DATALENGTH_1,param->hInstance,0);
  window_quickwatch->tab_hwnd_combo_length_alignment[1] = CreateWindowEx(0,"COMBOBOX", 0, CBS_DROPDOWNLIST | WS_CHILD | WS_VISIBLE,
                                                                         272,94,120,250,hwndQuickWatch,(HMENU)IDC_COMBOBOX_QUICKWATCH_DATALENGTH_2,param->hInstance,0);
  window_quickwatch->tab_hwnd_combo_length_alignment[2] = CreateWindowEx(0,"COMBOBOX", 0, CBS_DROPDOWNLIST | WS_CHILD | WS_VISIBLE,
                                                                         272,124,120,250,hwndQuickWatch,(HMENU)IDC_COMBOBOX_QUICKWATCH_DATALENGTH_3,param->hInstance,0);
  window_quickwatch->tab_hwnd_combo_length_alignment[3] = CreateWindowEx(0,"COMBOBOX", 0, CBS_DROPDOWNLIST | WS_CHILD | WS_VISIBLE,
                                                                         272,153,120,250,hwndQuickWatch,(HMENU)IDC_COMBOBOX_QUICKWATCH_DATALENGTH_4,param->hInstance,0);
  window_quickwatch->tab_hwnd_combo_length_alignment[4] = CreateWindowEx(0,"COMBOBOX", 0, CBS_DROPDOWNLIST | WS_CHILD | WS_VISIBLE,
                                                                         272,182,120,250,hwndQuickWatch,(HMENU)IDC_COMBOBOX_QUICKWATCH_DATALENGTH_5,param->hInstance,0);
  window_quickwatch->tab_hwnd_combo_length_alignment[5] = CreateWindowEx(0,"COMBOBOX", 0, CBS_DROPDOWNLIST | WS_CHILD | WS_VISIBLE,
                                                                         272,211,120,250,hwndQuickWatch,(HMENU)IDC_COMBOBOX_QUICKWATCH_DATALENGTH_6,param->hInstance,0);
  window_quickwatch->tab_hwnd_combo_length_alignment[6] = CreateWindowEx(0,"COMBOBOX", 0, CBS_DROPDOWNLIST | WS_CHILD | WS_VISIBLE,
                                                                         272,241,120,250,hwndQuickWatch,(HMENU)IDC_COMBOBOX_QUICKWATCH_DATALENGTH_7,param->hInstance,0);
  window_quickwatch->tab_hwnd_combo_length_alignment[7] = CreateWindowEx(0,"COMBOBOX", 0, CBS_DROPDOWNLIST | WS_CHILD | WS_VISIBLE,
                                                                         272,270,120,250,hwndQuickWatch,(HMENU)IDC_COMBOBOX_QUICKWATCH_DATALENGTH_8,param->hInstance,0);
  window_quickwatch->tab_hwnd_combo_length_alignment[8] = CreateWindowEx(0,"COMBOBOX", 0, CBS_DROPDOWNLIST | WS_CHILD | WS_VISIBLE,
                                                                         272,299,120,250,hwndQuickWatch,(HMENU)IDC_COMBOBOX_QUICKWATCH_DATALENGTH_9,param->hInstance,0);
  window_quickwatch->tab_hwnd_combo_length_alignment[9] = CreateWindowEx(0,"COMBOBOX", 0, CBS_DROPDOWNLIST | WS_CHILD | WS_VISIBLE,
                                                                         272,328,120,250,hwndQuickWatch,(HMENU)IDC_COMBOBOX_QUICKWATCH_DATALENGTH_10,param->hInstance,0);
  for(i=0; i<7; i++)
    {
      SendMessage(window_quickwatch->tab_hwnd_combo_length_alignment[0],CB_ADDSTRING,0,(LPARAM)data_length_alignment[i]);
      SendMessage(window_quickwatch->tab_hwnd_combo_length_alignment[1],CB_ADDSTRING,0,(LPARAM)data_length_alignment[i]);
      SendMessage(window_quickwatch->tab_hwnd_combo_length_alignment[2],CB_ADDSTRING,0,(LPARAM)data_length_alignment[i]);
      SendMessage(window_quickwatch->tab_hwnd_combo_length_alignment[3],CB_ADDSTRING,0,(LPARAM)data_length_alignment[i]);
      SendMessage(window_quickwatch->tab_hwnd_combo_length_alignment[4],CB_ADDSTRING,0,(LPARAM)data_length_alignment[i]);
      SendMessage(window_quickwatch->tab_hwnd_combo_length_alignment[5],CB_ADDSTRING,0,(LPARAM)data_length_alignment[i]);
      SendMessage(window_quickwatch->tab_hwnd_combo_length_alignment[6],CB_ADDSTRING,0,(LPARAM)data_length_alignment[i]);
      SendMessage(window_quickwatch->tab_hwnd_combo_length_alignment[7],CB_ADDSTRING,0,(LPARAM)data_length_alignment[i]);
      SendMessage(window_quickwatch->tab_hwnd_combo_length_alignment[8],CB_ADDSTRING,0,(LPARAM)data_length_alignment[i]);
      SendMessage(window_quickwatch->tab_hwnd_combo_length_alignment[9],CB_ADDSTRING,0,(LPARAM)data_length_alignment[i]);
    }

  /** Cration de la Police de caractre + Affectation de la police au Control Combo **/
  window_quickwatch->hFont_conbo = CreateFont(14,0,0,0,FW_DONTCARE,FALSE,FALSE,FALSE,ANSI_CHARSET,OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY,DEFAULT_PITCH|FF_SWISS,"Arial");
  for(i=0; i<10; i++)
    SendMessage(window_quickwatch->tab_hwnd_combo_length_alignment[i],WM_SETFONT,(WPARAM)window_quickwatch->hFont_conbo,TRUE);

  /** Init des Valeurs **/
  for(i=0; i<10; i++)
    {
      /** Valeur des Radio Boutons Ram Type (+ MAJ de la fentre) **/
      UpdateQuickWatchRamType(window_quickwatch,i,window_main->tab_quickwatch_data[i].ram_type,0);

      /** Adresses **/
      if(window_main->tab_quickwatch_data[i].is_data_error == 1)
        {
          SetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,tab_bank_id[i]),"");
          SetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,tab_address_id[i]),"");
        }
      else
        {
          sprintf(buffer,"%02X",window_main->tab_quickwatch_data[i].bank);
          SetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,tab_bank_id[i]),buffer);
          sprintf(buffer,"%04X",window_main->tab_quickwatch_data[i].address);
          SetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,tab_address_id[i]),buffer);
        }

      /** Init des valeurs Aligment **/
      SendMessage(window_quickwatch->tab_hwnd_combo_length_alignment[i],CB_SETCURSEL,window_main->tab_quickwatch_data[i].combo_length_alignment,0);

      /** Nom **/
      SetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,tab_name_id[i]),window_main->tab_quickwatch_data[i].name);
    }

  /** Construction des Data + Rafraichissement de la fentre **/
  RefreshQuickWatchWindowData(window_quickwatch);

  /* OK */
  return(0);
}


/*************************************************************************************************/
/*  subEditQuickWatchProcControl() :  Rcupre les vnements qui arrivent dans le Control Edit. */
/*************************************************************************************************/
LRESULT CALLBACK subEditQuickWatchProcControl(HWND hwnd_control, UINT msg, WPARAM wParam, LPARAM lParam)
{
  int index;
  struct window_quickwatch *window_quickwatch;
  my_Memory(MEMORY_GET_WINDOW_QUICKWATCH,&window_quickwatch,NULL);

  /* Rien  faire */
  if(window_quickwatch == NULL)
    return(0);

  /** Identifie le Control **/
  /* Src */
  if(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_BANK1) == hwnd_control)
    index = 1;
  else if(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_BANK2) == hwnd_control)
    index = 2;
  else if(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_BANK3) == hwnd_control)
    index = 3;
  else if(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_BANK4) == hwnd_control)
    index = 4;
  else if(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_BANK5) == hwnd_control)
    index = 5;
  else if(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_BANK6) == hwnd_control)
    index = 6;
  else if(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_BANK7) == hwnd_control)
    index = 7;
  else if(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_BANK8) == hwnd_control)
    index = 8;
  else if(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_BANK9) == hwnd_control)
    index = 9;
  else if(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_BANK10) == hwnd_control)
    index = 10;
  else if(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_ADDRESS1) == hwnd_control)
    index = 11;
  else if(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_ADDRESS2) == hwnd_control)
    index = 12;
  else if(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_ADDRESS3) == hwnd_control)
    index = 13;
  else if(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_ADDRESS4) == hwnd_control)
    index = 14;
  else if(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_ADDRESS5) == hwnd_control)
    index = 15;
  else if(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_ADDRESS6) == hwnd_control)
    index = 16;
  else if(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_ADDRESS7) == hwnd_control)
    index = 17;
  else if(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_ADDRESS8) == hwnd_control)
    index = 18;
  else if(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_ADDRESS9) == hwnd_control)
    index = 19;
  else if(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_ADDRESS10) == hwnd_control)
    index = 20;
  else if(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_LABEL1) == hwnd_control)
    index = 21;
  else if(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_LABEL2) == hwnd_control)
    index = 22;
  else if(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_LABEL3) == hwnd_control)
    index = 23;
  else if(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_LABEL4) == hwnd_control)
    index = 24;
  else if(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_LABEL5) == hwnd_control)
    index = 25;
  else if(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_LABEL6) == hwnd_control)
    index = 26;
  else if(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_LABEL7) == hwnd_control)
    index = 27;
  else if(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_LABEL8) == hwnd_control)
    index = 28;
  else if(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_LABEL9) == hwnd_control)
    index = 29;
  else if(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_LABEL10) == hwnd_control)
    index = 30;
  else
    return(0);      /* Pas trouv */

  /** Dtecte l'vnement du Return dans l'Edit **/
  if(msg == WM_CHAR && wParam == VK_RETURN)
    {
      /* Enlve le focus de l'Edit */
      SetFocus(window_quickwatch->hwnd_quickwatch);

      /* On force la MAJ de la fentre */
      RefreshQuickWatchWindowData(window_quickwatch);
    }
  else if(index == 1)
    return(CallWindowProc(window_quickwatch->oldEditProcBank1,hwnd_control,msg,wParam,lParam)); 
  else if(index == 2)
    return(CallWindowProc(window_quickwatch->oldEditProcBank2,hwnd_control,msg,wParam,lParam)); 
  else if(index == 3)
    return(CallWindowProc(window_quickwatch->oldEditProcBank3,hwnd_control,msg,wParam,lParam)); 
  else if(index == 4)
    return(CallWindowProc(window_quickwatch->oldEditProcBank4,hwnd_control,msg,wParam,lParam)); 
  else if(index == 5)
    return(CallWindowProc(window_quickwatch->oldEditProcBank5,hwnd_control,msg,wParam,lParam)); 
  else if(index == 6)
    return(CallWindowProc(window_quickwatch->oldEditProcBank6,hwnd_control,msg,wParam,lParam)); 
  else if(index == 7)
    return(CallWindowProc(window_quickwatch->oldEditProcBank7,hwnd_control,msg,wParam,lParam)); 
  else if(index == 8)
    return(CallWindowProc(window_quickwatch->oldEditProcBank8,hwnd_control,msg,wParam,lParam)); 
  else if(index == 9)
    return(CallWindowProc(window_quickwatch->oldEditProcBank9,hwnd_control,msg,wParam,lParam)); 
  else if(index == 10)
    return(CallWindowProc(window_quickwatch->oldEditProcBank10,hwnd_control,msg,wParam,lParam)); 
  else if(index == 11)
    return(CallWindowProc(window_quickwatch->oldEditProcAddress1,hwnd_control,msg,wParam,lParam)); 
  else if(index == 12)
    return(CallWindowProc(window_quickwatch->oldEditProcAddress2,hwnd_control,msg,wParam,lParam)); 
  else if(index == 13)
    return(CallWindowProc(window_quickwatch->oldEditProcAddress3,hwnd_control,msg,wParam,lParam)); 
  else if(index == 14)
    return(CallWindowProc(window_quickwatch->oldEditProcAddress4,hwnd_control,msg,wParam,lParam)); 
  else if(index == 15)
    return(CallWindowProc(window_quickwatch->oldEditProcAddress5,hwnd_control,msg,wParam,lParam)); 
  else if(index == 16)
    return(CallWindowProc(window_quickwatch->oldEditProcAddress6,hwnd_control,msg,wParam,lParam)); 
  else if(index == 17)
    return(CallWindowProc(window_quickwatch->oldEditProcAddress7,hwnd_control,msg,wParam,lParam)); 
  else if(index == 18)
    return(CallWindowProc(window_quickwatch->oldEditProcAddress8,hwnd_control,msg,wParam,lParam)); 
  else if(index == 19)
    return(CallWindowProc(window_quickwatch->oldEditProcAddress9,hwnd_control,msg,wParam,lParam)); 
  else if(index == 20)
    return(CallWindowProc(window_quickwatch->oldEditProcAddress10,hwnd_control,msg,wParam,lParam)); 
  else if(index == 21)
    return(CallWindowProc(window_quickwatch->oldEditProcLabel1,hwnd_control,msg,wParam,lParam)); 
  else if(index == 22)
    return(CallWindowProc(window_quickwatch->oldEditProcLabel2,hwnd_control,msg,wParam,lParam)); 
  else if(index == 23)
    return(CallWindowProc(window_quickwatch->oldEditProcLabel3,hwnd_control,msg,wParam,lParam)); 
  else if(index == 24)
    return(CallWindowProc(window_quickwatch->oldEditProcLabel4,hwnd_control,msg,wParam,lParam)); 
  else if(index == 25)
    return(CallWindowProc(window_quickwatch->oldEditProcLabel5,hwnd_control,msg,wParam,lParam)); 
  else if(index == 26)
    return(CallWindowProc(window_quickwatch->oldEditProcLabel6,hwnd_control,msg,wParam,lParam)); 
  else if(index == 27)
    return(CallWindowProc(window_quickwatch->oldEditProcLabel7,hwnd_control,msg,wParam,lParam)); 
  else if(index == 28)
    return(CallWindowProc(window_quickwatch->oldEditProcLabel8,hwnd_control,msg,wParam,lParam)); 
  else if(index == 29)
    return(CallWindowProc(window_quickwatch->oldEditProcLabel9,hwnd_control,msg,wParam,lParam)); 
  else if(index == 30)
    return(CallWindowProc(window_quickwatch->oldEditProcLabel10,hwnd_control,msg,wParam,lParam)); 

  /* OK */
  return(0);
}


/***********************************************************************************************/
/*  UpdateQuickWatchRefreshMode() :  MAJ du mode de rafraichissement de la fentre QuickWatch. */
/***********************************************************************************************/
static void UpdateQuickWatchRefreshMode(struct window_quickwatch *window_quickwatch, int refresh_mode)
{
  struct notification *current_notification;

  if(refresh_mode == QUICKWATCH_REFRESH_AUTOMATIC)
    {
      /* Visuel */
      window_quickwatch->refresh_mode = QUICKWATCH_REFRESH_AUTOMATIC;
      SendDlgItemMessage(window_quickwatch->hwnd_quickwatch,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_quickwatch->hwnd_quickwatch,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,0,0);

      /* Cration d'une Notification */
      current_notification = (struct notification *) calloc(1,sizeof(struct notification));
      if(current_notification == NULL)
        return;
      current_notification->hwnd_window = window_quickwatch->hwnd_quickwatch;
      current_notification->message = HM_QUICKWATCH_REFRESHWINDOW;
      current_notification->type = NOTIFICATION_TYPE_VIEW;
      current_notification->sub_type = NOTIFICATION_SUBTYPE_QUICKWATCH;

      /* Ajout de la notification */
      window_quickwatch->current_notification = current_notification;
      my_Notification(NOTIFICATION_ADD,current_notification,NULL);
    }
  else
    {
      /* Visuel */
      window_quickwatch->refresh_mode = QUICKWATCH_REFRESH_MANUAL;
      SendDlgItemMessage(window_quickwatch->hwnd_quickwatch,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_quickwatch->hwnd_quickwatch,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,1,0);

      /* Supprime la Notification */
      my_Notification(NOTIFICATION_REMOVE,window_quickwatch->current_notification,NULL);
      window_quickwatch->current_notification = NULL;
    }
}


/*************************************************************************************************************/
/*  UpdateQuickWatchDataLengthAlignment() :  MAJ du ComboBox Data Length Alignment de la fentre QuickWatch. */
/*************************************************************************************************************/
static void UpdateQuickWatchDataLengthAlignment(struct window_quickwatch *window_quickwatch, int combo_index, int data_length_alignment)
{
  struct window_main *window_main;
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);

  /* MAJ de la valeur */
  window_main->tab_quickwatch_data[combo_index-1].combo_length_alignment = data_length_alignment;

  /* Valeurs */
  if(data_length_alignment == QUICKWATCH_COMBO_8)
    {
      window_main->tab_quickwatch_data[combo_index-1].data_length = QUICKWATCH_LENGTH_8;
      window_main->tab_quickwatch_data[combo_index-1].data_alignment = QUICKWATCH_ALIGNMENT_INTEL;
    }
  else if(data_length_alignment == QUICKWATCH_COMBO_16_INTEL)
    {
      window_main->tab_quickwatch_data[combo_index-1].data_length = QUICKWATCH_LENGTH_16;
      window_main->tab_quickwatch_data[combo_index-1].data_alignment = QUICKWATCH_ALIGNMENT_INTEL;
    }
  else if(data_length_alignment == QUICKWATCH_COMBO_24_INTEL)
    {
      window_main->tab_quickwatch_data[combo_index-1].data_length = QUICKWATCH_LENGTH_24;
      window_main->tab_quickwatch_data[combo_index-1].data_alignment = QUICKWATCH_ALIGNMENT_INTEL;
    }
  else if(data_length_alignment == QUICKWATCH_COMBO_32_INTEL)
    {
      window_main->tab_quickwatch_data[combo_index-1].data_length = QUICKWATCH_LENGTH_32;
      window_main->tab_quickwatch_data[combo_index-1].data_alignment = QUICKWATCH_ALIGNMENT_INTEL;
    }
  else if(data_length_alignment == QUICKWATCH_COMBO_16_MOTOROLA)
    {
      window_main->tab_quickwatch_data[combo_index-1].data_length = QUICKWATCH_LENGTH_16;
      window_main->tab_quickwatch_data[combo_index-1].data_alignment = QUICKWATCH_ALIGNMENT_MOTOROLA;
    }
  else if(data_length_alignment == QUICKWATCH_COMBO_24_MOTOROLA)
    {
      window_main->tab_quickwatch_data[combo_index-1].data_length = QUICKWATCH_LENGTH_24;
      window_main->tab_quickwatch_data[combo_index-1].data_alignment = QUICKWATCH_ALIGNMENT_MOTOROLA;
    }
  else if(data_length_alignment == QUICKWATCH_COMBO_32_MOTOROLA)
    {
      window_main->tab_quickwatch_data[combo_index-1].data_length = QUICKWATCH_LENGTH_32;
      window_main->tab_quickwatch_data[combo_index-1].data_alignment = QUICKWATCH_ALIGNMENT_MOTOROLA;
    }

  /* MAJ des donnes */
  RefreshQuickWatchWindowData(window_quickwatch);
}


/***************************************************************************************/
/*  UpdateQuickWatchRamType() :  MAJ du Radio Bouton RamType de la fentre QuickWatch. */
/***************************************************************************************/
void UpdateQuickWatchRamType(struct window_quickwatch *window_quickwatch, int radio_index, int ram_type, int refresh_window)
{
  struct window_main *window_main;
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);

  if(radio_index == 0)
    {
      window_main->tab_quickwatch_data[radio_index].ram_type = ram_type;
      if(ram_type == QUICKWATCH_RAMTYPE_RAM)
        {
          /* Visuel */
          SendDlgItemMessage(window_quickwatch->hwnd_quickwatch,IDC_RADIO_RAM1,BM_SETCHECK,1,0);
          SendDlgItemMessage(window_quickwatch->hwnd_quickwatch,IDC_RADIO_DOCRAM1,BM_SETCHECK,0,0);

          /* Enable BANK */
          EnableWindow(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_BANK1),TRUE);
        }
      else
        {
          /* Visuel */
          SendDlgItemMessage(window_quickwatch->hwnd_quickwatch,IDC_RADIO_RAM1,BM_SETCHECK,0,0);
          SendDlgItemMessage(window_quickwatch->hwnd_quickwatch,IDC_RADIO_DOCRAM1,BM_SETCHECK,1,0);

          /* Disable BANK */
          EnableWindow(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_BANK1),FALSE);
        }
    }
  else if(radio_index == 1)
    {
      window_main->tab_quickwatch_data[radio_index].ram_type = ram_type;
      if(ram_type == QUICKWATCH_RAMTYPE_RAM)
        {
          /* Visuel */
          SendDlgItemMessage(window_quickwatch->hwnd_quickwatch,IDC_RADIO_RAM2,BM_SETCHECK,1,0);
          SendDlgItemMessage(window_quickwatch->hwnd_quickwatch,IDC_RADIO_DOCRAM2,BM_SETCHECK,0,0);

          /* Enable BANK */
          EnableWindow(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_BANK2),TRUE);
        }
      else
        {
          /* Visuel */
          SendDlgItemMessage(window_quickwatch->hwnd_quickwatch,IDC_RADIO_RAM2,BM_SETCHECK,0,0);
          SendDlgItemMessage(window_quickwatch->hwnd_quickwatch,IDC_RADIO_DOCRAM2,BM_SETCHECK,1,0);

          /* Disable BANK */
          EnableWindow(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_BANK2),FALSE);
        }
    }
  else if(radio_index == 2)
    {
      window_main->tab_quickwatch_data[radio_index].ram_type = ram_type;
      if(ram_type == QUICKWATCH_RAMTYPE_RAM)
        {
          /* Visuel */
          SendDlgItemMessage(window_quickwatch->hwnd_quickwatch,IDC_RADIO_RAM3,BM_SETCHECK,1,0);
          SendDlgItemMessage(window_quickwatch->hwnd_quickwatch,IDC_RADIO_DOCRAM3,BM_SETCHECK,0,0);

          /* Enable BANK */
          EnableWindow(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_BANK3),TRUE);
        }
      else
        {
          /* Visuel */
          SendDlgItemMessage(window_quickwatch->hwnd_quickwatch,IDC_RADIO_RAM3,BM_SETCHECK,0,0);
          SendDlgItemMessage(window_quickwatch->hwnd_quickwatch,IDC_RADIO_DOCRAM3,BM_SETCHECK,1,0);

          /* Disable BANK */
          EnableWindow(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_BANK3),FALSE);
        }
    }
  else if(radio_index == 3)
    {
      window_main->tab_quickwatch_data[radio_index].ram_type = ram_type;
      if(ram_type == QUICKWATCH_RAMTYPE_RAM)
        {
          /* Visuel */
          SendDlgItemMessage(window_quickwatch->hwnd_quickwatch,IDC_RADIO_RAM4,BM_SETCHECK,1,0);
          SendDlgItemMessage(window_quickwatch->hwnd_quickwatch,IDC_RADIO_DOCRAM4,BM_SETCHECK,0,0);

          /* Enable BANK */
          EnableWindow(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_BANK4),TRUE);
        }
      else
        {
          /* Visuel */
          SendDlgItemMessage(window_quickwatch->hwnd_quickwatch,IDC_RADIO_RAM4,BM_SETCHECK,0,0);
          SendDlgItemMessage(window_quickwatch->hwnd_quickwatch,IDC_RADIO_DOCRAM4,BM_SETCHECK,1,0);

          /* Disable BANK */
          EnableWindow(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_BANK4),FALSE);
        }
    }
  else if(radio_index == 4)
    {
      window_main->tab_quickwatch_data[radio_index].ram_type = ram_type;
      if(ram_type == QUICKWATCH_RAMTYPE_RAM)
        {
          /* Visuel */
          SendDlgItemMessage(window_quickwatch->hwnd_quickwatch,IDC_RADIO_RAM5,BM_SETCHECK,1,0);
          SendDlgItemMessage(window_quickwatch->hwnd_quickwatch,IDC_RADIO_DOCRAM5,BM_SETCHECK,0,0);

          /* Enable BANK */
          EnableWindow(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_BANK5),TRUE);
        }
      else
        {
          /* Visuel */
          SendDlgItemMessage(window_quickwatch->hwnd_quickwatch,IDC_RADIO_RAM5,BM_SETCHECK,0,0);
          SendDlgItemMessage(window_quickwatch->hwnd_quickwatch,IDC_RADIO_DOCRAM5,BM_SETCHECK,1,0);

          /* Disable BANK */
          EnableWindow(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_BANK5),FALSE);
        }
    }
  else if(radio_index == 5)
    {
      window_main->tab_quickwatch_data[radio_index].ram_type = ram_type;
      if(ram_type == QUICKWATCH_RAMTYPE_RAM)
        {
          /* Visuel */
          SendDlgItemMessage(window_quickwatch->hwnd_quickwatch,IDC_RADIO_RAM6,BM_SETCHECK,1,0);
          SendDlgItemMessage(window_quickwatch->hwnd_quickwatch,IDC_RADIO_DOCRAM6,BM_SETCHECK,0,0);

          /* Enable BANK */
          EnableWindow(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_BANK6),TRUE);
        }
      else
        {
          /* Visuel */
          SendDlgItemMessage(window_quickwatch->hwnd_quickwatch,IDC_RADIO_RAM6,BM_SETCHECK,0,0);
          SendDlgItemMessage(window_quickwatch->hwnd_quickwatch,IDC_RADIO_DOCRAM6,BM_SETCHECK,1,0);

          /* Disable BANK */
          EnableWindow(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_BANK6),FALSE);
        }
    }
  else if(radio_index == 6)
    {
      window_main->tab_quickwatch_data[radio_index].ram_type = ram_type;
      if(ram_type == QUICKWATCH_RAMTYPE_RAM)
        {
          /* Visuel */
          SendDlgItemMessage(window_quickwatch->hwnd_quickwatch,IDC_RADIO_RAM7,BM_SETCHECK,1,0);
          SendDlgItemMessage(window_quickwatch->hwnd_quickwatch,IDC_RADIO_DOCRAM7,BM_SETCHECK,0,0);

          /* Enable BANK */
          EnableWindow(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_BANK7),TRUE);
        }
      else
        {
          /* Visuel */
          SendDlgItemMessage(window_quickwatch->hwnd_quickwatch,IDC_RADIO_RAM7,BM_SETCHECK,0,0);
          SendDlgItemMessage(window_quickwatch->hwnd_quickwatch,IDC_RADIO_DOCRAM7,BM_SETCHECK,1,0);

          /* Disable BANK */
          EnableWindow(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_BANK7),FALSE);
        }
    }
  else if(radio_index == 7)
    {
      window_main->tab_quickwatch_data[radio_index].ram_type = ram_type;
      if(ram_type == QUICKWATCH_RAMTYPE_RAM)
        {
          /* Visuel */
          SendDlgItemMessage(window_quickwatch->hwnd_quickwatch,IDC_RADIO_RAM8,BM_SETCHECK,1,0);
          SendDlgItemMessage(window_quickwatch->hwnd_quickwatch,IDC_RADIO_DOCRAM8,BM_SETCHECK,0,0);

          /* Enable BANK */
          EnableWindow(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_BANK8),TRUE);
        }
      else
        {
          /* Visuel */
          SendDlgItemMessage(window_quickwatch->hwnd_quickwatch,IDC_RADIO_RAM8,BM_SETCHECK,0,0);
          SendDlgItemMessage(window_quickwatch->hwnd_quickwatch,IDC_RADIO_DOCRAM8,BM_SETCHECK,1,0);

          /* Disable BANK */
          EnableWindow(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_BANK8),FALSE);
        }
    }
  else if(radio_index == 8)
    {
      window_main->tab_quickwatch_data[radio_index].ram_type = ram_type;
      if(ram_type == QUICKWATCH_RAMTYPE_RAM)
        {
          /* Visuel */
          SendDlgItemMessage(window_quickwatch->hwnd_quickwatch,IDC_RADIO_RAM9,BM_SETCHECK,1,0);
          SendDlgItemMessage(window_quickwatch->hwnd_quickwatch,IDC_RADIO_DOCRAM9,BM_SETCHECK,0,0);

          /* Enable BANK */
          EnableWindow(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_BANK9),TRUE);
        }
      else
        {
          /* Visuel */
          SendDlgItemMessage(window_quickwatch->hwnd_quickwatch,IDC_RADIO_RAM9,BM_SETCHECK,0,0);
          SendDlgItemMessage(window_quickwatch->hwnd_quickwatch,IDC_RADIO_DOCRAM9,BM_SETCHECK,1,0);

          /* Disable BANK */
          EnableWindow(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_BANK9),FALSE);
        }
    }
  else if(radio_index == 9)
    {
      window_main->tab_quickwatch_data[radio_index].ram_type = ram_type;
      if(ram_type == QUICKWATCH_RAMTYPE_RAM)
        {
          /* Visuel */
          SendDlgItemMessage(window_quickwatch->hwnd_quickwatch,IDC_RADIO_RAM10,BM_SETCHECK,1,0);
          SendDlgItemMessage(window_quickwatch->hwnd_quickwatch,IDC_RADIO_DOCRAM10,BM_SETCHECK,0,0);

          /* Enable BANK */
          EnableWindow(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_BANK10),TRUE);
        }
      else
        {
          /* Visuel */
          SendDlgItemMessage(window_quickwatch->hwnd_quickwatch,IDC_RADIO_RAM10,BM_SETCHECK,0,0);
          SendDlgItemMessage(window_quickwatch->hwnd_quickwatch,IDC_RADIO_DOCRAM10,BM_SETCHECK,1,0);

          /* Disable BANK */
          EnableWindow(GetDlgItem(window_quickwatch->hwnd_quickwatch,IDC_EDIT_RAM_BANK10),FALSE);
        }
    }

  /** On va forcer le rafraichissement de la fentre **/
  if(refresh_window == 1)
    RefreshQuickWatchWindowData(window_quickwatch);
}


/******************************************************************************/
/*  RefreshQuickWatchWindowData() :  MAJ du contenu de la fentre QuickWatch. */
/******************************************************************************/
void RefreshQuickWatchWindowData(struct window_quickwatch *window_quickwatch)
{
  /** MAJ des Control Edit **/
  BuildQuickWatchFromData(window_quickwatch);
}


/************************************************************************************************************/
/*  BuildQuickWatchFromData() :  Construire les donnes hxadcimal  afficher  partir des donnes brutes. */
/************************************************************************************************************/
static void BuildQuickWatchFromData(struct window_quickwatch *window_quickwatch)
{
  int i, j, is_error_bank, is_error_address, one_int;
  unsigned char byte_1, byte_2, byte_3, byte_4;
  unsigned short one_word;
  unsigned int one_dword;
  BYTE bank;
  WORD address;
  DWORD bank_address;
  struct mmu *current_mmu;
  char value_buffer[1024];
  char buffer[BUFFER_SIZE];
  char buffer_byte1[256];
  char buffer_byte2[256];
  char buffer_byte3[256];
  char buffer_byte4[256];
  char buffer_hexa[256];
  char buffer_binary[256];
  char buffer_unsigned[256];
  char buffer_signed[256];
  int tab_bank_id[10]    = {IDC_EDIT_RAM_BANK1,IDC_EDIT_RAM_BANK2,IDC_EDIT_RAM_BANK3,IDC_EDIT_RAM_BANK4,IDC_EDIT_RAM_BANK5,IDC_EDIT_RAM_BANK6,IDC_EDIT_RAM_BANK7,IDC_EDIT_RAM_BANK8,IDC_EDIT_RAM_BANK9,IDC_EDIT_RAM_BANK10};
  int tab_address_id[10] = {IDC_EDIT_RAM_ADDRESS1,IDC_EDIT_RAM_ADDRESS2,IDC_EDIT_RAM_ADDRESS3,IDC_EDIT_RAM_ADDRESS4,IDC_EDIT_RAM_ADDRESS5,IDC_EDIT_RAM_ADDRESS6,IDC_EDIT_RAM_ADDRESS7,IDC_EDIT_RAM_ADDRESS8,IDC_EDIT_RAM_ADDRESS9,IDC_EDIT_RAM_ADDRESS10};
  int tab_value_id[10]   = {IDC_EDIT_VALUE1,IDC_EDIT_VALUE2,IDC_EDIT_VALUE3,IDC_EDIT_VALUE4,IDC_EDIT_VALUE5,IDC_EDIT_VALUE6,IDC_EDIT_VALUE7,IDC_EDIT_VALUE8,IDC_EDIT_VALUE9,IDC_EDIT_VALUE10};
  int tab_name_id[10]    = {IDC_EDIT_LABEL1,IDC_EDIT_LABEL2,IDC_EDIT_LABEL3,IDC_EDIT_LABEL4,IDC_EDIT_LABEL5,IDC_EDIT_LABEL6,IDC_EDIT_LABEL7,IDC_EDIT_LABEL8,IDC_EDIT_LABEL9,IDC_EDIT_LABEL10};
  struct window_main *window_main;
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);

  /* Rcupre le MMU */
  my_Memory(MEMORY_GET_S65816_MMU,&current_mmu,NULL);

  /** Rcupre les donnes **/
  for(i=0; i<10; i++)
    {
      /* Init */
      is_error_bank = 0;
      is_error_address = 0;

      /** Adresse des donnes **/
      if(window_main->tab_quickwatch_data[i].ram_type == QUICKWATCH_RAMTYPE_RAM)
        {
          /* Rcupre l'adresse cible */
          GetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,tab_bank_id[i]),value_buffer,1024-1);
          bank = EvaluateAsByte(value_buffer,&is_error_bank);
          GetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,tab_address_id[i]),value_buffer,1024-1);
          address = EvaluateAsWord(value_buffer,&is_error_address);
          bank_address = (bank << 16) | address;

          /* Conserve l'adresse */
          window_main->tab_quickwatch_data[i].bank = bank;
          window_main->tab_quickwatch_data[i].address = address;
          window_main->tab_quickwatch_data[i].is_data_error = 0;
        }
      else
        {
          /* Rcupre l'adresse cible */
          GetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,tab_address_id[i]),value_buffer,1024-1);
          address = EvaluateAsWord(value_buffer,&is_error_address);
          bank_address = address;

          /* Conserve l'adresse */
          window_main->tab_quickwatch_data[i].bank = 0x00;
          window_main->tab_quickwatch_data[i].address = address;
          window_main->tab_quickwatch_data[i].is_data_error = 0;
        }

      /* Erreur ? */
      if(is_error_bank || is_error_address)
        {
          window_main->tab_quickwatch_data[i].is_data_error = 1;
          continue;
        }

      /* Rcupre le Nom */
      GetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,tab_name_id[i]),window_main->tab_quickwatch_data[i].name,1024-1);

      /* On supprime la dpendance au Symbol */
      if(strcmp(window_main->tab_quickwatch_data[i].name,window_main->tab_quickwatch_data[i].symbol_name))
        {
          window_main->tab_quickwatch_data[i].has_symbol = 0;
          strcpy(window_main->tab_quickwatch_data[i].symbol_name,"");
          strcpy(window_main->tab_quickwatch_data[i].symbol_segment,"");
          strcpy(window_main->tab_quickwatch_data[i].symbol_file,"");
          window_main->tab_quickwatch_data[i].symbol_address = 0xFFFFFFFF;
        }

      /** Rcupre les donnes **/
      window_main->tab_quickwatch_data[i].is_data_error = 0;
      for(j=0; j<window_main->tab_quickwatch_data[i].data_length; j++)
        window_main->tab_quickwatch_data[i].data[j] = (window_main->tab_quickwatch_data[i].ram_type == QUICKWATCH_RAMTYPE_RAM) ? ReadMemoryByte(bank_address+j,current_mmu) : ReadDOCRAMByte(bank_address+j,current_mmu);
    }

  /** Afficher les Data dans les controles Edit **/
  for(i=0; i<10; i++)
    {
      /** Formatage **/
      if(window_main->tab_quickwatch_data[i].is_data_error == 1)
        strcpy(buffer,"");
      else
        {
          /** Byte **/
          if(window_main->tab_quickwatch_data[i].data_length == QUICKWATCH_LENGTH_8)
            {
              /* Data */
              byte_1 = window_main->tab_quickwatch_data[i].data[0];
              
              /* Affichage */
              sprintf(buffer_hexa,"       %02X",byte_1);
              FormatByteAsBinary(byte_1,&buffer_byte1[0]);
              sprintf(buffer_binary,"                              %s",buffer_byte1);
              sprintf(buffer_unsigned,"%13d",(unsigned int)byte_1);
              sprintf(buffer_signed,"%13d",(int)((char)byte_1));
            }
          /** Word **/
          else if(window_main->tab_quickwatch_data[i].data_length == QUICKWATCH_LENGTH_16)
            {
              /* Data */
              byte_1 = (window_main->tab_quickwatch_data[i].data_alignment == QUICKWATCH_ALIGNMENT_INTEL) ? window_main->tab_quickwatch_data[i].data[1] : window_main->tab_quickwatch_data[i].data[0];
              byte_2 = (window_main->tab_quickwatch_data[i].data_alignment == QUICKWATCH_ALIGNMENT_INTEL) ? window_main->tab_quickwatch_data[i].data[0] : window_main->tab_quickwatch_data[i].data[1];
                
              /* Affichage */
              sprintf(buffer_hexa,"     %02X%02X",byte_1,byte_2);
              FormatByteAsBinary(byte_1,&buffer_byte1[0]);
              FormatByteAsBinary(byte_2,&buffer_byte2[0]);
              sprintf(buffer_binary,"                    %s %s",buffer_byte1,buffer_byte2);
              one_word = byte_1*256 + byte_2;
              sprintf(buffer_unsigned,"%13d",(unsigned int)(byte_1*256 + byte_2));
              sprintf(buffer_signed,"%13d",(int)((short)one_word));
            }
          /** 24 Bit **/
          else if(window_main->tab_quickwatch_data[i].data_length == QUICKWATCH_LENGTH_24)
            {
              /* Data */
              byte_1 = (window_main->tab_quickwatch_data[i].data_alignment == QUICKWATCH_ALIGNMENT_INTEL) ? window_main->tab_quickwatch_data[i].data[2] : window_main->tab_quickwatch_data[i].data[0];
              byte_2 = (window_main->tab_quickwatch_data[i].data_alignment == QUICKWATCH_ALIGNMENT_INTEL) ? window_main->tab_quickwatch_data[i].data[1] : window_main->tab_quickwatch_data[i].data[1];
              byte_3 = (window_main->tab_quickwatch_data[i].data_alignment == QUICKWATCH_ALIGNMENT_INTEL) ? window_main->tab_quickwatch_data[i].data[0] : window_main->tab_quickwatch_data[i].data[2];
                
              /* Affichage */
              sprintf(buffer_hexa,"  %02X.%02X%02X",byte_1,byte_2,byte_3);
              FormatByteAsBinary(byte_1,&buffer_byte1[0]);
              FormatByteAsBinary(byte_2,&buffer_byte2[0]);
              FormatByteAsBinary(byte_3,&buffer_byte3[0]);
              sprintf(buffer_binary,"          %s %s %s",buffer_byte1,buffer_byte2,buffer_byte3);
              one_dword = byte_1*65536 + byte_2*256 + byte_3;
              sprintf(buffer_unsigned,"%13d",one_dword);
              if((byte_1 & 0x80) == 0x80)
                one_int = (-1)*(0x1000000 - one_dword);
              else
                one_int = one_dword;
              sprintf(buffer_signed,"%13d",one_int);
            }
          /** 32 Bit **/
          else if(window_main->tab_quickwatch_data[i].data_length == QUICKWATCH_LENGTH_32)
            {
              /* Data */
              byte_1 = (window_main->tab_quickwatch_data[i].data_alignment == QUICKWATCH_ALIGNMENT_INTEL) ? window_main->tab_quickwatch_data[i].data[3] : window_main->tab_quickwatch_data[i].data[0];
              byte_2 = (window_main->tab_quickwatch_data[i].data_alignment == QUICKWATCH_ALIGNMENT_INTEL) ? window_main->tab_quickwatch_data[i].data[2] : window_main->tab_quickwatch_data[i].data[1];
              byte_3 = (window_main->tab_quickwatch_data[i].data_alignment == QUICKWATCH_ALIGNMENT_INTEL) ? window_main->tab_quickwatch_data[i].data[1] : window_main->tab_quickwatch_data[i].data[2];
              byte_4 = (window_main->tab_quickwatch_data[i].data_alignment == QUICKWATCH_ALIGNMENT_INTEL) ? window_main->tab_quickwatch_data[i].data[0] : window_main->tab_quickwatch_data[i].data[3];
                
              /* Affichage */
              sprintf(buffer_hexa,"%02X%02X.%02X%02X",byte_1,byte_2,byte_3,byte_4);
              FormatByteAsBinary(byte_1,&buffer_byte1[0]);
              FormatByteAsBinary(byte_2,&buffer_byte2[0]);
              FormatByteAsBinary(byte_3,&buffer_byte3[0]);
              FormatByteAsBinary(byte_4,&buffer_byte4[0]);
              sprintf(buffer_binary,"%s %s %s %s",buffer_byte1,buffer_byte2,buffer_byte3,buffer_byte4);
              one_dword = byte_1*16777216 + byte_2*65536 + byte_3*256 + byte_4;
              sprintf(buffer_unsigned,"%13u",(unsigned int)one_dword);
              sprintf(buffer_signed,"%13d",(int)(one_dword));
            } 
                        
          /** Rsultat **/
          sprintf(buffer,"%s  %s  %s\r\n%s",buffer_hexa,buffer_unsigned,buffer_signed,buffer_binary);
        }

      /* Affichage */
      SetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,tab_value_id[i]),buffer);
    }
}


/***************************************************************************************/
/*  LoadQuickWatchDataFile() :  Charge en mmoire les donnes d'un fichier QuickWatch. */
/***************************************************************************************/
void LoadQuickWatchDataFile(char *quickwatch_file_path, struct window_quickwatch *window_quickwatch)
{
  int i, file_size;
  char *file_data;
  char file_path[1024];
  char error_buffer[1024];
  char buffer_name[1024];
  char buffer_value[1024];
  struct symbol *current_symbol;
  int tab_ram_id[10]     = {IDC_RADIO_RAM1,IDC_RADIO_RAM2,IDC_RADIO_RAM3,IDC_RADIO_RAM4,IDC_RADIO_RAM5,IDC_RADIO_RAM6,IDC_RADIO_RAM7,IDC_RADIO_RAM8,IDC_RADIO_RAM9,IDC_RADIO_RAM10};
  int tab_docram_id[10]  = {IDC_RADIO_DOCRAM1,IDC_RADIO_DOCRAM2,IDC_RADIO_DOCRAM3,IDC_RADIO_DOCRAM4,IDC_RADIO_DOCRAM5,IDC_RADIO_DOCRAM6,IDC_RADIO_DOCRAM7,IDC_RADIO_DOCRAM8,IDC_RADIO_DOCRAM9,IDC_RADIO_DOCRAM10};
  int tab_bank_id[10]    = {IDC_EDIT_RAM_BANK1,IDC_EDIT_RAM_BANK2,IDC_EDIT_RAM_BANK3,IDC_EDIT_RAM_BANK4,IDC_EDIT_RAM_BANK5,IDC_EDIT_RAM_BANK6,IDC_EDIT_RAM_BANK7,IDC_EDIT_RAM_BANK8,IDC_EDIT_RAM_BANK9,IDC_EDIT_RAM_BANK10};
  int tab_address_id[10] = {IDC_EDIT_RAM_ADDRESS1,IDC_EDIT_RAM_ADDRESS2,IDC_EDIT_RAM_ADDRESS3,IDC_EDIT_RAM_ADDRESS4,IDC_EDIT_RAM_ADDRESS5,IDC_EDIT_RAM_ADDRESS6,IDC_EDIT_RAM_ADDRESS7,IDC_EDIT_RAM_ADDRESS8,IDC_EDIT_RAM_ADDRESS9,IDC_EDIT_RAM_ADDRESS10};
  int tab_value_id[10]   = {IDC_EDIT_VALUE1,IDC_EDIT_VALUE2,IDC_EDIT_VALUE3,IDC_EDIT_VALUE4,IDC_EDIT_VALUE5,IDC_EDIT_VALUE6,IDC_EDIT_VALUE7,IDC_EDIT_VALUE8,IDC_EDIT_VALUE9,IDC_EDIT_VALUE10};
  int tab_name_id[10]    = {IDC_EDIT_LABEL1,IDC_EDIT_LABEL2,IDC_EDIT_LABEL3,IDC_EDIT_LABEL4,IDC_EDIT_LABEL5,IDC_EDIT_LABEL6,IDC_EDIT_LABEL7,IDC_EDIT_LABEL8,IDC_EDIT_LABEL9,IDC_EDIT_LABEL10};
  int tab_combo_id[10]   = {IDC_COMBOBOX_QUICKWATCH_DATALENGTH_1,IDC_COMBOBOX_QUICKWATCH_DATALENGTH_2,IDC_COMBOBOX_QUICKWATCH_DATALENGTH_3,IDC_COMBOBOX_QUICKWATCH_DATALENGTH_4,IDC_COMBOBOX_QUICKWATCH_DATALENGTH_5,
                            IDC_COMBOBOX_QUICKWATCH_DATALENGTH_6,IDC_COMBOBOX_QUICKWATCH_DATALENGTH_7,IDC_COMBOBOX_QUICKWATCH_DATALENGTH_8,IDC_COMBOBOX_QUICKWATCH_DATALENGTH_9,IDC_COMBOBOX_QUICKWATCH_DATALENGTH_10};
  struct window_main *window_main;
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);

  /* Demande le chemin du fichier */
  if(quickwatch_file_path == NULL)
    {
      GetOneFilePath(window_quickwatch->hwnd_quickwatch,"Select QuickWatch file","QuickWatch Files (*.txt)\0*.txt\0\0","",file_path);
      if(strlen(file_path) == 0)
        return;
    }
  else
    strcpy(file_path,quickwatch_file_path);

  /* Charge les donnes du fichier */
  file_data = (char *) LoadTextFile(file_path,&file_size,error_buffer);
  if(file_data == NULL)
    return;

  /** On va rcuprer les 10 QuickWatch **/
  for(i=0; i<10; i++)
    {
      /** RAM Type **/
      sprintf(buffer_name,"RamType%d=",i+1);
      GetTextValue(file_data,buffer_name,buffer_value);
      if(!stricmp(buffer_value,"0"))
        {
          /* RAM */
          window_main->tab_quickwatch_data[i].ram_type = 0;

          /* MAJ Fentre */
          if(window_quickwatch != NULL)
            {
              SendDlgItemMessage(window_quickwatch->hwnd_quickwatch,tab_ram_id[i],BM_SETCHECK,1,0);
              SendDlgItemMessage(window_quickwatch->hwnd_quickwatch,tab_docram_id[i],BM_SETCHECK,0,0);
            }
        }
      else
        {
          /* RAM DOC */
          window_main->tab_quickwatch_data[i].ram_type = 1;

          /* MAJ Fentre */
          if(window_quickwatch != NULL)
            {
              SendDlgItemMessage(window_quickwatch->hwnd_quickwatch,tab_ram_id[i],BM_SETCHECK,0,0);
              SendDlgItemMessage(window_quickwatch->hwnd_quickwatch,tab_docram_id[i],BM_SETCHECK,1,0);
            }
        }

      /** Bank **/
      sprintf(buffer_name,"Bank%d=",i+1);
      GetTextValue(file_data,buffer_name,buffer_value);
      if(strlen(buffer_value) == 2)
        {
          sscanf(buffer_value,"%X",&window_main->tab_quickwatch_data[i].bank);
          sprintf(buffer_value,"%02X",window_main->tab_quickwatch_data[i].bank);

          /* MAJ Fentre */
          if(window_quickwatch != NULL)
            SetDlgItemText(window_quickwatch->hwnd_quickwatch,tab_bank_id[i],buffer_value);
        }
      else
        {
          window_main->tab_quickwatch_data[i].bank = 0x00;

          /* MAJ Fentre */
          if(window_quickwatch != NULL)
            SetDlgItemText(window_quickwatch->hwnd_quickwatch,tab_bank_id[i],"");
        }

      /** Address **/
      sprintf(buffer_name,"Address%d=",i+1);
      GetTextValue(file_data,buffer_name,buffer_value);
      if(strlen(buffer_value) == 4)
        {
          sscanf(buffer_value,"%X",&window_main->tab_quickwatch_data[i].address);
          sprintf(buffer_value,"%04X",window_main->tab_quickwatch_data[i].address);

          /* MAJ Fentre */
          if(window_quickwatch != NULL)
            SetDlgItemText(window_quickwatch->hwnd_quickwatch,tab_address_id[i],buffer_value);
        }
      else
        {
          window_main->tab_quickwatch_data[i].address = 0x0000;

          /* MAJ Fentre */
          if(window_quickwatch != NULL)
            SetDlgItemText(window_quickwatch->hwnd_quickwatch,tab_address_id[i],"");
        }

      /** DataLength / DataAligment **/
      sprintf(buffer_name,"DataLength%d=",i+1);
      GetTextValue(file_data,buffer_name,buffer_value);
      window_main->tab_quickwatch_data[i].data_length = atoi(buffer_value);
      sprintf(buffer_name,"DataAlignment%d=",i+1);
      GetTextValue(file_data,buffer_name,buffer_value);
      window_main->tab_quickwatch_data[i].data_alignment = atoi(buffer_value);
      if(window_main->tab_quickwatch_data[i].data_length == QUICKWATCH_LENGTH_8 && window_main->tab_quickwatch_data[i].data_alignment == QUICKWATCH_ALIGNMENT_INTEL)
        window_main->tab_quickwatch_data[i].combo_length_alignment = QUICKWATCH_COMBO_8;
      else if(window_main->tab_quickwatch_data[i].data_length == QUICKWATCH_LENGTH_16 && window_main->tab_quickwatch_data[i].data_alignment == QUICKWATCH_ALIGNMENT_INTEL)
        window_main->tab_quickwatch_data[i].combo_length_alignment = QUICKWATCH_COMBO_16_INTEL;
      else if(window_main->tab_quickwatch_data[i].data_length == QUICKWATCH_LENGTH_24 && window_main->tab_quickwatch_data[i].data_alignment == QUICKWATCH_ALIGNMENT_INTEL)
        window_main->tab_quickwatch_data[i].combo_length_alignment = QUICKWATCH_COMBO_24_INTEL;
      else if(window_main->tab_quickwatch_data[i].data_length == QUICKWATCH_LENGTH_32 && window_main->tab_quickwatch_data[i].data_alignment == QUICKWATCH_ALIGNMENT_INTEL)
        window_main->tab_quickwatch_data[i].combo_length_alignment = QUICKWATCH_COMBO_32_INTEL;
      else if(window_main->tab_quickwatch_data[i].data_length == QUICKWATCH_LENGTH_16 && window_main->tab_quickwatch_data[i].data_alignment == QUICKWATCH_ALIGNMENT_MOTOROLA)
        window_main->tab_quickwatch_data[i].combo_length_alignment = QUICKWATCH_COMBO_16_MOTOROLA;
      else if(window_main->tab_quickwatch_data[i].data_length == QUICKWATCH_LENGTH_24 && window_main->tab_quickwatch_data[i].data_alignment == QUICKWATCH_ALIGNMENT_MOTOROLA)
        window_main->tab_quickwatch_data[i].combo_length_alignment = QUICKWATCH_COMBO_24_MOTOROLA;
      else if(window_main->tab_quickwatch_data[i].data_length == QUICKWATCH_LENGTH_32 && window_main->tab_quickwatch_data[i].data_alignment == QUICKWATCH_ALIGNMENT_MOTOROLA)
        window_main->tab_quickwatch_data[i].combo_length_alignment= QUICKWATCH_COMBO_32_MOTOROLA;

      /* MAJ Fentre */
      if(window_quickwatch != NULL)
        SendMessage(GetDlgItem(window_quickwatch->hwnd_quickwatch,tab_combo_id[i]),CB_SETCURSEL,window_main->tab_quickwatch_data[i].combo_length_alignment,0);

      /** Name **/
      sprintf(buffer_name,"Name%d=",i+1);
      GetTextValue(file_data,buffer_name,buffer_value);
      strcpy(window_main->tab_quickwatch_data[i].name,buffer_value);
      /* MAJ Fentre */
      if(window_quickwatch != NULL)
        SetDlgItemText(window_quickwatch->hwnd_quickwatch,tab_name_id[i],buffer_value);

      /** Symbol **/
      sprintf(buffer_name,"HasSymbol%d=",i+1);
      GetTextValue(file_data,buffer_name,buffer_value);
      if(!stricmp(buffer_value,"1"))
        {
          sprintf(buffer_name,"SymbolName%d=",i+1);
          GetTextValue(file_data,buffer_name,buffer_value);
          strcpy(window_main->tab_quickwatch_data[i].symbol_name,buffer_value);
          /* MAJ Fentre */
          if(window_quickwatch != NULL)
            SetDlgItemText(window_quickwatch->hwnd_quickwatch,tab_name_id[i],buffer_value);   /* Ecrase le nom par celui du Symbol */
          window_main->tab_quickwatch_data[i].has_symbol = 1;
          sprintf(buffer_name,"SymbolFile%d=",i+1);
          GetTextValue(file_data,buffer_name,buffer_value);
          strcpy(window_main->tab_quickwatch_data[i].symbol_file,buffer_value);
          sprintf(buffer_name,"SymbolSegment%d=",i+1);
          GetTextValue(file_data,buffer_name,buffer_value);
          strcpy(window_main->tab_quickwatch_data[i].symbol_segment,buffer_value);

          /** Recherche l'adresse du Symbol **/
          current_symbol = GetSymbolByNameSegmentFile(window_main->tab_quickwatch_data[i].symbol_name,window_main->tab_quickwatch_data[i].symbol_segment,window_main->tab_quickwatch_data[i].symbol_file);
          if(current_symbol)
            {
              /* Bank */
              window_main->tab_quickwatch_data[i].bank = (BYTE) ((current_symbol->symbol_relocated_address >> 16) & 0x000000FF);
              sprintf(buffer_value,"%02X",window_main->tab_quickwatch_data[i].bank);
              /* MAJ Fentre */
              if(window_quickwatch != NULL)
                SetDlgItemText(window_quickwatch->hwnd_quickwatch,tab_bank_id[i],buffer_value);

              /* Adresse */
              window_main->tab_quickwatch_data[i].address = (WORD) (current_symbol->symbol_relocated_address & 0x0000FFFF);
              sprintf(buffer_value,"%04X",window_main->tab_quickwatch_data[i].address);
              /* MAJ Fentre */
              if(window_quickwatch != NULL)
                SetDlgItemText(window_quickwatch->hwnd_quickwatch,tab_address_id[i],buffer_value);
            }
        }
      else
        {
          window_main->tab_quickwatch_data[i].has_symbol = 0;
          strcpy(window_main->tab_quickwatch_data[i].symbol_file,"");
          strcpy(window_main->tab_quickwatch_data[i].symbol_segment,"");
        }
    }

  /* Libration mmoire */
  free(file_data);

  /* On rafrachit la fentre */
  if(window_quickwatch != NULL)
    RefreshQuickWatchWindowData(window_quickwatch);
}


/***********************************************************************************/
/*  SaveQuickWatchDataFile() :  Sauvegarde dans un fichier les donnes Quickwatch. */
/***********************************************************************************/
static void SaveQuickWatchDataFile(struct window_quickwatch *window_quickwatch)
{
  int i;
  FILE *fd;
  BOOL is_OK;
  char quickwatch_file_path[1024];
  struct window_main *window_main;
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);

  /* On rafrachit la fentre pour forcer la rcupration des valeurs saisies */
  RefreshQuickWatchWindowData(window_quickwatch);

  /* Demande le nom de l'image */
  strcpy(quickwatch_file_path,"QuickWatch.txt");
      
  /** Fentre de slection de chemin de fichier **/
  is_OK = GetSaveFilePath(window_quickwatch->hwnd_quickwatch,"Select QuickWatch file","All Files (*.*)\0*.*\0\0\0",quickwatch_file_path);

  /* Si l'utilisateur l'annule */
  if(!is_OK)
    return;

  /* Cration du fichier */
  fd = fopen(quickwatch_file_path,"w");
  if(fd == NULL)
    {
      MessageBox(window_quickwatch->hwnd_quickwatch,"Impossible to create QuickWatch file","Create QuickWatch file Error",MB_OK|MB_ICONWARNING);    
      return;
    }

  /** Remplissage du fichier **/
  for(i=0; i<10; i++)
    {
      /* Adresse */
      fprintf(fd,"RamType%d=%d\n",i+1,window_main->tab_quickwatch_data[i].ram_type);
      if(window_main->tab_quickwatch_data[i].is_data_error == 1)
        {
          fprintf(fd,"Bank%d=\n",i+1);
          fprintf(fd,"Address%d=\n",i+1);
        }
      else
        {
          fprintf(fd,"Bank%d=%02X\n",i+1,window_main->tab_quickwatch_data[i].bank);
          fprintf(fd,"Address%d=%04X\n",i+1,window_main->tab_quickwatch_data[i].address);
        }
      fprintf(fd,"DataLength%d=%d\n",i+1,window_main->tab_quickwatch_data[i].data_length);
      fprintf(fd,"DataAlignment%d=%d\n",i+1,window_main->tab_quickwatch_data[i].data_alignment);

      /* Name */
      fprintf(fd,"Name%d=%s\n",i+1,window_main->tab_quickwatch_data[i].name);

      /* Symbol */
      fprintf(fd,"HasSymbol%d=%d\n",i+1,window_main->tab_quickwatch_data[i].has_symbol);
      fprintf(fd,"SymbolName%d=%s\n",i+1,window_main->tab_quickwatch_data[i].symbol_name);
      fprintf(fd,"SymbolSegment%d=%s\n",i+1,window_main->tab_quickwatch_data[i].symbol_segment);
      fprintf(fd,"SymbolFile%d=%s\n",i+1,window_main->tab_quickwatch_data[i].symbol_file);
    }

  /* Fermeture du fichier */
  fclose(fd);
}


/**********************************************************************************************/
/*  UpdateQuickWatchSymbolAddress() :  MAJ des adresses des QuickWatch utilisant des Symbols. */
/**********************************************************************************************/
void UpdateQuickWatchSymbolAddress(void)
{
  int i, refresh_window;
  struct symbol *current_symbol;
  struct window_main *window_main;
  struct window_quickwatch *window_quickwatch;
  struct quickwatch_data *current_quickwatch;
  char buffer[BUFFER_SIZE];
  int tab_bank_id[10]    = {IDC_EDIT_RAM_BANK1,IDC_EDIT_RAM_BANK2,IDC_EDIT_RAM_BANK3,IDC_EDIT_RAM_BANK4,IDC_EDIT_RAM_BANK5,IDC_EDIT_RAM_BANK6,IDC_EDIT_RAM_BANK7,IDC_EDIT_RAM_BANK8,IDC_EDIT_RAM_BANK9,IDC_EDIT_RAM_BANK10};
  int tab_address_id[10] = {IDC_EDIT_RAM_ADDRESS1,IDC_EDIT_RAM_ADDRESS2,IDC_EDIT_RAM_ADDRESS3,IDC_EDIT_RAM_ADDRESS4,IDC_EDIT_RAM_ADDRESS5,IDC_EDIT_RAM_ADDRESS6,IDC_EDIT_RAM_ADDRESS7,IDC_EDIT_RAM_ADDRESS8,IDC_EDIT_RAM_ADDRESS9,IDC_EDIT_RAM_ADDRESS10};

  /** Rcupre les fentres Main / QuickWatch **/
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  if(window_main == NULL)
    return;
  if(window_main->current_symbol_table == NULL)
    return;
  my_Memory(MEMORY_GET_WINDOW_QUICKWATCH,&window_quickwatch,NULL);

  /*** Passer en revue toutes les QuickWatch ***/
  refresh_window = 0;
  for(i=0; i<10; i++)
    {
      /* Rcupre un QuickWatch */
      current_quickwatch = &window_main->tab_quickwatch_data[i];

      /** QuickWtach Address : Symbol **/
      if(current_quickwatch->has_symbol == 1)
        {
          /* Recherche */
          for(current_symbol=window_main->current_symbol_table->first_symbol; current_symbol; current_symbol=current_symbol->next)
            if(!strcmp(current_symbol->name,current_quickwatch->symbol_name) &&                    /* Le Symbole est case sensitive */
               !stricmp(current_symbol->segment_name,current_quickwatch->symbol_segment) &&
               !stricmp(current_symbol->program_name,current_quickwatch->symbol_file))
              {
                /* Rcupre l'adresse du Symbol */
                current_quickwatch->symbol_address = current_symbol->symbol_relocated_address;
                current_quickwatch->bank           = (BYTE) ((current_quickwatch->symbol_address >> 16) & 0x000000FF);
                current_quickwatch->address        = (WORD) (current_quickwatch->symbol_address & 0x0000FFFF);

                /* Mise  jour de l'affichage */
                if(window_quickwatch != NULL)
                  {
                    sprintf(buffer,"%02X",current_quickwatch->bank);
                    SetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,tab_bank_id[i]),buffer);
                    sprintf(buffer,"%04X",current_quickwatch->address);
                    SetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,tab_address_id[i]),buffer);
                  }

                /* Il faudra mettre  jour la fentre  la fin */
                refresh_window = 1;
                break;
              }
        }
    }

  /* MAJ L'affichage */
  if(window_quickwatch != NULL && refresh_window == 1)
    RefreshQuickWatchWindowData(window_quickwatch);
}


/***************************************************************/
/*  RefreshQuickWatchWindow() :  MAJ de la fentre QuickWatch. */
/***************************************************************/
void RefreshQuickWatchWindow(struct window_quickwatch *window_quickwatch)
{
  int i;
  struct window_main *window_main;
  int tab_bank_id[10]    = {IDC_EDIT_RAM_BANK1,IDC_EDIT_RAM_BANK2,IDC_EDIT_RAM_BANK3,IDC_EDIT_RAM_BANK4,IDC_EDIT_RAM_BANK5,IDC_EDIT_RAM_BANK6,IDC_EDIT_RAM_BANK7,IDC_EDIT_RAM_BANK8,IDC_EDIT_RAM_BANK9,IDC_EDIT_RAM_BANK10};
  int tab_address_id[10] = {IDC_EDIT_RAM_ADDRESS1,IDC_EDIT_RAM_ADDRESS2,IDC_EDIT_RAM_ADDRESS3,IDC_EDIT_RAM_ADDRESS4,IDC_EDIT_RAM_ADDRESS5,IDC_EDIT_RAM_ADDRESS6,IDC_EDIT_RAM_ADDRESS7,IDC_EDIT_RAM_ADDRESS8,IDC_EDIT_RAM_ADDRESS9,IDC_EDIT_RAM_ADDRESS10};
  int tab_name_id[10]    = {IDC_EDIT_LABEL1,IDC_EDIT_LABEL2,IDC_EDIT_LABEL3,IDC_EDIT_LABEL4,IDC_EDIT_LABEL5,IDC_EDIT_LABEL6,IDC_EDIT_LABEL7,IDC_EDIT_LABEL8,IDC_EDIT_LABEL9,IDC_EDIT_LABEL10};
  char buffer[BUFFER_SIZE];

  /** Rcupre les fentres Main / QuickWatch **/
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  if(window_main == NULL)
    return;

  /** Init des Valeurs **/
  for(i=0; i<10; i++)
    {
      /** Valeur des Radio Boutons Ram Type (+ MAJ de la fentre) **/
      UpdateQuickWatchRamType(window_quickwatch,i,window_main->tab_quickwatch_data[i].ram_type,0);

      /** Adresses **/
      if(window_main->tab_quickwatch_data[i].is_data_error == 1)
        {
          SetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,tab_bank_id[i]),"");
          SetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,tab_address_id[i]),"");
        }
      else
        {
          sprintf(buffer,"%02X",window_main->tab_quickwatch_data[i].bank);
          SetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,tab_bank_id[i]),buffer);
          sprintf(buffer,"%04X",window_main->tab_quickwatch_data[i].address);
          SetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,tab_address_id[i]),buffer);
        }

      /** Init des valeurs Aligment **/
      SendMessage(window_quickwatch->tab_hwnd_combo_length_alignment[i],CB_SETCURSEL,window_main->tab_quickwatch_data[i].combo_length_alignment,0);

      /** Nom **/
      SetWindowText(GetDlgItem(window_quickwatch->hwnd_quickwatch,tab_name_id[i]),window_main->tab_quickwatch_data[i].name);
    }

  /** Construction des Data + Rafraichissement de la fentre **/
  RefreshQuickWatchWindowData(window_quickwatch);
}


/************************************************************************/
/*  DeleteQuickWatchChildWindows() :  Destruction des fentres enfants. */
/************************************************************************/
static void DeleteQuickWatchChildWindows(struct window_quickwatch *window_quickwatch)
{
}


/**************************************************************************/
/*  mem_free_window_quickwatch() :  Libration mmoire pour la structure. */
/**************************************************************************/
void mem_free_window_quickwatch(struct window_quickwatch *window_quickwatch)
{
  if(window_quickwatch)
    {
      /* Supprime la notification */
      my_Notification(NOTIFICATION_REMOVE,window_quickwatch->current_notification,NULL);
      window_quickwatch->current_notification = NULL;
      
      free(window_quickwatch);
    }
}

/**********************************************************************/
