/************************************************************************/
/*                                                                      */
/*  Window_Replace.c :  Module de gestion de la fentre Replace.        */
/*                                                                      */
/************************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe  *  Date : Octobre 2018  */
/************************************************************************/

#include <stdio.h>
#include <time.h>

#include <windows.h>
#include <windowsx.h>
#include <Commctrl.h>
#include <Winuser.h>
#include <Richedit.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_Project.h"
#include "Dc_MMU.h"
#include "Dc_Snapshot.h"
#include "Dc_Notification.h"
#include "Dc_Graphic.h"
#include "Dc_CPU.h"
#include "Dc_Request.h"
#include "Cyrene_Engine.h"
#include "Window_QuickWatch.h"
#include "Window_Main.h"
#include "Window_Main_Menu.h"
#include "Window_Replace.h"

static DWORD BuildReplaceWindowThread(void *);
LRESULT CALLBACK ProcHandlerReplaceWindow(HWND,UINT,WPARAM,LPARAM);
static int CreateReplaceChildWindows(HWND,struct window_replace *);
LRESULT CALLBACK subEditReplaceProcControl(HWND,UINT,WPARAM,LPARAM);
static void UpdateSearchType(struct window_replace *,int);
static void UpdateSearchSize(struct window_replace *,int);
static void UpdateSearchAlign(struct window_replace *,int);
static void UpdateSearchCharset(struct window_replace *,int);
static void UpdateSearchOptions(struct window_replace *,int);
static void UpdateSearchArea(struct window_replace *,int);
static void UpdateReplaceType(struct window_replace *,int);
static void UpdateReplaceSize(struct window_replace *,int);
static void UpdateReplaceAlign(struct window_replace *,int);
static void UpdateReplaceCharset(struct window_replace *,int);
static void SearchForData(struct window_replace *);
static void DisplaySearchResult(struct window_replace *,int);
static DWORD CALLBACK ReplaceRichEditStreamCallback(DWORD *,LPBYTE,LONG,LONG *);
static void DeleteReplaceChildWindows(struct window_replace *);
static void BuildSearchFromData(struct window_replace *,struct replace_result *);
static void EmptySearchResult(struct window_replace *);
static void ReplaceSearchResult(struct window_replace *,int);
static struct replace_result *mem_alloc_replaceresult(int,int,int,unsigned char *);
static void mem_free_all_replaceresult(struct replace_result *);


/****************************************************************************/
/*  BuildReplaceWindow() :  Construction / Affichage de la fentre Replace. */
/****************************************************************************/
struct window_replace *BuildReplaceWindow(HINSTANCE hInstance, int menu_id)
{
  struct window_replace *window_replace;

  /** Allocation de la structure window_replace **/
  window_replace = (struct window_replace *) calloc(1,sizeof(struct window_replace));
  if(window_replace == NULL)
    return(NULL);
  my_Memory(MEMORY_SET_WINDOW_REPLACE,window_replace,NULL);

  /* Lance un thread pour grer la fentre Replace */
  CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)BuildReplaceWindowThread,window_replace,0,NULL);

  /* Renvoi la structure mmoire grant la fentre */
  return(window_replace);
}


/**************************************************************************************/
/*  BuildReplaceWindowThread() :  Cration de la fentre Replace via un Thread ddi. */
/**************************************************************************************/
static DWORD BuildReplaceWindowThread(void *window_replace_param)
{
  HWND hwndReplace;
  MSG msg;
  BOOL bRet;
  struct window_main *window_main;
  struct window_replace *window_replace;
  struct parameter *param;

  /* Init */
  window_replace = (struct window_replace *) window_replace_param;
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Cration de la fentre Direct Page **/
  hwndReplace = CreateDialog(param->hInstance,MAKEINTRESOURCE(IDD_WINDOW_REPLACE),window_main->hwnd_main,(DLGPROC)ProcHandlerReplaceWindow);
  if(hwndReplace == NULL)
    return(1);

  /** Show Window + Paint Content **/
  ShowWindow(hwndReplace,SW_SHOWDEFAULT);
  UpdateWindow(hwndReplace);

  /** Message Loop pour toutes les fentres de ce Thread (fentre Replace...) **/
  while((bRet = GetMessage(&msg,NULL,0,0)) != 0)
    {
      if(bRet == -1)
        break;

      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }

  /* Fin du Thread */
  ExitThread(0);
}


/************************************************************************/
/*  ProcHandlerReplaceWindow() :  Gestion des vnements de la fentre. */
/************************************************************************/
LRESULT CALLBACK ProcHandlerReplaceWindow(HWND hwndReplace, UINT umsg, WPARAM wParam, LPARAM lParam)
{
  struct window_replace *window_replace;
  struct parameter *param;
  
  /* Init */
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  my_Memory(MEMORY_GET_WINDOW_REPLACE,&window_replace,NULL);

  switch(umsg)
    {
      case WM_INITDIALOG :
        /* Cration du contenu de la DialogBox */
        CreateReplaceChildWindows(hwndReplace,window_replace);
        return(TRUE);

      /****************************/
      /* Evnements de la fentre */
      /****************************/
      case WM_COMMAND :
        switch(LOWORD(wParam))
          {
            /** Bouton Radio : On change le type des donnes  rechercher **/
            case IDC_RADIO_TYPE_HEXA :
              if(HIWORD(wParam) == BN_CLICKED && window_replace->search_type != REPLACE_TYPE_HEXA)
                UpdateSearchType(window_replace,REPLACE_TYPE_HEXA);
              break;

            case IDC_RADIO_TYPE_DECIMAL :
              if(HIWORD(wParam) == BN_CLICKED && window_replace->search_type != REPLACE_TYPE_DECIMAL)
                UpdateSearchType(window_replace,REPLACE_TYPE_DECIMAL);
              break;

            case IDC_RADIO_TYPE_TEXT :
              if(HIWORD(wParam) == BN_CLICKED && window_replace->search_type != REPLACE_TYPE_TEXT)
                UpdateSearchType(window_replace,REPLACE_TYPE_TEXT);
              break;

            case IDC_RADIO_TYPE_BYTES :
              if(HIWORD(wParam) == BN_CLICKED && window_replace->search_type != REPLACE_TYPE_BYTES)
                UpdateSearchType(window_replace,REPLACE_TYPE_BYTES);
              break;

            /** Bouton Radio : On slectionne la taille des donnes  rechercher **/
            case IDC_RADIO_SIZE_8 :
              if(HIWORD(wParam) == BN_CLICKED && window_replace->search_size != REPLACE_SIZE_8)
                UpdateSearchSize(window_replace,REPLACE_SIZE_8);
              break;

            case IDC_RADIO_SIZE_16 :
              if(HIWORD(wParam) == BN_CLICKED && window_replace->search_size != REPLACE_SIZE_16)
                UpdateSearchSize(window_replace,REPLACE_SIZE_16);
              break;

            case IDC_RADIO_SIZE_24 :
              if(HIWORD(wParam) == BN_CLICKED && window_replace->search_size != REPLACE_SIZE_24)
                UpdateSearchSize(window_replace,REPLACE_SIZE_24);
              break;

            case IDC_RADIO_SIZE_32 :
              if(HIWORD(wParam) == BN_CLICKED && window_replace->search_size != REPLACE_SIZE_32)
                UpdateSearchSize(window_replace,REPLACE_SIZE_32);
              break;

            /** Bouton Radio : On slectionne l'alignement des donnes  chercher **/
            case IDC_RADIO_ALIGN_INTEL :
              if(HIWORD(wParam) == BN_CLICKED && window_replace->search_align != REPLACE_ALIGN_INTEL)
                UpdateSearchAlign(window_replace,REPLACE_ALIGN_INTEL);
              break;

            case IDC_RADIO_ALIGN_MOTOROLA :
              if(HIWORD(wParam) == BN_CLICKED && window_replace->search_align != REPLACE_ALIGN_MOTOROLA)
                UpdateSearchAlign(window_replace,REPLACE_ALIGN_MOTOROLA);
              break;

            /** Check Box : On choisit le Charset des chaines  chercher **/
            case IDC_CHECK_ASCII :
              if(HIWORD(wParam) == BN_CLICKED)
                UpdateSearchCharset(window_replace,REPLACE_CHARSET_ASCII);
              break;

            case IDC_CHECK_HIGHASCII :
              if(HIWORD(wParam) == BN_CLICKED)
                UpdateSearchCharset(window_replace,REPLACE_CHARSET_HIGHASCII);
              break;

            case IDC_CHECK_INVERSE :
              if(HIWORD(wParam) == BN_CLICKED)
                UpdateSearchCharset(window_replace,REPLACE_CHARSET_INVERSE);
              break;

            case IDC_CHECK_FLASH :
              if(HIWORD(wParam) == BN_CLICKED)
                UpdateSearchCharset(window_replace,REPLACE_CHARSET_FLASH);
              break;

            /** Check Box : On choisit les Options des chaines  chercher **/
            case IDC_CHECK_CASESENSITIVE :
              if(HIWORD(wParam) == BN_CLICKED)
                UpdateSearchOptions(window_replace,REPLACE_OPTIONS_CASESENSITIVE);
              break;

            case IDC_CHECK_REVERSEORDER :
              if(HIWORD(wParam) == BN_CLICKED)
                UpdateSearchOptions(window_replace,REPLACE_OPTIONS_REVERSEORDER);
              break;

            case IDC_CHECK_WILDCARD :
              if(HIWORD(wParam) == BN_CLICKED)
                UpdateSearchOptions(window_replace,REPLACE_OPTIONS_WILDCARD);
              break;

            /** Check Box : On choisit la zone de recherche **/
            case IDC_CHECK_FASTRAM :
              if(HIWORD(wParam) == BN_CLICKED)
                UpdateSearchArea(window_replace,REPLACE_AREA_FASTRAM);
              break;
              
            case IDC_CHECK_SLOWRAM :
              if(HIWORD(wParam) == BN_CLICKED)
                UpdateSearchArea(window_replace,REPLACE_AREA_SLOWRAM);
              break;
              
            case IDC_CHECK_ROM :
              if(HIWORD(wParam) == BN_CLICKED)
                UpdateSearchArea(window_replace,REPLACE_AREA_ROM);
              break;

            case IDC_CHECK_DOCRAM :
              if(HIWORD(wParam) == BN_CLICKED)
                UpdateSearchArea(window_replace,REPLACE_AREA_DOCRAM);
              break;

            /** Bouton Radio : On change le type des donnes  remplacer **/
            case IDC_RADIO_REPLACE_TYPE_HEXA :
              if(HIWORD(wParam) == BN_CLICKED && window_replace->replace_type != REPLACE_TYPE_HEXA)
                UpdateReplaceType(window_replace,REPLACE_TYPE_HEXA);
              break;

            case IDC_RADIO_REPLACE_TYPE_DECIMAL :
              if(HIWORD(wParam) == BN_CLICKED && window_replace->replace_type != REPLACE_TYPE_DECIMAL)
                UpdateReplaceType(window_replace,REPLACE_TYPE_DECIMAL);
              break;

            case IDC_RADIO_REPLACE_TYPE_TEXT :
              if(HIWORD(wParam) == BN_CLICKED && window_replace->replace_type != REPLACE_TYPE_TEXT)
                UpdateReplaceType(window_replace,REPLACE_TYPE_TEXT);
              break;

            case IDC_RADIO_REPLACE_TYPE_BYTES :
              if(HIWORD(wParam) == BN_CLICKED && window_replace->replace_type != REPLACE_TYPE_BYTES)
                UpdateReplaceType(window_replace,REPLACE_TYPE_BYTES);
              break;

            /** Bouton Radio : On slectionne la taille des donnes  remplacer **/
            case IDC_RADIO_REPLACE_SIZE_8 :
              if(HIWORD(wParam) == BN_CLICKED && window_replace->replace_size != REPLACE_SIZE_8)
                UpdateReplaceSize(window_replace,REPLACE_SIZE_8);
              break;

            case IDC_RADIO_REPLACE_SIZE_16 :
              if(HIWORD(wParam) == BN_CLICKED && window_replace->replace_size != REPLACE_SIZE_16)
                UpdateReplaceSize(window_replace,REPLACE_SIZE_16);
              break;

            case IDC_RADIO_REPLACE_SIZE_24 :
              if(HIWORD(wParam) == BN_CLICKED && window_replace->replace_size != REPLACE_SIZE_24)
                UpdateReplaceSize(window_replace,REPLACE_SIZE_24);
              break;

            case IDC_RADIO_REPLACE_SIZE_32 :
              if(HIWORD(wParam) == BN_CLICKED && window_replace->replace_size != REPLACE_SIZE_32)
                UpdateReplaceSize(window_replace,REPLACE_SIZE_32);
              break;

            /** Bouton Radio : On slectionne l'alignement des donnes  remplacer **/
            case IDC_RADIO_REPLACE_ALIGN_INTEL :
              if(HIWORD(wParam) == BN_CLICKED && window_replace->replace_align != REPLACE_ALIGN_INTEL)
                UpdateReplaceAlign(window_replace,REPLACE_ALIGN_INTEL);
              break;

            case IDC_RADIO_REPLACE_ALIGN_MOTOROLA :
              if(HIWORD(wParam) == BN_CLICKED && window_replace->replace_align != REPLACE_ALIGN_MOTOROLA)
                UpdateReplaceAlign(window_replace,REPLACE_ALIGN_MOTOROLA);
              break;

            /** Bouton Radio : On choisit le Charset des chaines  remplacer **/
            case IDC_RADIO_REPLACE_ASCII :
              if(HIWORD(wParam) == BN_CLICKED && window_replace->replace_charset != REPLACE_CHARSET_ASCII)
                UpdateReplaceCharset(window_replace,REPLACE_CHARSET_ASCII);
              break;

            case IDC_RADIO_REPLACE_HIGHASCII :
              if(HIWORD(wParam) == BN_CLICKED && window_replace->replace_charset != REPLACE_CHARSET_HIGHASCII)
                UpdateReplaceCharset(window_replace,REPLACE_CHARSET_HIGHASCII);
              break;

            case IDC_RADIO_REPLACE_INVERSE :
              if(HIWORD(wParam) == BN_CLICKED && window_replace->replace_charset != REPLACE_CHARSET_INVERSE)
                UpdateReplaceCharset(window_replace,REPLACE_CHARSET_INVERSE);
              break;

            case IDC_RADIO_REPLACE_FLASH :
              if(HIWORD(wParam) == BN_CLICKED && window_replace->replace_charset != REPLACE_CHARSET_FLASH)
                UpdateReplaceCharset(window_replace,REPLACE_CHARSET_FLASH);
              break;

            /** Bouton pour lancer la recherche **/
            case IDC_BUTTON_SEARCH :
              SearchForData(window_replace);
              break;

            /** Boutons pour faire dfiler les rsultats **/
            case IDC_BUTTON_PREVIOUS :
              DisplaySearchResult(window_replace,-1);
              break;
            case IDC_BUTTON_NEXT :
              DisplaySearchResult(window_replace,1);
              break;

            /** Boutons pour remplacer les valeurs **/
            case IDC_BUTTON_REPLACE :
              ReplaceSearchResult(window_replace,REPLACE_CURRENT_RESULT);
              break;
            case IDC_BUTTON_REPLACEALL :
              ReplaceSearchResult(window_replace,REPLACE_ALL_RESULT);
              break;
          }
        return(TRUE);

      /********************************************************************/
      /* Fermeture de window (le WM_CLOSE est appel avant le WM_DESTROY) */
      /********************************************************************/
      case WM_CLOSE :
        /* Fentre Replace */
        DestroyWindow(hwndReplace);
        return(0);

      case WM_DESTROY :
        /* Suppression des Fentres */
        DeleteReplaceChildWindows(window_replace);
        CheckMainWindowMenuItem(ID_MENU_EDIT_REPLACE,FALSE);
        my_Memory(MEMORY_FREE_WINDOW_REPLACE,NULL,NULL);
        /* Demande la sortie de la boucle des messages */
        PostQuitMessage(0);
        return(0);
    }

  /* Retourner FALSE pour relayer le message non trait au systme */
  return(FALSE);
}


/*****************************************************************************************************/
/*  CreateReplaceChildWindows() : Crer les fentres enfants et remplir la structure window_replace. */
/*****************************************************************************************************/
static int CreateReplaceChildWindows(HWND hwndReplace, struct window_replace *window_replace)
{
  HWND hwnd_group;
  DWORD dialog_background_color;  
  RECT client_rect, window_rect, group_rect;
  char buffer[256];
  struct s65816 *current_65816;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Modle mmoire **/
  my_Memory(MEMORY_GET_S65816,&current_65816,NULL);
  if(current_65816 != NULL)
    {
      /* Fast RAM */
      window_replace->nb_fastram_bank = current_65816->current_mmu->nb_extended_bank + 2;

      /* ROM */
      if(current_65816->current_mmu->rom_version == ROM_APPLEIIGS_01)
        window_replace->nb_rom_bank = 2;
      else if(current_65816->current_mmu->rom_version == ROM_APPLEIIGS_03)
        window_replace->nb_rom_bank = 4;
    }
    
  /** Surchage des Control Edit pour capter les RETURN lors des saisies **/
  window_replace->oldEditProcSearch = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndReplace,IDC_EDIT_SEARCH),GWLP_WNDPROC,(LONG_PTR)subEditReplaceProcControl);
  window_replace->oldEditProcFastramFromBank = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndReplace,IDC_EDIT_FASTRAM_FROM_BANK),GWLP_WNDPROC,(LONG_PTR)subEditReplaceProcControl);
  window_replace->oldEditProcFastramFromAddress = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndReplace,IDC_EDIT_FASTRAM_FROM_ADDRESS),GWLP_WNDPROC,(LONG_PTR)subEditReplaceProcControl);
  window_replace->oldEditProcFastramToBank = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndReplace,IDC_EDIT_FASTRAM_TO_BANK),GWLP_WNDPROC,(LONG_PTR)subEditReplaceProcControl);
  window_replace->oldEditProcFastramToAddress = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndReplace,IDC_EDIT_FASTRAM_TO_ADDRESS),GWLP_WNDPROC,(LONG_PTR)subEditReplaceProcControl);
  window_replace->oldEditProcSlowramFromBank = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndReplace,IDC_EDIT_SLOWRAM_FROM_BANK),GWLP_WNDPROC,(LONG_PTR)subEditReplaceProcControl);
  window_replace->oldEditProcSlowramFromAddress = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndReplace,IDC_EDIT_SLOWRAM_FROM_ADDRESS),GWLP_WNDPROC,(LONG_PTR)subEditReplaceProcControl);
  window_replace->oldEditProcSlowramToBank = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndReplace,IDC_EDIT_SLOWRAM_TO_BANK),GWLP_WNDPROC,(LONG_PTR)subEditReplaceProcControl);
  window_replace->oldEditProcSlowramToAddress = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndReplace,IDC_EDIT_SLOWRAM_TO_ADDRESS),GWLP_WNDPROC,(LONG_PTR)subEditReplaceProcControl);
  window_replace->oldEditProcRomFromBank = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndReplace,IDC_EDIT_ROM_FROM_BANK),GWLP_WNDPROC,(LONG_PTR)subEditReplaceProcControl);
  window_replace->oldEditProcRomFromAddress = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndReplace,IDC_EDIT_ROM_FROM_ADDRESS),GWLP_WNDPROC,(LONG_PTR)subEditReplaceProcControl);
  window_replace->oldEditProcRomToBank = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndReplace,IDC_EDIT_ROM_TO_BANK),GWLP_WNDPROC,(LONG_PTR)subEditReplaceProcControl);
  window_replace->oldEditProcRomToAddress = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndReplace,IDC_EDIT_ROM_TO_ADDRESS),GWLP_WNDPROC,(LONG_PTR)subEditReplaceProcControl);
  window_replace->oldEditProcDocramFromAddress = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndReplace,IDC_EDIT_DOCRAM_FROM_ADDRESS),GWLP_WNDPROC,(LONG_PTR)subEditReplaceProcControl);
  window_replace->oldEditProcDocramToAddress = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndReplace,IDC_EDIT_DOCRAM_TO_ADDRESS),GWLP_WNDPROC,(LONG_PTR)subEditReplaceProcControl);
  window_replace->oldEditProcReplace = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndReplace,IDC_EDIT_REPLACE),GWLP_WNDPROC,(LONG_PTR)subEditReplaceProcControl);
    
  /** Allocation Buffer mmoire Zone Texte **/
  window_replace->nb_line = 2; 
  window_replace->nb_column = 16;
  /* Nombre caractres visibles sur la ligne (on ne compte pas le \r\n final) */
  window_replace->text_data_width = (int) (strlen("00/0000   ") + window_replace->nb_column*3 + 2 + window_replace->nb_column - 1);   /* Enlve le dernier sparateur */
  /* Nombre d'octet pour encoder la zone Texte (on compte les \r\n en fin de ligne) */
  window_replace->rtf_data_size = REPLACE_RICHEDIT_HEADER_SIZE;
  window_replace->rtf_data_size += ((window_replace->text_data_width + (int)strlen("\r\n"))*window_replace->nb_line)*REPLACE_RICHEDIT_MAX_CHAR_SIZE;
  window_replace->rtf_data_size += REPLACE_RICHEDIT_FOOTER_SIZE;
  window_replace->rtf_data_hexadecimal = (char *) calloc(window_replace->rtf_data_size+1,sizeof(char));
  if(window_replace->rtf_data_hexadecimal == NULL)
    return(1);

  /** Window Handle **/
  window_replace->hwnd_replace = hwndReplace;

  /* Hauteur Titre */
  GetWindowRect(window_replace->hwnd_replace,&window_rect);
  GetClientRect(window_replace->hwnd_replace,&client_rect);
  window_replace->title_height = (window_rect.bottom - window_rect.top) - client_rect.bottom;

  /* Position de la GroupBox Replace Result */
  hwnd_group = GetDlgItem(window_replace->hwnd_replace,IDC_GROUP_REPLACE);     /* Rcupre le Handle du Control GroupBox li  IDC_GROUP_REPLACE */
  GetWindowRect(hwnd_group,&group_rect);
  window_replace->group_data_x = group_rect.left - window_rect.left;
  window_replace->group_data_y = (group_rect.top - window_rect.top) - window_replace->title_height;

  /* Position de la Zone Text Direct Page Data dans la fentre */
  window_replace->replace_data_x = window_replace->group_data_x + WINDOW_GROUP_BORDER_LEFT_SIZE + 3;
  window_replace->replace_data_y = window_replace->group_data_y + WINDOW_GROUP_BORDER_TOP_SIZE + 34;

  /* Dimension en Pixel du Control Text Replace Data */
  window_replace->replace_data_width = window_replace->text_data_width*7 + 2;
  window_replace->replace_data_height = window_replace->nb_line*REPLACE_EDIT_FONT_HEIGHT;

  /** Cration du Rich Edit **/
  window_replace->hwnd_richedit_data = CreateWindowEx(0,RICHEDIT_CLASS,"",WS_CHILD | WS_VISIBLE | ES_SAVESEL | ES_MULTILINE | ES_READONLY, 
                                                      window_replace->replace_data_x, window_replace->replace_data_y, window_replace->replace_data_width, window_replace->replace_data_height,   // set size in WM_SIZE message 
                                                      window_replace->hwnd_replace,
                                                      (HMENU)IDC_RICHEDIT_REPLACE_DATA,
                                                      param->hInstance,
                                                      NULL);

  /* Couleur de Fond du control Rich Edit : Gris comme la fentre d'accueil */
  dialog_background_color = GetSysColor(COLOR_3DFACE);
  SendMessage(window_replace->hwnd_richedit_data,EM_SETBKGNDCOLOR,0,(LPARAM)dialog_background_color); 

  /* Search Type */
  window_replace->search_type = REPLACE_TYPE_HEXA;
  SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_TYPE_HEXA,BM_SETCHECK,1,0);
  SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_TYPE_DECIMAL,BM_SETCHECK,0,0);
  SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_TYPE_TEXT,BM_SETCHECK,0,0);
  SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_TYPE_BYTES,BM_SETCHECK,0,0);

  /* Search Size */
  window_replace->search_size = REPLACE_SIZE_8;
  SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_SIZE_8,BM_SETCHECK,1,0);
  SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_SIZE_16,BM_SETCHECK,0,0);
  SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_SIZE_24,BM_SETCHECK,0,0);
  SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_SIZE_32,BM_SETCHECK,0,0);
    
  /* Search Align */
  window_replace->search_align = REPLACE_ALIGN_INTEL;
  SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_ALIGN_INTEL,BM_SETCHECK,1,0);
  SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_ALIGN_MOTOROLA,BM_SETCHECK,0,0);
  EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_ALIGN_INTEL),FALSE);
  EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_ALIGN_MOTOROLA),FALSE);
  
  /* Search Area */
  if(window_replace->nb_rom_bank == 0)
    {
      /* Apple II */
      window_replace->search_area_fastram = 1;
      window_replace->search_area_slowram = 0;
      window_replace->search_area_rom = 0;
      window_replace->search_area_docram = 0;
      SendMessage(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_FASTRAM),BM_SETCHECK,1,0);
      
      /* Adresses */
      window_replace->fastram_from_bank = 0x00;
      window_replace->fastram_from_address = 0x0000;
      window_replace->fastram_to_bank = 0x01;
      window_replace->fastram_to_address = 0xFFFF;

      window_replace->slowram_from_bank = -1;
      window_replace->slowram_from_address = -1;
      window_replace->slowram_to_bank = -1;
      window_replace->slowram_to_address = -1;

      window_replace->rom_from_bank = -1;
      window_replace->rom_from_address = -1;
      window_replace->rom_to_bank = -1;
      window_replace->rom_to_address = -1;

      window_replace->docram_from_address = -1;
      window_replace->docram_to_address = -1;
      
      /* Control Edit */
      SetWindowText(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_FASTRAM_FROM_BANK),"00");
      SetWindowText(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_FASTRAM_FROM_ADDRESS),"0000");
      SetWindowText(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_FASTRAM_TO_BANK),"01");
      SetWindowText(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_FASTRAM_TO_ADDRESS),"FFFF");
      SetWindowText(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_SLOWRAM_FROM_BANK),"");
      SetWindowText(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_SLOWRAM_FROM_ADDRESS),"");
      SetWindowText(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_SLOWRAM_TO_BANK),"");
      SetWindowText(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_SLOWRAM_TO_ADDRESS),"");
      SetWindowText(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_ROM_FROM_BANK),"");
      SetWindowText(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_ROM_FROM_ADDRESS),"");
      SetWindowText(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_ROM_TO_BANK),"");
      SetWindowText(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_ROM_TO_ADDRESS),"");
      SetWindowText(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_DOCRAM_FROM_ADDRESS),"");
      SetWindowText(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_DOCRAM_TO_ADDRESS),"");

      /* Disable Control */
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_SLOWRAM_FROM_BANK),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_SLOWRAM_FROM_ADDRESS),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_SLOWRAM_TO_BANK),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_SLOWRAM_TO_ADDRESS),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_ROM_FROM_BANK),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_ROM_FROM_ADDRESS),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_ROM_TO_BANK),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_ROM_TO_ADDRESS),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_DOCRAM_FROM_ADDRESS),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_DOCRAM_TO_ADDRESS),FALSE);
    }
  else
    {
      /* Apple IIgs */
      window_replace->search_area_fastram = 1;
      window_replace->search_area_slowram = 1;
      window_replace->search_area_rom = 1;
      window_replace->search_area_docram = 1;
      SendMessage(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_FASTRAM),BM_SETCHECK,1,0);
      SendMessage(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_SLOWRAM),BM_SETCHECK,1,0);
      SendMessage(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_ROM),BM_SETCHECK,1,0);
      SendMessage(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_DOCRAM),BM_SETCHECK,1,0);
      
      /* Adresses */
      window_replace->fastram_from_bank = 0x00;
      window_replace->fastram_from_address = 0x0000;
      window_replace->fastram_to_bank = window_replace->nb_fastram_bank-1;
      window_replace->fastram_to_address = 0xFFFF;

      window_replace->slowram_from_bank = 0xE0;
      window_replace->slowram_from_address = 0x0000;
      window_replace->slowram_to_bank = 0xE1;
      window_replace->slowram_to_address = 0xFFFF;

      window_replace->rom_from_bank = (window_replace->nb_rom_bank == 2) ? 0xFE : 0xFC;
      window_replace->rom_from_address = 0x0000;
      window_replace->rom_to_bank = 0xFF;
      window_replace->rom_to_address = 0xFFFF;

      window_replace->docram_from_address = 0x0000;
      window_replace->docram_to_address = 0xFFFF;
      
      /* Control Edit */
      SetWindowText(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_FASTRAM_FROM_BANK),"00");
      SetWindowText(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_FASTRAM_FROM_ADDRESS),"0000");
      sprintf(buffer,"%02X",window_replace->nb_fastram_bank-1);
      SetWindowText(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_FASTRAM_TO_BANK),buffer);
      SetWindowText(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_FASTRAM_TO_ADDRESS),"FFFF");
      SetWindowText(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_SLOWRAM_FROM_BANK),"E0");
      SetWindowText(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_SLOWRAM_FROM_ADDRESS),"0000");
      SetWindowText(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_SLOWRAM_TO_BANK),"E1");
      SetWindowText(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_SLOWRAM_TO_ADDRESS),"FFFF");
      sprintf(buffer,"%02X",(window_replace->nb_rom_bank == 2) ? 0xFE : 0xFC);
      SetWindowText(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_ROM_FROM_BANK),buffer);
      SetWindowText(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_ROM_FROM_ADDRESS),"0000");
      SetWindowText(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_ROM_TO_BANK),"FF");
      SetWindowText(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_ROM_TO_ADDRESS),"FFFF");
      SetWindowText(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_DOCRAM_FROM_ADDRESS),"0000");
      SetWindowText(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_DOCRAM_TO_ADDRESS),"FFFF");
    }

  /* Charset : Disabled + Hide */
  window_replace->charset_ascii = 1;
  window_replace->charset_highascii = 0;
  window_replace->charset_inverse = 0;
  window_replace->charset_flash = 0;
  SendMessage(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_ASCII),BM_SETCHECK,1,0);
  EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_ASCII),FALSE);
  EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_HIGHASCII),FALSE);
  EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_INVERSE),FALSE);
  EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_FLASH),FALSE);
  ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_ASCII),SW_HIDE);
  ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_HIGHASCII),SW_HIDE);
  ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_INVERSE),SW_HIDE);
  ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_FLASH),SW_HIDE);   

  /* Options : Disable + Hide */
  window_replace->options_casesensitive = 0;
  window_replace->options_reverseorder = 0;
  window_replace->options_wildcard = 0;  
  EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_CASESENSITIVE),FALSE);
  EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_REVERSEORDER),FALSE);
  EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_WILDCARD),FALSE);
  ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_CASESENSITIVE),SW_HIDE);
  ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_REVERSEORDER),SW_HIDE);
  ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_WILDCARD),SW_HIDE);

  /* Replace Type */
  window_replace->replace_type = REPLACE_TYPE_HEXA;
  SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_TYPE_HEXA,BM_SETCHECK,1,0);
  SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_TYPE_DECIMAL,BM_SETCHECK,0,0);
  SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_TYPE_TEXT,BM_SETCHECK,0,0);
  SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_TYPE_BYTES,BM_SETCHECK,0,0);

  /* Replace Size */
  window_replace->replace_size = REPLACE_SIZE_8;
  SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_8,BM_SETCHECK,1,0);
  SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_16,BM_SETCHECK,0,0);
  SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_24,BM_SETCHECK,0,0);
  SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_32,BM_SETCHECK,0,0);
    
  /* Replace Align */
  window_replace->replace_align = REPLACE_ALIGN_INTEL;
  SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_ALIGN_INTEL,BM_SETCHECK,1,0);
  SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_ALIGN_MOTOROLA,BM_SETCHECK,0,0);
  EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_ALIGN_INTEL),FALSE);
  EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_ALIGN_MOTOROLA),FALSE);

  /* Replace Charset */
  window_replace->replace_charset = REPLACE_CHARSET_ASCII;
  SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_ASCII,BM_SETCHECK,1,0);
  SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_HIGHASCII,BM_SETCHECK,0,0);
  SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_INVERSE,BM_SETCHECK,0,0);
  SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_FLASH,BM_SETCHECK,0,0);
  EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_ASCII),FALSE);
  EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_HIGHASCII),FALSE);
  EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_INVERSE),FALSE);
  EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_FLASH),FALSE);
  ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_ASCII),SW_HIDE);
  ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_HIGHASCII),SW_HIDE);
  ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_INVERSE),SW_HIDE);
  ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_FLASH),SW_HIDE);

  /* Dsactive les 2 boutons Search Results + 2 Boutons Replace */
  EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_BUTTON_PREVIOUS),FALSE);
  EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_BUTTON_NEXT),FALSE);
  EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_BUTTON_REPLACE),FALSE);
  EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_BUTTON_REPLACEALL),FALSE);
    
  /* OK */
  return(0);
}


/****************************************************************************************/
/*  subEditReplaceProcControl() :  Rcupre le RETURN pour valider les donnes saisies. */
/****************************************************************************************/
LRESULT CALLBACK subEditReplaceProcControl(HWND hwnd_control, UINT msg, WPARAM wParam, LPARAM lParam)
{
  int index;
  struct window_replace *window_replace;
  my_Memory(MEMORY_GET_WINDOW_REPLACE,&window_replace,NULL);

  /* Protection */
  if(window_replace == NULL)
    return(0);

  /** Identifie le Control **/
  if(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_SEARCH) == hwnd_control)
    index = 1;
  else if(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_FASTRAM_FROM_BANK) == hwnd_control)
    index = 2;
  else if(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_FASTRAM_FROM_ADDRESS) == hwnd_control)
    index = 3;
  else if(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_FASTRAM_TO_BANK) == hwnd_control)
    index = 4;
  else if(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_FASTRAM_TO_ADDRESS) == hwnd_control)
    index = 5;
  else if(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_SLOWRAM_FROM_BANK) == hwnd_control)
    index = 6;
  else if(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_SLOWRAM_FROM_ADDRESS) == hwnd_control)
    index = 7;
  else if(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_SLOWRAM_TO_BANK) == hwnd_control)
    index = 8;
  else if(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_SLOWRAM_TO_ADDRESS) == hwnd_control)
    index = 9;
  else if(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_ROM_FROM_BANK) == hwnd_control)
    index = 10;
  else if(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_ROM_FROM_ADDRESS) == hwnd_control)
    index = 11;
  else if(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_ROM_TO_BANK) == hwnd_control)
    index = 12;
  else if(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_ROM_TO_ADDRESS) == hwnd_control)
    index = 13;
  else if(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_DOCRAM_FROM_ADDRESS) == hwnd_control)
    index = 14;
  else if(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_DOCRAM_TO_ADDRESS) == hwnd_control)
    index = 15;
  else if(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_REPLACE) == hwnd_control)
    index = 16;
  else
    return(0);      /* Pas trouv */

  /** Dtecte la touche RETURN pour valider la donne **/
  if(msg == WM_CHAR && wParam == VK_RETURN)
    {
      /* Enlve le focus de l'Edit */
      SetFocus(window_replace->hwnd_replace);
    }
  else if(index == 1)
    return(CallWindowProc(window_replace->oldEditProcSearch,hwnd_control,msg,wParam,lParam));
  else if(index == 2)
    return(CallWindowProc(window_replace->oldEditProcFastramFromBank,hwnd_control,msg,wParam,lParam));
  else if(index == 3)
    return(CallWindowProc(window_replace->oldEditProcFastramFromAddress,hwnd_control,msg,wParam,lParam));
  else if(index == 4)
    return(CallWindowProc(window_replace->oldEditProcFastramToBank,hwnd_control,msg,wParam,lParam));
  else if(index == 5)
    return(CallWindowProc(window_replace->oldEditProcFastramToAddress,hwnd_control,msg,wParam,lParam));
  else if(index == 6)
    return(CallWindowProc(window_replace->oldEditProcSlowramFromBank,hwnd_control,msg,wParam,lParam));
  else if(index == 7)
    return(CallWindowProc(window_replace->oldEditProcSlowramFromAddress,hwnd_control,msg,wParam,lParam));  
  else if(index == 8)
    return(CallWindowProc(window_replace->oldEditProcSlowramToBank,hwnd_control,msg,wParam,lParam));
  else if(index == 9)
    return(CallWindowProc(window_replace->oldEditProcSlowramToAddress,hwnd_control,msg,wParam,lParam));
  else if(index == 10)
    return(CallWindowProc(window_replace->oldEditProcRomFromBank,hwnd_control,msg,wParam,lParam));
  else if(index == 11)
    return(CallWindowProc(window_replace->oldEditProcRomFromAddress,hwnd_control,msg,wParam,lParam));
  else if(index == 12)
    return(CallWindowProc(window_replace->oldEditProcRomToBank,hwnd_control,msg,wParam,lParam));
  else if(index == 13)
    return(CallWindowProc(window_replace->oldEditProcRomToAddress,hwnd_control,msg,wParam,lParam));
  else if(index == 14)
    return(CallWindowProc(window_replace->oldEditProcDocramFromAddress,hwnd_control,msg,wParam,lParam));
  else if(index == 15)
    return(CallWindowProc(window_replace->oldEditProcDocramToAddress,hwnd_control,msg,wParam,lParam));
  else if(index == 16)
    return(CallWindowProc(window_replace->oldEditProcReplace,hwnd_control,msg,wParam,lParam));

  /* OK */
  return(0);
}


/*****************************************************************/
/*  UpdateSearchType() :  Change le type des donnes  chercher. */
/*****************************************************************/
static void UpdateSearchType(struct window_replace *window_replace, int search_type)
{
  if(window_replace->search_type == search_type)
    return;
    
  /* Modification */
  if(search_type == REPLACE_TYPE_HEXA)
    {
      /* Type */
      window_replace->search_type = REPLACE_TYPE_HEXA;
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_TYPE_HEXA,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_TYPE_DECIMAL,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_TYPE_TEXT,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_TYPE_BYTES,BM_SETCHECK,0,0);

      /* Size : Enabled */
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_SIZE_8),TRUE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_SIZE_16),TRUE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_SIZE_24),TRUE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_SIZE_32),TRUE); 
      /* Size : Show */
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_SIZE_8),SW_SHOW);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_SIZE_16),SW_SHOW);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_SIZE_24),SW_SHOW);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_SIZE_32),SW_SHOW);
                  
      /* Align : Enabled / Disabled */
      if(window_replace->search_size == REPLACE_SIZE_8)
        {
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_ALIGN_INTEL),FALSE);
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_ALIGN_MOTOROLA),FALSE);
        }
      else
        {
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_ALIGN_INTEL),TRUE);
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_ALIGN_MOTOROLA),TRUE);
        }
      /* Align : Show */
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_ALIGN_INTEL),SW_SHOW);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_ALIGN_MOTOROLA),SW_SHOW);
      
      /* Charset : Disabled + Hide */
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_ASCII),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_HIGHASCII),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_INVERSE),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_FLASH),FALSE);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_ASCII),SW_HIDE);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_HIGHASCII),SW_HIDE);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_INVERSE),SW_HIDE);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_FLASH),SW_HIDE);      

      /* Options : Disable + Hide */
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_CASESENSITIVE),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_REVERSEORDER),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_WILDCARD),FALSE);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_CASESENSITIVE),SW_HIDE);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_REVERSEORDER),SW_HIDE);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_WILDCARD),SW_HIDE);

      /* Label Data Size + Align */
      SetWindowText(GetDlgItem(window_replace->hwnd_replace,IDC_STATIC_DATA_SIZE),"Data Size :");
      SetWindowText(GetDlgItem(window_replace->hwnd_replace,IDC_STATIC_ALIGN),"Alignment :");
    }
  else if(search_type == REPLACE_TYPE_DECIMAL)
    {
      /* Type */
      window_replace->search_type = REPLACE_TYPE_DECIMAL;
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_TYPE_HEXA,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_TYPE_DECIMAL,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_TYPE_TEXT,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_TYPE_BYTES,BM_SETCHECK,0,0);
      
      /* Size : Enabled */
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_SIZE_8),TRUE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_SIZE_16),TRUE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_SIZE_24),TRUE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_SIZE_32),TRUE); 
      /* Size : Show */
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_SIZE_8),SW_SHOW);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_SIZE_16),SW_SHOW);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_SIZE_24),SW_SHOW);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_SIZE_32),SW_SHOW);
                  
      /* Align : Enabled / Disabled */
      if(window_replace->search_size == REPLACE_SIZE_8)
        {
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_ALIGN_INTEL),FALSE);
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_ALIGN_MOTOROLA),FALSE);
        }
      else
        {
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_ALIGN_INTEL),TRUE);
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_ALIGN_MOTOROLA),TRUE);
        }
      /* Align : Show */
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_ALIGN_INTEL),SW_SHOW);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_ALIGN_MOTOROLA),SW_SHOW);        

      /* Charset : Disabled + Hide */
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_ASCII),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_HIGHASCII),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_INVERSE),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_FLASH),FALSE);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_ASCII),SW_HIDE);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_HIGHASCII),SW_HIDE);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_INVERSE),SW_HIDE);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_FLASH),SW_HIDE);

      /* Options : Disable + Hide */
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_CASESENSITIVE),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_REVERSEORDER),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_WILDCARD),FALSE);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_CASESENSITIVE),SW_HIDE);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_REVERSEORDER),SW_HIDE);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_WILDCARD),SW_HIDE);

      /* Label Data Size + Align */
      SetWindowText(GetDlgItem(window_replace->hwnd_replace,IDC_STATIC_DATA_SIZE),"Data Size :");
      SetWindowText(GetDlgItem(window_replace->hwnd_replace,IDC_STATIC_ALIGN),"Alignment :");
    }
  else if(search_type == REPLACE_TYPE_TEXT)
    {
      /* Type */
      window_replace->search_type = REPLACE_TYPE_TEXT;
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_TYPE_HEXA,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_TYPE_DECIMAL,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_TYPE_TEXT,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_TYPE_BYTES,BM_SETCHECK,0,0);
      
      /* Size : Disabled */
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_SIZE_8),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_SIZE_16),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_SIZE_24),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_SIZE_32),FALSE); 
      /* Size : Hide */
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_SIZE_8),SW_HIDE);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_SIZE_16),SW_HIDE);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_SIZE_24),SW_HIDE);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_SIZE_32),SW_HIDE);
        
      /* Align : Disabled */
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_ALIGN_INTEL),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_ALIGN_MOTOROLA),FALSE);
      /* Align : Hide */
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_ALIGN_INTEL),SW_HIDE);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_ALIGN_MOTOROLA),SW_HIDE);      

      /* Charset : Enabled + Show */
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_ASCII),TRUE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_HIGHASCII),TRUE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_INVERSE),TRUE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_FLASH),TRUE);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_ASCII),SW_SHOW);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_HIGHASCII),SW_SHOW);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_INVERSE),SW_SHOW);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_FLASH),SW_SHOW);

      /* Options : Enabled + Show */
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_CASESENSITIVE),TRUE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_REVERSEORDER),TRUE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_WILDCARD),TRUE);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_CASESENSITIVE),SW_SHOW);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_REVERSEORDER),SW_SHOW);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_WILDCARD),SW_SHOW);

      /* Label Charset + Options */
      SetWindowText(GetDlgItem(window_replace->hwnd_replace,IDC_STATIC_DATA_SIZE),"Charset :");
      SetWindowText(GetDlgItem(window_replace->hwnd_replace,IDC_STATIC_ALIGN),"Options :");
    }
  else if(search_type == REPLACE_TYPE_BYTES)
    {
      /* Type */
      window_replace->search_type = REPLACE_TYPE_BYTES;
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_TYPE_HEXA,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_TYPE_DECIMAL,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_TYPE_TEXT,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_TYPE_BYTES,BM_SETCHECK,1,0);
      
      /* Size : Disabled */
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_SIZE_8),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_SIZE_16),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_SIZE_24),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_SIZE_32),FALSE); 
      /* Size : Show */
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_SIZE_8),SW_SHOW);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_SIZE_16),SW_SHOW);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_SIZE_24),SW_SHOW);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_SIZE_32),SW_SHOW);
              
      /* Align : Disabled */
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_ALIGN_INTEL),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_ALIGN_MOTOROLA),FALSE);
      /* Align : Show */
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_ALIGN_INTEL),SW_SHOW);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_ALIGN_MOTOROLA),SW_SHOW);
      
      /* Charset : Disabled + Hide */
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_ASCII),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_HIGHASCII),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_INVERSE),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_FLASH),FALSE);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_ASCII),SW_HIDE);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_HIGHASCII),SW_HIDE);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_INVERSE),SW_HIDE);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_FLASH),SW_HIDE);

      /* Options : Disable + Hide */
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_CASESENSITIVE),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_REVERSEORDER),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_WILDCARD),FALSE);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_CASESENSITIVE),SW_HIDE);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_REVERSEORDER),SW_HIDE);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_CHECK_WILDCARD),SW_HIDE);

      /* Label Data Size + Align */
      SetWindowText(GetDlgItem(window_replace->hwnd_replace,IDC_STATIC_DATA_SIZE),"Data Size :");
      SetWindowText(GetDlgItem(window_replace->hwnd_replace,IDC_STATIC_ALIGN),"Alignment :");
    }
    
  /* Active / Dsactive le bouton Search */
  if(window_replace->search_type == REPLACE_TYPE_TEXT && 
     window_replace->charset_ascii == 0 && window_replace->charset_highascii == 0 && 
     window_replace->charset_inverse == 0 && window_replace->charset_flash == 0)
    EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_BUTTON_SEARCH),FALSE);
  else if(window_replace->search_area_fastram == 0 && window_replace->search_area_slowram == 0 &&
          window_replace->search_area_rom == 0 && window_replace->search_area_docram == 0)
    EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_BUTTON_SEARCH),FALSE);
  else
    EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_BUTTON_SEARCH),TRUE);    
}


/*******************************************************************/
/*  UpdateSearchSize() :  Change la taille des donnes  chercher. */
/*******************************************************************/
static void UpdateSearchSize(struct window_replace *window_replace, int search_size)
{
  if(window_replace->search_size == search_size)
    return;

  if(window_replace->search_type == REPLACE_TYPE_TEXT || window_replace->search_type == REPLACE_TYPE_BYTES)
    return;

  /* Modification */
  if(search_size == REPLACE_SIZE_8)
    {
      window_replace->search_size = REPLACE_SIZE_8;
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_SIZE_8,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_SIZE_16,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_SIZE_24,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_SIZE_32,BM_SETCHECK,0,0);
      
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_ALIGN_INTEL),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_ALIGN_MOTOROLA),FALSE);      
    }
  else if(search_size == REPLACE_SIZE_16)
    {
      window_replace->search_size = REPLACE_SIZE_16;
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_SIZE_8,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_SIZE_16,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_SIZE_24,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_SIZE_32,BM_SETCHECK,0,0);
      
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_ALIGN_INTEL),TRUE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_ALIGN_MOTOROLA),TRUE);      
    }
  else if(search_size == REPLACE_SIZE_24)
    {
      window_replace->search_size = REPLACE_SIZE_24;
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_SIZE_8,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_SIZE_16,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_SIZE_24,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_SIZE_32,BM_SETCHECK,0,0);
      
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_ALIGN_INTEL),TRUE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_ALIGN_MOTOROLA),TRUE);        
    }
  else if(search_size == REPLACE_SIZE_32)
    {
      window_replace->search_size = REPLACE_SIZE_32;
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_SIZE_8,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_SIZE_16,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_SIZE_24,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_SIZE_32,BM_SETCHECK,1,0);

      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_ALIGN_INTEL),TRUE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_ALIGN_MOTOROLA),TRUE);
    }
}


/***********************************************************************/
/*  UpdateSearchAlign() :  Change l'alignement des donnes  chercher. */
/***********************************************************************/
static void UpdateSearchAlign(struct window_replace *window_replace, int search_align)
{
  if(window_replace->search_align == search_align)
    return;
    
  /* Modification */
  if(search_align == REPLACE_ALIGN_INTEL)
    {
      window_replace->search_align = REPLACE_ALIGN_INTEL;
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_ALIGN_INTEL,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_ALIGN_MOTOROLA,BM_SETCHECK,0,0);
    }
  else if(search_align == REPLACE_ALIGN_MOTOROLA)
    {
      window_replace->search_align = REPLACE_ALIGN_MOTOROLA;
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_ALIGN_INTEL,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_ALIGN_MOTOROLA,BM_SETCHECK,1,0);
    }
}


/********************************************************************************/
/*  UpdateSearchCharset() :  Slectionne les Charset valides pour la recherche. */
/********************************************************************************/
static void UpdateSearchCharset(struct window_replace *window_replace, int search_charset)
{
   if(search_charset == REPLACE_CHARSET_ASCII)
     {
       if(window_replace->charset_ascii == 0)
         window_replace->charset_ascii = 1;
       else
         window_replace->charset_ascii = 0;
     }
   else if(search_charset == REPLACE_CHARSET_HIGHASCII)
     {
       if(window_replace->charset_highascii == 0)
         window_replace->charset_highascii = 1;
       else
         window_replace->charset_highascii = 0;     
     }     
   else if(search_charset == REPLACE_CHARSET_INVERSE)
     {
       if(window_replace->charset_inverse == 0)
         window_replace->charset_inverse = 1;
       else
         window_replace->charset_inverse = 0;         
     }     
   else if(search_charset == REPLACE_CHARSET_FLASH)
     {
       if(window_replace->charset_flash == 0)
         window_replace->charset_flash = 1;
       else
         window_replace->charset_flash = 0;      
     }
     
  /* Active / Dsactive le bouton Search */
  if(window_replace->search_area_fastram == 0 && window_replace->search_area_slowram == 0 &&
     window_replace->search_area_rom == 0 && window_replace->search_area_docram == 0)
    EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_BUTTON_SEARCH),FALSE);  
  else if(window_replace->charset_ascii == 0 && window_replace->charset_highascii == 0 && 
     window_replace->charset_inverse == 0 && window_replace->charset_flash == 0)
    EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_BUTTON_SEARCH),FALSE);
  else
    EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_BUTTON_SEARCH),TRUE);
}


/************************************************************************/
/*  UpdateSearchOptions() :  Slectionne les Options pour la recherche. */
/************************************************************************/
static void UpdateSearchOptions(struct window_replace *window_replace, int search_options)
{
   if(search_options == REPLACE_OPTIONS_CASESENSITIVE)
     {
       if(window_replace->options_casesensitive == 0)
         window_replace->options_casesensitive = 1;
       else
         window_replace->options_casesensitive = 0;
     }
   else if(search_options == REPLACE_OPTIONS_REVERSEORDER)
     {
       if(window_replace->options_reverseorder == 0)
         window_replace->options_reverseorder = 1;
       else
         window_replace->options_reverseorder = 0;
     }     
   else if(search_options == REPLACE_OPTIONS_WILDCARD)
     {
       if(window_replace->options_wildcard == 0)
         window_replace->options_wildcard = 1;
       else
         window_replace->options_wildcard = 0;
     }
}


/*********************************************************************/
/*  UpdateSearchArea() :  Change les zones de recherche des donnes. */
/*********************************************************************/
static void UpdateSearchArea(struct window_replace *window_replace, int search_area)
{
  if(search_area == REPLACE_AREA_FASTRAM)
    {
      if(window_replace->search_area_fastram == 0)
        {
          window_replace->search_area_fastram = 1;
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_FASTRAM_FROM_BANK),TRUE);
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_FASTRAM_FROM_ADDRESS),TRUE);
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_FASTRAM_TO_BANK),TRUE);
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_FASTRAM_TO_ADDRESS),TRUE);
        }
      else
        {
          window_replace->search_area_fastram = 0;
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_FASTRAM_FROM_BANK),FALSE);
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_FASTRAM_FROM_ADDRESS),FALSE);
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_FASTRAM_TO_BANK),FALSE);
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_FASTRAM_TO_ADDRESS),FALSE);
        }
    }
  else if(search_area == REPLACE_AREA_SLOWRAM)
    {
      if(window_replace->search_area_slowram == 0)
        {
          window_replace->search_area_slowram = 1;
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_SLOWRAM_FROM_BANK),TRUE);
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_SLOWRAM_FROM_ADDRESS),TRUE);
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_SLOWRAM_TO_BANK),TRUE);
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_SLOWRAM_TO_ADDRESS),TRUE);
        }
      else
        {
          window_replace->search_area_slowram = 0;
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_SLOWRAM_FROM_BANK),FALSE);
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_SLOWRAM_FROM_ADDRESS),FALSE);
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_SLOWRAM_TO_BANK),FALSE);
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_SLOWRAM_TO_ADDRESS),FALSE);
        }    
    }
  else if(search_area == REPLACE_AREA_ROM)
    {
      if(window_replace->search_area_rom == 0)
        {
          window_replace->search_area_rom = 1;
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_ROM_FROM_BANK),TRUE);
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_ROM_FROM_ADDRESS),TRUE);
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_ROM_TO_BANK),TRUE);
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_ROM_TO_ADDRESS),TRUE);
        }
      else
        {
          window_replace->search_area_rom = 0;
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_ROM_FROM_BANK),FALSE);
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_ROM_FROM_ADDRESS),FALSE);
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_ROM_TO_BANK),FALSE);
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_ROM_TO_ADDRESS),FALSE);
        }    
    }
  else if(search_area == REPLACE_AREA_DOCRAM)
    {
      if(window_replace->search_area_docram == 0)
        {
          window_replace->search_area_docram = 1;
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_DOCRAM_FROM_ADDRESS),TRUE);
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_DOCRAM_TO_ADDRESS),TRUE);
        }
      else
        {
          window_replace->search_area_docram = 0;
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_DOCRAM_FROM_ADDRESS),FALSE);
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_DOCRAM_TO_ADDRESS),FALSE);
        }    
    }
    
  /* Active / Dsactive le bouton Replace */
  if(window_replace->search_type == REPLACE_TYPE_TEXT && 
     window_replace->charset_ascii == 0 && window_replace->charset_highascii == 0 && 
     window_replace->charset_inverse == 0 && window_replace->charset_flash == 0)
    EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_BUTTON_SEARCH),FALSE);
  else if(window_replace->search_area_fastram == 0 && window_replace->search_area_slowram == 0 &&
          window_replace->search_area_rom == 0 && window_replace->search_area_docram == 0)
    EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_BUTTON_SEARCH),FALSE);
  else
    EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_BUTTON_SEARCH),TRUE);  
}


/*******************************************************************/
/*  UpdateReplaceType() :  Change le type des donnes  remplacer. */
/*******************************************************************/
static void UpdateReplaceType(struct window_replace *window_replace, int replace_type)
{
  if(window_replace->replace_type == replace_type)
    return;
    
  /* Modification */
  if(replace_type == REPLACE_TYPE_HEXA)
    {
      /* Type */
      window_replace->replace_type = REPLACE_TYPE_HEXA;
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_TYPE_HEXA,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_TYPE_DECIMAL,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_TYPE_TEXT,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_TYPE_BYTES,BM_SETCHECK,0,0);

      /* Size : Enabled */
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_8),TRUE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_16),TRUE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_24),TRUE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_32),TRUE); 
      /* Size : Show */
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_8),SW_SHOW);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_16),SW_SHOW);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_24),SW_SHOW);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_32),SW_SHOW);
                  
      /* Align : Enabled / Disabled */
      if(window_replace->replace_size == REPLACE_SIZE_8)
        {
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_ALIGN_INTEL),FALSE);
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_ALIGN_MOTOROLA),FALSE);
        }
      else
        {
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_ALIGN_INTEL),TRUE);
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_ALIGN_MOTOROLA),TRUE);
        }
      
      /* Charset : Disabled + Hide */
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_ASCII),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_HIGHASCII),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_INVERSE),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_FLASH),FALSE);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_ASCII),SW_HIDE);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_HIGHASCII),SW_HIDE);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_INVERSE),SW_HIDE);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_FLASH),SW_HIDE);      

      /* Label Data Size + Align */
      SetWindowText(GetDlgItem(window_replace->hwnd_replace,IDC_STATIC_REPLACE_DATA_SIZE),"Data Size :");
    }
  else if(replace_type == REPLACE_TYPE_DECIMAL)
    {
      /* Type */
      window_replace->replace_type = REPLACE_TYPE_DECIMAL;
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_TYPE_HEXA,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_TYPE_DECIMAL,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_TYPE_TEXT,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_TYPE_BYTES,BM_SETCHECK,0,0);
      
      /* Size : Enabled */
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_8),TRUE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_16),TRUE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_24),TRUE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_32),TRUE); 
      /* Size : Show */
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_8),SW_SHOW);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_16),SW_SHOW);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_24),SW_SHOW);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_32),SW_SHOW);
                  
      /* Align : Enabled / Disabled */
      if(window_replace->replace_size == REPLACE_SIZE_8)
        {
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_ALIGN_INTEL),FALSE);
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_ALIGN_MOTOROLA),FALSE);
        }
      else
        {
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_ALIGN_INTEL),TRUE);
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_ALIGN_MOTOROLA),TRUE);
        }

      /* Charset : Disabled + Hide */
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_ASCII),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_HIGHASCII),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_INVERSE),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_FLASH),FALSE);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_ASCII),SW_HIDE);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_HIGHASCII),SW_HIDE);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_INVERSE),SW_HIDE);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_FLASH),SW_HIDE);

      /* Label Data Size + Align */
      SetWindowText(GetDlgItem(window_replace->hwnd_replace,IDC_STATIC_REPLACE_DATA_SIZE),"Data Size :");
    }
  else if(replace_type == REPLACE_TYPE_TEXT)
    {
      /* Type */
      window_replace->replace_type = REPLACE_TYPE_TEXT;
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_TYPE_HEXA,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_TYPE_DECIMAL,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_TYPE_TEXT,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_TYPE_BYTES,BM_SETCHECK,0,0);
      
      /* Size : Disabled */
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_8),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_16),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_24),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_32),FALSE); 
      /* Size : Hide */
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_8),SW_HIDE);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_16),SW_HIDE);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_24),SW_HIDE);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_32),SW_HIDE);
        
      /* Align : Disabled */
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_ALIGN_INTEL),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_ALIGN_MOTOROLA),FALSE);

      /* Charset : Enabled + Show */
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_ASCII),TRUE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_HIGHASCII),TRUE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_INVERSE),TRUE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_FLASH),TRUE);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_ASCII),SW_SHOW);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_HIGHASCII),SW_SHOW);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_INVERSE),SW_SHOW);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_FLASH),SW_SHOW);

      /* Label Charset */
      SetWindowText(GetDlgItem(window_replace->hwnd_replace,IDC_STATIC_REPLACE_DATA_SIZE),"Charset :");
    }
  else if(replace_type == REPLACE_TYPE_BYTES)
    {
      /* Type */
      window_replace->replace_type = REPLACE_TYPE_BYTES;
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_TYPE_HEXA,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_TYPE_DECIMAL,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_TYPE_TEXT,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_TYPE_BYTES,BM_SETCHECK,1,0);
      
      /* Size : Disabled */
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_8),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_16),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_24),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_32),FALSE); 
      /* Size : Show */
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_8),SW_SHOW);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_16),SW_SHOW);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_24),SW_SHOW);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_32),SW_SHOW);
              
      /* Align : Disabled */
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_ALIGN_INTEL),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_ALIGN_MOTOROLA),FALSE);
      
      /* Charset : Disabled + Hide */
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_ASCII),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_HIGHASCII),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_INVERSE),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_FLASH),FALSE);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_ASCII),SW_HIDE);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_HIGHASCII),SW_HIDE);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_INVERSE),SW_HIDE);
      ShowWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_FLASH),SW_HIDE);

      /* Label Data Size */
      SetWindowText(GetDlgItem(window_replace->hwnd_replace,IDC_STATIC_REPLACE_DATA_SIZE),"Data Size :");
    }    
}


/*********************************************************************/
/*  UpdateReplaceSize() :  Change la taille des donnes  remplacer. */
/*********************************************************************/
static void UpdateReplaceSize(struct window_replace *window_replace, int replace_size)
{
  if(window_replace->replace_size == replace_size)
    return;

  if(window_replace->replace_type == REPLACE_TYPE_TEXT || window_replace->replace_type == REPLACE_TYPE_BYTES)
    return;

  /* Modification */
  if(replace_size == REPLACE_SIZE_8)
    {
      window_replace->replace_size = REPLACE_SIZE_8;
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_8,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_16,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_24,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_32,BM_SETCHECK,0,0);
      
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_ALIGN_INTEL),FALSE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_ALIGN_MOTOROLA),FALSE);      
    }
  else if(replace_size == REPLACE_SIZE_16)
    {
      window_replace->replace_size = REPLACE_SIZE_16;
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_8,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_16,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_24,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_32,BM_SETCHECK,0,0);
      
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_ALIGN_INTEL),TRUE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_ALIGN_MOTOROLA),TRUE);      
    }
  else if(replace_size == REPLACE_SIZE_24)
    {
      window_replace->replace_size = REPLACE_SIZE_24;
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_8,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_16,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_24,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_32,BM_SETCHECK,0,0);
      
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_ALIGN_INTEL),TRUE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_ALIGN_MOTOROLA),TRUE);        
    }
  else if(replace_size == REPLACE_SIZE_32)
    {
      window_replace->replace_size = REPLACE_SIZE_32;
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_8,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_16,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_24,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_SIZE_32,BM_SETCHECK,1,0);

      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_ALIGN_INTEL),TRUE);
      EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_RADIO_REPLACE_ALIGN_MOTOROLA),TRUE);
    }
}


/*************************************************************************/
/*  UpdateReplaceAlign() :  Change l'alignement des donnes  remplacer. */
/*************************************************************************/
static void UpdateReplaceAlign(struct window_replace *window_replace, int replace_align)
{
  if(window_replace->replace_align == replace_align)
    return;
    
  /* Modification */
  if(replace_align == REPLACE_ALIGN_INTEL)
    {
      window_replace->replace_align = REPLACE_ALIGN_INTEL;
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_ALIGN_INTEL,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_ALIGN_MOTOROLA,BM_SETCHECK,0,0);
    }
  else if(replace_align == REPLACE_ALIGN_MOTOROLA)
    {
      window_replace->replace_align = REPLACE_ALIGN_MOTOROLA;
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_ALIGN_INTEL,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_ALIGN_MOTOROLA,BM_SETCHECK,1,0);
    }
}


/************************************************************************************/
/*  UpdateReplaceCharset() :  Slectionne les Charset valides pour le remplacement. */
/************************************************************************************/
static void UpdateReplaceCharset(struct window_replace *window_replace, int replace_charset)
{
   if(replace_charset == REPLACE_CHARSET_ASCII)
     {
      window_replace->replace_charset = REPLACE_CHARSET_ASCII;
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_ASCII,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_HIGHASCII,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_INVERSE,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_FLASH,BM_SETCHECK,0,0);
     }
   else if(replace_charset == REPLACE_CHARSET_HIGHASCII)
     {
      window_replace->replace_charset = REPLACE_CHARSET_HIGHASCII;
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_ASCII,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_HIGHASCII,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_INVERSE,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_FLASH,BM_SETCHECK,0,0);
     }     
   else if(replace_charset == REPLACE_CHARSET_INVERSE)
     {
      window_replace->replace_charset = REPLACE_CHARSET_INVERSE;
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_ASCII,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_HIGHASCII,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_INVERSE,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_FLASH,BM_SETCHECK,0,0);
     }     
   else if(replace_charset == REPLACE_CHARSET_FLASH)
     {
      window_replace->replace_charset = REPLACE_CHARSET_FLASH;
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_ASCII,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_HIGHASCII,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_INVERSE,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_replace->hwnd_replace,IDC_RADIO_REPLACE_FLASH,BM_SETCHECK,1,0);
     }     
}


/**********************************************************************/
/*  SearchForData() :  Recherche les donnes dans les zones mmoires. */
/**********************************************************************/
static void SearchForData(struct window_replace *window_replace)
{
  BYTE one_byte;
  WORD one_word;
  DWORD one_dword;
  unsigned int value;
  int data_length;
  unsigned char data[1024];
  unsigned char one_data_char;
  unsigned char one_memory_char;
  unsigned char offset;
  int i, j, area, is_hexadecimal, is_decimal, is_odd, match;
  char search_data[1024];
  char buffer[1024];
  char *next_char;
  int memory_address_begin, memory_address_end, memory_length;
  char *ascii_charset = " !\"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklmnopqrstuvwxyz{|}~";
  char special_charset[64+1] = "@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_ !\"#$%&'()*+,-./0123456789:;<=>?";   /* INV et FLASH */
  struct replace_result *current_replaceresult;
  unsigned char *memory_data;
  struct mmu *current_mmu;
    
  /* Rcupre le MMU */
  my_Memory(MEMORY_GET_S65816_MMU,&current_mmu,NULL);
    
  /* Libration mmoire */
  mem_free_all_replaceresult(window_replace->first_replaceresult);
  window_replace->nb_found = 0;
  window_replace->nb_replaced = 0;
  window_replace->current_result = 0;
  window_replace->first_replaceresult = NULL;
  window_replace->last_replaceresult = NULL;
  
  /* RAZ de l'affichage */
  DisplaySearchResult(window_replace,0);
    
  /** Rcupre le critre de recherche **/
  GetWindowText(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_SEARCH),search_data,1024-1);
  if(strlen(search_data) == 0)
    {
      MessageBox(window_replace->hwnd_replace,"Empty Search criteria !","Search Criteria Error",MB_OK|MB_ICONWARNING);
      return;
    }
  
  /** Conversion **/
  if(window_replace->search_type == REPLACE_TYPE_HEXA)
    {
      /* Vrifie le format */
      is_hexadecimal = IsHexaDecimal(search_data,&is_odd);
      if(is_hexadecimal == 0)
        {
          sprintf(window_replace->buffer_log,"Error : The value '%s' is not in hexadecimal format",search_data);
          MessageBox(window_replace->hwnd_replace,window_replace->buffer_log,"Search Criteria Error",MB_OK|MB_ICONWARNING);
          return;
        }
        
      /** Conversion en Hexadcimal **/
      sscanf(search_data,"%X",&value);
      if(window_replace->search_size == REPLACE_SIZE_8)
        {
          one_byte = (BYTE) value;
          data_length = 1;
          memcpy(&data[0],&one_byte,data_length);
        }
      else if(window_replace->search_size == REPLACE_SIZE_16)
        {
          one_word = (WORD) value;
          data_length = 2;
          memcpy(&data[0],&one_word,data_length);
        }
      else if(window_replace->search_size == REPLACE_SIZE_24)
        {
          one_dword = (DWORD) value;
          data_length = 3;
          memcpy(&data[0],&one_dword,4);
        }        
      else if(window_replace->search_size == REPLACE_SIZE_32)
        {
          one_dword = (DWORD) value;
          data_length = 4;
          memcpy(&data[0],&one_dword,data_length);
        }
        
      /** Inverse les octets **/
      if(window_replace->search_align == REPLACE_ALIGN_MOTOROLA && data_length > 1)
        ExchangeByte(&data[0],data_length);
    }
  else if(window_replace->search_type == REPLACE_TYPE_DECIMAL)
    {
      /* Vrifie le format */
      is_decimal = IsDecimal(search_data,&is_odd);
      if(is_decimal == 0)
        {
          sprintf(window_replace->buffer_log,"Error : The value '%s' is not in decimal format",search_data);
          MessageBox(window_replace->hwnd_replace,window_replace->buffer_log,"Search Criteria Error",MB_OK|MB_ICONWARNING);
          return;
        }
        
      /** Conversion en Dcimal **/
      value = atoi(search_data);
      if(window_replace->search_size == REPLACE_SIZE_8)
        {
          one_byte = (BYTE) value;
          data_length = 1;
          memcpy(&data[0],&one_byte,data_length);          
        }
      else if(window_replace->search_size == REPLACE_SIZE_16)
        {
          one_word = (WORD) value;
          data_length = 2;
          memcpy(&data[0],&one_word,data_length);          
        }
      else if(window_replace->search_size == REPLACE_SIZE_24)
        {
          one_dword = (DWORD) value;
          data_length = 3;
          memcpy(&data[0],&one_dword,4);
        }
      else if(window_replace->search_size == REPLACE_SIZE_32)
        {
          one_dword = (DWORD) value;
          data_length = 4;
          memcpy(&data[0],&one_dword,data_length);          
        }
        
      /** Inverse les octets **/
      if(window_replace->search_align == REPLACE_ALIGN_MOTOROLA && data_length > 1)
        ExchangeByte(&data[0],data_length);     
    }    
  else if(window_replace->search_type == REPLACE_TYPE_TEXT)
    {
      /** Rcupre la chaine de caractre **/
      strcpy((char *)data,search_data);
      data_length = (int) strlen(search_data);
      
      /* On va valider les caractres saisis */
      for(i=0; i<(int)strlen((char *)data); i++)
        if(strchr(ascii_charset,data[i]) == NULL)
          {
            sprintf(window_replace->buffer_log,"Error : The character '%c' is not valid.",data[i]);
            MessageBox(window_replace->hwnd_replace,window_replace->buffer_log,"Search Text Error",MB_OK|MB_ICONWARNING);
            return;
          }
    } 
  else if(window_replace->search_type == REPLACE_TYPE_BYTES)
    {
      /* Supprime les espaces */
      RemoveSpace(search_data);
      if(strlen(search_data) == 0)
        {
          MessageBox(window_replace->hwnd_replace,"Empty Search criteria !","Search Criteria Error",MB_OK|MB_ICONWARNING);
          return;
        }
    
      /* Vrifie le format */
      is_hexadecimal = IsHexaDecimal(search_data,&is_odd);
      if(is_hexadecimal == 0)
        {
          sprintf(window_replace->buffer_log,"Error : The replaceed value is not in hexadecimal format",search_data);
          MessageBox(window_replace->hwnd_replace,window_replace->buffer_log,"Search Criteria Error",MB_OK|MB_ICONWARNING);
          return;
        }
      if(is_odd == 1)
        {
          sprintf(window_replace->buffer_log,"Error : The length of the value should be even",search_data);
          MessageBox(window_replace->hwnd_replace,window_replace->buffer_log,"Search Criteria Error",MB_OK|MB_ICONWARNING);
          return;
        }
        
      /** Conversion en Hexa **/
      for(i=0; i<(int)strlen(search_data); i+=2)
        {
          memcpy(buffer,&search_data[i],2);
          buffer[2] = '\0';
          sscanf(buffer,"%02X",&value);
          data[i/2] = (unsigned char) value;
        }
      data_length = (int) strlen(search_data)/2;
    }

  /* Conserve le critre de recherche */
  window_replace->search_query_size = data_length;
  memcpy(window_replace->search_query_data,data,window_replace->search_query_size);
    
  /*** Recherche dans les Zones mmoires ***/
  for(area=1; area<=4; area++)
    {
      /** Fast Ram | Slow Ram | Rom | DOC Ram **/
      if(area == 1 && window_replace->search_area_fastram == 1)
        {
          /* Zone mmoire */
          memory_address_begin = (window_replace->fastram_from_bank << 16) | window_replace->fastram_from_address;
          memory_address_end = (window_replace->fastram_to_bank << 16) | window_replace->fastram_to_address;
          memory_length = (memory_address_end - memory_address_begin) + 1;
          
          /* Allocation mmoire */
          memory_data = (unsigned char *) calloc(1,memory_length+16);
          if(memory_data == NULL)
            return;
          
          /* Rcupre les donnes brutes */
          ReadMemoryBlock(memory_address_begin,memory_length,memory_data,current_mmu);
        }
      else if(area == 2 && window_replace->search_area_slowram == 1)
        {
          /* Zone mmoire */
          memory_address_begin = (window_replace->slowram_from_bank << 16) | window_replace->slowram_from_address;
          memory_address_end = (window_replace->slowram_to_bank << 16) | window_replace->slowram_to_address;
          memory_length = (memory_address_end - memory_address_begin) + 1;
          
          /* Allocation mmoire */
          memory_data = (unsigned char *) calloc(1,memory_length+16);
          if(memory_data == NULL)
            return;
          
          /* Rcupre les donnes brutes */
          ReadMemoryBlock(memory_address_begin,memory_length,memory_data,current_mmu);        
        }
      else if(area == 3 && window_replace->search_area_rom == 1)
        {
          /* Zone mmoire */
          memory_address_begin = (window_replace->rom_from_bank << 16) | window_replace->rom_from_address;
          memory_address_end = (window_replace->rom_to_bank << 16) | window_replace->rom_to_address;
          memory_length = (memory_address_end - memory_address_begin) + 1;
          
          /* Allocation mmoire */
          memory_data = (unsigned char *) calloc(1,memory_length+16);
          if(memory_data == NULL)
            return;
          
          /* Rcupre les donnes brutes */
          ReadMemoryBlock(memory_address_begin,memory_length,memory_data,current_mmu);        
        }
      else if(area == 4 && window_replace->search_area_docram == 1)
        {
          /* Zone mmoire */
          memory_address_begin = window_replace->docram_from_address;
          memory_address_end = window_replace->docram_to_address;
          memory_length = (memory_address_end - memory_address_begin) + 1;
          
          /* Allocation mmoire */
          memory_data = (unsigned char *) calloc(1,memory_length+16);
          if(memory_data == NULL)
            return;
          
          /* Rcupre les donnes brutes */
          ReadDOCRAMBlock(memory_address_begin,memory_length,memory_data,current_mmu);
        }
        
      /** Recherche les octets dans la zone mmoire **/
      for(i=0; i<(memory_length-data_length); i++)
        {
          /** Recherche des octets **/
          if(window_replace->search_type == REPLACE_TYPE_TEXT)
            {
              /** Valide toute la chaine **/
              for(j=0,match=1; j<data_length; j++)
                {
                  /* Rcupre 2 caractres */
                  one_memory_char = memory_data[i+j];
                  if(window_replace->options_reverseorder == 1)
                    one_data_char = data[data_length-j-1];
                  else
                    one_data_char = data[j];
                    
                  /*** Comparaisons ***/
                  if(window_replace->options_wildcard == 1 && one_data_char == '?')
                    continue;
                  
                  /** Ascii **/                    
                  if(window_replace->charset_ascii == 1)
                    {
                      /* Plage de valeurs limite 0x20-0x7F */
                      if(one_memory_char >= 0x20 && one_memory_char <= 0x7F)
                        {
                          /* Case sensitive */
                          if(window_replace->options_casesensitive == 1)
                            if(one_memory_char == one_data_char)
                              continue;
                          
                          /* Case Insensitive */
                          if(window_replace->options_casesensitive == 0)
                            if(toupper(one_memory_char) == toupper(one_data_char))    /* On peut utiliser toupper car on est en Ascii */
                              continue;
                        }
                    }
                    
                  /** High Ascii **/
                  if(window_replace->charset_highascii == 1)
                    {
                      /* Plage de valeurs limite 0x80-0xFF */
                      if(one_memory_char >= 0x80 && one_memory_char <= 0xFF)
                        {
                          /* Enlve le High Bit */
                          one_memory_char -= 0x80;
                          
                          /* Case sensitive */
                          if(window_replace->options_casesensitive == 1)
                            if(one_memory_char == one_data_char)
                              continue;
                          
                          /* Case Insensitive */
                          if(window_replace->options_casesensitive == 0)
                            if(toupper(one_memory_char) == toupper(one_data_char))    /* On peut utiliser toupper car on est en Ascii */
                              continue;                          
                        }
                    }

                  /** INVERSE **/
                  if(window_replace->charset_inverse == 1)
                    {
                      /* Plage de valeurs limite : 0x00-0x3F */
                      if(one_memory_char >= 0x00 && one_memory_char <= 0x3F)
                        {
                          /* Case Insensitive => Force en majuscule */
                          if(window_replace->options_casesensitive == 0)
                            one_data_char = toupper(one_data_char);            /* Ascii */

                          /* Offset 0-63 du caractre dans la plage autorise (qui n'est qu'en majuscule) */
                          next_char = strchr(special_charset,one_data_char);
                          if(next_char != NULL)
                            {
                              offset = (unsigned char) (next_char - special_charset);
                              if(offset == one_memory_char)
                                continue;
                            }
                        }
                    }
                    
                  /** FLASH **/
                  if(window_replace->charset_flash == 1)
                    {
                      /* Plage de valeurs limite : 0x40-0x7F */
                      if(one_memory_char >= 0x40 && one_memory_char <= 0x7F)
                        {
                          /* On passe en 0x00 - 0x3F => 64 valeurs possibles */
                          one_memory_char -= 0x40;
                          
                          /* Case Insensitive => Force en majuscule */
                          if(window_replace->options_casesensitive == 0)
                            one_data_char = toupper(one_data_char);       /* Ascii */

                          /* Offset 0-63 du caractre dans la plage autorise (qui n'est qu'en majuscule) */
                          next_char = strchr(special_charset,one_data_char);
                          if(next_char != NULL)
                            {
                              offset = (unsigned char) (next_char - special_charset);
                              if(offset == one_memory_char)
                                continue;
                            }                          
                        }                    
                    }
                    
                  /* Pas trouv */
                  match = 0;
                  break;
                }
                
              /** On conserve cette Adresse **/
              if(match == 1)
                {
                  /* Allocation mmoire */
                  current_replaceresult = mem_alloc_replaceresult(area,memory_address_begin+i,data_length,&memory_data[i]);
                  if(current_replaceresult == NULL)
                    continue;

                  /* Attachement */
                  if(window_replace->first_replaceresult == NULL)
                    window_replace->first_replaceresult = current_replaceresult;
                  else
                    window_replace->last_replaceresult->next = current_replaceresult;
                  window_replace->last_replaceresult = current_replaceresult;
                  window_replace->nb_found++;
                }              
            }
          else
            {
              /* Recherche du binaire */
              if(!memcmp(&memory_data[i],data,data_length))
                {
                  /* Allocation mmoire */
                  current_replaceresult = mem_alloc_replaceresult(area,memory_address_begin+i,data_length,&memory_data[i]);
                  if(current_replaceresult == NULL)
                    continue;

                  /* Attachement */
                  if(window_replace->first_replaceresult == NULL)
                    window_replace->first_replaceresult = current_replaceresult;
                  else
                    window_replace->last_replaceresult->next = current_replaceresult;
                  window_replace->last_replaceresult = current_replaceresult;
                  window_replace->nb_found++;
                }
            }
        }
   
      /* Libration mmoire */
      if(memory_data != NULL)
        free(memory_data);
      memory_data = NULL;
    }
    
  /** On met en place le premier rsultat de la recherche **/
  DisplaySearchResult(window_replace,0);
}


/************************************************************************************/
/*  mem_alloc_replaceresult() :  Allocation mmoire de la structure replace_result. */
/************************************************************************************/
static struct replace_result *mem_alloc_replaceresult(int replace_area, int memory_address, int data_size, unsigned char *data)
{
  struct replace_result *current_replaceresult;
  
  /* Allocation mmoire */
  current_replaceresult = (struct replace_result *) calloc(1,sizeof(struct replace_result));
  if(current_replaceresult == NULL)
    return(NULL);
    
  /* Remplissage */
  current_replaceresult->replace_area = replace_area;
  current_replaceresult->bank = memory_address >> 16;
  current_replaceresult->address = memory_address & 0xFFFF;
  current_replaceresult->data_size = data_size;

  /* Renvoi la structure */
  return(current_replaceresult);
}


/***************************************************************************************/
/*  mem_free_all_replaceresult() :  Libration mmoire de la structure replace_result. */
/***************************************************************************************/
static void mem_free_all_replaceresult(struct replace_result *first_replaceresult)
{
  struct replace_result *current_replaceresult;
  struct replace_result *next_replaceresult;
  
  for(current_replaceresult = first_replaceresult; current_replaceresult; current_replaceresult = next_replaceresult)
    {
      next_replaceresult = current_replaceresult->next;
      mem_free_replaceresult(current_replaceresult);
    }
}


/***********************************************************************************/
/*  mem_free_replaceresult() :  Libration mmoire de la structure replace_result. */
/***********************************************************************************/
void mem_free_replaceresult(struct replace_result *current_replaceresult)
{
  if(current_replaceresult)
    {       
      free(current_replaceresult);
    }
}


/*****************************************************************/
/*  DisplaySearchResult() :  Affiche les rsultats de recherche. */
/*****************************************************************/
static void DisplaySearchResult(struct window_replace *window_replace, int direction)
{
  int i;
  struct replace_result *current_replaceresult;
  
  /** Premier affichage aprs une Recherche ou aprs un Replace **/
  if(direction == 0)
    {
      /* Rien trouv */
      if(window_replace->nb_found == 0)
        {
          /* Rien  afficher */
          window_replace->current_result = 0;
          
          /* Dsactive les 2 boutons Search Results + Replace */
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_BUTTON_PREVIOUS),FALSE);
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_BUTTON_NEXT),FALSE);
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_BUTTON_REPLACE),FALSE);
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_BUTTON_REPLACEALL),FALSE);
          
          /* Construit les rsultats */
          SetWindowText(GetDlgItem(window_replace->hwnd_replace,IDC_STATIC_RESULT),"0");
          EmptySearchResult(window_replace);
        }
      else if(window_replace->nb_found == 1)
        {
          /* Affiche la premire fois juste aprs la recherche */
          window_replace->current_result = 1;
          
          /* Dsactive les 2 boutons Search Results + Bouton ReplaceAll */
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_BUTTON_PREVIOUS),FALSE);
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_BUTTON_NEXT),FALSE);
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_BUTTON_REPLACEALL),FALSE);
          
          /* Active le bouton Replace */
          if(window_replace->first_replaceresult->replaced == 0)
            EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_BUTTON_REPLACE),TRUE);
          else
            EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_BUTTON_REPLACE),FALSE);

          /* Construit les rsultats */
          BuildSearchFromData(window_replace,window_replace->first_replaceresult);
        }
      else if(window_replace->current_result == 0)
        {
          /* Affiche la premire fois, jsyte aprs la recherche */
          window_replace->current_result = 1;

          /* Active les 2 boutons Search Results + Replace */
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_BUTTON_PREVIOUS),TRUE);
          EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_BUTTON_NEXT),TRUE);

          /* Active le bouton Replace */
          if(window_replace->first_replaceresult->replaced == 0)
            EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_BUTTON_REPLACE),TRUE);
          else
            EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_BUTTON_REPLACE),FALSE);

          /* Active le bouton ReplaceAll */
          if(window_replace->nb_found > window_replace->nb_replaced)
            EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_BUTTON_REPLACEALL),TRUE);
          else
            EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_BUTTON_REPLACEALL),FALSE);

          /* Construit les rsultats */
          BuildSearchFromData(window_replace,window_replace->first_replaceresult);        
        }
      else
        {
          /* Affiche aprs un remplacement */
          /* Rcupre le current_result */
          for(i=1, current_replaceresult=window_replace->first_replaceresult; i<=window_replace->nb_found; i++, current_replaceresult=current_replaceresult->next)
            if(i == window_replace->current_result)
              break;

          /* Active le bouton Replace */
          if(current_replaceresult->replaced == 0)
            EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_BUTTON_REPLACE),TRUE);
          else
            EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_BUTTON_REPLACE),FALSE);

          /* Active le bouton ReplaceAll */
          if(window_replace->nb_found > window_replace->nb_replaced)
            EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_BUTTON_REPLACEALL),TRUE);
          else
            EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_BUTTON_REPLACEALL),FALSE);

          /* Construit les rsultats */
          BuildSearchFromData(window_replace,current_replaceresult);
        }
    }
  else if(direction == -1)  /* Previous */
    {
      /* Borne infrieure */
      if(window_replace->current_result == 1)
        return;

      /* Affiche la precdente */
      window_replace->current_result -= 1;

      /* Construit les rsultats */
      for(i=1, current_replaceresult=window_replace->first_replaceresult; i<=window_replace->nb_found; i++, current_replaceresult=current_replaceresult->next)
        if(i == window_replace->current_result)
          break;

      /* Active le bouton Replace */
      if(current_replaceresult->replaced == 0)
        EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_BUTTON_REPLACE),TRUE);
      else
        EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_BUTTON_REPLACE),FALSE);

      /* Construit les rsultats */
      BuildSearchFromData(window_replace,current_replaceresult);          
    }
  else if(direction == 1)   /* Next */
    {
      /* Borne supprieure */
      if(window_replace->current_result == window_replace->nb_found)
        return;

      /* Affiche la suivante */
      window_replace->current_result += 1;
                
      /* Construit les rsultats */
      for(i=1, current_replaceresult=window_replace->first_replaceresult; i<=window_replace->nb_found; i++, current_replaceresult=current_replaceresult->next)
        if(i == window_replace->current_result)
          break;      

      /* Active le bouton Replace */
      if(current_replaceresult->replaced == 0)
        EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_BUTTON_REPLACE),TRUE);
      else
        EnableWindow(GetDlgItem(window_replace->hwnd_replace,IDC_BUTTON_REPLACE),FALSE);

      /* Construit les rsultats */
      BuildSearchFromData(window_replace,current_replaceresult);      
    }
}


/*********************************************************************************************************/
/*  BuildSearchFromData() :  Construire les donnes hxadcimal  afficher  partir des donnes brutes. */
/*********************************************************************************************************/
static void BuildSearchFromData(struct window_replace *window_replace, struct replace_result *current_replaceresult)
{
  EDITSTREAM edit_stream;
  int i, j, offset, offset_rtf, address_base_begin, address_base_end, nb_line;
  int search_data_address_begin, search_data_address_end, search_data_value, search_data_sep;
  char buffer[1024];
  char charset_tab[] = "@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_ !\"#$%&'()*+,-./0123456789:;<=>?";    /* INV et FLASH */
  unsigned char data[256];
  struct mmu *current_mmu;
    
  /* Rcupre le MMU */
  my_Memory(MEMORY_GET_S65816_MMU,&current_mmu,NULL);
  
  /* Numro de la recherche */
  sprintf(buffer,"%d / %d",window_replace->current_result,window_replace->nb_found);
  SetWindowText(GetDlgItem(window_replace->hwnd_replace,IDC_STATIC_RESULT),buffer);

  /* Zone de recherche des donnes */
  search_data_address_begin = (current_replaceresult->bank << 16) | current_replaceresult->address;
  search_data_address_end = search_data_address_begin + current_replaceresult->data_size - 1;

  /** Rcupre les donnes **/
  address_base_begin = search_data_address_begin & 0xFFFFFFF0;
  address_base_end = search_data_address_end & 0xFFFFFFF0;
  nb_line = (address_base_begin == address_base_end) ? 1 : 2;
  if(current_replaceresult->replace_area == REPLACE_AREA_DOCRAM)
    ReadDOCRAMBlock(address_base_begin,nb_line*16,data,current_mmu);
  else
    ReadMemoryBlock(address_base_begin,nb_line*16,data,current_mmu);    

  /******************************************************/
  /*** Cration d'un buffer RTF rpentant les donnes ***/
  window_replace->rtf_data_hexadecimal[0] = '\0';
  strcat(window_replace->rtf_data_hexadecimal,"{\\rtf1\\ansi\\ansicpg1252\\deff0\\deflang1036{\\fonttbl{\\f0\\fnil\\fcharset0 Courier New;}}\r\n");
  strcat(window_replace->rtf_data_hexadecimal,"{\\colortbl ;\\red51\\green153\\blue255;\\red0\\green77\\blue187;\\red0\\green176\\blue80;\\red192\\green80\\blue77;\\red128\\green128\\blue128;}\r\n");
  strcat(window_replace->rtf_data_hexadecimal,"{\\*\\generator Msftedit 5.41.21.2510;}\\viewkind4\\uc1\\pard\\sl240\\slmult1\\lang12\\f0\\fs");
  strcat(window_replace->rtf_data_hexadecimal,REPLACE_RICHEDIT_FONT_HEIGHT_RTF);  /* Taille de la Police : Si 8 dans WordPad => 16 ici (x2)*/
  strcat(window_replace->rtf_data_hexadecimal," ");
  offset_rtf = (int) strlen(window_replace->rtf_data_hexadecimal);

  /*** Transformer les donnes BYTE en Hexa ***/
  for(i=0,offset=0; i<nb_line; i++,offset+=16)
    {
      /* Adresse */
      if(current_replaceresult->replace_area == REPLACE_AREA_DOCRAM)
        sprintf(&window_replace->rtf_data_hexadecimal[offset_rtf],"DOC/%04X  ",(address_base_begin+offset)&0xFFFF);      
      else
        sprintf(&window_replace->rtf_data_hexadecimal[offset_rtf],"%02X/%04X   ",(address_base_begin+offset)>>16,(address_base_begin+offset)&0xFFFF);
      offset_rtf += (int) strlen("00/0000   ");

      /** 1 Lignes = Hexa + Ascii **/
      for(j=0; j<16; j++)
        {
          /* Est t'on dans la zone recherche ? */
          search_data_value = ((address_base_begin+offset+j) >= search_data_address_begin && (address_base_begin+offset+j) <= search_data_address_end) ? 1 : 0;
          search_data_sep = ((address_base_begin+offset+j) >= search_data_address_begin && (address_base_begin+offset+j) < search_data_address_end) ? 1 : 0;

          /* Insre le code couleur de slection */
          if(search_data_value == 1)
            {
              strcpy(&window_replace->rtf_data_hexadecimal[offset_rtf],"\\highlight1 ");
              offset_rtf += (int) strlen("\\highlight1 ");
            }

          /* Byte Hexa Ascii */
          sprintf(&window_replace->rtf_data_hexadecimal[offset_rtf],"%02X",data[offset+j]);
          offset_rtf += 2;

          /* Supprime la couleur de slection */
          if(search_data_value == 1)
            {
              strcpy(&window_replace->rtf_data_hexadecimal[offset_rtf],"\\highlight0 ");
              offset_rtf += (int) strlen("\\highlight0 ");
            }

          /* Insre le code couleur de slection */
          if(search_data_sep == 1)
            {
              strcpy(&window_replace->rtf_data_hexadecimal[offset_rtf],"\\highlight1 ");
              offset_rtf += (int) strlen("\\highlight1 ");
            }

          /* Sparateur */
          if(j < (16-1))
            {
              window_replace->rtf_data_hexadecimal[offset_rtf] = (j==7 && (i*window_replace->nb_column+j < nb_line*16)) ? '.' : ' ';
              offset_rtf++;
            }

          /* Supprime la couleur de slection */
          if(search_data_sep == 1)
            {
              strcpy(&window_replace->rtf_data_hexadecimal[offset_rtf],"\\highlight0 ");
              offset_rtf += (int) strlen("\\highlight0 ");
            }
        }

      /* Sparateur */
      strcpy(&window_replace->rtf_data_hexadecimal[offset_rtf],"  ");
      offset_rtf += 2;

      /* Ascii */
      for(j=0; j<16; j++)
        {
          /* Est t'on dans la zone recherche ? */
          search_data_value = ((address_base_begin+offset+j) >= search_data_address_begin && (address_base_begin+offset+j) <= search_data_address_end) ? 1 : 0;

          /* Insre le code couleur de slection */
          if(search_data_value == 1)
            {
              strcpy(&window_replace->rtf_data_hexadecimal[offset_rtf],"\\highlight1 ");
              offset_rtf += (int) strlen("\\highlight1 ");
            }

          /* Caractre Ascii */
          if(window_replace->search_type == REPLACE_TYPE_TEXT)
            {
              /* On ne prend les Charset dfinis */
              if(window_replace->charset_ascii == 1 && data[offset+j] >= 0x20 && data[offset+j] <= 0x7E)
                sprintf(&window_replace->rtf_data_hexadecimal[offset_rtf],"%c",data[offset+j]);
              else if(window_replace->charset_highascii == 1 && data[offset+j] >= 0xA0 && data[offset+j] <= 0xFE)
                sprintf(&window_replace->rtf_data_hexadecimal[offset_rtf],"%c",data[offset+j]-0x80);
              else if(window_replace->charset_inverse == 1 && data[offset+j] >= 0x00 && data[offset+j] <= 0x3F)
                sprintf(&window_replace->rtf_data_hexadecimal[offset_rtf],"%c",charset_tab[offset+j]);
              else if(window_replace->charset_flash == 1 && data[offset+j] >= 0x40 && data[offset+j] <= 0x7F)
                sprintf(&window_replace->rtf_data_hexadecimal[offset_rtf],"%c",charset_tab[offset+j]-0x40);
              else
                strcpy(&window_replace->rtf_data_hexadecimal[offset_rtf],".");
            }
          else
            {
              /* On ne prend que l'Ascii standard */
              if(data[offset+j] >= 0x20 && data[offset+j] <= 0x7E)
                sprintf(&window_replace->rtf_data_hexadecimal[offset_rtf],"%c",data[offset+j]);
              else
                strcpy(&window_replace->rtf_data_hexadecimal[offset_rtf],".");
            }
          offset_rtf++;

          /* Supprime la couleur de slection */
          if(search_data_value == 1)
            {
              strcpy(&window_replace->rtf_data_hexadecimal[offset_rtf],"\\highlight0 ");
              offset_rtf += (int) strlen("\\highlight0 ");
            }
        }

      /* Fin de ligne */
      strcpy(buffer,"\\par\r\n");
      strcpy(&window_replace->rtf_data_hexadecimal[offset_rtf],buffer);
      offset_rtf += (int) strlen(buffer);
    }
    
  /** Fin du RTF **/
  strcpy(&window_replace->rtf_data_hexadecimal[offset_rtf],"}\r\n");
  offset_rtf += (int) strlen("}\r\n");
  window_replace->rtf_data_hexadecimal[offset_rtf] = '\0';

  /* Taille du RTF (le \0 final compte) */
  window_replace->rtf_data_size = (int) strlen(window_replace->rtf_data_hexadecimal) + 1;

  /** Afficher le rsultat en RTF dans le control Rich Edit **/
  window_replace->rtf_write_offset = 0;
  memset(&edit_stream,0,sizeof(EDITSTREAM));
  edit_stream.pfnCallback = (EDITSTREAMCALLBACK) ReplaceRichEditStreamCallback;
  edit_stream.dwCookie = (DWORD_PTR) window_replace;          /* Paramtre  passer  la fonction Callback */
  SendMessage(window_replace->hwnd_richedit_data,EM_STREAMIN,SF_RTF,(LPARAM)&edit_stream);
}


/*********************************************************************************/
/*  EmptySearchResult() :  Vide la zone Rich Text des rsultats de la recherche. */
/*********************************************************************************/
static void EmptySearchResult(struct window_replace *window_replace)
{
  EDITSTREAM edit_stream;
  int offset_rtf;
  
  /******************************************************/
  /*** Cration d'un buffer RTF rpentant les donnes ***/
  window_replace->rtf_data_hexadecimal[0] = '\0';
  strcat(window_replace->rtf_data_hexadecimal,"{\\rtf1\\ansi\\ansicpg1252\\deff0\\deflang1036{\\fonttbl{\\f0\\fnil\\fcharset0 Courier New;}}\r\n");
  strcat(window_replace->rtf_data_hexadecimal,"{\\colortbl ;\\red255\\green0\\blue0;\\red0\\green77\\blue187;\\red0\\green176\\blue80;\\red192\\green80\\blue77;\\red128\\green128\\blue128;}\r\n");
  strcat(window_replace->rtf_data_hexadecimal,"{\\*\\generator Msftedit 5.41.21.2510;}\\viewkind4\\uc1\\pard\\sl240\\slmult1\\lang12\\f0\\fs");
  strcat(window_replace->rtf_data_hexadecimal,REPLACE_RICHEDIT_FONT_HEIGHT_RTF);  /* Taille de la Police : Si 8 dans WordPad => 16 ici (x2)*/
  strcat(window_replace->rtf_data_hexadecimal," ");
  offset_rtf = (int) strlen(window_replace->rtf_data_hexadecimal);  
  
  /** Fin du RTF **/
  strcpy(&window_replace->rtf_data_hexadecimal[offset_rtf],"}\r\n");
  offset_rtf += (int) strlen("}\r\n");
  window_replace->rtf_data_hexadecimal[offset_rtf] = '\0';

  /* Taille du RTF (le \0 final compte) */
  window_replace->rtf_data_size = (int) strlen(window_replace->rtf_data_hexadecimal) + 1;

  /** Afficher le rsultat en RTF dans le control Rich Edit **/
  window_replace->rtf_write_offset = 0;
  memset(&edit_stream,0,sizeof(EDITSTREAM));
  edit_stream.pfnCallback = (EDITSTREAMCALLBACK) ReplaceRichEditStreamCallback;
  edit_stream.dwCookie = (DWORD_PTR) window_replace;          /* Paramtre  passer  la fonction Callback */
  SendMessage(window_replace->hwnd_richedit_data,EM_STREAMIN,SF_RTF,(LPARAM)&edit_stream);  
}


/*************************************************************************************/
/*  ReplaceRichEditStreamCallback() :  Fonction Call back pour remplir le Rich Edit. */
/*************************************************************************************/
static DWORD CALLBACK ReplaceRichEditStreamCallback(DWORD *dwCookie, LPBYTE output_buffer, LONG nb_write_max, LONG *nb_write_rtn)
{
  int nb_remain_byte, nb_copy_byte;
  struct window_replace *window_replace;

  /* Rcupration du paramtre */
  window_replace = (struct window_replace *) dwCookie;

  /* Taille des donnes encore  copier */
  nb_remain_byte = window_replace->rtf_data_size - window_replace->rtf_write_offset;
  if(nb_remain_byte == 0)
    {
      /* Plus rien  copier */
      *nb_write_rtn = 0;
      return(0);
    }

  /* Taille des donnes que l'on est autoris  copier */
  nb_copy_byte = (nb_write_max >= nb_remain_byte) ? nb_remain_byte : nb_write_max;

  /* Nombre de byte transfrs */
  *nb_write_rtn = nb_copy_byte;
		  
  /** Copie les Byte dans le output buffer **/
  memcpy(output_buffer,&window_replace->rtf_data_hexadecimal[window_replace->rtf_write_offset],nb_copy_byte);

  /* Dplace l'offset */
  window_replace->rtf_write_offset += nb_copy_byte;

  /* OK */
  return(0);
}


/***************************************************************************************/
/*  ReplaceSearchResult() :  Remplace les valeurs trouves par celle qui les remplace. */
/***************************************************************************************/
static void ReplaceSearchResult(struct window_replace *window_replace, int replace_type)
{
  BYTE one_byte;
  WORD one_word;
  DWORD one_dword;
  unsigned int value;
  int data_length;
  unsigned char data[1024];
  int i, is_hexadecimal, is_decimal, is_odd;
  char replace_data[1024];
  char buffer[1024];
  char *ascii_charset = " !\"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklmnopqrstuvwxyz{|}~";
  char special_charset[64+1] = "@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_ !\"#$%&'()*+,-./0123456789:;<=>?";  
  struct replace_result *current_replaceresult;
  struct mmu *current_mmu;
    
  /* Rcupre le MMU */
  my_Memory(MEMORY_GET_S65816_MMU,&current_mmu,NULL);

  /*** Construit les octets de remplacement ***/
  /** Rcupre le critre de recherche **/
  GetWindowText(GetDlgItem(window_replace->hwnd_replace,IDC_EDIT_REPLACE),replace_data,1024-1);
  if(strlen(replace_data) == 0)
    {
      MessageBox(window_replace->hwnd_replace,"Empty Replace value !","Replace Criteria Error",MB_OK|MB_ICONWARNING);
      return;
    }
  
  /** Conversion **/
  if(window_replace->replace_type == REPLACE_TYPE_HEXA)
    {
      /* Vrifie le format */
      is_hexadecimal = IsHexaDecimal(replace_data,&is_odd);
      if(is_hexadecimal == 0)
        {
          sprintf(window_replace->buffer_log,"Error : The value '%s' is not in hexadecimal format",replace_data);
          MessageBox(window_replace->hwnd_replace,window_replace->buffer_log,"Replace Criteria Error",MB_OK|MB_ICONWARNING);
          return;
        }
        
      /** Conversion en Hexadcimal **/
      sscanf(replace_data,"%X",&value);
      if(window_replace->replace_size == REPLACE_SIZE_8)
        {
          one_byte = (BYTE) value;
          data_length = 1;
          memcpy(&data[0],&one_byte,data_length);
        }
      else if(window_replace->replace_size == REPLACE_SIZE_16)
        {
          one_word = (WORD) value;
          data_length = 2;
          memcpy(&data[0],&one_word,data_length);
        }
      else if(window_replace->replace_size == REPLACE_SIZE_24)
        {
          one_dword = (DWORD) value;
          data_length = 3;
          memcpy(&data[0],&one_dword,4);
        }        
      else if(window_replace->replace_size == REPLACE_SIZE_32)
        {
          one_dword = (DWORD) value;
          data_length = 4;
          memcpy(&data[0],&one_dword,data_length);
        }
        
      /** Inverse les octets **/
      if(window_replace->replace_align == REPLACE_ALIGN_MOTOROLA && data_length > 1)
        ExchangeByte(&data[0],data_length);
    }
  else if(window_replace->replace_type == REPLACE_TYPE_DECIMAL)
    {
      /* Vrifie le format */
      is_decimal = IsDecimal(replace_data,&is_odd);
      if(is_decimal == 0)
        {
          sprintf(window_replace->buffer_log,"Error : The value '%s' is not in decimal format",replace_data);
          MessageBox(window_replace->hwnd_replace,window_replace->buffer_log,"Replace Criteria Error",MB_OK|MB_ICONWARNING);
          return;
        }
        
      /** Conversion en Dcimal **/
      value = atoi(replace_data);
      if(window_replace->replace_size == REPLACE_SIZE_8)
        {
          one_byte = (BYTE) value;
          data_length = 1;
          memcpy(&data[0],&one_byte,data_length);          
        }
      else if(window_replace->replace_size == REPLACE_SIZE_16)
        {
          one_word = (WORD) value;
          data_length = 2;
          memcpy(&data[0],&one_word,data_length);          
        }
      else if(window_replace->replace_size == REPLACE_SIZE_24)
        {
          one_dword = (DWORD) value;
          data_length = 3;
          memcpy(&data[0],&one_dword,4);
        }
      else if(window_replace->replace_size == REPLACE_SIZE_32)
        {
          one_dword = (DWORD) value;
          data_length = 4;
          memcpy(&data[0],&one_dword,data_length);          
        }
        
      /** Inverse les octets **/
      if(window_replace->replace_align == REPLACE_ALIGN_MOTOROLA && data_length > 1)
        ExchangeByte(&data[0],data_length);     
    }    
  else if(window_replace->replace_type == REPLACE_TYPE_TEXT)
    {
      /** Rcupre la chaine de caractre **/
      strcpy((char *)data,replace_data);
      data_length = (int) strlen(replace_data);
      
      /* On va valider les caractres saisis : ASCII / HIGHASCII */
      if(window_replace->replace_charset == REPLACE_CHARSET_ASCII || window_replace->replace_charset == REPLACE_CHARSET_HIGHASCII)
        {
          /* Valide les caractres saisis */
          for(i=0; i<(int)strlen((char *)data); i++)
            if(strchr(ascii_charset,data[i]) == NULL)
              {
                sprintf(window_replace->buffer_log,"Error : The character '%c' is not valid.",data[i]);
                MessageBox(window_replace->hwnd_replace,window_replace->buffer_log,"Replace Text Error",MB_OK|MB_ICONWARNING);
                return;
              }
              
          /* On va adapter les valeurs */
          if(window_replace->replace_charset == REPLACE_CHARSET_HIGHASCII)
            for(i=0; i<(int)strlen((char *)data); i++)
              data[i] = data[i] | 0x80;
        }
      else if(window_replace->replace_charset == REPLACE_CHARSET_INVERSE || window_replace->replace_charset == REPLACE_CHARSET_FLASH)
        {
          /* Valide les caractres saisis */
          for(i=0; i<(int)strlen((char *)data); i++)
            if(strchr(special_charset,data[i]) == NULL)
              {
                sprintf(window_replace->buffer_log,"Error : The character '%c' is not valid.",data[i]);
                MessageBox(window_replace->hwnd_replace,window_replace->buffer_log,"Replace Text Error",MB_OK|MB_ICONWARNING);
                return;
              }
              
          /* On va adapter les valeurs */
          for(i=0; i<(int)strlen((char *)data); i++)
            {
              if(window_replace->replace_charset == REPLACE_CHARSET_INVERSE)
                {
                  /* 0x00 -> 0x3F */
                  if(data[i] >= '@' && data[i] <= '_')
                    data[i] = data[i] - 0x40;           /* @ A B C D E F G H I J K L M N O P Q R S T U V W X Y Z [ \ ] ^ _ */
                }
              else if(window_replace->replace_charset == REPLACE_CHARSET_FLASH)
                {
                  /* 0x40 -> 0x7F */
                  if(data[i] >= ' ' && data[i] <= '?')
                    data[i] = data[i] + 0x40;           /*   ! " # $ % & ' ( ) * + , - . / 0 1 2 3 4 5 6 7 8 9 : ; < = > ? */
                }
            }
        }
    } 
  else if(window_replace->replace_type == REPLACE_TYPE_BYTES)
    {
      /* Supprime les espaces */
      RemoveSpace(replace_data);
      if(strlen(replace_data) == 0)
        {
          MessageBox(window_replace->hwnd_replace,"Empty Replace value !","Replace Criteria Error",MB_OK|MB_ICONWARNING);
          return;
        }
    
      /* Vrifie le format */
      is_hexadecimal = IsHexaDecimal(replace_data,&is_odd);
      if(is_hexadecimal == 0)
        {
          sprintf(window_replace->buffer_log,"Error : The replace value is not in hexadecimal format",replace_data);
          MessageBox(window_replace->hwnd_replace,window_replace->buffer_log,"Replace Criteria Error",MB_OK|MB_ICONWARNING);
          return;
        }
      if(is_odd == 1)
        {
          sprintf(window_replace->buffer_log,"Error : The length of the value should be even",replace_data);
          MessageBox(window_replace->hwnd_replace,window_replace->buffer_log,"Replace Criteria Error",MB_OK|MB_ICONWARNING);
          return;
        }
        
      /** Conversion en Hexa **/
      for(i=0; i<(int)strlen(replace_data); i+=2)
        {
          memcpy(buffer,&replace_data[i],2);
          buffer[2] = '\0';
          sscanf(buffer,"%02X",&value);
          data[i/2] = (unsigned char) value;
        }
      data_length = (int) strlen(replace_data)/2;
    }

  /* Conserve la valeur de remplacement */
  window_replace->replace_query_size = data_length;
  memcpy(window_replace->replace_query_data,data,data_length);

  /** On veut une adquation entre le Search et le Result **/
  if(window_replace->search_query_size != window_replace->replace_query_size)
    {
      sprintf(window_replace->buffer_log,"Error : The length of the Replace value (%d) is different from the Search value (%d)",window_replace->replace_query_size,window_replace->search_query_size);
      MessageBox(window_replace->hwnd_replace,window_replace->buffer_log,"Search / Replace Criteria Error",MB_OK|MB_ICONWARNING);
      return;
    }

  /*** Remplace la/les valeur(s) ***/
  if(replace_type == REPLACE_CURRENT_RESULT)
    {
      /* Rcupre le current_result */
      for(i=1, current_replaceresult=window_replace->first_replaceresult; i<=window_replace->nb_found; i++, current_replaceresult=current_replaceresult->next)
        if(i == window_replace->current_result)
          break;

      /* Y a t'il qqchose  remplacer */
      if(current_replaceresult->replaced == 0)
        {
          /* On remplace tous les bytes */
          for(i=0; i<window_replace->replace_query_size; i++)
            {
              if(current_replaceresult->replace_area == REPLACE_AREA_DOCRAM)
                WriteDOCRAMByte(current_replaceresult->address+i,window_replace->replace_query_data[i],current_mmu);
              else
                WriteMemoryByte(((current_replaceresult->bank << 16) | current_replaceresult->address)+i,window_replace->replace_query_data[i],current_mmu);
            }

          /** Transfre les donnes dans la mmoire de l'mulateur **/
          if(current_replaceresult->replace_area != REPLACE_AREA_DOCRAM)
            WriteDataToEmulatorMemory(RAMTYPE_RAM,(current_replaceresult->bank << 16) | current_replaceresult->address,window_replace->replace_query_size,window_replace->replace_query_data);
          else
            WriteDataToEmulatorMemory(RAMTYPE_DOC,current_replaceresult->address,window_replace->replace_query_size,window_replace->replace_query_data);

          /* Cette valeur a t remplace */
          current_replaceresult->replaced = 1;
          window_replace->nb_replaced++;
        }

      /* MAJ de l'affichage des rsultats */
      DisplaySearchResult(window_replace,0);
    }
  else if(replace_type == REPLACE_ALL_RESULT)
    {
      /** Remplace partout o c'est possible **/
      for(current_replaceresult=window_replace->first_replaceresult; current_replaceresult; current_replaceresult=current_replaceresult->next)
        if(current_replaceresult->replaced == 0)
          {
            /* On remplace tous les bytes */
            for(i=0; i<window_replace->replace_query_size; i++)
              {
                if(current_replaceresult->replace_area == REPLACE_AREA_DOCRAM)
                  WriteDOCRAMByte(current_replaceresult->address+i,window_replace->replace_query_data[i],current_mmu);
                else
                  WriteMemoryByte(((current_replaceresult->bank << 16) | current_replaceresult->address)+i,window_replace->replace_query_data[i],current_mmu);
              }

            /** Transfre les donnes dans la mmoire de l'mulateur **/
            if(current_replaceresult->replace_area != REPLACE_AREA_DOCRAM)
              WriteDataToEmulatorMemory(RAMTYPE_RAM,(current_replaceresult->bank << 16) | current_replaceresult->address,window_replace->replace_query_size,window_replace->replace_query_data);
            else
              WriteDataToEmulatorMemory(RAMTYPE_DOC,current_replaceresult->address,window_replace->replace_query_size,window_replace->replace_query_data);

            /* Cette valeur a t remplace */
            current_replaceresult->replaced = 1;
            window_replace->nb_replaced++;
          }

      /* MAJ de l'affichage des rsultats */
      DisplaySearchResult(window_replace,0);
    }
}


/*********************************************************************/
/*  DeleteReplaceChildWindows() :  Destruction des fentres enfants. */
/*********************************************************************/
static void DeleteReplaceChildWindows(struct window_replace *window_replace)
{
  /** Libration des ressources **/  
  if(window_replace->rtf_data_hexadecimal != NULL)
    free(window_replace->rtf_data_hexadecimal);
  window_replace->rtf_data_hexadecimal = NULL;
}


/***********************************************************************/
/*  mem_free_window_replace() :  Libration mmoire pour la structure. */
/***********************************************************************/
void mem_free_window_replace(struct window_replace *window_replace)
{ 
  if(window_replace)
    {
      mem_free_all_replaceresult(window_replace->first_replaceresult);
           
      free(window_replace);
    }
}

/**********************************************************************/
