/************************************************************************/
/*                                                                      */
/*  Window_Softswitch.c :  Module de gestion de la fentre Softswitch.  */
/*                                                                      */
/************************************************************************/
/*  Auteur : Olivier ZARDINI  *   Brutal Deluxe   *   Date : Juin 2018  */
/************************************************************************/

#include <stdio.h>
#include <time.h>

#include <windows.h>
#include <windowsx.h>
#include <Commctrl.h>
#include <Winuser.h>
#include <Richedit.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_Notification.h"
#include "Dc_Graphic.h"
#include "Dc_CPU.h"
#include "Dc_MMU.h"
#include "Window_QuickWatch.h"
#include "Window_Main.h"
#include "Window_Main_Menu.h"
#include "Window_Softswitch.h"

static DWORD BuildSoftswitchWindowThread(void *);
LRESULT CALLBACK ProcHandlerSoftswitchWindow(HWND,UINT,WPARAM,LPARAM);
static int CreateSoftswitchChildWindows(HWND,struct window_softswitch *);
LRESULT CALLBACK subEditSoftswitchProcControl(HWND,UINT,WPARAM,LPARAM);
static void UpdateSoftswitchContextInformation(struct window_softswitch *,int,int);
static void UpdateSoftswitchRefreshMode(struct window_softswitch *,int);
static void RefreshSoftswitchWindowData(struct window_softswitch *);
static int IsSoftswitchDataModified(int,struct window_softswitch *);
static void ResizeSoftswitchWindow(struct window_softswitch *);
static void BuildSoftswitchFromData(struct window_softswitch *);
static void DeleteSoftswitchChildWindows(struct window_softswitch *);


/**********************************************************************************/
/*  BuildSoftswitchWindow() :  Construction / Affichage de la fentre Softswitch. */
/**********************************************************************************/
struct window_softswitch *BuildSoftswitchWindow(HINSTANCE hInstance, int menu_id)
{
  struct window_softswitch *window_softswitch;

  /** Allocation de la structure window_softswitch **/
  window_softswitch = (struct window_softswitch *) calloc(1,sizeof(struct window_softswitch));
  if(window_softswitch == NULL)
    return(NULL);
  my_Memory(MEMORY_SET_WINDOW_SOFTSWITCH,window_softswitch,NULL);

  /* Lance un thread pour grer la fentre Softswitch */
  CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)BuildSoftswitchWindowThread,window_softswitch,0,NULL);

  /* Renvoi la structure mmoire grant la fentre */
  return(window_softswitch);
}


/********************************************************************************************/
/*  BuildSoftswitchWindowThread() :  Cration de la fentre Softswitch via un Thread ddi. */
/********************************************************************************************/
static DWORD BuildSoftswitchWindowThread(void *window_softswitch_param)
{
  HWND hwndSoftswitch;
  MSG msg;
  BOOL bRet;
  struct window_main *window_main;
  struct window_softswitch *window_softswitch;
  struct parameter *param;

  /* Init */
  window_softswitch = (struct window_softswitch *) window_softswitch_param;
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Cration de la fentre Softswitch **/
  hwndSoftswitch = CreateDialog(param->hInstance,MAKEINTRESOURCE(IDD_WINDOW_SOFTSWITCH),window_main->hwnd_main,(DLGPROC)ProcHandlerSoftswitchWindow);
  if(hwndSoftswitch == NULL)
    return(1);

  /** Show Window + Paint Content **/
  ShowWindow(hwndSoftswitch,SW_SHOWDEFAULT);
  UpdateWindow(hwndSoftswitch);

  /** Message Loop pour toutes les fentres de ce Thread (fentre Softswitch...) **/
  while((bRet = GetMessage(&msg,NULL,0,0)) != 0)
    {
      if(bRet == -1)
        break;

      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }

  /* Fin du Thread */
  ExitThread(0);
}


/***************************************************************************/
/*  ProcHandlerSoftswitchWindow() :  Gestion des vnements de la fentre. */
/***************************************************************************/
LRESULT CALLBACK ProcHandlerSoftswitchWindow(HWND hwndSoftswitch, UINT umsg, WPARAM wParam, LPARAM lParam)
{
  struct window_softswitch *window_softswitch;
  struct parameter *param;
  
  /* Init */
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  my_Memory(MEMORY_GET_WINDOW_SOFTSWITCH,&window_softswitch,NULL);

  switch(umsg)
    {
      case WM_INITDIALOG :
        /* Cration du contenu de la DialogBox */
        CreateSoftswitchChildWindows(hwndSoftswitch,window_softswitch);
        return(TRUE);

      /***************************/
      /* Evnements de la souris */
      /***************************/
      case WM_MOUSEMOVE :
        SetWindowText(GetDlgItem(window_softswitch->hwnd_softswitch,IDC_STATIC_CONTEXT),"");
        return(TRUE);
        
      /****************************/
      /* Evnements de la fentre */
      /****************************/
      case WM_COMMAND :
        switch(LOWORD(wParam))
          {
            /** Bouton Radio : On change la mthode de rafraichissement de la fentre **/
            case IDC_RADIO_REFRESH_AUTOMATIC :
              if(HIWORD(wParam) == BN_CLICKED && window_softswitch->refresh_mode != SOFTSWITCH_REFRESH_AUTOMATIC)
                UpdateSoftswitchRefreshMode(window_softswitch,SOFTSWITCH_REFRESH_AUTOMATIC);
              break;

            case IDC_RADIO_REFRESH_MANUAL :
              if(HIWORD(wParam) == BN_CLICKED && window_softswitch->refresh_mode != SOFTSWITCH_REFRESH_MANUAL)
                UpdateSoftswitchRefreshMode(window_softswitch,SOFTSWITCH_REFRESH_MANUAL);
              break;

            /** Bouton de rafraichissement Manuel de la fentre **/
            case IDC_BUTTON_REFRESH :
              RefreshSoftswitchWindowData(window_softswitch);
              break;
          }
        return(TRUE);

      /*****************************************************************/
      /* Message extrieur demandant le rafraichissement de la fentre */
      /*****************************************************************/
      case HM_SOFTSWITCH_REFRESHWINDOW :
        RefreshSoftswitchWindowData(window_softswitch);
        return(TRUE);

      /********************************************************************/
      /* Fermeture de window (le WM_CLOSE est appel avant le WM_DESTROY) */
      /********************************************************************/
      case WM_CLOSE :
        /* Fentre Softswitch */
        DestroyWindow(hwndSoftswitch);
        return(0);

      case WM_DESTROY :
        /* Suppression des Fentres */
        DeleteSoftswitchChildWindows(window_softswitch);
        CheckMainWindowMenuItem(ID_MENU_VIEW_SOFTSWITCH,FALSE);
        my_Memory(MEMORY_FREE_WINDOW_SOFTSWITCH,NULL,NULL);
        /* Demande la sortie de la boucle des messages */
        PostQuitMessage(0);
        return(0);
    }

  /* Retourner FALSE pour relayer le message non trait au systme */
  return(FALSE);
}


/***********************************************************************************************************/
/*  CreateSoftswitchChildWindows() : Crer les fentres enfants et remplir la structure window_softswitch. */
/***********************************************************************************************************/
static int CreateSoftswitchChildWindows(HWND hwndSoftswitch, struct window_softswitch *window_softswitch)
{
  HWND hwnd_group;
  RECT client_rect, window_rect, group_rect;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Allocation Buffer mmoire Zone Texte : 256 Bytes de raw data **/
  window_softswitch->nb_line = 16; 
  window_softswitch->nb_column = 16;
  /* Nombre caractres visibles sur la ligne (on ne compte pas le \r\n final) */
  window_softswitch->text_data_width = (int)(strlen("00/0000   ") + window_softswitch->nb_column*3 - 1);   /* Enlve le dernier sparateur */
  /* Nombre d'octet pour encoder la zone Texte (on compte les \r\n en fin de ligne) */
  window_softswitch->text_data_size = window_softswitch->nb_line*(window_softswitch->text_data_width+2);  /* +2 pour */
  window_softswitch->text_data_hexadecimal = (char *) calloc(window_softswitch->text_data_size+1,sizeof(char));
  if(window_softswitch->text_data_hexadecimal == NULL)
    return(1);

  /** Window Handle **/
  window_softswitch->hwnd_softswitch = hwndSoftswitch;

  /* Force la MAJ du contenu de la fentre */
  window_softswitch->softswitch_address = -1;

  /* Refresh Mode : Automatic */
  UpdateSoftswitchRefreshMode(window_softswitch,SOFTSWITCH_REFRESH_AUTOMATIC);

  /* Hauteur Titre */
  GetWindowRect(window_softswitch->hwnd_softswitch,&window_rect);
  GetClientRect(window_softswitch->hwnd_softswitch,&client_rect);
  window_softswitch->title_height = (window_rect.bottom - window_rect.top) - client_rect.bottom;

  /* Position de la GroupBox Select + GroupBox Palette */
  hwnd_group = GetDlgItem(window_softswitch->hwnd_softswitch,IDC_GROUP_SOFTSWITCH);     /* Rcupre le Handle du Control GroupBox li  IDC_GROUP_SOFTSWITCH */
  GetWindowRect(hwnd_group,&group_rect);
  window_softswitch->group_data_x = group_rect.left - window_rect.left;
  window_softswitch->group_data_y = (group_rect.top - window_rect.top) - window_softswitch->title_height;

  /* Position de la Zone Text Direct Page Data dans la fenetre */
  window_softswitch->softswitch_data_x = window_softswitch->group_data_x + WINDOW_GROUP_BORDER_LEFT_SIZE;
  window_softswitch->softswitch_data_y = window_softswitch->group_data_y + WINDOW_GROUP_BORDER_TOP_SIZE;

  /* Dimension en Pixel du Control Text Softswitch Data */
  window_softswitch->softswitch_data_width = window_softswitch->text_data_width*7 + 2;
  window_softswitch->softswitch_data_height = window_softswitch->nb_line*SOFTSWITCH_EDIT_FONT_HEIGHT;

  /* Dimension en Pixel du Control Group Box + Resize */  
  window_softswitch->group_data_width = WINDOW_GROUP_BORDER_LEFT_SIZE + window_softswitch->softswitch_data_width + WINDOW_GROUP_BORDER_RIGHT_SIZE;
  window_softswitch->group_data_height = WINDOW_GROUP_BORDER_TOP_SIZE + window_softswitch->softswitch_data_height + WINDOW_GROUP_BORDER_BOTTOM_SIZE;
  MoveWindow(hwnd_group,window_softswitch->group_data_x,window_softswitch->group_data_y,window_softswitch->group_data_width,window_softswitch->group_data_height,TRUE);

  /** Cration du Control Text Edit Multi ligne ReadOnly **/
  window_softswitch->hwnd_edit_data = CreateWindowExA(0,"EDIT",NULL,WS_CHILD | WS_VISIBLE | ES_LEFT | ES_MULTILINE | ES_READONLY, 
                                                      window_softswitch->softswitch_data_x, window_softswitch->softswitch_data_y, window_softswitch->softswitch_data_width, window_softswitch->softswitch_data_height,   // set size in WM_SIZE message 
                                                      window_softswitch->hwnd_softswitch,
                                                      (HMENU)IDC_EDIT_SOFTSWITCH_DATA,            // edit control ID
                                                      param->hInstance,
                                                      NULL);

  /** Surchage des Control Edit pour capter les vnement **/
  window_softswitch->oldEditProcSoftswitchData = (WNDPROC) SetWindowLongPtr(window_softswitch->hwnd_edit_data,GWLP_WNDPROC,(LONG_PTR)subEditSoftswitchProcControl);

  /** Cration de la Police de caractre + Affectation de la police au Control Edit **/
  window_softswitch->hFont = CreateFont(SOFTSWITCH_EDIT_FONT_HEIGHT,0,0,0,FW_DONTCARE,FALSE,FALSE,FALSE,ANSI_CHARSET,OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY,DEFAULT_PITCH|FF_SWISS,TEXT("Courier New"));
  SendMessage(window_softswitch->hwnd_edit_data,WM_SETFONT,(WPARAM)window_softswitch->hFont,MAKELPARAM(FALSE,0));  

  /* Retaille la fentre */
  ResizeSoftswitchWindow(window_softswitch);

  /** Construction du Texte + Rafraichissement de la fentre **/
  RefreshSoftswitchWindowData(window_softswitch);
    
  /* OK */
  return(0);
}

/*************************************************************************************************/
/*  subEditSoftswitchProcControl() :  Rcupre les vnements qui arrivent dans le Control Edit. */
/*************************************************************************************************/
LRESULT CALLBACK subEditSoftswitchProcControl(HWND hwnd_control, UINT msg, WPARAM wParam, LPARAM lParam)
{
  int index, mouse_x, mouse_y;
  struct window_softswitch *window_softswitch;
  my_Memory(MEMORY_GET_WINDOW_SOFTSWITCH,&window_softswitch,NULL);

  /* Protection */
  if(window_softswitch == NULL)
    return(0);

  /** Identifie le Control **/
  /* Src */
  if(window_softswitch->hwnd_edit_data == hwnd_control)
    index = 1;
  else
    return(0);      /* Pas trouv */

  /** Dtecte l'vnement de la souris dans l'Edit **/
  if(msg == WM_MOUSEMOVE)
    {
      /* Corrdonnes de la souris dans l'Edit */
      mouse_x = GET_X_LPARAM(lParam) / 7;
      mouse_y = GET_Y_LPARAM(lParam) / SOFTSWITCH_EDIT_FONT_HEIGHT;
      UpdateSoftswitchContextInformation(window_softswitch,mouse_x,mouse_y);
      
      /* Transmet l'vnement au Control Text */
      return(CallWindowProc(window_softswitch->oldEditProcSoftswitchData,hwnd_control,msg,wParam,lParam)); 
    }
  else if(index == 1)
    return(CallWindowProc(window_softswitch->oldEditProcSoftswitchData,hwnd_control,msg,wParam,lParam)); 
         
  /* OK */
  return(0);
} 


/*******************************************************************************************************/
/*  UpdateSoftswitchContextInformation() :  MAJ des information de contexte de la fenetre Softswitch. */
/*******************************************************************************************************/
static void UpdateSoftswitchContextInformation(struct window_softswitch *window_softswitch, int grid_x, int grid_y)
{
  int grid_softswitch_x, softswitch_data_address, grid_softswitch_offset, softswitch_data_offset;
  unsigned char softswitch_data;
  
  /* Rien  faire */
  if(window_softswitch == NULL)
    return;
    
  /* En dehors de la zone */
  if(grid_x < 10)
    {
      SetWindowText(GetDlgItem(window_softswitch->hwnd_softswitch,IDC_STATIC_CONTEXT),"");
      return;
    }
  grid_softswitch_x = grid_x - 10;     /* Enlve les 10 caractres de gauche */
  if(((grid_softswitch_x+1) % 3) == 0)
    {
      SetWindowText(GetDlgItem(window_softswitch->hwnd_softswitch,IDC_STATIC_CONTEXT),"");
      return;
    }
  grid_softswitch_offset = (grid_softswitch_x+1) / 3;
  
  /* Rcupre l'adresse de la cellule */
  softswitch_data_address = window_softswitch->softswitch_address + grid_y*16;
  softswitch_data_address += grid_softswitch_offset;
  
  /* Rcupre la valeur de la cellule */
  softswitch_data_offset = softswitch_data_address - window_softswitch->softswitch_address;
  if(softswitch_data_offset > 255)
    {
      SetWindowText(GetDlgItem(window_softswitch->hwnd_softswitch,IDC_STATIC_CONTEXT),"");
      return;
    }
  softswitch_data = window_softswitch->softswitch_data[softswitch_data_offset];
  
  /* Coordonnes de la souris en coodonnes grid_x,grid_y */
  sprintf(window_softswitch->buffer_log,"Softswitch Data  %02X/%04X : %02X",softswitch_data_address>>16,softswitch_data_address&0xFFFF,softswitch_data);
  SetWindowText(GetDlgItem(window_softswitch->hwnd_softswitch,IDC_STATIC_CONTEXT),window_softswitch->buffer_log);
}


/************************************************************************************************/
/*  UpdateSoftswitchRefreshMode() :  MAJ du mode de rafraichissement de la fenetre Softswitch. */
/************************************************************************************************/
static void UpdateSoftswitchRefreshMode(struct window_softswitch *window_softswitch, int refresh_mode)
{
  struct notification *current_notification;

  if(refresh_mode == SOFTSWITCH_REFRESH_AUTOMATIC)
    {
      /* Visuel */
      window_softswitch->refresh_mode = SOFTSWITCH_REFRESH_AUTOMATIC;
      SendDlgItemMessage(window_softswitch->hwnd_softswitch,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_softswitch->hwnd_softswitch,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,0,0);

      /* Cration d'une Notification */
      current_notification = (struct notification *) calloc(1,sizeof(struct notification));
      if(current_notification == NULL)
        return;
      current_notification->hwnd_window = window_softswitch->hwnd_softswitch;
      current_notification->message = HM_SOFTSWITCH_REFRESHWINDOW;
      current_notification->type = NOTIFICATION_TYPE_VIEW;
      current_notification->sub_type = NOTIFICATION_SUBTYPE_SOFTSWITCH;

      /* Ajout de la notification */
      window_softswitch->current_notification = current_notification;
      my_Notification(NOTIFICATION_ADD,current_notification,NULL);
    }
  else
    {
      /* Visuel */
      window_softswitch->refresh_mode = SOFTSWITCH_REFRESH_MANUAL;
      SendDlgItemMessage(window_softswitch->hwnd_softswitch,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_softswitch->hwnd_softswitch,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,1,0);

      /* Supprime la Notification */
      my_Notification(NOTIFICATION_REMOVE,window_softswitch->current_notification,NULL);
      window_softswitch->current_notification = NULL;
    }
}


/******************************************************************************/
/*  RefreshSoftswitchWindowData() :  MAJ du contenu de la fentre Softswitch. */
/******************************************************************************/
static void RefreshSoftswitchWindowData(struct window_softswitch *window_softswitch)
{
  int softswitch_address, is_modified;

  /* Rcupre la valeur du Softswitch Pointer */
  softswitch_address = 0x00C000;
    
  /** Les donnes ont t'elles t modifies + Rcupration des nouvelles Data si besoin **/
  is_modified = IsSoftswitchDataModified(softswitch_address,window_softswitch);
  if(is_modified == 0)
    return;
    
  /** Cration de la Zone Rich Edit (RTF)  partir des nouvelles donnes **/
  BuildSoftswitchFromData(window_softswitch);

  /* Direct Page Pointer */
  sprintf(window_softswitch->buffer_log,"%02X/%04X",softswitch_address>>16,softswitch_address&0xFFFF);
  SetWindowText(GetDlgItem(window_softswitch->hwnd_softswitch,IDC_STATIC_SOFTSWITCHPOINTER),window_softswitch->buffer_log);

  /* Rafraichit la fentre */
  UpdateWindow(window_softswitch->hwnd_softswitch);
}


/*************************************************************************************************************/
/*  IsSoftswitchDataModified() :  Les donnes Softswitch ont t'elles t modifies depuis la dernirer fois. */
/*************************************************************************************************************/
static int IsSoftswitchDataModified(int softswitch_address, struct window_softswitch *window_softswitch)
{
  unsigned char softswitch_data[256];
  struct mmu *current_mmu;

  /* Rcupre le MMU */
  my_Memory(MEMORY_GET_S65816_MMU,&current_mmu,NULL);
  
  /* Rcupre les donnes */
  ReadMemoryBlock(softswitch_address,256,&softswitch_data[0],current_mmu);
  
  /* Compare les donnes */
  if(softswitch_address == window_softswitch->softswitch_address &&
     memcmp(&softswitch_data[0],&window_softswitch->softswitch_data[0],256) == 0)
    return(0);
    
  /* Les donnes sont diffrentes */
  window_softswitch->softswitch_address = softswitch_address;
  memcpy(&window_softswitch->softswitch_data[0],&softswitch_data[0],256);
  return(1);
}


/***********************************************************************/
/*  ResizeSoftswitchWindow() : Resize Softswitch window by given size. */
/***********************************************************************/
static void ResizeSoftswitchWindow(struct window_softswitch *window_softswitch)
{
  RECT window_rect;
  int window_x, window_y;
  
  /* Largeur de la fentre */
  window_softswitch->window_width = window_softswitch->group_data_x + window_softswitch->group_data_width + WINDOW_CLIENT_RIGHT_BORDER_SIZE + WINDOW_BORDER_SIZE;

  /* Hauteur de la fentre */
  window_softswitch->window_height = window_softswitch->title_height + window_softswitch->group_data_y + window_softswitch->group_data_height + WINDOW_CLIENT_BOTTOM_BORDER_SIZE;

  /* Position actuelle de la fentre dans l'cran */
  GetWindowRect(window_softswitch->hwnd_softswitch,&window_rect);
  window_x = window_rect.left;
  window_y = window_rect.top;

  /* Resize window */
  MoveWindow(window_softswitch->hwnd_softswitch,window_x,window_y,window_softswitch->window_width,window_softswitch->window_height,TRUE);
}


/************************************************************************************************************/
/*  BuildSoftswitchFromData() :  Construire les donnes hxadcimal  afficher  partir des donnes brutes. */
/************************************************************************************************************/
static void BuildSoftswitchFromData(struct window_softswitch *window_softswitch)
{
  int i, j, offset, offset_text, nb_byte_to_process;

  /*** Transformer les donnes BYTE en Hexa ***/
  nb_byte_to_process = window_softswitch->nb_column*window_softswitch->nb_line;
  for(i=0,offset=0,offset_text=0; i<window_softswitch->nb_line; i++,offset+=16)
    {
      /* Adresse */
      sprintf(&window_softswitch->text_data_hexadecimal[offset_text],"%02X/%04X   ",(offset+window_softswitch->softswitch_address)>>16,(offset+window_softswitch->softswitch_address) & 0xFFFF);
      offset_text += (int) strlen("00/0000   ");

      /* Hexa */
      for(j=0; j<window_softswitch->nb_column; j++)
        {
          if(i*window_softswitch->nb_column+j < nb_byte_to_process)
            sprintf(&window_softswitch->text_data_hexadecimal[offset_text],"%02X",window_softswitch->softswitch_data[offset+j]);
          else
            strcpy(&window_softswitch->text_data_hexadecimal[offset_text],"  ");
          offset_text += 2;

          /* Sparateur */
          if(j < (window_softswitch->nb_column-1))
            {
              window_softswitch->text_data_hexadecimal[offset_text] = (j==7 && (i*window_softswitch->nb_column+j < nb_byte_to_process)) ? '.' : ' ';
              offset_text++;
            }
        }

      /* Fin de ligne */
      strcpy(&window_softswitch->text_data_hexadecimal[offset_text],"\r\n");
      offset_text += 2;
    }
    
  /** Efface le contenu de la fentre **/
  SetWindowText(window_softswitch->hwnd_edit_data,"");

  /** Afficher le rsultat dans le control **/
  SetWindowText(window_softswitch->hwnd_edit_data,window_softswitch->text_data_hexadecimal);
}


/************************************************************************/
/*  DeleteSoftswitchChildWindows() :  Destruction des fentres enfants. */
/************************************************************************/
static void DeleteSoftswitchChildWindows(struct window_softswitch *window_softswitch)
{
  /** Libration des ressources **/  
  if(window_softswitch->text_data_hexadecimal != NULL)
    free(window_softswitch->text_data_hexadecimal);
  window_softswitch->text_data_hexadecimal = NULL;
}


/**************************************************************************/
/*  mem_free_window_softswitch() :  Libration mmoire pour la structure. */
/**************************************************************************/
void mem_free_window_softswitch(struct window_softswitch *window_softswitch)
{
  if(window_softswitch)
    {
      /* Supprime la notification */
      my_Notification(NOTIFICATION_REMOVE,window_softswitch->current_notification,NULL);
      window_softswitch->current_notification = NULL;
      
      free(window_softswitch);
    }
}

/**********************************************************************/
