/**********************************************************************/
/*                                                                    */
/*   Window_Stack.c :  Module de gestion de la fentre Stack.         */
/*                                                                    */
/**********************************************************************/
/*  Auteur : Olivier ZARDINI  *   Brutal Deluxe   *  Date : Mai 2018  */
/**********************************************************************/

#include <stdio.h>
#include <time.h>

#include <windows.h>
#include <windowsx.h>
#include <Commctrl.h>
#include <Winuser.h>
#include <Richedit.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_Notification.h"
#include "Dc_Graphic.h"
#include "Dc_CPU.h"
#include "Dc_MMU.h"
#include "Window_QuickWatch.h"
#include "Window_Main.h"
#include "Window_Main_Menu.h"
#include "Window_Stack.h"

static DWORD BuildStackWindowThread(void *);
LRESULT CALLBACK ProcHandlerStackWindow(HWND,UINT,WPARAM,LPARAM);
static int CreateStackChildWindows(HWND,struct window_stack *);
static void UpdateStackContextInformation(struct window_stack *,int,int);
static void UpdateStackRefreshMode(struct window_stack *,int);
static void RefreshStackWindowData(struct window_stack *);
static int IsStackDataModified(int,int,struct window_stack *);
static void ResizeStackWindow(struct window_stack *);
static void BuildStackFromData(struct window_stack *);
static DWORD CALLBACK StackRichEditStreamCallback(DWORD *,LPBYTE,LONG,LONG *);
static void DeleteStackChildWindows(struct window_stack *);


/************************************************************************/
/*  BuildStackWindow() :  Construction / Affichage de la fentre Stack. */
/************************************************************************/
struct window_stack *BuildStackWindow(HINSTANCE hInstance, int menu_id)
{
  struct window_stack *window_stack;

  /** Allocation de la structure window_stack **/
  window_stack = (struct window_stack *) calloc(1,sizeof(struct window_stack));
  if(window_stack == NULL)
    return(NULL);
  my_Memory(MEMORY_SET_WINDOW_STACK,window_stack,NULL);

  /* Lance un thread pour grer la fentre Stack */
  CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)BuildStackWindowThread,window_stack,0,NULL);

  /* Renvoi la structure mmoire grant la fentre */
  return(window_stack);
}


/**********************************************************************************/
/*  BuildStackWindowThread() :  Cration de la fentre Stack via un Thread ddi. */
/**********************************************************************************/
static DWORD BuildStackWindowThread(void *window_stack_param)
{
  HWND hwndStack;
  MSG msg;
  BOOL bRet;
  struct window_main *window_main;
  struct window_stack *window_stack;
  struct parameter *param;

  /* Init */
  window_stack = (struct window_stack *) window_stack_param;
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Cration de la fentre Stack **/
  hwndStack = CreateDialog(param->hInstance,MAKEINTRESOURCE(IDD_WINDOW_STACK),window_main->hwnd_main,(DLGPROC)ProcHandlerStackWindow);
  if(hwndStack == NULL)
    return(1);

  /** Show Window + Paint Content **/
  ShowWindow(hwndStack,SW_SHOWDEFAULT);
  UpdateWindow(hwndStack);

  /** Message Loop pour toutes les fentres de ce Thread (fentre Stack...) **/
  while((bRet = GetMessage(&msg,NULL,0,0)) != 0)
    {
      if(bRet == -1)
        break;

      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }

  /* Fin du Thread */
  ExitThread(0);
}


/**********************************************************************/
/*  ProcHandlerStackWindow() :  Gestion des vnements de la fentre. */
/**********************************************************************/
LRESULT CALLBACK ProcHandlerStackWindow(HWND hwndStack, UINT umsg, WPARAM wParam, LPARAM lParam)
{
  int mouse_x, mouse_y;
  MSGFILTER *message_filter;
  struct window_stack *window_stack;
  struct parameter *param;
  
  /* Init */
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  my_Memory(MEMORY_GET_WINDOW_STACK,&window_stack,NULL);

  switch(umsg)
    {
      case WM_INITDIALOG :
        /* Cration du contenu de la DialogBox */
        CreateStackChildWindows(hwndStack,window_stack);
        return(TRUE);

      /**********************************/
      /* Rafrachissement de la fentre */
      /**********************************/
      case WM_PAINT :
        break;

      /***************************/
      /* Evnements de la souris */
      /***************************/
      case WM_MOUSEMOVE :
        /* Indique les informations contextuelles aux coordonnes X,Y de la Souris */
        mouse_x = GET_X_LPARAM(lParam);
        mouse_y = GET_Y_LPARAM(lParam);
        SetWindowText(GetDlgItem(window_stack->hwnd_stack,IDC_STATIC_CONTEXT),"");
        return(TRUE);
        
      /****************************/
      /* Evnements de la fentre */
      /****************************/
      case WM_COMMAND :
        switch(LOWORD(wParam))
          {
            /** Bouton Radio : On change la mthode de rafraichissement de la fentre **/
            case IDC_RADIO_REFRESH_AUTOMATIC :
              if(HIWORD(wParam) == BN_CLICKED && window_stack->refresh_mode != STACK_REFRESH_AUTOMATIC)
                UpdateStackRefreshMode(window_stack,STACK_REFRESH_AUTOMATIC);
              break;

            case IDC_RADIO_REFRESH_MANUAL :
              if(HIWORD(wParam) == BN_CLICKED && window_stack->refresh_mode != STACK_REFRESH_MANUAL)
                UpdateStackRefreshMode(window_stack,STACK_REFRESH_MANUAL);
              break;

            /** Bouton de rafraichissement Manuel de la fentre **/
            case IDC_BUTTON_REFRESH :
              RefreshStackWindowData(window_stack);
              break;
          }
        return(TRUE);

      /**********************************************/
      /* Evnements des Controles  la fentre mre */
      /**********************************************/
      case WM_NOTIFY :
        message_filter = (MSGFILTER *) lParam;
        
        /** Evnement de la souris dans le Rich Edit transmis  la fentre mre **/
        if(message_filter && message_filter->nmhdr.hwndFrom == window_stack->hwnd_richedit_data)
          {
            mouse_x = GET_X_LPARAM(message_filter->lParam) / 7;
            mouse_y = GET_Y_LPARAM(message_filter->lParam) / STACK_RICHEDIT_FONT_HEIGHT;
            UpdateStackContextInformation(window_stack,mouse_x,mouse_y);
          }
        return(FALSE);

      /**************************************************************/
      /* Message extrieur demandant le rafraissiment de la fentre */
      /**************************************************************/
      case HM_STACK_REFRESHWINDOW :
        RefreshStackWindowData(window_stack);
        return(TRUE);

      /********************************************************************/
      /* Fermeture de window (le WM_CLOSE est appel avant le WM_DESTROY) */
      /********************************************************************/
      case WM_CLOSE :
        /* Fentre Stack */
        DestroyWindow(hwndStack);
        return(0);

      case WM_DESTROY :
        /* Suppression des Fentres */
        DeleteStackChildWindows(window_stack);
        CheckMainWindowMenuItem(ID_MENU_VIEW_STACK,FALSE);
        my_Memory(MEMORY_FREE_WINDOW_STACK,NULL,NULL);
        /* Demande la sortie de la boucle des messages */
        PostQuitMessage(0);
        return(0);
    }

  /* Retourner FALSE pour relayer le message non trait au systme */
  return(FALSE);
}


/*************************************************************************************************/
/*  CreateStackChildWindows() : Crer les fentres enfants et remplir la structure window_stack. */
/*************************************************************************************************/
static int CreateStackChildWindows(HWND hwndStack, struct window_stack *window_stack)
{
  HWND hwnd_group;
  DWORD dialog_background_color;
  RECT client_rect, window_rect, group_rect;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Allocation Buffer mmoire Zone Texte : 256 Bytes de raw data **/
  window_stack->nb_line = 16; 
  window_stack->nb_column = 16;
  /* Nombre caractres visibles sur la ligne (on ne compte pas le \r\n final) */
  window_stack->text_data_width = (int) (strlen("00/0000   ") + window_stack->nb_column*3 - 1);   /* Enlve le dernier sparateur */
  /* Nombre d'octet pour encoder la zone Texte (on compte les \r\n en fin de ligne) */
  window_stack->rtf_data_size = 4096 + window_stack->nb_line*(window_stack->text_data_width+2);
  window_stack->rtf_data_hexadecimal = (char *) calloc(window_stack->rtf_data_size+1,sizeof(char));
  if(window_stack->rtf_data_hexadecimal == NULL)
    return(1);

  /** Window Handle **/
  window_stack->hwnd_stack = hwndStack;

  /* Force la MAJ du contenu de la fentre */
  window_stack->stack_address = -1;

  /* Refresh Mode : Automatic */
  UpdateStackRefreshMode(window_stack,STACK_REFRESH_AUTOMATIC);

  /* Hauteur Titre */
  GetWindowRect(window_stack->hwnd_stack,&window_rect);
  GetClientRect(window_stack->hwnd_stack,&client_rect);
  window_stack->title_height = (window_rect.bottom - window_rect.top) - client_rect.bottom;

  /* Position de la GroupBox Select + GroupBox Palette */
  hwnd_group = GetDlgItem(window_stack->hwnd_stack,IDC_GROUP_STACK);     /* Rcupre le Handle du Control GroupBox li  IDC_GROUP_STACK */
  GetWindowRect(hwnd_group,&group_rect);
  window_stack->group_data_x = group_rect.left - window_rect.left;
  window_stack->group_data_y = (group_rect.top - window_rect.top) - window_stack->title_height;

  /* Position de la Zone Text Stack Data dans la fenetre */
  window_stack->stack_data_x = window_stack->group_data_x + WINDOW_GROUP_BORDER_LEFT_SIZE;
  window_stack->stack_data_y = window_stack->group_data_y + WINDOW_GROUP_BORDER_TOP_SIZE;

  /* Dimension en Pixel du Control Text Stack Data */
  window_stack->stack_data_width = window_stack->text_data_width*7 + 2;
  window_stack->stack_data_height = window_stack->nb_line*STACK_RICHEDIT_FONT_HEIGHT;   

  /* Dimension en Pixel du Control Group Box + Resize */  
  window_stack->group_data_width = WINDOW_GROUP_BORDER_LEFT_SIZE + window_stack->stack_data_width + WINDOW_GROUP_BORDER_RIGHT_SIZE;
  window_stack->group_data_height = WINDOW_GROUP_BORDER_TOP_SIZE + window_stack->stack_data_height + WINDOW_GROUP_BORDER_BOTTOM_SIZE;
  MoveWindow(hwnd_group,window_stack->group_data_x,window_stack->group_data_y,window_stack->group_data_width,window_stack->group_data_height,TRUE);

  /** Cration du Rich Edit **/
  window_stack->hwnd_richedit_data = CreateWindowEx(0,RICHEDIT_CLASS,"",WS_CHILD | WS_VISIBLE | ES_SAVESEL | ES_MULTILINE | ES_READONLY, 
                                                    window_stack->stack_data_x, window_stack->stack_data_y, window_stack->stack_data_width, window_stack->stack_data_height,   // set size in WM_SIZE message 
                                                    window_stack->hwnd_stack,
                                                    (HMENU)IDC_RICHEDIT_STACK_DATA,
                                                    param->hInstance,
                                                    NULL);
  
  /* Couleur de Fond du control Rich Edit : Gris comme la fentre d'accueil */
  dialog_background_color = GetSysColor(COLOR_3DFACE);
  SendMessage(window_stack->hwnd_richedit_data,EM_SETBKGNDCOLOR,0,(LPARAM)dialog_background_color); 
  SendMessage(window_stack->hwnd_richedit_data,EM_SETEVENTMASK,0,(LPARAM)ENM_MOUSEEVENTS);

  /* Retaille la fentre */
  ResizeStackWindow(window_stack);

  /** Construction du Texte + Rafraichissement de la fentre **/
  RefreshStackWindowData(window_stack);
    
  /* OK */
  return(0);
}


/*********************************************************************************************/
/*  UpdateStackContextInformation() :  MAJ des information de contexte de la fenetre Stack. */
/*********************************************************************************************/
static void UpdateStackContextInformation(struct window_stack *window_stack, int grid_x, int grid_y)
{
  int grid_stack_x, stack_data_address, grid_stack_offset, stack_data_offset;
  unsigned char stack_data;
  
  /* Rien  faire */
  if(window_stack == NULL)
    return;
    
  /* En dehors de la zone */
  if(grid_x < 10)
    {
      SetWindowText(GetDlgItem(window_stack->hwnd_stack,IDC_STATIC_CONTEXT),"");
      return;
    }
  grid_stack_x = grid_x - 10;     /* Enlve les 10 caractres de gauche */
  if(((grid_stack_x+1) % 3) == 0)
    {
      SetWindowText(GetDlgItem(window_stack->hwnd_stack,IDC_STATIC_CONTEXT),"");
      return;
    }
  grid_stack_offset = (grid_stack_x+1) / 3;
  
  /* Rcupre l'adresse de la cellule */
  stack_data_address = window_stack->stack_data_base_address + grid_y*16;
  stack_data_address += grid_stack_offset;
  
  /* Rcupre la valeur de la cellule */
  stack_data_offset = stack_data_address - window_stack->stack_data_base_address;
  if(stack_data_offset > 255)
    {
      SetWindowText(GetDlgItem(window_stack->hwnd_stack,IDC_STATIC_CONTEXT),"");
      return;
    }
  stack_data = window_stack->stack_data[stack_data_offset];
  
  /* Coordonnes de la souris en coodonnes grid_x,grid_y */
  sprintf(window_stack->buffer_log,"Stack Data  %02X/%04X : %02X",stack_data_address>>16,stack_data_address&0xFFFF,stack_data);
  SetWindowText(GetDlgItem(window_stack->hwnd_stack,IDC_STATIC_CONTEXT),window_stack->buffer_log);
}


/**************************************************************************************/
/*  UpdateStackRefreshMode() :  MAJ du mode de rafraichissement de la fenetre Stack. */
/**************************************************************************************/
static void UpdateStackRefreshMode(struct window_stack *window_stack, int refresh_mode)
{
  struct notification *current_notification;

  if(refresh_mode == STACK_REFRESH_AUTOMATIC)
    {
      /* Visuel */
      window_stack->refresh_mode = STACK_REFRESH_AUTOMATIC;
      SendDlgItemMessage(window_stack->hwnd_stack,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_stack->hwnd_stack,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,0,0);

      /* Cration d'une Notification */
      current_notification = (struct notification *) calloc(1,sizeof(struct notification));
      if(current_notification == NULL)
        return;
      current_notification->hwnd_window = window_stack->hwnd_stack;
      current_notification->message = HM_STACK_REFRESHWINDOW;
      current_notification->type = NOTIFICATION_TYPE_VIEW;
      current_notification->sub_type = NOTIFICATION_SUBTYPE_STACK;

      /* Ajout de la notification */
      window_stack->current_notification = current_notification;
      my_Notification(NOTIFICATION_ADD,current_notification,NULL);
    }
  else
    {
      /* Visuel */
      window_stack->refresh_mode = STACK_REFRESH_MANUAL;
      SendDlgItemMessage(window_stack->hwnd_stack,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_stack->hwnd_stack,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,1,0);

      /* Supprime la Notification */
      my_Notification(NOTIFICATION_REMOVE,window_stack->current_notification,NULL);
      window_stack->current_notification = NULL;
    }
}


/********************************************************************/
/*  RefreshStackWindowData() :  MAJ du contenu de la fentre Stack. */
/********************************************************************/
static void RefreshStackWindowData(struct window_stack *window_stack)
{
  int stack_address, stack_data_base_address, is_modified;
  struct cpu *current_cpu;

  /* Rcupre le CPU */
  my_Memory(MEMORY_GET_S65816_CPU,&current_cpu,NULL);

  /* Rcupre la valeur du Stack Pointer */
  stack_address = GetCPUData(CPU_ADDRESS_STACK,current_cpu);
  if((stack_address & 0x0000F0) <= 0x000070)
    stack_data_base_address = (stack_address & 0xFFFF00);
  else
    stack_data_base_address = (stack_address & 0xFFFF00) + 0x80;
    
  /** Les donnes ont t'elles t modifies + Rcupration des nouvelles Data si besoin **/
  is_modified = IsStackDataModified(stack_address,stack_data_base_address,window_stack);
  if(is_modified == 0)
    return;
    
  /** Cration de la Zone Rich Edit (RTF)  partir des nouvelles donnes **/
  BuildStackFromData(window_stack);

  /* Stack Pointer */
  sprintf(window_stack->buffer_log,"%02X/%04X",stack_address>>16,stack_address&0xFFFF);
  SetWindowText(GetDlgItem(window_stack->hwnd_stack,IDC_STATIC_STACKPOINTER),window_stack->buffer_log);

  /* Rafraichit la fentre */
  UpdateWindow(window_stack->hwnd_stack);
}


/***************************************************************************************************/
/*  IsStackDataModified() :  Les donnes Stack ont t'elles t modifies depuis la dernirer fois. */
/***************************************************************************************************/
static int IsStackDataModified(int stack_address, int stack_data_base_address, struct window_stack *window_stack)
{
  unsigned char stack_data[256];
  struct mmu *current_mmu;

  /* Rcupre le MMU */
  my_Memory(MEMORY_GET_S65816_MMU,&current_mmu,NULL);
  
  /* Rcupre les donnes */
  ReadMemoryBlock(stack_data_base_address,256,&stack_data[0],current_mmu);
  
  /* Compare les donnes */
  if(stack_address == window_stack->stack_address &&
     stack_data_base_address == window_stack->stack_data_base_address &&
     memcmp(&stack_data[0],&window_stack->stack_data[0],256) == 0)
    return(0);
    
  /* Les donnes sont diffrentes */
  window_stack->stack_address = stack_address;
  window_stack->stack_data_base_address = stack_data_base_address;
  memcpy(&window_stack->stack_data[0],&stack_data[0],256);
  return(1);
}


/*************************************************************/
/*  ResizeStackWindow() : Resize Stack window by given size. */
/*************************************************************/
static void ResizeStackWindow(struct window_stack *window_stack)
{
  RECT window_rect;
  int window_x, window_y;
  
  /* Largeur de la fentre */
  window_stack->window_width = window_stack->group_data_x + window_stack->group_data_width + WINDOW_CLIENT_RIGHT_BORDER_SIZE + WINDOW_BORDER_SIZE;

  /* Hauteur de la fentre */
  window_stack->window_height = window_stack->title_height + window_stack->group_data_y + window_stack->group_data_height + WINDOW_CLIENT_BOTTOM_BORDER_SIZE;

  /* Position actuelle de la fentre dans l'cran */
  GetWindowRect(window_stack->hwnd_stack,&window_rect);
  window_x = window_rect.left;
  window_y = window_rect.top;

  /* Resize window */
  MoveWindow(window_stack->hwnd_stack,window_x,window_y,window_stack->window_width,window_stack->window_height,TRUE);
}


/*******************************************************************************************************/
/*  BuildStackFromData() :  Construire les donnes hxadcimal  afficher  partir des donnes brutes. */
/*******************************************************************************************************/
static void BuildStackFromData(struct window_stack *window_stack)
{
  int i, j, offset, offset_rtf, nb_byte_to_process;
  char buffer[BUFFER_SIZE];
  EDITSTREAM edit_stream;

  /** Cration d'un buffer RTF rpentant les donnes **/
  window_stack->rtf_data_hexadecimal[0] = '\0';
  strcat(window_stack->rtf_data_hexadecimal,"{\\rtf1\\ansi\\ansicpg1252\\deff0\\deflang1036{\\fonttbl{\\f0\\fnil\\fcharset0 Courier New;}}\r\n");
  strcat(window_stack->rtf_data_hexadecimal,"{\\colortbl ;\\red51\\green153\\blue255;}\r\n");
  strcat(window_stack->rtf_data_hexadecimal,"{\\*\\generator Msftedit 5.41.21.2510;}\\viewkind4\\uc1\\pard\\sl240\\slmult1\\lang12\\f0\\fs");
  strcat(window_stack->rtf_data_hexadecimal,STACK_RICHEDIT_FONT_HEIGHT_RTF);  /* Taille de la Police : Si 8 dans WordPad => 16 ici (x2)*/
  strcat(window_stack->rtf_data_hexadecimal," ");
  offset_rtf = (int) strlen(window_stack->rtf_data_hexadecimal);

  /*** Transformer les donnes BYTE en Hexa Ascii ***/
  nb_byte_to_process = window_stack->nb_column*window_stack->nb_line;
  for(i=0,offset=0; i<window_stack->nb_line; i++,offset+=16)
    {
      /* Dbut de ligne : Adresse */
      sprintf(&window_stack->rtf_data_hexadecimal[offset_rtf],"%02X/%04X   ",(offset+window_stack->stack_data_base_address)>>16,(offset+window_stack->stack_data_base_address) & 0xFFFF);
      offset_rtf += (int) strlen("00/0000   ");

      /** Encode les Byte en Hexa **/
      for(j=0; j<window_stack->nb_column; j++)
        {
          if(window_stack->stack_address == (window_stack->stack_data_base_address+offset+j))
            {
              /* Passe ce Byte en Rouge */
              sprintf(buffer,"\\highlight1 %02X\\highlight0 ",window_stack->stack_data[offset+j]);
              strcpy(&window_stack->rtf_data_hexadecimal[offset_rtf],buffer);
              offset_rtf += (int) strlen(buffer);
            }
          else if(i*window_stack->nb_column+j < nb_byte_to_process)
            {
              /* Byte en noir */
              sprintf(&window_stack->rtf_data_hexadecimal[offset_rtf],"%02X",window_stack->stack_data[offset+j]);
              offset_rtf += 2;
            }
          else
            {
              /* Aucune donne  afficher */
              strcpy(&window_stack->rtf_data_hexadecimal[offset_rtf],"  ");
              offset_rtf += 2;
            }

          /* Sparateur */
          if(j < (window_stack->nb_column-1))
            {
              window_stack->rtf_data_hexadecimal[offset_rtf] = (j==7 && (i*window_stack->nb_column+j < nb_byte_to_process)) ? '.' : ' ';
              offset_rtf++;
            }
        }

      /* Fin de ligne */
      strcpy(&window_stack->rtf_data_hexadecimal[offset_rtf],"\\par\r\n");
      offset_rtf += (int) strlen("\\par\r\n");
    }

  /** Fin du RTF **/
  strcpy(&window_stack->rtf_data_hexadecimal[offset_rtf],"}\r\n");
  offset_rtf += (int) strlen("}\r\n");
  window_stack->rtf_data_hexadecimal[offset_rtf] = '\0';
  
  /* Taille du RTF (le \0 final compte) */
  window_stack->rtf_data_size = (int) strlen(window_stack->rtf_data_hexadecimal) + 1;

  /** Afficher le rsultat en RTF dans le control Rich Edit **/
  window_stack->rtf_write_offset = 0;
  memset(&edit_stream,0,sizeof(EDITSTREAM));
  edit_stream.pfnCallback = (EDITSTREAMCALLBACK) StackRichEditStreamCallback;
  edit_stream.dwCookie = (DWORD_PTR) window_stack;          /* Paramtre  passer  la fonction Callback */
  SendMessage(window_stack->hwnd_richedit_data,EM_STREAMIN,SF_RTF,(LPARAM)&edit_stream);
}


/***********************************************************************************/
/*  StackRichEditStreamCallback() :  Fonction Call back pour remplir le Rich Edit. */
/***********************************************************************************/
static DWORD CALLBACK StackRichEditStreamCallback(DWORD *dwCookie, LPBYTE output_buffer, LONG nb_write_max, LONG *nb_write_rtn)
{
  int nb_remain_byte, nb_copy_byte;
  struct window_stack *window_stack;

  /* Rcupration du paramtre */
  window_stack = (struct window_stack *) dwCookie;

  /* Taille des donnes encore  copier */
  nb_remain_byte = window_stack->rtf_data_size - window_stack->rtf_write_offset;
  if(nb_remain_byte == 0)
    {
      /* Plus rien  copier */
      *nb_write_rtn = 0;
      return(0);
    }

  /* Taille des donnes que l'on est autoris  copier */
  nb_copy_byte = (nb_write_max >= nb_remain_byte) ? nb_remain_byte : nb_write_max;

  /* Nombre de byte transfrs */
  *nb_write_rtn = nb_copy_byte;
		  
  /** Copie les Byte dans le output buffer **/
  memcpy(output_buffer,&window_stack->rtf_data_hexadecimal[window_stack->rtf_write_offset],nb_copy_byte);

  /* Dplace l'offset */
  window_stack->rtf_write_offset += nb_copy_byte;

  /* OK */
  return(0);
}


/*******************************************************************/
/*  DeleteStackChildWindows() :  Destruction des fentres enfants. */
/*******************************************************************/
static void DeleteStackChildWindows(struct window_stack *window_stack)
{
  /** Libration des ressources **/  
  if(window_stack->rtf_data_hexadecimal != NULL)
    free(window_stack->rtf_data_hexadecimal);
  window_stack->rtf_data_hexadecimal = NULL;
}


/*********************************************************************/
/*  mem_free_window_stack() :  Libration mmoire pour la structure. */
/*********************************************************************/
void mem_free_window_stack(struct window_stack *window_stack)
{
  if(window_stack)
    {
      /* Supprime la notification */
      my_Notification(NOTIFICATION_REMOVE,window_stack->current_notification,NULL);
      window_stack->current_notification = NULL;
      
      free(window_stack);
    }
}

/**********************************************************************/
