/***********************************************************************/
/*                                                                     */
/*  Window_Statistic.c :  Module de gestion de la fentre Statistic.   */
/*                                                                     */
/***********************************************************************/
/*  Auteur : Olivier ZARDINI  *   Brutal Deluxe   *   Date : May 2024  */
/***********************************************************************/

#include <stdio.h>
#include <time.h>

#include <windows.h>
#include <windowsx.h>
#include <Commctrl.h>
#include <Winuser.h>
#include <Richedit.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_Notification.h"
#include "Dc_Graphic.h"
#include "Dc_CPU.h"
#include "Dc_MMU.h"
#include "Dc_IIgsInternals.h"
#include "Dc_Disassembler.h"
#include "Dc_Name.h"
#include "Cyrene_Engine.h"
#include "Window_ProgressBar.h"
#include "Window_QuickWatch.h"
#include "Window_Main.h"
#include "Window_Main_Menu.h"
#include "Window_Disassembler.h"
#include "Window_MemoryArea.h"
#include "Window_Symbol.h"
#include "Window_Statistic.h"

static DWORD BuildStatisticWindowThread(void *);
LRESULT CALLBACK ProcHandlerStatisticWindow(HWND,UINT,WPARAM,LPARAM);
static int CreateStatisticChildWindows(HWND,struct window_statistic *);
static void UpdateStatisticRefreshMode(struct window_statistic *,int);
static void BuildStatisticListView(struct list_view *,HFONT,int,struct window_statistic *);
static void FillStatisticListView(struct list_view *,struct statistic_table *);
static void DeleteOneStatistic(DWORD,WORD,struct window_statistic *);
static void RefreshStatisticWindow(struct window_statistic *);
static void SaveFunctionTable(struct window_statistic *);
static void SaveFunctionHistory(struct window_statistic *);
static DWORD SaveFunctionHistoryThread(void *);
static struct statistic_function *GetStatisticByAddress(DWORD,WORD);
static void UserResizeStatisticWindow(struct window_statistic *);
static void MoveStatisticWindowControls(int,struct window_statistic *);
static void DeleteStatisticChildWindows(struct window_statistic *);
static int compare_statistic_address(const void *,const void *);


/********************************************************************************/
/*  BuildStatisticWindow() :  Construction / Affichage de la fentre Statistic. */
/********************************************************************************/
struct window_statistic *BuildStatisticWindow(HINSTANCE hInstance, int menu_id)
{
  struct window_statistic *window_statistic;

  /** Allocation de la structure window_statistic **/
  window_statistic = (struct window_statistic *) calloc(1,sizeof(struct window_statistic));
  if(window_statistic == NULL)
    return(NULL);
  my_Memory(MEMORY_SET_WINDOW_STATISTIC,window_statistic,NULL);

  /* Lance un thread pour grer la fentre Statistic */
  CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)BuildStatisticWindowThread,window_statistic,0,NULL);

  /* Renvoi la structure mmoire grant la fentre */
  return(window_statistic);
}


/******************************************************************************************/
/*  BuildStatisticWindowThread() :  Cration de la fentre Statistic via un Thread ddi. */
/******************************************************************************************/
static DWORD BuildStatisticWindowThread(void *window_statistic_param)
{
  HWND hwndStatistic;
  MSG msg;
  BOOL bRet;
  struct window_main *window_main;
  struct window_statistic *window_statistic;
  struct parameter *param;

  /* Init */
  window_statistic = (struct window_statistic *) window_statistic_param;
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Cration de la fentre Statistic **/
  hwndStatistic = CreateDialog(param->hInstance,MAKEINTRESOURCE(IDD_WINDOW_STATISTIC),window_main->hwnd_main,(DLGPROC)ProcHandlerStatisticWindow);
  if(hwndStatistic == NULL)
    return(1);

  /** Show Window + Paint Content **/
  ShowWindow(hwndStatistic,SW_SHOWDEFAULT);
  UpdateWindow(hwndStatistic);

  /** Message Loop pour toutes les fentres de ce Thread (fentre Statistic...) **/
  while((bRet = GetMessage(&msg,NULL,0,0)) != 0)
    {
      if(bRet == -1)
        break;

      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }

  /* Fin du Thread */
  ExitThread(0);
}


/**************************************************************************/
/*  ProcHandlerStatisticWindow() :  Gestion des vnements de la fentre. */
/**************************************************************************/
LRESULT CALLBACK ProcHandlerStatisticWindow(HWND hwndStatistic, UINT umsg, WPARAM wParam, LPARAM lParam)
{
  int i, is_error, select_id;
  UINT code, code_beginlabeledit, code_endlabeledit, code_getdispinfo, code_setdispinfo;
  DWORD statistic_address;
  BOOL result;
  NMLVDISPINFO *plvdi;
  LVITEM LvItem;
  NMLISTVIEW *pListView;
  char buffer_address[1024];
  struct list_view *current_listview;
  struct window_statistic *window_statistic;
  struct window_main *window_main;
  struct parameter *param;
  
  /* Init */
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  my_Memory(MEMORY_GET_WINDOW_STATISTIC,&window_statistic,NULL);
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);

  switch(umsg)
    {
      case WM_INITDIALOG :
        /* Cration du contenu de la DialogBox */
        CreateStatisticChildWindows(hwndStatistic,window_statistic);
        return(TRUE);
        
      /****************************/
      /* Evnements de la fentre */
      /****************************/
      case WM_COMMAND :
        switch(LOWORD(wParam))
          {
            /** Bouton Radio : On change la mthode de rafraichissement de la fentre **/
            case IDC_RADIO_REFRESH_AUTOMATIC :
              if(HIWORD(wParam) == BN_CLICKED && window_statistic->refresh_mode != STATISTIC_REFRESH_AUTOMATIC)
                UpdateStatisticRefreshMode(window_statistic,STATISTIC_REFRESH_AUTOMATIC);
              break;

            case IDC_RADIO_REFRESH_MANUAL :
              if(HIWORD(wParam) == BN_CLICKED && window_statistic->refresh_mode != STATISTIC_REFRESH_MANUAL)
                UpdateStatisticRefreshMode(window_statistic,STATISTIC_REFRESH_MANUAL);
              break;

            /** Bouton : Vide la table des Statistic **/
            case IDC_BUTTON_STATISTIC_CLEAR :
              if(HIWORD(wParam) == BN_CLICKED)
                ClearStatisticTable(window_statistic);
              break;

            /** Bouton : Supprime une Statistic **/
            case IDC_BUTTON_STATISTIC_DELETE :
              if(HIWORD(wParam) == BN_CLICKED)
                if(window_statistic->selected_statistic_address != 0xFFFFFFFF)
                  DeleteOneStatistic(window_statistic->selected_statistic_address,window_statistic->selected_statistic_call_number,window_statistic);
              break;

            /** Bouton de Sauvegarde la Table des Fonctions **/
            case IDC_BUTTON_HISTORY_SAVE :
              if(HIWORD(wParam) == BN_CLICKED)
                SaveFunctionHistory(window_statistic);
              break;

            /** Bouton de Sauvegarde la Table des Fonctions **/
            case IDC_BUTTON_SAVE_TABLE :
              if(HIWORD(wParam) == BN_CLICKED)
                SaveFunctionTable(window_statistic);
              break;

            /** Bouton de rafraichissement Manuel de la fentre **/
            case IDC_BUTTON_REFRESH :
              if(HIWORD(wParam) == BN_CLICKED)
                RefreshStatisticWindow(window_statistic);
              break; 
          }
        return(TRUE);

      /*****************************************************************/
      /* Message extrieur demandant le rafraichissement de la fentre */
      /*****************************************************************/
      case HM_STATISTIC_REFRESHWINDOW :
        RefreshStatisticWindow(window_statistic);
        return(TRUE);

      /********************************************/
      /* Evnements des CommonControls (ListView) */
      /********************************************/
      case WM_NOTIFY :
        code = ((LPNMHDR)lParam)->code;
        code_beginlabeledit = LVN_BEGINLABELEDIT;
        code_endlabeledit   = LVN_ENDLABELEDIT;
        code_getdispinfo    = LVN_GETDISPINFO;
        code_setdispinfo    = LVN_SETDISPINFO;

        /*****************************************************************************************************/
        /**  Affichage des Item de la fentre : Remplissage des Item dans les colonnes des lignes affiches **/
        /*****************************************************************************************************/
        if(code == LVN_GETDISPINFO)
          {
            plvdi = (NMLVDISPINFO *) lParam;

            /* Identifie la ListView : Statistics */
            if(LOWORD(wParam) == window_statistic->listview_statistic.idc_listview)
              current_listview = (struct list_view *) &window_statistic->listview_statistic;
            else
              break;

            /* Valide le numro de la colonne */
            if(plvdi->item.iSubItem < current_listview->nb_column)
              {
                plvdi->item.pszText = current_listview->tab_column[plvdi->item.iSubItem].tab_value_txt[(current_listview->has_column_sort == INTERNALS_COLUMN_SORT_NONE) ? plvdi->item.iItem : plvdi->item.lParam];
              }
          }
       
        /***************************************************************/
        /**  Click sur une ligne d'une ListView pour la slectionner  **/
        /***************************************************************/
        if(code == NM_CLICK)
          {
            /* Rcupre la ListView (Statistic) */
            if(LOWORD(wParam) == window_statistic->listview_statistic.idc_listview)
              current_listview = (struct list_view *) &window_statistic->listview_statistic;
            else
              break;

            /* Identifie la ligne clicke (0-N) */
            select_id = (int) SendMessage(current_listview->hwnd_listview,LVM_GETNEXTITEM,-1,LVNI_FOCUSED); // return item selected
            if(select_id == -1)
              return(TRUE);

            /* Rcupre l'adresse du Statistic dans la Colonne #0 */
            memset(&LvItem,0,sizeof(LVITEM));
            LvItem.mask = LVIF_TEXT;
            LvItem.iSubItem = 0;            /* Colonne #0 */
            LvItem.pszText = buffer_address;
            LvItem.cchTextMax = 256;
            LvItem.iItem = select_id;
            SendMessage(current_listview->hwnd_listview,LVM_GETITEMTEXT,select_id,(LPARAM)&LvItem);
            window_statistic->selected_statistic_address = EvaluateAsAddress(buffer_address,&is_error);

            /* Rcupre le Call Number du Statistic dans la Colonne #1 */
            memset(&LvItem,0,sizeof(LVITEM));
            LvItem.mask = LVIF_TEXT;
            LvItem.iSubItem = 1;            /* Colonne #1 */
            LvItem.pszText = buffer_address;
            LvItem.cchTextMax = 256;
            LvItem.iItem = select_id;
            SendMessage(current_listview->hwnd_listview,LVM_GETITEMTEXT,select_id,(LPARAM)&LvItem);
            window_statistic->selected_statistic_call_number = EvaluateAsWord(buffer_address,&is_error);
            if(is_error == 1)
              window_statistic->selected_statistic_call_number = 0xFFFF;     /* Indisponible */

            /* Active le bouton 'Delete' */
            EnableWindow(GetDlgItem(window_statistic->hwnd_statistic,IDC_BUTTON_STATISTIC_DELETE),TRUE);
          }

        /*************************************************************************/
        /**  Double-click sur une ligne d'une ListView pour activer une action  **/
        /*************************************************************************/
        if(code == NM_DBLCLK)
          {
            /* Rcupre la ListView (Statistic) */
            if(LOWORD(wParam) == window_statistic->listview_statistic.idc_listview)
              current_listview = (struct list_view *) &window_statistic->listview_statistic;
            else
              break;

            /* Identifie la ligne clicke (0-N) */
            select_id = (int) SendMessage(current_listview->hwnd_listview,LVM_GETNEXTITEM,-1,LVNI_FOCUSED); // return item selected
            if(select_id == -1)
              return(TRUE);

            /* Rcupre l'adresse du Statistic dans la Colonne #0 */
            memset(&LvItem,0,sizeof(LVITEM));
            LvItem.mask = LVIF_TEXT;
            LvItem.iSubItem = 0;            /* Colonne #0 */
            LvItem.pszText = buffer_address;
            LvItem.cchTextMax = 256;
            LvItem.iItem = select_id;
            SendMessage(current_listview->hwnd_listview,LVM_GETITEMTEXT,select_id,(LPARAM)&LvItem);
            statistic_address = EvaluateAsAddress(buffer_address,&is_error);

            /** On va ouvrir une fentre de dsassemblage **/
            BuildDisassemblerWindow(param->hInstance,ID_MENU_VIEW_CODEDISASSEMBLER,statistic_address);
          }

        /***************************************************************/
        /**  Click sur les titres des colonnes pour activer les tris  **/
        /***************************************************************/
        if(code == LVN_COLUMNCLICK)
          {
            pListView = (NMLISTVIEW *) lParam;
            select_id = (int) pListView->iSubItem;

            /* Rcupre la ListView (Statistic) */
            if(LOWORD(wParam) == window_statistic->listview_statistic.idc_listview)
              current_listview = (struct list_view *) &window_statistic->listview_statistic;
            else
              break;

            /* On fait le choix du type de tri */
            for(i=0; i<current_listview->nb_column; i++)
              {
                if(i == select_id)
                  {
                    if(current_listview->tab_column[i].sort == INTERNALS_COLUMN_SORT_NONE)
                      current_listview->tab_column[i].sort = INTERNALS_COLUMN_SORT_UP;
                    else if(current_listview->tab_column[i].sort == INTERNALS_COLUMN_SORT_UP)
                      current_listview->tab_column[i].sort = INTERNALS_COLUMN_SORT_DOWN;
                    else
                      current_listview->tab_column[i].sort = INTERNALS_COLUMN_SORT_NONE;
                  }
               else
                 current_listview->tab_column[i].sort = INTERNALS_COLUMN_SORT_NONE;
              }
             
            /* A t'on une colonne trie ? */
            current_listview->has_column_sort = INTERNALS_COLUMN_SORT_NONE;
            for(i=0; i<current_listview->nb_column; i++)
              if(current_listview->tab_column[i].sort != INTERNALS_COLUMN_SORT_NONE)
                current_listview->has_column_sort = current_listview->tab_column[i].sort;
             
            /* On va trier selon le tri mis en place */
            result = ListView_SortItems(current_listview->hwnd_listview,ProcessListViewCompareProc,current_listview);
           
            /* MAJ des images ^ / v des Tris des colonnes */
            setListViewSortIcon(current_listview);
          }

        /***********************************************************************************/
        /**  Gestion de l'affichage des Item pour dfinir une couleur de fond de cellule  **/
        /***********************************************************************************/
        if(code == NM_CUSTOMDRAW)
          {
            SetWindowLongPtr(window_statistic->hwnd_statistic,DWLP_MSGRESULT,(LONG_PTR)ProcessListViewWindowCustomDraw(lParam));
            return(CDRF_NOTIFYITEMDRAW);
          }
        return(TRUE);
        break;

      /**************************************/
      /*  On vient de retailler la fentre  */
      /**************************************/
      case WM_SIZE :
        /* On va tendre la fentre de donne mmoire si c'est un Resize User */
        if(window_statistic->user_resize == 1)
          UserResizeStatisticWindow(window_statistic);
        window_statistic->user_resize = 1;
        return(TRUE);

      /********************************************************************/
      /* Fermeture de window (le WM_CLOSE est appel avant le WM_DESTROY) */
      /********************************************************************/
      case WM_CLOSE :
        /* Fentre Statistic */
        DestroyWindow(hwndStatistic);
        return(0);

      case WM_DESTROY :
        /* Suppression des Fentres */
        DeleteStatisticChildWindows(window_statistic);
        CheckMainWindowMenuItem(ID_MENU_EXTRA_STATISTIC,FALSE);
        my_Memory(MEMORY_FREE_WINDOW_STATISTIC,NULL,NULL);
        /* Demande la sortie de la boucle des messages */
        PostQuitMessage(0);
        return(0);
    }

  /* Retourner FALSE pour relayer le message non trait au systme */
  return(FALSE);
}


/*********************************************************************************************************/
/*  CreateStatisticChildWindows() : Crer les fentres enfants et remplir la structure window_statistic. */
/*********************************************************************************************************/
static int CreateStatisticChildWindows(HWND hwndStatistic, struct window_statistic *window_statistic)
{
  RECT window_rect;
  struct window_main *window_main;
  char buffer[BUFFER_SIZE];
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Rcupre la fentre Main */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);

  /** Window Handle **/
  window_statistic->hwnd_statistic = hwndStatistic;

  /* Police de caractre de la fentre (14 points) */
  window_statistic->hFont = CreateFontA(LISTVIEW_FONT_HEIGHT,0,0,0,FW_DONTCARE,FALSE,FALSE,FALSE,ANSI_CHARSET,
                                        OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY,DEFAULT_PITCH|FF_SWISS,"Arial");

  /* Refresh Mode : Automatic */
  UpdateStatisticRefreshMode(window_statistic,STATISTIC_REFRESH_AUTOMATIC);

  /*****************/
  /*** Statistic ***/
  /*****************/
  window_statistic->statistic_x0 = 7;
  window_statistic->statistic_y0 = 7;

  /** List View **/
  BuildStatisticListView(&window_statistic->listview_statistic,window_statistic->hFont,IDC_LISTVIEW_STATISTIC,window_statistic);

  /** Remplissage du List View si les statistics sont disponibles **/
  if(window_main->current_statistic_table != NULL)
    FillStatisticListView(&window_statistic->listview_statistic,window_main->current_statistic_table);

  /* MAJ du nombre de cycle */
  if(window_main->current_statistic_table != NULL)
    {
      /* Total Call */
      sprintf(buffer,"%d",window_main->current_statistic_table->total_call_statistic_func);
      PunctuateAtThoudand(buffer);
      SetWindowText(GetDlgItem(window_statistic->hwnd_statistic,IDC_STATIC_TOTAL_FUNCTION),buffer);

      /* Total Cycle (64 bit) */
      sprintf(buffer,"%I64d",window_main->current_statistic_table->total_cycle_statistic_func);
      PunctuateAtThoudand(buffer);
      SetWindowText(GetDlgItem(window_statistic->hwnd_statistic,IDC_STATIC_TOTAL_CYCLE),buffer);
    }

  /* MAJ du titre de la fentre avec le nombre de Statistic */
  if(window_main->current_statistic_table != NULL)
    {
      sprintf(buffer," Function Statistics Table : %d entries",window_main->current_statistic_table->nb_statistic_func);
      SetWindowText(window_statistic->hwnd_statistic,buffer);
    }

  /* Aucune entre de slectionne dans la ListView */
  window_statistic->selected_statistic_address = 0xFFFFFFFF;

  /* Dsactive le bouton 'Delete' */
  EnableWindow(GetDlgItem(window_statistic->hwnd_statistic,IDC_BUTTON_STATISTIC_DELETE),FALSE);

  /** Taille de la fentre au dmarrage ((avec la barre de titre et les bordures)) **/
  GetWindowRect(window_statistic->hwnd_statistic,&window_rect);
  window_statistic->window_init_width  = window_rect.right  - window_rect.left;
  window_statistic->window_init_height = window_rect.bottom - window_rect.top;

  /* Nombre de ligne dans la ListView au dmarrage de la fentre */
  window_statistic->nb_init_data_row = STATISTIC_LISTVIEW_NB_ROW;

  /* Nombre de lignes qu'il est possible d'afficher dans la ListView */
  window_statistic->nb_row = window_statistic->nb_init_data_row;

  /* OK */
  return(0);
}


/*********************************************************************************************/
/*  UpdateStatisticRefreshMode() :  MAJ du mode de rafraichissement de la fentre Statistic. */
/*********************************************************************************************/
static void UpdateStatisticRefreshMode(struct window_statistic *window_statistic, int refresh_mode)
{
  struct notification *current_notification;

  if(refresh_mode == STATISTIC_REFRESH_AUTOMATIC)
    {
      /* Visuel */
      window_statistic->refresh_mode = STATISTIC_REFRESH_AUTOMATIC;
      SendDlgItemMessage(window_statistic->hwnd_statistic,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_statistic->hwnd_statistic,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,0,0);

      /* Cration d'une Notification */
      current_notification = (struct notification *) calloc(1,sizeof(struct notification));
      if(current_notification == NULL)
        return;
      current_notification->hwnd_window = window_statistic->hwnd_statistic;
      current_notification->message = HM_STATISTIC_REFRESHWINDOW;
      current_notification->type = NOTIFICATION_TYPE_VIEW;
      current_notification->sub_type = NOTIFICATION_SUBTYPE_STATISTIC;

      /* Ajout de la notification */
      window_statistic->current_notification = current_notification;
      my_Notification(NOTIFICATION_ADD,current_notification,NULL);
    }
  else
    {
      /* Visuel */
      window_statistic->refresh_mode = STATISTIC_REFRESH_MANUAL;
      SendDlgItemMessage(window_statistic->hwnd_statistic,IDC_RADIO_REFRESH_AUTOMATIC,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_statistic->hwnd_statistic,IDC_RADIO_REFRESH_MANUAL,BM_SETCHECK,1,0);

      /* Supprime la Notification */
      my_Notification(NOTIFICATION_REMOVE,window_statistic->current_notification,NULL);
      window_statistic->current_notification = NULL;
    }
}


/********************************************************************************/
/*  BuildStatisticListView() :  Construction d'une ListView pour les Statistic. */
/********************************************************************************/
static void BuildStatisticListView(struct list_view *current_listview, HFONT hFont, int idc_listview, struct window_statistic *window_statistic)
{
  /** Init de la structure List_view **/
  current_listview->idc_listview = idc_listview;

  /** Liste des Colonnes **/
  current_listview->nb_column = 0;

  /* Col 0 : Address */
  current_listview->tab_column[0].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[0].align = LVCFMT_RIGHT;
  current_listview->tab_column[0].length = 80;
  strcpy(current_listview->tab_column[0].title,"Address");
  current_listview->nb_column++;

  /* Col 1 : Call Number */
  current_listview->tab_column[1].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[1].align = LVCFMT_RIGHT;
  current_listview->tab_column[1].length = 60;
  strcpy(current_listview->tab_column[1].title,"Num");
  current_listview->nb_column++;

  /* Col 2 : Name */
  current_listview->tab_column[2].type = INTERNALS_COLUMN_TYPE_TXT;
  current_listview->tab_column[2].align = LVCFMT_LEFT;
  current_listview->tab_column[2].length = 140;
  strcpy(current_listview->tab_column[2].title,"Name");
  current_listview->nb_column++;

  /* Col 3 : Application */
  current_listview->tab_column[3].type = INTERNALS_COLUMN_TYPE_TXT;
  current_listview->tab_column[3].align = LVCFMT_LEFT;
  current_listview->tab_column[3].length = 220;
  strcpy(current_listview->tab_column[3].title,"Application");
  current_listview->nb_column++;

  /* Col 4 : # Call */
  current_listview->tab_column[4].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[4].align = LVCFMT_RIGHT;
  current_listview->tab_column[4].length = 65;
  strcpy(current_listview->tab_column[4].title,"# Call");
  current_listview->nb_column++;

  /* Col 5 : Total Cycle */
  current_listview->tab_column[5].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[5].align = LVCFMT_RIGHT;
  current_listview->tab_column[5].length = 90;
  strcpy(current_listview->tab_column[5].title,"Total Cycle");
  current_listview->nb_column++;

  /* Col 6 : % */
  current_listview->tab_column[6].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[6].align = LVCFMT_RIGHT;
  current_listview->tab_column[6].length = 50;
  strcpy(current_listview->tab_column[6].title,"%");
  current_listview->nb_column++;

  /* Col 7 : Min Cycle */
  current_listview->tab_column[7].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[7].align = LVCFMT_RIGHT;
  current_listview->tab_column[7].length = 90;
  strcpy(current_listview->tab_column[7].title,"Min Cycle");
  current_listview->nb_column++;

  /* Col 8 : Max Cycle */
  current_listview->tab_column[8].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[8].align = LVCFMT_RIGHT;
  current_listview->tab_column[8].length = 90;
  strcpy(current_listview->tab_column[8].title,"Max Cycle");
  current_listview->nb_column++;

  /* Col 9 : Average Cycle */
  current_listview->tab_column[9].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[9].align = LVCFMT_RIGHT;
  current_listview->tab_column[9].length = 90;
  strcpy(current_listview->tab_column[9].title,"Avg Cycle");
  current_listview->nb_column++;

  /** Cration du ListView (20 lignes) **/
  BuildListView(window_statistic->hwnd_statistic,hFont,window_statistic->statistic_x0,window_statistic->statistic_y0,STATISTIC_LISTVIEW_NB_ROW,current_listview);
}


/******************************************************************************/
/*  FillStatisticListView() :  Remplissage d'une ListView pour les Statistic. */
/******************************************************************************/
static void FillStatisticListView(struct list_view *current_listview, struct statistic_table *current_statistic_table)
{
  int i, result;
  QWORD percent;
  double percent_dbl;
  LVITEM listview_item;
  struct statistic_function *current_statistic_func;
  char buffer[1024];
  char buffer_name[1024];
  char buffer_type[1024];
  char buffer_application[1024];
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Libration mmoire des donnes affiches dans les colonnes de la ListView */
  mem_free_listviewcolumn(current_listview);

  /* Aucun Statistic */
  if(current_statistic_table == NULL)
    return;
  if(current_statistic_table->nb_statistic_func == 0)
    return;

  /** Remplissage avec les valeurs dans la structure mmoire **/
  current_listview->nb_row = current_statistic_table->nb_statistic_func;
  for(i=0; i<current_listview->nb_column; i++)
    {
      /* Allocation des tableaux */
      current_listview->tab_column[i].tab_value_num = (DWORD *) calloc(current_listview->nb_row,sizeof(DWORD));
      current_listview->tab_column[i].tab_value_txt = (char **) calloc(current_listview->nb_row,sizeof(char *));
    }
  /** Remplissage des tableaux **/
  for(i=0, current_statistic_func=current_statistic_table->first_statistic_func; current_statistic_func; current_statistic_func=current_statistic_func->next, i++)
    {
      /** Rcupre les informations lies  cette adresse **/
      GetNameFromAddress(current_statistic_func->address,current_statistic_func->call_number,buffer_name,buffer_type,buffer_application);

      /* Colonne 0 : Address */
      current_listview->tab_column[0].tab_value_num[i] = current_statistic_func->address;
      sprintf(buffer,"%02X/%04X",(current_statistic_func->address >> 16) & 0xFF,current_statistic_func->address & 0xFFFF);
      current_listview->tab_column[0].tab_value_txt[i] = strdup(buffer);

      /* Colonne 1 : Call Number */
      current_listview->tab_column[1].tab_value_num[i] = current_statistic_func->call_number;
      if(current_statistic_func->call_number == 0x0000 || current_statistic_func->call_number == 0xFFFF)
        strcpy(buffer,"");
      else
        sprintf(buffer,"%04X",current_statistic_func->call_number);
      current_listview->tab_column[1].tab_value_txt[i] = strdup(buffer);

      /* Colonne 2 : Name */
      current_listview->tab_column[2].tab_value_num[i] = i;
      current_listview->tab_column[2].tab_value_txt[i] = strdup(buffer_name);

      /* Colonne 3 : Application */
      current_listview->tab_column[3].tab_value_num[i] = i;
      if(strlen(buffer_type) > 0 && strlen(buffer_application) > 0)
        sprintf(buffer,"%s : %s",buffer_type,buffer_application);
      else if(strlen(buffer_type) > 0 && strlen(buffer_application) == 0)
        strcpy(buffer,buffer_type);
      else if(strlen(buffer_type) == 0 && strlen(buffer_application) > 0)
        strcpy(buffer,buffer_application);
      else
        strcpy(buffer,"");
      current_listview->tab_column[3].tab_value_txt[i] = strdup(buffer);

      /* Colonne 4 : # Call */
      current_listview->tab_column[4].tab_value_num[i] = current_statistic_func->nb_call;
      sprintf(buffer,"%d",current_statistic_func->nb_call);
      PunctuateAtThoudand(buffer);
      current_listview->tab_column[4].tab_value_txt[i] = strdup(buffer);

      /* Colonne 5 : Total Cycle */
      current_listview->tab_column[5].tab_value_num[i] = (DWORD) current_statistic_func->total_cycle;
      sprintf(buffer,"%d",(DWORD) current_statistic_func->total_cycle);
      PunctuateAtThoudand(buffer);
      current_listview->tab_column[5].tab_value_txt[i] = strdup(buffer);

      /* Colonne 6 : % Total Cycle */
      percent = (current_statistic_func->total_cycle*1000)/current_statistic_table->total_cycle_statistic_func;
      current_listview->tab_column[6].tab_value_num[i] = (DWORD) percent;
      percent_dbl = (double) percent;
      sprintf(buffer,"%.1f",percent_dbl/10.0);
      current_listview->tab_column[6].tab_value_txt[i] = strdup(buffer);

      /* Colonne 7 : Min Cycle */
      current_listview->tab_column[7].tab_value_num[i] = current_statistic_func->min_cycle;
      sprintf(buffer,"%d",current_statistic_func->min_cycle);
      PunctuateAtThoudand(buffer);
      current_listview->tab_column[7].tab_value_txt[i] = strdup(buffer);

      /* Colonne 8 : Max Cycle */
      current_listview->tab_column[8].tab_value_num[i] = current_statistic_func->max_cycle;
      sprintf(buffer,"%d",current_statistic_func->max_cycle);
      PunctuateAtThoudand(buffer);
      current_listview->tab_column[8].tab_value_txt[i] = strdup(buffer);

      /* Colonne 9 : Average Cycle */
      current_listview->tab_column[9].tab_value_num[i] = (DWORD) (current_statistic_func->total_cycle / current_statistic_func->nb_call);
      sprintf(buffer,"%d",(DWORD) (current_statistic_func->total_cycle / current_statistic_func->nb_call));
      PunctuateAtThoudand(buffer);
      current_listview->tab_column[9].tab_value_txt[i] = strdup(buffer);
    }

  /** Transfert des donnes dans la List View via un Msg LVN_GETDISPINFO **/
  /** Ajout des Item dans le ListView **/
  listview_item.pszText   = LPSTR_TEXTCALLBACK;           /* Sends an LVN_GETDISPINFO message to fill the item */
  listview_item.mask      = LVIF_TEXT | LVIF_IMAGE | LVIF_STATE | LVIF_PARAM;
  listview_item.stateMask = 0;
  listview_item.iSubItem  = 0;
  listview_item.state     = 0;

  /** Initialize LVITEM members that are different for each item **/
  for(i=0; i<current_listview->nb_row; i++)
    {
      listview_item.iItem  = i;
      listview_item.iImage = -1;         /* Identifiant de l'image dans la liste */
      listview_item.lParam = i;          /* Identifiant renvoy lors du tri */
      result = ListView_InsertItem(current_listview->hwnd_listview,&listview_item);
    }
}


/******************************************************************************/
/*  ProcessStatisticOperation() :  MAJ des Statistiques avec cette opration. */
/******************************************************************************/
void ProcessStatisticOperation(struct window_main *window_main, int is_interrupt)
{
  WORD func_call_number;
  DWORD nb_cycle, nb_operation, curr_op_address, func_address;
  int i, result, min, max, index, depth, prev_op_is_jsr_jsl, nb_unstack, exit_interrupt, nb_exit;
  struct statistic_function *current_statistic_func;
  struct statistic_stack *current_statistic_stack;
  struct statistic_stack *next_statistic_stack;
  struct statistic_stack *eof_statistic_stack;
  struct statistic_stack *previous_eof_statistic_stack;
  struct fonction_history *current_fonction_history;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Init */
  prev_op_is_jsr_jsl = (param->prev_operation_opcode == 0x20 || param->prev_operation_opcode == 0xFC || param->prev_operation_opcode == 0x22) ? 1 : 0;
  curr_op_address = (param->operation_PB << 16) | param->operation_PC;
  current_fonction_history = NULL;

  /** Si besoin, on alloue la structure statistic_table **/
  if(window_main->current_statistic_table == NULL)
    {
      window_main->current_statistic_table = (struct statistic_table *) calloc(1,sizeof(struct statistic_table));
      if(window_main->current_statistic_table == NULL)
        return;

      /* Init : Global Cycle Count */
      window_main->current_statistic_table->gcc_init = param->operation_gcc;
    }

  /** Vient t'on de sortir d'une fonction ? **/
  for(eof_statistic_stack = window_main->current_statistic_table->last_statistic_stack; eof_statistic_stack; eof_statistic_stack = eof_statistic_stack->previous)
    if(eof_statistic_stack->function_end_address == curr_op_address)
      break;

  /** Cas 1 : On vient de rentrer dans une fonction JSR/JSL **/
  if(prev_op_is_jsr_jsl == 1 && curr_op_address == param->prev_operation_next_op_addr)
    {
      /* Nouvelle entre dans la Statistic Stack */
      current_statistic_stack = (struct statistic_stack *) calloc(1,sizeof(struct statistic_stack));
      if(current_statistic_stack == NULL)
        {
          return;
        }
      current_statistic_stack->function_address     = curr_op_address;
      current_statistic_stack->call_number          = param->prev_operation_call_number;
      current_statistic_stack->gcc                  = param->operation_gcc;
      current_statistic_stack->goid                 = param->operation_goid;
      current_statistic_stack->function_end_address = param->prev_operation_sub_next_op_addr;
      current_statistic_stack->is_interrupt         = 0;
      current_statistic_stack->depth                = (window_main->current_statistic_table->last_statistic_stack == NULL) ? 0 : (window_main->current_statistic_table->last_statistic_stack->depth + 1);

      /* Ajoute  la fin de la liste doublement chaine */
      if(window_main->current_statistic_table->first_statistic_stack == NULL)
        {
          window_main->current_statistic_table->first_statistic_stack = current_statistic_stack;
          window_main->current_statistic_table->last_statistic_stack = current_statistic_stack;
        }
      else
        {
          current_statistic_stack->previous = window_main->current_statistic_table->last_statistic_stack;
          window_main->current_statistic_table->last_statistic_stack->next = current_statistic_stack;
          window_main->current_statistic_table->last_statistic_stack = current_statistic_stack;
        }

      /** Fonction History **/
      current_fonction_history = (struct fonction_history *) calloc(1,sizeof(struct fonction_history));
      if(current_fonction_history != NULL)
        {
          /* Remplissage */
          current_fonction_history->type                 = FUNCTION_HISTORY_TYPE_ENTER_FUNCTION;
          current_fonction_history->goid                 = current_statistic_stack->goid;
          current_fonction_history->cycle                = param->operation_gcc;
          current_fonction_history->function_address     = current_statistic_stack->function_address;
          current_fonction_history->call_number          = current_statistic_stack->call_number;
          current_fonction_history->operation_PB         = param->operation_PB;
          current_fonction_history->operation_PC         = param->operation_PC;
          current_fonction_history->prev_operation_PB    = param->prev_operation_PB;
          current_fonction_history->prev_operation_PC    = param->prev_operation_PC;
          current_fonction_history->function_end_address = current_statistic_stack->function_end_address;
          current_fonction_history->depth                = current_statistic_stack->depth;

          /* Attachement */
          if(window_main->current_statistic_table->first_fonction_history == NULL)
            window_main->current_statistic_table->first_fonction_history = current_fonction_history;
          else
            window_main->current_statistic_table->last_fonction_history->next = current_fonction_history;
          window_main->current_statistic_table->last_fonction_history = current_fonction_history;
          window_main->current_statistic_table->nb_fonction_history++;
        }
    }
  /** Cas 2 : On vient de rentrer dans une Interruption **/
  else if(curr_op_address != param->prev_operation_next_op_addr && param->is_first_operation == 0)
    {
      /* Nouvelle entre dans la Statistic Stack */
      current_statistic_stack = (struct statistic_stack *) calloc(1,sizeof(struct statistic_stack));
      if(current_statistic_stack == NULL)
        {
          return;
        }
      current_statistic_stack->function_address     = curr_op_address;
      current_statistic_stack->call_number          = param->prev_operation_call_number;
      current_statistic_stack->gcc                  = param->operation_gcc;
      current_statistic_stack->goid                 = param->operation_goid;
      current_statistic_stack->function_end_address = param->prev_operation_sub_next_op_addr;
      current_statistic_stack->is_interrupt         = 1;
      current_statistic_stack->depth                = (window_main->current_statistic_table->last_statistic_stack == NULL) ? 0 : (window_main->current_statistic_table->last_statistic_stack->depth + 1);

      /* Ajoute  la fin de la liste doublement chaine */
      if(window_main->current_statistic_table->first_statistic_stack == NULL)
        {
          window_main->current_statistic_table->first_statistic_stack = current_statistic_stack;
          window_main->current_statistic_table->last_statistic_stack = current_statistic_stack;
        }
      else
        {
          current_statistic_stack->previous = window_main->current_statistic_table->last_statistic_stack;
          window_main->current_statistic_table->last_statistic_stack->next = current_statistic_stack;
          window_main->current_statistic_table->last_statistic_stack = current_statistic_stack;
        }

      /** Fonction History **/
      current_fonction_history = (struct fonction_history *) calloc(1,sizeof(struct fonction_history));
      if(current_fonction_history != NULL)
        {
          /* Remplissage */
          current_fonction_history->type                 = FUNCTION_HISTORY_TYPE_ENTER_INTERRUPT;
          current_fonction_history->goid                 = current_statistic_stack->goid;
          current_fonction_history->cycle                = param->operation_gcc;
          current_fonction_history->function_address     = current_statistic_stack->function_address;
          current_fonction_history->call_number          = current_statistic_stack->call_number;
          current_fonction_history->operation_PB         = param->operation_PB;
          current_fonction_history->operation_PC         = param->operation_PC;
          current_fonction_history->prev_operation_PB    = param->prev_operation_PB;
          current_fonction_history->prev_operation_PC    = param->prev_operation_PC;
          current_fonction_history->function_end_address = current_statistic_stack->function_end_address;
          current_fonction_history->depth                = current_statistic_stack->depth;

          /* Attachement */
          if(window_main->current_statistic_table->first_fonction_history == NULL)
            window_main->current_statistic_table->first_fonction_history = current_fonction_history;
          else
            window_main->current_statistic_table->last_fonction_history->next = current_fonction_history;
          window_main->current_statistic_table->last_fonction_history = current_fonction_history;
          window_main->current_statistic_table->nb_fonction_history++;
        }
    }
  /** Cas 3 : On vient de sortir d'une fonction **/
  else if(eof_statistic_stack != NULL)
    {
      /** On va peut tre devoir dpiler plusieurs appels sur la mme opration (Interruption + Fonction) **/
      nb_exit = 1;
      while(eof_statistic_stack->function_end_address == curr_op_address)
        {
          /* Calcul du # de Cycle + # d'opration */
          nb_cycle = (DWORD) (param->operation_gcc - eof_statistic_stack->gcc);
          nb_operation = (DWORD) (param->operation_goid - eof_statistic_stack->goid);

          /* MAJ Stat Global Total */
          window_main->current_statistic_table->total_call_statistic_func++;
          window_main->current_statistic_table->total_cycle_statistic_func += nb_cycle;

          /** La fonction Statistic est t'elle connue ? **/
          current_statistic_func = GetStatisticByAddress(eof_statistic_stack->function_address,eof_statistic_stack->call_number);
          if(current_statistic_func != NULL)
            {
              /* MAJ des Stat */
              current_statistic_func->nb_call++;
              current_statistic_func->total_cycle += nb_cycle;
              if(current_statistic_func->min_cycle > nb_cycle)
                current_statistic_func->min_cycle = nb_cycle;
              if(current_statistic_func->max_cycle < nb_cycle)
                current_statistic_func->max_cycle = nb_cycle;
            }
          else
            {
              /** Allocation d'une nouvelle Function Statistic **/
              current_statistic_func = (struct statistic_function *) calloc(1,sizeof(struct statistic_function));
              if(current_statistic_func == NULL)
                return;
              current_statistic_func->address     = eof_statistic_stack->function_address;
              current_statistic_func->call_number = eof_statistic_stack->call_number;
              current_statistic_func->nb_call     = 1;
              current_statistic_func->total_cycle = nb_cycle;
              current_statistic_func->max_cycle   = nb_cycle;
              current_statistic_func->min_cycle   = nb_cycle;

              /** Ajout dans la liste **/
              if(window_main->current_statistic_table->nb_statistic_func == 0)
                {
                  /* 1re */
                  window_main->current_statistic_table->nb_statistic_func    = 1;
                  window_main->current_statistic_table->first_statistic_func = current_statistic_func;
                  window_main->current_statistic_table->last_statistic_func  = current_statistic_func;
                }
              else
                {
                  /** Insertion au bon endroit dans le tableau **/
                  /* En premire position */
                  result = compare_statistic_address(&current_statistic_func,&window_main->current_statistic_table->first_statistic_func);
                  if(result < 0)
                    {
                      current_statistic_func->next = window_main->current_statistic_table->first_statistic_func;
                      window_main->current_statistic_table->first_statistic_func = current_statistic_func;
                      window_main->current_statistic_table->nb_statistic_func++;
                    }
                  else
                    {
                      /* Dernire position */
                      result = compare_statistic_address(&current_statistic_func,&window_main->current_statistic_table->last_statistic_func);
                      if(result > 0)
                        {
                          window_main->current_statistic_table->last_statistic_func->next = current_statistic_func;
                          window_main->current_statistic_table->last_statistic_func = current_statistic_func;
                          window_main->current_statistic_table->nb_statistic_func++;
                        }
                      else
                        {
                          /* Au milieu : Recherche Dychotomique */
                          min = 0;
                          max = window_main->current_statistic_table->nb_statistic_func-1;
                          while(min < max)
                            {
                              index = min + ((max - min) / 2);
                              result = compare_statistic_address(&current_statistic_func,&window_main->current_statistic_table->tab_statistic_func[index]);
                              if(result < 0)
                                max = index-1;
                              else
                                min = index+1;
                            }
                          if(index < min)
                            index = min;
                          if(index > max)
                            index = max;

                          /* On teste pour insrer avant ou aprs l'index trouv */
                          result = compare_statistic_address(&current_statistic_func,&window_main->current_statistic_table->tab_statistic_func[index]);
                          if(result < 0)
                            {
                              /* Avant */
                              window_main->current_statistic_table->tab_statistic_func[index-1]->next = current_statistic_func;
                              current_statistic_func->next = window_main->current_statistic_table->tab_statistic_func[index];
                              window_main->current_statistic_table->nb_statistic_func++;
                            }
                          else
                            {
                              /* Aprs */
                              current_statistic_func->next = window_main->current_statistic_table->tab_statistic_func[index]->next;
                              window_main->current_statistic_table->tab_statistic_func[index]->next = current_statistic_func;
                              window_main->current_statistic_table->nb_statistic_func++;
                            }
                        }
                    }
                }

              /** Cration du tableau tri **/
              if(window_main->current_statistic_table->tab_statistic_func != NULL)
                free(window_main->current_statistic_table->tab_statistic_func);
              window_main->current_statistic_table->tab_statistic_func = (struct statistic_function **) calloc(window_main->current_statistic_table->nb_statistic_func,sizeof(struct statistic *));
              if(window_main->current_statistic_table->tab_statistic_func == NULL)
                {
                  return;
                }
              for(i=0, current_statistic_func=window_main->current_statistic_table->first_statistic_func; current_statistic_func; current_statistic_func=current_statistic_func->next, i++)
                window_main->current_statistic_table->tab_statistic_func[i] = current_statistic_func;
            }

          /* On conserve un pointeur sur la fonction prcdente */
          previous_eof_statistic_stack = eof_statistic_stack->previous;

          /** On va dpiler la Function Stack **/
          nb_unstack = 0;
          depth = eof_statistic_stack->depth;
          func_address = eof_statistic_stack->function_address;
          func_call_number = eof_statistic_stack->call_number;
          exit_interrupt = eof_statistic_stack->is_interrupt;
          if(window_main->current_statistic_table->first_statistic_stack == eof_statistic_stack)
            {
              nb_unstack = 1;
              free(eof_statistic_stack);
              window_main->current_statistic_table->first_statistic_stack = NULL;
              window_main->current_statistic_table->last_statistic_stack = NULL;
            }
          else
            {
              eof_statistic_stack->previous->next = NULL;
              window_main->current_statistic_table->last_statistic_stack = eof_statistic_stack->previous;
              for(current_statistic_stack = eof_statistic_stack; current_statistic_stack; current_statistic_stack = next_statistic_stack)
                {
                  next_statistic_stack = current_statistic_stack->next;
                  free(current_statistic_stack);
                  nb_unstack++;
                }
            }

          /** Fonction History **/
          current_fonction_history = (struct fonction_history *) calloc(1,sizeof(struct fonction_history));
          if(current_fonction_history != NULL)
            {
              /* Remplissage */
              current_fonction_history->type             = FUNCTION_HISTORY_TYPE_EXIT;
              current_fonction_history->goid             = param->operation_goid;
              current_fonction_history->cycle            = param->operation_gcc;
              current_fonction_history->function_address = func_address;
              current_fonction_history->call_number      = func_call_number;
              current_fonction_history->operation_PB     = param->operation_PB;
              current_fonction_history->operation_PC     = param->operation_PC;
              current_fonction_history->depth            = depth;
              current_fonction_history->exit_interrupt   = exit_interrupt;
              current_fonction_history->nb_unstack       = nb_unstack;
              current_fonction_history->nb_operation     = nb_operation;
              current_fonction_history->nb_cycle         = nb_cycle;
              current_fonction_history->nb_exit          = nb_exit;

              /* Attachement */
              if(window_main->current_statistic_table->first_fonction_history == NULL)
                window_main->current_statistic_table->first_fonction_history = current_fonction_history;
              else
                window_main->current_statistic_table->last_fonction_history->next = current_fonction_history;
              window_main->current_statistic_table->last_fonction_history = current_fonction_history;
              window_main->current_statistic_table->nb_fonction_history++;
            }

          /* On peut encore dpiler si on est sur la mme @ de retour */
          nb_exit++;
          if(previous_eof_statistic_stack == NULL)
            break;
          eof_statistic_stack = previous_eof_statistic_stack;
        }
    }
  /** Cas 4 : C'est une opration neutre **/
}


/******************************************************************************/
/*  DeleteOneStatistic() :  Supprime une des Fonctions Statistic de la Liste. */
/******************************************************************************/
static void DeleteOneStatistic(DWORD address, WORD call_number, struct window_statistic *window_statistic)
{
  int i;
  struct window_main *window_main;
  struct statistic_function *current_statistic_func;
  struct statistic_function *previous_statistic_func;

  /* Rcupre la fentre Main */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  if(window_main == NULL)
    return;
  if(window_main->current_statistic_table == NULL)
    return;

  /*** Recherche + Supprime la fonction | MAJ Liste ***/
  previous_statistic_func = NULL;
  for(current_statistic_func = window_main->current_statistic_table->first_statistic_func; current_statistic_func; current_statistic_func = current_statistic_func->next)
    {
      /** Supprime l'entre dans la liste **/
      if(current_statistic_func->address == address && 
         (current_statistic_func->call_number == call_number || current_statistic_func->call_number == 0x0000 || current_statistic_func->call_number == 0xFFFF))
        {
          /** Reconstruit la liste **/
          /* Une seule entre */
          if(window_main->current_statistic_table->nb_statistic_func == 1)
            {
              /* Liste */
              window_main->current_statistic_table->nb_statistic_func    = 0;
              window_main->current_statistic_table->first_statistic_func = NULL;
              window_main->current_statistic_table->last_statistic_func  = NULL;
              /* Stat */
              window_main->current_statistic_table->total_call_statistic_func  = 0;
              window_main->current_statistic_table->total_cycle_statistic_func = 0;
            }
          /* Premire entre */
          else if(current_statistic_func == window_main->current_statistic_table->first_statistic_func)
            {
              /* Liste */
              window_main->current_statistic_table->nb_statistic_func--;
              window_main->current_statistic_table->first_statistic_func = current_statistic_func->next;
              /* Stat */
              window_main->current_statistic_table->total_call_statistic_func  -= current_statistic_func->nb_call;
              window_main->current_statistic_table->total_cycle_statistic_func -= current_statistic_func->total_cycle;
            }
          /* Dernire entre */
          else if(current_statistic_func == window_main->current_statistic_table->last_statistic_func)
            {
              /* Liste */
              window_main->current_statistic_table->nb_statistic_func--;
              window_main->current_statistic_table->last_statistic_func = previous_statistic_func;
              previous_statistic_func->next = NULL;
              /* Stat */
              window_main->current_statistic_table->total_call_statistic_func  -= current_statistic_func->nb_call;
              window_main->current_statistic_table->total_cycle_statistic_func -= current_statistic_func->total_cycle;
            }
          /* Au milieu */
          else
            {
              /* Liste */
              window_main->current_statistic_table->nb_statistic_func--;
              previous_statistic_func->next = current_statistic_func->next;
              /* Stat */
              window_main->current_statistic_table->total_call_statistic_func  -= current_statistic_func->nb_call;
              window_main->current_statistic_table->total_cycle_statistic_func -= current_statistic_func->total_cycle;
            }

          /** Supprime l'entre **/
          mem_free_statistic(current_statistic_func);

          /** Reconstruit le tableau **/
          if(window_main->current_statistic_table->tab_statistic_func != NULL)
            free(window_main->current_statistic_table->tab_statistic_func);
          window_main->current_statistic_table->tab_statistic_func =  (struct statistic_function **) calloc(1+window_main->current_statistic_table->nb_statistic_func,sizeof(struct statistic_function *));
          if(window_main->current_statistic_table->tab_statistic_func == NULL)
            {
              return;
            }
          for(i=0,current_statistic_func=window_main->current_statistic_table->first_statistic_func; i<window_main->current_statistic_table->nb_statistic_func; i++,current_statistic_func=current_statistic_func->next)
            window_main->current_statistic_table->tab_statistic_func[i] = current_statistic_func;

          /* On sort */
          break;
        }

      /* Garde un pointeur sur l'entre prcdante */
      previous_statistic_func = current_statistic_func;
    }

  /** Reconstruit la ListView **/
  RefreshStatisticWindow(window_statistic);
}


/************************************************************************************/
/*  SaveFunctionTable() :  On va enregister la table des fonctions dans un fichier. */
/************************************************************************************/
static void SaveFunctionTable(struct window_statistic *window_statistic)
{
  BOOL is_OK;
  FILE *fd_statistic;
  DWORD total_cycle_count;
  int i;
  char buffer_call_number[256];
  char buffer_name[BUFFER_SIZE];
  char buffer_type[BUFFER_SIZE];
  char buffer_application[BUFFER_SIZE];
  char buffer_application_ext[BUFFER_SIZE];
  char file_path[BUFFER_SIZE];
  struct window_main *window_main;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Rcupre la fentre Main */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  if(window_main == NULL)
    return;
  if(window_main->current_statistic_table == NULL)
    return;

  /** Demande le nom du fichier **/
  sprintf(file_path,"Functions_Statistic.txt");
  is_OK = GetSaveFilePath(window_statistic->hwnd_statistic,"Select Functions Table file","All Files (*.txt)\0*.txt\0\0",file_path);
  if(is_OK = FALSE)
    return;

  /* Cration du fichier sur disque */
  fd_statistic = fopen(file_path,"w");
  if(fd_statistic == NULL)
    {
      MessageBox(window_statistic->hwnd_statistic,"Impossible to create File","Error",MB_OK|MB_ICONWARNING);
      return;
    }

  /* Global Total Cycle Count */
  total_cycle_count = (DWORD) (param->operation_gcc - window_main->current_statistic_table->gcc_init);
  if(total_cycle_count == 0)
    total_cycle_count = 1;

  /** Extraction des donnes de la table **/
  fprintf(fd_statistic,"Function Address;Call Number;Name;Application;# Call;Total Cycle;%%;Min Cycle;Max Cycle;Average Cycle\n");
  for(i=0; i<window_main->current_statistic_table->nb_statistic_func; i++)
    {
      /* Call Number */
      if(window_main->current_statistic_table->tab_statistic_func[i]->call_number == 0xFFFF)
        strcpy(buffer_call_number,"");
      else
        sprintf(buffer_call_number,"%04X",window_main->current_statistic_table->tab_statistic_func[i]->call_number);

      /* Nom de la fonction + Application */
      GetNameFromAddress(window_main->current_statistic_table->tab_statistic_func[i]->address,window_main->current_statistic_table->tab_statistic_func[i]->call_number,buffer_name,buffer_type,buffer_application);
      if(strlen(buffer_type) > 0 && strlen(buffer_application) > 0)
        sprintf(buffer_application_ext,"%s : %s",buffer_type,buffer_application);
      else if(strlen(buffer_type) > 0 && strlen(buffer_application) == 0)
        strcpy(buffer_application_ext,buffer_type);
      else if(strlen(buffer_type) == 0 && strlen(buffer_application) > 0)
        strcpy(buffer_application_ext,buffer_application);
      else
        strcpy(buffer_application_ext,"");

      /* Statistique d'une fonction */
      fprintf(fd_statistic,"%02X/%04X;%s;%s;%s;%d;%d;%d;%d;%d;%d\n",
              (window_main->current_statistic_table->tab_statistic_func[i]->address >> 16) & 0x000000FF,
              window_main->current_statistic_table->tab_statistic_func[i]->address & 0x0000FFFF,
              buffer_call_number,
              buffer_name,
              buffer_application_ext,
              window_main->current_statistic_table->tab_statistic_func[i]->nb_call,
              window_main->current_statistic_table->tab_statistic_func[i]->total_cycle,
              (100*window_main->current_statistic_table->tab_statistic_func[i]->total_cycle) / total_cycle_count,
              window_main->current_statistic_table->tab_statistic_func[i]->min_cycle,
              window_main->current_statistic_table->tab_statistic_func[i]->max_cycle,
              window_main->current_statistic_table->tab_statistic_func[i]->total_cycle / window_main->current_statistic_table->tab_statistic_func[i]->nb_call);
    }

  /* Fermeture du fichier */
  fclose(fd_statistic);
}


/**************************************************************************************/
/*  SaveFunctionHistory() :  On va enregister la liste des fonctions dans un fichier. */
/**************************************************************************************/
static void SaveFunctionHistory(struct window_statistic *window_statistic)
{
  BOOL is_OK;
  int i, result;
  struct window_main *window_main;
  char file_path[BUFFER_SIZE];

  /* Rcupre la fentre Main */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  if(window_main == NULL)
    return;
  if(window_main->current_statistic_table == NULL)
    return;

  /** Demande le nom du fichier **/
  sprintf(file_path,"Functions_List.txt");
  is_OK = GetSaveFilePath(window_statistic->hwnd_statistic,"Select Functions List file","All Files (*.txt)\0*.txt\0\0",file_path);
  if(is_OK = FALSE)
    return;
  for(i=(int)strlen(file_path); i>=0; i--)
    if(file_path[i] == '.')
      {
        file_path[i] = '\0';
        break;
      }

  /* Lancement du Thread de cration des fichiers */
  CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)SaveFunctionHistoryThread,(char *)file_path,0,NULL);

  /* Cration de la fentre de ProgressBar */
  result = ProgressBarWindow(window_statistic->hwnd_statistic,"Save Function List","Write Function List into files :",window_main->current_statistic_table->nb_fonction_history/1000);
}


/**************************************************************************************************/
/*  SaveOperationListThread() :  Thread ddi  la cration des fichiers contenant les Functions. */
/***************************************************************************************************/
static DWORD SaveFunctionHistoryThread(void *file_path)
{
  FILE *fd_statistic;
  int i, nb_fonction_history, file_id, has_simple_function_call;
  char buffer_application[BUFFER_SIZE];
  char buffer_call_number[BUFFER_SIZE];
  char buffer_info[BUFFER_SIZE];
  char buffer_name[BUFFER_SIZE];
  char buffer_offset[BUFFER_SIZE];
  char buffer_type[BUFFER_SIZE];
  char file_path_full[1024];
  char file_name[1024];
  char file_name_full[1024];
  char buffer[1024];
  struct fonction_history *current_fonction_history;
  struct window_main *window_main;
  struct window_statistic *window_statistic;
  struct window_progressbar *window_progressbar;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Rcupre la fentre Main */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  my_Memory(MEMORY_GET_WINDOW_STATISTIC,&window_statistic,NULL);

  /* Init */
  fd_statistic = NULL;

  /* Extrait le nom du fichier */
  strcpy(file_name,(char *)file_path);
  for(i=(int)strlen((char *)file_path); i>=0; i--)
    if(((char *)file_path)[i] == '\\' || ((char *)file_path)[i] == '/')
      {
        strcpy(file_name,&((char *)file_path)[i+1]);
        break;
      }

  /* On va attendre la cration de la fentre ProgressBar */
  while(1)
    {
      Sleep(1);
      my_Memory(MEMORY_GET_WINDOW_PROGRESSBAR,&window_progressbar,NULL);
      if(window_progressbar != NULL)
        break;
    }

  /** Extrait la liste des fonctions **/
  file_id = 1;
  nb_fonction_history = 1;
  for(current_fonction_history = window_main->current_statistic_table->first_fonction_history; current_fonction_history; current_fonction_history = current_fonction_history->next)
    {
      /* On va faire avancer la ProgressBar */
      if((nb_fonction_history % 1000) == 1)
        {
          EnterCriticalSection(&param->cs_progress_bar);
          my_Memory(MEMORY_GET_WINDOW_PROGRESSBAR,&window_progressbar,NULL);
          if(window_progressbar != NULL)
            SendMessage(window_progressbar->hwnd_progressbar,HM_PROGRESSBAR_BUTTON_INCREMENT,0,0);
          LeaveCriticalSection(&param->cs_progress_bar);
        }

      /** On va crer les fichiers Function : 1 M de lignes / fichier **/
      if((nb_fonction_history % 1000000) == 1)
        {
          /* Fermeture du fichier prcdent */
          if(fd_statistic != NULL)
            fclose(fd_statistic);
          fd_statistic = NULL;

          /* Nom du fichier */
          if(window_main->current_statistic_table->nb_fonction_history < 1000000)
            {
              sprintf(file_path_full,"%s.txt",file_path);
              sprintf(file_name_full,"%s.txt",file_name);
            }
          else
            {
              sprintf(file_path_full,"%s_%04d.txt",file_path,file_id);
              sprintf(file_name_full,"%s_%04d.txt",file_name,file_id);
            }
          file_id++;

          /* Cration du fichier */
          fd_statistic = fopen(file_path_full,"w");
          if(fd_statistic == NULL)
            {
              MessageBox(window_statistic->hwnd_statistic,"Impossible to create File","Error",MB_OK|MB_ICONWARNING);

              /* Fermeture de la fentre ProgressBar */
              EnterCriticalSection(&param->cs_progress_bar);
              my_Memory(MEMORY_GET_WINDOW_PROGRESSBAR,&window_progressbar,NULL);
              if(window_progressbar != NULL)
                SendMessage(window_progressbar->hwnd_progressbar,HM_PROGRESSBAR_BUTTON_CANCEL,0,0);
              LeaveCriticalSection(&param->cs_progress_bar);

              /* Fin du Thread */
              ExitThread(0);
              return(0);
            }

          /* Met  jour le Prompt de la fentre Progress */
          sprintf(buffer,"Write Function List into file '%s' :",file_name_full);
          EnterCriticalSection(&param->cs_progress_bar);
          my_Memory(MEMORY_GET_WINDOW_PROGRESSBAR,&window_progressbar,NULL);
          if(window_progressbar != NULL)
            SendMessage(window_progressbar->hwnd_progressbar,HM_PROGRESSBAR_UPDATE_PROMPT,0,(LPARAM)&buffer[0]);
          LeaveCriticalSection(&param->cs_progress_bar);

          /* Ligne entte */
          fprintf(fd_statistic,"Global Operation   Total Cycle Count  Type              Application                Name                       @Exit   #Pull   #Op   #Cycle   @From   Depth    Function Call Tree\n");
        }
      nb_fonction_history++;

      /** Dtecte un appel simple de fonction **/
      has_simple_function_call = 0;
      if(current_fonction_history->type == FUNCTION_HISTORY_TYPE_ENTER_FUNCTION && current_fonction_history->next != NULL)
        if(current_fonction_history->next->type == FUNCTION_HISTORY_TYPE_EXIT && current_fonction_history->function_address == current_fonction_history->next->function_address)
          has_simple_function_call = 1;

      /** Enter+Exit Simple Function **/
      if(has_simple_function_call == 1)
        {
          /* GOID */
          fprintf(fd_statistic,"%08X:%08X  ",(DWORD) ((current_fonction_history->goid >> 32) & 0x00000000FFFFFFFF),(DWORD) (current_fonction_history->goid & 0x00000000FFFFFFFF));

          /* Cycle */
          fprintf(fd_statistic,"%08X:%08X  ",(DWORD) ((current_fonction_history->cycle >> 32) & 0x00000000FFFFFFFF),(DWORD) (current_fonction_history->cycle & 0x00000000FFFFFFFF));

          /* Name */
          GetNameFromAddress(current_fonction_history->function_address,current_fonction_history->call_number,buffer_name,buffer_type,buffer_application);
          strcat(buffer_name,"                         ");
          buffer_name[25] = '\0';
          strcat(buffer_type,"                ");
          buffer_type[16] = '\0';
          strcat(buffer_application,"                         ");
          buffer_application[25] = '\0';
          sprintf(buffer_info,"%s::%s::%s",buffer_type,buffer_application,buffer_name);
          fprintf(fd_statistic,"%s ",buffer_info);

          /* @Exit */
          fprintf(fd_statistic,"$%02X/%04X ",(current_fonction_history->function_end_address >> 16) & 0x000000FF,current_fonction_history->function_end_address & 0x0000FFFF);

          /* # Pull */
          fprintf(fd_statistic,"%2d ",current_fonction_history->next->nb_unstack);

          /* # Operation */
          fprintf(fd_statistic,"%8d ",current_fonction_history->next->nb_operation);

          /* # Cycle */
          fprintf(fd_statistic,"%8d ",current_fonction_history->next->nb_cycle);

          /* PB/PC */
          fprintf(fd_statistic," $%02X/%04X ",current_fonction_history->prev_operation_PB,current_fonction_history->prev_operation_PC);

          /* Depth */
          fprintf(fd_statistic,"[%2d] :  ",current_fonction_history->depth);
          strcpy(buffer_offset,"");
          for(i=0; i<MY_MIN(current_fonction_history->depth,30); i++)
            strcat(buffer_offset,"  ");
          fprintf(fd_statistic,"%s ",buffer_offset);

          /* Function Address */
          fprintf(fd_statistic,"o $%02X/%04X ",(current_fonction_history->function_address >> 16) & 0x000000FF,current_fonction_history->function_address & 0x0000FFFF);

          /* Call Number */
          if(current_fonction_history->call_number == 0xFFFF)
            strcpy(buffer_call_number,"    ");
          else
            sprintf(buffer_call_number,"%04X",current_fonction_history->call_number);
          fprintf(fd_statistic,"%s ",buffer_call_number);

          /* Fin de ligne */
          fprintf(fd_statistic,"\n");

          /* Saute la fonction suivante */
          current_fonction_history = current_fonction_history->next;
        }
      /** Enter Function / Interrupt **/
      else if(current_fonction_history->type == FUNCTION_HISTORY_TYPE_ENTER_FUNCTION || current_fonction_history->type == FUNCTION_HISTORY_TYPE_ENTER_INTERRUPT)
        {
          /* Enter Interrupt */
          if(current_fonction_history->type == FUNCTION_HISTORY_TYPE_ENTER_INTERRUPT)
            fprintf(fd_statistic,"----------------------------------------------------------------------------- Enter Interrupt -----------------------------------------------------------------------------\n");

          /* GOID */
          fprintf(fd_statistic,"%08X:%08X  ",(DWORD) ((current_fonction_history->goid >> 32) & 0x00000000FFFFFFFF),(DWORD) (current_fonction_history->goid & 0x00000000FFFFFFFF));

          /* Cycle */
          fprintf(fd_statistic,"%08X:%08X  ",(DWORD) ((current_fonction_history->cycle >> 32) & 0x00000000FFFFFFFF),(DWORD) (current_fonction_history->cycle & 0x00000000FFFFFFFF));

          /* Name */
          GetNameFromAddress(current_fonction_history->function_address,current_fonction_history->call_number,buffer_name,buffer_type,buffer_application);
          strcat(buffer_name,"                         ");
          buffer_name[25] = '\0';
          strcat(buffer_type,"                ");
          buffer_type[16] = '\0';
          strcat(buffer_application,"                         ");
          buffer_application[25] = '\0';
          sprintf(buffer_info,"%s::%s::%s",buffer_type,buffer_application,buffer_name);
          fprintf(fd_statistic,"%s ",buffer_info);

          /* @Exit */
          fprintf(fd_statistic,"$%02X/%04X ",(current_fonction_history->function_end_address >> 16) & 0x000000FF,current_fonction_history->function_end_address & 0x0000FFFF);

          /* # Pull */
          fprintf(fd_statistic,"   ");

          /* # Operation */
          fprintf(fd_statistic,"         ");

          /* # Cycle */
          fprintf(fd_statistic,"         ");

          /* PB/PC */
          fprintf(fd_statistic," $%02X/%04X ",current_fonction_history->prev_operation_PB,current_fonction_history->prev_operation_PC);

          /* Depth */
          fprintf(fd_statistic,"[%2d] :  ",current_fonction_history->depth);
          strcpy(buffer_offset,"");
          for(i=0; i<MY_MIN(current_fonction_history->depth,30); i++)
            strcat(buffer_offset,"  ");
          fprintf(fd_statistic,"%s ",buffer_offset);

          /* Function Address */
          fprintf(fd_statistic,"> $%02X/%04X ",(current_fonction_history->function_address >> 16) & 0x000000FF,current_fonction_history->function_address & 0x0000FFFF);

          /* Call Number */
          if(current_fonction_history->call_number == 0xFFFF)
            strcpy(buffer_call_number,"    ");
          else
            sprintf(buffer_call_number,"%04X",current_fonction_history->call_number);
          fprintf(fd_statistic,"%s ",buffer_call_number);

          /* Fin de ligne */
          fprintf(fd_statistic,"\n");
        }
      /** Exit Function / Interrupt **/
      else if(current_fonction_history->type == FUNCTION_HISTORY_TYPE_EXIT)
        {
          /* GOID */
          fprintf(fd_statistic,"%08X:%08X  ",(DWORD) ((current_fonction_history->goid >> 32) & 0x00000000FFFFFFFF),(DWORD) (current_fonction_history->goid & 0x00000000FFFFFFFF));

          /* Cycle */
          fprintf(fd_statistic,"%08X:%08X  ",(DWORD) ((current_fonction_history->cycle >> 32) & 0x00000000FFFFFFFF),(DWORD) (current_fonction_history->cycle & 0x00000000FFFFFFFF));

          /* Name */
          GetNameFromAddress(current_fonction_history->function_address,current_fonction_history->call_number,buffer_name,buffer_type,buffer_application);
          strcat(buffer_name,"                         ");
          buffer_name[25] = '\0';
          strcat(buffer_type,"                ");
          buffer_type[16] = '\0';
          strcat(buffer_application,"                         ");
          buffer_application[25] = '\0';
          sprintf(buffer_info,"%s::%s::%s",buffer_type,buffer_application,buffer_name);
          fprintf(fd_statistic,"%s ",buffer_info);

          /* @Exit */
          fprintf(fd_statistic,"         ");

          /* # Pull */
          fprintf(fd_statistic,"%2d ",current_fonction_history->nb_unstack);

          /* # Operation */
          fprintf(fd_statistic,"%8d ",current_fonction_history->nb_operation);

          /* # Cycle */
          fprintf(fd_statistic,"%8d ",current_fonction_history->nb_cycle);

          /* PB/PC */
          fprintf(fd_statistic," $%02X/%04X ",current_fonction_history->operation_PB,current_fonction_history->operation_PC);

          /* Depth */
          fprintf(fd_statistic,"[%2d] :  ",current_fonction_history->depth);
          strcpy(buffer_offset,"");
          for(i=0; i<MY_MIN(current_fonction_history->depth,30); i++)
            strcat(buffer_offset,"  ");
          fprintf(fd_statistic,"%s ",buffer_offset);

          /* Function Address */
          fprintf(fd_statistic,"< $%02X/%04X ",(current_fonction_history->function_address >> 16) & 0x000000FF,current_fonction_history->function_address & 0x0000FFFF);

          /* Call Number */
          if(current_fonction_history->call_number == 0xFFFF)
            strcpy(buffer_call_number,"    ");
          else
            sprintf(buffer_call_number,"%04X",current_fonction_history->call_number);
          fprintf(fd_statistic,"%s ",buffer_call_number);

          /* # Exit */
          fprintf(fd_statistic,"%s",(current_fonction_history->nb_exit == 1)?"":"   <<<<< MULTIPLE EXIT >>>>>");

          /* Fin de ligne */
          fprintf(fd_statistic,"\n");

          /* Exit Interrupt */
          if(current_fonction_history->exit_interrupt == 1)
            fprintf(fd_statistic,"----------------------------------------------------------------------------- Exit Interrupt ------------------------------------------------------------------------------\n");
        }

      /* La ProgressBar a t'elle t ferme ? */
      my_Memory(MEMORY_GET_WINDOW_PROGRESSBAR,&window_progressbar,NULL);
      if(window_progressbar == NULL)
        break;
    }

  /* Fermeture du fichier */
  if(fd_statistic != NULL)
    fclose(fd_statistic);

  /* Fermeture de la fentre ProgressBar */
  EnterCriticalSection(&param->cs_progress_bar);
  my_Memory(MEMORY_GET_WINDOW_PROGRESSBAR,&window_progressbar,NULL);
  if(window_progressbar != NULL)
    SendMessage(window_progressbar->hwnd_progressbar,HM_PROGRESSBAR_BUTTON_CANCEL,0,0);
  LeaveCriticalSection(&param->cs_progress_bar);

  /* Fin du Thread */
  ExitThread(0);
  return(0);

}


/*********************************************************************************/
/*  RefreshStatisticWindow() :  Mise jour des entres de la fentre  Statistic. */
/*********************************************************************************/
static void RefreshStatisticWindow(struct window_statistic *window_statistic)
{
  int i;
  char buffer[BUFFER_SIZE];
  struct window_main *window_main;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Aucune entre slectionne */
  window_statistic->selected_statistic_address = 0xFFFFFFFF;
  EnableWindow(GetDlgItem(window_statistic->hwnd_statistic,IDC_BUTTON_STATISTIC_DELETE),FALSE);

  /* Rcupre la fentre Main */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  if(window_main == NULL)
    return;

  /** Supprime toutes les entres du Listview **/
  if(window_statistic != NULL)
    ListView_DeleteAllItems(window_statistic->listview_statistic.hwnd_listview);

  /** On refait le Listview avec les Statistic de la Table **/
  if(window_statistic != NULL)
    FillStatisticListView(&window_statistic->listview_statistic,window_main->current_statistic_table);

  /** RAZ des Tris **/
  for(i=0; i<window_statistic->listview_statistic.nb_column; i++)
    window_statistic->listview_statistic.tab_column[i].sort = INTERNALS_COLUMN_SORT_NONE;
  /* MAJ des images ^ / v des Tris des colonnes */
  setListViewSortIcon(&window_statistic->listview_statistic);

  /* MAJ du nombre de cycle */
  if(window_main->current_statistic_table != NULL)
    {
      /* Total Call */
      sprintf(buffer,"%d",window_main->current_statistic_table->total_call_statistic_func);
      PunctuateAtThoudand(buffer);
      SetWindowText(GetDlgItem(window_statistic->hwnd_statistic,IDC_STATIC_TOTAL_FUNCTION),buffer);

      /* Total Cycle (64 bit) */
      sprintf(buffer,"%I64d",window_main->current_statistic_table->total_cycle_statistic_func);
      PunctuateAtThoudand(buffer);
      SetWindowText(GetDlgItem(window_statistic->hwnd_statistic,IDC_STATIC_TOTAL_CYCLE),buffer);
    }

  /* MAJ du titre de la fentre avec le nombre de Statistic */
  if(window_main->current_statistic_table != NULL && window_statistic != NULL)
    {
      sprintf(buffer," Function Statistics Table : %d entries",window_main->current_statistic_table->nb_statistic_func);
      SetWindowText(window_statistic->hwnd_statistic,buffer);
    }
}


/**********************************************************/
/*  ClearStatisticTable() :  Vide la table des Statistic. */
/**********************************************************/
void ClearStatisticTable(struct window_statistic *window_statistic)
{
  struct window_main *window_main;

  /* Rcupre la fentre Main */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  if(window_main == NULL)
    return;

  /* Rien  faire */
  if(window_main->current_statistic_table == NULL)
    return;

  /* Libration mmoire */
  mem_free_statistic_table(window_main->current_statistic_table);
  window_main->current_statistic_table = NULL;

  /** Supprime toutes les entres du Listview **/
  ListView_DeleteAllItems(window_statistic->listview_statistic.hwnd_listview);

  /* MAJ du titre de la fentre avec le nombre de Statistic */
  SetWindowText(window_statistic->hwnd_statistic," Function Statistics Table : 0 entry");

  /* MAJ du nombre de total d'appel */
  SetWindowText(GetDlgItem(window_statistic->hwnd_statistic,IDC_STATIC_TOTAL_FUNCTION),"0");

  /* MAJ du nombre de total de cycle */
  SetWindowText(GetDlgItem(window_statistic->hwnd_statistic,IDC_STATIC_TOTAL_CYCLE),"0");
}


/*********************************************************************************/
/*  GetStatisticByAddress() :  Recherche une Statistic Function via son adresse. */
/*********************************************************************************/
static struct statistic_function *GetStatisticByAddress(DWORD address, WORD call_number)
{
  struct statistic_function local_statistic_func;
  struct statistic_function *local_statistic_func_ptr;
  struct statistic_function **found_statistic_func_ptr;
  struct statistic_function *current_statistic_func;
  struct window_main *window_main;

  /* Rcupre la fentre Main */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  if(window_main == NULL)
    return(NULL);
  if(window_main->current_statistic_table == NULL)
    return(NULL);
  if(window_main->current_statistic_table->tab_statistic_func == NULL)
    return(NULL);

  /* Recherche dychotomique */
  current_statistic_func = NULL;
  local_statistic_func.address = address;
  local_statistic_func.call_number = call_number;
  local_statistic_func_ptr = &local_statistic_func;
  found_statistic_func_ptr = (struct statistic_function **) bsearch(&local_statistic_func_ptr,(void *)window_main->current_statistic_table->tab_statistic_func,window_main->current_statistic_table->nb_statistic_func,sizeof(struct statistic *),compare_statistic_address);
  if(found_statistic_func_ptr != NULL)
    current_statistic_func = *found_statistic_func_ptr;

  /* Renvoi la Statistic Function */
  return(current_statistic_func);
}


/******************************************************************************/
/*  compare_statistic_address() : Fonction de comparaison pour le Quick Sort. */
/******************************************************************************/
static int compare_statistic_address(const void *data_1, const void *data_2)
{
  WORD call_number_1, call_number_2;
  struct statistic_function *statistic_1;
  struct statistic_function *statistic_2;

  /* Rcupration des paramtres */
  statistic_1 = *((struct statistic_function **) data_1);
  statistic_2 = *((struct statistic_function **) data_2);

  /* Comparaison des adresses | call_number */
  if(statistic_1->address < statistic_2->address)
    return(-1);
  else if(statistic_1->address > statistic_2->address)
    return(1);
  else
    {
      /* On inverse le Call Number pour trier par Numro de Tool */
      call_number_1 = ((statistic_1->call_number >> 8) & 0x00FF) | ((statistic_1->call_number << 8) & 0xFF00);
      call_number_2 = ((statistic_2->call_number >> 8) & 0x00FF) | ((statistic_2->call_number << 8) & 0xFF00);

      if(call_number_1 < call_number_2)
        return(-1);
      else if(call_number_1 > call_number_2)
        return(1);
      else
        return(0);
    }
}


/**************************************************************************/
/*  mem_free_statistic() :  Libration mmoire de la structure statistic. */
/**************************************************************************/
void mem_free_statistic(struct statistic_function *current_statistic)
{
  if(current_statistic)
    {
      free(current_statistic);
    }
}


/**************************************************************************************/
/*  mem_free_statistic_table() :  Libration mmoire de la structure statistic_table. */
/**************************************************************************************/
void mem_free_statistic_table(struct statistic_table *current_statistic_table)
{
  struct statistic_function *current_statistic;
  struct statistic_function *next_statistic;
  struct fonction_history *current_fonction_history;
  struct fonction_history *next_fonction_history;

  if(current_statistic_table)
    {
      /* Libration de la liste chaine Statistic */
      for(current_statistic = current_statistic_table->first_statistic_func; current_statistic; current_statistic = next_statistic)
        {
          next_statistic = current_statistic->next;
          mem_free_statistic(current_statistic);
        }
      if(current_statistic_table->tab_statistic_func)
        free(current_statistic_table->tab_statistic_func);

      /* Libartion de la liste chane History */
      for(current_fonction_history = current_statistic_table->first_fonction_history; current_fonction_history; current_fonction_history = next_fonction_history)
        {
          next_fonction_history = current_fonction_history->next;
          free(current_fonction_history);
        }

      free(current_statistic_table);
    }
}


/**********************************************************************/
/*  UserResizeStatisticWindow() :  Retaille la fentre des Statistic. */
/**********************************************************************/
static void UserResizeStatisticWindow(struct window_statistic *window_statistic)
{
  RECT window_rect;
  int window_x, window_y, window_new_height, minimum_window_height;
  int listview_pixel_height, nb_listview_data_row, delta_listview_nb_row, delta_pixel_height;

  /* Position + Dimension actuelle de la fentre retaille dans l'cran */
  GetWindowRect(window_statistic->hwnd_statistic,&window_rect);

  /* Nouvelle Dimension de la fentre (avec la barre de titre et les bordures) */
  window_x = window_rect.left;
  window_y = window_rect.top;
  window_new_height = window_rect.bottom - window_rect.top;

  /* Nombre de Pixels restant pour la ListView */
  minimum_window_height = window_statistic->window_init_height - window_statistic->nb_init_data_row*SYMBOL_LISTVIEW_ROW_HEIGHT;
  listview_pixel_height = window_new_height - minimum_window_height;

  /* Nombre de lignes de la ListView */
  nb_listview_data_row = listview_pixel_height / SYMBOL_LISTVIEW_ROW_HEIGHT;

  /** Teste les valeurs minimales autorises **/
  if(nb_listview_data_row < SYMBOL_LISTVIEW_MIN_NB_ROW)
    nb_listview_data_row = SYMBOL_LISTVIEW_MIN_NB_ROW;

  /* Delta de vla ListView */
  delta_listview_nb_row = nb_listview_data_row - window_statistic->nb_row;
  delta_pixel_height = delta_listview_nb_row*SYMBOL_LISTVIEW_ROW_HEIGHT;
  window_statistic->nb_row = nb_listview_data_row;

  /* Nouvelle hauteur de la fentre (on ajuste pour tomber sur un nombre extact de ligne dans la ListView) */
  window_new_height = minimum_window_height + nb_listview_data_row*SYMBOL_LISTVIEW_ROW_HEIGHT;

  /* Retaille la fentre */
  window_statistic->user_resize = 0;
  MoveWindow(window_statistic->hwnd_statistic,window_x,window_y,window_statistic->window_init_width,window_new_height,FALSE);

  /* Dplace les Contrles */
  MoveStatisticWindowControls(delta_pixel_height,window_statistic);

  /* Rafraichit la fentre */
  InvalidateRect(window_statistic->hwnd_statistic,NULL,TRUE);
  UpdateWindow(window_statistic->hwnd_statistic);
}


/********************************************************************************************/
/*  MoveStatisticWindowControls() :  On va dplacer les diffrents controles de la fentre. */
/********************************************************************************************/
static void MoveStatisticWindowControls(int delta_y, struct window_statistic *window_statistic)
{
  HWND hwnd_control, hwnd_listview;
  int i, control_x, control_y, control_width, control_height;
  int listview_x, listview_y, listview_width, listview_height;
  int tab_control_id[] = 
    {
      IDC_STATIC_REFRESH_MODE,   IDC_RADIO_REFRESH_AUTOMATIC, IDC_RADIO_REFRESH_MANUAL,
      IDC_BUTTON_REFRESH,        IDC_BUTTON_STATISTIC_CLEAR,  IDC_STATIC_TOTAL,
      IDC_STATIC_TOTAL_FUNCTION, IDC_STATIC_TOTAL_CYCLE,      IDC_BUTTON_STATISTIC_DELETE,
      IDC_BUTTON_SAVE_LIST,      IDC_BUTTON_SAVE_TABLE,
      0
    };

  /** On va modifier la hauteur de la ListBox **/
  /* Handle du Controle */
  hwnd_listview = window_statistic->listview_statistic.hwnd_listview;
  /* Position + Taille actuelle */
  GetWindowControlInfo(window_statistic->hwnd_statistic,hwnd_listview,&listview_x,&listview_y,&listview_width,&listview_height);
  /* Change la taille le controle verticalement */
  MoveWindow(hwnd_listview,listview_x,listview_y,listview_width,listview_height+delta_y,FALSE);

  /** On va dplacer tous les controles verticalement **/
  for(i=0; tab_control_id[i] != 0; i++)
    {
      /* Handle du Controle */
      hwnd_control = GetDlgItem(window_statistic->hwnd_statistic,tab_control_id[i]);

      /* Position + Taille actuelle */
      GetWindowControlInfo(window_statistic->hwnd_statistic,hwnd_control,&control_x,&control_y,&control_width,&control_height);

      /* Dplace le controle verticalement */
      MoveWindow(hwnd_control,control_x,control_y+delta_y,control_width,control_height,FALSE);
    }
}


/***********************************************************************/
/*  DeleteStatisticChildWindows() :  Destruction des fentres enfants. */
/***********************************************************************/
static void DeleteStatisticChildWindows(struct window_statistic *window_statistic)
{
}


/*************************************************************************/
/*  mem_free_window_statistic() :  Libration mmoire pour la structure. */
/*************************************************************************/
void mem_free_window_statistic(struct window_statistic *window_statistic)
{
  if(window_statistic)
    {
      /* Supprime la notification */
      my_Notification(NOTIFICATION_REMOVE,window_statistic->current_notification,NULL);
      window_statistic->current_notification = NULL;

      free(window_statistic);
    }
}

/**********************************************************************/
