/***********************************************************************/
/*                                                                     */
/*  Window_Symbol.c :  Module de gestion de la fentre Symbol.         */
/*                                                                     */
/***********************************************************************/
/*  Auteur : Olivier ZARDINI  *   Brutal Deluxe   *   Date : Oct 2023  */
/***********************************************************************/

#include <stdio.h>
#include <time.h>

#include <windows.h>
#include <windowsx.h>
#include <Commctrl.h>
#include <Winuser.h>
#include <Richedit.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_Notification.h"
#include "Dc_Graphic.h"
#include "Dc_CPU.h"
#include "Dc_MMU.h"
#include "Dc_IIgsInternals.h"
#include "Cyrene_Engine.h"
#include "Window_QuickWatch.h"
#include "Window_Main.h"
#include "Window_Main_Menu.h"
#include "Window_Disassembler.h"
#include "Window_MemoryArea.h"
#include "Window_Symbol.h"

static DWORD BuildSymbolWindowThread(void *);
LRESULT CALLBACK ProcHandlerSymbolWindow(HWND,UINT,WPARAM,LPARAM);
static int CreateSymbolChildWindows(HWND,struct window_symbol *);
static void BuildSymbolListView(struct list_view *,HFONT,int,struct window_symbol *);
static void FillSymbolListView(struct list_view *,struct symbol_table *);
static struct symbol *LoadSymbolFile(char *,struct reloc **);
static struct symbol *DecodeOneSymbol(char *,char *, struct segment *);
static struct reloc *DecodeOneReloc(char *,char *,struct segment *);
static int RelocateOneSymbol(struct symbol *);
static int RelocateOneReloc(struct reloc *);
static void RelocateSymbolSegment(struct window_symbol *);
static int InsertUniqueSymbol(struct symbol *,struct symbol_table *);
static int InsertUniqueReloc(struct reloc *,struct symbol_table *);
static void UpdateSymbolExtraInformation(struct window_symbol *,struct symbol *);
static void BuildValidSymbolTable(struct symbol_table *);
static void BuildValidRelocTable(struct symbol_table *);
static struct segment *BuildSegmentList(char *);
static void UserResizeSymbolWindow(struct window_symbol *);
static void MoveSymbolWindowControls(int,struct window_symbol *);
static void DeleteSymbolChildWindows(struct window_symbol *);
static int compare_symbol_address(const void *,const void *);
static int compare_symbol_name(const void *,const void *);
static int compare_reloc_address(const void *,const void *);


/**************************************************************************/
/*  BuildSymbolWindow() :  Construction / Affichage de la fentre Symbol. */
/**************************************************************************/
struct window_symbol *BuildSymbolWindow(HINSTANCE hInstance, int menu_id)
{
  struct window_symbol *window_symbol;

  /** Allocation de la structure window_symbol **/
  window_symbol = (struct window_symbol *) calloc(1,sizeof(struct window_symbol));
  if(window_symbol == NULL)
    return(NULL);
  my_Memory(MEMORY_SET_WINDOW_SYMBOL,window_symbol,NULL);

  /* Lance un thread pour grer la fentre Symbol */
  CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)BuildSymbolWindowThread,window_symbol,0,NULL);

  /* Renvoi la structure mmoire grant la fentre */
  return(window_symbol);
}


/************************************************************************************/
/*  BuildSymbolWindowThread() :  Cration de la fentre symbol via un Thread ddi. */
/************************************************************************************/
static DWORD BuildSymbolWindowThread(void *window_symbol_param)
{
  HWND hwndSymbol;
  MSG msg;
  BOOL bRet;
  struct window_main *window_main;
  struct window_symbol *window_symbol;
  struct parameter *param;

  /* Init */
  window_symbol = (struct window_symbol *) window_symbol_param;
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Cration de la fentre Symbol **/
  hwndSymbol = CreateDialog(param->hInstance,MAKEINTRESOURCE(IDD_WINDOW_SYMBOL),window_main->hwnd_main,(DLGPROC)ProcHandlerSymbolWindow);
  if(hwndSymbol == NULL)
    return(1);

  /** Show Window + Paint Content **/
  ShowWindow(hwndSymbol,SW_SHOWDEFAULT);
  UpdateWindow(hwndSymbol);

  /** Message Loop pour toutes les fentres de ce Thread (fentre Symbol...) **/
  while((bRet = GetMessage(&msg,NULL,0,0)) != 0)
    {
      if(bRet == -1)
        break;

      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }

  /* Fin du Thread */
  ExitThread(0);
}


/***********************************************************************/
/*  ProcHandlerSymbolWindow() :  Gestion des vnements de la fentre. */
/***********************************************************************/
LRESULT CALLBACK ProcHandlerSymbolWindow(HWND hwndSymbol, UINT umsg, WPARAM wParam, LPARAM lParam)
{
  int i, is_error, select_id, symbol_id;
  UINT code, code_beginlabeledit, code_endlabeledit, code_getdispinfo, code_setdispinfo;
  DWORD symbol_address;
  BOOL result;
  NMLVDISPINFO *plvdi;
  LVITEM LvItem;
  NMLISTVIEW *pListView;
  char buffer_symbol[256];
  char buffer_address[1024];
  char buffer_type[1024];
  struct list_view *current_listview;
  struct window_symbol *window_symbol;
  struct window_main *window_main;
  struct parameter *param;
  struct symbol *current_symbol;
  
  /* Init */
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  my_Memory(MEMORY_GET_WINDOW_SYMBOL,&window_symbol,NULL);
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);

  switch(umsg)
    {
      case WM_INITDIALOG :
        /* Cration du contenu de la DialogBox */
        CreateSymbolChildWindows(hwndSymbol,window_symbol);
        return(TRUE);
        
      /****************************/
      /* Evnements de la fentre */
      /****************************/
      case WM_COMMAND :
        switch(LOWORD(wParam))
          {
            /** Bouton : Charge un fichier Symbol **/
            case IDC_BUTTON_SYMBOL_LOAD :
              if(HIWORD(wParam) == BN_CLICKED)
                LoadAppendSymbolFile(NULL,window_symbol);
              break;

            /** Bouton : Vide la table des Symbol **/
            case IDC_BUTTON_SYMBOL_CLEAR :
              if(HIWORD(wParam) == BN_CLICKED)
                ClearSymbolTable(window_symbol);
              break;

            /** Bouton pour reloger le Segment slectionn **/
            case IDC_BUTTON_SYMBOL_RELOCATE :
              if(HIWORD(wParam) == BN_CLICKED)
                RelocateSymbolSegment(window_symbol);
              break;
          }
        return(TRUE);

      /********************************************/
      /* Evnements des CommonControls (ListView) */
      /********************************************/
      case WM_NOTIFY :
        code = ((LPNMHDR)lParam)->code;
        code_beginlabeledit = LVN_BEGINLABELEDIT;
        code_endlabeledit   = LVN_ENDLABELEDIT;
        code_getdispinfo    = LVN_GETDISPINFO;
        code_setdispinfo    = LVN_SETDISPINFO;

        /*****************************************************************************************************/
        /**  Affichage des Item de la fentre : Remplissage des Item dans les colonnes des lignes affiches **/
        /*****************************************************************************************************/
        if(code == LVN_GETDISPINFO)
          {
            plvdi = (NMLVDISPINFO *) lParam;

            /* Identifie la ListView : Breakpoint / Symbols */
            if(LOWORD(wParam) == window_symbol->listview_symbol.idc_listview)
              current_listview = (struct list_view *) &window_symbol->listview_symbol;
            else
              break;

            /* Valide le numro de la colonne */
            if(plvdi->item.iSubItem < current_listview->nb_column)
              {
                plvdi->item.pszText = current_listview->tab_column[plvdi->item.iSubItem].tab_value_txt[(current_listview->has_column_sort == INTERNALS_COLUMN_SORT_NONE) ? plvdi->item.iItem : plvdi->item.lParam];
              }
          }

        /***************************************************************/
        /**  Click sur une ligne d'une ListView pour la slectionner  **/
        /***************************************************************/
        if(code == NM_CLICK)
          {
            /* Rcupre la ListView (Breakpoint / Symbol) */
            if(LOWORD(wParam) == window_symbol->listview_symbol.idc_listview)
              current_listview = (struct list_view *) &window_symbol->listview_symbol;
            else
              break;

            /* Identifie la ligne clicke (0-N) */
            select_id = (int) SendMessage(current_listview->hwnd_listview,LVM_GETNEXTITEM,-1,LVNI_FOCUSED); // return item selected
            if(select_id == -1)
              return(TRUE);

            /* Rcupre le numro de la structure Symbol dans la Colonne #8 */
            memset(&LvItem,0,sizeof(LVITEM));
            LvItem.mask = LVIF_TEXT;
            LvItem.iSubItem = 8;            /* Colonne #8 */
            LvItem.pszText = buffer_symbol;
            LvItem.cchTextMax = 256;
            LvItem.iItem = select_id;
            SendMessage(current_listview->hwnd_listview,LVM_GETITEMTEXT,select_id,(LPARAM)&LvItem);
            symbol_id = atoi(buffer_symbol);

            /** On remplir les informations complmentaires du Symbol **/
            if(symbol_id < window_main->current_symbol_table->nb_symbol)
              {
                for(i=0,current_symbol = window_main->current_symbol_table->first_symbol; current_symbol; current_symbol=current_symbol->next,i++)
                  if(i == symbol_id)
                    {
                      UpdateSymbolExtraInformation(window_symbol,current_symbol);
                      break;
                    }
              }
          }
       
        /*************************************************************************/
        /**  Double-click sur une ligne d'une ListView pour activer une action  **/
        /*************************************************************************/
        if(code == NM_DBLCLK)
          {
            /* Rcupre la ListView (Breakpoint / Symbols) */
            if(LOWORD(wParam) == window_symbol->listview_symbol.idc_listview)
              current_listview = (struct list_view *) &window_symbol->listview_symbol;
            else
              break;

            /* Identifie la ligne clicke (0-N) */
            select_id = (int) SendMessage(current_listview->hwnd_listview,LVM_GETNEXTITEM,-1,LVNI_FOCUSED); // return item selected
            if(select_id == -1)
              return(TRUE);

            /* Rcupre l'adresse du Symbol dans la Colonne #1 */
            memset(&LvItem,0,sizeof(LVITEM));
            LvItem.mask = LVIF_TEXT;
            LvItem.iSubItem = 1;            /* Colonne #1 */
            LvItem.pszText = buffer_address;
            LvItem.cchTextMax = 256;
            LvItem.iItem = select_id;
            SendMessage(current_listview->hwnd_listview,LVM_GETITEMTEXT,select_id,(LPARAM)&LvItem);
            symbol_address = EvaluateAsAddress(buffer_address,&is_error);

            /* Rcupre le type du Symbol dans la Colonne #3 */
            memset(&LvItem,0,sizeof(LVITEM));
            LvItem.mask = LVIF_TEXT;
            LvItem.iSubItem = 3;            /* Colonne #3 */
            LvItem.pszText = buffer_type;
            LvItem.cchTextMax = 256;
            LvItem.iItem = select_id;
            SendMessage(current_listview->hwnd_listview,LVM_GETITEMTEXT,select_id,(LPARAM)&LvItem);

            /** On va ouvrir une fentre de dsassemblage | Fentre de Data **/
            if(!stricmp(buffer_type,"Code"))
              BuildDisassemblerWindow(param->hInstance,ID_MENU_VIEW_CODEDISASSEMBLER,symbol_address);
            else if(!strnicmp(buffer_type,"Data",strlen("Data")))
              BuildMemoryAreaWindow(param->hInstance,ID_MENU_VIEW_MEMORYAREA,symbol_address);
          }

        /***************************************************************/
        /**  Click sur les titres des colonnes pour activer les tris  **/
        /***************************************************************/
        if(code == LVN_COLUMNCLICK)
          {
            pListView = (NMLISTVIEW *) lParam;
            select_id = (int) pListView->iSubItem;

            /* Rcupre la ListView (Breakpoint / Symbols) */
            if(LOWORD(wParam) == window_symbol->listview_symbol.idc_listview)
              current_listview = (struct list_view *) &window_symbol->listview_symbol;
            else
              break;

            /* On fait le choix du type de tri */
            for(i=0; i<current_listview->nb_column; i++)
              {
                if(i == select_id)
                  {
                    if(current_listview->tab_column[i].sort == INTERNALS_COLUMN_SORT_NONE)
                      current_listview->tab_column[i].sort = INTERNALS_COLUMN_SORT_UP;
                    else if(current_listview->tab_column[i].sort == INTERNALS_COLUMN_SORT_UP)
                      current_listview->tab_column[i].sort = INTERNALS_COLUMN_SORT_DOWN;
                    else
                      current_listview->tab_column[i].sort = INTERNALS_COLUMN_SORT_NONE;
                  }
               else
                 current_listview->tab_column[i].sort = INTERNALS_COLUMN_SORT_NONE;
              }
             
            /* A t'on une colonne trie ? */
            current_listview->has_column_sort = INTERNALS_COLUMN_SORT_NONE;
            for(i=0; i<current_listview->nb_column; i++)
              if(current_listview->tab_column[i].sort != INTERNALS_COLUMN_SORT_NONE)
                current_listview->has_column_sort = current_listview->tab_column[i].sort;
             
            /* On va trier selon le tri mis en place */
            result = ListView_SortItems(current_listview->hwnd_listview,ProcessListViewCompareProc,current_listview);
           
            /* MAJ des images ^ / v des Tris des colonnes */
            setListViewSortIcon(current_listview);
          }

        /***********************************************************************************/
        /**  Gestion de l'affichage des Item pour dfinir une couleur de fond de cellule  **/
        /***********************************************************************************/
        if(code == NM_CUSTOMDRAW)
          {
            SetWindowLongPtr(window_symbol->hwnd_symbol,DWLP_MSGRESULT,(LONG_PTR)ProcessListViewWindowCustomDraw(lParam));
            return(CDRF_NOTIFYITEMDRAW);
          }

        return(TRUE);
        break;

      /**************************************/
      /*  On vient de retailler la fentre  */
      /**************************************/
      case WM_SIZE :
        /* On va tendre la fentre de donne mmoire si c'est un Resize User */
        if(window_symbol->user_resize == 1)
          UserResizeSymbolWindow(window_symbol);
        window_symbol->user_resize = 1;
        return(TRUE);

      /********************************************************************/
      /* Fermeture de window (le WM_CLOSE est appel avant le WM_DESTROY) */
      /********************************************************************/
      case WM_CLOSE :
        /* Fentre Symbol */
        DestroyWindow(hwndSymbol);
        return(0);

      case WM_DESTROY :
        /* Suppression des Fentres */
        DeleteSymbolChildWindows(window_symbol);
        CheckMainWindowMenuItem(ID_MENU_CODE_SYMBOL,FALSE);
        my_Memory(MEMORY_FREE_WINDOW_SYMBOL,NULL,NULL);
        /* Demande la sortie de la boucle des messages */
        PostQuitMessage(0);
        return(0);
    }

  /* Retourner FALSE pour relayer le message non trait au systme */
  return(FALSE);
}


/***************************************************************************************************/
/*  CreateSymbolChildWindows() : Crer les fentres enfants et remplir la structure window_symbol. */
/***************************************************************************************************/
static int CreateSymbolChildWindows(HWND hwndSymbol, struct window_symbol *window_symbol)
{
  RECT window_rect;
  struct window_main *window_main;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Rcupre la fentre Main */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);

  /** Window Handle **/
  window_symbol->hwnd_symbol = hwndSymbol;

  /* Police de caractre de la fentre (14 points) */
  window_symbol->hFont = CreateFontA(LISTVIEW_FONT_HEIGHT,0,0,0,FW_DONTCARE,FALSE,FALSE,FALSE,ANSI_CHARSET,
                                     OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY,DEFAULT_PITCH|FF_SWISS,"Arial");

  /**************/
  /*** Symbol ***/
  /**************/
  window_symbol->symbol_x0 = 7;
  window_symbol->symbol_y0 = 7;

  /** Cration de la List View **/
  BuildSymbolListView(&window_symbol->listview_symbol,window_symbol->hFont,IDC_LISTVIEW_SYMBOL,window_symbol);

  /** Remplissage du List View si les symbols sont disponibles + MAJ Titre Fentre + MAJ Status Bar **/
  RefreshSymbolWindow(window_symbol);

  /** Taille de la fentre au dmarrage ((avec la barre de titre et les bordures)) **/
  GetWindowRect(window_symbol->hwnd_symbol,&window_rect);
  window_symbol->window_init_width  = window_rect.right  - window_rect.left;
  window_symbol->window_init_height = window_rect.bottom - window_rect.top;

  /* Nombre de ligne dans la ListView au dmarrage de la fentre */
  window_symbol->nb_init_data_row = SYMBOL_LISTVIEW_NB_ROW;

  /* Nombre de lignes qu'il est possible d'afficher dans la ListView */
  window_symbol->nb_row = window_symbol->nb_init_data_row;

  /* OK */
  return(0);
}


/**************************************************************************/
/*  BuildSymbolListView() :  Construction d'une ListView pour les Symbol. */
/**************************************************************************/
static void BuildSymbolListView(struct list_view *current_listview, HFONT hFont, int idc_listview, struct window_symbol *window_symbol)
{
  /** Init de la structure List_view **/
  current_listview->idc_listview = idc_listview;

  /** Liste des Colonnes **/
  current_listview->nb_column = 0;

  /* Col 0 : Validit */
  current_listview->tab_column[0].type = INTERNALS_COLUMN_TYPE_TXT;
  current_listview->tab_column[0].align = LVCFMT_CENTER;
  current_listview->tab_column[0].length = 30;
  strcpy(current_listview->tab_column[0].title," ");
  current_listview->nb_column++;

  /* Col 1 : Address */
  current_listview->tab_column[1].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[1].align = LVCFMT_LEFT;
  current_listview->tab_column[1].length = 77;
  strcpy(current_listview->tab_column[1].title,"Address");
  current_listview->nb_column++;

  /* Col 2 : Name */
  current_listview->tab_column[2].type = INTERNALS_COLUMN_TYPE_TXT;
  current_listview->tab_column[2].align = LVCFMT_LEFT;
  current_listview->tab_column[2].length = 150;
  strcpy(current_listview->tab_column[2].title,"Name");
  current_listview->nb_column++;

  /* Col 3 : Type / Date Type */
  current_listview->tab_column[3].type = INTERNALS_COLUMN_TYPE_TXT;
  current_listview->tab_column[3].align = LVCFMT_LEFT;
  current_listview->tab_column[3].length = 75;
  strcpy(current_listview->tab_column[3].title,"Type");
  current_listview->nb_column++;

  /* Col 4 : Size */
  current_listview->tab_column[4].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[4].align = LVCFMT_RIGHT;
  current_listview->tab_column[4].length = 60;
  strcpy(current_listview->tab_column[4].title,"Size");
  current_listview->nb_column++;

  /* Col 5 : MX */
  current_listview->tab_column[5].type = INTERNALS_COLUMN_TYPE_TXT;
  current_listview->tab_column[5].align = LVCFMT_LEFT;
  current_listview->tab_column[5].length = 50;
  strcpy(current_listview->tab_column[5].title,"MX");
  current_listview->nb_column++;

  /* Col 6 : Source File Name */
  current_listview->tab_column[6].type = INTERNALS_COLUMN_TYPE_TXT;
  current_listview->tab_column[6].align = LVCFMT_LEFT;
  current_listview->tab_column[6].length = 110;
  strcpy(current_listview->tab_column[6].title,"File");
  current_listview->nb_column++;

  /* Col 7 : Segment Name */
  current_listview->tab_column[7].type = INTERNALS_COLUMN_TYPE_TXT;
  current_listview->tab_column[7].align = LVCFMT_LEFT;
  current_listview->tab_column[7].length = 100;
  strcpy(current_listview->tab_column[7].title,"Segment");
  current_listview->nb_column++;

  /* Col 8 : ID de la structure Symbol */
  current_listview->tab_column[8].type = INTERNALS_COLUMN_TYPE_TXT;
  current_listview->tab_column[8].align = LVCFMT_LEFT;
  current_listview->tab_column[8].length = 1;
  strcpy(current_listview->tab_column[8].title,"");
  current_listview->nb_column++;

  /** Cration du ListView **/
  BuildListView(window_symbol->hwnd_symbol,hFont,window_symbol->symbol_x0,window_symbol->symbol_y0,SYMBOL_LISTVIEW_NB_ROW,current_listview);
}


/************************************************************************/
/*  FillSymbolListView() :  Remplissage d'une ListView pour les Symbol. */
/************************************************************************/
static void FillSymbolListView(struct list_view *current_listview, struct symbol_table *current_symbol_table)
{
  int i, result;
  LVITEM listview_item;
  struct symbol *current_symbol;
  char buffer[1024];

  /* Libration mmoire des donnes affiches dans les colonnes de la ListView */
  mem_free_listviewcolumn(current_listview);

  /* Aucun Symbol */
  if(current_symbol_table == NULL)
    return;
  if(current_symbol_table->nb_symbol == 0)
    return;

  /** Remplissage avec les valeurs dans la structure mmoire **/
  current_listview->nb_row = current_symbol_table->nb_symbol;
  for(i=0; i<current_listview->nb_column; i++)
    {
      /* Allocation des tableaux */
      current_listview->tab_column[i].tab_value_num = (DWORD *) calloc(current_listview->nb_row,sizeof(DWORD));
      current_listview->tab_column[i].tab_value_txt = (char **) calloc(current_listview->nb_row,sizeof(char *));
    }
  /** Remplissage des tableaux **/
  for(i=0, current_symbol=current_symbol_table->first_symbol; current_symbol; current_symbol=current_symbol->next, i++)
    {
      /* Colonne 0 : Validit */
      current_listview->tab_column[0].tab_value_num[i] = current_symbol->is_valid;
      sprintf(buffer,"%s",(current_symbol->is_valid == 1) ? "Ok" : "?");
      current_listview->tab_column[0].tab_value_txt[i] = strdup(buffer);

      /* Colonne 1 : Address Reloge */
      current_listview->tab_column[1].tab_value_num[i] = current_symbol->symbol_relocated_address;
      sprintf(buffer,"%02X/%04X",(current_symbol->symbol_relocated_address >> 16) & 0xFF,current_symbol->symbol_relocated_address & 0xFFFF);
      current_listview->tab_column[1].tab_value_txt[i] = strdup(buffer);

      /* Colonne 2 : Name */
      current_listview->tab_column[2].tab_value_num[i] = i;
      current_listview->tab_column[2].tab_value_txt[i] = strdup(current_symbol->name);

      /* Colonne 3 : Type / Data Type */
      current_listview->tab_column[3].tab_value_num[i] = i;
      strcpy(buffer,current_symbol->type);
      if(strlen(current_symbol->data_type) > 0)
        sprintf(&buffer[strlen(buffer)]," (%s)",current_symbol->data_type);
      current_listview->tab_column[3].tab_value_txt[i] = strdup(buffer);

      /* Colonne 4 : Size */
      current_listview->tab_column[4].tab_value_num[i] = current_symbol->size;
      sprintf(buffer,"%d",current_symbol->size);
      current_listview->tab_column[4].tab_value_txt[i] = strdup(buffer);

      /* Colonne 5 : MX */
      current_listview->tab_column[5].tab_value_num[i] = i;
      sprintf(buffer,"%c%c",current_symbol->m,current_symbol->x);
      current_listview->tab_column[5].tab_value_txt[i] = strdup(buffer);

      /* Colonne 6 : File Name */
      current_listview->tab_column[6].tab_value_num[i] = i;
      current_listview->tab_column[6].tab_value_txt[i] = strdup(current_symbol->program_name);

      /* Colonne 7 : Segment Name */
      current_listview->tab_column[7].tab_value_num[i] = i;
      current_listview->tab_column[7].tab_value_txt[i] = strdup(current_symbol->segment_name);

      /* Colonne 8 : ID de la structure Symbol [0-N] */
      current_listview->tab_column[8].tab_value_num[i] = i;
      sprintf(buffer,"%d",i);
      current_listview->tab_column[8].tab_value_txt[i] = strdup(buffer);
    }

  /** Transfert des donnes dans la List View via un Msg LVN_GETDISPINFO **/
  /** Ajout des Item dans le ListView **/
  listview_item.pszText   = LPSTR_TEXTCALLBACK;           /* Sends an LVN_GETDISPINFO message to fill the item */
  listview_item.mask      = LVIF_TEXT | LVIF_IMAGE | LVIF_STATE | LVIF_PARAM;
  listview_item.stateMask = 0;
  listview_item.iSubItem  = 0;
  listview_item.state     = 0;

  /** Initialize LVITEM members that are different for each item **/
  for(i=0; i<current_listview->nb_row; i++)
    {
      listview_item.iItem  = i;
      listview_item.iImage = -1;         /* Identifiant de l'image dans la liste */
      listview_item.lParam = i;          /* Identifiant renvoy lors du tri */
      result = ListView_InsertItem(current_listview->hwnd_listview,&listview_item);
    }
}


/****************************************************************************************/
/*  LoadAppendSymbolFile() : Charge en mmoire un fichier Symbol et Ajoute les entres. */
/****************************************************************************************/
void LoadAppendSymbolFile(char *symbol_file_path, struct window_symbol *window_symbol)
{
  int i, is_duplicate;
  struct symbol *new_symbol_list;
  struct symbol *current_symbol;
  struct symbol *next_symbol;
  struct reloc *new_reloc_list;
  struct reloc *current_reloc;
  struct reloc *next_reloc;
  char file_path[BUFFER_SIZE];
  char buffer[BUFFER_SIZE];
  struct window_main *window_main;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Rcupre la fentre Main */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  if(window_main == NULL)
    return;

  /* Si besoin, on alloue la structure symbol_table */
  if(window_main->current_symbol_table == NULL)
    {
      window_main->current_symbol_table = (struct symbol_table *) calloc(1,sizeof(struct symbol_table));
      if(window_main->current_symbol_table == NULL)
        return;
    }

  /* Demande le chemin du fichier */
  if(symbol_file_path == NULL)
    {
      GetOneFilePath((window_symbol == NULL)?window_main->hwnd_main:window_symbol->hwnd_symbol,"Select Symbol file","Symbol Files (*.txt)\0*.txt\0\0","",file_path);
      if(strlen(file_path) == 0)
        return;
    }
  else
    strcpy(file_path,symbol_file_path);

  /* Charge en mmoire le fichier (Symbol + Reloc) */
  new_symbol_list = LoadSymbolFile(file_path,&new_reloc_list);

  /*** Ajoute les Symbol  la liste ***/
  for(current_symbol = new_symbol_list; current_symbol; current_symbol = next_symbol)
    {
      /* Isole le Symbol */
      next_symbol = current_symbol->next;
      current_symbol->next = NULL;

      /* Ajoute dans la liste chaine */
      is_duplicate = InsertUniqueSymbol(current_symbol,window_main->current_symbol_table);

      /* On va reloger le Symbol */
      if(is_duplicate == 0)
        RelocateOneSymbol(current_symbol);

      /* Supprime les doublons */
      if(is_duplicate == 1)
        mem_free_symbol(current_symbol);
    }

  /* Protge l'accs aux Symbols */
  EnterCriticalSection(&param->cs_symbol);

  /** Construit un tableau tri par Adresse **/
  window_main->current_symbol_table->nb_symbol_valid = 0;
  if(window_main->current_symbol_table->tab_symbol)
    free(window_main->current_symbol_table->tab_symbol);
  window_main->current_symbol_table->tab_symbol = (struct symbol **) calloc(window_main->current_symbol_table->nb_symbol,sizeof(struct symbol *));
  if(window_main->current_symbol_table->tab_symbol == NULL)
    {
      mem_free_symbol_table(window_main->current_symbol_table);
      window_main->current_symbol_table = NULL;
      LeaveCriticalSection(&param->cs_symbol);
      if(window_symbol != NULL)
        ListView_DeleteAllItems(window_symbol->listview_symbol.hwnd_listview);
      return;
    }
  for(i=0, current_symbol=window_main->current_symbol_table->first_symbol; current_symbol; current_symbol=current_symbol->next, i++)
    {
      window_main->current_symbol_table->tab_symbol[i] = current_symbol;
      if(current_symbol->is_valid == 1)
        window_main->current_symbol_table->nb_symbol_valid++;
    }
  qsort(window_main->current_symbol_table->tab_symbol,window_main->current_symbol_table->nb_symbol,sizeof(struct symbol *),compare_symbol_address);

  /* Refait les liens */
  if(window_main->current_symbol_table->nb_symbol > 0)
    {
      current_symbol=window_main->current_symbol_table->first_symbol = window_main->current_symbol_table->tab_symbol[0];
      current_symbol=window_main->current_symbol_table->last_symbol = window_main->current_symbol_table->tab_symbol[window_main->current_symbol_table->nb_symbol-1];
      for(i=0; i<window_main->current_symbol_table->nb_symbol; i++)
        if(i != (window_main->current_symbol_table->nb_symbol - 1))
          window_main->current_symbol_table->tab_symbol[i]->next = window_main->current_symbol_table->tab_symbol[i+1];
      window_main->current_symbol_table->tab_symbol[window_main->current_symbol_table->nb_symbol-1]->next = NULL;
    }
  else
    {
      current_symbol=window_main->current_symbol_table->first_symbol = NULL;
      current_symbol=window_main->current_symbol_table->last_symbol = NULL;
    }

  /** Construit le tableau des Symbol valides **/
  BuildValidSymbolTable(window_main->current_symbol_table);

  /** Construit un tableau tri par Name|Segment|Program **/
  if(window_main->current_symbol_table->tab_symbol_name)
    free(window_main->current_symbol_table->tab_symbol_name);
  window_main->current_symbol_table->tab_symbol_name = (struct symbol **) calloc(window_main->current_symbol_table->nb_symbol,sizeof(struct symbol *));
  if(window_main->current_symbol_table->tab_symbol_name == NULL)
    {
      mem_free_symbol_table(window_main->current_symbol_table);
      window_main->current_symbol_table = NULL;
      LeaveCriticalSection(&param->cs_symbol);
      if(window_symbol != NULL)
        ListView_DeleteAllItems(window_symbol->listview_symbol.hwnd_listview);
      return;
    }
  for(i=0, current_symbol=window_main->current_symbol_table->first_symbol; current_symbol; current_symbol=current_symbol->next, i++)
    window_main->current_symbol_table->tab_symbol_name[i] = current_symbol;
  qsort(window_main->current_symbol_table->tab_symbol_name,window_main->current_symbol_table->nb_symbol,sizeof(struct symbol *),compare_symbol_name);

  /*** Ajoute les Reloc  la liste ***/
  for(current_reloc = new_reloc_list; current_reloc; current_reloc = next_reloc)
    {
      /* Isole le Reloc */
      next_reloc = current_reloc->next;
      current_reloc->next = NULL;

      /* Ajoute dans la liste chaine */
      is_duplicate = InsertUniqueReloc(current_reloc,window_main->current_symbol_table);

      /* On va reloger le Reloc */
      if(is_duplicate == 0)
        RelocateOneReloc(current_reloc);

      /* Supprime les doublons */
      if(is_duplicate == 1)
        mem_free_reloc(current_reloc);
    }

  /** Construit un tableau Reloc tri par Adresse **/
  window_main->current_symbol_table->nb_reloc_valid = 0;
  if(window_main->current_symbol_table->tab_reloc)
    free(window_main->current_symbol_table->tab_reloc);
  window_main->current_symbol_table->tab_reloc = (struct reloc **) calloc(window_main->current_symbol_table->nb_reloc,sizeof(struct reloc *));
  if(window_main->current_symbol_table->tab_reloc == NULL)
    {
      mem_free_symbol_table(window_main->current_symbol_table);
      window_main->current_symbol_table = NULL;
      LeaveCriticalSection(&param->cs_symbol);
      if(window_symbol != NULL)
        ListView_DeleteAllItems(window_symbol->listview_symbol.hwnd_listview);
      return;
    }
  for(i=0, current_reloc=window_main->current_symbol_table->first_reloc; current_reloc; current_reloc=current_reloc->next, i++)
    window_main->current_symbol_table->tab_reloc[i] = current_reloc;
  qsort(window_main->current_symbol_table->tab_reloc,window_main->current_symbol_table->nb_reloc,sizeof(struct reloc *),compare_reloc_address);

  /** Tableau des Reloc valides **/
  BuildValidRelocTable(window_main->current_symbol_table);

  /* Libre l'accs aux Symbols */
  LeaveCriticalSection(&param->cs_symbol);

  /** Supprime toutes les entres du Listview **/
  if(window_symbol != NULL)
    ListView_DeleteAllItems(window_symbol->listview_symbol.hwnd_listview);

  /** On refait le Listview avec les Symbols de la Table **/
  if(window_symbol != NULL)
    FillSymbolListView(&window_symbol->listview_symbol,window_main->current_symbol_table);

  /* MAJ du titre de la fentre avec le nombre de Symbol */
  if(window_main->current_symbol_table != NULL && window_symbol != NULL)
    {
      sprintf(buffer," Symbol Table : %d entries",window_main->current_symbol_table->nb_symbol);
      SetWindowText(window_symbol->hwnd_symbol,buffer);
    }

  /* MAJ de la Status Bar */
  UpdateWindowMainStatusBar();
}


/************************************************************/
/*  LoadSymbolFile() : Charge en mmoire un fichier Symbol. */
/************************************************************/
static struct symbol *LoadSymbolFile(char *file_path, struct reloc **first_reloc_rtn)
{
  int i, file_size;
  char *file_data;
  char *begin_line;
  char *end_line;
  char program_name[BUFFER_SIZE] = "";
  char error_buffer[BUFFER_SIZE];
  struct symbol *first_symbol;
  struct symbol *last_symbol;
  struct symbol *current_symbol;
  struct reloc *first_reloc;
  struct reloc *last_reloc;
  struct reloc *current_reloc;
  struct segment *first_segment;
  struct segment *current_segment;
  struct segment *next_segment;

  /* Init */
  first_symbol = NULL;
  last_symbol = NULL;
  first_reloc = NULL;
  last_reloc = NULL;
  first_segment = NULL;
  *first_reloc_rtn = NULL;

  /* Isole le nom du Programme */
  for(i=(int)strlen(file_path); i>=0; i--)
    if(file_path[i] == '/' || file_path[i] == '\\')
      {
        /* Nom du Fichier */
        strcpy(program_name,&file_path[i+1]);

        /* Nom du Programme */
        for(i=(int)strlen(program_name); i>=0; i--)
          if(program_name[i] == '_')
            {
              program_name[i] = '\0';
              break;
            }
        break;
      }

  /* Charge en mmoire le fichier en mmoire */
  file_data = (char *) LoadTextFile(file_path,&file_size,error_buffer);
  if(file_data == NULL)
    return(NULL);

  /* Rcupre la liste des Segments du fichier et dtermine s'ils sont relogeables */
  first_segment = BuildSegmentList(file_data);

  /** Dcoupe le fichier en Symbol / Reloc **/
  begin_line = file_data;
  while(begin_line)
    {
      /* Fin de la ligne */
      end_line = strchr(begin_line,'\n');
      if(end_line)
        *end_line = '\0';

      /* Rcupre un Symbol + Ajoute dans la liste Symbol */
      current_symbol = DecodeOneSymbol(begin_line,program_name,first_segment);
      if(current_symbol != NULL)
        {
          if(first_symbol == NULL)
            first_symbol = current_symbol;
          else
            last_symbol->next = current_symbol;
          last_symbol = current_symbol;
        }

      /* Rcupre un Symbol Reloc + Ajoute dans la liste Reloc */
      current_reloc = DecodeOneReloc(begin_line,program_name,first_segment);
      if(current_reloc != NULL)
        {
          if(first_reloc == NULL)
            first_reloc = current_reloc;
          else
            last_reloc->next = current_reloc;
          last_reloc = current_reloc;
        }

      /* Ligne suivante */
      if(end_line == NULL)
        break;
      begin_line = end_line + 1;
    }

  /* Libration mmoire */
  free(file_data);
  for(current_segment = first_segment; current_segment; current_segment = next_segment)
    {
      next_segment = current_segment->next;
      free(current_segment);
    }

  /* Renvoi la liste */
  *first_reloc_rtn = first_reloc;
  return(first_symbol);
}


/**************************************************/
/*  DecodeOneSymbol() :  Dcode une ligne Symbol. */
/**************************************************/
static struct symbol *DecodeOneSymbol(char *symbol_line, char *program_name, struct segment *first_segment)
{
  int i, nb_item, length, is_odd, is_error, tab_index[16];
  struct symbol *current_symbol;
  struct segment *current_segment;
  char value[BUFFER_SIZE];

  /* Allocation mmoire */
  current_symbol = (struct symbol *) calloc(1,sizeof(struct symbol));
  if(current_symbol == NULL)
    return(NULL);
  current_symbol->program_name = strdup(program_name);
  if(current_symbol->program_name == NULL)
    {
      mem_free_symbol(current_symbol);
      return(NULL);
    }

  /** Segment Name;Segment Line;File Name;File Line;Address;Name;Type;Data Type;Size;MX;Reloc **/
  nb_item = 0;
  tab_index[nb_item++] = 0;
  for(i=0; i<(int)strlen(symbol_line); i++)
    if(symbol_line[i] == ';')
      tab_index[nb_item++] = i;
  tab_index[nb_item++] = (int) strlen(symbol_line);

  /** Valide le format de la ligne **/
  if(nb_item != 12)
    {
      mem_free_symbol(current_symbol);
      return(NULL);
    }

  /** Rcupre les valeurs **/
  for(i=0; i<nb_item-1; i++)
    {
      /* Rcupre la valeur */
      length = tab_index[i+1]-(tab_index[i]+(i==0?0:1));
      memcpy(value,&symbol_line[tab_index[i]+(i==0?0:1)],length);
      value[length] = '\0';

      /* Conserve la valeur */
      if(i == 0)
        {
          /* Segment Name */
          current_symbol->segment_name = strdup(value);
          if(current_symbol->segment_name == NULL)
            {
              mem_free_symbol(current_symbol);
              return(NULL);
            }
        }
      else if(i == 1)
        {
          /* Segment Line */
          if(IsDecimal(value,&is_odd) == 0)
            {
              mem_free_symbol(current_symbol);
              return(NULL);
            }
          current_symbol->segment_line = atoi(value);
        }
      else if(i == 2)
        {
          /* File Name */
          current_symbol->source_file_name = strdup(value);
          if(current_symbol->source_file_name == NULL)
            {
              mem_free_symbol(current_symbol);
              return(NULL);
            }
        }
      else if(i == 3)
        {
          /* File Line */
          if(IsDecimal(value,&is_odd) == 0)
            {
              mem_free_symbol(current_symbol);
              return(NULL);
            }
          current_symbol->source_file_line = atoi(value);
        }
      else if(i == 4)
        {
          /* Address Relative : 00/0052 */
          current_symbol->symbol_relative_address = (DWORD) EvaluateAsAddress(value,&is_error);
          if(is_error)
            {
              mem_free_symbol(current_symbol);
              return(NULL);
            }
        }
      else if(i == 5)
        {
          /* Name */
          current_symbol->name = strdup(value);
          if(current_symbol->name == NULL)
            {
              mem_free_symbol(current_symbol);
              return(NULL);
            }

          /* Pas de nom, pas de Symbol */
          if(strlen(current_symbol->name) == 0)
            {
              mem_free_symbol(current_symbol);
              return(NULL);
            }
        }
      else if(i == 6)
        {
          /* Type */
          current_symbol->type = strdup(value);
          if(current_symbol->type == NULL)
            {
              mem_free_symbol(current_symbol);
              return(NULL);
            }
        }
      else if(i == 7)
        {
          /* Data Type */
          current_symbol->data_type = strdup(value);
          if(current_symbol->data_type == NULL)
            {
              mem_free_symbol(current_symbol);
              return(NULL);
            }
        }
      else if(i == 8)
        {
          /* Size */
          current_symbol->size = atoi(value);
        }
      else if(i == 9)
        {
          /* MX */
          current_symbol->m = value[0];
          current_symbol->x = value[1];
        }
      else if(i == 10)
        {
          /* Reloc */
          strcpy(current_symbol->reloc,value);
        }
    }

  /* Ce Symbol ncessite un Relocate */
  for(current_segment = first_segment; current_segment; current_segment=current_segment->next)
    if(!strcmp(current_symbol->segment_name,current_segment->name))
      {
        current_symbol->require_relocate = current_segment->is_rellocated;
        break;
      }

  /* Un Symbol non relogeable est automatiquement valide */
  if(current_symbol->require_relocate == 0)
    current_symbol->is_valid = 1;

  /* Adresse de chargement du Segment */
  current_symbol->segment_begin_address = 0x000000;

  /* Adresse reloge = Adresse Segment + Adresse non reloge */
  current_symbol->symbol_relocated_address = current_symbol->segment_begin_address + current_symbol->symbol_relative_address;

  /* Renvoi un Symbol */
  return(current_symbol);
}


/************************************************/
/*  DecodeOneReloc() :  Dcode une ligne Reloc. */
/************************************************/
static struct reloc *DecodeOneReloc(char *symbol_line, char *program_name, struct segment *first_segment)
{
  int i, nb_item, length, is_error, tab_index[16];
  struct reloc *current_reloc;
  struct segment *current_segment;
  char value[BUFFER_SIZE];

  /* Allocation mmoire */
  current_reloc = (struct reloc *) calloc(1,sizeof(struct reloc));
  if(current_reloc == NULL)
    return(NULL);
  current_reloc->program_name = strdup(program_name);
  if(current_reloc->program_name == NULL)
    {
      mem_free_reloc(current_reloc);
      return(NULL);
    }

  /** 'Segment Name';Segment Line;File Name;File Line;'Address';Name;Type;Data Type;Size;MX;'Reloc' **/
  nb_item = 0;
  tab_index[nb_item++] = 0;
  for(i=0; i<(int)strlen(symbol_line); i++)
    if(symbol_line[i] == ';')
      tab_index[nb_item++] = i;
  tab_index[nb_item++] = (int) strlen(symbol_line);

  /** Valide le format de la ligne **/
  if(nb_item != 12)
    {
      mem_free_reloc(current_reloc);
      return(NULL);
    }

  /** Rcupre les valeurs **/
  for(i=0; i<nb_item-1; i++)
    {
      /* Rcupre la valeur */
      length = tab_index[i+1]-(tab_index[i]+(i==0?0:1));
      memcpy(value,&symbol_line[tab_index[i]+(i==0?0:1)],length);
      value[length] = '\0';

      /* Conserve la valeur */
      if(i == 0)
        {
          /* Segment Name */
          current_reloc->segment_name = strdup(value);
          if(current_reloc->segment_name == NULL)
            {
              mem_free_reloc(current_reloc);
              return(NULL);
            }
        }
      else if(i == 1)
        {
          /* Segment Line */
        }
      else if(i == 2)
        {
          /* File Name */
        }
      else if(i == 3)
        {
          /* File Line */
        }
      else if(i == 4)
        {
          /* Address Relative : 00/0052 */
          current_reloc->symbol_relative_address = (DWORD) EvaluateAsAddress(value,&is_error);
          if(is_error)
            {
              mem_free_reloc(current_reloc);
              return(NULL);
            }
        }
      else if(i == 5)
        {
          /* Name */
        }
      else if(i == 6)
        {
          /* Type */
        }
      else if(i == 7)
        {
          /* Data Type */
        }
      else if(i == 8)
        {
          /* Size */
        }
      else if(i == 9)
        {
          /* MX */
        }
      else if(i == 10)
        {
          /* Reloc */
          strcpy(current_reloc->reloc,value);

          /* Pas de Reloc ? */
          if(strlen(current_reloc->reloc) == 0)
            {
              mem_free_reloc(current_reloc);
              return(NULL);
            }
        }
    }

  /* Ce Reloc ncessite un Relocate */
  for(current_segment = first_segment; current_segment; current_segment=current_segment->next)
    if(!strcmp(current_reloc->segment_name,current_segment->name))
      {
        current_reloc->require_relocate = current_segment->is_rellocated;
        break;
      }

  /* Un Reloc non relogeable est automatiquement valide */
  if(current_reloc->require_relocate == 0)
    current_reloc->is_valid = 1;

  /* Adresse de chargement du Segment */
  current_reloc->segment_begin_address = 0x000000;

  /* Adresse reloge = Adresse Segment + Adresse non reloge */
  current_reloc->symbol_relocated_address = current_reloc->segment_begin_address + current_reloc->symbol_relative_address;

  /* Renvoi un Reloc */
  return(current_reloc);
}


/********************************************************************/
/*  BuildSegmentList() :  Extrait la liste des Segments du fichier. */
/********************************************************************/
static struct segment *BuildSegmentList(char *file_data)
{
  int i, nb_item, length, is_error, tab_index[16];
  char *begin_line;
  char *end_line;
  DWORD symbol_relative_address;
  struct segment *first_segment;
  struct segment *last_segment;
  struct segment *current_segment;
  char value[BUFFER_SIZE];
  char segment_name[BUFFER_SIZE];
  char reloc[BUFFER_SIZE];

  /* Init */
  first_segment = NULL;
  last_segment = NULL;

  /* Saute la premire ligne */
  begin_line = strchr(file_data,'\n');
  if(begin_line == NULL)
    return(NULL);
  begin_line++;

  /** Dcoupe le fichier en Symbol / Reloc **/
  while(begin_line)
    {
      /* Fin de la ligne */
      end_line = strchr(begin_line,'\n');
      if(end_line)
        *end_line = '\0';

      /** 'Segment Name';Segment Line;File Name;File Line;'Address';Name;Type;Data Type;Size;MX;'Reloc' **/
      nb_item = 0;
      tab_index[nb_item++] = 0;
      for(i=0; i<(int)strlen(begin_line); i++)
        if(begin_line[i] == ';')
          tab_index[nb_item++] = i;
      tab_index[nb_item++] = (int) strlen(begin_line);

      /** Valide le format de la ligne **/
      if(nb_item == 12)
        {
          /** Rcupre les valeurs Name + Address + Reloc **/
          for(i=0; i<nb_item-1; i++)
            {
              /* Rcupre la valeur */
              length = tab_index[i+1]-(tab_index[i]+(i==0?0:1));
              memcpy(value,&begin_line[tab_index[i]+(i==0?0:1)],length);
              value[length] = '\0';

              /* Conserve la valeur */
              if(i == 0)
                {
                  /* Segment Name */
                  strcpy(segment_name,value);
                }
              else if(i == 1)
                {
                  /* Segment Line */
                }
              else if(i == 2)
                {
                  /* File Name */
                }
              else if(i == 3)
                {
                  /* File Line */
                }
              else if(i == 4)
                {
                  /* Address Relative : 00/0052 */
                  symbol_relative_address = (DWORD) EvaluateAsAddress(value,&is_error);
                }
              else if(i == 5)
                {
                  /* Name */
                }
              else if(i == 6)
                {
                  /* Type */
                }
              else if(i == 7)
                {
                  /* Data Type */
                }
              else if(i == 8)
                {
                  /* Size */
                }
              else if(i == 9)
                {
                  /* MX */
                }
              else if(i == 10)
                {
                  /* Reloc */
                  strcpy(reloc,value);
                }
            }

          /* New Segment ? */
          for(current_segment = first_segment; current_segment; current_segment = current_segment->next)
            if(!strcmp(current_segment->name,segment_name))
              {
                if(current_segment->is_rellocated == 0 && strlen(reloc) > 0)
                  current_segment->is_rellocated = 1;
                break;
              }

          /* Allocation */
          if(current_segment == NULL)
            {
              current_segment = (struct segment *) calloc(1,sizeof(struct segment));
              if(current_segment)
                {
                  /* Remplissage */
                  strcpy(current_segment->name,segment_name);
                  current_segment->is_rellocated = (strlen(reloc) > 0) ? 1 : 0;

                  /* Attachement */
                  if(first_segment == NULL)
                    first_segment = current_segment;
                  else
                    last_segment->next = current_segment;
                  last_segment = current_segment;
                }
            }
        }

      /* Ligne suivante */
      if(end_line == NULL)
        break;
      *end_line = '\n';
      begin_line = end_line + 1;
    }

  /* Renvoie la liste */
  return(first_segment);
}


/************************************************************/
/*  InsertUniqueSymbol() :  Ajoute un symbol dans la liste. */
/************************************************************/
static int InsertUniqueSymbol(struct symbol *current_symbol, struct symbol_table *current_symbol_table)
{
  struct symbol *one_symbol;

  /* Ajoute si l'entre n'est pas l */
  for(one_symbol = current_symbol_table->first_symbol; one_symbol; one_symbol = one_symbol->next)
    if(!stricmp(one_symbol->program_name,current_symbol->program_name) && 
       !stricmp(one_symbol->segment_name,current_symbol->segment_name) &&
       !strcmp(one_symbol->name,current_symbol->name))                         /* Le Label est Case Sensitive */
      return(1);

  /* Ajoute  la fin */
  current_symbol_table->nb_symbol++;
  if(current_symbol_table->first_symbol == NULL)
    current_symbol_table->first_symbol = current_symbol;
  else
    current_symbol_table->last_symbol->next = current_symbol;
  current_symbol_table->last_symbol = current_symbol;

  /* OK */
  return(0);
}


/**********************************************************/
/*  InsertUniqueReloc() :  Ajoute un reloc dans la liste. */
/**********************************************************/
static int InsertUniqueReloc(struct reloc *current_reloc, struct symbol_table *current_symbol_table)
{
  struct reloc *one_reloc;

  /* Ajoute si l'entre n'est pas l */
  for(one_reloc = current_symbol_table->first_reloc; one_reloc; one_reloc = one_reloc->next)
    if(!stricmp(one_reloc->program_name,current_reloc->program_name) && 
       !stricmp(one_reloc->segment_name,current_reloc->segment_name) &&
       one_reloc->symbol_relative_address == current_reloc->symbol_relative_address)
      return(1);

  /* Ajoute  la fin */
  current_symbol_table->nb_reloc++;
  if(current_symbol_table->first_reloc == NULL)
    current_symbol_table->first_reloc = current_reloc;
  else
    current_symbol_table->last_reloc->next = current_reloc;
  current_symbol_table->last_reloc = current_reloc;

  /* OK */
  return(0);
}


/********************************************************/
/*  RelocateOneSymbol() :  Reloge le Symbol en mmoire. */
/********************************************************/
static int RelocateOneSymbol(struct symbol *current_symbol)
{
  int i;
  struct file_segment *current_filesegment;
  struct s65816 *current_65816;
  my_Memory(MEMORY_GET_S65816,&current_65816,NULL);

  /* Rien  faire */
  if(current_65816 == NULL)
    return(0);
  if(current_symbol->require_relocate == 0)
    return(0);

  /** Recherche le Nom / segment dans les fichiers **/
  for(i=0; i<current_65816->current_file_segment->nb_segment_table; i++)
    {
      current_filesegment = current_65816->current_file_segment->tab_file_segment[i];

      if(!stricmp(current_symbol->program_name,current_filesegment->file_name) &&
         IsSameSegmentName(current_symbol->segment_name,current_filesegment->segment_name) == 1)
        {
          current_symbol->segment_begin_address    = current_filesegment->address;
          current_symbol->symbol_relocated_address = current_symbol->segment_begin_address + current_symbol->symbol_relative_address;
          current_symbol->is_valid = 1;
          return(1);
        }
    }

  /* On invalide un Symbol relogeable non relog */
  if(current_symbol->is_valid == 1)
    {
      current_symbol->segment_begin_address    = 0x00000000;
      current_symbol->symbol_relocated_address = current_symbol->segment_begin_address + current_symbol->symbol_relative_address;
      current_symbol->is_valid = 0;
      return(1);
    }

  /* Pas modifie */
  return(0);
}


/****************************************************************************/
/*  IsSameSegmentName() :  On va vrifier si le nom du Segment est le mme. */
/****************************************************************************/
int IsSameSegmentName(char *symbol_reloc_segment_name, char *file_segment_name)
{
  /* Mme nom */
  if(!stricmp(symbol_reloc_segment_name,file_segment_name))
    return(1);

  /* Le premier Segment d'un binaire n'a pas forcment de nom */
  if(!stricmp(file_segment_name,""))
    return(1);

  /* Diffrent */
  return(0);
}


/******************************************************/
/*  RelocateOneReloc() :  Reloge le Reloc en mmoire. */
/******************************************************/
static int RelocateOneReloc(struct reloc *current_reloc)
{
  int i;
  struct file_segment *current_filesegment;
  struct s65816 *current_65816;
  my_Memory(MEMORY_GET_S65816,&current_65816,NULL);

  /* Rien  faire */
  if(current_65816 == NULL)
    return(0);
  if(current_reloc->require_relocate == 0)
    return(0);

  /** Recherche le Nom / segment dans les fichiers **/
  for(i=0; i<current_65816->current_file_segment->nb_segment_table; i++)
    {
      current_filesegment = current_65816->current_file_segment->tab_file_segment[i];

      if(!stricmp(current_reloc->program_name,current_filesegment->file_name) &&
         IsSameSegmentName(current_reloc->segment_name,current_filesegment->segment_name) == 1)
        {
          current_reloc->segment_begin_address    = current_filesegment->address;
          current_reloc->symbol_relocated_address = current_reloc->segment_begin_address + current_reloc->symbol_relative_address;
          current_reloc->is_valid = 1;
          return(1);
        }
    }

  /* On invalide un Reloc relogeable non relog */
  if(current_reloc->is_valid == 1)
    {
      current_reloc->segment_begin_address    = 0x00000000;
      current_reloc->symbol_relocated_address = current_reloc->segment_begin_address + current_reloc->symbol_relative_address;
      current_reloc->is_valid = 0;
      return(1);
    }

  /* Pas modifi */
  return(0);
}


/****************************************************/
/*  ClearSymbolTable() :  Vide la table des Symbol. */
/****************************************************/
void ClearSymbolTable(struct window_symbol *window_symbol)
{
  struct window_main *window_main;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Rcupre la fentre Main */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  if(window_main == NULL)
    return;

  /* Rien  faire */
  if(window_main->current_symbol_table == NULL)
    return;

  /* Protge l'accs aux Symbols */
  EnterCriticalSection(&param->cs_symbol);

  /* Libration mmoire */
  mem_free_symbol_table(window_main->current_symbol_table);
  window_main->current_symbol_table = NULL;

  /* Libre l'accs aux Symbols */
  LeaveCriticalSection(&param->cs_symbol);

  /** Supprime toutes les entres du Listview **/
  ListView_DeleteAllItems(window_symbol->listview_symbol.hwnd_listview);

  /** On efface les donnes contextuelles du Symbol **/
  UpdateSymbolExtraInformation(window_symbol,NULL);

  /* MAJ du titre de la fentre avec le nombre de Symbol */
  SetWindowText(window_symbol->hwnd_symbol," Symbol Table : 0 entry");
  UpdateWindowMainStatusBar();
}


/*******************************************************/
/*  RefreshSymbolWindow() :  MAJ de la fentre Symbol. */
/*******************************************************/
void RefreshSymbolWindow(struct window_symbol *window_symbol)
{
  struct window_main *window_main;
  char buffer[BUFFER_SIZE];

  /* Rcupre la fentre Main */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  if(window_main == NULL)
    return;

  /** Remplissage du List View si les symbols sont disponibles **/
  if(window_main->current_symbol_table != NULL)
    FillSymbolListView(&window_symbol->listview_symbol,window_main->current_symbol_table);

  /* MAJ du titre de la fentre avec le nombre de Symbol */
  if(window_main->current_symbol_table != NULL)
    {
      sprintf(buffer," Symbol Table : %d entries",window_main->current_symbol_table->nb_symbol);
      SetWindowText(window_symbol->hwnd_symbol,buffer);
    }

  /* MAJ Status Bar */
  UpdateWindowMainStatusBar();
}


/************************************************************************/
/*  RelocateSymbolSegment() :  Change l'adresse du Segment slectionn. */
/************************************************************************/
static void RelocateSymbolSegment(struct window_symbol *window_symbol)
{
  int length, is_error;
  DWORD segment_address;
  char buffer_value[BUFFER_SIZE];
  struct symbol *current_symbol;
  struct window_main *window_main;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Rcupre la fentre Main */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  if(window_main == NULL)
    return;

  /* Rien de slectionn ? */
  if(strlen(window_symbol->current_segment_name) == 0 || strlen(window_symbol->current_program_name) == 0)
    return;

  /* Dcode la nouvelle adresse */
  length = GetDlgItemText(window_symbol->hwnd_symbol,IDC_EDIT_SEGMENT_ADDRESS,buffer_value,1024);
  segment_address = EvaluateAsAddress(buffer_value,&is_error);
  if(is_error == 1)
    return;

  /* Rien n'a chang */
  if(segment_address == window_symbol->current_segment_address)
    return;

  /* Protge l'accs aux Symbols */
  EnterCriticalSection(&param->cs_symbol);

  /** On va repasser sur tous les Symbols concerns pour MAJ l'adresse **/
  for(current_symbol = window_main->current_symbol_table->first_symbol; current_symbol; current_symbol=current_symbol->next)
    if(!stricmp(current_symbol->program_name,window_symbol->current_program_name) && 
       !stricmp(current_symbol->segment_name,window_symbol->current_segment_name))
      {
        current_symbol->segment_begin_address = segment_address;
        current_symbol->symbol_relocated_address = current_symbol->segment_begin_address + current_symbol->symbol_relative_address;
        current_symbol->is_valid = 1;
      }

  /* Tri  nouveau le tableau */
  qsort(window_main->current_symbol_table->tab_symbol,window_main->current_symbol_table->nb_symbol,sizeof(struct symbol *),compare_symbol_address);

  /** Cration du Tableau des Symbols valides **/
  BuildValidSymbolTable(window_main->current_symbol_table);

  /* Libre l'accs aux Symbols */
  LeaveCriticalSection(&param->cs_symbol);

  /** Supprime toutes les entres du Listview **/
  ListView_DeleteAllItems(window_symbol->listview_symbol.hwnd_listview);

  /** On refait le Listview avec les Symbols de la Table **/
  FillSymbolListView(&window_symbol->listview_symbol,window_main->current_symbol_table);
}


/**************************************************************************/
/*  BuildValidSymbolTable() :  Construit le tableau des Symboles valides. */
/**************************************************************************/
static void BuildValidSymbolTable(struct symbol_table *current_symbol_table)
{
  int i, j, nb_symbol_valid;
  struct symbol **tab_symbol_valid;

  /* Compte le nombre de symboles valides */
  nb_symbol_valid = 0;
  for(i=0; i<current_symbol_table->nb_symbol; i++)
    if(current_symbol_table->tab_symbol[i]->is_valid == 1)
      nb_symbol_valid++;

  /* Allocation du tableau */
  tab_symbol_valid = (struct symbol **) calloc(1+nb_symbol_valid,sizeof(struct symbol *));
  if(tab_symbol_valid == NULL)
    return;

  /* Remplissage du tableau */
  for(i=0, j=0; i<current_symbol_table->nb_symbol; i++)
    if(current_symbol_table->tab_symbol[i]->is_valid == 1)
      tab_symbol_valid[j++] = current_symbol_table->tab_symbol[i];

  /* Libration mmoire */
  if(current_symbol_table->tab_symbol_valid != NULL)
    free(current_symbol_table->tab_symbol_valid);

  /* Remplace le tableau */
  current_symbol_table->nb_symbol_valid = nb_symbol_valid;
  current_symbol_table->tab_symbol_valid = tab_symbol_valid;
}


/**********************************************************************/
/*  BuildValidRelocTable() :  Construit le tableau des Reloc valides. */
/**********************************************************************/
static void BuildValidRelocTable(struct symbol_table *current_symbol_table)
{
  int i, j, nb_reloc_valid;
  struct reloc **tab_reloc_valid;

  /* Compte le nombre de reloc valides */
  nb_reloc_valid = 0;
  for(i = 0; i<current_symbol_table->nb_reloc; i++)
    if(current_symbol_table->tab_reloc[i]->is_valid == 1)
      nb_reloc_valid++;

  /* Allocation du tableau */
  tab_reloc_valid = (struct reloc **) calloc(1+nb_reloc_valid,sizeof(struct reloc *));
  if(tab_reloc_valid == NULL)
    return;

  /* Remplissage du tableau */
  for(i=0, j=0; i<current_symbol_table->nb_reloc; i++)
    if(current_symbol_table->tab_reloc[i]->is_valid == 1)
      tab_reloc_valid[j++] = current_symbol_table->tab_reloc[i];

  /* Libration mmoire */
  if(current_symbol_table->tab_reloc_valid != NULL)
    free(current_symbol_table->tab_reloc_valid);

  /* Remplace le tableau */
  current_symbol_table->nb_reloc_valid = nb_reloc_valid;
  current_symbol_table->tab_reloc_valid = tab_reloc_valid;
}


/**************************************************************************************/
/*  UpdateSymbolExtraInformation() :  Met  jour les donnes contextuelles du Symbol. */
/**************************************************************************************/
static void UpdateSymbolExtraInformation(struct window_symbol *window_symbol, struct symbol *current_symbol)
{
  char buffer_value[BUFFER_SIZE];

  /** On met  jour avec les donnes du Symbol **/
  if(current_symbol != NULL)
    {
      /* Conserve l'identifiant du Segment */
      strcpy(window_symbol->current_program_name,current_symbol->program_name);
      strcpy(window_symbol->current_segment_name,current_symbol->segment_name);
      window_symbol->current_segment_address = current_symbol->segment_begin_address;

      /* Met  jour les donnes  l'cran */
      SetWindowText(GetDlgItem(window_symbol->hwnd_symbol,IDC_EDIT_SOURCE_FILE),current_symbol->source_file_name);
      sprintf(buffer_value,"%d",current_symbol->source_file_line);
      SetWindowText(GetDlgItem(window_symbol->hwnd_symbol,IDC_EDIT_SOURCE_LINE),buffer_value);
      sprintf(buffer_value,"%d",current_symbol->segment_line);
      SetWindowText(GetDlgItem(window_symbol->hwnd_symbol,IDC_EDIT_SEGMENT_LINE),buffer_value);
      sprintf(buffer_value,"%02X/%04X",(current_symbol->symbol_relative_address>>16)&0xFF,current_symbol->symbol_relative_address&0xFFFF);
      SetWindowText(GetDlgItem(window_symbol->hwnd_symbol,IDC_EDIT_SOURCE_ADDRESS),buffer_value);
      sprintf(buffer_value,"%02X/%04X",(current_symbol->segment_begin_address>>16)&0xFF,current_symbol->segment_begin_address&0xFFFF);
      SetWindowText(GetDlgItem(window_symbol->hwnd_symbol,IDC_EDIT_SEGMENT_ADDRESS),buffer_value);
    }
  else
    {
      /* On efface tout */
      strcpy(window_symbol->current_program_name,"");
      strcpy(window_symbol->current_segment_name,"");
      window_symbol->current_segment_address = 0;
      SetWindowText(GetDlgItem(window_symbol->hwnd_symbol,IDC_EDIT_SOURCE_FILE),"");
      SetWindowText(GetDlgItem(window_symbol->hwnd_symbol,IDC_EDIT_SOURCE_LINE),"");
      SetWindowText(GetDlgItem(window_symbol->hwnd_symbol,IDC_EDIT_SEGMENT_LINE),"");
      SetWindowText(GetDlgItem(window_symbol->hwnd_symbol,IDC_EDIT_SOURCE_ADDRESS),"");
      SetWindowText(GetDlgItem(window_symbol->hwnd_symbol,IDC_EDIT_SEGMENT_ADDRESS),"");
    }
}


/*****************************************************************/
/*  GetSymbolByAddress() :  Recherche un Symbol via son adresse. */
/*****************************************************************/
struct symbol *GetSymbolByAddress(DWORD symbol_address)
{
  struct symbol local_symbol;
  struct symbol *local_symbol_ptr;
  struct symbol **found_symbol_ptr;
  struct symbol *current_symbol;
  struct window_main *window_main;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Rcupre la fentre Main */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  if(window_main == NULL)
    return(NULL);
  if(window_main->current_symbol_table == NULL)
    return(NULL);

  /* Protge l'accs aux Symbols */
  EnterCriticalSection(&param->cs_symbol);
  if(window_main->current_symbol_table->tab_symbol_valid == NULL || window_main->current_symbol_table->nb_symbol_valid == 0)
    {
      LeaveCriticalSection(&param->cs_symbol);
      return(NULL);
    }

  /* Recherche dychotomique d'un Symbopl relog */
  current_symbol = NULL;
  local_symbol.symbol_relocated_address = symbol_address;
  local_symbol_ptr = &local_symbol;
  found_symbol_ptr = (struct symbol **) bsearch(&local_symbol_ptr,(void *)window_main->current_symbol_table->tab_symbol_valid,window_main->current_symbol_table->nb_symbol_valid,sizeof(struct symbol *),compare_symbol_address);
  if(found_symbol_ptr != NULL)
    current_symbol = *found_symbol_ptr;

  /* Libre l'accs aux Symbols */
  LeaveCriticalSection(&param->cs_symbol);

  /* Renvoi le Symbol */
  return(current_symbol);
}


/***************************************************************/
/*  GetRelocByAddress() :  Recherche un Reloc via son adresse. */
/***************************************************************/
struct reloc *GetRelocByAddress(DWORD symbol_address)
{
  struct reloc local_reloc;
  struct reloc *local_reloc_ptr;
  struct reloc **found_reloc_ptr;
  struct reloc *current_reloc;
  struct window_main *window_main;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Rcupre la fentre Main */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  if(window_main == NULL)
    return(NULL);
  if(window_main->current_symbol_table == NULL)
    return(NULL);

  /* Protge l'accs aux Symbols */
  EnterCriticalSection(&param->cs_symbol);
  if(window_main->current_symbol_table->tab_reloc_valid == NULL || window_main->current_symbol_table->nb_reloc_valid == 0)
    {
      LeaveCriticalSection(&param->cs_symbol);
      return(NULL);
    }

  /* Recherche dychotomique */
  current_reloc = NULL;
  local_reloc.symbol_relocated_address = symbol_address;
  local_reloc_ptr = &local_reloc;
  found_reloc_ptr = (struct reloc **) bsearch(&local_reloc_ptr,(void *)window_main->current_symbol_table->tab_reloc_valid,window_main->current_symbol_table->nb_reloc_valid,sizeof(struct reloc *),compare_reloc_address);
  if(found_reloc_ptr != NULL)
    current_reloc = *found_reloc_ptr;

  /* Libre l'accs aux Symbols */
  LeaveCriticalSection(&param->cs_symbol);

  /* Renvoi le Reloc */
  return(current_reloc);
}


/************************************************************************************/
/*  GetSymbolByPreviousAddress() :  Recherche un Symbol via une adresse prcdante. */
/************************************************************************************/
struct symbol *GetSymbolByPreviousAddress(DWORD memory_address)
{
  int min_index, max_index, current_index;
  struct symbol **tab_symbol_valid;
  struct window_main *window_main;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Rcupre la fentre Main */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  if(window_main == NULL)
    return(NULL);
  if(window_main->current_symbol_table == NULL)
    return(NULL);

  /* Protge l'accs aux Symbols */
  EnterCriticalSection(&param->cs_symbol);
  if(window_main->current_symbol_table->tab_symbol_valid == NULL || window_main->current_symbol_table->nb_symbol_valid == 0)
    {
      LeaveCriticalSection(&param->cs_symbol);
      return(NULL);
    }

  /* Init */
  min_index = 0;
  max_index = (window_main->current_symbol_table->nb_symbol_valid-1);
  current_index = max_index/2;
  tab_symbol_valid = window_main->current_symbol_table->tab_symbol_valid;

  /* Gestion des extrmes */
  if(tab_symbol_valid[min_index]->symbol_relocated_address > memory_address)
    {
      LeaveCriticalSection(&param->cs_symbol);
      return(NULL);
    }
  if((tab_symbol_valid[max_index]->symbol_relocated_address & 0x00FF0000) < (memory_address & 0x00FF0000))
    {
      LeaveCriticalSection(&param->cs_symbol);
      return(NULL);
    }

  /*** Recherche dichotomique ***/
  while(1)
    {
      /* Trouv ? */
      if((current_index+1) <= (window_main->current_symbol_table->nb_symbol_valid-1))
        if(tab_symbol_valid[current_index]->symbol_relocated_address <= memory_address && tab_symbol_valid[current_index+1]->symbol_relocated_address > memory_address)
          {
            if((tab_symbol_valid[current_index]->symbol_relocated_address + tab_symbol_valid[current_index]->size) > memory_address)
              {
                /* Cherche le bon Symbol avec une adresse Valide */
                if(tab_symbol_valid[current_index]->require_relocate == 0)
                  {
                    LeaveCriticalSection(&param->cs_symbol);
                    return(tab_symbol_valid[current_index]);      /* Pas besoin de le reloger => on conserve */
                  }
                else if(tab_symbol_valid[current_index]->require_relocate == 1 && tab_symbol_valid[current_index]->symbol_relocated_address != tab_symbol_valid[current_index]->symbol_relative_address)
                  {
                    LeaveCriticalSection(&param->cs_symbol);
                    return(tab_symbol_valid[current_index]);      /* Ce Symbol relogeable a t relog => On garde */
                  }
                else
                  {
                    LeaveCriticalSection(&param->cs_symbol);
                    return(NULL);
                  }
              }
            LeaveCriticalSection(&param->cs_symbol);
            return(NULL);
          }

      /* Termin */
      if(current_index == min_index && min_index == max_index)
        {
          LeaveCriticalSection(&param->cs_symbol);
          return(NULL);
        }

      /** Divise le restant  vrifier par 2 **/
      if(tab_symbol_valid[current_index]->symbol_relocated_address > memory_address)
        {
          max_index = current_index-1;
          if(max_index < 0)
            max_index = 0;
          current_index = min_index + (max_index-min_index)/2;
        }
      else
        {
          min_index = current_index+1;
          if(min_index > (window_main->current_symbol_table->nb_symbol_valid-1))
            min_index = (window_main->current_symbol_table->nb_symbol_valid-1);
          current_index = min_index + (max_index-min_index)/2;
        }
    }
}


/*********************************************************/
/*  GetSymbolByName() :  Rcupre un Symbol via son Nom. */
/*********************************************************/
struct symbol *GetSymbolByNameSegmentFile(char *name, char *segment, char *file)
{
  int i;
  struct symbol *current_symbol;
  struct window_main *window_main;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Rcupre la fentre Main */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  if(window_main == NULL)
    return(NULL);
  if(window_main->current_symbol_table == NULL)
    return(NULL);

  /* Protge l'accs aux Symbols */
  EnterCriticalSection(&param->cs_symbol);
  if(window_main->current_symbol_table->tab_symbol == NULL || window_main->current_symbol_table->nb_symbol == 0)
    {
      LeaveCriticalSection(&param->cs_symbol);
      return(NULL);
    }

  /** Recherche le Symbol dans la Table **/
  for(i=0;i<window_main->current_symbol_table->nb_symbol; i++)
    {
      current_symbol = window_main->current_symbol_table->tab_symbol[i];
      if(!stricmp(current_symbol->program_name,file)    && 
         !stricmp(current_symbol->segment_name,segment) &&
         !strcmp(current_symbol->name,name))                         /* Le Label est Case Sensitive */
        {
          LeaveCriticalSection(&param->cs_symbol);
          return(current_symbol);
        }
    }

  /* Libre l'accs aux Symbols */
  LeaveCriticalSection(&param->cs_symbol);

  /* Pas trouv */
  return(NULL);
}


/********************************************************************/
/*  UpdateSymbolAddress() :  Mise  jour des adresses des Symboles. */
/********************************************************************/
void UpdateSymbolAddress(void)
{
  int nb_symbol_relocated, nb_reloc_relocated;
  struct symbol *current_symbol;
  struct reloc *current_reloc;
  struct window_main *window_main;
  struct window_symbol *window_symbol;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Rcupre la fentre Main */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  if(window_main == NULL)
    return;
  if(window_main->current_symbol_table == NULL)
    return;
  my_Memory(MEMORY_GET_WINDOW_SYMBOL,&window_symbol,NULL);

  /* Protge l'accs aux Symbols */
  EnterCriticalSection(&param->cs_symbol);

  /** On va repasser sur tous les Symbols pour MAJ l'adresse **/
  nb_symbol_relocated = 0;
  for(current_symbol = window_main->current_symbol_table->first_symbol; current_symbol; current_symbol=current_symbol->next)
    nb_symbol_relocated += RelocateOneSymbol(current_symbol);
  /* Tri  nouveau le tableau */
  if(window_main->current_symbol_table->tab_symbol != NULL && nb_symbol_relocated > 0)
    qsort(window_main->current_symbol_table->tab_symbol,window_main->current_symbol_table->nb_symbol,sizeof(struct symbol *),compare_symbol_address);
  /* MAJ du tableau de Symbol valid */
  BuildValidSymbolTable(window_main->current_symbol_table);

  /** On va repasser sur tous les Reloc pour MAJ de l'adresse **/
  nb_reloc_relocated = 0;
  for(current_reloc = window_main->current_symbol_table->first_reloc; current_reloc; current_reloc=current_reloc->next)
    nb_reloc_relocated += RelocateOneReloc(current_reloc);
  /* Tri  nouveau le tableau */
  if(window_main->current_symbol_table->tab_reloc != NULL && nb_reloc_relocated > 0)
    qsort(window_main->current_symbol_table->tab_reloc,window_main->current_symbol_table->nb_reloc,sizeof(struct reloc *),compare_reloc_address);
  /* MAJ du tableau de Reloc valid */
  BuildValidRelocTable(window_main->current_symbol_table);

  /* Libre l'accs aux Symbols */
  LeaveCriticalSection(&param->cs_symbol);

  /** MAJ de la LstView de la fentre Symbol **/
  if(window_symbol != NULL && nb_symbol_relocated > 0)
    {
      /* Supprime toutes les entres du Listview */
      ListView_DeleteAllItems(window_symbol->listview_symbol.hwnd_listview);

      /* On refait le Listview avec les Symbols de la Table */
      FillSymbolListView(&window_symbol->listview_symbol,window_main->current_symbol_table);
    }
}


/***************************************************************************/
/*  compare_symbol_address() : Fonction de comparaison pour le Quick Sort. */
/***************************************************************************/
static int compare_symbol_address(const void *data_1, const void *data_2)
{
  struct symbol *symbol_1;
  struct symbol *symbol_2;

  /* Rcupration des paramtres */
  symbol_1 = *((struct symbol **) data_1);
  symbol_2 = *((struct symbol **) data_2);

  /* Comparaison des adresses */
  if(symbol_1->symbol_relocated_address < symbol_2->symbol_relocated_address)
    return(-1);
  else if(symbol_1->symbol_relocated_address > symbol_2->symbol_relocated_address)
    return(1);
  else
    return(0);
}


/************************************************************************/
/*  compare_symbol_name() : Fonction de comparaison pour le Quick Sort. */
/************************************************************************/
static int compare_symbol_name(const void *data_1, const void *data_2)
{
  struct symbol *symbol_1;
  struct symbol *symbol_2;
  char full_name_1[1024];
  char full_name_2[1024];

  /* Rcupration des paramtres */
  symbol_1 = *((struct symbol **) data_1);
  symbol_2 = *((struct symbol **) data_2);
  sprintf(full_name_1,"%s|%s|%s",symbol_1->name,symbol_1->segment_name,symbol_1->program_name);
  sprintf(full_name_2,"%s|%s|%s",symbol_2->name,symbol_2->segment_name,symbol_2->program_name);

  /* Comparaison des noms */
  return(stricmp(full_name_1,full_name_2));
}


/**************************************************************************/
/*  compare_reloc_address() : Fonction de comparaison pour le Quick Sort. */
/**************************************************************************/
static int compare_reloc_address(const void *data_1, const void *data_2)
{
  struct reloc *reloc_1;
  struct reloc *reloc_2;

  /* Rcupration des paramtres */
  reloc_1 = *((struct reloc **) data_1);
  reloc_2 = *((struct reloc **) data_2);

  /* Comparaison des adresses */
  if(reloc_1->symbol_relocated_address < reloc_2->symbol_relocated_address)
    return(-1);
  else if(reloc_1->symbol_relocated_address > reloc_2->symbol_relocated_address)
    return(1);
  else
    return(0);
}


/********************************************************************/
/*  mem_free_symbol() :  Libration mmoire de la structure symbol. */
/********************************************************************/
void mem_free_symbol(struct symbol *current_symbol)
{
  if(current_symbol)
    {
      if(current_symbol->program_name)
        free(current_symbol->program_name);

      if(current_symbol->segment_name)
        free(current_symbol->segment_name);

      if(current_symbol->source_file_name)
        free(current_symbol->source_file_name);

      if(current_symbol->name)
        free(current_symbol->name);

      if(current_symbol->type)
        free(current_symbol->type);

      if(current_symbol->data_type)
        free(current_symbol->data_type);

      free(current_symbol);
    }
}


/******************************************************************/
/*  mem_free_reloc() :  Libration mmoire de la structure reloc. */
/******************************************************************/
void mem_free_reloc(struct reloc *current_reloc)
{
  if(current_reloc)
    {
      if(current_reloc->program_name)
        free(current_reloc->program_name);

      if(current_reloc->segment_name)
        free(current_reloc->segment_name);

      free(current_reloc);
    }
}


/********************************************************************************/
/*  mem_free_symbol_table() :  Libration mmoire de la structure symbol_table. */
/********************************************************************************/
void mem_free_symbol_table(struct symbol_table *current_symbol_table)
{
  struct symbol *current_symbol;
  struct symbol *next_symbol;
  struct reloc *current_reloc;
  struct reloc *next_reloc;

  if(current_symbol_table)
    {
      /* Libration de la liste chaine Symbol */
      for(current_symbol = current_symbol_table->first_symbol; current_symbol; current_symbol = next_symbol)
        {
          next_symbol = current_symbol->next;
          mem_free_symbol(current_symbol);
        }
      if(current_symbol_table->tab_symbol)
        free(current_symbol_table->tab_symbol);
      if(current_symbol_table->tab_symbol_name)
        free(current_symbol_table->tab_symbol_name);
      if(current_symbol_table->tab_symbol_valid)
        free(current_symbol_table->tab_symbol_valid);

      /* Libration de la liste chaine Reloc */
      for(current_reloc = current_symbol_table->first_reloc; current_reloc; current_reloc = next_reloc)
        {
          next_reloc = current_reloc->next;
          mem_free_reloc(current_reloc);
        }
      if(current_symbol_table->tab_reloc)
        free(current_symbol_table->tab_reloc);
      if(current_symbol_table->tab_reloc_valid)
        free(current_symbol_table->tab_reloc_valid);

      free(current_symbol_table);
    }
}


/*****************************************************************/
/*  UserResizeSymbolWindow() :  Retaille la fentre des Symbols. */
/*****************************************************************/
static void UserResizeSymbolWindow(struct window_symbol *window_symbol)
{
  RECT window_rect;
  int window_x, window_y, window_new_height, minimum_window_height;
  int listview_pixel_height, nb_listview_data_row, delta_listview_nb_row, delta_pixel_height;

  /* Position + Dimension actuelle de la fentre retaille dans l'cran */
  GetWindowRect(window_symbol->hwnd_symbol,&window_rect);

  /* Nouvelle Dimension de la fentre (avec la barre de titre et les bordures) */
  window_x = window_rect.left;
  window_y = window_rect.top;
  window_new_height = window_rect.bottom - window_rect.top;

  /* Nombre de Pixels restant pour la ListView */
  minimum_window_height = window_symbol->window_init_height - window_symbol->nb_init_data_row*SYMBOL_LISTVIEW_ROW_HEIGHT;
  listview_pixel_height = window_new_height - minimum_window_height;

  /* Nombre de lignes de la ListView */
  nb_listview_data_row = listview_pixel_height / SYMBOL_LISTVIEW_ROW_HEIGHT;

  /** Teste les valeurs minimales autorises **/
  if(nb_listview_data_row < SYMBOL_LISTVIEW_MIN_NB_ROW)
    nb_listview_data_row = SYMBOL_LISTVIEW_MIN_NB_ROW;

  /* Delta de vla ListView */
  delta_listview_nb_row = nb_listview_data_row - window_symbol->nb_row;
  delta_pixel_height = delta_listview_nb_row*SYMBOL_LISTVIEW_ROW_HEIGHT;
  window_symbol->nb_row = nb_listview_data_row;

  /* Nouvelle hauteur de la fentre (on ajuste pour tomber sur un nombre extact de ligne dans la ListView) */
  window_new_height = minimum_window_height + nb_listview_data_row*SYMBOL_LISTVIEW_ROW_HEIGHT;

  /* Retaille la fentre */
  window_symbol->user_resize = 0;
  MoveWindow(window_symbol->hwnd_symbol,window_x,window_y,window_symbol->window_init_width,window_new_height,FALSE);

  /* Dplace les Contrles */
  MoveSymbolWindowControls(delta_pixel_height,window_symbol);

  /* Rafraichit la fentre */
  InvalidateRect(window_symbol->hwnd_symbol,NULL,TRUE);
  UpdateWindow(window_symbol->hwnd_symbol);
}


/*****************************************************************************************/
/*  MoveSymbolWindowControls() :  On va dplacer les diffrents controles de la fentre. */
/*****************************************************************************************/
static void MoveSymbolWindowControls(int delta_y, struct window_symbol *window_symbol)
{
  HWND hwnd_control, hwnd_listview;
  int i, control_x, control_y, control_width, control_height;
  int listview_x, listview_y, listview_width, listview_height;
  int tab_control_id[] = 
    {
      IDC_GROUP_SYMBOL, 
      IDC_STATIC_CODE_FILE,        IDC_STATIC_CODE_LINE,     IDC_STATIC_SEGMENT_LINE,
      IDC_EDIT_SOURCE_FILE,        IDC_EDIT_SOURCE_LINE,     IDC_EDIT_SEGMENT_LINE,
      IDC_STATIC_RELATIVE_ADDRESS, IDC_STATIC_LOAD_ADDRESS,
      IDC_EDIT_SOURCE_ADDRESS,     IDC_EDIT_SEGMENT_ADDRESS,
      IDC_BUTTON_SYMBOL_LOAD,      IDC_BUTTON_SYMBOL_CLEAR,  IDC_BUTTON_SYMBOL_RELOCATE,
      0
    };

  /** On va modifier la hauteur de la ListBox **/
  /* Handle du Controle */
  hwnd_listview = window_symbol->listview_symbol.hwnd_listview;
  /* Position + Taille actuelle */
  GetWindowControlInfo(window_symbol->hwnd_symbol,hwnd_listview,&listview_x,&listview_y,&listview_width,&listview_height);
  /* Change la taille le controle verticalement */
  MoveWindow(hwnd_listview,listview_x,listview_y,listview_width,listview_height+delta_y,FALSE);

  /** On va dplacer tous les controles verticalement **/
  for(i=0; tab_control_id[i] != 0; i++)
    {
      /* Handle du Controle */
      hwnd_control = GetDlgItem(window_symbol->hwnd_symbol,tab_control_id[i]);

      /* Position + Taille actuelle */
      GetWindowControlInfo(window_symbol->hwnd_symbol,hwnd_control,&control_x,&control_y,&control_width,&control_height);

      /* Dplace le controle verticalement */
      MoveWindow(hwnd_control,control_x,control_y+delta_y,control_width,control_height,FALSE);
    }
}


/********************************************************************/
/*  DeleteSymbolChildWindows() :  Destruction des fentres enfants. */
/********************************************************************/
static void DeleteSymbolChildWindows(struct window_symbol *window_symbol)
{
}


/**********************************************************************/
/*  mem_free_window_symbol() :  Libration mmoire pour la structure. */
/**********************************************************************/
void mem_free_window_symbol(struct window_symbol *window_symbol)
{
  if(window_symbol)
    {      
      free(window_symbol);
    }
}

/**********************************************************************/
