/****************************************************************************/
/*                                                                          */
/*  Window_SymbolPickup.c :  Module de gestion de la fentre SymbolPickup.  */
/*                                                                          */
/****************************************************************************/
/*  Auteur : Olivier ZARDINI    *    Brutal Deluxe    *    Date : Jun 2024  */
/****************************************************************************/

#include <stdio.h>
#include <time.h>

#include <windows.h>
#include <windowsx.h>
#include <Commctrl.h>
#include <Winuser.h>
#include <Richedit.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_Notification.h"
#include "Dc_Graphic.h"
#include "Dc_CPU.h"
#include "Dc_MMU.h"
#include "Dc_IIgsInternals.h"
#include "Cyrene_Engine.h"
#include "Window_QuickWatch.h"
#include "Window_Main.h"
#include "Window_Main_Menu.h"
#include "Window_Disassembler.h"
#include "Window_MemoryArea.h"
#include "Window_Symbol.h"
#include "Window_SymbolPickup.h"

static DWORD BuildSymbolPickupWindowThread(void *);
LRESULT CALLBACK ProcHandlerSymbolPickupWindow(HWND,UINT,WPARAM,LPARAM);
static int CreateSymbolPickupChildWindows(HWND,struct window_symbol_pickup *);
static void BuildSymbolPickupListView(struct list_view *,HFONT,int,struct window_symbol_pickup *);
static void FillSymbolPickupListView(struct list_view *,struct symbol_table *);
static void UserResizeSymbolPickupWindow(struct window_symbol_pickup *);
static void MoveSymbolPickupWindowControls(int,struct window_symbol_pickup *);
static void DeleteSymbolPickupChildWindows(struct window_symbol_pickup *);


/**************************************************************************************/
/*  BuildSymbolPickupWindow() :  Construction / Affichage de la fentre SymbolPickup. */
/**************************************************************************************/
struct window_symbol_pickup *BuildSymbolPickupWindow(HINSTANCE hInstance, HWND hwndParent, int refresh_message_id)
{
  struct window_symbol_pickup *window_symbol_pickup;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Init */
  param->symbol_is_available = 0;

  /** Allocation de la structure window_symbol **/
  window_symbol_pickup = (struct window_symbol_pickup *) calloc(1,sizeof(struct window_symbol_pickup));
  if(window_symbol_pickup == NULL)
    return(NULL);
  window_symbol_pickup->hwnd_parent = hwndParent;
  window_symbol_pickup->refresh_message_id = refresh_message_id;
  window_symbol_pickup->symbol_address = 0xFFFFFFFF;
  window_symbol_pickup->symbol_size = 0;
  my_Memory(MEMORY_SET_WINDOW_SYMBOL_PICKUP,window_symbol_pickup,NULL);

  /* Lance un thread pour grer la fentre SymbolPickup */
  CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)BuildSymbolPickupWindowThread,window_symbol_pickup,0,NULL);

  /* Renvoi la structure mmoire grant la fentre */
  return(window_symbol_pickup);
}


/************************************************************************************************/
/*  BuildSymbolPickupWindowThread() :  Cration de la fentre SymbolPickup via un Thread ddi. */
/************************************************************************************************/
static DWORD BuildSymbolPickupWindowThread(void *window_symbol_pickup_param)
{
  HWND hwndSymbolPickup;
  MSG msg;
  BOOL bRet;
  struct window_main *window_main;
  struct window_symbol_pickup *window_symbol_pickup;
  struct parameter *param;

  /* Init */
  window_symbol_pickup = (struct window_symbol_pickup *) window_symbol_pickup_param;
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Cration de la fentre SymbolPickup **/
  hwndSymbolPickup = CreateDialog(param->hInstance,MAKEINTRESOURCE(IDD_WINDOW_SYMBOL_PICKUP),window_main->hwnd_main,(DLGPROC)ProcHandlerSymbolPickupWindow);
  if(hwndSymbolPickup == NULL)
    return(1);

  /** Show Window + Paint Content **/
  ShowWindow(hwndSymbolPickup,SW_SHOWDEFAULT);
  UpdateWindow(hwndSymbolPickup);

  /** Message Loop pour toutes les fentres de ce Thread (fentre SymbolPickup...) **/
  while((bRet = GetMessage(&msg,NULL,0,0)) != 0)
    {
      if(bRet == -1)
        break;

      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }

  /* Fin du Thread */
  ExitThread(0);
}


/*****************************************************************************/
/*  ProcHandlerSymbolPickupWindow() :  Gestion des vnements de la fentre. */
/*****************************************************************************/
LRESULT CALLBACK ProcHandlerSymbolPickupWindow(HWND hwndSymbolPickup, UINT umsg, WPARAM wParam, LPARAM lParam)
{
  int i, select_id, symbol_id;
  UINT code, code_beginlabeledit, code_endlabeledit, code_getdispinfo, code_setdispinfo;
  BOOL result;
  NMLVDISPINFO *plvdi;
  LVITEM LvItem;
  NMLISTVIEW *pListView;
  char buffer_symbol[256];
  struct list_view *current_listview;
  struct window_symbol_pickup *window_symbol_pickup;
  struct window_main *window_main;
  struct parameter *param;
  struct symbol *current_symbol;
  
  /* Init */
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  my_Memory(MEMORY_GET_WINDOW_SYMBOL_PICKUP,&window_symbol_pickup,NULL);
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);

  switch(umsg)
    {
      case WM_INITDIALOG :
        /* Cration du contenu de la DialogBox */
        CreateSymbolPickupChildWindows(hwndSymbolPickup,window_symbol_pickup);

        /* On Disable la fentre mre */
        EnableWindow(window_symbol_pickup->hwnd_parent,FALSE);
        return(TRUE);
        
      /****************************/
      /* Evnements de la fentre */
      /****************************/
      case WM_COMMAND :
        switch(LOWORD(wParam))
          {
            /** Bouton : Vide la table des Symbol **/
            case IDC_BUTTON_SYMBOL_OK :
              if(HIWORD(wParam) == BN_CLICKED)
                {
                  /* Transfre les informations du Symbol */
                  if(window_symbol_pickup->symbol_address != 0xFFFFFFFF)
                    {
                      param->symbol_is_available = 1;
                      param->symbol_address = window_symbol_pickup->symbol_address;
                      param->symbol_size = window_symbol_pickup->symbol_size;
                      strcpy(param->symbol_name,window_symbol_pickup->symbol_name);
                      strcpy(param->symbol_segment,window_symbol_pickup->symbol_segment);
                      strcpy(param->symbol_file,window_symbol_pickup->symbol_file);

                      /* Demande la MAJ de la fentre Parent */
                      SendMessage(window_symbol_pickup->hwnd_parent,window_symbol_pickup->refresh_message_id,0,0);
                    }

                  /* Ferme la fentre SymbolPickup */
                  SendMessage(window_symbol_pickup->hwnd_symbol_pickup,WM_CLOSE,0,0);
                }
              break;

            /** Bouton pour reloger le Segment slectionn :  **/
            case IDC_BUTTON_SYMBOL_CANCEL :
              if(HIWORD(wParam) == BN_CLICKED)
                SendMessage(window_symbol_pickup->hwnd_symbol_pickup,WM_CLOSE,0,0);
              break;
          }
        return(TRUE);

      /********************************************/
      /* Evnements des CommonControls (ListView) */
      /********************************************/
      case WM_NOTIFY :
        code = ((LPNMHDR)lParam)->code;
        code_beginlabeledit = LVN_BEGINLABELEDIT;
        code_endlabeledit   = LVN_ENDLABELEDIT;
        code_getdispinfo    = LVN_GETDISPINFO;
        code_setdispinfo    = LVN_SETDISPINFO;

        /*****************************************************************************************************/
        /**  Affichage des Item de la fentre : Remplissage des Item dans les colonnes des lignes affiches **/
        /*****************************************************************************************************/
        if(code == LVN_GETDISPINFO)
          {
            plvdi = (NMLVDISPINFO *) lParam;

            /* Identifie la ListView : Breakpoint / Symbols */
            if(LOWORD(wParam) == window_symbol_pickup->listview_symbol_pickup.idc_listview)
              current_listview = (struct list_view *) &window_symbol_pickup->listview_symbol_pickup;
            else
              break;

            /* Valide le numro de la colonne */
            if(plvdi->item.iSubItem < current_listview->nb_column)
              {
                plvdi->item.pszText = current_listview->tab_column[plvdi->item.iSubItem].tab_value_txt[(current_listview->has_column_sort == INTERNALS_COLUMN_SORT_NONE) ? plvdi->item.iItem : plvdi->item.lParam];
              }
          }

        /***************************************************************/
        /**  Click sur une ligne d'une ListView pour la slectionner  **/
        /***************************************************************/
        if(code == NM_CLICK)
          {
            /* Rcupre la ListView (Breakpoint / Symbol) */
            if(LOWORD(wParam) == window_symbol_pickup->listview_symbol_pickup.idc_listview)
              current_listview = (struct list_view *) &window_symbol_pickup->listview_symbol_pickup;
            else
              break;

            /* Identifie la ligne clicke (0-N) */
            select_id = (int) SendMessage(current_listview->hwnd_listview,LVM_GETNEXTITEM,-1,LVNI_FOCUSED); // return item selected
            if(select_id == -1)
              return(TRUE);

            /* Rcupre le numro de la structure Symbol dans la Colonne #7 */
            memset(&LvItem,0,sizeof(LVITEM));
            LvItem.mask = LVIF_TEXT;
            LvItem.iSubItem = 8;            /* Colonne #8 */
            LvItem.pszText = buffer_symbol;
            LvItem.cchTextMax = 256;
            LvItem.iItem = select_id;
            SendMessage(current_listview->hwnd_listview,LVM_GETITEMTEXT,select_id,(LPARAM)&LvItem);
            symbol_id = atoi(buffer_symbol);

            /** On remplir les informations complmentaires du Symbol **/
            if(symbol_id < window_main->current_symbol_table->nb_symbol)
              {
                for(i=0,current_symbol = window_main->current_symbol_table->first_symbol; current_symbol; current_symbol=current_symbol->next,i++)
                  if(i == symbol_id)
                    {
                      window_symbol_pickup->symbol_address = current_symbol->symbol_relocated_address;
                      window_symbol_pickup->symbol_size = current_symbol->size;
                      strcpy(window_symbol_pickup->symbol_name,current_symbol->name);
                      strcpy(window_symbol_pickup->symbol_segment,current_symbol->segment_name);
                      strcpy(window_symbol_pickup->symbol_file,current_symbol->program_name);
                      break;
                    }
              }
          }
       
        /*************************************************************************/
        /**  Double-click sur une ligne d'une ListView pour activer une action  **/
        /*************************************************************************/
        if(code == NM_DBLCLK)
          {
            /* Rcupre la ListView (Breakpoint / Symbols) */
            if(LOWORD(wParam) == window_symbol_pickup->listview_symbol_pickup.idc_listview)
              current_listview = (struct list_view *) &window_symbol_pickup->listview_symbol_pickup;
            else
              break;

            /* Identifie la ligne clicke (0-N) */
            select_id = (int) SendMessage(current_listview->hwnd_listview,LVM_GETNEXTITEM,-1,LVNI_FOCUSED); // return item selected
            if(select_id == -1)
              return(TRUE);

            /* Rcupre le numro de la structure Symbol dans la Colonne #7 */
            memset(&LvItem,0,sizeof(LVITEM));
            LvItem.mask = LVIF_TEXT;
            LvItem.iSubItem = 8;            /* Colonne #8 */
            LvItem.pszText = buffer_symbol;
            LvItem.cchTextMax = 256;
            LvItem.iItem = select_id;
            SendMessage(current_listview->hwnd_listview,LVM_GETITEMTEXT,select_id,(LPARAM)&LvItem);
            symbol_id = atoi(buffer_symbol);

            /** On remplir les informations complmentaires du Symbol + Ferme la fentre **/
            if(symbol_id < window_main->current_symbol_table->nb_symbol)
              {
                for(i=0,current_symbol = window_main->current_symbol_table->first_symbol; current_symbol; current_symbol=current_symbol->next,i++)
                  if(i == symbol_id)
                    {
                      /* Transfert les donnes du Symbol */
                      param->symbol_is_available = 1;
                      param->symbol_address = window_symbol_pickup->symbol_address;
                      param->symbol_size = window_symbol_pickup->symbol_size;
                      strcpy(param->symbol_name,window_symbol_pickup->symbol_name);
                      strcpy(param->symbol_segment,window_symbol_pickup->symbol_segment);
                      strcpy(param->symbol_file,window_symbol_pickup->symbol_file);

                      /* Demande la MAJ de la fentre Parent */
                      SendMessage(window_symbol_pickup->hwnd_parent,window_symbol_pickup->refresh_message_id,0,0);

                      /* Ferme la fentre SymbolPickup */
                      SendMessage(window_symbol_pickup->hwnd_symbol_pickup,WM_CLOSE,0,0);
                      break;
                    }
              }
          }

        /***************************************************************/
        /**  Click sur les titres des colonnes pour activer les tris  **/
        /***************************************************************/
        if(code == LVN_COLUMNCLICK)
          {
            pListView = (NMLISTVIEW *) lParam;
            select_id = (int) pListView->iSubItem;

            /* Rcupre la ListView (Breakpoint / Symbols) */
            if(LOWORD(wParam) == window_symbol_pickup->listview_symbol_pickup.idc_listview)
              current_listview = (struct list_view *) &window_symbol_pickup->listview_symbol_pickup;
            else
              break;

            /* On fait le choix du type de tri */
            for(i=0; i<current_listview->nb_column; i++)
              {
                if(i == select_id)
                  {
                    if(current_listview->tab_column[i].sort == INTERNALS_COLUMN_SORT_NONE)
                      current_listview->tab_column[i].sort = INTERNALS_COLUMN_SORT_UP;
                    else if(current_listview->tab_column[i].sort == INTERNALS_COLUMN_SORT_UP)
                      current_listview->tab_column[i].sort = INTERNALS_COLUMN_SORT_DOWN;
                    else
                      current_listview->tab_column[i].sort = INTERNALS_COLUMN_SORT_NONE;
                  }
               else
                 current_listview->tab_column[i].sort = INTERNALS_COLUMN_SORT_NONE;
              }
             
            /* A t'on une colonne trie ? */
            current_listview->has_column_sort = INTERNALS_COLUMN_SORT_NONE;
            for(i=0; i<current_listview->nb_column; i++)
              if(current_listview->tab_column[i].sort != INTERNALS_COLUMN_SORT_NONE)
                current_listview->has_column_sort = current_listview->tab_column[i].sort;
             
            /* On va trier selon le tri mis en place */
            result = ListView_SortItems(current_listview->hwnd_listview,ProcessListViewCompareProc,current_listview);
           
            /* MAJ des images ^ / v des Tris des colonnes */
            setListViewSortIcon(current_listview);
          }

        /***********************************************************************************/
        /**  Gestion de l'affichage des Item pour dfinir une couleur de fond de cellule  **/
        /***********************************************************************************/
        if(code == NM_CUSTOMDRAW)
          {
            SetWindowLongPtr(window_symbol_pickup->hwnd_symbol_pickup,DWLP_MSGRESULT,(LONG_PTR)ProcessListViewWindowCustomDraw(lParam));
            return(CDRF_NOTIFYITEMDRAW);
          }

        return(TRUE);
        break;
  
      /**************************************/
      /*  On vient de retailler la fentre  */
      /**************************************/
      case WM_SIZE :
        /* On va tendre la fentre de donne mmoire si c'est un Resize User */
        if(window_symbol_pickup->user_resize == 1)
          UserResizeSymbolPickupWindow(window_symbol_pickup);
        window_symbol_pickup->user_resize = 1;
        return(TRUE);
        
      /********************************************************************/
      /* Fermeture de window (le WM_CLOSE est appel avant le WM_DESTROY) */
      /********************************************************************/
      case WM_CLOSE :
        /* Fentre SymbolPickup */
        DestroyWindow(hwndSymbolPickup);
        return(0);

      case WM_DESTROY :
        /* On Enable la fentre mre */
        EnableWindow(window_symbol_pickup->hwnd_parent,TRUE);

        /* Suppression des Fentres */
        DeleteSymbolPickupChildWindows(window_symbol_pickup);
        my_Memory(MEMORY_FREE_WINDOW_SYMBOL_PICKUP,NULL,NULL);

        /* Demande la sortie de la boucle des messages */
        PostQuitMessage(0);
        return(0);
    }

  /* Retourner FALSE pour relayer le message non trait au systme */
  return(FALSE);
}


/****************************************************************************************************************/
/*  CreateSymbolPickupChildWindows() : Crer les fentres enfants et remplir la structure window_symbol_pickup. */
/****************************************************************************************************************/
static int CreateSymbolPickupChildWindows(HWND hwndSymbolPickup, struct window_symbol_pickup *window_symbol_pickup)
{
  RECT window_rect;
  struct window_main *window_main;
  char buffer[BUFFER_SIZE];
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Rcupre la fentre Main */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);

  /** Window Handle **/
  window_symbol_pickup->hwnd_symbol_pickup = hwndSymbolPickup;

  /* Police de caractre de la fentre (14 points) */
  window_symbol_pickup->hFont = CreateFontA(LISTVIEW_FONT_HEIGHT,0,0,0,FW_DONTCARE,FALSE,FALSE,FALSE,ANSI_CHARSET,
                                            OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY,DEFAULT_PITCH|FF_SWISS,"Arial");

  /*********************/
  /*** Symbol Pickup ***/
  /*********************/
  window_symbol_pickup->symbol_x0 = 7;
  window_symbol_pickup->symbol_y0 = 7;

  /** List View **/
  BuildSymbolPickupListView(&window_symbol_pickup->listview_symbol_pickup,window_symbol_pickup->hFont,IDC_LISTVIEW_SYMBOL_PICKUP,window_symbol_pickup);

  /** Remplissage du List View si les symbols sont disponibles **/
  if(window_main->current_symbol_table != NULL)
    FillSymbolPickupListView(&window_symbol_pickup->listview_symbol_pickup,window_main->current_symbol_table);

  /* MAJ du titre de la fentre avec le nombre de Symbol */
  if(window_main->current_symbol_table != NULL)
    {
      sprintf(buffer,"  Pickup Symbol : %d entries",window_main->current_symbol_table->nb_symbol);
      SetWindowText(window_symbol_pickup->hwnd_symbol_pickup,buffer);
    }

  /** Taille de la fentre au dmarrage ((avec la barre de titre et les bordures)) **/
  GetWindowRect(window_symbol_pickup->hwnd_symbol_pickup,&window_rect);
  window_symbol_pickup->window_init_width  = window_rect.right  - window_rect.left;
  window_symbol_pickup->window_init_height = window_rect.bottom - window_rect.top;

  /* Nombre de ligne dans la ListView au dmarrage de la fentre */
  window_symbol_pickup->nb_init_data_row = SYMBOL_PICKUP_LISTVIEW_NB_ROW;

  /* Nombre de lignes qu'il est possible d'afficher dans la ListView */
  window_symbol_pickup->nb_row = window_symbol_pickup->nb_init_data_row;

  /* OK */
  return(0);
}


/**************************************************************************************/
/*  BuildSymbolPickupListView() :  Construction d'une ListView pour les SymbolPickup. */
/**************************************************************************************/
static void BuildSymbolPickupListView(struct list_view *current_listview, HFONT hFont, int idc_listview, struct window_symbol_pickup *window_symbol_pickup)
{
  /** Init de la structure List_view **/
  current_listview->idc_listview = idc_listview;

  /** Liste des Colonnes **/
  current_listview->nb_column = 0;

  /* Col 0 : Validit */
  current_listview->tab_column[0].type = INTERNALS_COLUMN_TYPE_TXT;
  current_listview->tab_column[0].align = LVCFMT_CENTER;
  current_listview->tab_column[0].length = 30;
  strcpy(current_listview->tab_column[0].title," ");
  current_listview->nb_column++;

  /* Col 1 : Address */
  current_listview->tab_column[1].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[1].align = LVCFMT_LEFT;
  current_listview->tab_column[1].length = 77;
  strcpy(current_listview->tab_column[1].title,"Address");
  current_listview->nb_column++;

  /* Col 2 : Name */
  current_listview->tab_column[2].type = INTERNALS_COLUMN_TYPE_TXT;
  current_listview->tab_column[2].align = LVCFMT_LEFT;
  current_listview->tab_column[2].length = 150;
  strcpy(current_listview->tab_column[2].title,"Name");
  current_listview->nb_column++;

  /* Col 3 : Type / Date Type */
  current_listview->tab_column[3].type = INTERNALS_COLUMN_TYPE_TXT;
  current_listview->tab_column[3].align = LVCFMT_LEFT;
  current_listview->tab_column[3].length = 75;
  strcpy(current_listview->tab_column[3].title,"Type");
  current_listview->nb_column++;

  /* Col 4 : Size */
  current_listview->tab_column[4].type = INTERNALS_COLUMN_TYPE_NUM;
  current_listview->tab_column[4].align = LVCFMT_RIGHT;
  current_listview->tab_column[4].length = 60;
  strcpy(current_listview->tab_column[4].title,"Size");
  current_listview->nb_column++;

  /* Col 5 : MX */
  current_listview->tab_column[5].type = INTERNALS_COLUMN_TYPE_TXT;
  current_listview->tab_column[5].align = LVCFMT_LEFT;
  current_listview->tab_column[5].length = 50;
  strcpy(current_listview->tab_column[5].title,"MX");
  current_listview->nb_column++;

  /* Col 6 : Source File Name */
  current_listview->tab_column[6].type = INTERNALS_COLUMN_TYPE_TXT;
  current_listview->tab_column[6].align = LVCFMT_LEFT;
  current_listview->tab_column[6].length = 110;
  strcpy(current_listview->tab_column[6].title,"File");
  current_listview->nb_column++;

  /* Col 7 : Segment Name */
  current_listview->tab_column[7].type = INTERNALS_COLUMN_TYPE_TXT;
  current_listview->tab_column[7].align = LVCFMT_LEFT;
  current_listview->tab_column[7].length = 100;
  strcpy(current_listview->tab_column[7].title,"Segment");
  current_listview->nb_column++;

  /* Col 8 : ID de la structure Symbol */
  current_listview->tab_column[8].type = INTERNALS_COLUMN_TYPE_TXT;
  current_listview->tab_column[8].align = LVCFMT_LEFT;
  current_listview->tab_column[8].length = 1;
  strcpy(current_listview->tab_column[8].title,"");
  current_listview->nb_column++;

  /** Cration du ListView **/
  BuildListView(window_symbol_pickup->hwnd_symbol_pickup,hFont,window_symbol_pickup->symbol_x0,window_symbol_pickup->symbol_y0,SYMBOL_PICKUP_LISTVIEW_NB_ROW,current_listview);
}


/************************************************************************************/
/*  FillSymbolPickupListView() :  Remplissage d'une ListView pour les SymbolPickup. */
/************************************************************************************/
static void FillSymbolPickupListView(struct list_view *current_listview, struct symbol_table *current_symbol_table)
{
  int i, result;
  LVITEM listview_item;
  struct symbol *current_symbol;
  char buffer[1024];

  /* Libration mmoire des donnes affiches dans les colonnes de la ListView */
  mem_free_listviewcolumn(current_listview);

  /* Aucun Symbol */
  if(current_symbol_table == NULL)
    return;
  if(current_symbol_table->nb_symbol == 0)
    return;

  /** Remplissage avec les valeurs dans la structure mmoire **/
  current_listview->nb_row = current_symbol_table->nb_symbol;
  for(i=0; i<current_listview->nb_column; i++)
    {
      /* Allocation des tableaux */
      current_listview->tab_column[i].tab_value_num = (DWORD *) calloc(current_listview->nb_row,sizeof(DWORD));
      current_listview->tab_column[i].tab_value_txt = (char **) calloc(current_listview->nb_row,sizeof(char *));
    }
  /** Remplissage des tableaux **/
  for(i=0, current_symbol=current_symbol_table->first_symbol; current_symbol; current_symbol=current_symbol->next, i++)
    {
      /* Colonne 0 : Validit */
      current_listview->tab_column[0].tab_value_num[i] = current_symbol->is_valid;
      sprintf(buffer,"%s",(current_symbol->is_valid == 1) ? "Ok" : "?");
      current_listview->tab_column[0].tab_value_txt[i] = strdup(buffer);

      /* Colonne 1 : Address Reloge */
      current_listview->tab_column[1].tab_value_num[i] = current_symbol->symbol_relocated_address;
      sprintf(buffer,"%02X/%04X",(current_symbol->symbol_relocated_address >> 16) & 0xFF,current_symbol->symbol_relocated_address & 0xFFFF);
      current_listview->tab_column[1].tab_value_txt[i] = strdup(buffer);

      /* Colonne 2 : Name */
      current_listview->tab_column[2].tab_value_num[i] = i;
      current_listview->tab_column[2].tab_value_txt[i] = strdup(current_symbol->name);

      /* Colonne 3 : Type / Data Type */
      current_listview->tab_column[3].tab_value_num[i] = i;
      strcpy(buffer,current_symbol->type);
      if(strlen(current_symbol->data_type) > 0)
        sprintf(&buffer[strlen(buffer)]," (%s)",current_symbol->data_type);
      current_listview->tab_column[3].tab_value_txt[i] = strdup(buffer);

      /* Colonne 4 : Size */
      current_listview->tab_column[4].tab_value_num[i] = current_symbol->size;
      sprintf(buffer,"%d",current_symbol->size);
      current_listview->tab_column[4].tab_value_txt[i] = strdup(buffer);

      /* Colonne 5 : MX */
      current_listview->tab_column[5].tab_value_num[i] = i;
      sprintf(buffer,"%c%c",current_symbol->m,current_symbol->x);
      current_listview->tab_column[5].tab_value_txt[i] = strdup(buffer);

      /* Colonne 6 : File Name */
      current_listview->tab_column[6].tab_value_num[i] = i;
      current_listview->tab_column[6].tab_value_txt[i] = strdup(current_symbol->program_name);

      /* Colonne 7 : Segment Name */
      current_listview->tab_column[7].tab_value_num[i] = i;
      current_listview->tab_column[7].tab_value_txt[i] = strdup(current_symbol->segment_name);

      /* Colonne 8 : ID de la structure Symbol [0-N] */
      current_listview->tab_column[8].tab_value_num[i] = i;
      sprintf(buffer,"%d",i);
      current_listview->tab_column[8].tab_value_txt[i] = strdup(buffer);
    }

  /** Transfert des donnes dans la List View via un Msg LVN_GETDISPINFO **/
  /** Ajout des Item dans le ListView **/
  listview_item.pszText   = LPSTR_TEXTCALLBACK;           /* Sends an LVN_GETDISPINFO message to fill the item */
  listview_item.mask      = LVIF_TEXT | LVIF_IMAGE | LVIF_STATE | LVIF_PARAM;
  listview_item.stateMask = 0;
  listview_item.iSubItem  = 0;
  listview_item.state     = 0;

  /** Initialize LVITEM members that are different for each item **/
  for(i=0; i<current_listview->nb_row; i++)
    {
      listview_item.iItem  = i;
      listview_item.iImage = -1;         /* Identifiant de l'image dans la liste */
      listview_item.lParam = i;          /* Identifiant renvoy lors du tri */
      result = ListView_InsertItem(current_listview->hwnd_listview,&listview_item);
    }
}


/*****************************************************************************/
/*  UserResizeSymbolPickupWindow() :  Retaille la fentre des Symbol Pickup. */
/*****************************************************************************/
static void UserResizeSymbolPickupWindow(struct window_symbol_pickup *window_symbol_pickup)
{
  RECT window_rect;
  int window_x, window_y, window_new_height, minimum_window_height;
  int listview_pixel_height, nb_listview_data_row, delta_listview_nb_row, delta_pixel_height;

  /* Position + Dimension actuelle de la fentre retaille dans l'cran */
  GetWindowRect(window_symbol_pickup->hwnd_symbol_pickup,&window_rect);

  /* Nouvelle Dimension de la fentre (avec la barre de titre et les bordures) */
  window_x = window_rect.left;
  window_y = window_rect.top;
  window_new_height = window_rect.bottom - window_rect.top;

  /* Nombre de Pixels restant pour la ListView */
  minimum_window_height = window_symbol_pickup->window_init_height - window_symbol_pickup->nb_init_data_row*SYMBOL_PICKUP_LISTVIEW_ROW_HEIGHT;
  listview_pixel_height = window_new_height - minimum_window_height;

  /* Nombre de lignes de la ListView */
  nb_listview_data_row = listview_pixel_height / SYMBOL_LISTVIEW_ROW_HEIGHT;

  /** Teste les valeurs minimales autorises **/
  if(nb_listview_data_row < SYMBOL_LISTVIEW_MIN_NB_ROW)
    nb_listview_data_row = SYMBOL_LISTVIEW_MIN_NB_ROW;

  /* Delta de vla ListView */
  delta_listview_nb_row = nb_listview_data_row - window_symbol_pickup->nb_row;
  delta_pixel_height = delta_listview_nb_row*SYMBOL_LISTVIEW_ROW_HEIGHT;
  window_symbol_pickup->nb_row = nb_listview_data_row;

  /* Nouvelle hauteur de la fentre (on ajuste pour tomber sur un nombre extact de ligne dans la ListView) */
  window_new_height = minimum_window_height + nb_listview_data_row*SYMBOL_LISTVIEW_ROW_HEIGHT;

  /* Retaille la fentre */
  window_symbol_pickup->user_resize = 0;
  MoveWindow(window_symbol_pickup->hwnd_symbol_pickup,window_x,window_y,window_symbol_pickup->window_init_width,window_new_height,FALSE);

  /* Dplace les Contrles */
  MoveSymbolPickupWindowControls(delta_pixel_height,window_symbol_pickup);

  /* Rafraichit la fentre */
  InvalidateRect(window_symbol_pickup->hwnd_symbol_pickup,NULL,TRUE);
  UpdateWindow(window_symbol_pickup->hwnd_symbol_pickup);
}


/***********************************************************************************************/
/*  MoveSymbolPickupWindowControls() :  On va dplacer les diffrents controles de la fentre. */
/***********************************************************************************************/
static void MoveSymbolPickupWindowControls(int delta_y, struct window_symbol_pickup *window_symbol_pickup)
{
  HWND hwnd_control, hwnd_listview;
  int i, control_x, control_y, control_width, control_height;
  int listview_x, listview_y, listview_width, listview_height;
  int tab_control_id[] = 
    {
      IDC_BUTTON_SYMBOL_CANCEL, IDC_BUTTON_SYMBOL_OK,
      0
    };

  /** On va modifier la hauteur de la ListBox **/
  /* Handle du Controle */
  hwnd_listview = window_symbol_pickup->listview_symbol_pickup.hwnd_listview;
  /* Position + Taille actuelle */
  GetWindowControlInfo(window_symbol_pickup->hwnd_symbol_pickup,hwnd_listview,&listview_x,&listview_y,&listview_width,&listview_height);
  /* Change la taille le controle verticalement */
  MoveWindow(hwnd_listview,listview_x,listview_y,listview_width,listview_height+delta_y,FALSE);

  /** On va dplacer tous les controles verticalement **/
  for(i=0; tab_control_id[i] != 0; i++)
    {
      /* Handle du Controle */
      hwnd_control = GetDlgItem(window_symbol_pickup->hwnd_symbol_pickup,tab_control_id[i]);

      /* Position + Taille actuelle */
      GetWindowControlInfo(window_symbol_pickup->hwnd_symbol_pickup,hwnd_control,&control_x,&control_y,&control_width,&control_height);

      /* Dplace le controle verticalement */
      MoveWindow(hwnd_control,control_x,control_y+delta_y,control_width,control_height,FALSE);
    }
}


/**************************************************************************/
/*  DeleteSymbolPickupChildWindows() :  Destruction des fentres enfants. */
/**************************************************************************/
static void DeleteSymbolPickupChildWindows(struct window_symbol_pickup *window_symbol_pickup)
{
}


/*****************************************************************************/
/*  mem_free_window_symbol_pickup() :  Libration mmoire pour la structure. */
/*****************************************************************************/
void mem_free_window_symbol_pickup(struct window_symbol_pickup *window_symbol_pickup)
{
  if(window_symbol_pickup)
    {      
      free(window_symbol_pickup);
    }
}

/**********************************************************************/
