/***********************************************************************/
/*                                                                     */
/*   Dc_Cyrene.c :  Module de gestion des changes IPC avec Cyrene.    */
/*                                                                     */
/***********************************************************************/
/*  Auteur : Olivier ZARDINI  *   Brutal Deluxe   *   Date : May 2023  */
/***********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/timeb.h>

#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>

#include "defc.h"
#include "sound.h"
#include "Cyrene.h"
#include "Cyrene_Shared.h"
#include "Cyrene_Operation.h"
#include "Cyrene_Breakpoint.h"
#include "Cyrene_SystemCall.h"
#include "Cyrene_WindowIPC.h"

/* Cyrene Executable IPC Window */
#define CYRENE_IPC_WINDOW_CLASS   "CyreneIPC"
#define CYRENE_IPC_WINDOW_NAME    "Cyrene_%d"

static void mem_free_cyrene_parameter(struct cyrene_parameter *);


/******************************************************************************/
/*  InitCyrene() :  Init Cyrene Layer + Launch IPC Thread + Build IPC Window. */
/******************************************************************************/
struct cyrene_parameter *InitCyrene(void)
{
  DWORD thread_id, raz_value;
  HANDLE thread_handle;
  struct cyrene_parameter *cyrene_param;

  /* Init */
  my_Cyrene(CYRENE_INIT,NULL,NULL);

  /** Allocation de la structure cyrene_param **/
  cyrene_param = (struct cyrene_parameter *) calloc(1,sizeof(struct cyrene_parameter));
  if(cyrene_param == NULL)
    {
      printf("%s[KEGS//CYRENE]    => Error, Can't allocate memory for structure cyrene_param\n",GetCurrentTimeMS());
      return(NULL);
    }
  /* Header (1 KB) + 16 MB RAM/ROM + 64 KB DOC RAM */
  cyrene_param->request_data = (unsigned char *) calloc(1024+16*1024*1024+64*1024,sizeof(unsigned char));
  if(cyrene_param->request_data == NULL)
    {
      free(cyrene_param);
      printf("%s[KEGS//CYRENE]    => Error, Can't allocate memory for structure cyrene_param\n",GetCurrentTimeMS());
      return(NULL);
    }
  my_Cyrene(CYRENE_SET_PARAM,cyrene_param,NULL);

  /* Cyrene IPC Window Class Name */
  strcpy(cyrene_param->cyrene_ipc_window_class_name,CYRENE_IPC_WINDOW_CLASS);

  /* KEGS Process ID */
  cyrene_param->kegs_process_id = GetCurrentProcessId();
  
  /* Status de la connection */
  cyrene_param->cyrene_connection_status = CONNECTION_STATUS_CLOSED;

  /* No next expected address available */
  cyrene_param->next_operation_address = 0xFFFFFFFF;

  /** Cration de la Shared Memory 'Local\Cyrene_<kegs_pid>' **/
  sprintf(cyrene_param->shared_memory_path,"Local\\"CYRENE_IPC_WINDOW_NAME,cyrene_param->kegs_process_id);
  cyrene_param->hSharedMemory = CreateFileMapping(INVALID_HANDLE_VALUE,NULL,PAGE_READWRITE,0,IPC_SHARED_MEMORY_TOTAL_SIZE,cyrene_param->shared_memory_path);
  if(cyrene_param->hSharedMemory == NULL)
    {
      printf("%s[KEGS//CYRENE]    => Error, Can't create KeCyrene Shared Memory\n",GetCurrentTimeMS());
      my_Cyrene(CYRENE_FREE_PARAM,NULL,NULL);
      return(NULL);
    }
  cyrene_param->shared_memory_ptr = (unsigned char *) MapViewOfFile(cyrene_param->hSharedMemory,FILE_MAP_ALL_ACCESS,0,0,IPC_SHARED_MEMORY_TOTAL_SIZE);
  if(cyrene_param->shared_memory_ptr == NULL)
    {
      printf("%s[KEGS//CYRENE]    => Error, Can't create Cyrene Shared Memory\n",GetCurrentTimeMS());
      my_Cyrene(CYRENE_FREE_PARAM,NULL,NULL);
      return(NULL);
    }
  /* RAZ */
  memset(cyrene_param->shared_memory_ptr,0x00,IPC_SHARED_MEMORY_TOTAL_SIZE);
  raz_value = 0;
  memcpy(&cyrene_param->shared_memory_ptr[IPC_SHARED_MEMORY_OFFSET_OUTPUT_READ],&raz_value,sizeof(DWORD));
  memcpy(&cyrene_param->shared_memory_ptr[IPC_SHARED_MEMORY_OFFSET_OUTPUT_WRITE],&raz_value,sizeof(DWORD));
  memcpy(&cyrene_param->shared_memory_ptr[IPC_SHARED_MEMORY_OUTPUT_WRITE_MISSED],&raz_value,sizeof(DWORD));

  /** Lancement du Thread  + Cration de la fenetre IPC **/
  cyrene_param->kegs_ipc_window_status = IPC_WINDOW_STATUS_WAIT;
  thread_handle = CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)CyreneIPCThreadInit,NULL,0,&thread_id);
  if(thread_handle == NULL)
    {
      printf("%s[KEGS//CYRENE]    => Error, Can't create Thread for Cyrene IPC Window\n",GetCurrentTimeMS());
      my_Cyrene(CYRENE_FREE_PARAM,NULL,NULL);
      return(NULL);
    }
  
  /* Conserve le Handle de ce Thread */
  cyrene_param->kegs_ipc_thread_handle = thread_handle;
  cyrene_param->kegs_ipc_thread_id = thread_id;

  /** Attend la cration de la fentre **/
  while(cyrene_param->kegs_ipc_window_status == IPC_WINDOW_STATUS_WAIT)
    Sleep(1);
  if(cyrene_param->kegs_ipc_window_status == IPC_WINDOW_STATUS_ERROR)
    {
      my_Cyrene(CYRENE_FREE_PARAM,NULL,NULL);
      return(NULL);
    }

  /* Info */
  printf("%s[KEGS//CYRENE] : Initialization Complete\n",GetCurrentTimeMS());

  /* OK */
  return(cyrene_param);
}


/****************************************************************************/
/*  FreeCyrene() :  Free Cyrene Layer (Stop IPC Thread + Close IPC Window). */
/****************************************************************************/
void FreeCyrene(void)
{
  BOOL result;
  DWORD exit_code;
  struct cyrene_parameter *cyrene_param;
  my_Cyrene(CYRENE_GET_PARAM,&cyrene_param,NULL);

  /* Log */
  printf("%s[KEGS//CYRENE] : Closing Service...\n",GetCurrentTimeMS());

  /* Not Started */
  if(cyrene_param == NULL)
    return;

  /* Ferme la fentre Kegs IPC */
  SendMessage(cyrene_param->hwnd_kegs_ipc,WM_CLOSE,0,0);

  /* Attend la fin du Thread Kegs IPC */
  while(1)
    {
      result = GetExitCodeThread(cyrene_param->kegs_ipc_thread_handle,&exit_code);
      if(result && exit_code != STILL_ACTIVE)
        {
          CloseHandle(cyrene_param->kegs_ipc_thread_handle);
          break;
        }
    }

  /* Libre la mmoire */
  my_Cyrene(CYRENE_FREE_PARAM,NULL,NULL);
}


/*************************************************************/
/*  my_Cyrene() :  Gestion du module des ressources mmoire. */
/*************************************************************/
void my_Cyrene(int code, void *data, void *value)
{
  static struct cyrene_parameter *cyrene_param;

  switch(code)
    {
      case CYRENE_INIT :
        cyrene_param = NULL;
        break;

      case CYRENE_FREE :
        if(cyrene_param)
          free(cyrene_param);
        cyrene_param = NULL;
        break;

      case CYRENE_SET_PARAM :
        cyrene_param = (struct cyrene_parameter *) data;
        break;

      case CYRENE_GET_PARAM :
        *((struct cyrene_parameter **) data) = cyrene_param;
        break;

      case CYRENE_FREE_PARAM :
        if(cyrene_param)
          mem_free_cyrene_parameter(cyrene_param);
        cyrene_param = NULL;
        break;
    }
}


/*************************************************************************/
/*  mem_free_cyrene_parameter() :  Libration mmoire pour la structure. */
/*************************************************************************/
static void mem_free_cyrene_parameter(struct cyrene_parameter *cyrene_param)
{
  struct cyrene_breakpoint *current_breakpoint;
  struct cyrene_breakpoint *next_breakpoint;
  struct cyrene_system_call *current_system_call;
  struct cyrene_system_call *next_system_call;

  if(cyrene_param)
    {
      if(cyrene_param->request_data)
        free(cyrene_param->request_data);

      /* Shared Memory */
      if(cyrene_param->shared_memory_ptr != NULL)
        UnmapViewOfFile(cyrene_param->shared_memory_ptr);
      if(cyrene_param->hSharedMemory != NULL)
        CloseHandle(cyrene_param->hSharedMemory);

      /* Breakpoint List */
      for(current_breakpoint = cyrene_param->first_breakpoint; current_breakpoint; current_breakpoint = next_breakpoint)
        {
          next_breakpoint = current_breakpoint->next;
          mem_free_breakpoint(current_breakpoint);
        }

      /* System Call List */
      for(current_system_call = cyrene_param->first_system_call; current_system_call; current_system_call = next_system_call)
        {
          next_system_call = current_system_call->next;
          mem_free_system_call(current_system_call);
        }

      free(cyrene_param);
    }
}

/*****************************************************************************/
