/***********************************************************************/
/*                                                                     */
/*   Dc_Cyrene.h :  Header de gestion des changes IPC avec Cyrene.    */
/*                                                                     */
/***********************************************************************/
/*  Auteur : Olivier ZARDINI  *   Brutal Deluxe   *   Date : May 2023  */
/***********************************************************************/

typedef unsigned __int64          QWORD;    /* Unsigned 64 bit */
#if defined(WIN32)
typedef unsigned long             DWORD;    /* Unsigned 32 bit */
#else
typedef uint32_t                  DWORD;    /* Unsigned 32 bit */
#endif
typedef unsigned short            WORD;     /* Unsigned 16 bit */
typedef unsigned char             BYTE;     /* Unsigned 8 bit */

typedef int (*IO_READ)(word32, dword64 *);
typedef word32 (*GET_LINES_SINCE_VBL)(dword64);
typedef void (* X_UPDATE_DISPLAY)(void *);  /* Window_info * */
typedef void (* CHECK_EVENTS)();

/* Maximum number of Operations kept in Cache before release */
#define MAX_OPERATION_CACHE                   20000   

/* Operation Data Buffer Size */
#define OPERATION_DATA_SIZE                      59   /* 1 Operation Data Size */

/* Cyrene Memory Management */
#define CYRENE_INIT                              10   /* Memory Structure Management */
#define CYRENE_FREE                              11
#define CYRENE_SET_PARAM                         20
#define CYRENE_GET_PARAM                         21
#define CYRENE_FREE_PARAM                        22

struct cyrene_parameter
{
  /* KEGS Pid */
  DWORD kegs_process_id;

  /** KEGS IPC Thread **/
  HANDLE kegs_ipc_thread_handle;
  DWORD kegs_ipc_thread_id;

  /** KEGS IPC Window **/
  /* Window creation status */
  int kegs_ipc_window_status;           /* 0=NotCreated, 1=Created, 2=Error */

  /* Window Class */
  int kegs_ipc_register_class_status;

  /* KEGS IPC Window Handle */
  HWND hwnd_kegs_ipc;
  
  /* Device Context */
  HDC hDC;
  HDC hCompatibleDC;

  /* Window Title */
  char window_title[1024];

  /* Window Title height (in pixels) */
  int title_height;
  int window_width;
  int window_height;

  /** CYRENE IPC Window **/
  char cyrene_ipc_window_class_name[256];
  char cyrene_ipc_window_name[256];
  HWND hwnd_cyrene_ipc;
  int send_error_count;                   /* Count Send Data Error */

  /** CYRENE IPC Shared Memory **/
  char shared_memory_path[256];           /* Local\cyrene_ipc_window_name */
  HANDLE hSharedMemory;
  unsigned char *shared_memory_ptr;

  /* Breakpoint List */
  struct cyrene_breakpoint *first_breakpoint;
  struct cyrene_breakpoint *last_breakpoint;

  /* System Call Return Values List */
  struct cyrene_system_call *first_system_call;
  struct cyrene_system_call *last_system_call;

  /* Global Operation ID */
  QWORD goid;

  /* Global Cycle Count */
  QWORD gcc;

  /* VBL Status */
  int previous_vbl_status;
  int current_vbl_status;

  /* Previous Operation Address (+Taille) */
  DWORD previous_operation_address;
  int previous_operation_size;

  /* Interrupt */
  DWORD next_operation_address;

  /* Status de la connection avec Cyrene */
  int cyrene_connection_status;           /* CONNECTION_STATUS_CLOSED / CONNECTION_STATUS_OPEN */

  /* Type de donnes attendu par Cyrene */
  int cyrene_connection_query;            /* CONNECTION_QUERY_NOTHING / CONNECTION_QUERY_SNAPSHOT_DATA / CONNECTION_QUERY_OPERATION_DATA */

  /** Send Request Data **/
  DWORD request_type;                     /* Function identifier = REQUEST_TYPE_XXX */
  DWORD request_data_size;                /* Size of data */
  unsigned char *request_data;            /* Request Data (Register / Opcode / Operand / Hardware / Softswitch / Memory...) */

  /** Operation Cache **/
  unsigned char operation_cache_data[MAX_OPERATION_CACHE*OPERATION_DATA_SIZE];   /* Operation Cache Data */
  int nb_operation_cache;                 /* Current number of cached Operation Data */
  int stop_condition;                     /* STOP_CONDITION_XXX */
  int stop_address;                       /* Stop Address (00/0000) */
  unsigned __int64 stop_goid;             /* Stop on this GOID */

  /** Receive Request Data **/
  int cyrene_answer_status;               /* CYRENE_ANSWER_XXX */
  int operation_continue_status;          /* Continue on next instruction=True/False */

  /*** KEGS Memory Data ***/
  /** Engine **/
  Engine_reg *engine;

  /** SoftSwitch **/
  DWORD *g_c035_shadow_reg;
  DWORD *g_c068_statereg;
  IO_READ _io_read;
  GET_LINES_SINCE_VBL _get_lines_since_vbl;

  /* Windows */
  X_UPDATE_DISPLAY _x_update_display;
  CHECK_EVENTS _check_input_events;
  void *g_mainwin_info;                 /* Window_info * */
  void *g_debugwin_info;                /* Window_info * */
  int *g_a2_fatal_err;                  /* Kegs Exit Code */

  /* Video */
  dword64 *g_last_vbl_dfcyc;

  /** GS RAM **/
  int size_ram;
  BYTE *gs_fast_ram;        /* Pointer to Fast RAM Space : $00/0000 - $FD/FFFF [DO_NOT_FREE] */
  BYTE *gs_slow_ram;        /* Pointer to Slow RAM Space : $E0/0000 - $E1/FFFF [DO_NOT_FREE] */
  Page_info *page_info_rd_wr;
  DWORD max_ram_address;

  /** GS ROM FC-FF **/
  int version_rom;          /* Rom01 = 1, Rom03 = 3 */
  BYTE *gs_rom;             /* Pointer to ROM Space : $FC/0000 - $FF/FFFF [DO_NOT_FREE] */
  
  /** DOC RAM : 64 KB **/
  BYTE *doc_ram;            /* Pointer to DOC RAM : $0000 - $FFFF [DO_NOT_FREE] */

  /** DOC Register **/
  Doc_reg *tab_doc_reg;     /* 32 DOC Registers */
  word32 *ptr_doc_reg_e0;
  int *ptr_g_doc_num_osc_en;

  /** BATTERY RAM **/
  BYTE *battery_ram;        /* 256 Bytes */

  /* Compteur Unique */
  int counter;
};

struct cyrene_parameter *InitCyrene(void);                       /* Allocate Memory / Build IPC Window */
void FreeCyrene(void);                                           /* Close IPC Window / Free Memory     */
void my_Cyrene(int,void *,void *);                               /* Get / Set / Free cyrene_parameter  */

/*************************************************************************/
