/********************************************************************/
/*                                                                  */
/*   Cyrene_Breakpoint.c :  Module de gestion des Breakpoints.      */
/*                                                                  */
/********************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe  *  Date : May 2023  */
/********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/timeb.h>

#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>

#include "defc.h"
#include "sound.h"
#include "Cyrene.h"
#include "Cyrene_Shared.h"
#include "Cyrene_Operation.h"
#include "Cyrene_WindowIPC.h"
#include "Cyrene_SystemCall.h"
#include "Cyrene_Breakpoint.h"

static int CheckBreakpointOperationAddress(DWORD,BYTE,WORD,int,int,WORD,struct cyrene_breakpoint *);
static int CompareBreakpointValue(int,WORD,int,int,WORD);
static int CheckBreakpointAddress(DWORD,struct cyrene_breakpoint *);
static struct cyrene_breakpoint *DecodeOneBreakpointLine(char *,struct cyrene_breakpoint *);


/***************************************************************************************/
/*  CheckForBreakpointMatch() :  Check if this Checkpoint match the current operation. */
/***************************************************************************************/
int CheckForBreakpointMatch(struct cyrene_breakpoint *current_breakpoint, struct cyrene_operation *current_operation, unsigned char *operation_data)
{
  WORD reg_value, hold_value;
  int is_match, is_match_read, is_match_write, reg_value_size, hold_value_size, status_x, status_m;

  /* Init */
  is_match = 0;

  /** This Breakpoint has exceeded its execution times */
  if(current_breakpoint->repeat == BREAKPOINT_REPEAT_ONLY && current_breakpoint->nb_times >= current_breakpoint->repeat_only_times)
    return(0);

  /*** Check Source : Address / SystemCall / Operation / Register ***/
  if(current_breakpoint->source == BREAKPOINT_SOURCE_ADDRESS)
    {
      /** Read **/
      if(current_breakpoint->trigger == BREAKPOINT_TRIGGER_READ)
        {
          /* Read Operation */
          is_match_read = CheckBreakpointOperationAddress(current_operation->read_address, current_operation->read_value_size, current_operation->read_value, 
                                                          current_breakpoint->trigger_read_operator, current_breakpoint->trigger_read_value_size, current_breakpoint->trigger_read_value, 
                                                          current_breakpoint);
          if(is_match_read == 0)
            return(0);
        }
      /** Write **/
      else if(current_breakpoint->trigger == BREAKPOINT_TRIGGER_WRITE)
        {
          /* Write Operation */
          is_match_write = CheckBreakpointOperationAddress(current_operation->write_address, current_operation->write_value_size, current_operation->write_value, 
                                                           current_breakpoint->trigger_write_operator, current_breakpoint->trigger_write_value_size, current_breakpoint->trigger_write_value, 
                                                           current_breakpoint);
          if(is_match_write == 0)
            return(0);
        }
      /** ReadWrite **/
      else if(current_breakpoint->trigger == BREAKPOINT_TRIGGER_READWRITE)
        {
          /* Read Operation */
          is_match_read = CheckBreakpointOperationAddress(current_operation->read_address, current_operation->read_value_size, current_operation->read_value, 
                                                          current_breakpoint->trigger_read_operator, current_breakpoint->trigger_read_value_size, current_breakpoint->trigger_read_value, 
                                                          current_breakpoint);

          /* Write Operation */
          is_match_write = CheckBreakpointOperationAddress(current_operation->write_address, current_operation->write_value_size, current_operation->write_value, 
                                                           current_breakpoint->trigger_write_operator, current_breakpoint->trigger_write_value_size, current_breakpoint->trigger_write_value, 
                                                           current_breakpoint);

          if(is_match_read == 0 && is_match_write == 0)
            return(0);
        }
      /** Execute **/ 
      else if(current_breakpoint->trigger == BREAKPOINT_TRIGGER_EXECUTE)
        {
          /* Check Execute Address */
          is_match = CheckBreakpointAddress(current_operation->execute_address,current_breakpoint);
          if(is_match == 0)
            return(0);
        }

      /* OK */
      is_match = 1;
    }
  else if(current_breakpoint->source == BREAKPOINT_SOURCE_SYSTEMCALL)
    {
      /* Pas un appel  une sous fonction */
      if(current_operation->jump_address == 0xFFFFFFFF)
        return(0);

      /* Pas le bon Call Number */
      if(current_operation->call_number != current_breakpoint->source_systemcall_number)
        return(0);

      /* On vrifie le type d'Appel System */
      if(current_breakpoint->source_systemcall_type == TYPE_SYSTEM_CALL_P8)
        {
          /* [Prodos8] : JSR $00/BF00 */
          if((current_operation->opcode == 0x20 || current_operation->opcode == 0xFC) && current_operation->jump_address == 0x00BF00)
            is_match = 1;
          else
            return(0);
        }
      else if(current_breakpoint->source_systemcall_type == TYPE_SYSTEM_CALL_P16)
        {
          /* [Prodos16] : JSL $E1/00A8 / JSL $E1/00B0 */
          if(current_operation->opcode == 0x22 && (current_operation->jump_address == 0xE100A8 || current_operation->jump_address == 0xE100B0))
            is_match = 1;
          else
            return(0);
        }
      else if(current_breakpoint->source_systemcall_type == TYPE_SYSTEM_CALL_GSOS)
        {
          /* [GS/OS] : JSL $E1/00A8 / JSL $E1/00B0 */
          if(current_operation->opcode == 0x22 && (current_operation->jump_address == 0xE100A8 || current_operation->jump_address == 0xE100B0))
            is_match = 1;
          else
            return(0);
        }
      else if(current_breakpoint->source_systemcall_type == TYPE_SYSTEM_CALL_TOOLBOX)
        {
          /* [Toolbox] : JSL $E1/0000 */
          if(current_operation->opcode == 0x22 && current_operation->jump_address == 0xE10000)
            is_match = 1;
          else
            return(0);
        }
    }
  else if(current_breakpoint->source == BREAKPOINT_SOURCE_OPERATION)
    {
      /* On compare la longueur de l'opration (Opcode + Operand) */
      if(current_breakpoint->source_operation_data_nb != (1 + current_operation->operand_data_size))
        return(0);

      /* On compare le byte d'Opcode */
      if(current_breakpoint->source_operation_data[0] != current_operation->opcode)
        return(0);

      /** On compare les bytes d'Operand **/
      if(current_operation->operand_data_size >= 1 && current_breakpoint->source_operation_data[1] != current_operation->operand_1)
        return(0);
      if(current_operation->operand_data_size >= 2 && current_breakpoint->source_operation_data[2] != current_operation->operand_2)
        return(0);
      if(current_operation->operand_data_size >= 3 && current_breakpoint->source_operation_data[3] != current_operation->operand_3)
        return(0);

      /* OK */
      is_match = 1;
    }
  else if(current_breakpoint->source == BREAKPOINT_SOURCE_REGISTER)
    {
      /* Status Register */
      status_x = (current_operation->status_reg >> 4) & 0x01;     /* X */
      status_m = (current_operation->status_reg >> 5) & 0x01;     /* M */

      /** Register Value **/
      if(current_breakpoint->source_register == BREAKPOINT_REGISTER_A)
        {
          reg_value_size = (status_m == 0) ? 2 : 1;
          reg_value = current_operation->a_reg & ((reg_value_size == 1) ? 0x00FF : 0xFFFF);
        }
      else if(current_breakpoint->source_register == BREAKPOINT_REGISTER_X)
        {
          reg_value_size = (status_x == 0) ? 2 : 1;
          reg_value = current_operation->x_reg & ((reg_value_size == 1) ? 0x00FF : 0xFFFF);
        }
      else if(current_breakpoint->source_register == BREAKPOINT_REGISTER_Y)
        {
          reg_value_size = (status_x == 0) ? 2 : 1;
          reg_value = current_operation->y_reg & ((reg_value_size == 1) ? 0x00FF : 0xFFFF);
        }
      else if(current_breakpoint->source_register == BREAKPOINT_REGISTER_PC)
        {
          reg_value_size = 2;
          reg_value = current_operation->pc_reg;
        }
      else if(current_breakpoint->source_register == BREAKPOINT_REGISTER_STACK)
        {
          reg_value_size = 2;
          reg_value = current_operation->stack_reg;
        }
      else if(current_breakpoint->source_register == BREAKPOINT_REGISTER_DIRECT)
        {
          reg_value_size = 2;
          reg_value = current_operation->direct_reg;
        }      
      else if(current_breakpoint->source_register == BREAKPOINT_REGISTER_DBR)
        {
          reg_value_size = 1;
          reg_value = current_operation->db_reg;
        }
      else if(current_breakpoint->source_register == BREAKPOINT_REGISTER_PBR)
        {
          reg_value_size = 1;
          reg_value = current_operation->pb_reg;
        }
      else if(current_breakpoint->source_register == BREAKPOINT_REGISTER_STATUS)
        {
          reg_value_size = 1;
          reg_value = current_operation->status_reg;
        }
      else
        {
          /* Never Here */
          reg_value_size = 0;
          reg_value = 0x0000;
        }

      /** Hold Value **/
      hold_value = current_breakpoint->trigger_hold_value;
      /* Expand Value Size if register is 16 bit */
      hold_value_size = current_breakpoint->trigger_hold_value_size;    /* 0:None / 1:8bit / 2:16bit */
      if(hold_value_size == 1 && reg_value_size == 2)
        hold_value_size = 2;

      /** Compare **/
      is_match = CompareBreakpointValue(reg_value_size,reg_value,current_breakpoint->trigger_hold_operator,hold_value_size,hold_value);
    }

  /** Update Breakpoint Statistics **/
  if(is_match == 1)
    current_breakpoint->nb_times++;

  /** Don't raise Breakpoint too early **/
  if(is_match == 1 && current_breakpoint->repeat == BREAKPOINT_REPEAT_AFTER && current_breakpoint->nb_times < current_breakpoint->repeat_only_times)
    is_match = 0;

  /* Return status */
  return(is_match);
}


/**************************************************************************/
/*  CheckBreakpointOperationAddress() :  Vrifie l'opration Read/Write . */
/**************************************************************************/
static int CheckBreakpointOperationAddress(DWORD operation_address, BYTE operation_value_size, WORD operation_value, 
                                           int breakpoint_trigger_operator, int breakpoint_trigger_value_size, WORD breakpoint_trigger_value, 
                                           struct cyrene_breakpoint *current_breakpoint)
{
  int is_match;

  /* Not a Read/Write Operation ? */
  if(operation_address == 0xFFFFFFFF)
    return(0);

  /* Check Read/Write Value (in any) */
  if(breakpoint_trigger_value_size != 0)
    {
      /* Wrong Value Size */
      if(breakpoint_trigger_value_size != operation_value_size)
        return(0);

      /* Check Value with operator */
      is_match = CompareBreakpointValue(operation_value_size,operation_value,breakpoint_trigger_operator,breakpoint_trigger_value_size,breakpoint_trigger_value);
      if(is_match == 0)
        return(0);
    }

  /* Check Read/Write Address */
  is_match = CheckBreakpointAddress(operation_address,current_breakpoint);
  if(is_match == 0)
    return(0);

  /* Return status */
  return(is_match);
}


/****************************************************************************/
/*  CompareBreakpointValue() :  Compare la valeur avec celle du Breakpoint. */
/****************************************************************************/
static int CompareBreakpointValue(int value_size, WORD value, int bp_operator, int bp_value_size, WORD bp_value)
{
  /** Compare Size **/
  if(value_size != bp_value_size)
    return(0);

  /** Compare Values **/
  if(bp_operator == BREAKPOINT_OPERATOR_EQUAL)
    {
      if(value == bp_value)
        return(1);
    }
  else if(bp_operator == BREAKPOINT_OPERATOR_DIFFERENT)
    {
      if(value != bp_value)
        return(1);
    }
  else if(bp_operator == BREAKPOINT_OPERATOR_GREATER)
    {
      if(value > bp_value)
        return(1);
    }
  else if(bp_operator == BREAKPOINT_OPERATOR_GREATER_EQUAL)
    {
      if(value >= bp_value)
        return(1);
    }
  else if(bp_operator == BREAKPOINT_OPERATOR_LOWER)
    {
      if(value < bp_value)
        return(1);
    }
  else if(bp_operator == BREAKPOINT_OPERATOR_LOWER_EQUAL)
    {
      if(value <= bp_value)
        return(1);
    }

  /* No Match */
  return(0);
}


/******************************************************************************************************/
/*  CheckBreakpointAddress() :  On va vrifier que l'adresse de l'opration match avec le Breakpoint. */
/******************************************************************************************************/
static int CheckBreakpointAddress(DWORD operation_address, struct cyrene_breakpoint *current_breakpoint)
{
  int is_match;

  /** One Address : Source Address **/
  if(current_breakpoint->source_address_type_dst == BREAKPOINT_ADDRESS_EMPTY)
    {
      /* Compare Address with Mask */
      is_match = ((operation_address & current_breakpoint->source_address_mask) == current_breakpoint->source_address_masked) ? 1: 0;
      return(is_match);
    }
  else  /* Interval : [Source Address,Destination Address] */
    {
      /* Compare interval */
      if((operation_address & current_breakpoint->source_address_mask)     >= current_breakpoint->source_address_masked &&
         (operation_address & current_breakpoint->source_address_mask_dst) <= current_breakpoint->source_address_masked_dst)
        return(1);

      /* Outside */
      return(0);
    }
}


/**************************************************************************************/
/*  UpdateBreakpointList() :  Rcupre la liste des Breapoints dans la Shared Memory. */
/**************************************************************************************/
void UpdateBreakpointList(struct cyrene_parameter *cyrene_param)
{
  int status, nb_breakpoint;
  char *begin_data;
  char *end_line;
  struct cyrene_breakpoint *first_breakpoint;
  struct cyrene_breakpoint *last_breakpoint;
  struct cyrene_breakpoint *current_breakpoint;
  struct cyrene_breakpoint *next_breakpoint;

  /* Init */
  first_breakpoint = NULL;
  last_breakpoint = NULL;

  /* A t'on des choses  lire ? */
  memcpy(&status,&cyrene_param->shared_memory_ptr[IPC_SHARED_MEMORY_OFFSET_STATUS_BREAKPOINT],sizeof(int));
  if(status == 0)
    return;

  /*** On va rcuprer la liste des Breakpoints ***/
  /* Nombre de Breakpoint */
  memcpy(&nb_breakpoint,&cyrene_param->shared_memory_ptr[IPC_SHARED_MEMORY_OFFSET_NB_BREAKPOINT],sizeof(int));
  if(nb_breakpoint > 0)
    {
      /* On se positionne au dbut de la zone des Breakpoints */
      begin_data = (char *) &cyrene_param->shared_memory_ptr[IPC_SHARED_MEMORY_OFFSET_DATA_BREAKPOINT];
      while(begin_data)
        {
          /* Fin de la ligne */
          end_line = strchr(begin_data,'\n');
          if(end_line)
            *end_line = '\0';

          /** Rcupre un Breakpoint + Ajoute dans la liste Breakpoint **/
          current_breakpoint = DecodeOneBreakpointLine(begin_data,cyrene_param->first_breakpoint);
          if(current_breakpoint != NULL)
            {
              if(first_breakpoint == NULL)
                first_breakpoint = current_breakpoint;
              else
                last_breakpoint->next = current_breakpoint;
              last_breakpoint = current_breakpoint;
            }

          /* Ligne suivante */
          if(end_line == NULL)
            break;
          begin_data = end_line + 1;
        }
    }

  /** On libre la liste des Breakpoints dj en mmoire **/
  for(current_breakpoint = cyrene_param->first_breakpoint; current_breakpoint; current_breakpoint = next_breakpoint)
    {
      next_breakpoint = current_breakpoint->next;
      free(current_breakpoint);
    }
  cyrene_param->first_breakpoint = NULL;
  cyrene_param->last_breakpoint = NULL;

  /* Rattache les nouveaux */
  cyrene_param->first_breakpoint = first_breakpoint;
  cyrene_param->last_breakpoint = last_breakpoint;

  /** On a fini de tout lire **/
  status = 0;
  memcpy(&cyrene_param->shared_memory_ptr[IPC_SHARED_MEMORY_OFFSET_STATUS_BREAKPOINT],&status,sizeof(int));
}


/**************************************************************/
/*  DecodeOneBreakpointLine() :  Dcode une ligne Breakpoint. */
/**************************************************************/
static struct cyrene_breakpoint *DecodeOneBreakpointLine(char *breakpoint_line, struct cyrene_breakpoint *first_breakpoint)
{
  int i, j, nb_item, length, tab_index[16], is_error, found;
  DWORD call_number;
  char *data;
  char *next_sep;
  struct cyrene_breakpoint *current_breakpoint;
  struct cyrene_breakpoint *existing_breakpoint;
  char address_txt[16];
  char mask_txt[16];
  char value[BUFFER_SIZE];
  char *tab_register[] = {"A","X","Y","PC","STACK","DIRECT","DBR","PBR","STATUS",NULL};
  char *tab_operator[] = {"=","!=",">",">=","<","<=",NULL};

  /* Allocation mmoire */
  current_breakpoint = (struct cyrene_breakpoint *) calloc(1,sizeof(struct cyrene_breakpoint));
  if(current_breakpoint == NULL)
    return(NULL);
  current_breakpoint->status = 1;
  strcpy(current_breakpoint->breakpoint_line,breakpoint_line);

  /** ID;Source;Trigger;Action;Repeat **/
  nb_item = 0;
  tab_index[nb_item++] = 0;
  for(i=0; i<(int)strlen(breakpoint_line); i++)
    if(breakpoint_line[i] == ';')
      tab_index[nb_item++] = i;
  tab_index[nb_item++] = (int) strlen(breakpoint_line);

  /** Valide le format de la ligne **/
  if(nb_item != 6)
    {
      mem_free_breakpoint(current_breakpoint);
      return(NULL);
    }

  /** Rcupre les valeurs **/
  for(i=0; i<nb_item-1; i++)
    {
      /* Rcupre la valeur */
      length = tab_index[i+1]-(tab_index[i]+(i==0?0:1));
      memcpy(value,&breakpoint_line[tab_index[i]+(i==0?0:1)],length);
      value[length] = '\0';

      /*** Conserve la valeur ***/
      if(i == 0)
        {
          /** ID **/
          current_breakpoint->id = atoi(value);
        }
      else if(i == 1)
        {
          /** Source **/
          if(!strnicmp(value,"Address=",strlen("Address=")))
            {
              data = &value[strlen("Address=")];
              if(strlen(data) == strlen("00/0000") && data[2] == '/')
                {
                  /* Type */
                  current_breakpoint->source = BREAKPOINT_SOURCE_ADDRESS;

                  /** Adresse Src Txt **/
                  memcpy(&address_txt[0],&data[0],2);
                  memcpy(&address_txt[2],&data[3],4);
                  address_txt[6] = '\0';

                  /* Source Address Type */
                  current_breakpoint->source_address_type = (strchr(address_txt,'*') == NULL) ? BREAKPOINT_ADDRESS_NUM : BREAKPOINT_ADDRESS_TXT;

                  /* Address Src Mask */
                  for(j=0; j<6; j++)
                    {
                      mask_txt[j]    = (address_txt[j] == '*') ? '0' : 'F';
                      address_txt[j] = (address_txt[j] == '*') ? '0' : address_txt[j];
                    }
                  mask_txt[6] = '\0';

                  /* Address Src Num */
                  sscanf(mask_txt,"%06X",&current_breakpoint->source_address_mask);
                  sscanf(address_txt,"%06X",&current_breakpoint->source_address_masked);

                  /* Adresse Dst Type */
                  current_breakpoint->source_address_type_dst = BREAKPOINT_ADDRESS_EMPTY;
                }
              else if(strlen(data) == strlen("00/0000-00/0000") && data[2] == '/' && data[7] == '-' && data[10] == '/')
                {
                  /* Type */
                  current_breakpoint->source = BREAKPOINT_SOURCE_ADDRESS;

                  /** Adresse Src Txt **/
                  memcpy(&address_txt[0],&data[0],2);
                  memcpy(&address_txt[2],&data[3],4);
                  address_txt[6] = '\0';

                  /* Source Address Type */
                  current_breakpoint->source_address_type = (strchr(address_txt,'*') == NULL) ? BREAKPOINT_ADDRESS_NUM : BREAKPOINT_ADDRESS_TXT;

                  /* Address Src Mask */
                  for(j=0; j<6; j++)
                    {
                      mask_txt[j] = (address_txt[j] == '*') ? '0' : 'F';
                      address_txt[j] = (address_txt[j] == '*') ? '0' : address_txt[j];
                    }
                  mask_txt[6] = '\0';

                  /* Address Src Num */
                  sscanf(mask_txt,"%06X",&current_breakpoint->source_address_mask);
                  sscanf(address_txt,"%06X",&current_breakpoint->source_address_masked);

                  /** Adresse Dst Txt **/
                  memcpy(&address_txt[0],&data[8],2);
                  memcpy(&address_txt[2],&data[11],4);
                  address_txt[6] = '\0';

                  /* Destination Address Type */
                  current_breakpoint->source_address_type_dst = (strchr(address_txt,'*') == NULL) ? BREAKPOINT_ADDRESS_NUM : BREAKPOINT_ADDRESS_TXT;

                  /* Address Dst Mask */
                  for(j=0; j<6; j++)
                    {
                      mask_txt[j]    = (address_txt[j] == '*') ? '0' : 'F';
                      address_txt[j] = (address_txt[j] == '*') ? '0' : address_txt[j];
                    }
                  mask_txt[6] = '\0';

                  /* Address Dst Num */
                  sscanf(mask_txt,"%06X",&current_breakpoint->source_address_mask_dst);
                  sscanf(address_txt,"%06X",&current_breakpoint->source_address_masked_dst);
                }
              else
                {
                  mem_free_breakpoint(current_breakpoint);
                  return(NULL);
                }
            }
          else if(!strnicmp(value,"Symbol=",strlen("Symbol=")))
            {
              /* Le Symbol a t rsolu dans Cyrene, on est pass en Address */
            }
          else if(!strnicmp(value,"SystemCall=",strlen("SystemCall=")))
            {
              /* SystemCall */
              data = &value[strlen("SystemCall=")];
              if(strlen(data) == 0)
                {
                  mem_free_breakpoint(current_breakpoint);
                  return(NULL);
                }
              current_breakpoint->source = BREAKPOINT_SOURCE_SYSTEMCALL;
              if(!strnicmp(data,"P8-",3))
                {
                  current_breakpoint->source_systemcall_type = TYPE_SYSTEM_CALL_P8;
                  sscanf(&data[strlen("P8-")],"%X",&call_number);
                  current_breakpoint->source_systemcall_number = (BYTE) call_number;
                }
              else if(!strnicmp(data,"P16-",4))
                {
                  current_breakpoint->source_systemcall_type = TYPE_SYSTEM_CALL_P16;
                  sscanf(&data[strlen("P16-")],"%X",&call_number);
                  current_breakpoint->source_systemcall_number = (WORD) call_number;
                }
              else if(!strnicmp(data,"GSOS-",5))
                {
                  current_breakpoint->source_systemcall_type = TYPE_SYSTEM_CALL_GSOS;
                  sscanf(&data[strlen("GSOS-")],"%X",&call_number);
                  current_breakpoint->source_systemcall_number = (WORD) call_number;
                }
              else if(!strnicmp(data,"TOOL-",5))
                {
                  current_breakpoint->source_systemcall_type = TYPE_SYSTEM_CALL_TOOLBOX;
                  sscanf(&data[strlen("TOOL-")],"%X",&call_number);
                  current_breakpoint->source_systemcall_number = (WORD) call_number;
                }
              else
                {
                  mem_free_breakpoint(current_breakpoint);
                  return(NULL);
                }
            }
          else if(!strnicmp(value,"Operation=",strlen("Operation=")))
            {
              data = &value[strlen("Operation=")];
              if(strlen(data) != strlen("0011223344"))
                {
                  mem_free_breakpoint(current_breakpoint);
                  return(NULL);
                }
              for(j=0; j<(int)strlen(data); j++)
                if(IsHexaDecimalChar(data[j]) == 0)
                  {
                    mem_free_breakpoint(current_breakpoint);
                    return(NULL);
                  }
              current_breakpoint->source = BREAKPOINT_SOURCE_OPERATION;

              /* On rcupre directement les byte code de l'opration */
              current_breakpoint->source_operation_data_nb = EvaluateAsOneByte(&data[0],&is_error);
              current_breakpoint->source_operation_data[0] = EvaluateAsOneByte(&data[2],&is_error);   /* Opcode */
              current_breakpoint->source_operation_data[1] = EvaluateAsOneByte(&data[4],&is_error);   /* Operand Byte #1 */
              current_breakpoint->source_operation_data[2] = EvaluateAsOneByte(&data[6],&is_error);   /* Operand Byte #2 */
              current_breakpoint->source_operation_data[3] = EvaluateAsOneByte(&data[8],&is_error);   /* Operand Byte #3 */
            }
          else if(!strnicmp(value,"Register=",strlen("Register=")))
            {
              data = &value[strlen("Register=")];
              if(strlen(data) == 0)
                {
                  mem_free_breakpoint(current_breakpoint);
                  return(NULL);
                }
              current_breakpoint->source = BREAKPOINT_SOURCE_REGISTER;
              for(found=0,j=0; tab_register[j]!=NULL; j++)
                if(!stricmp(data,tab_register[j]))
                  {
                    current_breakpoint->source_register = j;
                    found = 1;
                    break;
                  }
              if(found == 0)
                {
                  mem_free_breakpoint(current_breakpoint);
                  return(NULL);
                }
            }
        }
      else if(i == 2)
        {
          /** Trigger **/
          /* ReadWrite */
          next_sep = my_stristr(value,"ReadWrite:");
          if(next_sep)
            {
              data = next_sep + strlen("ReadWrite:");
              /* Operator */
              next_sep = strchr(data,' ');
              if(next_sep == NULL)
                {
                  mem_free_breakpoint(current_breakpoint);
                  return(NULL);                  
                }
              *next_sep = '\0';
              for(found=0,j=0; tab_operator[j]!=NULL; j++)
                if(!stricmp(data,tab_operator[j]))
                  {
                    found = 1;
                    break;
                  }
              if(found == 0)
                {
                  mem_free_breakpoint(current_breakpoint);
                  return(NULL);                  
                }
              current_breakpoint->trigger_readwrite_operator = i;
              *next_sep = ' ';
              /* Value */
              data = next_sep+1;
              next_sep = strchr(data,',');
              if(next_sep)
                *next_sep = '\0';
              if(strlen(data) == 2)
                {
                  /* BYTE */
                  current_breakpoint->trigger_readwrite_value_size = 1;
                  current_breakpoint->trigger_readwrite_value = (WORD) EvaluateAsByte(data,&is_error);
                  if(is_error)
                    {
                      mem_free_breakpoint(current_breakpoint);
                      return(NULL);  
                    }
                }
              else if(strlen(data) == 4)
                {
                  /* WORD */
                  current_breakpoint->trigger_readwrite_value_size = 2;
                  current_breakpoint->trigger_readwrite_value = (WORD) EvaluateAsWord(data,&is_error);
                  if(is_error)
                    {
                      mem_free_breakpoint(current_breakpoint);
                      return(NULL);  
                    }
                }
              if(next_sep)
                *next_sep = ',';
              current_breakpoint->trigger = BREAKPOINT_TRIGGER_READWRITE;
            }
          else if(my_stristr(value,"ReadWrite"))
            {
              current_breakpoint->trigger = BREAKPOINT_TRIGGER_READWRITE;
              current_breakpoint->trigger_readwrite_value_size = 0;
            }

          /* Read */
          next_sep = my_stristr(value,"Read:");
          if(next_sep != NULL && current_breakpoint->trigger == 0)
            {
              data = next_sep + strlen("Read:");
              /* Operator */
              next_sep = strchr(data,' ');
              if(next_sep == NULL)
                {
                  mem_free_breakpoint(current_breakpoint);
                  return(NULL);                  
                }
              *next_sep = '\0';
              for(found=0,j=0; tab_operator[j]!=NULL; j++)
                if(!stricmp(data,tab_operator[j]))
                  {
                    found = 1;
                    break;
                  }
              if(found == 0)
                {
                  mem_free_breakpoint(current_breakpoint);
                  return(NULL);                  
                }
              current_breakpoint->trigger_read_operator = i;
              *next_sep = ' ';
              /* Value */
              data = next_sep+1;
              next_sep = strchr(data,',');
              if(next_sep)
                *next_sep = '\0';
              if(strlen(data) == 2)
                {
                  /* BYTE */
                  current_breakpoint->trigger_read_value_size = 1;
                  current_breakpoint->trigger_read_value = (WORD) EvaluateAsByte(data,&is_error);
                  if(is_error)
                    {
                      mem_free_breakpoint(current_breakpoint);
                      return(NULL);  
                    }
                }
              else if(strlen(data) == 4)
                {
                  /* WORD */
                  current_breakpoint->trigger_read_value_size = 2;
                  current_breakpoint->trigger_read_value = (WORD) EvaluateAsWord(data,&is_error);
                  if(is_error)
                    {
                      mem_free_breakpoint(current_breakpoint);
                      return(NULL);  
                    }
                }
              if(next_sep)
                *next_sep = ',';
              current_breakpoint->trigger = BREAKPOINT_TRIGGER_READ;
            }
          else if(my_stristr(value,"Read") != NULL && current_breakpoint->trigger == 0)
            {
              current_breakpoint->trigger = BREAKPOINT_TRIGGER_READ;
              current_breakpoint->trigger_read_value_size = 0;
            }

          /* Write */
          next_sep = my_stristr(value,"Write:");
          if(next_sep != NULL && current_breakpoint->trigger == 0)
            {
              data = next_sep + strlen("Write:");
              /* Operator */
              next_sep = strchr(data,' ');
              if(next_sep == NULL)
                {
                  mem_free_breakpoint(current_breakpoint);
                  return(NULL);                  
                }
              *next_sep = '\0';
              for(found=0,j=0; tab_operator[j]!=NULL; j++)
                if(!stricmp(data,tab_operator[j]))
                  {
                    found = 1;
                    break;
                  }
              if(found == 0)
                {
                  mem_free_breakpoint(current_breakpoint);
                  return(NULL);                  
                }
              current_breakpoint->trigger_write_operator = i;
              *next_sep = ' ';
              /* Value */
              data = next_sep+1;
              next_sep = strchr(data,',');
              if(next_sep)
                *next_sep = '\0';
              if(strlen(data) == 2)
                {
                  /* BYTE */
                  current_breakpoint->trigger_write_value_size = 1;
                  current_breakpoint->trigger_write_value = (WORD) EvaluateAsByte(data,&is_error);
                  if(is_error)
                    {
                      mem_free_breakpoint(current_breakpoint);
                      return(NULL);  
                    }
                }
              else if(strlen(data) == 4)
                {
                  /* WORD */
                  current_breakpoint->trigger_write_value_size = 2;
                  current_breakpoint->trigger_write_value = (WORD) EvaluateAsWord(data,&is_error);
                  if(is_error)
                    {
                      mem_free_breakpoint(current_breakpoint);
                      return(NULL);  
                    }
                }
              if(next_sep)
                *next_sep = ',';
              current_breakpoint->trigger = BREAKPOINT_TRIGGER_WRITE;
            }
          else if(my_stristr(value,"Write") != NULL  && current_breakpoint->trigger == 0)
            {
              current_breakpoint->trigger = BREAKPOINT_TRIGGER_WRITE;
              current_breakpoint->trigger_write_value_size = 0;
            }

          /* Execute */
          if(my_stristr(value,"Execute"))
            current_breakpoint->trigger = BREAKPOINT_TRIGGER_EXECUTE;

          /* Hold */
          next_sep = my_stristr(value,"Hold:");
          if(next_sep)
            {
              data = next_sep + strlen("Hold:");
              /* Operator */
              next_sep = strchr(data,' ');
              if(next_sep == NULL)
                {
                  mem_free_breakpoint(current_breakpoint);
                  return(NULL);                  
                }
              *next_sep = '\0';
              for(found=0,j=0; tab_operator[j]!=NULL; j++)
                if(!stricmp(data,tab_operator[j]))
                  {
                    found = 1;
                    break;
                  }
              if(found == 0)
                {
                  mem_free_breakpoint(current_breakpoint);
                  return(NULL);                  
                }
              current_breakpoint->trigger_hold_operator = i;
              *next_sep = ' ';
              /* Value */
              data = next_sep+1;
              next_sep = strchr(data,',');
              if(next_sep)
                *next_sep = '\0';
              if(strlen(data) == 2)
                {
                  /* BYTE */
                  current_breakpoint->trigger_hold_value_size = 1;
                  current_breakpoint->trigger_hold_value = (WORD) EvaluateAsByte(data,&is_error);
                  if(is_error)
                    {
                      mem_free_breakpoint(current_breakpoint);
                      return(NULL);  
                    }
                }
              else if(strlen(data) == 4)
                {
                  /* WORD */
                  current_breakpoint->trigger_hold_value_size = 2;
                  current_breakpoint->trigger_hold_value = (WORD) EvaluateAsWord(data,&is_error);
                  if(is_error)
                    {
                      mem_free_breakpoint(current_breakpoint);
                      return(NULL);  
                    }
                }
              if(next_sep)
                *next_sep = ',';
              current_breakpoint->trigger = BREAKPOINT_TRIGGER_HOLD;
            }
          else if(my_stristr(value,"Hold"))
            {
              current_breakpoint->trigger = BREAKPOINT_TRIGGER_HOLD;
              current_breakpoint->trigger_hold_value_size = 0;
            }

          /* Il faut au moins 1 coche */
          if(current_breakpoint->trigger == 0)
            {
              mem_free_breakpoint(current_breakpoint);
              return(NULL);  
            }
        }
      else if(i == 3)
        {
          /** Action **/
          if(!strnicmp(value,"Action:Stop",strlen("Action:Stop")))
            current_breakpoint->action = BREAKPOINT_ACTION_STOP;
          else if(!strnicmp(value,"Action:Print'",strlen("Action:Print'")))
            {
              data = &value[strlen("Action:Print'")];
              next_sep = strchr(data,'\'');
              if(next_sep == NULL)
                {
                  mem_free_breakpoint(current_breakpoint);
                  return(NULL);
                }
              *next_sep = '\0';
              current_breakpoint->action = BREAKPOINT_ACTION_PRINT;
              strcpy(current_breakpoint->action_print_text,data);
            }
          else if(!strnicmp(value,"Action:File'",strlen("Action:File'")))
            {
              data = &value[strlen("Action:File'")];
              next_sep = strchr(data,'\'');
              if(next_sep == NULL)
                {
                  mem_free_breakpoint(current_breakpoint);
                  return(NULL);
                }
              *next_sep = '\0';
              strcpy(current_breakpoint->action_print_text,data);
              data = next_sep+1;
              if(!strnicmp(data,",'",strlen(",'")))
                {
                  data += 2;
                  next_sep = strchr(data,'\'');
                  if(next_sep == NULL)
                    {
                      mem_free_breakpoint(current_breakpoint);
                      return(NULL);
                    }
                  *next_sep = '\0';
                  strcpy(current_breakpoint->action_file_path,data);
                }
              else
                {
                  mem_free_breakpoint(current_breakpoint);
                  return(NULL);
                }
              current_breakpoint->action = BREAKPOINT_ACTION_FILE;
            }
        }
      else if(i == 4)
        {
          /** Repeat **/
          if(!strnicmp(value,"Repeat:Always",strlen("Repeat:Always")))
            current_breakpoint->repeat = BREAKPOINT_REPEAT_ALWAYS;
          else if(!strnicmp(value,"Repeat:Only",strlen("Repeat:Only")))
            {
              data = &value[strlen("Repeat:Only")];
              current_breakpoint->repeat = BREAKPOINT_REPEAT_ONLY;
              current_breakpoint->repeat_only_times = (DWORD) atoi(data);
            }
          else if(!strnicmp(value,"Repeat:After",strlen("Repeat:After")))
            {
              data = &value[strlen("Repeat:After")];
              current_breakpoint->repeat = BREAKPOINT_REPEAT_AFTER;
              current_breakpoint->repeat_only_times = (DWORD) atoi(data);
            }
        }
    }

  /** Fusionne avec le Breakpoint dj en mmoire pour conserver les Stat **/
  for(existing_breakpoint=first_breakpoint; existing_breakpoint; existing_breakpoint=existing_breakpoint->next)
    if(existing_breakpoint->id == current_breakpoint->id)
      {
        current_breakpoint->nb_times = existing_breakpoint->nb_times;
        break;
      }

  /* Renvoi un Breakpoint */
  return(current_breakpoint);
}


/***********************************************************************************************/
/*  BuildBreakpointOutputLine() :  Construit la ligne du Breakpoint en remplaant les Keyword. */
/***********************************************************************************************/
void BuildBreakpointOutputLine(char *line_template, struct cyrene_operation *current_operation, struct cyrene_parameter *cyrene_param, char *line_rtn)
{
  BYTE value;
  DWORD address;
  char *next_sep;
  int i, j, k, length, status_x, status_m, is_address;
  char buffer_value[256];
  char buffer_keyword[BUFFER_SIZE];

  status_x = (current_operation->status_reg >> 4) & 0x01;     /* X */
  status_m = (current_operation->status_reg >> 5) & 0x01;     /* M */

  /** On va Rechercher / Remplacer les [Keyword] **/
  length = strlen(line_template);
  for(i=0,j=0; i<length; i++)
    {
      /* [Keyword] : A X Y PC PB DB STACK DIRECT STATUS GOID GCC @E12000 */
      if(line_template[i] == '[')
        {
          /* Isole le nom du Keyword */
          strcpy(buffer_keyword,&line_template[i+1]);
          next_sep = strchr(buffer_keyword,']');
          if(next_sep == NULL)
            break;
          *next_sep = '\0';

          /** Valeur du Keyword **/
          strcpy(buffer_value,"");
          if(!stricmp(buffer_keyword,"A"))
            {
              if(status_m == 1)
                sprintf(buffer_value,"%02X",current_operation->a_reg & 0x00FF);
              else
                sprintf(buffer_value,"%04X",current_operation->a_reg);
            }
          else if(!stricmp(buffer_keyword,"X"))
            {
              if(status_x == 1)
                sprintf(buffer_value,"%02X",current_operation->x_reg & 0x00FF);
              else
                sprintf(buffer_value,"%04X",current_operation->x_reg);
            }
          else if(!stricmp(buffer_keyword,"Y"))
            {
              if(status_x == 1)
                sprintf(buffer_value,"%02X",current_operation->y_reg & 0x00FF);
              else
                sprintf(buffer_value,"%04X",current_operation->y_reg);
            }
          else if(!stricmp(buffer_keyword,"PC"))
            sprintf(buffer_value,"%04X",current_operation->pc_reg);
          else if(!stricmp(buffer_keyword,"PB"))
            sprintf(buffer_value,"%02X",current_operation->pb_reg);
          else if(!stricmp(buffer_keyword,"DB"))
            sprintf(buffer_value,"%02X",current_operation->db_reg);
          else if(!stricmp(buffer_keyword,"STACK"))
            sprintf(buffer_value,"%04X",current_operation->stack_reg);
          else if(!stricmp(buffer_keyword,"DIRECT"))
            sprintf(buffer_value,"%04X",current_operation->direct_reg);
          else if(!stricmp(buffer_keyword,"STATUS"))
            sprintf(buffer_value,"%04X",current_operation->status_reg);
          else if(!stricmp(buffer_keyword,"GOID"))
            sprintf(buffer_value,"%08X:%08X",(DWORD)(cyrene_param->goid>>32),(DWORD)(cyrene_param->goid & 0x00000000FFFFFFFF));
          else if(!stricmp(buffer_keyword,"GCC"))
            sprintf(buffer_value,"%08X:%08X",(DWORD)(cyrene_param->gcc>>32),(DWORD)(cyrene_param->gcc & 0x00000000FFFFFFFF));
          else if(buffer_keyword[0] == '@' && strlen(buffer_keyword) == 7)
            {
              /* Value from Memory @ Location */
              is_address = 1;
              for(k=0; k<6; k++)
                if(IsHexaDecimalChar(buffer_keyword[1+k]) == 0)
                  is_address = 0;

              /* Rcupre la valeur */
              if(is_address == 1)
                {
                  sscanf(&buffer_keyword[1],"%06X",&address);
                  value = GetMemoryByte(address,current_operation,cyrene_param);
                  sprintf(buffer_value,"%04X",value);
                }
            }

          /* Copie la valeur */
          memcpy(&line_rtn[j],buffer_value,strlen(buffer_value));
          j += strlen(buffer_value);

          /* Saute le Keyword */
          i += strlen(buffer_keyword) + 1;
        }
      else 
        line_rtn[j++] = line_template[i];
    }

  /* Fin */
  line_rtn[j] = '\0';
}


/****************************************************************************/
/*  mem_free_breakpoint() :  Libration mmoire de la structure breakpoint. */
/****************************************************************************/
void mem_free_breakpoint(struct cyrene_breakpoint *current_breakpoint)
{
  if(current_breakpoint)
    {
      free(current_breakpoint);
    }
}

/*********************************************************************************/
