/*************************************************************************/
/*                                                                       */
/*   Cyrene_Operation.c :  Module de gestion des donnes des Oprations. */
/*                                                                       */
/*************************************************************************/
/*  Auteur : Olivier ZARDINI   *   Brutal Deluxe   *   Date : May 2023   */
/*************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/timeb.h>

#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>

#include "defc.h"
#include "sound.h"
#include "Cyrene.h"
#include "Cyrene_Shared.h"
#include "Cyrene_Breakpoint.h"
#include "Cyrene_SystemCall.h"
#include "Cyrene_WindowIPC.h"
#include "Cyrene_Operation.h"

/* 65c816 Address Mode */
#define AM_UNKOWN                                0  /* Not recognize */
#define AM_IMPLICIT                              1  /* A          Implicit                             */
#define AM_ABSOLUTE                              2  /* addr2      Absolute                             */
#define AM_ABSOLUTE_INDEXED_X_INDIRECT           3  /* (addr2,X)  Absolute Indexed,X Indirect Jmp/Jsr  */
#define AM_ABSOLUTE_INDEXED_X                    4  /* addr2,X    Absolute Indexed,X                   */
#define AM_ABSOLUTE_INDEXED_Y                    5  /* addr2,Y    Absolute Indexed,Y                   */
#define AM_ABSOLUTE_INDIRECT                     6  /* (addr2)    Absolute Indirect                    */
#define AM_ABSOLUTE_INDIRECT_LONG                7  /* [addr2]    Absolute Indirect Long               */
#define AM_ABSOLUTE_LONG                         8  /* addr3      Absolute Long                        */
#define AM_ABSOLUTE_LONG_INDEXED_X               9  /* addr3,X    Absolute Long Indexed,X              */
#define AM_DIRECT_PAGE                          10  /* dp         Direct Page                          */
#define AM_DIRECT_PAGE_INDEXED_X                11  /* dp,X       Direct Page Indexed,X                */
#define AM_DIRECT_PAGE_INDEXED_Y                12  /* dp,Y       Direct Page Indexed,Y                */
#define AM_DIRECT_PAGE_INDIRECT                 13  /* (dp)       Direct Page Indirect                 */
#define AM_DIRECT_PAGE_INDIRECT_LONG            14  /* [dp]       Direct Page Indirect Long            */
#define AM_DIRECT_PAGE_INDEXED_X_INDIRECT       15  /* (dp,X)     Direct Page Indexed,X Indirect       */
#define AM_DIRECT_PAGE_INDIRECT_INDEXED_Y       16  /* (dp),Y     Direct Page Indirect Indexed,Y       */
#define AM_DIRECT_PAGE_INDIRECT_LONG_INDEXED_Y  17  /* [dp],Y     Direct Page Indirect Long Indexed,Y  */
#define AM_IMMEDIATE_8                          18  /* #const 8   Immediate 8 bit                      */
#define AM_IMMEDIATE_16                         19  /* #const 16  Immediate 16 bit                      */
#define AM_PC_RELATIVE                          20  /* relative1  Program Counter Relative 8 bit (BRA, BEQ, BPL...) */
#define AM_PC_RELATIVE_LONG                     21  /* relative2  Program Counter Relative Long (BRL)  */
#define AM_STACK_RELATIVE_INDIRECT_INDEXED_Y    22  /* (sr,S),Y   Stack Relative Indirect Indexed,Y    */
#define AM_STACK_PC_RELATIVE_LONG               23  /* label      Stack PC Relative Long (PER) =  AM_PC_RELATIVE_LONG) */
#define AM_STACK_RELATIVE                       24  /* sr,S       Stack Relative                       */
#define AM_BLOCK_MOVE                           25  /* Src,Dst    Block Move (MVN, MVP)                */
#define AM_ABSOLUTE_JUMP                        26  /* addr2      Absolute Jmp/Jsr                     */

#define ADDRESS_DIRECT_STACK_MASK       0x0000FFFF  /* Remove Bank from Stack or Direct Page address (Force Bank 00) */

static void GetSoftswitchValues(struct cyrene_operation *,struct cyrene_parameter *);
static DWORD GetOperationAddress(int,struct cyrene_operation *,DWORD *);


/*************************************************************************************************************/
/*  GetOperationExtraInformation() :  Calcule le nombre de cycles + Informations Breakpoint d'une opration. */
/*************************************************************************************************************/
void GetOperationExtraInformation(struct cyrene_operation *current_operation, struct cyrene_parameter *cyrene_param)
{
  int nb_cycle, operand_data_size, has_jump, change_page, change_page_index, is_p16_call;
  BYTE byte_1, byte_2, read_value_size, write_value_size, read_value_wrap, write_value_wrap, smartport_command, trace_system_call;
  WORD read_value, write_value, call_number, address_param, address_wrap;
  int status_c, status_z, status_i, status_d, status_x, status_m, status_v, status_n, status_e, status_b;
  DWORD read_address, write_address, execute_address, jump_address, next_address, address_stack, address_dp, read_address_noindex, write_address_noindex, jump_address_noindex, smartport_address, return_address;
  struct cyrene_system_call *current_system_call;

  /* Init */
  operand_data_size = 0;
  read_address = 0xFFFFFFFF;
  read_value = 0xFFFF;
  read_value_size = 0;
  read_value_wrap = 0;
  write_address = 0xFFFFFFFF;
  write_value = 0xFFFF;
  write_value_size = 0;
  write_value_wrap = 0;
  execute_address = 0xFFFFFFFF;
  jump_address = 0xFFFFFFFF;
  call_number = 0xFFFF;

  /* Isole les Flags */
  status_c = (current_operation->status_reg >> 0) & 0x01;     /* C */
  status_z = (current_operation->status_reg >> 1) & 0x01;     /* Z */
  status_i = (current_operation->status_reg >> 2) & 0x01;     /* I */
  status_d = (current_operation->status_reg >> 3) & 0x01;     /* D */
  status_x = (current_operation->status_reg >> 4) & 0x01;     /* X */
  status_m = (current_operation->status_reg >> 5) & 0x01;     /* M */
  status_v = (current_operation->status_reg >> 6) & 0x01;     /* V */
  status_n = (current_operation->status_reg >> 7) & 0x01;     /* N */
  status_e = (current_operation->status_reg >> 8) & 0x01;     /* E */
  status_b = (current_operation->status_reg >> 9) & 0x01;     /* B */

  /** Get Softswitch values : $C019 + $C02E + $C02F + $C035 + $C068 **/
  GetSoftswitchValues(current_operation,cyrene_param);

  /** Adresses Operation + Pile + DP (Teste si la pile et la page direct sont en banc 01) **/
  execute_address = (current_operation->pb_reg << 16) | current_operation->pc_reg;
  address_stack   = (((current_operation->s_C068 & 0x30) == 0x30) ? 0x00010000 : 0x00000000) + current_operation->stack_reg;     /* Pointe sur la prochaine place libre */
  address_dp      = (((current_operation->s_C068 & 0x30) == 0x30) ? 0x00010000 : 0x00000000) + current_operation->direct_reg;

  /** Traite les Instructions **/
  switch(current_operation->opcode)
    {
      /** BRK data_8 (65816 65c02 6502) **/
      case 0x00 :
        /* Data Size */
        operand_data_size = 1;
        /* Cycle */
        nb_cycle = 7;
        nb_cycle += (status_e == 0) ? 1 : 0;                                         /* +1 si Natif */
        break;

      /** ORA (dp,x) (65816 65c02 6502) **/
      case 0x01 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDEXED_X_INDIRECT,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 6;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** COP data_8 (65816) **/
      case 0x02 :
        /* Data Size */
        operand_data_size = 1;
        /* Cycle */
        nb_cycle = 7;
        nb_cycle += (status_e == 0) ? 1 : 0;                                         /* +1 si Natif */
        break;

      /** ORA sr,s (65816) **/
      case 0x03 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_STACK_RELATIVE,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** TSB dp (65816 65c02) **/
      case 0x04 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE,current_operation,&read_address_noindex);
        write_address = read_address;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        write_value_size = read_value_size;
        write_value = (read_value | current_operation->a_reg) & (write_value_size == 1 ? 0x00FF : 0xFFFF);
        /* Cycle */
        nb_cycle = 5;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** ORA dp (65816 65c02 6502) **/
      case 0x05 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 3;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** ASL dp (65816 65c02 6502) **/
      case 0x06 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE,current_operation,&read_address_noindex);
        write_address = read_address;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        write_value_size = read_value_size;
        write_value = (read_value << 1) & (write_value_size == 1 ? 0x00FF : 0xFFFF);
        /* Cycle */
        nb_cycle = 5;
        nb_cycle += (status_m == 0) ? 2 : 0;                                         /* +2 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** ORAL [dp] (65816) **/
      case 0x07 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDIRECT_LONG,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 6;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** PHP (65816 65c02 6502) **/
      case 0x08 :
        /* Address */
        write_address = address_stack;
        /* Value */
        write_value_size = 1;
        write_value = current_operation->status_reg & 0x00FF;
        /* Cycle */
        nb_cycle = 3;
        break;

      /** ORA #const (65816 65c02 6502) **/
      case 0x09 :
        /* Data Size */
        operand_data_size = (status_m == 0) ? 2 : 1;
        /* Cycle */
        nb_cycle = 2;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** ASL (65816 65c02 6502) **/
      case 0x0A :
        /* Cycle */
        nb_cycle = 2;
        break;

      /** PHD (65816) **/
      case 0x0B :
        /* Address */
        write_address = address_stack;
        /* Value */
        write_value_size = 2;
        write_value = current_operation->direct_reg;
        /* Cycle */
        nb_cycle = 4;
        break;

      /** TSB addr2 (65816 65c02) **/
      case 0x0C :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE,current_operation,&read_address_noindex);
        write_address = read_address;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        write_value_size = read_value_size;
        write_value = (read_value | current_operation->a_reg) & (write_value_size == 1 ? 0x00FF : 0xFFFF);
        /* Cycle */
        nb_cycle = 6;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** ORA addr2 (65816 65c02 6502) **/
      case 0x0D :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** ASL addr2 (65816 65c02 6502) **/
      case 0x0E :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE,current_operation,&read_address_noindex);
        write_address = read_address;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        write_value_size = read_value_size;
        write_value = (read_value << 1) & (write_value_size == 1 ? 0x00FF : 0xFFFF);
        /* Cycle */
        nb_cycle = 6;
        nb_cycle += (status_m == 0) ? 2 : 0;                                         /* +2 si 16 bit */
        break;

      /** ORAL addr3 (65816) **/
      case 0x0F :
        /* Data Size */
        operand_data_size = 3;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE_LONG,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 5;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** BPL delta_8 (65816 65c02 6502) **/
      case 0x10 :
        /* Jump */
        has_jump = (status_n == 0) ? 1 : 0;
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        next_address = (current_operation->pb_reg << 16) + ((execute_address + 2) & 0x0000FFFF);
        jump_address = (has_jump == 0) ? next_address : GetOperationAddress(AM_PC_RELATIVE,current_operation,&jump_address_noindex);
        change_page = ((jump_address & 0x0000FF00) != (current_operation->pc_reg & 0x0000FF00)) ? 1 : 0;
        /* Cycle */
        nb_cycle = 2;
        nb_cycle += has_jump;                                                        /* +1 si Saut */
        nb_cycle += (status_e == 1 && has_jump == 1 && change_page == 1) ? 1 : 0;    /* +1 si le saut fait changer de page */
        break;

      /** ORA (dp),y (65816 65c02 6502) **/
      case 0x11 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDIRECT_INDEXED_Y,current_operation,&read_address_noindex);
        change_page_index = ((read_address_noindex & 0x0000FF00) != (read_address & 0x0000FF00)) ? 1 : 0;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 5;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        nb_cycle += (change_page_index == 1) ? 1 : 0;                                /* +1 si changement de Page avec l'Index  */
        break;

      /** ORA (dp) (65816 65c02) **/
      case 0x12 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDIRECT,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 5;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** ORA (sr,s),y (65816) **/
      case 0x13 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_STACK_RELATIVE_INDIRECT_INDEXED_Y,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 7;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** TRB dp (65816 65c02) **/
      case 0x14 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE,current_operation,&read_address_noindex);
        write_address = read_address;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        write_value_size = read_value_size;
        write_value = (read_value & ~current_operation->a_reg) & (write_value_size == 1 ? 0x00FF : 0xFFFF);
        /* Cycle */
        nb_cycle = 5;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** ORA dp,x (65816 65c02 6502) **/
      case 0x15 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDEXED_X,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** ASL dp,x (65816 65c02 6502) **/
      case 0x16 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDEXED_X,current_operation,&read_address_noindex);
        write_address = read_address;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        write_value_size = read_value_size;
        write_value = (read_value << 1) & (write_value_size == 1 ? 0x00FF : 0xFFFF);
        /* Cycle */
        nb_cycle = 6;
        nb_cycle += (status_m == 0) ? 2 : 0;                                         /* +2 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** ORAL [dp],y (65816) **/
      case 0x17 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDIRECT_LONG_INDEXED_Y,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 6;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** CLC (65816 65c02 6502) **/
      case 0x18 :
        /* Cycle */
        nb_cycle = 2;
        break;

      /** ORA addr2,y (65816 65c02 6502) **/
      case 0x19 :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE_INDEXED_Y,current_operation,&read_address_noindex);
        change_page_index = ((read_address_noindex & 0x0000FF00) != (read_address & 0x0000FF00)) ? 1 : 0;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += (change_page_index == 1) ? 1 : 0;                                /* +1 si changement de Page avec l'Index  */
        break;

      /** INC (65816 65c02) **/
      case 0x1A :
        /* Cycle */
        nb_cycle = 2;
        break;

      /** TCS (65816) **/
      case 0x1B :
        /* Cycle */
        nb_cycle = 2;
        break;

      /** TRB addr2 (65816 65c02) **/
      case 0x1C :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE,current_operation,&read_address_noindex);
        write_address = read_address;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        write_value_size = read_value_size;
        write_value = (read_value & ~current_operation->a_reg) & (write_value_size == 1 ? 0x00FF : 0xFFFF);
        /* Cycle */
        nb_cycle = 6;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** ORA addr2,X (65816 65c02 6502) **/
      case 0x1D :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE_INDEXED_X,current_operation,&read_address_noindex);
        change_page_index = ((read_address_noindex & 0x0000FF00) != (read_address & 0x0000FF00)) ? 1 : 0;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += (change_page_index == 1) ? 1 : 0;                                /* +1 si changement de Page avec l'Index  */
        break;

      /** ASL addr2,X (65816 65c02 6502) **/
      case 0x1E :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE_INDEXED_X,current_operation,&read_address_noindex);
        write_address = read_address;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        write_value_size = read_value_size;
        write_value = (read_value << 1) & (write_value_size == 1 ? 0x00FF : 0xFFFF);
        /* Cycle */
        nb_cycle = 7;
        nb_cycle += (status_m == 0) ? 2 : 0;                                         /* +2 si 16 bit */
        break;

      /** ORAL addr3,X (65816) **/
      case 0x1F :
        /* Data Size */
        operand_data_size = 3;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE_LONG_INDEXED_X,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 5;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** JSR addr2 (65816 65c02 6502) **/
      case 0x20 :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        write_address = address_stack;
        jump_address = GetOperationAddress(AM_ABSOLUTE_JUMP,current_operation,&jump_address_noindex);
        /* Value */
        write_value_size = 2;
        write_value = current_operation->pc_reg + 3;
        /* Cycle */
        nb_cycle = 6;
        /** Prodos 8 **/
        if(jump_address == 0x0000BF00)   /* Prodos8 : In-Line Call */
          {
            address_param = current_operation->pc_reg + 3;
            byte_1 = GetMemoryByte((current_operation->pb_reg<<16)|address_param,current_operation,cyrene_param);
            call_number = (WORD) byte_1;
          }
        break;

      /** AND (dp,x) (65816 65c02 6502) **/
      case 0x21 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDEXED_X_INDIRECT,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 6;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** JSL addr3 (65816) **/
      case 0x22 :
        /* Data Size */
        operand_data_size = 3;
        /* Address */
        write_address = address_stack;
        jump_address = GetOperationAddress(AM_ABSOLUTE_LONG,current_operation,&jump_address_noindex);
        /* Value */
        write_value_size = 2;
        write_value = current_operation->pc_reg + 4;
        /* Cycle */
        nb_cycle = 8;
        /** Toolbox / GS/OS / Prodos 16 **/
        if(jump_address == 0x00E10000)        /* Toolbox */
          call_number = current_operation->x_reg;
        else if(jump_address == 0x00E100A8)   /* GS/OS|Prodos16 : In-Line Call */
          {
            address_param = current_operation->pc_reg + 4;
            byte_1 = GetMemoryByte((current_operation->pb_reg<<16)|address_param,current_operation,cyrene_param);
            address_param = current_operation->pc_reg + 5;
            byte_2 = GetMemoryByte((current_operation->pb_reg<<16)|address_param,current_operation,cyrene_param);
            call_number = (byte_2 << 8) | byte_1;
          }
        else if(jump_address == 0x00E100B0)   /* GS/OS|Prodos16 : Call Stack */
          {
            address_param = current_operation->stack_reg + 1;
            byte_1 = GetMemoryByte((0x00<<16)|address_param,current_operation,cyrene_param);
            address_param = current_operation->stack_reg + 2;
            byte_2 = GetMemoryByte((0x00<<16)|address_param,current_operation,cyrene_param);
            call_number = (byte_2 << 8) | byte_1;
          }
        break;

      /** AND sr,s (65816) **/
      case 0x23 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_STACK_RELATIVE,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** BIT dp (65816 65c02 6502) **/
      case 0x24 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 3;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** AND dp (65816 65c02 6502) **/
      case 0x25 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 3;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** ROL dp (65816 65c02 6502) **/
      case 0x26 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE,current_operation,&read_address_noindex);
        write_address = read_address;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        write_value_size = read_value_size;
        write_value = ((read_value << 1) & (write_value_size == 1 ? 0x00FF : 0xFFFF)) | status_c;
        /* Cycle */
        nb_cycle = 5;
        nb_cycle += (status_m == 0) ? 2 : 0;                                         /* +2 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** ANDL [dp] (65816) **/
      case 0x27 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDIRECT_LONG,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 6;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** PLP (65816 65c02 6502) **/
      case 0x28 :
        /* Address */
        read_address = address_stack - 1;
        /* Value */
        read_value_size = 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        break;

      /** AND #const (65816 65c02 6502) **/
      case 0x29 :
        /* Data Size */
        operand_data_size = (status_m == 0) ? 2 : 1;
        /* Cycle */
        nb_cycle = 2;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** ROL (65816 65c02 6502) **/
      case 0x2A :
        /* Cycle */
        nb_cycle = 2;
        break;

      /** PLD (65816) **/
      case 0x2B :
        /* Address */
        read_address = address_stack - 2;
        /* Value */
        read_value_size = 2;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 5;
        break;

      /** BIT addr2 (65816 65c02 6502) **/
      case 0x2C :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** AND addr2 (65816 65c02 6502) **/
      case 0x2D :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** ROL addr2 (65816 65c02 6502) **/
      case 0x2E :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE,current_operation,&read_address_noindex);
        write_address = read_address;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        write_value_size = read_value_size;
        write_value = ((read_value << 1) & (write_value_size == 1 ? 0x00FF : 0xFFFF)) | status_c;
        /* Cycle */
        nb_cycle = 6;
        nb_cycle += (status_m == 0) ? 2 : 0;                                         /* +2 si 16 bit */
        break;

      /** ANDL addr3 (65816) **/
      case 0x2F :
        /* Data Size */
        operand_data_size = 3;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE_LONG,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 5;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** BMI delta_8 (65816 65c02 6502) **/
      case 0x30 :
        /* Jump */
        has_jump = (status_n == 1) ? 1 : 0;
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        next_address = (current_operation->pb_reg << 16) + ((execute_address + 2) & 0x0000FFFF);
        jump_address = (has_jump == 0) ? next_address : GetOperationAddress(AM_PC_RELATIVE,current_operation,&jump_address_noindex);
        change_page = ((jump_address & 0x0000FF00) != (current_operation->pc_reg & 0x0000FF00)) ? 1 : 0;
        /* Cycle */
        nb_cycle = 2;
        nb_cycle += has_jump;                                                        /* +1 si Saut */
        nb_cycle += (status_e == 1 && has_jump == 1 && change_page == 1) ? 1 : 0;    /* +1 si le saut fait changer de page */
        break;

      /** AND (dp),y (65816 65c02 6502) **/
      case 0x31 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDIRECT_INDEXED_Y,current_operation,&read_address_noindex);
        change_page_index = ((read_address_noindex & 0x0000FF00) != (read_address & 0x0000FF00)) ? 1 : 0;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 5;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        nb_cycle += (change_page_index == 1) ? 1 : 0;                                /* +1 si changement de Page avec l'Index  */
        break;

      /** AND (dp) (65816 65c02) **/
      case 0x32 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDIRECT,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 5;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** AND (sr,s),y (65816) **/
      case 0x33 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_STACK_RELATIVE_INDIRECT_INDEXED_Y,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 7;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** BIT dp,x (65816 65c02) **/
      case 0x34 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDEXED_X,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** AND dp,x (65816 65c02 6502) **/
      case 0x35 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDEXED_X,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** ROL dp,x (65816 65c02 6502) **/
      case 0x36 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDEXED_X,current_operation,&read_address_noindex);
        write_address = read_address;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        write_value_size = read_value_size;
        write_value = ((read_value << 1) & (write_value_size == 1 ? 0x00FF : 0xFFFF)) | status_c;
        /* Cycle */
        nb_cycle = 6;
        nb_cycle += (status_m == 0) ? 2 : 0;                                         /* +2 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** ANDL [dp],y (65816) **/
      case 0x37 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDIRECT_LONG_INDEXED_Y,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 6;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** SEC (65816 65c02 6502) **/
      case 0x38 :
        /* Cycle */
        nb_cycle = 2;
        break;

      /** AND addr2,y (65816 65c02 6502) **/
      case 0x39 :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE_INDEXED_Y,current_operation,&read_address_noindex);
        change_page_index = ((read_address_noindex & 0x0000FF00) != (read_address & 0x0000FF00)) ? 1 : 0;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += (change_page_index == 1) ? 1 : 0;                                /* +1 si changement de Page avec l'Index  */
        break;

      /** DEC (65816 65c02) **/
      case 0x3A :
        /* Cycle */
        nb_cycle = 2;
        break;

      /** TSC (65816) **/
      case 0x3B :
        /* Cycle */
        nb_cycle = 2;
        break;

      /** BIT addr2,X (65816 65c02) **/
      case 0x3C :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE_INDEXED_X,current_operation,&read_address_noindex);
        change_page_index = ((read_address_noindex & 0x0000FF00) != (read_address & 0x0000FF00)) ? 1 : 0;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += (change_page_index == 1) ? 1 : 0;                                /* +1 si changement de Page avec l'Index  */
        break;

      /** AND addr2,X (65816 65c02 6502) **/
      case 0x3D :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE_INDEXED_X,current_operation,&read_address_noindex);
        change_page_index = ((read_address_noindex & 0x0000FF00) != (read_address & 0x0000FF00)) ? 1 : 0;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += (change_page_index == 1) ? 1 : 0;                                /* +1 si changement de Page avec l'Index  */
        break;

      /** ROL addr2,X (65816 65c02 6502) **/
      case 0x3E :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE_INDEXED_X,current_operation,&read_address_noindex);
        write_address = read_address;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        write_value_size = read_value_size;
        write_value = ((read_value << 1) & (write_value_size == 1 ? 0x00FF : 0xFFFF)) | status_c;
        /* Cycle */
        nb_cycle = 7;
        nb_cycle += (status_m == 0) ? 2 : 0;                                         /* +2 si 16 bit */
        break;

      /** ANDL addr3,X (65816) **/
      case 0x3F :
        /* Data Size */
        operand_data_size = 3;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE_LONG_INDEXED_X,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 5;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** RTI (65816 65c02 6502) **/
      case 0x40 :
        /* Address / Value */
        read_address = address_stack +  2;
        jump_address = GetMemoryValue(read_address,3,current_operation,cyrene_param);
        read_value_size = 2;
        read_value = (WORD) (jump_address & 0x0000FFFF);
        /* Cycle */
        nb_cycle = 6;
        nb_cycle += (status_e == 0) ? 1 : 0;                                         /* +1 si Natif */
        break;

      /** EOR (dp,x) (65816 65c02 6502) **/
      case 0x41 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDEXED_X_INDIRECT,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 6;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** WDM const_8 (65816) **/
      case 0x42 :
        /* Data Size */
        operand_data_size = 1;
        /* Cycle */
        nb_cycle = 2;
        break;

      /** EOR sr,s (65816) **/
      case 0x43 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_STACK_RELATIVE,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** MVP Src,Dst (65816) **/
      case 0x44 :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        read_address = (current_operation->operand_2 << 16) + current_operation->x_reg;
        write_address = (current_operation->operand_1 << 16) + current_operation->y_reg;
        /* Values */
        read_value_size = 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        write_value_size = 1;
        write_value = read_value;
        /* Cycle */
        nb_cycle = 7;
        break;

      /** EOR dp (65816 65c02 6502) **/
      case 0x45 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 3;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** LSR dp (65816 65c02 6502) **/
      case 0x46 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE,current_operation,&read_address_noindex);
        write_address = read_address;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        write_value_size = read_value_size;
        write_value = (read_value >> 1) & (write_value_size == 1 ? 0x007F : 0x7FFF);
        /* Cycle */
        nb_cycle = 5;
        nb_cycle += (status_m == 0) ? 2 : 0;                                         /* +2 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** EORL [dp] (65816) **/
      case 0x47 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDIRECT_LONG,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 6;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** PHA (65816 65c02 6502) **/
      case 0x48 :
        /* Address */
        write_address = address_stack;
        /* Value */
        write_value_size = (status_m == 0) ? 2 : 1; 
        write_value = current_operation->a_reg & ((write_value_size == 1) ? 0x00FF : 0xFFFF);
        /* Cycle */
        nb_cycle = 3;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** EOR #const (65816 65c02 6502) **/
      case 0x49 :
        /* Data Size */
        operand_data_size = (status_m == 0) ? 2 : 1;
        /* Cycle */
        nb_cycle = 2;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** LSR (65816 65c02 6502) **/
      case 0x4A :
        /* Cycle */
        nb_cycle = 2;
        break;

      /** PHK (65816) **/
      case 0x4B :
        /* Address */
        write_address = address_stack;
        /* Value */
        write_value_size = 1;
        write_value = (WORD) current_operation->pb_reg;
        /* Cycle */
        nb_cycle = 3;
        break;

      /** JMP addr2 (65816 65c02 6502) **/
      case 0x4C :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        jump_address = GetOperationAddress(AM_ABSOLUTE_JUMP,current_operation,&jump_address_noindex);
        /* Cycle */
        nb_cycle = 3;
        break;

      /** EOR addr2 (65816 65c02 6502) **/
      case 0x4D :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** LSR addr2 (65816 65c02 6502) **/
      case 0x4E :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE,current_operation,&read_address_noindex);
        write_address = read_address;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        write_value_size = read_value_size;
        write_value = (read_value >> 1) & (write_value_size == 1 ? 0x007F : 0x7FFF);
        /* Cycle */
        nb_cycle = 6;
        nb_cycle += (status_m == 0) ? 2 : 0;                                         /* +2 si 16 bit */
        break;

      /** EORL addr3 (65816) **/
      case 0x4F :
        /* Data Size */
        operand_data_size = 3;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE_LONG,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 5;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** BVC delta_8 (65816 65c02 6502) **/
      case 0x50 :
        /* Jump */
        has_jump = (status_v == 0) ? 1 : 0;
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        next_address = (current_operation->pb_reg << 16) + ((execute_address + 2) & 0x0000FFFF);
        jump_address = (has_jump == 0) ? next_address : GetOperationAddress(AM_PC_RELATIVE,current_operation,&jump_address_noindex);
        change_page = ((jump_address & 0x0000FF00) != (current_operation->pc_reg & 0x0000FF00)) ? 1 : 0;
        /* Cycle */
        nb_cycle = 2;
        nb_cycle += has_jump;                                                        /* +1 si Saut */
        nb_cycle += (status_e == 1 && has_jump == 1 && change_page == 1) ? 1 : 0;    /* +1 si le saut fait changer de page */
        break;

      /** EOR (dp),y (65816 65c02 6502) **/
      case 0x51 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDIRECT_INDEXED_Y,current_operation,&read_address_noindex);
        change_page_index = ((read_address_noindex & 0x0000FF00) != (read_address & 0x0000FF00)) ? 1 : 0;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 5;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        nb_cycle += (change_page_index == 1) ? 1 : 0;                                /* +1 si changement de Page avec l'Index  */
        break;

      /** EOR (dp) (65816 65c02) **/
      case 0x52 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDIRECT,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 5;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** EOR (sr,s),y (65816) **/
      case 0x53 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_STACK_RELATIVE_INDIRECT_INDEXED_Y,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 7;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** MVN Src,Dst (65816) **/
      case 0x54 :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        read_address = (current_operation->operand_2<< 16) + current_operation->x_reg;
        write_address = (current_operation->operand_1 << 16) + current_operation->y_reg;
        /* Values */
        read_value_size = 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        write_value_size = 1;
        write_value = read_value;
        /* Cycle */
        nb_cycle = 7;
        break;

      /** EOR dp,x (65816 65c02 6502) **/
      case 0x55 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDEXED_X,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** LSR dp,x (65816 65c02 6502) **/
      case 0x56 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDEXED_X,current_operation,&read_address_noindex);
        write_address = read_address;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        write_value_size = read_value_size;
        write_value = (read_value >> 1) & (write_value_size == 1 ? 0x007F : 0x7FFF);
        /* Cycle */
        nb_cycle = 6;
        nb_cycle += (status_m == 0) ? 2 : 0;                                         /* +2 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** EORL [dp],y (65816) **/
      case 0x57 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDIRECT_LONG_INDEXED_Y,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 6;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** CLI (65816 65c02 6502) **/
      case 0x58 :
        /* Cycle */
        nb_cycle = 2;
        break;

      /** EOR addr2,y (65816 65c02 6502) **/
      case 0x59 :
        /* Data Size */
        operand_data_size = 2;
        /* Address */                 /* En 6502, on reste dans le banc */
        read_address = GetOperationAddress(AM_ABSOLUTE_INDEXED_Y,current_operation,&read_address_noindex);
        change_page_index = ((read_address_noindex & 0x0000FF00) != (read_address & 0x0000FF00)) ? 1 : 0;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += (change_page_index == 1) ? 1 : 0;                                /* +1 si changement de Page avec l'Index  */
        break;

      /** PHY (65816 65c02) **/
      case 0x5A :
        /* Address */
        write_address = address_stack;
        /* Value */
        write_value_size = (status_x == 0) ? 2 : 1;
        write_value = current_operation->y_reg & ((write_value_size == 1) ? 0x00FF : 0xFFFF);
        /* Cycle */
        nb_cycle = 3;
        nb_cycle += (status_x == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** TCD (65816) **/
      case 0x5B :
        /* Cycle */
        nb_cycle = 2;
        break;

      /** JMPL addr3 (65816) **/
      case 0x5C :
        /* Data Size */
        operand_data_size = 3;
        /* Address */
        jump_address = GetOperationAddress(AM_ABSOLUTE_LONG,current_operation,&jump_address_noindex);
        /* Cycle */
        nb_cycle = 4;
        break;

      /** EOR addr2,X (65816 65c02 6502) **/
      case 0x5D :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE_INDEXED_X,current_operation,&read_address_noindex);
        change_page_index = ((read_address_noindex & 0x0000FF00) != (read_address & 0x0000FF00)) ? 1 : 0;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += (change_page_index == 1) ? 1 : 0;                                /* +1 si changement de Page avec l'Index  */
        break;

      /** LSR addr2,X (65816 65c02 6502) **/
      case 0x5E :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE_INDEXED_X,current_operation,&read_address_noindex);
        write_address = read_address;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        write_value_size = read_value_size;
        write_value = (read_value >> 1) & (write_value_size == 1 ? 0x007F : 0x7FFF);
        /* Cycle */
        nb_cycle = 7;
        nb_cycle += (status_m == 0) ? 2 : 0;                                         /* +2 si 16 bit */
        break;

      /** EORL addr3,X (65816) **/
      case 0x5F :
        /* Data Size */
        operand_data_size = 3;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE_LONG_INDEXED_X,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 5;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** RTS (65816 65c02 6502) **/
      case 0x60 :
        /* Address / Value */
        read_address = address_stack + 1;
        read_value_size = 2;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        address_wrap = read_value + 1;
        jump_address = (current_operation->pb_reg << 16) | address_wrap;
        /* Cycle */
        nb_cycle = 6;
        break;

      /** ADC (dp,x) (65816 65c02 6502) **/
      case 0x61 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDEXED_X_INDIRECT,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 6;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        nb_cycle += (status_d == 1 && status_e == 1) ? 1 : 0;                        /* +1 si Decimal + Emulation */
        break;

      /** PER delta_16 => label (65816) **/
      case 0x62 :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        write_address = address_stack;
        /* Value */
        write_value_size = 2;
        write_value = (WORD) (GetOperationAddress(AM_STACK_PC_RELATIVE_LONG,current_operation,&read_address_noindex) & 0x0000FFFF);
        /* Cycle */
        nb_cycle = 6;
        break;

      /** ADC sr,s (65816) **/
      case 0x63 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_STACK_RELATIVE,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += (status_d == 1 && status_e == 1) ? 1 : 0;                        /* +1 si Decimal + Emulation */
        break;

      /** STZ dp (65816 65c02) **/
      case 0x64 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        write_address = GetOperationAddress(AM_DIRECT_PAGE,current_operation,&write_address_noindex);
        /* Value */
        write_value_size = (status_m == 0) ? 2 : 1;
        write_value = 0x0000;
        /* Cycle */
        nb_cycle = 3;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** ADC dp (65816 65c02 6502) **/
      case 0x65 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 3;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        nb_cycle += (status_d == 1 && status_e == 1) ? 1 : 0;                        /* +1 si Decimal + Emulation */
        break;

      /** ROR dp (65816 65c02 6502) **/
      case 0x66 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE,current_operation,&read_address_noindex);
        write_address = read_address;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        write_value_size = read_value_size;
        write_value = (read_value >> 1) & (write_value_size == 1 ? (0x007F | (status_c << 7)) : (0x7FFF | (status_c << 15)));
        /* Cycle */
        nb_cycle = 5;
        nb_cycle += (status_m == 0) ? 2 : 0;                                         /* +2 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** ADCL [dp] (65816) **/
      case 0x67 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDIRECT_LONG,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 6;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        nb_cycle += (status_d == 1 && status_e == 1) ? 1 : 0;                        /* +1 si Decimal + Emulation */
        break;

      /** PLA (65816 65c02 6502) **/
      case 0x68 :
        /* Address */
        read_address = address_stack - ((status_m == 0) ? 2 : 1);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** ADC #const (65816 65c02 6502) **/
      case 0x69 :
        /* Data Size */
        operand_data_size = (status_m == 0) ? 2 : 1;
        /* Cycle */
        nb_cycle = 2;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += (status_d == 1 && status_e == 1) ? 1 : 0;                        /* +1 si Decimal + Emulation */
        break;

      /** ROR (65816 65c02 6502) **/
      case 0x6A :
        /* Cycle */
        nb_cycle = 2;
        break;

      /** RTL (65816) **/
      case 0x6B :
        /* Address / Value */
        read_address = address_stack + 1;
        jump_address = GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,3,current_operation,cyrene_param);
        address_wrap = jump_address & 0xFFFF;
        address_wrap += 1;
        jump_address = (jump_address & 0xFFFF0000) | address_wrap;
        read_value_size = 2;
        read_value = (WORD) (jump_address & 0x0000FFFF);
        /* Cycle */
        nb_cycle = 6;
        break;

      /** JMP (addr2) (65816 65c02 6502) [BANK 00] **/
      case 0x6C :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        jump_address = GetOperationAddress(AM_ABSOLUTE_INDIRECT,current_operation,&jump_address_noindex);
        /* Cycle */
        nb_cycle = 5;
        break;

      /** ADC addr2 (65816 65c02 6502) **/
      case 0x6D :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += (status_d == 1 && status_e == 1) ? 1 : 0;                        /* +1 si Decimal + Emulation */
        break;

      /** ROR addr2 (65816 65c02 6502) **/
      case 0x6E :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE,current_operation,&read_address_noindex);
        write_address = read_address;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        write_value_size = read_value_size;
        write_value = (read_value >> 1) & (write_value_size == 1 ? (0x007F | (status_c << 7)) : (0x7FFF | (status_c << 15)));
        /* Cycle */
        nb_cycle = 6;
        nb_cycle += (status_m == 0) ? 2 : 0;                                         /* +2 si 16 bit */
        break;

      /** ADCL addr3 (65816) **/
      case 0x6F :
        /* Data Size */
        operand_data_size = 3;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE_LONG,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 5;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += (status_d == 1 && status_e == 1) ? 1 : 0;                        /* +1 si Decimal + Emulation */
        break;

      /** BVS delta_8 (65816 65c02 6502) **/
      case 0x70 :
        /* Jump */
        has_jump = (status_v == 1) ? 1 : 0;
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        next_address = (current_operation->pb_reg << 16) + ((execute_address + 2) & 0x0000FFFF);
        jump_address = (has_jump == 0) ? next_address : GetOperationAddress(AM_PC_RELATIVE,current_operation,&jump_address_noindex);
        change_page = ((jump_address & 0x0000FF00) != (current_operation->pc_reg & 0x0000FF00)) ? 1 : 0;
        /* Cycle */
        nb_cycle = 2;
        nb_cycle += has_jump;                                                        /* +1 si Saut */
        nb_cycle += (status_e == 1 && has_jump == 1 && change_page == 1) ? 1 : 0;    /* +1 si le saut fait changer de page */
        break;

      /** ADC (dp),Y (65816 65c02 6502) **/
      case 0x71 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDIRECT_INDEXED_Y,current_operation,&read_address_noindex);
        change_page_index = ((read_address_noindex & 0x0000FF00) != (read_address & 0x0000FF00)) ? 1 : 0;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 5;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        nb_cycle += (status_d == 1 && status_e == 1) ? 1 : 0;                        /* +1 si Decimal + Emulation */
        nb_cycle += (change_page_index == 1) ? 1 : 0;                                /* +1 si changement de Page avec l'Index  */
        break;

      /** ADC (dp) (65816 65c02) **/
      case 0x72 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDIRECT,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 5;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        nb_cycle += (status_d == 1 && status_e == 1) ? 1 : 0;                        /* +1 si Decimal + Emulation */
        break;

      /** ADC (sr,s),y (65816) **/
      case 0x73 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_STACK_RELATIVE_INDIRECT_INDEXED_Y,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 7;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += (status_d == 1 && status_e == 1) ? 1 : 0;                        /* +1 si Decimal + Emulation */
        break;

      /** STZ dp,x (65816 65c02) **/
      case 0x74 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        write_address = GetOperationAddress(AM_DIRECT_PAGE_INDEXED_X,current_operation,&write_address_noindex);
        /* Value */
        write_value_size = (status_m == 0) ? 2 : 1;
        write_value = 0x0000;
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** ADC dp,X (65816 65c02 6502) **/
      case 0x75 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDEXED_X,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        nb_cycle += (status_d == 1 && status_e == 1) ? 1 : 0;                        /* +1 si Decimal + Emulation */
        break;

      /** ROR dp,x (65816 65c02 6502) **/
      case 0x76 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDEXED_X,current_operation,&read_address_noindex);
        write_address = read_address;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        write_value_size = read_value_size;
        write_value = (read_value >> 1) & (write_value_size == 1 ? (0x007F | (status_c << 7)) : (0x7FFF | (status_c << 15)));
        /* Cycle */
        nb_cycle = 6;
        nb_cycle += (status_m == 0) ? 2 : 0;                                         /* +2 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** ADCL [dp],y (65816) **/
      case 0x77 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDIRECT_LONG_INDEXED_Y,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 6;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        nb_cycle += (status_d == 1 && status_e == 1) ? 1 : 0;                        /* +1 si Decimal + Emulation */
        break;

      /** SEI (65816 65c02 6502) **/
      case 0x78 :
        /* Cycle */
        nb_cycle = 2;
        break;

      /** ADC addr2,y (65816 65c02 6502) **/
      case 0x79 :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE_INDEXED_Y,current_operation,&read_address_noindex);
        change_page_index = ((read_address_noindex & 0x0000FF00) != (read_address & 0x0000FF00)) ? 1 : 0;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += (status_d == 1 && status_e == 1) ? 1 : 0;                        /* +1 si Decimal + Emulation */
        nb_cycle += (change_page_index == 1) ? 1 : 0;                                /* +1 si changement de Page avec l'Index  */
        break;

      /** PLY (65816 65c02) **/
      case 0x7A :
        /* Address */
        read_address = address_stack - ((status_x == 0) ? 2 : 1);
        /* Value */
        read_value_size = (status_x == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_x == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** TDC (65816) **/
      case 0x7B :
        /* Cycle */
        nb_cycle = 2;
        break;

      /** JMP (addr2,x) (65816 65c02) [PHK] **/
      case 0x7C :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        jump_address = GetOperationAddress(AM_ABSOLUTE_INDEXED_X_INDIRECT,current_operation,&jump_address_noindex); 
        /* Cycle */
        nb_cycle = 6;
        break;

      /** ADC addr2,X (65816 65c02 6502) **/
      case 0x7D :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE_INDEXED_X,current_operation,&read_address_noindex);
        change_page_index = ((read_address_noindex & 0x0000FF00) != (read_address & 0x0000FF00)) ? 1 : 0;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += (status_d == 1 && status_e == 1) ? 1 : 0;                        /* +1 si Decimal + Emulation */
        nb_cycle += (change_page_index == 1) ? 1 : 0;                                /* +1 si changement de Page avec l'Index  */
        break;

      /** ROR addr2,X (65816 65c02 6502) **/
      case 0x7E :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE_INDEXED_X,current_operation,&read_address_noindex);
        write_address = read_address;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        write_value_size = read_value_size;
        write_value = (read_value >> 1) & (write_value_size == 1 ? (0x007F | (status_c << 7)) : (0x7FFF | (status_c << 15)));
        /* Cycle */
        nb_cycle = 7;
        nb_cycle += (status_m == 0) ? 2 : 0;                                         /* +2 si 16 bit */
        break;

      /** ADCL addr3,X (65816) **/
      case 0x7F :
        /* Data Size */
        operand_data_size = 3;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE_LONG_INDEXED_X,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 5;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += (status_d == 1 && status_e == 1) ? 1 : 0;                        /* +1 si Decimal + Emulation */
        break;

      /** BRA delta_8 (65816 65c02) **/
      case 0x80 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        jump_address = GetOperationAddress(AM_PC_RELATIVE,current_operation,&jump_address_noindex);
        change_page = ((jump_address & 0x0000FF00) != (current_operation->pc_reg & 0x0000FF00)) ? 1 : 0;
        /* Cycle */
        nb_cycle = 3;
        nb_cycle += (status_e == 1 && change_page == 1) ? 1 : 0;                     /* +1 si le saut fait changer de page */
        break;

      /** STA (dp,x) (65816 65c02 6502) **/
      case 0x81 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        write_address = GetOperationAddress(AM_DIRECT_PAGE_INDEXED_X_INDIRECT,current_operation,&write_address_noindex);
        /* Value */
        write_value_size = (status_m == 0) ? 2 : 1;
        write_value = current_operation->a_reg & ((write_value_size == 1) ? 0x00FF : 0xFFFF);
        /* Cycle */
        nb_cycle = 6;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** BRL delta_16 (65816) **/
      case 0x82 :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        jump_address = GetOperationAddress(AM_PC_RELATIVE_LONG,current_operation,&jump_address_noindex);
        /* Cycle */
        nb_cycle = 4;
        break;

      /** STA sr,s (65816) **/
      case 0x83 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        write_address = GetOperationAddress(AM_STACK_RELATIVE,current_operation,&write_address_noindex);
        /* Value */
        write_value_size = (status_m == 0) ? 2 : 1;
        write_value = current_operation->a_reg & ((write_value_size == 1) ? 0x00FF : 0xFFFF);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** STY dp (65816 65c02 6502) **/
      case 0x84 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        write_address = GetOperationAddress(AM_DIRECT_PAGE,current_operation,&write_address_noindex);
        /* Value */
        write_value_size = (status_x == 0) ? 2 : 1;
        write_value = current_operation->y_reg & ((write_value_size == 1) ? 0x00FF : 0xFFFF);
        /* Cycle */
        nb_cycle = 3;
        nb_cycle += (status_x == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** STA dp (65816 65c02 6502) **/
      case 0x85 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        write_address = GetOperationAddress(AM_DIRECT_PAGE,current_operation,&write_address_noindex);
        /* Value */
        write_value_size = (status_m == 0) ? 2 : 1;
        write_value = current_operation->a_reg & ((write_value_size == 1) ? 0x00FF : 0xFFFF);
        /* Cycle */
        nb_cycle = 3;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** STX dp (65816 65c02 6502) **/
      case 0x86 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        write_address = GetOperationAddress(AM_DIRECT_PAGE,current_operation,&write_address_noindex);
        /* Value */
        write_value_size = (status_x == 0) ? 2 : 1;
        write_value = current_operation->x_reg & ((write_value_size == 1) ? 0x00FF : 0xFFFF);
        /* Cycle */
        nb_cycle = 3;
        nb_cycle += (status_x == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** STAL [dp] (65816) **/
      case 0x87 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        write_address = GetOperationAddress(AM_DIRECT_PAGE_INDIRECT_LONG,current_operation,&write_address_noindex);
        /* Value */
        write_value_size = (status_m == 0) ? 2 : 1;
        write_value = current_operation->a_reg & ((write_value_size == 1) ? 0x00FF : 0xFFFF);
        /* Cycle */
        nb_cycle = 6;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** DEY (65816 65c02 6502) **/
      case 0x88 :
        /* Cycle */
        nb_cycle = 2;
        break;

      /** BIT #const (65816 65c02) **/
      case 0x89 :
        /* Data Size */
        operand_data_size = (status_m == 0) ? 2 : 1;
        /* Cycle */
        nb_cycle = 2;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** TXA (65816 65c02 6502) **/
      case 0x8A :
        /* Cycle */
        nb_cycle = 2;
        break;

      /** PHB (65816) **/
      case 0x8B :
        /* Address */
        write_address = address_stack;
        /* Value */
        write_value_size = 1;
        write_value = (WORD) current_operation->db_reg;
        /* Cycle */
        nb_cycle = 3;
        break;

      /** STY addr2 (65816 65c02 6502) **/
      case 0x8C :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        write_address = GetOperationAddress(AM_ABSOLUTE,current_operation,&write_address_noindex);
        /* Value */
        write_value_size = (status_x == 0) ? 2 : 1;
        write_value = current_operation->y_reg & ((write_value_size == 1) ? 0x00FF : 0xFFFF);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_x == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** STA addr2 (65816 65c02 6502) **/
      case 0x8D :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        write_address = GetOperationAddress(AM_ABSOLUTE,current_operation,&write_address_noindex);
        /* Value */
        write_value_size = (status_m == 0) ? 2 : 1;
        write_value = current_operation->a_reg & ((write_value_size == 1) ? 0x00FF : 0xFFFF);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** STX addr2 (65816 65c02 6502) **/
      case 0x8E :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        write_address = GetOperationAddress(AM_ABSOLUTE,current_operation,&write_address_noindex);
        /* Value */
        write_value_size = (status_x == 0) ? 2 : 1;
        write_value = current_operation->x_reg & ((write_value_size == 1) ? 0x00FF : 0xFFFF);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_x == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** STAL addr3 (65816) **/
      case 0x8F :
        /* Data Size */
        operand_data_size = 3;
        /* Address */
        write_address = GetOperationAddress(AM_ABSOLUTE_LONG,current_operation,&write_address_noindex);
        /* Value */
        write_value_size = (status_m == 0) ? 2 : 1;
        write_value = current_operation->a_reg & ((write_value_size == 1) ? 0x00FF : 0xFFFF);
        /* Cycle */
        nb_cycle = 5;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** BCC delta_8 (65816 65c02 6502) **/
      case 0x90 :
        /* Jump */
        has_jump = (status_c == 0) ? 1 : 0;
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        next_address = (current_operation->pb_reg << 16) + ((execute_address + 2) & 0x0000FFFF);
        jump_address = (has_jump == 0) ? next_address : GetOperationAddress(AM_PC_RELATIVE,current_operation,&jump_address_noindex);
        change_page = ((jump_address & 0x0000FF00) != (current_operation->pc_reg & 0x0000FF00)) ? 1 : 0;
        /* Cycle */
        nb_cycle = 2;
        nb_cycle += has_jump;                                                        /* +1 si Saut */
        nb_cycle += (status_e == 1 && has_jump == 1 && change_page == 1) ? 1 : 0;    /* +1 si le saut fait changer de page */
        break;

      /** STA (dp),y (65816 65c02 6502) **/
      case 0x91 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        write_address = GetOperationAddress(AM_DIRECT_PAGE_INDIRECT_INDEXED_Y,current_operation,&write_address_noindex);
        /* Value */
        write_value_size = (status_m == 0) ? 2 : 1;
        write_value = current_operation->a_reg & ((write_value_size == 1) ? 0x00FF : 0xFFFF);
        /* Cycle */
        nb_cycle = 6;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** STA (dp) (65816 65c02) **/
      case 0x92 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        write_address = GetOperationAddress(AM_DIRECT_PAGE_INDIRECT,current_operation,&write_address_noindex);
        /* Value */
        write_value_size = (status_m == 0) ? 2 : 1;
        write_value = current_operation->a_reg & ((write_value_size == 1) ? 0x00FF : 0xFFFF);
        /* Cycle */
        nb_cycle = 5;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** STA (sr,s),y (65816) **/
      case 0x93 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        write_address = GetOperationAddress(AM_STACK_RELATIVE_INDIRECT_INDEXED_Y,current_operation,&write_address_noindex);
        /* Value */
        write_value_size = (status_m == 0) ? 2 : 1;
        write_value = current_operation->a_reg & ((write_value_size == 1) ? 0x00FF : 0xFFFF);
        /* Cycle */
        nb_cycle = 7;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** STY dp,x (65816 65c02 6502) **/
      case 0x94 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        write_address = GetOperationAddress(AM_DIRECT_PAGE_INDEXED_X,current_operation,&write_address_noindex);
        /* Value */
        write_value_size = (status_x == 0) ? 2 : 1;
        write_value = current_operation->y_reg & ((write_value_size == 1) ? 0x00FF : 0xFFFF);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_x == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** STA dp,x (65816 65c02 6502) **/
      case 0x95 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        write_address = GetOperationAddress(AM_DIRECT_PAGE_INDEXED_X,current_operation,&write_address_noindex);
        /* Value */
        write_value_size = (status_m == 0) ? 2 : 1;
        write_value = current_operation->a_reg & ((write_value_size == 1) ? 0x00FF : 0xFFFF);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** STX dp,y (65816 65c02 6502) **/
      case 0x96 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        write_address = GetOperationAddress(AM_DIRECT_PAGE_INDEXED_Y,current_operation,&write_address_noindex);
        /* Value */
        write_value_size = (status_x == 0) ? 2 : 1;
        write_value = current_operation->x_reg & ((write_value_size == 1) ? 0x00FF : 0xFFFF);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_x == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** STAL [dp],y (65816) **/
      case 0x97 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        write_address = GetOperationAddress(AM_DIRECT_PAGE_INDIRECT_LONG_INDEXED_Y,current_operation,&write_address_noindex);
        /* Value */
        write_value_size = (status_m == 0) ? 2 : 1;
        write_value = current_operation->a_reg & ((write_value_size == 1) ? 0x00FF : 0xFFFF);
        /* Cycle */
        nb_cycle = 6;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** TYA (65816 65c02 6502) **/
      case 0x98 :
        /* Cycle */
        nb_cycle = 2;
        break;

      /** STA addr2,Y (65816 65c02 6502) **/
      case 0x99 :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        write_address = GetOperationAddress(AM_ABSOLUTE_INDEXED_Y,current_operation,&write_address_noindex); 
        /* Value */
        write_value_size = (status_m == 0) ? 2 : 1;
        write_value = current_operation->a_reg & ((write_value_size == 1) ? 0x00FF : 0xFFFF);
        /* Cycle */
        nb_cycle = 5;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** TXS (65816 65c02 6502) **/
      case 0x9A :
        /* Cycle */
        nb_cycle = 2;
        break;

      /** TXY (65816) **/
      case 0x9B :
        /* Cycle */
        nb_cycle = 2;
        break;

      /** STZ addr2 (65816 65c02) **/
      case 0x9C :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        write_address = GetOperationAddress(AM_ABSOLUTE,current_operation,&write_address_noindex);
        /* Value */
        write_value_size = (status_m == 0) ? 2 : 1;
        write_value = 0x0000;
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** STA addr2,X (65816 65c02 6502) **/
      case 0x9D :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        write_address = GetOperationAddress(AM_ABSOLUTE_INDEXED_X,current_operation,&write_address_noindex);
        /* Value */
        write_value_size = (status_m == 0) ? 2 : 1;
        write_value = current_operation->a_reg & ((write_value_size == 1) ? 0x00FF : 0xFFFF);
        /* Cycle */
        nb_cycle = 5;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** STZ addr2,X (65816 65c02) **/
      case 0x9E :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        write_address = GetOperationAddress(AM_ABSOLUTE_INDEXED_X,current_operation,&write_address_noindex);
        /* Value */
        write_value_size = (status_m == 0) ? 2 : 1;
        write_value = 0x0000;
        /* Cycle */
        nb_cycle = 5;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** STAL addr3,X (65816) **/
      case 0x9F :
        /* Data Size */
        operand_data_size = 3;
        /* Address */
        write_address = GetOperationAddress(AM_ABSOLUTE_LONG_INDEXED_X,current_operation,&write_address_noindex);
        /* Value */
        write_value_size = (status_m == 0) ? 2 : 1;
        write_value = current_operation->a_reg & ((write_value_size == 1) ? 0x00FF : 0xFFFF);
        /* Cycle */
        nb_cycle = 5;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** LDY #const (65816 65c02 6502) **/
      case 0xA0 :
        /* Data Size */
        operand_data_size = (status_x == 0) ? 2 : 1;
        /* Cycle */
        nb_cycle = 2;
        nb_cycle += (status_x == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** LDA (dp,x) (65816 65c02 6502) **/
      case 0xA1 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDEXED_X_INDIRECT,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 6;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** LDX #const (65816 65c02 6502) **/
      case 0xA2 :
        /* Data Size */
        operand_data_size = (status_x == 0) ? 2 : 1;
        /* Cycle */
        nb_cycle = 2;
        nb_cycle += (status_x == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** LDA sr,s (65816) **/
      case 0xA3 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_STACK_RELATIVE,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** LDY dp (65816 65c02 6502) **/
      case 0xA4 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_x == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 3;
        nb_cycle += (status_x == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** LDA dp (65816 65c02 6502) **/
      case 0xA5 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 3;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** LDX dp (65816 65c02 6502) **/
      case 0xA6 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_x == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 3;
        nb_cycle += (status_x == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** LDAL [dp] (65816) **/
      case 0xA7 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDIRECT_LONG,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 6;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** TAY (65816 65c02 6502) **/
      case 0xA8 :
        /* Cycle */
        nb_cycle = 2;
        break;

      /** LDA #const (65816 65c02 6502) **/
      case 0xA9 :
        /* Data Size */
        operand_data_size = (status_m == 0) ? 2 : 1;
        /* Cycle */
        nb_cycle = 2;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** TAX (65816 65c02 6502) **/
      case 0xAA :
        /* Cycle */
        nb_cycle = 2;
        break;

      /** PLB (65816) **/
      case 0xAB :
        /* Address */
        read_address = address_stack - 1;
        /* Value */
        read_value_size = 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        break;

      /** LDY addr2 (65816 65c02 6502) **/
      case 0xAC :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_x == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_x == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** LDA addr2 (65816 65c02 6502) **/
      case 0xAD :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** LDX addr2 (65816 65c02 6502) **/
      case 0xAE :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_x == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_x == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** LDAL addr3 (65816) **/
      case 0xAF :
        /* Data Size */
        operand_data_size = 3;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE_LONG,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 5;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** BCS delta_8 (65816 65c02 6502) **/
      case 0xB0 :
        /* Jump */
        has_jump = (status_c == 1) ? 1 : 0;
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        next_address = (current_operation->pb_reg << 16) + ((execute_address + 2) & 0x0000FFFF);
        jump_address = (has_jump == 0) ? next_address : GetOperationAddress(AM_PC_RELATIVE,current_operation,&jump_address_noindex);
        change_page = ((jump_address & 0x0000FF00) != (current_operation->pc_reg & 0x0000FF00)) ? 1 : 0;
        /* Cycle */
        nb_cycle = 2;
        nb_cycle += has_jump;                                                        /* +1 si Saut */
        nb_cycle += (status_e == 1 && has_jump == 1 && change_page == 1) ? 1 : 0;    /* +1 si le saut fait changer de page */
        break;

      /** LDA (dp),y (65816 65c02 6502) **/
      case 0xB1 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDIRECT_INDEXED_Y,current_operation,&read_address_noindex);
        change_page_index = ((read_address_noindex & 0x0000FF00) != (read_address & 0x0000FF00)) ? 1 : 0;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 5;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        nb_cycle += (change_page_index == 1) ? 1 : 0;                                /* +1 si changement de Page avec l'Index  */
        break;

      /** LDA (dp) (65816 65c02) **/
      case 0xB2 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDIRECT,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 5;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** LDA (sr,s),y (65816) **/
      case 0xB3 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_STACK_RELATIVE_INDIRECT_INDEXED_Y,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 7;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** LDY dp,x (65816 65c02 6502) **/
      case 0xB4 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDEXED_X,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_x == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_x == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** LDA dp,x (65816 65c02 6502) **/
      case 0xB5 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDEXED_X,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** LDX (dp),y (65816 65c02 6502) **/
      case 0xB6 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDIRECT_INDEXED_Y,current_operation,&read_address_noindex);
        change_page_index = ((read_address_noindex & 0x0000FF00) != (read_address & 0x0000FF00)) ? 1 : 0;
        /* Value */
        read_value_size = (status_x == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 5;
        nb_cycle += (status_x == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        nb_cycle += (change_page_index == 1) ? 1 : 0;                                /* +1 si changement de Page avec l'Index  */
        break;

      /** LDAL [dp],y (65816) **/
      case 0xB7 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDIRECT_LONG_INDEXED_Y,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 6;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** CLV (65816 65c02 6502) **/
      case 0xB8 :
        /* Cycle */
        nb_cycle = 2;
        break;

      /** LDA addr2,y (65816 65c02 6502) **/
      case 0xB9 :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE_INDEXED_Y,current_operation,&read_address_noindex);
        change_page_index = ((read_address_noindex & 0x0000FF00) != (read_address & 0x0000FF00)) ? 1 : 0;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += (change_page_index == 1) ? 1 : 0;                                /* +1 si changement de Page avec l'Index  */
        break;

      /** TSX (65816 65c02 6502) **/
      case 0xBA :
        /* Cycle */
        nb_cycle = 2;
        break;

      /** TYX (65816) **/
      case 0xBB :
        /* Cycle */
        nb_cycle = 2;
        break;

      /** LDY addr2,X (65816 65c02 6502) **/
      case 0xBC :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE_INDEXED_X,current_operation,&read_address_noindex);
        change_page_index = ((read_address_noindex & 0x0000FF00) != (read_address & 0x0000FF00)) ? 1 : 0;
        /* Value */
        read_value_size = (status_x == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_x == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += (change_page_index == 1) ? 1 : 0;                                /* +1 si changement de Page avec l'Index  */
        break;

      /** LDA addr2,X (65816 65c02 6502) **/
      case 0xBD :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE_INDEXED_X,current_operation,&read_address_noindex);
        change_page_index = ((read_address_noindex & 0x0000FF00) != (read_address & 0x0000FF00)) ? 1 : 0;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += (change_page_index == 1) ? 1 : 0;                                /* +1 si changement de Page avec l'Index  */
        break;

      /** LDX addr2,y (65816 65c02 6502) **/
      case 0xBE :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE_INDEXED_Y,current_operation,&read_address_noindex);
        change_page_index = ((read_address_noindex & 0x0000FF00) != (read_address & 0x0000FF00)) ? 1 : 0;
        /* Value */
        read_value_size = (status_x == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_x == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += (change_page_index == 1) ? 1 : 0;                                /* +1 si changement de Page avec l'Index  */
        break;

      /** LDAL addr3,X (65816) **/
      case 0xBF :
        /* Data Size */
        operand_data_size = 3;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE_LONG_INDEXED_X,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 5;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** CPY #const (65816 65c02 6502) **/
      case 0xC0 :
        /* Data Size */
        operand_data_size = (status_x == 0) ? 2 : 1;
        /* Cycle */
        nb_cycle = 2;
        nb_cycle += (status_x == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** CMP (dp,x) (65816 65c02 6502) **/
      case 0xC1 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDEXED_X_INDIRECT,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 6;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** REP const_8 (65816) **/
      case 0xC2 :
        /* Data Size */
        operand_data_size = 1;
        /* Cycle */
        nb_cycle = 3;
        break;

      /** CMP sr,s (65816) **/
      case 0xC3 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_STACK_RELATIVE,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** CPY dp (65816 65c02 6502) **/
      case 0xC4 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_x == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 3;
        nb_cycle += (status_x == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** CMP dp (65816 65c02 6502) **/
      case 0xC5 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 3;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** DEC dp (65816 65c02 6502) **/
      case 0xC6 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE,current_operation,&read_address_noindex);
        write_address = read_address;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        write_value_size = read_value_size;
        write_value = (read_value - 1) & ((status_m == 0) ? 0xFFFF : 0x00FF);
        /* Cycle */
        nb_cycle = 5;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** CMPL [dp] (65816) **/
      case 0xC7 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDIRECT_LONG,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 6;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** INY (65816 65c02 6502) **/
      case 0xC8 :
        /* Cycle */
        nb_cycle = 2;
        break;

      /** CMP #const (65816 65c02 6502) **/
      case 0xC9 :
        /* Data Size */
        operand_data_size = (status_m == 0) ? 2 : 1;
        /* Cycle */
        nb_cycle = 2;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** DEX (65816 65c02 6502) **/
      case 0xCA :
        /* Cycle */
        nb_cycle = 2;
        break;

      /** WAI (65816) **/
      case 0xCB :
        /* Cycle */
        nb_cycle = 3;
        break;

      /** CPY addr2 (65816 65c02 6502) **/
      case 0xCC :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_x == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_x == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** CMP addr2 (65816 65c02 6502) **/
      case 0xCD :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** DEC addr2 (65816 65c02 6502) **/
      case 0xCE :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE,current_operation,&read_address_noindex);
        write_address = read_address;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        write_value_size = read_value_size;
        write_value = (read_value - 1) & ((status_m == 0) ? 0xFFFF : 0x00FF);
        /* Cycle */
        nb_cycle = 6;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** CMPL addr3 (65816) **/
      case 0xCF :
        /* Data Size */
        operand_data_size = 3;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE_LONG,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 5;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** BNE delta_8 (65816 65c02 6502) **/
      case 0xD0 :
        /* Jump */
        has_jump = (status_z == 0) ? 1 : 0;
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        next_address = (current_operation->pb_reg << 16) + ((execute_address + 2) & 0x0000FFFF);
        jump_address = (has_jump == 0) ? next_address : GetOperationAddress(AM_PC_RELATIVE,current_operation,&jump_address_noindex);
        change_page = ((jump_address & 0x0000FF00) != (current_operation->pc_reg & 0x0000FF00)) ? 1 : 0;
        /* Cycle */
        nb_cycle = 2;
        nb_cycle += has_jump;                                                        /* +1 si Saut */
        nb_cycle += (status_e == 1 && has_jump == 1 && change_page == 1) ? 1 : 0;    /* +1 si le saut fait changer de page */
        break;

      /** CMP (dp),y (65816 65c02 6502) **/
      case 0xD1 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDIRECT_INDEXED_Y,current_operation,&read_address_noindex);
        change_page_index = ((read_address_noindex & 0x0000FF00) != (read_address & 0x0000FF00)) ? 1 : 0;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 5;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        nb_cycle += (change_page_index == 1) ? 1 : 0;                                /* +1 si changement de Page avec l'Index  */
        break;

      /** CMP (dp) (65816 65c02) **/
      case 0xD2 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDIRECT,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 5;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** CMP (sr,s),y (65816) **/
      case 0xD3 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_STACK_RELATIVE_INDIRECT_INDEXED_Y,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 7;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** PEI (dp) (65816) **/
      case 0xD4 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDIRECT,current_operation,&read_address_noindex);
        write_address = address_stack;
        /* Value */
        read_value_size = 2;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        write_value_size = 2;
        write_value = read_value;
        /* Cycle */
        nb_cycle = 6;
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** CMP dp,x (65816 65c02 6502) **/
      case 0xD5 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDEXED_X,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** DEC dp,x (65816 65c02 6502) **/
      case 0xD6 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDEXED_X,current_operation,&read_address_noindex);
        write_address = read_address;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        write_value_size = read_value_size;
        write_value = (read_value - 1) & ((status_m == 0) ? 0xFFFF : 0x00FF);
        /* Cycle */
        nb_cycle = 6;
        nb_cycle += (status_m == 0) ? 2 : 0;                                         /* +2 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** CMPL [dp],y (65816) **/
      case 0xD7 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDIRECT_LONG_INDEXED_Y,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 6;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** CLD (65816 65c02 6502) **/
      case 0xD8 :
        /* Cycle */
        nb_cycle = 2;
        break;

      /** CMP addr2,y (65816 65c02 6502) **/
      case 0xD9 :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE_INDEXED_Y,current_operation,&read_address_noindex);
        change_page_index = ((read_address_noindex & 0x0000FF00) != (read_address & 0x0000FF00)) ? 1 : 0;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += (change_page_index == 1) ? 1 : 0;                                /* +1 si changement de Page avec l'Index  */
        break;

      /** PHX (65816 65c02) **/
      case 0xDA :
        /* Address */
        write_address = address_stack;
        /* Value */
        write_value_size = (status_x == 0) ? 2 : 1; 
        write_value = current_operation->x_reg & ((write_value_size == 1) ? 0x00FF : 0xFFFF);
        /* Cycle */
        nb_cycle = 3;
        nb_cycle += (status_x == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** STP (65816) **/
      case 0xDB :
        /* Cycle */
        nb_cycle = 3;
        break;

      /** JMPL [addr2] (65816) [BANK 00] **/
      case 0xDC :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        jump_address = GetOperationAddress(AM_ABSOLUTE_INDIRECT_LONG,current_operation,&jump_address_noindex);
        /* Cycle */
        nb_cycle = 6;
        break;

      /** CMP addr2,X (65816 65c02 6502) **/
      case 0xDD :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE_INDEXED_X,current_operation,&read_address_noindex);
        change_page_index = ((read_address_noindex & 0x0000FF00) != (read_address & 0x0000FF00)) ? 1 : 0;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += (change_page_index == 1) ? 1 : 0;                                /* +1 si changement de Page avec l'Index  */
        break;

      /** DEC addr2,X (65816 65c02 6502) **/
      case 0xDE :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE_INDEXED_X,current_operation,&read_address_noindex);
        write_address = read_address;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        write_value_size = read_value_size;
        write_value = (read_value - 1) & ((status_m == 0) ? 0xFFFF : 0x00FF);
        /* Cycle */
        nb_cycle = 7;
        nb_cycle += (status_m == 0) ? 2 : 0;                                         /* +2 si 16 bit */
        break;

      /** CMPL addr3,X (65816) **/
      case 0xDF :
        /* Data Size */
        operand_data_size = 3;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE_LONG_INDEXED_X,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 5;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** CPX #const (65816 65c02 6502) **/
      case 0xE0 :
        /* Data Size */
        operand_data_size = (status_x == 0) ? 2 : 1;
        /* Cycle */
        nb_cycle = 2;
        nb_cycle += (status_x == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** SBC (dp,x) (65816 65c02 6502) **/
      case 0xE1 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDEXED_X_INDIRECT,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 6;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        nb_cycle += (status_d == 1 && status_e == 1) ? 1 : 0;                        /* +1 si Decimal + Emulation */
        break;

      /** SEP const_8 (65816) **/
      case 0xE2 :
        /* Data Size */
        operand_data_size = 1;
        /* Cycle */
        nb_cycle = 3;
        break;

      /** SBC sr,s (65816) **/
      case 0xE3 :
        /* Data Size */
        operand_data_size = 1;
        /* Address (Reste dans le Banc) */
        read_address = GetOperationAddress(AM_STACK_RELATIVE,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += (status_d == 1 && status_e == 1) ? 1 : 0;                        /* +1 si Decimal + Emulation */
        break;

      /** CPX dp (65816 65c02 6502) **/
      case 0xE4 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_x == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 3;
        nb_cycle += (status_x == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** SBC dp (65816 65c02 6502) **/
      case 0xE5 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 3;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        nb_cycle += (status_d == 1 && status_e == 1) ? 1 : 0;                        /* +1 si Decimal + Emulation */
        break;

      /** INC dp (65816 65c02 6502) **/
      case 0xE6 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE,current_operation,&read_address_noindex);
        write_address = read_address;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        write_value_size = read_value_size;
        write_value = (read_value + 1) & ((status_m == 0) ? 0xFFFF : 0x00FF);
        /* Cycle */
        nb_cycle = 5;
        nb_cycle += (status_m == 0) ? 2 : 0;                                         /* +2 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** SBCL [dp] (65816) **/
      case 0xE7 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDIRECT_LONG,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 6;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        nb_cycle += (status_d == 1 && status_e == 1) ? 1 : 0;                        /* +1 si Decimal + Emulation */
        break;

      /** INX (65816 65c02 6502) **/
      case 0xE8 :
        /* Cycle */
        nb_cycle = 2;
        break;

      /** SBC #const (65816 65c02 6502) **/
      case 0xE9 :
        /* Data Size */
        operand_data_size = (status_m == 0) ? 2 : 1;
        /* Cycle */
        nb_cycle = 2;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += (status_d == 1 && status_e == 1) ? 1 : 0;                        /* +1 si Decimal + Emulation */
        break;

      /** NOP (65816 65c02 6502) **/
      case 0xEA :
        /* Cycle */
        nb_cycle = 2;
        break;

      /** XBA (65816) **/
      case 0xEB :
        /* Cycle */
        nb_cycle = 3;
        break;

      /** CPX addr2 (65816 65c02 6502) **/
      case 0xEC :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_x == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_x == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** SBC addr2 (65816 65c02 6502) **/
      case 0xED :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += (status_d == 1 && status_e == 1) ? 1 : 0;                        /* +1 si Decimal + Emulation */
        break;

      /** INC addr2 (65816 65c02 6502) **/
      case 0xEE :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE,current_operation,&read_address_noindex);
        write_address = read_address;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        write_value_size = read_value_size;
        write_value = (read_value + 1) & ((status_m == 0) ? 0xFFFF : 0x00FF);
        /* Cycle */
        nb_cycle = 6;
        nb_cycle += (status_m == 0) ? 2 : 0;    /* +2 si 16 bit */
        break;

      /** SBCL addr3 (65816) **/
      case 0xEF :
        /* Data Size */
        operand_data_size = 3;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE_LONG,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 5;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += (status_d == 1 && status_e == 1) ? 1 : 0;                        /* +1 si Decimal + Emulation */
        break;

      /** BEQ delta_8 (65816 65c02 6502) **/
      case 0xF0 :
        /* Jump */
        has_jump = (status_z == 1) ? 1 : 0;
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        next_address = (current_operation->pb_reg << 16) + ((execute_address + 2) & 0x0000FFFF);
        jump_address = (has_jump == 0) ? next_address : GetOperationAddress(AM_PC_RELATIVE,current_operation,&jump_address_noindex);
        change_page = ((jump_address & 0x0000FF00) != (current_operation->pc_reg & 0x0000FF00)) ? 1 : 0;
        /* Cycle */
        nb_cycle = 2;
        nb_cycle += has_jump;                                                        /* +1 si Saut */
        nb_cycle += (status_e == 1 && has_jump == 1 && change_page == 1) ? 1 : 0;    /* +1 si le saut fait changer de page */
        break;

      /** SBC (dp),y (65816 65c02 6502) **/
      case 0xF1 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDIRECT_INDEXED_Y,current_operation,&read_address_noindex);
        change_page_index = ((read_address_noindex & 0x0000FF00) != (read_address & 0x0000FF00)) ? 1 : 0;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 5;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        nb_cycle += (status_d == 1 && status_e == 1) ? 1 : 0;                        /* +1 si Decimal + Emulation */
        nb_cycle += (change_page_index == 1) ? 1 : 0;                                /* +1 si changement de Page avec l'Index  */
        break;

      /** SBC (dp) (65816 65c02) **/
      case 0xF2 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDIRECT,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 6;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        nb_cycle += (status_d == 1 && status_e == 1) ? 1 : 0;                        /* +1 si Decimal + Emulation */
        break;

      /** SBC (sr,s),y (65816) **/
      case 0xF3 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_STACK_RELATIVE_INDIRECT_INDEXED_Y,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 7;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += (status_d == 1 && status_e == 1) ? 1 : 0;                        /* +1 si Decimal + Emulation */
        break;

      /** PEA const_16 (65816) **/
      case 0xF4 :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        write_address = address_stack;
        /* Value */
        write_value_size = 2;
        write_value = (current_operation->operand_2 << 8) | current_operation->operand_1;
        /* Cycle */
        nb_cycle = 5;
        break;

      /** SBC dp,x (65816 65c02 6502) **/
      case 0xF5 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDEXED_X,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        nb_cycle += (status_d == 1 && status_e == 1) ? 1 : 0;                        /* +1 si Decimal + Emulation */
        break;

      /** INC dp,x (65816 65c02 6502) **/
      case 0xF6 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDEXED_X,current_operation,&read_address_noindex);
        write_address = read_address;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address & ADDRESS_DIRECT_STACK_MASK,read_value_size,current_operation,cyrene_param);
        write_value_size = read_value_size;
        write_value = (read_value + 1) & ((status_m == 0) ? 0xFFFF : 0x00FF);
        /* Cycle */
        nb_cycle = 6;
        nb_cycle += (status_m == 0) ? 2 : 0;                                         /* +2 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        break;

      /** SBCL [dp],y (65816) **/
      case 0xF7 :
        /* Data Size */
        operand_data_size = 1;
        /* Address */
        read_address = GetOperationAddress(AM_DIRECT_PAGE_INDIRECT_LONG_INDEXED_Y,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 6;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += ((current_operation->direct_reg & 0x00FF) == 0x0000) ? 0 : 1;    /* +1 si la Page Direct n'est pas aligne */
        nb_cycle += (status_d == 1 && status_e == 1) ? 1 : 0;                        /* +1 si Decimal + Emulation */
        break;

      /** SED (65816 65c02 6502) **/
      case 0xF8 :
        /* Cycle */
        nb_cycle = 2;
        break;

      /** SBC addr2,y (65816 65c02 6502) **/
      case 0xF9 :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE_INDEXED_Y,current_operation,&read_address_noindex);
        change_page_index = ((read_address_noindex & 0x0000FF00) != (read_address & 0x0000FF00)) ? 1 : 0;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += (status_d == 1 && status_e == 1) ? 1 : 0;                        /* +1 si Decimal + Emulation */
        nb_cycle += (change_page_index == 1) ? 1 : 0;                                /* +1 si changement de Page avec l'Index  */
        break;

      /** PLX (65816 65c02) **/
      case 0xFA :
        /* Address */
        read_address = address_stack - ((status_x == 0) ? 2 : 1);
        /* Value */
        read_value_size = (status_x == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_x == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        break;

      /** XCE (65816) **/
      case 0xFB :
        /* Cycle */
        nb_cycle = 2;
        break;

      /** JSR (addr2,x) (65816) **/
      case 0xFC :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        write_address = address_stack;
        jump_address = GetOperationAddress(AM_ABSOLUTE_INDEXED_X_INDIRECT,current_operation,&jump_address_noindex);
        /* Value */
        write_value_size = 2;
        write_value = current_operation->pc_reg + 3;
        /* Cycle */
        nb_cycle = 6;
        /** Prodos 8 **/
        if(jump_address == 0x0000BF00)   /* Prodos8 : In-Line Call */
          {
            address_param = current_operation->pc_reg + 3;
            byte_1 = GetMemoryByte((current_operation->pb_reg<<16)|address_param,current_operation,cyrene_param);
            call_number = (WORD) byte_1;
          }
        break;

      /** SBC addr2,X (65816 65c02 6502) **/
      case 0xFD :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE_INDEXED_X,current_operation,&read_address_noindex);
        change_page_index = ((read_address_noindex & 0x0000FF00) != (read_address & 0x0000FF00)) ? 1 : 0;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 4;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += (status_d == 1 && status_e == 1) ? 1 : 0;                        /* +1 si Decimal + Emulation */
        nb_cycle += (change_page_index == 1) ? 1 : 0;                                /* +1 si changement de Page avec l'Index  */
        break;

      /** INC addr2,X (65816 65c02 6502) **/
      case 0xFE :
        /* Data Size */
        operand_data_size = 2;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE_INDEXED_X,current_operation,&read_address_noindex);
        write_address = read_address;
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        write_value_size = read_value_size;
        write_value = (read_value + 1) & ((status_m == 0) ? 0xFFFF : 0x00FF);
        /* Cycle */
        nb_cycle = 7;
        nb_cycle += (status_m == 0) ? 2 : 0;                                         /* +2 si 16 bit */
        break;

      /** SBCL addr3,X (65816) **/
      case 0xFF :
        /* Data Size */
        operand_data_size = 3;
        /* Address */
        read_address = GetOperationAddress(AM_ABSOLUTE_LONG_INDEXED_X,current_operation,&read_address_noindex);
        /* Value */
        read_value_size = (status_m == 0) ? 2 : 1;
        read_value = (WORD) GetMemoryValue(read_address,read_value_size,current_operation,cyrene_param);
        /* Cycle */
        nb_cycle = 5;
        nb_cycle += (status_m == 0) ? 1 : 0;                                         /* +1 si 16 bit */
        nb_cycle += (status_d == 1 && status_e == 1) ? 1 : 0;                        /* +1 si Decimal + Emulation */
        break;
    }

  /** Smartport : Gestion de l'adresse de la prochaine opration **/
  if(execute_address == 0x0000C70D)
    {
      /* Adresse du bloc d'appel du SmartPort = Command [1 byte] + Param [2 ou 4] */
      smartport_address = cyrene_param->previous_operation_address + cyrene_param->previous_operation_size;
      smartport_command = GetMemoryByte(smartport_address,current_operation,cyrene_param);
      jump_address = smartport_address + 1 + ((smartport_command >= 0x40) ? 4 : 2);
    }

  /** Renvoi le nombre de cycles + Data Size + Adresses **/
  current_operation->nb_cycle          = nb_cycle;
  current_operation->operand_data_size = operand_data_size;
  current_operation->read_address      = read_address;
  current_operation->read_value        = read_value;
  current_operation->read_value_size   = read_value_size;
  current_operation->read_value_wrap   = read_value_wrap;
  current_operation->write_address     = write_address;
  current_operation->write_value       = write_value;
  current_operation->write_value_size  = write_value_size;
  current_operation->write_value_wrap  = write_value_wrap;
  current_operation->execute_address   = execute_address;
  current_operation->jump_address      = jump_address;
  current_operation->call_number       = call_number;

  /** Flags : IWWRRwr- **/
  current_operation->flags = 0x00;
  /* Bit 7 : Interrupt */
  if(cyrene_param->next_operation_address != 0xFFFFFFFF && execute_address != cyrene_param->next_operation_address)
    current_operation->flags |= OPERATION_FLAG_INTERRUPT;
  /* Bit 6-5 : WriteValue Size */
  current_operation->flags |= ((current_operation->write_value_size & 0x03) << 5);
  /* Bit 4-3 : ReadValue Size */
  current_operation->flags |= ((current_operation->read_value_size & 0x03) << 3);
  /* Bit 2 : WriteValue Wrap */
  current_operation->flags |= ((current_operation->write_value_wrap & 0x01) << 2);
  /* Bit 1 : ReadValue Wrap */
  current_operation->flags |= ((current_operation->read_value_wrap & 0x01) << 1);

  /** Adresse de la prochaine opration **/
  if(current_operation->jump_address != 0xFFFFFFFF)
    cyrene_param->next_operation_address = current_operation->jump_address;
  else
    {
      /* On doit grer les MVN / MVP spcifiquement */
      if((current_operation->opcode == 0x54 || current_operation->opcode == 0x44) && current_operation->a_reg != 0x0000)
        cyrene_param->next_operation_address = current_operation->execute_address;     /* Reste sur la ligne du MVN/MVP */
      else
        cyrene_param->next_operation_address = (current_operation->pb_reg << 16) | ((current_operation->pc_reg + 1 + current_operation->operand_data_size) & 0x0000FFFF);
    }

  /** Conserve l'Adresse + Taille de l'opration pour la prochaine **/
  cyrene_param->previous_operation_address = current_operation->execute_address;
  cyrene_param->previous_operation_size    = 1 + current_operation->operand_data_size;

  /** Trace des Appels Systmes **/
  trace_system_call = cyrene_param->shared_memory_ptr[IPC_SHARED_MEMORY_OFFSET_OUTPUT_SYSCALL];
  if(trace_system_call != 0x00)
    {
      /** Valeurs de retour d'un Appel Systme **/
      for(current_system_call = cyrene_param->last_system_call; current_system_call; current_system_call = current_system_call->previous)
        if(current_system_call->return_address == execute_address)
          {
            /** Dcode les valeurs de retour des Appels Systme **/
            ProcessTraceSystemCallReturnValues(current_system_call,current_operation->a_reg,status_c,current_operation,cyrene_param);

            /* Libration mmoire */
            RemoveSystemCallFromList(current_system_call,cyrene_param);
            break;
          }

      /* P16 Call ? */
      is_p16_call = IsProdos16Call(jump_address,call_number);

      /** TraceCall [GS/OS] : JSL $E1/00A8 / JSL $E1/00B0 **/
      if((trace_system_call & TRACE_SYSTEM_CALL_GSOS) == TRACE_SYSTEM_CALL_GSOS)
        if(current_operation->opcode == 0x22 && (jump_address == 0xE100A8 || jump_address == 0xE100B0) && is_p16_call == 0)
          {
            /* Adresse de retour de la fonction (dans le mme banc) */
            if(jump_address == 0xE100A8)
              return_address = (execute_address & 0xFFFF0000) | ((execute_address+10) & 0x0000FFFF);   /* InLine : JSL @@@ + DA + ADRL */
            else if(jump_address == 0xE100B0)
              return_address = (execute_address & 0xFFFF0000) | ((execute_address+4) & 0x0000FFFF);    /* Stack  : JSL @@@ */

            /** Dcode l'Appel Systme GS/OS **/
            ProcessTraceSystemCall(address_stack,execute_address,jump_address,return_address,call_number,current_operation,cyrene_param);
          }

      /** TraceCall [Prodos16] : JSL $E1/00A8 / JSL $E1/00B0 **/
      if((trace_system_call & TRACE_SYSTEM_CALL_P16) == TRACE_SYSTEM_CALL_P16)
        if(current_operation->opcode == 0x22 && (jump_address == 0xE100A8 || jump_address == 0xE100B0) && is_p16_call == 1)
          {
            /* Adresse de retour de la fonction (dans le mme banc) */
            if(jump_address == 0xE100A8)
              return_address = (execute_address & 0xFFFF0000) | ((execute_address+10) & 0x0000FFFF);   /* InLine : JSL @@@ + DA + ADRL */
            else if(jump_address == 0xE100B0)
              return_address = (execute_address & 0xFFFF0000) | ((execute_address+4) & 0x0000FFFF);    /* Stack  : JSL @@@ */

            /** Dcode l'Appel Systme P16 **/
            ProcessTraceSystemCall(address_stack,execute_address,jump_address,return_address,call_number,current_operation,cyrene_param);
          }

      /** TraceCall [Prodos8] : JSR $00/BF00 **/
      if((trace_system_call & TRACE_SYSTEM_CALL_P8) == TRACE_SYSTEM_CALL_P8)
        if((current_operation->opcode == 0x20 || current_operation->opcode == 0xFC) && jump_address == 0x00BF00)
          {
            /* Adresse de retour de la fonction (dans le mme banc) */
            return_address = (execute_address & 0xFFFF0000) | ((execute_address+6) & 0x0000FFFF);    /* InLine : JSR @@ + DFB + DA */

            /* Dcode l'Appel Systme P8 */
            ProcessTraceSystemCall(address_stack,execute_address,jump_address,return_address,call_number,current_operation,cyrene_param);
          }

      /** TraceCall [Toolbox] : JSL $E1/0000 **/
      if((trace_system_call & TRACE_SYSTEM_CALL_TOOLBOX) == TRACE_SYSTEM_CALL_TOOLBOX)
        if(current_operation->opcode == 0x22 && jump_address == 0xE10000)
          {
            /* Adresse de retour de la fonction (dans le mme banc) */
            return_address = (execute_address & 0xFFFF0000) | ((execute_address+4) & 0x0000FFFF);   /* Stack : JSL @@@ */

            /* Dcode l'Appel Systme Toolbox */
            ProcessTraceSystemCall(address_stack,execute_address,jump_address,return_address,call_number,current_operation,cyrene_param);
          }
    }
}


/**********************************************************************/
/*  GetSoftswitchValues() :  Rcupration des valeurs des Softswitch. */
/**********************************************************************/
static void GetSoftswitchValues(struct cyrene_operation *current_operation, struct cyrene_parameter *cyrene_param)
{
  double dusecs_since_last_vbl, dlines_since_vbl, dcyc_line_start;
  int lines_since_vbl, line_offset;
  DWORD mask;

  /** Get Softswitch values : $C019 + $C02E + $C02F + $C035 + $C068 **/
  current_operation->s_C035 = (BYTE) (*cyrene_param->g_c035_shadow_reg);
  current_operation->s_C068 = (BYTE) (*cyrene_param->g_c068_statereg);

  /** Build Video Softswitch (based on read_vid_counters() code) **/
  dusecs_since_last_vbl = (double)((current_operation->dfcyc >> 16) - (*cyrene_param->g_last_vbl_dfcyc >> 16));
  dlines_since_vbl = dusecs_since_last_vbl * (1.0 / 65.0);
  lines_since_vbl = (int) dlines_since_vbl;
  dcyc_line_start = (double)lines_since_vbl * 65.0;
  line_offset = ((int)(dusecs_since_last_vbl - dcyc_line_start)) & 0xff;
  lines_since_vbl = (lines_since_vbl << 8) + line_offset;
  /* $C019 : VBL */
  current_operation->s_C019 = (lines_since_vbl >= 0xc000) ? 0x80 : 0x00;
  /* $C02E + $C02F : Video Spot location */
  lines_since_vbl += 0x10000;
  if(lines_since_vbl >= 0x20000)
    lines_since_vbl = lines_since_vbl - 0x20000 + 0xfa00;
  current_operation->s_C02E = (lines_since_vbl >> 9) & 0xff;     /* Vertical count */
  mask = (lines_since_vbl >> 1) & 0x80;
  lines_since_vbl = (lines_since_vbl & 0xff);
  if(lines_since_vbl >= 0x01)
    lines_since_vbl = (lines_since_vbl + 0x3f) & 0x7f;
  current_operation->s_C02F = (BYTE) (mask | (lines_since_vbl & 0xff));   /* Horizontal count */
}


/***************************************************************/
/*  GetOperationAddress() :  Calcule d'adresse de l'opration. */
/***************************************************************/
static DWORD GetOperationAddress(int address_mode, struct cyrene_operation *current_operation, DWORD *address_noindex_rtn)
{
  WORD address_wrap, delta_16;
  DWORD address, address_stack, address_dp, address_indirect;
  int status_x, status_m, status_e;
  struct cyrene_parameter *cyrene_param;
  my_Cyrene(CYRENE_GET_PARAM,&cyrene_param,NULL);

  /* Isole les Flags */
  status_x = (current_operation->status_reg >> 4) & 0x01;     /* X */
  status_m = (current_operation->status_reg >> 5) & 0x01;     /* M */
  status_e = (current_operation->status_reg >> 8) & 0x01;     /* E */

  /** Adresses Pile + DP (Teste si la pile et la page direct sont en banc 01) **/
  address_stack = (((current_operation->s_C068 & 0x30) == 0x30) ? 0x00010000 : 0x00000000) + current_operation->stack_reg;     /* Pointe sur la prochaine place libre */
  address_dp    = (((current_operation->s_C068 & 0x30) == 0x30) ? 0x00010000 : 0x00000000) + current_operation->direct_reg;

  /*** Address Mode ***/
  switch(address_mode)
    {
      /* addr2      Absolute */
      case AM_ABSOLUTE :
        address = (current_operation->db_reg << 16) + (current_operation->operand_2 << 8) + current_operation->operand_1;
        *address_noindex_rtn = address;
        break;

      /* addr2      Absolute Jump/Jsr */
      case AM_ABSOLUTE_JUMP :
        address = (current_operation->pb_reg << 16) + (current_operation->operand_2 << 8) + current_operation->operand_1;
        *address_noindex_rtn = address;
        break;

      /* (addr2)    Absolute Indirect JMP [BANK 00] */
      case AM_ABSOLUTE_INDIRECT :
        address_indirect = (0x00 << 16) + (current_operation->operand_2 << 8) + current_operation->operand_1;
        address = (current_operation->pb_reg << 16) + GetMemoryValue(address_indirect,2,current_operation,cyrene_param);
        *address_noindex_rtn = address;
        break;

      /* [addr2]    Absolute Indirect Long JML [BANK 00] */
      case AM_ABSOLUTE_INDIRECT_LONG :
        address_indirect = (0x00 << 16) + (current_operation->operand_2 << 8) + current_operation->operand_1;
        address = GetMemoryValue(address_indirect,3,current_operation,cyrene_param);
        *address_noindex_rtn = address;
        break;

      /* (addr2,X)  Absolute Indexed,X Indirect Jmp/Jsr [PBR] */
      case AM_ABSOLUTE_INDEXED_X_INDIRECT :
        address_indirect = (current_operation->pb_reg << 16) + (current_operation->operand_2 << 8) + current_operation->operand_1;
        address_indirect += (status_x == 0) ? current_operation->x_reg : (current_operation->x_reg & 0x00FF);
        address = (current_operation->pb_reg << 16) + GetMemoryValue(address_indirect,2,current_operation,cyrene_param);
        *address_noindex_rtn = address;
        break;

      /* addr2,X    Absolute Indexed,X */
      case AM_ABSOLUTE_INDEXED_X :
        if(status_e == 1)
          {
            /* En 6502, on reste dans le banc */
            address_wrap = (current_operation->operand_2 << 8) + current_operation->operand_1;
            *address_noindex_rtn = (current_operation->db_reg << 16) + address_wrap;
            address_wrap += current_operation->x_reg & 0x00FF;       
            address = (current_operation->db_reg << 16) + address_wrap;
          }
        else
          {
            address = ((current_operation->db_reg << 16) + (current_operation->operand_2 << 8) + current_operation->operand_1);
            *address_noindex_rtn = address;
            address += (status_x == 0) ? current_operation->x_reg : (current_operation->x_reg & 0x00FF);
          }
        break;

      /* addr2,Y    Absolute Indexed,Y */
      case AM_ABSOLUTE_INDEXED_Y :
        if(status_e == 1)
          {
            /* En 6502, on reste dans le banc */
            address_wrap = (current_operation->operand_2 << 8) + current_operation->operand_1;
            *address_noindex_rtn = (current_operation->db_reg << 16) + address_wrap;
            address_wrap += current_operation->y_reg & 0x00FF;
            address = (current_operation->db_reg << 16) + address_wrap; 
          }
        else
          {
            address = (current_operation->db_reg << 16) + (current_operation->operand_2 << 8) + current_operation->operand_1;
            *address_noindex_rtn = address;
            address += (status_x == 0) ? current_operation->y_reg : (current_operation->y_reg & 0x00FF);
          }
        break;

      /* addr3      Absolute Long */
      case AM_ABSOLUTE_LONG :
        address = (current_operation->operand_3 << 16) + (current_operation->operand_2 << 8) + current_operation->operand_1;
        *address_noindex_rtn = address;
        break;

      /* addr3,X    Absolute Long Indexed,X */
      case AM_ABSOLUTE_LONG_INDEXED_X :
        address = (current_operation->operand_3 << 16) + (current_operation->operand_2 << 8) + current_operation->operand_1;
        *address_noindex_rtn = address;
        address += (status_x == 0) ? current_operation->x_reg : (current_operation->x_reg & 0x00FF);
        break;

      /* dp         Direct Page */
      case AM_DIRECT_PAGE :
        address_wrap = (WORD) (address_dp & 0x0000FFFF);
        address_wrap += current_operation->operand_1;
        address = (address_dp & 0xFFFF0000) | address_wrap;
        *address_noindex_rtn = address;
        break;

      /* dp,X       Direct Page Indexed,X */
      case AM_DIRECT_PAGE_INDEXED_X :
        if(status_e == 1)
          {
            /* En 6502, on reste dans le banc */
            address_wrap = (WORD) (address_dp & 0x0000FFFF);
            address_wrap += current_operation->operand_1;
            *address_noindex_rtn = (address_dp & 0xFFFF0000) + address_wrap;
            address_wrap += current_operation->x_reg & 0x00FF;
            address = (address_dp & 0xFFFF0000) + address_wrap;
          }
        else
          {
            address_wrap = (WORD) (address_dp & 0x0000FFFF);
            address_wrap += current_operation->operand_1;
            address = (address_dp & 0xFFFF0000) | address_wrap;
            *address_noindex_rtn = address;
            address += (status_x == 0) ? current_operation->x_reg : (current_operation->x_reg & 0x00FF);
          }
        break;

      /* dp,Y       Direct Page Indexed,Y */
      case AM_DIRECT_PAGE_INDEXED_Y :
        if(status_e == 1)
          {
            /* En 6502, on reste dans le banc */
            address_wrap = (WORD) (address_dp & 0x0000FFFF);
            address_wrap += current_operation->operand_1;
            *address_noindex_rtn = (address_dp & 0xFFFF0000) + address_wrap;
            address_wrap += current_operation->y_reg & 0x00FF;
            address = (address_dp & 0xFFFF0000) + address_wrap;
          }
        else
          {
            address_wrap = (WORD) (address_dp & 0x0000FFFF);
            address_wrap += current_operation->operand_1;
            address = (address_dp & 0xFFFF0000) | address_wrap;
            *address_noindex_rtn = address;
            address += (status_x == 0) ? current_operation->y_reg : (current_operation->y_reg & 0x00FF);
          }
        break;

      /* (dp)       Direct Page Indirect */
      case AM_DIRECT_PAGE_INDIRECT :
        address_indirect = address_dp + current_operation->operand_1;
        address = (current_operation->db_reg << 16) + GetMemoryValue(address_indirect,2,current_operation,cyrene_param);
        *address_noindex_rtn = address;
        break;

      /* [dp]       Direct Page Indirect Long */
      case AM_DIRECT_PAGE_INDIRECT_LONG :
        address_indirect = address_dp + current_operation->operand_1;
        address = GetMemoryValue(address_indirect,3,current_operation,cyrene_param);
        *address_noindex_rtn = address;
        break;

      /* (dp,X)     Direct Page Indexed,X Indirect */
      case AM_DIRECT_PAGE_INDEXED_X_INDIRECT :
        address_indirect = address_dp + current_operation->operand_1;
        address_indirect += (status_x == 0) ? current_operation->x_reg : (current_operation->x_reg & 0x00FF);
        address = (current_operation->db_reg << 16) + GetMemoryValue(address_indirect,2,current_operation,cyrene_param);
        *address_noindex_rtn = address;
        break;

      /* (dp),Y     Direct Page Indirect Indexed,Y */
      case AM_DIRECT_PAGE_INDIRECT_INDEXED_Y :
        address_indirect = address_dp + current_operation->operand_1;
        address = (current_operation->db_reg << 16) + GetMemoryValue(address_indirect,2,current_operation,cyrene_param);
        *address_noindex_rtn = address;
        address += (status_x == 0) ? current_operation->y_reg : (current_operation->y_reg & 0x00FF);
        break;

      /* [dp],Y     Direct Page Indirect Long Indexed,Y */
      case AM_DIRECT_PAGE_INDIRECT_LONG_INDEXED_Y :
        address_indirect = address_dp + current_operation->operand_1;
        address = GetMemoryValue(address_indirect,3,current_operation,cyrene_param);
        *address_noindex_rtn = address;
        address += (status_x == 0) ? current_operation->y_reg : (current_operation->y_reg & 0x00FF);
        break;

      /* relative1  Program Counter Relative 8 bit (BRA, BEQ, BPL...) */
      case AM_PC_RELATIVE :
        /* Reste dans le mme banc */
        address_wrap = (current_operation->pc_reg + 2) + (((current_operation->operand_1 & 0x80) == 0x00) ? current_operation->operand_1 : (-1)*(256-current_operation->operand_1));    
        address = (current_operation->pb_reg << 16) + address_wrap;
        *address_noindex_rtn = address;
        break;

      /* relative2  Program Counter Relative Long (BRL) */
      case AM_PC_RELATIVE_LONG :
        /* Reste dans le mme banc */
        delta_16 = (current_operation->operand_2 << 8) + current_operation->operand_1;
        address_wrap = (current_operation->pc_reg + 3) + (((delta_16 & 0x8000) == 0x0000) ? delta_16 : (-1)*(65536-delta_16));
        address = (current_operation->pb_reg << 16) + address_wrap;
        *address_noindex_rtn = address;
        break;

      /* (sr,S),Y   Stack Relative Indirect Indexed,Y */
      case AM_STACK_RELATIVE_INDIRECT_INDEXED_Y :
        /* Reste dans le Banc de la Pile */
        address_indirect = (address_stack & 0xFFFF0000) + current_operation->stack_reg + current_operation->operand_1;
        address = (address_stack & 0xFFFF0000) + GetMemoryValue(address_indirect,2,current_operation,cyrene_param);
        *address_noindex_rtn = address;
        /* L'ajout de Y permet de sortir du banc de la stack */
        address += (status_x == 0) ? current_operation->y_reg : (current_operation->y_reg & 0x00FF);
        break;

      /* label      Stack PC Relative Long (PER) =  AM_PC_RELATIVE_LONG) */
      case AM_STACK_PC_RELATIVE_LONG :
        /* Reste dans le mme banc */
        delta_16 = (current_operation->operand_2 << 8) + current_operation->operand_1;
        address_wrap = (current_operation->pc_reg + 3) + (((delta_16 & 0x8000) == 0x0000) ? delta_16 : (-1)*(65536-delta_16));
        address = (current_operation->pb_reg << 16) + address_wrap;
        *address_noindex_rtn = address;
        break;

      /* sr,S       Stack Relative */
      case AM_STACK_RELATIVE :
        /* Reste dans le Banc de la Pile */
        address_wrap = current_operation->stack_reg + current_operation->operand_1;
        address = (address_stack & 0xFFFF0000) + address_wrap;
        *address_noindex_rtn = address;
        break;
    }

  /* Renvoi l'adresse */
  return(address);
}

/**************************************************************************/
