/********************************************************************/
/*                                                                  */
/*   Cyrene_WindowIPC.c :  Module de gestion de la fentre IPC.     */
/*                                                                  */
/********************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe  *  Date : May 2023  */
/********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/timeb.h>

#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>

#include "defc.h"
#include "sound.h"
#include "Cyrene.h"
#include "Cyrene_Shared.h"
#include "Cyrene_Operation.h"
#include "Cyrene_Breakpoint.h"
#include "Cyrene_SystemCall.h"
#include "Cyrene_WindowIPC.h"

/* Kegs Executable IPC Window */
#define KEGS_IPC_WINDOW_CLASS           "KegsIPC"
#define KEGS_IPC_WINDOW_NAME            "Kegs_%d"

/* IPC Window Minimum Size */
#define IPC_WINDOW_MIN_WIDTH                  240
#define IPC_WINDOW_MIN_HEIGHT                 120

/* Close Connection if errors */
#define MAX_SEND_DATA_ERROR_COUNT             255   /* Maximum errors allowed */

/* Status for Cyrene Answer */
#define CYRENE_ANSWER_WAIT                      1   /* Wait for Cyrene Answer */
#define CYRENE_ANSWER_DONE                      2   /* Cyrene Answer ready */

/* Operation Stop */
#define STOP_CONDITION_GOID                       1   /* Next XXX operations */     
#define STOP_CONDITION_JMP                        2   /* Next JMP BRA BEQ BPL... */
#define STOP_CONDITION_JSR                        3   /* Next JSR JSL */
#define STOP_CONDITION_RTS                        4   /* Next RTS RTL RTI */
#define STOP_CONDITION_ADDRESS                    5   /* Next Specified Address */
#define STOP_CONDITION_VBL                        6   /* Next VBL */
#define STOP_CONDITION_INTERRUPT                  7   /* Next Interrupt */
#define STOP_CONDITION_TOOLBOX                    8   /* Next Toolbox */
#define STOP_CONDITION_OS                         9   /* Next P8 / P16 / GS/OS */
#define STOP_CONDITION_BREAKPOINT                10   /* Next Breakpoint */

/* Operation Data Buffer */
#define OPERATION_DATA_OFFSET_A_W                 0   /* Offset of A Bytes in the Operation Data Buffer */
#define OPERATION_DATA_OFFSET_X_W                 2   /* Offset of X Bytes in the Operation Data Buffer */
#define OPERATION_DATA_OFFSET_Y_W                 4   /* Offset of Y Bytes in the Operation Data Buffer */
#define OPERATION_DATA_OFFSET_PC_W                6   /* Offset of PC Bytes in the Operation Data Buffer */
#define OPERATION_DATA_OFFSET_PBR_B               8   /* Offset of PBR Byte in the Operation Data Buffer */
#define OPERATION_DATA_OFFSET_DBR_B               9   /* Offset of DBR Byte in the Operation Data Buffer */
#define OPERATION_DATA_OFFSET_STACK_W            10   /* Offset of STACK Bytes in the Operation Data Buffer */
#define OPERATION_DATA_OFFSET_DIRECT_W           12   /* Offset of DIRECT Bytes in the Operation Data Buffer */
#define OPERATION_DATA_OFFSET_STATUS_W           14   /* Offset of STATUS Bytes in the Operation Data Buffer */
#define OPERATION_DATA_OFFSET_OPCODE_B           16   /* Offset of Opcode Byte in the Operation Data buffer */
#define OPERATION_DATA_OFFSET_OPERAND1_B         17   /* Offset of Operand #1 Byte in the Operation Data buffer */
#define OPERATION_DATA_OFFSET_OPERAND2_B         18   /* Offset of Operand #2 Byte in the Operation Data buffer */
#define OPERATION_DATA_OFFSET_OPERAND3_B         19   /* Offset of Operand #3 Byte in the Operation Data buffer */
#define OPERATION_DATA_OFFSET_DATASIZE_CYCLE_B   20   /* Offset of DataSize and #Cycle Byte in the Operation Data Buffer */
#define OPERATION_DATA_OFFSET_READ_ADDRESSB_B    21   /* Offset of Read Address Bank Byte in the Operation Data buffer */
#define OPERATION_DATA_OFFSET_READ_ADDRESSH_B    22   /* Offset of Read Address High Byte in the Operation Data buffer */
#define OPERATION_DATA_OFFSET_READ_ADDRESSL_B    23   /* Offset of Read Address Low Byte in the Operation Data buffer */
#define OPERATION_DATA_OFFSET_WRITE_ADDRESSB_B   24   /* Offset of Write Address Bank Byte in the Operation Data buffer */
#define OPERATION_DATA_OFFSET_WRITE_ADDRESSH_B   25   /* Offset of Write Address High Byte in the Operation Data buffer */
#define OPERATION_DATA_OFFSET_WRITE_ADDRESSL_B   26   /* Offset of Write Address Low Byte in the Operation Data buffer */
#define OPERATION_DATA_OFFSET_JUMP_ADDRESSB_B    27   /* Offset of Jump Address Bank Byte in the Operation Data buffer */
#define OPERATION_DATA_OFFSET_JUMP_ADDRESSH_B    28   /* Offset of Jump Address High Byte in the Operation Data buffer */
#define OPERATION_DATA_OFFSET_JUMP_ADDRESSL_B    29   /* Offset of Jump Address Low Byte in the Operation Data buffer */
#define OPERATION_DATA_OFFSET_READ_VALUE_W       30   /* Offset of Read Value Bytes in the Operation Data buffer */
#define OPERATION_DATA_OFFSET_WRITE_VALUE_W      32   /* Offset of Write Value Bytes in the Operation Data buffer */
#define OPERATION_DATA_OFFSET_FLAGS_B            34   /* Offset of Flags Byte in the Operation Data buffer */
#define OPERATION_DATA_OFFSET_CALL_NUMBER_W      35   /* Offset of Call Number Bytes in the Operation Data Buffer */
#define OPERATION_DATA_OFFSET_ROM_B              37   /* Offset of ROM (01/03) Byte in the Operation Data buffer */
#define OPERATION_DATA_OFFSET_RAM_B              38   /* Offset of RAM Banks (4-255) Byte in the Operation Data buffer */
#define OPERATION_DATA_OFFSET_C02E_B             39   /* Offset of Softswitch $C02E Byte in the Operation Data buffer */
#define OPERATION_DATA_OFFSET_C02F_B             40   /* Offset of Softswitch $C02F Byte in the Operation Data buffer */
#define OPERATION_DATA_OFFSET_C035_B             41   /* Offset of Softswitch $C035 Byte in the Operation Data buffer */
#define OPERATION_DATA_OFFSET_C068_B             42   /* Offset of Softswitch $C068 Byte in the Operation Data buffer */
#define OPERATION_DATA_OFFSET_GLOBALID_Q         43   /* Offset of Global ID Bytes in the Operation Data buffer */
#define OPERATION_DATA_OFFSET_GLOBALCYCLE_Q      51   /* Offset of Global Cycle Bytes in the Operation Data buffer */

#define SNAPSHOT_DATA_OFFSET_NEXT_OPERATION       0   /* Offset of Next Operation Data in the Snapshot Data buffer */
#define SNAPSHOT_DATA_OFFSET_STOP               100   /* Offset of Stop Data in the Snapshot Data buffer v*/
#define SNAPSHOT_DATA_OFFSET_DOC_REGISTER       512   /* Offset of DOC Register 224+3 bytes in the Snapshot Data buffer */
#define SNAPSHOT_DATA_OFFSET_BATTERY_RAM        768   /* Offset of Battery Ram 256 bytes in the Snapshot Data buffer */
#define SNAPSHOT_DATA_OFFSET_RAM               1024   /* Offset of RAM 00 -> XX + E0 + E1 bytes in the Snapshot Data buffer */
                                                      /* ROM (128 KB or 256 KB) */
                                                      /* DOC RAM (64 KB) */

LRESULT CALLBACK ProcHandlerKegsIPCWindow(HWND,UINT,WPARAM,LPARAM);
static int CreateIPCStaticChildWindows(HWND);
static int DeleteIPCChildWindows(void);
static void ProcessCyreneToKegsData(int,int,unsigned char *);
static int SendDataToCyrene(struct cyrene_parameter *);
static int IsStopOperation(struct cyrene_operation *,unsigned char *,BYTE *,DWORD *,struct cyrene_parameter *);


/***************************************************************************************/
/*  CyreneIPCThreadInit() :  Traite la gestion des IPC Cyrene/Kegs via 1 Thread ddi. */
/***************************************************************************************/
DWORD CyreneIPCThreadInit(void *nothing)
{
  MSG msg;
  WNDCLASS wndclass;
  HINSTANCE hInstance;
  HWND hwndIPC;
  RECT client_rect, window_rect;
  struct cyrene_parameter *cyrene_param;
  my_Cyrene(CYRENE_GET_PARAM,&cyrene_param,NULL);

  /* Get Kegs Instance */
  hInstance = GetModuleHandle(NULL);

  /** Register IPC Windows Class **/
  if(cyrene_param->kegs_ipc_register_class_status == 0)
    {
      memset(&wndclass,0,sizeof(WNDCLASS));
      wndclass.style         = 0;
      wndclass.lpfnWndProc   = (WNDPROC) ProcHandlerKegsIPCWindow;
      wndclass.cbClsExtra    = 0;
      wndclass.cbWndExtra    = 0;
      wndclass.hInstance     = hInstance;
      wndclass.hIcon         = LoadIcon(NULL,IDI_APPLICATION);
      wndclass.hCursor       = LoadCursor((HINSTANCE)NULL,IDC_ARROW);
      wndclass.hbrBackground = (HBRUSH) COLOR_BACKGROUND;   /* Couleur de Fond */
      wndclass.lpszClassName = KEGS_IPC_WINDOW_CLASS;
      if(!RegisterClass(&wndclass))
        {
          printf("%s[KEGS//CYRENE]    => Error, Can't register Class for Cyrene IPC Window\n",GetCurrentTimeMS());
          cyrene_param->kegs_ipc_window_status = IPC_WINDOW_STATUS_ERROR;      /* Error */
          return(0);
        }
        
      /* La classe de fentre est enregistre */
      cyrene_param->kegs_ipc_register_class_status = 1;
    }

  /** Create IPC Window **/
  sprintf(cyrene_param->window_title,KEGS_IPC_WINDOW_NAME,cyrene_param->kegs_process_id);
  hwndIPC = CreateWindowEx(0,KEGS_IPC_WINDOW_CLASS,cyrene_param->window_title,WS_CAPTION|WS_SYSMENU|WS_MINIMIZEBOX,CW_USEDEFAULT,CW_USEDEFAULT,
                           IPC_WINDOW_MIN_WIDTH,IPC_WINDOW_MIN_HEIGHT,NULL,NULL,hInstance,NULL);
  if(hwndIPC == NULL)
    {
      printf("%s[KEGS//CYRENE]    => Error, Can't create Cyrene IPC Window\n",GetCurrentTimeMS());
      cyrene_param->kegs_ipc_window_status = IPC_WINDOW_STATUS_ERROR;      /* Error */
      return(0);
    }
  cyrene_param->hwnd_kegs_ipc = hwndIPC;

  /* Display Window (hidden) */
  ShowWindow(cyrene_param->hwnd_kegs_ipc,SW_HIDE);

  /* Title + Menu height */
  GetWindowRect(hwndIPC,&window_rect);
  GetClientRect(hwndIPC,&client_rect);
  cyrene_param->title_height = (window_rect.bottom - window_rect.top) - client_rect.bottom;

  /* IPC Window ready */
  cyrene_param->kegs_ipc_window_status = IPC_WINDOW_STATUS_READY;

  /**********************************************************************************************/
  /** Rentre dans la boucle de message de la fentre => Rception des donnes venant de Cyrene **/
  msg.wParam = 0;
  while(GetMessage(&msg,NULL,0,0))
    {
      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }

  /* IPC Window closed */
  cyrene_param->kegs_ipc_window_status = IPC_WINDOW_STATUS_CLOSE;

  /* Fin du Thread */
  ExitThread(0);

  /* OK */
  return(0);
}


/*****************************************************************/
/*  ProcHandlerKegsIPCWindow() :  Handle Kegs IPC Window Events. */
/*****************************************************************/
LRESULT CALLBACK ProcHandlerKegsIPCWindow(HWND hwndIPC, UINT umsg, WPARAM wParam, LPARAM lParam)
{
  COPYDATASTRUCT *cyrene_copy_data;
  WORD low_word_param, high_word_param;
  struct cyrene_parameter *cyrene_param;
  my_Cyrene(CYRENE_GET_PARAM,&cyrene_param,NULL);

  switch(umsg)
    {
      /****************************/
      /* IPC Window Initialization */
      /****************************/
      case WM_CREATE:
        CreateIPCStaticChildWindows(hwndIPC);
        return(0);

      /*******************************/
      /* IPC Window Resize / Refresh */
      /*******************************/
      case WM_SIZE:
        return(0);

      case WM_PAINT:
        break;
        
      /**************************************/
      /* Misc Events (Keyboard, buttons...) */
      /**************************************/
      case WM_COMMAND :
        low_word_param = LOWORD(wParam);
        high_word_param = HIWORD(wParam);
        return(0);

      /********************************************************/
      /* Inter Process Communication data from Cyrene to Kegs */
      /********************************************************/
      case WM_COPYDATA :
        /* Kegs is receiving Message from Cyrene */
        cyrene_copy_data = (COPYDATASTRUCT *) lParam;
        ProcessCyreneToKegsData((int)cyrene_copy_data->dwData,(int)cyrene_copy_data->cbData,(unsigned char *)cyrene_copy_data->lpData);
        return(1);         /* Send back 1 to Cyrene.exe to tell that message has been taken into account */

      /********************/
      /* IPC Window Close */
      /********************/
      case WM_CLOSE :
        /* IPC Window */
        DestroyWindow(hwndIPC);
        return(0);

      case WM_DESTROY :
        DeleteIPCChildWindows();
        PostQuitMessage(0);
        return(0);
    }

  /* Send back message to calling window */
  return(DefWindowProc(hwndIPC,umsg,wParam,lParam));
}


/***************************************************************************************************/
/*  CreateIPCStaticChildWindows() : Crer les fentres enfants et remplir la structure window_ipc. */
/***************************************************************************************************/
static int CreateIPCStaticChildWindows(HWND hwndKegsIPC)
{
  struct cyrene_parameter *cyrene_param;
  my_Cyrene(CYRENE_GET_PARAM,&cyrene_param,NULL);

  /* Fill the structure member ipc window */
  cyrene_param->hwnd_kegs_ipc = hwndKegsIPC;

  /** Device Context **/
  cyrene_param->hDC = GetDC(hwndKegsIPC);
  cyrene_param->hCompatibleDC = CreateCompatibleDC(cyrene_param->hDC);
  
  /* OK */
  return(0);
}


/****************************************************************/
/*  DeleteIPCChildWindows() : Destruction des fentres enfants. */
/****************************************************************/
static int DeleteIPCChildWindows(void)
{
  struct cyrene_parameter *cyrene_param;
  my_Cyrene(CYRENE_GET_PARAM,&cyrene_param,NULL);

  /* OK */
  return(0);
}


/**************************************************************************/
/*  ProcessCyreneToKegsData() :  Process Data coming from Cyrene to Kegs. */
/**************************************************************************/
static void ProcessCyreneToKegsData(int ipc_outputdata_type, int outputdata_size, unsigned char *output_data)
{
  int ram_type;
  DWORD raz_value, address;
  char buffer_value[2048];
  struct cyrene_parameter *cyrene_param;
  my_Cyrene(CYRENE_GET_PARAM,&cyrene_param,NULL);

  /*********************************************************/
  /*** C2K_CONNECT : Cyrene vient de se connecter  Kegs ***/
  if(ipc_outputdata_type == IPC_REQUEST_TYPE_C2K_OPEN_CONNECTION)
    {
      /* On va rcuprer le titre de la fentre IPC de Cyrene */
      GetNamedValueText((char *)output_data,"CyreneIPCWindowName=",buffer_value);
      
      /* On va rcuprer le Handle de la fentre IPC + Shared Memory de Cyrene */
      if(strlen(buffer_value) > 0)
        {
          /* Fentre IPC */
          strcpy(cyrene_param->cyrene_ipc_window_name,buffer_value);
          cyrene_param->hwnd_cyrene_ipc = FindWindow(cyrene_param->cyrene_ipc_window_class_name,cyrene_param->cyrene_ipc_window_name);
          if(cyrene_param->hwnd_cyrene_ipc != NULL)
            printf("%s[KEGS<-CYRENE] [%08X:%08X]  o Detect connection from Cyrene : %s\n",GetCurrentTimeMS(),(DWORD)(cyrene_param->goid>>32),(DWORD)(cyrene_param->goid & 0x00000000FFFFFFFF),cyrene_param->cyrene_ipc_window_name);
        }

      /* Kegs va devoir envoyer les Data Snapshot */
      cyrene_param->cyrene_connection_status = CONNECTION_STATUS_OPEN;
      cyrene_param->cyrene_connection_query = CONNECTION_QUERY_SNAPSHOT_DATA;

      /* Cyrene Data Received */
      cyrene_param->cyrene_answer_status = CYRENE_ANSWER_DONE;

      /* RAZ Shared Memory */
      memset(cyrene_param->shared_memory_ptr,0x00,IPC_SHARED_MEMORY_TOTAL_SIZE);
      raz_value = 0;
      memcpy(&cyrene_param->shared_memory_ptr[IPC_SHARED_MEMORY_OFFSET_OUTPUT_READ],&raz_value,sizeof(DWORD));
      memcpy(&cyrene_param->shared_memory_ptr[IPC_SHARED_MEMORY_OFFSET_OUTPUT_WRITE],&raz_value,sizeof(DWORD));
      memcpy(&cyrene_param->shared_memory_ptr[IPC_SHARED_MEMORY_OUTPUT_WRITE_MISSED],&raz_value,sizeof(DWORD));

      /** Rcupre / MAJ la liste des Breakpoints **/
      UpdateBreakpointList(cyrene_param);
    }

  /**************************************************************/
  /*** C2K_DISCONNECT : Cyrene vient de se dconnecterde Kegs ***/
  else if(ipc_outputdata_type == IPC_REQUEST_TYPE_C2K_CLOSE_CONNECTION)
    {
      /* Info */
      printf("%s[KEGS<-CYRENE] [%08X:%08X]  o Close Connection with Cyrene\n",GetCurrentTimeMS(),(DWORD)(cyrene_param->goid>>32),(DWORD)(cyrene_param->goid & 0x00000000FFFFFFFF));

      /* Close Cyrene Connection */
      strcpy(cyrene_param->cyrene_ipc_window_name,"");
      cyrene_param->hwnd_cyrene_ipc = NULL;

      /* La connection est ferme */
      cyrene_param->cyrene_connection_status = CONNECTION_STATUS_CLOSED;
      cyrene_param->cyrene_connection_query  = CONNECTION_QUERY_NOTHING;

      /* Cyrene Data Received */
      cyrene_param->cyrene_answer_status = CYRENE_ANSWER_DONE;

      /* No Previous Operation Address available */
      cyrene_param->next_operation_address = 0xFFFFFFFF;
    }

  /*************************************************************************/
  /*** C2K_GET_SNAPSHOT : Cyrene demande les donnes Snaphot IIgs  Kegs ***/ 
  else if(ipc_outputdata_type == IPC_REQUEST_TYPE_C2K_GET_SNAPSHOT)
    {
      /* Info */
      printf("%s[KEGS<-CYRENE] [%08X:%08X]  o Ask for Snapshot Data\n",GetCurrentTimeMS(),(DWORD)(cyrene_param->goid>>32),(DWORD)(cyrene_param->goid & 0x00000000FFFFFFFF));

      /* Kegs va devoir envoyer les Data Snapshot */
      cyrene_param->cyrene_connection_query = CONNECTION_QUERY_SNAPSHOT_DATA;

      /* Cyrene Data Received */
      cyrene_param->cyrene_answer_status = CYRENE_ANSWER_DONE;

      /** Rcupre / MAJ la liste des Breakpoints **/
      UpdateBreakpointList(cyrene_param);
    }

  /****************************************************************************/
  /*** C2K_GET_OPERATION : Cyrene demande les donnes Next Operation  Kegs ***/ 
  else if(ipc_outputdata_type == IPC_REQUEST_TYPE_C2K_GET_OPERATION)
    {
      /* Info */
      printf("%s[KEGS<-CYRENE] [%08X:%08X]  o Ask for Next Operations Data (%s)\n",GetCurrentTimeMS(),(DWORD)(cyrene_param->goid>>32),(DWORD)(cyrene_param->goid & 0x00000000FFFFFFFF),(char *)output_data);

      /* Kegs va devoir envoyer les Data Snapshot */
      cyrene_param->cyrene_connection_query = CONNECTION_QUERY_OPERATION_DATA;

      /* Cyrene Data Received */
      cyrene_param->cyrene_answer_status = CYRENE_ANSWER_DONE;

      /** Decode le Stop condition **/
      cyrene_param->stop_condition = 0;
      if(!stricmp((char *)output_data,"Stop=x1"))
        {
          cyrene_param->stop_condition = STOP_CONDITION_GOID;
          cyrene_param->stop_goid = cyrene_param->goid + 1;
        }
      else if(!stricmp((char *)output_data,"Stop=x10"))
        {
          cyrene_param->stop_condition = STOP_CONDITION_GOID;
          cyrene_param->stop_goid = cyrene_param->goid + 10;
        }
      else if(!stricmp((char *)output_data,"Stop=x100"))
        {
          cyrene_param->stop_condition = STOP_CONDITION_GOID;
          cyrene_param->stop_goid = cyrene_param->goid + 100;
        }
      else if(!stricmp((char *)output_data,"Stop=x1000"))
        {
          cyrene_param->stop_condition = STOP_CONDITION_GOID;
          cyrene_param->stop_goid = cyrene_param->goid + 1000;
        }
      else if(!stricmp((char *)output_data,"Stop=x10000"))
        {
          cyrene_param->stop_condition = STOP_CONDITION_GOID;
          cyrene_param->stop_goid = cyrene_param->goid + 10000;
        }
      else if(!stricmp((char *)output_data,"Stop=JMP"))
        cyrene_param->stop_condition = STOP_CONDITION_JMP;
      else if(!stricmp((char *)output_data,"Stop=JSR"))
        cyrene_param->stop_condition = STOP_CONDITION_JSR;
      else if(!stricmp((char *)output_data,"Stop=RTS"))
        cyrene_param->stop_condition = STOP_CONDITION_RTS;
      else if(!strnicmp((char *)output_data,"Stop=@",strlen("Stop=@")) && strlen((char *)output_data) == strlen("Stop=@XXXXXX"))
        {
          cyrene_param->stop_condition = STOP_CONDITION_ADDRESS;
          sscanf((char *)&output_data[strlen("Stop=@")],"%06X",&cyrene_param->stop_address);
        }
      else if(!stricmp((char *)output_data,"Stop=VBL"))
        cyrene_param->stop_condition = STOP_CONDITION_VBL;
      else if(!stricmp((char *)output_data,"Stop=INTERRUPT"))
        cyrene_param->stop_condition = STOP_CONDITION_INTERRUPT;
      else if(!stricmp((char *)output_data,"Stop=TOOLBOX"))
        cyrene_param->stop_condition = STOP_CONDITION_TOOLBOX;
      else if(!stricmp((char *)output_data,"Stop=OS"))
        cyrene_param->stop_condition = STOP_CONDITION_OS;

      /** Rcupre / MAJ la liste des Breakpoints **/
      UpdateBreakpointList(cyrene_param);
    }

  /*****************************************************************************************/
  /*** IPC_REQUEST_TYPE_C2K_WRITE_DATA : Cyrene demande des modifications Mmoire  Kegs ***/ 
  else if(ipc_outputdata_type == IPC_REQUEST_TYPE_C2K_WRITE_DATA)
    {
      /* Info */
      printf("%s[KEGS<-CYRENE] [%08X:%08X]  o Update KEGS Memory Data\n",GetCurrentTimeMS(),(DWORD)(cyrene_param->goid>>32),(DWORD)(cyrene_param->goid & 0x00000000FFFFFFFF));

      /* Cyrene Data Received */
      cyrene_param->cyrene_answer_status = CYRENE_ANSWER_DONE;

      /** On va mettre  jour la mmoire avec les donnes reues **/
      memcpy(&address,output_data,sizeof(DWORD));
      ram_type = ((address & 0xFF000000) == 0x00000000) ? RAMTYPE_RAM : RAMTYPE_DOC;
      SetMemoryData(ram_type,address&0x00FFFFFF,(DWORD)(outputdata_size-sizeof(DWORD)),output_data+sizeof(DWORD),cyrene_param);
    }

  /***************************************************************/
  /*** C2K_PAUSE : Cyrene demande  Kegs de se mettre en Pause ***/ 
  else if(ipc_outputdata_type == IPC_REQUEST_TYPE_C2K_PAUSE)
    {
      /* Info */
      printf("%s[KEGS<-CYRENE] [%08X:%08X]  o Ask for Pause\n",GetCurrentTimeMS(),(DWORD)(cyrene_param->goid>>32),(DWORD)(cyrene_param->goid & 0x00000000FFFFFFFF));

      /* Kegs va devoir envoyer les Data Snapshot */
      cyrene_param->cyrene_connection_query = CONNECTION_QUERY_PAUSE;

      /* Cyrene Data Received */
      cyrene_param->cyrene_answer_status = CYRENE_ANSWER_DONE;

      /** Rcupre / MAJ la liste des Breakpoints **/
      UpdateBreakpointList(cyrene_param);
    }
}


/********************************************************/
/*  SendCyreneData() :  Send Apple IIgs Data to Cyrene. */
/********************************************************/
void SendCyreneData(int connection_query, struct cyrene_operation *current_operation)
{
  HWND hwnd_cyrene_ipc;
  BYTE stop_condition;
  DWORD address, breakpoint_id;
  int i, j, error, wait_cycle, offset_data, nb_bank, is_stop_operation, exit_now;
  unsigned char buffer_operation_data[OPERATION_DATA_SIZE];
  struct cyrene_parameter *cyrene_param;
  my_Cyrene(CYRENE_GET_PARAM,&cyrene_param,NULL);

  /* Init */
  is_stop_operation = 0;

  /*** Prepare Next Operation Data (OPERATION_DATA_SIZE bytes long) ***/
  offset_data = OPERATION_DATA_SIZE;
  /* A [0] */
  memcpy(&buffer_operation_data[OPERATION_DATA_OFFSET_A_W],&current_operation->a_reg,sizeof(WORD));
  /* X [2] */
  memcpy(&buffer_operation_data[OPERATION_DATA_OFFSET_X_W],&current_operation->x_reg,sizeof(WORD));
  /* Y [4] */
  memcpy(&buffer_operation_data[OPERATION_DATA_OFFSET_Y_W],&current_operation->y_reg,sizeof(WORD));
  /* PC [6] */
  memcpy(&buffer_operation_data[OPERATION_DATA_OFFSET_PC_W],&current_operation->pc_reg,sizeof(WORD));
  /* PBR [8] */
  buffer_operation_data[OPERATION_DATA_OFFSET_PBR_B] = current_operation->pb_reg;
  /* DBR [9] */
  buffer_operation_data[OPERATION_DATA_OFFSET_DBR_B] = current_operation->db_reg;
  /* Stack [10] */
  memcpy(&buffer_operation_data[OPERATION_DATA_OFFSET_STACK_W],&current_operation->stack_reg,sizeof(WORD));
  /* Direct Page [12] */
  memcpy(&buffer_operation_data[OPERATION_DATA_OFFSET_DIRECT_W],&current_operation->direct_reg,sizeof(WORD));
  /* Status Register [14] */
  memcpy(&buffer_operation_data[OPERATION_DATA_OFFSET_STATUS_W],&current_operation->status_reg,sizeof(WORD));
  /* Opcode  [16] */
  buffer_operation_data[OPERATION_DATA_OFFSET_OPCODE_B] = current_operation->opcode;
  /* Operand #1 [17] */
  buffer_operation_data[OPERATION_DATA_OFFSET_OPERAND1_B] = current_operation->operand_1;
  /* Operand #2 [18] */
  buffer_operation_data[OPERATION_DATA_OFFSET_OPERAND2_B] = current_operation->operand_2;
  /* Operand #3 [19] */
  buffer_operation_data[OPERATION_DATA_OFFSET_OPERAND3_B] = current_operation->operand_3;

  /*** Extra Operation Information ***/
  /* Data Size + Nb Cycle [20] */
  buffer_operation_data[OPERATION_DATA_OFFSET_DATASIZE_CYCLE_B] = (BYTE) ((current_operation->operand_data_size << 4) & 0xF0) |
                                                                          (current_operation->nb_cycle & 0x0F);
  /* Read Address Bank [21] */
  buffer_operation_data[OPERATION_DATA_OFFSET_READ_ADDRESSB_B] = (BYTE) ((current_operation->read_address >> 16) & 0x000000FF);
  /* Read Address High [22] */
  buffer_operation_data[OPERATION_DATA_OFFSET_READ_ADDRESSH_B] = (BYTE) ((current_operation->read_address >> 8) & 0x000000FF);
  /* Read Address Low [23] */
  buffer_operation_data[OPERATION_DATA_OFFSET_READ_ADDRESSL_B] = (BYTE) (current_operation->read_address & 0x000000FF);
  /* Write Address Bank [24] */
  buffer_operation_data[OPERATION_DATA_OFFSET_WRITE_ADDRESSB_B] = (BYTE) ((current_operation->write_address >> 16) & 0x000000FF);
  /* Write Address High [25] */
  buffer_operation_data[OPERATION_DATA_OFFSET_WRITE_ADDRESSH_B] = (BYTE) ((current_operation->write_address >> 8) & 0x000000FF);
  /* Write Address Low [26] */
  buffer_operation_data[OPERATION_DATA_OFFSET_WRITE_ADDRESSL_B] = (BYTE) (current_operation->write_address & 0x000000FF);
  /* Jump Address Bank [27] */
  buffer_operation_data[OPERATION_DATA_OFFSET_JUMP_ADDRESSB_B] = (BYTE) ((current_operation->jump_address >> 16) & 0x000000FF);
  /* Jump Address High [28] */
  buffer_operation_data[OPERATION_DATA_OFFSET_JUMP_ADDRESSH_B] = (BYTE) ((current_operation->jump_address >> 8) & 0x000000FF);
  /* Jump Address Low [29] */
  buffer_operation_data[OPERATION_DATA_OFFSET_JUMP_ADDRESSL_B] = (BYTE) (current_operation->jump_address & 0x000000FF);
  /* Read Value [30] */
  memcpy(&buffer_operation_data[OPERATION_DATA_OFFSET_READ_VALUE_W],&current_operation->read_value,sizeof(WORD));
  /* Write Value [32] */
  memcpy(&buffer_operation_data[OPERATION_DATA_OFFSET_WRITE_VALUE_W],&current_operation->write_value,sizeof(WORD));
  /* Flags [34] */
  buffer_operation_data[OPERATION_DATA_OFFSET_FLAGS_B] = current_operation->flags;
  /* Call Number [35] */
  memcpy(&buffer_operation_data[OPERATION_DATA_OFFSET_CALL_NUMBER_W],&current_operation->call_number,sizeof(WORD));
  
  /*** Ajoute les informations ***/
  /* 1=ROM_01, 3=ROM_03 [37] */
  buffer_operation_data[OPERATION_DATA_OFFSET_ROM_B] = (unsigned char) (cyrene_param->version_rom);
  /* # RAM Banks (Min=4,Max=225) [38] */
  buffer_operation_data[OPERATION_DATA_OFFSET_RAM_B] = (unsigned char) (cyrene_param->size_ram / (64*1024));
  /* Softswitch $C02E : VertCnt [39] */
  buffer_operation_data[OPERATION_DATA_OFFSET_C02E_B] = current_operation->s_C02E;
  /* Softswitch $C02F : HorizCnt [40] */
  buffer_operation_data[OPERATION_DATA_OFFSET_C02F_B] = current_operation->s_C02F;
  /* Softswitch $C035 : Shadowing [41]*/
  buffer_operation_data[OPERATION_DATA_OFFSET_C035_B] = (unsigned char) ((*cyrene_param->g_c035_shadow_reg) & 0xFF);
  /* Softswitch $C068 : StateReg [42] */
  buffer_operation_data[OPERATION_DATA_OFFSET_C068_B] = (unsigned char) ((*cyrene_param->g_c068_statereg) & 0xFF);
  /* Global Operation ID : [43] */
  memcpy(&buffer_operation_data[OPERATION_DATA_OFFSET_GLOBALID_Q],&cyrene_param->goid,sizeof(QWORD));
  /* Global Cycle Count : Low [51] */
  memcpy(&buffer_operation_data[OPERATION_DATA_OFFSET_GLOBALCYCLE_Q],&cyrene_param->gcc,sizeof(QWORD));

  /*************************************************************************************/
  /**  Prepare Operation Context : Operation data + Hardware info + SoftSwitch value  **/
  /*************************************************************************************/
  if(connection_query == CONNECTION_QUERY_OPERATION_DATA)
    {
      /** Do we need to flush to Cyrene the Cached Operation Data ? **/
      if(cyrene_param->nb_operation_cache == MAX_OPERATION_CACHE)
        {
          /** Prepare Buffer Data **/
          memcpy(cyrene_param->request_data,cyrene_param->operation_cache_data,cyrene_param->nb_operation_cache*OPERATION_DATA_SIZE);
          cyrene_param->request_data_size = cyrene_param->nb_operation_cache*OPERATION_DATA_SIZE;
          cyrene_param->request_type = IPC_REQUEST_TYPE_K2C_SEND_OPERATION;

          /** Flush Cached Operation Data To Cyrene IPC Window **/
          printf("%s[KEGS->CYRENE] [%08X:%08X]  o Flush %d cached Operation%s Data to Cyrene\n",GetCurrentTimeMS(),(DWORD)(cyrene_param->goid>>32),(DWORD)(cyrene_param->goid & 0x00000000FFFFFFFF),cyrene_param->nb_operation_cache,(cyrene_param->nb_operation_cache>1)?"s":"");
          cyrene_param->cyrene_answer_status = CYRENE_ANSWER_WAIT;
          error = SendDataToCyrene(cyrene_param);

          /* Reset Cache Data */
          cyrene_param->nb_operation_cache = 0;
        }

      /** Did we reach the Stop Operation ? **/
      is_stop_operation = IsStopOperation(current_operation,buffer_operation_data,&stop_condition,&breakpoint_id,cyrene_param);

      /** If the Stop operation is not reached, Keep operation in Cache and Continue **/
      if(is_stop_operation == 0)
        {
          memcpy(&cyrene_param->operation_cache_data[cyrene_param->nb_operation_cache*OPERATION_DATA_SIZE],buffer_operation_data,OPERATION_DATA_SIZE);
          cyrene_param->nb_operation_cache++;   /* One more Operation Kept */
          return;
        }

      /** Stop Operation has been reached, Send the current Operation as Snapshot Data **/
    }

  /**********************************************************************************************/
  /**  Prepare Full Context : Operation data + Hardware info + SoftSwitch value + Memory data  **/
  /**********************************************************************************************/
  if(connection_query == CONNECTION_QUERY_SNAPSHOT_DATA || is_stop_operation == 1)
    {
      /** Empty Operation Cache Data **/
      if(cyrene_param->nb_operation_cache > 0)
        {
          /** Prepare Buffer Data **/
          memcpy(cyrene_param->request_data,cyrene_param->operation_cache_data,cyrene_param->nb_operation_cache*OPERATION_DATA_SIZE);
          cyrene_param->request_data_size = cyrene_param->nb_operation_cache*OPERATION_DATA_SIZE;
          cyrene_param->request_type = IPC_REQUEST_TYPE_K2C_SEND_OPERATION;

          /** Send Data To Cyrene IPC Window **/
          printf("%s[KEGS->CYRENE] [%08X:%08X]  o Send %d Operation%s Data to Cyrene\n",GetCurrentTimeMS(),(DWORD)(cyrene_param->goid>>32),(DWORD)(cyrene_param->goid & 0x00000000FFFFFFFF),cyrene_param->nb_operation_cache,(cyrene_param->nb_operation_cache>1)?"s":"");
          cyrene_param->cyrene_answer_status = CYRENE_ANSWER_WAIT;
          error = SendDataToCyrene(cyrene_param);

          /* Reset Cache Data */
          cyrene_param->nb_operation_cache = 0;
        }

      /** Snapshot Data **/
      cyrene_param->request_type = IPC_REQUEST_TYPE_K2C_SEND_SNAPSHOT;

      /*** Build Snapshot Data ***/
      memset(cyrene_param->request_data,0,1024);

      /** Next Operation Data **/
      memcpy(&cyrene_param->request_data[SNAPSHOT_DATA_OFFSET_NEXT_OPERATION],buffer_operation_data,OPERATION_DATA_SIZE);   

      /** Stop Data **/
      memcpy(&cyrene_param->request_data[SNAPSHOT_DATA_OFFSET_STOP+0],&stop_condition,1);
      if(stop_condition == STOP_CONDITION_BREAKPOINT)
        memcpy(&cyrene_param->request_data[SNAPSHOT_DATA_OFFSET_STOP+1],&breakpoint_id,sizeof(DWORD));

      /** DOC Register (224+3 bytes) **/
      offset_data = SNAPSHOT_DATA_OFFSET_DOC_REGISTER;
      for(i=0; i<32; i++)
        {
          cyrene_param->request_data[offset_data+0x00+i] = (BYTE) (cyrene_param->tab_doc_reg[i].freq & 0x000000FF);          /* 0x00-0x1F : Freq Low */
          cyrene_param->request_data[offset_data+0x20+i] = (BYTE) ((cyrene_param->tab_doc_reg[i].freq >> 8) & 0x000000FF);   /* 0x20-0x3F : Freq High */
          cyrene_param->request_data[offset_data+0x40+i] = (BYTE) (cyrene_param->tab_doc_reg[i].vol & 0x000000FF);           /* 0x40-0x5F : Volume */
          cyrene_param->request_data[offset_data+0x60+i] = (BYTE) (cyrene_param->tab_doc_reg[i].last_samp_val & 0x000000FF); /* 0x60-0x7F : Data */
          cyrene_param->request_data[offset_data+0x80+i] = (BYTE) (cyrene_param->tab_doc_reg[i].waveptr & 0x000000FF);       /* 0x80-0x9F : Wave Ptr */
          cyrene_param->request_data[offset_data+0xA0+i] = (BYTE) (cyrene_param->tab_doc_reg[i].ctl & 0x000000FF);           /* 0xA0-0xBF : Control */
          cyrene_param->request_data[offset_data+0xC0+i] = (BYTE) (cyrene_param->tab_doc_reg[i].wavesize & 0x000000FF);      /* 0xC0-0xDF : Wave Size */
        }
      offset_data += 224;
      /* E0 (Interruption) + E1 (#Osc*2) + E2 (Digit Data) */
      cyrene_param->request_data[offset_data+0] = (BYTE) *cyrene_param->ptr_doc_reg_e0;
      cyrene_param->request_data[offset_data+1] = (BYTE) (*cyrene_param->ptr_g_doc_num_osc_en - 1) << 1;
      cyrene_param->request_data[offset_data+2] = 0x80;   /* Silence Sound */
      offset_data += 3;

      /** BATTERY RAM (256 bytes) **/
      offset_data = SNAPSHOT_DATA_OFFSET_BATTERY_RAM;
      for(i=0; i<256; i++)
        cyrene_param->request_data[offset_data++] = cyrene_param->battery_ram[i];

      /** RAM 00 -> XX + E0 + E1 **/
      offset_data = 1024;
      nb_bank = cyrene_param->size_ram / (64*1024);
      for(i=0; i<nb_bank-2; i++)
        {
          /* Remplissage du Buffer de 64 KB */
          for(j=0; j<64*1024; j++)
            {
              address = (i<<16) | j;
              cyrene_param->request_data[offset_data++] = GetMemoryByte(address,current_operation,cyrene_param);
            }
        }
      for(i=0xE0; i<0xE2; i++)
        {
          /* Remplissage du Buffer de 64 KB */
          for(j=0; j<64*1024; j++)
            {
              address = (i<<16) | j;
              cyrene_param->request_data[offset_data++] = GetMemoryByte(address,current_operation,cyrene_param);
            }
        }

      /** ROM : 128 KB or 256 KB **/
      if(cyrene_param->version_rom == 0x01)
        {
          /* Rom 01 : 128 KB */
          memcpy(&cyrene_param->request_data[offset_data],cyrene_param->gs_rom+2*64*1024,2*64*1024);
          offset_data += 2*64*1024;
        }
      else
        {
          /* Rom 03 : 256 KB */
          memcpy(&cyrene_param->request_data[offset_data],cyrene_param->gs_rom,4*64*1024);
          offset_data += 4*64*1024;
        }

      /** DOC RAM **/
      memcpy(&cyrene_param->request_data[offset_data],cyrene_param->doc_ram,64*1024);
      offset_data += 64*1024;

      /* Data Size */
      cyrene_param->request_data_size = offset_data;
    }
  
  /** Send Data To Cyrene IPC Window **/
  printf("%s[KEGS->CYRENE] [%08X:%08X]  o Send 1 Operation + Snapshot Data to Cyrene\n",GetCurrentTimeMS(),(DWORD)(cyrene_param->goid>>32),(DWORD)(cyrene_param->goid & 0x00000000FFFFFFFF));
  cyrene_param->cyrene_answer_status = CYRENE_ANSWER_WAIT;
  error = SendDataToCyrene(cyrene_param);
  if(error)
    {
      /* Stop Sending Data if too many errors */
      cyrene_param->send_error_count++;
      if(cyrene_param->send_error_count > MAX_SEND_DATA_ERROR_COUNT)
        {
          /* Close Cyrene connection */
          printf("%s[KEGS//CYRENE] [%08X:%08X]    => Too many errors : Close Cyrene connection\n",GetCurrentTimeMS(),(DWORD)(cyrene_param->goid>>32),(DWORD)(cyrene_param->goid & 0x00000000FFFFFFFF));
          strcpy(cyrene_param->cyrene_ipc_window_name,"");
          cyrene_param->hwnd_cyrene_ipc = NULL;
          cyrene_param->cyrene_connection_status = CONNECTION_STATUS_CLOSED;
          cyrene_param->next_operation_address = 0xFFFFFFFF;
          return;
        }
    }

  /** Wait for Cyrene Answer (from dedicated IPC Thread) **/
  wait_cycle = 0;
  while(cyrene_param->cyrene_answer_status == CYRENE_ANSWER_WAIT)
    {
      /* Count */
      wait_cycle++;

      /* Connection has been closed by Cyrene in between */
      if(cyrene_param->cyrene_connection_status == CONNECTION_STATUS_CLOSED)
        return;

      /* Check if Cyrene is still there */
      hwnd_cyrene_ipc = FindWindow(cyrene_param->cyrene_ipc_window_class_name,cyrene_param->cyrene_ipc_window_name);
      if(hwnd_cyrene_ipc == NULL)
        {
          printf("%s[KEGS//CYRENE] [%08X:%08X]   => Connection lost with Cyrene\n",GetCurrentTimeMS(),(DWORD)(cyrene_param->goid>>32),(DWORD)(cyrene_param->goid & 0x00000000FFFFFFFF));
          /* Clean up Connection */
          strcpy(cyrene_param->cyrene_ipc_window_name,"");
          cyrene_param->hwnd_cyrene_ipc = NULL;
          cyrene_param->cyrene_connection_status = CONNECTION_STATUS_CLOSED;
          cyrene_param->next_operation_address = 0xFFFFFFFF;
          return;
        }
    }

  /* Reset for next Request */
  cyrene_param->cyrene_answer_status = CYRENE_ANSWER_WAIT;

  /***********************************/
  /**  Wait for Cyrene Instruction  **/
  /***********************************/
  if(cyrene_param->cyrene_connection_query == CONNECTION_QUERY_PAUSE)
    {
      /* Wait for Cyrene order */
      printf("%s[KEGS//CYRENE] [%08X:%08X]  >> Paused << : Wait for Cyrene instruction [%d]\n",GetCurrentTimeMS(),(DWORD)(cyrene_param->goid>>32),(DWORD)(cyrene_param->goid & 0x00000000FFFFFFFF),cyrene_param->counter++);
      while(cyrene_param->cyrene_connection_query == CONNECTION_QUERY_PAUSE)
        {
          /* Check if Cyrene is still there */
          hwnd_cyrene_ipc = FindWindow(cyrene_param->cyrene_ipc_window_class_name,cyrene_param->cyrene_ipc_window_name);
          if(hwnd_cyrene_ipc == NULL)
            {
              printf("%s[KEGS//CYRENE] [%08X:%08X]   => Connection lost with Cyrene\n",GetCurrentTimeMS(),(DWORD)(cyrene_param->goid>>32),(DWORD)(cyrene_param->goid & 0x00000000FFFFFFFF));
              /* Clean up Connection */
              strcpy(cyrene_param->cyrene_ipc_window_name,"");
              cyrene_param->hwnd_cyrene_ipc = NULL;
              cyrene_param->cyrene_connection_status = CONNECTION_STATUS_CLOSED;
              cyrene_param->next_operation_address = 0xFFFFFFFF;
            }

          /* Connection has been closed in between */
          if(cyrene_param->cyrene_connection_status == CONNECTION_STATUS_CLOSED)
            return;

          /** Refresh KEGS Windows **/
          exit_now = RefreshEmulatorWindows(cyrene_param);
          if(exit_now)
            {
              /** Send Data To Cyrene IPC Window **/
              printf("%s[KEGS->CYRENE] [%08X:%08X]  o Close Connection with Cyrene\n",GetCurrentTimeMS(),(DWORD)(cyrene_param->goid>>32),(DWORD)(cyrene_param->goid & 0x00000000FFFFFFFF));
              cyrene_param->request_data_size = 0;
              cyrene_param->request_type = IPC_REQUEST_TYPE_K2C_CLOSE_CONNECTION;
              cyrene_param->cyrene_answer_status = CYRENE_ANSWER_WAIT;
              error = SendDataToCyrene(cyrene_param);

              /* Clean up Connection */
              strcpy(cyrene_param->cyrene_ipc_window_name,"");
              cyrene_param->hwnd_cyrene_ipc = NULL;
              cyrene_param->cyrene_connection_status = CONNECTION_STATUS_CLOSED;
              cyrene_param->next_operation_address = 0xFFFFFFFF;

              return;
            }

          /* Wait */
          Sleep(10);
        }
      return;
    }
}


/*********************************************************************************/
/*  IsStopOperation() :  Dtermine si cette Opration est une Opration d'arrt. */
/*********************************************************************************/
static int IsStopOperation(struct cyrene_operation *current_operation, unsigned char *buffer_operation_data, BYTE *stop_condition_rtn, DWORD *breakpoint_id_rtn, struct cyrene_parameter *cyrene_param)
{
  BYTE flags;
  DWORD address_sub;
  int address, is_stop_operation, is_match;
  unsigned char opcode;
  struct cyrene_breakpoint *current_breakpoint;
  char breakpoint_output_line[8192];

  /* Init */
  is_stop_operation = 0;
  *stop_condition_rtn = 0x00,
  *breakpoint_id_rtn = 0x00000000;

  /** Vrifie les conditions d'arrt **/
  if(cyrene_param->stop_condition == STOP_CONDITION_GOID && cyrene_param->goid == cyrene_param->stop_goid)
    {
      is_stop_operation = 1;
      *stop_condition_rtn = STOP_CONDITION_GOID;
    }
  else if(cyrene_param->stop_condition == STOP_CONDITION_JMP)
    {
      /* Rcupration de l'Opcode */
      opcode = buffer_operation_data[OPERATION_DATA_OFFSET_OPCODE_B];

      /* JMP @@ JMP @@@ JMP (@@) JMP (@@,X) JMP [@@] BRA BRL BEQ BNE BCC BCS BMI BPL BVC BVS */
      if(opcode == 0x4C || opcode == 0x5C ||opcode == 0x6C || opcode == 0x7C || opcode == 0xDC ||
         opcode == 0x80 || opcode == 0x82 ||
         opcode == 0xF0 || opcode == 0xD0 || opcode == 0x90 || opcode == 0xB0 || opcode == 0x30 || opcode == 0x10 || opcode == 0x50 || opcode == 0x70)
        {
          is_stop_operation = 1;
          *stop_condition_rtn = STOP_CONDITION_JMP;
        }
    }
  else if(cyrene_param->stop_condition == STOP_CONDITION_JSR)
    {
      /* Rcupration de l'Opcode */
      opcode = buffer_operation_data[OPERATION_DATA_OFFSET_OPCODE_B];

      /* JSR @@ JSR (@@,X) JSL @@@ */
      if(opcode == 0x20 || opcode == 0xFC || opcode == 0x22)
        {
          is_stop_operation = 1;
          *stop_condition_rtn = STOP_CONDITION_JSR;
        }
    }
  else if(cyrene_param->stop_condition == STOP_CONDITION_RTS)
    {
      /* Rcupration de l'Opcode */
      opcode = buffer_operation_data[OPERATION_DATA_OFFSET_OPCODE_B];

      /* RTS RTL RTI */
      if(opcode == 0x60 || opcode == 0x6B || opcode == 0x40)
        {
          is_stop_operation = 1;
          *stop_condition_rtn = STOP_CONDITION_RTS;
        }
    }
  else if(cyrene_param->stop_condition == STOP_CONDITION_ADDRESS)
    {
      /* Address */
      address = (buffer_operation_data[OPERATION_DATA_OFFSET_PBR_B] << 16) | (buffer_operation_data[OPERATION_DATA_OFFSET_PC_W+1] << 8) | buffer_operation_data[OPERATION_DATA_OFFSET_PC_W+0];
      if(address == cyrene_param->stop_address)
        {
          is_stop_operation = 1;
          *stop_condition_rtn = STOP_CONDITION_ADDRESS;
        }
    }
  else if(cyrene_param->stop_condition == STOP_CONDITION_VBL)
    {
      if(cyrene_param->previous_vbl_status == 0 && cyrene_param->current_vbl_status == 1)
        {
          is_stop_operation = 1;
          *stop_condition_rtn = STOP_CONDITION_VBL;
        }
    }
  else if(cyrene_param->stop_condition == STOP_CONDITION_INTERRUPT)
    {
      /* Rcupration des flags */
      flags = buffer_operation_data[OPERATION_DATA_OFFSET_FLAGS_B];

      if((flags & OPERATION_FLAG_INTERRUPT) == OPERATION_FLAG_INTERRUPT)
        {
          is_stop_operation = 1;
          *stop_condition_rtn = STOP_CONDITION_INTERRUPT;
        }
    }
  else if(cyrene_param->stop_condition == STOP_CONDITION_TOOLBOX)
    {
      /* Rcupration de l'Opcode */
      opcode = buffer_operation_data[OPERATION_DATA_OFFSET_OPCODE_B];
      address_sub = (buffer_operation_data[OPERATION_DATA_OFFSET_OPERAND3_B] << 16) | 
                    (buffer_operation_data[OPERATION_DATA_OFFSET_OPERAND2_B] <<  8) | 
                     buffer_operation_data[OPERATION_DATA_OFFSET_OPERAND1_B];

      /* TOOLBOX : JSL $E1/0000 */
      if(opcode == 0x22 && address_sub == 0xE10000)
        {
          is_stop_operation = 1;
          *stop_condition_rtn = STOP_CONDITION_TOOLBOX;
        }
    }
  else if(cyrene_param->stop_condition == STOP_CONDITION_OS)
    {
      /* Rcupration de l'Opcode */
      opcode = buffer_operation_data[OPERATION_DATA_OFFSET_OPCODE_B];
      address_sub = (buffer_operation_data[OPERATION_DATA_OFFSET_OPERAND3_B] << 16) | 
                    (buffer_operation_data[OPERATION_DATA_OFFSET_OPERAND2_B] <<  8) | 
                     buffer_operation_data[OPERATION_DATA_OFFSET_OPERAND1_B];

      /* P16 / GS/OS : JSL $E1/00A8 / JSL $E1/00B0 */
      if(opcode == 0x22 && (address_sub == 0xE100A8 || address_sub == 0xE100B0))
        {
          is_stop_operation = 1;
          *stop_condition_rtn = STOP_CONDITION_OS;
        }
      /* P8 : JSR $00/BF00 */
      if((opcode == 0x20 || opcode == 0xFC) && address_sub == 0x00BF00)
        {
          is_stop_operation = 1;
          *stop_condition_rtn = STOP_CONDITION_OS;
        }
    }

  /** Check for Breakpoints **/
  if(is_stop_operation == 0 && cyrene_param->first_breakpoint != NULL)
    {
      /** Loop over all breakpoints **/
      for(current_breakpoint = cyrene_param->first_breakpoint; current_breakpoint; current_breakpoint = current_breakpoint->next)
        {
          /* Check if this Checkpoint match */
          is_match = CheckForBreakpointMatch(current_breakpoint,current_operation,buffer_operation_data);
          if(is_match == 1)
            {
              /* Log */
              printf("%s[KEGS//CYRENE] [%08X:%08X]  !! Breakpoint !! [%d] %s\n",GetCurrentTimeMS(),
                                                                                (DWORD)(cyrene_param->goid>>32),(DWORD)(cyrene_param->goid & 0x00000000FFFFFFFF),
                                                                                current_breakpoint->id,&current_breakpoint->breakpoint_line[2]);

              /* Action : Stop */
              if(current_breakpoint->action == BREAKPOINT_ACTION_STOP)
                {
                  is_stop_operation = 1;
                  *breakpoint_id_rtn = current_breakpoint->id;
                  *stop_condition_rtn = STOP_CONDITION_BREAKPOINT;
                  break;
                }
              else
                {
                  /* Construit la ligne */
                  if(current_breakpoint->action == BREAKPOINT_ACTION_PRINT)
                    BuildBreakpointOutputLine(current_breakpoint->action_print_text,current_operation,cyrene_param,&breakpoint_output_line[0]);
                  else if(current_breakpoint->action == BREAKPOINT_ACTION_FILE)
                    BuildBreakpointOutputLine(current_breakpoint->action_print_text,current_operation,cyrene_param,&breakpoint_output_line[0]);

                  /* Copie la ligne dans la Shared Memory */
                  CopyOutlineIntoSharedMemory(current_breakpoint->action,current_breakpoint->id,0,0,breakpoint_output_line,cyrene_param);
                }
            }
        }
    }

  /* Return Stop Operation status */
  return(is_stop_operation);
}


/*************************************************************************************/
/*  SendDataToCyrene() :  Envoie les donnes de la requte  Cyrene pour traitement. */
/*************************************************************************************/
static int SendDataToCyrene(struct cyrene_parameter *cyrene_param)
{
  LRESULT result;
  COPYDATASTRUCT copy_data;

  /** Fill the COPYDATA structure **/
  memset(&copy_data,0,sizeof(COPYDATASTRUCT));
  copy_data.dwData = (DWORD) cyrene_param->request_type;                     /* Function identifier = IPC_REQUEST_TYPE_XXX */
  copy_data.cbData = (DWORD) cyrene_param->request_data_size;                /* Size of data */
  copy_data.lpData = (unsigned char *) cyrene_param->request_data;           /* Request Data (Operation Data, Snapshot Data, Ram Data...) */

  /** Envoi la Requte  Cyrene via un message **/
  result = SendMessage(cyrene_param->hwnd_cyrene_ipc,WM_COPYDATA,(WPARAM)cyrene_param->hwnd_kegs_ipc,(LPARAM)(LPVOID)&copy_data);

  /* Process SendMessage Errors */
  if(result == FALSE)
    return(1);

  /* OK */
  return(0);
}


/*******************************************************************************/
/*  GetSharedMemoryCircularValue() :  Rcupre un octet dans la Shared Memory. */
/*******************************************************************************/
BYTE GetSharedMemoryCircularValue(int offset, unsigned char *shared_memory)
{
  int offset_circular;

  /* Offset dans un buffer circulaire */
  offset_circular = (offset < IPC_SHARED_MEMORY_OUTPUT_SIZE) ? offset : (offset - IPC_SHARED_MEMORY_OUTPUT_SIZE);

  /* Renvoie la valeur */
  return(shared_memory[IPC_SHARED_MEMORY_OFFSET_DATA_OUTPUT + offset_circular]);
}


/******************************************************************************/
/*  SetSharedMemoryCircularValue() :  Modifie un octet dans la Shared Memory. */
/******************************************************************************/
void SetSharedMemoryCircularValue(int offset, unsigned char *shared_memory, BYTE value)
{
  int offset_circular;

  /* Offset dans un buffer circulaire */
  offset_circular = (offset < IPC_SHARED_MEMORY_OUTPUT_SIZE) ? offset : (offset - IPC_SHARED_MEMORY_OUTPUT_SIZE);

  /* Modifie la valeur */
  shared_memory[IPC_SHARED_MEMORY_OFFSET_DATA_OUTPUT + offset_circular] = value;
}


/*****************************************************************************/
/*  CopyOutlineIntoSharedMemory() :  Copie la Outline dans la Shared Memory. */
/*****************************************************************************/
void CopyOutlineIntoSharedMemory(int action, DWORD breakpoint_id, int call_origin, WORD call_number, char *line, struct cyrene_parameter *cyrene_param)
{
  BYTE value;
  DWORD offset, new_offset, missed;
  int i, line_length, shared_memory_length, cyrene_connection_status;
  // char buffer_log[2048];
  char shared_memory_line[4096];

  /* On n'envoie rien dans la Shared Memory si on n'est pas connect  Cyrene */
  cyrene_connection_status = (cyrene_param->hwnd_cyrene_ipc == NULL) ? CONNECTION_STATUS_CLOSED : cyrene_param->cyrene_connection_status;
  if(cyrene_connection_status == CONNECTION_STATUS_CLOSED)
    return;

  /* Longueur de la ligne */
  line_length = (int) strlen(line);

  /** Construction de la ligne au format attendu dans la Shared Memory : [P/F/T][BreakpointID/Origin+CallNumber][Line][\n] **/
  /* Dbut de la ligne : Destination (Print / File / Trace) : 1 byte */
  if(action == BREAKPOINT_ACTION_PRINT)
    shared_memory_line[0] = 'P';
  else if(action == BREAKPOINT_ACTION_FILE)
    shared_memory_line[0] = 'F';
  else if(action == BREAKPOINT_ACTION_TRACE_SYSTEM_CALL)
    shared_memory_line[0] = 'T';
  /* Breakpoint ID | Call Number : 8 bytes */
  if(action == BREAKPOINT_ACTION_TRACE_SYSTEM_CALL)
    {
      if(call_origin == TRACE_CALL_ORIGIN_GSOS)
        sprintf(&shared_memory_line[1],"GSOS%04X",call_number);
      else if(call_origin == TRACE_CALL_ORIGIN_P16)
        sprintf(&shared_memory_line[1],"P16 %04X",call_number);
      else if(call_origin == TRACE_CALL_ORIGIN_P8)
        sprintf(&shared_memory_line[1],"P8  %04X",call_number);
      else if(call_origin == TRACE_CALL_ORIGIN_TOOL)
        sprintf(&shared_memory_line[1],"TOOL%04X",call_number);
      else if(call_origin == TRACE_CALL_ORIGIN_RETURN)
        strcpy(&shared_memory_line[1],"RETU0000");
    }
  else
    sprintf(&shared_memory_line[1],"%08X",breakpoint_id);
  /* Ligne */
  memcpy(&shared_memory_line[1+8],line,line_length);
  /* Fin de ligne */
  strcpy(&shared_memory_line[1+8+line_length],"\n");

  /* Taille de la ligne  crire dans la Shared Memory */
  shared_memory_length = strlen(shared_memory_line);

  /** Offset de dpart pour les Write dans la Shared Memory **/
  memcpy(&offset,&cyrene_param->shared_memory_ptr[IPC_SHARED_MEMORY_OFFSET_OUTPUT_WRITE],sizeof(DWORD));

  /** On va vrifier qu'il y a de la place dans la Shared Memory **/
  for(i=0; i<shared_memory_length; i++)
    {
      /* Rcupre la valeur */
      value = GetSharedMemoryCircularValue(offset+i,cyrene_param->shared_memory_ptr);
      if(value != 0x00)
        {
          /* Log */
          //LogTextLineDirect("c:\\AppleIIgs\\Kegs_SharedMemory.txt","  > Buffer OverFlow",1);

          /* le buffer est plein, on signale l'overflow */
          memcpy(&missed,&cyrene_param->shared_memory_ptr[IPC_SHARED_MEMORY_OUTPUT_WRITE_MISSED],sizeof(DWORD));
          missed++;
          memcpy(&cyrene_param->shared_memory_ptr[IPC_SHARED_MEMORY_OUTPUT_WRITE_MISSED],&missed,sizeof(DWORD));
          return;
        }
    }

  /** Copie la Ligne Output dans la Shared Memory (on recopie  l'envers) **/
  for(i=shared_memory_length-1; i>=0; i--)
    SetSharedMemoryCircularValue(offset+i,cyrene_param->shared_memory_ptr,shared_memory_line[i]);

  /** MAJ de l'offset pour les Write dans la Shared Memory **/
  new_offset = offset + shared_memory_length;
  new_offset = (new_offset < IPC_SHARED_MEMORY_OUTPUT_SIZE) ? new_offset : (new_offset - IPC_SHARED_MEMORY_OUTPUT_SIZE);
  memcpy(&cyrene_param->shared_memory_ptr[IPC_SHARED_MEMORY_OFFSET_OUTPUT_WRITE],&new_offset,sizeof(DWORD));

  /* Log */
 // sprintf(buffer_log,"%s  O=%08X  L=%X  O=%08X  '",GetCurrentTimeMS(),offset,shared_memory_length,new_offset);
 // LogTextLineDirect("c:\\AppleIIgs\\Kegs_SharedMemory.txt",buffer_log,0);
 // LogTextLineDirect("c:\\AppleIIgs\\Kegs_SharedMemory.txt",shared_memory_line,0);
 // LogTextLineDirect("c:\\AppleIIgs\\Kegs_SharedMemory.txt","'",1);
}

/************************************************************************************/
