/***********************************************************************/
/*                                                                     */
/*  Window_Breakpoint.c :  Module de gestion de la fentre Breakpoint. */
/*                                                                     */
/***********************************************************************/
/*  Auteur : Olivier ZARDINI  *   Brutal Deluxe   *   Date : Nov 2023  */
/***********************************************************************/

#include <stdio.h>
#include <time.h>

#include <windows.h>
#include <windowsx.h>
#include <Commctrl.h>
#include <WinUser.h>
#include <Richedit.h>
#include "resource.h"

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_Notification.h"
#include "Dc_Name.h"
#include "Dc_Graphic.h"
#include "Dc_CPU.h"
#include "Dc_MMU.h"
#include "Dc_IIgsInternals.h"
#include "Cyrene_Engine.h"
#include "Window_QuickWatch.h"
#include "Window_Main.h"
#include "Window_Main_Menu.h"
#include "Window_Disassembler.h"
#include "Window_MemoryArea.h"
#include "Window_Symbol.h"
#include "Window_SymbolPickup.h"
#include "Window_Breakpoint.h"

#define AM_UNKOWN                                0  /* Not recognize */
#define AM_IMPLICIT                              1  /* A          Implicit                             */
#define AM_ABSOLUTE                              2  /* addr2      Absolute                             */
#define AM_ABSOLUTE_INDEXED_X_INDIRECT           3  /* (addr2,X)  Absolute Indexed,X Indirect          */
#define AM_ABSOLUTE_INDEXED_X                    4  /* addr2,X    Absolute Indexed,X                   */
#define AM_ABSOLUTE_INDEXED_Y                    5  /* addr2,Y    Absolute Indexed,Y                   */
#define AM_ABSOLUTE_INDIRECT                     6  /* (addr2)    Absolute Indirect                    */
#define AM_ABSOLUTE_INDIRECT_LONG                7  /* [addr2]    Absolute Indirect Long               */
#define AM_ABSOLUTE_LONG                         8  /* addr3      Absolute Long                        */
#define AM_ABSOLUTE_LONG_INDEXED_X               9  /* addr3,X    Absolute Long Indexed,X              */
#define AM_DIRECT_PAGE                          10  /* dp         Direct Page                          */
#define AM_DIRECT_PAGE_INDEXED_X                11  /* dp,X       Direct Page Indexed,X                */
#define AM_DIRECT_PAGE_INDEXED_Y                12  /* dp,Y       Direct Page Indexed,Y                */
#define AM_DIRECT_PAGE_INDIRECT                 13  /* (dp)       Direct Page Indirect                 */
#define AM_DIRECT_PAGE_INDIRECT_LONG            14  /* [dp]       Direct Page Indirect Long            */
#define AM_DIRECT_PAGE_INDEXED_X_INDIRECT       15  /* (dp,X)     Direct Page Indexed,X Indirect       */
#define AM_DIRECT_PAGE_INDIRECT_INDEXED_Y       16  /* (dp),Y     Direct Page Indirect Indexed,Y       */
#define AM_DIRECT_PAGE_INDIRECT_LONG_INDEXED_Y  17  /* [dp],Y     Direct Page Indirect Long Indexed,Y  */
#define AM_IMMEDIATE_8                          18  /* #const 8   Immediate 8 bit                      */
#define AM_IMMEDIATE_16                         19  /* #const 16  Immediate 16 bit                      */
#define AM_PC_RELATIVE                          20  /* relative1  Program Counter Relative 8 bit (BRA, BEQ, BPL...) */
#define AM_PC_RELATIVE_LONG                     21  /* relative2  Program Counter Relative Long (BRL et PER)  */
#define AM_STACK_RELATIVE_INDIRECT_INDEXED_Y    22  /* (sr,S),Y   Stack Relative Indirect Indexed,Y    */
#define AM_STACK_PC_RELATIVE_LONG               23  /* label      Stack PC Relative Long               */
#define AM_STACK_RELATIVE                       24  /* sr,S       Stack Relative                       */
#define AM_BLOCK_MOVE                           25  /* Src,Dst    Block Move (MVN, MVP)                */

static DWORD BuildBreakpointWindowThread(void *);
LRESULT CALLBACK ProcHandlerBreakpointWindow(HWND,UINT,WPARAM,LPARAM);
static int CreateBreakpointChildWindows(HWND,struct window_breakpoint *);
LRESULT CALLBACK subEditBreakpointProcControl(HWND,UINT,WPARAM,LPARAM);
static void BuildBreakpointListView(struct list_view *,HFONT,int,struct window_breakpoint *);
static void FillBreakpointListView(struct list_view *,struct breakpoint_table *);
static struct cyrene_breakpoint *DecodeOneBreakpointLine(char *);
static void CreateBreakpoint(struct window_breakpoint *);
static void ModifyBreakpoint(struct window_breakpoint *);
static void DeleteBreakpoint(struct window_breakpoint *);
static void EnableBreakpoint(struct window_breakpoint *);
static void DisableBreakpoint(struct window_breakpoint *);
static void SaveBreakpointFile(struct window_breakpoint *);
static void CancelBreakpointOperation(struct window_breakpoint *);
static void ValidateBreakpointUpdate(struct window_breakpoint *);
static struct cyrene_breakpoint *LoadBreakpointFile(char *);
static int DecodeBreakpointSourceOperation(struct cyrene_breakpoint *);
static int GetAddressModeData(char *,int *,unsigned char *);
static int InsertUniqueBreakpoint(struct cyrene_breakpoint *,struct breakpoint_table *);
static int CompareBreakpoint(struct cyrene_breakpoint *,struct cyrene_breakpoint *);
static void ValidateBreakpointSourceOperation(struct window_breakpoint *);
static void DisplayHideBreakpointButton(struct window_breakpoint *,int);
static void EnableDisableBreakpointButton(struct window_breakpoint *,int,int);
static void UpdateBreakpointExtraInformation(struct window_breakpoint *,struct cyrene_breakpoint *);
static void RAZBreakpoint(struct window_breakpoint *);
static void UpdateBreakpointEditMode(struct window_breakpoint *,int);
static void UpdateBreakpointSource(struct window_breakpoint *,int);
static void UpdateBreakpointTrigger(struct window_breakpoint *,int);
static void UpdateBreakpointAction(struct window_breakpoint *,int);
static void UpdateBreakpointRepeat(struct window_breakpoint *,int);
static void DeleteBreakpointChildWindows(struct window_breakpoint *);
static int compare_breakpoint_address(const void *,const void *);
static int compare_reloc_address(const void *,const void *);
static int CheckBreakpointHexaValue(char *,int);
static void RebuildListViewContent(struct window_breakpoint *);
static DWORD GetBreakpopintHighestID(void);
static void BuildOutputLine(char *,char *);
static void BuildSymbolKeywordAddress(char *,char *);
static void UserResizeBreakpointWindow(struct window_breakpoint *);
static void MoveBreakpointWindowControls(int,struct window_breakpoint *);


/**********************************************************************************/
/*  BuildBreakpointWindow() :  Construction / Affichage de la fentre Breakpoint. */
/**********************************************************************************/
struct window_breakpoint *BuildBreakpointWindow(HINSTANCE hInstance, int menu_id)
{
  struct window_breakpoint *window_breakpoint;

  /** Allocation de la structure window_breakpoint **/
  window_breakpoint = (struct window_breakpoint *) calloc(1,sizeof(struct window_breakpoint));
  if(window_breakpoint == NULL)
    return(NULL);
  my_Memory(MEMORY_SET_WINDOW_BREAKPOINT,window_breakpoint,NULL);

  /* Lance un thread pour grer la fentre Breakpoint */
  CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)BuildBreakpointWindowThread,window_breakpoint,0,NULL);

  /* Renvoi la structure mmoire grant la fentre */
  return(window_breakpoint);
}


/********************************************************************************************/
/*  BuildBreakpointWindowThread() :  Cration de la fentre breakpoint via un Thread ddi. */
/********************************************************************************************/
static DWORD BuildBreakpointWindowThread(void *window_breakpoint_param)
{
  HWND hwndBreakpoint;
  MSG msg;
  BOOL bRet;
  struct window_main *window_main;
  struct window_breakpoint *window_breakpoint;
  struct parameter *param;

  /* Init */
  window_breakpoint = (struct window_breakpoint *) window_breakpoint_param;
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /** Cration de la fentre Breakpoint **/
  hwndBreakpoint = CreateDialog(param->hInstance,MAKEINTRESOURCE(IDD_WINDOW_BREAKPOINT),window_main->hwnd_main,(DLGPROC)ProcHandlerBreakpointWindow);
  if(hwndBreakpoint == NULL)
    return(1);

  /** Show Window + Paint Content **/
  ShowWindow(hwndBreakpoint,SW_SHOWDEFAULT);
  UpdateWindow(hwndBreakpoint);

  /** Message Loop pour toutes les fentres de ce Thread (fentre Breakpoint...) **/
  while((bRet = GetMessage(&msg,NULL,0,0)) != 0)
    {
      if(bRet == -1)
        break;

      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }

  /* Fin du Thread */
  ExitThread(0);
}


/***************************************************************************/
/*  ProcHandlerBreakpointWindow() :  Gestion des vnements de la fentre. */
/***************************************************************************/
LRESULT CALLBACK ProcHandlerBreakpointWindow(HWND hwndBreakpoint, UINT umsg, WPARAM wParam, LPARAM lParam)
{
  int i, select_id, breakpoint_id;
  UINT code, code_beginlabeledit, code_endlabeledit, code_getdispinfo, code_setdispinfo;
  BOOL result_bool;
  LRESULT result;
  NMLVDISPINFO *plvdi;
  LVITEM LvItem;
  NMLISTVIEW *pListView;
  char buffer_breakpoint[1024];
  struct list_view *current_listview;
  struct window_breakpoint *window_breakpoint;
  struct window_main *window_main;
  struct parameter *param;
  
  /* Init */
  my_Memory(MEMORY_GET_PARAM,&param,NULL);
  my_Memory(MEMORY_GET_WINDOW_BREAKPOINT,&window_breakpoint,NULL);
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);

  switch(umsg)
    {
      case WM_INITDIALOG :
        /* Cration du contenu de la DialogBox */
        CreateBreakpointChildWindows(hwndBreakpoint,window_breakpoint);
        return(TRUE);
        
      /****************************/
      /* Evnements de la fentre */
      /****************************/
      case WM_COMMAND :
        switch(LOWORD(wParam))
          {
            /** Bouton : Nouveau Breakpoint **/
            case IDC_BUTTON_NEW :
              if(HIWORD(wParam) == BN_CLICKED)
                CreateBreakpoint(window_breakpoint);
              break;

            /** Bouton : Modifier Breakpoint **/
            case IDC_BUTTON_MODIFY :
              if(HIWORD(wParam) == BN_CLICKED)
                ModifyBreakpoint(window_breakpoint);
              break;

            /** Bouton : Supprimer Breakpoint **/
            case IDC_BUTTON_DELETE :
              if(HIWORD(wParam) == BN_CLICKED)
                DeleteBreakpoint(window_breakpoint);
              break;

            /** Bouton : Supprimer tous les Breakpoints **/
            case IDC_BUTTON_DELETE_ALL :
              if(HIWORD(wParam) == BN_CLICKED)
                ClearBreakpointTable(window_breakpoint);
              break;

            /** Bouton : Activer Breakpoint **/
            case IDC_BUTTON_ENABLE :
              if(HIWORD(wParam) == BN_CLICKED)
                EnableBreakpoint(window_breakpoint);
              break;

            /** Bouton : Dsactiver Breakpoint **/
            case IDC_BUTTON_DISABLE :
              if(HIWORD(wParam) == BN_CLICKED)
                DisableBreakpoint(window_breakpoint);
              break;

            /** Bouton : Sauvegarde dans un fichier Breakpoint **/
            case IDC_BUTTON_SAVE :
              if(HIWORD(wParam) == BN_CLICKED)
                SaveBreakpointFile(window_breakpoint);
              break;

            /** Bouton : Charge un fichier Breakpoint **/
            case IDC_BUTTON_LOAD :
              if(HIWORD(wParam) == BN_CLICKED)
                LoadAppendBreakpointFile(NULL,window_breakpoint);
              break;

            /** Bouton : Annule la modification en cours **/
            case IDC_BUTTON_CANCEL :
              if(HIWORD(wParam) == BN_CLICKED)
                CancelBreakpointOperation(window_breakpoint);
              break;

            /** Bouton : Valide la modification en cours **/
            case IDC_BUTTON_OK :
              if(HIWORD(wParam) == BN_CLICKED)
                ValidateBreakpointUpdate(window_breakpoint);
              break;

            /** Bouton de slection de l'adresse via Symbol **/
            case IDC_BUTTON_SYMBOL :
              BuildSymbolPickupWindow(param->hInstance,window_breakpoint->hwnd_breakpoint,HM_BREAKPOINT_REFRESHSYMBOL);
              break;

            /** Bouton Radio : Source **/ 
            case IDC_RADIO_ADDRESS :
              if(HIWORD(wParam) == BN_CLICKED && window_breakpoint->source_radio != BREAKPOINT_SOURCE_ADDRESS)
                {
                  window_breakpoint->source_radio = BREAKPOINT_SOURCE_ADDRESS;
                  UpdateBreakpointSource(window_breakpoint,1);
                }
              break;
            case IDC_RADIO_SYMBOL :
              if(HIWORD(wParam) == BN_CLICKED && window_breakpoint->source_radio != BREAKPOINT_SOURCE_SYMBOL)
                {
                  window_breakpoint->source_radio = BREAKPOINT_SOURCE_SYMBOL;
                  UpdateBreakpointSource(window_breakpoint,1);
                }
              break;
            case IDC_RADIO_SYSTEMCALL :
              if(HIWORD(wParam) == BN_CLICKED && window_breakpoint->source_radio != BREAKPOINT_SOURCE_SYSTEMCALL)
                {
                  window_breakpoint->source_radio = BREAKPOINT_SOURCE_SYSTEMCALL;
                  UpdateBreakpointSource(window_breakpoint,1);
                }
              break;
            case IDC_RADIO_OPERATION :
              if(HIWORD(wParam) == BN_CLICKED && window_breakpoint->source_radio != BREAKPOINT_SOURCE_OPERATION)
                {
                  window_breakpoint->source_radio = BREAKPOINT_SOURCE_OPERATION;
                  UpdateBreakpointSource(window_breakpoint,1);
                }
              break; 
            case IDC_RADIO_REGISTER :
              if(HIWORD(wParam) == BN_CLICKED && window_breakpoint->source_radio != BREAKPOINT_SOURCE_REGISTER)
                {
                  window_breakpoint->source_radio = BREAKPOINT_SOURCE_REGISTER;
                  UpdateBreakpointSource(window_breakpoint,1);
                }
              break;

            /** Bouton Radio : Trigger **/ 
            case IDC_RADIO_READ :
              if(HIWORD(wParam) == BN_CLICKED && window_breakpoint->trigger_radio != BREAKPOINT_TRIGGER_READ)
                {
                  window_breakpoint->trigger_radio = BREAKPOINT_TRIGGER_READ;
                  UpdateBreakpointTrigger(window_breakpoint,1);
                }
              break;
            case IDC_RADIO_WRITE :
              if(HIWORD(wParam) == BN_CLICKED && window_breakpoint->trigger_radio != BREAKPOINT_TRIGGER_WRITE)
                {
                  window_breakpoint->trigger_radio = BREAKPOINT_TRIGGER_WRITE;
                  UpdateBreakpointTrigger(window_breakpoint,1);
                }
              break;
            case IDC_RADIO_READWRITE :
              if(HIWORD(wParam) == BN_CLICKED && window_breakpoint->trigger_radio != BREAKPOINT_TRIGGER_READWRITE)
                {
                  window_breakpoint->trigger_radio = BREAKPOINT_TRIGGER_READWRITE;
                  UpdateBreakpointTrigger(window_breakpoint,1);
                }
              break;
            case IDC_RADIO_EXECUTE :
              if(HIWORD(wParam) == BN_CLICKED && window_breakpoint->trigger_radio != BREAKPOINT_TRIGGER_EXECUTE)
                {
                  window_breakpoint->trigger_radio = BREAKPOINT_TRIGGER_EXECUTE;
                  UpdateBreakpointTrigger(window_breakpoint,1);
                }
              break;
            case IDC_RADIO_HOLD :
              if(HIWORD(wParam) == BN_CLICKED && window_breakpoint->trigger_radio != BREAKPOINT_TRIGGER_HOLD)
                {
                  window_breakpoint->trigger_radio = BREAKPOINT_TRIGGER_HOLD;
                  UpdateBreakpointTrigger(window_breakpoint,1);
                }
              break;

            /** Bouton Radio : Action **/
            case IDC_RADIO_STOP_EXECUTION :
              if(HIWORD(wParam) == BN_CLICKED && window_breakpoint->action_radio != BREAKPOINT_ACTION_STOP)
                {
                  window_breakpoint->action_radio = BREAKPOINT_ACTION_STOP;
                  UpdateBreakpointAction(window_breakpoint,1);
                }
              break;
            case IDC_RADIO_PRINT :
              if(HIWORD(wParam) == BN_CLICKED && window_breakpoint->action_radio != BREAKPOINT_ACTION_PRINT)
                {
                  window_breakpoint->action_radio = BREAKPOINT_ACTION_PRINT;
                  UpdateBreakpointAction(window_breakpoint,1);
                }
              break;
            case IDC_RADIO_FILE :
              if(HIWORD(wParam) == BN_CLICKED && window_breakpoint->action_radio != BREAKPOINT_ACTION_FILE)
                {
                  window_breakpoint->action_radio = BREAKPOINT_ACTION_FILE;
                  UpdateBreakpointAction(window_breakpoint,1);
                }
              break;

            /** Bouton Radio : Repeat **/
            case IDC_RADIO_REPEAT_ALWAYS :
              if(HIWORD(wParam) == BN_CLICKED && window_breakpoint->repeat_radio != BREAKPOINT_REPEAT_ALWAYS)
                {
                  window_breakpoint->repeat_radio = BREAKPOINT_REPEAT_ALWAYS;
                  UpdateBreakpointRepeat(window_breakpoint,1);
                }
              break;
            case IDC_RADIO_REPEAT_ONLY :
              if(HIWORD(wParam) == BN_CLICKED && window_breakpoint->repeat_radio != BREAKPOINT_ACTION_PRINT)
                {
                  window_breakpoint->repeat_radio = BREAKPOINT_ACTION_PRINT;
                  UpdateBreakpointRepeat(window_breakpoint,1);
                }
              break;
            case IDC_RADIO_REPEAT_AFTER :
              if(HIWORD(wParam) == BN_CLICKED && window_breakpoint->repeat_radio != BREAKPOINT_ACTION_FILE)
                {
                  window_breakpoint->repeat_radio = BREAKPOINT_ACTION_FILE;
                  UpdateBreakpointRepeat(window_breakpoint,1);
                }
              break;

            /** Slection dans la ComboBox : SystemCall **/
            case IDC_COMBOBOX_REGISTER :
              if(HIWORD(wParam) == CBN_CLOSEUP)
                {
                  /* Rcupre l'index de ce qui est slectionn 0-N */
                  result = SendMessage(window_breakpoint->hwnd_combo_register,CB_GETCURSEL,0,0);
                  if(result != CB_ERR)
                    window_breakpoint->source_register = (int) result;
                }
              break;

            /** Slection dans la ComboBox : Source SystemCall **/
            case IDC_COMBOBOX_SYSTEMCALL :
              if(HIWORD(wParam) == CBN_CLOSEUP)
                {
                  /* Rcupre l'index de ce qui est slectionn 0-N */
                  result = SendMessage(window_breakpoint->hwnd_combo_systemcall,CB_GETCURSEL,0,0);
                  if(result != CB_ERR)
                    {
                      if(param->tab_systemcall != NULL)
                        {
                          window_breakpoint->source_systemcall_type = param->tab_systemcall[(int)result]->type;
                          window_breakpoint->source_systemcall_number = param->tab_systemcall[(int)result]->call_number;
                          strcpy(window_breakpoint->source_systemcall_text,param->tab_systemcall[(int)result]->text);
                          window_breakpoint->source_systemcall_index = (int) result;
                        }
                    }
                }
              break;

            /** Slection dans la ComboBox : Source Read Operator **/
            case IDC_COMBOBOX_READ_OPERATOR :
              if(HIWORD(wParam) == CBN_CLOSEUP)
                {
                  /* Rcupre l'index de ce qui est slectionn : 0-N */
                  result = SendMessage(window_breakpoint->hwnd_combo_read_operator,CB_GETCURSEL,0,0);
                  if(result != CB_ERR)
                    window_breakpoint->trigger_read_operator = (int) result;
                }
              break;

            /** Slection dans la ComboBox : Source Write Operator **/
            case IDC_COMBOBOX_WRITE_OPERATOR :
              if(HIWORD(wParam) == CBN_CLOSEUP)
                {
                  /* Rcupre l'index de ce qui est slectionn */
                  result = SendMessage(window_breakpoint->hwnd_combo_write_operator,CB_GETCURSEL,0,0);
                  if(result != CB_ERR)
                    window_breakpoint->trigger_write_operator = (int) result;
                }
              break;

            /** Slection dans la ComboBox : Source ReadWrite Operator **/
            case IDC_COMBOBOX_READWRITE_OPERATOR :
              if(HIWORD(wParam) == CBN_CLOSEUP)
                {
                  /* Rcupre l'index de ce qui est slectionn */
                  result = SendMessage(window_breakpoint->hwnd_combo_readwrite_operator,CB_GETCURSEL,0,0);
                  if(result != CB_ERR)
                    window_breakpoint->trigger_readwrite_operator = (int) result;
                }
              break;

            /** Slection dans la ComboBox : Source Hold Operator **/
            case IDC_COMBOBOX_HOLD_OPERATOR :
              if(HIWORD(wParam) == CBN_CLOSEUP)
                {
                  /* Rcupre l'index de ce qui est slectionn */
                  result = SendMessage(window_breakpoint->hwnd_combo_hold_operator,CB_GETCURSEL,0,0);
                  if(result != CB_ERR)
                    window_breakpoint->trigger_hold_operator = (int) result;
                }
              break;
          }
        return(TRUE);

      /********************************************/
      /* Evnements des CommonControls (ListView) */
      /********************************************/
      case WM_NOTIFY :
        code = ((LPNMHDR)lParam)->code;
        code_beginlabeledit = LVN_BEGINLABELEDIT;
        code_endlabeledit   = LVN_ENDLABELEDIT;
        code_getdispinfo    = LVN_GETDISPINFO;
        code_setdispinfo    = LVN_SETDISPINFO;

        /*****************************************************************************************************/
        /**  Affichage des Item de la fentre : Remplissage des Item dans les colonnes des lignes affiches **/
        /*****************************************************************************************************/
        if(code == LVN_GETDISPINFO)
          {
            plvdi = (NMLVDISPINFO *) lParam;

            /* Identifie la ListView : Breakpoint / Breakpoints */
            if(LOWORD(wParam) == window_breakpoint->listview_breakpoint.idc_listview)
              current_listview = (struct list_view *) &window_breakpoint->listview_breakpoint;
            else
              break;

            /* Valide le numro de la colonne */
            if(plvdi->item.iSubItem < current_listview->nb_column)
              {
                plvdi->item.pszText = current_listview->tab_column[plvdi->item.iSubItem].tab_value_txt[(current_listview->has_column_sort == INTERNALS_COLUMN_SORT_NONE) ? plvdi->item.iItem : plvdi->item.lParam];
              }
          }

        /***************************************************************/
        /**  Click sur une ligne d'une ListView pour la slectionner  **/
        /***************************************************************/
        if(code == NM_CLICK &&  window_breakpoint->edit_mode == 0)
          {
            /* Rcupre la ListView (Breakpoint / Breakpoint) */
            if(LOWORD(wParam) == window_breakpoint->listview_breakpoint.idc_listview)
              current_listview = (struct list_view *) &window_breakpoint->listview_breakpoint;
            else
              break;

            /* Identifie la ligne clicke (0-N) */
            select_id = (int) SendMessage(current_listview->hwnd_listview,LVM_GETNEXTITEM,-1,LVNI_FOCUSED); // return item selected
            if(select_id == -1)
              return(TRUE);

            /* Rcupre le numro de la structure Breakpoint dans la Colonne #5 */
            memset(&LvItem,0,sizeof(LVITEM));
            LvItem.mask = LVIF_TEXT;
            LvItem.iSubItem = 5;            /* Colonne #5 */
            LvItem.pszText = buffer_breakpoint;
            LvItem.cchTextMax = 256;
            LvItem.iItem = select_id;
            SendMessage(current_listview->hwnd_listview,LVM_GETITEMTEXT,select_id,(LPARAM)&LvItem);
            breakpoint_id = atoi(buffer_breakpoint);

            /** On remplir les informations complmentaires du Breakpoint **/
            UpdateBreakpointExtraInformation(window_breakpoint,window_main->current_breakpoint_table->tab_breakpoint[breakpoint_id]);

            /** On active les boutons MODIFY DELETE ENABLE DISABLE **/
            EnableDisableBreakpointButton(window_breakpoint,1,window_main->current_breakpoint_table->tab_breakpoint[breakpoint_id]->status);
          }

        /***************************************************************/
        /**  Click sur les titres des colonnes pour activer les tris  **/
        /***************************************************************/
        if(code == LVN_COLUMNCLICK)
          {
            pListView = (NMLISTVIEW *) lParam;
            select_id = (int) pListView->iSubItem;

            /* Rcupre la ListView (Breakpoint / Breakpoints) */
            if(LOWORD(wParam) == window_breakpoint->listview_breakpoint.idc_listview)
              current_listview = (struct list_view *) &window_breakpoint->listview_breakpoint;
            else
              break;

            /* On fait le choix du type de tri */
            for(i=0; i<current_listview->nb_column; i++)
              {
                if(i == select_id)
                  {
                    if(current_listview->tab_column[i].sort == INTERNALS_COLUMN_SORT_NONE)
                      current_listview->tab_column[i].sort = INTERNALS_COLUMN_SORT_UP;
                    else if(current_listview->tab_column[i].sort == INTERNALS_COLUMN_SORT_UP)
                      current_listview->tab_column[i].sort = INTERNALS_COLUMN_SORT_DOWN;
                    else
                      current_listview->tab_column[i].sort = INTERNALS_COLUMN_SORT_NONE;
                  }
               else
                 current_listview->tab_column[i].sort = INTERNALS_COLUMN_SORT_NONE;
              }
             
            /* A t'on une colonne trie ? */
            current_listview->has_column_sort = INTERNALS_COLUMN_SORT_NONE;
            for(i=0; i<current_listview->nb_column; i++)
              if(current_listview->tab_column[i].sort != INTERNALS_COLUMN_SORT_NONE)
                current_listview->has_column_sort = current_listview->tab_column[i].sort;
             
            /* On va trier selon le tri mis en place */
            result_bool = ListView_SortItems(current_listview->hwnd_listview,ProcessListViewCompareProc,current_listview);
           
            /* MAJ des images ^ / v des Tris des colonnes */
            setListViewSortIcon(current_listview);
          }

        /***********************************************************************************/
        /**  Gestion de l'affichage des Item pour dfinir une couleur de fond de cellule  **/
        /***********************************************************************************/
        if(code == NM_CUSTOMDRAW)
          {
            SetWindowLongPtr(window_breakpoint->hwnd_breakpoint,DWLP_MSGRESULT,ProcessListViewWindowCustomDraw(lParam));
            return(CDRF_NOTIFYITEMDRAW);
          }

        return(TRUE);
        break;

      /*****************************************************************/
      /* Message extrieur demandant le rafraichissement de la fentre */
      /*****************************************************************/
      /** On vient de demander la MAJ de l'adresse via un Symbol **/
      case HM_BREAKPOINT_REFRESHSYMBOL :
        if(param->symbol_is_available == 1)
          {
            /* Conserve le Symbol */
            window_breakpoint->source_symbol_begin_address = param->symbol_address;
            window_breakpoint->source_symbol_end_address = window_breakpoint->source_symbol_begin_address + param->symbol_size - 1;
            strcpy(window_breakpoint->source_symbol_name,param->symbol_name);
            strcpy(window_breakpoint->source_symbol_segment,param->symbol_segment);
            strcpy(window_breakpoint->source_symbol_program,param->symbol_file);
            /* MAJ L'affichage Nom Symbol */
            sprintf(buffer_breakpoint,"%s",param->symbol_name);
            SetWindowText(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_LABEL_SYMBOL1),buffer_breakpoint);
            sprintf(buffer_breakpoint,"[%s | %s]",param->symbol_segment,param->symbol_file);
            SetWindowText(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_LABEL_SYMBOL2),buffer_breakpoint);
            /* MAJ des Adresses */
            sprintf(buffer_breakpoint,"%02X",(BYTE)((window_breakpoint->source_symbol_begin_address>>16) & 0x000000FF));
            SetWindowText(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_BANK),buffer_breakpoint);
            sprintf(buffer_breakpoint,"%04X",(WORD)(window_breakpoint->source_symbol_begin_address & 0x0000FFFF));
            SetWindowText(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_ADDRESS),buffer_breakpoint);
            sprintf(buffer_breakpoint,"%02X",(BYTE)((window_breakpoint->source_symbol_end_address>>16) & 0x000000FF));
            SetWindowText(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_BANK_DST),buffer_breakpoint);
            sprintf(buffer_breakpoint,"%04X",(WORD)(window_breakpoint->source_symbol_end_address & 0x0000FFFF));
            SetWindowText(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_ADDRESS_DST),buffer_breakpoint);
          }
        return(TRUE);

      /**************************************/
      /*  On vient de retailler la fentre  */
      /**************************************/
      case WM_SIZE :
        /* On va tendre la fentre de donne mmoire si c'est un Resize User */
        if(window_breakpoint->user_resize == 1)
          UserResizeBreakpointWindow(window_breakpoint);
        window_breakpoint->user_resize = 1;
        return(TRUE);

      /********************************************************************/
      /* Fermeture de window (le WM_CLOSE est appel avant le WM_DESTROY) */
      /********************************************************************/
      case WM_CLOSE :
        /* Fentre Breakpoint */
        DestroyWindow(hwndBreakpoint);
        return(0);

      case WM_DESTROY :
        /* Suppression des Fentres */
        DeleteBreakpointChildWindows(window_breakpoint);
        CheckMainWindowMenuItem(ID_MENU_CODE_BREAKPOINT,FALSE);
        my_Memory(MEMORY_FREE_WINDOW_BREAKPOINT,NULL,NULL);
        /* Demande la sortie de la boucle des messages */
        PostQuitMessage(0);
        return(0);
    }

  /* Retourner FALSE pour relayer le message non trait au systme */
  return(FALSE);
}


/***********************************************************************************************************/
/*  CreateBreakpointChildWindows() : Crer les fentres enfants et remplir la structure window_breakpoint. */
/***********************************************************************************************************/
static int CreateBreakpointChildWindows(HWND hwndBreakpoint, struct window_breakpoint *window_breakpoint)
{
  int i;
  RECT window_rect;
  char buffer_systemcall[BUFFER_SIZE];
  struct window_main *window_main;
  struct system_call *current_systemcall;
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Rcupre la fentre Main */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);

  /** Window Handle **/
  window_breakpoint->hwnd_breakpoint = hwndBreakpoint;

  /** Surchage des Control Edit pour capter les vnements **/
  window_breakpoint->oldEditProcSrcBank        = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndBreakpoint,IDC_EDIT_ADDRESS_BANK),GWLP_WNDPROC,(LONG_PTR)subEditBreakpointProcControl);
  window_breakpoint->oldEditProcSrcAddress     = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndBreakpoint,IDC_EDIT_ADDRESS_ADDRESS),GWLP_WNDPROC,(LONG_PTR)subEditBreakpointProcControl);
  window_breakpoint->oldEditProcDstBank        = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndBreakpoint,IDC_EDIT_ADDRESS_BANK_DST),GWLP_WNDPROC,(LONG_PTR)subEditBreakpointProcControl);
  window_breakpoint->oldEditProcDstAddress     = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndBreakpoint,IDC_EDIT_ADDRESS_ADDRESS_DST),GWLP_WNDPROC,(LONG_PTR)subEditBreakpointProcControl);
  window_breakpoint->oldEditProcOperationData  = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndBreakpoint,IDC_EDIT_OPERATION_DATA),GWLP_WNDPROC,(LONG_PTR)subEditBreakpointProcControl);
  window_breakpoint->oldEditProcReadValue      = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndBreakpoint,IDC_EDIT_READ_VALUE),GWLP_WNDPROC,(LONG_PTR)subEditBreakpointProcControl);
  window_breakpoint->oldEditProcWriteValue     = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndBreakpoint,IDC_EDIT_WRITE_VALUE),GWLP_WNDPROC,(LONG_PTR)subEditBreakpointProcControl);
  window_breakpoint->oldEditProcReadWriteValue = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndBreakpoint,IDC_EDIT_READWRITE_VALUE),GWLP_WNDPROC,(LONG_PTR)subEditBreakpointProcControl);
  window_breakpoint->oldEditProcHoldValue      = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndBreakpoint,IDC_EDIT_HOLD_VALUE),GWLP_WNDPROC,(LONG_PTR)subEditBreakpointProcControl);
  window_breakpoint->oldEditProcPrintData      = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndBreakpoint,IDC_EDIT_PRINT_DATA),GWLP_WNDPROC,(LONG_PTR)subEditBreakpointProcControl);
  window_breakpoint->oldEditProcFilePath       = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndBreakpoint,IDC_EDIT_FILE_PATH),GWLP_WNDPROC,(LONG_PTR)subEditBreakpointProcControl);
  window_breakpoint->oldEditProcRepeatOnly     = (WNDPROC) SetWindowLongPtr(GetDlgItem(hwndBreakpoint,IDC_EDIT_REPEAT_ONLY_DATA),GWLP_WNDPROC,(LONG_PTR)subEditBreakpointProcControl);

  /* Police de caractre de la fentre (14 points) */
  window_breakpoint->hFont = CreateFontA(LISTVIEW_FONT_HEIGHT,0,0,0,FW_DONTCARE,FALSE,FALSE,FALSE,ANSI_CHARSET,
                                         OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY,DEFAULT_PITCH|FF_SWISS,"Arial");

  /******************/
  /*** Breakpoint ***/
  /******************/
  window_breakpoint->breakpoint_x0 = 7;
  window_breakpoint->breakpoint_y0 = 7;

  /** Ajoute les images sur les boutons **/
  SendMessage(GetDlgItem(hwndBreakpoint,IDC_BUTTON_SYMBOL),BM_SETIMAGE,IMAGE_ICON,(LPARAM)LoadIcon(param->hInstance,MAKEINTRESOURCE(IDI_ICON_SYMBOL)));

  /** Cration ComboBox SystemCall **/
  window_breakpoint->hwnd_combo_systemcall = CreateWindowEx(0,"COMBOBOX", 0, CBS_DROPDOWNLIST | WS_CHILD | WS_VISIBLE | WS_VSCROLL,
                                                            89+12,320+4,205,340,hwndBreakpoint,(HMENU)IDC_COMBOBOX_SYSTEMCALL,param->hInstance,0);
  for(i=0; i<param->nb_systemcall; i++)
    {
      current_systemcall = param->tab_systemcall[i];
      if(current_systemcall->type == TYPE_SYSTEM_CALL_P8)
        sprintf(buffer_systemcall,"Prodos8 :  %s  [%02X]",current_systemcall->name,current_systemcall->call_number);
      else if(current_systemcall->type == TYPE_SYSTEM_CALL_P16)
        sprintf(buffer_systemcall,"Prodos16 :  %s  [%04X]",current_systemcall->name,current_systemcall->call_number);
      else if(current_systemcall->type == TYPE_SYSTEM_CALL_GSOS)
        sprintf(buffer_systemcall,"GS/OS :  %s  [%04X]",current_systemcall->name,current_systemcall->call_number);
      else if(current_systemcall->type == TYPE_SYSTEM_CALL_TOOLBOX)
        sprintf(buffer_systemcall,"Tool %03d :  %s  [%04X]",current_systemcall->call_number & 0x00FF,current_systemcall->name,current_systemcall->call_number);
      SendMessage(window_breakpoint->hwnd_combo_systemcall,CB_ADDSTRING,0,(LPARAM)buffer_systemcall);
    }
  SendMessage(window_breakpoint->hwnd_combo_systemcall,WM_SETFONT,(WPARAM)window_breakpoint->hFont,TRUE);

  /** Cration ComboBox Register **/
  window_breakpoint->hwnd_combo_register = CreateWindowEx(0,"COMBOBOX", 0, CBS_DROPDOWNLIST | WS_CHILD | WS_VISIBLE,
                                                          89+12,320+62,117,170,hwndBreakpoint,(HMENU)IDC_COMBOBOX_REGISTER,param->hInstance,0);
  SendMessage(window_breakpoint->hwnd_combo_register,CB_ADDSTRING,0,(LPARAM)"A");
  SendMessage(window_breakpoint->hwnd_combo_register,CB_ADDSTRING,0,(LPARAM)"X");
  SendMessage(window_breakpoint->hwnd_combo_register,CB_ADDSTRING,0,(LPARAM)"Y");
  SendMessage(window_breakpoint->hwnd_combo_register,CB_ADDSTRING,0,(LPARAM)"PC");
  SendMessage(window_breakpoint->hwnd_combo_register,CB_ADDSTRING,0,(LPARAM)"STACK");
  SendMessage(window_breakpoint->hwnd_combo_register,CB_ADDSTRING,0,(LPARAM)"DIRECT");
  SendMessage(window_breakpoint->hwnd_combo_register,CB_ADDSTRING,0,(LPARAM)"DBR");
  SendMessage(window_breakpoint->hwnd_combo_register,CB_ADDSTRING,0,(LPARAM)"PBR");
  SendMessage(window_breakpoint->hwnd_combo_register,CB_ADDSTRING,0,(LPARAM)"STATUS");
  SendMessage(window_breakpoint->hwnd_combo_register,WM_SETFONT,(WPARAM)window_breakpoint->hFont,TRUE);

  /** Cration ComboBox Read Operator **/
  window_breakpoint->hwnd_combo_read_operator = CreateWindowEx(0,"COMBOBOX", 0, CBS_DROPDOWNLIST | WS_CHILD | WS_VISIBLE,
                                                               377+43,259,40,170,hwndBreakpoint,(HMENU)IDC_COMBOBOX_READ_OPERATOR,param->hInstance,0);
  SendMessage(window_breakpoint->hwnd_combo_read_operator,CB_ADDSTRING,0,(LPARAM)"=");
  SendMessage(window_breakpoint->hwnd_combo_read_operator,CB_ADDSTRING,0,(LPARAM)"!=");
  SendMessage(window_breakpoint->hwnd_combo_read_operator,CB_ADDSTRING,0,(LPARAM)">");
  SendMessage(window_breakpoint->hwnd_combo_read_operator,CB_ADDSTRING,0,(LPARAM)">=");
  SendMessage(window_breakpoint->hwnd_combo_read_operator,CB_ADDSTRING,0,(LPARAM)"<");
  SendMessage(window_breakpoint->hwnd_combo_read_operator,CB_ADDSTRING,0,(LPARAM)"<=");
  SendMessage(window_breakpoint->hwnd_combo_read_operator,WM_SETFONT,(WPARAM)window_breakpoint->hFont,TRUE);

  /** Cration ComboBox Write Operator **/
  window_breakpoint->hwnd_combo_write_operator = CreateWindowEx(0,"COMBOBOX", 0, CBS_DROPDOWNLIST | WS_CHILD | WS_VISIBLE,
                                                                377+43,290,40,170,hwndBreakpoint,(HMENU)IDC_COMBOBOX_WRITE_OPERATOR,param->hInstance,0);
  SendMessage(window_breakpoint->hwnd_combo_write_operator,CB_ADDSTRING,0,(LPARAM)"=");
  SendMessage(window_breakpoint->hwnd_combo_write_operator,CB_ADDSTRING,0,(LPARAM)"!=");
  SendMessage(window_breakpoint->hwnd_combo_write_operator,CB_ADDSTRING,0,(LPARAM)">");
  SendMessage(window_breakpoint->hwnd_combo_write_operator,CB_ADDSTRING,0,(LPARAM)">=");
  SendMessage(window_breakpoint->hwnd_combo_write_operator,CB_ADDSTRING,0,(LPARAM)"<");
  SendMessage(window_breakpoint->hwnd_combo_write_operator,CB_ADDSTRING,0,(LPARAM)"<=");
  SendMessage(window_breakpoint->hwnd_combo_write_operator,WM_SETFONT,(WPARAM)window_breakpoint->hFont,TRUE);

  /** Cration ComboBox Read|Write Operator **/
  window_breakpoint->hwnd_combo_readwrite_operator = CreateWindowEx(0,"COMBOBOX", 0, CBS_DROPDOWNLIST | WS_CHILD | WS_VISIBLE,
                                                                    377+43,290+31,40,170,hwndBreakpoint,(HMENU)IDC_COMBOBOX_READWRITE_OPERATOR,param->hInstance,0);
  SendMessage(window_breakpoint->hwnd_combo_readwrite_operator,CB_ADDSTRING,0,(LPARAM)"=");
  SendMessage(window_breakpoint->hwnd_combo_readwrite_operator,CB_ADDSTRING,0,(LPARAM)"!=");
  SendMessage(window_breakpoint->hwnd_combo_readwrite_operator,CB_ADDSTRING,0,(LPARAM)">");
  SendMessage(window_breakpoint->hwnd_combo_readwrite_operator,CB_ADDSTRING,0,(LPARAM)">=");
  SendMessage(window_breakpoint->hwnd_combo_readwrite_operator,CB_ADDSTRING,0,(LPARAM)"<");
  SendMessage(window_breakpoint->hwnd_combo_readwrite_operator,CB_ADDSTRING,0,(LPARAM)"<=");
  SendMessage(window_breakpoint->hwnd_combo_readwrite_operator,WM_SETFONT,(WPARAM)window_breakpoint->hFont,TRUE);

  /** Cration ComboBox Hold Operator **/
  window_breakpoint->hwnd_combo_hold_operator = CreateWindowEx(0,"COMBOBOX", 0, CBS_DROPDOWNLIST | WS_CHILD | WS_VISIBLE,
                                                               377+43,290+97,40,170,hwndBreakpoint,(HMENU)IDC_COMBOBOX_HOLD_OPERATOR,param->hInstance,0);
  SendMessage(window_breakpoint->hwnd_combo_hold_operator,CB_ADDSTRING,0,(LPARAM)"=");
  SendMessage(window_breakpoint->hwnd_combo_hold_operator,CB_ADDSTRING,0,(LPARAM)"!=");
  SendMessage(window_breakpoint->hwnd_combo_hold_operator,CB_ADDSTRING,0,(LPARAM)">");
  SendMessage(window_breakpoint->hwnd_combo_hold_operator,CB_ADDSTRING,0,(LPARAM)">=");
  SendMessage(window_breakpoint->hwnd_combo_hold_operator,CB_ADDSTRING,0,(LPARAM)"<");
  SendMessage(window_breakpoint->hwnd_combo_hold_operator,CB_ADDSTRING,0,(LPARAM)"<=");
  SendMessage(window_breakpoint->hwnd_combo_hold_operator,WM_SETFONT,(WPARAM)window_breakpoint->hFont,TRUE);

  /** Cration de la List View **/
  BuildBreakpointListView(&window_breakpoint->listview_breakpoint,window_breakpoint->hFont,IDC_LISTVIEW_BREAKPOINT,window_breakpoint);

  /** Reconstruit le tableau des Breakpoint + ListView + MAJ Stat / Display+Hide Buttons / Enable+Disable Buttons / Reset de la partie basse **/
  RefreshBreakpointWindow(window_breakpoint);

  /** Taille de la fentre au dmarrage ((avec la barre de titre et les bordures)) **/
  GetWindowRect(window_breakpoint->hwnd_breakpoint,&window_rect);
  window_breakpoint->window_init_width  = window_rect.right  - window_rect.left;
  window_breakpoint->window_init_height = window_rect.bottom - window_rect.top;

  /* Nombre de ligne dans la ListView au dmarrage de la fentre */
  window_breakpoint->nb_init_data_row = BREAKPOINT_LISTVIEW_NB_ROW;

  /* Nombre de lignes qu'il est possible d'afficher dans la ListView */
  window_breakpoint->nb_row = window_breakpoint->nb_init_data_row;

  /* OK */
  return(0);
}


/*************************************************************************************************/
/*  subEditBreakpointProcControl() :  Rcupre les vnements qui arrivent dans le Control Edit. */
/*************************************************************************************************/
LRESULT CALLBACK subEditBreakpointProcControl(HWND hwnd_control, UINT msg, WPARAM wParam, LPARAM lParam)
{
  int index;
  struct window_breakpoint *window_breakpoint;
  my_Memory(MEMORY_GET_WINDOW_BREAKPOINT,&window_breakpoint,NULL);

  /* Rien  faire */
  if(window_breakpoint == NULL)
    return(0);

  /** Identifie le Control **/
  /* Src */
  if(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_BANK) == hwnd_control)
    index = 1;
  else if(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_ADDRESS) == hwnd_control)
    index = 2;
  else if(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_BANK_DST) == hwnd_control)
    index = 3;
  else if(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_ADDRESS_DST) == hwnd_control)
    index = 4;
  else if(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_OPERATION_DATA) == hwnd_control)
    index = 5;
  else if(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_READ_VALUE) == hwnd_control)
    index = 6;
  else if(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_WRITE_VALUE) == hwnd_control)
    index = 7;
  else if(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_READWRITE_VALUE) == hwnd_control)
    index = 8;
  else if(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_HOLD_VALUE) == hwnd_control)
    index = 9;
  else if(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_PRINT_DATA) == hwnd_control)
    index = 10;
  else if(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_FILE_PATH) == hwnd_control)
    index = 11;
  else if(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_REPEAT_ONLY_DATA) == hwnd_control)
    index = 12;
  else
    return(0);      /* Pas trouv */

  /** Dtecte l'vnement du Return dans l'Edit **/
  if(msg == WM_CHAR && wParam == VK_RETURN)
    {
      /* Enlve le focus de l'Edit */
      SetFocus(window_breakpoint->hwnd_breakpoint);

      /* Valide l'opration saisie */
      if(window_breakpoint->source_radio == BREAKPOINT_SOURCE_OPERATION)
        ValidateBreakpointSourceOperation(window_breakpoint);
    }
  else if(index == 1)
    return(CallWindowProc(window_breakpoint->oldEditProcSrcBank,hwnd_control,msg,wParam,lParam)); 
  else if(index == 2)
    return(CallWindowProc(window_breakpoint->oldEditProcSrcAddress,hwnd_control,msg,wParam,lParam)); 
  else if(index == 3)
    return(CallWindowProc(window_breakpoint->oldEditProcDstBank,hwnd_control,msg,wParam,lParam)); 
  else if(index == 4)
    return(CallWindowProc(window_breakpoint->oldEditProcDstAddress,hwnd_control,msg,wParam,lParam)); 
  else if(index == 5)
    return(CallWindowProc(window_breakpoint->oldEditProcOperationData,hwnd_control,msg,wParam,lParam)); 
  else if(index == 6)
    return(CallWindowProc(window_breakpoint->oldEditProcReadValue,hwnd_control,msg,wParam,lParam)); 
  else if(index == 7)
    return(CallWindowProc(window_breakpoint->oldEditProcWriteValue,hwnd_control,msg,wParam,lParam)); 
  else if(index == 8)
    return(CallWindowProc(window_breakpoint->oldEditProcReadWriteValue,hwnd_control,msg,wParam,lParam)); 
  else if(index == 9)
    return(CallWindowProc(window_breakpoint->oldEditProcHoldValue,hwnd_control,msg,wParam,lParam)); 
  else if(index == 10)
    return(CallWindowProc(window_breakpoint->oldEditProcPrintData,hwnd_control,msg,wParam,lParam)); 
  else if(index == 11)
    return(CallWindowProc(window_breakpoint->oldEditProcFilePath,hwnd_control,msg,wParam,lParam)); 
  else if(index == 12)
    return(CallWindowProc(window_breakpoint->oldEditProcRepeatOnly,hwnd_control,msg,wParam,lParam)); 

  /* OK */
  return(0);
}


/**********************************************************************************/
/*  BuildBreakpointListView() :  Construction d'une ListView pour les Breakpoint. */
/**********************************************************************************/
static void BuildBreakpointListView(struct list_view *current_listview, HFONT hFont, int idc_listview, struct window_breakpoint *window_breakpoint)
{
  /** Init de la structure List_view **/
  current_listview->idc_listview = idc_listview;

  /** Liste des Colonnes **/
  current_listview->nb_column = 0;

  /* Col 0 : Status */
  current_listview->tab_column[0].type = INTERNALS_COLUMN_TYPE_TXT;
  current_listview->tab_column[0].align = LVCFMT_LEFT;
  current_listview->tab_column[0].length = 77;
  strcpy(current_listview->tab_column[0].title,"Status");
  current_listview->nb_column++;

  /* Col 1 : Source */
  current_listview->tab_column[1].type = INTERNALS_COLUMN_TYPE_TXT;
  current_listview->tab_column[1].align = LVCFMT_LEFT;
  current_listview->tab_column[1].length = 260;
  strcpy(current_listview->tab_column[1].title,"Source");
  current_listview->nb_column++;

  /* Col 2 : Trigger */
  current_listview->tab_column[2].type = INTERNALS_COLUMN_TYPE_TXT;
  current_listview->tab_column[2].align = LVCFMT_LEFT;
  current_listview->tab_column[2].length = 125;
  strcpy(current_listview->tab_column[2].title,"Trigger");
  current_listview->nb_column++;

  /* Col 3 : Action */
  current_listview->tab_column[3].type = INTERNALS_COLUMN_TYPE_TXT;
  current_listview->tab_column[3].align = LVCFMT_LEFT;
  current_listview->tab_column[3].length = 220;
  strcpy(current_listview->tab_column[3].title,"Action");
  current_listview->nb_column++;

  /* Col 4 : Repeat */
  current_listview->tab_column[4].type = INTERNALS_COLUMN_TYPE_TXT;
  current_listview->tab_column[4].align = LVCFMT_LEFT;
  current_listview->tab_column[4].length = 140;
  strcpy(current_listview->tab_column[4].title,"Repeat");
  current_listview->nb_column++;

  /* Col 5 : ID de la structure Breakpoint */
  current_listview->tab_column[5].type = INTERNALS_COLUMN_TYPE_TXT;
  current_listview->tab_column[5].align = LVCFMT_LEFT;
  current_listview->tab_column[5].length = 1;
  strcpy(current_listview->tab_column[5].title,"");
  current_listview->nb_column++;

  /** Cration du ListView **/
  BuildListView(window_breakpoint->hwnd_breakpoint,hFont,window_breakpoint->breakpoint_x0,window_breakpoint->breakpoint_y0,BREAKPOINT_LISTVIEW_NB_ROW,current_listview);
}


/********************************************************************************/
/*  FillBreakpointListView() :  Remplissage d'une ListView pour les Breakpoint. */
/********************************************************************************/
static void FillBreakpointListView(struct list_view *current_listview, struct breakpoint_table *current_breakpoint_table)
{
  int i, result;
  char *tab_operator[6] = {"=","!=",">",">=","<","<="};
  char *tab_register[] = {"A","X","Y","PC","STACK","DIRECT","DBR","PBR","STATUS",NULL};
  LVITEM listview_item;
  struct cyrene_breakpoint *current_breakpoint;
  char buffer[1024];

  /* Libration mmoire des donnes affiches dans les colonnes de la ListView */
  mem_free_listviewcolumn(current_listview);

  /* Aucun Breakpoint */
  if(current_breakpoint_table == NULL)
    return;
  if(current_breakpoint_table->nb_breakpoint == 0)
    return;

  /** Remplissage avec les valeurs dans la structure mmoire **/
  current_listview->nb_row = current_breakpoint_table->nb_breakpoint;
  for(i=0; i<current_listview->nb_column; i++)
    {
      /* Allocation des tableaux */
      current_listview->tab_column[i].tab_value_num = (DWORD *) calloc(current_listview->nb_row,sizeof(DWORD));
      current_listview->tab_column[i].tab_value_txt = (char **) calloc(current_listview->nb_row,sizeof(char *));
    }
  /*** Remplissage des tableaux ***/
  for(i=0; i<current_breakpoint_table->nb_breakpoint; i++)
    {
      current_breakpoint=current_breakpoint_table->tab_breakpoint[i];

      /** Colonne 0 : Status **/
      current_listview->tab_column[0].tab_value_num[i] = current_breakpoint->status;
      sprintf(buffer,"%s",(current_breakpoint->status == 0) ? "Disable" : "Enable");
      current_listview->tab_column[0].tab_value_txt[i] = strdup(buffer);

      /** Colonne 1 : Source **/
      current_listview->tab_column[1].tab_value_num[i] = i;
      if(current_breakpoint->source == BREAKPOINT_SOURCE_ADDRESS)
        {
          sprintf(buffer,"Address : %s/%s",current_breakpoint->source_address_bank,current_breakpoint->source_address_address);
          if(strlen(current_breakpoint->source_address_bank_dst) > 0)
            sprintf(&buffer[strlen(buffer)]," - %s/%s",current_breakpoint->source_address_bank_dst,current_breakpoint->source_address_address_dst);
        }
      else if(current_breakpoint->source == BREAKPOINT_SOURCE_SYMBOL)
        sprintf(buffer,"Symbol : %s [%s | %s]",current_breakpoint->source_symbol_name,current_breakpoint->source_symbol_segment,current_breakpoint->source_symbol_program);
      else if(current_breakpoint->source == BREAKPOINT_SOURCE_SYSTEMCALL)
        sprintf(buffer,"SystemCall : %s",current_breakpoint->source_systemcall_text);
      else if(current_breakpoint->source == BREAKPOINT_SOURCE_REGISTER)
        sprintf(buffer,"Register : %s",tab_register[current_breakpoint->source_register]);
      else if(current_breakpoint->source == BREAKPOINT_SOURCE_OPERATION)
        sprintf(buffer,"Operation : %s",current_breakpoint->source_operation_text);
      current_listview->tab_column[1].tab_value_txt[i] = strdup(buffer);

      /** Colonne 2 : Trigger **/
      current_listview->tab_column[2].tab_value_num[i] = i;
      strcpy(buffer,"");
      if(current_breakpoint->trigger == BREAKPOINT_TRIGGER_READ)
        {
          if(current_breakpoint->trigger_read_value_size == 0)
            sprintf(&buffer[strlen(buffer)],"Read");
          else if(current_breakpoint->trigger_read_value_size == 1)
            sprintf(&buffer[strlen(buffer)],"Read [%s %02X]",tab_operator[current_breakpoint->trigger_read_operator],current_breakpoint->trigger_read_value);
          else if(current_breakpoint->trigger_read_value_size == 2)
            sprintf(&buffer[strlen(buffer)],"Read [%s %04X]",tab_operator[current_breakpoint->trigger_read_operator],current_breakpoint->trigger_read_value);
        }
      else if(current_breakpoint->trigger == BREAKPOINT_TRIGGER_WRITE)
        {
          if(current_breakpoint->trigger_write_value_size == 0)
            sprintf(&buffer[strlen(buffer)],"Write");
          else if(current_breakpoint->trigger_write_value_size == 1)
            sprintf(&buffer[strlen(buffer)],"Write [%s %02X]",tab_operator[current_breakpoint->trigger_write_operator],current_breakpoint->trigger_write_value);
          else if(current_breakpoint->trigger_write_value_size == 2)
            sprintf(&buffer[strlen(buffer)],"Write [%s %04X]",tab_operator[current_breakpoint->trigger_write_operator],current_breakpoint->trigger_write_value);
        }
      else if(current_breakpoint->trigger == BREAKPOINT_TRIGGER_READWRITE)
        {
          if(current_breakpoint->trigger_readwrite_value_size == 0)
            sprintf(&buffer[strlen(buffer)],"Read|Write");
          else if(current_breakpoint->trigger_readwrite_value_size == 1)
            sprintf(&buffer[strlen(buffer)],"Read|Write [%s %02X]",tab_operator[current_breakpoint->trigger_readwrite_operator],current_breakpoint->trigger_readwrite_value);
          else if(current_breakpoint->trigger_readwrite_value_size == 2)
            sprintf(&buffer[strlen(buffer)],"Read|Write [%s %04X]",tab_operator[current_breakpoint->trigger_readwrite_operator],current_breakpoint->trigger_readwrite_value);
        }
      else if(current_breakpoint->trigger == BREAKPOINT_TRIGGER_EXECUTE)
        sprintf(&buffer[strlen(buffer)],"Execute");
      else if(current_breakpoint->trigger == BREAKPOINT_TRIGGER_HOLD)
        {
          if(current_breakpoint->trigger_hold_value_size == 0)
            sprintf(&buffer[strlen(buffer)],"Hold");
          else if(current_breakpoint->trigger_hold_value_size == 1)
            sprintf(&buffer[strlen(buffer)],"Hold [%s %02X]",tab_operator[current_breakpoint->trigger_hold_operator],current_breakpoint->trigger_hold_value);
          else if(current_breakpoint->trigger_hold_value_size == 2)
            sprintf(&buffer[strlen(buffer)],"Hold [%s %04X]",tab_operator[current_breakpoint->trigger_hold_operator],current_breakpoint->trigger_hold_value);
        }
      current_listview->tab_column[2].tab_value_txt[i] = strdup(buffer);

      /** Colonne 3 : Action **/
      current_listview->tab_column[3].tab_value_num[i] = i;
      if(current_breakpoint->action == BREAKPOINT_ACTION_STOP)
        strcpy(buffer,"Action : Stop");
      else if(current_breakpoint->action == BREAKPOINT_ACTION_PRINT)
        sprintf(buffer,"Action : Print '%s'",current_breakpoint->action_print_text);
      else if(current_breakpoint->action == BREAKPOINT_ACTION_FILE)
        sprintf(buffer,"Action : File '%s'",current_breakpoint->action_print_text);
      current_listview->tab_column[3].tab_value_txt[i] = strdup(buffer);

      /** Colonne 4 : Repeat **/
      current_listview->tab_column[4].tab_value_num[i] = i;
      if(current_breakpoint->repeat == BREAKPOINT_REPEAT_ALWAYS)
        sprintf(buffer,"Repeat : Always");
      else if(current_breakpoint->repeat == BREAKPOINT_REPEAT_ONLY)
        sprintf(buffer,"Repeat : Only %d times",current_breakpoint->repeat_only_times);
      else if(current_breakpoint->repeat == BREAKPOINT_REPEAT_AFTER)
        sprintf(buffer,"Repeat : After %d times",current_breakpoint->repeat_only_times);
      current_listview->tab_column[4].tab_value_txt[i] = strdup(buffer);

      /** Colonne 5 : ID de la structure Breakpoint [0-N] **/
      current_listview->tab_column[5].tab_value_num[i] = i;
      sprintf(buffer,"%d",i);
      current_listview->tab_column[5].tab_value_txt[i] = strdup(buffer);
    }

  /** Transfert des donnes dans la List View via un Msg LVN_GETDISPINFO **/
  /** Ajout des Item dans le ListView **/
  listview_item.pszText   = LPSTR_TEXTCALLBACK;           /* Sends an LVN_GETDISPINFO message to fill the item */
  listview_item.mask      = LVIF_TEXT | LVIF_IMAGE | LVIF_STATE | LVIF_PARAM;
  listview_item.stateMask = 0;
  listview_item.iSubItem  = 0;
  listview_item.state     = 0;

  /** Initialize LVITEM members that are different for each item **/
  for(i=0; i<current_listview->nb_row; i++)
    {
      listview_item.iItem  = i;
      listview_item.iImage = -1;         /* Identifiant de l'image dans la liste */
      listview_item.lParam = i;          /* Identifiant renvoy lors du tri */
      result = ListView_InsertItem(current_listview->hwnd_listview,&listview_item);
    }
}


/************************************************************/
/*  CreateBreakpoint() :  Cration d'un nouveau Breakpoint. */
/************************************************************/
static void CreateBreakpoint(struct window_breakpoint *window_breakpoint)
{
  /* On est en Edit Mode */
  window_breakpoint->edit_mode= 1;

  /** RAZ + Init **/
  RAZBreakpoint(window_breakpoint);
  SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_ADDRESS,BM_SETCHECK,1,0);
  SendMessage(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_READ),BM_SETCHECK,1,0);
  SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_STOP_EXECUTION,BM_SETCHECK,1,0);
  SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_REPEAT_ALWAYS,BM_SETCHECK,1,0);

  /** Display/Hide Buttons **/
  DisplayHideBreakpointButton(window_breakpoint,0);

  /** Les controles sont modifiables **/
  UpdateBreakpointEditMode(window_breakpoint,1);

  /* Pas de Breakpoint slectionn */
  window_breakpoint->selected_breakpoint = NULL;
}


/******************************************************************/
/*  ModifyBreakpoint() :  Modification du Breakpoint slectionn. */
/******************************************************************/
static void ModifyBreakpoint(struct window_breakpoint *window_breakpoint)
{
  /* Rien n'est slectionn */
  if(window_breakpoint->selected_breakpoint == NULL)
    return;

  /* On est en Edit Mode */
  window_breakpoint->edit_mode= 1;

  /** Display/Hide Buttons **/
  DisplayHideBreakpointButton(window_breakpoint,0);

  /** Les controles sont modifiables **/
  UpdateBreakpointEditMode(window_breakpoint,1);
}


/*****************************************************************/
/*  DeleteBreakpoint() :  Suppression du Breakpoint slectionn. */
/*****************************************************************/
static void DeleteBreakpoint(struct window_breakpoint *window_breakpoint)
{
  struct cyrene_breakpoint *current_breakpoint;
  struct window_main *window_main;

  /* Rcupre la fentre Main */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  if(window_main == NULL)
    return;

  /* Rien de slectionn */
  if(window_breakpoint->selected_breakpoint == NULL)
    return;

  /** Supprime le Breakpoint **/
  if(window_main->current_breakpoint_table->first_breakpoint == window_breakpoint->selected_breakpoint &&
    window_main->current_breakpoint_table->nb_breakpoint == 1)
    {
      /* Breakpoint unique */
      mem_free_breakpoint(window_main->current_breakpoint_table->first_breakpoint);
      window_main->current_breakpoint_table->first_breakpoint = NULL;
      window_main->current_breakpoint_table->last_breakpoint = NULL;
      window_main->current_breakpoint_table->nb_breakpoint = 0;
    }
  else if(window_main->current_breakpoint_table->first_breakpoint == window_breakpoint->selected_breakpoint)
    {
      /* Premier Breakpoint */
      window_main->current_breakpoint_table->first_breakpoint = window_breakpoint->selected_breakpoint->next;
      window_main->current_breakpoint_table->nb_breakpoint--;
      mem_free_breakpoint(window_breakpoint->selected_breakpoint);
    }
  else if(window_main->current_breakpoint_table->last_breakpoint == window_breakpoint->selected_breakpoint)
    {
      /* Dernier Breakpoint */
      for(current_breakpoint=window_main->current_breakpoint_table->first_breakpoint; current_breakpoint; current_breakpoint=current_breakpoint->next)
        if(current_breakpoint->next == window_main->current_breakpoint_table->last_breakpoint)
          {
            window_main->current_breakpoint_table->last_breakpoint = current_breakpoint;
            current_breakpoint->next = NULL;
            break;
          }
      window_main->current_breakpoint_table->nb_breakpoint--;
      mem_free_breakpoint(window_breakpoint->selected_breakpoint);
    }
  else
    {
      /* Au milieu */
      for(current_breakpoint=window_main->current_breakpoint_table->first_breakpoint; current_breakpoint; current_breakpoint=current_breakpoint->next)
        if(current_breakpoint->next == window_breakpoint->selected_breakpoint)
          {
            current_breakpoint->next = current_breakpoint->next->next;
            break;
          }
      window_main->current_breakpoint_table->nb_breakpoint--;
      mem_free_breakpoint(window_breakpoint->selected_breakpoint);
    }

  /** Reconstruit le ListView + MAJ des Stat **/
  RebuildListViewContent(window_breakpoint);

  /** On efface les donnes contextuelles du Breakpoint **/
  UpdateBreakpointExtraInformation(window_breakpoint,NULL);

  /** On met  jour la zone basse **/
  RAZBreakpoint(window_breakpoint);
  UpdateBreakpointEditMode(window_breakpoint,0);

  /** Enable/Disable Buttons **/
  EnableDisableBreakpointButton(window_breakpoint,0,0);
}


/************************************************************/
/*  ClearBreakpointTable() :  Vide la table des Breakpoint. */
/************************************************************/
void ClearBreakpointTable(struct window_breakpoint *window_breakpoint)
{
  struct window_main *window_main;

  /* Rcupre la fentre Main */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  if(window_main == NULL)
    return;

  /* Rien  faire */
  if(window_main->current_breakpoint_table == NULL)
    return;

  /* Libration mmoire */
  mem_free_breakpoint_table(window_main->current_breakpoint_table);
  window_main->current_breakpoint_table = NULL;

  /** Reconstruit le ListView + MAJ des Stat **/
  RebuildListViewContent(window_breakpoint);

  /** On efface les donnes contextuelles du Breakpoint **/
  UpdateBreakpointExtraInformation(window_breakpoint,NULL);

  /** On met  jour la zone basse **/
  RAZBreakpoint(window_breakpoint);
  UpdateBreakpointEditMode(window_breakpoint,0);

  /** Enable / Disable Buttons **/
  EnableDisableBreakpointButton(window_breakpoint,0,0);
}


/***************************************************************/
/*  RefreshBreakpointWindow() :  MAJ de la fentre Breakpoint. */
/***************************************************************/
void RefreshBreakpointWindow(struct window_breakpoint *window_breakpoint)
{
  struct window_main *window_main;

  /* Rcupre la fentre Main */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  if(window_main == NULL)
    return;

  /** Reconstruit le tableau des Breakpoint + ListView + MAJ Stat **/
  RebuildListViewContent(window_breakpoint);

  /** Display/Hide Buttons **/
  DisplayHideBreakpointButton(window_breakpoint,1);

  /** Enable/Disable Buttons **/
  EnableDisableBreakpointButton(window_breakpoint,0,0);

  /** Reset de la partie basse **/
  RAZBreakpoint(window_breakpoint);
  UpdateBreakpointEditMode(window_breakpoint,0);

  /* MAJ Status Bar */
  UpdateWindowMainStatusBar();
}


/****************************************************************/
/*  EnableBreakpoint() :  Activation du Breakpoint slectionn. */
/****************************************************************/
static void EnableBreakpoint(struct window_breakpoint *window_breakpoint)
{
  /* Rien de slectionn */
  if(window_breakpoint->selected_breakpoint == NULL)
    return;

  /* Active le Breakpoint */
  window_breakpoint->selected_breakpoint->status = 1;

  /** Reconstruit le ListView + MAJ des Stat **/
  RebuildListViewContent(window_breakpoint);

  /** On efface les donnes contextuelles du Breakpoint **/
  UpdateBreakpointExtraInformation(window_breakpoint,NULL);

  /** On met  jour la zone basse **/
  RAZBreakpoint(window_breakpoint);
  UpdateBreakpointEditMode(window_breakpoint,0);

  /** Enable/Disable Buttons **/
  EnableDisableBreakpointButton(window_breakpoint,0,0);
}


/********************************************************************/
/*  DisableBreakpoint() :  DsaAtivation du Breakpoint slectionn. */
/********************************************************************/
static void DisableBreakpoint(struct window_breakpoint *window_breakpoint)
{
  /* Rien de slectionn */
  if(window_breakpoint->selected_breakpoint == NULL)
    return;

  /* Dsactive le Breakpoint */
  window_breakpoint->selected_breakpoint->status = 0;

  /** Reconstruit le ListView + MAJ des Stat **/
  RebuildListViewContent(window_breakpoint);

  /** On efface les donnes contextuelles du Breakpoint **/
  UpdateBreakpointExtraInformation(window_breakpoint,NULL);

  /** On met  jour la zone basse **/
  RAZBreakpoint(window_breakpoint);
  UpdateBreakpointEditMode(window_breakpoint,0);

  /** Enable/Disable Buttons **/
  EnableDisableBreakpointButton(window_breakpoint,0,0);
}


/************************************************************************/
/*  SaveBreakpointFile() :  Sauvegarde des Breakpoints dans un fichier. */
/************************************************************************/
static void SaveBreakpointFile(struct window_breakpoint *window_breakpoint)
{
  BOOL is_OK;
  char file_path[1024];
  char breakpoint_line_txt[1024];
  FILE *fd_breakpoint;
  struct cyrene_breakpoint *current_breakpoint;
  struct window_main *window_main;

  /* Rcupre la fentre Main */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  if(window_main == NULL)
    return;

  /** Demande le nom du fichier **/
  sprintf(file_path,"Breakpoint_List.txt");
  is_OK = GetSaveFilePath(window_breakpoint->hwnd_breakpoint,"Select Breakpoint file","All Files (*.txt)\0*.txt\0\0",file_path);
  if(is_OK = FALSE)
    return;

  /* Cration du fichier sur disque */
  fd_breakpoint = fopen(file_path,"w");
  if(fd_breakpoint == NULL)
    {
      MessageBox(window_breakpoint->hwnd_breakpoint,"Impossible to create File","Error",MB_OK|MB_ICONWARNING);
      return;
    }

  /*** Passe en revue tous les breakpoints ***/
  if(window_main->current_breakpoint_table != NULL)
    for(current_breakpoint = window_main->current_breakpoint_table->first_breakpoint; current_breakpoint; current_breakpoint=current_breakpoint->next)
      {
        /** On construit la ligne du Breakpoint au format Texte **/
        BuildBreakpointTextLine(current_breakpoint,BREAKPOINT_LINE_TARGET_FILE,breakpoint_line_txt);

        /* Ajoute la ligne dans le fichier */
        fprintf(fd_breakpoint,breakpoint_line_txt);
      }

  /* Fermeture du fichier */
  fclose(fd_breakpoint);
}


/*************************************************************************/
/*  BuildBreakpointTextLine() :  Construit une ligne Breapoint en texte. */
/*************************************************************************/
void BuildBreakpointTextLine(struct cyrene_breakpoint *current_breakpoint, int target, char *breakpoint_line_rtn)
{
  char buffer_output_line[BUFFER_SIZE];
  char buffer_trigger[256];
  char *tab_operator[6] = {"=","!=",">",">=","<","<="};
  char *tab_register[] = {"A","X","Y","PC","STACK","DIRECT","DBR","PBR","STATUS",NULL};

  /* Init */
  strcpy(breakpoint_line_rtn,"");

  /** Status ou ID (pour KEGS) **/
  if(target == BREAKPOINT_LINE_TARGET_KEGS)
    sprintf(&breakpoint_line_rtn[strlen(breakpoint_line_rtn)],"%d;",current_breakpoint->id);
  else if(target == BREAKPOINT_LINE_TARGET_FILE)
    strcpy(&breakpoint_line_rtn[strlen(breakpoint_line_rtn)],(current_breakpoint->status == 0) ? "Disable;" : "Enable;");
  else if(target == BREAKPOINT_LINE_TARGET_STATUS)
    sprintf(&breakpoint_line_rtn[strlen(breakpoint_line_rtn)],"/!\\ Breakpoint #%d;",current_breakpoint->id);

  /** Source **/
  if(current_breakpoint->source == BREAKPOINT_SOURCE_ADDRESS)
    {
      if(strlen(current_breakpoint->source_address_bank_dst) == 0)
        sprintf(&breakpoint_line_rtn[strlen(breakpoint_line_rtn)],"Address=%s/%s;",current_breakpoint->source_address_bank,current_breakpoint->source_address_address);
      else
        sprintf(&breakpoint_line_rtn[strlen(breakpoint_line_rtn)],"Address=%s/%s-%s/%s;",current_breakpoint->source_address_bank,current_breakpoint->source_address_address,
                                                                                         current_breakpoint->source_address_bank_dst,current_breakpoint->source_address_address_dst);
    }
  else if(current_breakpoint->source == BREAKPOINT_SOURCE_SYMBOL)
    {
      /* Si la cible est KEGS, on ne peut pas encoder les symboles donc on passe en adresse */
      if(target == BREAKPOINT_LINE_TARGET_KEGS)
        {
          if(current_breakpoint->source_symbol_end_address == 0xFFFFFFFF)
            sprintf(&breakpoint_line_rtn[strlen(breakpoint_line_rtn)],"Address=%02X/%04X;",(current_breakpoint->source_symbol_begin_address>>16) & 0xFF,current_breakpoint->source_symbol_begin_address & 0xFFFF);
          else
            sprintf(&breakpoint_line_rtn[strlen(breakpoint_line_rtn)],"Address=%02X/%04X-%02X/%04X;",(current_breakpoint->source_symbol_begin_address>>16) & 0xFF,current_breakpoint->source_symbol_begin_address & 0xFFFF,
                                                                                                     (current_breakpoint->source_symbol_end_address>>16) & 0xFF,current_breakpoint->source_symbol_end_address & 0xFFFF);
        }
      else
        sprintf(&breakpoint_line_rtn[strlen(breakpoint_line_rtn)],"Symbol=%s|%s|%s;",current_breakpoint->source_symbol_program,current_breakpoint->source_symbol_segment,current_breakpoint->source_symbol_name);
    }
  else if(current_breakpoint->source == BREAKPOINT_SOURCE_SYSTEMCALL)
    {
      if(current_breakpoint->source_systemcall_type == TYPE_SYSTEM_CALL_P8)
        sprintf(&breakpoint_line_rtn[strlen(breakpoint_line_rtn)],"SystemCall=P8-%02X;",current_breakpoint->source_systemcall_number & 0x00FF);
      else if(current_breakpoint->source_systemcall_type == TYPE_SYSTEM_CALL_P16)
        sprintf(&breakpoint_line_rtn[strlen(breakpoint_line_rtn)],"SystemCall=P16-%04X;",current_breakpoint->source_systemcall_number);
      else if(current_breakpoint->source_systemcall_type == TYPE_SYSTEM_CALL_GSOS)
        sprintf(&breakpoint_line_rtn[strlen(breakpoint_line_rtn)],"SystemCall=GSOS-%04X;",current_breakpoint->source_systemcall_number);
      else if(current_breakpoint->source_systemcall_type == TYPE_SYSTEM_CALL_TOOLBOX)
        sprintf(&breakpoint_line_rtn[strlen(breakpoint_line_rtn)],"SystemCall=TOOL-%04X;",current_breakpoint->source_systemcall_number);
    }
  else if(current_breakpoint->source == BREAKPOINT_SOURCE_REGISTER)
    sprintf(&breakpoint_line_rtn[strlen(breakpoint_line_rtn)],"Register=%s;",tab_register[current_breakpoint->source_register]);
  else if(current_breakpoint->source == BREAKPOINT_SOURCE_OPERATION)
    {
      /* Si la cible est KEGS, on stocke directement les bytes de l'opration (avec un leading byte codant le nb de bytes significatifs) */
      if(target == BREAKPOINT_LINE_TARGET_KEGS)
        sprintf(&breakpoint_line_rtn[strlen(breakpoint_line_rtn)],"Operation=%02X%02X%02X%02X%02X;",current_breakpoint->source_operation_data_nb,
                                                                                                    current_breakpoint->source_operation_data[0],
                                                                                                    current_breakpoint->source_operation_data[1],
                                                                                                    current_breakpoint->source_operation_data[2],
                                                                                                    current_breakpoint->source_operation_data[3]);
      else
        sprintf(&breakpoint_line_rtn[strlen(breakpoint_line_rtn)],"Operation=%s;",current_breakpoint->source_operation_text);
    }

  /** Trigger **/
  strcpy(buffer_trigger,"");
  /* Read */
  if(current_breakpoint->trigger == BREAKPOINT_TRIGGER_READ)
    {
      if(current_breakpoint->trigger_read_value_size == 0)
        sprintf(buffer_trigger,"Read");
      else if(current_breakpoint->trigger_read_value_size == 1)
        sprintf(buffer_trigger,"Read:%s %02X",tab_operator[current_breakpoint->trigger_read_operator],current_breakpoint->trigger_read_value);
      else if(current_breakpoint->trigger_read_value_size == 2)
        sprintf(buffer_trigger,"Read:%s %04X",tab_operator[current_breakpoint->trigger_read_operator],current_breakpoint->trigger_read_value);
    }
  /* Write */
  else if(current_breakpoint->trigger == BREAKPOINT_TRIGGER_WRITE)
    {
      if(current_breakpoint->trigger_write_value_size == 0)
        sprintf(buffer_trigger,"Write");
      else if(current_breakpoint->trigger_write_value_size == 1)
        sprintf(buffer_trigger,"Write:%s %02X",tab_operator[current_breakpoint->trigger_write_operator],current_breakpoint->trigger_write_value);
      else if(current_breakpoint->trigger_write_value_size == 2)
        sprintf(buffer_trigger,"Write:%s %04X",tab_operator[current_breakpoint->trigger_write_operator],current_breakpoint->trigger_write_value);
    }
  /* Read|Write */
  else if(current_breakpoint->trigger == BREAKPOINT_TRIGGER_READWRITE)
    {
      if(current_breakpoint->trigger_readwrite_value_size == 0)
        sprintf(buffer_trigger,"Read|Write");
      else if(current_breakpoint->trigger_readwrite_value_size == 1)
        sprintf(buffer_trigger,"Read|Write:%s %02X",tab_operator[current_breakpoint->trigger_readwrite_operator],current_breakpoint->trigger_readwrite_value);
      else if(current_breakpoint->trigger_readwrite_value_size == 2)
        sprintf(buffer_trigger,"Read|Write:%s %04X",tab_operator[current_breakpoint->trigger_readwrite_operator],current_breakpoint->trigger_readwrite_value);
    }
  /* Execute */
  else if(current_breakpoint->trigger == BREAKPOINT_TRIGGER_EXECUTE)
    sprintf(&buffer_trigger[strlen(buffer_trigger)],"Execute");
  /* Hold */
  else if(current_breakpoint->trigger == BREAKPOINT_TRIGGER_HOLD)
    {
      if(current_breakpoint->trigger_hold_value_size == 0)
        sprintf(buffer_trigger,"Hold");
      else if(current_breakpoint->trigger_hold_value_size == 1)
        sprintf(buffer_trigger,"Hold:%s %02X",tab_operator[current_breakpoint->trigger_hold_operator],current_breakpoint->trigger_hold_value);
      else if(current_breakpoint->trigger_hold_value_size == 2)
        sprintf(buffer_trigger,"Hold:%s %04X",tab_operator[current_breakpoint->trigger_hold_operator],current_breakpoint->trigger_hold_value);
    }
  /* Stocke le buffer */
  sprintf(&breakpoint_line_rtn[strlen(breakpoint_line_rtn)],"%s;",buffer_trigger);

  /** Action **/
  if(current_breakpoint->action == BREAKPOINT_ACTION_STOP)
    {
      if(target == BREAKPOINT_LINE_TARGET_KEGS || target == BREAKPOINT_LINE_TARGET_FILE)
        sprintf(&breakpoint_line_rtn[strlen(breakpoint_line_rtn)],"Action:Stop;");
    }
  else if(current_breakpoint->action == BREAKPOINT_ACTION_PRINT)
    {
      if(target == BREAKPOINT_LINE_TARGET_KEGS)
        {
          /* Remplace les adresses des Symbols dans la ligne output */
          BuildOutputLine(current_breakpoint->action_print_text,buffer_output_line);
          sprintf(&breakpoint_line_rtn[strlen(breakpoint_line_rtn)],"Action:Print'%s';",buffer_output_line);
        }
      else if(target == BREAKPOINT_LINE_TARGET_FILE)
        sprintf(&breakpoint_line_rtn[strlen(breakpoint_line_rtn)],"Action:Print'%s';",current_breakpoint->action_print_text);
    }
  else if(current_breakpoint->action == BREAKPOINT_ACTION_FILE)
    {
      if(target == BREAKPOINT_LINE_TARGET_KEGS)
        {
          /* Remplace les adresses des Symbols dans la ligne output */
          BuildOutputLine(current_breakpoint->action_print_text,buffer_output_line);
          sprintf(&breakpoint_line_rtn[strlen(breakpoint_line_rtn)],"Action:File'%s','%s';",buffer_output_line,current_breakpoint->action_file_path);
        }
      else if(target == BREAKPOINT_LINE_TARGET_FILE)
        sprintf(&breakpoint_line_rtn[strlen(breakpoint_line_rtn)],"Action:File'%s','%s';",current_breakpoint->action_print_text,current_breakpoint->action_file_path);
    }

  /** Repeat **/
  if(target == BREAKPOINT_LINE_TARGET_KEGS || target == BREAKPOINT_LINE_TARGET_FILE)
    {
      if(current_breakpoint->repeat == BREAKPOINT_REPEAT_ALWAYS)
        sprintf(&breakpoint_line_rtn[strlen(breakpoint_line_rtn)],"Repeat:Always");
      else if(current_breakpoint->repeat == BREAKPOINT_REPEAT_ONLY)
        sprintf(&breakpoint_line_rtn[strlen(breakpoint_line_rtn)],"Repeat:Only%d",current_breakpoint->repeat_only_times);
      else if(current_breakpoint->repeat == BREAKPOINT_REPEAT_AFTER)
        sprintf(&breakpoint_line_rtn[strlen(breakpoint_line_rtn)],"Repeat:After%d",current_breakpoint->repeat_only_times);
    }

  /* Fin de ligne */
  if(target == BREAKPOINT_LINE_TARGET_KEGS || target == BREAKPOINT_LINE_TARGET_FILE)
    sprintf(&breakpoint_line_rtn[strlen(breakpoint_line_rtn)],"\n");
}


/****************************************************************************************/
/*  BuildOutputLine() :  On va remplacer les adresses des Symbols dans la ligne output. */
/****************************************************************************************/
static void BuildOutputLine(char *action_text, char *output_line_rtn)
{
  int i, j, length;
  char *end_sep;
  char buffer_keyword_name[BUFFER_SIZE];
  char buffer_keyword_address[BUFFER_SIZE];

  /* Init */
  strcpy(output_line_rtn,action_text);

  /** Passe en revue les Keywords de la ligne output **/
  for(i=0,j=0; i<strlen(action_text); i++)
    {
      if(action_text[i] == '[' && action_text[i+1] == '@')
        {
          /* Isole le Keyword */
          end_sep = strchr(&action_text[i],']');
          if(end_sep == NULL)
            {
              strcpy(output_line_rtn,action_text);
              return;
            }
          length = (int) (end_sep - &action_text[i]) + 1;
          memcpy(buffer_keyword_name,&action_text[i],length);
          buffer_keyword_name[length] = '\0';

          /* Adresse ou Symbol */
          BuildSymbolKeywordAddress(buffer_keyword_name,buffer_keyword_address);

          /* Remplace le Keyword */
          strcpy(&output_line_rtn[j],buffer_keyword_address);
          j += (int) strlen(buffer_keyword_address);
          i += (int) strlen(buffer_keyword_name) - 1;
        }
      else
        output_line_rtn[j++] = action_text[i];
    }
  output_line_rtn[j] = '\0';
}


/***********************************************************************/
/*  BuildSymbolKeywordAddress() :  Remplace un Symbol par son adresse. */
/***********************************************************************/
static void BuildSymbolKeywordAddress(char *symbol_name, char *symbol_address_rtn)
{
  int length;
  char *end_sep;
  char *begin_name;
  char *begin_segment;
  char *begin_program;
  struct symbol *current_symbol;
  struct window_main *window_main;
  char buffer_name[BUFFER_SIZE];
  char buffer_segment[BUFFER_SIZE];
  char buffer_program[BUFFER_SIZE];

  /* Init */
  strcpy(symbol_address_rtn,symbol_name);

  /** Rcupre la fentre Main **/
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  if(window_main == NULL)
    return;
  if(window_main->current_breakpoint_table == NULL)
    return;
  if(window_main->current_symbol_table == NULL)
    return;

  /** Extrait Name|Segment|Program **/
  /* Name */
  begin_name = symbol_name + 2;
  end_sep = strchr(begin_name,'|');
  if(end_sep == NULL)
    return;
  length = (int) (end_sep - begin_name);
  memcpy(buffer_name,begin_name,length);
  buffer_name[length] = '\0';

  /* Segment */
  begin_segment = end_sep + 1;
  end_sep = strchr(begin_segment,'|');
  if(end_sep == NULL)
    return;
  length = (int) (end_sep - begin_segment);
  memcpy(buffer_segment,begin_segment,length);
  buffer_segment[length] = '\0';

  /* Program */
  begin_program = end_sep + 1;
  end_sep = strchr(begin_program,']');
  if(end_sep == NULL)
    return;
  length = (int) (end_sep - begin_program);
  memcpy(buffer_program,begin_program,length);
  buffer_program[length] = '\0';

  /** Recherche le Symbol **/
  for(current_symbol=window_main->current_symbol_table->first_symbol; current_symbol; current_symbol=current_symbol->next)
    if(!strcmp(current_symbol->name,buffer_name) &&                 /* Le Symbol est case sensitive */
       !stricmp(current_symbol->segment_name,buffer_segment) &&
       !stricmp(current_symbol->program_name,buffer_program))
      {
        sprintf(symbol_address_rtn,"[@%06X]",current_symbol->symbol_relocated_address);
        return;
      }
}


/************************************************************************************************/
/*  LoadAppendBreakpointFile() : Charge en mmoire un fichier Breakpoint et Ajoute les entres. */
/************************************************************************************************/
void LoadAppendBreakpointFile(char *breakpoint_file_path, struct window_breakpoint *window_breakpoint)
{
  int is_duplicate;
  struct cyrene_breakpoint *new_breakpoint_list;
  struct cyrene_breakpoint *current_breakpoint;
  struct cyrene_breakpoint *next_breakpoint;
  char file_path[BUFFER_SIZE];
  struct window_main *window_main;

  /* Rcupre la fentre Main */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  if(window_main == NULL)
    return;

  /* Si besoin, on alloue la structure breakpoint_table */
  if(window_main->current_breakpoint_table == NULL)
    {
      window_main->current_breakpoint_table = (struct breakpoint_table *) calloc(1,sizeof(struct breakpoint_table));
      if(window_main->current_breakpoint_table == NULL)
        return;
    }

  /** Demande le chemin du fichier **/
  if(breakpoint_file_path == NULL)
    {
      GetOneFilePath((window_breakpoint==NULL)?window_main->hwnd_main:window_breakpoint->hwnd_breakpoint,"Select Breakpoint file","Breakpoint Files (*.txt)\0*.txt\0\0","",file_path);
      if(strlen(file_path) == 0)
        return;
    }
  else
    strcpy(file_path,breakpoint_file_path);

  /* Charge en mmoire le fichier (Breakpoint + Reloc) */
  new_breakpoint_list = LoadBreakpointFile(file_path);

  /*** Ajoute les Breakpoint  la liste ***/
  for(current_breakpoint = new_breakpoint_list; current_breakpoint; current_breakpoint = next_breakpoint)
    {
      /* Isole le Breakpoint */
      next_breakpoint = current_breakpoint->next;
      current_breakpoint->next = NULL;

      /* Ajoute dans la liste chaine */
      is_duplicate = InsertUniqueBreakpoint(current_breakpoint,window_main->current_breakpoint_table);

      /* Supprime les doublons */
      if(is_duplicate == 1)
        mem_free_breakpoint(current_breakpoint);
    }

  /** Reconstruit le tableau des Breakpoint + ListView + MAJ Stat **/
  if(window_breakpoint != NULL)
    RebuildListViewContent(window_breakpoint);
}


/****************************************************************************************/
/*  CancelBreakpointOperation() :  Abandonne l'opration de modification du breakpoint. */
/****************************************************************************************/
static void CancelBreakpointOperation(struct window_breakpoint *window_breakpoint)
{
  /** Display/Hide Buttons **/
  DisplayHideBreakpointButton(window_breakpoint,1);

  /** Enabel/Disable Buttons **/
  EnableDisableBreakpointButton(window_breakpoint,0,0);

  /** Reset de la partie basse **/
  RAZBreakpoint(window_breakpoint);
  UpdateBreakpointEditMode(window_breakpoint,0);

  /* On est en Read Mode */
  window_breakpoint->edit_mode = 0;
}


/************************************************************************************/
/*  ValidateBreakpointUpdate() :  Valide l'opration de modification du breakpoint. */
/************************************************************************************/
static void ValidateBreakpointUpdate(struct window_breakpoint *window_breakpoint)
{
  int i, length, value, has_error, is_odd, is_same;
  char buffer[1024];
  char buffer_dst[1024];
  struct cyrene_breakpoint *current_breakpoint;
  struct cyrene_breakpoint *one_breakpoint;
  struct window_main *window_main;
  char *tab_register[] = {"A","X","Y","PC","STACK","DIRECT","DBR","PBR","STATUS",NULL};
  char *tab_operator[] = {"=","!=",">",">=","<","<=",NULL};

  /* Rcupre la fentre Main */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  if(window_main == NULL)
    return;

  /** Source **/
  if(window_breakpoint->source_radio == BREAKPOINT_SOURCE_ADDRESS)
    {
      /* Bank */
      length = GetDlgItemText(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_BANK,buffer,1024);
      has_error = CheckBreakpointHexaValue(buffer,2);
      if(has_error)
        {
          MessageBox(window_breakpoint->hwnd_breakpoint,"You have to provide a valid Bank value","Wrong Breakpoint Data",MB_OK|MB_ICONWARNING);
          return;
        }
      strcpy(window_breakpoint->source_address_bank,buffer);

      /* Address */
      length = GetDlgItemText(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_ADDRESS,buffer,1024);
      has_error = CheckBreakpointHexaValue(buffer,4);
      if(has_error)
        {
          MessageBox(window_breakpoint->hwnd_breakpoint,"You have to provide a valid Address value","Wrong Breakpoint Data",MB_OK|MB_ICONWARNING);
          return;
        }
      strcpy(window_breakpoint->source_address_address,buffer);

      /* Dst */
      length = GetDlgItemText(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_BANK_DST,buffer,1024);
      length = GetDlgItemText(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_ADDRESS_DST,buffer_dst,1024);
      if(strlen(buffer) > 0 || strlen(buffer_dst) > 0)
        {
          /* Bank */
          has_error = CheckBreakpointHexaValue(buffer,2);
          if(has_error)
            {
              MessageBox(window_breakpoint->hwnd_breakpoint,"You have to provide a valid Bank value","Wrong Breakpoint Data",MB_OK|MB_ICONWARNING);
              return;
            }
          strcpy(window_breakpoint->source_address_bank_dst,buffer);

          /* Address */
          has_error = CheckBreakpointHexaValue(buffer_dst,4);
          if(has_error)
            {
              MessageBox(window_breakpoint->hwnd_breakpoint,"You have to provide a valid Address value","Wrong Breakpoint Data",MB_OK|MB_ICONWARNING);
              return;
            }
          strcpy(window_breakpoint->source_address_address_dst,buffer_dst);
        }
    }
  else if(window_breakpoint->source_radio == BREAKPOINT_SOURCE_SYMBOL)
    {
      /* No Symbol Table ? */
      if(window_main->current_symbol_table == NULL ||  
         window_breakpoint->source_symbol_begin_address == 0xFFFFFFFF ||
         window_breakpoint->source_symbol_end_address == 0xFFFFFFFF)
        {
          MessageBox(window_breakpoint->hwnd_breakpoint,"You have to provide a valid Symbol name","Wrong Breakpoint Data",MB_OK|MB_ICONWARNING);
          return;
        }
    }
  else if(window_breakpoint->source_radio == BREAKPOINT_SOURCE_SYSTEMCALL)
    {
      /* SystemCall */
    }
  else if(window_breakpoint->source_radio == BREAKPOINT_SOURCE_REGISTER)
    {
      /* Register */
    }
  else if(window_breakpoint->source_radio == BREAKPOINT_SOURCE_OPERATION)
    {
      /* Valide l'opration saisie */
      ValidateBreakpointSourceOperation(window_breakpoint);
    }

  /** Trigger **/
  /* Read */
  if(window_breakpoint->trigger_radio == BREAKPOINT_TRIGGER_READ)
    {
      length = GetDlgItemText(window_breakpoint->hwnd_breakpoint,IDC_EDIT_READ_VALUE,buffer,1024);
      if(strlen(buffer) > 0)
        {
          has_error = IsHexaDecimal(buffer,&is_odd);
          if(has_error == 0)
            {
              MessageBox(window_breakpoint->hwnd_breakpoint,"You have to provide a valid Read value","Wrong Breakpoint Data",MB_OK|MB_ICONWARNING);
              return;
            }
          sscanf(buffer,"%X",&value);
          window_breakpoint->trigger_read_operator_value = (WORD) value;
        }
      else
        window_breakpoint->trigger_read_operator_value = (WORD) 0;
      /* Taille des donnes : 0/1/2 bytes */
      if(strlen(buffer) == 0)
        window_breakpoint->trigger_read_operator_value_size = 0;
      else if(strlen(buffer) >= 3)
        window_breakpoint->trigger_read_operator_value_size = 2;
      else
        window_breakpoint->trigger_read_operator_value_size = 1;
    }
  /* Write */
  else if(window_breakpoint->trigger_radio == BREAKPOINT_TRIGGER_WRITE)
    {
      length = GetDlgItemText(window_breakpoint->hwnd_breakpoint,IDC_EDIT_WRITE_VALUE,buffer,1024);
      if(strlen(buffer) > 0)
        {
          has_error = IsHexaDecimal(buffer,&is_odd);
          if(has_error == 0)
            {
              MessageBox(window_breakpoint->hwnd_breakpoint,"You have to provide a valid Write value","Wrong Breakpoint Data",MB_OK|MB_ICONWARNING);
              return;
            }
          sscanf(buffer,"%X",&value);
          window_breakpoint->trigger_write_operator_value = (WORD) value;
        }
      else
        window_breakpoint->trigger_write_operator_value = (WORD) 0;
      /* Taille des donnes : 0/1/2 bytes */
      if(strlen(buffer) == 0)
        window_breakpoint->trigger_write_operator_value_size = 0;
      else if(strlen(buffer) >= 3)
        window_breakpoint->trigger_write_operator_value_size = 2;
      else
        window_breakpoint->trigger_write_operator_value_size = 1;
    }
  /* Read|Write */
  else if(window_breakpoint->trigger_radio == BREAKPOINT_TRIGGER_READWRITE)
    {
      length = GetDlgItemText(window_breakpoint->hwnd_breakpoint,IDC_EDIT_READWRITE_VALUE,buffer,1024);
      if(strlen(buffer) > 0)
        {
          has_error = IsHexaDecimal(buffer,&is_odd);
          if(has_error == 0)
            {
              MessageBox(window_breakpoint->hwnd_breakpoint,"You have to provide a valid Read|Write value","Wrong Breakpoint Data",MB_OK|MB_ICONWARNING);
              return;
            }
          sscanf(buffer,"%X",&value);
          window_breakpoint->trigger_readwrite_operator_value = (WORD) value;
        }
      else
        window_breakpoint->trigger_readwrite_operator_value = (WORD) 0;
      /* Taille des donnes : 0/1/2 bytes */
      if(strlen(buffer) == 0)
        window_breakpoint->trigger_readwrite_operator_value_size = 0;
      else if(strlen(buffer) >= 3)
        window_breakpoint->trigger_readwrite_operator_value_size = 2;
      else
        window_breakpoint->trigger_readwrite_operator_value_size = 1;
    }
  /* Execute */
  else if(window_breakpoint->trigger_radio == BREAKPOINT_TRIGGER_EXECUTE)
    ;
  /* Hold */
  else if(window_breakpoint->trigger_radio == BREAKPOINT_TRIGGER_HOLD)
    {
      length = GetDlgItemText(window_breakpoint->hwnd_breakpoint,IDC_EDIT_HOLD_VALUE,buffer,1024);
      if(strlen(buffer) > 0)
        {
          has_error = IsHexaDecimal(buffer,&is_odd);
          if(has_error == 0)
            {
              MessageBox(window_breakpoint->hwnd_breakpoint,"You have to provide a valid Hold value","Wrong Breakpoint Data",MB_OK|MB_ICONWARNING);
              return;
            }
          sscanf(buffer,"%X",&value);
          window_breakpoint->trigger_hold_operator_value = (WORD) value;
        }
      else
        {
          MessageBox(window_breakpoint->hwnd_breakpoint,"You have to provide a valid Hold value","Wrong Breakpoint Data",MB_OK|MB_ICONWARNING);
          return;
        }

      if(strlen(buffer) == 0)
        window_breakpoint->trigger_hold_operator_value_size = 0;
      else if(strlen(buffer) >= 3)
        window_breakpoint->trigger_hold_operator_value_size = 2;
      else
        window_breakpoint->trigger_hold_operator_value_size = 1;
    }

  /** Action **/
  if(window_breakpoint->action_radio == BREAKPOINT_ACTION_STOP)
    ;
  else if(window_breakpoint->action_radio == BREAKPOINT_ACTION_PRINT)
    {
      /* Message  afficher */
      length = GetDlgItemText(window_breakpoint->hwnd_breakpoint,IDC_EDIT_PRINT_DATA,window_breakpoint->action_print_data,1024);
      if(strlen(window_breakpoint->action_print_data) == 0)
        {
          MessageBox(window_breakpoint->hwnd_breakpoint,"You have to provide a valid Print Data value","Wrong Breakpoint Data",MB_OK|MB_ICONWARNING);
          return;
        }
    }
  else if(window_breakpoint->action_radio == BREAKPOINT_ACTION_FILE)
    {
      /* Message  afficher */
      length = GetDlgItemText(window_breakpoint->hwnd_breakpoint,IDC_EDIT_PRINT_DATA,window_breakpoint->action_print_data,1024);
      if(strlen(window_breakpoint->action_print_data) == 0)
        {
          MessageBox(window_breakpoint->hwnd_breakpoint,"You have to provide a valid Print Data value","Wrong Breakpoint Data",MB_OK|MB_ICONWARNING);
          return;
        }

      /* Chemin du fichier */
      length = GetDlgItemText(window_breakpoint->hwnd_breakpoint,IDC_EDIT_FILE_PATH,window_breakpoint->action_file_path,1024);
      if(strlen(window_breakpoint->action_file_path) == 0)
        {
          MessageBox(window_breakpoint->hwnd_breakpoint,"You have to provide a valid File Path value","Wrong Breakpoint Data",MB_OK|MB_ICONWARNING);
          return;
        }
    }

  /** Repeat **/
  if(window_breakpoint->repeat_radio == BREAKPOINT_REPEAT_ALWAYS)
    ;
  else if(window_breakpoint->repeat_radio == BREAKPOINT_REPEAT_ONLY)
    {
      /* Message  afficher */
      length = GetDlgItemText(window_breakpoint->hwnd_breakpoint,IDC_EDIT_REPEAT_ONLY_DATA,buffer,1024);
      has_error = IsDecimal(buffer,&is_odd);
      if(has_error == 0)
        {
          MessageBox(window_breakpoint->hwnd_breakpoint,"You have to provide a valid Repeat value","Wrong Breakpoint Data",MB_OK|MB_ICONWARNING);
          return;
        }
      window_breakpoint->repeat_only_times = atoi(buffer);
    }
  else if(window_breakpoint->repeat_radio == BREAKPOINT_REPEAT_AFTER)
    {
      /* Message  afficher */
      length = GetDlgItemText(window_breakpoint->hwnd_breakpoint,IDC_EDIT_REPEAT_ONLY_DATA,buffer,1024);
      has_error = IsDecimal(buffer,&is_odd);
      if(has_error == 0)
        {
          MessageBox(window_breakpoint->hwnd_breakpoint,"You have to provide a valid After value","Wrong Breakpoint Data",MB_OK|MB_ICONWARNING);
          return;
        }
      window_breakpoint->repeat_only_times = atoi(buffer);
    }

  /** On modifie le Breakpoint slectionn **/
  if(window_breakpoint->selected_breakpoint != NULL)
    current_breakpoint = window_breakpoint->selected_breakpoint;
  else
    {
      /** Cration d'un nouveau Breakpoint **/
      /* Allocation */
      current_breakpoint = (struct cyrene_breakpoint  *) calloc(1,sizeof(struct cyrene_breakpoint));
      if(current_breakpoint == NULL)
        return;

      /* ID unique */
      current_breakpoint->id = GetBreakpopintHighestID() + 1;

      /* Le Breakpoint est Disable */
      current_breakpoint->status = 0;     /* 0:Disable / 1:Enable */
    }

  /** Remplissage **/
  /* Source */
  current_breakpoint->source = window_breakpoint->source_radio;     /* BREAKPOINT_SOURCE_XXX */
  strcpy(current_breakpoint->source_address_bank,window_breakpoint->source_address_bank);
  strcpy(current_breakpoint->source_address_address,window_breakpoint->source_address_address);
  strcpy(current_breakpoint->source_address_bank_dst,window_breakpoint->source_address_bank_dst);
  strcpy(current_breakpoint->source_address_address_dst,window_breakpoint->source_address_address_dst);
  strcpy(current_breakpoint->source_symbol_program,window_breakpoint->source_symbol_program);
  strcpy(current_breakpoint->source_symbol_segment,window_breakpoint->source_symbol_segment);
  strcpy(current_breakpoint->source_symbol_name,window_breakpoint->source_symbol_name);
  current_breakpoint->source_symbol_begin_address = window_breakpoint->source_symbol_begin_address;
  current_breakpoint->source_symbol_end_address = window_breakpoint->source_symbol_end_address;
  current_breakpoint->source_systemcall_type = window_breakpoint->source_systemcall_type;
  current_breakpoint->source_systemcall_number = window_breakpoint->source_systemcall_number;
  current_breakpoint->source_systemcall_index = window_breakpoint->source_systemcall_index;
  strcpy(current_breakpoint->source_systemcall_text,window_breakpoint->source_systemcall_text);
  current_breakpoint->source_register = window_breakpoint->source_register;
  strcpy(current_breakpoint->source_operation_text,window_breakpoint->source_operation_text);
  current_breakpoint->source_operation_data_nb = window_breakpoint->source_operation_data_nb;
  for(i=0; i<4; i++)
    current_breakpoint->source_operation_data[i] = window_breakpoint->source_operation_data[i];
  /* Trigger */
  current_breakpoint->trigger = window_breakpoint->trigger_radio;                                       /* BREAKPOINT_TRIGGER_XXX */
  current_breakpoint->trigger_read_operator = window_breakpoint->trigger_read_operator;
  current_breakpoint->trigger_read_value_size = window_breakpoint->trigger_read_operator_value_size;    /* 0:None / 1:8bit / 2:16bit */
  current_breakpoint->trigger_read_value = window_breakpoint->trigger_read_operator_value;
  current_breakpoint->trigger_write_operator = window_breakpoint->trigger_write_operator;
  current_breakpoint->trigger_write_value_size = window_breakpoint->trigger_write_operator_value_size;  /* 0:None / 1:8bit / 2:16bit */
  current_breakpoint->trigger_write_value = window_breakpoint->trigger_write_operator_value;
  current_breakpoint->trigger_readwrite_operator = window_breakpoint->trigger_readwrite_operator;
  current_breakpoint->trigger_readwrite_value_size = window_breakpoint->trigger_readwrite_operator_value_size;  /* 0:None / 1:8bit / 2:16bit */
  current_breakpoint->trigger_readwrite_value = window_breakpoint->trigger_readwrite_operator_value;
  current_breakpoint->trigger_hold_operator = window_breakpoint->trigger_hold_operator;
  current_breakpoint->trigger_hold_value_size = window_breakpoint->trigger_hold_operator_value_size;    /* 0:None / 1:8bit / 2:16bit */
  current_breakpoint->trigger_hold_value = window_breakpoint->trigger_hold_operator_value;
  /* Action */
  current_breakpoint->action = window_breakpoint->action_radio;                                         /* BREAKPOINT_ACTION_XXX */
  strcpy(current_breakpoint->action_print_text,window_breakpoint->action_print_data);
  strcpy(current_breakpoint->action_file_path,window_breakpoint->action_file_path);
  /* Repeat */
  current_breakpoint->repeat = window_breakpoint->repeat_radio;                                         /* BREAKPOINT_REPEAT_XXX */
  current_breakpoint->repeat_only_times = window_breakpoint->repeat_only_times;

  /** Ajout de ce nouvel Breakpoint  la liste **/
  if(window_breakpoint->selected_breakpoint == NULL)
    {
      /* Vrifie si on a dj un Breakpoint identique */
      for(one_breakpoint = window_main->current_breakpoint_table->first_breakpoint; one_breakpoint; one_breakpoint = one_breakpoint->next)
        {
          /* Compare les 2 breakpoints */
          is_same = CompareBreakpoint(current_breakpoint,one_breakpoint);
          if(is_same == 1)
            {
              MessageBox(window_breakpoint->hwnd_breakpoint,"The same Breakpoint already exists !","Wrong Breakpoint Data",MB_OK|MB_ICONWARNING);
              mem_free_breakpoint(current_breakpoint);
              return;
            }
        }

      /* Allocation mmoire de la structure breakpoint_table */
      if(window_main->current_breakpoint_table == NULL)
        {
          window_main->current_breakpoint_table = (struct breakpoint_table *) calloc(1,sizeof(struct breakpoint_table));
          if(window_main->current_breakpoint_table == NULL)
            {
              mem_free_breakpoint(current_breakpoint);
              return;
            }
        }

      /* Un nouveau Breakpoint */
      window_main->current_breakpoint_table->nb_breakpoint++;
      if(window_main->current_breakpoint_table->first_breakpoint == NULL)
        window_main->current_breakpoint_table->first_breakpoint = current_breakpoint;
      else
        window_main->current_breakpoint_table->last_breakpoint->next = current_breakpoint;
      window_main->current_breakpoint_table->last_breakpoint = current_breakpoint;
    }
  else
    {
      /* Vrifie si on a dj un Breakpoint identique */
      for(one_breakpoint = window_main->current_breakpoint_table->first_breakpoint; one_breakpoint; one_breakpoint = one_breakpoint->next)
        {
          /* Compare les 2 breakpoints */
          if(current_breakpoint != one_breakpoint)
            {
              is_same = CompareBreakpoint(current_breakpoint,one_breakpoint);
              if(is_same == 1)
                {
                  MessageBox(window_breakpoint->hwnd_breakpoint,"The same Breakpoint already exists !","Wrong Breakpoint Data",MB_OK|MB_ICONWARNING);
                  return;
                }
            }
        }
    }

  /** Reconstruit le tableau des Breakpoint + ListView + MAJ Stat **/
  RebuildListViewContent(window_breakpoint);

  /** Display/Hide Buttons **/
  DisplayHideBreakpointButton(window_breakpoint,1);

  /** Enable/Disable Buttons **/
  EnableDisableBreakpointButton(window_breakpoint,0,0);

  /** Reset de la partie basse **/
  RAZBreakpoint(window_breakpoint);
  UpdateBreakpointEditMode(window_breakpoint,0);

  /* On est en Read Mode */
  window_breakpoint->edit_mode = 0;
}


/*********************************************************************************************/
/*  ValidateBreakpointSourceOperation() :  Valide l'opration saisie et affiche les donnes. */
/*********************************************************************************************/
static void ValidateBreakpointSourceOperation(struct window_breakpoint *window_breakpoint)
{
  int i, length, has_error;
  char buffer_value[BUFFER_SIZE];
  struct cyrene_breakpoint local_breakpoint;

  /* Operation */
  length = GetDlgItemText(window_breakpoint->hwnd_breakpoint,IDC_EDIT_OPERATION_DATA,window_breakpoint->source_operation_text,1024);

  /* Dcode les valeurs de l'opration */
  strcpy(local_breakpoint.source_operation_text,window_breakpoint->source_operation_text);
  has_error = DecodeBreakpointSourceOperation(&local_breakpoint);
  if(has_error)
    {
      MessageBox(window_breakpoint->hwnd_breakpoint,"Impossible to decode Operation data","Wrong Breakpoint Data",MB_OK|MB_ICONWARNING);
      return;
    }
  window_breakpoint->source_operation_data_nb = local_breakpoint.source_operation_data_nb;
  strcpy(window_breakpoint->source_operation_text,local_breakpoint.source_operation_text);
  for(i=0; i<local_breakpoint.source_operation_data_nb; i++)
    window_breakpoint->source_operation_data[i] = local_breakpoint.source_operation_data[i];

  /* Met  jour le visuel du contrle */
  strcpy(buffer_value,"");
  for(i=0; i<window_breakpoint->source_operation_data_nb; i++)
      sprintf(&buffer_value[strlen(buffer_value)],"%02X ",window_breakpoint->source_operation_data[i]);
  SetWindowText(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_STATIC_OPERATION_DATA),buffer_value);
}


/******************************************************************************/
/*  DisplayHideBreakpointButton() :  On met  jour la visibilit des boutons. */
/******************************************************************************/
static void DisplayHideBreakpointButton(struct window_breakpoint *window_breakpoint, int display_mode)
{
  int mode;

  /** Bouton **/
  mode = (display_mode == 1) ? SW_SHOW : SW_HIDE;
  ShowWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_BUTTON_NEW),mode);
  ShowWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_BUTTON_MODIFY),mode);
  ShowWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_BUTTON_DELETE),mode);
  ShowWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_BUTTON_DELETE_ALL),mode);
  ShowWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_BUTTON_ENABLE),mode);
  ShowWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_BUTTON_DISABLE),mode);
  ShowWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_BUTTON_LOAD),mode);
  ShowWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_BUTTON_SAVE),mode);

  /** Bouton Edit **/
  mode = (display_mode == 0) ? SW_SHOW : SW_HIDE;
  ShowWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_BUTTON_CANCEL),mode);
  ShowWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_BUTTON_OK),mode);
}


/*******************************************************************************/
/*  EnableDisableBreakpointButton() :  On met  jour l'activation des boutons. */
/*******************************************************************************/
static void EnableDisableBreakpointButton(struct window_breakpoint *window_breakpoint, int has_select, int status)
{
  int has_breakpoint;
  struct window_main *window_main;

  /* Rcupre la fentre Main */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  if(window_main == NULL)
    return;

  /* Y a t'il des Breapoint dans la table ? */
  if(window_main->current_breakpoint_table == NULL)
    has_breakpoint = 0;
  else if(window_main->current_breakpoint_table->nb_breakpoint == 0)
    has_breakpoint = 0;
  else
    has_breakpoint = 1;

  /** Enable / Disable Buttons en fonction de la slection d'un Breakpoint dans la liste (NEW et LOAD sont tjs actifs) **/
  EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_BUTTON_MODIFY),has_select);
  EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_BUTTON_DELETE),has_select);
  if(has_select == 0)
    {
      /* Disable */
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_BUTTON_ENABLE),has_select);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_BUTTON_DISABLE),has_select);
    }
  else
    {
      /* Selected Breakpoint is Enabled */
      if(status == 1)
        {
          /* Enable */
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_BUTTON_ENABLE),0);
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_BUTTON_DISABLE),1);
        }
      else
        {
          /* Disable */
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_BUTTON_ENABLE),1);
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_BUTTON_DISABLE),0);
        }
    }
  /* Dsactive DELETE_ALL et SAVE si aucun Breakpoint */
  EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_BUTTON_DELETE_ALL),has_breakpoint);
  EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_BUTTON_SAVE),has_breakpoint);
}


/********************************************************************/
/*  LoadBreakpointFile() : Charge en mmoire un fichier Breakpoint. */
/********************************************************************/
static struct cyrene_breakpoint *LoadBreakpointFile(char *file_path)
{
  int i, file_size;
  char *file_data;
  char *begin_line;
  char *end_line;
  DWORD highest_id;
  char program_name[BUFFER_SIZE] = "";
  char error_buffer[BUFFER_SIZE];
  struct cyrene_breakpoint *first_breakpoint;
  struct cyrene_breakpoint *last_breakpoint;
  struct cyrene_breakpoint *current_breakpoint;

  /* Init */
  first_breakpoint = NULL;
  last_breakpoint = NULL;
  highest_id = GetBreakpopintHighestID();

  /* Isole le nom du Programme */
  for(i=(int)strlen(file_path); i>=0; i--)
    if(file_path[i] == '/' || file_path[i] == '\\')
      {
        /* Nom du Fichier */
        strcpy(program_name,&file_path[i+1]);

        /* Nom du Programme */
        for(i=(int)strlen(program_name); i>=0; i--)
          if(program_name[i] == '_')
            {
              program_name[i] = '\0';
              break;
            }
        break;
      }

  /* Charge en mmoire le fichier en mmoire */
  file_data = (char *) LoadTextFile(file_path,&file_size,error_buffer);
  if(file_data == NULL)
    return(NULL);

  /** Dcoupe le fichier en Breakpoint / Reloc **/
  begin_line = file_data;
  while(begin_line)
    {
      /* Fin de la ligne */
      end_line = strchr(begin_line,'\n');
      if(end_line)
        *end_line = '\0';

      /* Rcupre un Breakpoint + Ajoute dans la liste Breakpoint */
      current_breakpoint = DecodeOneBreakpointLine(begin_line);
      if(current_breakpoint != NULL)
        {
          current_breakpoint->id = ++highest_id;
          if(first_breakpoint == NULL)
            first_breakpoint = current_breakpoint;
          else
            last_breakpoint->next = current_breakpoint;
          last_breakpoint = current_breakpoint;
        }

      /* Ligne suivante */
      if(end_line == NULL)
        break;
      begin_line = end_line + 1;
    }

  /* Renvoi la liste */
  return(first_breakpoint);
}


/**************************************************************/
/*  DecodeOneBreakpointLine() :  Dcode une ligne Breakpoint. */
/**************************************************************/
static struct cyrene_breakpoint *DecodeOneBreakpointLine(char *breakpoint_line)
{
  int i, j, nb_item, length, tab_index[16], is_error, found;
  DWORD call_number;
  char *data;
  char *next_sep;
  struct window_main *window_main;
  struct cyrene_breakpoint *current_breakpoint;
  struct symbol *current_symbol;
  char value[BUFFER_SIZE];
  char *tab_register[] = {"A","X","Y","PC","STACK","DIRECT","DBR","PBR","STATUS",NULL};
  char *tab_operator[] = {"=","!=",">",">=","<","<=",NULL};
  struct parameter *param;
  my_Memory(MEMORY_GET_PARAM,&param,NULL);

  /* Rcupre la fentre Main */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  if(window_main == NULL)
    return(NULL);

  /* Allocation mmoire */
  current_breakpoint = (struct cyrene_breakpoint *) calloc(1,sizeof(struct cyrene_breakpoint));
  if(current_breakpoint == NULL)
    return(NULL);
  current_breakpoint->source_symbol_begin_address = 0xFFFFFFFF;
  current_breakpoint->source_symbol_end_address = 0xFFFFFFFF;
  /** Status;Source;Trigger;Action;Repeat **/
  nb_item = 0;
  tab_index[nb_item++] = 0;
  for(i=0; i<(int)strlen(breakpoint_line); i++)
    if(breakpoint_line[i] == ';')
      tab_index[nb_item++] = i;
  tab_index[nb_item++] = (int) strlen(breakpoint_line);

  /** Valide le format de la ligne **/
  if(nb_item != 6)
    {
      mem_free_breakpoint(current_breakpoint);
      return(NULL);
    }

  /** Rcupre les valeurs **/
  for(i=0; i<nb_item-1; i++)
    {
      /* Rcupre la valeur */
      length = tab_index[i+1]-(tab_index[i]+(i==0?0:1));
      memcpy(value,&breakpoint_line[tab_index[i]+(i==0?0:1)],length);
      value[length] = '\0';

      /*** Conserve la valeur ***/
      if(i == 0)
        {
          /** Status **/
          if(!stricmp(value,"Enable"))
            current_breakpoint->status = 1;
          else if(!stricmp(value,"Disable"))
            current_breakpoint->status = 0;
          else
            {
              mem_free_breakpoint(current_breakpoint);
              return(NULL);
            }
        }
      else if(i == 1)
        {
          /** Source **/
          if(!strnicmp(value,"Address=",strlen("Address=")))
            {
              data = &value[strlen("Address=")];
              if(strlen(data) == strlen("00/0000") && data[2] == '/')
                {
                  current_breakpoint->source = BREAKPOINT_SOURCE_ADDRESS;
                  memcpy(current_breakpoint->source_address_bank,data,2);
                  current_breakpoint->source_address_bank[2] = '\0';
                  memcpy(current_breakpoint->source_address_address,&data[3],4);
                  current_breakpoint->source_address_address[4] = '\0';
                  strcpy(current_breakpoint->source_address_bank_dst,"");
                  strcpy(current_breakpoint->source_address_address_dst,"");
                }
              else if(strlen(data) == strlen("00/0000-00/0000") && data[2] == '/' && data[7] == '-' && data[10] == '/')
                {
                  current_breakpoint->source = BREAKPOINT_SOURCE_ADDRESS;
                  memcpy(current_breakpoint->source_address_bank,data,2);
                  current_breakpoint->source_address_bank[2] = '\0';
                  memcpy(current_breakpoint->source_address_address,&data[3],4);
                  current_breakpoint->source_address_address[4] = '\0';
                  memcpy(current_breakpoint->source_address_bank_dst,&data[8],2);
                  current_breakpoint->source_address_bank_dst[2] = '\0';
                  memcpy(current_breakpoint->source_address_address_dst,&data[11],4);
                  current_breakpoint->source_address_address_dst[4] = '\0';
                }
              else
                {
                  mem_free_breakpoint(current_breakpoint);
                  return(NULL);
                }
            }
          else if(!strnicmp(value,"Symbol=",strlen("Symbol=")))
            {
              /* Rcupre les donnes */
              data = &value[strlen("Symbol=")];
              if(strlen(data) == 0 || strlen(data) > 255)
                {
                  mem_free_breakpoint(current_breakpoint);
                  return(NULL);
                }
              current_breakpoint->source = BREAKPOINT_SOURCE_SYMBOL;
              strcpy(current_breakpoint->source_symbol_program,data);
              next_sep = strchr(current_breakpoint->source_symbol_program,'|');
              if(next_sep == NULL)
                {
                  mem_free_breakpoint(current_breakpoint);
                  return(NULL);
                }
              *next_sep = '\0';
              strcpy(current_breakpoint->source_symbol_segment,next_sep+1);
              next_sep = strchr(current_breakpoint->source_symbol_segment,'|');
              if(next_sep == NULL)
                {
                  mem_free_breakpoint(current_breakpoint);
                  return(NULL);
                }
              *next_sep = '\0';
              strcpy(current_breakpoint->source_symbol_name,next_sep+1);

              /* Recherche l'adresse en mmoire de ce Symbol */
              current_breakpoint->source_symbol_begin_address = 0xFFFFFFFF;
              current_breakpoint->source_symbol_end_address = 0xFFFFFFFF;
              if(window_main->current_symbol_table != NULL)
                {
                  for(current_symbol=window_main->current_symbol_table->first_symbol; current_symbol; current_symbol=current_symbol->next)
                    if(!strcmp(current_symbol->name,current_breakpoint->source_symbol_name) &&              /* Le Symbol est case sensitive */
                       !stricmp(current_symbol->segment_name,current_breakpoint->source_symbol_segment) &&
                       !stricmp(current_symbol->program_name,current_breakpoint->source_symbol_program))
                      {
                         current_breakpoint->source_symbol_begin_address = current_symbol->symbol_relocated_address;

                         /* Pour les Symbol, on calcule l'adresse de fin (y compris le Code) */
                         current_breakpoint->source_symbol_end_address = current_breakpoint->source_symbol_begin_address + current_symbol->size - 1;
                         break;
                      }
                }
            }
          else if(!strnicmp(value,"SystemCall=",strlen("SystemCall=")))
            {
              data = &value[strlen("SystemCall=")];
              if(strlen(data) == 0)
                {
                  mem_free_breakpoint(current_breakpoint);
                  return(NULL);
                }
              current_breakpoint->source = BREAKPOINT_SOURCE_SYSTEMCALL;
              if(!strnicmp(data,"P8-",3))
                {
                  current_breakpoint->source_systemcall_type = TYPE_SYSTEM_CALL_P8;
                  sscanf(&data[strlen("P8-")],"%X",&call_number);
                  current_breakpoint->source_systemcall_number = (BYTE) call_number;
                }
              else if(!strnicmp(data,"P16-",4))
                {
                  current_breakpoint->source_systemcall_type = TYPE_SYSTEM_CALL_P16;
                  sscanf(&data[strlen("P16-")],"%X",&call_number);
                  current_breakpoint->source_systemcall_number = (WORD) call_number;
                }
              else if(!strnicmp(data,"GSOS-",5))
                {
                  current_breakpoint->source_systemcall_type = TYPE_SYSTEM_CALL_GSOS;
                  sscanf(&data[strlen("GSOS-")],"%X",&call_number);
                  current_breakpoint->source_systemcall_number = (WORD) call_number;
                }
              else if(!strnicmp(data,"TOOL-",5))
                {
                  current_breakpoint->source_systemcall_type = TYPE_SYSTEM_CALL_TOOLBOX;
                  sscanf(&data[strlen("TOOL-")],"%X",&call_number);
                  current_breakpoint->source_systemcall_number = (WORD) call_number;
                }
              else
                {
                  mem_free_breakpoint(current_breakpoint);
                  return(NULL);
                }

              /* Recherche du Text et de l'index dans le tableau */
              for(i=0; i<param->nb_systemcall; i++)
                if(param->tab_systemcall[i]->type == current_breakpoint->source_systemcall_type &&
                   param->tab_systemcall[i]->call_number == current_breakpoint->source_systemcall_number)
                    {
                      strcpy(current_breakpoint->source_systemcall_text,param->tab_systemcall[i]->text);
                      current_breakpoint->source_systemcall_index = i;
                      break;
                    }
            }
          else if(!strnicmp(value,"Register=",strlen("Register=")))
            {
              data = &value[strlen("Register=")];
              if(strlen(data) == 0)
                {
                  mem_free_breakpoint(current_breakpoint);
                  return(NULL);
                }
              current_breakpoint->source = BREAKPOINT_SOURCE_REGISTER;
              for(found=0,j=0; tab_register[j]!=NULL; j++)
                if(!stricmp(data,tab_register[j]))
                  {
                    current_breakpoint->source_register = j;
                    found = 1;
                    break;
                  }
              if(found == 0)
                {
                  mem_free_breakpoint(current_breakpoint);
                  return(NULL);
                }
            }
          else if(!strnicmp(value,"Operation=",strlen("Operation=")))
            {
              data = &value[strlen("Operation=")];
              if(strlen(data) == 0)
                {
                  mem_free_breakpoint(current_breakpoint);
                  return(NULL);
                }
              current_breakpoint->source = BREAKPOINT_SOURCE_OPERATION;
              strcpy(current_breakpoint->source_operation_text,data);
              is_error = DecodeBreakpointSourceOperation(current_breakpoint);
              if(is_error)
                {
                  mem_free_breakpoint(current_breakpoint);
                  return(NULL);
                }
            }
        }
      else if(i == 2)
        {
          /** Trigger **/
          /* ReadWrite */
          next_sep = my_stristr(value,"ReadWrite:");
          if(next_sep)
            {
              data = next_sep + strlen("ReadWrite:");
              /* Operator */
              next_sep = strchr(data,' ');
              if(next_sep == NULL)
                {
                  mem_free_breakpoint(current_breakpoint);
                  return(NULL);                  
                }
              *next_sep = '\0';
              for(found=0,j=0; tab_operator[j]!=NULL; j++)
                if(!stricmp(data,tab_operator[j]))
                  {
                    found = 1;
                    break;
                  }
              if(found == 0)
                {
                  mem_free_breakpoint(current_breakpoint);
                  return(NULL);                  
                }
              current_breakpoint->trigger_readwrite_operator = i;
              *next_sep = ' ';
              /* Value */
              data = next_sep+1;
              next_sep = strchr(data,',');
              if(next_sep)
                *next_sep = '\0';
              if(strlen(data) == 2)
                {
                  /* BYTE */
                  current_breakpoint->trigger_readwrite_value_size = 1;
                  current_breakpoint->trigger_readwrite_value = (WORD) EvaluateAsByte(data,&is_error);
                  if(is_error)
                    {
                      mem_free_breakpoint(current_breakpoint);
                      return(NULL);  
                    }
                }
              else if(strlen(data) == 4)
                {
                  /* WORD */
                  current_breakpoint->trigger_readwrite_value_size = 2;
                  current_breakpoint->trigger_readwrite_value = (WORD) EvaluateAsWord(data,&is_error);
                  if(is_error)
                    {
                      mem_free_breakpoint(current_breakpoint);
                      return(NULL);  
                    }
                }
              if(next_sep)
                *next_sep = ',';
              current_breakpoint->trigger = BREAKPOINT_TRIGGER_READWRITE;
            }
          else if(my_stristr(value,"ReadWrite"))
            {
              current_breakpoint->trigger = BREAKPOINT_TRIGGER_READWRITE;
              current_breakpoint->trigger_readwrite_value_size = 0;
            }

          /* Read */
          next_sep = my_stristr(value,"Read:");
          if(next_sep != NULL && current_breakpoint->trigger == 0)
            {
              data = next_sep + strlen("Read:");
              /* Operator */
              next_sep = strchr(data,' ');
              if(next_sep == NULL)
                {
                  mem_free_breakpoint(current_breakpoint);
                  return(NULL);                  
                }
              *next_sep = '\0';
              for(found=0,j=0; tab_operator[j]!=NULL; j++)
                if(!stricmp(data,tab_operator[j]))
                  {
                    found = 1;
                    break;
                  }
              if(found == 0)
                {
                  mem_free_breakpoint(current_breakpoint);
                  return(NULL);                  
                }
              current_breakpoint->trigger_read_operator = i;
              *next_sep = ' ';
              /* Value */
              data = next_sep+1;
              next_sep = strchr(data,',');
              if(next_sep)
                *next_sep = '\0';
              if(strlen(data) == 2)
                {
                  /* BYTE */
                  current_breakpoint->trigger_read_value_size = 1;
                  current_breakpoint->trigger_read_value = (WORD) EvaluateAsByte(data,&is_error);
                  if(is_error)
                    {
                      mem_free_breakpoint(current_breakpoint);
                      return(NULL);  
                    }
                }
              else if(strlen(data) == 4)
                {
                  /* WORD */
                  current_breakpoint->trigger_read_value_size = 2;
                  current_breakpoint->trigger_read_value = (WORD) EvaluateAsWord(data,&is_error);
                  if(is_error)
                    {
                      mem_free_breakpoint(current_breakpoint);
                      return(NULL);  
                    }
                }
              if(next_sep)
                *next_sep = ',';
              current_breakpoint->trigger = BREAKPOINT_TRIGGER_READ;
            }
          else if(my_stristr(value,"Read") && current_breakpoint->trigger == 0)
            {
              current_breakpoint->trigger = BREAKPOINT_TRIGGER_READ;
              current_breakpoint->trigger_read_value_size = 0;
            }

          /* Write */
          next_sep = my_stristr(value,"Write:");
          if(next_sep != NULL && current_breakpoint->trigger == 0)
            {
              data = next_sep + strlen("Write:");
              /* Operator */
              next_sep = strchr(data,' ');
              if(next_sep == NULL)
                {
                  mem_free_breakpoint(current_breakpoint);
                  return(NULL);                  
                }
              *next_sep = '\0';
              for(found=0,j=0; tab_operator[j]!=NULL; j++)
                if(!stricmp(data,tab_operator[j]))
                  {
                    found = 1;
                    break;
                  }
              if(found == 0)
                {
                  mem_free_breakpoint(current_breakpoint);
                  return(NULL);                  
                }
              current_breakpoint->trigger_write_operator = i;
              *next_sep = ' ';
              /* Value */
              data = next_sep+1;
              next_sep = strchr(data,',');
              if(next_sep)
                *next_sep = '\0';
              if(strlen(data) == 2)
                {
                  /* BYTE */
                  current_breakpoint->trigger_write_value_size = 1;
                  current_breakpoint->trigger_write_value = (WORD) EvaluateAsByte(data,&is_error);
                  if(is_error)
                    {
                      mem_free_breakpoint(current_breakpoint);
                      return(NULL);  
                    }
                }
              else if(strlen(data) == 4)
                {
                  /* WORD */
                  current_breakpoint->trigger_write_value_size = 2;
                  current_breakpoint->trigger_write_value = (WORD) EvaluateAsWord(data,&is_error);
                  if(is_error)
                    {
                      mem_free_breakpoint(current_breakpoint);
                      return(NULL);  
                    }
                }
              if(next_sep)
                *next_sep = ',';
              current_breakpoint->trigger = BREAKPOINT_TRIGGER_WRITE;
            }
          else if(my_stristr(value,"Write") && current_breakpoint->trigger == 0)
            {
              current_breakpoint->trigger = BREAKPOINT_TRIGGER_WRITE;
              current_breakpoint->trigger_write_value_size = 0;
            }

          /* Execute */
          if(my_stristr(value,"Execute"))
            current_breakpoint->trigger = BREAKPOINT_TRIGGER_EXECUTE;

          /* Hold */
          next_sep = my_stristr(value,"Hold:");
          if(next_sep)
            {
              data = next_sep + strlen("Hold:");
              /* Operator */
              next_sep = strchr(data,' ');
              if(next_sep == NULL)
                {
                  mem_free_breakpoint(current_breakpoint);
                  return(NULL);                  
                }
              *next_sep = '\0';
              for(found=0,j=0; tab_operator[j]!=NULL; j++)
                if(!stricmp(data,tab_operator[j]))
                  {
                    found = 1;
                    break;
                  }
              if(found == 0)
                {
                  mem_free_breakpoint(current_breakpoint);
                  return(NULL);                  
                }
              current_breakpoint->trigger_hold_operator = i;
              *next_sep = ' ';
              /* Value */
              data = next_sep+1;
              next_sep = strchr(data,',');
              if(next_sep)
                *next_sep = '\0';
              if(strlen(data) == 2)
                {
                  /* BYTE */
                  current_breakpoint->trigger_hold_value_size = 1;
                  current_breakpoint->trigger_hold_value = (WORD) EvaluateAsByte(data,&is_error);
                  if(is_error)
                    {
                      mem_free_breakpoint(current_breakpoint);
                      return(NULL);  
                    }
                }
              else if(strlen(data) == 4)
                {
                  /* WORD */
                  current_breakpoint->trigger_hold_value_size = 2;
                  current_breakpoint->trigger_hold_value = (WORD) EvaluateAsWord(data,&is_error);
                  if(is_error)
                    {
                      mem_free_breakpoint(current_breakpoint);
                      return(NULL);  
                    }
                }
              if(next_sep)
                *next_sep = ',';
              current_breakpoint->trigger = BREAKPOINT_TRIGGER_HOLD;
            }
          else if(my_stristr(value,"Hold"))
            {
              current_breakpoint->trigger = BREAKPOINT_TRIGGER_HOLD;
              current_breakpoint->trigger_hold_value_size = 0;
            }

          /* Il faut au moins 1 slectionn */
          if(current_breakpoint->trigger == 0)
            {
              mem_free_breakpoint(current_breakpoint);
              return(NULL);  
            }
        }
      else if(i == 3)
        {
          /** Action **/
          if(!strnicmp(value,"Action:Stop",strlen("Action:Stop")))
            current_breakpoint->action = BREAKPOINT_ACTION_STOP;
          else if(!strnicmp(value,"Action:Print'",strlen("Action:Print'")))
            {
              data = &value[strlen("Action:Print'")];
              next_sep = strchr(data,'\'');
              if(next_sep == NULL)
                {
                  mem_free_breakpoint(current_breakpoint);
                  return(NULL);
                }
              *next_sep = '\0';
              current_breakpoint->action = BREAKPOINT_ACTION_PRINT;
              strcpy(current_breakpoint->action_print_text,data);
            }
          else if(!strnicmp(value,"Action:File'",strlen("Action:File'")))
            {
              data = &value[strlen("Action:File'")];
              next_sep = strchr(data,'\'');
              if(next_sep == NULL)
                {
                  mem_free_breakpoint(current_breakpoint);
                  return(NULL);
                }
              *next_sep = '\0';
              strcpy(current_breakpoint->action_print_text,data);
              data = next_sep+1;
              if(!strnicmp(data,",'",strlen(",'")))
                {
                  data += 2;
                  next_sep = strchr(data,'\'');
                  if(next_sep == NULL)
                    {
                      mem_free_breakpoint(current_breakpoint);
                      return(NULL);
                    }
                  *next_sep = '\0';
                  strcpy(current_breakpoint->action_file_path,data);
                }
              else
                {
                  mem_free_breakpoint(current_breakpoint);
                  return(NULL);
                }
              current_breakpoint->action = BREAKPOINT_ACTION_FILE;
            }
        }
      else if(i == 4)
        {
          /** Repeat **/
          if(!strnicmp(value,"Repeat:Always",strlen("Repeat:Always")))
            current_breakpoint->repeat = BREAKPOINT_REPEAT_ALWAYS;
          else if(!strnicmp(value,"Repeat:Only",strlen("Repeat:Only")))
            {
              data = &value[strlen("Repeat:Only")];
              current_breakpoint->repeat = BREAKPOINT_REPEAT_ONLY;
              current_breakpoint->repeat_only_times = atoi(data);
            }
          else if(!strnicmp(value,"Repeat:After",strlen("Repeat:After")))
            {
              data = &value[strlen("Repeat:After")];
              current_breakpoint->repeat = BREAKPOINT_REPEAT_AFTER;
              current_breakpoint->repeat_only_times = atoi(data);
            }
        }
    }

  /* Renvoi un Breakpoint */
  return(current_breakpoint);
}


/****************************************************************************/
/*  DecodeBreakpointSourceOperation() :  Dcode la partie Source Operation. */
/****************************************************************************/
static int DecodeBreakpointSourceOperation(struct cyrene_breakpoint *current_breakpoint)
{
  int i, nb_value, is_error, no_only_byte, address_mode;
  char *next_sep;
  char **tab_value;
  char *tab_opcode_txt[] = 
    {
      "ASL","BRK","CLC","CLD","CLI","CLV","COP","DEC","DEX","DEY",   /* Opcode 1 Byte */
      "INC","INX","INY","LSR","NOP","PHA","PHB","PHD","PHK","PHP",
      "PHX","PHY","PLA","PLB","PLD","PLP","PLX","PLY","ROL","ROR",
      "RTI","RTL","RTS","SEC","SED","SEI","STP","TAX","TAY","TCD",
      "TCS","TDC","TSC","TSX","TXA","TXS","TXY","TYA","TYX","WAI",
      "WDM","XBA","XCE",
      NULL
    };
  unsigned char tab_opcode_byte[] = 
    {
      0x0A,0x00,0x18,0xD8,0x58,0xB8,0x02,0x3A,0xCA,0x88,
      0x1A,0xE8,0xC8,0x4A,0xEA,0x48,0x8B,0x0B,0x4B,0x08,
      0xDA,0x5A,0x68,0xAB,0x2B,0x28,0xFA,0x7A,0x2A,0x6A,
      0x40,0x6B,0x60,0x38,0xF8,0x78,0xDB,0xAA,0xA8,0x5B,
      0x1B,0x7B,0x3B,0xBA,0x8A,0x9A,0x9B,0x98,0xBB,0xCB,
      0x42,0xEB,0xFB
    };
  char operand[256];
  int nb_byte_operand;
  unsigned char data_operand[4];

  /* Etape 1 : Nettoyage */
  CleanBuffer(current_breakpoint->source_operation_text);
  next_sep = strstr(current_breakpoint->source_operation_text,"  ");
  while(next_sep)
    {
      memmove(next_sep,next_sep+1,strlen(next_sep+1)+1);
      next_sep = strstr(current_breakpoint->source_operation_text,"  ");
    }

  /* Etape 2 : Dcoupage des valeurs */
  tab_value = BuildUniqueListFromText(current_breakpoint->source_operation_text,' ',&nb_value);
  if(tab_value == NULL)
    return(1);
  if(nb_value == 0)
    {
      mem_free_list(nb_value,tab_value);
      return(1);
    }

  /** Option #1 : Que des bytes **/
  no_only_byte = 0;
  current_breakpoint->source_operation_data_nb = 0;
  if(nb_value <= 4)
    {
      /* Decode Byte */
      for(i=0; i<nb_value; i++)
        {
          if(strlen(tab_value[i]) != 2)
            {
              no_only_byte = 1;
              break;
            }
          if(IsHexaDecimalChar(tab_value[i][0]) && IsHexaDecimalChar(tab_value[i][1]))
            current_breakpoint->source_operation_data[current_breakpoint->source_operation_data_nb++] = EvaluateAsByte(tab_value[i],&is_error);
          else
            {
              no_only_byte = 1;
              break;
            }
        }
      if(no_only_byte == 0)
        {
          mem_free_list(nb_value,tab_value);
          return(0);
        }
    }

  /** Option #2 : Code  assembler **/
  if(nb_value == 1 && strlen(tab_value[0]) == 3)
    {
      /** Recherche l'Opcode **/
      for(i=0; tab_opcode_txt[i] != NULL; i++)
        if(!stricmp(tab_opcode_txt[i],tab_value[0]))
          {
            current_breakpoint->source_operation_data_nb = 1;
            current_breakpoint->source_operation_data[0] = tab_opcode_byte[i];
            mem_free_list(nb_value,tab_value);
            return(0);
          }
    }
  else if(nb_value == 2 && (strlen(tab_value[0]) == 3 || strlen(tab_value[0]) == 4) && strlen(tab_value[1]) >= 3)
    {
      /** Dcode l'oprande : Mode d'adressage + Data **/
      strcpy(operand,tab_value[1]);
      address_mode = GetAddressModeData(operand,&nb_byte_operand,&data_operand[0]);
      if(address_mode == AM_UNKOWN)
        {
          mem_free_list(nb_value,tab_value);
          return(1);
        }
      current_breakpoint->source_operation_data_nb = nb_byte_operand+1;
      memcpy(&current_breakpoint->source_operation_data[1],data_operand,nb_byte_operand);

      /** Encodage **/
      if(!stricmp(tab_value[0],"ADC"))
        {
          if(address_mode == AM_IMMEDIATE_8)
            {
              current_breakpoint->source_operation_data[0] = 0x69;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_IMMEDIATE_16)
            {
              current_breakpoint->source_operation_data[0] = 0x69;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE)
            {
              current_breakpoint->source_operation_data[0] = 0x6D;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE_LONG)
            {
              current_breakpoint->source_operation_data[0] = 0x6F;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE)
            {
              current_breakpoint->source_operation_data[0] = 0x65;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDIRECT)
            {
              current_breakpoint->source_operation_data[0] = 0x72;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDIRECT_LONG)
            {
              current_breakpoint->source_operation_data[0] = 0x67;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE_INDEXED_X)
            {
              current_breakpoint->source_operation_data[0] = 0x7D;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE_LONG_INDEXED_X)
            {
              current_breakpoint->source_operation_data[0] = 0x7F;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE_INDEXED_Y || address_mode == AM_DIRECT_PAGE_INDEXED_Y)    /* $08,Y doit aussi pouvoir tre interprett comme $0008,Y car DP,Y n'existe pas */
            {
              current_breakpoint->source_operation_data[0] = 0x79;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDEXED_X)
            {
              current_breakpoint->source_operation_data[0] = 0x75;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDEXED_X_INDIRECT)
            {
              current_breakpoint->source_operation_data[0] = 0x61;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDIRECT_INDEXED_Y)
            {
              current_breakpoint->source_operation_data[0] = 0x71;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDIRECT_LONG_INDEXED_Y)
            {
              current_breakpoint->source_operation_data[0] = 0x77;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_STACK_RELATIVE)
            {
              current_breakpoint->source_operation_data[0] = 0x63;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_STACK_RELATIVE_INDIRECT_INDEXED_Y)
            {
              current_breakpoint->source_operation_data[0] = 0x73;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
        }
      else if(!stricmp(tab_value[0],"AND"))
        {
          if(address_mode == AM_IMMEDIATE_8)
            {
              current_breakpoint->source_operation_data[0] = 0x29;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_IMMEDIATE_16)
            {
              current_breakpoint->source_operation_data[0] = 0x29;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE)
            {
              current_breakpoint->source_operation_data[0] = 0x2D;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE_LONG)
            {
              current_breakpoint->source_operation_data[0] = 0x2F;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE)
            {
              current_breakpoint->source_operation_data[0] = 0x25;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDIRECT)
            {
              current_breakpoint->source_operation_data[0] = 0x32;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDIRECT_LONG)
            {
              current_breakpoint->source_operation_data[0] = 0x27;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE_INDEXED_X)
            {
              current_breakpoint->source_operation_data[0] = 0x3D;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE_LONG_INDEXED_X)
            {
              current_breakpoint->source_operation_data[0] = 0x3F;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE_INDEXED_Y || address_mode == AM_DIRECT_PAGE_INDEXED_Y)    /* $08,Y doit aussi pouvoir tre interprett comme $0008,Y car DP,Y n'existe pas */
            {
              current_breakpoint->source_operation_data[0] = 0x39;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDEXED_X)
            {
              current_breakpoint->source_operation_data[0] = 0x35;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDEXED_X_INDIRECT)
            {
              current_breakpoint->source_operation_data[0] = 0x21;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDIRECT_INDEXED_Y)
            {
              current_breakpoint->source_operation_data[0] = 0x31;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDIRECT_LONG_INDEXED_Y)
            {
              current_breakpoint->source_operation_data[0] = 0x37;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_STACK_RELATIVE)
            {
              current_breakpoint->source_operation_data[0] = 0x23;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_STACK_RELATIVE_INDIRECT_INDEXED_Y)
            {
              current_breakpoint->source_operation_data[0] = 0x33;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
        }
      else if(!stricmp(tab_value[0],"ASL"))
        {
          if(address_mode == AM_ABSOLUTE)
            {
              current_breakpoint->source_operation_data[0] = 0x0E;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE)
            {
              current_breakpoint->source_operation_data[0] = 0x06;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE_INDEXED_X)
            {
              current_breakpoint->source_operation_data[0] = 0x1E;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDEXED_X)
            {
              current_breakpoint->source_operation_data[0] = 0x16;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
        }
      else if(!stricmp(tab_value[0],"BIT"))
        {
          if(address_mode == AM_IMMEDIATE_8)
            {
              current_breakpoint->source_operation_data[0] = 0x89;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_IMMEDIATE_16)
            {
              current_breakpoint->source_operation_data[0] = 0x89;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE)
            {
              current_breakpoint->source_operation_data[0] = 0x2C;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE)
            {
              current_breakpoint->source_operation_data[0] = 0x24;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE_INDEXED_X)
            {
              current_breakpoint->source_operation_data[0] = 0x3C;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDEXED_X)
            {
              current_breakpoint->source_operation_data[0] = 0x34;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
        }
      else if(!stricmp(tab_value[0],"BRK"))
        {
          if(address_mode == AM_IMMEDIATE_8 || address_mode == AM_DIRECT_PAGE)
            {
              current_breakpoint->source_operation_data[0] = 0x00;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
        }
      else if(!stricmp(tab_value[0],"CMP"))
        {
          if(address_mode == AM_IMMEDIATE_8)
            {
              current_breakpoint->source_operation_data[0] = 0xC9;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_IMMEDIATE_16)
            {
              current_breakpoint->source_operation_data[0] = 0xC9;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE)
            {
              current_breakpoint->source_operation_data[0] = 0xCD;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE_LONG)
            {
              current_breakpoint->source_operation_data[0] = 0xCF;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE)
            {
              current_breakpoint->source_operation_data[0] = 0xC5;
              mem_free_list(nb_value,tab_value);
              return(0);
          }
          else if(address_mode == AM_DIRECT_PAGE_INDIRECT)
            {
              current_breakpoint->source_operation_data[0] = 0xD2;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDIRECT_LONG)
            {
              current_breakpoint->source_operation_data[0] = 0xC7;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE_INDEXED_X)
            {
              current_breakpoint->source_operation_data[0] = 0xDD;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE_LONG_INDEXED_X)
            {
              current_breakpoint->source_operation_data[0] = 0xDF;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE_INDEXED_Y || address_mode == AM_DIRECT_PAGE_INDEXED_Y)    /* $08,Y doit aussi pouvoir tre interprett comme $0008,Y car DP,Y n'existe pas */
            {
              current_breakpoint->source_operation_data[0] = 0xD9;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDEXED_X)
            {
              current_breakpoint->source_operation_data[0] = 0xD5;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDEXED_X_INDIRECT)
            {
              current_breakpoint->source_operation_data[0] = 0xC1;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDIRECT_INDEXED_Y)
            {
              current_breakpoint->source_operation_data[0] = 0xD1;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDIRECT_LONG_INDEXED_Y)
            {
              current_breakpoint->source_operation_data[0] = 0xD7;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_STACK_RELATIVE)
            {
              current_breakpoint->source_operation_data[0] = 0xC3;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_STACK_RELATIVE_INDIRECT_INDEXED_Y)
            {
              current_breakpoint->source_operation_data[0] = 0xD3;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
        }
      else if(!stricmp(tab_value[0],"COP"))
        {
          if(address_mode == AM_IMMEDIATE_8 || address_mode == AM_DIRECT_PAGE)
            {
              current_breakpoint->source_operation_data[0] = 0x02;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
        }
      else if(!stricmp(tab_value[0],"CPX"))
        {
          if(address_mode == AM_IMMEDIATE_8)
            {
              current_breakpoint->source_operation_data[0] = 0xE0;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_IMMEDIATE_16)
            {
              current_breakpoint->source_operation_data[0] = 0xE0;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE)
            {
              current_breakpoint->source_operation_data[0] = 0xEC;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE)
            {
              current_breakpoint->source_operation_data[0] = 0xE4;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
        }
      else if(!stricmp(tab_value[0],"CPY"))
        {
          if(address_mode == AM_IMMEDIATE_8)
            {
              current_breakpoint->source_operation_data[0] = 0xC0;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_IMMEDIATE_16)
            {
              current_breakpoint->source_operation_data[0] = 0xC0;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE)
            {
              current_breakpoint->source_operation_data[0] = 0xCC;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE)
            {
              current_breakpoint->source_operation_data[0] = 0xC4;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
        }
      else if(!stricmp(tab_value[0],"DEC"))
        {
          if(address_mode == AM_ABSOLUTE)
            {
              current_breakpoint->source_operation_data[0] = 0xCE;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE)
            {
              current_breakpoint->source_operation_data[0] = 0xC6;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE_INDEXED_X)
            {
              current_breakpoint->source_operation_data[0] = 0xDE;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDEXED_X)
            {
              current_breakpoint->source_operation_data[0] = 0xD6;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
        }
      else if(!stricmp(tab_value[0],"EOR"))
        {
          if(address_mode == AM_IMMEDIATE_8)
            {
              current_breakpoint->source_operation_data[0] = 0x49;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_IMMEDIATE_16)
            {
              current_breakpoint->source_operation_data[0] = 0x49;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE)
            {
              current_breakpoint->source_operation_data[0] = 0x4D;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE_LONG)
            {
              current_breakpoint->source_operation_data[0] = 0x4F;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE)
            {
              current_breakpoint->source_operation_data[0] = 0x45;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDIRECT)
            {
              current_breakpoint->source_operation_data[0] = 0x52;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDIRECT_LONG)
            {
              current_breakpoint->source_operation_data[0] = 0x47;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE_INDEXED_X)
            {
              current_breakpoint->source_operation_data[0] = 0x5D;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE_LONG_INDEXED_X)
            {
              current_breakpoint->source_operation_data[0] = 0x5F;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE_INDEXED_Y || address_mode == AM_DIRECT_PAGE_INDEXED_Y)    /* $08,Y doit aussi pouvoir tre interprett comme $0008,Y car DP,Y n'existe pas */
            {
              current_breakpoint->source_operation_data[0] = 0x59;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDEXED_X)
            {
              current_breakpoint->source_operation_data[0] = 0x55;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDEXED_X_INDIRECT)
            {
              current_breakpoint->source_operation_data[0] = 0x41;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDIRECT_INDEXED_Y)
            {
              current_breakpoint->source_operation_data[0] = 0x51;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDIRECT_LONG_INDEXED_Y)
            {
              current_breakpoint->source_operation_data[0] = 0x57;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_STACK_RELATIVE)
            {
              current_breakpoint->source_operation_data[0] = 0x43;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_STACK_RELATIVE_INDIRECT_INDEXED_Y)
            {
              current_breakpoint->source_operation_data[0] = 0x53;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
        }
      else if(!stricmp(tab_value[0],"INC"))
        {
          if(address_mode == AM_ABSOLUTE)
            {
              current_breakpoint->source_operation_data[0] = 0xEE;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE)
            {
              current_breakpoint->source_operation_data[0] = 0xE6;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE_INDEXED_X)
            {
              current_breakpoint->source_operation_data[0] = 0xFE;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDEXED_X)
            {
              current_breakpoint->source_operation_data[0] = 0xF6;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
        }
      else if(!stricmp(tab_value[0],"JMP"))
        {
          if(address_mode == AM_ABSOLUTE || address_mode == AM_DIRECT_PAGE)       /* On fait passer le $00 pour un $0000 */
            {
              current_breakpoint->source_operation_data[0] = 0x4C;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE_INDIRECT || address_mode == AM_DIRECT_PAGE_INDIRECT)       /* On fait passer le ($00) pour un ($0000) */
            {
              current_breakpoint->source_operation_data[0] = 0x6C;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE_INDEXED_X_INDIRECT)
            {
              current_breakpoint->source_operation_data[0] = 0x7C;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE_LONG)
            {
              current_breakpoint->source_operation_data[0] = 0x5C;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE_INDIRECT_LONG)
            {
              current_breakpoint->source_operation_data[0] = 0xDC;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
        }
      else if(!stricmp(tab_value[0],"JSL"))
        {
          if(address_mode == AM_DIRECT_PAGE || address_mode == AM_ABSOLUTE || address_mode == AM_ABSOLUTE_LONG)
            {
              current_breakpoint->source_operation_data[0] = 0x22;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
        }
      else if(!stricmp(tab_value[0],"JSR"))
        {
          if(address_mode == AM_ABSOLUTE || address_mode == AM_DIRECT_PAGE)       /* On fait passer le $00 pour un $0000 */
            {
              current_breakpoint->source_operation_data[0] = 0x20;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE_INDEXED_X_INDIRECT || address_mode == AM_DIRECT_PAGE_INDEXED_X_INDIRECT)       /* On fait passer le ($00,X) pour un ($0000,X) */
            {
              current_breakpoint->source_operation_data[0] = 0xFC;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
        }
      else if(!stricmp(tab_value[0],"LDA") || !stricmp(tab_value[0],"LDAL"))
        {
          if(address_mode == AM_IMMEDIATE_8)
            {
              current_breakpoint->source_operation_data[0] = 0xA9;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_IMMEDIATE_16)
            {
              current_breakpoint->source_operation_data[0] = 0xA9;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE)
            {
              current_breakpoint->source_operation_data[0] = 0xAD;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE_LONG)
            {
              current_breakpoint->source_operation_data[0] = 0xAF;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE)
            {
              current_breakpoint->source_operation_data[0] = 0xA5;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDIRECT || address_mode == AM_ABSOLUTE_INDIRECT)   /* le LDA ($0000) doit tre interprett comme un LDA ($00) */
            {
              current_breakpoint->source_operation_data[0] = 0xB2;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDIRECT_LONG || address_mode == AM_ABSOLUTE_INDIRECT_LONG)  /* le LDA [$0000] doit tre interprett comme un LDA [$00] */
            {
              current_breakpoint->source_operation_data[0] = 0xA7;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE_INDEXED_X)
            {
              current_breakpoint->source_operation_data[0] = 0xBD;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE_LONG_INDEXED_X)
            {
              current_breakpoint->source_operation_data[0] = 0xBF;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE_INDEXED_Y || address_mode == AM_DIRECT_PAGE_INDEXED_Y)   /* on avait un LDA $00,Y or le LDA DP,Y n'existe pas, donc a devait tre LDA $0000,Y */
            {
              current_breakpoint->source_operation_data[0] = 0xB9;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDEXED_X)
            {
              current_breakpoint->source_operation_data[0] = 0xB5;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDEXED_X_INDIRECT)
            {
              current_breakpoint->source_operation_data[0] = 0xA1;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDIRECT_INDEXED_Y)
            {
              current_breakpoint->source_operation_data[0] = 0xB1;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDIRECT_LONG_INDEXED_Y)
            {
              current_breakpoint->source_operation_data[0] = 0xB7;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_STACK_RELATIVE)
            {
              current_breakpoint->source_operation_data[0] = 0xA3;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_STACK_RELATIVE_INDIRECT_INDEXED_Y)
            {
              current_breakpoint->source_operation_data[0] = 0xB3;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
        }
      else if(!stricmp(tab_value[0],"LDX"))
        {
          if(address_mode == AM_IMMEDIATE_8)
            {
              current_breakpoint->source_operation_data[0] = 0xA2;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_IMMEDIATE_16)
            {
              current_breakpoint->source_operation_data[0] = 0xA2;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE || address_mode == AM_ABSOLUTE_LONG)   /* On cast l'adressage Long en adressage Abs */
            {
              current_breakpoint->source_operation_data[0] = 0xAE;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE)
            {
              current_breakpoint->source_operation_data[0] = 0xA6;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE_INDEXED_Y)
            {
              current_breakpoint->source_operation_data[0] = 0xBE;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDEXED_Y)
            {
              current_breakpoint->source_operation_data[0] = 0xB6;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
        }
      else if(!stricmp(tab_value[0],"LDY"))
        {
          if(address_mode == AM_IMMEDIATE_8)
            {
              current_breakpoint->source_operation_data[0] = 0xA0;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_IMMEDIATE_16)
            {
              current_breakpoint->source_operation_data[0] = 0xA0;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE || address_mode == AM_ABSOLUTE_LONG)   /* On cast l'adressage Long en adressage Abs */
            {
              current_breakpoint->source_operation_data[0] = 0xAC;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE)
            {
              current_breakpoint->source_operation_data[0] = 0xA4;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE_INDEXED_X)
            {
              current_breakpoint->source_operation_data[0] = 0xBC;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDEXED_X)
            {
              current_breakpoint->source_operation_data[0] = 0xB4;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
        }
      else if(!stricmp(tab_value[0],"LSR"))
        {
          if(address_mode == AM_ABSOLUTE)
            {
              current_breakpoint->source_operation_data[0] = 0x4E;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE)
            {
              current_breakpoint->source_operation_data[0] = 0x46;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE_INDEXED_X)
            {
              current_breakpoint->source_operation_data[0] = 0x5E;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDEXED_X)
            {
              current_breakpoint->source_operation_data[0] = 0x56;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
        }
      else if(!stricmp(tab_value[0],"MVN"))
        {
          if(address_mode == AM_BLOCK_MOVE)
            {
              current_breakpoint->source_operation_data[0] = 0x54;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
        }
      else if(!stricmp(tab_value[0],"MVP"))
        {
          if(address_mode == AM_BLOCK_MOVE)
            {
              current_breakpoint->source_operation_data[0] = 0x44;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
        }
      else if(!stricmp(tab_value[0],"ORA"))
        {
          if(address_mode == AM_IMMEDIATE_8)
            {
              current_breakpoint->source_operation_data[0] = 0x09;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_IMMEDIATE_16)
            {
              current_breakpoint->source_operation_data[0] = 0x09;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE)
            {
              current_breakpoint->source_operation_data[0] = 0x0D;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE_LONG)
            {
              current_breakpoint->source_operation_data[0] = 0x0F;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE)
            {
              current_breakpoint->source_operation_data[0] = 0x05;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDIRECT)
            {
              current_breakpoint->source_operation_data[0] = 0x12;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDIRECT_LONG)
            {
              current_breakpoint->source_operation_data[0] = 0x07;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE_INDEXED_X)
            {
              current_breakpoint->source_operation_data[0] = 0x1D;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE_LONG_INDEXED_X)
            {
              current_breakpoint->source_operation_data[0] = 0x1F;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE_INDEXED_Y || address_mode == AM_DIRECT_PAGE_INDEXED_Y)    /* $08,Y doit aussi pouvoir tre interprett comme $0008,Y car DP,Y n'existe pas */
            {
              current_breakpoint->source_operation_data[0] = 0x19;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDEXED_X)
            {
              current_breakpoint->source_operation_data[0] = 0x15;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDEXED_X_INDIRECT)
            {
              current_breakpoint->source_operation_data[0] = 0x01;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDIRECT_INDEXED_Y)
            {
              current_breakpoint->source_operation_data[0] = 0x11;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDIRECT_LONG_INDEXED_Y)
            {
              current_breakpoint->source_operation_data[0] = 0x17;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_STACK_RELATIVE)
            {
              current_breakpoint->source_operation_data[0] = 0x03;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_STACK_RELATIVE_INDIRECT_INDEXED_Y)
            {
              current_breakpoint->source_operation_data[0] = 0x13;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
        }
      else if(!stricmp(tab_value[0],"PEA"))
        {
          if(address_mode == AM_ABSOLUTE_LONG || address_mode == AM_ABSOLUTE || address_mode == AM_DIRECT_PAGE || address_mode == AM_IMMEDIATE_8 || address_mode == AM_IMMEDIATE_16)    /* On doit pouvoir crire PEA $1234 ou PEA $0 ou PEA #0 ou PEA #$1234 */
            {
              current_breakpoint->source_operation_data[0] = 0xF4;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
        }
      else if(!stricmp(tab_value[0],"PEI"))
        {
          if(address_mode == AM_DIRECT_PAGE_INDIRECT || address_mode == AM_DIRECT_PAGE || address_mode == AM_ABSOLUTE)    /* On doit pouvoir crire PEI $12 ou PEI ($12) ou PEI Label */
            {
              current_breakpoint->source_operation_data[0] = 0xD4;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
        }
      else if(!stricmp(tab_value[0],"PER"))
        {
          if(address_mode == AM_ABSOLUTE || address_mode == AM_IMMEDIATE_16)    /* PER Label ou PER #Label ou PER $address => Relatif Long comme le BRL */
            {
              current_breakpoint->source_operation_data[0] = 0x62;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
        }
      else if(!stricmp(tab_value[0],"REP"))
        {
          if(address_mode == AM_IMMEDIATE_8 || address_mode == AM_IMMEDIATE_16 || address_mode == AM_DIRECT_PAGE)   /* On peut tomber sur du REP $30 */
            {
              current_breakpoint->source_operation_data[0] = 0xC2;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
        }
      else if(!stricmp(tab_value[0],"ROL"))
        {
          if(address_mode == AM_ABSOLUTE || address_mode == AM_ABSOLUTE_LONG)   /* On cast le Long en Absolute */
            {
              current_breakpoint->source_operation_data[0] = 0x2E;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE)
            {
              current_breakpoint->source_operation_data[0] = 0x26;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE_INDEXED_X)
            {
              current_breakpoint->source_operation_data[0] = 0x3E;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDEXED_X)
            {
              current_breakpoint->source_operation_data[0] = 0x36;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
        }
      else if(!stricmp(tab_value[0],"ROR"))
        {
          if(address_mode == AM_ABSOLUTE || address_mode == AM_ABSOLUTE_LONG)   /* On cast le Long en Absolute */
            {
              current_breakpoint->source_operation_data[0] = 0x6E;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE)
            {
              current_breakpoint->source_operation_data[0] = 0x66;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE_INDEXED_X)
            {
              current_breakpoint->source_operation_data[0] = 0x7E;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDEXED_X)
            {
              current_breakpoint->source_operation_data[0] = 0x76;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
        }
      else if(!stricmp(tab_value[0],"SBC"))
        {
          if(address_mode == AM_IMMEDIATE_8)
            {
              current_breakpoint->source_operation_data[0] = 0xE9;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_IMMEDIATE_16)
            {
              current_breakpoint->source_operation_data[0] = 0xE9;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE)
            {
              current_breakpoint->source_operation_data[0] = 0xED;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE_LONG)
            {
              current_breakpoint->source_operation_data[0] = 0xEF;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE)
            {
              current_breakpoint->source_operation_data[0] = 0xE5;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDIRECT)
            {
              current_breakpoint->source_operation_data[0] = 0xF2;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDIRECT_LONG)
            {
              current_breakpoint->source_operation_data[0] = 0xE7;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE_INDEXED_X)
            {
              current_breakpoint->source_operation_data[0] = 0xFD;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE_LONG_INDEXED_X)
            {
              current_breakpoint->source_operation_data[0] = 0xFF;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE_INDEXED_Y || address_mode == AM_DIRECT_PAGE_INDEXED_Y)    /* $08,Y doit aussi pouvoir tre interprett comme $0008,Y car DP,Y n'existe pas */
            {
              current_breakpoint->source_operation_data[0] = 0xF9;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDEXED_X)
            {
              current_breakpoint->source_operation_data[0] = 0xF5;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDEXED_X_INDIRECT)
            {
              current_breakpoint->source_operation_data[0] = 0xE1;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDIRECT_INDEXED_Y)
            {
              current_breakpoint->source_operation_data[0] = 0xF1;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDIRECT_LONG_INDEXED_Y)
            {
              current_breakpoint->source_operation_data[0] = 0xF7;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_STACK_RELATIVE)
            {
              current_breakpoint->source_operation_data[0] = 0xE3;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_STACK_RELATIVE_INDIRECT_INDEXED_Y)
            {
              current_breakpoint->source_operation_data[0] = 0xF3;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
        }
      else if(!stricmp(tab_value[0],"SEP"))
        {
          if(address_mode == AM_IMMEDIATE_8 || address_mode == AM_IMMEDIATE_16 || address_mode == AM_DIRECT_PAGE)         /* On peut tompber sur du SEP $20 */
            {
              current_breakpoint->source_operation_data[0] = 0xE2;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
        }
      else if(!stricmp(tab_value[0],"STA") || !stricmp(tab_value[0],"STAL"))
        {
          if(address_mode == AM_ABSOLUTE)
            {
              current_breakpoint->source_operation_data[0] = 0x8D;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE_LONG)
            {
              current_breakpoint->source_operation_data[0] = 0x8F;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE)
            {
              current_breakpoint->source_operation_data[0] = 0x85;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDIRECT || address_mode == AM_ABSOLUTE_INDIRECT)             /* Le STA ($0000) peut aussi tre intepret comme STA ($00) */
            {
              current_breakpoint->source_operation_data[0] = 0x92;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDIRECT_LONG || address_mode == AM_ABSOLUTE_INDIRECT_LONG)   /* Le STA [$0000] peut aussi tre interprt comme STA [$00] */
            {
              current_breakpoint->source_operation_data[0] = 0x87;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE_INDEXED_X)
            {
              current_breakpoint->source_operation_data[0] = 0x9D;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE_LONG_INDEXED_X)
            {
              current_breakpoint->source_operation_data[0] = 0x9F;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE_INDEXED_Y || address_mode == AM_DIRECT_PAGE_INDEXED_Y)    /* On fait passer le $00,Y pour un $0000,Y */
            {
              current_breakpoint->source_operation_data[0] = 0x99;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDEXED_X)
            {
              current_breakpoint->source_operation_data[0] = 0x95;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDEXED_X_INDIRECT)
            {
              current_breakpoint->source_operation_data[0] = 0x81;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDIRECT_INDEXED_Y)
            {
              current_breakpoint->source_operation_data[0] = 0x91;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDIRECT_LONG_INDEXED_Y)
            {
              current_breakpoint->source_operation_data[0] = 0x97;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_STACK_RELATIVE)
            {
              current_breakpoint->source_operation_data[0] = 0x83;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_STACK_RELATIVE_INDIRECT_INDEXED_Y)
            {
              current_breakpoint->source_operation_data[0] = 0x93;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
        }
      else if(!stricmp(tab_value[0],"STX"))
        {
          if(address_mode == AM_ABSOLUTE)
            {
              current_breakpoint->source_operation_data[0] = 0x8E;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE)
            {
              current_breakpoint->source_operation_data[0] = 0x86;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDEXED_Y || address_mode == AM_ABSOLUTE_INDEXED_Y)             /* Le STX $0000,X peut aussi tre intepret comme STX $00,X */        
            {
              current_breakpoint->source_operation_data[0] = 0x96;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
        }
      else if(!stricmp(tab_value[0],"STY"))
        {
          if(address_mode == AM_ABSOLUTE)
            {
              current_breakpoint->source_operation_data[0] = 0x8C;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE)
            {
              current_breakpoint->source_operation_data[0] = 0x84;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDEXED_X)
            {
              current_breakpoint->source_operation_data[0] = 0x94;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
        }
      else if(!stricmp(tab_value[0],"STZ"))
        {
          if(address_mode == AM_ABSOLUTE)
            {
              current_breakpoint->source_operation_data[0] = 0x9C;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE)
            {
              current_breakpoint->source_operation_data[0] = 0x64;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_ABSOLUTE_INDEXED_X)
            {
              current_breakpoint->source_operation_data[0] = 0x9E;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE_INDEXED_X)
            {
              current_breakpoint->source_operation_data[0] = 0x74;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
        }
      else if(!stricmp(tab_value[0],"TRB"))
        {
          if(address_mode == AM_ABSOLUTE)
            {
              current_breakpoint->source_operation_data[0] = 0x1C;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE)
            {
              current_breakpoint->source_operation_data[0] = 0x14;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
        }
      else if(!stricmp(tab_value[0],"TSB"))
        {
          if(address_mode == AM_ABSOLUTE)
            {
              current_breakpoint->source_operation_data[0] = 0x0C;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
          else if(address_mode == AM_DIRECT_PAGE)
            {
              current_breakpoint->source_operation_data[0] = 0x04;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
        }
      else if(!stricmp(tab_value[0],"WDM"))
        {
          if(address_mode == AM_IMMEDIATE_8 || address_mode == AM_DIRECT_PAGE)
            {
              current_breakpoint->source_operation_data[0] = 0x42;
              mem_free_list(nb_value,tab_value);
              return(0);
            }
        }
      else
        {
          mem_free_list(nb_value,tab_value);
          return(1);
        }
    }

  /* Libration mmoire */
  mem_free_list(nb_value,tab_value);

  /* Error */
  return(1);
}


/**********************************************************************************************/
/*  GetAddressModeData() :  Dcode le mode d'adressage et rcupre les donnes de l'oprande. */
/**********************************************************************************************/
static int GetAddressModeData(char *operand, int *nb_byte_oprand_rtn, unsigned char *data_operand)
{
  int is_odd;

  /* #const 8 Immediate 8 bit : #$12 */
  if(strlen(operand) == 4 && operand[0] == '#' && operand[1] == '$' && 
     IsHexaDecimalChar(operand[2]) && IsHexaDecimalChar(operand[3]))
    {
      *nb_byte_oprand_rtn = 1;
      data_operand[0] = EvaluateAsOneByte(&operand[2],&is_odd);
      return(AM_IMMEDIATE_8);
    }

  /* #const 16  Immediate 16 bit : #$1234 */
  if(strlen(operand) == 6 && operand[0] == '#' && operand[1] == '$' && 
     IsHexaDecimalChar(operand[2]) && IsHexaDecimalChar(operand[3]) && IsHexaDecimalChar(operand[4]) && IsHexaDecimalChar(operand[5]))
    {
      *nb_byte_oprand_rtn = 2;
      data_operand[1] = EvaluateAsOneByte(&operand[2],&is_odd);
      data_operand[0] = EvaluateAsOneByte(&operand[4],&is_odd);
      return(AM_IMMEDIATE_16);
    }

  /* dp Direct Page : $12 */
  if(strlen(operand) == 3 && operand[0] == '$' && 
     IsHexaDecimalChar(operand[1]) && IsHexaDecimalChar(operand[2]))
    {
      *nb_byte_oprand_rtn = 1;
      data_operand[0] = EvaluateAsOneByte(&operand[1],&is_odd);
      return(AM_DIRECT_PAGE);
    }

  /* addr2 Absolute : $1243 */
  if(strlen(operand) == 5 && operand[0] == '$' && 
     IsHexaDecimalChar(operand[1]) && IsHexaDecimalChar(operand[2]) && IsHexaDecimalChar(operand[3]) && IsHexaDecimalChar(operand[4]))
    {
      *nb_byte_oprand_rtn = 2;
      data_operand[1] = EvaluateAsOneByte(&operand[1],&is_odd);
      data_operand[0] = EvaluateAsOneByte(&operand[3],&is_odd);
      return(AM_ABSOLUTE);
    }

  /* addr3 Absolute Long : $123456 */
  if(strlen(operand) == 7 && operand[0] == '$' && 
     IsHexaDecimalChar(operand[1]) && IsHexaDecimalChar(operand[2]) && IsHexaDecimalChar(operand[3]) && IsHexaDecimalChar(operand[4]) && IsHexaDecimalChar(operand[5]) && IsHexaDecimalChar(operand[6]))
    {
      *nb_byte_oprand_rtn = 3;
      data_operand[0] = EvaluateAsOneByte(&operand[5],&is_odd);
      data_operand[1] = EvaluateAsOneByte(&operand[3],&is_odd);
      data_operand[2] = EvaluateAsOneByte(&operand[1],&is_odd);
      return(AM_ABSOLUTE_LONG);
    }

  /* dp,X Direct Page Indexed,X : $12,X */
  if(strlen(operand) == 5 && operand[0] == '$' && operand[3] == ',' && toupper(operand[4]) == 'X' && 
     IsHexaDecimalChar(operand[1]) && IsHexaDecimalChar(operand[2]))
    {
      *nb_byte_oprand_rtn = 1;
      data_operand[0] = EvaluateAsOneByte(&operand[1],&is_odd);
      return(AM_DIRECT_PAGE_INDEXED_X);
    }

  /* dp,Y Direct Page Indexed,Y : $12,Y */
  if(strlen(operand) == 5 && operand[0] == '$' && operand[3] == ',' && toupper(operand[4]) == 'Y' && 
     IsHexaDecimalChar(operand[1]) && IsHexaDecimalChar(operand[2]))
    {
      *nb_byte_oprand_rtn = 1;
      data_operand[0] = EvaluateAsOneByte(&operand[1],&is_odd);
      return(AM_DIRECT_PAGE_INDEXED_Y);
    }

  /* addr2,X Absolute Indexed,X : $1234,X */
  if(strlen(operand) == 7 && operand[0] == '$' && operand[5] == ',' && toupper(operand[6]) == 'X' && 
     IsHexaDecimalChar(operand[1]) && IsHexaDecimalChar(operand[2]) && IsHexaDecimalChar(operand[3]) && IsHexaDecimalChar(operand[4]))
    {
      *nb_byte_oprand_rtn = 2;
      data_operand[0] = EvaluateAsOneByte(&operand[3],&is_odd);
      data_operand[1] = EvaluateAsOneByte(&operand[1],&is_odd);
      return(AM_ABSOLUTE_INDEXED_X);
    }

  /* addr2,Y Absolute Indexed,Y : $1234,Y */
  if(strlen(operand) == 7 && operand[0] == '$' && operand[5] == ',' && toupper(operand[6]) == 'Y' && 
     IsHexaDecimalChar(operand[1]) && IsHexaDecimalChar(operand[2]) && IsHexaDecimalChar(operand[3]) && IsHexaDecimalChar(operand[4]))
    {
      *nb_byte_oprand_rtn = 2;
      data_operand[0] = EvaluateAsOneByte(&operand[3],&is_odd);
      data_operand[1] = EvaluateAsOneByte(&operand[1],&is_odd);
      return(AM_ABSOLUTE_INDEXED_Y);
    }

  /* addr3,X Absolute Long Indexed,X : $123456,X */
  if(strlen(operand) == 9 && operand[0] == '$' && operand[7] == ',' && toupper(operand[8]) == 'X' && 
     IsHexaDecimalChar(operand[1]) && IsHexaDecimalChar(operand[2]) && IsHexaDecimalChar(operand[3]) && IsHexaDecimalChar(operand[4]) && IsHexaDecimalChar(operand[5]) && IsHexaDecimalChar(operand[6]))
    {
      *nb_byte_oprand_rtn = 3;
      data_operand[0] = EvaluateAsOneByte(&operand[5],&is_odd);
      data_operand[1] = EvaluateAsOneByte(&operand[3],&is_odd);
      data_operand[2] = EvaluateAsOneByte(&operand[1],&is_odd);
      return(AM_ABSOLUTE_LONG_INDEXED_X);
    }

  /* (dp) Direct Page Indirect : ($12) */
  if(strlen(operand) == 5 && operand[0] == '(' && operand[1] == '$' && operand[4] == ')' && 
     IsHexaDecimalChar(operand[2]) && IsHexaDecimalChar(operand[3]))
    {
      *nb_byte_oprand_rtn = 1;
      data_operand[0] = EvaluateAsOneByte(&operand[2],&is_odd);
      return(AM_DIRECT_PAGE_INDIRECT);
    }

  /* [dp] Direct Page Indirect Long : [$12] */
  if(strlen(operand) == 5 && operand[0] == '[' && operand[1] == '$' && operand[4] == ']' && 
     IsHexaDecimalChar(operand[2]) && IsHexaDecimalChar(operand[3]))
    {
      *nb_byte_oprand_rtn = 1;
      data_operand[0] = EvaluateAsOneByte(&operand[2],&is_odd);
      return(AM_DIRECT_PAGE_INDIRECT_LONG);
    }

  /* (addr2) Absolute Indirect : ($1234) */
  if(strlen(operand) == 7 && operand[0] == '(' && operand[1] == '$' && operand[6] == ')' && 
     IsHexaDecimalChar(operand[2]) && IsHexaDecimalChar(operand[3]))
    {
      *nb_byte_oprand_rtn = 2;
      data_operand[1] = EvaluateAsOneByte(&operand[2],&is_odd);
      data_operand[0] = EvaluateAsOneByte(&operand[4],&is_odd);
      return(AM_ABSOLUTE_INDIRECT);
    }

  /* [addr2] Absolute Indirect Long : [$1234] */
  if(strlen(operand) == 7 && operand[0] == '[' && operand[1] == '$' && operand[6] == ']' && 
     IsHexaDecimalChar(operand[2]) && IsHexaDecimalChar(operand[3]))
    {
      *nb_byte_oprand_rtn = 2;
      data_operand[1] = EvaluateAsOneByte(&operand[2],&is_odd);
      data_operand[0] = EvaluateAsOneByte(&operand[4],&is_odd);
      return(AM_ABSOLUTE_INDIRECT_LONG);
    }

  /* (addr2,X) Absolute Indexed,X Indirect : ($1234,X) */
  if(strlen(operand) == 9 && operand[0] == '(' && operand[1] == '$' && operand[6] == ',' && toupper(operand[7]) == 'X' && operand[8] == ',' && 
     IsHexaDecimalChar(operand[2]) && IsHexaDecimalChar(operand[3]))
    {
      *nb_byte_oprand_rtn = 2;
      data_operand[1] = EvaluateAsOneByte(&operand[2],&is_odd);
      data_operand[0] = EvaluateAsOneByte(&operand[4],&is_odd);
      return(AM_ABSOLUTE_INDEXED_X_INDIRECT);
    }

  /* (dp,X) Direct Page Indexed,X Indirect  : ($12,X) */
  if(strlen(operand) == 7 && operand[0] == '(' && operand[1] == '$' && operand[4] == ',' && toupper(operand[5]) == 'X' && operand[6] == ')' &&
     IsHexaDecimalChar(operand[2]) && IsHexaDecimalChar(operand[3]))
    {
      *nb_byte_oprand_rtn = 1;
      data_operand[0] = EvaluateAsOneByte(&operand[2],&is_odd);
      return(AM_DIRECT_PAGE_INDEXED_X_INDIRECT);
    }
  
  /* (dp),Y Direct Page Indirect Indexed,Y : ($12),Y */
  if(strlen(operand) == 7 && operand[0] == '(' && operand[1] == '$' && operand[4] == ')' && operand[5] == ',' && toupper(operand[6]) == 'Y' &&
     IsHexaDecimalChar(operand[2]) && IsHexaDecimalChar(operand[3]))
    {
      *nb_byte_oprand_rtn = 1;
      data_operand[0] = EvaluateAsOneByte(&operand[2],&is_odd);
      return(AM_DIRECT_PAGE_INDIRECT_INDEXED_Y);
    }

  /* [dp],Y Direct Page Indirect Long Indexed,Y : [$12],Y */
  if(strlen(operand) == 7 && operand[0] == '[' && operand[1] == '$' && operand[4] == ']' && operand[5] == ',' && toupper(operand[6]) == 'Y' &&
     IsHexaDecimalChar(operand[2]) && IsHexaDecimalChar(operand[3]))
    {
      *nb_byte_oprand_rtn = 1;
      data_operand[0] = EvaluateAsOneByte(&operand[2],&is_odd);
      return(AM_DIRECT_PAGE_INDIRECT_LONG_INDEXED_Y);
    }

  /* (sr,S),Y Stack Relative Indirect Indexed,Y : ($12,S),Y */
  if(strlen(operand) == 9 && operand[0] == '5' && operand[1] == '$' && operand[4] == ',' && toupper(operand[5]) == 'S' && operand[6] == ')' && operand[7] == ',' && toupper(operand[8]) == 'Y' &&
     IsHexaDecimalChar(operand[2]) && IsHexaDecimalChar(operand[3]))
    {
      *nb_byte_oprand_rtn = 1;
      data_operand[0] = EvaluateAsOneByte(&operand[2],&is_odd);
      return(AM_STACK_RELATIVE_INDIRECT_INDEXED_Y);
    }

  /* sr,S Stack Relative : $12,S */
  if(strlen(operand) == 5 && operand[0] == '5' && operand[3] == ',' && toupper(operand[4]) == 'S' &&
     IsHexaDecimalChar(operand[1]) && IsHexaDecimalChar(operand[2]))
    {
      *nb_byte_oprand_rtn = 1;
      data_operand[0] = EvaluateAsOneByte(&operand[1],&is_odd);
      return(AM_STACK_RELATIVE);
    }

  /* Src,Dst Block Move (MVN, MVP) : $12,$13 */
  if(strlen(operand) == 7 && operand[0] == '$' && operand[3] == ',' && operand[4] == '$' &&
     IsHexaDecimalChar(operand[1]) && IsHexaDecimalChar(operand[2]) && IsHexaDecimalChar(operand[5]) && IsHexaDecimalChar(operand[6]))
    {
      *nb_byte_oprand_rtn = 2;
      data_operand[1] = EvaluateAsOneByte(&operand[1],&is_odd);
      data_operand[0] = EvaluateAsOneByte(&operand[5],&is_odd);
      return(AM_BLOCK_MOVE);
    }

  /* Erreur */
  return(AM_UNKOWN);
}


/********************************************************************/
/*  InsertUniqueBreakpoint() :  Ajoute un breakpoint dans la liste. */
/********************************************************************/
static int InsertUniqueBreakpoint(struct cyrene_breakpoint *current_breakpoint, struct breakpoint_table *current_breakpoint_table)
{
  int is_same;
  struct cyrene_breakpoint *one_breakpoint;

  /* Ajoute si l'entre n'est pas l */
  for(one_breakpoint = current_breakpoint_table->first_breakpoint; one_breakpoint; one_breakpoint = one_breakpoint->next)
    {
      /* Compare les 2 breakpoints */
      is_same = CompareBreakpoint(current_breakpoint,one_breakpoint);
      if(is_same == 1)
        return(1);
    }

  /* Breakpoint ID */
  current_breakpoint->id = GetBreakpopintHighestID() + 1;

  /* Ajoute  la fin */
  current_breakpoint_table->nb_breakpoint++;
  if(current_breakpoint_table->first_breakpoint == NULL)
    current_breakpoint_table->first_breakpoint = current_breakpoint;
  else
    current_breakpoint_table->last_breakpoint->next = current_breakpoint;
  current_breakpoint_table->last_breakpoint = current_breakpoint;

  /* OK */
  return(0);
}


/****************************************************/
/*  CompareBreakpoint() :  Compare deux Breakpoint. */
/****************************************************/
static int CompareBreakpoint(struct cyrene_breakpoint *breakpoint_1, struct cyrene_breakpoint *breakpoint_2)
{
  int i;

  /** Source **/
  if(breakpoint_1->source != breakpoint_2->source)
    return(0);
  if(breakpoint_1->source == BREAKPOINT_SOURCE_ADDRESS)
    {
      if(strcmp(breakpoint_1->source_address_bank,breakpoint_2->source_address_bank))
        return(0);
      if(strcmp(breakpoint_1->source_address_address,breakpoint_2->source_address_address))
        return(0);
      if(strcmp(breakpoint_1->source_address_bank_dst,breakpoint_2->source_address_bank_dst))
        return(0);
      if(strcmp(breakpoint_1->source_address_address_dst,breakpoint_2->source_address_address_dst))
        return(0);
    }
  else if(breakpoint_1->source == BREAKPOINT_SOURCE_SYMBOL)
    {
      if(strcmp(breakpoint_1->source_symbol_name,breakpoint_2->source_symbol_name))
        return(0);
      if(strcmp(breakpoint_1->source_symbol_segment,breakpoint_2->source_symbol_segment))
        return(0);
      if(strcmp(breakpoint_1->source_symbol_program,breakpoint_2->source_symbol_program))
        return(0);
    }
  else if(breakpoint_1->source == BREAKPOINT_SOURCE_SYSTEMCALL)
    {
      if(breakpoint_1->source_systemcall_type != breakpoint_2->source_systemcall_type)
        return(0);
      if(breakpoint_1->source_systemcall_number != breakpoint_2->source_systemcall_number)
        return(0);
    }
  else if(breakpoint_1->source == BREAKPOINT_SOURCE_REGISTER)
    {
      if(breakpoint_1->source_register != breakpoint_2->source_register)
        return(0);
    }
  else if(breakpoint_1->source == BREAKPOINT_SOURCE_OPERATION)
    {
      if(breakpoint_1->source_operation_data_nb != breakpoint_2->source_operation_data_nb)
        return(0);
      for(i=0; i<breakpoint_1->source_operation_data_nb; i++)
        if(breakpoint_1->source_operation_data[i] != breakpoint_2->source_operation_data[i])
          return(0);
    }

  /** Trigger **/
  if(breakpoint_1->trigger != breakpoint_2->trigger)
    return(0);
  if(breakpoint_1->trigger == BREAKPOINT_TRIGGER_READ)
    {
      if(breakpoint_1->trigger_read_operator != breakpoint_2->trigger_read_operator)
        return(0);
      if(breakpoint_1->trigger_read_value_size != breakpoint_2->trigger_read_value_size)
        return(0);
      if(breakpoint_1->trigger_read_value != breakpoint_2->trigger_read_value)
        return(0);
    }
  else if(breakpoint_1->trigger == BREAKPOINT_TRIGGER_WRITE)
    {
      if(breakpoint_1->trigger_write_operator != breakpoint_2->trigger_write_operator)
        return(0);
      if(breakpoint_1->trigger_write_value_size != breakpoint_2->trigger_write_value_size)
        return(0);
      if(breakpoint_1->trigger_write_value != breakpoint_2->trigger_write_value)
        return(0);
    }
  else if(breakpoint_1->trigger == BREAKPOINT_TRIGGER_READWRITE)
    {
      if(breakpoint_1->trigger_readwrite_operator != breakpoint_2->trigger_readwrite_operator)
        return(0);
      if(breakpoint_1->trigger_readwrite_value_size != breakpoint_2->trigger_readwrite_value_size)
        return(0);
      if(breakpoint_1->trigger_readwrite_value != breakpoint_2->trigger_readwrite_value)
        return(0);
    }
  else if(breakpoint_1->trigger == BREAKPOINT_TRIGGER_HOLD)
    {
      if(breakpoint_1->trigger_hold_operator != breakpoint_2->trigger_hold_operator)
        return(0);
      if(breakpoint_1->trigger_hold_value_size != breakpoint_2->trigger_hold_value_size)
        return(0);
      if(breakpoint_1->trigger_hold_value != breakpoint_2->trigger_hold_value)
        return(0);
    }

  /** Action **/
  if(breakpoint_1->action != breakpoint_2->action)
    return(0);
  if(breakpoint_1->action == BREAKPOINT_ACTION_STOP)
    ;
  else if(breakpoint_1->action == BREAKPOINT_ACTION_PRINT)
    {
      if(strcmp(breakpoint_1->action_print_text,breakpoint_2->action_print_text))
        return(0);
    }
  else if(breakpoint_1->action == BREAKPOINT_ACTION_FILE)
    {
      if(strcmp(breakpoint_1->action_print_text,breakpoint_2->action_print_text))
        return(0);
      if(strcmp(breakpoint_1->action_file_path,breakpoint_2->action_file_path))
        return(0);
    }

  /** Repeat **/
  if(breakpoint_1->repeat != breakpoint_2->repeat)
    return(0);
  if(breakpoint_1->repeat == BREAKPOINT_REPEAT_ALWAYS)
    ;
  else if(breakpoint_1->repeat == BREAKPOINT_REPEAT_ONLY)
    {
      if(breakpoint_1->repeat_only_times != breakpoint_2->repeat_only_times)
        return(0);
    }
  else if(breakpoint_1->repeat == BREAKPOINT_REPEAT_AFTER)
    {
      if(breakpoint_1->repeat_only_times != breakpoint_2->repeat_only_times)
        return(0);
    }

  /* Identique */
  return(1);
}


/**********************************************************************************************/
/*  UpdateBreakpointExtraInformation() :  Met  jour les donnes contextuelles du Breakpoint. */
/**********************************************************************************************/
static void UpdateBreakpointExtraInformation(struct window_breakpoint *window_breakpoint, struct cyrene_breakpoint *current_breakpoint)
{
  int i;
  struct window_main *window_main;
  char buffer_value[BUFFER_SIZE];

  /* Rcupre la fentre Main */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  if(window_main == NULL)
    return;

  /** On met  jour avec les donnes du Breakpoint **/
  if(current_breakpoint != NULL)
    {
      /* RAZ */
      RAZBreakpoint(window_breakpoint);

      /*** Met  jour les donnes  l'cran ***/
      /** Source **/
      if(current_breakpoint->source == BREAKPOINT_SOURCE_ADDRESS)
        {
          /* Controles */
          SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_ADDRESS,BM_SETCHECK,1,0);
          SetWindowText(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_BANK),current_breakpoint->source_address_bank);
          SetWindowText(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_ADDRESS),current_breakpoint->source_address_address);
          SetWindowText(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_BANK_DST),current_breakpoint->source_address_bank_dst);
          SetWindowText(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_ADDRESS_DST),current_breakpoint->source_address_address_dst);

          /* Breakpoint de la fentre */
          window_breakpoint->source_radio = BREAKPOINT_SOURCE_ADDRESS;
          strcpy(window_breakpoint->source_address_bank,current_breakpoint->source_address_bank);
          strcpy(window_breakpoint->source_address_address,current_breakpoint->source_address_address);
          strcpy(window_breakpoint->source_address_bank_dst,current_breakpoint->source_address_bank_dst);
          strcpy(window_breakpoint->source_address_address_dst,current_breakpoint->source_address_address_dst);
        }
      else if(current_breakpoint->source == BREAKPOINT_SOURCE_SYMBOL)
        {
          /* Controles */
          SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_SYMBOL,BM_SETCHECK,1,0);
          if(current_breakpoint->source_symbol_begin_address != 0xFFFFFFFF)
            {
              /* Adresse Dbut */
              sprintf(buffer_value,"%02X",(current_breakpoint->source_symbol_begin_address >> 16) & 0x000000FF);
              SetWindowText(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_BANK),buffer_value);
              sprintf(buffer_value,"%04X",current_breakpoint->source_symbol_begin_address & 0x0000FFFF);
              SetWindowText(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_ADDRESS),buffer_value);
            }
          if(current_breakpoint->source_symbol_end_address != 0xFFFFFFFF)
            {
              /* Adresse Fin de la zone Data */
              sprintf(buffer_value,"%02X",(current_breakpoint->source_symbol_end_address >> 16) & 0x000000FF);
              SetWindowText(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_BANK_DST),buffer_value);
              sprintf(buffer_value,"%04X",current_breakpoint->source_symbol_end_address & 0x0000FFFF);
              SetWindowText(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_ADDRESS_DST),buffer_value);
            }

          /* Breakpoint de la fentre */
          window_breakpoint->source_radio = BREAKPOINT_SOURCE_SYMBOL;
          strcpy(window_breakpoint->source_symbol_name,current_breakpoint->source_symbol_name);
          strcpy(window_breakpoint->source_symbol_segment,current_breakpoint->source_symbol_segment);
          strcpy(window_breakpoint->source_symbol_program,current_breakpoint->source_symbol_program);
          window_breakpoint->source_symbol_begin_address = current_breakpoint->source_symbol_begin_address;
          window_breakpoint->source_symbol_end_address = current_breakpoint->source_symbol_end_address;
          /* MAJ L'affichage Nom Symbol */
          sprintf(buffer_value,"%s",current_breakpoint->source_symbol_name);
          SetWindowText(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_LABEL_SYMBOL1),buffer_value);
          sprintf(buffer_value,"[%s | %s]",current_breakpoint->source_symbol_segment,current_breakpoint->source_symbol_program);
          SetWindowText(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_LABEL_SYMBOL2),buffer_value);
        }
      else if(current_breakpoint->source == BREAKPOINT_SOURCE_SYSTEMCALL)
        {
          /* Controles */
          SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_SYSTEMCALL,BM_SETCHECK,1,0);
          SendMessage(window_breakpoint->hwnd_combo_systemcall,CB_SETCURSEL,current_breakpoint->source_systemcall_index,0);
          SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_ADDRESS,BM_SETCHECK,0,0);

          /* Breakpoint de la fentre */
          window_breakpoint->source_radio = BREAKPOINT_SOURCE_SYSTEMCALL;
          window_breakpoint->source_systemcall_type = current_breakpoint->source_systemcall_type;
          window_breakpoint->source_systemcall_number = current_breakpoint->source_systemcall_number;
          strcpy(window_breakpoint->source_systemcall_text,current_breakpoint->source_systemcall_text);
          window_breakpoint->source_systemcall_index = current_breakpoint->source_systemcall_index;
        }
      else if(current_breakpoint->source == BREAKPOINT_SOURCE_REGISTER)
        {
          /* Controles */
          SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_REGISTER,BM_SETCHECK,1,0);
          SendMessage(window_breakpoint->hwnd_combo_register,CB_SETCURSEL,current_breakpoint->source_register,0);
          SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_ADDRESS,BM_SETCHECK,0,0);

          /* Breakpoint de la fentre */
          window_breakpoint->source_radio = BREAKPOINT_SOURCE_REGISTER;
          window_breakpoint->source_register = current_breakpoint->source_register;
        }
      else if(current_breakpoint->source == BREAKPOINT_SOURCE_OPERATION)
        {
          /* Controles */
          SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_OPERATION,BM_SETCHECK,1,0);
          SetWindowText(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_OPERATION_DATA),current_breakpoint->source_operation_text);
          strcpy(buffer_value,"");
          for(i=0; i<current_breakpoint->source_operation_data_nb; i++)
             sprintf(&buffer_value[strlen(buffer_value)],"%02X ",current_breakpoint->source_operation_data[i]);
          SetWindowText(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_STATIC_OPERATION_DATA),buffer_value);

          /* Breakpoint de la fentre */
          window_breakpoint->source_radio = BREAKPOINT_SOURCE_OPERATION;
          window_breakpoint->source_operation_data_nb = current_breakpoint->source_operation_data_nb;
          for(i=0; i<current_breakpoint->source_operation_data_nb; i++)
             window_breakpoint->source_operation_data[i] = current_breakpoint->source_operation_data[i];
          strcpy(window_breakpoint->source_operation_text,current_breakpoint->source_operation_text);
        }

      /** Trigger **/
      if(current_breakpoint->trigger == BREAKPOINT_TRIGGER_READ)
        {
          /* Controles */
          SendMessage(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_READ),BM_SETCHECK,1,0);
          SendMessage(window_breakpoint->hwnd_combo_read_operator,CB_SETCURSEL,current_breakpoint->trigger_read_operator,0);  
          if(current_breakpoint->trigger_read_value_size == 2)
            sprintf(buffer_value,"%04X",current_breakpoint->trigger_read_value);
          else if(current_breakpoint->trigger_read_value_size == 1)
            sprintf(buffer_value,"%02X",current_breakpoint->trigger_read_value);
          else
            strcpy(buffer_value,"");
          SetWindowText(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_READ_VALUE),buffer_value);

          /* Breakpoint de la fentre */
          window_breakpoint->trigger_radio = BREAKPOINT_TRIGGER_READ;
          window_breakpoint->trigger_read_operator = current_breakpoint->trigger_read_operator;
          window_breakpoint->trigger_read_operator_value_size = current_breakpoint->trigger_read_value_size;
          window_breakpoint->trigger_read_operator_value = current_breakpoint->trigger_read_value;
        }
      else if(current_breakpoint->trigger == BREAKPOINT_TRIGGER_WRITE)
        {
          /* Controles */
          SendMessage(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_READ),BM_SETCHECK,0,0);
          SendMessage(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_WRITE),BM_SETCHECK,1,0);
          SendMessage(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_READWRITE),BM_SETCHECK,0,0);
          SendMessage(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_EXECUTE),BM_SETCHECK,0,0);
          SendMessage(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_HOLD),BM_SETCHECK,0,0);
          SendMessage(window_breakpoint->hwnd_combo_write_operator,CB_SETCURSEL,current_breakpoint->trigger_write_operator,0);   
          if(current_breakpoint->trigger_write_value_size == 2)
            sprintf(buffer_value,"%04X",current_breakpoint->trigger_write_value);
          else if(current_breakpoint->trigger_write_value_size == 1)
            sprintf(buffer_value,"%02X",current_breakpoint->trigger_write_value);
          else
            strcpy(buffer_value,"");
          SetWindowText(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_WRITE_VALUE),buffer_value);

          /* Breakpoint de la fentre */
          window_breakpoint->trigger_radio = BREAKPOINT_TRIGGER_WRITE;
          window_breakpoint->trigger_write_operator = current_breakpoint->trigger_write_operator;
          window_breakpoint->trigger_write_operator_value_size = current_breakpoint->trigger_write_value_size;
          window_breakpoint->trigger_write_operator_value = current_breakpoint->trigger_write_value;
        }
      else if(current_breakpoint->trigger == BREAKPOINT_TRIGGER_READWRITE)
        {
          /* Controles */
          SendMessage(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_READ),BM_SETCHECK,0,0);
          SendMessage(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_WRITE),BM_SETCHECK,0,0);
          SendMessage(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_READWRITE),BM_SETCHECK,1,0);
          SendMessage(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_EXECUTE),BM_SETCHECK,0,0);
          SendMessage(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_HOLD),BM_SETCHECK,0,0);
          SendMessage(window_breakpoint->hwnd_combo_readwrite_operator,CB_SETCURSEL,current_breakpoint->trigger_readwrite_operator,0);   
          if(current_breakpoint->trigger_readwrite_value_size == 2)
            sprintf(buffer_value,"%04X",current_breakpoint->trigger_readwrite_value);
          else if(current_breakpoint->trigger_readwrite_value_size == 1)
            sprintf(buffer_value,"%02X",current_breakpoint->trigger_readwrite_value);
          else
            strcpy(buffer_value,"");
          SetWindowText(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_READWRITE_VALUE),buffer_value);

          /* Breakpoint de la fentre */
          window_breakpoint->trigger_radio = BREAKPOINT_TRIGGER_READWRITE;
          window_breakpoint->trigger_readwrite_operator = current_breakpoint->trigger_readwrite_operator;
          window_breakpoint->trigger_readwrite_operator_value_size = current_breakpoint->trigger_readwrite_value_size;
          window_breakpoint->trigger_readwrite_operator_value = current_breakpoint->trigger_readwrite_value;
        }
      else if(current_breakpoint->trigger == BREAKPOINT_TRIGGER_EXECUTE)
        {
          /* Controles */
          SendMessage(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_READ),BM_SETCHECK,0,0);
          SendMessage(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_WRITE),BM_SETCHECK,0,0);
          SendMessage(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_EXECUTE),BM_SETCHECK,1,0);
          SendMessage(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_HOLD),BM_SETCHECK,0,0);

          /* Breakpoint de la fentre */
          window_breakpoint->trigger_radio = BREAKPOINT_TRIGGER_EXECUTE;
        }
      else if(current_breakpoint->trigger == BREAKPOINT_TRIGGER_HOLD)
        {
          /* Controles */
          SendMessage(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_READ),BM_SETCHECK,0,0);
          SendMessage(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_WRITE),BM_SETCHECK,0,0);
          SendMessage(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_EXECUTE),BM_SETCHECK,0,0);
          SendMessage(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_HOLD),BM_SETCHECK,1,0);
          SendMessage(window_breakpoint->hwnd_combo_hold_operator,CB_SETCURSEL,current_breakpoint->trigger_hold_operator,0);   
          if(current_breakpoint->trigger_hold_value_size == 2)
            sprintf(buffer_value,"%04X",current_breakpoint->trigger_hold_value);
          else if(current_breakpoint->trigger_hold_value_size == 1)
            sprintf(buffer_value,"%02X",current_breakpoint->trigger_hold_value);
          else
            strcpy(buffer_value,"");
          SetWindowText(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_HOLD_VALUE),buffer_value);

          /* Breakpoint de la fentre */
          window_breakpoint->trigger_radio = BREAKPOINT_TRIGGER_HOLD;
          window_breakpoint->trigger_hold_operator = current_breakpoint->trigger_hold_operator;
          window_breakpoint->trigger_hold_operator_value_size = current_breakpoint->trigger_hold_value_size;
          window_breakpoint->trigger_hold_operator_value = current_breakpoint->trigger_hold_value;
        }

      /** Action **/
      if(current_breakpoint->action == BREAKPOINT_ACTION_STOP)
        {
          /* Controles */
          SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_STOP_EXECUTION,BM_SETCHECK,1,0);

          /* Breakpoint de la fentre */
          window_breakpoint->action_radio = BREAKPOINT_ACTION_STOP;
        }
      else if(current_breakpoint->action == BREAKPOINT_ACTION_PRINT)
        {
          /* Controles */
          SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_PRINT,BM_SETCHECK,1,0);
          SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_STOP_EXECUTION,BM_SETCHECK,0,0);
          SetWindowText(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_PRINT_DATA),current_breakpoint->action_print_text);

          /* Breakpoint de la fentre */
          window_breakpoint->action_radio = BREAKPOINT_ACTION_PRINT;
          strcpy(window_breakpoint->action_print_data,current_breakpoint->action_print_text);
        }
      else if(current_breakpoint->action == BREAKPOINT_ACTION_FILE)
        {
          /* Controles */
          SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_FILE,BM_SETCHECK,1,0);
          SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_STOP_EXECUTION,BM_SETCHECK,0,0);
          SetWindowText(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_PRINT_DATA),current_breakpoint->action_print_text);
          SetWindowText(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_FILE_PATH),current_breakpoint->action_file_path);

          /* Breakpoint de la fentre */
          window_breakpoint->action_radio = BREAKPOINT_ACTION_FILE;
          strcpy(window_breakpoint->action_print_data,current_breakpoint->action_print_text);
          strcpy(window_breakpoint->action_file_path,current_breakpoint->action_file_path);
        }

      /** Repeat **/
      if(current_breakpoint->repeat == BREAKPOINT_REPEAT_ALWAYS)
        {
          /* Controles */
          SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_REPEAT_ALWAYS,BM_SETCHECK,1,0);

          /* Breakpoint de la fentre */
          window_breakpoint->repeat_radio = BREAKPOINT_REPEAT_ALWAYS;
        }
      else if(current_breakpoint->repeat == BREAKPOINT_REPEAT_ONLY)
        {
          /* Controles */
          SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_REPEAT_ONLY,BM_SETCHECK,1,0);
          SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_REPEAT_ALWAYS,BM_SETCHECK,0,0);
          sprintf(buffer_value,"%d",current_breakpoint->repeat_only_times);
          SetWindowText(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_REPEAT_ONLY_DATA),buffer_value);

          /* Breakpoint de la fentre */
          window_breakpoint->repeat_radio = BREAKPOINT_REPEAT_ONLY;
          window_breakpoint->repeat_only_times = current_breakpoint->repeat_only_times;
        }
      else if(current_breakpoint->repeat == BREAKPOINT_REPEAT_AFTER)
        {
          /* Controles */
          SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_REPEAT_AFTER,BM_SETCHECK,1,0);
          SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_REPEAT_ALWAYS,BM_SETCHECK,0,0);
          sprintf(buffer_value,"%d",current_breakpoint->repeat_only_times);
          SetWindowText(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_REPEAT_ONLY_DATA),buffer_value);

          /* Breakpoint de la fentre */
          window_breakpoint->repeat_radio = BREAKPOINT_REPEAT_AFTER;
          window_breakpoint->repeat_only_times = current_breakpoint->repeat_only_times;
        }

      /* On est en mode lecture seule */
      UpdateBreakpointEditMode(window_breakpoint,0);   /* Edit Mode = Disable */

      /* Breakpoint slectionn */
      window_breakpoint->selected_breakpoint = current_breakpoint;
    }
  else
    {
      /* On efface tout */
      RAZBreakpoint(window_breakpoint);                /* Clear */
      UpdateBreakpointEditMode(window_breakpoint,0);   /* Edit Mode = Disable */
      window_breakpoint->selected_breakpoint = NULL;   /* Aucun Breakpoint selectionn */
    }
}


/*****************************************************************************/
/*  UpdateBreakpointSource() :  Met  jour les donnes Source du Breakpoint. */
/*****************************************************************************/
static void UpdateBreakpointSource(struct window_breakpoint *window_breakpoint, int edit_mode_status)
{
  /*** Met  jour les donnes Source  l'cran ***/
  if(window_breakpoint->source_radio == BREAKPOINT_SOURCE_ADDRESS)
    {
      /* On slectionne 'Address' */
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_ADDRESS,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_SYMBOL,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_SYSTEMCALL,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_OPERATION,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_REGISTER,BM_SETCHECK,0,0);

      /* Enable les contrles de Address */
      if(edit_mode_status == 1)
        {
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_BANK),1);
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_ADDRESS),1);
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_BANK_DST),1);
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_ADDRESS_DST),1);
        }

      /* Disable les contrles de Symbol / SystemCall / Operation / Register */
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_LABEL_SYMBOL1),0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_LABEL_SYMBOL2),0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_BUTTON_SYMBOL),0);
      EnableWindow(window_breakpoint->hwnd_combo_systemcall,0);
      EnableWindow(window_breakpoint->hwnd_combo_register,0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_OPERATION_DATA),0);

      /* Trigger : On va forcer 'Read' */
      window_breakpoint->trigger_radio = BREAKPOINT_TRIGGER_READ;
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_READ,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_WRITE,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_READWRITE,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_EXECUTE,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_HOLD,BM_SETCHECK,0,0);

      /* Enable Trigger : Read / Write / ReadWrite / Execute */
      if(edit_mode_status == 1)
        {
          /* Read / Write / ReadWrite / Execute */
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_READ),1);
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_WRITE),1);
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_READWRITE),1);
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_EXECUTE),1);

          /* Enable : Read Operator + Value */
          EnableWindow(window_breakpoint->hwnd_combo_read_operator,1);
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_READ_VALUE),1);
        }

      /* Disable : Write Operator + Value */
      EnableWindow(window_breakpoint->hwnd_combo_write_operator,0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_WRITE_VALUE),0);

      /* Disable : ReadWrite Operator + Value */
      EnableWindow(window_breakpoint->hwnd_combo_readwrite_operator,0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_READWRITE_VALUE),0);

      /* Disable Trigger : Pas de Hold pour les Address */
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_HOLD),0);
      EnableWindow(window_breakpoint->hwnd_combo_hold_operator,0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_HOLD_VALUE),0);
    }
  else if(window_breakpoint->source_radio == BREAKPOINT_SOURCE_SYMBOL)
    {
      /* On slectionne 'Symbol' */
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_ADDRESS,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_SYMBOL,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_SYSTEMCALL,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_OPERATION,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_REGISTER,BM_SETCHECK,0,0);

      /* Enable les contrles de Symbol */
      if(edit_mode_status == 1)
        {
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_LABEL_SYMBOL1),1);
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_LABEL_SYMBOL2),1);
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_BUTTON_SYMBOL),1);
        }

      /* Disable les controles de Adresse / SystemCall / Operatiopn / Register */
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_BANK),0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_ADDRESS),0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_BANK_DST),0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_ADDRESS_DST),0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_OPERATION_DATA),0);
      EnableWindow(window_breakpoint->hwnd_combo_systemcall,0);
      EnableWindow(window_breakpoint->hwnd_combo_register,0);

      /* Trigger : On va forcer 'Read' */
      window_breakpoint->trigger_radio = BREAKPOINT_TRIGGER_READ;
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_READ,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_WRITE,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_READWRITE,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_EXECUTE,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_HOLD,BM_SETCHECK,0,0);

      /* Enable Trigger : Read / Write / ReadWrite / Execute */
      if(edit_mode_status == 1)
        {
          /* Read / Write / ReadWrite / Execute */
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_READ),1);
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_WRITE),1);
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_READWRITE),1);
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_EXECUTE),1);

          /* Read Operator + Value */
          EnableWindow(window_breakpoint->hwnd_combo_read_operator,1);
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_READ_VALUE),1);
        }

      /* Disable : Write Operator + Value */
      EnableWindow(window_breakpoint->hwnd_combo_write_operator,0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_WRITE_VALUE),0);

      /* Disable : ReadWrite Operator + Value */
      EnableWindow(window_breakpoint->hwnd_combo_readwrite_operator,0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_READWRITE_VALUE),0);

      /* Disable Trigger : Pas de Hold pour les Symbol */
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_HOLD),0);
      EnableWindow(window_breakpoint->hwnd_combo_hold_operator,0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_HOLD_VALUE),0);
    }
  else if(window_breakpoint->source_radio == BREAKPOINT_SOURCE_SYSTEMCALL)
    {
      /* On slectionne 'SystemCall' */
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_ADDRESS,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_SYMBOL,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_SYSTEMCALL,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_OPERATION,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_REGISTER,BM_SETCHECK,0,0);

      /* Enable les contrles de SystemCall */
      if(edit_mode_status == 1)
        EnableWindow(window_breakpoint->hwnd_combo_systemcall,1);

      /* Disable les controles de Adresse / Symbol / SystemCall / Operatiopn / Register */
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_BANK),0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_ADDRESS),0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_BANK_DST),0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_ADDRESS_DST),0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_OPERATION_DATA),0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_BUTTON_SYMBOL),0);
      EnableWindow(window_breakpoint->hwnd_combo_register,0);

      /* Trigger : On va forcer 'Execute' */
      window_breakpoint->trigger_radio = BREAKPOINT_TRIGGER_EXECUTE;
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_READ,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_WRITE,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_READWRITE,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_EXECUTE,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_HOLD,BM_SETCHECK,0,0);

      /* Enable Trigger : Execute */
      if(edit_mode_status == 1)
        {
          /* Disable Read / Write / ReadWrite */
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_READ),0);
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_WRITE),0);
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_READWRITE),0);

          /* Enable Execute */
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_EXECUTE),1);
        }

      /* Disable : Read Operator + Value */
      EnableWindow(window_breakpoint->hwnd_combo_read_operator,0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_READ_VALUE),0);

      /* Disable : Write Operator + Value */
      EnableWindow(window_breakpoint->hwnd_combo_write_operator,0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_WRITE_VALUE),0);

      /* Disable : ReadWrite Operator + Value */
      EnableWindow(window_breakpoint->hwnd_combo_readwrite_operator,0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_READWRITE_VALUE),0);

      /* Disable Trigger : Pas de Hold pour les SystemCall */
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_HOLD),0);
      EnableWindow(window_breakpoint->hwnd_combo_hold_operator,0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_HOLD_VALUE),0);
    }
  else if(window_breakpoint->source_radio == BREAKPOINT_SOURCE_OPERATION)
    {
      /* On slectionne 'Operation' */
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_ADDRESS,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_SYMBOL,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_SYSTEMCALL,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_OPERATION,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_REGISTER,BM_SETCHECK,0,0);

      /* Enable des contrles de Operation */
      if(edit_mode_status == 1)
        EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_OPERATION_DATA),1);

      /* Disable les controles de Address / SystemCall / Symbol / Register */
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_BANK),0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_ADDRESS),0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_BANK_DST),0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_ADDRESS_DST),0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_LABEL_SYMBOL1),0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_LABEL_SYMBOL2),0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_BUTTON_SYMBOL),0);
      EnableWindow(window_breakpoint->hwnd_combo_systemcall,0);
      EnableWindow(window_breakpoint->hwnd_combo_register,0);

      /* Trigger : On va forcer 'Execute' */
      window_breakpoint->trigger_radio = BREAKPOINT_TRIGGER_EXECUTE;
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_READ,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_WRITE,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_EXECUTE,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_HOLD,BM_SETCHECK,0,0);

      /* Enable Trigger : Execute */
      if(edit_mode_status == 1)
        EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_EXECUTE),1);

      /* Disable Trigger : Pas de Read / Write / ReadWrite / Hold pour les Operation */
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_READ),0);
      EnableWindow(window_breakpoint->hwnd_combo_read_operator,0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_READ_VALUE),0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_WRITE),0);
      EnableWindow(window_breakpoint->hwnd_combo_write_operator,0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_WRITE_VALUE),0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_READWRITE),0);
      EnableWindow(window_breakpoint->hwnd_combo_readwrite_operator,0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_READWRITE_VALUE),0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_HOLD),0);
      EnableWindow(window_breakpoint->hwnd_combo_hold_operator,0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_HOLD_VALUE),0);
    }
  else if(window_breakpoint->source_radio == BREAKPOINT_SOURCE_REGISTER)
    {
      /* On slectionne 'Register' */
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_ADDRESS,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_SYMBOL,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_SYSTEMCALL,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_OPERATION,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_REGISTER,BM_SETCHECK,1,0);

      /* Enable des contrles de Register */
      if(edit_mode_status == 1)
        EnableWindow(window_breakpoint->hwnd_combo_register,1);

      /* Disable les controles de Address / SystemCall / Symbol / Operation */
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_BANK),0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_ADDRESS),0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_BANK_DST),0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_ADDRESS_DST),0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_LABEL_SYMBOL1),0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_LABEL_SYMBOL2),0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_BUTTON_SYMBOL),0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_OPERATION_DATA),0);
      EnableWindow(window_breakpoint->hwnd_combo_systemcall,0);

      /* Trigger : On va forcer 'Hold' */
      window_breakpoint->trigger_radio = BREAKPOINT_TRIGGER_HOLD;
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_READ,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_WRITE,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_READWRITE,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_EXECUTE,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_HOLD,BM_SETCHECK,1,0);

      /* Enable Trigger : Hold */
      if(edit_mode_status == 1)
        {
          /* Hold */
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_HOLD),1);

          /* Hold Operator + Value */
          EnableWindow(window_breakpoint->hwnd_combo_hold_operator,1);
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_HOLD_VALUE),1);
        }

      /* Disable Trigger : Pas de Read / Write / ReadWrite /Execute pour les Register */
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_READ),0);
      EnableWindow(window_breakpoint->hwnd_combo_read_operator,0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_READ_VALUE),0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_WRITE),0);
      EnableWindow(window_breakpoint->hwnd_combo_write_operator,0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_WRITE_VALUE),0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_READWRITE),0);
      EnableWindow(window_breakpoint->hwnd_combo_readwrite_operator,0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_READWRITE_VALUE),0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_EXECUTE),0);
    }
}


/*******************************************************************************/
/*  UpdateBreakpointTrigger() :  Met  jour les donnes Trigger du Breakpoint. */
/*******************************************************************************/
static void UpdateBreakpointTrigger(struct window_breakpoint *window_breakpoint, int edit_mode_status)
{
  /*** Met  jour les donnes Trigger  l'cran ***/
  /** Disable **/
  /* Read */
  SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_READ,BM_SETCHECK,0,0);
  EnableWindow(window_breakpoint->hwnd_combo_read_operator,0);
  EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_READ_VALUE),0);
  /* Write */
  SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_WRITE,BM_SETCHECK,0,0);
  EnableWindow(window_breakpoint->hwnd_combo_write_operator,0);
  EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_WRITE_VALUE),0);
  /* ReadWrite */
  SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_READWRITE,BM_SETCHECK,0,0);
  EnableWindow(window_breakpoint->hwnd_combo_readwrite_operator,0);
  EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_READWRITE_VALUE),0);
  /* Execute */
  SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_EXECUTE,BM_SETCHECK,0,0);
  /* Hold */
  SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_HOLD,BM_SETCHECK,0,0);
  EnableWindow(window_breakpoint->hwnd_combo_hold_operator,0);
  EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_HOLD_VALUE),0);

  /** Read **/
  if(window_breakpoint->trigger_radio == BREAKPOINT_TRIGGER_READ)
    {
      /* Read */
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_READ,BM_SETCHECK,1,0);
      /* Enable */
      if(edit_mode_status == 1)
        {
          EnableWindow(window_breakpoint->hwnd_combo_read_operator,1);
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_READ_VALUE),1);
        }
    }
  /** Write **/
  else if(window_breakpoint->trigger_radio == BREAKPOINT_TRIGGER_WRITE)
    {
      /* Write */
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_WRITE,BM_SETCHECK,1,0);
      /* Enable */
      if(edit_mode_status == 1)
        {
          EnableWindow(window_breakpoint->hwnd_combo_write_operator,1);
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_WRITE_VALUE),1);
        }
    }
  /** Read | Write **/
  else if(window_breakpoint->trigger_radio == BREAKPOINT_TRIGGER_READWRITE)
    {
      /* Write */
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_READWRITE,BM_SETCHECK,1,0);
      /* Enable */
      if(edit_mode_status == 1)
        {
          EnableWindow(window_breakpoint->hwnd_combo_readwrite_operator,1);
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_READWRITE_VALUE),1);
        }
    }
  /** Execute **/
  else if(window_breakpoint->trigger_radio == BREAKPOINT_TRIGGER_EXECUTE)
    SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_EXECUTE,BM_SETCHECK,1,0);
  /** Hold **/
  else if(window_breakpoint->trigger_radio == BREAKPOINT_TRIGGER_HOLD)
    {
      /* Hold */
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_HOLD,BM_SETCHECK,1,0);
      /* Enable */
      if(edit_mode_status == 1)
        {
          EnableWindow(window_breakpoint->hwnd_combo_hold_operator,1);
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_HOLD_VALUE),1);
        }
    }
}


/*****************************************************************************/
/*  UpdateBreakpointAction() :  Met  jour les donnes Action du Breakpoint. */
/*****************************************************************************/
static void UpdateBreakpointAction(struct window_breakpoint *window_breakpoint, int edit_mode_status)
{
  /*** Met  jour les donnes Action  l'cran ***/
  if(window_breakpoint->action_radio == BREAKPOINT_ACTION_STOP)
    {
      /* Stop */
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_STOP_EXECUTION,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_PRINT,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_FILE,BM_SETCHECK,0,0);
      /* Disable */
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_LABEL_LINE_DATA),0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_PRINT_DATA),0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_LABEL_FILE_PATH),0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_FILE_PATH),0);
    }
  else if(window_breakpoint->action_radio == BREAKPOINT_ACTION_PRINT)
    {
      /* Log */
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_STOP_EXECUTION,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_PRINT,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_FILE,BM_SETCHECK,0,0);
      /* Disable */
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_LABEL_FILE_PATH),0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_FILE_PATH),0);
      /* Enable */
      if(edit_mode_status == 1)
        {
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_LABEL_LINE_DATA),1);
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_PRINT_DATA),1);
        }
    }
  else if(window_breakpoint->action_radio == BREAKPOINT_ACTION_FILE)
    {
      /* File */
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_STOP_EXECUTION,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_PRINT,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_FILE,BM_SETCHECK,1,0);
      /* Enable */
      if(edit_mode_status == 1)
        {
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_LABEL_LINE_DATA),1);
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_PRINT_DATA),1);
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_LABEL_FILE_PATH),1);
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_FILE_PATH),1);
        }
    }
}


/*****************************************************************************/
/*  UpdateBreakpointRepeat() :  Met  jour les donnes Repeat du Breakpoint. */
/*****************************************************************************/
static void UpdateBreakpointRepeat(struct window_breakpoint *window_breakpoint, int edit_mode_status)
{
  /*** Met  jour les donnes Repeat  l'cran ***/
  if(window_breakpoint->repeat_radio == BREAKPOINT_REPEAT_ALWAYS)
    {
      /* Stop */
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_REPEAT_ALWAYS,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_REPEAT_ONLY,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_REPEAT_AFTER,BM_SETCHECK,0,0);
      /* Disable */
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_LABEL_REPEAT_COUNTER),0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_REPEAT_ONLY_DATA),0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_LABEL_REPEAT_ONLY),0);
    }
  else if(window_breakpoint->repeat_radio == BREAKPOINT_REPEAT_ONLY)
    {
      /* Stop */
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_REPEAT_ALWAYS,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_REPEAT_ONLY,BM_SETCHECK,1,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_REPEAT_AFTER,BM_SETCHECK,0,0);
      /* Enable */
      if(edit_mode_status == 1)
        {
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_LABEL_REPEAT_COUNTER),1);
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_REPEAT_ONLY_DATA),1);
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_LABEL_REPEAT_ONLY),1);
        }
    }
  else if(window_breakpoint->repeat_radio == BREAKPOINT_REPEAT_AFTER)
    {
      /* Stop */
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_REPEAT_ALWAYS,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_REPEAT_ONLY,BM_SETCHECK,0,0);
      SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_REPEAT_AFTER,BM_SETCHECK,1,0);
      /* Enable */
      if(edit_mode_status == 1)
        {
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_LABEL_REPEAT_COUNTER),1);
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_REPEAT_ONLY_DATA),1);
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_LABEL_REPEAT_ONLY),1);
        }
    }
}


/**************************************************/
/*  RAZBreakpoint() :  RAZ de la zone Breakpoint. */
/**************************************************/
static void RAZBreakpoint(struct window_breakpoint *window_breakpoint)
{
  int i;

  /*** Init + RAZ ***/
  /** Address **/
  /* Breakpoint de la fentre */
  window_breakpoint->source_radio = BREAKPOINT_SOURCE_ADDRESS;
  strcpy(window_breakpoint->source_address_bank,"");
  strcpy(window_breakpoint->source_address_address,"");
  strcpy(window_breakpoint->source_address_bank_dst,"");
  strcpy(window_breakpoint->source_address_address_dst,"");
  strcpy(window_breakpoint->source_operation_text,"");
  window_breakpoint->source_register = 0;
  window_breakpoint->source_operation_data_nb = 0;
  window_breakpoint->source_systemcall_type = 0;
  window_breakpoint->source_systemcall_number = 0;
  strcpy(window_breakpoint->source_systemcall_text,"");
  window_breakpoint->source_systemcall_index = 0;
  for(i=0; i<4; i++)
    window_breakpoint->source_operation_data[i] = 0x00;
  strcpy(window_breakpoint->source_operation_text,"");
  strcpy(window_breakpoint->source_symbol_name,"");
  strcpy(window_breakpoint->source_symbol_segment,"");
  strcpy(window_breakpoint->source_symbol_program,"");
  /* Controles */
  SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_ADDRESS,BM_SETCHECK,0,0);
  SetWindowText(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_BANK),"");
  SetWindowText(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_ADDRESS),"");
  SetWindowText(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_BANK_DST),"");
  SetWindowText(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_ADDRESS_DST),"");
  SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_SYMBOL,BM_SETCHECK,0,0);
  SetWindowText(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_LABEL_SYMBOL1),"");
  SetWindowText(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_LABEL_SYMBOL2),"");
  SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_SYSTEMCALL,BM_SETCHECK,0,0);
  SendMessage(window_breakpoint->hwnd_combo_systemcall,CB_SETCURSEL,0,0);
  SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_OPERATION,BM_SETCHECK,0,0);
  SetWindowText(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_OPERATION_DATA),"");
  SetWindowText(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_STATIC_OPERATION_DATA),"");
  SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_REGISTER,BM_SETCHECK,0,0);
  SendMessage(window_breakpoint->hwnd_combo_register,CB_SETCURSEL,0,0); 

  /** Trigger **/
  /* Breakpoint de la fentre */
  window_breakpoint->trigger_radio = BREAKPOINT_TRIGGER_READ;
  window_breakpoint->trigger_read_operator = 0;
  window_breakpoint->trigger_read_operator_value_size = 0;
  window_breakpoint->trigger_read_operator_value = 0x0000;
  window_breakpoint->trigger_write_operator = 0;
  window_breakpoint->trigger_write_operator_value_size = 0;
  window_breakpoint->trigger_write_operator_value = 0x0000;
  window_breakpoint->trigger_readwrite_operator = 0;
  window_breakpoint->trigger_readwrite_operator_value_size = 0;
  window_breakpoint->trigger_readwrite_operator_value = 0x0000;
  window_breakpoint->trigger_hold_operator = 0;
  window_breakpoint->trigger_hold_operator_value_size = 0;
  window_breakpoint->trigger_hold_operator_value = 0x0000;
  /* Controles */
  SendMessage(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_READ),BM_SETCHECK,0,0);
  SendMessage(window_breakpoint->hwnd_combo_read_operator,CB_SETCURSEL,0,0);
  SetWindowText(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_READ_VALUE),"");
  SendMessage(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_WRITE),BM_SETCHECK,0,0);
  SendMessage(window_breakpoint->hwnd_combo_write_operator,CB_SETCURSEL,0,0);
  SetWindowText(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_WRITE_VALUE),"");
  SendMessage(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_READWRITE),BM_SETCHECK,0,0);
  SendMessage(window_breakpoint->hwnd_combo_readwrite_operator,CB_SETCURSEL,0,0);
  SetWindowText(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_READWRITE_VALUE),"");
  SendMessage(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_EXECUTE),BM_SETCHECK,0,0);
  SendMessage(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_HOLD),BM_SETCHECK,0,0);
  SendMessage(window_breakpoint->hwnd_combo_hold_operator,CB_SETCURSEL,0,0);
  SetWindowText(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_HOLD_VALUE),"");

  /** Action **/
  /* Breakpoint de la fentre */
  window_breakpoint->action_radio = BREAKPOINT_ACTION_STOP;
  strcpy(window_breakpoint->action_print_data,"");
  strcpy(window_breakpoint->action_file_path,"");
  /* Controles */
  SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_STOP_EXECUTION,BM_SETCHECK,0,0);
  SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_PRINT,BM_SETCHECK,0,0);
  SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_FILE,BM_SETCHECK,0,0);
  SetWindowText(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_PRINT_DATA),"");
  SetWindowText(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_FILE_PATH),"");

  /** Repeat **/
  /* Breakpoint de la fentre */
  window_breakpoint->repeat_radio = BREAKPOINT_REPEAT_ALWAYS;
  window_breakpoint->repeat_only_times = 0;
  /* Controles */
  SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_REPEAT_ALWAYS,BM_SETCHECK,0,0);
  SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_REPEAT_ONLY,BM_SETCHECK,0,0);
  SendDlgItemMessage(window_breakpoint->hwnd_breakpoint,IDC_RADIO_REPEAT_AFTER,BM_SETCHECK,0,0);
  SetWindowText(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_REPEAT_ONLY_DATA),"");
}


/*******************************************************************************/
/*  UpdateBreakpointEditMode() :  MAJ du status de modification des contrles. */
/*******************************************************************************/
static void UpdateBreakpointEditMode(struct window_breakpoint *window_breakpoint, int edit_mode)
{
  int status;

  /* Status */
  status = edit_mode;   /* 0:Disable / 1:Enable */

  /** Enable / Disable **/
  /* Source */
  EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_ADDRESS),status);
  EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_SYMBOL),status);
  EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_SYSTEMCALL),status);
  EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_REGISTER),status);
  EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_OPERATION),status);

  /* Trigger */
  EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_READ),status);
  EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_WRITE),status);
  EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_READWRITE),status);
  EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_EXECUTE),status);
  EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_HOLD),status);
  /* Une Address ne peut tre en 'Hold' */
  if(edit_mode == 1 && window_breakpoint->source_radio == BREAKPOINT_SOURCE_ADDRESS)
    EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_HOLD),0);
  /* Un Symbol ne peut tre en 'Hold' */
  if(edit_mode == 1 && window_breakpoint->source_radio == BREAKPOINT_SOURCE_SYMBOL)
    EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_HOLD),0);
  /* Un SystemCall ne peut tre en 'Hold' */
  if(edit_mode == 1 && window_breakpoint->source_radio == BREAKPOINT_SOURCE_SYSTEMCALL)
    EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_HOLD),0);
  /* Un SystemCall ne peut tre qu'en 'Execute' */
  if(edit_mode == 1 && window_breakpoint->source_radio == BREAKPOINT_SOURCE_SYSTEMCALL)
    {
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_READ),0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_WRITE),0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_READWRITE),0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_HOLD),0);
    }
  /* Une Opration ne peut tre qu'en 'Execute' */
  if(edit_mode == 1 && window_breakpoint->source_radio == BREAKPOINT_SOURCE_OPERATION)
    {
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_READ),0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_WRITE),0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_READWRITE),0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_HOLD),0);
    }
  /* Les Registres ne peut tre qu'en 'Hold' */
  if(edit_mode == 1 && window_breakpoint->source_radio == BREAKPOINT_SOURCE_REGISTER)
    {
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_READ),0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_WRITE),0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_READWRITE),0);
      EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_EXECUTE),0);
    }

  /* Action */
  EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_STOP_EXECUTION),status);
  EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_PRINT),status);
  EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_FILE),status);

  /* Repeat */
  EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_REPEAT_ALWAYS),status);
  EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_REPEAT_ONLY),status);
  EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_RADIO_REPEAT_AFTER),status);

  /** Source **/
  /* Disable */
  EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_BANK),0);
  EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_ADDRESS),0);
  EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_BANK_DST),0);
  EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_ADDRESS_DST),0);
  EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_LABEL_SYMBOL1),0);
  EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_LABEL_SYMBOL2),0);
  EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_BUTTON_SYMBOL),0);
  EnableWindow(window_breakpoint->hwnd_combo_systemcall,0);
  EnableWindow(window_breakpoint->hwnd_combo_register,0);
  EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_OPERATION_DATA),0);
  EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_STATIC_OPERATION_DATA),0);
  /* Enable */
  if(edit_mode == 1)
    {
      if(window_breakpoint->source_radio == BREAKPOINT_SOURCE_ADDRESS)
        {
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_BANK),1);
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_ADDRESS),1);
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_BANK_DST),1);
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_ADDRESS_ADDRESS_DST),1);
        }
      else if(window_breakpoint->source_radio == BREAKPOINT_SOURCE_SYMBOL)
        {
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_LABEL_SYMBOL1),1);
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_LABEL_SYMBOL2),1);
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_BUTTON_SYMBOL),1);
        }
      else if(window_breakpoint->source_radio == BREAKPOINT_SOURCE_SYSTEMCALL)
        EnableWindow(window_breakpoint->hwnd_combo_systemcall,1);
      else if(window_breakpoint->source_radio == BREAKPOINT_SOURCE_REGISTER)
        EnableWindow(window_breakpoint->hwnd_combo_register,1);
      else if(window_breakpoint->source_radio == BREAKPOINT_SOURCE_OPERATION)
        EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_OPERATION_DATA),1);
    }

  /** Trigger **/
  /* Disable */
  EnableWindow(window_breakpoint->hwnd_combo_read_operator,0);
  EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_READ_VALUE),0);
  EnableWindow(window_breakpoint->hwnd_combo_write_operator,0);
  EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_WRITE_VALUE),0);
  EnableWindow(window_breakpoint->hwnd_combo_readwrite_operator,0);
  EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_READWRITE_VALUE),0);
  EnableWindow(window_breakpoint->hwnd_combo_hold_operator,0);
  EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_HOLD_VALUE),0);
  /* Enable */
  if(edit_mode == 1)
    {
      if(window_breakpoint->trigger_radio == BREAKPOINT_TRIGGER_READ)
        {
          EnableWindow(window_breakpoint->hwnd_combo_read_operator,1);          
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_READ_VALUE),1);
        }
      else if(window_breakpoint->trigger_radio == BREAKPOINT_TRIGGER_WRITE)
        {
          EnableWindow(window_breakpoint->hwnd_combo_write_operator,1);
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_WRITE_VALUE),1);
        }
      else if(window_breakpoint->trigger_radio == BREAKPOINT_TRIGGER_READWRITE)
        {
          EnableWindow(window_breakpoint->hwnd_combo_readwrite_operator,1);
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_READWRITE_VALUE),1);
        }
      else if(window_breakpoint->trigger_radio == BREAKPOINT_TRIGGER_HOLD)
        {
          EnableWindow(window_breakpoint->hwnd_combo_hold_operator,1);
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_HOLD_VALUE),1);
        }
    }

  /** Action **/
  /* Disable */
  EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_LABEL_ACTION),status);
  EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_LABEL_LINE_DATA),0);
  EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_PRINT_DATA),0);
  EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_LABEL_FILE_PATH),0);
  EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_FILE_PATH),0);
  
  /* Enable */
  if(edit_mode == 1)
    {
      if(window_breakpoint->action_radio == BREAKPOINT_ACTION_PRINT)
        {
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_LABEL_LINE_DATA),1);
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_PRINT_DATA),1);
        }
      else if(window_breakpoint->action_radio == BREAKPOINT_ACTION_FILE)
        {
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_LABEL_LINE_DATA),1);
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_PRINT_DATA),1);
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_LABEL_FILE_PATH),1);
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_FILE_PATH),1);
        }
    }

  /** Repeat **/
  /* Disable */
  EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_LABEL_REPEAT_MODE),status);
  EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_LABEL_REPEAT_COUNTER),0);
  EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_REPEAT_ONLY_DATA),0);
  EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_LABEL_REPEAT_ONLY),0);
  /* Enable */
  if(edit_mode == 1)
    {
      if(window_breakpoint->repeat_radio == BREAKPOINT_REPEAT_ONLY || window_breakpoint->repeat_radio == BREAKPOINT_REPEAT_AFTER)
        {
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_LABEL_REPEAT_COUNTER),1);
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_EDIT_REPEAT_ONLY_DATA),1);
          EnableWindow(GetDlgItem(window_breakpoint->hwnd_breakpoint,IDC_LABEL_REPEAT_ONLY),1);
        }
    }
}


/***************************************************************/
/*  GetBreakpointByID() :  Recherche un Breakpoint via son ID. */
/***************************************************************/
struct cyrene_breakpoint *GetBreakpointByID(DWORD breakpoint_id)
{
  struct cyrene_breakpoint *current_breakpoint;
  struct window_main *window_main;

  /* Rcupre la fentre Main */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  if(window_main == NULL)
    return(NULL);
  if(window_main->current_breakpoint_table == NULL)
    return(NULL);

  /* Renvoi le Breakpoint */
  for(current_breakpoint = window_main->current_breakpoint_table->first_breakpoint; current_breakpoint; current_breakpoint = current_breakpoint->next)
    if(current_breakpoint->id == breakpoint_id)
      return(current_breakpoint);

  /* Pas trouv */
  return(NULL);
}


/********************************************************************************/
/*  CheckBreakpointHexaValue() : Valide une valeur Hexa pouvant contenir des *. */
/********************************************************************************/
static int CheckBreakpointHexaValue(char *buffer, int nb_byte)
{
  int i;

  /* Vrifie la taille */
  if(strlen(buffer) != nb_byte)
    return(1);

  /* Vrifie les caractres */
  for(i=0; i<nb_byte; i++)
    if(!(IsHexaDecimalChar(buffer[i]) || buffer[i] == '*'))
      return(1);

  /* OK */
  return(0);
}


/*****************************************************************************/
/*  RebuildListViewContent() :  Reconstruit le ListView avec les Breakpoint. */
/*****************************************************************************/
static void RebuildListViewContent(struct window_breakpoint *window_breakpoint)
{
  int i;
  char buffer[1024];
  struct window_main *window_main;
  struct cyrene_breakpoint *current_breakpoint;

  /* Rcupre la fentre Main */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  if(window_main == NULL)
    return;

  /* Allocation mmoire */
  if(window_main->current_breakpoint_table == NULL)
    {
      window_main->current_breakpoint_table = (struct breakpoint_table *) calloc(1,sizeof(struct breakpoint_table));
      if(window_main->current_breakpoint_table == NULL)
        return;
    }

  /** Construit le tableau **/
  if(window_main->current_breakpoint_table->tab_breakpoint != NULL)
    free(window_main->current_breakpoint_table->tab_breakpoint);
  window_main->current_breakpoint_table->tab_breakpoint = (struct cyrene_breakpoint **) calloc(window_main->current_breakpoint_table->nb_breakpoint,sizeof(struct cyrene_breakpoint *));
  if(window_main->current_breakpoint_table->tab_breakpoint == NULL)
    {
      mem_free_breakpoint_table(window_main->current_breakpoint_table);
      window_main->current_breakpoint_table = NULL;
      ListView_DeleteAllItems(window_breakpoint->listview_breakpoint.hwnd_listview);
      FillBreakpointListView(&window_breakpoint->listview_breakpoint,window_main->current_breakpoint_table);
      return;
    }

  /** Remplit le tableau en mettant en haut les Enable et ensuite les Disable **/
  i = 0;
  for(current_breakpoint=window_main->current_breakpoint_table->first_breakpoint; current_breakpoint; current_breakpoint=current_breakpoint->next)
    if(current_breakpoint->status == 1)
      window_main->current_breakpoint_table->tab_breakpoint[i++] = current_breakpoint;
  for(current_breakpoint=window_main->current_breakpoint_table->first_breakpoint; current_breakpoint; current_breakpoint=current_breakpoint->next)
    if(current_breakpoint->status == 0)
      window_main->current_breakpoint_table->tab_breakpoint[i++] = current_breakpoint;

  /* Refait les liens */
  if(window_main->current_breakpoint_table->nb_breakpoint > 0)
    {
      window_main->current_breakpoint_table->first_breakpoint = window_main->current_breakpoint_table->tab_breakpoint[0];
      window_main->current_breakpoint_table->last_breakpoint = window_main->current_breakpoint_table->tab_breakpoint[window_main->current_breakpoint_table->nb_breakpoint-1];
      window_main->current_breakpoint_table->last_breakpoint->next = NULL;
      for(i=0; i<window_main->current_breakpoint_table->nb_breakpoint-1; i++)
        window_main->current_breakpoint_table->tab_breakpoint[i]->next = window_main->current_breakpoint_table->tab_breakpoint[i+1];
    }
  else
    {
      window_main->current_breakpoint_table->first_breakpoint = NULL;
      window_main->current_breakpoint_table->last_breakpoint = NULL;
    }

  /** Supprime toutes les entres du Listview **/
  ListView_DeleteAllItems(window_breakpoint->listview_breakpoint.hwnd_listview);

  /** On refait le Listview avec les Breakpoints de la Table **/
  FillBreakpointListView(&window_breakpoint->listview_breakpoint,window_main->current_breakpoint_table);

  /* MAJ du titre de la fentre avec le nombre de Breakpoint */
  if(window_main->current_breakpoint_table != NULL)
    {
      if(window_main->current_breakpoint_table->nb_breakpoint < 2)
        sprintf(buffer," Breakpoint Table : %d entry",window_main->current_breakpoint_table->nb_breakpoint);
      else
        sprintf(buffer," Breakpoint Table : %d entries",window_main->current_breakpoint_table->nb_breakpoint);
      SetWindowText(window_breakpoint->hwnd_breakpoint,buffer);
    }

  /* MAJ Status Bar */
  UpdateWindowMainStatusBar();
}


/**********************************************************************************/
/*  GetBreakpopintHighestID() :  Rcupre l'ID de plus haut rang des breakpoints. */
/**********************************************************************************/
static DWORD GetBreakpopintHighestID(void)
{
  DWORD highest_id;
  struct window_main *window_main;
  struct cyrene_breakpoint *current_breakpoint;

  /* Init */
  highest_id = 0;

  /* Rcupre la fentre Main */
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  if(window_main == NULL)
    return(highest_id);
  if(window_main->current_breakpoint_table == NULL)
    return(highest_id);

  /** Boucle sur tous les Breakpoints **/
  for(current_breakpoint=window_main->current_breakpoint_table->first_breakpoint; current_breakpoint; current_breakpoint=current_breakpoint->next)
    if(current_breakpoint->id > highest_id)
      highest_id = current_breakpoint->id;

  /* Renvoie l'ID */
  return(highest_id);
}


/****************************************************************************/
/*  mem_free_breakpoint() :  Libration mmoire de la structure breakpoint. */
/****************************************************************************/
void mem_free_breakpoint(struct cyrene_breakpoint *current_breakpoint)
{
  if(current_breakpoint)
    {
      free(current_breakpoint);
    }
}


/****************************************************************************************/
/*  mem_free_breakpoint_table() :  Libration mmoire de la structure breakpoint_table. */
/****************************************************************************************/
void mem_free_breakpoint_table(struct breakpoint_table *current_breakpoint_table)
{
  struct cyrene_breakpoint *current_breakpoint;
  struct cyrene_breakpoint *next_breakpoint;

  if(current_breakpoint_table)
    {
      /* Libration de la liste chaine Breakpoint */
      for(current_breakpoint = current_breakpoint_table->first_breakpoint; current_breakpoint; current_breakpoint = next_breakpoint)
        {
          next_breakpoint = current_breakpoint->next;
          mem_free_breakpoint(current_breakpoint);
        }

      if(current_breakpoint_table->tab_breakpoint)
        free(current_breakpoint_table->tab_breakpoint);

      free(current_breakpoint_table);
    }
}


/***********************************************************************************************/
/*  UpdateBreakpointSymbolAddress() :  MAJ des adresses des Breakpoints utilisant des Symbols. */
/***********************************************************************************************/
void UpdateBreakpointSymbolAddress(void)
{
  struct symbol *current_symbol;
  struct window_main *window_main;
  struct cyrene_breakpoint *current_breakpoint;

  /** Rcupre la fentre Main **/
  my_Memory(MEMORY_GET_WINDOW_MAIN,&window_main,NULL);
  if(window_main == NULL)
    return;
  if(window_main->current_breakpoint_table == NULL)
    return;
  if(window_main->current_symbol_table == NULL)
    return;

  /*** Passer en revue tous les Breakpoints ***/
  for(current_breakpoint = window_main->current_breakpoint_table->first_breakpoint; current_breakpoint; current_breakpoint = current_breakpoint->next)
    {
      /** Source Address : Symbol **/
      if(current_breakpoint->source == BREAKPOINT_SOURCE_SYMBOL)
        {
          /* Symbol Address */
          current_breakpoint->source_symbol_begin_address = 0xFFFFFFFF;
          current_breakpoint->source_symbol_end_address = 0xFFFFFFFF;
          /* Recherche */
          for(current_symbol=window_main->current_symbol_table->first_symbol; current_symbol; current_symbol=current_symbol->next)
            if(!strcmp(current_symbol->name,current_breakpoint->source_symbol_name) &&                 /* Le Symbole est case sensitive */
               !stricmp(current_symbol->segment_name,current_breakpoint->source_symbol_segment) &&
               !stricmp(current_symbol->program_name,current_breakpoint->source_symbol_program))
              {
                current_breakpoint->source_symbol_begin_address = current_symbol->symbol_relocated_address;

                /* Pour les Symbol, on calcule l'adresse de fin (y compris pour le Code) */
                current_breakpoint->source_symbol_end_address = current_breakpoint->source_symbol_begin_address + current_symbol->size - 1;
                break;
              }
        }
    }
}


/************************************************************************/
/*  UserResizeBreakpointWindow() :  Retaille la fentre des Breakpoint. */
/************************************************************************/
static void UserResizeBreakpointWindow(struct window_breakpoint *window_breakpoint)
{
  RECT window_rect;
  int window_x, window_y, window_new_height, minimum_window_height;
  int listview_pixel_height, nb_listview_data_row, delta_listview_nb_row, delta_pixel_height;

  /* Position + Dimension actuelle de la fentre retaille dans l'cran */
  GetWindowRect(window_breakpoint->hwnd_breakpoint,&window_rect);

  /* Nouvelle Dimension de la fentre (avec la barre de titre et les bordures) */
  window_x = window_rect.left;
  window_y = window_rect.top;
  window_new_height = window_rect.bottom - window_rect.top;

  /* Nombre de Pixels restant pour la ListView */
  minimum_window_height = window_breakpoint->window_init_height - window_breakpoint->nb_init_data_row*SYMBOL_LISTVIEW_ROW_HEIGHT;
  listview_pixel_height = window_new_height - minimum_window_height;

  /* Nombre de lignes de la ListView */
  nb_listview_data_row = listview_pixel_height / SYMBOL_LISTVIEW_ROW_HEIGHT;

  /** Teste les valeurs minimales autorises **/
  if(nb_listview_data_row < SYMBOL_LISTVIEW_MIN_NB_ROW)
    nb_listview_data_row = SYMBOL_LISTVIEW_MIN_NB_ROW;

  /* Delta de vla ListView */
  delta_listview_nb_row = nb_listview_data_row - window_breakpoint->nb_row;
  delta_pixel_height = delta_listview_nb_row*SYMBOL_LISTVIEW_ROW_HEIGHT;
  window_breakpoint->nb_row = nb_listview_data_row;

  /* Nouvelle hauteur de la fentre (on ajuste pour tomber sur un nombre extact de ligne dans la ListView) */
  window_new_height = minimum_window_height + nb_listview_data_row*SYMBOL_LISTVIEW_ROW_HEIGHT;

  /* Retaille la fentre */
  window_breakpoint->user_resize = 0;
  MoveWindow(window_breakpoint->hwnd_breakpoint,window_x,window_y,window_breakpoint->window_init_width,window_new_height,TRUE);

  /* Dplace les Contrles */
  MoveBreakpointWindowControls(delta_pixel_height,window_breakpoint);

  /* Rafraichit la fentre */
  InvalidateRect(window_breakpoint->hwnd_breakpoint,NULL,TRUE);
  UpdateWindow(window_breakpoint->hwnd_breakpoint);
}


/*********************************************************************************************/
/*  MoveBreakpointWindowControls() :  On va dplacer les diffrents controles de la fentre. */
/*********************************************************************************************/
static void MoveBreakpointWindowControls(int delta_y, struct window_breakpoint *window_breakpoint)
{
  HWND hwnd_control, hwnd_listview;
  int i, control_x, control_y, control_width, control_height;
  int listview_x, listview_y, listview_width, listview_height;
  int tab_control_id[] = 
    {
      IDC_BUTTON_NEW,             IDC_BUTTON_MODIFY,          IDC_BUTTON_DELETE,            IDC_BUTTON_DELETE_ALL, 
      IDC_BUTTON_ENABLE,          IDC_BUTTON_DISABLE,         IDC_BUTTON_SAVE,              IDC_BUTTON_CANCEL,
      IDC_BUTTON_LOAD,            IDC_BUTTON_OK,              IDC_GROUP_SOURCE,             IDC_RADIO_ADDRESS,
      IDC_RADIO_SYMBOL,           IDC_RADIO_SYSTEMCALL,       IDC_RADIO_OPERATION,          IDC_RADIO_REGISTER,
      IDC_EDIT_ADDRESS_BANK,      IDC_STATIC_SEP_1,           IDC_EDIT_ADDRESS_ADDRESS,     IDC_STATIC_SEP_2,
      IDC_EDIT_ADDRESS_BANK_DST,  IDC_STATIC_SEP_3,           IDC_EDIT_ADDRESS_ADDRESS_DST, IDC_BUTTON_SYMBOL,
      IDC_EDIT_OPERATION_DATA,    IDC_GROUP_TRIGGER,          IDC_RADIO_READ,               IDC_RADIO_WRITE,
      IDC_RADIO_READWRITE,        IDC_RADIO_EXECUTE,          IDC_RADIO_HOLD,               IDC_EDIT_READ_VALUE,
      IDC_EDIT_WRITE_VALUE,       IDC_EDIT_READWRITE_VALUE,   IDC_EDIT_HOLD_VALUE,          IDC_GROUP_ACTION,
      IDC_LABEL_ACTION,           IDC_RADIO_STOP_EXECUTION,   IDC_RADIO_PRINT,              IDC_RADIO_FILE,
      IDC_LABEL_LINE_DATA,        IDC_EDIT_PRINT_DATA,        IDC_LABEL_FILE_PATH,          IDC_EDIT_FILE_PATH,
      IDC_GROUP_REPEAT,           IDC_LABEL_REPEAT_MODE,      IDC_LABEL_REPEAT_COUNTER,     IDC_RADIO_REPEAT_ALWAYS,
      IDC_RADIO_REPEAT_ONLY,      IDC_RADIO_REPEAT_AFTER,     IDC_EDIT_REPEAT_ONLY_DATA,    IDC_LABEL_REPEAT_ONLY,
      IDC_LABEL_SYMBOL1,          IDC_LABEL_SYMBOL2,          IDC_STATIC_OPERATION_DATA,    IDC_COMBOBOX_SYSTEMCALL,
      IDC_COMBOBOX_REGISTER,      IDC_COMBOBOX_READ_OPERATOR, IDC_COMBOBOX_WRITE_OPERATOR,  IDC_COMBOBOX_READWRITE_OPERATOR,
      IDC_COMBOBOX_HOLD_OPERATOR, IDC_COMBOBOX_SYMBOL,
      0
    };

  /** On va modifier la hauteur de la ListBox **/
  /* Handle du Controle */
  hwnd_listview = window_breakpoint->listview_breakpoint.hwnd_listview;
  /* Position + Taille actuelle */
  GetWindowControlInfo(window_breakpoint->hwnd_breakpoint,hwnd_listview,&listview_x,&listview_y,&listview_width,&listview_height);
  /* Change la hauteur du controle */
  MoveWindow(hwnd_listview,listview_x,listview_y,listview_width,listview_height+delta_y,TRUE);

  /** On va dplacer tous les controles verticalement **/
  for(i=0; tab_control_id[i] != 0; i++)
    {
      /* Handle du Controle */
      hwnd_control = GetDlgItem(window_breakpoint->hwnd_breakpoint,tab_control_id[i]);

      /* Position + Taille actuelle */
      GetWindowControlInfo(window_breakpoint->hwnd_breakpoint,hwnd_control,&control_x,&control_y,&control_width,&control_height);

      /* Dplace le controle verticalement */
      MoveWindow(hwnd_control,control_x,control_y+delta_y,control_width,control_height,TRUE);
    }
}


/************************************************************************/
/*  DeleteBreakpointChildWindows() :  Destruction des fentres enfants. */
/************************************************************************/
static void DeleteBreakpointChildWindows(struct window_breakpoint *window_breakpoint)
{
}


/**************************************************************************/
/*  mem_free_window_breakpoint() :  Libration mmoire pour la structure. */
/**************************************************************************/
void mem_free_window_breakpoint(struct window_breakpoint *window_breakpoint)
{
  if(window_breakpoint)
    {
      free(window_breakpoint);
    }
}

/****************************************************************************/
