/*********************************************************************/
/*                                                                   */
/*   Cyrene_SystemCall.c :  Module de gestion des appels systmes.   */
/*                                                                   */
/*********************************************************************/
/*  Auteur : Olivier ZARDINI  *  Brutal Deluxe  *  Date : May 2023   */
/*********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/timeb.h>

#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>

#include "defc.h"
#include "sound.h"
#include "Cyrene.h"
#include "Cyrene_Shared.h"
#include "Cyrene_WindowIPC.h"
#include "Cyrene_Breakpoint.h"
#include "Cyrene_SystemCall.h"

/* Type des paramtres */
#define TYPE_GSOS_PARAM_BYTE                    1
#define TYPE_GSOS_PARAM_WORD                    2
#define TYPE_GSOS_PARAM_24BIT                   3
#define TYPE_GSOS_PARAM_DWORD                   4
#define TYPE_GSOS_PARAM_QWORD                   5
#define TYPE_GSOS_PARAM_STRING                  6    /* 2 bytes lead (GS/OS) */
#define TYPE_GSOS_PARAM_P8STRING                7    /* 1 byte lead (Prodos 8) */
#define TYPE_GSOS_PARAM_PSTRING                 8    /* 1 byte lead (Prodos 16) */
#define TYPE_GSOS_PARAM_RESULT_STRING           9    /* 4 bytes lead : Buffer Size (WORD) + String Length (WORD) */
#define TYPE_GSOS_PARAM_STRING_PTR             10    /* Adresse 4 bytes d'un pointeur vers une String GS/OS (2 bytes lead) */
#define TYPE_GSOS_PARAM_TXT                    50    /* + Longueur de la chaine */

static char *GetCallGsOsAllParameter(DWORD,DWORD,DWORD,DWORD,WORD,struct cyrene_operation *,struct cyrene_parameter *);
static char *GetCallProdos16AllParameter(DWORD,DWORD,DWORD,DWORD,WORD,struct cyrene_operation *,struct cyrene_parameter *);
static char *GetCallProdos8AllParameter(DWORD,DWORD,DWORD,BYTE,struct cyrene_operation *,struct cyrene_parameter *);
static char *GetCallToolboxAllParameter(DWORD,DWORD,DWORD,DWORD,WORD,struct cyrene_operation *,struct cyrene_parameter *);
static void GetEmptyParameter(WORD,WORD,char *);
static void GetGsOsCallOneParameter(WORD,WORD,DWORD,int,char *,struct cyrene_operation *,struct cyrene_parameter *);
static void BuildSystemCallReturnValues(DWORD,WORD,int,int,int,DWORD,int,struct cyrene_parameter *);


/**************************************************************************************/
/*  ProcessTraceSystemCall() :  On va tracer l'Appel Systme / Toolbox dans l'Output. */
/**************************************************************************************/
void ProcessTraceSystemCall(DWORD stack_address, DWORD pc_address, DWORD call_address, DWORD return_address, WORD call_number, struct cyrene_operation *current_operation, struct cyrene_parameter *cyrene_param)
{
  int call_origin, is_p16_call;
  char *fonction_parameter;
  char buffer_line[BUFFER_SIZE];

  /* Is Prodos 16 calls ? */
  is_p16_call = IsProdos16Call(call_address,call_number);

  /** Init : GOID PC_ADDRESS CALL_NUMBER **/
  if(call_address == 0x00E10000)
    sprintf(buffer_line,"[%08X:%08X][%02X/%04X][%04X] o T%03d::",(DWORD)(cyrene_param->goid>>32),(DWORD)(cyrene_param->goid & 0x00000000FFFFFFFF),(pc_address>>16) & 0x000000FF,pc_address & 0x0000FFFF,call_number,call_number & 0x00FF);
  else if(call_address == 0x0000BF00)
    sprintf(buffer_line,"[%08X:%08X][%02X/%04X][  %02X] o   P8::",(DWORD)(cyrene_param->goid>>32),(DWORD)(cyrene_param->goid & 0x00000000FFFFFFFF),(pc_address>>16) & 0x000000FF,pc_address & 0x0000FFFF,(BYTE)(call_number & 0x00FF));
  else if(is_p16_call == 1)
    sprintf(buffer_line,"[%08X:%08X][%02X/%04X][%04X] o  P16::",(DWORD)(cyrene_param->goid>>32),(DWORD)(cyrene_param->goid & 0x00000000FFFFFFFF),(pc_address>>16) & 0x000000FF,pc_address & 0x0000FFFF,call_number);
  else
    sprintf(buffer_line,"[%08X:%08X][%02X/%04X][%04X] o GSOS::",(DWORD)(cyrene_param->goid>>32),(DWORD)(cyrene_param->goid & 0x00000000FFFFFFFF),(pc_address>>16) & 0x000000FF,pc_address & 0x0000FFFF,call_number);

  /*** Rcupration du nom de la fonction + paramtre ***/
  if(call_address == 0x00E10000)
    {
      /* Trace Toolbox Calls */
      fonction_parameter = GetCallToolboxAllParameter(stack_address,pc_address,call_address,return_address,call_number,current_operation,cyrene_param);
      call_origin = TRACE_CALL_ORIGIN_TOOL;
    }
  else if(call_address == 0x0000BF00)
    {
      /* Trace Prodos 8 calls */
      fonction_parameter = GetCallProdos8AllParameter(pc_address,call_address,return_address,(BYTE)(call_number & 0x00FF),current_operation,cyrene_param);
      call_origin = TRACE_CALL_ORIGIN_P8;
    }
  else if(is_p16_call == 1)
    {
      /* Trace Prodos 16 calls */
      fonction_parameter = GetCallProdos16AllParameter(stack_address,pc_address,call_address,return_address,call_number,current_operation,cyrene_param);
      call_origin = TRACE_CALL_ORIGIN_P16;
    }
  else
    {
      /* Trace GS/OS calls */
      fonction_parameter = GetCallGsOsAllParameter(stack_address,pc_address,call_address,return_address,call_number,current_operation,cyrene_param);
      call_origin = TRACE_CALL_ORIGIN_GSOS;
    }
  if(fonction_parameter == NULL)
    return;
  strcat(buffer_line,fonction_parameter);

  /** Ecriture de la ligne dans la Shared Memory **/
  CopyOutlineIntoSharedMemory(BREAKPOINT_ACTION_TRACE_SYSTEM_CALL,0,call_origin,call_number,buffer_line,cyrene_param);
}


/*******************************************************************************************************/
/*  ProcessTraceSystemCallReturnValues() :  Rcupration des paramtres de retour des Appels Systmes. */
/*******************************************************************************************************/
void ProcessTraceSystemCallReturnValues(struct cyrene_system_call *current_system_call, WORD a_reg, int status_c, struct cyrene_operation *current_operation, struct cyrene_parameter *cyrene_param)
{
  int i;
  DWORD total_cycle;
  char buffer_line[BUFFER_SIZE];

  /** Init : GOID + Address + Call Number **/
  sprintf(buffer_line,"[%08X:%08X][%02X/%04X][%04X] => ",(DWORD)(current_system_call->operation_goid>>32),(DWORD)(current_system_call->operation_goid & 0x00000000FFFFFFFF),
                                                         (current_system_call->return_address >> 16) & 0x000000FF,current_system_call->return_address & 0x0000FFFF,
                                                         current_system_call->call_number);

  /* # Cycle */
  total_cycle = (DWORD) (cyrene_param->gcc - current_system_call->operation_gcc);
  sprintf(&buffer_line[strlen(buffer_line)],"[%d]",total_cycle);

  /* Status : Error GS/OS(c+A), P16(c+A), P8(c+A), Toolbox(c+A) */
  if(status_c == 1 && current_system_call->origin == TRACE_CALL_ORIGIN_P8)
    sprintf(&buffer_line[strlen(buffer_line)],"Error (%02X)",a_reg & 0x00FF);
  else if(status_c == 1)
    sprintf(&buffer_line[strlen(buffer_line)],"Error (%04X)",a_reg & 0x00FF);
  else
    {
      /* Dbut des valeurs */
      strcat(buffer_line,"(");

      /** Rcupration des valeurs **/
      for(i=0; i<current_system_call->nb_param; i++)
        GetGsOsCallOneParameter(1,1,current_system_call->tab_param_address[i],current_system_call->tab_param_type[i],buffer_line,current_operation,cyrene_param);

      /* Fin des valeurs */
      strcat(buffer_line,")");
    }

  /** Ecriture de la ligne dans la Shared Memory **/
  CopyOutlineIntoSharedMemory(BREAKPOINT_ACTION_TRACE_SYSTEM_CALL,0,TRACE_CALL_ORIGIN_RETURN,current_system_call->call_number,buffer_line,cyrene_param);
}


/*****************************************************************************/
/*  GetCallGsOsAllParameter() :  Rcupre les paramtres de l'appel  GS/OS. */
/*****************************************************************************/
static char *GetCallGsOsAllParameter(DWORD stack_address, DWORD pc_address, DWORD call_address, DWORD return_address, WORD call_number, struct cyrene_operation *current_operation, struct cyrene_parameter *cyrene_param)
{
  WORD nb_param_max;
  DWORD param_address, string_address, param_address_prt;
  static char buffer_line[BUFFER_SIZE];

  /** In Line **/
  if(call_address == 0x00E100A8)
    param_address_prt = pc_address + 4 + 2;
  /** Stack **/
  else if(call_address == 0x00E100B0)
    param_address_prt = stack_address + 3;

  /* Adresse du dbut des paramtres */
  param_address = GetMemoryValue(param_address_prt,3,current_operation,cyrene_param);

  /** On va traiter les Appels Systmes les plus courrants */
  switch(call_number)
    {
      case 0x2034 :
        strcpy(buffer_line,"AddNotifyProc(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        /* #01 DWORD : procPointer       IN  +2 */
        GetGsOsCallOneParameter(1,nb_param_max,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,0,0,0,0,cyrene_param);
        break;

      case 0x201D :
        strcpy(buffer_line,"BeginSession(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,0,0,0,0,cyrene_param);
        break;

      case 0x2031 :
        strcpy(buffer_line,"BindInt(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        /* #01 WORD  : intNum         OUT +2 */
        GetEmptyParameter(1,nb_param_max,buffer_line);
        /* #02 WORD  : vrn            IN  +4 */
        GetGsOsCallOneParameter(2,nb_param_max,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : intCode        IN  +6 */
        GetGsOsCallOneParameter(3,nb_param_max,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,1,nb_param_max,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x2004 :
        strcpy(buffer_line,"ChangePath(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        /* #01 DWORD : pathname       IN  +2 */
        string_address = GetMemoryValue(param_address+2,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,nb_param_max,string_address,TYPE_GSOS_PARAM_STRING,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : newPathname    IN  +6 */
        string_address = GetMemoryValue(param_address+6,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param_max,string_address,TYPE_GSOS_PARAM_STRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,0,0,0,0,cyrene_param);
        break;

      case 0x200B :
        strcpy(buffer_line,"ClearBackupBit(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        /* #01 DWORD : pathname       IN  +4 */
        string_address = GetMemoryValue(param_address+2,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,nb_param_max,string_address,TYPE_GSOS_PARAM_STRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,0,0,0,0,cyrene_param);
        break;

      case 0x2014 :
        strcpy(buffer_line,"Close(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        /* #01 WORD  : refNum        IN  +2 */
        GetGsOsCallOneParameter(1,nb_param_max,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,0,0,0,0,cyrene_param);
        break;

      case 0x2001 :
        strcpy(buffer_line,"Create(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        /* #01 DWORD : pathname      IN  +2 */
        string_address = GetMemoryValue(param_address+2,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,nb_param_max,string_address,TYPE_GSOS_PARAM_STRING,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : access        IN  +6 */
        GetGsOsCallOneParameter(2,nb_param_max,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : fileType      IN  +8 */
        GetGsOsCallOneParameter(3,nb_param_max,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD : auxType       IN  +10 */
        GetGsOsCallOneParameter(4,nb_param_max,param_address+10,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #05 WORD  : storageType   IN  +14 */
        GetGsOsCallOneParameter(5,nb_param_max,param_address+14,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #06 DWORD : eof           IN  +16 */
        GetGsOsCallOneParameter(6,nb_param_max,param_address+16,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #07 DWORD : ressourceEOF  IN  +20 */
        GetGsOsCallOneParameter(7,nb_param_max,param_address+20,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,0,0,0,0,cyrene_param);
        break;

      case 0x202E :
        strcpy(buffer_line,"DControl(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        /* #01 WORD  : devNum        IN  +2 */
        GetGsOsCallOneParameter(1,nb_param_max,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : code          IN  +4 */
        GetGsOsCallOneParameter(2,nb_param_max,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : list          IN  +6 */
        GetGsOsCallOneParameter(3,nb_param_max,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD : requestCount  IN  +10 */
        GetGsOsCallOneParameter(4,nb_param_max,param_address+10,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #05 DWORD : transferCount OUT +14 */
        GetEmptyParameter(5,nb_param_max,buffer_line);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,5,nb_param_max,param_address+14,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x2035 :
        strcpy(buffer_line,"DelNotifyProc(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        /* #01 DWORD : procPointer     IN  +2 */
        GetGsOsCallOneParameter(1,nb_param_max,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,0,0,0,0,cyrene_param);
        break;

      case 0x2002 :
        strcpy(buffer_line,"Destroy(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        /* #01 DWORD : pathname     IN  +2 */
        string_address = GetMemoryValue(param_address+2,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,nb_param_max,string_address,TYPE_GSOS_PARAM_STRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,0,0,0,0,cyrene_param);
        break;

      case 0x202C :
        strcpy(buffer_line,"DInfo(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        /* #01 WORD  : devNum          IN  +2 */
        GetGsOsCallOneParameter(1,nb_param_max,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : devname         IN  +4 */
        GetGsOsCallOneParameter(2,nb_param_max,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : characteristics OUT +8 */
        GetEmptyParameter(3,nb_param_max,buffer_line);
        /* #04 DWORD : totalBlocks     OUT +10 */
        GetEmptyParameter(4,nb_param_max,buffer_line);
        /* #05 WORD  : slotNum         OUT +14 */
        GetEmptyParameter(5,nb_param_max,buffer_line);
        /* #06 WORD  : unitNum         OUT +16 */
        GetEmptyParameter(6,nb_param_max,buffer_line);
        /* #07 WORD  : version         OUT +18 */
        GetEmptyParameter(7,nb_param_max,buffer_line);
        /* #08 WORD  : deviceID        OUT +20 */
        GetEmptyParameter(8,nb_param_max,buffer_line);
        /* #09 WORD  : headLink        OUT +22 */
        GetEmptyParameter(9,nb_param_max,buffer_line);
        /* #10 WORD  : forwardLink     OUT +24 */
        GetEmptyParameter(10,nb_param_max,buffer_line);
        /* #11 DWORD : extendedDIBptr  IN  +26 */
        GetGsOsCallOneParameter(11,nb_param_max,param_address+26,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** Return Values **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,3,nb_param_max,param_address+8,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,4,nb_param_max,param_address+10,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,5,nb_param_max,param_address+14,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,6,nb_param_max,param_address+16,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,7,nb_param_max,param_address+18,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,8,nb_param_max,param_address+20,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,9,nb_param_max,param_address+22,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,10,nb_param_max,param_address+24,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x202F :
        strcpy(buffer_line,"DRead(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        /* #01 WORD  : devNum          IN  +2 */
        GetGsOsCallOneParameter(1,nb_param_max,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : buffer          IN  +4 */
        GetGsOsCallOneParameter(2,nb_param_max,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : requestCount    IN  +8 */
        GetGsOsCallOneParameter(3,nb_param_max,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD : startingBlock   IN  +12 */
        GetGsOsCallOneParameter(4,nb_param_max,param_address+12,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #05 WORD  : blockSize       IN  +16 */
        GetGsOsCallOneParameter(5,nb_param_max,param_address+16,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #06 DWORD : transferCount   OUT +18 */
        GetEmptyParameter(6,nb_param_max,buffer_line);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,6,nb_param_max,param_address+18,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x2036 :
        strcpy(buffer_line,"DRename(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        /* #01 WORD  : devNum          IN  +2 */
        GetGsOsCallOneParameter(1,nb_param_max,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : strPtr          IN  +4 */
        string_address = GetMemoryValue(param_address+4,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param_max,string_address,TYPE_GSOS_PARAM_STRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,0,0,0,0,cyrene_param);
        break;

      case 0x202D :
        strcpy(buffer_line,"DStatus(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        /* #01 WORD  : devNum          IN  +2 */
        GetGsOsCallOneParameter(1,nb_param_max,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : code            IN  +4 */
        GetGsOsCallOneParameter(2,nb_param_max,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : list            IN  +6 */
        GetGsOsCallOneParameter(3,nb_param_max,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD : requestCount    IN  +10 */
        GetGsOsCallOneParameter(4,nb_param_max,param_address+10,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #05 DWORD : transferCount   OUT +14 */
        GetEmptyParameter(5,nb_param_max,buffer_line);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,5,nb_param_max,param_address+14,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x2030 :
        strcpy(buffer_line,"DWrite(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        /* #01 WORD  : devNum          IN  +2 */
        GetGsOsCallOneParameter(1,nb_param_max,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : buffer          IN  +4 */
        GetGsOsCallOneParameter(2,nb_param_max,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : requestCount    IN  +8 */
        GetGsOsCallOneParameter(3,nb_param_max,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD : startingBlock   IN  +12 */
        GetGsOsCallOneParameter(4,nb_param_max,param_address+12,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #05 WORD  : blockSize       IN  +16 */
        GetGsOsCallOneParameter(5,nb_param_max,param_address+16,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #06 DWORD : transferCount   OUT +18 */
        GetEmptyParameter(6,nb_param_max,buffer_line);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,6,nb_param_max,param_address+18,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x201E :
        strcpy(buffer_line,"EndSession(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,0,0,0,0,cyrene_param);
        break;

      case 0x2025 :
        strcpy(buffer_line,"EraseDisk(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        /* #01 DWORD : devName         IN  +2 */
        string_address = GetMemoryValue(param_address+2,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,nb_param_max,string_address,TYPE_GSOS_PARAM_STRING,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : volName         IN  +6 */
        string_address = GetMemoryValue(param_address+6,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param_max,string_address,TYPE_GSOS_PARAM_STRING,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : fileSysID       OUT +10 */
        GetEmptyParameter(3,nb_param_max,buffer_line);
        /* #04 WORD  : reqfileSysID    IN  +12 */
        GetGsOsCallOneParameter(4,nb_param_max,param_address+12,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,3,nb_param_max,param_address+10,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x200E :
        strcpy(buffer_line,"ExpandPath(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        /* #01 DWORD : inputPath       IN  +2 */
        string_address = GetMemoryValue(param_address+2,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,nb_param_max,string_address,TYPE_GSOS_PARAM_STRING,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : outputPath      OUT +6 */
        GetEmptyParameter(2,nb_param_max,buffer_line);
        /* #03 WORD  : flags           IN  +10 */
        GetGsOsCallOneParameter(3,nb_param_max,param_address+10,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        string_address = GetMemoryValue(param_address+6,4,current_operation,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,2,nb_param_max,string_address,TYPE_GSOS_PARAM_RESULT_STRING,cyrene_param);
        break;

      case 0x2015 :
        strcpy(buffer_line,"Flush(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        /* #01 WORD  : refNum          IN  +2 */
        GetGsOsCallOneParameter(1,nb_param_max,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,0,0,0,0,cyrene_param);
        break;

      case 0x2024 :
        strcpy(buffer_line,"Format(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        /* #01 DWORD : devName         IN  +2 */
        string_address = GetMemoryValue(param_address+2,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,nb_param_max,string_address,TYPE_GSOS_PARAM_STRING,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : volName         IN  +6 */
        string_address = GetMemoryValue(param_address+6,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param_max,string_address,TYPE_GSOS_PARAM_STRING,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : fileSysID       OUT +10 */
        GetEmptyParameter(3,nb_param_max,buffer_line);
        /* #04 WORD  : reqFileSysID    IN  +12 */
        GetGsOsCallOneParameter(4,nb_param_max,param_address+12,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,3,nb_param_max,param_address+10,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x2033 :
        strcpy(buffer_line,"FSTSpecific(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        /* #01 WORD  : fileSysID       IN  +2 */
        GetGsOsCallOneParameter(1,nb_param_max,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : commandNum      IN  +4 */
        GetGsOsCallOneParameter(2,nb_param_max,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03-N     : Subcall-specific param... */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,0,0,0,0,cyrene_param);
        break;

      case 0x2028 :
        strcpy(buffer_line,"GetBoolVol(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        /* #01 DWORD : dataBuffer      IN  +2 */
        GetGsOsCallOneParameter(1,nb_param_max,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,0,0,0,0,cyrene_param);
        break;

      case 0x2020 :
        strcpy(buffer_line,"GetDevNumber(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        /* #01 DWORD : devName         IN  +2 */
        string_address = GetMemoryValue(param_address+2,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,nb_param_max,string_address,TYPE_GSOS_PARAM_STRING,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : devNum          OUT +6 */
        GetEmptyParameter(2,nb_param_max,buffer_line);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,2,nb_param_max,param_address+6,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x201C :
        strcpy(buffer_line,"GetDirEntry(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        /* #01 WORD  : refNum          IN  +2 */
        GetGsOsCallOneParameter(1,nb_param_max,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : flags           OUT +4 */
        GetEmptyParameter(2,nb_param_max,buffer_line);
        /* #03 WORD  : base            IN  +6 */
        GetGsOsCallOneParameter(3,nb_param_max,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD  : displacement    IN  +8 */
        GetGsOsCallOneParameter(4,nb_param_max,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #05 DWORD : name buffer     OUT +10 */
        GetEmptyParameter(5,nb_param_max,buffer_line);
        /* #06 WORD  : entryNum        OUT +14 */
        GetEmptyParameter(6,nb_param_max,buffer_line);
        /* #07 WORD  : fileType        OUT +16 */
        GetEmptyParameter(7,nb_param_max,buffer_line);
        /* #08 DWORD : eof             OUT +18 */
        GetEmptyParameter(8,nb_param_max,buffer_line);
        /* #09 DWORD : blockCount      OUT +22 */
        GetEmptyParameter(9,nb_param_max,buffer_line);
        /* #10 QWORD : createDateTime  OUT +26 */
        GetEmptyParameter(10,nb_param_max,buffer_line);
        /* #11 QWORD : modDateTime     OUT +34*/
        GetEmptyParameter(11,nb_param_max,buffer_line);
        /* #12 WORD  : access          OUT +42 */
        GetEmptyParameter(12,nb_param_max,buffer_line);
        /* #13 DWORD : auxType         OUT +44 */
        GetEmptyParameter(13,nb_param_max,buffer_line);
        /* #14 WORD  : fileSysID       OUT +48 */
        GetEmptyParameter(14,nb_param_max,buffer_line);
        /* #15 DWORD : optionList      IN  +50 */
        GetGsOsCallOneParameter(15,nb_param_max,param_address+50,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #16 DWORD : resourceEOF     OUT +54 */
        GetEmptyParameter(16,nb_param_max,buffer_line);
        /* #17 DWORD : resourceBlocks  OUT +58 */
        GetEmptyParameter(17,nb_param_max,buffer_line);

        /** Return Values **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,2,nb_param_max,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        string_address = GetMemoryValue(param_address+10,4,current_operation,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,5,nb_param_max,string_address,TYPE_GSOS_PARAM_RESULT_STRING,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,6,nb_param_max,param_address+14,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,7,nb_param_max,param_address+16,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,8,nb_param_max,param_address+18,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,9,nb_param_max,param_address+22,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,10,nb_param_max,param_address+26,TYPE_GSOS_PARAM_QWORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,11,nb_param_max,param_address+34,TYPE_GSOS_PARAM_QWORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,12,nb_param_max,param_address+42,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,13,nb_param_max,param_address+44,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,14,nb_param_max,param_address+48,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,16,nb_param_max,param_address+54,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,17,nb_param_max,param_address+58,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x2019 :
        strcpy(buffer_line,"GetEOF(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        /* #01 WORD  : refNum          IN  +2 */
        GetGsOsCallOneParameter(1,nb_param_max,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : eof             OUT +4 */
        GetEmptyParameter(2,nb_param_max,buffer_line);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,2,nb_param_max,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x2006 :
        strcpy(buffer_line,"GetFileInfo(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        /* #01 DWORD : pathName        IN  +2 */
        string_address = GetMemoryValue(param_address+2,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,nb_param_max,string_address,TYPE_GSOS_PARAM_STRING,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : access          OUT +6 */
        GetEmptyParameter(2,nb_param_max,buffer_line);
        /* #03 WORD  : fileType        OUT +8 */
        GetEmptyParameter(3,nb_param_max,buffer_line);
        /* #04 DWORD : auxType         OUT +10 */
        GetEmptyParameter(4,nb_param_max,buffer_line);
        /* #05 WORD  : storageType     OUT +14 */
        GetEmptyParameter(5,nb_param_max,buffer_line);
        /* #06 QWORD : createDateTime  OUT +16 */
        GetEmptyParameter(6,nb_param_max,buffer_line);
        /* #07 QWORD : modDateTime     OUT +24 */
        GetEmptyParameter(7,nb_param_max,buffer_line);
        /* #08 DWORD : optionList      IN  +32 */
        GetGsOsCallOneParameter(8,nb_param_max,param_address+32,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #09 DWORD : eof             OUT +36 */
        GetEmptyParameter(9,nb_param_max,buffer_line);
        /* #10 DWORD : blocksUsed      OUT +40 */
        GetEmptyParameter(10,nb_param_max,buffer_line);
        /* #11 DWORD : resourceEOF     OUT +44 */
        GetEmptyParameter(11,nb_param_max,buffer_line);
        /* #12 DWORD : resourceBlocks  OUT +48 */
        GetEmptyParameter(12,nb_param_max,buffer_line);

        /** Return Values **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,2,nb_param_max,param_address+6,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,3,nb_param_max,param_address+8,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,4,nb_param_max,param_address+10,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,5,nb_param_max,param_address+14,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,6,nb_param_max,param_address+16,TYPE_GSOS_PARAM_QWORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,7,nb_param_max,param_address+24,TYPE_GSOS_PARAM_QWORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,9,nb_param_max,param_address+36,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,10,nb_param_max,param_address+40,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,11,nb_param_max,param_address+44,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,12,nb_param_max,param_address+48,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x202B :
        strcpy(buffer_line,"GetFSTInfo(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        /* #01 WORD  : fstNum          IN  +2 */
        GetGsOsCallOneParameter(1,nb_param_max,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : fileSysID       OUT +4 */
        GetEmptyParameter(2,nb_param_max,buffer_line);
        /* #03 DWORD : fstName         OUT +6 */
        GetEmptyParameter(3,nb_param_max,buffer_line);
        /* #04 WORD  : version         OUT +10 */
        GetEmptyParameter(4,nb_param_max,buffer_line);
        /* #05 WORD  : attributes      OUT +12 */
        GetEmptyParameter(5,nb_param_max,buffer_line);
        /* #06 WORD  : blockSize       OUT +14 */
        GetEmptyParameter(6,nb_param_max,buffer_line);
        /* #07 DWORD : maxVolSize      OUT +16 */
        GetEmptyParameter(7,nb_param_max,buffer_line);
        /* #08 DWORD : maxFileSize     OUT +20 */
        GetEmptyParameter(8,nb_param_max,buffer_line);

        /** Return Values **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,2,nb_param_max,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        string_address = GetMemoryValue(param_address+6,4,current_operation,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,3,nb_param_max,string_address,TYPE_GSOS_PARAM_RESULT_STRING,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,4,nb_param_max,param_address+10,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,5,nb_param_max,param_address+12,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,6,nb_param_max,param_address+14,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,7,nb_param_max,param_address+16,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,8,nb_param_max,param_address+20,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x201B :
        strcpy(buffer_line,"GetLevel(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        /* #01 WORD  : level           OUT  +2 */
        GetEmptyParameter(1,nb_param_max,buffer_line);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,1,nb_param_max,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x2017 :
        strcpy(buffer_line,"GetMark(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        /* #01 WORD  : refNum          IN  +2 */
        GetGsOsCallOneParameter(1,nb_param_max,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : position        IN  +4 */
        GetEmptyParameter(2,nb_param_max,buffer_line);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,2,nb_param_max,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x2027 :
        strcpy(buffer_line,"GetName(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        /* #01 DWORD : dataBuffer      IN  +2 */
        GetGsOsCallOneParameter(1,nb_param_max,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,0,0,0,0,cyrene_param);
        break;

      case 0x200A :
        strcpy(buffer_line,"GetPrefix(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        /* #01 WORD  : prefixNum      IN  +2 */
        GetGsOsCallOneParameter(1,nb_param_max,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : prefix         IN  +4 */
        GetEmptyParameter(2,nb_param_max,buffer_line);

        /** Return Value **/
        string_address = GetMemoryValue(param_address+4,4,current_operation,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,2,nb_param_max,string_address,TYPE_GSOS_PARAM_RESULT_STRING,cyrene_param);
        break;

      case 0x2039 :
        strcpy(buffer_line,"GetRefInfo(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        /* #01 WORD  : refNum         IN  +2 */
        GetGsOsCallOneParameter(1,nb_param_max,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : access         OUT +4 */
        GetEmptyParameter(2,nb_param_max,buffer_line);
        /* #03 WORD  : pathname       IN  +6 */
        string_address = GetMemoryValue(param_address+6,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(3,nb_param_max,string_address,TYPE_GSOS_PARAM_STRING,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,2,nb_param_max,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x2038 :
        strcpy(buffer_line,"GetRefNum(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        /* #01 DWORD : pathname       IN  +2 */
        string_address = GetMemoryValue(param_address+2,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,nb_param_max,string_address,TYPE_GSOS_PARAM_STRING,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : refNum         OUT +6 */
        GetEmptyParameter(2,nb_param_max,buffer_line);
        /* #03 WORD  : access         OUT +8 */
        GetEmptyParameter(3,nb_param_max,buffer_line);
        /* #04 WORD  : resNum         IN  +10 */
        GetGsOsCallOneParameter(4,nb_param_max,param_address+10,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #05 WORD  : caseSense      IN  +12 */
        GetGsOsCallOneParameter(5,nb_param_max,param_address+12,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #06 WORD  : displacement   IN  +14 */
        GetGsOsCallOneParameter(6,nb_param_max,param_address+14,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** Return Values **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,2,nb_param_max,param_address+6,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,3,nb_param_max,param_address+8,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x2037 :
        strcpy(buffer_line,"GetStdRefNum(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        /* #01 WORD  : prefixNum      IN  +2 */
        GetGsOsCallOneParameter(1,nb_param_max,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : refNum         OUT +4 */
        GetEmptyParameter(2,nb_param_max,buffer_line);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,2,nb_param_max,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x200F :
        strcpy(buffer_line,"GetSysPrefs(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        /* #01 WORD  : preferences       OUT +2 */
        GetEmptyParameter(1,nb_param_max,buffer_line);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,1,nb_param_max,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x202A :
        strcpy(buffer_line,"GetVersion(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        /* #01 WORD  : version         OUT +2 */
        GetEmptyParameter(1,nb_param_max,buffer_line);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,1,nb_param_max,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x2011 :
        strcpy(buffer_line,"NewLine(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        /* #01 WORD  : refNum         IN  +2 */
        GetGsOsCallOneParameter(1,nb_param_max,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : enableMask     IN  +4 */
        GetGsOsCallOneParameter(2,nb_param_max,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : numChars       IN  +6 */
        GetGsOsCallOneParameter(3,nb_param_max,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD : newlineTable   IN  +8 */
        GetGsOsCallOneParameter(4,nb_param_max,param_address+10,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,0,0,0,0,cyrene_param);
        break;

      case 0x200D :
        strcpy(buffer_line,"Null(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,0,0,0,0,cyrene_param);
        break;

      case 0x2010 :
        strcpy(buffer_line,"Open(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        /* #01 WORD  : refNum         OUT +2 */
        GetEmptyParameter(1,nb_param_max,buffer_line);
        /* #02 DWORD : pathname       IN  +4 */
        string_address = GetMemoryValue(param_address+4,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param_max,string_address,TYPE_GSOS_PARAM_STRING,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : requestAccess  IN  +8 */
        GetGsOsCallOneParameter(3,nb_param_max,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD  : resourceNumber IN  +10 */
        GetGsOsCallOneParameter(4,nb_param_max,param_address+10,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #05 WORD  : access         OUT +12 */
        GetEmptyParameter(5,nb_param_max,buffer_line);
        /* #06 WORD  : fileType       OUT +14 */
        GetEmptyParameter(6,nb_param_max,buffer_line);
        /* #07 DWORD : auxType        OUT +16 */
        GetEmptyParameter(7,nb_param_max,buffer_line);
        /* #08 WORD  : storageType    OUT +20 */
        GetEmptyParameter(8,nb_param_max,buffer_line);
        /* #09 QWORD : createDateTime OUT +22 */
        GetEmptyParameter(9,nb_param_max,buffer_line);
        /* #10 QWORD : modDateTime    OUT +30 */
        GetEmptyParameter(10,nb_param_max,buffer_line);
        /* #11 DWORD : optionList     IN  +38 */
        GetGsOsCallOneParameter(11,nb_param_max,param_address+38,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #12 DWORD : eof            OUT +42 */
        GetEmptyParameter(12,nb_param_max,buffer_line);
        /* #13 DWORD : blocksUsed     OUT +46 */
        GetEmptyParameter(13,nb_param_max,buffer_line);
        /* #14 DWORD : resourceEOF    OUT +50 */
        GetEmptyParameter(14,nb_param_max,buffer_line);
        /* #15 DWORD : resourceBlocks OUT +54 */
        GetEmptyParameter(15,nb_param_max,buffer_line);

        /** Return Values **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,1,nb_param_max,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,5,nb_param_max,param_address+12,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,6,nb_param_max,param_address+14,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,7,nb_param_max,param_address+16,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,8,nb_param_max,param_address+20,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,9,nb_param_max,param_address+22,TYPE_GSOS_PARAM_QWORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,10,nb_param_max,param_address+30,TYPE_GSOS_PARAM_QWORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,11,nb_param_max,param_address+42,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,12,nb_param_max,param_address+46,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,14,nb_param_max,param_address+50,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,15,nb_param_max,param_address+54,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x2003 :
        strcpy(buffer_line,"OSShutdown(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        /* #01 WORD  : shutdownFlag   IN  +2 */
        GetGsOsCallOneParameter(1,nb_param_max,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,0,0,0,0,cyrene_param);
        break;

      case 0x2029 :
        strcpy(buffer_line,"Quit(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        /* #01 DWORD : pathname       IN  +2 */
        string_address = GetMemoryValue(param_address+2,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,nb_param_max,string_address,TYPE_GSOS_PARAM_STRING,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : flags          IN  +6 */
        GetGsOsCallOneParameter(2,nb_param_max,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,0,0,0,0,cyrene_param);
        break;

      case 0x2012 :
        strcpy(buffer_line,"Read(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        /* #01 WORD  : refNum         IN  +2 */
        GetGsOsCallOneParameter(1,nb_param_max,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : dataBuffer     IN  +4 */
        GetGsOsCallOneParameter(2,nb_param_max,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : requestCount   IN  +8 */
        GetGsOsCallOneParameter(3,nb_param_max,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD : transferCount  OUT +12 */
        GetEmptyParameter(4,nb_param_max,buffer_line);
        /* #05 WORD  : cachePriority  IN  +16 */
        GetGsOsCallOneParameter(5,nb_param_max,param_address+16,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,4,nb_param_max,param_address+12,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x2026 :
        strcpy(buffer_line,"ResetCache(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,0,0,0,0,cyrene_param);
        break;

      case 0x201F :
        strcpy(buffer_line,"SessionStatus(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        /* #01 WORD  : status          IN  +2 */
        GetGsOsCallOneParameter(1,nb_param_max,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,0,0,0,0,cyrene_param);
        break;

      case 0x2018 :
        strcpy(buffer_line,"SetEOF(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        /* #01 WORD  : refNum          IN  +2 */
        GetGsOsCallOneParameter(1,nb_param_max,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : base            IN  +4 */
        GetGsOsCallOneParameter(2,nb_param_max,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : displacement    IN  +6 */
        GetGsOsCallOneParameter(3,nb_param_max,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,0,0,0,0,cyrene_param);
        break;

      case 0x2005 :
        strcpy(buffer_line,"SetFileInfo(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        /* #01 DWORD : pathname        IN  +2 */
        GetGsOsCallOneParameter(1,nb_param_max,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : access          IN  +6 */
        GetGsOsCallOneParameter(2,nb_param_max,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : fileType        IN  +8 */
        GetGsOsCallOneParameter(3,nb_param_max,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD : auxType         IN  +10 */
        GetGsOsCallOneParameter(4,nb_param_max,param_address+10,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #05 WORD  : reserved        IN  +14 */
        GetGsOsCallOneParameter(5,nb_param_max,param_address+14,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #06 QWORD : createDateTime  IN  +16 */
        GetGsOsCallOneParameter(6,nb_param_max,param_address+16,TYPE_GSOS_PARAM_QWORD,buffer_line,current_operation,cyrene_param);
        /* #07 QWORD : modDateTime     IN  +24 */
        GetGsOsCallOneParameter(7,nb_param_max,param_address+24,TYPE_GSOS_PARAM_QWORD,buffer_line,current_operation,cyrene_param);
        /* #08 DWORD : optionList      IN  +32 */
        GetGsOsCallOneParameter(8,nb_param_max,param_address+32,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #09 DWORD : reserved        IN  +36 */
        GetGsOsCallOneParameter(9,nb_param_max,param_address+36,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #10 DWORD : reserved        IN  +40 */
        GetGsOsCallOneParameter(10,nb_param_max,param_address+40,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #11 DWORD : reserved        IN  +44 */
        GetGsOsCallOneParameter(11,nb_param_max,param_address+44,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #12 DWORD : reserved        IN  +48 */
        GetGsOsCallOneParameter(12,nb_param_max,param_address+48,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,0,0,0,0,cyrene_param);
        break;

      case 0x201A :
        strcpy(buffer_line,"SetLevel(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        /* #01 WORD  : level           IN  +2 */
        GetGsOsCallOneParameter(1,nb_param_max,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,0,0,0,0,cyrene_param);
        break;

      case 0x2016 :
        strcpy(buffer_line,"SetMark(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        /* #01 WORD  : refNum          IN  +2 */
        GetGsOsCallOneParameter(1,nb_param_max,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : base            IN  +4 */
        GetGsOsCallOneParameter(2,nb_param_max,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : displacement    IN  +6 */
        GetGsOsCallOneParameter(3,nb_param_max,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,0,0,0,0,cyrene_param);
        break;

      case 0x2009 :
        strcpy(buffer_line,"SetPrefix(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        /* #01 WORD  : prefixNum       IN  +2 */
        GetGsOsCallOneParameter(1,nb_param_max,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : prefix          IN  +4 */
        string_address = GetMemoryValue(param_address+4,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param_max,string_address,TYPE_GSOS_PARAM_STRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,0,0,0,0,cyrene_param);
        break;

      case 0x200C :
        strcpy(buffer_line,"SetSysPrefs(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        /* #01 WORD  : preferences       IN  +2 */
        GetGsOsCallOneParameter(1,nb_param_max,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,0,0,0,0,cyrene_param);
        break;

      case 0x2032 :
        strcpy(buffer_line,"UnbindInt(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        /* #01 WORD  : intNum           IN  +2 */
        GetGsOsCallOneParameter(1,nb_param_max,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,0,0,0,0,cyrene_param);
        break;

      case 0x2008 : 
        strcpy(buffer_line,"Volume(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        /* #01 DWORD : devName          IN  +2 */
        string_address = GetMemoryValue(param_address+2,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,nb_param_max,string_address,TYPE_GSOS_PARAM_STRING,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : volName          OUT +6 */
        GetEmptyParameter(2,nb_param_max,buffer_line);
        /* #03 DWORD : totalBlocks      OUT +10 */
        GetEmptyParameter(3,nb_param_max,buffer_line);
        /* #04 DWORD : freeBlocks       OUT +14 */
        GetEmptyParameter(4,nb_param_max,buffer_line);
        /* #05 WORD  : fileSysID        OUT +18 */
        GetEmptyParameter(5,nb_param_max,buffer_line);
        /* #06 WORD  : blockSize        OUT +20 */
        GetEmptyParameter(6,nb_param_max,buffer_line);

        /** Return Values **/
        string_address = GetMemoryValue(param_address+6,4,current_operation,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,2,nb_param_max,string_address,TYPE_GSOS_PARAM_RESULT_STRING,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,3,nb_param_max,param_address+10,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,4,nb_param_max,param_address+14,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,5,nb_param_max,param_address+18,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,6,nb_param_max,param_address+20,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x2013 :
        strcpy(buffer_line,"Write(");
        /* #00 WORD  : pCount */
        nb_param_max = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        /* #01 WORD  : refNum           IN  +2 */
        GetGsOsCallOneParameter(1,nb_param_max,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : dataBuffer       IN  +4 */
        GetGsOsCallOneParameter(2,nb_param_max,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : requestCount     IN  +8 */
        GetGsOsCallOneParameter(3,nb_param_max,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD : transferCount    OUT +12 */
        GetEmptyParameter(4,nb_param_max,buffer_line);
        /* #05 WORD  : cachePriority    IN  +16 */
        GetGsOsCallOneParameter(5,nb_param_max,param_address+16,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_GSOS,4,nb_param_max,param_address+12,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      default :
        return(NULL);
        break;
    }

  /* Renvoi la ligne */
  strcat(buffer_line,")");
  return(&buffer_line[0]);
}


/************************************************************************************/
/*  GetCallProdos16AllParameter() :  Rcupre les paramtres de l'appel  Prodos16. */
/************************************************************************************/
static char *GetCallProdos16AllParameter(DWORD stack_address, DWORD pc_address, DWORD call_address, DWORD return_address, WORD call_number, struct cyrene_operation *current_operation, struct cyrene_parameter *cyrene_param)
{
  WORD nb_param;
  DWORD param_address, string_address, param_address_prt;
  static char buffer_line[BUFFER_SIZE];

  /** In Line **/
  if(call_address == 0x00E100A8)
    param_address_prt = pc_address + 4 + 2;
  /** Stack **/
  else if(call_address == 0x00E100B0)
    param_address_prt = stack_address + 3;

  /* Adresse du dbut des paramtres */
  param_address = GetMemoryValue(param_address_prt,3,current_operation,cyrene_param);

  /** On va traiter les Appels Systmes Prodos 16 **/
  switch(call_number)
    {
      case 0x0031 :
        strcpy(buffer_line,"ALLOC_INTERRUPT(");
        nb_param = 2;
        /* #01 WORD  : intN um         IN  +0 */
        GetGsOsCallOneParameter(1,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : intCode         IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,0,0,0,0,cyrene_param);
        break;

      case 0x0004 :
        strcpy(buffer_line,"CHANGE_PATH(");
        nb_param = 2;
        /* #01 DWORD : pathname       IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : newPathname    IN  +4 */
        string_address = GetMemoryValue(param_address+4,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,0,0,0,0,cyrene_param);
        break;

      case 0x000B :
        strcpy(buffer_line,"CLEAR_BACKUP_BIT(");
        nb_param = 1;
        /* #01 DWORD : pathname       IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,0,0,0,0,cyrene_param);
        break;

      case 0x0014 :
        strcpy(buffer_line,"CLOSE(");
        nb_param = 1;
        /* #01 WORD  : fileRefNum     IN  +0 */
        GetGsOsCallOneParameter(1,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,0,0,0,0,cyrene_param);
        break;

      case 0x0001 :
        strcpy(buffer_line,"CREATE(");
        nb_param = 7;
        /* #01 DWORD : pathname       IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : fAccess        IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : fileType       IN  +6 */
        GetGsOsCallOneParameter(3,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD : auxType        IN  +8 */
        GetGsOsCallOneParameter(4,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #05 WORD  : storageType    IN  +12 */
        GetGsOsCallOneParameter(5,nb_param,param_address+12,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #06 WORD  : createDate     IN  +14 */
        GetGsOsCallOneParameter(6,nb_param,param_address+14,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #07 WORD  : createTime     IN  +16 */
        GetGsOsCallOneParameter(7,nb_param,param_address+16,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,0,0,0,0,cyrene_param);
        break;

      case 0x0032 :
        strcpy(buffer_line,"DEALLOC_INTERRUPT(");
        nb_param = 1;
        /* #01 WORD  : intNum          IN  +0 */
        GetGsOsCallOneParameter(1,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,0,0,0,0,cyrene_param);
        break;

      case 0x0002 :
        strcpy(buffer_line,"DESTROY(");
        nb_param = 1;
        /* #01 DWORD : pathname          IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,0,0,0,0,cyrene_param);
        break;

      case 0x002C :
        strcpy(buffer_line,"D_INFO(");
        nb_param = 2;
        /* #01 WORD  : devNum            IN  +0 */
        GetGsOsCallOneParameter(1,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : devName           OUT +2 */
        GetEmptyParameter(2,nb_param,buffer_line);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,2,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0025 :
        strcpy(buffer_line,"ERASE_DISK(");
        nb_param = 3;
        /* #01 DWORD : devName          IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : volName          IN  +4 */
        string_address = GetMemoryValue(param_address+4,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : fileSysID        IN  +8 */
        GetGsOsCallOneParameter(3,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,0,0,0,0,cyrene_param);
        break;

      case 0x000E :
        strcpy(buffer_line,"EXPAND_PATH(");
        nb_param = 3;
        /* #01 DWORD : inputPath        IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : outputPath       OUT +4 */
        GetEmptyParameter(2,nb_param,buffer_line);
        /* #03 WORD  : flags            IN  +8 */
        GetGsOsCallOneParameter(3,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0015 :
        strcpy(buffer_line,"FLUSH(");
        nb_param = 1;
        /* #01 WORD  : fileRefNum       IN  +0 */
        GetGsOsCallOneParameter(1,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,0,0,0,0,cyrene_param);
        break;

      case 0x0024 :
        strcpy(buffer_line,"FORMAT(");
        nb_param = 3;
        /* #01 DWORD : devName          IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : volName          IN  +4 */
        string_address = GetMemoryValue(param_address+4,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : fileSysID        IN  +8 */
        GetGsOsCallOneParameter(3,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,0,0,0,0,cyrene_param);
        break;

      case 0x0028 :
        strcpy(buffer_line,"GET_BOOT_VOL(");
        nb_param = 1;
        /* #01 DWORD : dataBuffer     OUT +0 */
        GetEmptyParameter(1,nb_param,buffer_line);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,1,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0020 :
        strcpy(buffer_line,"GET_DEV_NUM(");
        nb_param = 2;
        /* #01 DWORD : devName          IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : devNum           OUT +4 */
        GetEmptyParameter(2,nb_param,buffer_line);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x001C :
        strcpy(buffer_line,"GET_DIR_ENTRY(");
        nb_param = 14;
        /* #01 WORD  : refNum           IN  +0 */
        GetGsOsCallOneParameter(1,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : flags            OUT +2 */
        GetEmptyParameter(2,nb_param,buffer_line);
        /* #03 WORD  : base             IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD  : displacement     IN  +6 */
        GetGsOsCallOneParameter(4,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #05 DWORD : nameBuffer       OUT +8 */
        GetEmptyParameter(5,nb_param,buffer_line);
        /* #06 WORD  : entryNum         OUT +12 */
        GetEmptyParameter(6,nb_param,buffer_line);
        /* #07 WORD  : fileType         OUT +14 */
        GetEmptyParameter(7,nb_param,buffer_line);
        /* #08 DWORD : endOfFile        OUT +16 */
        GetEmptyParameter(8,nb_param,buffer_line);
        /* #09 DWORD : blockCount       OUT +20 */
        GetEmptyParameter(9,nb_param,buffer_line);
        /* #10 QWORD : createTime       OUT +24 */
        GetEmptyParameter(10,nb_param,buffer_line);
        /* #11 QWORD : modTime          OUT +32 */
        GetEmptyParameter(11,nb_param,buffer_line);
        /* #12 WORD  : access           OUT +40 */
        GetEmptyParameter(12,nb_param,buffer_line);
        /* #13 DWORD : auxType          OUT +42 */
        GetEmptyParameter(13,nb_param,buffer_line);
        /* #14 WORD  : fileSysID        OUT +46 */
        GetEmptyParameter(14,nb_param,buffer_line);

        /** Return Values **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        string_address = GetMemoryValue(param_address+8,4,current_operation,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,5,nb_param,string_address,TYPE_GSOS_PARAM_RESULT_STRING,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,6,nb_param,param_address+12,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,7,nb_param,param_address+14,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,8,nb_param,param_address+16,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,9,nb_param,param_address+20,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,10,nb_param,param_address+24,TYPE_GSOS_PARAM_QWORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,11,nb_param,param_address+32,TYPE_GSOS_PARAM_QWORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,12,nb_param,param_address+40,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,13,nb_param,param_address+42,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,14,nb_param,param_address+46,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0019 :
        strcpy(buffer_line,"GET_EOF(");
        nb_param = 2;
        /* #01 WORD  : refNum          IN  +0 */
        GetGsOsCallOneParameter(1,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : eof             OUT +2 */
        GetEmptyParameter(2,nb_param,buffer_line);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,2,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0006 :
        strcpy(buffer_line,"GET_FILE_INFO(");
        nb_param = 10;
        /* #01 DWORD : pathname         IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : FAccess          OUT +4 */
        GetEmptyParameter(2,nb_param,buffer_line);
        /* #03 WORD  : fileType         OUT +6 */
        GetEmptyParameter(3,nb_param,buffer_line);
        /* #04 DWORD : auxType          OUT +8 */
        GetEmptyParameter(4,nb_param,buffer_line);
        /* #05 WORD  : storageType      OUT +12 */
        GetEmptyParameter(5,nb_param,buffer_line);
        /* #06 WORD  : createDate       OUT +14 */
        GetEmptyParameter(6,nb_param,buffer_line);
        /* #07 WORD  : createTime       OUT +16 */
        GetEmptyParameter(7,nb_param,buffer_line);
        /* #08 WORD  : modDate          OUT +18 */
        GetEmptyParameter(8,nb_param,buffer_line);
        /* #09 WORD  : modTime          OUT +20 */
        GetEmptyParameter(9,nb_param,buffer_line);
        /* #10 DWORD : blocksUsed       OUT +22 */
        GetEmptyParameter(10,nb_param,buffer_line);

        /** Return Values **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,3,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,4,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,5,nb_param,param_address+12,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,6,nb_param,param_address+14,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,7,nb_param,param_address+16,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,8,nb_param,param_address+18,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,9,nb_param,param_address+20,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,10,nb_param,param_address+22,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0021 :
        strcpy(buffer_line,"GET_LAST_DEV(");
        nb_param = 1;
        /* #01 WORD  : devNum           OUT +0 */
        GetEmptyParameter(1,nb_param,buffer_line);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,1,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x001B :
        strcpy(buffer_line,"GET_LEVEL(");
        nb_param = 1;
        /* #01 WORD  : level            OUT +0 */
        GetEmptyParameter(1,nb_param,buffer_line);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,1,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0017 :
        strcpy(buffer_line,"GET_MARK(");
        nb_param = 2;
        /* #01 WORD  : markRefNum      IN  +0 */
        GetGsOsCallOneParameter(1,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : position        OUT +2 */
        GetEmptyParameter(2,nb_param,buffer_line);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,2,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0027 :
        strcpy(buffer_line,"GET_NAME(");
        nb_param = 1;
        /* #01 DWORD : dataBuffer      OUT +0 */
        GetEmptyParameter(1,nb_param,buffer_line);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,1,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x000A :
        strcpy(buffer_line,"GET_PREFIX(");
        nb_param = 2;
        /* #01 WORD  : prefixNum       IN  +0 */
        GetGsOsCallOneParameter(1,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : prefix          OUT +2 */
        GetEmptyParameter(2,nb_param,buffer_line);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,2,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x002A :
        strcpy(buffer_line,"GET_VERSION(");
        nb_param = 1;
        /* #01 WORD  : version           OUT +0 */
        GetEmptyParameter(1,nb_param,buffer_line);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,1,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0011 :
        strcpy(buffer_line,"NEWLINE(");
        nb_param = 3;
        /* #01 WORD  : newLRefNum     IN  +0 */
        GetGsOsCallOneParameter(1,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : enableMask     IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : newlineChar    IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,0,0,0,0,cyrene_param);
        break;

      case 0x0010 :
        strcpy(buffer_line,"OPEN(");
        nb_param = 3;
        /* #01 WORD  : openRefNum     OUT +0 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD : openPathname   IN  +2 */
        string_address = GetMemoryValue(param_address+2,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : ioBuffer       OUT +6 */
        GetEmptyParameter(3,nb_param,buffer_line);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,1,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,3,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0029 :
        strcpy(buffer_line,"QUIT(");
        nb_param = 2;
        /* #01 DWORD : quitPathname     IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : flags            IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,0,0,0,0,cyrene_param);
        break;

      case 0x0012 :
        strcpy(buffer_line,"READ(");
        nb_param = 4;
        /* #01 WORD  : fileRefNum      IN  +0 */
        GetGsOsCallOneParameter(1,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : dataBuffer      IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : requestCount    IN  +6 */
        GetGsOsCallOneParameter(3,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD : transferCount   OUT +10 */
        GetEmptyParameter(4,nb_param,buffer_line);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,4,nb_param,param_address+10,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0022 :
        strcpy(buffer_line,"READ_BLOCK(");
        nb_param = 3;
        /* #01 WORD  : blockDevNum      IN  +0 */
        GetGsOsCallOneParameter(1,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : blockDataBuffer  IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : blockNum         IN  +6 */
        GetGsOsCallOneParameter(3,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,0,0,0,0,cyrene_param);
        break;

      case 0x0018 :
        strcpy(buffer_line,"SET_EOF(");
        nb_param = 2;
        /* #01 WORD  : eofRefNum       IN  +0 */
        GetGsOsCallOneParameter(1,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : eofPosition     IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,0,0,0,0,cyrene_param);
        break;

      case 0x0005 :
        strcpy(buffer_line,"SET_FILE_INFO(");
        nb_param = 9;
        /* #01 DWORD : pathname       IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : fAccess        IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : fileType       IN  +6 */
        GetGsOsCallOneParameter(3,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD : auxType        IN  +8 */
        GetGsOsCallOneParameter(4,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #05 WORD  : unused         IN  +12 */
        GetEmptyParameter(5,nb_param,buffer_line);
        /* #06 WORD  : createDate     IN  +14 */
        GetGsOsCallOneParameter(6,nb_param,param_address+14,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #07 WORD  : createTime     IN  +16 */
        GetGsOsCallOneParameter(7,nb_param,param_address+16,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #08 WORD  : modDate        IN  +18 */
        GetGsOsCallOneParameter(8,nb_param,param_address+18,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #09 WORD  : modTime        IN  +20 */
        GetGsOsCallOneParameter(9,nb_param,param_address+20,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,0,0,0,0,cyrene_param);
        break;

      case 0x001A :
        strcpy(buffer_line,"SET_LEVEL(");
        nb_param = 1;
        /* #01 WORD  : level          IN  +0 */
        GetGsOsCallOneParameter(1,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,0,0,0,0,cyrene_param);
        break;

      case 0x0016 :
        strcpy(buffer_line,"SET_MARK(");
        nb_param = 2;
        /* #01 WORD  : markRefNum      IN  +0 */
        GetGsOsCallOneParameter(1,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : position        IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,0,0,0,0,cyrene_param);
        break;

      case 0x0009 :
        strcpy(buffer_line,"SET_PREFIX(");
        nb_param = 2;
        /* #01 WORD  : prefixNum      IN  +0 */
        GetGsOsCallOneParameter(1,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : prefix         IN  +2 */
        string_address = GetMemoryValue(param_address+2,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,0,0,0,0,cyrene_param);
        break;

      case 0x0008 :
        strcpy(buffer_line,"VOLUME(");
        nb_param = 5;
        /* #01 DWORD : devName        IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : volName        OUT +4 */
        GetEmptyParameter(2,nb_param,buffer_line);
        /* #03 DWORD : total_blocks   OUT +8 */
        GetEmptyParameter(3,nb_param,buffer_line);
        /* #04 DWORD : freeBlocks     OUT +12 */
        GetEmptyParameter(4,nb_param,buffer_line);
        /* #05 WORD  : fileSysID      OUT +16 */
        GetEmptyParameter(5,nb_param,buffer_line);

        /** Return Values **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,3,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,4,nb_param,param_address+12,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,5,nb_param,param_address+16,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0013 :
        strcpy(buffer_line,"WRITE(");
        nb_param = 4;
        /* #01 WORD  : refNum          IN  +0 */
        GetGsOsCallOneParameter(1,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : dataBuffer      IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : requestCount    IN  +6 */
        GetGsOsCallOneParameter(3,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD : transferCount   OUT +10 */
        GetEmptyParameter(4,nb_param,buffer_line);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,4,nb_param,param_address+10,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0023 :
        strcpy(buffer_line,"WRITE_BLOCK(");
        nb_param = 3;
        /* #01 WORD  : blockDevNum      IN  +0 */
        GetGsOsCallOneParameter(1,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : blockDataBuffer  IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : blockNum         IN  +6 */
        GetGsOsCallOneParameter(3,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P16,0,0,0,0,cyrene_param);
        break;

      default :
        return(NULL);
        break;
    }

  /* Renvoi la ligne */
  strcat(buffer_line,")");
  return(&buffer_line[0]);
}


/*********************************************************************/
/*  IsProdos16Call() :  Dtermine s'il s'agit d'un appel Prodos 16 . */
/*********************************************************************/
int IsProdos16Call(DWORD call_address, WORD call_number)
{
  /* GS/OS | Prodos16 */
  if(call_address != 0xE100A8 && call_address != 0xE100B0)
    return(0);

  /** Prodos 16 Calls **/
  if(call_number == 0x0001 || /* CREATE */
     call_number == 0x0002 || /* DESTROY */
     call_number == 0x0004 || /* CHANGE_PATH */ 
     call_number == 0x0005 || /* SET_FILE_INFO */ 
     call_number == 0x0006 || /* GET_FILE_INFO */
     call_number == 0x0008 || /* VOLUME */
     call_number == 0x0009 || /* SET_PREFIX */
     call_number == 0x000A || /* GET_PREFIX */
     call_number == 0x000B || /* CLEAR_BACKUP_BIT */
     call_number == 0x000E || /* EXPAND_PATH */
     call_number == 0x0010 || /* OPEN */
     call_number == 0x0011 || /* NEWLINE */ 
     call_number == 0x0012 || /* READ */
     call_number == 0x0013 || /* WRITE */ 
     call_number == 0x0014 || /* CLOSE */
     call_number == 0x0015 || /* FLUSH */
     call_number == 0x0016 || /* SET_MARK */ 
     call_number == 0x0017 || /* GET_MARK */
     call_number == 0x0018 || /* SET_EOF */
     call_number == 0x0019 || /* GET_EOF */
     call_number == 0x001A || /* SET_LEVEL */
     call_number == 0x001B || /* GET_LEVEL */
     call_number == 0x001C || /* GET_DIR_ENTRY */
     call_number == 0x0020 || /* GET_DEV_NUM */
     call_number == 0x0021 || /* GET_LAST_DEV */ 
     call_number == 0x0022 || /* READ_BLOCK */
     call_number == 0x0023 || /* WRITE_BLOCK */
     call_number == 0x0024 || /* FORMAT */
     call_number == 0x0025 || /* ERASE_DISK */
     call_number == 0x0027 || /* GET_NAME */
     call_number == 0x0028 || /* GET_BOOT_VOL */
     call_number == 0x0029 || /* QUIT */
     call_number == 0x002A || /* GET_VERSION */
     call_number == 0x002C || /* D_INFO */
     call_number == 0x0031 || /* ALLOC_INTERRUPT */
     call_number == 0x0032)   /* DEALLOC_INTERRUPT */
      return(1);

  /* Other */
  return(0);
}


/**********************************************************************************/
/*  GetCallProdos8AllParameter() :  Rcupre les paramtres de l'appel  Prodos8. */
/**********************************************************************************/
static char *GetCallProdos8AllParameter(DWORD pc_address, DWORD call_address, DWORD return_address, BYTE call_number, struct cyrene_operation *current_operation, struct cyrene_parameter *cyrene_param)
{
  BYTE nb_param;
  DWORD param_address, string_address, param_address_prt;
  static char buffer_line[BUFFER_SIZE];

  /** In Line **/
  param_address_prt = pc_address + 3 + 1;

  /* Adresse du dbut des paramtres (  $0000 si aucun paramtre) */
  param_address = GetMemoryValue(param_address_prt,2,current_operation,cyrene_param);

  /** On va traiter les Appels Systmes les plus courrants */
  switch(call_number)
    {
      case 0xC0 :
        strcpy(buffer_line,"CREATE(");
        /* #00 BYTE  : pCount */
        nb_param = (param_address == 0x00000000) ? 0x00 : (BYTE) GetMemoryValue(param_address+0,1,current_operation,cyrene_param);
        /* #01 WORD  : PathName       IN  +1 */
        string_address = GetMemoryValue(param_address+1,2,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,nb_param,string_address,TYPE_GSOS_PARAM_P8STRING,buffer_line,current_operation,cyrene_param);
        /* #02 BYTE  : Access Bits    IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+3,TYPE_GSOS_PARAM_BYTE,buffer_line,current_operation,cyrene_param);
        /* #03 BYTE  : File Type      IN  +2 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_BYTE,buffer_line,current_operation,cyrene_param);
        /* #04 WORD  : Aux File Type  IN  +2 */
        GetGsOsCallOneParameter(4,nb_param,param_address+5,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #05 BYTE  : Storage Type   IN  +2 */
        GetGsOsCallOneParameter(5,nb_param,param_address+7,TYPE_GSOS_PARAM_BYTE,buffer_line,current_operation,cyrene_param);
        /* #06 WORD  : Creation Date  IN  +2 */
        GetGsOsCallOneParameter(6,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #07 WORD  : Creatipn Time  IN  +2 */
        GetGsOsCallOneParameter(7,nb_param,param_address+10,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P8,0,0,0,0,cyrene_param);
        break;

      case 0xC1 :
        strcpy(buffer_line,"DESTROY(");
        /* #00 BYTE  : pCount */
        nb_param = (param_address == 0x00000000) ? 0x00 : (BYTE) GetMemoryValue(param_address+0,1,current_operation,cyrene_param);
        /* #01 WORD  : PathName       IN  +1 */
        string_address = GetMemoryValue(param_address+1,2,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,nb_param,string_address,TYPE_GSOS_PARAM_P8STRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P8,0,0,0,0,cyrene_param);
        break;

      case 0xC2 :
        strcpy(buffer_line,"RENAME(");
        /* #00 BYTE  : pCount */
        nb_param = (param_address == 0x00000000) ? 0x00 : (BYTE) GetMemoryValue(param_address+0,1,current_operation,cyrene_param);
        /* #01 WORD  : Old PathName   IN  +1 */
        string_address = GetMemoryValue(param_address+1,2,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,nb_param,string_address,TYPE_GSOS_PARAM_P8STRING,buffer_line,current_operation,cyrene_param);
        /* #01 WORD  : New PathName   IN  +3 */
        string_address = GetMemoryValue(param_address+3,2,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param,string_address,TYPE_GSOS_PARAM_P8STRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P8,0,0,0,0,cyrene_param);
        break;

      case 0xC3 :
        strcpy(buffer_line,"SET_FILE_INFO(");
        /* #00 BYTE  : pCount */
        nb_param = (param_address == 0x00000000) ? 0x00 : (BYTE) GetMemoryValue(param_address+0,1,current_operation,cyrene_param);
        /* #01 WORD  : PathName        IN  +1 */
        string_address = GetMemoryValue(param_address+1,2,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,nb_param,string_address,TYPE_GSOS_PARAM_P8STRING,buffer_line,current_operation,cyrene_param);
        /* #02 BYTE  : Access Bits     IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+3,TYPE_GSOS_PARAM_BYTE,buffer_line,current_operation,cyrene_param);
        /* #03 BYTE  : File Type       IN  +2 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_BYTE,buffer_line,current_operation,cyrene_param);
        /* #04 WORD  : Aux File Type   IN  +2 */
        GetGsOsCallOneParameter(4,nb_param,param_address+5,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* 3 bytes   : Reserved */
        /* #05 WORD  : LastModif Date  IN  +2 */
        GetGsOsCallOneParameter(5,nb_param,param_address+10,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #06 WORD  : LastModif Time  IN  +2 */
        GetGsOsCallOneParameter(6,nb_param,param_address+12,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P8,0,0,0,0,cyrene_param);
        break;

      case 0xC4 :
        strcpy(buffer_line,"GET_FILE_INFO(");
        /* #00 BYTE  : pCount */
        nb_param = (param_address == 0x00000000) ? 0x00 : (BYTE) GetMemoryValue(param_address+0,1,current_operation,cyrene_param);
        /* #01 WORD  : PathName        IN  +1 */
        string_address = GetMemoryValue(param_address+1,2,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,nb_param,string_address,TYPE_GSOS_PARAM_P8STRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P8,0,0,0,0,cyrene_param);
        break;

      case 0xC5 :
        strcpy(buffer_line,"ON_LINE(");
        /* #00 BYTE  : pCount */
        nb_param = (param_address == 0x00000000) ? 0x00 : (BYTE) GetMemoryValue(param_address+0,1,current_operation,cyrene_param);
        /* #01 BYTE  : Unit Number          IN  +1 */
        GetGsOsCallOneParameter(1,nb_param,param_address+1,TYPE_GSOS_PARAM_BYTE,buffer_line,current_operation,cyrene_param);
        /* #01 WORD  : Data Buffer Address  IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P8,0,0,0,0,cyrene_param);
        break;

      case 0xC6 :
        strcpy(buffer_line,"SET_PREFIX(");
        /* #00 BYTE  : pCount */
        nb_param = (param_address == 0x00000000) ? 0x00 : (BYTE) GetMemoryValue(param_address+0,1,current_operation,cyrene_param);
        /* #01 WORD  : PathName        IN  +1 */
        string_address = GetMemoryValue(param_address+1,2,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,nb_param,string_address,TYPE_GSOS_PARAM_P8STRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P8,0,0,0,0,cyrene_param);
        break;

      case 0xC7 :
        strcpy(buffer_line,"GET_PREFIX(");
        /* #00 BYTE  : pCount */
        nb_param = (param_address == 0x00000000) ? 0x00 : (BYTE) GetMemoryValue(param_address+0,1,current_operation,cyrene_param);
        /* #01 WORD  : PathName        IN  +1 */
        GetGsOsCallOneParameter(1,nb_param,param_address+1,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P8,0,0,0,0,cyrene_param);
        break;

      case 0xC8 :
        strcpy(buffer_line,"OPEN(");
        /* #00 BYTE  : pCount */
        nb_param = (param_address == 0x00000000) ? 0x00 : (BYTE) GetMemoryValue(param_address+0,1,current_operation,cyrene_param);
        /* #01 WORD  : PathName            IN  +1 */
        string_address = GetMemoryValue(param_address+1,2,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,nb_param,string_address,TYPE_GSOS_PARAM_P8STRING,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : FileBuffer Address  IN  +3 */
        GetGsOsCallOneParameter(2,nb_param,param_address+3,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 BYTE  : Reference Number    OUT +5 */
        GetEmptyParameter(3,nb_param,buffer_line);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P8,3,nb_param,param_address+5,TYPE_GSOS_PARAM_BYTE,cyrene_param);
        break;

      case 0xC9 :
        strcpy(buffer_line,"NEWLINE(");
        /* #00 BYTE  : pCount */
        nb_param = (param_address == 0x00000000) ? 0x00 : (BYTE) GetMemoryValue(param_address+0,1,current_operation,cyrene_param);
        /* #01 BYTE  : Reference Number    IN  +1 */
        GetGsOsCallOneParameter(1,nb_param,param_address+1,TYPE_GSOS_PARAM_BYTE,buffer_line,current_operation,cyrene_param);
        /* #02 BYTE  : And Mask            IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_BYTE,buffer_line,current_operation,cyrene_param);
        /* #03 BYTE  : Newline Character   IN  +3 */
        GetGsOsCallOneParameter(3,nb_param,param_address+3,TYPE_GSOS_PARAM_BYTE,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P8,0,0,0,0,cyrene_param);
        break;

      case 0xCA :
        strcpy(buffer_line,"READ(");
        /* #00 BYTE  : pCount */
        nb_param = (param_address == 0x00000000) ? 0x00 : (BYTE) GetMemoryValue(param_address+0,1,current_operation,cyrene_param);
        /* #01 BYTE  : Reference Number     IN  +1 */
        GetGsOsCallOneParameter(1,nb_param,param_address+1,TYPE_GSOS_PARAM_BYTE,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : Data Buffer Address  IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : Requested Length     IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD  : Actual Length        OUT +6 */
        GetEmptyParameter(4,nb_param,buffer_line);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P8,4,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0xCB :
        strcpy(buffer_line,"WRITE(");
        /* #00 BYTE  : pCount */
        nb_param = (param_address == 0x00000000) ? 0x00 : (BYTE) GetMemoryValue(param_address+0,1,current_operation,cyrene_param);
        /* #01 BYTE  : Reference Number     IN  +1 */
        GetGsOsCallOneParameter(1,nb_param,param_address+1,TYPE_GSOS_PARAM_BYTE,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : Data Buffer Address  IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : Requested Length     IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD  : Actual Length        OUT +6 */
        GetEmptyParameter(4,nb_param,buffer_line);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P8,4,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0xCC :
        strcpy(buffer_line,"CLOSE(");
        /* #00 BYTE  : pCount */
        nb_param = (param_address == 0x00000000) ? 0x00 : (BYTE) GetMemoryValue(param_address+0,1,current_operation,cyrene_param);
        /* #01 BYTE  : Reference Number     IN  +1 */
        GetGsOsCallOneParameter(1,nb_param,param_address+1,TYPE_GSOS_PARAM_BYTE,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P8,0,0,0,0,cyrene_param);
        break;

      case 0xCD :
        strcpy(buffer_line,"FLUSH(");
        /* #00 BYTE  : pCount */
        nb_param = (param_address == 0x00000000) ? 0x00 : (BYTE) GetMemoryValue(param_address+0,1,current_operation,cyrene_param);
        /* #01 BYTE  : Reference Number     IN  +1 */
        GetGsOsCallOneParameter(1,nb_param,param_address+1,TYPE_GSOS_PARAM_BYTE,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P8,0,0,0,0,cyrene_param);
        break;

      case 0xCE :
        strcpy(buffer_line,"SET_MARK(");
        /* #00 BYTE  : pCount */
        nb_param = (param_address == 0x00000000) ? 0x00 : (BYTE) GetMemoryValue(param_address+0,1,current_operation,cyrene_param);
        /* #01 BYTE  : Reference Number     IN  +1 */
        GetGsOsCallOneParameter(1,nb_param,param_address+1,TYPE_GSOS_PARAM_BYTE,buffer_line,current_operation,cyrene_param);
        /* #02 24Bit : New File Position    IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_24BIT,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P8,0,0,0,0,cyrene_param);
        break;

      case 0xCF :
        strcpy(buffer_line,"GET_MARK(");
        /* #00 BYTE  : pCount */
        nb_param = (param_address == 0x00000000) ? 0x00 : (BYTE) GetMemoryValue(param_address+0,1,current_operation,cyrene_param);
        /* #01 BYTE  : Reference Number     IN  +1 */
        GetGsOsCallOneParameter(1,nb_param,param_address+1,TYPE_GSOS_PARAM_BYTE,buffer_line,current_operation,cyrene_param);
        /* #02 24Bit : File Position        OUT +2 */
        GetEmptyParameter(2,nb_param,buffer_line);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P8,2,nb_param,param_address+2,TYPE_GSOS_PARAM_24BIT,cyrene_param);
        break;

      case 0xD0 :
        strcpy(buffer_line,"SET_EOF(");
        /* #00 BYTE  : pCount */
        nb_param = (param_address == 0x00000000) ? 0x00 : (BYTE) GetMemoryValue(param_address+0,1,current_operation,cyrene_param);
        /* #01 BYTE  : Reference Number     IN  +1 */
        GetGsOsCallOneParameter(1,nb_param,param_address+1,TYPE_GSOS_PARAM_BYTE,buffer_line,current_operation,cyrene_param);
        /* #02 24Bit : New EOF Position    IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_24BIT,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P8,0,0,0,0,cyrene_param);
        break;

      case 0xD1 :
        strcpy(buffer_line,"GET_EOF(");
        /* #00 BYTE  : pCount */
        nb_param = (param_address == 0x00000000) ? 0x00 : (BYTE) GetMemoryValue(param_address+0,1,current_operation,cyrene_param);
        /* #01 BYTE  : Reference Number     IN  +1 */
        GetGsOsCallOneParameter(1,nb_param,param_address+1,TYPE_GSOS_PARAM_BYTE,buffer_line,current_operation,cyrene_param);
        /* #02 24Bit : EOF Position         OUT +2 */
        GetEmptyParameter(2,nb_param,buffer_line);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P8,2,nb_param,param_address+2,TYPE_GSOS_PARAM_24BIT,cyrene_param);
        break;

      case 0xD2 :
        strcpy(buffer_line,"SET_BUF(");
        /* #00 BYTE  : pCount */
        nb_param = (param_address == 0x00000000) ? 0x00 : (BYTE) GetMemoryValue(param_address+0,1,current_operation,cyrene_param);
        /* #01 BYTE  : Reference Number         IN  +1 */
        GetGsOsCallOneParameter(1,nb_param,param_address+1,TYPE_GSOS_PARAM_BYTE,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : New File Buffer Address  IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P8,0,0,0,0,cyrene_param);
        break;

      case 0xD3 :
        strcpy(buffer_line,"GET_BUF(");
        /* #00 BYTE  : pCount */
        nb_param = (param_address == 0x00000000) ? 0x00 : (BYTE) GetMemoryValue(param_address+0,1,current_operation,cyrene_param);
        /* #01 BYTE  : Reference Number         IN  +1 */
        GetGsOsCallOneParameter(1,nb_param,param_address+1,TYPE_GSOS_PARAM_BYTE,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : File Buffer Address      OUT +2 */
        GetEmptyParameter(2,nb_param,buffer_line);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P8,2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x40 :
        strcpy(buffer_line,"ALLOC_INTERRUPT(");
        /* #00 BYTE  : pCount */
        nb_param = (param_address == 0x00000000) ? 0x00 : (BYTE) GetMemoryValue(param_address+0,1,current_operation,cyrene_param);
        /* #01 BYTE  : Priority        IN  +1 */
        GetGsOsCallOneParameter(1,nb_param,param_address+1,TYPE_GSOS_PARAM_BYTE,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : Handler Address IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P8,0,0,0,0,cyrene_param);
        break;

      case 0x41 :
        strcpy(buffer_line,"DEALLOC_INTERRUPT(");
        /* #00 BYTE  : pCount */
        nb_param = (param_address == 0x00000000) ? 0x00 : (BYTE) GetMemoryValue(param_address+0,1,current_operation,cyrene_param);
        /* #01 BYTE  : Priority        IN  +1 */
        GetGsOsCallOneParameter(1,nb_param,param_address+1,TYPE_GSOS_PARAM_BYTE,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P8,0,0,0,0,cyrene_param);
        break;

      case 0x65 :
        strcpy(buffer_line,"QUIT(");
        /* #00 BYTE  : pCount */
        nb_param = (param_address == 0x00000000) ? 0x00 : (BYTE) GetMemoryValue(param_address+0,1,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P8,0,0,0,0,cyrene_param);
        break;

      case 0x80 :
        strcpy(buffer_line,"READ_BLOCK(");
        /* #00 BYTE  : pCount */
        nb_param = (param_address == 0x00000000) ? 0x00 : (BYTE) GetMemoryValue(param_address+0,1,current_operation,cyrene_param);
        /* #01 BYTE  : Unit Number         IN  +1 */
        GetGsOsCallOneParameter(1,nb_param,param_address+1,TYPE_GSOS_PARAM_BYTE,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : Data Buffer Address IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : Block Number        IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P8,0,0,0,0,cyrene_param);
        break;

      case 0x81 :
        strcpy(buffer_line,"WRITE_BLOCK(");
        /* #00 BYTE  : pCount */
        nb_param = (param_address == 0x00000000) ? 0x00 : (BYTE) GetMemoryValue(param_address+0,1,current_operation,cyrene_param);
        /* #01 BYTE  : Unit Number         IN  +1 */
        GetGsOsCallOneParameter(1,nb_param,param_address+1,TYPE_GSOS_PARAM_BYTE,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : Data Buffer Address IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : Block Number        IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P8,0,0,0,0,cyrene_param);
        break;

      case 0x82 :
        strcpy(buffer_line,"GET_TIME(");
        /* #00 BYTE  : pCount */
        nb_param = (param_address == 0x00000000) ? 0x00 : (BYTE) GetMemoryValue(param_address+0,1,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_P8,0,0,0,0,cyrene_param);
        break;

      default :
        return(NULL);
        break;
    }

  /* Renvoi la ligne */
  strcat(buffer_line,")");
  return(&buffer_line[0]);
}


/*************************************************************************************/
/*  GetCallToolboxAllParameter() :  Rcupre les paramtres de l'appel  la Toolbox. */
/*************************************************************************************/
static char *GetCallToolboxAllParameter(DWORD stack_address, DWORD pc_address, DWORD call_address, DWORD return_address, WORD call_number, struct cyrene_operation *current_operation, struct cyrene_parameter *cyrene_param)
{
  int nb_param;
  WORD string_length;
  DWORD param_address, string_address;
  static char buffer_line[BUFFER_SIZE];

  /** Adresse des paramtres **/
  param_address = stack_address + 1;

  /** On va traiter les Appels Toolbox les plus courrants */
  switch(call_number)
    {
      /*******************************/
      /*  Tools 0x01 : Tool Locator  */
      /*******************************/
      case 0x0101 :
        strcpy(buffer_line,"TLBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0201 :
        strcpy(buffer_line,"TLStartUp(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0301 :
        strcpy(buffer_line,"TLShutDown(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0401 :
        strcpy(buffer_line,"TLVersion(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0501 :
        strcpy(buffer_line,"TLReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0601 :
        strcpy(buffer_line,"TLStatus(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0B01 :
        strcpy(buffer_line,"GetFuncPtr(");
        /* #01 DWORD : longspace        OUT +4 */
        GetEmptyParameter(1,4,buffer_line);
        /* #02 WORD  : userOrSystem     IN  +2 */
        GetGsOsCallOneParameter(2,4,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 BYTE  : funcNum          IN  +1 */
        GetGsOsCallOneParameter(3,4,param_address+1,TYPE_GSOS_PARAM_BYTE,buffer_line,current_operation,cyrene_param);
        /* #04 BYTE  : tsNum            IN  +0 */
        GetGsOsCallOneParameter(4,4,param_address+0,TYPE_GSOS_PARAM_BYTE,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,4,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0901 :
        strcpy(buffer_line,"GetTSPtr(");
        /* #01 DWORD : longspace        OUT +4 */
        GetEmptyParameter(1,3,buffer_line);
        /* #02 WORD  : userOrSystem     IN  +2 */
        GetGsOsCallOneParameter(2,3,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : tsNum            IN  +0 */
        GetGsOsCallOneParameter(3,3,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,3,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0C01 :
        strcpy(buffer_line,"GetWAP(");
        /* #01 DWORD : longspace        OUT +4 */
        GetEmptyParameter(1,3,buffer_line);
        /* #02 WORD  : userOrSystem     IN  +2 */
        GetGsOsCallOneParameter(2,3,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : tsNum            IN  +0 */
        GetGsOsCallOneParameter(3,3,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,3,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0F01 :
        strcpy(buffer_line,"LoadOneTool(");
        /* #01 WORD  : toolNumber       IN  +2 */
        GetGsOsCallOneParameter(1,2,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : minVersion       IN  +0 */
        GetGsOsCallOneParameter(2,2,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0E01 :
        strcpy(buffer_line,"LoadTools(");
        /* #01 DWORD : toolTablePtr     IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1501 :
        strcpy(buffer_line,"MessageCenter(");
        /* #01 WORD  : action           IN  +6 */
        GetGsOsCallOneParameter(1,3,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : type             IN  +4 */
        GetGsOsCallOneParameter(2,3,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : messageHandle    IN  +0 */
        GetGsOsCallOneParameter(3,3,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1401 :
        strcpy(buffer_line,"RestoreTextState(");
        /* #01 DWORD : stateHandle      IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1301 :
        strcpy(buffer_line,"SaveTextState(");
        /* #01 DWORD : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0A01 :
        strcpy(buffer_line,"SetTSPtr(");
        /* #01 WORD  : userOrSystem     IN  +6 */
        GetGsOsCallOneParameter(1,3,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : tsNum            IN  +4 */
        GetGsOsCallOneParameter(2,3,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : ftpPtr           IN  +0 */
        GetGsOsCallOneParameter(3,3,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0D01 :
        strcpy(buffer_line,"SetWAP(");
        /* #01 WORD  : userOrSystem     IN  +6 */
        GetGsOsCallOneParameter(1,3,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : tsNum            IN  +4 */
        GetGsOsCallOneParameter(2,3,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : waptPtr          IN  +0 */
        GetGsOsCallOneParameter(3,3,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1101 :
        strcpy(buffer_line,"TLMountVolume(");
        /* #01 WORD  : wordspace        OUT +20 */
        GetEmptyParameter(1,7,buffer_line);
        /* #02 WORD  : whereX           IN  +18 */
        GetGsOsCallOneParameter(2,7,param_address+18,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : whereY           IN  +16 */
        GetGsOsCallOneParameter(3,7,param_address+16,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD : line1Ptr         IN  +12 */
        string_address = GetMemoryValue(param_address+12,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(4,7,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #05 DWORD : line2Ptr         IN  +8 */
        string_address = GetMemoryValue(param_address+8,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(5,7,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #06 DWORD : but1Ptr          IN  +4 */
        string_address = GetMemoryValue(param_address+4,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(6,7,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #07 DWORD : but2Ptr          IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(7,7,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,7,param_address+20,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1201 :
        strcpy(buffer_line,"TLTextMountVolume(");
        /* #01 WORD  : wordspace        OUT +16 */
        GetEmptyParameter(1,5,buffer_line);
        /* #02 DWORD : line1Ptr         IN  +12 */
        string_address = GetMemoryValue(param_address+12,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,5,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : line2Ptr         IN  +8 */
        string_address = GetMemoryValue(param_address+8,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(3,5,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD : button1Ptr       IN  +4 */
        string_address = GetMemoryValue(param_address+4,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(4,5,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #05 DWORD : button2Ptr       IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(5,5,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,5,param_address+16,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1001 :
        strcpy(buffer_line,"UnloadOneTool(");
        /* #01 WORD  : toolNumber       IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1701 :
        strcpy(buffer_line,"MessageByName(");
        /* #01 DWORD : Space            OUT +6 */
        GetEmptyParameter(1,3,buffer_line);
        /* #02 WORD  : createItFlag     IN  +4 */
        GetGsOsCallOneParameter(2,3,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : recordPointer    IN  +0 */
        GetGsOsCallOneParameter(3,3,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,3,param_address+6,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x1601 :
        strcpy(buffer_line,"SetDefaultTPT(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1901 :
        strcpy(buffer_line,"ShutDownTools(");
        /* #01 WORD  : startStopRefDesc IN  +4 */
        GetGsOsCallOneParameter(1,2,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : startStopRef     IN  +0 */
        GetGsOsCallOneParameter(2,2,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1801 :
        strcpy(buffer_line,"StartUpTools(");
        /* #01 DWORD : Space            OUT +8 */
        GetEmptyParameter(1,4,buffer_line);
        /* #02 WORD  : userID           IN  +6 */
        GetGsOsCallOneParameter(2,4,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : startStopRefDesc IN  +4 */
        GetGsOsCallOneParameter(3,4,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD : startStopRef     IN  +0 */
        GetGsOsCallOneParameter(4,4,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,4,param_address+8,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x1B01 :
        strcpy(buffer_line,"AcceptRequests(");
        /* #02 DWORD : nameString       IN  +6 */
        string_address = GetMemoryValue(param_address+6,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,3,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : userID           IN  +4 */
        GetGsOsCallOneParameter(2,3,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD : requestProc      IN  +0 */
        GetGsOsCallOneParameter(3,3,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1A01 :
        strcpy(buffer_line,"GetMsgHandle(");
        /* #01 DWORD : Space            OUT +6 */
        GetEmptyParameter(1,3,buffer_line);
        /* #02 WORD  : flags            IN  +4 */
        GetGsOsCallOneParameter(2,3,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : messageRef       IN  +0 */
        GetGsOsCallOneParameter(3,3,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,3,param_address+6,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x1C01 :
        strcpy(buffer_line,"SendRequest(");
        /* #01 WORD  : reqCode          IN  +14 */
        GetGsOsCallOneParameter(1,5,param_address+14,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : sendHow          IN  +12 */
        GetGsOsCallOneParameter(2,5,param_address+12,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : target           IN  +8 */
        GetGsOsCallOneParameter(3,5,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD : dataIn           IN  +4 */
        GetGsOsCallOneParameter(4,5,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #05 DWORD : dataOut          IN  +0 */
        GetGsOsCallOneParameter(5,5,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;


      /*********************************/
      /*  Tools 0x02 : Memory Manager  */
      /*********************************/
      case 0x0102 :
        strcpy(buffer_line,"MMBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0202 :
        strcpy(buffer_line,"MMStartUp(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0302 :
        strcpy(buffer_line,"MMShutDown(");
        /* #01 WORD  : userID           IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0402 :
        strcpy(buffer_line,"MMVersion(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0502 :
        strcpy(buffer_line,"MMReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0602 :
        strcpy(buffer_line,"MMStatus(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x2B02 :
        strcpy(buffer_line,"BlockMove(");
        /* #01 DWORD : sourcePtr        IN  +8 */
        GetGsOsCallOneParameter(1,3,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : destPtr          IN  +4 */
        GetGsOsCallOneParameter(2,3,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : count            IN  +0 */
        GetGsOsCallOneParameter(3,3,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1E02 :
        strcpy(buffer_line,"CheckHandle(");
        /* #01 DWORD : theHandle        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1F02 :
        strcpy(buffer_line,"CompactMem(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1102 :
        strcpy(buffer_line,"DisposeAll(");
        /* #01 WORD  : userID           IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1002 :
        strcpy(buffer_line,"DisposeHandle(");
        /* #01 DWORD : theHandle        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1A02 :
        strcpy(buffer_line,"FindHandle(");
        /* #01 DWORD : longspace        OUT +4 */
        GetEmptyParameter(1,2,buffer_line);
        /* #02 DWORD : locationPtr      IN  +0 */
        GetGsOsCallOneParameter(2,2,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,2,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x1B02 :
        strcpy(buffer_line,"FreeMem(");
        /* #01 DWORD : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x1802 :
        strcpy(buffer_line,"GetHandleSize(");
        /* #01 DWORD : longspace        OUT +4 */
        GetEmptyParameter(1,2,buffer_line);
        /* #02 DWORD : theHandle        IN  +0 */
        GetGsOsCallOneParameter(2,2,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,2,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x2A02 :
        strcpy(buffer_line,"HandToHand(");
        /* #01 DWORD : sourceHandle     IN  +8 */
        GetGsOsCallOneParameter(1,3,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : destHandle       IN  +4 */
        GetGsOsCallOneParameter(2,3,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : count            IN  +0 */
        GetGsOsCallOneParameter(3,3,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2902 :
        strcpy(buffer_line,"HandToPtr(");
        /* #01 DWORD : sourceHandle     IN  +8 */
        GetGsOsCallOneParameter(1,3,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : destPtr          IN  +4 */
        GetGsOsCallOneParameter(2,3,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : count            IN  +0 */
        GetGsOsCallOneParameter(3,3,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2002 :
        strcpy(buffer_line,"HLock(");
        /* #01 DWORD : theHandle        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2102 :
        strcpy(buffer_line,"HLockAll(");
        /* #01 WORD  : userID           IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2202 :
        strcpy(buffer_line,"HUnlock(");
        /* #01 DWORD : theHandle        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2302 :
        strcpy(buffer_line,"HUnlockAll(");
        /* #01 WORD  : userID           IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1C02 :
        strcpy(buffer_line,"MaxBlock(");
        /* #01 DWORD : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0902 :
        strcpy(buffer_line,"NewHandle(");
        /* #01 DWORD : longspace        OUT +12 */
        GetEmptyParameter(1,5,buffer_line);
        /* #02 DWORD : blockSize        IN  +8 */
        GetGsOsCallOneParameter(2,5,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : userID           IN  +6 */
        GetGsOsCallOneParameter(3,5,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD  : attributes       IN  +4 */
        GetGsOsCallOneParameter(4,5,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #05 DWORD : locationPtr      IN  +0 */
        GetGsOsCallOneParameter(5,5,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,5,param_address+12,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x2802 :
        strcpy(buffer_line,"PtrToHand(");
        /* #01 DWORD : sourcePtr        IN  +8 */
        GetGsOsCallOneParameter(1,3,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : destHandle       IN  +4 */
        GetGsOsCallOneParameter(2,3,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : count            IN  +0 */
        GetGsOsCallOneParameter(3,3,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1302 :
        strcpy(buffer_line,"PurgeAll(");
        /* #01 WORD  : userID           IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1202 :
        strcpy(buffer_line,"PurgeHandle(");
        /* #01 DWORD : theHandle        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0A02 :
        strcpy(buffer_line,"ReallocHandle(");
        /* #01 DWORD : blockSize        IN  +12 */
        GetGsOsCallOneParameter(1,5,param_address+12,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02  WORD : userID           IN  +10 */
        GetGsOsCallOneParameter(2,5,param_address+10,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03  WORD : attributes       IN  +8 */
        GetGsOsCallOneParameter(3,5,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD : locationPtr      IN  +4 */
        GetGsOsCallOneParameter(4,5,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #05 DWORD : theHandle        IN  +0 */
        GetGsOsCallOneParameter(5,5,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0B02 :
        strcpy(buffer_line,"RestoreHandle(");
        /* #01 DWORD : theHandle        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1902 :
        strcpy(buffer_line,"SetHandleSize(");
        /* #01 DWORD : newSize          IN  +4 */
        GetGsOsCallOneParameter(1,2,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : theHandle        IN  +0 */
        GetGsOsCallOneParameter(2,2,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2402 :
        strcpy(buffer_line,"SetPurge(");
        /* #01 WORD  : newPurgeLevel    IN  +4 */
        GetGsOsCallOneParameter(1,2,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : theHandle        IN  +0 */
        GetGsOsCallOneParameter(2,2,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2502 :
        strcpy(buffer_line,"SetPurgeAll(");
        /* #01 WORD  : newPurgeLevel    IN  +2 */
        GetGsOsCallOneParameter(1,2,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : userID           IN  +0 */
        GetGsOsCallOneParameter(2,2,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1D02 :
        strcpy(buffer_line,"TotalMem(");
        /* #01 DWORD : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0C02 :
        strcpy(buffer_line,"AddToOOMQueue(");
        /* #01 DWORD : headerPtr        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2F02 :
        strcpy(buffer_line,"RealFreeMem(");
        /* #01 DWORD : Space            OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0D02 :
        strcpy(buffer_line,"RemoveFromOOMQueue(");
        /* #01 DWORD : headerPtr        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3002 :
        strcpy(buffer_line,"SetHandleID(");
        /* #01 WORD  : Space            OUT +6 */
        GetEmptyParameter(1,3,buffer_line);
        /* #02 WORD  : newID            IN  +4 */
        GetGsOsCallOneParameter(2,3,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : theHandle        IN  +0 */
        GetGsOsCallOneParameter(3,3,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,3,param_address+6,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;


      /********************************/
      /*  Tools 0x03 : Miscellaneous  */
      /********************************/
      case 0x0103 :
        strcpy(buffer_line,"MTBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0203 :
        strcpy(buffer_line,"MTStartUp(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0303 :
        strcpy(buffer_line,"MTShutDown(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0403 :
        strcpy(buffer_line,"MTVersion(");
        /* #01 WORD  : wordSpace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0503 :
        strcpy(buffer_line,"MTReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0603 :
        strcpy(buffer_line,"MTStatus(");
        /* #01 WORD  : wordSpace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0903 :
        strcpy(buffer_line,"WriteBRam(");
        /* #01 DWORD : bufferPtr        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0A03 :
        strcpy(buffer_line,"ReadBRam(");
        /* #01 DWORD : bufferPtr        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0B03 :
        strcpy(buffer_line,"WriteBParam(");
        /* #01 WORD  : theData          IN  +2 */
        GetGsOsCallOneParameter(1,2,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : paramRefNum      IN  +0 */
        GetGsOsCallOneParameter(2,2,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0C03 :
        strcpy(buffer_line,"ReadBParam(");
        /* #01 WORD  : wordSpace        OUT +2 */
        GetEmptyParameter(1,1,buffer_line);
        /* #02 WORD  : paramRefNum      IN  +0 */
        GetGsOsCallOneParameter(2,2,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,2,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0D03 :
        strcpy(buffer_line,"ReadTimeHex(");
        /* #01 WORD  : wordSpace        OUT +6 */
        GetEmptyParameter(1,4,buffer_line);
        /* #02 WORD  : wordSpace        OUT +4 */
        GetEmptyParameter(2,4,buffer_line);
        /* #03 WORD  : wordSpace        OUT +2 */
        GetEmptyParameter(3,4,buffer_line);
        /* #04 WORD  : wordSpace        OUT +0 */
        GetEmptyParameter(4,4,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,4,param_address+6,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,2,4,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,3,4,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,4,4,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0E03 :
        strcpy(buffer_line,"WriteTimeHex(");
        /* #01 BYTE  : month            IN  +5 */
        GetGsOsCallOneParameter(1,6,param_address+5,TYPE_GSOS_PARAM_BYTE,buffer_line,current_operation,cyrene_param);
        /* #02 BYTE  : day              IN  +4 */
        GetGsOsCallOneParameter(2,6,param_address+4,TYPE_GSOS_PARAM_BYTE,buffer_line,current_operation,cyrene_param);
        /* #03 BYTE  : curYear          IN  +3 */
        GetGsOsCallOneParameter(3,6,param_address+3,TYPE_GSOS_PARAM_BYTE,buffer_line,current_operation,cyrene_param);
        /* #04 BYTE  : hour             IN  +2 */
        GetGsOsCallOneParameter(4,6,param_address+2,TYPE_GSOS_PARAM_BYTE,buffer_line,current_operation,cyrene_param);
        /* #05 BYTE  : minute           IN  +1 */
        GetGsOsCallOneParameter(5,6,param_address+1,TYPE_GSOS_PARAM_BYTE,buffer_line,current_operation,cyrene_param);
        /* #06 BYTE  : second           IN  +0 */
        GetGsOsCallOneParameter(6,6,param_address+0,TYPE_GSOS_PARAM_BYTE,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0F03 :
        strcpy(buffer_line,"ReadAsciiTime(");
        /* #01 DWORD : bufferPtr        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2403 :
        strcpy(buffer_line,"FWEntry(");
        /* #01 WORD  : wordSpace        OUT +14 */
        GetEmptyParameter(1,8,buffer_line);
        /* #02 WORD  : wordSpace        OUT +12 */
        GetEmptyParameter(2,8,buffer_line);
        /* #03 WORD  : wordSpace        OUT +10 */
        GetEmptyParameter(3,8,buffer_line);
        /* #04 WORD  : wordSpace        OUT +8 */
        GetEmptyParameter(4,8,buffer_line);
        /* #05 WORD  : aRegValue        IN  +6 */
        GetGsOsCallOneParameter(5,8,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #06 WORD  : xRegValue        IN  +4 */
        GetGsOsCallOneParameter(6,8,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #07 WORD  : yRegValue        IN  +2 */
        GetGsOsCallOneParameter(7,8,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #08 WORD  : eModeEntryPt     IN  +0 */
        GetGsOsCallOneParameter(8,8,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,8,param_address+14,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,2,8,param_address+12,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,3,8,param_address+10,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,4,8,param_address+8,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1603 :
        strcpy(buffer_line,"GetAddr(");
        /* #01 DWORD : longSpace        OUT +2 */
        GetEmptyParameter(1,2,buffer_line);
        /* #02 WORD  : refNum           IN  +0 */
        GetGsOsCallOneParameter(2,2,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,2,param_address+2,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x2503 :
        strcpy(buffer_line,"GetTick(");
        /* #01 DWORD : longSpace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x2903 :
        strcpy(buffer_line,"GetIRQEnable(");
        /* #01 WORD  : wordSpace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x2303 :
        strcpy(buffer_line,"IntSource(");
        /* #01 WORD  : srcRefNum           IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1C03 :
        strcpy(buffer_line,"ClampMouse(");
        /* #01 WORD  : xMinClamp           IN  +6 */
        GetGsOsCallOneParameter(1,4,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : xMaxClamp           IN  +4 */
        GetGsOsCallOneParameter(2,4,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : yMinClamp           IN  +2 */
        GetGsOsCallOneParameter(3,4,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD  : yMaxClamp           IN  +0 */
        GetGsOsCallOneParameter(4,4,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1B03 :
        strcpy(buffer_line,"ClearMouse(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1D03 :
        strcpy(buffer_line,"GetMouseClamp(");
        /* #01 WORD  : wordSpace        OUT +6 */
        GetEmptyParameter(1,4,buffer_line);
        /* #02 WORD  : wordSpace        OUT +4 */
        GetEmptyParameter(2,4,buffer_line);
        /* #03 WORD  : wordSpace        OUT +2 */
        GetEmptyParameter(3,4,buffer_line);
        /* #04 WORD  : wordSpace        OUT +0 */
        GetEmptyParameter(4,4,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,4,param_address+6,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,2,4,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,3,4,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,4,4,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1A03 :
        strcpy(buffer_line,"HomeMouse(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1803 :
        strcpy(buffer_line,"InitMouse(");
        /* #01 WORD  : mouseSlot        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1E03 :
        strcpy(buffer_line,"PosMouse(");
        /* #01 WORD  : xPos             IN  +2 */
        GetGsOsCallOneParameter(1,2,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : yPos             IN  +0 */
        GetGsOsCallOneParameter(2,2,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1703 :
        strcpy(buffer_line,"ReadMouse(");
        /* #01 WORD  : wordSpace        OUT +4 */
        GetEmptyParameter(1,3,buffer_line);
        /* #02 WORD  : wordSpace        OUT +2 */
        GetEmptyParameter(2,3,buffer_line);
        /* #03 WORD  : wordSpace        OUT +0 */
        GetEmptyParameter(3,3,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,3,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,2,3,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,3,3,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1F03 :
        strcpy(buffer_line,"ServeMouse(");
        /* #01 WORD  : wordSpace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1903 :
        strcpy(buffer_line,"SetMouse(");
        /* #01 WORD  : mouseMode        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2A03 :
        strcpy(buffer_line,"SetAbsClamp(");
        /* #01 WORD  : xMinClamp        IN  +6 */
        GetGsOsCallOneParameter(1,4,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : xMaxClamp        IN  +4 */
        GetGsOsCallOneParameter(2,4,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : yMinClamp        IN  +2 */
        GetGsOsCallOneParameter(3,4,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD  : yMaxClamp        IN  +0 */
        GetGsOsCallOneParameter(4,4,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2B03 :
        strcpy(buffer_line,"GetAbsClamp(");
        /* #01 WORD  : wordSpace        OUT +6 */
        GetEmptyParameter(1,4,buffer_line);
        /* #02 WORD  : wordSpace        OUT +4 */
        GetEmptyParameter(2,4,buffer_line);
        /* #03 WORD  : wordSpace        OUT +2 */
        GetEmptyParameter(3,4,buffer_line);
        /* #04 WORD  : wordSpace        OUT +0 */
        GetEmptyParameter(4,4,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,4,param_address+6,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,2,4,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,3,4,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,4,4,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x2603 :
        strcpy(buffer_line,"PackBytes(");
        /* #01 WORD  : wordSpace        OUT +14 */
        GetEmptyParameter(1,5,buffer_line);
        /* #02 DWORD : startHandle      IN  +10 */
        GetGsOsCallOneParameter(2,5,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : sizePtr          IN  +6 */
        GetGsOsCallOneParameter(3,5,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD : bufferPtr        IN  +2 */
        GetGsOsCallOneParameter(4,5,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #05 WORD  : bufferSize       IN  +0 */
        GetGsOsCallOneParameter(5,5,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,5,param_address+14,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x2703 :
        strcpy(buffer_line,"UnPackBytes(");
        /* #01 WORD  : wordSpace        OUT +14 */
        GetEmptyParameter(1,5,buffer_line);
        /* #02 DWORD : bufferPtr        IN  +10 */
        GetGsOsCallOneParameter(2,5,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : bufferSize       IN  +8 */
        GetGsOsCallOneParameter(3,5,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD : startHandle      IN  +4 */
        GetGsOsCallOneParameter(4,5,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #05 DWORD : sizePtr          IN  +0 */
        GetGsOsCallOneParameter(5,5,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,5,param_address+14,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x2803 :
        strcpy(buffer_line,"Munger(");
        /* #01 WORD  : wordSpace        OUT +24 */
        GetEmptyParameter(1,8,buffer_line);
        /* #02 DWORD : destPtr          IN  +20 */
        GetGsOsCallOneParameter(2,8,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : destLenPtr       IN  +16 */
        GetGsOsCallOneParameter(3,8,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD : targPtr          IN  +12 */
        GetGsOsCallOneParameter(4,8,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #05 WORD  : targLen          IN  +10 */
        GetGsOsCallOneParameter(5,8,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #06 DWORD : replPtr          IN  +6 */
        GetGsOsCallOneParameter(6,8,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #07 WORD  : replLen          IN  +4 */
        GetGsOsCallOneParameter(7,8,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #08 DWORD : padPtr           IN  +0 */
        GetGsOsCallOneParameter(8,8,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,8,param_address+24,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1203 :
        strcpy(buffer_line,"SetHeartBeat(");
        /* #01 DWORD : taskPtr          IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1303 :
        strcpy(buffer_line,"DelHeartBeat(");
        /* #01 DWORD : taskPtr          IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1403 :
        strcpy(buffer_line,"ClrHeartBeat(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2C03 :
        strcpy(buffer_line,"SysBeep(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1503 :
        strcpy(buffer_line,"SysFailMgr(");
        /* #01 WORD  : errorCode        IN  +4 */
        GetGsOsCallOneParameter(1,2,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #01 DWORD : strPtr           IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,2,string_address,TYPE_GSOS_PARAM_STRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2003 :
        strcpy(buffer_line,"GetNewID(");
        /* #01 WORD  : wordSpace        OUT +2 */
        GetEmptyParameter(1,2,buffer_line);
        /* #02 WORD  : idTag            IN  +0 */
        GetGsOsCallOneParameter(2,2,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,2,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x2103 :
        strcpy(buffer_line,"DeleteID(");
        /* #01 WORD  : idTag            IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2203 :
        strcpy(buffer_line,"StatusID(");
        /* #01 WORD  : idTag            IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1003 :
        strcpy(buffer_line,"SetVector(");
        /* #01 WORD  : vectorRefNum     IN  +4 */
        GetGsOsCallOneParameter(1,2,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : vectorPtr        IN  +0 */
        GetGsOsCallOneParameter(2,2,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1103 :
        strcpy(buffer_line,"GetVector(");
        /* #01 DWORD : longSpace        OUT +2 */
        GetEmptyParameter(1,2,buffer_line);
        /* #02 WORD  : vectorRefNum     IN  +0 */
        GetGsOsCallOneParameter(2,2,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,2,param_address+2,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x2E03 :
        strcpy(buffer_line,"AddToQueue(");
        /* #01 DWORD : newEntryPtr      IN  +4 */
        GetGsOsCallOneParameter(1,2,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : headerPtr        IN  +0 */
        GetGsOsCallOneParameter(2,2,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2F03 :
        strcpy(buffer_line,"DeleteFromQueue(");
        /* #01 DWORD : EntryPtr         IN  +4 */
        GetGsOsCallOneParameter(1,2,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : headerPtr        IN  +0 */
        GetGsOsCallOneParameter(2,2,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3403 :
        strcpy(buffer_line,"GetCodeResConverter(");
        /* #01 DWORD : Space            OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x3103 :
        strcpy(buffer_line,"GetInterruptState(");
        /* #01 DWORD : intStateRcdPtr   IN  +2 */
        GetGsOsCallOneParameter(1,2,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : bytesDesired     IN  +0 */
        GetGsOsCallOneParameter(2,2,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3203 :
        strcpy(buffer_line,"GetIntStateRecSize(");
        /* #01 WORD  : Space            OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x3503 :
        strcpy(buffer_line,"GetROMResource(");
        /* Unknown Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3303 :
        strcpy(buffer_line,"ReadMouse2(");
        /* #01 WORD  : Space            OUT +4 */
        GetEmptyParameter(1,3,buffer_line);
        /* #02 WORD  : Space            OUT +2 */
        GetEmptyParameter(2,3,buffer_line);
        /* #03 WORD  : Space            OUT +0 */
        GetEmptyParameter(3,3,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,3,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,2,3,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,3,3,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x3603 :
        strcpy(buffer_line,"ReleaseROMResource(");
        /* Unknown Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3003 :
        strcpy(buffer_line,"SetInterruptState(");
        /* #01 DWORD : intStateRcdPtr   IN  +2 */
        GetGsOsCallOneParameter(1,2,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : bytesDesired     IN  +0 */
        GetGsOsCallOneParameter(2,2,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3703 :
        strcpy(buffer_line,"ConvSeconds(");
        /* #01 DWORD : Space            OUT +10 */
        GetEmptyParameter(1,4,buffer_line);
        /* #02 WORD  : convVerb         IN  +8 */
        GetGsOsCallOneParameter(2,4,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : seconds          IN  +4 */
        GetGsOsCallOneParameter(3,4,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD : datePtr          IN  +0 */
        GetGsOsCallOneParameter(4,4,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,4,param_address+10,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x3D03 :
        strcpy(buffer_line,"ScanDevices(");
        /* #01 DWORD : Space            OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x3C03 :
        strcpy(buffer_line,"ShowBootInfo(");
        /* #01 DWORD : cStringPtr       IN  +4 */
        string_address = GetMemoryValue(param_address+4,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,2,string_address,TYPE_GSOS_PARAM_STRING,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : iconPtr          IN  +0 */
        GetGsOsCallOneParameter(2,2,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3B03 :
        strcpy(buffer_line,"StringToText(");
        /* #01 WORD  : Space            OUT +14 */
        GetEmptyParameter(1,6,buffer_line);
        /* #02 WORD  : Space            OUT +12 */
        GetEmptyParameter(2,6,buffer_line);
        /* #03 WORD  : flags            IN  +10 */
        GetGsOsCallOneParameter(3,6,param_address+10,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD : textPtr          IN  +6 */
        string_length = (WORD) GetMemoryValue(param_address+4,2,current_operation,cyrene_param);
        string_address = GetMemoryValue(param_address+6,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(4,6,string_address,TYPE_GSOS_PARAM_TXT+string_length,buffer_line,current_operation,cyrene_param);
        /* #05 WORD  : textLen          IN  +4 */
        GetGsOsCallOneParameter(5,6,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #06 DWORD : resultPtr        IN  +0 */
        GetGsOsCallOneParameter(6,6,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,6,param_address+14,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,2,6,param_address+12,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x3803 :
        strcpy(buffer_line,"SysBeep2(");
        /* #01 WORD  : beepType         IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3903 :
        strcpy(buffer_line,"VersionString(");
        /* #01 WORD  : flags            IN  +8 */
        GetGsOsCallOneParameter(3,6,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : theVersion       IN  +4 */
        GetGsOsCallOneParameter(2,3,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : stringPtr        IN  +0 */
        string_length = 10;
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(3,3,string_address,TYPE_GSOS_PARAM_TXT+string_length,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3A03 :
        strcpy(buffer_line,"WaitUntil(");
        /* #01 WORD  : Space            OUT +4 */
        GetEmptyParameter(1,3,buffer_line);
        /* #02 WORD  : delayFrom        IN  +2 */
        GetGsOsCallOneParameter(2,3,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : delayAmount      IN  +0 */
        GetGsOsCallOneParameter(3,3,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,3,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x3E03 :
        strcpy(buffer_line,"AlertMessage(");
        /* #01 WORD  : Space            OUT +10 */
        GetEmptyParameter(1,4,buffer_line);
        /* #02 DWORD : tablePtr         IN  +6 */
        GetGsOsCallOneParameter(2,4,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : msgNumber        IN  +4 */
        GetGsOsCallOneParameter(3,4,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD : substitutions    IN  +0 */
        GetGsOsCallOneParameter(4,4,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,4,param_address+10,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x3F03 :
        strcpy(buffer_line,"DoSysPrefs(");
        /* #01 WORD  : Space            OUT +4 */
        GetEmptyParameter(1,3,buffer_line);
        /* #02 WORD  : bitsToClear      IN  +2 */
        GetGsOsCallOneParameter(2,3,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : bitsToSet        IN  +0 */
        GetGsOsCallOneParameter(3,3,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,3,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;


      /*******************************/
      /*  Tools 0x04 : QuickDraw II  */
      /*******************************/
      case 0x0104 :
        strcpy(buffer_line,"QDBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0204 :
        strcpy(buffer_line,"QDStartUp(");
        nb_param = 4;
        /* #01 WORD  : dPageAddr         IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : masterSCB         IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : maxWidth          IN  +2 */
        GetGsOsCallOneParameter(3,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD  : userID            IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0304 :
        strcpy(buffer_line,"QDShutDown(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0404 :
        strcpy(buffer_line,"QDVersion(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0504 :
        strcpy(buffer_line,"QDReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0604 :
        strcpy(buffer_line,"QDStatus(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0904 :
        strcpy(buffer_line,"GetAddress(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : tableID          IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0A04 : 
        strcpy(buffer_line,"GrafOn(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0B04 :
        strcpy(buffer_line,"GrafOff(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0C04 :
        strcpy(buffer_line,"GetStandardSCB(");
        /* #01 WORD   : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0D04 :
        strcpy(buffer_line,"InitColorTable(");
        /* #01 DWORD  : tablePtr         IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0E04 :
        strcpy(buffer_line,"SetColorTable(");
        nb_param = 2;
        /* #01 WORD   : tableNumber      IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : srcTablePtr      IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0F04 :
        strcpy(buffer_line,"GetColorTable(");
        nb_param = 2;
        /* #01 WORD   : tableNumber      IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : destTablePtr     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1004 :
        strcpy(buffer_line,"SetColorEntry(");
        nb_param = 3;
        /* #01 WORD   : tableNumber      IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : entryNumber      IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : newColor         IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1104 :
        strcpy(buffer_line,"GetColorEntry(");
        nb_param = 3;
        /* #01 WORD   : wordspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : tableNumber      IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : entryNumber      IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1204 :
        strcpy(buffer_line,"SetSCB(");
        nb_param = 2;
        /* #01 WORD   : scanLine        IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : newSCB          IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1304 :
        strcpy(buffer_line,"GetSCB(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : scanLine         IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1404 :
        strcpy(buffer_line,"SetAllSCBs(");
        /* #01 WORD   : newSCB           IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1504 :
        strcpy(buffer_line,"ClearScreen(");
        /* #01 WORD   : colorWord        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1604 :
        strcpy(buffer_line,"SetMasterSCB(");
        /* #01 WORD   : masterSCB        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1704 :
        strcpy(buffer_line,"GetMasterSCB(");
        /* #01 WORD   : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1804 :
        strcpy(buffer_line,"OpenPort(");
        /* #01 DWORD  : portPtr          IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1904 :
        strcpy(buffer_line,"InitPort(");
        /* #01 DWORD  : portPtr          IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1A04 :
        strcpy(buffer_line,"ClosePort(");
        /* #01 DWORD  : portPtr          IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1B04 :
        strcpy(buffer_line,"SetPort(");
        /* #01 DWORD  : portPtr          IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1C04 :
        strcpy(buffer_line,"GetPort(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x1D04 :
        strcpy(buffer_line,"SetPortLoc(");
        /* #01 DWORD  : locInfoPtr       IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1E04 :
        strcpy(buffer_line,"GetPortLoc(");
        /* #01 DWORD  : locInfoPtr       IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1F04 :
        strcpy(buffer_line,"SetPortRect(");
        /* #01 DWORD  : rectPtr          IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2004 :
        strcpy(buffer_line,"GetPortRect(");
        /* #01 DWORD  : rectPtr          IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2104 :
        strcpy(buffer_line,"SetPortSize(");
        nb_param = 2;
        /* #01 WORD   : portWidth        IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : portHeight       IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2204 :
        strcpy(buffer_line,"MovePortTo(");
        nb_param = 2;
        /* #01 WORD   : h                IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : v                IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2304 :
        strcpy(buffer_line,"SetOrigin(");
        nb_param = 2;
        /* #01 WORD   : h                IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : v                IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2404 :
        strcpy(buffer_line,"SetClip(");
        /* #01 DWORD  : rgnHandle        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2504 :
        strcpy(buffer_line,"GetClip(");
        /* #01 DWORD  : rgnHandle        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2604 :
        strcpy(buffer_line,"ClipRect(");
        /* #01 DWORD  : rectPtr          IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2704 :
        strcpy(buffer_line,"HidePen(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2804 :
        strcpy(buffer_line,"ShowPen(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2904 :
        strcpy(buffer_line,"GetPen(");
        /* #01 DWORD  : pointPtr         IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2A04 :
        strcpy(buffer_line,"SetPenState(");
        /* #01 DWORD  : penStatePtr      IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2B04 :
        strcpy(buffer_line,"GetPenState(");
        /* #01 DWORD  : penStatePtr      IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2C04 :
        strcpy(buffer_line,"SetPenSize(");
        nb_param = 2;
        /* #01 WORD   : penWidth         IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : penHeight        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2D04 :
        strcpy(buffer_line,"GetPenSize(");
        /* #01 DWORD  : pointPtr         IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2E04 :
        strcpy(buffer_line,"SetPenMode(");
        /* #01 WORD   : pendMode         IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2F04 :
        strcpy(buffer_line,"GetPenMode(");
        /* #01 WORD   : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x3004 :
        strcpy(buffer_line,"SetPenPat(");
        /* #01 DWORD  : patternPtr       IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3104 :
        strcpy(buffer_line,"GetPenPat(");
        /* #01 DWORD  : patternPtr       IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3204 :
        strcpy(buffer_line,"SetPenMask(");
        /* #01 DWORD  : maskPtr          IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3304 :
        strcpy(buffer_line,"GetPenMask(");
        /* #01 DWORD  : maskPtr          IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3404 :
        strcpy(buffer_line,"SetBackPat(");
        /* #01 DWORD  : patternPtr       IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3504 :
        strcpy(buffer_line,"GetBackPat(");
        /* #01 DWORD  : patternPtr       IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3604 :
        strcpy(buffer_line,"PenNormal(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3704 :
        strcpy(buffer_line,"SetSolidPenPat(");
        /* #01 WORD   : colorNum         IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3804 :
        strcpy(buffer_line,"SetSolidBackPat(");
        /* #01 WORD   : colorNum         IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3904 :
        strcpy(buffer_line,"SolidPattern(");
        nb_param = 2;
        /* #01 WORD   : colorNum         IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : patternPtr       IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3A04 :
        strcpy(buffer_line,"MoveTo(");
        nb_param = 2;
        /* #01 WORD   : h                IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : v                IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3B04 :
        strcpy(buffer_line,"Move(");
        nb_param = 2;
        /* #01 WORD   : dH               IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : dV               IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3C04 :
        strcpy(buffer_line,"LineTo(");
        nb_param = 2;
        /* #01 WORD   : h                IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : v                IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3D04 :
        strcpy(buffer_line,"Line(");
        nb_param = 2;
        /* #01 WORD   : dH               IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : dV               IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3E04 :
        strcpy(buffer_line,"SetPicSave(");
        /* #01 DWORD  : picSaveValue     IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3F04 :
        strcpy(buffer_line,"GetPicSave(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x4004 :
        strcpy(buffer_line,"SetRgnSave(");
        /* #01 DWORD  : rgnSaveValue     IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x4104 :
        strcpy(buffer_line,"GetRgnSave(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x4204 :
        strcpy(buffer_line,"SetPolySave(");
        /* #01 DWORD  : polySaveValue    IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x4304 :
        strcpy(buffer_line,"GetPolySave(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x4404 :
        strcpy(buffer_line,"SetGrafProcs(");
        /* #01 DWORD  : grafProcsPtr     IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x4504 :
        strcpy(buffer_line,"GetGrafProcs(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x4604 :
        strcpy(buffer_line,"SetUserField(");
        /* #01 DWORD  : userFieldValue   IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x4704 :
        strcpy(buffer_line,"GetUserField(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x4804 :
        strcpy(buffer_line,"SetSysField(");
        /* #01 DWORD  : sysFieldValue    IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x4904 :
        strcpy(buffer_line,"GetSysField(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x4A04 :
        strcpy(buffer_line,"SetRect(");
        nb_param = 5;
        /* #01 DWORD  : rectPtr          IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : left             IN  +6 */
        GetGsOsCallOneParameter(2,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : top              IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : right            IN  +2 */
        GetGsOsCallOneParameter(4,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #05 WORD   : bottom           IN  +0 */
        GetGsOsCallOneParameter(5,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x4B04 :
        strcpy(buffer_line,"OffsetRect(");
        nb_param = 3;
        /* #01 DWORD  : rectPtr          IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : dH               IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : dV               IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x4C04 :
        strcpy(buffer_line,"InsetRect(");
        nb_param = 3;
        /* #01 DWORD  : rectPtr          IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : dH               IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : dV               IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x4D04 :
        strcpy(buffer_line,"SectRect(");
        nb_param = 4;
        /* #01 WORD   : wordspace        OUT +12 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : rect1Ptr         IN  +8 */
        GetGsOsCallOneParameter(2,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : rect2Ptr         IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : intersectRectPtr IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+12,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x4E04 :
        strcpy(buffer_line,"UnionRect(");
        nb_param = 3;
        /* #01 DWORD  : rect1Ptr         IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : rect2Ptr         IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : unionRecPtr      IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x4F04 :
        strcpy(buffer_line,"PtInRect(");
        nb_param = 3;
        /* #01 WORD   : wordspace        OUT +8 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : pointPtr         IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : rectPtr          IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x5004 :
        strcpy(buffer_line,"Pt2Rect(");
        nb_param = 3;
        /* #01 DWORD  : point1Ptr        IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : point2Ptr        IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : rectPtr          IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x5104 :
        strcpy(buffer_line,"EqualRect(");
        nb_param = 3;
        /* #01 WORD   : wordspace        OUT +8 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : rect1Ptr         IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : rect2Ptr         IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x5204 :
        strcpy(buffer_line,"NotEmptyRect(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : rectPtr          IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x5304 :
        strcpy(buffer_line,"FrameRect(");
        /* #01 DWORD  : rectPtr          IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x5404 :
        strcpy(buffer_line,"PaintRect(");
        /* #01 DWORD  : rectPtr          IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x5504 :
        strcpy(buffer_line,"EraseRect(");
        /* #01 DWORD  : rectPtr          IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x5604 :
        strcpy(buffer_line,"InvertRect(");
        /* #01 DWORD  : rectPtr          IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x5704 :
        strcpy(buffer_line,"FillRect(");
        nb_param = 2;
        /* #01 DWORD  : rectPtr          IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : patternPtr       IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x5804 :
        strcpy(buffer_line,"FrameOval(");
        /* #01 DWORD  : rectPtr          IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x5904 :
        strcpy(buffer_line,"PaintOval(");
        /* #01 DWORD  : rectPtr          IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x5A04 :
        strcpy(buffer_line,"EraseOval(");
        /* #01 DWORD  : rectPtr          IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x5B04 :
        strcpy(buffer_line,"InvertOval(");
        /* #01 DWORD  : rectPtr          IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x5C04 :
        strcpy(buffer_line,"FillOval(");
        nb_param = 4;
        /* #01 DWORD  : rectPtr          IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : startAngle       IN  +6 */
        GetGsOsCallOneParameter(2,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : arcAngle         IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : patternPtr       IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x5D04 :
        strcpy(buffer_line,"FrameRRect(");
        nb_param = 3;
        /* #01 DWORD  : rectPtr          IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : ovalWidth        IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : ovalHeight       IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x5E04 :
        strcpy(buffer_line,"PaintRRect(");
        nb_param = 3;
        /* #01 DWORD  : rectPtr          IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : ovalWidth        IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : ovalHeight       IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x5F04 :
        strcpy(buffer_line,"EraseRRect(");
        nb_param = 2;
        /* #01 DWORD  : rectPtr          IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : patternPtr       IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x6004 :
        strcpy(buffer_line,"InvertRRect(");
        nb_param = 3;
        /* #01 DWORD  : rectPtr          IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : ovalWidth        IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : ovalHeight       IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x6104 :
        strcpy(buffer_line,"FillRRect(");
        nb_param = 4;
        /* #01 DWORD  : rectPtr          IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : ovalWidth        IN  +6 */
        GetGsOsCallOneParameter(2,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : ovalHeight       IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : patternPtr       IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x6204 :
        strcpy(buffer_line,"FrameArc(");
        nb_param = 3;
        /* #01 DWORD  : rectPtr          IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : startAngle       IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : arcAngle         IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x6304 :
        strcpy(buffer_line,"PaintArc(");
        nb_param = 3;
        /* #01 DWORD  : rectPtr          IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : startAngle       IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : arcAngle         IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x6404 :
        strcpy(buffer_line,"EraseArc(");
        nb_param = 3;
        /* #01 DWORD  : rectPtr          IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : startAngle       IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : arcAngle         IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x6504 :
        strcpy(buffer_line,"InvertArc(");
        nb_param = 3;
        /* #01 DWORD  : rectPtr          IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : startAngle       IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : arcAngle         IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x6604 :
        strcpy(buffer_line,"FillArc(");
        nb_param = 4;
        /* #01 DWORD  : rectPtr          IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : startAngle       IN  +6 */
        GetGsOsCallOneParameter(2,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : arcAngle         IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : patternPtr       IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x6704 :
        strcpy(buffer_line,"NewRgn(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x6804 :
        strcpy(buffer_line,"DisposeRgn(");
        /* #01 DWORD  : rgnHandle        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x6904 :
        strcpy(buffer_line,"CopyRgn(");
        nb_param = 2;
        /* #01 DWORD  : srcRgnHandle     IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : destRgnHandle    IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x6A04 :
        strcpy(buffer_line,"SetEmptyRgn(");
        /* #01 DWORD  : rgnHandle        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x6B04 :
        strcpy(buffer_line,"SetRectRgn(");
        nb_param = 5;
        /* #01 DWORD  : rgnHandle        IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : left             IN  +6 */
        GetGsOsCallOneParameter(2,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : top              IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : right            IN  +2 */
        GetGsOsCallOneParameter(4,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #05 WORD   : bottom           IN  +0 */
        GetGsOsCallOneParameter(5,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x6C04 :
        strcpy(buffer_line,"RectRgn(");
        nb_param = 2;
        /* #01 DWORD  : rgnHandle        IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : rectPtr          IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x6D04 :
        strcpy(buffer_line,"OpenRgn(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x6E04 :
        strcpy(buffer_line,"CloseRgn(");
        /* #01 DWORD  : rgnHandle        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x6F04 :
        strcpy(buffer_line,"OffsetRgn(");
        nb_param = 3;
        /* #01 DWORD  : rgnHandle        IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : dH               IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : dV               IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x7004 :
        strcpy(buffer_line,"InsetRgn(");
        nb_param = 3;
        /* #01 DWORD  : rgnHandle        IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : dH               IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : dV               IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x7104 :
        strcpy(buffer_line,"SectRgn(");
        nb_param = 3;
        /* #01 DWORD  : rgn1Handle       IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : rgn2Handle       IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : destRgnHandle    IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x7204 :
        strcpy(buffer_line,"UnionRgn(");
        nb_param = 3;
        /* #01 DWORD  : rgn1Handle       IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : rgn2Handle       IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : unionRgnHandle   IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x7304 :
        strcpy(buffer_line,"DiffRgn(");
        nb_param = 3;
        /* #01 DWORD  : rgn1Handle       IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : rgn2Handle       IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : diffRgnHandle    IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x7404 :
        strcpy(buffer_line,"XorRgn(");
        nb_param = 3;
        /* #01 DWORD  : rgn1Handle       IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : rgn2Handle       IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : xorRgnHandle     IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x7504 :
        strcpy(buffer_line,"PtInRgn(");
        nb_param = 3;
        /* #01 WORD   : wordspace        OUT +8 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : pointPtr         IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : rgnHandle        IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x7604 :
        strcpy(buffer_line,"RectInRgn(");
        nb_param = 3;
        /* #01 WORD   : wordspace        OUT +8 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : rectPtr          IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : rgnHandle        IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x7704 :
        strcpy(buffer_line,"EqualRgn(");
        nb_param = 3;
        /* #01 WORD   : wordspace        OUT +8 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : rgn1Handle       IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : rgn2Handle       IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x7804 :
        strcpy(buffer_line,"EmptyRgn(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : rgnHandle        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x7904 :
        strcpy(buffer_line,"FrameRgn(");
        /* #01 DWORD  : rgnHandle        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x7A04 :
        strcpy(buffer_line,"PaintRgn(");
        /* #01 DWORD  : rgnHandle        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x7B04 :
        strcpy(buffer_line,"EraseRgn(");
        /* #01 DWORD  : rgnHandle        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x7C04 :
        strcpy(buffer_line,"InvertRgn(");
        /* #01 DWORD  : rgnHandle        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x7D04 :
        strcpy(buffer_line,"FillRgn(");
        nb_param = 2;
        /* #01 DWORD  : rgnHandle        IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : patternPtr       IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x7E04 :
        strcpy(buffer_line,"ScrollRect(");
        nb_param = 4;
        /* #01 DWORD  : rectPtr          IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : dH               IN  +6 */
        GetGsOsCallOneParameter(2,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : dV               IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : updateRgnHandle  IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x7F04 :
        strcpy(buffer_line,"PaintPixels(");
        /* #01 DWORD  : paintParamPtr    IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x8004 :
        strcpy(buffer_line,"AddPt(");
        nb_param = 2;
        /* #01 DWORD  : srcPtPtr         IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : destPtPtr        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x8104 :
        strcpy(buffer_line,"SubPt(");
        nb_param = 2;
        /* #01 DWORD  : srcPtPtr         IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : destPrPtr        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x8204 :
        strcpy(buffer_line,"SetPt(");
        nb_param = 3;
        /* #01 DWORD  : srcPtPtr         IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : h                IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : v                IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x8304 :
        strcpy(buffer_line,"EqualPt(");
        nb_param = 3;
        /* #01 WORD   : wordspace        OUT +8 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : point1Ptr        IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : point2Ptr        IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x8404 :
        strcpy(buffer_line,"LocalToGlobal(");
        /* #01 DWORD  : pointPtr         IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x8504 :
        strcpy(buffer_line,"GlobalToLocal(");
        /* #01 DWORD  : pointPtr         IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x8604 :
        strcpy(buffer_line,"Random(");
        /* #01 WORD   : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x8704 :
        strcpy(buffer_line,"SetRandSeed(");
        /* #01 DWORD  : randomSeed       IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x8804 :
        strcpy(buffer_line,"GetPixel(");
        nb_param = 3;
        /* #01 WORD   : wordspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : h                IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : v                IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x8904 :
        strcpy(buffer_line,"ScalePt(");
        nb_param = 3;
        /* #01 DWORD  : pointPtr         IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : srcRectPtr       IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : destRectPtr      IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x8A04 :
        strcpy(buffer_line,"MapPt(");
        nb_param = 3;
        /* #01 DWORD  : pointPtr         IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : srcRectPtr       IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : destRectPtr      IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x8B04 :
        strcpy(buffer_line,"MapRect(");
        nb_param = 3;
        /* #01 DWORD  : pointPtr         IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : srcRectPtr       IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : destRectPtr      IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x8C04 :
        strcpy(buffer_line,"MapRgn(");
        nb_param = 3;
        /* #01 DWORD  : rgnHandle        IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : srcRectPtr       IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : destRectPtr      IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x8D04 :
        strcpy(buffer_line,"SetStdProcs(");
        /* #01 DWORD  : stdProcRecPtr    IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x8E04 :
        strcpy(buffer_line,"SetCursor(");
        /* #01 DWORD  : cursorPtr        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x8F04 :
        strcpy(buffer_line,"GetCursorAdr(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x9004 :
        strcpy(buffer_line,"HideCursor(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x9104 :
        strcpy(buffer_line,"ShowCursor(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x9204 :
        strcpy(buffer_line,"ObscureCursor(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x9304 :
        strcpy(buffer_line,"SetMouseLoc(");
        nb_param = 2;
        /* #01 WORD   : posX             IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : posY             IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x9404 :
        strcpy(buffer_line,"SetFont(");
        /* #01 DWORD  : newFontHandle    IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x9504 :
        strcpy(buffer_line,"GetFont(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x9604 :
        strcpy(buffer_line,"GetFontInfo(");
        /* #01 DWORD  : fontInfoRecPtr   IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x9704 :
        strcpy(buffer_line,"GetFontGlobals(");
        /* #01 DWORD  : fgRecPtr         IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x9804 :
        strcpy(buffer_line,"SetFontFlags(");
        /* #01 WORD   : fontFlags        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x9904 :
        strcpy(buffer_line,"GetFontFlags(");
        /* #01 WORD   : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x9A04 :
        strcpy(buffer_line,"SetTextFace(");
        /* #01 WORD  : textFace          IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x9B04 :
        strcpy(buffer_line,"GetTextFace(");
        /* #01 WORD   : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x9C04 :
        strcpy(buffer_line,"SetTextMode(");
        /* #01 WORD  : textMode          IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x9D04 :
        strcpy(buffer_line,"GetTextMode(");
        /* #01 WORD   : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x9E04 :
        strcpy(buffer_line,"SetSpaceExtra(");
        /* #01 DWORD  : spaceExtra      IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x9F04 :
        strcpy(buffer_line,"GetSpaceExtra(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0xA004 :
        strcpy(buffer_line,"SetForeColor(");
        /* #01 WORD   : foreColor        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0xA104 :
        strcpy(buffer_line,"GetForeColor(");
        /* #01 WORD   : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0xA204 :
        strcpy(buffer_line,"SetBackColor(");
        /* #01 WORD   : backColor        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0xA304 :
        strcpy(buffer_line,"GetBackColor(");
        /* #01 WORD   : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0xA404 :
        strcpy(buffer_line,"DrawChar(");
        /* #01 WORD   : theChar          IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0xA504 :
        strcpy(buffer_line,"DrawString(");
        /* #01 DWORD  : cStringPtr       IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,1,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0xA604 :
        strcpy(buffer_line,"DrawCString(");
        /* #01 DWORD  : cStringPtr       IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,1,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0xA704 :
        strcpy(buffer_line,"DrawText(");
        nb_param = 2;
        /* #01 DWORD  : textPtr          IN  +0 */
        string_address = GetMemoryValue(param_address+2,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #01 WORD   : textLength       IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0xA804 :
        strcpy(buffer_line,"CharWidth(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : theChar          IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0xA904 :
        strcpy(buffer_line,"StringWidth(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : stringPtr        IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0xAA04 :
        strcpy(buffer_line,"CStringWidth(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : cStringPtr       IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0xAB04 :
        strcpy(buffer_line,"TextWidth(");
        nb_param = 3;
        /* #01 WORD   : wordspace        OUT +6 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : textPtr          IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : textLength       IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0xAC04 :
        strcpy(buffer_line,"CharBounds(");
        nb_param = 2;
        /* #01 WORD   : theChar          IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : resultPtr        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0xAD04 :
        strcpy(buffer_line,"StringBounds(");
        nb_param = 2;
        /* #01 DWORD  : stringPtr        IN  +4 */
        string_address = GetMemoryValue(param_address+4,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : resultPtr        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0xAE04 :
        strcpy(buffer_line,"CStringBounds(");
        nb_param = 2;
        /* #01 DWORD  : cStringPtr       IN  +4 */
        string_address = GetMemoryValue(param_address+4,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : resultPtr        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0xAF04 :
        strcpy(buffer_line,"TextBounds(");
        nb_param = 3;
        /* #01 DWORD  : textPtr          IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : textLength       IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : resultPtr        IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0xB004 :
        strcpy(buffer_line,"SetArcRot(");
        /* #01 WORD   : arcRotValue      IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0xB104 :
        strcpy(buffer_line,"GetArcRot(");
        /* #01 WORD   : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0xB204 :
        strcpy(buffer_line,"SetSysFont(");
        /* #01 DWORD  : fontHandle       IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0xB304 :
        strcpy(buffer_line,"GetSysFont(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0xB404 :
        strcpy(buffer_line,"SetVisRgn(");
        /* #01 DWORD  : rgnHandle        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0xB504 :
        strcpy(buffer_line,"GetVisRgn(");
        /* #01 DWORD  : rgnHandle        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0xB604 :
        strcpy(buffer_line,"SetIntUse(");
        /* #01 WORD   : useInt           IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0xB704 :
        strcpy(buffer_line,"OpenPicture(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : picFrame         IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0xB804 :
        strcpy(buffer_line,"PicComment(");
        nb_param = 3;
        /* #01 WORD   : commentKind      IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : dataSize         IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : dataHandle       IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0xB904 :
        strcpy(buffer_line,"ClosePicture(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0xBA04 :
        strcpy(buffer_line,"DrawPicture(");
        nb_param = 2;
        /* #01 DWORD  : picHandle        IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : destRect         IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0xBB04 :
        strcpy(buffer_line,"KillPicture(");
        /* #01 DWORD  : picHandle        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0xBC04 :
        strcpy(buffer_line,"FramePoly(");
        /* #01 DWORD  : polyHandle       IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0xBD04 :
        strcpy(buffer_line,"PaintPoly(");
        /* #01 DWORD  : polyHandle       IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0xBE04 :
        strcpy(buffer_line,"ErasePoly(");
        /* #01 DWORD  : polyHandle       IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0xBF04 :
        strcpy(buffer_line,"InvertPoly(");
        /* #01 DWORD  : polyHandle       IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0xC004 :
        strcpy(buffer_line,"FillPoly(");
        nb_param = 2;
        /* #01 DWORD  : polyHandle       IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : patternPtr       IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0xC104 :
        strcpy(buffer_line,"OpenPoly(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0xC204 :
        strcpy(buffer_line,"ClosePoly(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0xC304 :
        strcpy(buffer_line,"KillPoly(");
        /* #01 DWORD  : polyHandle       IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0xC404 :
        strcpy(buffer_line,"OffsetPoly(");
        nb_param = 3;
        /* #01 DWORD  : polyHandle       IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : dH               IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : dV               IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0xC504 :
        strcpy(buffer_line,"MapPoly(");
        nb_param = 3;
        /* #01 DWORD  : polyHandle       IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : srcRectPtr       IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : destRectPtr      IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0xC604 :
        strcpy(buffer_line,"SetClipHandle(");
        /* #01 DWORD  : rgnHandle        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0xC704 :
        strcpy(buffer_line,"GetClipHandle(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0xC804 :
        strcpy(buffer_line,"SetVisHandle(");
        /* #01 DWORD  : rgnHandle        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0xC904 :
        strcpy(buffer_line,"GetVisHandle(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0xCA04 :
        strcpy(buffer_line,"InitCursor(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0xCB04 :
        strcpy(buffer_line,"SetBufDims(");
        nb_param = 3;
        /* #01 WORD   : maxWidth         IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : maxFontHeight    IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : maxFBRExtent     IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0xCC04 :
        strcpy(buffer_line,"ForceBufDims(");
        nb_param = 3;
        /* #01 WORD   : maxWidth         IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : maxFontHeight    IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : maxFBRExtent     IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0xCD04 :
        strcpy(buffer_line,"SaveBufDims(");
        /* #01 DWORD  : sizeInfoPtr      IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0xCE04 :
        strcpy(buffer_line,"RestoreBufDims(");
        /* #01 DWORD  : sizeInfoPtr      IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0xCF04 :
        strcpy(buffer_line,"GetFGSize(");
        /* #01 WORD   : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0xD004 :
        strcpy(buffer_line,"SetFontID(");
        /* #01 DWORD  : fontID           IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0xD104 :
        strcpy(buffer_line,"GetFontID(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0xD204 :
        strcpy(buffer_line,"SetTextSize(");
        /* #01 WORD  : textSize          IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0xD304 :
        strcpy(buffer_line,"GetTextSize(");
        /* #01 WORD   : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0xD404 :
        strcpy(buffer_line,"SetCharExtra(");
        /* #01 DWORD  : charExtra        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0xD504 :
        strcpy(buffer_line,"GetCharExtra(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0xD604 :
        strcpy(buffer_line,"PPToPort(");
        nb_param = 5;
        /* #01 DWORD  : srcLocPtr        IN  +10 */
        GetGsOsCallOneParameter(1,nb_param,param_address+10,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : srcRectPtr       IN  +6 */
        GetGsOsCallOneParameter(2,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : destX            IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : destY            IN  +2 */
        GetGsOsCallOneParameter(4,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #05 WORD   : transferMode     IN  +0 */
        GetGsOsCallOneParameter(5,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0xD704 :
        strcpy(buffer_line,"InflateTextBuffer(");
        nb_param = 2;
        /* #01 WORD   : newWidth         IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : newHeight        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0xD804 :
        strcpy(buffer_line,"GetRomFont(");
        /* #01 DWORD  : recordPtr        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0xD904 :
        strcpy(buffer_line,"GetFontLore(");
        nb_param = 3;
        /* #01 WORD   : wordspace        OUT +6 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : recordPtr        IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : recordSize       IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0xDA04 :
        strcpy(buffer_line,"Get640Colors(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0xDB04 :
        strcpy(buffer_line,"Set640Color(");
        /* #01 WORD   : colorNum         IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;


      /*******************************/
      /*  Tools 0x05 : Desk Manager  */
      /*******************************/
      case 0x0105 :
        strcpy(buffer_line,"DeskBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0205 :
        strcpy(buffer_line,"DeskStartUp(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0305 :
        strcpy(buffer_line,"DeskShutDown(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0405 :
        strcpy(buffer_line,"DeskVersion(");
        /* #01 WORD  : Space            OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0505 :
        strcpy(buffer_line,"DeskReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0605 :
        strcpy(buffer_line,"DeskStatus(");
        /* #01 WORD  : Space            OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1105 :
        strcpy(buffer_line,"ChooseCDA(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1D05 :
        strcpy(buffer_line,"CloseAllNDAs(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1605 :
        strcpy(buffer_line,"CloseNDA(");
        /* #01 WORD  : refNum           IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1C05 :
        strcpy(buffer_line,"CloseNDAbyWinPtr(");
        /* #01 DWORD : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1E05 :
        strcpy(buffer_line,"FixAppleMenu(");
        /* #01 WORD  : startingID       IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1405 :
        strcpy(buffer_line,"GetDAStrPtr(");
        /* #01 DWORD : Space            OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x1B05 :
        strcpy(buffer_line,"GetNumNDAs(");
        /* #01 WORD  : Space            OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0F05 :
        strcpy(buffer_line,"InstallCDA(");
        /* #01 DWORD : idHandle         IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0E05 :
        strcpy(buffer_line,"InstallNDA(");
        /* #01 DWORD : idHandle         IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1505 :
        strcpy(buffer_line,"OpenNDA(");
        /* #01 WORD  : Space            OUT +2 */
        GetEmptyParameter(1,2,buffer_line);
        /* #02 WORD  : idNum            IN  +0 */
        GetGsOsCallOneParameter(2,2,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,2,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0C05 :
        strcpy(buffer_line,"RestAll(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0A05 :
        strcpy(buffer_line,"RestScrn(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0B05 :
        strcpy(buffer_line,"SaveAll(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0905 :
        strcpy(buffer_line,"SaveScrn(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1305 :
        strcpy(buffer_line,"SetDAStrPtr(");
        /* #01 DWORD : altDispHandle    IN  +4 */
        GetGsOsCallOneParameter(1,2,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : stringTablePtr   IN  +0 */
        GetGsOsCallOneParameter(2,2,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1705 :
        strcpy(buffer_line,"SystemClick(");
        /* #01 DWORD : eventRectPtr     IN  +6 */
        GetGsOsCallOneParameter(1,3,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : theWindowPtr     IN  +2 */
        GetGsOsCallOneParameter(2,3,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : findWndwResult   IN  +0 */
        GetGsOsCallOneParameter(3,3,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1805 :
        strcpy(buffer_line,"SystemEdit(");
        /* #01 WORD  : wordSpace        OUT +2 */
        GetEmptyParameter(1,2,buffer_line);
        /* #03 WORD  : editType         IN  +0 */
        GetGsOsCallOneParameter(2,2,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,2,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1A05 :
        strcpy(buffer_line,"SystemEvent(");
        /* #01 WORD  : wordSpace        OUT +16 */
        GetEmptyParameter(1,6,buffer_line);
        /* #02 WORD  : editWhat         IN  +14 */
        GetGsOsCallOneParameter(2,6,param_address+14,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : eventMessage     IN  +10 */
        GetGsOsCallOneParameter(3,6,param_address+10,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD : eventWhen        IN  +6 */
        GetGsOsCallOneParameter(4,6,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #05 DWORD : eventWhere       IN  +2 */
        GetGsOsCallOneParameter(5,6,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #06 WORD  : eventMods        IN  +0 */
        GetGsOsCallOneParameter(6,6,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,6,param_address+16,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1905 :
        strcpy(buffer_line,"SystemTask(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1F05 :
        strcpy(buffer_line,"AddToRunQ(");
        /* #01 DWORD : runItemPtr       IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2105 :
        strcpy(buffer_line,"RemoveCDA(");
        /* #01 DWORD : idHandle         IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2005 :
        strcpy(buffer_line,"RemoveFromRunQ(");
        /* #01 DWORD : runItemPtr       IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2205 :
        strcpy(buffer_line,"RemoveNDA(");
        /* #01 DWORD : idHandle         IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2405 :
        strcpy(buffer_line,"CallDeskAcc(");
        /* #01 WORD  : Space            OUT +12 */
        GetEmptyParameter(1,5,buffer_line);
        /* #02 WORD  : flags            IN  +10 */
        GetGsOsCallOneParameter(2,5,param_address+10,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : daReference      IN  +6 */
        GetGsOsCallOneParameter(3,5,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD  : action           IN  +4 */
        GetGsOsCallOneParameter(4,5,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #05 DWORD : data             IN  +0 */
        GetGsOsCallOneParameter(5,5,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,5,param_address+12,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x2305 :
        strcpy(buffer_line,"GetDeskAccInfo(");
        /* #02 WORD  : flags            IN  +10 */
        GetGsOsCallOneParameter(1,4,param_address+10,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : daReference      IN  +6 */
        GetGsOsCallOneParameter(2,4,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD  : buffSize         IN  +4 */
        GetGsOsCallOneParameter(3,4,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #05 DWORD : bufferPtr        IN  +0 */
        GetGsOsCallOneParameter(4,4,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2505 :
        strcpy(buffer_line,"GetDeskGlobal(");
        /* #01 DWORD : Space            OUT +2 */
        GetEmptyParameter(1,2,buffer_line);
        /* #02 WORD  : selector         IN  +0 */
        GetGsOsCallOneParameter(2,2,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,2,param_address+2,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;


      /********************************/
      /*  Tools 0x06 : Event Manager  */
      /********************************/
      case 0x0106 :
        strcpy(buffer_line,"EMBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0206 :
        strcpy(buffer_line,"EMStartUp(");
        /* #01 WORD  : dPageAddr        IN  +12 */
        GetGsOsCallOneParameter(1,7,param_address+12,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : queueSize        IN  +10 */
        GetGsOsCallOneParameter(2,7,param_address+10,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : xMinClamp        IN  +8 */
        GetGsOsCallOneParameter(3,7,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD  : xMaxClamp        IN  +6 */
        GetGsOsCallOneParameter(4,7,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #05 WORD  : yMinClamp        IN  +4 */
        GetGsOsCallOneParameter(5,7,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #06 WORD  : yMaxClamp        IN  +2 */
        GetGsOsCallOneParameter(6,7,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #07 WORD  : userID           IN  +0 */
        GetGsOsCallOneParameter(7,7,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0306 :
        strcpy(buffer_line,"EMShutDown(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0406 :
        strcpy(buffer_line,"EMVersion(");
        /* #01 WORD  : wordSpace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0506 :
        strcpy(buffer_line,"EMReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0606 :
        strcpy(buffer_line,"EMStatus(");
        /* #01 WORD  : wordSpace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0D06 :
        strcpy(buffer_line,"Button(");
        /* #01 WORD  : wordSpace        OUT +2 */
        GetEmptyParameter(1,2,buffer_line);
        /* #02 WORD  : buttonNum        IN  +0 */
        GetGsOsCallOneParameter(2,2,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,2,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0906 :
        strcpy(buffer_line,"DoWindows(");
        /* #01 WORD  : wordSpace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0B06 :
        strcpy(buffer_line,"EventAvail(");
        /* #01 WORD  : wordSpace        OUT +6 */
        GetEmptyParameter(1,3,buffer_line);
        /* #02 WORD  : eventMask        IN  +4 */
        GetGsOsCallOneParameter(2,3,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : eventPtr         IN  +0 */
        GetGsOsCallOneParameter(3,3,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,3,param_address+6,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1906 :
        strcpy(buffer_line,"FakeMouse(");
        /* #01 WORD  : changedFlag      IN  +8 */
        GetGsOsCallOneParameter(1,6,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 BYTE  : modLatch         IN  +7 */
        GetGsOsCallOneParameter(2,6,param_address+7,TYPE_GSOS_PARAM_BYTE,buffer_line,current_operation,cyrene_param);
        /* #03 BYTE  : padding          IN  +6 */
        GetGsOsCallOneParameter(3,6,param_address+6,TYPE_GSOS_PARAM_BYTE,buffer_line,current_operation,cyrene_param);
        /* #04 WORD  : xPosition        IN  +4 */
        GetGsOsCallOneParameter(4,6,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #05 WORD  : yPosition        IN  +2 */
        GetGsOsCallOneParameter(5,6,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #06 WORD  : buttonStatus     IN  +0 */
        GetGsOsCallOneParameter(6,6,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1506 :
        strcpy(buffer_line,"FlushEvents(");
        /* #01 WORD  : wordSpace        OUT +4 */
        GetEmptyParameter(1,3,buffer_line);
        /* #02 WORD  : eventMask        IN  +2 */
        GetGsOsCallOneParameter(2,3,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : stopMask         IN  +0 */
        GetGsOsCallOneParameter(3,3,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,3,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1206 :
        strcpy(buffer_line,"GetCaretTime(");
        /* #01 DWORD : longSpace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x1106 :
        strcpy(buffer_line,"GetDblTime(");
        /* #01 DWORD : longSpace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0C06 :
        strcpy(buffer_line,"GetMouse(");
        /* #01 DWORD : mouseLocPtr      IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0A06 :
        strcpy(buffer_line,"GetNextEvent(");
        /* #01 WORD  : wordSpace        OUT +6 */
        GetEmptyParameter(1,3,buffer_line);
        /* #02 WORD  : eventMask        IN  +4 */
        GetGsOsCallOneParameter(2,3,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : eventPtr         IN  +0 */
        GetGsOsCallOneParameter(3,3,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,3,param_address+6,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1606 :
        strcpy(buffer_line,"GetOSEvent(");
        /* #01 WORD  : wordSpace        OUT +6 */
        GetEmptyParameter(1,3,buffer_line);
        /* #02 WORD  : eventMask        IN  +4 */
        GetGsOsCallOneParameter(2,3,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : eventPtr         IN  +0 */
        GetGsOsCallOneParameter(3,3,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,3,param_address+6,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1706 :
        strcpy(buffer_line,"OSEventAvail(");
        /* #01 WORD  : wordSpace        OUT +6 */
        GetEmptyParameter(1,3,buffer_line);
        /* #02 WORD  : eventMask        IN  +4 */
        GetGsOsCallOneParameter(2,3,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : eventPtr         IN  +0 */
        GetGsOsCallOneParameter(3,3,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,3,param_address+6,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1406 :
        strcpy(buffer_line,"PostEvent(");
        /* #01 WORD  : wordSpace        OUT +6 */
        GetEmptyParameter(1,3,buffer_line);
        /* #02 WORD  : eventCode        IN  +4 */
        GetGsOsCallOneParameter(2,3,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : eventMsg         IN  +0 */
        GetGsOsCallOneParameter(3,3,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,3,param_address+6,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1806 :
        strcpy(buffer_line,"SetEventMask(");
        /* #01 WORD  : sysEventMask     IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1306 :
        strcpy(buffer_line,"SetSwitch(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0E06 :
        strcpy(buffer_line,"StillDown(");
        /* #01 WORD  : wordSpace        OUT +2 */
        GetEmptyParameter(1,2,buffer_line);
        /* #02 WORD  : buttonNum        IN  +0 */
        GetGsOsCallOneParameter(2,2,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,2,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1006 :
        strcpy(buffer_line,"TickCount(");
        /* #01 DWORD : longSpace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0F06 :
        strcpy(buffer_line,"WaitMouseUp(");
        /* #01 WORD  : wordSpace        OUT +2 */
        GetEmptyParameter(1,2,buffer_line);
        /* #02 WORD  : buttonNum        IN  +0 */
        GetGsOsCallOneParameter(2,2,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,2,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1B06 :
        strcpy(buffer_line,"GetKeyTranslation(");
        /* #01 WORD  : wordSpace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1A06 :
        strcpy(buffer_line,"SetAutoKeyLimit(");
        /* #01 WORD  : newLimit         IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1C06 :
        strcpy(buffer_line,"SetKeyTranslation(");
        /* #01 WORD  : kTransID         IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;


      /****************************/
      /*  Tools 0x07 : Scheduler  */
      /****************************/
      case 0x0107 :
        strcpy(buffer_line,"SchBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0207 :
        strcpy(buffer_line,"SchStartUp(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0307 :
        strcpy(buffer_line,"SchShutDown(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0407 :
        strcpy(buffer_line,"SchVersion(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0507 :
        strcpy(buffer_line,"SchReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0607 :
        strcpy(buffer_line,"SchStatus(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0907 :
        strcpy(buffer_line,"SchAddTask(");
        nb_param = 2;
        /* #01 WORD  : wordspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD : taskPtr          IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0A07 : 
        strcpy(buffer_line,"SchFlush(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;


      /********************************/
      /*  Tools 0x08 : Sound Manager  */
      /********************************/
      case 0x0108 :
        strcpy(buffer_line,"SoundBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0208 :
        strcpy(buffer_line,"SoundStartUp(");
        /* #01 WORD  : dPageAddr         IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0308 :
        strcpy(buffer_line,"SoundShutDown(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0408 :
        strcpy(buffer_line,"SoundVersion(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0508 :
        strcpy(buffer_line,"SoundReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0608 :
        strcpy(buffer_line,"SoundToolStatus(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0908 :
        strcpy(buffer_line,"WriteRamBlock(");
        nb_param = 3;
        /* #01 DWORD  : srcPtr           IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : docStart         IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : byteCount        IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0A08 :
        strcpy(buffer_line,"ReadRamBlock(");
        nb_param = 3;
        /* #01 DWORD  : destPtr          IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : docStart         IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : byteCount        IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0B08 :
        strcpy(buffer_line,"GetTableAddress(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0C08 :
        strcpy(buffer_line,"GetSoundVolume(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : genNumber        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;
        
      case 0x0D08 :
        strcpy(buffer_line,"SetSoundVolume("); 
        nb_param = 2;
        /* #01 WORD   : volume           IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : genNumber        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0E08 :
        strcpy(buffer_line,"FFStartSound(");
        nb_param = 2;
        /* #01 WORD   : genNumFFSynth    IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : pBlockPtr        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0F08 :
        strcpy(buffer_line,"FFStopSound(");
        /* #01 WORD   : genMask          IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1008 :
        strcpy(buffer_line,"FFSoundStatus(");
        /* #01 WORD   : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1108 :
        strcpy(buffer_line,"FFGeneratorStatus(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : genNumber        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1208 :
        strcpy(buffer_line,"SetSoundMIRQV(");
        /* #01 DWORD  : sMasterIRQ       IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1308 :
        strcpy(buffer_line,"SetUserSoundIRQV(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : userIRQVector    IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x1408 :
        strcpy(buffer_line,"FFSoundDoneStatus(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : genNumber        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1508 :
        strcpy(buffer_line,"FFSetUpSound(");
        nb_param = 2;
        /* #01 WORD   : channelGen       IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : paramBlockPtr    IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1608 :
        strcpy(buffer_line,"FFStartPlaying(");
        /* #01 WORD   : genWord          IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1708 :
        strcpy(buffer_line,"SetDocReg(");
        /* #01 DWORD  : pBlockPtr        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1808 :
        strcpy(buffer_line,"ReadDocReg(");
        /* #01 DWORD  : pBlockPtr        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;


      /************************************/
      /*  Tools 0x09 : Apple Desktop Bus  */
      /************************************/
      case 0x0109 :
        strcpy(buffer_line,"ADBBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0209 :
        strcpy(buffer_line,"ADBStartUp(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0309 :
        strcpy(buffer_line,"ADBShutDown(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0409 :
        strcpy(buffer_line,"ADBVersion(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0509 :
        strcpy(buffer_line,"ADBReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0609 :
        strcpy(buffer_line,"ADBStatus(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0909 :
        strcpy(buffer_line,"SendInfo(");
        nb_param = 3;
        /* #01 WORD   : dataLength       IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : dataPtr          IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : adbCommand       IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0A09 :
        strcpy(buffer_line,"ReadKeyMicroData(");
        nb_param = 3;
        /* #01 WORD   : dataLength       IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : dataPtr          IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : adbCommand       IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0B09 :
        strcpy(buffer_line,"ReadKeyMicroMemory(");
        nb_param = 3;
        /* #01 DWORD  : dataOutPtr       IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : dataInPtr        IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : adbCommand       IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0D09 :
        strcpy(buffer_line,"AsyncADBReceive(");
        nb_param = 2;
        /* #01 DWORD  : compPtr          IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : adbCommand       IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0E09 :
        strcpy(buffer_line,"SyncADBReceive(");
        nb_param = 3;
        /* #01 WORD   : inputWord        IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : compPtr          IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : adbCommand       IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0F09 :
        strcpy(buffer_line,"AbsOn(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1009 :
        strcpy(buffer_line,"AbsOff(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1109 :
        strcpy(buffer_line,"ReadAbs(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1209 :
        strcpy(buffer_line,"SetAbsScale(");
        /* #01 DWORD  : dataOutPtr       IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1309 :
        strcpy(buffer_line,"GetAbsScale(");
        /* #01 DWORD  : dataInPtr        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1409 :
        strcpy(buffer_line,"SRQPoll(");
        nb_param = 2;
        /* #01 DWORD  : compPtr          IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #01 WORD   : adbRegAddr       IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1509 :
        strcpy(buffer_line,"SRQRemove(");
        /* #01 WORD   : adbRegAddr       IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1609 :
        strcpy(buffer_line,"ClearSRQTable(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;


      /***********************/
      /*  Tools 0x0A : SANE  */
      /***********************/
      case 0x010A :
        strcpy(buffer_line,"SANEBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x020A :
        strcpy(buffer_line,"SANEStartUp(");
        /* #01 WORD  : dPageAddr         IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x030A :
        strcpy(buffer_line,"SANEShutDown(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x040A :
        strcpy(buffer_line,"SANEVersion(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x050A :
        strcpy(buffer_line,"SANEReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x060A :
        strcpy(buffer_line,"SANEStatus(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x090A :
        strcpy(buffer_line,"SANEFP816(");
        /* Variable number of params */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0A0A :
        strcpy(buffer_line,"SANEDecStr816(");
        /* Variable number of params */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0B0A :
        strcpy(buffer_line,"SANEElems816(");
        /* Variable number of params */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;


      /*******************************/
      /*  Tools 0x0B : Integer Math  */
      /*******************************/
      case 0x010B :
        strcpy(buffer_line,"IMBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x020B :
        strcpy(buffer_line,"IMStartUp(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x030B :
        strcpy(buffer_line,"IMShutDown(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x040B :
        strcpy(buffer_line,"IMVersion(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x050B :
        strcpy(buffer_line,"IMReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x060B :
        strcpy(buffer_line,"IMStatus(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x090B :
        strcpy(buffer_line,"Multiply(");
        nb_param = 3;
        /* #01 DWORD  : longspace       OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD  : multiplicand     IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : multiplier       IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0A0B :
        strcpy(buffer_line,"SDivide(");
        nb_param = 4;
        /* #01 WORD   : wordspace        OUT +6 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : wordspace        IN  +4 */
        GetEmptyParameter(2,nb_param,buffer_line);
        /* #03 WORD   : dividend         IN  +2 */
        GetGsOsCallOneParameter(3,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : divisor          IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0B0B :
        strcpy(buffer_line,"UDivide(");
        nb_param = 4;
        /* #01 WORD   : wordspace        OUT +6 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : wordspace        OUT +4 */
        GetEmptyParameter(2,nb_param,buffer_line);
        /* #03 WORD   : dividend         IN  +2 */
        GetGsOsCallOneParameter(3,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : divisor          IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0C0B :
        strcpy(buffer_line,"LongMul(");
        nb_param = 4;
        /* #01 DWORD  : longspace        OUT +12 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : longspace        OUT +8 */
        GetEmptyParameter(2,nb_param,buffer_line);
        /* #03 DWORD  : multiplicand     IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : multiplier       IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+12,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,2,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0D0B :
        strcpy(buffer_line,"LongDivide(");
        nb_param = 4;
        /* #01 DWORD  : longspace        OUT +12 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : longspace        OUT +8 */
        GetEmptyParameter(2,nb_param,buffer_line);
        /* #03 DWORD  : dividend         IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : divisor          IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+12,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,2,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0E0B :
        strcpy(buffer_line,"FixRatio(");
        nb_param = 3;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : numerator        IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : denominator      IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0F0B :
        strcpy(buffer_line,"FixMul(");
        nb_param = 3;
        /* #01 DWORD  : longspace        OUT +8 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : multiplicand     IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : multiplier       IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x100B :
        strcpy(buffer_line,"FracMul(");
        nb_param = 3;
        /* #01 DWORD  : longspace        OUT +8 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : multiplicand     IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : multiplier       IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x110B :
        strcpy(buffer_line,"FixDiv(");
        nb_param = 3;
        /* #01 DWORD  : longspace        OUT +8 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : dividend         IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : divisor          IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x120B :
        strcpy(buffer_line,"FracDiv(");
        nb_param = 3;
        /* #01 DWORD  : longspace        OUT +8 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : dividend         IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : divisor          IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x130B :
        strcpy(buffer_line,"FixRound(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : fixedValue       IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x140B :
        strcpy(buffer_line,"FracSqrt(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : fracValue        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x150B :
        strcpy(buffer_line,"FracCos(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : angle            IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x160B :
        strcpy(buffer_line,"FracSin(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : angle            IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x170B :
        strcpy(buffer_line,"FixATan2(");
        nb_param = 3;
        /* #01 DWORD  : longspace        OUT +8 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : input1           IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : input2           IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x180B :
        strcpy(buffer_line,"HiWord(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : longValue        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x190B :
        strcpy(buffer_line,"LoWord(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : longValue        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1A0B :
        strcpy(buffer_line,"Long2Fix(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : longIntValue     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x1B0B :
        strcpy(buffer_line,"Fix2Long(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : fixedValue       IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x1C0B :
        strcpy(buffer_line,"Fix2Frac(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : fixedValue       IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x1D0B :
        strcpy(buffer_line,"Frac2Fix(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : fracValue        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x1E0B :
        strcpy(buffer_line,"Fix2X(");
        nb_param = 2;
        /* #01 DWORD  : fixedValue       IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : extendPtr        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1F0B :
        strcpy(buffer_line,"Frac2X(");
        nb_param = 2;
        /* #01 DWORD  : fracValue        IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : extendPtr        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x200B :
        strcpy(buffer_line,"X2Fix(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : extendPtr        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x210B :
        strcpy(buffer_line,"X2Frac(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : extendPtr        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x220B :
        strcpy(buffer_line,"Int2Hex(");
        nb_param = 3;
        /* #01 WORD   : intValue         IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : strPtr           OUT +2 */
        GetEmptyParameter(2,nb_param,buffer_line);
        /* #03 WORD   : strLength        IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        string_length = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        string_address = GetMemoryValue(param_address+2,4,current_operation,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,2,nb_param,string_address,TYPE_GSOS_PARAM_TXT+string_length,cyrene_param);
        break;

      case 0x230B :
        strcpy(buffer_line,"Long2Hex(");
        nb_param = 3;
        /* #01 DWORD  : longValue        IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : strPtr           OUT +2 */
        GetEmptyParameter(2,nb_param,buffer_line);
        /* #03 WORD   : strLength        IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        string_length = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        string_address = GetMemoryValue(param_address+2,4,current_operation,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,2,nb_param,string_address,TYPE_GSOS_PARAM_TXT+string_length,cyrene_param);
        break;

      case 0x240B :
        strcpy(buffer_line,"Hex2Int(");
        nb_param = 3;
        /* #01 WORD   : wordspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : strPtr           IN  +2 */
        string_length = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        string_address = GetMemoryValue(param_address+2,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param,string_address,TYPE_GSOS_PARAM_TXT+string_length,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : strLength        IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x250B :
        strcpy(buffer_line,"Hex2Long(");
        nb_param = 3;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : strPtr           IN  +2 */
        string_length = (WORD) GetMemoryValue(param_address+0,2,current_operation,cyrene_param);
        string_address = GetMemoryValue(param_address+2,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param,string_address,TYPE_GSOS_PARAM_TXT+string_length,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : strLength        IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x260B :
        strcpy(buffer_line,"Int2Dec(");
        nb_param = 4;
        /* #01 WORD   : wordValue        IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : strPtr           OUT +4 */
        GetEmptyParameter(2,nb_param,buffer_line);
        /* #03 WORD   : strLength        IN  +2 */
        GetGsOsCallOneParameter(3,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : signedFlag       IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        string_length = (WORD) GetMemoryValue(param_address+2,2,current_operation,cyrene_param);
        string_address = GetMemoryValue(param_address+4,4,current_operation,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,2,nb_param,string_address,TYPE_GSOS_PARAM_TXT+string_length,cyrene_param);
        break;

      case 0x270B :
        strcpy(buffer_line,"Long2Dec(");
        nb_param = 4;
        /* #01 DWORD  : longValue        IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : strPtr           OUT +4 */
        GetEmptyParameter(2,nb_param,buffer_line);
        /* #03 WORD   : strLength        IN  +2 */
        GetGsOsCallOneParameter(3,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : signedFlag       IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        string_length = (WORD) GetMemoryValue(param_address+2,2,current_operation,cyrene_param);
        string_address = GetMemoryValue(param_address+4,4,current_operation,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,2,nb_param,string_address,TYPE_GSOS_PARAM_TXT+string_length,cyrene_param);
        break;

      case 0x280B :
        strcpy(buffer_line,"Dec2Int(");
        nb_param = 4;
        /* #01 WORD   : wordspace        OUT +8 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : strPtr           IN  +4 */
        string_length = (WORD) GetMemoryValue(param_address+2,2,current_operation,cyrene_param);
        string_address = GetMemoryValue(param_address+4,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param,string_address,TYPE_GSOS_PARAM_TXT+string_length,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : strLength        IN  +2 */
        GetGsOsCallOneParameter(3,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : signedFlag       IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x290B :
        strcpy(buffer_line,"Dec2Long(");
        nb_param = 4;
        /* #01 DWORD  : longspace        OUT +8 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : strPtr           IN  +4 */
        string_length = (WORD) GetMemoryValue(param_address+2,2,current_operation,cyrene_param);
        string_address = GetMemoryValue(param_address+4,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param,string_address,TYPE_GSOS_PARAM_TXT+string_length,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : strLength        IN  +2 */
        GetGsOsCallOneParameter(3,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : signedFlag       IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x2A0B :
        strcpy(buffer_line,"HexIt(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : intValue         IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;


      /***********************/
      /*  Tools 0x0C : Text  */
      /***********************/
      case 0x010C :
        strcpy(buffer_line,"TextBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x020C :
        strcpy(buffer_line,"TextStartUp(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x030C :
        strcpy(buffer_line,"TextShutDown(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x040C :
        strcpy(buffer_line,"TextVersion(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x050C :
        strcpy(buffer_line,"TextReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x060C :
        strcpy(buffer_line,"TextStatus(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x090C :
        strcpy(buffer_line,"SetInGlobals(");
        nb_param = 2;
        /* #01 WORD   : andMask          IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : orMask           IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0A0C :
        strcpy(buffer_line,"SetOutGlobals(");
        nb_param = 2;
        /* #01 WORD   : andMask          IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : orMask           IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0B0C :
        strcpy(buffer_line,"SetErrGlobals(");
        nb_param = 2;
        /* #01 WORD   : andMask          IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : orMask           IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0C0C :
        strcpy(buffer_line,"GetInGlobals(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : wordspace        OUT +0 */
        GetEmptyParameter(2,nb_param,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0D0C :
        strcpy(buffer_line,"GetOutGlobals(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : wordspace        OUT +0 */
        GetEmptyParameter(2,nb_param,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0E0C :
        strcpy(buffer_line,"GetErrGlobals(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : wordspace        OUT +0 */
        GetEmptyParameter(2,nb_param,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0F0C :
        strcpy(buffer_line,"SetInputDevice(");
        nb_param = 2;
        /* #01 WORD   : deviceType       IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : ptrOrSlot        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x100C :
        strcpy(buffer_line,"SetOutputDevice(");
        nb_param = 2;
        /* #01 WORD   : deviceType       IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : ptrOrSlot        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x110C :
        strcpy(buffer_line,"SetErrorDevice(");
        nb_param = 2;
        /* #01 WORD   : deviceType       IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : ptrOrSlot        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x120C :
        strcpy(buffer_line,"GetInputDevice(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(2,nb_param,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x130C :
        strcpy(buffer_line,"GetOutputDevice(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(2,nb_param,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x140C :
        strcpy(buffer_line,"GetErrorDevice(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(2,nb_param,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x150C :
        strcpy(buffer_line,"InitTextDev(");
        /* #01 WORD   : deviceNum        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x160C :
        strcpy(buffer_line,"CtlTextDev(");
        nb_param = 2;
        /* #01 WORD   : deviceNum        IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : controlCode      IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x170C :
        strcpy(buffer_line,"StatusTextDev(");
        nb_param = 2;
        /* #01 WORD   : deviceNum        IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : requestCode      IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x180C :
        strcpy(buffer_line,"WriteChar(");
        /* #01 WORD   : theChar          IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x190C :
        strcpy(buffer_line,"ErrWriteChar(");
        /* #01 WORD   : theChar          IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1A0C :
        strcpy(buffer_line,"WriteLine(");
        /* #01 DWORD  : strPtr           IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,1,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1B0C : 
        strcpy(buffer_line,"ErrWriteLine(");
        /* #01 DWORD  : strPtr           IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,1,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1C0C :
        strcpy(buffer_line,"WriteString(");
        /* #01 DWORD  : strPtr           IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,1,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1D0C :
        strcpy(buffer_line,"ErrWriteString(");
        /* #01 DWORD  : strPtr           IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,1,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1E0C :
        strcpy(buffer_line,"TextWriteBlock(");
        nb_param = 3;
        /* #01 DWORD  : textPtr          IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : offset           IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : count            IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1F0C :
        strcpy(buffer_line,"ErrWriteBlock(");
        nb_param = 3;
        /* #01 DWORD  : textPtr          IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : offset           IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : count            IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x200C :
        strcpy(buffer_line,"WriteCString(");
        /* #01 DWORD  : cStrPtr          IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,1,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x210C :
        strcpy(buffer_line,"ErrWriteCString(");
        /* #01 DWORD  : cStrPtr          IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,1,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x220C :
        strcpy(buffer_line,"ReadChar(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : echoFlag         IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x230C :
        strcpy(buffer_line,"TextReadBlock(");
        nb_param = 4;
        /* #01 DWORD  : bufferPtr        IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : offset           IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : blockSize        IN  +2 */
        GetGsOsCallOneParameter(3,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : echoFlag         IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
          
        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x240C :
        strcpy(buffer_line,"ReadLine(");
        nb_param = 5;
        /* #01 WORD   : wordspace        OUT +10 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : cStrPtr          IN  +6 */
        GetGsOsCallOneParameter(2,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : maxCount         IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : eolChar          IN  +2 */
        GetGsOsCallOneParameter(4,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #05 WORD   : echoFlag         IN  +0 */
        GetGsOsCallOneParameter(5,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+10,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;


      /**********************************/
      /*  Tool 0x0D : RAMDisk Tool Set  */
      /**********************************/
      case 0x010D :
        strcpy(buffer_line,"RDBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x020D :
        strcpy(buffer_line,"RDStartUp(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x030D :
        strcpy(buffer_line,"RDShutDown(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x040D :
        strcpy(buffer_line,"RDVersion(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x050D :
        strcpy(buffer_line,"RDReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x060D :
        strcpy(buffer_line,"RDStatus(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x090D :
        strcpy(buffer_line,"Kill(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0A0D :
        strcpy(buffer_line,"RDExecPCCmd(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0B0D :
        strcpy(buffer_line,"RDSetDiskSize(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0C0D :
        strcpy(buffer_line,"RDGetDiskSize(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;


      /*********************************/
      /*  Tools 0x0E : Window Manager  */
      /*********************************/
      case 0x010E :
        strcpy(buffer_line,"WindBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x020E :
        strcpy(buffer_line,"WindStartUp(");
        /* #01 WORD  : userID            IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x030E :
        strcpy(buffer_line,"WindShutDown(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x040E :
        strcpy(buffer_line,"WindVersion(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x050E :
        strcpy(buffer_line,"WindReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x060E :
        strcpy(buffer_line,"WindStatus(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x090E :
        strcpy(buffer_line,"NewWindow(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : paramListPtr     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0A0E :
        strcpy(buffer_line,"CheckUpdate(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : theEventPtr      IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0B0E :
        strcpy(buffer_line,"CloseWindow(");
        /* #01 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0C0E :
        strcpy(buffer_line,"Desktop(");
        nb_param = 3;
        /* #01 DWORD  : longspace        OUT +6 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : desktopOP        IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : dtParam          IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0D0E :
        strcpy(buffer_line,"SetWTitle(");
        nb_param = 2;
        /* #01 DWORD  : titlePtr         IN  +4 */
        string_address = GetMemoryValue(param_address+4,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0E0E :
        strcpy(buffer_line,"GetWTitle(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0F0E :
        strcpy(buffer_line,"SetFrameColor(");
        nb_param = 2;
        /* #01 DWORD  : newColorPtr      IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x100E :
        strcpy(buffer_line,"GetFrameColor(");
        nb_param = 2;
        /* #01 DWORD  : colorPtr         IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x110E :
        strcpy(buffer_line,"SelectWindow(");
        /* #01 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x120E :
        strcpy(buffer_line,"HideWindow(");
        /* #01 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x130E :
        strcpy(buffer_line,"ShowWindow(");
        /* #01 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x140E :
        strcpy(buffer_line,"SendBehind(");
        nb_param = 2;
        /* #01 DWORD  : behindWindowPtr  IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x150E :
        strcpy(buffer_line,"FrontWindow(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x160E :
        strcpy(buffer_line,"SetInfoDraw(");
        nb_param = 2;
        /* #01 DWORD  : infoDrawPtr      IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x170E :
        strcpy(buffer_line,"FindWindow(");
        nb_param = 4;
        /* #01 WORD   : wordspace        OUT +8 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : whichWindowPtr   IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : pointX           IN  +2 */
        GetGsOsCallOneParameter(3,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : pointY           IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x180E :
        strcpy(buffer_line,"TrackGoAway(");
        nb_param = 4;
        /* #01 WORD   : wordspace        OUT +8 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : startX           IN  +6 */
        GetGsOsCallOneParameter(2,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : startY           IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x190E :
        strcpy(buffer_line,"MoveWindow(");
        nb_param = 3;
        /* #01 WORD   : newX             IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : newY             IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1A0E :
        strcpy(buffer_line,"DragWindow(");
        nb_param = 6;
        /* #01 WORD   : grid             IN  +14 */
        GetGsOsCallOneParameter(1,nb_param,param_address+14,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : startX           IN  +12 */
        GetGsOsCallOneParameter(2,nb_param,param_address+12,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : startY           IN  +10 */
        GetGsOsCallOneParameter(3,nb_param,param_address+10,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : grace            IN  +8 */
        GetGsOsCallOneParameter(4,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #05 DWORD  : boundsRectPtr    IN  +4 */
        GetGsOsCallOneParameter(5,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #06 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(6,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1B0E :
        strcpy(buffer_line,"GrowWindow(");
        nb_param = 6;
        /* #01 DWORD  : longspace        OUT +12 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : minWidth         IN  +10 */
        GetGsOsCallOneParameter(2,nb_param,param_address+10,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : minHeight        IN  +8 */
        GetGsOsCallOneParameter(3,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : startX           IN  +6 */
        GetGsOsCallOneParameter(4,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #05 WORD   : startY           IN  +4 */
        GetGsOsCallOneParameter(5,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #06 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(6,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+12,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x1C0E :
        strcpy(buffer_line,"SizeWindow(");
        nb_param = 3;
        /* #01 WORD   : newWidth         IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : newHeight        IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1D0E :
        strcpy(buffer_line,"TaskMaster(");
        nb_param = 3;
        /* #01 WORD   : wordspace        OUT +6 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : taskMask         IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : taskRecPtr       IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1E0E :
        strcpy(buffer_line,"BeginUpdate(");
        /* #01 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1F0E :
        strcpy(buffer_line,"EndUpdate(");
        /* #01 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x200E :
        strcpy(buffer_line,"GetWMgrPort(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x210E :
        strcpy(buffer_line,"PinRect(");
        nb_param = 4;
        /* #01 DWORD  : longspace        OUT +8 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : theXPt           IN  +6 */
        GetGsOsCallOneParameter(2,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : theYPt           IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : rectPtr          IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x220E :
        strcpy(buffer_line,"HiliteWindow(");
        nb_param = 2;
        /* #01 WORD   : fHiliteFlag      IN  +4 */
        GetGsOsCallOneParameter(1,1,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x230E :
        strcpy(buffer_line,"ShowHide(");
        nb_param = 2;
        /* #01 WORD   : showFlag         IN  +4 */
        GetGsOsCallOneParameter(1,1,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x240E :
        strcpy(buffer_line,"BringToFront(");
        /* #01 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x250E :
        strcpy(buffer_line,"WindNewRes(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x260E :
        strcpy(buffer_line,"TrackZoom(");
        nb_param = 4;
        /* #01 WORD   : wordspace        OUT +8 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : startX           IN  +6 */
        GetGsOsCallOneParameter(2,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : startY           IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x270E :
        strcpy(buffer_line,"ZoomWindow(");
        /* #01 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x280E :
        strcpy(buffer_line,"SetWRefCon(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x290E :
        strcpy(buffer_line,"GetWRefCon(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x2A0E :
        strcpy(buffer_line,"GetNextWindow(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x2B0E :
        strcpy(buffer_line,"GetWKind(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x2C0E :
        strcpy(buffer_line,"GetWFrame(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x2D0E :
        strcpy(buffer_line,"SetWFrame(");
        nb_param = 2;
        /* #01 WORD   : wFrame           IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2E0E :
        strcpy(buffer_line,"GetStructRgn(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x2F0E :
        strcpy(buffer_line,"GetContentRgn(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x300E :
        strcpy(buffer_line,"GetUpdateRgn(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x310E :
        strcpy(buffer_line,"GetDefProc(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x320E :
        strcpy(buffer_line,"SetDefProc(");
        nb_param = 2;
        /* #01 DWORD  : wDefProcPtr      IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x330E :
        strcpy(buffer_line,"GetWControls(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x340E :
        strcpy(buffer_line,"SetOriginMask(");
        nb_param = 2;
        /* #01 WORD   : originMask       IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x350E :
        strcpy(buffer_line,"GetInfoRefCon(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x360E :
        strcpy(buffer_line,"SetInfoRefCon(");
        nb_param = 2;
        /* #01 DWORD  : infoRefCon       IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x370E :
        strcpy(buffer_line,"GetZoomRect(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x380E :
        strcpy(buffer_line,"SetZoomRect(");
        nb_param = 2;
        /* #01 DWORD  : wZoomSizePtr     IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x390E :
        strcpy(buffer_line,"RefreshDesktop(");
        /* #01 DWORD  : redrawRect       IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3A0E :
        strcpy(buffer_line,"InvalRect(");
        /* #01 DWORD  : badRectPtr       IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3B0E :
        strcpy(buffer_line,"InvalRgn(");
        /* #01 DWORD  : badRgnHandle     IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3C0E :
        strcpy(buffer_line,"ValidRect(");
        /* #01 DWORD  : goodRectPtr      IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3D0E :
        strcpy(buffer_line,"ValidRgn(");
        /* #01 DWORD  : goodRgnHandle    IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3E0E :
        strcpy(buffer_line,"GetContentOrigin(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x3F0E :
        strcpy(buffer_line,"SetContentOrigin(");
        nb_param = 3;
        /* #01 WORD   : xOrigin          IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : yOrigin          IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x400E :
        strcpy(buffer_line,"GetDataSize(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x410E :
        strcpy(buffer_line,"SetDataSize(");
        nb_param = 3;
        /* #01 WORD   : dataWidth        IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : dataHeight       IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x420E :
        strcpy(buffer_line,"GetMaxGrow(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x430E :
        strcpy(buffer_line,"SetMaxGrow(");
        nb_param = 3;
        /* #01 WORD   : maxWidth         IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : maxHeight        IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x440E :
        strcpy(buffer_line,"GetScroll(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x450E :
        strcpy(buffer_line,"SetScroll(");
        nb_param = 3;
        /* #01 WORD   : hScroll          IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : vScroll          IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x460E :
        strcpy(buffer_line,"GetPage(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x470E :
        strcpy(buffer_line,"SetPage(");
        nb_param = 3;
        /* #01 WORD   : hPage            IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : vPage            IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x480E :
        strcpy(buffer_line,"GetContentDraw(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x490E :
        strcpy(buffer_line,"SetContentDraw(");
        nb_param = 2;
        /* #01 DWORD  : contentProcPtr   IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x4A0E :
        strcpy(buffer_line,"GetInfoDraw(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x4B0E :
        strcpy(buffer_line,"SetSysWindow(");
        /* #01 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x4C0E :
        strcpy(buffer_line,"GetSysWFlag(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x4D0E :
        strcpy(buffer_line,"StartDrawing(");
        /* #01 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x4E0E :
        strcpy(buffer_line,"SetWindowIcons(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x4F0E :
        strcpy(buffer_line,"GetRectInfo(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x500E :
        strcpy(buffer_line,"StartInfoDrawing(");
        nb_param = 2;
        /* #01 DWORD  : infoRectPtr      IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #01 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        break;

      case 0x510E :
        strcpy(buffer_line,"EndInfoDrawing(");
        /* No Parameters */
        break;

      case 0x520E :
        strcpy(buffer_line,"GetFirstWindow(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x530E :
        strcpy(buffer_line,"WindDragRect(");
        nb_param = 9;
        /* #01 DWORD  : longspace        OUT +26 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : actionProcPtr    IN  +22 */
        GetGsOsCallOneParameter(2,nb_param,param_address+22,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : dragPatternPtr   IN  +18 */
        GetGsOsCallOneParameter(3,nb_param,param_address+18,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : startX           IN  +16 */
        GetGsOsCallOneParameter(4,nb_param,param_address+16,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #05 WORD   : startY           IN  +14 */
        GetGsOsCallOneParameter(5,nb_param,param_address+14,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #06 DWORD  : dragRectPtr      IN  +10 */
        GetGsOsCallOneParameter(6,nb_param,param_address+10,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #07 DWORD  : limitRectPtr     IN  +6 */
        GetGsOsCallOneParameter(7,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #08 DWORD  : stopRectPtr      IN  +2 */
        GetGsOsCallOneParameter(8,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #09 WORD   : dragFlag         IN  +0 */
        GetGsOsCallOneParameter(9,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+26,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x540E :
        strcpy(buffer_line,"GetDragRect(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x550E :
        strcpy(buffer_line,"DrawInfoBar(");
        /* #01 DWORD  : grafPortPtr      IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x560E :
        strcpy(buffer_line,"WindowGlobal(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : windowGlobalMask IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x570E :
        strcpy(buffer_line,"SetContentOrigin2(");
        nb_param = 4;
        /* #01 WORD   : scrollFlag       IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : xOrigin          IN  +6 */
        GetGsOsCallOneParameter(2,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : yOrigin          IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x580E :
        strcpy(buffer_line,"GetWindowMgrGlobals(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x590E :
        strcpy(buffer_line,"AlertWindow(");
        nb_param = 4;
        /* #01 WORD   : wordspace        OUT +10 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : alertFlags       IN  +8 */
        GetGsOsCallOneParameter(2,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : subStrPtr        IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : alertStrRef      IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+10,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x5A0E :
        strcpy(buffer_line,"StartFrameDrawing(");
        /* #01 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x5B0E :
        strcpy(buffer_line,"EndFrameDrawing(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x5C0E :
        strcpy(buffer_line,"ResizeWindow(");
        nb_param = 3;
        /* #01 WORD   : hiddenFlag       IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : rectPtr          IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : grafPortPtr      IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x5D0E :
        strcpy(buffer_line,"TaskMasterContent(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : taskRecPtr       IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x5E0E :
        strcpy(buffer_line,"TaskMasterKey(");
        nb_param = 3;
        /* #01 DWORD  : longspace        OUT +8 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : taskRecPtr       IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : windPtr          IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x5F0E :
        strcpy(buffer_line,"TaskMasterDA(");
        nb_param = 3;
        /* #01 WORD   : wordspace        OUT +6 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : eventMask        IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : taskRecPtr       IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x600E :
        strcpy(buffer_line,"CompileText(");
        nb_param = 5;
        /* #01 DWORD  : longspace        OUT +12 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : subType          IN  +10 */
        GetGsOsCallOneParameter(2,nb_param,param_address+10,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : subStringsPtr    IN  +6 */
        GetGsOsCallOneParameter(3,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : srcStringRef     IN  +2 */
        GetGsOsCallOneParameter(4,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #05 WORD   : srcSize          IN  +0 */
        GetGsOsCallOneParameter(5,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+12,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x610E :
        strcpy(buffer_line,"NewWindow2(");
        nb_param = 8;
        /* #01 DWORD  : longspace        OUT +24 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : titlePtr         IN  +20 */
        GetGsOsCallOneParameter(2,nb_param,param_address+20,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : refCon           IN  +16 */
        GetGsOsCallOneParameter(3,nb_param,param_address+16,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : contentDrawPtr   IN  +12 */
        GetGsOsCallOneParameter(4,nb_param,param_address+12,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #05 DWORD  : defProcPtr       IN  +8 */
        GetGsOsCallOneParameter(5,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #06 WORD   : paramTableDesc   IN  +6 */
        GetGsOsCallOneParameter(6,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #07 DWORD  : paramTableRef    IN  +2 */
        GetGsOsCallOneParameter(7,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #08 WORD   : resourceType     IN  +0 */
        GetGsOsCallOneParameter(8,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+24,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x620E :
        strcpy(buffer_line,"ErrorWindow(");
        nb_param = 4;
        /* #01 WORD   : wordspace        OUT +8 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : subType          IN  +6 */
        GetGsOsCallOneParameter(2,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : subStringPtr     IN  +2 */
        GetGsOsCallOneParameter(3,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : errNum           IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x630E :
        strcpy(buffer_line,"GetAuxWindInfo(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : windPtr          IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x640E :
        strcpy(buffer_line,"DoModalWindow(");
        nb_param = 6;
        /* #01 DWORD  : longspace        OUT +18 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : eventPtr         IN  +14 */
        GetGsOsCallOneParameter(2,nb_param,param_address+14,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : updateProc       IN  +10 */
        GetGsOsCallOneParameter(3,nb_param,param_address+10,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : eventHook        IN  +6 */
        GetGsOsCallOneParameter(4,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #05 DWORD  : beepProc         IN  +2 */
        GetGsOsCallOneParameter(5,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #06 WORD   : flags            IN  +0 */
        GetGsOsCallOneParameter(6,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+18,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x650E :
        strcpy(buffer_line,"MWGetCtlPart(");
        /* #01 WORD   : wordspace        IN  +0 */
        GetEmptyParameter(1,1,buffer_line);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x660E :
        strcpy(buffer_line,"MWSetMenuProc(");
        nb_param = 2;
        /* #0 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #0 DWORD  : newMenuProc      IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x670E :
        strcpy(buffer_line,"MWStdDrawProc(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x680E :
        strcpy(buffer_line,"MWSetUpEditMenu(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x690E :
        strcpy(buffer_line,"FindCursorCtl(");
        nb_param = 5;
        /* #01 WORD   : wordspace        OUT +12 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : ctlHandlePtr     IN  +8 */
        GetGsOsCallOneParameter(2,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : xLoc             IN  +6 */
        GetGsOsCallOneParameter(3,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : yLoc             IN  +4 */
        GetGsOsCallOneParameter(4,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #05 DWORD  : windowPtr        IN  +0 */
        GetGsOsCallOneParameter(5,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+12,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x6A0E :
        strcpy(buffer_line,"ResizeInfoBar(");
        nb_param = 3;
        /* #01 WORD   : flags            IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : newSize          IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : windPtr          IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x6B0E :
        strcpy(buffer_line,"HandleDiskInsert(");
        nb_param = 4;
        /* #01 WORD   : wordspace        OUT +6 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : wordspace        OUT +4 */
        GetEmptyParameter(2,nb_param,buffer_line);
        /* #03 WORD   : flags            IN  +2 */
        GetGsOsCallOneParameter(3,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : devNum           IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x6C0E :
        strcpy(buffer_line,"UpdateWindow(");
        nb_param = 2;
        /* #01 WORD   : flags            IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : windowPtr        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;


      /*******************************/
      /*  Tools 0x0F : Menu Manager  */
      /*******************************/
      case 0x010F :
        strcpy(buffer_line,"MenuBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x020F :
        strcpy(buffer_line,"MenuStartUp(");
        nb_param = 2;
        /* #01 WORD  : userID            IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : dPageAddr         IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x030F :
        strcpy(buffer_line,"MenuShutDown(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x040F :
        strcpy(buffer_line,"MenuVersion(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x050F :
        strcpy(buffer_line,"MenuReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x060F :
        strcpy(buffer_line,"MenuStatus(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x090F :
        strcpy(buffer_line,"MenuKey(");
        nb_param = 2;
        /* #01 DWORD  : taskRecPtr       IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : barHandle        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0A0F :
        strcpy(buffer_line,"GetMenuBar(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0B0F :
        strcpy(buffer_line,"MenuRefresh(");
        /* #01 DWORD  : redrawRoutinePtr IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0C0F :
        strcpy(buffer_line,"FlashMenuBar(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0D0F :
        strcpy(buffer_line,"InsertMenu(");
        nb_param = 2;
        /* #01 DWORD  : addMenuHandle    IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : insertAfter      IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0E0F :
        strcpy(buffer_line,"DeleteMenu(");
        /* #01 WORD   : menuNum          IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0F0F :
        strcpy(buffer_line,"InsertMItem(");
        nb_param = 3;
        /* #01 DWORD  : addItemPtr       IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : insertAfter      IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : menuNum          IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x100F :
        strcpy(buffer_line,"DeleteMItem(");
        /* #01 WORD   : itemNum          IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x110F :
        strcpy(buffer_line,"GetSysBar(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x120F :
        strcpy(buffer_line,"SetSysBar(");
        /* #01 DWORD  : barHandle        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x130F :
        strcpy(buffer_line,"FixMenuBar(");
        /* #01 WORD   : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x140F :
        strcpy(buffer_line,"CountMItems(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : menuNum          IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x150F :
        strcpy(buffer_line,"NewMenuBar(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x160F :
        strcpy(buffer_line,"GetMHandle(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : menuNum          IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x170F :
        strcpy(buffer_line,"SetBarColors(");
        nb_param = 3;
        /* #01 WORD   : newBarColor      IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : newInvertColor   IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : newOutColor      IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x180F :
        strcpy(buffer_line,"GetBarColors(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x190F :
        strcpy(buffer_line,"SetMTitleStart(");
        /* #01 WORD   : xStart           IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1A0F :
        strcpy(buffer_line,"GetMTitleStart(");
        /* #01 WORD   : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1B0F :
        strcpy(buffer_line,"GetMenuMgrPort(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x1C0F :
        strcpy(buffer_line,"CalcMenuSize(");
        nb_param = 3;
        /* #01 WORD   : newWidth         IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : newHeight        IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : menuNum          IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1D0F :
        strcpy(buffer_line,"SetMTitleWidth(");
        nb_param = 2;
        /* #01 WORD   : newWidth         IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : menuNum          IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1E0F :
        strcpy(buffer_line,"GetMTitleWidth(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : menuNum          IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1F0F :
        strcpy(buffer_line,"SetMenuFlag(");
        nb_param = 2;
        /* #01 WORD   : newValue         IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : menuNum          IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x200F :
        strcpy(buffer_line,"GetMenuFlag(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : menuNum          IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x210F :
        strcpy(buffer_line,"SetMenuTitle(");
        nb_param = 2;
        /* #01 DWORD  : newStrPtr        IN  +2 */
        string_address = GetMemoryValue(param_address+2,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : menuNum          IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x220F :
        strcpy(buffer_line,"GetMenuTitle(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : menuNum          IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x230F :
        strcpy(buffer_line,"MenuGlobal(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : menuGlobalMask   IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x240F :
        strcpy(buffer_line,"SetMItem(");
        nb_param = 2;
        /* #01 DWORD  : newItemLinePtr   IN  +2 */
        string_address = GetMemoryValue(param_address+2,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : itemNum          IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x250F :
        strcpy(buffer_line,"GetMItem(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : itemNum          IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x260F :
        strcpy(buffer_line,"SetMItemFlag(");
        nb_param = 2;
        /* #01 WORD   : newValue         IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : itemNum          IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x270F :
        strcpy(buffer_line,"GetMItemFlag(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : itemNum          IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x280F :
        strcpy(buffer_line,"SetMItemBlink(");
        /* #01 WORD   : count            IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x290F :
        strcpy(buffer_line,"MenuNewRes(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2A0F :
        strcpy(buffer_line,"DrawMenuBar(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2B0F :
        strcpy(buffer_line,"MenuSelect(");
        nb_param = 2;
        /* #01 DWORD  : taskRecPtr       IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : barHandle        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2C0F :
        strcpy(buffer_line,"HiliteMenu(");
        nb_param = 2;
        /* #01 WORD   : hiliteFlag       IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : menuNum          IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2D0F :
        strcpy(buffer_line,"NewMenu(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : menuStringPtr    IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x2E0F :
        strcpy(buffer_line,"DisposeMenu(");
        /* #01 DWORD  : menuHandle       IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2F0F :
        strcpy(buffer_line,"InitPalette(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x300F :
        strcpy(buffer_line,"EnableMItem(");
        /* #01 WORD   : itemNum          IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x310F :
        strcpy(buffer_line,"DisableMItem(");
        /* #01 WORD   : itemNum          IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x320F :
        strcpy(buffer_line,"CheckMItem(");
        nb_param = 2;
        /* #01 WORD   : checkedFlag      IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : itemNum          IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x330F :
        strcpy(buffer_line,"SetMItemMark(");
        nb_param = 2;
        /* #01 WORD   : mark             IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : itemNum          IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x340F :
        strcpy(buffer_line,"GetMItemMark(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : itemNum          IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x350F :
        strcpy(buffer_line,"SetMItemStyle(");
        nb_param = 2;
        /* #01 WORD   : textStyle        IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : itemNum          IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x360F :
        strcpy(buffer_line,"GetMItemStyle(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : itemNum          IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x370F :
        strcpy(buffer_line,"SetMenuID(");
        nb_param = 2;
        /* #01 WORD   : newMenuNum       IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : curMenuNum       IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x380F :
        strcpy(buffer_line,"SetMItemID(");
        nb_param = 2;
        /* #01 WORD   : newItemNum       IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : curItemNum       IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x390F :
        strcpy(buffer_line,"SetMenuBar(");
        /* #01 DWORD  : barHandle        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3A0F :
        strcpy(buffer_line,"SetMItemName(");
        nb_param = 2;
        /* #01 DWORD  : strPtr           IN  +2 */
        string_address = GetMemoryValue(param_address+2,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : itemNum          IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3B0F :
        strcpy(buffer_line,"GetPopUpDefProc(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x3C0F :
        strcpy(buffer_line,"PopUpMenuSelect(");
        nb_param = 6;
        /* #01 WORD   : wordspace        OUT +12 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : selection        IN  +10 */
        GetGsOsCallOneParameter(2,nb_param,param_address+10,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : currentLeft      IN  +8 */
        GetGsOsCallOneParameter(3,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : currentTop       IN  +6 */
        GetGsOsCallOneParameter(4,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #05 WORD   : flag             IN  +4 */
        GetGsOsCallOneParameter(5,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #06 DWORD  : menuHandle       IN  +0 */
        GetGsOsCallOneParameter(6,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+12,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x3D0F :
        strcpy(buffer_line,"DrawPopUp(");
        /* Deprecated */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3E0F :
        strcpy(buffer_line,"NewMenu2(");
        nb_param = 3;
        /* #01 DWORD  : longspace        OUT +6 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : refDesc          IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : menuTRef         IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x3F0F :
        strcpy(buffer_line,"InsertMItem2(");
        nb_param = 4;
        /* #01 WORD   : refDesc          IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : menuItemTRef     IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : insertAfter      IN  +2 */
        GetGsOsCallOneParameter(3,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : menuNumber       IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x400F :
        strcpy(buffer_line,"SetMenuTitle2(");
        nb_param = 3;
        /* #01 WORD   : refDesc          IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : titleRef         IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : menuNum          IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x410F :
        strcpy(buffer_line,"SetMItem2(");
        nb_param = 3;
        /* #01 WORD   : refDesc          IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : menuItemTRef     IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : menuItemID       IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x420F :
        strcpy(buffer_line,"SetMItemName2(");
        nb_param = 3;
        /* #01 WORD   : refDesc          IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : titleRef         IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : menuItemID       IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x430F :
        strcpy(buffer_line,"NewMenuBar2(");
        nb_param = 4;
        /* #01 DWORD  : longspace        OUT +10 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : refDesc          IN  +8 */
        GetGsOsCallOneParameter(2,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : menuBarTRef      IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : windowPtr        IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+10,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x440F :
        strcpy(buffer_line,"GetResMTitle(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : menuHandle       IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x450F :
        strcpy(buffer_line,"HideMenuBar(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x460F :
        strcpy(buffer_line,"ShowMenuBar(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x470F :
        strcpy(buffer_line,"SetItemIcon(");
        nb_param = 3;
        /* #01 WORD   : iconDesc         IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : iconRef          IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : itemID           IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x480F :
        strcpy(buffer_line,"GetItemIcon(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : itemID           IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x490F :
        strcpy(buffer_line,"SetMItemStruct(");
        nb_param = 3;
        /* #01 WORD   : itemStructDesc   IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : itemStructRef    IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : itemID           IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x4A0F :
        strcpy(buffer_line,"GetItemStruct(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : itemID           IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x4B0F :
        strcpy(buffer_line,"RemoveMItemStruct(");
        /* #01 WORD   : itemID           IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x4C0F :
        strcpy(buffer_line,"GetItemFlag2(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : itemID           IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x4D0F :
        strcpy(buffer_line,"SetMItemFlag2(");
        nb_param = 2;
        /* #01 WORD   : newValue         IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : itemID           IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x4E0F :
        strcpy(buffer_line,"GetMItemWidth(");
        nb_param = 3;
        /* #01 WORD   : wordspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : wordspace        OUT +2 */
        GetEmptyParameter(2,nb_param,buffer_line);
        /* #03 WORD   : itemID           IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x4F0F :
        strcpy(buffer_line,"GetMItemBlink(");
        /* #01 WORD   : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x500F :
        strcpy(buffer_line,"InsertPathMItems(");
        nb_param = 7;
        /* #01 WORD   : flags            IN  +16 */
        GetGsOsCallOneParameter(1,nb_param,param_address+16,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : pathPtr          IN  +12 */
        string_address = GetMemoryValue(param_address+12,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : deviceNum        IN  +10 */
        GetGsOsCallOneParameter(3,nb_param,param_address+10,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : menuID           IN  +8 */
        GetGsOsCallOneParameter(4,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #05 WORD   : afterID          IN  +6 */
        GetGsOsCallOneParameter(5,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #06 WORD   : startingID       IN  +4 */
        GetGsOsCallOneParameter(6,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #07 DWORD  : resultPtr        IN  +0 */
        GetGsOsCallOneParameter(7,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;


      /**********************************/
      /*  Tools 0x10 : Control Manager  */
      /**********************************/
      case 0x0110 :
        strcpy(buffer_line,"CtlBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0210 :
        strcpy(buffer_line,"CtlStartUp(");
        nb_param = 2;
        /* #01 WORD  : userID            IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : dPageAddr         IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0310 :
        strcpy(buffer_line,"CtlShutDown(");
        /* No Parameters */
 
        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0410 :
        strcpy(buffer_line,"CtlVersion(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0510 :
        strcpy(buffer_line,"CtlReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0610 :
        strcpy(buffer_line,"CtlStatus(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0910 :
        strcpy(buffer_line,"NewControl(");
        nb_param = 11;
        /* #01 DWORD  : longspace        OUT +32 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : theWindowPtr     IN  +28 */
        GetGsOsCallOneParameter(2,nb_param,param_address+28,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : boundsRectPtr    IN  +24 */
        GetGsOsCallOneParameter(3,nb_param,param_address+24,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : titlePtr         IN  +20 */
        GetGsOsCallOneParameter(4,nb_param,param_address+20,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #05 WORD   : flag             IN  +18 */
        GetGsOsCallOneParameter(5,nb_param,param_address+18,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #06 WORD   : value            IN  +16 */
        GetGsOsCallOneParameter(6,nb_param,param_address+16,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #07 WORD   : param1           IN  +14 */
        GetGsOsCallOneParameter(7,nb_param,param_address+14,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #08 WORD   : param2           IN  +12 */
        GetGsOsCallOneParameter(8,nb_param,param_address+12,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #09 DWORD  : defProcPtr       IN  +8 */
        GetGsOsCallOneParameter(9,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #10 DWORD  : refCon           IN  +4 */
        GetGsOsCallOneParameter(10,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #11 DWORD  : colorTablePtr    IN  +0 */
        GetGsOsCallOneParameter(11,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+32,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0A10 :
        strcpy(buffer_line,"DisposeControl(");
        /* #01 DWORD  : theControlHandle IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0B10 :
        strcpy(buffer_line,"KillControls(");
        /* #01 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0C10 :
        strcpy(buffer_line,"SetCtlTitle(");
        nb_param = 2;
        /* #01 DWORD  : titlePtr         IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : theControlHandle IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0D10 :
        strcpy(buffer_line,"GetCtlTitle(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : theControlHandle IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0E10 :
        strcpy(buffer_line,"HideControl(");
        /* #01 DWORD  : theControlHandle  IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0F10 :
        strcpy(buffer_line,"ShowControl(");
        /* #01 DWORD  : theControlHandle  IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1010 :
        strcpy(buffer_line,"DrawControls(");
        /* #01 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1110 :
        strcpy(buffer_line,"HiliteControl(");
        nb_param = 2;
        /* #01 WORD   : hiliteState      IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : theControlHandle IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1210 :
        strcpy(buffer_line,"CtlNewRes(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1310 :
        strcpy(buffer_line,"FindControl(");
        nb_param = 5;
        /* #01 WORD   : wordspace        OUT +10 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : foundCtlPtr      IN  +6 */
        GetGsOsCallOneParameter(2,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : pointX           IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : pointY           IN  +2 */
        GetGsOsCallOneParameter(4,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #05 DWORD  : theWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(5,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+10,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1410 :
        strcpy(buffer_line,"TestControl(");
        nb_param = 4;
        /* #01 WORD   : wordspace        OUT +8 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : pointX           IN  +6 */
        GetGsOsCallOneParameter(2,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : pointY           IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : theControlHandle IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1510 :
        strcpy(buffer_line,"TrackControl(");
        nb_param = 5;
        /* #01 WORD   : wordspace        OUT +12 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : startX           IN  +10 */
        GetGsOsCallOneParameter(2,nb_param,param_address+10,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : startY           IN  +8 */
        GetGsOsCallOneParameter(3,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : actionProcPtr    IN  +4 */
        GetGsOsCallOneParameter(4,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #05 DWORD  : theControlHandle IN  +0 */
        GetGsOsCallOneParameter(5,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+12,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1610 :
        strcpy(buffer_line,"MoveControl(");
        nb_param = 3;
        /* #01 WORD   : newX             IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : newY             IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : theControlHandle IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1710 :
        strcpy(buffer_line,"DrawControl(");
        nb_param = 6;
        /* #01 WORD   : startX           IN  +16 */
        GetGsOsCallOneParameter(1,nb_param,param_address+16,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : startY           IN  +14 */
        GetGsOsCallOneParameter(2,nb_param,param_address+14,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : limitRectPtr     IN  +10 */
        GetGsOsCallOneParameter(3,nb_param,param_address+10,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : slopRectPtr      IN  +6 */
        GetGsOsCallOneParameter(4,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #05 WORD   : axis             IN  +4 */
        GetGsOsCallOneParameter(5,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #06 DWORD  : theControlHandle IN  +0 */
        GetGsOsCallOneParameter(6,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1810 :
        strcpy(buffer_line,"SetCtlIcons(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : newFontHandle    IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x1910 :
        strcpy(buffer_line,"SetCtlValue(");
        nb_param = 2;
        /* #01 WORD   : curValue         IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : theControlHandle IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1A10 :
        strcpy(buffer_line,"GetCtlValue(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : theControlHandle IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1B10 :
        strcpy(buffer_line,"SetCtlParams(");
        nb_param = 3;
        /* #01 WORD   : param2           IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : param1           IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : theControlHandle IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1C10 :
        strcpy(buffer_line,"GetCtlParams(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : theControlHandle IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x1D10 :
        strcpy(buffer_line,"DragRect(");
        nb_param = 9;
        /* #01 DWORD  : longspace        OUT +26 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : actionProcPtr    IN  +22 */
        GetGsOsCallOneParameter(2,nb_param,param_address+22,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : dragPatternPtr   IN  +18 */
        GetGsOsCallOneParameter(3,nb_param,param_address+18,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : startX           IN  +16 */
        GetGsOsCallOneParameter(4,nb_param,param_address+16,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #05 WORD   : startY           IN  +14 */
        GetGsOsCallOneParameter(5,nb_param,param_address+14,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #06 DWORD  : dragRectPtr      IN  +10 */
        GetGsOsCallOneParameter(6,nb_param,param_address+10,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #07 DWORD  : limitRectPtr     IN  +6 */
        GetGsOsCallOneParameter(7,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #08 DWORD  : slopRectPtr      IN  +2 */
        GetGsOsCallOneParameter(8,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #09 WORD   : dragFlag         IN  +0 */
        GetGsOsCallOneParameter(9,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+26,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x1E10 :
        strcpy(buffer_line,"GrowSize(");
        /* #0 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x1F10 :
        strcpy(buffer_line,"GetCtlDpage(");
        /* #0 WORD   : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x2010 :
        strcpy(buffer_line,"SetCtlAction(");
        nb_param = 2;
        /* #01 DWORD  : newActionPtr     IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : theControlHandle IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2110 :
        strcpy(buffer_line,"GetCtlAction(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : theControlHandle IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x2210 :
        strcpy(buffer_line,"SetCtlRefCon(");
        nb_param = 2;
        /* #01 DWORD  : newRefCon        IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : theControlHandle IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2310 :
        strcpy(buffer_line,"GetCtlRefCon(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : theControlHandle IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x2410 :
        strcpy(buffer_line,"EraseControl(");
        /* #01 DWORD  : theControlHandle IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2510 :
        strcpy(buffer_line,"DrawOneCtl(");
        /* #01 DWORD  : theControlHandle IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2610 :
        strcpy(buffer_line,"FindTargetCtl(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x2710 :
        strcpy(buffer_line,"MakeNextCtlTarget(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x2810 :
        strcpy(buffer_line,"MakeThisCtlTarget(");
        /* #01 DWORD  : ctlToBeTarget    IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2910 :
        strcpy(buffer_line,"SendEventToCtl(");
        nb_param = 4;
        /* #01 DWORD  : longspace        OUT +10 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : targetOnlyFlag   IN  +8 */
        GetGsOsCallOneParameter(2,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : ctlWindowPtr     IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : eTaskRecPtr      IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+10,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x2A10 :
        strcpy(buffer_line,"GetCtlID(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : ctlHandle        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x2B10 :
        strcpy(buffer_line,"SetCtlID(");
        nb_param = 2;
        /* #01 DWORD  : newID            IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : ctlHandle        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2C10 :
        strcpy(buffer_line,"CallCtlDefProc(");
        nb_param = 4;
        /* #01 DWORD  : longspace        OUT +10 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : ctlHandle        IN  +6 */
        GetGsOsCallOneParameter(2,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : ctlMessage       IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : ctlParam         IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+10,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x2D10 :
        strcpy(buffer_line,"NotifyCtls(");
        nb_param = 4;
        /* #01 WORD   : mask             IN  +10 */
        GetGsOsCallOneParameter(1,nb_param,param_address+10,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : message          IN  +8 */
        GetGsOsCallOneParameter(2,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : param            IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : window           IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2E10 :
        strcpy(buffer_line,"GetCtlMoreFlags(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : ctlHandle        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x2F10 :
        strcpy(buffer_line,"SetCtlMoreFlags(");
        nb_param = 2;
        /* #01 WORD   : newMoreFlags     IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : ctlHandle        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3010 :
        strcpy(buffer_line,"GetCtlHandleFromID(");
        nb_param = 3;
        /* #01 DWORD  : longspace        OUT +8 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : ctlWindowPtr     IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : ctlID            IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x3110 :
        strcpy(buffer_line,"NewControl2(");
        nb_param = 4;
        /* #01 DWORD  : longspace        OUT +10 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : ownerPtr         IN  +6 */
        GetGsOsCallOneParameter(2,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : inputDesc        IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : inputRef         IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+10,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x3210 :
        strcpy(buffer_line,"CMLoadResource(");
        nb_param = 3;
        /* #01 DWORD  : longspace        OUT +6 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : resourceType     IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : resourceID       IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x3310 :
        strcpy(buffer_line,"CMReleaseResource(");
        nb_param = 2;
        /* #01 WORD   : resourceType     IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : resourceID       IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3410 :
        strcpy(buffer_line,"SetCtlParamPtr(");
        /* #01 DWORD  : subArrayPtr      IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3510 :
        strcpy(buffer_line,"GetCtlParamPtr(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x3710 :
        strcpy(buffer_line,"InvalCtls(");
        /* #01 DWORD  : ctlWindowPtr     IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3910 :
        strcpy(buffer_line,"FindRadioButton(");
        nb_param = 3;
        /* #01 WORD   : wordspace        OUT +6 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : windPtr          IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : famNum           IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x3A10 :
        strcpy(buffer_line,"SetLETextByID(");
        nb_param = 3;
        /* #01 DWORD  : windPtr          IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : LECtlID          IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : textPtr          IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(3,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3B10 :
        strcpy(buffer_line,"GetLETextByID(");
        nb_param = 3;
        /* #01 DWORD  : windPtr          IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : LECtlID          IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : textPtr          IN  +0 */
        GetEmptyParameter(3,nb_param,buffer_line);

        /** Return Value **/
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,3,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,cyrene_param);
        break;

      case 0x3C10 :
        strcpy(buffer_line,"SetCtlValueByID(");
        nb_param = 3;
        /* #01 WORD   : newValue         IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : windPtr          IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : ctlID            IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(3,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3D10 :
        strcpy(buffer_line,"GetCtlValueByID(");
        nb_param = 3;
        /* #01 WORD   : wordspace        OUT +8 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : windPtr          IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : ctlID            IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x3E10 :
        strcpy(buffer_line,"InvalOneCtlByID(");
        nb_param = 2;
        /* #01 DWORD  : windPtr          IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : ctlID            IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3F10 :
        strcpy(buffer_line,"HiliteCtlByID(");
        nb_param = 3;
        /* #01 WORD   : hiliteState      IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : windPtr          IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : ctlID            IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;


      /*******************************/
      /*  Tools 0x11 : Load Manager  */
      /*******************************/
      case 0x0F11 :
        strcpy(buffer_line,"GetLoadSegInfo(");
        /* #01 WORD  : userID           IN  +8 */
        GetGsOsCallOneParameter(1,4,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : fileNum          IN  +6 */
        GetGsOsCallOneParameter(2,4,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : segNum           IN  +4 */
        GetGsOsCallOneParameter(3,4,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD : buffAddr         IN  +0 */
        GetGsOsCallOneParameter(4,4,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1011 :
        strcpy(buffer_line,"GetUserID(");
        /* #01 WORD  : Space            OUT +4 */
        GetEmptyParameter(1,2,buffer_line);
        /* #02 DWORD : pathnameAddr     IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,2,string_address,TYPE_GSOS_PARAM_STRING,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,2,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x2111 :
        strcpy(buffer_line,"GetUserID2(");
        /* #01 WORD  : Space            OUT +4 */
        GetEmptyParameter(1,2,buffer_line);
        /* #02 DWORD : pathnameAddr     IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,2,string_address,TYPE_GSOS_PARAM_STRING,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,2,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0911 :
        strcpy(buffer_line,"InitialLoad(");
        /* #01 WORD  : Space            OUT +16 */
        GetEmptyParameter(1,7,buffer_line);
        /* #02 WORD  : Space            OUT +14 */
        GetEmptyParameter(2,7,buffer_line);
        /* #03 DWORD : Space            OUT +10 */
        GetEmptyParameter(3,7,buffer_line);
        /* #04 WORD  : Space            OUT +8 */
        GetEmptyParameter(4,7,buffer_line);
        /* #05 WORD  : userID           IN  +6 */
        GetGsOsCallOneParameter(5,7,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #06 DWORD : pathnameAddr     IN  +2 */
        string_address = GetMemoryValue(param_address+2,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(6,7,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #07 WORD  : flagWord         IN  +0 */
        GetGsOsCallOneParameter(7,7,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,7,param_address+16,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,2,7,param_address+14,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,3,7,param_address+10,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,4,7,param_address+8,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x2011 :
        strcpy(buffer_line,"InitialLoad2(");
        /* #01 WORD  : Space            OUT +18 */
        GetEmptyParameter(1,8,buffer_line);
        /* #02 WORD  : Space            OUT +16 */
        GetEmptyParameter(2,8,buffer_line);
        /* #03 DWORD : Space            OUT +12 */
        GetEmptyParameter(3,8,buffer_line);
        /* #04 WORD  : Space            OUT +10 */
        GetEmptyParameter(4,8,buffer_line);
        /* #05 WORD  : userID           IN  +8 */
        GetGsOsCallOneParameter(5,8,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #06 DWORD : buffAddr         IN  +4 */
        GetGsOsCallOneParameter(6,8,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #07 WORD  : flagWord         IN  +2 */
        GetGsOsCallOneParameter(7,8,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #08 WORD  : inputType        IN  +0 */
        GetGsOsCallOneParameter(8,8,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,8,param_address+18,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,2,8,param_address+16,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,3,8,param_address+12,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,4,8,param_address+10,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1111 :
        strcpy(buffer_line,"LGetPathname(");
        /* #01 DWORD : Space            OUT +4 */
        GetEmptyParameter(1,3,buffer_line);
        /* #02 WORD  : userID           IN  +2 */
        GetGsOsCallOneParameter(2,3,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : fileNum          IN  +0 */
        GetGsOsCallOneParameter(3,3,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,3,param_address+4,TYPE_GSOS_PARAM_STRING_PTR,cyrene_param);
        break;

      case 0x2211 :
        strcpy(buffer_line,"LGetPathname2(");
        /* #01 DWORD : Space            OUT +4 */
        GetEmptyParameter(1,3,buffer_line);
        /* #02 WORD  : userID           IN  +2 */
        GetGsOsCallOneParameter(2,3,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : fileNum          IN  +0 */
        GetGsOsCallOneParameter(3,3,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,3,param_address+4,TYPE_GSOS_PARAM_STRING_PTR,cyrene_param);
        break;

      case 0x0111 :
        strcpy(buffer_line,"LoaderInitialization(");
        /* No Param */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0511 :
        strcpy(buffer_line,"LoaderReset(");
        /* No Param */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0311 :
        strcpy(buffer_line,"LoaderShutDown(");
        /* No Param */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0211 :
        strcpy(buffer_line,"LoaderStartUp(");
        /* No Param */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0611 :
        strcpy(buffer_line,"LoaderStatus(");
        /* #01 WORD  : Space            OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0411 :
        strcpy(buffer_line,"LoaderVersion(");
        /* #01 WORD  : Space            OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0D11 :
        strcpy(buffer_line,"LoadSegName(");
        /* #01 WORD  : Space            OUT +18 */
        GetEmptyParameter(1,7,buffer_line);
        /* #02 WORD  : Space            OUT +16 */
        GetEmptyParameter(2,7,buffer_line);
        /* #03 WORD  : Space            OUT +14 */
        GetEmptyParameter(3,7,buffer_line);
        /* #04 DWORD : Space            OUT +10 */
        GetEmptyParameter(4,7,buffer_line);
        /* #05 WORD  : userID           IN  +8 */
        GetGsOsCallOneParameter(5,7,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #06 DWORD : filenameAddr     IN  +4 */
        string_address = GetMemoryValue(param_address+4,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(6,7,string_address,TYPE_GSOS_PARAM_STRING,buffer_line,current_operation,cyrene_param);
        /* #07 DWORD : segNameAddr      IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(7,7,string_address,TYPE_GSOS_PARAM_STRING,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,7,param_address+18,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,2,7,param_address+16,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,3,7,param_address+14,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,4,7,param_address+10,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0B11 :
        strcpy(buffer_line,"LoadSegNum(");
        /* #01 DWORD : Space            OUT +6 */
        GetEmptyParameter(1,4,buffer_line);
        /* #02 WORD  : userID           IN  +4 */
        GetGsOsCallOneParameter(2,4,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : fileNum          IN  +2 */
        GetGsOsCallOneParameter(3,4,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD  : segNum           IN  +0 */
        GetGsOsCallOneParameter(4,4,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,4,param_address+6,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x1311 :
        strcpy(buffer_line,"RenamePathname(");
        /* #01 DWORD : oldAddress       IN  +4 */
        GetGsOsCallOneParameter(1,2,param_address+4,TYPE_GSOS_PARAM_STRING,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : newAddress       IN  +0 */
        GetGsOsCallOneParameter(2,2,param_address+0,TYPE_GSOS_PARAM_STRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0A11 :
        strcpy(buffer_line,"Restart(");
        /* #01 WORD  : Space            OUT +10 */
        GetEmptyParameter(1,5,buffer_line);
        /* #02 WORD  : Space            OUT +8 */
        GetEmptyParameter(2,5,buffer_line);
        /* #03 DWORD : Space            OUT +4 */
        GetEmptyParameter(3,5,buffer_line);
        /* #04 WORD  : Space            OUT +2 */
        GetEmptyParameter(4,5,buffer_line);
        /* #05 WORD  : userID           IN  +0 */
        GetGsOsCallOneParameter(5,5,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,5,param_address+10,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,2,5,param_address+8,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,3,5,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,4,5,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0E11 :
        strcpy(buffer_line,"UnloadSeg(");
        /* #01 WORD  : Space            OUT +8 */
        GetEmptyParameter(1,4,buffer_line);
        /* #02 WORD  : Space            OUT +6 */
        GetEmptyParameter(2,4,buffer_line);
        /* #03 WORD  : Space            OUT +4 */
        GetEmptyParameter(3,4,buffer_line);
        /* #04 DWORD : address          IN  +0 */
        GetGsOsCallOneParameter(4,4,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,4,param_address+8,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,2,4,param_address+6,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,3,4,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0C11 :
        strcpy(buffer_line,"UnloadSegNum(");
        /* #01 WORD  : userID           IN  +4 */
        GetGsOsCallOneParameter(1,3,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : fileNum          IN  +2 */
        GetGsOsCallOneParameter(2,3,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : segNum           IN  +0 */
        GetGsOsCallOneParameter(3,3,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1211 :
        strcpy(buffer_line,"UserShutDown(");
        /* #01 WORD  : Space            OUT +4 */
        GetEmptyParameter(1,3,buffer_line);
        /* #02 WORD  : UserID           IN  +2 */
        GetGsOsCallOneParameter(2,3,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : flag             IN  +0 */
        GetGsOsCallOneParameter(3,3,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,3,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;


      /********************************/
      /*  Tools 0x12 : QuickDraw Aux  */
      /********************************/
      case 0x0112 :
        strcpy(buffer_line,"QDAuxBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0212 :
        strcpy(buffer_line,"QDAuxStartUp(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0312 :
        strcpy(buffer_line,"QDAuxShutDown(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0412 :
        strcpy(buffer_line,"QDAuxVersion(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0512 :
        strcpy(buffer_line,"QDAuxReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0612 :
        strcpy(buffer_line,"QDAuxStatus(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0912 :
        strcpy(buffer_line,"CopyPixels(");
        nb_param = 6;
        /* #01 DWORD  : srcLocInfoPtr    IN  +18 */
        GetGsOsCallOneParameter(1,nb_param,param_address+18,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : destLocPtr       IN  +14 */
        GetGsOsCallOneParameter(2,nb_param,param_address+14,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : srcRect          IN  +10 */
        GetGsOsCallOneParameter(3,nb_param,param_address+10,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : destRect         IN  +6 */
        GetGsOsCallOneParameter(4,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #05 WORD   : xferMode         IN  +4 */
        GetGsOsCallOneParameter(5,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #06 DWORD  : maskRgn          IN  +0 */
        GetGsOsCallOneParameter(6,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0A12 :
        strcpy(buffer_line,"WaitCursor(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0B12 :
        strcpy(buffer_line,"DrawIcon(");
        nb_param = 4;
        /* #01 DWORD  : srcLocInfoPtr    IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : displayMode      IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : xPos             IN  +2 */
        GetGsOsCallOneParameter(3,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : yPos             IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0C12 :
        strcpy(buffer_line,"SpecialRect(");
        nb_param = 3;
        /* #01 DWORD  : rectPtr          IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : frameColor       IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : fillColor        IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0D12 :
        strcpy(buffer_line,"SeedFill(");
        nb_param = 9;
        /* #01 DWORD  : srcLocInfoPtr    IN  +26 */
        GetGsOsCallOneParameter(1,nb_param,param_address+26,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : srcRect          IN  +22 */
        GetGsOsCallOneParameter(2,nb_param,param_address+22,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : destLocInfoPtr   IN  +18 */
        GetGsOsCallOneParameter(3,nb_param,param_address+18,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : destRect         IN  +14 */
        GetGsOsCallOneParameter(4,nb_param,param_address+14,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #05 WORD   : seedH            IN  +12 */
        GetGsOsCallOneParameter(5,nb_param,param_address+12,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #06 WORD   : seedV            IN  +10 */
        GetGsOsCallOneParameter(6,nb_param,param_address+10,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #07 WORD   : resMode          IN  +8 */
        GetGsOsCallOneParameter(7,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #08 DWORD  : patternPtr       IN  +4 */
        GetGsOsCallOneParameter(8,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #09 DWORD  : leakTblPtr       IN  +0 */
        GetGsOsCallOneParameter(9,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0E12 :
        strcpy(buffer_line,"CalcMask(");
        nb_param = 7;
        /* #01 DWORD  : srcLocInfoPtr    IN  +22 */
        GetGsOsCallOneParameter(1,nb_param,param_address+22,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : srcRect          IN  +18 */
        GetGsOsCallOneParameter(2,nb_param,param_address+18,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : destLocInfoPtr   IN  +14 */
        GetGsOsCallOneParameter(3,nb_param,param_address+14,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : destRect         IN  +10 */
        GetGsOsCallOneParameter(4,nb_param,param_address+10,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #05 WORD   : resMode          IN  +8 */
        GetGsOsCallOneParameter(5,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #06 DWORD  : patternPtr       IN  +4 */
        GetGsOsCallOneParameter(6,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #07 DWORD  : leakTblPtr       IN  +0 */
        GetGsOsCallOneParameter(7,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0F12 :
        strcpy(buffer_line,"GetSysIcon(");
        nb_param = 4;
        /* #01 DWORD  : longspace        OUT +8 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : flags            IN  +6 */
        GetGsOsCallOneParameter(2,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : value            IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : auxValue         IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x1012 :
        strcpy(buffer_line,"PixelMap2Rgn(");
        nb_param = 4;
        /* #01 DWORD  : longspace        OUT +8 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : srcLocInfo       IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : bitsPerPixxel    IN  +2 */
        GetGsOsCallOneParameter(3,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : colorsToInclude  IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x1312 :
        strcpy(buffer_line,"IBeamCursor(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1412 :
        strcpy(buffer_line,"WhooshRect(");
        nb_param = 3;
        /* #01 DWORD  : flags            IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : smallRect        IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : bigRect          IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1512 :
        strcpy(buffer_line,"DrawStringWidth(");
        nb_param = 3;
        /* #01 WORD   : flags            IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : ref              IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : width            IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1612 :
        strcpy(buffer_line,"UseColorTable(");
        nb_param = 4;
        /* #01 DWORD  : longspace        OUT +8 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : tableNum         IN  +6 */
        GetGsOsCallOneParameter(2,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : tablePtr         IN  +2 */
        GetGsOsCallOneParameter(3,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : flags            IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x1712 :
        strcpy(buffer_line,"RestoreColorTable(");
        nb_param = 2;
        /* #01 DWORD  : colorInfo        IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : flags            IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;


      /********************************/
      /*  Tools 0x13 : Print Manager  */
      /********************************/
      case 0x0113 :
        strcpy(buffer_line,"PMBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0213 :
        strcpy(buffer_line,"PMStartUp(");
        nb_param = 2;
        /* #01 WORD  : userID            IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : dPageAddr         IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0313 :
        strcpy(buffer_line,"PMShutDown(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0413 :
        strcpy(buffer_line,"PMVersion(");
        /* #01 WORD  : wordspace         OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0513 :
        strcpy(buffer_line,"PMReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0613 :
        strcpy(buffer_line,"PMStatus(");
        /* #01 WORD  : wordspace         OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0913 :
        strcpy(buffer_line,"PrDefault(");
        /* #01 DWORD : printRecordHandle IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0A13 :
        strcpy(buffer_line,"PrValidate(");
        nb_param = 2;
        /* #01 WORD  : wordspace         OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD : printRecordHandle IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0B13 :
        strcpy(buffer_line,"PrStlDialog(");
        nb_param = 2;
        /* #01 WORD  : wordspace         OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD : printRecordHandle IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0C13 :
        strcpy(buffer_line,"PrJobDialog(");
        nb_param = 2;
        /* #01 WORD  : wordspace         OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD : printRecordHandle IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0D13 :
        strcpy(buffer_line,"PrPixelMap(");
        nb_param = 3;
        /* #01 DWORD : srcLocPtr         IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : srcRectPtr        IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : colorFlag         IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0E13 :
        strcpy(buffer_line,"PrOpenDoc(");
        nb_param = 3;
        /* #01 DWORD : longspace         OUT +8 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD : printRecordHandle IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : printGrafPortPtr  IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0F13 :
        strcpy(buffer_line,"PrCloseDoc(");
        /* #01 DWORD  : printGrafPortPtr IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1013 :
        strcpy(buffer_line,"PrOpenPage(");
        nb_param = 2;
        /* #01 DWORD : printGrafPortPtr  IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : pageFramePtr      IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1113 :
        strcpy(buffer_line,"PrClosePage(");
        /* #01 DWORD  : printGrafPortPtr IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1213 :
        strcpy(buffer_line,"PrPicFile(");
        nb_param = 3;
        /* #01 DWORD : printRecordHandle IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : printGrafPortPtr  IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : statusRecPtr      IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1313 :
        strcpy(buffer_line,"PrControl(");
        /* No Parameters */
        break;

      case 0x1413 :
        strcpy(buffer_line,"PrError(");
        /* #01 WORD  :  wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1513 :
        strcpy(buffer_line,"PrSetError(");
        /* #01 WORD  :  errorNumber      IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1613 :
        strcpy(buffer_line,"PrChoosePrinter(");
        /* #01 WORD  :  wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1713 :
        strcpy(buffer_line,"GetDeviceName(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1813 :
        strcpy(buffer_line,"PrGetPrinterSpecs(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #01 WORD   : wordspace        OUT +0 */
        GetEmptyParameter(2,nb_param,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1913 :
        strcpy(buffer_line,"PrDevPrChanged(");
        /* #01 DWORD : printerNameStr    IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,1,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1A13 :
        strcpy(buffer_line,"PrDevStartup(");
        nb_param = 2;
        /* #01 DWORD  : curNetName       IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : curZoneName      IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1B13 :
        strcpy(buffer_line,"PrDevShutDown(");
        /* #01 DWORD  : portDriver       IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1C13 :
        strcpy(buffer_line,"PrDevOpen(");
        nb_param = 2;
        /* #01 DWORD  : completionRtn    IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : reserved         IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1D13 :
        strcpy(buffer_line,"PrDevRead(");
        nb_param = 3;
        /* #01 WORD   : wordspace        OUT +6 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : bufferPtr        IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : bufferSize       IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1E13 :
        strcpy(buffer_line,"PrDevWrite(");
        nb_param = 3;
        /* #01 DWORD  : completionRtn    IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : bufferPtr        IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : bufferSize       IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1F13 :
        strcpy(buffer_line,"PrDevClose(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2013 :
        strcpy(buffer_line,"PrDevStatus(");
        /* #01 DWORD  : infoStrBuffer    IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2113 :
        strcpy(buffer_line,"PrDevAsyncRead(");
        nb_param = 4;
        /* #01 WORD   : wordspace        OUT +10 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : completionRtn    IN  +6 */
        GetGsOsCallOneParameter(2,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : bufferSize       IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : bufferPtr        IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+10,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x2213 :
        strcpy(buffer_line,"PrDevWriteBackground(");
        nb_param = 3;
        /* #01 DWORD  : completionRtn    IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : bufferSize       IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : bufferPtr        IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2313 :
        strcpy(buffer_line,"PrDriverVer(");
        /* #01 WORD  : wordspace         OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x2413 :
        strcpy(buffer_line,"PrPortVer(");
        /* #01 WORD  : wordspace         OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x2513 :
        strcpy(buffer_line,"PrGetZoneName(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x2813 :
        strcpy(buffer_line,"PrGetPrinterDvrName(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x2913 :
        strcpy(buffer_line,"PrGetPortDvrName(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x2A13 :
        strcpy(buffer_line,"PrGetUserName(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x2B13 :
        strcpy(buffer_line,"PrGetNetworkName(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x3013 :
        strcpy(buffer_line,"PrDevIsItSafe(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x3113 :
        strcpy(buffer_line,"GetZoneList(");
        /* Not used */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3213 :
        strcpy(buffer_line,"GetMyZone(");
        /* Not used */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3313 :
        strcpy(buffer_line,"GetPrinterList(");
        /* Not used */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3413 :
        strcpy(buffer_line,"PMUnloadDriver(");
        /* #01 WORD   : whichDriver      IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3513 :
        strcpy(buffer_line,"PMLoadDriver(");
        /* #01 WORD   : whichDriver      IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3613 :
        strcpy(buffer_line,"PrGetDocName(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x3713 :
        strcpy(buffer_line,"PrSetDocName(");
        /* #01 DWORD  : docNamePtr       IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,1,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3813 :
        strcpy(buffer_line,"PrGetPgOrientation(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : prRecordHandle   IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;


      /********************************/
      /*  Tool 0x14 : Line Edit Tool  */
      /********************************/
      case 0x0114 :
        strcpy(buffer_line,"LEBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0214 :
        strcpy(buffer_line,"LEStartUp(");
        nb_param = 2;
        /* #01 WORD  : userID            IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : dPageAddr         IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0314 :
        strcpy(buffer_line,"LEShutDown(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0414 :
        strcpy(buffer_line,"LEVersion(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0514 :
        strcpy(buffer_line,"LEReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0614 :
        strcpy(buffer_line,"LEStatus(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0914 :
        strcpy(buffer_line,"LENew(");
        nb_param = 4;
        /* #01 DWORD  : longspace        OUT +10 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : destRectPtr      IN  +6 */
        GetGsOsCallOneParameter(2,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : viewRectPtr      IN  +2 */
        GetGsOsCallOneParameter(3,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : maxTextLen       IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+10,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0A14 :
        strcpy(buffer_line,"LEDispose(");
        /* #01 DWORD  : leRecHandle      IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0B14 :
        strcpy(buffer_line,"LESetText(");
        nb_param = 3;
        /* #01 DWORD  : textPtr          IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : textLength       IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : leRecHandle      IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0C14 :
        strcpy(buffer_line,"LEIdle(");
        /* #01 DWORD  : leRecHandle      IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0D14 :
        strcpy(buffer_line,"LEClick(");
        nb_param = 2;
        /* #01 DWORD  : eventPtr         IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #01 DWORD  : leRecHandle      IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0E14 :
        strcpy(buffer_line,"LESetSelect(");
        nb_param = 3;
        /* #01 WORD   : selStart         IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : selEnd           IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : leRecHandle      IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0F14 :
        strcpy(buffer_line,"LEActivate(");
        /* #01 DWORD  : leRecHandle      IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1014 :
        strcpy(buffer_line,"LEDeactivate(");
        /* #01 DWORD  : leRecHandle      IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1114 :
        strcpy(buffer_line,"LEKey(");
        nb_param = 3;
        /* #01 WORD   : theKey           IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : modifiers        IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : leRecHandle      IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1214 :
        strcpy(buffer_line,"LECut(");
        /* #01 DWORD  : leRecHandle      IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1314 :
        strcpy(buffer_line,"LECopy(");
        /* #01 DWORD  : leRecHandle      IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1414 :
        strcpy(buffer_line,"LEPaste(");
        /* #01 DWORD  : leRecHandle      IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1514 :
        strcpy(buffer_line,"LEDelete(");
        /* #01 DWORD  : leRecHandle      IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1614 :
        strcpy(buffer_line,"LEInsert(");
        nb_param = 3;
        /* #01 DWORD  : textPtr          IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : textLength       IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : leRecHandle      IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1714 :
        strcpy(buffer_line,"LEUpdate(");
        /* #01 DWORD  : leRecHandle      IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1814 :
        strcpy(buffer_line,"LETextBox(");
        nb_param = 4;
        /* #01 DWORD  : textPtr          IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : textLength       IN  +6 */
        GetGsOsCallOneParameter(2,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : rectPtr          IN  +2 */
        GetGsOsCallOneParameter(3,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : just             IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1914 :
        strcpy(buffer_line,"LEFromScrap(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1A14 :
        strcpy(buffer_line,"LEToScrap(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1B14 :
        strcpy(buffer_line,"LEScrapHandle(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x1C14 :
        strcpy(buffer_line,"LEGetScrapLen(");
        /* #01 WORD  : wordspace         OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x1D14 :
        strcpy(buffer_line,"LESetScrapLen(");
        /* #01 WORD   : newLength        IN  +4 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1E14 :
        strcpy(buffer_line,"LESetHilite(");
        nb_param = 2;
        /* #01 DWORD  : hiliteProcPtr    IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #01 DWORD  : leRecHandle      IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1F14 :
        strcpy(buffer_line,"LESetCaret(");
        nb_param = 2;
        /* #01 DWORD  : caretProcPtr     IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #01 DWORD  : leRecHandle      IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2014 :
        strcpy(buffer_line,"LETextBox2(");
        nb_param = 4;
        /* #01 DWORD  : textPtr          IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : textLength       IN  +6 */
        GetGsOsCallOneParameter(2,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : rectPtr          IN  +2 */
        GetGsOsCallOneParameter(3,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : just             IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2114 :
        strcpy(buffer_line,"LESetJust(");
        nb_param = 2;
        /* #01 WORD   : just             IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : leRecHandle      IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2214 :
        strcpy(buffer_line,"LEGetTextHand(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : leRecHandle      IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x2314 :
        strcpy(buffer_line,"LEGetTextLen(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : leRecHandle      IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x2414 :
        strcpy(buffer_line,"GetLEDefProc(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x2514 :
        strcpy(buffer_line,"LEClassifyKey(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : theEvent         IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;


      /*********************************/
      /*  Tools 0x15 : Dialog Manager  */
      /*********************************/
      case 0x0115 :
        strcpy(buffer_line,"DialogBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0215 :
        strcpy(buffer_line,"DialogStartUp(");
        /* #01 WORD  : userID            IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0315 :
        strcpy(buffer_line,"DialogShutDown(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0415 :
        strcpy(buffer_line,"DialogVersion(");
        /* #01 WORD  : wordspace         OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0515 :
        strcpy(buffer_line,"DialogReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0615 :
        strcpy(buffer_line,"DialogStatus(");
        /* #01 WORD  : wordspace         OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0915 :
        strcpy(buffer_line,"ErrorSound(");
        /* #01 DWORD  : soundProcPtr     IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0A15 :
        strcpy(buffer_line,"NewModalDialog(");
        nb_param = 4;
        /* #01 DWORD  : longspace        OUT +10 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : dBoundsRectPtr   IN  +6 */
        GetGsOsCallOneParameter(2,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : dVisibleFlag     IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : dRefCon          IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+10,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0B15 :
        strcpy(buffer_line,"NewModelessDialog(");
        nb_param = 7;
        /* #01 DWORD  : longspace        OUT +22 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : dBoundsRectPtr   IN  +18 */
        GetGsOsCallOneParameter(2,nb_param,param_address+18,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : dTitlePtr        IN  +14 */
        string_address = GetMemoryValue(param_address+14,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(3,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : dBehindPtr       IN  +10 */
        GetGsOsCallOneParameter(4,nb_param,param_address+10,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #05 WORD   : dFlag            IN  +8 */
        GetGsOsCallOneParameter(5,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #06 DWORD  : dRefCon          IN  +4 */
        GetGsOsCallOneParameter(6,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #07 DWORD  : dFullSizePtr     IN  +0 */
        GetGsOsCallOneParameter(7,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+22,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0C15 :
        strcpy(buffer_line,"CloseDialog(");
        /* #01 DWORD  : theDialogPtr     IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0D15 :
        strcpy(buffer_line,"NewDItem(");
        nb_param = 8;
        /* #01 DWORD  : theDialogPtr     IN  +20 */
        GetGsOsCallOneParameter(1,nb_param,param_address+20,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : itemID           IN  +18 */
        GetGsOsCallOneParameter(2,nb_param,param_address+18,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : itemRecttPtr     IN  +14 */
        GetGsOsCallOneParameter(3,nb_param,param_address+14,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : itemType        IN  +12 */
        GetGsOsCallOneParameter(4,nb_param,param_address+12,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #05 DWORD  : itemDescr        IN  +8 */
        GetGsOsCallOneParameter(5,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #06 WORD   : itemValue        IN  +6 */
        GetGsOsCallOneParameter(6,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #07 WORD   : itemFlag         IN  +4 */
        GetGsOsCallOneParameter(7,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #08 DWORD  : itemColorPtr     IN  +0 */
        GetGsOsCallOneParameter(8,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0E15 :
        strcpy(buffer_line,"RemoveDItem(");
        nb_param = 2;
        /* #01 DWORD  : theDialogPtr     IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : itemID           IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0F15 :
        strcpy(buffer_line,"ModalDialog(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : filterProcPtr    IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1015 :
        strcpy(buffer_line,"IsDialogEvent(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : theEventPtr      IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1115 :
        strcpy(buffer_line,"DialogSelect(");
        nb_param = 4;
        /* #01 WORD   : wordspace        OUT +12 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : theDialogPtr     IN  +8 */
        GetGsOsCallOneParameter(2,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : resultPtr        IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : itemHitPtr       IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+12,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1215 :
        strcpy(buffer_line,"DlgCut(");
        /* #01 DWORD  : theDialogPtr     IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1315 :
        strcpy(buffer_line,"DlgCopy(");
        /* #01 DWORD  : theDialogPtr     IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1415 :
        strcpy(buffer_line,"DlgPaste(");
        /* #01 DWORD  : theDialogPtr     IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1515 :
        strcpy(buffer_line,"DlgDelete(");
        /* #01 DWORD  : theDialogPtr     IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        break;

      case 0x1615 :
        strcpy(buffer_line,"DrawDialog(");
        /* #01 DWORD  : theDialogPtr     IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1715 :
        strcpy(buffer_line,"Alert(");
        nb_param = 3;
        /* #01 WORD   : wordspace        OUT +8 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : alertTemplatePtr IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : filterProcPtr    IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1815 :
        strcpy(buffer_line,"StopAlert(");
        nb_param = 3;
        /* #01 WORD   : wordspace        OUT +8 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : alertTemplatePtr IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : filterProcPtr    IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1915 :
        strcpy(buffer_line,"NoteAlert(");
        nb_param = 3;
        /* #01 WORD   : wordspace        OUT +8 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : alertTemplatePtr IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : filterProcPtr    IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1A15 :
        strcpy(buffer_line,"CautionAlert(");
        nb_param = 3;
        /* #01 WORD   : wordspace        OUT +8 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : alertTemplatePtr IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : filterProcPtr    IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1B15 :
        strcpy(buffer_line,"ParamText(");
        nb_param = 4;
        /* #01 DWORD  : param0Ptr        IN  +12 */
        string_address = GetMemoryValue(param_address+12,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : param1Ptr        IN  +8 */
        string_address = GetMemoryValue(param_address+8,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : param2Ptr        IN  +4 */
        string_address = GetMemoryValue(param_address+4,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(3,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : param3Ptr        IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(4,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1C15 :
        strcpy(buffer_line,"SetDAFont(");
        /* #01 DWORD  : fontHandle       IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1E15 :
        strcpy(buffer_line,"GetControlDItem(");
        nb_param = 3;
        /* #01 DWORD  : longspace        OUT +6 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : theDialogPtr     IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : itemID           IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x1F15 :
        strcpy(buffer_line,"GetIText(");
        nb_param = 3;
        /* #01 DWORD  : theDialogPtr     IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : itemID           IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : resultPtr        IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2015 :
        strcpy(buffer_line,"SetIText(");
        nb_param = 3;
        /* #01 DWORD  : theDialogPtr     IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : itemID           IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : theStringPtr     IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(3,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2115 :
        strcpy(buffer_line,"SelectIText(");
        nb_param = 4;
        /* #01 DWORD  : theDialogPtr     IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : itemID           IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : startSel         IN  +2 */
        GetGsOsCallOneParameter(3,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : endSel           IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2215 :
        strcpy(buffer_line,"HideDItem(");
        nb_param = 2;
        /* #01 DWORD  : theDialogPtr     IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : itemID           IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2315 :
        strcpy(buffer_line,"ShowDItem(");
        nb_param = 2;
        /* #01 DWORD  : theDialogPtr     IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : itemID           IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2415 :
        strcpy(buffer_line,"FindDItem(");
        nb_param = 3;
        /* #01 WORD   : wordspace        OUT +8 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : theDialogPtr     IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : thePoint         IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x2515 :
        strcpy(buffer_line,"UpdateDialog(");
        nb_param = 2;
        /* #01 DWORD  : theDialogPtr     IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : updateRgnHandle  IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2615 :
        strcpy(buffer_line,"GetDItemType(");
        nb_param = 3;
        /* #01 WORD   : wordspace        OUT +6 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : theDialogPtr     IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : itemID           IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x2715 :
        strcpy(buffer_line,"SetDItemType(");
        nb_param = 3;
        /* #01 WORD   : itemType         IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : theDialogPtr     IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : itemID           IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2815 :
        strcpy(buffer_line,"GetDItemBox(");
        nb_param = 3;
        /* #01 DWORD  : theDialogPtr     IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : itemID           IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : itemBoxPtr       IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2915 :
        strcpy(buffer_line,"SetDItemBox(");
        nb_param = 3;
        /* #01 DWORD  : theDialogPtr     IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : itemID           IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : itemBoxPtr       IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2A15 :
        strcpy(buffer_line,"GetFirstDItem(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : theDialogPtr     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x2B15 :
        strcpy(buffer_line,"GetNextDItem(");
        nb_param = 3;
        /* #01 WORD   : wordspace        OUT +6 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : theDialogPtr     IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : itemID           IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x2C15 :
        strcpy(buffer_line,"ModalDialog2(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : filterProcPtr    IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x2E15 :
        strcpy(buffer_line,"GetDItemValue(");
        nb_param = 3;
        /* #01 WORD   : wordspace        OUT +6 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : theDialogPtr     IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : itemID           IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x2F15 :
        strcpy(buffer_line,"SetDItemValue(");
        nb_param = 3;
        /* #01 WORD   : itemValue        IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : theDialogPtr     IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : itemID           IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3215 :
        strcpy(buffer_line,"GetNewModalDialog(");
        nb_param = 2;
        /* #01 DWORD : longspace         OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD : dialogTemplatePtr IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x3315 :
        strcpy(buffer_line,"GetNewDItem(");
        nb_param = 2;
        /* #01 DWORD  : theDialogPtr     IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : itemTemplatePtr  IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3415 :
        strcpy(buffer_line,"GetAlertStage(");
        /* #01 WORD   : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x3515 :
        strcpy(buffer_line,"ResetAlertStage(");
        /* No Parameters */
 
        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);        
        break;

      case 0x3615 :
        strcpy(buffer_line,"DefaultFilter(");
        nb_param = 4;
        /* #01 WORD   : wordspace        OUT +12 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : theDialogPtr     IN  +8 */
        GetGsOsCallOneParameter(2,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : theEventPtr      IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : itemHitPtr       IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+12,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x3715 :
        strcpy(buffer_line,"GetDefButton(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : theDialogPtr     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x3815 :
        strcpy(buffer_line,"SetDefButton(");
        nb_param = 2;
        /* #01 WORD   : itemID           IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : theDialogPtr     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3915 :
        strcpy(buffer_line,"DisableDItem(");
        nb_param = 2;
        /* #01 DWORD  : theDialogPtr     IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : itemID           IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3A15 :
        strcpy(buffer_line,"EnableDItem(");
        nb_param = 2;
        /* #01 DWORD  : theDialogPtr     IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : itemID           IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;


      /*********************************/
      /*  Tools 0x16 : Scrap Manager  */
      /*********************************/
      case 0x0116 :
        strcpy(buffer_line,"ScrapBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0216 :
        strcpy(buffer_line,"ScrapStartUp(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0316 :
        strcpy(buffer_line,"ScrapShutDown(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0416 :
        strcpy(buffer_line,"ScrapVersion(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0516 :
        strcpy(buffer_line,"ScrapReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0616 :
        strcpy(buffer_line,"ScrapStatus(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0916 :
        strcpy(buffer_line,"UnloadScrap(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0A16 :
        strcpy(buffer_line,"LoadScrap(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0B16 :
        strcpy(buffer_line,"ZeroScrap(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0C16 :
        strcpy(buffer_line,"PutScrap(");
        nb_param = 3;
        /* #01 DWORD  : numBytes         IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : scrapType        IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : srcPtr           IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0D16 :
        strcpy(buffer_line,"GetScrap(");
        nb_param = 2;
        /* #01 DWORD  : destHandle       IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : scrapType        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0E16 :
        strcpy(buffer_line,"GetScrapHandle(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : scrapType        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0F16 :
        strcpy(buffer_line,"GetScrapSize(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : scrapType        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x1016 :
        strcpy(buffer_line,"GetScrapPath(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x1116 :
        strcpy(buffer_line,"SetScrapPath(");
        /* #01 DWORD  : pathPtr          IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,1,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1216 :
        strcpy(buffer_line,"GetScrapCount(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1316 :
        strcpy(buffer_line,"GetScrapState(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1416 :
        strcpy(buffer_line,"GetIndScrap(");
        nb_param = 2;
        /* #01 WORD   : index            IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : bufferPtr        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1516 :
        strcpy(buffer_line,"ShowClipboard(");
        nb_param = 3;
        /* #01 DWORD  : longspace        OUT +6 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : flags            IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : reserved         IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;


      /*******************************************/
      /*  Tools 0x17 : Standard File Operations  */
      /*******************************************/
      case 0x0117 :
        strcpy(buffer_line,"SFBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0217 :
        strcpy(buffer_line,"SFStartUp(");
        nb_param = 2;
        /* #01 WORD  : userID            IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : dPageAddr         IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
 
        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);        
        break;

      case 0x0317 :
        strcpy(buffer_line,"SFShutDown(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0417 :
        strcpy(buffer_line,"SFVersion(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0517 :
        strcpy(buffer_line,"SFReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0617 :
        strcpy(buffer_line,"SFStatus(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0917 :
        strcpy(buffer_line,"SFGetFile(");
        nb_param = 6;
        /* #01 WORD   : whereX           IN  +18 */
        GetGsOsCallOneParameter(1,nb_param,param_address+18,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : whereY           IN  +16 */
        GetGsOsCallOneParameter(2,nb_param,param_address+16,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : promptPtr        IN  +12 */
        string_address = GetMemoryValue(param_address+12,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(3,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : filterProcPtr    IN  +8 */
        GetGsOsCallOneParameter(4,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #05 DWORD  : typeListPtr      IN  +4 */
        GetGsOsCallOneParameter(5,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #06 DWORD  : replyPtr         IN  +0 */
        GetGsOsCallOneParameter(6,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0A17 :
        strcpy(buffer_line,"SFPutFile(");
        nb_param = 6;
        /* #01 WORD   : whereX           IN  +16 */
        GetGsOsCallOneParameter(1,nb_param,param_address+16,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : whereY           IN  +14 */
        GetGsOsCallOneParameter(2,nb_param,param_address+14,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : promptPtr        IN  +10 */
        string_address = GetMemoryValue(param_address+10,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(3,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : origNamePtr      IN  +6 */
        GetGsOsCallOneParameter(4,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #05 WORD   : maxLen           IN  +4 */
        GetGsOsCallOneParameter(5,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #06 DWORD  : replyPtr         IN  +0 */
        GetGsOsCallOneParameter(6,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0B17 :
        strcpy(buffer_line,"SFPGetFile(");
        nb_param = 8;
        /* #01 WORD   : whereX           IN  +26 */
        GetGsOsCallOneParameter(1,nb_param,param_address+26,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : whereY           IN  +24 */
        GetGsOsCallOneParameter(2,nb_param,param_address+24,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : promptPtr        IN  +20 */
        string_address = GetMemoryValue(param_address+20,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(3,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : filterProcPtr    IN  +16 */
        GetGsOsCallOneParameter(4,nb_param,param_address+16,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #05 DWORD  : typeListPtr      IN  +12 */
        GetGsOsCallOneParameter(5,nb_param,param_address+12,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #06 DWORD  : dialogTempPtr    IN  +8 */
        GetGsOsCallOneParameter(6,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #07 DWORD  : dialogHookPtr    IN  +4 */
        GetGsOsCallOneParameter(7,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #08 DWORD  : replyPtr         IN  +0 */
        GetGsOsCallOneParameter(8,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0C17 :
        strcpy(buffer_line,"SFPPutFile(");
        nb_param = 8;
        /* #01 WORD   : whereX           IN  +24 */
        GetGsOsCallOneParameter(1,nb_param,param_address+24,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : whereY           IN  +22 */
        GetGsOsCallOneParameter(2,nb_param,param_address+22,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : promptPtr        IN  +18 */
        string_address = GetMemoryValue(param_address+18,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(3,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : origNamePtr      IN  +14 */
        GetGsOsCallOneParameter(4,nb_param,param_address+14,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #05 WORD   : maxLen           IN  +12 */
        GetGsOsCallOneParameter(5,nb_param,param_address+12,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #06 DWORD  : dlgTempPtr       IN  +8 */
        GetGsOsCallOneParameter(6,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #07 DWORD  : dialogHookPtr    IN  +4 */
        GetGsOsCallOneParameter(7,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #08 DWORD  : replyPtr         IN  +0 */
        GetGsOsCallOneParameter(8,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0D17 :
        strcpy(buffer_line,"SFAllCaps(");
        /* #01 WORD  : allCapsFlag       IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0E17 :
        strcpy(buffer_line,"SFGetFile2(");
        nb_param = 7;
        /* #01 WORD   : whereX           IN  +20 */
        GetGsOsCallOneParameter(1,nb_param,param_address+20,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : whereY           IN  +18 */
        GetGsOsCallOneParameter(2,nb_param,param_address+18,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : promptRefDesc    IN  +16 */
        GetGsOsCallOneParameter(3,nb_param,param_address+16,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : promptRef        IN  +12 */
        GetGsOsCallOneParameter(4,nb_param,param_address+12,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #05 DWORD  : filterProcPtr    IN  +8 */
        GetGsOsCallOneParameter(5,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #06 DWORD  : typeListPtr      IN  +4 */
        GetGsOsCallOneParameter(6,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #07 DWORD  : replyPtr         IN  +0 */
        GetGsOsCallOneParameter(7,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0F17 :
        strcpy(buffer_line,"SFPutFile2(");
        nb_param = 7;
        /* #01 WORD   : whereX           IN  +18 */
        GetGsOsCallOneParameter(1,nb_param,param_address+18,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : whereY           IN  +16 */
        GetGsOsCallOneParameter(2,nb_param,param_address+16,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : promptRefDesc    IN  +14 */
        GetGsOsCallOneParameter(3,nb_param,param_address+14,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : promptRef        IN  +10 */
        GetGsOsCallOneParameter(4,nb_param,param_address+10,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #05 WORD   : origNameRefDesc  IN  +8 */
        GetGsOsCallOneParameter(5,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #06 DWORD  : origNameRef      IN  +4 */
        GetGsOsCallOneParameter(6,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #07 DWORD  : replyPtr         IN  +0 */
        GetGsOsCallOneParameter(7,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1017 :
        strcpy(buffer_line,"SFPGetFile2(");
        nb_param = 10;
        /* #01 WORD   : whereX           IN  +32 */
        GetGsOsCallOneParameter(1,nb_param,param_address+32,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : whereY           IN  +30 */
        GetGsOsCallOneParameter(2,nb_param,param_address+30,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : itemDrawPtr      IN  +26 */
        GetGsOsCallOneParameter(3,nb_param,param_address+26,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : promptRefDesc    IN  +24 */
        GetGsOsCallOneParameter(4,nb_param,param_address+24,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #05 DWORD  : promptRef        IN  +20 */
        GetGsOsCallOneParameter(5,nb_param,param_address+20,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #06 DWORD  : filterProcPtr    IN  +16 */
        GetGsOsCallOneParameter(6,nb_param,param_address+16,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #07 DWORD  : typeListPtr      IN  +12 */
        GetGsOsCallOneParameter(7,nb_param,param_address+12,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #08 DWORD  : dialogTempPtr    IN  +8 */
        GetGsOsCallOneParameter(8,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #09 DWORD  : dialogHookPtr    IN  +4 */
        GetGsOsCallOneParameter(9,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #10 DWORD  : replyPtr         IN  +0 */
        GetGsOsCallOneParameter(10,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1117 :
        strcpy(buffer_line,"SFPPutFile2(");
        nb_param = 10;
        /* #01 WORD   : whereX           IN  +30 */
        GetGsOsCallOneParameter(1,nb_param,param_address+30,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : whereY           IN  +28 */
        GetGsOsCallOneParameter(2,nb_param,param_address+28,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : itemDrawPtr      IN  +24 */
        GetGsOsCallOneParameter(3,nb_param,param_address+24,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : promptRefDesc    IN  +22 */
        GetGsOsCallOneParameter(4,nb_param,param_address+22,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #05 DWORD  : promptRef        IN  +18 */
        GetGsOsCallOneParameter(5,nb_param,param_address+18,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #06 WORD   : origNameRefDesc  IN  +16 */
        GetGsOsCallOneParameter(6,nb_param,param_address+16,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #07 DWORD  : origNameRef      IN  +12 */
        GetGsOsCallOneParameter(7,nb_param,param_address+12,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #08 DWORD  : dialogTempPtr    IN  +8 */
        GetGsOsCallOneParameter(8,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #09 DWORD  : dialogHookPtr    IN  +4 */
        GetGsOsCallOneParameter(9,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #10 DWORD  : replyPtr         IN  +0 */
        GetGsOsCallOneParameter(10,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1217 :
        strcpy(buffer_line,"SFShowInvisible(");
        nb_param = 2;
        /* #01 WORD  : wordspace         OUT +0 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD  : invisibleState    IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1317 :
        strcpy(buffer_line,"SFReScan(");
        nb_param = 2;
        /* #01 DWORD  : filterProcPtr    IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : typeListPtr      IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1417 :
        strcpy(buffer_line,"SFMultiGet2(");
        nb_param = 7;
        /* #01 WORD   : whereX           IN  +20 */
        GetGsOsCallOneParameter(1,nb_param,param_address+20,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : whereY           IN  +18 */
        GetGsOsCallOneParameter(2,nb_param,param_address+18,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : promptRefDesc    IN  +16 */
        GetGsOsCallOneParameter(3,nb_param,param_address+16,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : promptRef        IN  +12 */
        GetGsOsCallOneParameter(4,nb_param,param_address+12,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #05 DWORD  : filterProcPtr    IN  +8 */
        GetGsOsCallOneParameter(5,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #06 DWORD  : typeListPtr      IN  +4 */
        GetGsOsCallOneParameter(6,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #07 DWORD  : replyPtr         IN  +0 */
        GetGsOsCallOneParameter(7,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1517 :
        strcpy(buffer_line,"SFPMultiGet2(");
        nb_param = 10;
        /* #01 WORD   : whereX           IN  +32 */
        GetGsOsCallOneParameter(1,nb_param,param_address+32,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : whereY           IN  +30 */
        GetGsOsCallOneParameter(2,nb_param,param_address+30,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : itemDrawPtr      IN  +26 */
        GetGsOsCallOneParameter(3,nb_param,param_address+26,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : promptRefDesc    IN  +24 */
        GetGsOsCallOneParameter(4,nb_param,param_address+24,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #05 DWORD  : promptRef        IN  +20 */
        GetGsOsCallOneParameter(5,nb_param,param_address+20,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #06 DWORD  : filterProcPtr    IN  +16 */
        GetGsOsCallOneParameter(6,nb_param,param_address+16,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #07 DWORD  : typeListPtr      IN  +12 */
        GetGsOsCallOneParameter(7,nb_param,param_address+12,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #08 DWORD  : dialogTempPtr    IN  +8 */
        GetGsOsCallOneParameter(8,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #09 DWORD  : dialogHookPtr    IN  +4 */
        GetGsOsCallOneParameter(9,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #10 DWORD  : replyPtr         IN  +0 */
        GetGsOsCallOneParameter(10,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;


      /***********************************/
      /*  Tools 0x19 : Note Synthesizer  */
      /***********************************/
      case 0x0119 :
        strcpy(buffer_line,"NSBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0219 :
        strcpy(buffer_line,"NSStartUp(");
        nb_param = 2;
        /* #01 WORD   : updateRate       IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : userUpdateRtnPtr IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0319 :
        strcpy(buffer_line,"NSShutDown(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0419 :
        strcpy(buffer_line,"NSVersion(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0519 :
        strcpy(buffer_line,"NSReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0619 :
        strcpy(buffer_line,"NSStatus(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0919 :
        strcpy(buffer_line,"AllocGen(");
        nb_param = 2;
        /* #01 WORD  : wordspace         OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD  : requestPriority   IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0A19 :
        strcpy(buffer_line,"DeallocGen(");
        /* #01 WORD   : genNumber        IN  +8 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0B19 :
        strcpy(buffer_line,"NoteOn(");
        nb_param = 4;
        /* #01 WORD   : genNumber        IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : semitone         IN  +6 */
        GetGsOsCallOneParameter(2,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : volume           IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : instrumentPtr    IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0C19 :
        strcpy(buffer_line,"NoteOff(");
        nb_param = 2;
        /* #01 WORD   : genNumber        IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : semitone         IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0D19 :
        strcpy(buffer_line,"AllNotesOff(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0E19 :
        strcpy(buffer_line,"NSSetUpdateRate(");
        nb_param = 2;
        /* #01 WORD  : wordspace         OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD  : updateRate        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0F19 :
        strcpy(buffer_line,"NSSetUserUpdateRtn(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : updateRtn        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;


      /*********************************/
      /*  Tools 0x1A : Note Sequencer  */
      /*********************************/
      case 0x011A :
        strcpy(buffer_line,"SeqBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x021A :
        strcpy(buffer_line,"SeqStartUp(");
        nb_param = 4;
        /* #01 WORD  : dPageAddr         IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : mode              IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : updateRate        IN  +2 */
        GetGsOsCallOneParameter(3,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD  : increment         IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x031A :
        strcpy(buffer_line,"SeqShutDown(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x041A :
        strcpy(buffer_line,"SeqVersion(");
        /* #01 WORD  : wordspace         OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x051A :
        strcpy(buffer_line,"SeqReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x061A :
        strcpy(buffer_line,"SeqStatus(");
        /* #01 WORD  : wordspace         OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x091A :
        strcpy(buffer_line,"SetIncr(");
        /* #01 WORD  : increment         IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0A1A :
        strcpy(buffer_line,"ClearIncr(");
        /* #01 WORD  : wordspace         OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0B1A :
        strcpy(buffer_line,"GetTimer(");
        /* #01 WORD  : wordspace         OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0C1A :
        strcpy(buffer_line,"GetLoc(");
        nb_param = 3;
        /* #01 WORD  : wordspace         OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD  : wordspace         OUT +2 */
        GetEmptyParameter(2,nb_param,buffer_line);
        /* #03 WORD  : wordspace         OUT +0 */
        GetEmptyParameter(3,nb_param,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0D1A :
        strcpy(buffer_line,"SeqAllNotesOff(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0E1A :
        strcpy(buffer_line,"SetTrkInfo(");
        nb_param = 3;
        /* #01 WORD  : priority          IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : instIndex         IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : trackNum          IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0F1A :
        strcpy(buffer_line,"StartSeq(");
        nb_param = 3;
        /* #01 DWORD  : errHndlerPtr     IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : compRoutine      IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : sequence         IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x101A :
        strcpy(buffer_line,"StepSeq(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x111A :
        strcpy(buffer_line,"StopSeq(");
        /* #01 WORD  : next              IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
 
        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);        
        break;

      case 0x121A :
        strcpy(buffer_line,"SetInstTable(");
        /* #01 DWORD  : instTable        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x131A :
        strcpy(buffer_line,"StartInts(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x141A :
        strcpy(buffer_line,"StopInts(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x151A :
        strcpy(buffer_line,"StartSeqRel(");
        nb_param = 3;
        /* #01 DWORD  : errHndlerPtr     IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : compRoutine      IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : sequence         IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;


      /*******************************/
      /*  Tools 0x1B : Font Manager  */
      /*******************************/
      case 0x011B :
        strcpy(buffer_line,"FMBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x021B :
        strcpy(buffer_line,"FMStartUp(");
        nb_param = 2;
        /* #01 WORD  : userID            IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : dPageAddr         IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x031B :
        strcpy(buffer_line,"FMShutDown(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x041B :
        strcpy(buffer_line,"FMVersion(");
        /* #01 WORD  : wordspace         OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x051B :
        strcpy(buffer_line,"FMReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x061B :
        strcpy(buffer_line,"FMStatus(");
        /* #01 WORD  : wordspace         OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x091B :
        strcpy(buffer_line,"CountFamilies(");
        nb_param = 2;
        /* #01 WORD  : wordspace         OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD  : famSpecs          IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0A1B :
        strcpy(buffer_line,"FindFamily(");
        nb_param = 4;
        /* #01 WORD   : wordspace        OUT +8 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD  : famSpecs          IN  +6 */
        GetGsOsCallOneParameter(2,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : positionNum       IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : namePtr          IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(4,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0B1B :
        strcpy(buffer_line,"GetFamInfo(");
        nb_param = 3;
        /* #01 WORD   : wordspace        OUT +6 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : famNum           IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : namePtr          IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(3,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0C1B :
        strcpy(buffer_line,"GetFamNum(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : namePtr          IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0D1B :
        strcpy(buffer_line,"AddFamily(");
        nb_param = 2;
        /* #01 WORD   : famNum           IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : namePtr          IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0E1B :
        strcpy(buffer_line,"InstallFont(");
        nb_param = 2;
        /* #01 DWORD  : desiredID        IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : scaleWord        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0F1B :
        strcpy(buffer_line,"SetPurgeStat(");
        nb_param = 2;
        /* #02 DWORD  : fontID           IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : purgeStat        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x101B :
        strcpy(buffer_line,"CountFonts(");
        nb_param = 3;
        /* #01 WORD   : wordspace        OUT +6 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : desiredID        IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : fontSpecs        IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x111B :
        strcpy(buffer_line,"FindFontStats(");
        nb_param = 4;
        /* #01 DWORD  : desiredID        IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : fontSpecs        IN  +6 */
        GetGsOsCallOneParameter(2,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : positionNum      IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : resultPtr        IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x121B :
        strcpy(buffer_line,"LoadFont(");
        nb_param = 4;
        /* #01 DWORD  : desiredID        IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : fontSpecs        IN  +6 */
        GetGsOsCallOneParameter(2,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : positionNum      IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : resultPtr        IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x131B :
        strcpy(buffer_line,"LoadSysFont(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x141B :
        strcpy(buffer_line,"AddFontVar(");
        nb_param = 2;
        /* #01 DWORD  : fontHandle       IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : newSpecs         IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x151B :
        strcpy(buffer_line,"FixFontMenu(");
        nb_param = 3;
        /* #01 WORD  : menuID            IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : startingID        IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : famSpecs          IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x161B :
        strcpy(buffer_line,"ChooseFont(");
        nb_param = 3;
        /* #01 DWORD  : longspace        OUT +6 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : currentID        IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : famSpecs         IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x171B :
        strcpy(buffer_line,"ItemID2FamNum(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #03 WORD   : itemID           IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x181B :
        strcpy(buffer_line,"FMSetSysFont(");
        /* #01 DWORD  : fontID           IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x191B :
        strcpy(buffer_line,"FMGetSysFID(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x1A1B :
        strcpy(buffer_line,"FMGetCurFID(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x1B1B :
        strcpy(buffer_line,"FamNum2ItemID(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD  : famNum            IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1C1B :
        strcpy(buffer_line,"InstallWithStats(");
        nb_param = 3;
        /* #01 DWORD  : desiredID        IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : scaleWord        IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : resultPtr        IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;


      /*******************************/
      /*  Tools 0x1C : List Manager  */
      /*******************************/
      case 0x011C :
        strcpy(buffer_line,"ListBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x021C :
        strcpy(buffer_line,"ListStartup(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x031C :
        strcpy(buffer_line,"ListShutDown(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x041C :
        strcpy(buffer_line,"ListVersion(");
        /* #01 WORD  : wordspace         OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x051C :
        strcpy(buffer_line,"ListReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x061C :
        strcpy(buffer_line,"ListStatus(");
        /* #01 WORD  : wordspace         OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x091C :
        strcpy(buffer_line,"CreateList(");
        nb_param = 3;
        /* #01 DWORD  : longspace        OUT +8 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : theWindowPtr     IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : listRecPtr       IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0A1C :
        strcpy(buffer_line,"SortList(");
        nb_param = 2;
        /* #01 DWORD  : comparePtr       IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : listRecPtr       IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0B1C :
        strcpy(buffer_line,"NextMember(");
        nb_param = 3;
        /* #01 DWORD  : longspace        OUT +8 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : memberPtr        IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : listRecPtr       IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0C1C :
        strcpy(buffer_line,"DrawMember(");
        nb_param = 2;
        /* #01 DWORD  : memberPtr        IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : listRecPtr       IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0D1C :
        strcpy(buffer_line,"SelectMember(");
        nb_param = 2;
        /* #01 DWORD  : memberPtr        IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : listRecPtr       IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0E1C :
        strcpy(buffer_line,"GetListDefProc(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0F1C :
        strcpy(buffer_line,"ResetMember(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : listRecPtr       IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x101C :
        strcpy(buffer_line,"NewList(");
        nb_param = 2;
        /* #01 DWORD  : memberPtr        IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : listRecPtr       IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x111C :
        strcpy(buffer_line,"DrawMember2(");
        nb_param = 2;
        /* #01 WORD   : itemNumber       IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : ctlHandle        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x121C :
        strcpy(buffer_line,"NextMember2(");
        nb_param = 3;
        /* #01 WORD   : wordspace        OUT +6 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : itemNumber       IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : ctlHandle        IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x131C :
        strcpy(buffer_line,"ResetMember2(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : ctlHandle        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x141C :
        strcpy(buffer_line,"SelectMember2(");
        nb_param = 2;
        /* #01 WORD   : itemNumber       IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : ctlHandle        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x151C :
        strcpy(buffer_line,"SortList2(");
        nb_param = 2;
        /* #01 DWORD  : comparePtr       IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : ctlHandle        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x161C :
        strcpy(buffer_line,"NewList2(");
        nb_param = 6;
        /* #01 DWORD  : drawProcPtr      IN  +14 */
        GetGsOsCallOneParameter(1,nb_param,param_address+14,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : listStart        IN  +12 */
        GetGsOsCallOneParameter(2,nb_param,param_address+12,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : listRef          IN  +8 */
        GetGsOsCallOneParameter(3,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : listRefDesc      IN  +6 */
        GetGsOsCallOneParameter(4,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #05 WORD   : listSize         IN  +4 */
        GetGsOsCallOneParameter(5,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #06 DWORD  : ctlHandle        IN  +0 */
        GetGsOsCallOneParameter(6,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x171C :
        strcpy(buffer_line,"ListKey(");
        nb_param = 3;
        /* #01 WORD   : flags            IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : theEventRec      IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : listCtlHndl      IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x181C :
        strcpy(buffer_line,"CompareStrings(");
        nb_param = 4;
        /* #01 WORD   : wordspace        OUT +10 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : flags            IN  +8 */
        GetGsOsCallOneParameter(2,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : String1          IN  +4 */
        string_address = GetMemoryValue(param_address+4,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(3,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : String2          IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(4,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+10,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;


      /**********************/
      /*  Tools 0x1D : ACE  */
      /**********************/
      case 0x011D :
        strcpy(buffer_line,"ACEBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x021D :
        strcpy(buffer_line,"ACEStartUp(");
        /* #01 WORD  : dPageAddr        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x031D :
        strcpy(buffer_line,"ACEShutDown(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x041D :
        strcpy(buffer_line,"ACEVersion(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x051D :
        strcpy(buffer_line,"ACEReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x061D :
        strcpy(buffer_line,"ACEStatus(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x071D :
        strcpy(buffer_line,"ACEInfo(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : infoItemCode     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;
    
      case 0x091D :
        strcpy(buffer_line,"ACECompress(");
        nb_param = 6;
        /* #01 DWORD  : src              IN  +16 */
        GetGsOsCallOneParameter(1,nb_param,param_address+16,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : srcOffset        IN  +12 */
        GetGsOsCallOneParameter(2,nb_param,param_address+12,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : dest             IN  +8 */
        GetGsOsCallOneParameter(3,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : destOffset       IN  +4 */
        GetGsOsCallOneParameter(4,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #05 WORD   : nBlks            IN  +2 */
        GetGsOsCallOneParameter(5,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #06 WORD   : method           IN  +0 */
        GetGsOsCallOneParameter(6,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0A1D :
        strcpy(buffer_line,"ACEEXpand(");
        nb_param = 6;
        /* #01 DWORD  : src              IN  +16 */
        GetGsOsCallOneParameter(1,nb_param,param_address+16,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : srcOffset        IN  +12 */
        GetGsOsCallOneParameter(2,nb_param,param_address+12,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : dest             IN  +8 */
        GetGsOsCallOneParameter(3,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : destOffset       IN  +4 */
        GetGsOsCallOneParameter(4,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #05 WORD   : nBlks            IN  +2 */
        GetGsOsCallOneParameter(5,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #06 WORD   : method           IN  +0 */
        GetGsOsCallOneParameter(6,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0B1D :
        strcpy(buffer_line,"ACECompBegin(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0C1D :
        strcpy(buffer_line,"ACEExpBegin(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0D1D :
        strcpy(buffer_line,"GetACEExpState(");
        /* #01 DWORD  : theBuffer        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0E1D :
        strcpy(buffer_line,"SetACEExpState(");
        /* #01 DWORD  : theBuffer        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;


      /***************************/
      /*  Tools 0x1E : Resource  */
      /***************************/
      case 0x011E :
        strcpy(buffer_line,"ResourceBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x021E :
        strcpy(buffer_line,"ResourceStartUp(");
        /* #01 WORD  : userID            IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x031E :
        strcpy(buffer_line,"ResourceShutDown(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x041E :
        strcpy(buffer_line,"ResourceVersion(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x051E :
        strcpy(buffer_line,"ResourceReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x061E :
        strcpy(buffer_line,"ResourceStatus(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x091E :
        strcpy(buffer_line,"CreateResourceFile(");
        nb_param = 4;
        /* #01 DWORD  : auxType          IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : fileType         IN  +6 */
        GetGsOsCallOneParameter(2,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : fileAccess       IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : fileName         IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(4,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0A1E :
        strcpy(buffer_line,"OpenResourceFile(");
        nb_param = 4;
        /* #01 WORD   : wordspace        IN  +10 */
        GetGsOsCallOneParameter(1,nb_param,param_address+10,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : openAccess       IN  +8 */
        GetGsOsCallOneParameter(2,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : resourceMapPtr   IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : fileName         IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(4,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0B1E :
        strcpy(buffer_line,"CloseResourceFile(");
        /* #01 WORD  : fileID            IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0C1E :
        strcpy(buffer_line,"AddResource(");
        nb_param = 4;
        /* #01 DWORD  : resourceHandle   IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : resourceAttr     IN  +6 */
        GetGsOsCallOneParameter(2,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : resourceType     IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : resourceID       IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0D1E :
        strcpy(buffer_line,"UpdateResourceFile(");
        /* #01 WORD   : fileID           IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0E1E :
        strcpy(buffer_line,"LoadResource(");
        nb_param = 3;
        /* #01 DWORD  : longspace        OUT +6 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : resourceType     IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : resourceID       IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0F1E :
        strcpy(buffer_line,"RemoveResource(");
        nb_param = 2;
        /* #01 WORD   : resourceType     IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : resourceID       IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x101E :
        strcpy(buffer_line,"MarkResourceChange(");
        nb_param = 3;
        /* #01 WORD   : changeFlag       IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : resourceType     IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : resourceID       IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x111E :
        strcpy(buffer_line,"SetCurResourceFile(");
        /* #01 WORD   : fileID           IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x121E :
        strcpy(buffer_line,"GetCurResourceFile(");
        /* #01 WORD  : wordspace         OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x131E :
        strcpy(buffer_line,"SetCurResourceApp(");
        /* #01 WORD   : userID           IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x141E :
        strcpy(buffer_line,"GetCurResourceApp(");
        /* #01 WORD  : wordspace         OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x151E :
        strcpy(buffer_line,"HomeResourceFile(");
        nb_param = 3;
        /* #01 WORD   : wordspace        OUT +6 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : resourceType     IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : resourceID       IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x161E :
        strcpy(buffer_line,"WriteResource(");
        nb_param = 2;
        /* #01 WORD   : resourceType     IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : resourceID       IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x171E :
        strcpy(buffer_line,"ReleaseResource(");
        nb_param = 3;
        /* #01 WORD   : purgeLevel       IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : resourceType     IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : resourceID       IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x181E :
        strcpy(buffer_line,"DetachResource(");
        nb_param = 2;
        /* #01 WORD   : resourceType     IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : resourceID       IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x191E :
        strcpy(buffer_line,"UniqueResourceID(");
        nb_param = 3;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : IDRange          IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : resourceType     IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x1A1E :
        strcpy(buffer_line,"SetResourceID(");
        nb_param = 3;
        /* #01 DWORD  : newID            IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : resourceType     IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : currentID        IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1B1E :
        strcpy(buffer_line,"GetResourceAttr(");
        nb_param = 3;
        /* #01 WORD   : wordspace        OUT +6 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : resourceType     IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : resourceID       IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1C1E :
        strcpy(buffer_line,"SetResourceAttr(");
        nb_param = 3;
        /* #01 WORD   : resourceAttr     IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : resourceType     IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : resourceID       IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1D1E :
        strcpy(buffer_line,"GetResourceSize(");
        nb_param = 3;
        /* #01 DWORD  : longspace        OUT +6 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : resourceType     IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : resourceID       IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x1E1E :
        strcpy(buffer_line,"MatchResourceHandle(");
        nb_param = 2;
        /* #01 DWORD  : foundRec         IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : resourceHandle   IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1F1E :
        strcpy(buffer_line,"GetOpenFileRefNum(");
        nb_param = 2;
        /* #01 WORD  : wordspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD  : fileID           IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x201E :
        strcpy(buffer_line,"CountTypes(");
        /* #01 WORD  : wordspace         OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x211E :
        strcpy(buffer_line,"GetIndType(");
        nb_param = 2;
        /* #01 WORD  : wordspace         OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD  : typeIndex         IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x221E :
        strcpy(buffer_line,"CountResources(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : resourceType     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x231E :
        strcpy(buffer_line,"GetIndResource(");
        nb_param = 3;
        /* #01 DWORD  : longspace        OUT +6 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : resourceType     IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : resourceIndex    IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x241E :
        strcpy(buffer_line,"SetResourceLoad(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : readFlag         IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x251E :
        strcpy(buffer_line,"SetResourceFileDepth(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : searchDepth      IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x261E :
        strcpy(buffer_line,"GetMapHandle(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : fileID           IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x271E :
        strcpy(buffer_line,"LoadAbsResource(");
        nb_param = 5;
        /* #01 DWORD  : longspace        OUT +14 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : loadAddress      IN  +10 */
        GetGsOsCallOneParameter(2,nb_param,param_address+10,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : maxSize          IN  +6 */
        GetGsOsCallOneParameter(3,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : resourceType     IN  +4 */
        GetGsOsCallOneParameter(4,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #05 DWORD  : resourceID       IN  +0 */
        GetGsOsCallOneParameter(5,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+14,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x281E :
        strcpy(buffer_line,"ResourceConverter(");
        nb_param = 3;
        /* #01 DWORD  : converterProc    IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : resourceType     IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : logFlags         IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x291E :
        strcpy(buffer_line,"LoadResource2(");
        nb_param = 5;
        /* #01 DWORD  : longspace        OUT +12 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : flags            IN  +10 */
        GetGsOsCallOneParameter(2,nb_param,param_address+10,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : bufferPtr        IN  +6 */
        GetGsOsCallOneParameter(3,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : resourceType     IN  +4 */
        GetGsOsCallOneParameter(4,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #05 DWORD  : resourceID       IN  +0 */
        GetGsOsCallOneParameter(5,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+12,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x2A1E :
        strcpy(buffer_line,"RMFindNamedResource(");
        nb_param = 4;
        /* #01 DWORD  : longspace        OUT +10 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : rType            IN  +8 */
        GetGsOsCallOneParameter(2,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : namePtr          IN  +4 */
        string_address = GetMemoryValue(param_address+4,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(3,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : fileNumPtr       IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+10,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x2B1E :
        strcpy(buffer_line,"RMGetResourceName(");
        nb_param = 3;
        /* #01 WORD   : rType            IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : rID              IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : namePtr          IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(3,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2C1E :
        strcpy(buffer_line,"RMLoadNamedResource(");
        nb_param = 3;
        /* #01 DWORD  : longspace        OUT +6 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : rType            IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : namePtr          IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(3,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x2D1E :
        strcpy(buffer_line,"RMSetResourceName(");
        nb_param = 3;
        /* #01 WORD   : rType            IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : rID              IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : namePtr          IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(3,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2E1E :
        strcpy(buffer_line,"OpenResourceFileByID(");
        nb_param = 3;
        /* #01 WORD  : wordspace         IN  +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD  : openAccess        IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : userID            IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2F1E :
        strcpy(buffer_line,"CompactResourceFile(");
        nb_param = 2;
        /* #01 WORD  : flags             IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : fileID            IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;


      /***********************/
      /*  Tools 0x20 : MIDI  */
      /***********************/
      case 0x0120 :
        strcpy(buffer_line,"MidiBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0220 :
        strcpy(buffer_line,"MidiStartUp(");
        nb_param = 2;
        /* #01 WORD  : userID           IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : dPageAddr        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0320 :
        strcpy(buffer_line,"MidiShutDown(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0420 :
        strcpy(buffer_line,"MidiVersion(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0520 :
        strcpy(buffer_line,"MidiReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0620 :
        strcpy(buffer_line,"MidiStatus(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0920 :
        strcpy(buffer_line,"MIDIControl(");
        nb_param = 2;
        /* #01 WORD   : funcNum         IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : arg             IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0A20 :
        strcpy(buffer_line,"MIDIDevice(");
        nb_param = 2;
        /* #01 WORD   : funcNum         IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : driverInfo      IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0B20 :
        strcpy(buffer_line,"MIDIClock(");
        nb_param = 2;
        /* #01 WORD   : funcNum         IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : arg             IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0C20 :
        strcpy(buffer_line,"MIDIInfo(");
        nb_param = 2;
        /* #01 DWORD  : longspace       OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : funcNum         IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0D20 :
        strcpy(buffer_line,"MIDIReadPacket(");
        nb_param = 3;
        /* #01 WORD   : wordspace       OUT +6 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : arrayAddr       IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : arraySize       IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0E20 :
        strcpy(buffer_line,"MIDIWritePacket(");
        nb_param = 2;
        /* #01 WORD   : wordspace       OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : arrayAddr       IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;


      /********************************/
      /*  Tools 0x21 : Video Overlay  */
      /********************************/
      case 0x0121 :
        strcpy(buffer_line,"VDBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0221 :
        strcpy(buffer_line,"VDStartUp(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0321 :
        strcpy(buffer_line,"VDShutDown(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0421 :
        strcpy(buffer_line,"VDVersion(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0521 :
        strcpy(buffer_line,"VDReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0621 :
        strcpy(buffer_line,"VDStatus(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0921 :
        strcpy(buffer_line,"VDInStatus(");
        nb_param = 2;
        /* #01 WORD  : wordspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD  : Selector         IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0A21 :
        strcpy(buffer_line,"VDInSetStd(");
        /* #01 WORD  : InStandard       IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0B21 :
        strcpy(buffer_line,"VDInGetStd(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0C21 :
        strcpy(buffer_line,"VDInConvAdj(");
        nb_param = 2;
        /* #01 WORD  : Selector         IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : AdjFunction      IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0D21 :
        strcpy(buffer_line,"VDKeyControl(");
        nb_param = 2;
        /* #01 WORD  : Selector         IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : KeyerCtrlVal     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0E21 :
        strcpy(buffer_line,"VDKeyStatus(");
        nb_param = 2;
        /* #01 WORD  : wordspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD  : Selector         IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0F21 :
        strcpy(buffer_line,"VDKeySetKCol(");
        nb_param = 3;
        /* #01 WORD  : RedValue         IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : GreenValue       IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : BlueValue        IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1021 :
        strcpy(buffer_line,"VDKeyGetKRCol(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1121 :
        strcpy(buffer_line,"VDKeyGetKBCol(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1221 :
        strcpy(buffer_line,"VDKeyGetKBCol(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1321 :
        strcpy(buffer_line,"VDKeySetKDiss(");
        /* #01 WORD  : KDissolve        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1421 :
        strcpy(buffer_line,"VDKeyGetKDiss(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1521 :
        strcpy(buffer_line,"VDKeySetNKDiss(");
        /* #01 WORD  : NKDissolve       IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1621 :
        strcpy(buffer_line,"VDKeyGetNKDiss(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1721 :
        strcpy(buffer_line,"VDOutSetStd(");
        /* #01 WORD  : OutStandard      IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1821 :
        strcpy(buffer_line,"VDOutGetStd(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1921 :
        strcpy(buffer_line,"VDOutControl(");
        nb_param = 2;
        /* #01 WORD  : Selector         IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : OutCtrlValue     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1A21 :
        strcpy(buffer_line,"VDOutStatus(");
        nb_param = 2;
        /* #01 WORD  : wordspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD  : Selector         IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1B21 :
        strcpy(buffer_line,"VDGetFeatures(");
        nb_param = 2;
        /* #01 WORD  : wordspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD  : Selector         IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1C21 :
        strcpy(buffer_line,"VDInControl(");
        nb_param = 2;
        /* #01 WORD  : Selector         IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : InCtrlValue      IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1D21 :
        strcpy(buffer_line,"VDGGControl(");
        nb_param = 2;
        /* #01 WORD  : Selector         IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : GGCtrlValue      IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1E21 :
        strcpy(buffer_line,"VDGGStatus(");
        nb_param = 2;
        /* #01 WORD  : wordspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD  : Selector         IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;


      /***************************/
      /*  Tools 0x22 : TextEdit  */
      /***************************/
      case 0x0122 :
        strcpy(buffer_line,"TEBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0222 :
        strcpy(buffer_line,"TEStartUp(");
        nb_param = 2;
        /* #01 WORD  : userID           IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : directPage       IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0322 :
        strcpy(buffer_line,"TEShutDown(");

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0422 :
        strcpy(buffer_line,"TEVersion(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0522 :
        strcpy(buffer_line,"TEReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0622 :
        strcpy(buffer_line,"TEStatus(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0922 :
        strcpy(buffer_line,"TENew(");
        nb_param = 2;
        /* #01 DWORD  : longspace       OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD  : parameterBlock   IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0A22 :
        strcpy(buffer_line,"TEKill(");
        /* #01 DWORD  : teH          IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0B22 :
        strcpy(buffer_line,"TESetText(");
        nb_param = 6;
        /* #01 WORD  : textDescriptor   IN  +18 */
        GetGsOsCallOneParameter(1,nb_param,param_address+18,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : textRef         IN  +14 */
        GetGsOsCallOneParameter(2,nb_param,param_address+14,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : textLength      IN  +10 */
        GetGsOsCallOneParameter(3,nb_param,param_address+10,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD  : styleDescriptor  IN  +8 */
        GetGsOsCallOneParameter(4,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #05 DWORD  : styleRef        IN  +4 */
        GetGsOsCallOneParameter(5,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #06 DWORD  : teH             IN  +0 */
        GetGsOsCallOneParameter(6,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0C22 :
        strcpy(buffer_line,"TEGetText(");
        nb_param = 7;
        /* #01 DWORD  : longspace       OUT +20 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : commonStylePtr  IN  +18 */
        GetGsOsCallOneParameter(2,nb_param,param_address+18,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : commonStylePtr  IN  +14 */
        GetGsOsCallOneParameter(3,nb_param,param_address+14,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : styleHandle     IN  +10 */
        GetGsOsCallOneParameter(4,nb_param,param_address+10,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #05 WORD   : commonStylePtr  IN  +8 */
        GetGsOsCallOneParameter(5,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #06 DWORD  : styleHandle     IN  +4 */
        GetGsOsCallOneParameter(6,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #07 DWORD  : teH             IN  +0 */
        GetGsOsCallOneParameter(7,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+20,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0D22 :
        strcpy(buffer_line,"TEGetTextInfo(");
        nb_param = 3;
        /* #01 DWORD  : infoRecPtr      IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : parameterCount   IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : teH             IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0E22 :
        strcpy(buffer_line,"TEIdle(");
        /* #01 DWORD  : teH          IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0F22 :
        strcpy(buffer_line,"TEActivate(");
        /* #01 DWORD  : teH          IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1022 :
        strcpy(buffer_line,"TEDeactivate(");
        /* #01 DWORD  : teH          IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1122 :
        strcpy(buffer_line,"TEClick(");
        nb_param = 2;
        /* #01 DWORD  : eventRecordPtr   IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : teH              IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1222 :
        strcpy(buffer_line,"TEUpdate(");
        /* #01 DWORD  : teH          IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1322 :
        strcpy(buffer_line,"TEPaintText(");
        nb_param = 6;
        /* #01 DWORD  : longspace       OUT +18 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : grafPort        IN  +14 */
        GetGsOsCallOneParameter(2,nb_param,param_address+14,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : startingLine    IN  +10 */
        GetGsOsCallOneParameter(3,nb_param,param_address+10,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : rectPtr         IN  +6 */
        GetGsOsCallOneParameter(4,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #05 WORD  : flags            IN  +4 */
        GetGsOsCallOneParameter(5,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #06 DWORD  : teH             IN  +0 */
        GetGsOsCallOneParameter(6,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+18,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x1422 :
        strcpy(buffer_line,"TEKey(");
        nb_param = 2;
        /* #01 DWORD  : eventRecordPtr   IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : teH              IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1522 :
        strcpy(buffer_line,"TEUndo(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1622 :
        strcpy(buffer_line,"TECut(");
        /* #01 DWORD  : teH          IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1722 :
        strcpy(buffer_line,"TECopy(");
        /* #01 DWORD  : teH          IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1822 :
        strcpy(buffer_line,"TEPaste(");
        /* #01 DWORD  : teH          IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1922 :
        strcpy(buffer_line,"TEClear(");
        /* #01 DWORD  : teH          IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1A22 :
        strcpy(buffer_line,"TEInsert(");
        nb_param = 6;
        /* #01 WORD  : textDescriptor   IN  +18 */
        GetGsOsCallOneParameter(1,nb_param,param_address+18,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : textRef         IN  +14 */
        GetGsOsCallOneParameter(2,nb_param,param_address+14,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : textLength      IN  +10 */
        GetGsOsCallOneParameter(3,nb_param,param_address+10,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD  : styleDescriptor  IN  +8 */
        GetGsOsCallOneParameter(4,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #05 DWORD  : styleRef        IN  +4 */
        GetGsOsCallOneParameter(5,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #06 DWORD  : teH             IN  +0 */
        GetGsOsCallOneParameter(6,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1B22 :
        strcpy(buffer_line,"TEReplace(");
        nb_param = 6;
        /* #01 WORD  : textDescriptor   IN  +18 */
        GetGsOsCallOneParameter(1,nb_param,param_address+18,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : textRef         IN  +14 */
        GetGsOsCallOneParameter(2,nb_param,param_address+14,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : textLength      IN  +10 */
        GetGsOsCallOneParameter(3,nb_param,param_address+10,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD  : styleDescriptor  IN  +8 */
        GetGsOsCallOneParameter(4,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #05 DWORD  : styleRef        IN  +4 */
        GetGsOsCallOneParameter(5,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #06 DWORD  : teH             IN  +0 */
        GetGsOsCallOneParameter(6,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1C22 :
        strcpy(buffer_line,"TEGetSelection(");
        nb_param = 3;
        /* #01 DWORD  : selectionStart  IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : selectionEnd    IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : teH             IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1D22 :
        strcpy(buffer_line,"TESetSelection(");
        nb_param = 3;
        /* #01 DWORD  : selectionStart  IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : selectionEnd    IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : teH             IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1E22 :
        strcpy(buffer_line,"TEGetSelectionStyle(");
        nb_param = 4;
        /* #01 WORD   : wordspace       OUT +10 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : commonStylePtr  IN  +8 */
        GetGsOsCallOneParameter(2,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : styleHandle     IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : teH             IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+10,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1F22 :
        strcpy(buffer_line,"TEStyleChange(");
        nb_param = 3;
        /* #01 WORD   : flags           IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : stylePtr        IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : teH             IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2022 :
        strcpy(buffer_line,"TEOffsetToPoint(");
        nb_param = 4;
        /* #01 DWORD  : textOffset      IN  +12 */
        GetGsOsCallOneParameter(1,nb_param,param_address+12,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : vertPosPtr      IN  +8 */
        GetGsOsCallOneParameter(2,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : horzPosPtr      IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : teH             IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2122 :
        strcpy(buffer_line,"TEPointToOffset(");
        nb_param = 4;
        /* #01 DWORD  : longspace       OUT +12 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : vertPos         IN  +8 */
        GetGsOsCallOneParameter(2,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : horzPos         IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : teH             IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+12,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x2222 :
        strcpy(buffer_line,"TEGetDefProc(");
        /* #01 DWORD  : longspace       OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x2322 :
        strcpy(buffer_line,"TEGetRuler(");
        nb_param = 3;
        /* #01 WORD   : rulerDescriptor IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : rulerRef        IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : teH             IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2422 :
        strcpy(buffer_line,"TESetRuler(");
        nb_param = 3;
        /* #01 WORD   : rulerDescriptor IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : rulerRef        IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : teH             IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2522 :
        strcpy(buffer_line,"TEScroll(");
        nb_param = 4;
        /* #01 WORD   : scrollDescriptor IN  +12 */
        GetGsOsCallOneParameter(1,nb_param,param_address+12,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : vertAmount       IN  +8 */
        GetGsOsCallOneParameter(2,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : horzAmount       IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : teH              IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2622 :
        strcpy(buffer_line,"TEGetInternalProc(");
        /* #01 DWORD  : longspace       OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x2722 :
        strcpy(buffer_line,"TEGetLastError(");
        nb_param = 3;
        /* #01 WORD   : wordspace       OUT +6 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : clearFlag       IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : teH             IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x2822 :
        strcpy(buffer_line,"TECompactRecord(");
        /* #01 DWORD  : teH          IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;


      /*****************************/
      /*  Tools 0x23 : Midi Synth  */
      /*****************************/
      case 0x0123 :
        strcpy(buffer_line,"MSBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0223 :
        strcpy(buffer_line,"MSStartUp(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0323 :
        strcpy(buffer_line,"MSShutDown(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0423 :
        strcpy(buffer_line,"MSVersion(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0523 :
        strcpy(buffer_line,"MSReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0623 :
        strcpy(buffer_line,"MSStatus(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0923 :
        strcpy(buffer_line,"SetBasicChannel(");
        /* #01 WORD  : channelNo        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0A23 :
        strcpy(buffer_line,"SetMIDIMode(");
        /* #01 WORD  : MIDImode         IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0B23 :
        strcpy(buffer_line,"PlayNote(");
        nb_param = 3;
        /* #01 WORD  : channelNo        IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : MIDInote         IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : keyVelocity      IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0C23 :
        strcpy(buffer_line,"StopNote(");
        nb_param = 2;
        /* #01 WORD  : channelNo        IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : MIDInote         IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
 
        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);        
        break;

      case 0x0D23 :
        strcpy(buffer_line,"KillAllNotes(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0E23 :
        strcpy(buffer_line,"SetRecTrack(");
        /* #01 WORD  : recTrack         IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0F23 :
        strcpy(buffer_line,"SetPlayTrack(");
        nb_param = 2;
        /* #01 WORD  : trackNo          IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : trackPlayState   IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1023 :
        strcpy(buffer_line,"TrackToChannel(");
        nb_param = 2;
        /* #01 WORD  : trackNo          IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : channelNo        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1123 :
        strcpy(buffer_line,"Locate(");
        nb_param = 3;
        /* #01 DWORD  : longspace       OUT +8 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : timestamp       IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : seqBuffPtr      IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x1223 :
        strcpy(buffer_line,"SetVelComp(");
        /* #01 WORD  : velOffset        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
 
        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);        
        break;

      case 0x1323 :
        strcpy(buffer_line,"SetMIDIPort(");
        nb_param = 2;
        /* #01 WORD  : MIDIin           IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : MIDIout          IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1423 :
        strcpy(buffer_line,"SetInstrument(");
        nb_param = 2;
        /* #01 DWORD  : instRecPtr      IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : instNo          IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1523 :
        strcpy(buffer_line,"SeqPlayer(");
        /* #01 DWORD  : seqPlayerRec    IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1623 :
        strcpy(buffer_line,"SetTempo(");
        /* #01 WORD  : tempo           IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1723 :
        strcpy(buffer_line,"SetCallBack(");
        /* #01 DWORD  : longspace       OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x1823 :
        strcpy(buffer_line,"SysExOut(");
        nb_param = 3;
        /* #01 DWORD  : compMsgPtr      IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : delay           IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : monitorPtr      IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1923 :
        strcpy(buffer_line,"SetBeat(");
        /* #01 WORD  : beat            IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1A23 :
        strcpy(buffer_line,"MIDIMessage(");
        nb_param = 5;
        /* #01 WORD  : destination      IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : validDataNo      IN  +6 */
        GetGsOsCallOneParameter(2,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : MIDIStatusMsg    IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD  : dataByte1        IN  +2 */
        GetGsOsCallOneParameter(4,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #05 WORD  : dataByte2        IN  +0 */
        GetGsOsCallOneParameter(5,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1B23 :
        strcpy(buffer_line,"LocateEnd(");
        nb_param = 2;
        /* #01 DWORD  : longspace       OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : seqPtr          IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x1C23 :
        strcpy(buffer_line,"Merge(");
        nb_param = 2;
        /* #01 DWORD  : buffer1Ptr      IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : buffer2Ptr      IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1D23 :
        strcpy(buffer_line,"DeleteTrack(");
        nb_param = 2;
        /* #01 WORD   : trackNo         IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : seqPtr          IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1E23 :
        strcpy(buffer_line,"SetMetro(");
        nb_param = 3;
        /* #01 WORD   : metronomeValue  IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : metroFrequency  IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : metroWavePtr    IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1F23 :
        strcpy(buffer_line,"GetMSData(");
        nb_param = 2;
        /* #01 DWORD  : longspace       OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : longspace       OUT +0 */
        GetEmptyParameter(2,nb_param,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x2023 :
        strcpy(buffer_line,"ConvertToTime(");
        nb_param = 5;
        /* #01 DWORD  : longspace       OUT +8 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : ticksPerBeat    IN  +6 */
        GetGsOsCallOneParameter(2,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : beatsPerMeasure IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : beatNumber      IN  +2 */
        GetGsOsCallOneParameter(4,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #05 WORD   : measureNumber   IN  +0 */
        GetGsOsCallOneParameter(5,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x2123 :
        strcpy(buffer_line,"ConvertToMeasure(");
        nb_param = 6;
        /* #01 WORD  : wordspace        OUT +12 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD  : wordspace        OUT +10 */
        GetEmptyParameter(2,nb_param,buffer_line);
        /* #03 WORD  : wordspace        OUT +8 */
        GetEmptyParameter(3,nb_param,buffer_line);
        /* #04 WORD   : ticksPerBeat    IN  +6 */
        GetGsOsCallOneParameter(4,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #05 WORD   : beatsPerMeasure IN  +4 */
        GetGsOsCallOneParameter(5,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #06 DWORD  : seqClockTicks   IN  +0 */
        GetGsOsCallOneParameter(6,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+12,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,2,nb_param,param_address+10,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,3,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x2223 :
        strcpy(buffer_line,"MSSuspend(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2323 :
        strcpy(buffer_line,"MSResume(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2423 :
        strcpy(buffer_line,"SetTuningTable(");
        /* #01 DWORD  : tuningTblPtr   IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2523 :
        strcpy(buffer_line,"GetTuningTable(");
        /* #01 DWORD  : tuningTblPtr   IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2623 :
        strcpy(buffer_line,"SetTrackOut(");
        nb_param = 2;
        /* #01 WORD  : trackNo          IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : pathValue        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2723 :
        strcpy(buffer_line,"InitMIDIDriver(");
        nb_param = 4;
        /* #01 WORD  : slot            IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : Int/Ext         IN  +6 */
        GetGsOsCallOneParameter(2,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : UserID          IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD : MIDIDriverPtr   IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2823 :
        strcpy(buffer_line,"RemoveMIDIDriver(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;


      /********************************/
      /*  Tools 0x25 : Anim Tool Set  */
      /********************************/
      case 0x0125 :
        strcpy(buffer_line,"AnimBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0225 :
        strcpy(buffer_line,"AnimStartUp(");
        nb_param = 4;
        /* #01 WORD  : memID           IN  +10 */
        GetGsOsCallOneParameter(1,nb_param,param_address+10,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : DirectPage      IN  +8 */
        GetGsOsCallOneParameter(2,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : ModeFlag        IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD : PatchTablePtr   IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0325 :
        strcpy(buffer_line,"AnimShutDown(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0425 :
        strcpy(buffer_line,"AnimVersion(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0525 :
        strcpy(buffer_line,"AnimReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0625 :
        strcpy(buffer_line,"AnimStatus(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0725 :
        strcpy(buffer_line,"CleanExit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0825 :
        strcpy(buffer_line,"AnimIdleDebug(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0925 :
        strcpy(buffer_line,"StartScene(");
        nb_param = 2;
        /* #01 WORD  : Mode           IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : RectPTr        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0A25 :
        strcpy(buffer_line,"StopScene(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0B25 :
        strcpy(buffer_line,"StartFrameTimer(");
        /* #01 WORD  : FrameInterval           IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0C25 :
        strcpy(buffer_line,"StopFrameTimer(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0D25 :
        strcpy(buffer_line,"SetBackGndPort(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0E25 :
        strcpy(buffer_line,"RefreshBackCall(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0F25 :
        strcpy(buffer_line,"StartCharCall(");
        nb_param = 5;
        /* #01 WORD  : PassY              IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : PassX              IN  +6 */
        GetGsOsCallOneParameter(2,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : PassMaxP/PassFlag  IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD  : PassMinP/PassDir   IN  +2 */
        GetGsOsCallOneParameter(4,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #05 WORD  : LPassChar          IN  +0 */
        GetGsOsCallOneParameter(5,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1025 :
        strcpy(buffer_line,"MoveCharCall(");
        nb_param = 4;
        /* #01 WORD  : PassIter          IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : PassDY/PassFlag   IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : PassDX/PassSpeed  IN  +2 */
        GetGsOsCallOneParameter(3,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD  : LPassChar         IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1125 :
        strcpy(buffer_line,"GetCharRecPtr(");
        nb_param = 2;
        /* #01 DWORD  : longspace  OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : CharNum    IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x1225 :
        strcpy(buffer_line,"KillCharCall(");
        /* #01 WORD  : CharNum          IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1325 :
        strcpy(buffer_line,"LoadActor(");
        nb_param = 2;
        /* #01 DWORD  : ActorPtr       IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : CharNum        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1425 :
        strcpy(buffer_line,"SetCharScript(");
        nb_param = 2;
        /* #01 DWORD  : theScript      IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : CharNum        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1525 :
        strcpy(buffer_line,"RunAnimScripts(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1625 :
        strcpy(buffer_line,"FillAddressTbl(");
        /* #01 DWORD  : TablePtr      IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1725 :
        strcpy(buffer_line,"CreateReblitCode(");
        nb_param = 2;
        /* #01 DWORD  : RectPtr       IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : GenFlags      IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1825 :
        strcpy(buffer_line,"StartTockTask(");
        nb_param = 4;
        /* #01 DWORD  : thePtr        IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : theData       IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : theID         IN  +2 */
        GetGsOsCallOneParameter(3,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : theCount      IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1925 :
        strcpy(buffer_line,"FireTockTask(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;


      /***********************************/
      /*  Tools 0x26 : Media Controller  */
      /***********************************/
      case 0x0126 :
        strcpy(buffer_line,"MCBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0226 :
        strcpy(buffer_line,"MCStartUp(");
        /* #01 WORD   : userID      IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0326 :
        strcpy(buffer_line,"MCShutDown(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0426 :
        strcpy(buffer_line,"MCVersion(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0526 :
        strcpy(buffer_line,"MCReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0626 :
        strcpy(buffer_line,"MCStatus(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0926 :
        strcpy(buffer_line,"MCGetErrorMsg(");
        nb_param = 2;
        /* #01 WORD   : mcErrorNo   IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : PStrPtr     IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0A26 :
        strcpy(buffer_line,"MCLoadDriver(");
        /* #01 WORD   : mcChannelNo      IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0B26 :
        strcpy(buffer_line,"MCUnLoadDriver(");
        /* #01 WORD   : mcChannelNo      IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0C26 :
        strcpy(buffer_line,"MCTime2Bin(");
        nb_param = 2;
        /* #01 DWORD  : longspace     OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : mcTimeValue   IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0D26 :
        strcpy(buffer_line,"MCBin2Time(");
        nb_param = 2;
        /* #01 DWORD  : longspace     OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : mcBinVal      IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0E26 :
        strcpy(buffer_line,"MCGetTrackTitle(");
        nb_param = 3;
        /* #01 DWORD  : mcDiscID      IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : mcTrackNo     IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : PStrPtr       IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(3,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0F26 :
        strcpy(buffer_line,"MCSetTrackTitle(");
        nb_param = 3;
        /* #01 DWORD  : mcDiscID      IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : mcTrackNo     IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : TitlePtr      IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(3,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1026 :
        strcpy(buffer_line,"MCGetProgram(");
        nb_param = 2;
        /* #01 DWORD  : mcDiscID      IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : resultPtr     IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1126 :
        strcpy(buffer_line,"MCSetProgram(");
        nb_param = 2;
        /* #01 DWORD  : mcDiscID      IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : mcProgPtr     IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1226 :
        strcpy(buffer_line,"MCGetDiscTitle(");
        nb_param = 2;
        /* #01 DWORD  : mcDiscID      IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : PStrPtr       IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1326 :
        strcpy(buffer_line,"MCSetDiscTitle(");
        nb_param = 2;
        /* #01 DWORD  : mcDiscID      IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : titleptr      IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        break;

      case 0x1426 :
        strcpy(buffer_line,"MCDStartUp(");
        nb_param = 2;
        /* #01 WORD   : mcChannelNo   IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : portnameptr   IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1526 :
        strcpy(buffer_line,"MCDShutDown(");
        /* #01 WORD   : mcChannelNo   IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1626 :
        strcpy(buffer_line,"MCGetFeatures(");
        nb_param = 3;
        /* #01 DWORD  : longspace     IN  +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : mcChannelNo   IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : mcFeatSel     IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1726 :
        strcpy(buffer_line,"MCPlay(");
        /* #01 WORD   : mcChannelNo   IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1826 :
        strcpy(buffer_line,"MCPause(");
        /* #01 WORD   : mcChannelNo   IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1926 :
        strcpy(buffer_line,"MCSendRawData(");
        nb_param = 2;
        /* #01 WORD   : mcChannelNo   IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : mcNativePtr   IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1A26 :
        strcpy(buffer_line,"MCGetStatus(");
        nb_param = 3;
        /* #01 WORD  : wordspace      OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : mcChannelNo   IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : mcStatSel     IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1B26 :
        strcpy(buffer_line,"MCControl(");
        nb_param = 2;
        /* #01 WORD  : mcChannelNo    IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : ctlcommand     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1C26 :
        strcpy(buffer_line,"MCScan(");
        nb_param = 2;
        /* #01 WORD   : mcChannelNo   IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : mcDirection   IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1D26 :
        strcpy(buffer_line,"MCGetSpeeds(");
        nb_param = 2;
        /* #01 WORD   : mcChannelNo   IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : PStrPtr       IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1E26 :
        strcpy(buffer_line,"MCSpeed(");
        nb_param = 2;
        /* #01 WORD  : mcChannelNo    IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : mcFPS          IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1F26 :
        strcpy(buffer_line,"MCStopAt(");
        nb_param = 3;
        /* #01 WORD   : mcChannelNo   IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : mcUnitType    IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : mcStopLoc     IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2026 :
        strcpy(buffer_line,"MCJog(");
        nb_param = 4;
        /* #01 WORD   : mcChannelNo   IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : mcUnitType    IN  +6 */
        GetGsOsCallOneParameter(2,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : mcNJog        IN  +2 */
        GetGsOsCallOneParameter(3,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : mcJobRepeat   IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2126 :
        strcpy(buffer_line,"MCSearchTo(");
        nb_param = 3;
        /* #01 WORD   : mcChannelNo   IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : mcUnitType    IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : searchLoc     IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2226 :
        strcpy(buffer_line,"MCSearchDone(");
        nb_param = 2;
        /* #01 WORD   : wordspace     OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : mcChannelNo   IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x2326 :
        strcpy(buffer_line,"MCSearchWait(");
        /* #01 WORD   : mcChannelNo   IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2426 :
        strcpy(buffer_line,"MCGetPosition(");
        nb_param = 3;
        /* #01 DWORD  : longspace     OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : mcChannelNo   IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : mcUnitType    IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x2526 :
        strcpy(buffer_line,"MCSetAudio(");
        nb_param = 2;
        /* #01 WORD   : mcChannelNo   IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : mcAudioCtl    IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2626 :
        strcpy(buffer_line,"MCGetTimes(");
        nb_param = 3;
        /* #01 DWORD  : longspace     OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : mcChannelNo   IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : mcTimeSel     IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x2726 :
        strcpy(buffer_line,"MCGetDiscTOC(");
        nb_param = 3;
        /* #01 DWORD  : longspace     OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : mcChannelNo   IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : mcTrackNo     IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x2826 :
        strcpy(buffer_line,"MCGetDiscID(");
        nb_param = 2;
        /* #01 DWORD  : longspace     OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : mcChannelNo   IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x2926 :
        strcpy(buffer_line,"MCGetNoTracks(");
        nb_param = 2;
        /* #01 WORD  : wordspace     OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD  : mcChannelNo   IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x2A26 :
        strcpy(buffer_line,"MCRecord(");
        /* #01 WORD   : mcChannelNo   IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2B26 :
        strcpy(buffer_line,"MCStop(");
        /* #01 WORD   : mcChannelNo   IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2C26 :
        strcpy(buffer_line,"MCWaitRawData(");
        nb_param = 4;
        /* #01 WORD   : mcChannelNo   IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : resultPtr     IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : tickWait      IN  +2 */
        GetGsOsCallOneParameter(3,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : term_mask     IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2D26 :
        strcpy(buffer_line,"MCGetName(");
        nb_param = 2;
        /* #01 WORD   : mcChannelNo   IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : PStrPtr       IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2E26 :
        strcpy(buffer_line,"MCSetVolume(");
        nb_param = 3;
        /* #01 WORD   : mcChannelNo   IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : mcLeftVol     IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : mcRightVol    IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
 
        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;


      /*****************************/
      /*  Tools 0x32 : Male Voice  */
      /*****************************/
      case 0x0132 :
        strcpy(buffer_line,"MaleBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0232 :
        strcpy(buffer_line,"MaleStartUp(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0332 :
        strcpy(buffer_line,"MaleShutDown(");
        /* No Parameters */
 
        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);        
        break;

      case 0x0432 :
        strcpy(buffer_line,"MaleVersion(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0532 :
        strcpy(buffer_line,"MaleReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0632 :
        strcpy(buffer_line,"MaleStatus(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0932 :
        strcpy(buffer_line,"MaleSpeak(");
        nb_param = 4;
        /* #01 WORD  : volume           IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : speed            IN  +6 */
        GetGsOsCallOneParameter(2,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : pitch            IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD : phonetic string  IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(4,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
 
        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);        
        break;


      /*******************************/
      /*  Tools 0x33 : Female Voice  */
      /*******************************/
      case 0x0133 :
        strcpy(buffer_line,"FemaleBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0233 :
        strcpy(buffer_line,"FemaleStartUp(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0333 :
        strcpy(buffer_line,"FemaleShutDown(");
        /* No Parameters */
 
        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);        
        break;

      case 0x0433 :
        strcpy(buffer_line,"FemaleVersion(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0533 :
        strcpy(buffer_line,"FemaleReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0633 :
        strcpy(buffer_line,"FemaleStatus(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0933 :
        strcpy(buffer_line,"FemaleSpeak(");
        nb_param = 4;
        /* #01 WORD  : volume           IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : speed            IN  +6 */
        GetGsOsCallOneParameter(2,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : pitch            IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD : phonetic string  IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(4,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;


      /*********************************/
      /*  Tools 0x34 : Speech Toolset  */
      /*********************************/
      case 0x0134 :
        strcpy(buffer_line,"ParseBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0234 :
        strcpy(buffer_line,"ParseStartUp(");
        /* #01 WORD  : userID           IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0343 :
        strcpy(buffer_line,"ParseShutDown(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0434 :
        strcpy(buffer_line,"ParseVersion(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0534 :
        strcpy(buffer_line,"ParseReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0634 :
        strcpy(buffer_line,"ParseStatus(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0934 :
        strcpy(buffer_line,"Parse(");
        nb_param = 4;
        /* #01 WORD  : wordspace        OUT +10 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD : English string   IN  +6 */
        string_address = GetMemoryValue(param_address+6,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : phonetic string  IN  +2 */
        string_address = GetMemoryValue(param_address+2,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(3,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #04 WORD  : start            IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+10,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0A34 :
        strcpy(buffer_line,"DictInsert(");
        nb_param = 2;
        /* #01 DWORD : English word     IN  +4 */
        string_address = GetMemoryValue(param_address+4,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : phonetic word    IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0B34 :
        strcpy(buffer_line,"DictDelete(");
        /* #01 DWORD : English word     IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,1,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0C34 :
        strcpy(buffer_line,"DictDump(");
        nb_param = 3;
        /* #01 DWORD : longspace        OUT +8 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD : English word     IN  +4 */
        string_address = GetMemoryValue(param_address+4,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : phonetic word    IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(3,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0D34 :
        strcpy(buffer_line,"SetSayGlobals(");
        nb_param = 5;
        /* #01 WORD  : voice       IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : tone        IN  +6 */
        GetGsOsCallOneParameter(2,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : pitch       IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD  : speed       IN  +2 */
        GetGsOsCallOneParameter(4,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #05 WORD  : volume      IN  +0 */
        GetGsOsCallOneParameter(5,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0E34 :
        strcpy(buffer_line,"DictInit(");
        /* #01 WORD  : initFlag         IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0F34 :
        strcpy(buffer_line,"Say(");
        /* #01 DWORD : English string   IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,1,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1034 :
         strcpy(buffer_line,"DictActivate(");
        /* #01 WORD  : activeFlag       IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;


      /*********************************/
      /*  Tools 0x35 : Speech Toolkit  */
      /*********************************/
      case 0x0135 :
        strcpy(buffer_line,"SpeechBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0235 :
        strcpy(buffer_line,"SpeechStartUp(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0335 :
        strcpy(buffer_line,"SpeechShutDown(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0435 :
        strcpy(buffer_line,"SpeechVersion(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0535 :
        strcpy(buffer_line,"SpeechReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0635 :
        strcpy(buffer_line,"SpeechStatus(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;


      /**********************************/
      /*  Tool 0x36 : Marinetti TCP/IP  */
      /**********************************/
      case 0x0136 :
        strcpy(buffer_line,"TCPIPBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0236 :
        strcpy(buffer_line,"TCPIPStartUp(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0336 :
        strcpy(buffer_line,"TCPIPShutDown(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0436 :
        strcpy(buffer_line,"TCPIPVersion(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0536 :
        strcpy(buffer_line,"TCPIPReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0636 :
        strcpy(buffer_line,"TCPIPStatus(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0836 :
        strcpy(buffer_line,"TCPIPLongVersion(");
        /* #01 WORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0936 :
        strcpy(buffer_line,"TCPIPGetConnectStatus(");
        /* #01 WORD   : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0A36 :
        strcpy(buffer_line,"TCPIPGetErrorTable(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0B36 :
        strcpy(buffer_line,"TCPIPGetReconnectStatus(");
        /* #01 WORD   : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0C36 :
        strcpy(buffer_line,"TCPIPReconnect(");
        /* #01 DWORD  : displayPtr       IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0D36 :
        strcpy(buffer_line,"TCPIPConvertIPToHex(");
        nb_param = 2;
        /* #01 DWORD  : cvtRecPtr        IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : ddippstring      IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0E36 :
        strcpy(buffer_line,"TCPIPConvertIPToASCII(");
        nb_param = 4;
        /* #01 WORD   : wordspace        OUT +10 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : ipaddress        IN  +6 */
        GetGsOsCallOneParameter(2,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : ddpstring        IN  +2 */
        GetGsOsCallOneParameter(3,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : flags            IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+10,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0F36 :
        strcpy(buffer_line,"TCPIPGetMyIPAddress(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x1036 :
        strcpy(buffer_line,"TCPIPGetConnectMethod(");
        /* #01 WORD   : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1136 :
        strcpy(buffer_line,"TCPIPSetConnectMethod(");
        /* #01 WORD   : IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1236 :
        strcpy(buffer_line,"TCPIPConnect(");
        /* #01 DWORD  : displayPtr       IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1336 :
        strcpy(buffer_line,"TCPIPDisconnect(");
        nb_param = 2;
        /* #01 WORD   : forceFlag        IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : displayPtr       IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1436 :
        strcpy(buffer_line,"TCPIPGetMTU(");
        /* #01 WORD   : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1536 :
        strcpy(buffer_line,"TCPIPValidateIPCString(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : toPtr            IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1636 :
        strcpy(buffer_line,"TCPIPGetConnectData(");
        nb_param = 3;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : userid           IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : method           IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x1736 :
        strcpy(buffer_line,"TCPIPSetConnectData(");
        nb_param = 2;
        /* #01 WORD   : method           IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : conHandle        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1836 :
        strcpy(buffer_line,"TCPIPGetDisconnectData(");
        nb_param = 3;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : userid           IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : method           IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x1936 :
        strcpy(buffer_line,"TCPIPSetDisconnectData(");
        nb_param = 2;
        /* #01 WORD   : method           IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : disconHandle     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1A36 :
        strcpy(buffer_line,"TCPIPLoadPreferences(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1B36 :
        strcpy(buffer_line,"TCPIPSavePreferences(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1C36 :
        strcpy(buffer_line,"TCPIPGetDNS(");
        /* #01 DWORD  : DNSRecPtr        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1D36 :
        strcpy(buffer_line,"TCPIPSetDNS(");
        /* #01 DWORD  : DNSRecPtr        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1E36 :
        strcpy(buffer_line,"TCPIPGetTuningTable(");
        /* #01 DWORD  : tunePtr          IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1F36 :
        strcpy(buffer_line,"TCPIPSetTuningTable(");
        /* #01 DWORD  : tunePtr          IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2036 :
        strcpy(buffer_line,"TCPIPCancelDNR(");
        /* #01 DWORD  : dnrBufferPtr     IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2136 :
        strcpy(buffer_line,"TCPIPDNRNameToIP(");
        nb_param = 2;
        /* #01 DWORD  : nameptr          IN  +4 */
        string_address = GetMemoryValue(param_address+4,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : dnrBufferPtr     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2236 :
        strcpy(buffer_line,"TCPIPPoll(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2336 :
        strcpy(buffer_line,"TCPIPLogin(");
        nb_param = 6;
        /* #01 WORD   : wordspace        OUT +12 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : userid           IN  +10 */
        GetGsOsCallOneParameter(2,nb_param,param_address+10,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : destip           IN  +6 */
        GetGsOsCallOneParameter(3,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : destport         IN  +4 */
        GetGsOsCallOneParameter(4,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #05 WORD   : defaultTOS       IN  +2 */
        GetGsOsCallOneParameter(5,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #06 WORD   : defaultTTL       IN  +0 */
        GetGsOsCallOneParameter(6,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+12,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x2436 :
        strcpy(buffer_line,"TCPIPLogout(");
        /* #01 WORD   : ipid              IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2536 :
        strcpy(buffer_line,"TCPIPSendICMP(");
        nb_param = 3;
        /* #01 WORD   : ipid             IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : messagePtr       IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : messageLen       IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2636 :
        strcpy(buffer_line,"TCPIPSendUDP(");
        nb_param = 3;
        /* #01 WORD   : ipid             IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : udpPtr           IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : udpLen           IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2736 :
        strcpy(buffer_line,"TCPIPGetDatagramCount(");
        nb_param = 3;
        /* #01 WORD   : wordspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : ipid             IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : protocol         IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x2836 :
        strcpy(buffer_line,"TCPIPGetNextDatagram(");
        nb_param = 4;
        /* #01 DWORD  : longspace        OUT +6 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : ipid             IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : protocol         IN  +2 */
        GetGsOsCallOneParameter(3,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : flags            IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x2936 :
        strcpy(buffer_line,"TCPIPGetLoginCount(");
        /* #01 WORD   : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x2A36 :
        strcpy(buffer_line,"TCPIPSendICMPEcho(");
        nb_param = 2;
        /* #01 WORD   : ipid             IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : seqNum           IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x2B36 :
        strcpy(buffer_line,"TCPIPReceiveICMPEcho(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : ipid             IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x2C36 :
        strcpy(buffer_line,"TCPIPOpenTCP(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : ipid             IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x2D36 :
        strcpy(buffer_line,"TCPIPWriteTCP(");
        nb_param = 6;
        /* #01 WORD   : wordspace        OUT +14 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : ipid             IN  +12 */
        GetGsOsCallOneParameter(2,nb_param,param_address+12,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : dataPtr          IN  +8 */
        GetGsOsCallOneParameter(3,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : dataLength       IN  +4 */
        GetGsOsCallOneParameter(4,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #05 WORD   : pushFlag         IN  +2 */
        GetGsOsCallOneParameter(5,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #06 WORD   : urgentFlag       IN  +0 */
        GetGsOsCallOneParameter(6,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+14,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x2E36 :
        strcpy(buffer_line,"TCPIPReadTCP(");
        nb_param = 6;
        /* #01 WORD   : wordspace        OUT +16 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : ipid             IN  +14 */
        GetGsOsCallOneParameter(2,nb_param,param_address+14,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : buffType         IN  +12 */
        GetGsOsCallOneParameter(3,nb_param,param_address+12,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : buffData         IN  +8 */
        GetGsOsCallOneParameter(4,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #05 DWORD  : buffLen          IN  +4 */
        GetGsOsCallOneParameter(5,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #06 DWORD  : rrBuffPtr        IN  +0 */
        GetGsOsCallOneParameter(6,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+16,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x2F36 :
        strcpy(buffer_line,"TCPIPCloseTCP(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : ipid             IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x3036 :
        strcpy(buffer_line,"TCPIPAbortTCP(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : ipid             IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x3136 :
        strcpy(buffer_line,"TCPIPStatusTCP(");
        nb_param = 3;
        /* #01 WORD   : wordspace        OUT +6 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : ipid             IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : srBuffPtr        IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x3236 :
        strcpy(buffer_line,"TCPIPGetSourcePort(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : ipid             IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x3336 :
        strcpy(buffer_line,"TCPIPGetTOS(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : ipid             IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x3436 :
        strcpy(buffer_line,"TCPIPSetTOS(");
        nb_param = 2;
        /* #01 WORD   : ipid             IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : TOS              IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3536 :
        strcpy(buffer_line,"TCPIPGetTTL(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : ipid             IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x3636 :
        strcpy(buffer_line,"TCPIPSetTTL(");
        nb_param = 2;
        /* #01 WORD   : ipid             IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : TTL              IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x3736 :
        strcpy(buffer_line,"TCPIPSetSourcePort(");
        nb_param = 2;
        /* #01 WORD   : ipid             IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : sourcePort       IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        break;

      case 0x3F36 :
        strcpy(buffer_line,"TCPIPConvertIPCToHex(");
        nb_param = 2;
        /* #01 DWORD  : cvtRecPtr        IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : ddippstring      IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x4036 :
        strcpy(buffer_line,"TCPIPSendIPDatagram(");
        /* #01 DWORD  : datagramPtr      IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x4136 :
        strcpy(buffer_line,"TCPIPConvertIPToClass(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : ipaddress        IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x4236 :
        strcpy(buffer_line,"TCPIPGetConnectMsgFlag(");
        /* #01 WORD   : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x4336 :
        strcpy(buffer_line,"TCPIPSetConnectMsgFlag(");
        /* #01 WORD   : conMsgFlag       IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x4436 :
        strcpy(buffer_line,"TCPIPGetUsername(");
        /* #01 DWORD  : unBuffPtr        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x4536 :
        strcpy(buffer_line,"TCPIPSetUsername(");
        /* #01 DWORD  : usernamePtr      IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x4636 :
        strcpy(buffer_line,"TCPIPGetPassword(");
        /* #01 DWORD  : pwBuffPtr        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x4736 :
        strcpy(buffer_line,"TCPIPSetPassword(");
        /* #01 DWORD  : passwordPtr      IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,1,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x4836 :
        strcpy(buffer_line,"TCPIPValidateIPString(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : toPtr            IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x4936 :
        strcpy(buffer_line,"TCPIPGetUserStatistic(");
        nb_param = 3;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : ipid             IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : statisticNum     IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x4A36 :
        strcpy(buffer_line,"TCPIPGetLinkVariables(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x4B36 :
        strcpy(buffer_line,"TCPIPEditLinkConfig(");
        nb_param = 2;
        /* #0 DWORD  : connectHandle     IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #0 DWORD  : disconnectHandle  IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x4C36 :
        strcpy(buffer_line,"TCPIPGetModuleNames(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x4E36 :
        strcpy(buffer_line,"TCPIPListenTCP(");
        nb_param = 2;
        /* #01 WORD   : wordspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : ipid             IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x4F36 :
        strcpy(buffer_line,"TCPIPAcceptTCP(");
        nb_param = 3;
        /* #01 WORD   : wordspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : ipid             IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : reserved         IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x5036 :
        strcpy(buffer_line,"TCPIPSetNewDestination(");
        nb_param = 3;
        /* #01 WORD   : ipid             IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : destip           IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : destport         IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x5136 :
        strcpy(buffer_line,"TCPIPGetHostName(");
        /* #01 DWORD  : hnBuffPtr        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x5236 :
        strcpy(buffer_line,"TCPIPSetHostName(");
        /* #01 DWORD  : hostNamePtr      IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x5336 :
        strcpy(buffer_line,"TCPIPStatusUDP(");
        nb_param = 2;
        /* #01 WORD   : ipid             IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : updVarsPtr       IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x5436 :
        strcpy(buffer_line,"TCPIPGetLinkLayer(");
        /* #01 DWORD  : linkInfoBlkPtr   IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x5536 :
        strcpy(buffer_line,"TCPIPPtrToPtr(");
        nb_param = 3;
        /* #01 DWORD  : fromPtr          IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : toPtr            IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : length           IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x5636 :
        strcpy(buffer_line,"TCPIPPtrToPtrNeg(");
        nb_param = 3;
        /* #01 DWORD  : fromEndPtr       IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : toEndPtr         IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : length           IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x5736 :
        strcpy(buffer_line,"TCPIPGetAuthMessage(");
        nb_param = 2;
        /* #01 DWORD  : longspace       OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : userid          IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x5836 :
        strcpy(buffer_line,"TCPIPConvertIPToCASCII(");
        nb_param = 4;
        /* #01 WORD   : wordspace        OUT +10 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : ipaddress        IN  +6 */
        GetGsOsCallOneParameter(2,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : ddpstring        IN  +2 */
        GetGsOsCallOneParameter(3,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : flags            IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+10,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x5936 :
        strcpy(buffer_line,"TCPIPMangleDomainName(");
        nb_param = 3;
        /* #01 WORD   : wordspace        OUT +6 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : flags            IN  +4 */
        GetGsOsCallOneParameter(4,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : dnPstringPtr     IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(3,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x5A36 :
        strcpy(buffer_line,"TCPIPGetAliveFlag(");
        /* #01 WORD   : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x5B36 :
        strcpy(buffer_line,"TCPIPSetAliveFlag(");
        /* #01 WORD   : aliveFlag        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x5C36 :
        strcpy(buffer_line,"TCPIPGetAliveMinutes(");
        /* #01 WORD   : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x5D36 :
        strcpy(buffer_line,"TCPIPSetAliveMinutes(");
        /* #01 WORD   : aliveMinutes     IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x5E36 :
        strcpy(buffer_line,"TCPIPReadLineTCP(");
        nb_param = 7;
        /* #01 WORD   : wordspace        OUT +20 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : ipid             IN  +18 */
        GetGsOsCallOneParameter(2,nb_param,param_address+18,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : delimitStrPtr    IN  +14 */
        string_address = GetMemoryValue(param_address+14,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(3,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : buffType         IN  +12 */
        GetGsOsCallOneParameter(4,nb_param,param_address+12,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #05 DWORD  : buffData         IN  +8 */
        GetGsOsCallOneParameter(5,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #06 DWORD  : buffLen          IN  +4 */
        GetGsOsCallOneParameter(6,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #07 DWORD  : rlrBuffPtr       IN  +0 */
        GetGsOsCallOneParameter(7,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+20,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x5F36 :
        strcpy(buffer_line,"TCPIPGetBootConnectFlag(");
        /* #01 WORD   : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;
 
      case 0x6036 :
        strcpy(buffer_line,"TCPIPSetBootConnectFlag(");
        /* #01 WORD   : bootConnectFlag  IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x6136 :
        strcpy(buffer_line,"TCPIPSetUDPDispatch(");
        nb_param = 2;
        /* #01 WORD   : ipid             IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : dispatchFlag     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x6236 :
        strcpy(buffer_line,"TCPIPGetDestination(");
        nb_param = 2;
        /* #01 WORD   : ipid             IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : destRecPtr       IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x6336 :
        strcpy(buffer_line,"TCPIPGetUserEventTrigger(");
        nb_param = 3;
        /* #01 DWORD  : longspace        OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : triggerNumber    IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : ipid             IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x6436 :
        strcpy(buffer_line,"TCPIPSetUserEventTrigger(");
        nb_param = 3;
        /* #01 WORD   : triggerNumber    IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : ipid             IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : triggerProcPtr   IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x6536 :
        strcpy(buffer_line,"TCPIPGetSysEventTrigger(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : triggerNumber    IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x6636 :
        strcpy(buffer_line,"TCPIPSetSysEventTrigger(");
        nb_param = 2;
        /* #01 WORD   : triggerNumber    IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : triggerProcPtr   IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x6736 :
        strcpy(buffer_line,"TCPIPGetDNRTimeouts(");
        /* #01 DWORD  : dnrTimeoutsBufferPtr  IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;


      case 0x6836 :
        strcpy(buffer_line,"TCPIPSetDNRTimeouts(");
        /* #01 DWORD  : dnrTimeoutsBufferPtr  IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;
      

      /*******************************/
      /*  Tool 0x38 : Time Tool Set  */
      /*******************************/
      case 0x0138 :
        strcpy(buffer_line,"tiBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0238 :
        strcpy(buffer_line,"tiStartUp(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0338 :
        strcpy(buffer_line,"tiShutDown(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0438 :
        strcpy(buffer_line,"tiVersion(");
        /* #01 WORD  : wordspace         OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0538 :
        strcpy(buffer_line,"tiReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0638 :
        strcpy(buffer_line,"tiStatus(");
        /* #01 WORD  : wordspace         OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0938 :
        strcpy(buffer_line,"tiGetTimePrefs(");
        /* #01 DWORD  : tIPrefRecRef     IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0A38 :
        strcpy(buffer_line,"tiSetTimePrefs(");
        /* #01 DWORD  : tIPrefRecRef     IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0B38 :
        strcpy(buffer_line,"tiDateString2Sec(");
        nb_param = 3;
        /* #01 DWORD  : secondPtr       IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : tiInDatePString IN  +2 */
        string_address = GetMemoryValue(param_address+2,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : tiParseFlag      IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0C38 :
        strcpy(buffer_line,"tiDateString2Epoch(");
        nb_param = 3;
        /* #01 DWORD  : secondPtr       IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : tiInDatePString IN  +2 */
        string_address = GetMemoryValue(param_address+2,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : tiParseFlag      IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0D38 :
        strcpy(buffer_line,"tiToday2GMTString(");
        /* #01 DWORD  : tiOutDatePString IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,1,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0E38 :
        strcpy(buffer_line,"tiToday2LocalString(");
        /* #01 DWORD  : tiOutDatePString IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,1,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0F38 :
        strcpy(buffer_line,"tiParseDateString(");
        nb_param = 3;
        /* #01 DWORD  : timeGSRecPtr    IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : tiInDatePString IN  +2 */
        string_address = GetMemoryValue(param_address+2,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : tiParseFlag      IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1038 :
        strcpy(buffer_line,"tiTimeZoneString2Offset(");
        nb_param = 2;
        /* #01 DWORD  : longspace       OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : tiNamePString   IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x1138 :
        strcpy(buffer_line,"tiOffset2TimeZoneString(");
        nb_param = 3;
        /* #01 DWORD  : tiNamePString   IN  +6 */
        string_address = GetMemoryValue(param_address+6,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : secondOffset    IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : MangleFlag       IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;


      /********************************/
      /*  Tool 0x45 : MJuke Tool Set  */
      /********************************/
      case 0x0145 :
        strcpy(buffer_line,"MJBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0245 :
        strcpy(buffer_line,"MJStartUp(");
        nb_param = 2;
        /* #01 WORD  : userID           IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : handleErrFlag    IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0345 :
        strcpy(buffer_line,"MJShutDown(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0445 :
        strcpy(buffer_line,"MJVersion(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0545 :
        strcpy(buffer_line,"MJReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0645 :
        strcpy(buffer_line,"MJStatus(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0945 :
        strcpy(buffer_line,"MJMount(");
        nb_param = 2;
        /* #02 DWORD  : pathToSong       IN  +2 */
        string_address = GetMemoryValue(param_address+2,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #01 WORD   : prefixIndex      IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0A45 :
        strcpy(buffer_line,"MJPlay(");
        /* #01 WORD   : whichSong        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0B45 :
        strcpy(buffer_line,"MJJuke(");
        /* #01 WORD   : firstSong        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0C45 :
        strcpy(buffer_line,"MJCont(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0D45 :
        strcpy(buffer_line,"MJStop(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0E45 :
        strcpy(buffer_line,"MJWave(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0F45 :
        strcpy(buffer_line,"MJNext(");
        /* #01 DWORD  : nextRtnPtr       IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1045 :
        strcpy(buffer_line,"MJSong(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1145 :
        strcpy(buffer_line,"MJTimer(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;
 

      /***************************/
      /*  Tool 0x80 : Hash Tool  */
      /***************************/
      case 0x0180 :
        strcpy(buffer_line,"hashBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0280 :
        strcpy(buffer_line,"hashStartUp(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0380 :
        strcpy(buffer_line,"hashShutDown(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0480 :
        strcpy(buffer_line,"hashVersion(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0580 :
        strcpy(buffer_line,"hashReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0680 :
        strcpy(buffer_line,"hashStatus(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0980 :
        strcpy(buffer_line,"md5Init(");
        /* #01 DWORD : md5WorkBlkPtr   IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0A80 :
        strcpy(buffer_line,"md5Append(");
        nb_param = 3;
        /* #01 DWORD : md5WorkBlkPtr   IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : memoryPtr       IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : memoryLength    IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0B80 :
        strcpy(buffer_line,"md5Finish(");
        nb_param = 2;
        /* #01 DWORD : md5WorkBlkPtr   IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : digestPtr       IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0C80 :
        strcpy(buffer_line,"md4Init(");
        /* #01 DWORD : md4WorkBlkPtr   IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0D80 :
        strcpy(buffer_line,"md4Append(");
        nb_param = 3;
        /* #01 DWORD : md4WorkBlkPtr   IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : memoryPtr       IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : memoryLength    IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0E80 :
        strcpy(buffer_line,"md4Finish(");
        nb_param = 2;
        /* #01 DWORD : md4WorkBlkPtr   IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : digestPtr       IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0F80 :
        strcpy(buffer_line,"md2Init(");
        /* #01 DWORD : md2WorkBlkPtr   IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1080 :
        strcpy(buffer_line,"md2Update(");
        nb_param = 3;
        /* #01 DWORD : md2WorkBlkPtr   IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : memoryPtr       IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : memoryLength    IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1180 :
        strcpy(buffer_line,"md2Final(");
        nb_param = 2;
        /* #01 DWORD : md2WorkBlkPtr   IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : digestPtr       IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1280 :
        strcpy(buffer_line,"sha1Init(");
        /* #01 DWORD : sha1WorkBlkPtr   IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1380 :
        strcpy(buffer_line,"sha1Update(");
        nb_param = 3;
        /* #01 DWORD : sha1WorkBlkPtr   IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : memoryPtr       IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : memoryLength    IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1480 :
        strcpy(buffer_line,"sha1Final(");
        nb_param = 2;
        /* #01 DWORD : sha1WorkBlkPtr   IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : digestPtr       IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;


      /*********************************/
      /*  Tool 0x81 : Crypto Tool Set  */
      /*********************************/
      case 0x0181 :
        strcpy(buffer_line,"cryptoBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0281 :
        strcpy(buffer_line,"cryptoStartUp(");
        /* #01 WORD  : dPageAddr        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0381 :
        strcpy(buffer_line,"cryptoShutDown(");
        /* No Parameters */
        break;

      case 0x0481 :
        strcpy(buffer_line,"cryptoVersion(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0581 :
        strcpy(buffer_line,"cryptoReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0681 :
        strcpy(buffer_line,"cryptoStatus(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0981 :
        strcpy(buffer_line,"desCipher(");
        nb_param = 4;
        /* #01 DWORD : outputPtr       IN  +10 */
        GetGsOsCallOneParameter(1,nb_param,param_address+10,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : keyPtr          IN  +6 */
        GetGsOsCallOneParameter(2,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : inputPtr        IN  +2 */
        GetGsOsCallOneParameter(3,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD  : mode            IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0A81 :
        strcpy(buffer_line,"desAddParity(");
        nb_param = 2;
        /* #01 DWORD : outputPtr         IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : 56bitKeyPtr       IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;


      /*******************************/
      /*  Tool 0x82 : HTML Tool Set  */
      /*******************************/
      case 0x0182 :
        strcpy(buffer_line,"HTMLBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0282 :
        strcpy(buffer_line,"HTMLStartUp(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0382 :
        strcpy(buffer_line,"HTMLShutDown(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0482 :
        strcpy(buffer_line,"HTMLVersion(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0582 :
        strcpy(buffer_line,"HTMLReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0682 :
        strcpy(buffer_line,"HTMLStatus(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0982 :
        strcpy(buffer_line,"HTMLInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0A82 :
        strcpy(buffer_line,"HTMLParse(");
        nb_param = 4;
        /* #01 DWORD  : longspace       OUT +10 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD  : textDescriptor   IN  +8 */
        GetGsOsCallOneParameter(2,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : textRef         IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : textLength      IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+10,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0B82 :
        strcpy(buffer_line,"HTMLOpenDisplay(");
        /* #01 DWORD  : longspace       OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0C82 :
        strcpy(buffer_line,"HTMLCloseDisplay(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0D82 :
        strcpy(buffer_line,"HTMLInsertTERecord(");
        /* #01 DWORD  : teH          IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0E82 :
        strcpy(buffer_line,"HTMLClearDisplay(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0F82 :
        strcpy(buffer_line,"HTMLEventTask(");
        nb_param = 2;
        /* #01 WORD   : wordspace       OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : longspace       OUT +0 */
        GetEmptyParameter(2,nb_param,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x1082 :
        strcpy(buffer_line,"HTMLGetTitle(");
        /* #01 DWORD  : longspace       OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x1182 :
        strcpy(buffer_line,"HTMLGoToName(");
        nb_param = 2;
        /* #01 DWORD  : longspace       OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : stringPointer   IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x1282 :
        strcpy(buffer_line,"HTMLFindString(");
        nb_param = 2;
        /* #01 DWORD  : longspace       OUT +4 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : stringPointer   IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x1382 :
        strcpy(buffer_line,"HTMLGetForm(");
        nb_param = 4;
        /* #01 WORD   : wordspace       OUT +10 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : longspace       OUT +6 */
        GetEmptyParameter(2,nb_param,buffer_line);
        /* #03 DWORD  : longspace       OUT +2 */
        GetEmptyParameter(3,nb_param,buffer_line);
        /* #04 WORD  : formNumber       IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+10,TYPE_GSOS_PARAM_WORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,2,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,3,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x1482 :
        strcpy(buffer_line,"HTMLCheckForForm(");
        /* #01 WORD   : wordspace       OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1582 :
        strcpy(buffer_line,"HTMLGetError(");
        /* #01 WORD   : wordspace       OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1682 :
        strcpy(buffer_line,"HTMLParse2(");
        nb_param = 5;
        /* #01 DWORD  : longspace       OUT +12 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD  : textDescriptor   IN  +10 */
        GetGsOsCallOneParameter(2,nb_param,param_address+10,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : textRef         IN  +6 */
        GetGsOsCallOneParameter(3,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : textLength      IN  +2 */
        GetGsOsCallOneParameter(4,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #05 WORD  : rightMargin      IN  +0 */
        GetGsOsCallOneParameter(5,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+22,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x1782 :
        strcpy(buffer_line,"HTMLEventTask2(");
        /* #01 DWORD  : buffPtr         IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1882 :
        strcpy(buffer_line,"HTMLGetForm2(");
        /* #01 DWORD  : buffPtr         IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1982 :
        strcpy(buffer_line,"HTMLOpenDisplay2(");
        nb_param = 3;
        /* #01 DWORD  : longspace       OUT +6 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : templateFormat  IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : windowTemplate  IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x1A82 :
        strcpy(buffer_line,"HTMLEventToDisplay(");
        nb_param = 2;
        /* #01 DWORD  : ptrEventRecord  IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : buffPtr         IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1B82 :
        strcpy(buffer_line,"HTMLGetPage(");
        nb_param = 4;
        /* #01 WORD   : appID       IN  +12 */
        GetGsOsCallOneParameter(1,nb_param,param_address+12,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : dataIn      IN  +8 */
        GetGsOsCallOneParameter(2,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : dataOut     IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : proxyBlock  IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1C82 :
        strcpy(buffer_line,"HTMLPostForm(");
        nb_param = 4;
        /* #01 WORD   : appID       IN  +12 */
        GetGsOsCallOneParameter(1,nb_param,param_address+12,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : dataIn      IN  +8 */
        GetGsOsCallOneParameter(2,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : dataOut     IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : proxyBlock  IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1D82 :
        strcpy(buffer_line,"HTMLCacheControl(");
        /* #01 DWORD  : buffPtr         IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1E82 :
        strcpy(buffer_line,"HTMLSetTextTERecord(");
        /* #01 DWORD  : teH             IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1F82 :
        strcpy(buffer_line,"HTMLFindString2(");
        nb_param = 4;
        /* #01 DWORD  : longspace       OUT +10 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : stringPointer   IN  +6 */
        string_address = GetMemoryValue(param_address+6,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : teH             IN  +2 */
        GetGsOsCallOneParameter(3,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : findFlags       IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+10,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x2082 :
        strcpy(buffer_line,"HTMLExtractText(");
        nb_param = 4;
        /* #01 DWORD  : longspace       OUT +10 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : flags           IN  +8 */
        GetGsOsCallOneParameter(2,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : textLength      IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD  : teH             IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+10,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x2182 :
        strcpy(buffer_line,"HTMLWriteToLog(");
        nb_param = 2;
        /* #01 WORD   : appID           IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : buffPtr         IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
 
        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);        
        break;

      case 0x2282 :
        strcpy(buffer_line,"HTMLSetLogPath(");
        nb_param = 3;
        /* #01 WORD   : appID           IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : pathName        IN  +2 */
        string_address = GetMemoryValue(param_address+2,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : function        IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;


      /******************************/
      /*  Tool 0x83 : FTP Tool Set  */
      /******************************/
      case 0x0183 :
        strcpy(buffer_line,"FTPBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0283 :
        strcpy(buffer_line,"FTPStartUp(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0383 :
        strcpy(buffer_line,"FTPShutDown(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0483 :
        strcpy(buffer_line,"FTPVersion(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0583 :
        strcpy(buffer_line,"FTPReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0683 :
        strcpy(buffer_line,"FTPStatus(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0983 :
        strcpy(buffer_line,"FTPSendFile(");
        nb_param = 8;
        /* #01 WORD  : ftpUserID        IN  +26 */
        GetGsOsCallOneParameter(1,nb_param,param_address+26,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : userPassword     IN  +22 */
        string_address = GetMemoryValue(param_address+22,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : userName         IN  +18 */
        string_address = GetMemoryValue(param_address+18,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(3,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD : filePath         IN  +14 */
        string_address = GetMemoryValue(param_address+14,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(4,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #05 DWORD : hostPath         IN  +10 */
        string_address = GetMemoryValue(param_address+10,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(5,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #06 DWORD : IPAddress        IN  +6 */
        string_address = GetMemoryValue(param_address+6,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(6,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #07 DWORD : responseBuffer   IN  +2 */
        GetGsOsCallOneParameter(7,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #08 WORD  : flags            IN  +0 */
        GetGsOsCallOneParameter(8,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0A83 :
        strcpy(buffer_line,"FTPReceiveFile(");
        nb_param = 8;
        /* #01 WORD  : ftpUserID        IN  +26 */
        GetGsOsCallOneParameter(1,nb_param,param_address+26,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : userPassword     IN  +22 */
        string_address = GetMemoryValue(param_address+22,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : userName         IN  +18 */
        string_address = GetMemoryValue(param_address+18,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(3,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD : filePath         IN  +14 */
        string_address = GetMemoryValue(param_address+14,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(4,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #05 DWORD : hostPath         IN  +10 */
        string_address = GetMemoryValue(param_address+10,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(5,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #06 DWORD : IPAddress        IN  +6 */
        string_address = GetMemoryValue(param_address+6,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(6,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #07 DWORD : responseBuffer   IN  +2 */
        GetGsOsCallOneParameter(7,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #08 WORD  : flags            IN  +0 */
        GetGsOsCallOneParameter(8,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0B83 :
        strcpy(buffer_line,"FTPDeleteFile(");
        nb_param = 6;
        /* #01 WORD  : ftpUserID        IN  +18 */
        GetGsOsCallOneParameter(1,nb_param,param_address+18,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : userPassword     IN  +14 */
        string_address = GetMemoryValue(param_address+14,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : userName         IN  +10 */
        string_address = GetMemoryValue(param_address+10,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(3,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD : hostPath         IN  +6 */
        string_address = GetMemoryValue(param_address+6,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(4,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #05 DWORD : IPAddress        IN  +2 */
        string_address = GetMemoryValue(param_address+2,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(5,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #06 WORD  : flags            IN  +0 */
        GetGsOsCallOneParameter(6,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0C83 :
        strcpy(buffer_line,"FTPRenameFile(");
        nb_param = 7;
        /* #01 WORD  : ftpUserID        IN  +22 */
        GetGsOsCallOneParameter(1,nb_param,param_address+22,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : userPassword     IN  +18 */
        string_address = GetMemoryValue(param_address+18,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : userName         IN  +14 */
        string_address = GetMemoryValue(param_address+14,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(3,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD : hostPath1        IN  +10 */
        string_address = GetMemoryValue(param_address+10,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(4,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #05 DWORD : hostPath2        IN  +6 */
        string_address = GetMemoryValue(param_address+6,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(5,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #06 DWORD : IPAddress        IN  +2 */
        string_address = GetMemoryValue(param_address+2,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(6,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #07 WORD  : flags            IN  +0 */
        GetGsOsCallOneParameter(7,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0D83 :
        strcpy(buffer_line,"FTPDeleteDir(");
        nb_param = 6;
        /* #01 WORD  : ftpUserID        IN  +18 */
        GetGsOsCallOneParameter(1,nb_param,param_address+18,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : userPassword     IN  +14 */
        string_address = GetMemoryValue(param_address+14,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : userName         IN  +10 */
        string_address = GetMemoryValue(param_address+10,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(3,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD : hostPath         IN  +6 */
        string_address = GetMemoryValue(param_address+6,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(4,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #05 DWORD : IPAddress        IN  +2 */
        string_address = GetMemoryValue(param_address+2,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(5,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #06 WORD  : flags            IN  +0 */
        GetGsOsCallOneParameter(6,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0E83 :
        strcpy(buffer_line,"FTPMakeDir(");
        nb_param = 6;
        /* #01 WORD  : ftpUserID        IN  +18 */
        GetGsOsCallOneParameter(1,nb_param,param_address+18,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : userPassword     IN  +14 */
        string_address = GetMemoryValue(param_address+14,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : userName         IN  +10 */
        string_address = GetMemoryValue(param_address+10,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(3,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD : hostPath         IN  +6 */
        string_address = GetMemoryValue(param_address+6,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(4,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #05 DWORD : IPAddress        IN  +2 */
        string_address = GetMemoryValue(param_address+2,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(5,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #06 WORD  : flags            IN  +0 */
        GetGsOsCallOneParameter(6,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
 
        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);        
        break;

      case 0x0F83 :
        strcpy(buffer_line,"FTPListDir(");
        nb_param = 7;
        /* #01 WORD  : ftpUserID        IN  +22 */
        GetGsOsCallOneParameter(1,nb_param,param_address+22,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : userPassword     IN  +18 */
        string_address = GetMemoryValue(param_address+18,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(2,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : userName         IN  +14 */
        string_address = GetMemoryValue(param_address+14,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(3,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD : hostPath         IN  +10 */
        string_address = GetMemoryValue(param_address+10,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(4,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #05 DWORD : IPAddress        IN  +6 */
        string_address = GetMemoryValue(param_address+6,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(5,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #06 DWORD : ftpbuffData      IN  +2 */
        GetGsOsCallOneParameter(6,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #07 WORD  : flags            IN  +0 */
        GetGsOsCallOneParameter(7,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1083 :
        strcpy(buffer_line,"FTPLogIn(");
        nb_param = 2;
        /* #01 WORD  : wordspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD  : applicationID    IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x1183 :
        strcpy(buffer_line,"FTPLogOut(");
        /* #01 WORD  : ftpUserID        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;


      /******************************/
      /*  Tool 0x84 : Undo Manager  */
      /******************************/
      case 0x0184 :
        strcpy(buffer_line,"UMBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0284 :
        strcpy(buffer_line,"UMStartUp(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0384 :
        strcpy(buffer_line,"UMShutDown(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0484 :
        strcpy(buffer_line,"UMVersion(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0584 :
        strcpy(buffer_line,"UMReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0684 :
        strcpy(buffer_line,"UMStatus(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0984 :
        strcpy(buffer_line,"UMLogIn(");
        nb_param = 2;
        /* #01 WORD   : refID           IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : DataTable       IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0A84 :
        strcpy(buffer_line,"UMLogOut(");
        /* #01 WORD   : refID           IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0B84 :
        strcpy(buffer_line,"UMKeyAction(");
        /* #01 WORD   : refID           IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0C84 :
        strcpy(buffer_line,"UMMenuAction(");
        /* #01 WORD   : refID           IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0D84 :
        strcpy(buffer_line,"UMSaveClip(");
        /* #01 WORD   : refID           IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0E84 :
        strcpy(buffer_line,"UMClearClips(");
        /* #01 WORD   : refID           IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0F84 :
        strcpy(buffer_line,"UMClear(");
        /* #01 WORD   : refID           IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1084 :
        strcpy(buffer_line,"UMClearWindowClips(");
        /* #01 WORD   : refID           IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1184 :
        strcpy(buffer_line,"UMLogOutAll(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1284 :
        strcpy(buffer_line,"UMClipStatus(");
        nb_param = 3;
        /* #01 WORD   : refID             IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : UndoStatusBuffer  IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : ControlHandle     IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1384 :
        strcpy(buffer_line,"UMKeyAction2(");
        nb_param = 2;
        /* #01 WORD   : refID           IN  +2 */
        GetGsOsCallOneParameter(1,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : eventMask       IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1484 :
        strcpy(buffer_line,"UMPushUndoObject(");
        nb_param = 3;
        /* #01 WORD   : refID           IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 WORD   : ObjectType      IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD  : ObjectHandle    IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x1584 :
        strcpy(buffer_line,"UMPopUndoObject(");
        nb_param = 4;
        /* #01 DWORD  : longspace       OUT +6 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : refID           IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : ObjectType      IN  +2 */
        GetGsOsCallOneParameter(3,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : popObjectFlags  IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;


      /****************************/
      /*  Tool 0xDB : SoundSmith  */
      /****************************/
      case 0x01DB :
        strcpy(buffer_line,"STBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x02DB :
        strcpy(buffer_line,"STStartUp(");
        /* #01 WORD  : MemID           IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x03DB :
        strcpy(buffer_line,"STShutDown(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x04DB :
        strcpy(buffer_line,"STVersion(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x05DB :
        strcpy(buffer_line,"STReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x06DB :
        strcpy(buffer_line,"STStatus(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x09DB :
        strcpy(buffer_line,"STLoadOneMusic(");
        /* #01 DWORD : Filename   IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,1,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0ADB :
        strcpy(buffer_line,"STPlayMusic(");
        /* #01 WORD  : LoopMode         IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0BDB :
        strcpy(buffer_line,"STStopMusic(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0CDB :
        strcpy(buffer_line,"STGetEOfMusic(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0DDB :
        strcpy(buffer_line,"STAddToBatch(");
        nb_param = 2;
        /* #01 DWORD : Filename   IN  +2 */
        string_address = GetMemoryValue(param_address+2,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : BatchNb    IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0EDB :
        strcpy(buffer_line,"STSelectBatch(");
        /* #01 WORD  : BatchNb         IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0FDB :
        strcpy(buffer_line,"STKillBatch(");
        /* #01 WORD  : BatchNb         IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x10DB :
        strcpy(buffer_line,"STGetPlayingMusic(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x11DB :
        strcpy(buffer_line,"STPlayBatch(");
        /* #01 DWORD : List   IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x12DB :
        strcpy(buffer_line,"STGetTrackVu(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x13DB :
        strcpy(buffer_line,"STPauseMusic(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x14DB :
        strcpy(buffer_line,"STContinueMusic(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x15DB :
        strcpy(buffer_line,"STinternal15(");
        /* #01 DWORD : musicPtr  IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x16DB :
        strcpy(buffer_line,"STinternal16(");
        /* #01 DWORD : wavebankPtr  IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
 
        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);        
        break;


      /******************************/
      /*  Tool 0xDC : NoiseTracker  */
      /******************************/
      case 0x01DC :
        strcpy(buffer_line,"NTBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x02DC :
        strcpy(buffer_line,"NTStartUp(");
        /* #01 WORD  : DirectPage         IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x03DC :
        strcpy(buffer_line,"NTShutDown(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x04DC :
        strcpy(buffer_line,"NTVersion(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x05DC :
        strcpy(buffer_line,"NTReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x06DC :
        strcpy(buffer_line,"NTStatus(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x09DC :
        strcpy(buffer_line,"NTInitMusic(");
        /* #01 DWORD : ModulePtr   IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0ADC :
        strcpy(buffer_line,"NTLaunchMusic(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0BDC :
        strcpy(buffer_line,"NTUpdateSound(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0CDC :
        strcpy(buffer_line,"NTStopMusic(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;


      /******************************/
      /*  Tool 0xDD : NinjaTracker  */
      /******************************/
      case 0x01DD :
        strcpy(buffer_line,"NTBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x02DD :
        strcpy(buffer_line,"NTStartUp(");
        /* #01 WORD  : MemID           IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x03DD :
        strcpy(buffer_line,"NTShutDown(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x04DD :
        strcpy(buffer_line,"NTVersion(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x05DD :
        strcpy(buffer_line,"NTReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x06DD :
        strcpy(buffer_line,"NTStatus(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x09DD :
        strcpy(buffer_line,"NTLoadOneMusic(");
        /* #01 DWORD : Filename   IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,1,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0ADD :
        strcpy(buffer_line,"NTPlayMusic(");
        /* #01 WORD  : LoopMode         IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0BDD :
        strcpy(buffer_line,"NTStopMusic(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0CDD :
        strcpy(buffer_line,"NTGetEOfMusic(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0DDD :
        strcpy(buffer_line,"NTAddToBatch(");
        nb_param = 2;
        /* #01 DWORD : Filename   IN  +2 */
        string_address = GetMemoryValue(param_address+2,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : BatchNb    IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0EDD :
        strcpy(buffer_line,"NTSelectBatch(");
        /* #01 WORD  : BatchNb         IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0FDD :
        strcpy(buffer_line,"NTKillBatch(");
        /* #01 WORD  : BatchNb         IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x10DD :
        strcpy(buffer_line,"NTGetPlayingMusic(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x11DD :
        strcpy(buffer_line,"NTPlayBatch(");
        /* #01 DWORD : List   IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x12DD :
        strcpy(buffer_line,"NTGetTrackVu(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x13DD :
        strcpy(buffer_line,"NTPauseMusic(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x14DD :
        strcpy(buffer_line,"NTContinueMusic(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x15DD :
        strcpy(buffer_line,"NTinternal15(");
        /* #01 DWORD : musicPtr  IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x16DD :
        strcpy(buffer_line,"NTinternal16(");
        /* #01 DWORD : wavebankPtr  IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;


      /**********************************/
      /*  Tool 0xDE : NinjaTrackerPlus  */
      /**********************************/
      case 0x01DE :
        strcpy(buffer_line,"NTPBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x02DE :
        strcpy(buffer_line,"NTPStartUp(");
        /* #01 WORD  : MemID           IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x03DE :
        strcpy(buffer_line,"NTPShutDown(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x04DE :
        strcpy(buffer_line,"NTPVersion(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x05DE :
        strcpy(buffer_line,"NTPReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x06DE :
        strcpy(buffer_line,"NTPStatus(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x09DE :
        strcpy(buffer_line,"NTPLoadOneMusic(");
        /* #01 DWORD : Filename   IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,1,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
 
        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);        
        break;

      case 0x0ADE :
        strcpy(buffer_line,"NTPPlayMusic(");
        /* #01 WORD  : LoopMode         IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0BDE :
        strcpy(buffer_line,"NTPStopMusic(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0CDE :
        strcpy(buffer_line,"NTPGetEOfMusic(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x0DDE :
        strcpy(buffer_line,"NTPAddToBatch(");
        nb_param = 2;
        /* #01 DWORD : Filename   IN  +2 */
        string_address = GetMemoryValue(param_address+2,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,nb_param,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);
        /* #02 WORD  : BatchNb    IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0EDE :
        strcpy(buffer_line,"NTPSelectBatch(");
        /* #01 WORD  : BatchNb         IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        break;

      case 0x0FDE :
        strcpy(buffer_line,"NTPKillBatch(");
        /* #01 WORD  : BatchNb         IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x10DE :
        strcpy(buffer_line,"NTPGetPlayingMusic(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x11DE :
        strcpy(buffer_line,"NTPPlayBatch(");
        /* #01 DWORD : List   IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x12DE :
        strcpy(buffer_line,"NTPGetTrackVu(");
        /* #01 DWORD  : longspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x13DE :
        strcpy(buffer_line,"NTPPauseMusic(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x14DE :
        strcpy(buffer_line,"NTPContinueMusic(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;


      /***********************/
      /*  Tool 0xE6 : ZPack  */
      /***********************/
      case 0x01E6 :
        strcpy(buffer_line,"ZPBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x02E6 :
        strcpy(buffer_line,"ZPStartUp(");
        /* #01 WORD  : MemID           IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x03E6 :
        strcpy(buffer_line,"ZPShutDown(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x04E6 :
        strcpy(buffer_line,"ZPVersion(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x05E6 :
        strcpy(buffer_line,"ZPReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x06E6 :
        strcpy(buffer_line,"ZPStatus(");
        /* #01 WORD  : wordspace        OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x09E6 :
        strcpy(buffer_line,"ZUnPack(");
        nb_param = 4;
        /* #01 WORD  : BlockSize    IN  +12 */
        GetGsOsCallOneParameter(1,nb_param,param_address+12,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : BlockPtr     IN  +8 */
        GetGsOsCallOneParameter(2,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : OldBlockPtr  IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD : BufferHandle IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0AE6 :
        strcpy(buffer_line,"ZPack(");
        nb_param = 4;
        /* #01 WORD  : BlockSize    IN  +12 */
        GetGsOsCallOneParameter(1,nb_param,param_address+12,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : BlockPtr     IN  +8 */
        GetGsOsCallOneParameter(2,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : OldBlockPtr  IN  +4 */
        GetGsOsCallOneParameter(3,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #04 DWORD : BufferHandle IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0BE6 :
        strcpy(buffer_line,"ZP1Pack(");
        nb_param = 3;
        /* #01 DWORD : longspace    OUT +8 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD : PicturePtr   IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 DWORD : PackPtr      IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0CE6 :
        strcpy(buffer_line,"ZP1UnPack(");
        nb_param = 2;
        /* #01 DWORD : PicturePtr   IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : PackPtr      IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0DE6 :
        strcpy(buffer_line,"ZP1LoadPic(");
        nb_param = 2;
        /* #01 DWORD : PicturePtr   IN  +4 */
        GetGsOsCallOneParameter(1,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : FileName     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0FE6 :
        strcpy(buffer_line,"APFUnPackMain(");
        nb_param = 3;
        /* #01 DWORD : PicturePtr   IN  +6 */
        GetGsOsCallOneParameter(1,nb_param,param_address+6,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD : PackPtr      IN  +2 */
        GetGsOsCallOneParameter(2,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD  : PacSize      IN  +0 */
        GetGsOsCallOneParameter(3,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;


      /************************/
      /*  Tool 0xFF : GS/Bug  */
      /************************/
      case 0x01FF :
        strcpy(buffer_line,"DebugBootInit(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x02FF :
        strcpy(buffer_line,"DebugStartUp(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x03FF :
        strcpy(buffer_line,"DebugShutDown(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x04FF :
        strcpy(buffer_line,"DebugVersion(");
        /* #01 WORD  : wordspace         OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x05FF :
        strcpy(buffer_line,"DebugReset(");
        /* No Parameters */

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x06FF :
        strcpy(buffer_line,"DebugStatus(");
        /* #01 WORD  : wordspace         OUT +0 */
        GetEmptyParameter(1,1,buffer_line);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,1,param_address+0,TYPE_GSOS_PARAM_WORD,cyrene_param);
        break;

      case 0x09FF :
        strcpy(buffer_line,"DebugStr(");
        /* #01 DWORD : strPtr            IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,1,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0AFF :
        strcpy(buffer_line,"SetMileStone(");
        /* #01 DWORD : strPtr            IN  +0 */
        string_address = GetMemoryValue(param_address+0,4,current_operation,cyrene_param);
        GetGsOsCallOneParameter(1,1,string_address,TYPE_GSOS_PARAM_PSTRING,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0BFF :
        strcpy(buffer_line,"DebugSetHook(");
        /* #01 DWORD : hookRtnPtr        IN  +0 */
        GetGsOsCallOneParameter(1,1,param_address+0,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0CFF :
        strcpy(buffer_line,"DebugGetInfo(");
        nb_param = 2;
        /* #01 DWORD  : longspace        OUT +2 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 WORD   : wordSelector     IN  +0 */
        GetGsOsCallOneParameter(2,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+2,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      case 0x0DFF :
        strcpy(buffer_line,"DebugControl(");
        nb_param = 4;
        /* #01 DWORD  : data             IN  +8 */
        GetGsOsCallOneParameter(1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #02 DWORD  : extraData        IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : operation        IN  +2 */
        GetGsOsCallOneParameter(3,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : type             IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /** No Return Value **/
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,0,0,0,0,cyrene_param);
        break;

      case 0x0EFF :
        strcpy(buffer_line,"DebugQuery(");
        nb_param = 4;
        /* #01 DWORD  : longspace        OUT +8 */
        GetEmptyParameter(1,nb_param,buffer_line);
        /* #02 DWORD  : data             IN  +4 */
        GetGsOsCallOneParameter(2,nb_param,param_address+4,TYPE_GSOS_PARAM_DWORD,buffer_line,current_operation,cyrene_param);
        /* #03 WORD   : operation        IN  +2 */
        GetGsOsCallOneParameter(3,nb_param,param_address+2,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);
        /* #04 WORD   : type             IN  +0 */
        GetGsOsCallOneParameter(4,nb_param,param_address+0,TYPE_GSOS_PARAM_WORD,buffer_line,current_operation,cyrene_param);

        /* Return Value */
        BuildSystemCallReturnValues(return_address,call_number,TRACE_CALL_ORIGIN_TOOL,1,nb_param,param_address+8,TYPE_GSOS_PARAM_DWORD,cyrene_param);
        break;

      default :
        return(NULL);
        break;
    }

  /* Renvoi la ligne */
  strcat(buffer_line,")");
  return(&buffer_line[0]);
}


/******************************************************************/
/*  GetGsOsCallOneParameter() :  Rcupre / Formate un paramtre. */
/******************************************************************/
static void GetGsOsCallOneParameter(WORD param_num, WORD nb_param_max, DWORD param_string_address, int param_type, char *buffer_line_rtn, struct cyrene_operation *current_operation, struct cyrene_parameter *cyrene_param)
{
  int buffer_length, string_offset;
  BYTE one_byte, byte_1, byte_2, byte_3;
  WORD i, one_word, string_length, buffer_size;
  DWORD one_dword, two_dword, string_address;
  char buffer_hexa[16];
  char buffer_parameter[BUFFER_SIZE];
  unsigned char buffer_string[BUFFER_SIZE];

  /* Init */
  strcpy(buffer_parameter,"");
  string_offset = 0;

  /* Vrifie le nombre de paramtres */
  if(param_num > nb_param_max)
    return;

  /** Traite le paramtre en fonction de son type **/
  if(param_type == TYPE_GSOS_PARAM_BYTE)
    {
      one_byte = (BYTE) GetMemoryValue(param_string_address,1,current_operation,cyrene_param);
      sprintf(buffer_parameter,"%02X",one_byte);
    }
  else if(param_type == TYPE_GSOS_PARAM_WORD)
    {
      one_word = (WORD) (DWORD) GetMemoryValue(param_string_address,2,current_operation,cyrene_param);
      sprintf(buffer_parameter,"%04X",one_word);
    }
  else if(param_type == TYPE_GSOS_PARAM_24BIT)
    {
      byte_1 = (BYTE) GetMemoryValue(param_string_address+0,1,current_operation,cyrene_param);
      byte_2 = (BYTE) GetMemoryValue(param_string_address+1,1,current_operation,cyrene_param);
      byte_3 = (BYTE) GetMemoryValue(param_string_address+2,1,current_operation,cyrene_param);
      sprintf(buffer_parameter,"%02X%02X%02X",byte_3,byte_2,byte_1);
    }
  else if(param_type == TYPE_GSOS_PARAM_DWORD)
    {
      one_dword = GetMemoryValue(param_string_address,4,current_operation,cyrene_param);
      sprintf(buffer_parameter,"%08X",one_dword);
    }
  else if(param_type == TYPE_GSOS_PARAM_QWORD)
    {
      one_dword = GetMemoryValue(param_string_address,4,current_operation,cyrene_param);
      two_dword = GetMemoryValue(param_string_address+4,4,current_operation,cyrene_param);
      sprintf(buffer_parameter,"%08X%08X",one_dword,two_dword);
    }
  else if(param_type == TYPE_GSOS_PARAM_STRING)
    {
      /* String Address : DWORD */
      string_address = param_string_address;
      if(string_address == 0x00000000)
        {
          /* NULL => Empty String */
          strcpy(buffer_parameter,"0\"\"");
        }
      else
        {
          /* String Length : 2 bytes */
          string_offset = 2;
          string_length = (WORD) GetMemoryValue(string_address,2,current_operation,cyrene_param);
          if(string_length > 512)
            {
              /* Chaine en Erreur : On code l'adresse, la longueur et les 24 premiers Bytes */
              sprintf((char *)buffer_string,"@=%06X,L=%04X,",string_address,string_length);
              string_length = strlen((char *)buffer_string);
              for(i=0; i<24; i++)
                buffer_string[string_length+i] = GetMemoryByte(string_address+string_offset+i,current_operation,cyrene_param);
              string_length += 24;
            }
          else
            {
              /* Rcupre la chaine */
              for(i=0; i<string_length; i++)
                buffer_string[i] = GetMemoryByte(string_address+string_offset+i,current_operation,cyrene_param);
            }
              
          /* String Length Encoding */
          sprintf(buffer_parameter,"%d",string_length*2);

          /* String Data Encoded */
          strcat(buffer_parameter,"\"");
          buffer_length = strlen(buffer_parameter);
          for(i=0; i<string_length; i++)
            {
              sprintf(buffer_hexa,"%02X",buffer_string[i]);
              memcpy(&buffer_parameter[buffer_length+2*i],buffer_hexa,3);
            }
          strcat(buffer_parameter,"\"");
        }
    }
  else if(param_type == TYPE_GSOS_PARAM_STRING_PTR)
    {
      /* String Address Pointer */
      if(param_string_address == 0x00000000)
        {
          /* NULL => Empty String */
          strcpy(buffer_parameter,"0\"\"");
        }
      else
        {
          /* String Address : DWORD */
          string_address = (DWORD) GetMemoryValue(param_string_address,4,current_operation,cyrene_param);
          if(string_address == 0x00000000)
            {
              /* NULL => Empty String */
              strcpy(buffer_parameter,"0\"\"");
            }
          else
            {
              /* String Length : 2 bytes */
              string_offset = 2;
              string_length = (WORD) GetMemoryValue(string_address,2,current_operation,cyrene_param);
              if(string_length > 512)
                {
                  /* Chaine en Erreur : On code l'adresse, la longueur et les 24 premiers Bytes */
                  sprintf((char *)buffer_string,"@=%06X,L=%04X,",string_address,string_length);
                  string_length = strlen((char *)buffer_string);
                  for(i=0; i<24; i++)
                    buffer_string[string_length+i] = GetMemoryByte(string_address+string_offset+i,current_operation,cyrene_param);
                  string_length += 24;
                }
              else
                {
                  /* Rcupre la chaine */
                  for(i=0; i<string_length; i++)
                    buffer_string[i] = GetMemoryByte(string_address+string_offset+i,current_operation,cyrene_param);
                }
              
              /* String Length Encoding */
              sprintf(buffer_parameter,"%d",string_length*2);

              /* String Data Encoded */
              strcat(buffer_parameter,"\"");
              buffer_length = strlen(buffer_parameter);
              for(i=0; i<string_length; i++)
                {
                  sprintf(buffer_hexa,"%02X",buffer_string[i]);
                  memcpy(&buffer_parameter[buffer_length+2*i],buffer_hexa,3);
                }
              strcat(buffer_parameter,"\"");
            }
        }
    }
  else if(param_type == TYPE_GSOS_PARAM_RESULT_STRING)
    {
      /* String Address : DWORD */
      string_address = param_string_address;
      if(string_address == 0x00000000)
        {
          /* NULL => Empty String */
          strcpy(buffer_parameter,"0\"\"");
        }
      else
        {
          /* Buffer Size : 2 bytes */
          buffer_size = (WORD) GetMemoryValue(string_address,2,current_operation,cyrene_param);

          /* String Length : 2 bytes */
          string_offset = 4;
          string_length = (WORD) GetMemoryValue(string_address+2,2,current_operation,cyrene_param);
          if(string_length > 512)
            {
              /* Chaine en Erreur : On code l'adresse, la longueur et les 24 premiers Bytes */
              sprintf((char *)buffer_string,"@=%06X,B=%04X,L=%04X,",string_address,buffer_size,string_length);
              string_length = strlen((char *)buffer_string);
              for(i=0; i<24; i++)
                buffer_string[string_length+i] = GetMemoryByte(string_address+string_offset+i,current_operation,cyrene_param);
              string_length += 24;
            }
          else
            {
              /* Rcupre la chaine */
              for(i=0; i<string_length; i++)
                buffer_string[i] = GetMemoryByte(string_address+string_offset+i,current_operation,cyrene_param);
            }

          /* String Length Encoding */
          sprintf(buffer_parameter,"%d",string_length*2);

          /* String Data */
          strcat(buffer_parameter,"\"");
          buffer_length = strlen(buffer_parameter);
          for(i=0; i<string_length; i++)
            {
              sprintf(buffer_hexa,"%02X",buffer_string[i]);
              memcpy(&buffer_parameter[buffer_length+2*i],buffer_hexa,3);
            }
          strcat(buffer_parameter,"\"");
        }
    }
  else if(param_type == TYPE_GSOS_PARAM_P8STRING)
    {
      /* String Address : 16 bit in Bank 00 */
      string_address = param_string_address;
      if(string_address == 0x00000000)
        {
          /* NULL => Empty String */
          strcpy(buffer_parameter,"0\"\"");
        }
      else
        {
          /* String Length : 1 byte */
          string_offset = 1;
          string_length = (WORD) GetMemoryValue(string_address,1,current_operation,cyrene_param);
          if(string_length > 100)
            {
              /* Chaine en Erreur : On code l'adresse, la longueur et les 24 premiers Bytes */
              sprintf((char *)buffer_string,"@=%06X,L=%02X,",string_address,(BYTE)string_length);
              string_length = strlen((char *)buffer_string);
              for(i=0; i<24; i++)
                buffer_string[string_length+i] = GetMemoryByte(string_address+string_offset+i,current_operation,cyrene_param);
              string_length += 24;
            }
          else
            {
              /* Rcupre la chaine */
              for(i=0; i<string_length; i++)
                buffer_string[i] = GetMemoryByte(string_address+string_offset+i,current_operation,cyrene_param);
            }

          /* String Length Encoding */
          sprintf(buffer_parameter,"%d",string_length*2);

          /* String Data */
          strcat(buffer_parameter,"\"");
          buffer_length = strlen(buffer_parameter);
          for(i=0; i<string_length; i++)
            {
              sprintf(buffer_hexa,"%02X",buffer_string[i]);
              memcpy(&buffer_parameter[buffer_length+2*i],buffer_hexa,3);
            }
          strcat(buffer_parameter,"\"");
        }
    }
  else if(param_type == TYPE_GSOS_PARAM_PSTRING)
    {
      /* String Address : 32 bit */
      string_address = param_string_address;
      if(string_address == 0x00000000)
        {
          /* NULL => Empty String */
          strcpy(buffer_parameter,"0\"\"");
        }
      else
        {
          /* String Length : 1 byte */
          string_offset = 1;
          string_length = (WORD) GetMemoryValue(string_address,1,current_operation,cyrene_param);
          if(string_length > 100)
            {
              /* Chaine en Erreur : On code l'adresse, la longueur et les 24 premiers Bytes */
              sprintf((char *)buffer_string,"@=%06X,L=%02X,",string_address,(BYTE)string_length);
              string_length = strlen((char *)buffer_string);
              for(i=0; i<24; i++)
                buffer_string[string_length+i] = GetMemoryByte(string_address+string_offset+i,current_operation,cyrene_param);
              string_length += 24;
            }
          else
            {
              /* Rcupre la chaine */
              for(i=0; i<string_length; i++)
                buffer_string[i] = GetMemoryByte(string_address+string_offset+i,current_operation,cyrene_param);
            }

          /* String Length Encoding */
          sprintf(buffer_parameter,"%d",string_length*2);

          /* String Data */
          strcat(buffer_parameter,"\"");
          buffer_length = strlen(buffer_parameter);
          for(i=0; i<string_length; i++)
            {
              sprintf(buffer_hexa,"%02X",buffer_string[i]);
              memcpy(&buffer_parameter[buffer_length+2*i],buffer_hexa,3);
            }
          strcat(buffer_parameter,"\"");
        }
    }
  else if(param_type >= TYPE_GSOS_PARAM_TXT)
    {
      /* String Address : 32 bit */
      string_address = param_string_address;
      if(string_address == 0x00000000)
        {
          /* NULL => Empty String */
          strcpy(buffer_parameter,"0\"\"");
        }
      else
        {
          /* String Length : au bout du buffer_size_rtn */
          string_length = param_type - TYPE_GSOS_PARAM_TXT;
          if(string_length > 512)
            string_length = 32;

          /* String Length Encoding */
          sprintf(buffer_parameter,"%d",string_length*2);

          /* String Data */
          strcat(buffer_parameter,"\"");
          buffer_length = strlen(buffer_parameter);
          for(i=0; i<string_length; i++)
            {
              one_byte = GetMemoryByte(string_address+i,current_operation,cyrene_param);
              sprintf(buffer_hexa,"%02X",one_byte);
              memcpy(&buffer_parameter[buffer_length+2*i],buffer_hexa,3);
            }
          strcat(buffer_parameter,"\"");
        }
    }

  /** Ajoute le paramtre  la fin de la ligne **/
  if(buffer_line_rtn[strlen(buffer_line_rtn)-1] != '(')
    strcat(buffer_line_rtn,",");
  strcat(buffer_line_rtn,buffer_parameter);
}


/*****************************************************/
/*  GetEmptyParameter() :  Insre un paramtre vide. */
/*****************************************************/
static void GetEmptyParameter(WORD param_num, WORD nb_param_max, char *buffer_line_rtn)
{
  /* Vrifie le nombre de paramtres */
  if(param_num > nb_param_max)
    return;

  /** Ajoute le paramtre vide  la fin de la ligne **/
  if(buffer_line_rtn[strlen(buffer_line_rtn)-1] != '(')
    strcat(buffer_line_rtn,",");
  strcat(buffer_line_rtn,"-");
}


/***************************************************************************************************************************/
/*  BuildSystemCallReturnValues() :  Cration d'une structure pour rcuprer les paramtres de retour des appels systmes. */
/***************************************************************************************************************************/
static void BuildSystemCallReturnValues(DWORD return_address, WORD call_number, int origin, int nb_param, int nb_param_max, DWORD param_string_address, int param_type, struct cyrene_parameter *cyrene_param)
{
  struct cyrene_system_call *current_system_call;

  /* Non disponible */
  if(nb_param > nb_param_max)
    return;

  /** Cherche une structure pour cet appel **/
  for(current_system_call = cyrene_param->last_system_call; current_system_call; current_system_call = current_system_call->previous)
    if(current_system_call->operation_goid == cyrene_param->goid)
      {
        current_system_call->tab_param_address[current_system_call->nb_param] = param_string_address;
        current_system_call->tab_param_type[current_system_call->nb_param]    = param_type;
        current_system_call->nb_param++;
        return;
      }

  /* Allocation mmoire */
  current_system_call = (struct cyrene_system_call *) calloc(1,sizeof(struct cyrene_system_call));
  if(current_system_call == NULL)
    return;

  /** Remplissage avec le premier paramtre **/
  current_system_call->operation_goid = cyrene_param->goid;
  current_system_call->operation_gcc  = cyrene_param->gcc;
  current_system_call->return_address = return_address;
  current_system_call->call_number    = call_number;
  current_system_call->origin         = origin;

  /* Paramtres */
  if(nb_param != 0)
    {
      current_system_call->nb_param             = 1;
      current_system_call->tab_param_address[0] = param_string_address;
      current_system_call->tab_param_type[0]    = param_type;
    }

  /** Attachement **/
  if(cyrene_param->first_system_call == NULL)
    {
      cyrene_param->first_system_call = current_system_call;
      current_system_call->previous = NULL;
    }
  else
    {
      cyrene_param->last_system_call->next = current_system_call;
      current_system_call->previous = cyrene_param->last_system_call;
    }
  cyrene_param->last_system_call = current_system_call;
}


/*****************************************************************************************/
/*  RemoveSystemCallFromList() :  Supprime une structure cyrene_system_call de la liste. */
/*****************************************************************************************/
void RemoveSystemCallFromList(struct cyrene_system_call *current_system_call, struct cyrene_parameter *cyrene_param)
{
  /* Une seule structure */
  if(cyrene_param->first_system_call == current_system_call && cyrene_param->first_system_call == cyrene_param->last_system_call)  
    {
      cyrene_param->first_system_call = NULL;
      cyrene_param->last_system_call = NULL;
    }
  /* Premire structure */
  else if(cyrene_param->first_system_call == current_system_call)
    {
      cyrene_param->first_system_call = current_system_call->next;
      cyrene_param->first_system_call->previous = NULL;
    }
  /* Dernire structure */
  else if(cyrene_param->last_system_call == current_system_call)
    {
      current_system_call->previous->next = NULL;
      cyrene_param->last_system_call = current_system_call->previous;
    }
  /* Au milieu */
  else
    {
      current_system_call->previous->next = current_system_call->next;
      current_system_call->next->previous = current_system_call->previous;
    }

  /* Libration mmoire */
  mem_free_system_call(current_system_call);
}


/*************************************************************************************/
/*  mem_free_system_call() :  Libration mmoire de la structure cyrene_system_call. */
/*************************************************************************************/
void mem_free_system_call(struct cyrene_system_call *current_system_call)
{
  if(current_system_call)
    {
      free(current_system_call);
    }
}

/*********************************************************************/
